use super::merge_iter::MergeIterInner;
use super::node::{self, Root};
use core::alloc::Allocator;
use core::iter::FusedIterator;

impl<K, V> Root<K, V> {
    /// Appends all key-value pairs from the union of two ascending iterators, incrementing a `length` variable along the way. <br>从两个升序的迭代器的并集追加所有键值对，并在此过程中递增 `length` 变量。<br> The latter makes it easier for the caller to avoid a leak when a drop handler panicks. <br>后者使调用者在丢弃处理程序 panic 时更容易避免泄漏。<br>
    ///
    /// If both iterators produce the same key, this method drops the pair from the left iterator and appends the pair from the right iterator. <br>如果两个迭代器都产生相同的键，则此方法将从左迭代器中丢弃 pair，并从右迭代器中追加 pair。<br>
    ///
    /// If you want the tree to end up in a strictly ascending order, like for a `BTreeMap`, both iterators should produce keys in strictly ascending order, each greater than all keys in the tree, including any keys already in the tree upon entry. <br>如果要使树以严格的升序结束 (例如 `BTreeMap`)，则两个迭代器都应以严格的升序生成键，每个键都大于树中的所有键，包括输入时树中已存在的任何键。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    pub fn append_from_sorted_iters<I, A: Allocator + Clone>(
        &mut self,
        left: I,
        right: I,
        length: &mut usize,
        alloc: A,
    ) where
        K: Ord,
        I: Iterator<Item = (K, V)> + FusedIterator,
    {
        // We prepare to merge `left` and `right` into a sorted sequence in linear time. <br>我们准备在线性时间内将 `left` 和 `right` 合并为一个排序的序列。<br>
        let iter = MergeIter(MergeIterInner::new(left, right));

        // Meanwhile, we build a tree from the sorted sequence in linear time. <br>同时，我们根据线性时间中的排序序列构建一棵树。<br>
        self.bulk_push(iter, length, alloc)
    }

    /// Pushes all key-value pairs to the end of the tree, incrementing a `length` variable along the way. <br>将所有键值对推入树的末尾，并在此过程中递增 `length` 变量。<br>
    /// The latter makes it easier for the caller to avoid a leak when the iterator panicks. <br>后者使调用者在迭代器出现 panic 时更容易避免泄漏。<br>
    ///
    pub fn bulk_push<I, A: Allocator + Clone>(&mut self, iter: I, length: &mut usize, alloc: A)
    where
        I: Iterator<Item = (K, V)>,
    {
        let mut cur_node = self.borrow_mut().last_leaf_edge().into_node();
        // Iterate through all key-value pairs, pushing them into nodes at the right level. <br>遍历所有键值对，将它们推入正确级别的节点。<br>
        for (key, value) in iter {
            // Try to push key-value pair into the current leaf node. <br>尝试将键值对推入当前的叶子节点。<br>
            if cur_node.len() < node::CAPACITY {
                cur_node.push(key, value);
            } else {
                // No space left, go up and push there. <br>没有剩余空间了，上去推一下。<br>
                let mut open_node;
                let mut test_node = cur_node.forget_type();
                loop {
                    match test_node.ascend() {
                        Ok(parent) => {
                            let parent = parent.into_node();
                            if parent.len() < node::CAPACITY {
                                // Found a node with space left, push here. <br>找到一个剩余空间的节点，将其推入此处。<br>
                                open_node = parent;
                                break;
                            } else {
                                // Go up again. <br>再上去<br>
                                test_node = parent.forget_type();
                            }
                        }
                        Err(_) => {
                            // We are at the top, create a new root node and push there. <br>我们在顶部，创建一个新的根节点并推送到那里。<br>
                            open_node = self.push_internal_level(alloc.clone());
                            break;
                        }
                    }
                }

                // Push key-value pair and new right subtree. <br>push 键值对和新的右子树。<br>
                let tree_height = open_node.height() - 1;
                let mut right_tree = Root::new(alloc.clone());
                for _ in 0..tree_height {
                    right_tree.push_internal_level(alloc.clone());
                }
                open_node.push(key, value, right_tree);

                // Go down to the right-most leaf again. <br>再次下降到最右边的叶子。<br>
                cur_node = open_node.forget_type().last_leaf_edge().into_node();
            }

            // Increment length every iteration, to make sure the map drops the appended elements even if advancing the iterator panicks. <br>每次迭代都增加长度，以确保即使推进迭代器出现 panic，map 也会丢弃附加的元素。<br>
            //
            *length += 1;
        }
        self.fix_right_border_of_plentiful();
    }
}

// An iterator for merging two sorted sequences into one <br>用于将两个排序的序列合并为一个的迭代器<br>
struct MergeIter<K, V, I: Iterator<Item = (K, V)>>(MergeIterInner<I>);

impl<K: Ord, V, I> Iterator for MergeIter<K, V, I>
where
    I: Iterator<Item = (K, V)> + FusedIterator,
{
    type Item = (K, V);

    /// If two keys are equal, returns the key-value pair from the right source. <br>如果两个键相等，则从正确的源返回键值对。<br>
    fn next(&mut self) -> Option<(K, V)> {
        let (a_next, b_next) = self.0.nexts(|a: &(K, V), b: &(K, V)| K::cmp(&a.0, &b.0));
        b_next.or(a_next)
    }
}
