use core::intrinsics;
use core::mem;
use core::ptr;

/// This replaces the value behind the `v` unique reference by calling the relevant function. <br>这将通过调用相关函数来替换 `v` 唯一引用后面的值。<br>
///
///
/// If a panic occurs in the `change` closure, the entire process will be aborted. <br>如果 `change` 闭包中出现 panic，则整个进程将被中止。<br>
#[allow(dead_code)] // keep as illustration and for future use <br>作为插图保存，以备将来使用<br>
#[inline]
pub fn take_mut<T>(v: &mut T, change: impl FnOnce(T) -> T) {
    replace(v, |value| (change(value), ()))
}

/// This replaces the value behind the `v` unique reference by calling the relevant function, and returns a result obtained along the way. <br>这将通过调用相关函数来替换 `v` 唯一引用后面的值，并返回沿途获得的结果。<br>
///
///
/// If a panic occurs in the `change` closure, the entire process will be aborted. <br>如果 `change` 闭包中出现 panic，则整个进程将被中止。<br>
#[inline]
pub fn replace<T, R>(v: &mut T, change: impl FnOnce(T) -> (T, R)) -> R {
    struct PanicGuard;
    impl Drop for PanicGuard {
        fn drop(&mut self) {
            intrinsics::abort()
        }
    }
    let guard = PanicGuard;
    let value = unsafe { ptr::read(v) };
    let (new_value, ret) = change(value);
    unsafe {
        ptr::write(v, new_value);
    }
    mem::forget(guard);
    ret
}
