//! A doubly-linked list with owned nodes. <br>具有所属节点的双向链表。<br>
//!
//! The `LinkedList` allows pushing and popping elements at either end in constant time. <br>`LinkedList` 允许在恒定时间内在任一端推送和弹出元素。<br>
//!
//! NOTE: It is almost always better to use [`Vec`] or [`VecDeque`] because array-based containers are generally faster, more memory efficient, and make better use of CPU cache. <br>使用 [`Vec`] 或 [`VecDeque`] 几乎总是更好，因为基于数组的容器通常更快，内存效率更高，并且可以更好地利用 CPU 缓存。<br>
//!
//!
//! [`Vec`]: crate::vec::Vec
//! [`VecDeque`]: super::vec_deque::VecDeque
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

use core::cmp::Ordering;
use core::fmt;
use core::hash::{Hash, Hasher};
use core::iter::{FromIterator, FusedIterator};
use core::marker::PhantomData;
use core::mem;
use core::ptr::NonNull;

use super::SpecExtend;
use crate::boxed::Box;

#[cfg(test)]
mod tests;

/// A doubly-linked list with owned nodes. <br>具有所属节点的双向链表。<br>
///
/// The `LinkedList` allows pushing and popping elements at either end in constant time. <br>`LinkedList` 允许在恒定时间内在任一端推送和弹出元素。<br>
///
/// A `LinkedList` with a known list of items can be initialized from an array: <br>可以从数组初始化具有已知项列表的 `LinkedList`：<br>
///
/// ```
/// use std::collections::LinkedList;
///
/// let list = LinkedList::from([1, 2, 3]);
/// ```
///
/// NOTE: It is almost always better to use [`Vec`] or [`VecDeque`] because array-based containers are generally faster, more memory efficient, and make better use of CPU cache. <br>使用 [`Vec`] 或 [`VecDeque`] 几乎总是更好，因为基于数组的容器通常更快，内存效率更高，并且可以更好地利用 CPU 缓存。<br>
///
///
/// [`Vec`]: crate::vec::Vec
/// [`VecDeque`]: super::vec_deque::VecDeque
///
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "LinkedList")]
#[rustc_insignificant_dtor]
pub struct LinkedList<T> {
    head: Option<NonNull<Node<T>>>,
    tail: Option<NonNull<Node<T>>>,
    len: usize,
    marker: PhantomData<Box<Node<T>>>,
}

struct Node<T> {
    next: Option<NonNull<Node<T>>>,
    prev: Option<NonNull<Node<T>>>,
    element: T,
}

/// An iterator over the elements of a `LinkedList`. <br>`LinkedList` 元素上的迭代器。<br>
///
/// This `struct` is created by [`LinkedList::iter()`]. <br>该 `struct` 由 [`LinkedList::iter()`] 创建。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Iter<'a, T: 'a> {
    head: Option<NonNull<Node<T>>>,
    tail: Option<NonNull<Node<T>>>,
    len: usize,
    marker: PhantomData<&'a Node<T>>,
}

#[stable(feature = "collection_debug", since = "1.17.0")]
impl<T: fmt::Debug> fmt::Debug for Iter<'_, T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple("Iter")
            .field(&*mem::ManuallyDrop::new(LinkedList {
                head: self.head,
                tail: self.tail,
                len: self.len,
                marker: PhantomData,
            }))
            .field(&self.len)
            .finish()
    }
}

// FIXME(#26925) Remove in favor of `#[derive(Clone)]` <br>删除以支持 `#[derive(Clone)]`<br>
#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Clone for Iter<'_, T> {
    fn clone(&self) -> Self {
        Iter { ..*self }
    }
}

/// A mutable iterator over the elements of a `LinkedList`. <br>`LinkedList` 元素上的可变迭代器。<br>
///
/// This `struct` is created by [`LinkedList::iter_mut()`]. <br>该 `struct` 由 [`LinkedList::iter_mut()`] 创建。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct IterMut<'a, T: 'a> {
    head: Option<NonNull<Node<T>>>,
    tail: Option<NonNull<Node<T>>>,
    len: usize,
    marker: PhantomData<&'a mut Node<T>>,
}

#[stable(feature = "collection_debug", since = "1.17.0")]
impl<T: fmt::Debug> fmt::Debug for IterMut<'_, T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple("IterMut")
            .field(&*mem::ManuallyDrop::new(LinkedList {
                head: self.head,
                tail: self.tail,
                len: self.len,
                marker: PhantomData,
            }))
            .field(&self.len)
            .finish()
    }
}

/// An owning iterator over the elements of a `LinkedList`. <br>`LinkedList` 元素上的拥有的迭代器。<br>
///
/// This `struct` is created by the [`into_iter`] method on [`LinkedList`] (provided by the [`IntoIterator`] trait). <br>这个 `struct` 是通过 [`LinkedList`] 上的 [`into_iter`] 方法创建的 (由 [`IntoIterator`] trait 提供)。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// [`into_iter`]: LinkedList::into_iter
/// [`IntoIterator`]: core::iter::IntoIterator
#[derive(Clone)]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct IntoIter<T> {
    list: LinkedList<T>,
}

#[stable(feature = "collection_debug", since = "1.17.0")]
impl<T: fmt::Debug> fmt::Debug for IntoIter<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple("IntoIter").field(&self.list).finish()
    }
}

impl<T> Node<T> {
    fn new(element: T) -> Self {
        Node { next: None, prev: None, element }
    }

    fn into_element(self: Box<Self>) -> T {
        self.element
    }
}

// private methods <br>private 方法<br>
impl<T> LinkedList<T> {
    /// Adds the given node to the front of the list. <br>将给定节点添加到列表的最前面。<br>
    #[inline]
    fn push_front_node(&mut self, mut node: Box<Node<T>>) {
        // This method takes care not to create mutable references to whole nodes, to maintain validity of aliasing pointers into `element`. <br>此方法注意不要对整个节点创建变量引用，以保持 `element` 中的别名指针的有效性。<br>
        //
        unsafe {
            node.next = self.head;
            node.prev = None;
            let node = Some(Box::leak(node).into());

            match self.head {
                None => self.tail = node,
                // Not creating new mutable (unique!) references overlapping `element`. <br>不创建新的可变 (unique!) 引用重叠的 `element`。<br>
                Some(head) => (*head.as_ptr()).prev = node,
            }

            self.head = node;
            self.len += 1;
        }
    }

    /// Removes and returns the node at the front of the list. <br>删除并返回列表前面的节点。<br>
    #[inline]
    fn pop_front_node(&mut self) -> Option<Box<Node<T>>> {
        // This method takes care not to create mutable references to whole nodes, to maintain validity of aliasing pointers into `element`. <br>此方法注意不要对整个节点创建变量引用，以保持 `element` 中的别名指针的有效性。<br>
        //
        self.head.map(|node| unsafe {
            let node = Box::from_raw(node.as_ptr());
            self.head = node.next;

            match self.head {
                None => self.tail = None,
                // Not creating new mutable (unique!) references overlapping `element`. <br>不创建新的可变 (unique!) 引用重叠的 `element`。<br>
                Some(head) => (*head.as_ptr()).prev = None,
            }

            self.len -= 1;
            node
        })
    }

    /// Adds the given node to the back of the list. <br>将给定节点添加到列表的后面。<br>
    #[inline]
    fn push_back_node(&mut self, mut node: Box<Node<T>>) {
        // This method takes care not to create mutable references to whole nodes, to maintain validity of aliasing pointers into `element`. <br>此方法注意不要对整个节点创建变量引用，以保持 `element` 中的别名指针的有效性。<br>
        //
        unsafe {
            node.next = None;
            node.prev = self.tail;
            let node = Some(Box::leak(node).into());

            match self.tail {
                None => self.head = node,
                // Not creating new mutable (unique!) references overlapping `element`. <br>不创建新的可变 (unique!) 引用重叠的 `element`。<br>
                Some(tail) => (*tail.as_ptr()).next = node,
            }

            self.tail = node;
            self.len += 1;
        }
    }

    /// Removes and returns the node at the back of the list. <br>删除并返回列表后面的节点。<br>
    #[inline]
    fn pop_back_node(&mut self) -> Option<Box<Node<T>>> {
        // This method takes care not to create mutable references to whole nodes, to maintain validity of aliasing pointers into `element`. <br>此方法注意不要对整个节点创建变量引用，以保持 `element` 中的别名指针的有效性。<br>
        //
        self.tail.map(|node| unsafe {
            let node = Box::from_raw(node.as_ptr());
            self.tail = node.prev;

            match self.tail {
                None => self.head = None,
                // Not creating new mutable (unique!) references overlapping `element`. <br>不创建新的可变 (unique!) 引用重叠的 `element`。<br>
                Some(tail) => (*tail.as_ptr()).next = None,
            }

            self.len -= 1;
            node
        })
    }

    /// Unlinks the specified node from the current list. <br>从当前列表中取消指定节点的链接。<br>
    ///
    /// Warning: this will not check that the provided node belongs to the current list. <br>警告：这不会检查提供的节点是否属于当前列表。<br>
    ///
    /// This method takes care not to create mutable references to `element`, to maintain validity of aliasing pointers. <br>此方法注意不要对 `element` 创建变量引用，以保持别名指针的有效性。<br>
    ///
    #[inline]
    unsafe fn unlink_node(&mut self, mut node: NonNull<Node<T>>) {
        let node = unsafe { node.as_mut() }; // this one is ours now, we can create an &mut. <br>现在这是我们的，我们可以创建一个 &mut。<br>

        // Not creating new mutable (unique!) references overlapping `element`. <br>不创建新的可变 (unique!) 引用重叠的 `element`。<br>
        match node.prev {
            Some(prev) => unsafe { (*prev.as_ptr()).next = node.next },
            // this node is the head node <br>这个节点是头节点<br>
            None => self.head = node.next,
        };

        match node.next {
            Some(next) => unsafe { (*next.as_ptr()).prev = node.prev },
            // this node is the tail node <br>这个节点是尾节点<br>
            None => self.tail = node.prev,
        };

        self.len -= 1;
    }

    /// Splices a series of nodes between two existing nodes. <br>在两个现有节点之间拼接一系列节点。<br>
    ///
    /// Warning: this will not check that the provided node belongs to the two existing lists. <br>警告：这不会检查提供的节点是否属于两个现有列表。<br>
    #[inline]
    unsafe fn splice_nodes(
        &mut self,
        existing_prev: Option<NonNull<Node<T>>>,
        existing_next: Option<NonNull<Node<T>>>,
        mut splice_start: NonNull<Node<T>>,
        mut splice_end: NonNull<Node<T>>,
        splice_length: usize,
    ) {
        // This method takes care not to create multiple mutable references to whole nodes at the same time, to maintain validity of aliasing pointers into `element`. <br>此方法注意不要同时对整个节点创建多个变量引用，以保持 `element` 中的别名指针的有效性。<br>
        //
        if let Some(mut existing_prev) = existing_prev {
            unsafe {
                existing_prev.as_mut().next = Some(splice_start);
            }
        } else {
            self.head = Some(splice_start);
        }
        if let Some(mut existing_next) = existing_next {
            unsafe {
                existing_next.as_mut().prev = Some(splice_end);
            }
        } else {
            self.tail = Some(splice_end);
        }
        unsafe {
            splice_start.as_mut().prev = existing_prev;
            splice_end.as_mut().next = existing_next;
        }

        self.len += splice_length;
    }

    /// Detaches all nodes from a linked list as a series of nodes. <br>将一个链表中的所有节点分离为一系列节点。<br>
    #[inline]
    fn detach_all_nodes(mut self) -> Option<(NonNull<Node<T>>, NonNull<Node<T>>, usize)> {
        let head = self.head.take();
        let tail = self.tail.take();
        let len = mem::replace(&mut self.len, 0);
        if let Some(head) = head {
            // SAFETY: In a LinkedList, either both the head and tail are None because the list is empty, or both head and tail are Some because the list is populated. <br>在 LinkedList 中，要么头部和尾部都是 None 因为列表是空的，或者头部和尾部都是 Some 因为列表被填充。<br>
            //
            // Since we have verified the head is Some, we are sure the tail is Some too. <br>由于我们已经验证了头部是 Some，我们确定尾部也是 Some。<br>
            let tail = unsafe { tail.unwrap_unchecked() };
            Some((head, tail, len))
        } else {
            None
        }
    }

    #[inline]
    unsafe fn split_off_before_node(
        &mut self,
        split_node: Option<NonNull<Node<T>>>,
        at: usize,
    ) -> Self {
        // The split node is the new head node of the second part <br>拆分节点是第二部分的新头节点<br>
        if let Some(mut split_node) = split_node {
            let first_part_head;
            let first_part_tail;
            unsafe {
                first_part_tail = split_node.as_mut().prev.take();
            }
            if let Some(mut tail) = first_part_tail {
                unsafe {
                    tail.as_mut().next = None;
                }
                first_part_head = self.head;
            } else {
                first_part_head = None;
            }

            let first_part = LinkedList {
                head: first_part_head,
                tail: first_part_tail,
                len: at,
                marker: PhantomData,
            };

            // Fix the head ptr of the second part <br>修复第二部分的头部 ptr<br>
            self.head = Some(split_node);
            self.len = self.len - at;

            first_part
        } else {
            mem::replace(self, LinkedList::new())
        }
    }

    #[inline]
    unsafe fn split_off_after_node(
        &mut self,
        split_node: Option<NonNull<Node<T>>>,
        at: usize,
    ) -> Self {
        // The split node is the new tail node of the first part and owns the head of the second part. <br>拆分节点是第一部分的新尾节点，并拥有第二部分的头。<br>
        //
        if let Some(mut split_node) = split_node {
            let second_part_head;
            let second_part_tail;
            unsafe {
                second_part_head = split_node.as_mut().next.take();
            }
            if let Some(mut head) = second_part_head {
                unsafe {
                    head.as_mut().prev = None;
                }
                second_part_tail = self.tail;
            } else {
                second_part_tail = None;
            }

            let second_part = LinkedList {
                head: second_part_head,
                tail: second_part_tail,
                len: self.len - at,
                marker: PhantomData,
            };

            // Fix the tail ptr of the first part <br>修复第一部分的尾部点<br>
            self.tail = Some(split_node);
            self.len = at;

            second_part
        } else {
            mem::replace(self, LinkedList::new())
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Default for LinkedList<T> {
    /// Creates an empty `LinkedList<T>`. <br>创建一个空的 `LinkedList<T>`。<br>
    #[inline]
    fn default() -> Self {
        Self::new()
    }
}

impl<T> LinkedList<T> {
    /// Creates an empty `LinkedList`. <br>创建一个空的 `LinkedList`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let list: LinkedList<u32> = LinkedList::new();
    /// ```
    #[inline]
    #[rustc_const_stable(feature = "const_linked_list_new", since = "1.39.0")]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub const fn new() -> Self {
        LinkedList { head: None, tail: None, len: 0, marker: PhantomData }
    }

    /// Moves all elements from `other` to the end of the list. <br>将所有元素从 `other` 移动到列表的末尾。<br>
    ///
    /// This reuses all the nodes from `other` and moves them into `self`. <br>这将重用 `other` 中的所有节点并将它们移到 `self` 中。<br>
    /// After this operation, `other` becomes empty. <br>完成此操作后，`other` 变为空。<br>
    ///
    /// This operation should compute in *O*(1) time and *O*(1) memory. <br>此操作应在 *O*(1) 时间和 *O*(1) 内存中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut list1 = LinkedList::new();
    /// list1.push_back('a');
    ///
    /// let mut list2 = LinkedList::new();
    /// list2.push_back('b');
    /// list2.push_back('c');
    ///
    /// list1.append(&mut list2);
    ///
    /// let mut iter = list1.iter();
    /// assert_eq!(iter.next(), Some(&'a'));
    /// assert_eq!(iter.next(), Some(&'b'));
    /// assert_eq!(iter.next(), Some(&'c'));
    /// assert!(iter.next().is_none());
    ///
    /// assert!(list2.is_empty());
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn append(&mut self, other: &mut Self) {
        match self.tail {
            None => mem::swap(self, other),
            Some(mut tail) => {
                // `as_mut` is okay here because we have exclusive access to the entirety of both lists. <br>`as_mut` 在这里没问题，因为我们可以独占访问两个列表的全部内容。<br>
                //
                if let Some(mut other_head) = other.head.take() {
                    unsafe {
                        tail.as_mut().next = Some(other_head);
                        other_head.as_mut().prev = Some(tail);
                    }

                    self.tail = other.tail.take();
                    self.len += mem::replace(&mut other.len, 0);
                }
            }
        }
    }

    /// Provides a forward iterator. <br>提供一个正向迭代器。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut list: LinkedList<u32> = LinkedList::new();
    ///
    /// list.push_back(0);
    /// list.push_back(1);
    /// list.push_back(2);
    ///
    /// let mut iter = list.iter();
    /// assert_eq!(iter.next(), Some(&0));
    /// assert_eq!(iter.next(), Some(&1));
    /// assert_eq!(iter.next(), Some(&2));
    /// assert_eq!(iter.next(), None);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn iter(&self) -> Iter<'_, T> {
        Iter { head: self.head, tail: self.tail, len: self.len, marker: PhantomData }
    }

    /// Provides a forward iterator with mutable references. <br>提供具有可变引用的正向迭代器。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut list: LinkedList<u32> = LinkedList::new();
    ///
    /// list.push_back(0);
    /// list.push_back(1);
    /// list.push_back(2);
    ///
    /// for element in list.iter_mut() {
    ///     *element += 10;
    /// }
    ///
    /// let mut iter = list.iter();
    /// assert_eq!(iter.next(), Some(&10));
    /// assert_eq!(iter.next(), Some(&11));
    /// assert_eq!(iter.next(), Some(&12));
    /// assert_eq!(iter.next(), None);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn iter_mut(&mut self) -> IterMut<'_, T> {
        IterMut { head: self.head, tail: self.tail, len: self.len, marker: PhantomData }
    }

    /// Provides a cursor at the front element. <br>在前元素处提供游标。<br>
    ///
    /// The cursor is pointing to the "ghost" non-element if the list is empty. <br>如果列表为空，则游标指向 "ghost" 非元素。<br>
    #[inline]
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn cursor_front(&self) -> Cursor<'_, T> {
        Cursor { index: 0, current: self.head, list: self }
    }

    /// Provides a cursor with editing operations at the front element. <br>在前面的元素上为游标提供编辑操作。<br>
    ///
    /// The cursor is pointing to the "ghost" non-element if the list is empty. <br>如果列表为空，则游标指向 "ghost" 非元素。<br>
    #[inline]
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn cursor_front_mut(&mut self) -> CursorMut<'_, T> {
        CursorMut { index: 0, current: self.head, list: self }
    }

    /// Provides a cursor at the back element. <br>在 back 元素上提供游标。<br>
    ///
    /// The cursor is pointing to the "ghost" non-element if the list is empty. <br>如果列表为空，则游标指向 "ghost" 非元素。<br>
    #[inline]
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn cursor_back(&self) -> Cursor<'_, T> {
        Cursor { index: self.len.checked_sub(1).unwrap_or(0), current: self.tail, list: self }
    }

    /// Provides a cursor with editing operations at the back element. <br>在 back 元素上为游标提供编辑操作。<br>
    ///
    /// The cursor is pointing to the "ghost" non-element if the list is empty. <br>如果列表为空，则游标指向 "ghost" 非元素。<br>
    #[inline]
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn cursor_back_mut(&mut self) -> CursorMut<'_, T> {
        CursorMut { index: self.len.checked_sub(1).unwrap_or(0), current: self.tail, list: self }
    }

    /// Returns `true` if the `LinkedList` is empty. <br>如果 `LinkedList` 为空，则返回 `true`。<br>
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut dl = LinkedList::new();
    /// assert!(dl.is_empty());
    ///
    /// dl.push_front("foo");
    /// assert!(!dl.is_empty());
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn is_empty(&self) -> bool {
        self.head.is_none()
    }

    /// Returns the length of the `LinkedList`. <br>返回 `LinkedList` 的长度。<br>
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut dl = LinkedList::new();
    ///
    /// dl.push_front(2);
    /// assert_eq!(dl.len(), 1);
    ///
    /// dl.push_front(1);
    /// assert_eq!(dl.len(), 2);
    ///
    /// dl.push_back(3);
    /// assert_eq!(dl.len(), 3);
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn len(&self) -> usize {
        self.len
    }

    /// Removes all elements from the `LinkedList`. <br>从 `LinkedList` 删除所有元素。<br>
    ///
    /// This operation should compute in *O*(*n*) time. <br>此运算应在 *O*(*n*) 时间中计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut dl = LinkedList::new();
    ///
    /// dl.push_front(2);
    /// dl.push_front(1);
    /// assert_eq!(dl.len(), 2);
    /// assert_eq!(dl.front(), Some(&1));
    ///
    /// dl.clear();
    /// assert_eq!(dl.len(), 0);
    /// assert_eq!(dl.front(), None);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn clear(&mut self) {
        *self = Self::new();
    }

    /// Returns `true` if the `LinkedList` contains an element equal to the given value. <br>如果 `LinkedList` 包含等于给定值的元素，则返回 `true`。<br>
    ///
    ///
    /// This operation should compute linearly in *O*(*n*) time. <br>此操作应在 *O*(*n*) 时间内线性计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut list: LinkedList<u32> = LinkedList::new();
    ///
    /// list.push_back(0);
    /// list.push_back(1);
    /// list.push_back(2);
    ///
    /// assert_eq!(list.contains(&0), true);
    /// assert_eq!(list.contains(&10), false);
    /// ```
    #[stable(feature = "linked_list_contains", since = "1.12.0")]
    pub fn contains(&self, x: &T) -> bool
    where
        T: PartialEq<T>,
    {
        self.iter().any(|e| e == x)
    }

    /// Provides a reference to the front element, or `None` if the list is empty. <br>提供对前元素的引用，如果列表为空，则为 `None`。<br>
    ///
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut dl = LinkedList::new();
    /// assert_eq!(dl.front(), None);
    ///
    /// dl.push_front(1);
    /// assert_eq!(dl.front(), Some(&1));
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn front(&self) -> Option<&T> {
        unsafe { self.head.as_ref().map(|node| &node.as_ref().element) }
    }

    /// Provides a mutable reference to the front element, or `None` if the list is empty. <br>提供对前元素的可变引用，如果列表为空，则为 `None`。<br>
    ///
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut dl = LinkedList::new();
    /// assert_eq!(dl.front(), None);
    ///
    /// dl.push_front(1);
    /// assert_eq!(dl.front(), Some(&1));
    ///
    /// match dl.front_mut() {
    ///     None => {},
    ///     Some(x) => *x = 5,
    /// }
    /// assert_eq!(dl.front(), Some(&5));
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn front_mut(&mut self) -> Option<&mut T> {
        unsafe { self.head.as_mut().map(|node| &mut node.as_mut().element) }
    }

    /// Provides a reference to the back element, or `None` if the list is empty. <br>提供对 back 元素的引用，如果列表为空，则提供 `None`。<br>
    ///
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut dl = LinkedList::new();
    /// assert_eq!(dl.back(), None);
    ///
    /// dl.push_back(1);
    /// assert_eq!(dl.back(), Some(&1));
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn back(&self) -> Option<&T> {
        unsafe { self.tail.as_ref().map(|node| &node.as_ref().element) }
    }

    /// Provides a mutable reference to the back element, or `None` if the list is empty. <br>提供对 back 元素的可变引用，如果列表为空，则为 `None`。<br>
    ///
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut dl = LinkedList::new();
    /// assert_eq!(dl.back(), None);
    ///
    /// dl.push_back(1);
    /// assert_eq!(dl.back(), Some(&1));
    ///
    /// match dl.back_mut() {
    ///     None => {},
    ///     Some(x) => *x = 5,
    /// }
    /// assert_eq!(dl.back(), Some(&5));
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn back_mut(&mut self) -> Option<&mut T> {
        unsafe { self.tail.as_mut().map(|node| &mut node.as_mut().element) }
    }

    /// Adds an element first in the list. <br>首先在列表中添加一个元素。<br>
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut dl = LinkedList::new();
    ///
    /// dl.push_front(2);
    /// assert_eq!(dl.front().unwrap(), &2);
    ///
    /// dl.push_front(1);
    /// assert_eq!(dl.front().unwrap(), &1);
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn push_front(&mut self, elt: T) {
        self.push_front_node(Box::new(Node::new(elt)));
    }

    /// Removes the first element and returns it, or `None` if the list is empty. <br>删除第一个元素并返回它; 如果列表为空，则返回 `None`。<br>
    ///
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut d = LinkedList::new();
    /// assert_eq!(d.pop_front(), None);
    ///
    /// d.push_front(1);
    /// d.push_front(3);
    /// assert_eq!(d.pop_front(), Some(3));
    /// assert_eq!(d.pop_front(), Some(1));
    /// assert_eq!(d.pop_front(), None);
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn pop_front(&mut self) -> Option<T> {
        self.pop_front_node().map(Node::into_element)
    }

    /// Appends an element to the back of a list. <br>将元素追加到列表的后面。<br>
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut d = LinkedList::new();
    /// d.push_back(1);
    /// d.push_back(3);
    /// assert_eq!(3, *d.back().unwrap());
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn push_back(&mut self, elt: T) {
        self.push_back_node(Box::new(Node::new(elt)));
    }

    /// Removes the last element from a list and returns it, or `None` if it is empty. <br>从列表中删除最后一个元素并返回它; 如果它为空，则返回 `None`。<br>
    ///
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut d = LinkedList::new();
    /// assert_eq!(d.pop_back(), None);
    /// d.push_back(1);
    /// d.push_back(3);
    /// assert_eq!(d.pop_back(), Some(3));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn pop_back(&mut self) -> Option<T> {
        self.pop_back_node().map(Node::into_element)
    }

    /// Splits the list into two at the given index. <br>在给定的索引处将列表分为两部分。<br>
    /// Returns everything after the given index, including the index. <br>返回给定索引之后的所有内容，包括索引。<br>
    ///
    /// This operation should compute in *O*(*n*) time. <br>此运算应在 *O*(*n*) 时间中计算。<br>
    ///
    /// # Panics
    ///
    /// Panics if `at > len`. <br>如果为 `at > len`，就会出现 panics。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let mut d = LinkedList::new();
    ///
    /// d.push_front(1);
    /// d.push_front(2);
    /// d.push_front(3);
    ///
    /// let mut split = d.split_off(2);
    ///
    /// assert_eq!(split.pop_front(), Some(1));
    /// assert_eq!(split.pop_front(), None);
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn split_off(&mut self, at: usize) -> LinkedList<T> {
        let len = self.len();
        assert!(at <= len, "Cannot split off at a nonexistent index");
        if at == 0 {
            return mem::take(self);
        } else if at == len {
            return Self::new();
        }

        // Below, we iterate towards the `i-1`th node, either from the start or the end, depending on which would be faster. <br>在下面，我们从头到尾迭代第 i-1 个节点，具体取决于哪个会更快。<br>
        //
        let split_node = if at - 1 <= len - 1 - (at - 1) {
            let mut iter = self.iter_mut();
            // instead of skipping using .skip() (which creates a new struct), we skip manually so we can access the head field without depending on implementation details of Skip <br>无需跳过使用 .skip() (它会创建一个新的结构体) 的方法，而是手动跳过，因此我们可以访问 head 字段，而无需依赖于 Skip 的实现细节<br>
            //
            //
            for _ in 0..at - 1 {
                iter.next();
            }
            iter.head
        } else {
            // better off starting from the end <br>从头开始更好<br>
            let mut iter = self.iter_mut();
            for _ in 0..len - 1 - (at - 1) {
                iter.next_back();
            }
            iter.tail
        };
        unsafe { self.split_off_after_node(split_node, at) }
    }

    /// Removes the element at the given index and returns it. <br>删除给定索引处的元素并返回它。<br>
    ///
    /// This operation should compute in *O*(*n*) time. <br>此运算应在 *O*(*n*) 时间中计算。<br>
    ///
    /// # Panics
    /// Panics if at >= len <br>如果 >= len 就会出现 panics<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(linked_list_remove)]
    /// use std::collections::LinkedList;
    ///
    /// let mut d = LinkedList::new();
    ///
    /// d.push_front(1);
    /// d.push_front(2);
    /// d.push_front(3);
    ///
    /// assert_eq!(d.remove(1), 2);
    /// assert_eq!(d.remove(0), 3);
    /// assert_eq!(d.remove(0), 1);
    /// ```
    #[unstable(feature = "linked_list_remove", issue = "69210")]
    pub fn remove(&mut self, at: usize) -> T {
        let len = self.len();
        assert!(at < len, "Cannot remove at an index outside of the list bounds");

        // Below, we iterate towards the node at the given index, either from the start or the end, depending on which would be faster. <br>下面，我们从头到尾迭代给定索引处的节点，具体取决于哪个会更快。<br>
        //
        let offset_from_end = len - at - 1;
        if at <= offset_from_end {
            let mut cursor = self.cursor_front_mut();
            for _ in 0..at {
                cursor.move_next();
            }
            cursor.remove_current().unwrap()
        } else {
            let mut cursor = self.cursor_back_mut();
            for _ in 0..offset_from_end {
                cursor.move_prev();
            }
            cursor.remove_current().unwrap()
        }
    }

    /// Creates an iterator which uses a closure to determine if an element should be removed. <br>创建一个迭代器，该迭代器使用闭包确定是否应删除元素。<br>
    ///
    /// If the closure returns true, then the element is removed and yielded. <br>如果闭包返回 true，则删除并生成元素。<br>
    /// If the closure returns false, the element will remain in the list and will not be yielded by the iterator. <br>如果闭包返回 false，则该元素将保留在列表中，并且不会由迭代器产生。<br>
    ///
    /// Note that `drain_filter` lets you mutate every element in the filter closure, regardless of whether you choose to keep or remove it. <br>请注意，无论选择保留还是删除 `drain_filter`，您都可以对过滤器闭包中的每个元素进行可变的。<br>
    ///
    ///
    /// # Examples
    ///
    /// Splitting a list into evens and odds, reusing the original list: <br>将列表分成偶数和几率，重新使用原始列表：<br>
    ///
    /// ```
    /// #![feature(drain_filter)]
    /// use std::collections::LinkedList;
    ///
    /// let mut numbers: LinkedList<u32> = LinkedList::new();
    /// numbers.extend(&[1, 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, 15]);
    ///
    /// let evens = numbers.drain_filter(|x| *x % 2 == 0).collect::<LinkedList<_>>();
    /// let odds = numbers;
    ///
    /// assert_eq!(evens.into_iter().collect::<Vec<_>>(), vec![2, 4, 6, 8, 14]);
    /// assert_eq!(odds.into_iter().collect::<Vec<_>>(), vec![1, 3, 5, 9, 11, 13, 15]);
    /// ```
    ///
    #[unstable(feature = "drain_filter", reason = "recently added", issue = "43244")]
    pub fn drain_filter<F>(&mut self, filter: F) -> DrainFilter<'_, T, F>
    where
        F: FnMut(&mut T) -> bool,
    {
        // avoid borrow issues. <br>避免借用问题。<br>
        let it = self.head;
        let old_len = self.len;

        DrainFilter { list: self, it, pred: filter, idx: 0, old_len }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<#[may_dangle] T> Drop for LinkedList<T> {
    fn drop(&mut self) {
        struct DropGuard<'a, T>(&'a mut LinkedList<T>);

        impl<'a, T> Drop for DropGuard<'a, T> {
            fn drop(&mut self) {
                // Continue the same loop we do below. <br>继续执行以下相同的循环。<br> This only runs when a destructor has panicked. <br>这仅在析构函数崩溃时运行。<br>
                // If another one panics this will abort. <br>如果另一个 panics 将终止。<br>
                while self.0.pop_front_node().is_some() {}
            }
        }

        while let Some(node) = self.pop_front_node() {
            let guard = DropGuard(self);
            drop(node);
            mem::forget(guard);
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, T> Iterator for Iter<'a, T> {
    type Item = &'a T;

    #[inline]
    fn next(&mut self) -> Option<&'a T> {
        if self.len == 0 {
            None
        } else {
            self.head.map(|node| unsafe {
                // Need an unbound lifetime to get 'a <br>需要无限的生命周期来获得<br>
                let node = &*node.as_ptr();
                self.len -= 1;
                self.head = node.next;
                &node.element
            })
        }
    }

    #[inline]
    fn size_hint(&self) -> (usize, Option<usize>) {
        (self.len, Some(self.len))
    }

    #[inline]
    fn last(mut self) -> Option<&'a T> {
        self.next_back()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, T> DoubleEndedIterator for Iter<'a, T> {
    #[inline]
    fn next_back(&mut self) -> Option<&'a T> {
        if self.len == 0 {
            None
        } else {
            self.tail.map(|node| unsafe {
                // Need an unbound lifetime to get 'a <br>需要无限的生命周期来获得<br>
                let node = &*node.as_ptr();
                self.len -= 1;
                self.tail = node.prev;
                &node.element
            })
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> ExactSizeIterator for Iter<'_, T> {}

#[stable(feature = "fused", since = "1.26.0")]
impl<T> FusedIterator for Iter<'_, T> {}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, T> Iterator for IterMut<'a, T> {
    type Item = &'a mut T;

    #[inline]
    fn next(&mut self) -> Option<&'a mut T> {
        if self.len == 0 {
            None
        } else {
            self.head.map(|node| unsafe {
                // Need an unbound lifetime to get 'a <br>需要无限的生命周期来获得<br>
                let node = &mut *node.as_ptr();
                self.len -= 1;
                self.head = node.next;
                &mut node.element
            })
        }
    }

    #[inline]
    fn size_hint(&self) -> (usize, Option<usize>) {
        (self.len, Some(self.len))
    }

    #[inline]
    fn last(mut self) -> Option<&'a mut T> {
        self.next_back()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, T> DoubleEndedIterator for IterMut<'a, T> {
    #[inline]
    fn next_back(&mut self) -> Option<&'a mut T> {
        if self.len == 0 {
            None
        } else {
            self.tail.map(|node| unsafe {
                // Need an unbound lifetime to get 'a <br>需要无限的生命周期来获得<br>
                let node = &mut *node.as_ptr();
                self.len -= 1;
                self.tail = node.prev;
                &mut node.element
            })
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> ExactSizeIterator for IterMut<'_, T> {}

#[stable(feature = "fused", since = "1.26.0")]
impl<T> FusedIterator for IterMut<'_, T> {}

/// A cursor over a `LinkedList`. <br>`LinkedList` 上的游标。<br>
///
/// A `Cursor` is like an iterator, except that it can freely seek back-and-forth. <br>`Cursor` 类似于迭代器，不同之处在于它可以自由地来回查找。<br>
///
/// Cursors always rest between two elements in the list, and index in a logically circular way. <br>游标始终位于列表中的两个元素之间，并以逻辑循环的方式进行索引。<br>
/// To accommodate this, there is a "ghost" non-element that yields `None` between the head and tail of the list. <br>为了适应这一点，有一个 "ghost" 非元素在列表的开头和结尾之间产生 `None`。<br>
///
///
/// When created, cursors start at the front of the list, or the "ghost" non-element if the list is empty. <br>创建后，游标从列表的开头开始，如果列表为空，则从 "ghost" 非元素开始。<br>
#[unstable(feature = "linked_list_cursors", issue = "58533")]
pub struct Cursor<'a, T: 'a> {
    index: usize,
    current: Option<NonNull<Node<T>>>,
    list: &'a LinkedList<T>,
}

#[unstable(feature = "linked_list_cursors", issue = "58533")]
impl<T> Clone for Cursor<'_, T> {
    fn clone(&self) -> Self {
        let Cursor { index, current, list } = *self;
        Cursor { index, current, list }
    }
}

#[unstable(feature = "linked_list_cursors", issue = "58533")]
impl<T: fmt::Debug> fmt::Debug for Cursor<'_, T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple("Cursor").field(&self.list).field(&self.index()).finish()
    }
}

/// A cursor over a `LinkedList` with editing operations. <br>带有编辑操作的 `LinkedList` 上的游标。<br>
///
/// A `Cursor` is like an iterator, except that it can freely seek back-and-forth, and can safely mutate the list during iteration. <br>`Cursor` 类似于迭代器，不同之处在于它可以自由地来回查找，并且可以在迭代过程中安全地修改列表。<br>
/// This is because the lifetime of its yielded references is tied to its own lifetime, instead of just the underlying list. <br>这是因为其产生的引用的生命周期与其自身的生命周期相关联，而不仅仅是底层列表。<br>
/// This means cursors cannot yield multiple elements at once. <br>这意味着游标不能一次产生多个元素。<br>
///
/// Cursors always rest between two elements in the list, and index in a logically circular way. <br>游标始终位于列表中的两个元素之间，并以逻辑循环的方式进行索引。<br>
/// To accommodate this, there is a "ghost" non-element that yields `None` between the head and tail of the list. <br>为了适应这一点，有一个 "ghost" 非元素在列表的开头和结尾之间产生 `None`。<br>
///
///
#[unstable(feature = "linked_list_cursors", issue = "58533")]
pub struct CursorMut<'a, T: 'a> {
    index: usize,
    current: Option<NonNull<Node<T>>>,
    list: &'a mut LinkedList<T>,
}

#[unstable(feature = "linked_list_cursors", issue = "58533")]
impl<T: fmt::Debug> fmt::Debug for CursorMut<'_, T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple("CursorMut").field(&self.list).field(&self.index()).finish()
    }
}

impl<'a, T> Cursor<'a, T> {
    /// Returns the cursor position index within the `LinkedList`. <br>返回 `LinkedList` 中的游标位置索引。<br>
    ///
    /// This returns `None` if the cursor is currently pointing to the "ghost" non-element. <br>如果游标当前指向 "ghost" 非元素，则返回 `None`。<br>
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn index(&self) -> Option<usize> {
        let _ = self.current?;
        Some(self.index)
    }

    /// Moves the cursor to the next element of the `LinkedList`. <br>将游标移动到 `LinkedList` 的下一个元素。<br>
    ///
    /// If the cursor is pointing to the "ghost" non-element then this will move it to the first element of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，那么它将移动到 `LinkedList` 的第一个元素。<br>
    /// If it is pointing to the last element of the `LinkedList` then this will move it to the "ghost" non-element. <br>如果它指向 `LinkedList` 的最后一个元素，那么它将把它移到 "ghost" 非元素。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn move_next(&mut self) {
        match self.current.take() {
            // We had no current element; <br>我们没有当前的要素；<br> the cursor was sitting at the start position Next element should be the head of the list <br>游标位于开始位置，下一个元素应位于列表的开头<br>
            //
            None => {
                self.current = self.list.head;
                self.index = 0;
            }
            // We had a previous element, so let's go to its next <br>我们有一个上一个元素，所以让我们转到下一个元素<br>
            Some(current) => unsafe {
                self.current = current.as_ref().next;
                self.index += 1;
            },
        }
    }

    /// Moves the cursor to the previous element of the `LinkedList`. <br>将游标移动到 `LinkedList` 的上一个元素。<br>
    ///
    /// If the cursor is pointing to the "ghost" non-element then this will move it to the last element of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，那么它将移动到 `LinkedList` 的最后一个元素。<br>
    /// If it is pointing to the first element of the `LinkedList` then this will move it to the "ghost" non-element. <br>如果它指向 `LinkedList` 的第一个元素，那么它将把它移到 "ghost" 非元素。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn move_prev(&mut self) {
        match self.current.take() {
            // No current. <br>不是 current。<br> We're at the start of the list. <br>我们位于列表的开头。<br> Yield None and jump to the end. <br>Yield None 并跳到最后。<br>
            None => {
                self.current = self.list.tail;
                self.index = self.list.len().checked_sub(1).unwrap_or(0);
            }
            // Have a prev. <br>有上一个。<br> Yield it and go to the previous element. <br>Yield 并转到上一个元素。<br>
            Some(current) => unsafe {
                self.current = current.as_ref().prev;
                self.index = self.index.checked_sub(1).unwrap_or_else(|| self.list.len());
            },
        }
    }

    /// Returns a reference to the element that the cursor is currently pointing to. <br>返回对游标当前指向的元素的引用。<br>
    ///
    /// This returns `None` if the cursor is currently pointing to the "ghost" non-element. <br>如果游标当前指向 "ghost" 非元素，则返回 `None`。<br>
    ///
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn current(&self) -> Option<&'a T> {
        unsafe { self.current.map(|current| &(*current.as_ptr()).element) }
    }

    /// Returns a reference to the next element. <br>返回下一个元素的引用。<br>
    ///
    /// If the cursor is pointing to the "ghost" non-element then this returns the first element of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，则返回 `LinkedList` 的第一个元素。<br>
    /// If it is pointing to the last element of the `LinkedList` then this returns `None`. <br>如果它指向 `LinkedList` 的最后一个元素，则返回 `None`。<br>
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn peek_next(&self) -> Option<&'a T> {
        unsafe {
            let next = match self.current {
                None => self.list.head,
                Some(current) => current.as_ref().next,
            };
            next.map(|next| &(*next.as_ptr()).element)
        }
    }

    /// Returns a reference to the previous element. <br>返回上一个元素的引用。<br>
    ///
    /// If the cursor is pointing to the "ghost" non-element then this returns the last element of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，则返回 `LinkedList` 的最后一个元素。<br>
    /// If it is pointing to the first element of the `LinkedList` then this returns `None`. <br>如果它指向 `LinkedList` 的第一个元素，则返回 `None`。<br>
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn peek_prev(&self) -> Option<&'a T> {
        unsafe {
            let prev = match self.current {
                None => self.list.tail,
                Some(current) => current.as_ref().prev,
            };
            prev.map(|prev| &(*prev.as_ptr()).element)
        }
    }

    /// Provides a reference to the front element of the cursor's parent list, or None if the list is empty. <br>提供对游标父列表前元素的引用，如果列表为空，则为 None。<br>
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn front(&self) -> Option<&'a T> {
        self.list.front()
    }

    /// Provides a reference to the back element of the cursor's parent list, or None if the list is empty. <br>提供对游标父列表的后部元素的引用，如果列表为空，则为 None。<br>
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn back(&self) -> Option<&'a T> {
        self.list.back()
    }
}

impl<'a, T> CursorMut<'a, T> {
    /// Returns the cursor position index within the `LinkedList`. <br>返回 `LinkedList` 中的游标位置索引。<br>
    ///
    /// This returns `None` if the cursor is currently pointing to the "ghost" non-element. <br>如果游标当前指向 "ghost" 非元素，则返回 `None`。<br>
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn index(&self) -> Option<usize> {
        let _ = self.current?;
        Some(self.index)
    }

    /// Moves the cursor to the next element of the `LinkedList`. <br>将游标移动到 `LinkedList` 的下一个元素。<br>
    ///
    /// If the cursor is pointing to the "ghost" non-element then this will move it to the first element of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，那么它将移动到 `LinkedList` 的第一个元素。<br>
    /// If it is pointing to the last element of the `LinkedList` then this will move it to the "ghost" non-element. <br>如果它指向 `LinkedList` 的最后一个元素，那么它将把它移到 "ghost" 非元素。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn move_next(&mut self) {
        match self.current.take() {
            // We had no current element; <br>我们没有当前的要素；<br> the cursor was sitting at the start position Next element should be the head of the list <br>游标位于开始位置，下一个元素应位于列表的开头<br>
            //
            None => {
                self.current = self.list.head;
                self.index = 0;
            }
            // We had a previous element, so let's go to its next <br>我们有一个上一个元素，所以让我们转到下一个元素<br>
            Some(current) => unsafe {
                self.current = current.as_ref().next;
                self.index += 1;
            },
        }
    }

    /// Moves the cursor to the previous element of the `LinkedList`. <br>将游标移动到 `LinkedList` 的上一个元素。<br>
    ///
    /// If the cursor is pointing to the "ghost" non-element then this will move it to the last element of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，那么它将移动到 `LinkedList` 的最后一个元素。<br>
    /// If it is pointing to the first element of the `LinkedList` then this will move it to the "ghost" non-element. <br>如果它指向 `LinkedList` 的第一个元素，那么它将把它移到 "ghost" 非元素。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn move_prev(&mut self) {
        match self.current.take() {
            // No current. <br>不是 current。<br> We're at the start of the list. <br>我们位于列表的开头。<br> Yield None and jump to the end. <br>Yield None 并跳到最后。<br>
            None => {
                self.current = self.list.tail;
                self.index = self.list.len().checked_sub(1).unwrap_or(0);
            }
            // Have a prev. <br>有上一个。<br> Yield it and go to the previous element. <br>Yield 并转到上一个元素。<br>
            Some(current) => unsafe {
                self.current = current.as_ref().prev;
                self.index = self.index.checked_sub(1).unwrap_or_else(|| self.list.len());
            },
        }
    }

    /// Returns a reference to the element that the cursor is currently pointing to. <br>返回对游标当前指向的元素的引用。<br>
    ///
    /// This returns `None` if the cursor is currently pointing to the "ghost" non-element. <br>如果游标当前指向 "ghost" 非元素，则返回 `None`。<br>
    ///
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn current(&mut self) -> Option<&mut T> {
        unsafe { self.current.map(|current| &mut (*current.as_ptr()).element) }
    }

    /// Returns a reference to the next element. <br>返回下一个元素的引用。<br>
    ///
    /// If the cursor is pointing to the "ghost" non-element then this returns the first element of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，则返回 `LinkedList` 的第一个元素。<br>
    /// If it is pointing to the last element of the `LinkedList` then this returns `None`. <br>如果它指向 `LinkedList` 的最后一个元素，则返回 `None`。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn peek_next(&mut self) -> Option<&mut T> {
        unsafe {
            let next = match self.current {
                None => self.list.head,
                Some(current) => current.as_ref().next,
            };
            next.map(|next| &mut (*next.as_ptr()).element)
        }
    }

    /// Returns a reference to the previous element. <br>返回上一个元素的引用。<br>
    ///
    /// If the cursor is pointing to the "ghost" non-element then this returns the last element of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，则返回 `LinkedList` 的最后一个元素。<br>
    /// If it is pointing to the first element of the `LinkedList` then this returns `None`. <br>如果它指向 `LinkedList` 的第一个元素，则返回 `None`。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn peek_prev(&mut self) -> Option<&mut T> {
        unsafe {
            let prev = match self.current {
                None => self.list.tail,
                Some(current) => current.as_ref().prev,
            };
            prev.map(|prev| &mut (*prev.as_ptr()).element)
        }
    }

    /// Returns a read-only cursor pointing to the current element. <br>返回指向当前元素的只读游标。<br>
    ///
    /// The lifetime of the returned `Cursor` is bound to that of the `CursorMut`, which means it cannot outlive the `CursorMut` and that the `CursorMut` is frozen for the lifetime of the `Cursor`. <br>返回的 `Cursor` 的生命周期与 `CursorMut` 的生命周期绑定在一起，这意味着它不能超过 `CursorMut` 的生命周期，并且 `CursorMut` 被冻结为 `Cursor` 的生命周期。<br>
    ///
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn as_cursor(&self) -> Cursor<'_, T> {
        Cursor { list: self.list, current: self.current, index: self.index }
    }
}

// Now the list editing operations <br>现在列表编辑操作<br>

impl<'a, T> CursorMut<'a, T> {
    /// Inserts a new element into the `LinkedList` after the current one. <br>在当前元素之后将新元素插入 `LinkedList`。<br>
    ///
    /// If the cursor is pointing at the "ghost" non-element then the new element is inserted at the front of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，则将新元素插入 `LinkedList` 的前面。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn insert_after(&mut self, item: T) {
        unsafe {
            let spliced_node = Box::leak(Box::new(Node::new(item))).into();
            let node_next = match self.current {
                None => self.list.head,
                Some(node) => node.as_ref().next,
            };
            self.list.splice_nodes(self.current, node_next, spliced_node, spliced_node, 1);
            if self.current.is_none() {
                // The "ghost" non-element's index has changed. <br>"ghost" 非元素的索引已更改。<br>
                self.index = self.list.len;
            }
        }
    }

    /// Inserts a new element into the `LinkedList` before the current one. <br>在当前元素之前在 `LinkedList` 中插入一个新元素。<br>
    ///
    /// If the cursor is pointing at the "ghost" non-element then the new element is inserted at the end of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，则将新元素插入 `LinkedList` 的末尾。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn insert_before(&mut self, item: T) {
        unsafe {
            let spliced_node = Box::leak(Box::new(Node::new(item))).into();
            let node_prev = match self.current {
                None => self.list.tail,
                Some(node) => node.as_ref().prev,
            };
            self.list.splice_nodes(node_prev, self.current, spliced_node, spliced_node, 1);
            self.index += 1;
        }
    }

    /// Removes the current element from the `LinkedList`. <br>从 `LinkedList` 中删除当前元素。<br>
    ///
    /// The element that was removed is returned, and the cursor is moved to point to the next element in the `LinkedList`. <br>返回已删除的元素，并移动游标以指向 `LinkedList` 中的下一个元素。<br>
    ///
    ///
    /// If the cursor is currently pointing to the "ghost" non-element then no element is removed and `None` is returned. <br>如果游标当前指向 "ghost" 非元素，则不删除任何元素，并返回 `None`。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn remove_current(&mut self) -> Option<T> {
        let unlinked_node = self.current?;
        unsafe {
            self.current = unlinked_node.as_ref().next;
            self.list.unlink_node(unlinked_node);
            let unlinked_node = Box::from_raw(unlinked_node.as_ptr());
            Some(unlinked_node.element)
        }
    }

    /// Removes the current element from the `LinkedList` without deallocating the list node. <br>在不释放列表节点的情况下从 `LinkedList` 中删除当前元素。<br>
    ///
    /// The node that was removed is returned as a new `LinkedList` containing only this node. <br>被删除的节点作为仅包含该节点的新 `LinkedList` 返回。<br>
    /// The cursor is moved to point to the next element in the current `LinkedList`. <br>游标将移至当前 `LinkedList` 中的下一个元素。<br>
    ///
    /// If the cursor is currently pointing to the "ghost" non-element then no element is removed and `None` is returned. <br>如果游标当前指向 "ghost" 非元素，则不删除任何元素，并返回 `None`。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn remove_current_as_list(&mut self) -> Option<LinkedList<T>> {
        let mut unlinked_node = self.current?;
        unsafe {
            self.current = unlinked_node.as_ref().next;
            self.list.unlink_node(unlinked_node);

            unlinked_node.as_mut().prev = None;
            unlinked_node.as_mut().next = None;
            Some(LinkedList {
                head: Some(unlinked_node),
                tail: Some(unlinked_node),
                len: 1,
                marker: PhantomData,
            })
        }
    }

    /// Inserts the elements from the given `LinkedList` after the current one. <br>将给定 `LinkedList` 中的元素插入当前元素之后。<br>
    ///
    /// If the cursor is pointing at the "ghost" non-element then the new elements are inserted at the start of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，则新元素将插入 `LinkedList` 的开头。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn splice_after(&mut self, list: LinkedList<T>) {
        unsafe {
            let (splice_head, splice_tail, splice_len) = match list.detach_all_nodes() {
                Some(parts) => parts,
                _ => return,
            };
            let node_next = match self.current {
                None => self.list.head,
                Some(node) => node.as_ref().next,
            };
            self.list.splice_nodes(self.current, node_next, splice_head, splice_tail, splice_len);
            if self.current.is_none() {
                // The "ghost" non-element's index has changed. <br>"ghost" 非元素的索引已更改。<br>
                self.index = self.list.len;
            }
        }
    }

    /// Inserts the elements from the given `LinkedList` before the current one. <br>将给定 `LinkedList` 中的元素插入到当前元素之前。<br>
    ///
    /// If the cursor is pointing at the "ghost" non-element then the new elements are inserted at the end of the `LinkedList`. <br>如果游标指向 "ghost" 非元素，则新元素将插入 `LinkedList` 的末尾。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn splice_before(&mut self, list: LinkedList<T>) {
        unsafe {
            let (splice_head, splice_tail, splice_len) = match list.detach_all_nodes() {
                Some(parts) => parts,
                _ => return,
            };
            let node_prev = match self.current {
                None => self.list.tail,
                Some(node) => node.as_ref().prev,
            };
            self.list.splice_nodes(node_prev, self.current, splice_head, splice_tail, splice_len);
            self.index += splice_len;
        }
    }

    /// Splits the list into two after the current element. <br>在当前元素之后将列表分为两部分。<br>
    /// This will return a new list consisting of everything after the cursor, with the original list retaining everything before. <br>这将返回一个新列表，其中包含游标之后的所有内容，而原始列表将保留之前的所有内容。<br>
    ///
    ///
    /// If the cursor is pointing at the "ghost" non-element then the entire contents of the `LinkedList` are moved. <br>如果游标指向 "ghost" 非元素，那么将移动 `LinkedList` 的全部内容。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn split_after(&mut self) -> LinkedList<T> {
        let split_off_idx = if self.index == self.list.len { 0 } else { self.index + 1 };
        if self.index == self.list.len {
            // The "ghost" non-element's index has changed to 0. <br>"ghost" 非元素的索引已更改为 0。<br>
            self.index = 0;
        }
        unsafe { self.list.split_off_after_node(self.current, split_off_idx) }
    }

    /// Splits the list into two before the current element. <br>在当前元素之前将列表分为两部分。<br>
    /// This will return a new list consisting of everything before the cursor, with the original list retaining everything after. <br>这将返回一个新列表，该列表包含游标之前的所有内容，而原始列表保留之后的所有内容。<br>
    ///
    ///
    /// If the cursor is pointing at the "ghost" non-element then the entire contents of the `LinkedList` are moved. <br>如果游标指向 "ghost" 非元素，那么将移动 `LinkedList` 的全部内容。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn split_before(&mut self) -> LinkedList<T> {
        let split_off_idx = self.index;
        self.index = 0;
        unsafe { self.list.split_off_before_node(self.current, split_off_idx) }
    }

    /// Appends an element to the front of the cursor's parent list. <br>将一个元素追加到游标的父列表的前面。<br>
    /// The node that the cursor points to is unchanged, even if it is the "ghost" node. <br>游标指向的节点不变，即使是 "ghost" 节点。<br>
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    // `push_front` continues to point to "ghost" when it addes a node to mimic the behavior of `insert_before` on an empty list. <br>当 `push_front` 添加一个节点来模拟 `insert_before` 在空列表上的行为时，它继续指向 "ghost"。<br>
    //
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn push_front(&mut self, elt: T) {
        // Safety: We know that `push_front` does not change the position in memory of other nodes. <br>安全性：我们知道 `push_front` 不会改变其他节点在内存中的位置。<br>
        // This ensures that `self.current` remains valid. <br>这确保 `self.current` 保持有效。<br>
        //
        self.list.push_front(elt);
        self.index += 1;
    }

    /// Appends an element to the back of the cursor's parent list. <br>将一个元素追加到游标父列表的后面。<br>
    /// The node that the cursor points to is unchanged, even if it is the "ghost" node. <br>游标指向的节点不变，即使是 "ghost" 节点。<br>
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn push_back(&mut self, elt: T) {
        // Safety: We know that `push_back` does not change the position in memory of other nodes. <br>安全性：我们知道 `push_back` 不会改变其他节点在内存中的位置。<br>
        // This ensures that `self.current` remains valid. <br>这确保 `self.current` 保持有效。<br>
        //
        self.list.push_back(elt);
        if self.current().is_none() {
            // The index of "ghost" is the length of the list, so we just need to increment self.index to reflect the new length of the list. <br>"ghost" 的索引是列表的长度，所以我们只需要增加 self.index 来反映列表的新长度。<br>
            //
            self.index += 1;
        }
    }

    /// Removes the first element from the cursor's parent list and returns it, or None if the list is empty. <br>从游标的父列表中删除第一个元素并返回它，如果列表为空，则返回 None。<br>
    /// The element the cursor points to remains unchanged, unless it was pointing to the front element. <br>游标指向的元素保持不变，除非它指向前面的元素。<br>
    /// In that case, it points to the new front element. <br>在这种情况下，它指向新的前端元素。<br>
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn pop_front(&mut self) -> Option<T> {
        // We can't check if current is empty, we must check the list directly. <br>我们不能检查 current 是否为空，我们必须直接检查列表。<br>
        // It is possible for `self.current == None` and the list to be non-empty. <br>`self.current == None` 和列表可能不为空。<br>
        //
        if self.list.is_empty() {
            None
        } else {
            // We can't point to the node that we pop. <br>我们不能指向我们弹出的节点。<br>
            // Copying the behavior of `remove_current`, we move on the the next node in the sequence. <br>复制 `remove_current` 的行为，我们移动到序列中的下一个节点。<br>
            // If the list is of length 1 then we end pointing to the "ghost" node at index 0, which is expected. <br>如果列表的长度为 1，那么我们结束指向索引 0 处的 "ghost" 节点，这是预期的。<br>
            //
            if self.list.head == self.current {
                self.move_next();
            } else {
                self.index -= 1;
            }
            self.list.pop_front()
        }
    }

    /// Removes the last element from the cursor's parent list and returns it, or None if the list is empty. <br>从游标的父列表中删除最后一个元素并返回它，如果列表为空，则返回 None。<br>
    /// The element the cursor points to remains unchanged, unless it was pointing to the back element. <br>游标指向的元素保持不变，除非它指向后面的元素。<br>
    /// In that case, it points to the "ghost" element. <br>在这种情况下，它指向 "ghost" 元素。<br>
    ///
    /// This operation should compute in *O*(1) time. <br>此运算应在 *O*(1) 时间中进行计算。<br>
    ///
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn pop_back(&mut self) -> Option<T> {
        if self.list.is_empty() {
            None
        } else {
            if self.list.tail == self.current {
                // The index now reflects the length of the list. <br>索引现在反映了列表的长度。<br>
                // It was the length of the list minus 1, but now the list is 1 smaller. <br>它是列表的长度减 1，但现在列表小 1。<br>
                // No change is needed for `index`. <br>`index` 不需要更改。<br>
                self.current = None;
            } else if self.current.is_none() {
                self.index = self.list.len - 1;
            }
            self.list.pop_back()
        }
    }

    /// Provides a reference to the front element of the cursor's parent list, or None if the list is empty. <br>提供对游标父列表前元素的引用，如果列表为空，则为 None。<br>
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn front(&self) -> Option<&T> {
        self.list.front()
    }

    /// Provides a mutable reference to the front element of the cursor's parent list, or None if the list is empty. <br>提供对光标父列表的前元素的可变引用，如果列表为空，则为 None。<br>
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn front_mut(&mut self) -> Option<&mut T> {
        self.list.front_mut()
    }

    /// Provides a reference to the back element of the cursor's parent list, or None if the list is empty. <br>提供对游标父列表的后部元素的引用，如果列表为空，则为 None。<br>
    ///
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn back(&self) -> Option<&T> {
        self.list.back()
    }

    /// Provides a mutable reference to back element of the cursor's parent list, or `None` if the list is empty. <br>提供一个循环引用来返回游标的父列表的元素，如果列表为空，则提供 `None`。<br>
    ///
    /// # Examples
    /// Building and mutating a list with a cursor, then getting the back element: <br>使用游标构建和可变列表，然后获取返回元素：<br>
    ///
    /// ```
    /// #![feature(linked_list_cursors)]
    /// use std::collections::LinkedList;
    /// let mut dl = LinkedList::new();
    /// dl.push_front(3);
    /// dl.push_front(2);
    /// dl.push_front(1);
    /// let mut cursor = dl.cursor_front_mut();
    /// *cursor.current().unwrap() = 99;
    /// *cursor.back_mut().unwrap() = 0;
    /// let mut contents = dl.into_iter();
    /// assert_eq!(contents.next(), Some(99));
    /// assert_eq!(contents.next(), Some(2));
    /// assert_eq!(contents.next(), Some(0));
    /// assert_eq!(contents.next(), None);
    /// ```
    #[must_use]
    #[unstable(feature = "linked_list_cursors", issue = "58533")]
    pub fn back_mut(&mut self) -> Option<&mut T> {
        self.list.back_mut()
    }
}

/// An iterator produced by calling `drain_filter` on LinkedList. <br>通过在 LinkedList 上调用 `drain_filter` 生成的迭代器。<br>
#[unstable(feature = "drain_filter", reason = "recently added", issue = "43244")]
pub struct DrainFilter<'a, T: 'a, F: 'a>
where
    F: FnMut(&mut T) -> bool,
{
    list: &'a mut LinkedList<T>,
    it: Option<NonNull<Node<T>>>,
    pred: F,
    idx: usize,
    old_len: usize,
}

#[unstable(feature = "drain_filter", reason = "recently added", issue = "43244")]
impl<T, F> Iterator for DrainFilter<'_, T, F>
where
    F: FnMut(&mut T) -> bool,
{
    type Item = T;

    fn next(&mut self) -> Option<T> {
        while let Some(mut node) = self.it {
            unsafe {
                self.it = node.as_ref().next;
                self.idx += 1;

                if (self.pred)(&mut node.as_mut().element) {
                    // `unlink_node` is okay with aliasing `element` references. <br>`unlink_node` 可以使用别名 `element` 来引用。<br>
                    self.list.unlink_node(node);
                    return Some(Box::from_raw(node.as_ptr()).element);
                }
            }
        }

        None
    }

    fn size_hint(&self) -> (usize, Option<usize>) {
        (0, Some(self.old_len - self.idx))
    }
}

#[unstable(feature = "drain_filter", reason = "recently added", issue = "43244")]
impl<T, F> Drop for DrainFilter<'_, T, F>
where
    F: FnMut(&mut T) -> bool,
{
    fn drop(&mut self) {
        struct DropGuard<'r, 'a, T, F>(&'r mut DrainFilter<'a, T, F>)
        where
            F: FnMut(&mut T) -> bool;

        impl<'r, 'a, T, F> Drop for DropGuard<'r, 'a, T, F>
        where
            F: FnMut(&mut T) -> bool,
        {
            fn drop(&mut self) {
                self.0.for_each(drop);
            }
        }

        while let Some(item) = self.next() {
            let guard = DropGuard(self);
            drop(item);
            mem::forget(guard);
        }
    }
}

#[unstable(feature = "drain_filter", reason = "recently added", issue = "43244")]
impl<T: fmt::Debug, F> fmt::Debug for DrainFilter<'_, T, F>
where
    F: FnMut(&mut T) -> bool,
{
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple("DrainFilter").field(&self.list).finish()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Iterator for IntoIter<T> {
    type Item = T;

    #[inline]
    fn next(&mut self) -> Option<T> {
        self.list.pop_front()
    }

    #[inline]
    fn size_hint(&self) -> (usize, Option<usize>) {
        (self.list.len, Some(self.list.len))
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> DoubleEndedIterator for IntoIter<T> {
    #[inline]
    fn next_back(&mut self) -> Option<T> {
        self.list.pop_back()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> ExactSizeIterator for IntoIter<T> {}

#[stable(feature = "fused", since = "1.26.0")]
impl<T> FusedIterator for IntoIter<T> {}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> FromIterator<T> for LinkedList<T> {
    fn from_iter<I: IntoIterator<Item = T>>(iter: I) -> Self {
        let mut list = Self::new();
        list.extend(iter);
        list
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> IntoIterator for LinkedList<T> {
    type Item = T;
    type IntoIter = IntoIter<T>;

    /// Consumes the list into an iterator yielding elements by value. <br>将列表消耗到迭代器中，该迭代器按值产生元素。<br>
    #[inline]
    fn into_iter(self) -> IntoIter<T> {
        IntoIter { list: self }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, T> IntoIterator for &'a LinkedList<T> {
    type Item = &'a T;
    type IntoIter = Iter<'a, T>;

    fn into_iter(self) -> Iter<'a, T> {
        self.iter()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, T> IntoIterator for &'a mut LinkedList<T> {
    type Item = &'a mut T;
    type IntoIter = IterMut<'a, T>;

    fn into_iter(self) -> IterMut<'a, T> {
        self.iter_mut()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Extend<T> for LinkedList<T> {
    fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) {
        <Self as SpecExtend<I>>::spec_extend(self, iter);
    }

    #[inline]
    fn extend_one(&mut self, elem: T) {
        self.push_back(elem);
    }
}

impl<I: IntoIterator> SpecExtend<I> for LinkedList<I::Item> {
    default fn spec_extend(&mut self, iter: I) {
        iter.into_iter().for_each(move |elt| self.push_back(elt));
    }
}

impl<T> SpecExtend<LinkedList<T>> for LinkedList<T> {
    fn spec_extend(&mut self, ref mut other: LinkedList<T>) {
        self.append(other);
    }
}

#[stable(feature = "extend_ref", since = "1.2.0")]
impl<'a, T: 'a + Copy> Extend<&'a T> for LinkedList<T> {
    fn extend<I: IntoIterator<Item = &'a T>>(&mut self, iter: I) {
        self.extend(iter.into_iter().cloned());
    }

    #[inline]
    fn extend_one(&mut self, &elem: &'a T) {
        self.push_back(elem);
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: PartialEq> PartialEq for LinkedList<T> {
    fn eq(&self, other: &Self) -> bool {
        self.len() == other.len() && self.iter().eq(other)
    }

    fn ne(&self, other: &Self) -> bool {
        self.len() != other.len() || self.iter().ne(other)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Eq> Eq for LinkedList<T> {}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: PartialOrd> PartialOrd for LinkedList<T> {
    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
        self.iter().partial_cmp(other)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Ord> Ord for LinkedList<T> {
    #[inline]
    fn cmp(&self, other: &Self) -> Ordering {
        self.iter().cmp(other)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Clone> Clone for LinkedList<T> {
    fn clone(&self) -> Self {
        self.iter().cloned().collect()
    }

    fn clone_from(&mut self, other: &Self) {
        let mut iter_other = other.iter();
        if self.len() > other.len() {
            self.split_off(other.len());
        }
        for (elem, elem_other) in self.iter_mut().zip(&mut iter_other) {
            elem.clone_from(elem_other);
        }
        if !iter_other.is_empty() {
            self.extend(iter_other.cloned());
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: fmt::Debug> fmt::Debug for LinkedList<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_list().entries(self).finish()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Hash> Hash for LinkedList<T> {
    fn hash<H: Hasher>(&self, state: &mut H) {
        state.write_length_prefix(self.len());
        for elt in self {
            elt.hash(state);
        }
    }
}

#[stable(feature = "std_collections_from_array", since = "1.56.0")]
impl<T, const N: usize> From<[T; N]> for LinkedList<T> {
    /// Converts a `[T; N]` into a `LinkedList<T>`. <br>将 `[T; N]` 转换为 `LinkedList<T>`。<br>
    ///
    /// ```
    /// use std::collections::LinkedList;
    ///
    /// let list1 = LinkedList::from([1, 2, 3, 4]);
    /// let list2: LinkedList<_> = [1, 2, 3, 4].into();
    /// assert_eq!(list1, list2);
    /// ```
    fn from(arr: [T; N]) -> Self {
        Self::from_iter(arr)
    }
}

// Ensure that `LinkedList` and its read-only iterators are covariant in their type parameters. <br>确保 `LinkedList` 及其只读迭代器的类型参数是协变的。<br>
#[allow(dead_code)]
fn assert_covariance() {
    fn a<'a>(x: LinkedList<&'static str>) -> LinkedList<&'a str> {
        x
    }
    fn b<'i, 'a>(x: Iter<'i, &'static str>) -> Iter<'i, &'a str> {
        x
    }
    fn c<'a>(x: IntoIter<&'static str>) -> IntoIter<&'a str> {
        x
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<T: Send> Send for LinkedList<T> {}

#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<T: Sync> Sync for LinkedList<T> {}

#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<T: Sync> Send for Iter<'_, T> {}

#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<T: Sync> Sync for Iter<'_, T> {}

#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<T: Send> Send for IterMut<'_, T> {}

#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<T: Sync> Sync for IterMut<'_, T> {}

#[unstable(feature = "linked_list_cursors", issue = "58533")]
unsafe impl<T: Sync> Send for Cursor<'_, T> {}

#[unstable(feature = "linked_list_cursors", issue = "58533")]
unsafe impl<T: Sync> Sync for Cursor<'_, T> {}

#[unstable(feature = "linked_list_cursors", issue = "58533")]
unsafe impl<T: Send> Send for CursorMut<'_, T> {}

#[unstable(feature = "linked_list_cursors", issue = "58533")]
unsafe impl<T: Sync> Sync for CursorMut<'_, T> {}
