//! A dynamically-sized view into a contiguous sequence, `[T]`. <br>一个动态大小的视图到一个连续的序列，`[T]`。<br>
//!
//! *[See also the slice primitive type](slice).*
//!
//! Slices are a view into a block of memory represented as a pointer and a length. <br>切片是一个内存块的视图，表示为一个指针和一个长度。<br>
//!
//! ```
//! // slicing a Vec <br>切片 Vec<br>
//! let vec = vec![1, 2, 3];
//! let int_slice = &vec[..];
//! // coercing an array to a slice <br>将数组强制转换为切片<br>
//! let str_slice: &[&str] = &["one", "two", "three"];
//! ```
//!
//! Slices are either mutable or shared. <br>切片是可变的或共享的。<br>
//! The shared slice type is `&[T]`, while the mutable slice type is `&mut [T]`, where `T` represents the element type. <br>共享切片类型为 `&[T]`，而可变切片类型为 `&mut [T]`，其中 `T` 表示元素类型。<br>
//! For example, you can mutate the block of memory that a mutable slice points to: <br>例如，您可以更改可变切片所指向的内存块：<br>
//!
//! ```
//! let x = &mut [1, 2, 3];
//! x[1] = 7;
//! assert_eq!(x, &[1, 7, 3]);
//! ```
//!
//! Here are some of the things this module contains: <br>以下是此模块包含的一些内容：<br>
//!
//! ## Structs
//!
//! There are several structs that are useful for slices, such as [`Iter`], which represents iteration over a slice. <br>有几种结构切片可用于切片，例如 [`Iter`]，它表示切片上的迭代。<br>
//!
//! ## Trait Implementations <br>Trait 实现<br>
//!
//! There are several implementations of common traits for slices. <br>切片具有常见的 traits 的几种实现。<br> Some examples include: <br>一些示例包括：<br>
//!
//! * [`Clone`]
//! * [`Eq`], [`Ord`] - for slices whose element type are [`Eq`] or [`Ord`]. <br>[`Eq`]、[`Ord`] - 适用于元素类型为 [`Eq`] 或 [`Ord`] 的切片。<br>
//! * [`Hash`] - for slices whose element type is [`Hash`]. <br>[`Hash`] - 用于元素类型为 [`Hash`] 的切片。<br>
//!
//! ## Iteration
//!
//! The slices implement `IntoIterator`. <br>切片实现 `IntoIterator`。<br> The iterator yields references to the slice elements. <br>迭代器产生对切片元素的引用。<br>
//!
//! ```
//! let numbers = &[0, 1, 2];
//! for n in numbers {
//!     println!("{n} is a number!");
//! }
//! ```
//!
//! The mutable slice yields mutable references to the elements: <br>可变切片对以下元素产生了可变引用：<br>
//!
//! ```
//! let mut scores = [7, 8, 9];
//! for score in &mut scores[..] {
//!     *score += 1;
//! }
//! ```
//!
//! This iterator yields mutable references to the slice's elements, so while the element type of the slice is `i32`, the element type of the iterator is `&mut i32`. <br>此迭代器产生对切片元素的可变引用，因此，当切片的元素类型为 `i32` 时，迭代器的元素类型为 `&mut i32`。<br>
//!
//!
//! * [`.iter`] and [`.iter_mut`] are the explicit methods to return the default iterators. <br>[`.iter`] 和 [`.iter_mut`] 是返回默认迭代器的显式方法。<br>
//! * Further methods that return iterators are [`.split`], [`.splitn`], [`.chunks`], [`.windows`] and more. <br>返回迭代器的其他方法是 [`.split`]，[`.splitn`]，[`.chunks`]，[`.windows`] 等。<br>
//!
//! [`Hash`]: core::hash::Hash
//! [`.iter`]: slice::iter
//! [`.iter_mut`]: slice::iter_mut
//! [`.split`]: slice::split
//! [`.splitn`]: slice::splitn
//! [`.chunks`]: slice::chunks
//! [`.windows`]: slice::windows
//!
//!
//!
//!
//!
//!
//!
//!
#![stable(feature = "rust1", since = "1.0.0")]
// Many of the usings in this module are only used in the test configuration. <br>该模块中的许多用法仅在测试配置中使用。<br>
// It's cleaner to just turn off the unused_imports warning than to fix them. <br>仅关闭 unused_imports 警告比解决它们更干净。<br>
#![cfg_attr(test, allow(unused_imports, dead_code))]

use core::borrow::{Borrow, BorrowMut};
#[cfg(not(no_global_oom_handling))]
use core::cmp::Ordering::{self, Less};
#[cfg(not(no_global_oom_handling))]
use core::mem;
#[cfg(not(no_global_oom_handling))]
use core::mem::size_of;
#[cfg(not(no_global_oom_handling))]
use core::ptr;

use crate::alloc::Allocator;
#[cfg(not(no_global_oom_handling))]
use crate::alloc::Global;
#[cfg(not(no_global_oom_handling))]
use crate::borrow::ToOwned;
use crate::boxed::Box;
use crate::vec::Vec;

#[unstable(feature = "slice_range", issue = "76393")]
pub use core::slice::range;
#[unstable(feature = "array_chunks", issue = "74985")]
pub use core::slice::ArrayChunks;
#[unstable(feature = "array_chunks", issue = "74985")]
pub use core::slice::ArrayChunksMut;
#[unstable(feature = "array_windows", issue = "75027")]
pub use core::slice::ArrayWindows;
#[stable(feature = "inherent_ascii_escape", since = "1.60.0")]
pub use core::slice::EscapeAscii;
#[stable(feature = "slice_get_slice", since = "1.28.0")]
pub use core::slice::SliceIndex;
#[stable(feature = "from_ref", since = "1.28.0")]
pub use core::slice::{from_mut, from_ref};
#[unstable(feature = "slice_from_ptr_range", issue = "89792")]
pub use core::slice::{from_mut_ptr_range, from_ptr_range};
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::slice::{from_raw_parts, from_raw_parts_mut};
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::slice::{Chunks, Windows};
#[stable(feature = "chunks_exact", since = "1.31.0")]
pub use core::slice::{ChunksExact, ChunksExactMut};
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::slice::{ChunksMut, Split, SplitMut};
#[unstable(feature = "slice_group_by", issue = "80552")]
pub use core::slice::{GroupBy, GroupByMut};
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::slice::{Iter, IterMut};
#[stable(feature = "rchunks", since = "1.31.0")]
pub use core::slice::{RChunks, RChunksExact, RChunksExactMut, RChunksMut};
#[stable(feature = "slice_rsplit", since = "1.27.0")]
pub use core::slice::{RSplit, RSplitMut};
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::slice::{RSplitN, RSplitNMut, SplitN, SplitNMut};
#[stable(feature = "split_inclusive", since = "1.51.0")]
pub use core::slice::{SplitInclusive, SplitInclusiveMut};

////////////////////////////////////////////////////////////////////////////////
// Basic slice extension methods <br>基本切片扩展方法<br>
////////////////////////////////////////////////////////////////////////////////

// HACK(japaric) needed for the implementation of `vec!` macro during testing N.B., see the `hack` module in this file for more details. <br>在测试 NB 期间需要实现 `vec!` 宏，请参阅此文件中的 `hack` 模块以获取更多详细信息。<br>
//
#[cfg(test)]
pub use hack::into_vec;

// HACK(japaric) needed for the implementation of `Vec::clone` during testing N.B., see the `hack` module in this file for more details. <br>在测试 NB 期间需要实现 `Vec::clone`，请参阅此文件中的 `hack` 模块以获取更多详细信息。<br>
//
#[cfg(test)]
pub use hack::to_vec;

// HACK(japaric): With cfg(test) `impl [T]` is not available, these three functions are actually methods that are in `impl [T]` but not in `core::slice::SliceExt` - we need to supply these functions for the `test_permutations` test <br>cfg(test) `impl [T]` 不可用，这三个函数实际上是 `impl [T]` 中的方法，`core::slice::SliceExt` 中没有 - 我们需要为 `test_permutations` 测试提供这些函数<br>
//
//
//
pub(crate) mod hack {
    use core::alloc::Allocator;

    use crate::boxed::Box;
    use crate::vec::Vec;

    // We shouldn't add inline attribute to this since this is used in `vec!` macro mostly and causes perf regression. <br>我们不应该向此属性添加内联属性，因为该属性主要在 `vec!` 宏中使用，并且会导致性能下降。<br>
    // See #71204 for discussion and perf results. <br>有关讨论和性能结果，请参见 #71204。<br>
    //
    pub fn into_vec<T, A: Allocator>(b: Box<[T], A>) -> Vec<T, A> {
        unsafe {
            let len = b.len();
            let (b, alloc) = Box::into_raw_with_allocator(b);
            Vec::from_raw_parts_in(b as *mut T, len, len, alloc)
        }
    }

    #[cfg(not(no_global_oom_handling))]
    #[inline]
    pub fn to_vec<T: ConvertVec, A: Allocator>(s: &[T], alloc: A) -> Vec<T, A> {
        T::to_vec(s, alloc)
    }

    #[cfg(not(no_global_oom_handling))]
    pub trait ConvertVec {
        fn to_vec<A: Allocator>(s: &[Self], alloc: A) -> Vec<Self, A>
        where
            Self: Sized;
    }

    #[cfg(not(no_global_oom_handling))]
    impl<T: Clone> ConvertVec for T {
        #[inline]
        default fn to_vec<A: Allocator>(s: &[Self], alloc: A) -> Vec<Self, A> {
            struct DropGuard<'a, T, A: Allocator> {
                vec: &'a mut Vec<T, A>,
                num_init: usize,
            }
            impl<'a, T, A: Allocator> Drop for DropGuard<'a, T, A> {
                #[inline]
                fn drop(&mut self) {
                    // SAFETY:
                    // items were marked initialized in the loop below <br>项在下面的循环中被标记为已初始化<br>
                    unsafe {
                        self.vec.set_len(self.num_init);
                    }
                }
            }
            let mut vec = Vec::with_capacity_in(s.len(), alloc);
            let mut guard = DropGuard { vec: &mut vec, num_init: 0 };
            let slots = guard.vec.spare_capacity_mut();
            // .take(slots.len()) is necessary for LLVM to remove bounds checks and has better codegen than zip. <br>.take(slots.len()) 是 LLVM 删除边界检查所必需的，并且具有比 zip 更好的 codegen。<br>
            //
            for (i, b) in s.iter().enumerate().take(slots.len()) {
                guard.num_init = i;
                slots[i].write(b.clone());
            }
            core::mem::forget(guard);
            // SAFETY:
            // the vec was allocated and initialized above to at least this length. <br>vec 的分配和初始化至少要达到此长度。<br>
            unsafe {
                vec.set_len(s.len());
            }
            vec
        }
    }

    #[cfg(not(no_global_oom_handling))]
    impl<T: Copy> ConvertVec for T {
        #[inline]
        fn to_vec<A: Allocator>(s: &[Self], alloc: A) -> Vec<Self, A> {
            let mut v = Vec::with_capacity_in(s.len(), alloc);
            // SAFETY:
            // allocated above with the capacity of `s`, and initialize to `s.len()` in ptr::copy_to_non_overlapping below. <br>在上面分配了 `s` 的容量，并在下面的 ptr::copy_to_non_overlapping 中初始化为 `s.len()`。<br>
            //
            unsafe {
                s.as_ptr().copy_to_nonoverlapping(v.as_mut_ptr(), s.len());
                v.set_len(s.len());
            }
            v
        }
    }
}

#[cfg(not(test))]
impl<T> [T] {
    /// Sorts the slice. <br>对切片进行排序。<br>
    ///
    /// This sort is stable (i.e., does not reorder equal elements) and *O*(*n* \* log(*n*)) worst-case. <br>这种排序是稳定的 (即，不对相等的元素重新排序)，并且 *O*(*n*\*log(* n*)) 最坏的情况)。<br>
    ///
    /// When applicable, unstable sorting is preferred because it is generally faster than stable sorting and it doesn't allocate auxiliary memory. <br>在适用时，首选不稳定排序，因为它通常比稳定排序快，并且不分配辅助内存。<br>
    /// See [`sort_unstable`](slice::sort_unstable). <br>请参见 [`sort_unstable`](slice::sort_unstable)。<br>
    ///
    /// # Current implementation <br>当前实现<br>
    ///
    /// The current algorithm is an adaptive, iterative merge sort inspired by [timsort](https://en.wikipedia.org/wiki/Timsort). <br>当前的算法是一种受 [timsort](https://en.wikipedia.org/wiki/Timsort) 启发的自适应迭代合并排序。<br>
    /// It is designed to be very fast in cases where the slice is nearly sorted, or consists of two or more sorted sequences concatenated one after another. <br>在切片几乎被排序或由两个或多个依次连接的排序序列组成的情况下，它设计得非常快。<br>
    ///
    ///
    /// Also, it allocates temporary storage half the size of `self`, but for short slices a non-allocating insertion sort is used instead. <br>同样，它分配临时存储空间的大小是 `self` 的一半，但是对于短片，则使用非分配插入排序。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut v = [-5, 4, 1, -3, 2];
    ///
    /// v.sort();
    /// assert!(v == [-5, -3, 1, 2, 4]);
    /// ```
    ///
    ///
    ///
    #[cfg(not(no_global_oom_handling))]
    #[rustc_allow_incoherent_impl]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[inline]
    pub fn sort(&mut self)
    where
        T: Ord,
    {
        merge_sort(self, |a, b| a.lt(b));
    }

    /// Sorts the slice with a comparator function. <br>用比较器函数对切片进行排序。<br>
    ///
    /// This sort is stable (i.e., does not reorder equal elements) and *O*(*n* \* log(*n*)) worst-case. <br>这种排序是稳定的 (即，不对相等的元素重新排序)，并且 *O*(*n*\*log(* n*)) 最坏的情况)。<br>
    ///
    /// The comparator function must define a total ordering for the elements in the slice. <br>比较器函数必须为切片中的元素定义总顺序。<br> If the ordering is not total, the order of the elements is unspecified. <br>如果排序不全，则元素的顺序是未指定的。<br>
    /// An order is a total order if it is (for all `a`, `b` and `c`): <br>如果一个顺序是 (对于所有的`a`, `b` 和 `c`)，那么它就是一个总体顺序<br>
    ///
    /// * total and antisymmetric: exactly one of `a < b`, `a == b` or `a > b` is true, and <br>完全和反对称的: `a < b`，`a == b` 或 `a > b` 之一正确，并且<br>
    /// * transitive, `a < b` and `b < c` implies `a < c`. <br>可传递的，`a < b` 和 `b < c` 表示 `a < c`。<br> The same must hold for both `==` and `>`. <br>`==` 和 `>` 必须保持相同。<br>
    ///
    /// For example, while [`f64`] doesn't implement [`Ord`] because `NaN != NaN`, we can use `partial_cmp` as our sort function when we know the slice doesn't contain a `NaN`. <br>例如，虽然 [`f64`] 由于 `NaN != NaN` 而不实现 [`Ord`]，但是当我们知道切片不包含 `NaN` 时，可以将 `partial_cmp` 用作我们的排序函数。<br>
    ///
    ///
    /// ```
    /// let mut floats = [5f64, 4.0, 1.0, 3.0, 2.0];
    /// floats.sort_by(|a, b| a.partial_cmp(b).unwrap());
    /// assert_eq!(floats, [1.0, 2.0, 3.0, 4.0, 5.0]);
    /// ```
    ///
    /// When applicable, unstable sorting is preferred because it is generally faster than stable sorting and it doesn't allocate auxiliary memory. <br>在适用时，首选不稳定排序，因为它通常比稳定排序快，并且不分配辅助内存。<br>
    /// See [`sort_unstable_by`](slice::sort_unstable_by). <br>请参见 [`sort_unstable_by`](slice::sort_unstable_by)。<br>
    ///
    /// # Current implementation <br>当前实现<br>
    ///
    /// The current algorithm is an adaptive, iterative merge sort inspired by [timsort](https://en.wikipedia.org/wiki/Timsort). <br>当前的算法是一种受 [timsort](https://en.wikipedia.org/wiki/Timsort) 启发的自适应迭代合并排序。<br>
    /// It is designed to be very fast in cases where the slice is nearly sorted, or consists of two or more sorted sequences concatenated one after another. <br>在切片几乎被排序或由两个或多个依次连接的排序序列组成的情况下，它设计得非常快。<br>
    ///
    /// Also, it allocates temporary storage half the size of `self`, but for short slices a non-allocating insertion sort is used instead. <br>同样，它分配临时存储空间的大小是 `self` 的一半，但是对于短片，则使用非分配插入排序。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut v = [5, 4, 1, 3, 2];
    /// v.sort_by(|a, b| a.cmp(b));
    /// assert!(v == [1, 2, 3, 4, 5]);
    ///
    /// // reverse sorting <br>反向排序<br>
    /// v.sort_by(|a, b| b.cmp(a));
    /// assert!(v == [5, 4, 3, 2, 1]);
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[cfg(not(no_global_oom_handling))]
    #[rustc_allow_incoherent_impl]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[inline]
    pub fn sort_by<F>(&mut self, mut compare: F)
    where
        F: FnMut(&T, &T) -> Ordering,
    {
        merge_sort(self, |a, b| compare(a, b) == Less);
    }

    /// Sorts the slice with a key extraction function. <br>用键提取函数对切片进行排序。<br>
    ///
    /// This sort is stable (i.e., does not reorder equal elements) and *O*(*m* \* *n* \* log(*n*)) worst-case, where the key function is *O*(*m*). <br>这种排序是稳定的 (即，不对相等的元素重新排序)，并且是 *O*(*m*\* * n *\* log(*n*)) 最坏的情况，其中键函数为 *O*(*m*)。<br>
    ///
    /// For expensive key functions (e.g. <br>对于昂贵的键函数 (例如<br>
    /// functions that are not simple property accesses or basic operations), [`sort_by_cached_key`](slice::sort_by_cached_key) is likely to be significantly faster, as it does not recompute element keys. <br>不是简单的属性访问或基本操作的函数)，[`sort_by_cached_key`](slice::sort_by_cached_key) 可能会显着提高速度，因为它不会重新计算元素键。<br>
    ///
    ///
    /// When applicable, unstable sorting is preferred because it is generally faster than stable sorting and it doesn't allocate auxiliary memory. <br>在适用时，首选不稳定排序，因为它通常比稳定排序快，并且不分配辅助内存。<br>
    /// See [`sort_unstable_by_key`](slice::sort_unstable_by_key). <br>请参见 [`sort_unstable_by_key`](slice::sort_unstable_by_key)。<br>
    ///
    /// # Current implementation <br>当前实现<br>
    ///
    /// The current algorithm is an adaptive, iterative merge sort inspired by [timsort](https://en.wikipedia.org/wiki/Timsort). <br>当前的算法是一种受 [timsort](https://en.wikipedia.org/wiki/Timsort) 启发的自适应迭代合并排序。<br>
    /// It is designed to be very fast in cases where the slice is nearly sorted, or consists of two or more sorted sequences concatenated one after another. <br>在切片几乎被排序或由两个或多个依次连接的排序序列组成的情况下，它设计得非常快。<br>
    ///
    /// Also, it allocates temporary storage half the size of `self`, but for short slices a non-allocating insertion sort is used instead. <br>同样，它分配临时存储空间的大小是 `self` 的一半，但是对于短片，则使用非分配插入排序。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut v = [-5i32, 4, 1, -3, 2];
    ///
    /// v.sort_by_key(|k| k.abs());
    /// assert!(v == [1, 2, -3, 4, -5]);
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[cfg(not(no_global_oom_handling))]
    #[rustc_allow_incoherent_impl]
    #[stable(feature = "slice_sort_by_key", since = "1.7.0")]
    #[inline]
    pub fn sort_by_key<K, F>(&mut self, mut f: F)
    where
        F: FnMut(&T) -> K,
        K: Ord,
    {
        merge_sort(self, |a, b| f(a).lt(&f(b)));
    }

    /// Sorts the slice with a key extraction function. <br>用键提取函数对切片进行排序。<br>
    ///
    /// During sorting, the key function is called at most once per element, by using temporary storage to remember the results of key evaluation. <br>在排序过程中，每个元素最多调用一次 key 函数，通过使用临时存储来记住 key 评估的结果。<br>
    /// The order of calls to the key function is unspecified and may change in future versions of the standard library. <br>对 key 函数的调用顺序是未指定的，并且在标准库的未来版本中可能会发生变化。<br>
    ///
    /// This sort is stable (i.e., does not reorder equal elements) and *O*(*m* \* *n* + *n* \* log(*n*)) worst-case, where the key function is *O*(*m*). <br>这种排序是稳定的 (即，不对相等的元素重新排序)，并且 *O*(*m*\* * n *+* n *\* log(*n*)) 最坏的情况是，其中键函数为 *O*(*m*)。<br>
    ///
    /// For simple key functions (e.g., functions that are property accesses or basic operations), [`sort_by_key`](slice::sort_by_key) is likely to be faster. <br>对于简单的键函数 (例如，作为属性访问或基本操作的函数)，[`sort_by_key`](slice::sort_by_key) 可能会更快。<br>
    ///
    /// # Current implementation <br>当前实现<br>
    ///
    /// The current algorithm is based on [pattern-defeating quicksort][pdqsort] by Orson Peters, which combines the fast average case of randomized quicksort with the fast worst case of heapsort, while achieving linear time on slices with certain patterns. <br>当前算法基于 Orson Peters 的 [pattern-defeating 的快速排序][pdqsort]，该算法将随机快速排序的快速平均情况与堆排序的快速最坏情况相结合，同时在具有特定模式的切片上实现了线性时间。<br>
    /// It uses some randomization to avoid degenerate cases, but with a fixed seed to always provide deterministic behavior. <br>它使用一些随机化来避免退化的情况，但是使用固定的 seed 来始终提供确定性的行为。<br>
    ///
    /// In the worst case, the algorithm allocates temporary storage in a `Vec<(K, usize)>` the length of the slice. <br>在最坏的情况下，该算法在 `Vec<(K, usize)>` 中分配切片长度的临时存储。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut v = [-5i32, 4, 32, -3, 2];
    ///
    /// v.sort_by_cached_key(|k| k.to_string());
    /// assert!(v == [-3, -5, 2, 32, 4]);
    /// ```
    ///
    /// [pdqsort]: https://github.com/orlp/pdqsort
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[cfg(not(no_global_oom_handling))]
    #[rustc_allow_incoherent_impl]
    #[stable(feature = "slice_sort_by_cached_key", since = "1.34.0")]
    #[inline]
    pub fn sort_by_cached_key<K, F>(&mut self, f: F)
    where
        F: FnMut(&T) -> K,
        K: Ord,
    {
        // Helper macro for indexing our vector by the smallest possible type, to reduce allocation. <br>辅助宏，用于通过最小的类型索引 vector，以减少分配。<br>
        macro_rules! sort_by_key {
            ($t:ty, $slice:ident, $f:ident) => {{
                let mut indices: Vec<_> =
                    $slice.iter().map($f).enumerate().map(|(i, k)| (k, i as $t)).collect();
                // The elements of `indices` are unique, as they are indexed, so any sort will be stable with respect to the original slice. <br>`indices` 的元素是唯一的，因为它们已被索引，因此任何种类相对于原始切片都是稳定的。<br>
                // We use `sort_unstable` here because it requires less memory allocation. <br>我们在这里使用 `sort_unstable` 是因为它需要较少的内存分配。<br>
                //
                indices.sort_unstable();
                for i in 0..$slice.len() {
                    let mut index = indices[i].1;
                    while (index as usize) < i {
                        index = indices[index as usize].1;
                    }
                    indices[i].1 = index;
                    $slice.swap(i, index as usize);
                }
            }};
        }

        let sz_u8 = mem::size_of::<(K, u8)>();
        let sz_u16 = mem::size_of::<(K, u16)>();
        let sz_u32 = mem::size_of::<(K, u32)>();
        let sz_usize = mem::size_of::<(K, usize)>();

        let len = self.len();
        if len < 2 {
            return;
        }
        if sz_u8 < sz_u16 && len <= (u8::MAX as usize) {
            return sort_by_key!(u8, self, f);
        }
        if sz_u16 < sz_u32 && len <= (u16::MAX as usize) {
            return sort_by_key!(u16, self, f);
        }
        if sz_u32 < sz_usize && len <= (u32::MAX as usize) {
            return sort_by_key!(u32, self, f);
        }
        sort_by_key!(usize, self, f)
    }

    /// Copies `self` into a new `Vec`. <br>将 `self` 复制到新的 `Vec` 中。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let s = [10, 40, 30];
    /// let x = s.to_vec();
    /// // Here, `s` and `x` can be modified independently. <br>在此，`s` 和 `x` 可以独立修改。<br>
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[rustc_allow_incoherent_impl]
    #[rustc_conversion_suggestion]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[inline]
    pub fn to_vec(&self) -> Vec<T>
    where
        T: Clone,
    {
        self.to_vec_in(Global)
    }

    /// Copies `self` into a new `Vec` with an allocator. <br>使用分配器将 `self` 复制到新的 `Vec` 中。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(allocator_api)]
    ///
    /// use std::alloc::System;
    ///
    /// let s = [10, 40, 30];
    /// let x = s.to_vec_in(System);
    /// // Here, `s` and `x` can be modified independently. <br>在此，`s` 和 `x` 可以独立修改。<br>
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[rustc_allow_incoherent_impl]
    #[inline]
    #[unstable(feature = "allocator_api", issue = "32838")]
    pub fn to_vec_in<A: Allocator>(&self, alloc: A) -> Vec<T, A>
    where
        T: Clone,
    {
        // N.B., see the `hack` module in this file for more details. <br>注意，有关更多详细信息，请参见此文件中的 `hack` 模块。<br>
        hack::to_vec(self, alloc)
    }

    /// Converts `self` into a vector without clones or allocation. <br>将 `self` 转换为 vector，而无需克隆或分配。<br>
    ///
    /// The resulting vector can be converted back into a box via `Vec<T>`'s `into_boxed_slice` method. <br>产生的 vector 可以通过 `Vec 转换回 box<T>` 的 `into_boxed_slice` 方法。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let s: Box<[i32]> = Box::new([10, 40, 30]);
    /// let x = s.into_vec();
    /// // `s` cannot be used anymore because it has been converted into `x`. <br>`s` 不能再使用了，因为它已经转换成 `x`。<br>
    ///
    /// assert_eq!(x, vec![10, 40, 30]);
    /// ```
    #[rustc_allow_incoherent_impl]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[inline]
    pub fn into_vec<A: Allocator>(self: Box<Self, A>) -> Vec<T, A> {
        // N.B., see the `hack` module in this file for more details. <br>注意，有关更多详细信息，请参见此文件中的 `hack` 模块。<br>
        hack::into_vec(self)
    }

    /// Creates a vector by repeating a slice `n` times. <br>通过重复切片 `n` 次来创建 vector。<br>
    ///
    /// # Panics
    ///
    /// This function will panic if the capacity would overflow. <br>如果容量溢出，此函数将为 panic。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// assert_eq!([1, 2].repeat(3), vec![1, 2, 1, 2, 1, 2]);
    /// ```
    ///
    /// A panic upon overflow: <br>溢出时为 panic：<br>
    ///
    /// ```should_panic
    /// // this will panic at runtime <br>这将在运行时 panic<br>
    /// b"0123456789abcdef".repeat(usize::MAX);
    /// ```
    #[rustc_allow_incoherent_impl]
    #[cfg(not(no_global_oom_handling))]
    #[stable(feature = "repeat_generic_slice", since = "1.40.0")]
    pub fn repeat(&self, n: usize) -> Vec<T>
    where
        T: Copy,
    {
        if n == 0 {
            return Vec::new();
        }

        // If `n` is larger than zero, it can be split as `n = 2^expn + rem (2^expn > rem, expn >= 0, rem >= 0)`. <br>如果 `n` 大于零，则可以将其拆分为 `n = 2^expn + rem (2^expn > rem, expn >= 0, rem >= 0)`。<br>
        // `2^expn` is the number represented by the leftmost '1' bit of `n`, and `rem` is the remaining part of `n`. <br>`2^expn` 是 `n` 最左边的 '1' 位所代表的数字，`rem` 是 `n` 的剩余部分。<br>
        //
        //

        // Using `Vec` to access `set_len()`. <br>使用 `Vec` 访问 `set_len()`。<br>
        let capacity = self.len().checked_mul(n).expect("capacity overflow");
        let mut buf = Vec::with_capacity(capacity);

        // `2^expn` repetition is done by doubling `buf` `expn`-times. <br>`2^expn` 重复是通过将 `buf` `expn` 次加倍来完成的。<br>
        buf.extend(self);
        {
            let mut m = n >> 1;
            // If `m > 0`, there are remaining bits up to the leftmost '1'. <br>如果是 `m > 0`，则剩余的位将保留到最左边的 '1'。<br>
            while m > 0 {
                // `buf.extend(buf)`:
                unsafe {
                    ptr::copy_nonoverlapping(
                        buf.as_ptr(),
                        (buf.as_mut_ptr() as *mut T).add(buf.len()),
                        buf.len(),
                    );
                    // `buf` has capacity of `self.len() * n`. <br>`buf` 的容量为 `self.len() * n`。<br>
                    let buf_len = buf.len();
                    buf.set_len(buf_len * 2);
                }

                m >>= 1;
            }
        }

        // `rem` (`= n - 2^expn`) repetition is done by copying first `rem` repetitions from `buf` itself. <br>`rem` (`= n - 2^expn`) 重复是通过从 `buf` 本身复制第一个 `rem` 重复来完成的。<br>
        //
        let rem_len = capacity - buf.len(); // `self.len() * rem`
        if rem_len > 0 {
            // `buf.extend(buf[0 .. rem_len])`:
            unsafe {
                // This is non-overlapping since `2^expn > rem`. <br>自 `2^expn > rem` 起，这是不重叠的。<br>
                ptr::copy_nonoverlapping(
                    buf.as_ptr(),
                    (buf.as_mut_ptr() as *mut T).add(buf.len()),
                    rem_len,
                );
                // `buf.len() + rem_len` equals to `buf.capacity()` (`= self.len() * n`). <br>`buf.len() + rem_len` 等于 `buf.capacity()` (`= self.len() * n`)。<br>
                buf.set_len(capacity);
            }
        }
        buf
    }

    /// Flattens a slice of `T` into a single value `Self::Output`. <br>将 `T` 的切片展平为单个值 `Self::Output`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// assert_eq!(["hello", "world"].concat(), "helloworld");
    /// assert_eq!([[1, 2], [3, 4]].concat(), [1, 2, 3, 4]);
    /// ```
    #[rustc_allow_incoherent_impl]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn concat<Item: ?Sized>(&self) -> <Self as Concat<Item>>::Output
    where
        Self: Concat<Item>,
    {
        Concat::concat(self)
    }

    /// Flattens a slice of `T` into a single value `Self::Output`, placing a given separator between each. <br>将 `T` 的切片展平为单个值 `Self::Output`，并在每个值之间放置一个给定的分隔符。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// assert_eq!(["hello", "world"].join(" "), "hello world");
    /// assert_eq!([[1, 2], [3, 4]].join(&0), [1, 2, 0, 3, 4]);
    /// assert_eq!([[1, 2], [3, 4]].join(&[0, 0][..]), [1, 2, 0, 0, 3, 4]);
    /// ```
    #[rustc_allow_incoherent_impl]
    #[stable(feature = "rename_connect_to_join", since = "1.3.0")]
    pub fn join<Separator>(&self, sep: Separator) -> <Self as Join<Separator>>::Output
    where
        Self: Join<Separator>,
    {
        Join::join(self, sep)
    }

    /// Flattens a slice of `T` into a single value `Self::Output`, placing a given separator between each. <br>将 `T` 的切片展平为单个值 `Self::Output`，并在每个值之间放置一个给定的分隔符。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// # #![allow(deprecated)]
    /// assert_eq!(["hello", "world"].connect(" "), "hello world");
    /// assert_eq!([[1, 2], [3, 4]].connect(&0), [1, 2, 0, 3, 4]);
    /// ```
    #[rustc_allow_incoherent_impl]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[deprecated(since = "1.3.0", note = "renamed to join")]
    pub fn connect<Separator>(&self, sep: Separator) -> <Self as Join<Separator>>::Output
    where
        Self: Join<Separator>,
    {
        Join::join(self, sep)
    }
}

#[cfg(not(test))]
impl [u8] {
    /// Returns a vector containing a copy of this slice where each byte is mapped to its ASCII upper case equivalent. <br>返回一个 vector，其中包含此切片的副本，其中每个字节都映射到其等效的 ASCII 大写字母。<br>
    ///
    ///
    /// ASCII letters 'a' to 'z' are mapped to 'A' to 'Z', but non-ASCII letters are unchanged. <br>ASCII 字母 'a' 到 'z' 映射到 'A' 到 'Z'，但是非 ASCII 字母不变。<br>
    ///
    /// To uppercase the value in-place, use [`make_ascii_uppercase`]. <br>要就地将值大写，请使用 [`make_ascii_uppercase`]。<br>
    ///
    /// [`make_ascii_uppercase`]: slice::make_ascii_uppercase
    ///
    #[cfg(not(no_global_oom_handling))]
    #[rustc_allow_incoherent_impl]
    #[must_use = "this returns the uppercase bytes as a new Vec, \
                  without modifying the original"]
    #[stable(feature = "ascii_methods_on_intrinsics", since = "1.23.0")]
    #[inline]
    pub fn to_ascii_uppercase(&self) -> Vec<u8> {
        let mut me = self.to_vec();
        me.make_ascii_uppercase();
        me
    }

    /// Returns a vector containing a copy of this slice where each byte is mapped to its ASCII lower case equivalent. <br>返回一个 vector，其中包含该切片的副本，其中每个字节均映射为其等效的 ASCII 小写字母。<br>
    ///
    ///
    /// ASCII letters 'A' to 'Z' are mapped to 'a' to 'z', but non-ASCII letters are unchanged. <br>ASCII 字母 'A' 到 'Z' 映射到 'a' 到 'z'，但是非 ASCII 字母不变。<br>
    ///
    /// To lowercase the value in-place, use [`make_ascii_lowercase`]. <br>要就地小写该值，请使用 [`make_ascii_lowercase`]。<br>
    ///
    /// [`make_ascii_lowercase`]: slice::make_ascii_lowercase
    ///
    #[cfg(not(no_global_oom_handling))]
    #[rustc_allow_incoherent_impl]
    #[must_use = "this returns the lowercase bytes as a new Vec, \
                  without modifying the original"]
    #[stable(feature = "ascii_methods_on_intrinsics", since = "1.23.0")]
    #[inline]
    pub fn to_ascii_lowercase(&self) -> Vec<u8> {
        let mut me = self.to_vec();
        me.make_ascii_lowercase();
        me
    }
}

////////////////////////////////////////////////////////////////////////////////
// Extension traits for slices over specific kinds of data <br>扩展 traits 用于切片特定类型的数据<br>
////////////////////////////////////////////////////////////////////////////////

/// Helper trait for [`[T]::concat`](slice::concat). <br>[`[T]::concat`](slice::concat) 的辅助程序 trait。<br>
///
/// Note: the `Item` type parameter is not used in this trait, but it allows impls to be more generic. <br>`Item` 类型参数未在此 trait 中使用，但它使 impls 更具泛型性。<br>
/// Without it, we get this error: <br>没有它，我们将收到此错误：<br>
///
/// ```error
/// error[E0207]: the type parameter `T` is not constrained by the impl trait, self type, or predica
///    --> src/liballoc/slice.rs:608:6
///     |
/// 608 | impl<T: Clone, V: Borrow<[T]>> Concat for [V] {
///     |      ^ unconstrained type parameter
/// ```
///
/// This is because there could exist `V` types with multiple `Borrow<[_]>` impls, such that multiple `T` types would apply: <br>这是因为可能存在具有多个 `Borrow<[_]>` 表示的 `V` 类型，因此将应用多个 `T` 类型：<br>
///
///
/// ```
/// # #[allow(dead_code)]
/// pub struct Foo(Vec<u32>, Vec<String>);
///
/// impl std::borrow::Borrow<[u32]> for Foo {
///     fn borrow(&self) -> &[u32] { &self.0 }
/// }
///
/// impl std::borrow::Borrow<[String]> for Foo {
///     fn borrow(&self) -> &[String] { &self.1 }
/// }
/// ```
///
#[unstable(feature = "slice_concat_trait", issue = "27747")]
pub trait Concat<Item: ?Sized> {
    #[unstable(feature = "slice_concat_trait", issue = "27747")]
    /// The resulting type after concatenation <br>串联后的结果类型<br>
    type Output;

    /// Implementation of [`[T]::concat`](slice::concat) <br>[`[T]::concat`](slice::concat) 的实现<br>
    #[unstable(feature = "slice_concat_trait", issue = "27747")]
    fn concat(slice: &Self) -> Self::Output;
}

/// Helper trait for [`[T]::join`](slice::join) <br>[`[T]::join`](slice::join) 的辅助 trait<br>
#[unstable(feature = "slice_concat_trait", issue = "27747")]
pub trait Join<Separator> {
    #[unstable(feature = "slice_concat_trait", issue = "27747")]
    /// The resulting type after concatenation <br>串联后的结果类型<br>
    type Output;

    /// Implementation of [`[T]::join`](slice::join) <br>[`[T]::join`](slice::join) 的实现<br>
    #[unstable(feature = "slice_concat_trait", issue = "27747")]
    fn join(slice: &Self, sep: Separator) -> Self::Output;
}

#[cfg(not(no_global_oom_handling))]
#[unstable(feature = "slice_concat_ext", issue = "27747")]
impl<T: Clone, V: Borrow<[T]>> Concat<T> for [V] {
    type Output = Vec<T>;

    fn concat(slice: &Self) -> Vec<T> {
        let size = slice.iter().map(|slice| slice.borrow().len()).sum();
        let mut result = Vec::with_capacity(size);
        for v in slice {
            result.extend_from_slice(v.borrow())
        }
        result
    }
}

#[cfg(not(no_global_oom_handling))]
#[unstable(feature = "slice_concat_ext", issue = "27747")]
impl<T: Clone, V: Borrow<[T]>> Join<&T> for [V] {
    type Output = Vec<T>;

    fn join(slice: &Self, sep: &T) -> Vec<T> {
        let mut iter = slice.iter();
        let first = match iter.next() {
            Some(first) => first,
            None => return vec![],
        };
        let size = slice.iter().map(|v| v.borrow().len()).sum::<usize>() + slice.len() - 1;
        let mut result = Vec::with_capacity(size);
        result.extend_from_slice(first.borrow());

        for v in iter {
            result.push(sep.clone());
            result.extend_from_slice(v.borrow())
        }
        result
    }
}

#[cfg(not(no_global_oom_handling))]
#[unstable(feature = "slice_concat_ext", issue = "27747")]
impl<T: Clone, V: Borrow<[T]>> Join<&[T]> for [V] {
    type Output = Vec<T>;

    fn join(slice: &Self, sep: &[T]) -> Vec<T> {
        let mut iter = slice.iter();
        let first = match iter.next() {
            Some(first) => first,
            None => return vec![],
        };
        let size =
            slice.iter().map(|v| v.borrow().len()).sum::<usize>() + sep.len() * (slice.len() - 1);
        let mut result = Vec::with_capacity(size);
        result.extend_from_slice(first.borrow());

        for v in iter {
            result.extend_from_slice(sep);
            result.extend_from_slice(v.borrow())
        }
        result
    }
}

////////////////////////////////////////////////////////////////////////////////
// Standard trait implementations for slices <br>切片的标准 trait 实现<br>
////////////////////////////////////////////////////////////////////////////////

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Borrow<[T]> for Vec<T> {
    fn borrow(&self) -> &[T] {
        &self[..]
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> BorrowMut<[T]> for Vec<T> {
    fn borrow_mut(&mut self) -> &mut [T] {
        &mut self[..]
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Clone> ToOwned for [T] {
    type Owned = Vec<T>;
    #[cfg(not(test))]
    fn to_owned(&self) -> Vec<T> {
        self.to_vec()
    }

    #[cfg(test)]
    fn to_owned(&self) -> Vec<T> {
        hack::to_vec(self, Global)
    }

    fn clone_into(&self, target: &mut Vec<T>) {
        // drop anything in target that will not be overwritten <br>丢弃目标中不会被覆盖的任何内容<br>
        target.truncate(self.len());

        // target.len <= self.len due to the truncate above, so the slices here are always in-bounds. <br>target.len <= self.len 由于上面的截断，所以这里的切片总是在边界内。<br>
        //
        let (init, tail) = self.split_at(target.len());

        // reuse the contained values' allocations/resources. <br>重用包含的值的 allocations/resources。<br>
        target.clone_from_slice(init);
        target.extend_from_slice(tail);
    }
}

////////////////////////////////////////////////////////////////////////////////
// Sorting
////////////////////////////////////////////////////////////////////////////////

/// Inserts `v[0]` into pre-sorted sequence `v[1..]` so that whole `v[..]` becomes sorted. <br>将 `v[0]` 插入到预排序序列 `v[1..]` 中，以便整个 `v[..]` 都被排序。<br>
///
/// This is the integral subroutine of insertion sort. <br>这是插入排序的必不可少的子例程。<br>
#[cfg(not(no_global_oom_handling))]
fn insert_head<T, F>(v: &mut [T], is_less: &mut F)
where
    F: FnMut(&T, &T) -> bool,
{
    if v.len() >= 2 && is_less(&v[1], &v[0]) {
        unsafe {
            // There are three ways to implement insertion here: <br>这里有三种实现插入的方法：<br>
            //
            // 1. Swap adjacent elements until the first one gets to its final destination. <br>交换相邻的元素，直到第一个到达其最终目的地。<br>
            //    However, this way we copy data around more than is necessary. <br>但是，这样一来，我们就可以复制不必要的数据。<br>
            //    If elements are big structures (costly to copy), this method will be slow. <br>如果元素是大型结构 (复制成本很高)，则此方法的速度会很慢。<br>
            //
            // 2. Iterate until the right place for the first element is found. <br>迭代直到找到第一个元素的正确位置。<br>
            // Then shift the elements succeeding it to make room for it and finally place it into the remaining hole. <br>然后，移动后继的元素为其腾出空间，最后将其放入剩余的 hole 中。<br>
            // This is a good method. <br>这是一个好方法。<br>
            //
            // 3. Copy the first element into a temporary variable. <br>将第一个元素复制到一个临时变量中。<br> Iterate until the right place for it is found. <br>迭代直到找到正确的位置。<br>
            // As we go along, copy every traversed element into the slot preceding it. <br>在继续操作时，将每个遍历的元素复制到它前面的插槽中。<br>
            // Finally, copy data from the temporary variable into the remaining hole. <br>最后，将数据从临时变量复制到剩余的 hole 中。<br>
            // This method is very good. <br>这个方法很好。<br>
            // Benchmarks demonstrated slightly better performance than with the 2nd method. <br>基准测试显示出比第二种方法更好的性能。<br>
            //
            // All methods were benchmarked, and the 3rd showed best results. <br>所有方法均进行了基准测试，第 3 种方法显示最佳结果。<br> So we chose that one. <br>因此，我们选择了那个。<br>
            let tmp = mem::ManuallyDrop::new(ptr::read(&v[0]));

            // Intermediate state of the insertion process is always tracked by `hole`, which serves two purposes: <br>`hole` 始终跟踪插入过程的中间状态，这有两个目的：<br>
            // 1. Protects integrity of `v` from panics in `is_less`. <br>从 `is_less` 中的 panics 保护 `v` 的完整性。<br>
            // 2. Fills the remaining hole in `v` in the end. <br>最后将 `v` 中剩余的 hole 填充。<br>
            //
            // Panic safety: <br>Panic 安全：<br>
            //
            // If `is_less` panics at any point during the process, `hole` will get dropped and fill the hole in `v` with `tmp`, thus ensuring that `v` still holds every object it initially held exactly once. <br>如果在此过程中的任何时候 `is_less` panics，`hole` 都会被丢弃，并用 `tmp` 填充 `v` 中的 hole，从而确保 `v` 仍将其最初持有的每个对象精确地保留一次。<br>
            //
            //
            //
            let mut hole = InsertionHole { src: &*tmp, dest: &mut v[1] };
            ptr::copy_nonoverlapping(&v[1], &mut v[0], 1);

            for i in 2..v.len() {
                if !is_less(&v[i], &*tmp) {
                    break;
                }
                ptr::copy_nonoverlapping(&v[i], &mut v[i - 1], 1);
                hole.dest = &mut v[i];
            }
            // `hole` gets dropped and thus copies `tmp` into the remaining hole in `v`. <br>`hole` 被丢弃，因此将 `tmp` 复制到 `v` 中剩余的 hole 中。<br>
        }
    }

    // When dropped, copies from `src` into `dest`. <br>丢弃时，从 `src` 复制到 `dest`。<br>
    struct InsertionHole<T> {
        src: *const T,
        dest: *mut T,
    }

    impl<T> Drop for InsertionHole<T> {
        fn drop(&mut self) {
            unsafe {
                ptr::copy_nonoverlapping(self.src, self.dest, 1);
            }
        }
    }
}

/// Merges non-decreasing runs `v[..mid]` and `v[mid..]` using `buf` as temporary storage, and stores the result into `v[..]`. <br>使用 `buf` 作为临时存储合并非递减运行 `v[..mid]` 和 `v[mid..]`，并将结果存储到 `v[..]` 中。<br>
///
/// # Safety
///
/// The two slices must be non-empty and `mid` must be in bounds. <br>这两个片必须是非空的，并且 `mid` 必须在范围之内。<br>
/// Buffer `buf` must be long enough to hold a copy of the shorter slice. <br>缓冲区 `buf` 必须足够长才能容纳较短切片的副本。<br>
/// Also, `T` must not be a zero-sized type. <br>另外，`T` 不能为零大小类型。<br>
#[cfg(not(no_global_oom_handling))]
unsafe fn merge<T, F>(v: &mut [T], mid: usize, buf: *mut T, is_less: &mut F)
where
    F: FnMut(&T, &T) -> bool,
{
    let len = v.len();
    let v = v.as_mut_ptr();
    let (v_mid, v_end) = unsafe { (v.add(mid), v.add(len)) };

    // The merge process first copies the shorter run into `buf`. <br>合并过程首先将较短的运行复制到 `buf` 中。<br>
    // Then it traces the newly copied run and the longer run forwards (or backwards), comparing their next unconsumed elements and copying the lesser (or greater) one into `v`. <br>然后，它将跟踪新复制的运行，以及向前运行 (或向后运行) 的较长运行，比较它们的下一个未消耗元素，并将较小 (或较大) 的运行复制到 `v` 中。<br>
    //
    // As soon as the shorter run is fully consumed, the process is done. <br>一旦较短的运行时间被完全用尽，该过程就完成了。<br> If the longer run gets consumed first, then we must copy whatever is left of the shorter run into the remaining hole in `v`. <br>如果较长的运行首先被消耗，那么我们必须将较短的运行剩下的任何内容复制到 `v` 中剩余的 hole 中。<br>
    //
    // Intermediate state of the process is always tracked by `hole`, which serves two purposes: <br>`hole` 始终跟踪过程的中间状态，这有两个目的：<br>
    // 1. Protects integrity of `v` from panics in `is_less`. <br>从 `is_less` 中的 panics 保护 `v` 的完整性。<br>
    // 2. Fills the remaining hole in `v` if the longer run gets consumed first. <br>如果较长时间的运行首先被消耗，则将 `v` 中剩余的 hole 填充。<br>
    //
    // Panic safety: <br>Panic 安全：<br>
    //
    // If `is_less` panics at any point during the process, `hole` will get dropped and fill the hole in `v` with the unconsumed range in `buf`, thus ensuring that `v` still holds every object it initially held exactly once. <br>如果在此过程中的任何时候 `is_less` panics，`hole` 都会丢弃并用 `buf` 中的未消耗范围填充 `v` 中的 hole，从而确保 `v` 仍将其最初持有的每个对象精确地保留一次。<br>
    //
    //
    //
    //
    //
    let mut hole;

    if mid <= len - mid {
        // The left run is shorter. <br>左边的运行更短。<br>
        unsafe {
            ptr::copy_nonoverlapping(v, buf, mid);
            hole = MergeHole { start: buf, end: buf.add(mid), dest: v };
        }

        // Initially, these pointers point to the beginnings of their arrays. <br>最初，这些指针指向其数组的开头。<br>
        let left = &mut hole.start;
        let mut right = v_mid;
        let out = &mut hole.dest;

        while *left < hole.end && right < v_end {
            // Consume the lesser side. <br>消耗较小的一面。<br>
            // If equal, prefer the left run to maintain stability. <br>如果相等，则选择左旋以保持稳定性。<br>
            unsafe {
                let to_copy = if is_less(&*right, &**left) {
                    get_and_increment(&mut right)
                } else {
                    get_and_increment(left)
                };
                ptr::copy_nonoverlapping(to_copy, get_and_increment(out), 1);
            }
        }
    } else {
        // The right run is shorter. <br>右边的运行更短<br>
        unsafe {
            ptr::copy_nonoverlapping(v_mid, buf, len - mid);
            hole = MergeHole { start: buf, end: buf.add(len - mid), dest: v_mid };
        }

        // Initially, these pointers point past the ends of their arrays. <br>最初，这些指针指向其数组的两端。<br>
        let left = &mut hole.dest;
        let right = &mut hole.end;
        let mut out = v_end;

        while v < *left && buf < *right {
            // Consume the greater side. <br>消耗更大的一面。<br>
            // If equal, prefer the right run to maintain stability. <br>如果相等，则选择正确的行程以保持稳定性。<br>
            unsafe {
                let to_copy = if is_less(&*right.offset(-1), &*left.offset(-1)) {
                    decrement_and_get(left)
                } else {
                    decrement_and_get(right)
                };
                ptr::copy_nonoverlapping(to_copy, decrement_and_get(&mut out), 1);
            }
        }
    }
    // Finally, `hole` gets dropped. <br>最后，`hole` 被丢弃。<br>
    // If the shorter run was not fully consumed, whatever remains of it will now be copied into the hole in `v`. <br>如果较短的运行没有被完全消耗，则其剩余的任何内容现在都将被复制到 `v` 的 hole 中。<br>

    unsafe fn get_and_increment<T>(ptr: &mut *mut T) -> *mut T {
        let old = *ptr;
        *ptr = unsafe { ptr.offset(1) };
        old
    }

    unsafe fn decrement_and_get<T>(ptr: &mut *mut T) -> *mut T {
        *ptr = unsafe { ptr.offset(-1) };
        *ptr
    }

    // When dropped, copies the range `start..end` into `dest..`. <br>丢弃时，将范围 `start..end` 复制到 `dest..`。<br>
    struct MergeHole<T> {
        start: *mut T,
        end: *mut T,
        dest: *mut T,
    }

    impl<T> Drop for MergeHole<T> {
        fn drop(&mut self) {
            // `T` is not a zero-sized type, and these are pointers into a slice's elements. <br>`T` 不是零大小的类型，它们是指向切片元素的指针。<br>
            unsafe {
                let len = self.end.sub_ptr(self.start);
                ptr::copy_nonoverlapping(self.start, self.dest, len);
            }
        }
    }
}

/// This merge sort borrows some (but not all) ideas from TimSort, which is described in detail [here](https://github.com/python/cpython/blob/main/Objects/listsort.txt). <br>这个归并排序借用了 TimSort 的一些 (但不是全部) 想法，在 [这里](https://github.com/python/cpython/blob/main/Objects/listsort.txt) 有详细描述。<br>
///
///
/// The algorithm identifies strictly descending and non-descending subsequences, which are called natural runs. <br>该算法识别严格降序和非降序的子序列，这些子序列称为自然行程。<br> There is a stack of pending runs yet to be merged. <br>有待合并的待处理运行栈。<br>
/// Each newly found run is pushed onto the stack, and then some pairs of adjacent runs are merged until these two invariants are satisfied: <br>将每个新发现的运行推入栈中，然后合并一些相邻的运行对，直到这两个不变量得到满足：<br>
///
/// 1. for every `i` in `1..runs.len()`: `runs[i - 1].len > runs[i].len` <br>对于 `1..runs.len()` 中的每个 `i`: `runs[i - 1].len > runs[i].len`<br>
/// 2. for every `i` in `2..runs.len()`: `runs[i - 2].len > runs[i - 1].len + runs[i].len` <br>对于 `2..runs.len()` 中的每个 `i`: `runs[i - 2].len > runs[i - 1].len + runs[i].len`<br>
///
/// The invariants ensure that the total running time is *O*(*n* \* log(*n*)) worst-case. <br>不变量确保最坏情况下的总运行时间为 *O*(*n*\*log(* n*))。<br>
///
///
#[cfg(not(no_global_oom_handling))]
fn merge_sort<T, F>(v: &mut [T], mut is_less: F)
where
    F: FnMut(&T, &T) -> bool,
{
    // Slices of up to this length get sorted using insertion sort. <br>长度不超过此长度的切片将使用插入排序进行排序。<br>
    const MAX_INSERTION: usize = 20;
    // Very short runs are extended using insertion sort to span at least this many elements. <br>使用插入排序可扩展非常短的运行时间，以至少涵盖这么多的元素。<br>
    const MIN_RUN: usize = 10;

    // Sorting has no meaningful behavior on zero-sized types. <br>零大小类型的排序没有有意义的行为。<br>
    if size_of::<T>() == 0 {
        return;
    }

    let len = v.len();

    // Short arrays get sorted in-place via insertion sort to avoid allocations. <br>短数组通过插入排序进行就地排序，以避免分配。<br>
    if len <= MAX_INSERTION {
        if len >= 2 {
            for i in (0..len - 1).rev() {
                insert_head(&mut v[i..], &mut is_less);
            }
        }
        return;
    }

    // Allocate a buffer to use as scratch memory. <br>分配缓冲区以用作暂存器。<br> We keep the length 0 so we can keep in it shallow copies of the contents of `v` without risking the dtors running on copies if `is_less` panics. <br>我们将长度保持为 0，这样就可以在其中保留 `v` 内容的浅表副本，而不会冒 `is_less` panics 在副本上运行 dtor 的风险。<br>
    //
    // When merging two sorted runs, this buffer holds a copy of the shorter run, which will always have length at most `len / 2`. <br>合并两个已排序的运行时，此缓冲区将保存一个较短运行的副本，该副本的长度始终最多为 `len / 2`。<br>
    //
    let mut buf = Vec::with_capacity(len / 2);

    // In order to identify natural runs in `v`, we traverse it backwards. <br>为了识别 `v` 中的自然行程，我们将其向后移动。<br>
    // That might seem like a strange decision, but consider the fact that merges more often go in the opposite direction (forwards). <br>这看起来似乎是一个奇怪的决定，但请考虑以下事实：合并更多是朝相反的方向 (forwards) 进行。<br>
    // According to benchmarks, merging forwards is slightly faster than merging backwards. <br>根据基准，向前合并比向后合并要快一些。<br>
    // To conclude, identifying runs by traversing backwards improves performance. <br>总而言之，通过向后遍历来识别运行可提高性能。<br>
    let mut runs = vec![];
    let mut end = len;
    while end > 0 {
        // Find the next natural run, and reverse it if it's strictly descending. <br>找到下一个自然行程，如果严格下降，则将其反转。<br>
        let mut start = end - 1;
        if start > 0 {
            start -= 1;
            unsafe {
                if is_less(v.get_unchecked(start + 1), v.get_unchecked(start)) {
                    while start > 0 && is_less(v.get_unchecked(start), v.get_unchecked(start - 1)) {
                        start -= 1;
                    }
                    v[start..end].reverse();
                } else {
                    while start > 0 && !is_less(v.get_unchecked(start), v.get_unchecked(start - 1))
                    {
                        start -= 1;
                    }
                }
            }
        }

        // Insert some more elements into the run if it's too short. <br>如果过短，请在运行中插入更多元素。<br>
        // Insertion sort is faster than merge sort on short sequences, so this significantly improves performance. <br>插入排序比短序列上的合并排序要快，因此可以显着提高性能。<br>
        while start > 0 && end - start < MIN_RUN {
            start -= 1;
            insert_head(&mut v[start..end], &mut is_less);
        }

        // Push this run onto the stack. <br>将此运行推入栈。<br>
        runs.push(Run { start, len: end - start });
        end = start;

        // Merge some pairs of adjacent runs to satisfy the invariants. <br>合并一些成对的相邻行程以满足不变性。<br>
        while let Some(r) = collapse(&runs) {
            let left = runs[r + 1];
            let right = runs[r];
            unsafe {
                merge(
                    &mut v[left.start..right.start + right.len],
                    left.len,
                    buf.as_mut_ptr(),
                    &mut is_less,
                );
            }
            runs[r] = Run { start: left.start, len: left.len + right.len };
            runs.remove(r + 1);
        }
    }

    // Finally, exactly one run must remain in the stack. <br>最后，必须在栈中仅保留一次运行。<br>
    debug_assert!(runs.len() == 1 && runs[0].start == 0 && runs[0].len == len);

    // Examines the stack of runs and identifies the next pair of runs to merge. <br>检查运行栈，并确定要合并的下一对运行。<br>
    // More specifically, if `Some(r)` is returned, that means `runs[r]` and `runs[r + 1]` must be merged next. <br>更具体地说，如果返回 `Some(r)`，则意味着接下来必须合并 `runs[r]` 和 `runs[r + 1]`。<br>
    // If the algorithm should continue building a new run instead, `None` is returned. <br>如果算法应继续构建新的运行，则返回 `None`。<br>
    //
    // TimSort is infamous for its buggy implementations, as described here: <br>TimSort 因其 buggy 实现而臭名昭著，如下所述：<br>
    // http://envisage-project.eu/timsort-specification-and-verification/
    //
    // The gist of the story is: we must enforce the invariants on the top four runs on the stack. <br>这个故事的重点是：我们必须在栈的前四次运行中强制执行不变量。<br>
    // Enforcing them on just top three is not sufficient to ensure that the invariants will still hold for *all* runs in the stack. <br>仅在前三个上强制执行它们不足以确保不变量仍然适用于栈中的所有运行。<br>
    //
    // This function correctly checks invariants for the top four runs. <br>此函数正确检查前四次运行的不变量。<br>
    // Additionally, if the top run starts at index 0, it will always demand a merge operation until the stack is fully collapsed, in order to complete the sort. <br>另外，如果最高运行从索引 0 开始，它将始终要求合并操作，直到栈完全展开为止，以完成排序。<br>
    //
    //
    #[inline]
    fn collapse(runs: &[Run]) -> Option<usize> {
        let n = runs.len();
        if n >= 2
            && (runs[n - 1].start == 0
                || runs[n - 2].len <= runs[n - 1].len
                || (n >= 3 && runs[n - 3].len <= runs[n - 2].len + runs[n - 1].len)
                || (n >= 4 && runs[n - 4].len <= runs[n - 3].len + runs[n - 2].len))
        {
            if n >= 3 && runs[n - 3].len < runs[n - 1].len { Some(n - 3) } else { Some(n - 2) }
        } else {
            None
        }
    }

    #[derive(Clone, Copy)]
    struct Run {
        start: usize,
        len: usize,
    }
}
