//! A UTF-8–encoded, growable string. <br>一个 UTF-8 编码的可增长字符串。<br>
//!
//! This module contains the [`String`] type, the [`ToString`] trait for converting to strings, and several error types that may result from working with [`String`]s. <br>该模块包含 [`String`] 类型，用于转换为字符串的 [`ToString`] trait 以及使用 [`String`] 可能导致的几种错误类型。<br>
//!
//!
//! # Examples
//!
//! There are multiple ways to create a new [`String`] from a string literal: <br>有多种方法可从字符串字面量创建新的 [`String`]：<br>
//!
//! ```
//! let s = "Hello".to_string();
//!
//! let s = String::from("world");
//! let s: String = "also this".into();
//! ```
//!
//! You can create a new [`String`] from an existing one by concatenating with <br>您可以通过与现有的 [`String`] 串联来创建一个新的 [`String`]。<br>
//! `+`:
//!
//! ```
//! let s = "Hello".to_string();
//!
//! let message = s + " world!";
//! ```
//!
//! If you have a vector of valid UTF-8 bytes, you can make a [`String`] out of it. <br>如果您有一个有效的 UTF-8 字节 vector，则可以用它制作一个 [`String`]。<br> You can do the reverse too. <br>您也可以做相反的事情。<br>
//!
//! ```
//! let sparkle_heart = vec![240, 159, 146, 150];
//!
//! // We know these bytes are valid, so we'll use `unwrap()`. <br>我们知道这些字节是有效的，因此我们将使用 `unwrap()`。<br>
//! let sparkle_heart = String::from_utf8(sparkle_heart).unwrap();
//!
//! assert_eq!("💖", sparkle_heart);
//!
//! let bytes = sparkle_heart.into_bytes();
//!
//! assert_eq!(bytes, [240, 159, 146, 150]);
//! ```
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

#[cfg(not(no_global_oom_handling))]
use core::char::{decode_utf16, REPLACEMENT_CHARACTER};
use core::fmt;
use core::hash;
#[cfg(not(no_global_oom_handling))]
use core::iter::FromIterator;
use core::iter::{from_fn, FusedIterator};
#[cfg(not(no_global_oom_handling))]
use core::ops::Add;
#[cfg(not(no_global_oom_handling))]
use core::ops::AddAssign;
#[cfg(not(no_global_oom_handling))]
use core::ops::Bound::{Excluded, Included, Unbounded};
use core::ops::{self, Index, IndexMut, Range, RangeBounds};
use core::ptr;
use core::slice;
#[cfg(not(no_global_oom_handling))]
use core::str::lossy;
use core::str::pattern::Pattern;

#[cfg(not(no_global_oom_handling))]
use crate::borrow::{Cow, ToOwned};
use crate::boxed::Box;
use crate::collections::TryReserveError;
use crate::str::{self, Chars, Utf8Error};
#[cfg(not(no_global_oom_handling))]
use crate::str::{from_boxed_utf8_unchecked, FromStr};
use crate::vec::Vec;

/// A UTF-8–encoded, growable string. <br>一个 UTF-8 编码的可增长字符串。<br>
///
/// The `String` type is the most common string type that has ownership over the contents of the string. <br>`String` 类型是最常见的字符串类型，拥有对该字符串内容的所有权。<br> It has a close relationship with its borrowed counterpart, the primitive [`str`]. <br>它与其借用的对应物，原始的 [`str`] 有着密切的关系。<br>
///
/// # Examples
///
/// You can create a `String` from [a literal string][`&str`] with [`String::from`]: <br>您可以使用 [`String::from`] 从一个 [字符串字面量][`&str`] 创建一个 `String`：<br>
///
/// [`String::from`]: From::from
///
/// ```
/// let hello = String::from("Hello, world!");
/// ```
///
/// You can append a [`char`] to a `String` with the [`push`] method, and append a [`&str`] with the [`push_str`] method: <br>您可以使用 [`push`] 方法将 [`char`] 追加到 `String` 上，并使用 [`push_str`] 方法追加 [`&str`]：<br>
///
/// ```
/// let mut hello = String::from("Hello, ");
///
/// hello.push('w');
/// hello.push_str("orld!");
/// ```
///
/// [`push`]: String::push
/// [`push_str`]: String::push_str
///
/// If you have a vector of UTF-8 bytes, you can create a `String` from it with the [`from_utf8`] method: <br>如果具有 UTF-8 字节的 vector，则可以使用 [`from_utf8`] 方法从中创建一个 `String`：<br>
///
/// ```
/// // some bytes, in a vector <br>vector 中的一些字节<br>
/// let sparkle_heart = vec![240, 159, 146, 150];
///
/// // We know these bytes are valid, so we'll use `unwrap()`. <br>我们知道这些字节是有效的，因此我们将使用 `unwrap()`。<br>
/// let sparkle_heart = String::from_utf8(sparkle_heart).unwrap();
///
/// assert_eq!("💖", sparkle_heart);
/// ```
///
/// [`from_utf8`]: String::from_utf8
///
/// # UTF-8
///
/// `String`s are always valid UTF-8. <br>`String`s 始终是有效的 UTF-8。<br> If you need a non-UTF-8 string, consider [`OsString`]. <br>如果您需要非 UTF-8 字符串，请考虑 [`OsString`]。<br> It is similar, but without the UTF-8 constraint. <br>它是相似的，但是没有 UTF-8 约束。<br> Because UTF-8 is a variable width encoding, `String`s are typically smaller than an array of the same `chars`: <br>因为 UTF-8 是可变宽度的编码，`String`s 通常小于相同 `chars` 的数组:<br>
///
/// ```
/// use std::mem;
///
/// // `s` is ASCII which represents each `char` as one byte <br>`s` 是 ASCII 码，它表示每个 `char` 为一个字节<br>
/// let s = "hello";
/// assert_eq!(s.len(), 5);
///
/// // A `char` array with the same contents would be longer because every `char` is four bytes <br>具有相同内容的 `char` 数组会更长，因为每个 `char` 都是四个字节<br>
/////
/// let s = ['h', 'e', 'l', 'l', 'o'];
/// let size: usize = s.into_iter().map(|c| mem::size_of_val(&c)).sum();
/// assert_eq!(size, 20);
///
/// // However, for non-ASCII strings, the difference will be smaller and sometimes they are the same <br>但是，对于非 ASCII 字符串，差异会更小，有时它们是相同的<br>
/////
/// let s = "💖💖💖💖💖";
/// assert_eq!(s.len(), 20);
///
/// let s = ['💖', '💖', '💖', '💖', '💖'];
/// let size: usize = s.into_iter().map(|c| mem::size_of_val(&c)).sum();
/// assert_eq!(size, 20);
/// ```
///
/// This raises interesting questions as to how `s[i]` should work. <br>这就提出了一些有趣的问题，比如 `s[i]` 应该如何工作。<br>
/// What should `i` be here? <br>`i` 在这里应该是什么?<br> Several options include byte indices and `char` indices but, because of UTF-8 encoding, only byte indices would provide constant time indexing. <br>几个选项包括字节索引和 `char` 索引，但由于 UTF-8 编码，只有字节索引会提供时间特性索引。<br> Getting the `i`th `char`, for example, is available using [`chars`]: <br>例如，可以使用 [`chars`] 获取第 i 个 `char`:<br>
///
/// ```
/// let s = "hello";
/// let third_character = s.chars().nth(2);
/// assert_eq!(third_character, Some('l'));
///
/// let s = "💖💖💖💖💖";
/// let third_character = s.chars().nth(2);
/// assert_eq!(third_character, Some('💖'));
/// ```
///
/// Next, what should `s[i]` return? <br>接下来，`s[i]` 应该返回什么?<br> Because indexing returns a reference to underlying data it could be `&u8`, `&[u8]`, or something else similar. <br>因为索引会返回对，底层，数据的引用，所以它可能是 `&u8`、`&[u8]` 或其他类似的东西。<br>
/// Since we're only providing one index, `&u8` makes the most sense but that might not be what the user expects and can be explicitly achieved with <br>由于我们只提供一个索引，`&u8` 是最有意义的，但这可能不是用户所期望的，并且可以通过<br>
/// [`as_bytes()`]:
///
/// ```
/// // The first byte is 104 - the byte value of `'h'` <br>第一个字节是 104-`'h'` 的字节值<br>
/// let s = "hello";
/// assert_eq!(s.as_bytes()[0], 104);
/// // or
/// assert_eq!(s.as_bytes()[0], b'h');
///
/// // The first byte is 240 which isn't obviously useful <br>第一个字节是 240 这显然没有用<br>
/// let s = "💖💖💖💖💖";
/// assert_eq!(s.as_bytes()[0], 240);
/// ```
///
/// Due to these ambiguities/restrictions, indexing with a `usize` is simply forbidden: <br>由于这些歧义或限制，简单地禁止使用 `usize` 进行索引:<br>
///
/// ```compile_fail,E0277
/// let s = "hello";
///
/// // The following will not compile! <br>以下内容将不会编译!<br>
/// println!("The first letter of s is {}", s[0]);
/// ```
///
/// It is more clear, however, how `&s[i..j]` should work (that is, indexing with a range). <br>然而，更清楚的是 `&s[i..j]` 应该如何工作 (即，使用范围进行索引)。<br> It should accept byte indices (to be constant-time) and return a `&str` which is UTF-8 encoded. <br>它应该接受字节索引 (作为特征时间) 并返回一个 `&str`，它是 UTF-8 编码的。<br> This is also called "string slicing". <br>这也称为 "字符串切片"。<br>
/// Note this will panic if the byte indices provided are not character boundaries - see [`is_char_boundary`] for more details. <br>请注意，如果提供的字节索引不是字符边界，这将导致 panic - 有关更多详细信息，请参见 [`is_char_boundary`]。<br> See the implementations for [`SliceIndex<str>`] for more details on string slicing. <br>有关字符串切片的更多详细信息，请参见 [`SliceIndex<str>`] 的实现。<br> For a non-panicking version of string slicing, see [`get`]. <br>有关字符串切片的非 panic 版本，请参见 [`get`]。<br>
///
/// [`OsString`]: ../../std/ffi/struct.OsString.html "ffi::OsString"
/// [`SliceIndex<str>`]: core::slice::SliceIndex
/// [`as_bytes()`]: str::as_bytes
/// [`get`]: str::get
/// [`is_char_boundary`]: str::is_char_boundary
///
/// The [`bytes`] and [`chars`] methods return iterators over the bytes and codepoints of the string, respectively. <br>[`bytes`] 和 [`chars`] 方法分别返回字符串的字节和代码点的迭代器。<br> To iterate over codepoints along with byte indices, use [`char_indices`]. <br>要迭代代码点和字节索引，请使用 [`char_indices`]。<br>
///
/// [`bytes`]: str::bytes
/// [`chars`]: str::chars
/// [`char_indices`]: str::char_indices
///
/// # Deref
///
/// `String` implements <code>[Deref]<Target = [str]></code>, and so inherits all of [`str`]'s methods. <br>`String` 实现了 <code>[Deref]<Target = [str]></code>，因此继承了 [`str`] 的所有方法。<br> In addition, this means that you can pass a `String` to a function which takes a [`&str`] by using an ampersand (`&`): <br>另外，这意味着您可以使用与号 (`&`) 将 `String` 传递给采用 [`&str`] 的函数：<br>
///
/// ```
/// fn takes_str(s: &str) { }
///
/// let s = String::from("Hello");
///
/// takes_str(&s);
/// ```
///
/// This will create a [`&str`] from the `String` and pass it in. <br>这将从 `String` 创建一个 [`&str`]，并将其传入。<br> This conversion is very inexpensive, and so generally, functions will accept [`&str`]s as arguments unless they need a `String` for some specific reason. <br>这种转换非常便宜，因此通常，函数会接受 [`&str`] 作为参数，除非出于某些特定原因它们需要 `String`。<br>
///
/// In certain cases Rust doesn't have enough information to make this conversion, known as [`Deref`] coercion. <br>在某些情况下，Rust 没有足够的信息来进行此转换，称为 [`Deref`] 强制多态。<br> In the following example a string slice [`&'a str`][`&str`] implements the trait `TraitExample`, and the function `example_func` takes anything that implements the trait. <br>在以下示例中，字符串切片 [`&'a str`][`&str`] 实现 `TraitExample` trait，函数 `example_func` 接受实现 trait 的所有内容。<br>
/// In this case Rust would need to make two implicit conversions, which Rust doesn't have the means to do. <br>在这种情况下，Rust 将需要进行两次隐式转换，而 Rust 没有办法进行转换。<br>
/// For that reason, the following example will not compile. <br>因此，以下示例将无法编译。<br>
///
/// ```compile_fail,E0277
/// trait TraitExample {}
///
/// impl<'a> TraitExample for &'a str {}
///
/// fn example_func<A: TraitExample>(example_arg: A) {}
///
/// let example_string = String::from("example_string");
/// example_func(&example_string);
/// ```
///
/// There are two options that would work instead. <br>有两种选择可以代替。<br> The first would be to change the line `example_func(&example_string);` to `example_func(example_string.as_str());`, using the method [`as_str()`] to explicitly extract the string slice containing the string. <br>第一种是使用方法 [`as_str()`] 显式提取包含该字符串的字符串切片，从而将 `example_func(&example_string);` 行更改为 `example_func(example_string.as_str());`。<br>
/// The second way changes `example_func(&example_string);` to `example_func(&*example_string);`. <br>第二种方法将 `example_func(&example_string);` 更改为 `example_func(&*example_string);`。<br>
/// In this case we are dereferencing a `String` to a [`str`], then referencing the [`str`] back to [`&str`]. <br>在这种情况下，我们将 `String` 解引用到 [`str`]，然后将 [`str`] 引用回 [`&str`]。<br>
/// The second way is more idiomatic, however both work to do the conversion explicitly rather than relying on the implicit conversion. <br>第二种方法更惯用，但是两种方法都可以显式地进行转换，而不是依赖于隐式转换。<br>
///
/// # Representation
///
/// A `String` is made up of three components: a pointer to some bytes, a length, and a capacity. <br>`String` 由三个部分组成：指向某些字节的指针，长度和容量。<br> The pointer points to an internal buffer `String` uses to store its data. <br>指针指向 `String` 用于存储其数据的内部缓冲区。<br> The length is the number of bytes currently stored in the buffer, and the capacity is the size of the buffer in bytes. <br>长度是当前存储在缓冲区中的字节数，容量是缓冲区的大小 (以字节为单位)。<br>
///
/// As such, the length will always be less than or equal to the capacity. <br>这样，长度将始终小于或等于容量。<br>
///
/// This buffer is always stored on the heap. <br>此缓冲区始终存储在堆中。<br>
///
/// You can look at these with the [`as_ptr`], [`len`], and [`capacity`] methods: <br>您可以使用 [`as_ptr`]，[`len`] 和 [`capacity`] 方法查看它们：<br>
///
/// ```
/// use std::mem;
///
/// let story = String::from("Once upon a time...");
///
// FIXME Update this when vec_into_raw_parts is stabilized <br>在 vec_into_raw_parts 稳定后更新它<br>
/// // Prevent automatically dropping the String's data <br>防止自动丢弃字符串的数据<br>
/// let mut story = mem::ManuallyDrop::new(story);
///
/// let ptr = story.as_mut_ptr();
/// let len = story.len();
/// let capacity = story.capacity();
///
/// // story has nineteen bytes <br>story 有十九个字节<br>
/// assert_eq!(19, len);
///
/// // We can re-build a String out of ptr, len, and capacity. <br>我们可以用 ptr，len 和 capacity 重新构建一个 String。<br>
/// // This is all unsafe because we are responsible for making sure the components are valid: <br>这都是不安全的，因为我们有责任确保组件有效：<br>
/////
/// let s = unsafe { String::from_raw_parts(ptr, len, capacity) } ;
///
/// assert_eq!(String::from("Once upon a time..."), s);
/// ```
///
/// [`as_ptr`]: str::as_ptr
/// [`len`]: String::len
/// [`capacity`]: String::capacity
///
/// If a `String` has enough capacity, adding elements to it will not re-allocate. <br>如果 `String` 具有足够的容量，则向其添加元素将不会重新分配。<br> For example, consider this program: <br>例如，考虑以下程序：<br>
///
/// ```
/// let mut s = String::new();
///
/// println!("{}", s.capacity());
///
/// for _ in 0..5 {
///     s.push_str("hello");
///     println!("{}", s.capacity());
/// }
/// ```
///
/// This will output the following: <br>这将输出以下内容：<br>
///
/// ```text
/// 0
/// 5
/// 10
/// 20
/// 20
/// 40
/// ```
///
/// At first, we have no memory allocated at all, but as we append to the string, it increases its capacity appropriately. <br>最初，我们根本没有分配任何内存，但是当我们追加到字符串后，它会适当地增加其容量。<br> If we instead use the [`with_capacity`] method to allocate the correct capacity initially: <br>如果我们改为使用 [`with_capacity`] 方法来初始分配正确的容量，请执行以下操作：<br>
///
/// ```
/// let mut s = String::with_capacity(25);
///
/// println!("{}", s.capacity());
///
/// for _ in 0..5 {
///     s.push_str("hello");
///     println!("{}", s.capacity());
/// }
/// ```
///
/// [`with_capacity`]: String::with_capacity
///
/// We end up with a different output: <br>我们最终得到了不同的输出：<br>
///
/// ```text
/// 25
/// 25
/// 25
/// 25
/// 25
/// 25
/// ```
///
/// Here, there's no need to allocate more memory inside the loop. <br>在这里，不需要在循环内分配更多的内存。<br>
///
/// [str]: prim@str "str"
/// [`str`]: prim@str "str"
/// [`&str`]: prim@str "&str"
/// [Deref]: core::ops::Deref "ops::Deref"
/// [`Deref`]: core::ops::Deref "ops::Deref"
/// [`as_str()`]: String::as_str
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
#[derive(PartialOrd, Eq, Ord)]
#[cfg_attr(not(test), rustc_diagnostic_item = "String")]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct String {
    vec: Vec<u8>,
}

/// A possible error value when converting a `String` from a UTF-8 byte vector. <br>从 UTF-8 字节 vector 转换 `String` 时可能的错误值。<br>
///
/// This type is the error type for the [`from_utf8`] method on [`String`]. <br>该类型是 [`String`] 上 [`from_utf8`] 方法的错误类型。<br>
/// It is designed in such a way to carefully avoid reallocations: the [`into_bytes`] method will give back the byte vector that was used in the conversion attempt. <br>它的设计方式旨在避免重新分配: [`into_bytes`] 方法将返回转换尝试中使用的字节 vector。<br>
///
///
/// [`from_utf8`]: String::from_utf8
/// [`into_bytes`]: FromUtf8Error::into_bytes
///
/// The [`Utf8Error`] type provided by [`std::str`] represents an error that may occur when converting a slice of [`u8`]s to a [`&str`]. <br>[`std::str`] 提供的 [`Utf8Error`] 类型表示将 [`u8`] 的切片转换为 [`&str`] 时可能发生的错误。<br>
/// In this sense, it's an analogue to `FromUtf8Error`, and you can get one from a `FromUtf8Error` through the [`utf8_error`] method. <br>从这个意义上讲，它是 `FromUtf8Error` 的类似物，您可以通过 [`utf8_error`] 方法从 `FromUtf8Error` 中获得一个。<br>
///
/// [`Utf8Error`]: str::Utf8Error "std::str::Utf8Error"
/// [`std::str`]: core::str "std::str"
/// [`&str`]: prim@str "&str"
/// [`utf8_error`]: FromUtf8Error::utf8_error
///
/// # Examples
///
/// Basic usage: <br>基本用法：<br>
///
/// ```
/// // some invalid bytes, in a vector <br>vector 中的一些无效字节<br>
/// let bytes = vec![0, 159];
///
/// let value = String::from_utf8(bytes);
///
/// assert!(value.is_err());
/// assert_eq!(vec![0, 159], value.unwrap_err().into_bytes());
/// ```
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(no_global_oom_handling), derive(Clone))]
#[derive(Debug, PartialEq, Eq)]
pub struct FromUtf8Error {
    bytes: Vec<u8>,
    error: Utf8Error,
}

/// A possible error value when converting a `String` from a UTF-16 byte slice. <br>从 UTF-16 字节切片转换 `String` 时可能的错误值。<br>
///
/// This type is the error type for the [`from_utf16`] method on [`String`]. <br>该类型是 [`String`] 上 [`from_utf16`] 方法的错误类型。<br>
///
/// [`from_utf16`]: String::from_utf16
/// # Examples
///
/// Basic usage: <br>基本用法：<br>
///
/// ```
/// // 𝄞mu<invalid>ic
/// let v = &[0xD834, 0xDD1E, 0x006d, 0x0075,
///           0xD800, 0x0069, 0x0063];
///
/// assert!(String::from_utf16(v).is_err());
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct FromUtf16Error(());

impl String {
    /// Creates a new empty `String`. <br>创建一个新的空 `String`。<br>
    ///
    /// Given that the `String` is empty, this will not allocate any initial buffer. <br>由于 `String` 为空，因此不会分配任何初始缓冲区。<br> While that means that this initial operation is very inexpensive, it may cause excessive allocation later when you add data. <br>虽然这意味着该初始操作非常便宜，但在以后添加数据时可能会导致过多的分配。<br>
    ///
    /// If you have an idea of how much data the `String` will hold, consider the [`with_capacity`] method to prevent excessive re-allocation. <br>如果您对 `String` 可以容纳多少数据有所了解，请考虑使用 [`with_capacity`] 方法来防止过多的重新分配。<br>
    ///
    /// [`with_capacity`]: String::with_capacity
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let s = String::new();
    /// ```
    ///
    ///
    ///
    #[inline]
    #[rustc_const_stable(feature = "const_string_new", since = "1.39.0")]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub const fn new() -> String {
        String { vec: Vec::new() }
    }

    /// Creates a new empty `String` with at least the specified capacity. <br>创建一个至少具有指定容量的新空 `String`。<br>
    ///
    /// `String`s have an internal buffer to hold their data. <br>`String` 有一个内部缓冲区来保存其数据。<br>
    /// The capacity is the length of that buffer, and can be queried with the [`capacity`] method. <br>容量是该缓冲区的长度，可以使用 [`capacity`] 方法进行查询。<br>
    /// This method creates an empty `String`, but one with an initial buffer that can hold at least `capacity` bytes. <br>此方法创建一个空的 `String`，但它的初始缓冲区至少可容纳 `capacity` 字节。<br>
    /// This is useful when you may be appending a bunch of data to the `String`, reducing the number of reallocations it needs to do. <br>当您可能将一堆数据追加到 `String` 时，这很有用，从而减少了它需要进行的重新分配的次数。<br>
    ///
    ///
    /// [`capacity`]: String::capacity
    ///
    /// If the given capacity is `0`, no allocation will occur, and this method is identical to the [`new`] method. <br>如果给定的容量为 `0`，则不会进行分配，并且此方法与 [`new`] 方法相同。<br>
    ///
    /// [`new`]: String::new
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::with_capacity(10);
    ///
    /// // The String contains no chars, even though it has capacity for more <br>字符串不包含任何字符，即使它可以容纳更多字符<br>
    /// assert_eq!(s.len(), 0);
    ///
    /// // These are all done without reallocating... <br>这些都无需重新分配即可完成...<br>
    /// let cap = s.capacity();
    /// for _ in 0..10 {
    ///     s.push('a');
    /// }
    ///
    /// assert_eq!(s.capacity(), cap);
    ///
    /// // ...but this may make the string reallocate <br>... 但这可能会使字符串重新分配<br>
    /// s.push('a');
    /// ```
    ///
    ///
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn with_capacity(capacity: usize) -> String {
        String { vec: Vec::with_capacity(capacity) }
    }

    // HACK(japaric): with cfg(test) the inherent `[T]::to_vec` method, which is required for this method definition, is not available. <br>对于 cfg(test)，此方法定义所需的固有 `[T]::to_vec` 方法不可用。<br>
    // Since we don't require this method for testing purposes, I'll just stub it <br>由于我们不需要出于测试目的使用此方法，因此我将其存根。<br>
    // NB see the slice::hack module in slice.rs for more information <br>有关更多信息，请参见 slice.rs 中的 slice::hack 模块<br>
    //
    #[inline]
    #[cfg(test)]
    pub fn from_str(_: &str) -> String {
        panic!("not available with cfg(test)");
    }

    /// Converts a vector of bytes to a `String`. <br>将字节的 vector 转换为 `String`。<br>
    ///
    /// A string ([`String`]) is made of bytes ([`u8`]), and a vector of bytes ([`Vec<u8>`]) is made of bytes, so this function converts between the two. <br>字符串 ([`String`]) 由字节 ([`u8`]) 组成，字节 ([`Vec<u8>`]) 的 vector 由字节组成，因此此函数在两者之间进行转换。<br>
    /// Not all byte slices are valid `String`s, however: `String` requires that it is valid UTF-8. <br>并非所有的字节片都是有效的 `String`，但是: `String` 要求它是有效的 UTF-8。<br>
    /// `from_utf8()` checks to ensure that the bytes are valid UTF-8, and then does the conversion. <br>`from_utf8()` 检查以确保字节是有效的 UTF-8，然后进行转换。<br>
    ///
    /// If you are sure that the byte slice is valid UTF-8, and you don't want to incur the overhead of the validity check, there is an unsafe version of this function, [`from_utf8_unchecked`], which has the same behavior but skips the check. <br>如果您确定字节切片是有效的 UTF-8，并且不想增加有效性检查的开销，则此函数有一个不安全的版本 [`from_utf8_unchecked`]，它具有相同的行为，但是会跳过检查。<br>
    ///
    ///
    /// This method will take care to not copy the vector, for efficiency's sake. <br>为了提高效率，此方法将注意不要复制 vector。<br>
    ///
    /// If you need a [`&str`] instead of a `String`, consider [`str::from_utf8`]. <br>如果需要 [`&str`] 而不是 `String`，请考虑使用 [`str::from_utf8`]。<br>
    ///
    /// The inverse of this method is [`into_bytes`]. <br>与此方法的相反的是 [`into_bytes`]。<br>
    ///
    /// # Errors
    ///
    /// Returns [`Err`] if the slice is not UTF-8 with a description as to why the provided bytes are not UTF-8. <br>如果切片不是 UTF-8，则返回 [`Err`]，并说明为什么提供的字节不是 UTF-8。<br> The vector you moved in is also included. <br>还包括您移入的 vector。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// // some bytes, in a vector <br>vector 中的一些字节<br>
    /// let sparkle_heart = vec![240, 159, 146, 150];
    ///
    /// // We know these bytes are valid, so we'll use `unwrap()`. <br>我们知道这些字节是有效的，因此我们将使用 `unwrap()`。<br>
    /// let sparkle_heart = String::from_utf8(sparkle_heart).unwrap();
    ///
    /// assert_eq!("💖", sparkle_heart);
    /// ```
    ///
    /// Incorrect bytes: <br>字节不正确：<br>
    ///
    /// ```
    /// // some invalid bytes, in a vector <br>vector 中的一些无效字节<br>
    /// let sparkle_heart = vec![0, 159, 146, 150];
    ///
    /// assert!(String::from_utf8(sparkle_heart).is_err());
    /// ```
    ///
    /// See the docs for [`FromUtf8Error`] for more details on what you can do with this error. <br>请参见 [`FromUtf8Error`] 文档，以获取有关此错误的更多详细信息。<br>
    ///
    /// [`from_utf8_unchecked`]: String::from_utf8_unchecked
    /// [`Vec<u8>`]: crate::vec::Vec "Vec"
    /// [`&str`]: prim@str "&str"
    /// [`into_bytes`]: String::into_bytes
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn from_utf8(vec: Vec<u8>) -> Result<String, FromUtf8Error> {
        match str::from_utf8(&vec) {
            Ok(..) => Ok(String { vec }),
            Err(e) => Err(FromUtf8Error { bytes: vec, error: e }),
        }
    }

    /// Converts a slice of bytes to a string, including invalid characters. <br>将字节切片转换为字符串，包括无效字符。<br>
    ///
    /// Strings are made of bytes ([`u8`]), and a slice of bytes ([`&[u8]`][byteslice]) is made of bytes, so this function converts between the two. <br>字符串由字节 ([`u8`]) 组成，而字节 ([`&[u8]`][byteslice]) 的切片由字节组成，因此此函数在两者之间进行转换。<br> Not all byte slices are valid strings, however: strings are required to be valid UTF-8. <br>然而，并非所有的字节片都是有效的字符串，字符串必须是有效的 UTF-8。<br>
    /// During this conversion, `from_utf8_lossy()` will replace any invalid UTF-8 sequences with [`U+FFFD REPLACEMENT CHARACTER`][U+FFFD], which looks like this: <br>在此转换过程中，`from_utf8_lossy()` 会将所有无效的 UTF-8 序列替换为 [`U+FFFD REPLACEMENT CHARACTER`][U+FFFD]，如下所示：<br>
    ///
    /// [byteslice]: prim@slice
    /// [U+FFFD]: core::char::REPLACEMENT_CHARACTER
    ///
    /// If you are sure that the byte slice is valid UTF-8, and you don't want to incur the overhead of the conversion, there is an unsafe version of this function, [`from_utf8_unchecked`], which has the same behavior but skips the checks. <br>如果您确定字节切片是有效的 UTF-8，并且不想增加转换的开销，则此函数有一个不安全的版本 [`from_utf8_unchecked`]，它具有相同的行为，但是会跳过检查。<br>
    ///
    ///
    /// [`from_utf8_unchecked`]: String::from_utf8_unchecked
    ///
    /// This function returns a [`Cow<'a, str>`]. <br>此函数返回 [`Cow<'a, str>`]。<br> If our byte slice is invalid UTF-8, then we need to insert the replacement characters, which will change the size of the string, and hence, require a `String`. <br>如果字节切片的 UTF-8 无效，则需要插入替换字符，这将更改字符串的大小，因此需要 `String`。<br>
    /// But if it's already valid UTF-8, we don't need a new allocation. <br>但是，如果它已经是有效的 UTF-8，则不需要新的分配。<br>
    /// This return type allows us to handle both cases. <br>这种返回类型使我们能够处理两种情况。<br>
    ///
    /// [`Cow<'a, str>`]: crate::borrow::Cow "borrow::Cow"
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// // some bytes, in a vector <br>vector 中的一些字节<br>
    /// let sparkle_heart = vec![240, 159, 146, 150];
    ///
    /// let sparkle_heart = String::from_utf8_lossy(&sparkle_heart);
    ///
    /// assert_eq!("💖", sparkle_heart);
    /// ```
    ///
    /// Incorrect bytes: <br>字节不正确：<br>
    ///
    /// ```
    /// // some invalid bytes <br>一些无效的字节<br>
    /// let input = b"Hello \xF0\x90\x80World";
    /// let output = String::from_utf8_lossy(input);
    ///
    /// assert_eq!("Hello �World", output);
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[must_use]
    #[cfg(not(no_global_oom_handling))]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn from_utf8_lossy(v: &[u8]) -> Cow<'_, str> {
        let mut iter = lossy::Utf8Lossy::from_bytes(v).chunks();

        let first_valid = if let Some(chunk) = iter.next() {
            let lossy::Utf8LossyChunk { valid, broken } = chunk;
            if broken.is_empty() {
                debug_assert_eq!(valid.len(), v.len());
                return Cow::Borrowed(valid);
            }
            valid
        } else {
            return Cow::Borrowed("");
        };

        const REPLACEMENT: &str = "\u{FFFD}";

        let mut res = String::with_capacity(v.len());
        res.push_str(first_valid);
        res.push_str(REPLACEMENT);

        for lossy::Utf8LossyChunk { valid, broken } in iter {
            res.push_str(valid);
            if !broken.is_empty() {
                res.push_str(REPLACEMENT);
            }
        }

        Cow::Owned(res)
    }

    /// Decode a UTF-16–encoded vector `v` into a `String`, returning [`Err`] if `v` contains any invalid data. <br>将 UTF-16 编码的 vector `v` 解码为 `String`，如果 `v` 包含任何无效数据，则返回 [`Err`]。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// // 𝄞music
    /// let v = &[0xD834, 0xDD1E, 0x006d, 0x0075,
    ///           0x0073, 0x0069, 0x0063];
    /// assert_eq!(String::from("𝄞music"),
    ///            String::from_utf16(v).unwrap());
    ///
    /// // 𝄞mu<invalid>ic
    /// let v = &[0xD834, 0xDD1E, 0x006d, 0x0075,
    ///           0xD800, 0x0069, 0x0063];
    /// assert!(String::from_utf16(v).is_err());
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn from_utf16(v: &[u16]) -> Result<String, FromUtf16Error> {
        // This isn't done via collect::<Result<_, _>>() for performance reasons. <br>这不是通过 collect:: 完成的: <Result<_, _>> () 出于性能原因。<br>
        // FIXME: the function can be simplified again when #48994 is closed. <br>关闭 #48994 时，可以再次简化函数。<br>
        let mut ret = String::with_capacity(v.len());
        for c in decode_utf16(v.iter().cloned()) {
            if let Ok(c) = c {
                ret.push(c);
            } else {
                return Err(FromUtf16Error(()));
            }
        }
        Ok(ret)
    }

    /// Decode a UTF-16–encoded slice `v` into a `String`, replacing invalid data with [the replacement character (`U+FFFD`)][U+FFFD]. <br>将 UTF-16 编码的切片 `v` 解码为 `String`，将无效数据替换为 [替换字符 (`U+FFFD`)][U+FFFD]。<br>
    ///
    /// Unlike [`from_utf8_lossy`] which returns a [`Cow<'a, str>`], `from_utf16_lossy` returns a `String` since the UTF-16 to UTF-8 conversion requires a memory allocation. <br>与 [`from_utf8_lossy`] 返回 [`Cow<'a, str>`] 不同，`from_utf16_lossy` 返回 `String`，因为 UTF-16 到 UTF-8 的转换需要分配内存。<br>
    ///
    ///
    /// [`from_utf8_lossy`]: String::from_utf8_lossy
    /// [`Cow<'a, str>`]: crate::borrow::Cow "borrow::Cow"
    /// [U+FFFD]: core::char::REPLACEMENT_CHARACTER
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// // 𝄞mus<invalid>ic<invalid>
    /// let v = &[0xD834, 0xDD1E, 0x006d, 0x0075,
    ///           0x0073, 0xDD1E, 0x0069, 0x0063,
    ///           0xD834];
    ///
    /// assert_eq!(String::from("𝄞mus\u{FFFD}ic\u{FFFD}"),
    ///            String::from_utf16_lossy(v));
    /// ```
    ///
    ///
    #[cfg(not(no_global_oom_handling))]
    #[must_use]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn from_utf16_lossy(v: &[u16]) -> String {
        decode_utf16(v.iter().cloned()).map(|r| r.unwrap_or(REPLACEMENT_CHARACTER)).collect()
    }

    /// Decomposes a `String` into its raw components. <br>将 `String` 分解为其原始组件。<br>
    ///
    /// Returns the raw pointer to the underlying data, the length of the string (in bytes), and the allocated capacity of the data (in bytes). <br>返回指向底层数据的裸指针，字符串的长度 (以字节为单位) 和数据的已分配容量 (以字节为单位)。<br>
    /// These are the same arguments in the same order as the arguments to [`from_raw_parts`]. <br>这些参数与 [`from_raw_parts`] 的参数顺序相同。<br>
    ///
    /// After calling this function, the caller is responsible for the memory previously managed by the `String`. <br>调用此函数后，调用者将负责先前由 `String` 管理的内存。<br>
    /// The only way to do this is to convert the raw pointer, length, and capacity back into a `String` with the [`from_raw_parts`] function, allowing the destructor to perform the cleanup. <br>唯一的方法是使用 [`from_raw_parts`] 函数将裸指针，长度和容量转换回 `String`，从而允许析构函数执行清除操作。<br>
    ///
    ///
    /// [`from_raw_parts`]: String::from_raw_parts
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(vec_into_raw_parts)]
    /// let s = String::from("hello");
    ///
    /// let (ptr, len, cap) = s.into_raw_parts();
    ///
    /// let rebuilt = unsafe { String::from_raw_parts(ptr, len, cap) };
    /// assert_eq!(rebuilt, "hello");
    /// ```
    ///
    ///
    ///
    ///
    #[must_use = "`self` will be dropped if the result is not used"]
    #[unstable(feature = "vec_into_raw_parts", reason = "new API", issue = "65816")]
    pub fn into_raw_parts(self) -> (*mut u8, usize, usize) {
        self.vec.into_raw_parts()
    }

    /// Creates a new `String` from a length, capacity, and pointer. <br>根据长度，容量和指针创建一个新的 `String`。<br>
    ///
    /// # Safety
    ///
    /// This is highly unsafe, due to the number of invariants that aren't checked: <br>这是非常不安全的，因为没有检查的不变量的数量：<br>
    ///
    /// * The memory at `buf` needs to have been previously allocated by the same allocator the standard library uses, with a required alignment of exactly 1. <br>`buf` 处的内存必须事先由标准库使用的同一分配器分配，且所需的对齐方式正好为 1。<br>
    /// * `length` needs to be less than or equal to `capacity`. <br>`length` 需要小于或等于 `capacity`。<br>
    /// * `capacity` needs to be the correct value. <br>`capacity` 需要是正确的值。<br>
    /// * The first `length` bytes at `buf` need to be valid UTF-8. <br>`buf` 的前 `length` 字节必须为有效的 UTF-8。<br>
    ///
    /// Violating these may cause problems like corrupting the allocator's internal data structures. <br>违反这些可能会导致一些问题，比如破坏分配器的内部数据结构。<br>
    /// For example, it is normally **not** safe to build a `String` from a pointer to a C `char` array containing UTF-8 _unless_ you are certain that array was originally allocated by the Rust standard library's allocator. <br>例如，从指向包含 UTF-8 的 C `char` 数组的指针构建 `String` 通常**不**安全，除非您确定该数组最初是由 Rust 标准库的分配器分配的。<br>
    ///
    ///
    /// The ownership of `buf` is effectively transferred to the `String` which may then deallocate, reallocate or change the contents of memory pointed to by the pointer at will. <br>`buf` 的所有权有效地转移到 `String`，然后 `String` 可以随意释放，重新分配或更改指针所指向的内存的内容。<br>
    /// Ensure that nothing else uses the pointer after calling this function. <br>调用此函数后，请确保没有其他任何东西使用该指针。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// use std::mem;
    ///
    /// unsafe {
    ///     let s = String::from("hello");
    ///
    // FIXME Update this when vec_into_raw_parts is stabilized <br>在 vec_into_raw_parts 稳定后更新它<br>
    ///     // Prevent automatically dropping the String's data <br>防止自动丢弃字符串的数据<br>
    ///     let mut s = mem::ManuallyDrop::new(s);
    ///
    ///     let ptr = s.as_mut_ptr();
    ///     let len = s.len();
    ///     let capacity = s.capacity();
    ///
    ///     let s = String::from_raw_parts(ptr, len, capacity);
    ///
    ///     assert_eq!(String::from("hello"), s);
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub unsafe fn from_raw_parts(buf: *mut u8, length: usize, capacity: usize) -> String {
        unsafe { String { vec: Vec::from_raw_parts(buf, length, capacity) } }
    }

    /// Converts a vector of bytes to a `String` without checking that the string contains valid UTF-8. <br>将字节的 vector 转换为 `String`，而无需检查字符串是否包含有效的 UTF-8。<br>
    ///
    /// See the safe version, [`from_utf8`], for more details. <br>有关更多详细信息，请参见安全版本 [`from_utf8`]。<br>
    ///
    /// [`from_utf8`]: String::from_utf8
    ///
    /// # Safety
    ///
    /// This function is unsafe because it does not check that the bytes passed to it are valid UTF-8. <br>此函数不安全，因为它不检查传递给它的字节是否为有效的 UTF-8。<br>
    /// If this constraint is violated, it may cause memory unsafety issues with future users of the `String`, as the rest of the standard library assumes that `String`s are valid UTF-8. <br>如果违反了此约束，则 `String` 的未来用户可能会导致内存不安全问题，因为标准库的其余部分都假定 `String` 是有效的 UTF-8。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// // some bytes, in a vector <br>vector 中的一些字节<br>
    /// let sparkle_heart = vec![240, 159, 146, 150];
    ///
    /// let sparkle_heart = unsafe {
    ///     String::from_utf8_unchecked(sparkle_heart)
    /// };
    ///
    /// assert_eq!("💖", sparkle_heart);
    /// ```
    ///
    ///
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub unsafe fn from_utf8_unchecked(bytes: Vec<u8>) -> String {
        String { vec: bytes }
    }

    /// Converts a `String` into a byte vector. <br>将 `String` 转换为字节 vector。<br>
    ///
    /// This consumes the `String`, so we do not need to copy its contents. <br>这会消耗 `String`，因此我们不需要复制其内容。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let s = String::from("hello");
    /// let bytes = s.into_bytes();
    ///
    /// assert_eq!(&[104, 101, 108, 108, 111][..], &bytes[..]);
    /// ```
    #[inline]
    #[must_use = "`self` will be dropped if the result is not used"]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn into_bytes(self) -> Vec<u8> {
        self.vec
    }

    /// Extracts a string slice containing the entire `String`. <br>提取包含整个 `String` 的字符串切片。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let s = String::from("foo");
    ///
    /// assert_eq!("foo", s.as_str());
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "string_as_str", since = "1.7.0")]
    pub fn as_str(&self) -> &str {
        self
    }

    /// Converts a `String` into a mutable string slice. <br>将 `String` 转换为可变字符串切片。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("foobar");
    /// let s_mut_str = s.as_mut_str();
    ///
    /// s_mut_str.make_ascii_uppercase();
    ///
    /// assert_eq!("FOOBAR", s_mut_str);
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "string_as_str", since = "1.7.0")]
    pub fn as_mut_str(&mut self) -> &mut str {
        self
    }

    /// Appends a given string slice onto the end of this `String`. <br>将给定的字符串切片追加到这个 `String` 的末尾。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("foo");
    ///
    /// s.push_str("bar");
    ///
    /// assert_eq!("foobar", s);
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn push_str(&mut self, string: &str) {
        self.vec.extend_from_slice(string.as_bytes())
    }

    /// Copies elements from `src` range to the end of the string. <br>将 `src` 范围内的元素复制到字符串的末尾。<br>
    ///
    /// ## Panics
    ///
    /// Panics if the starting point or end point do not lie on a [`char`] boundary, or if they're out of bounds. <br>如果起始点或结束点不在 [`char`] 边界上，或超出边界，就会出现 panic。<br>
    ///
    ///
    /// ## Examples
    ///
    /// ```
    /// #![feature(string_extend_from_within)]
    /// let mut string = String::from("abcde");
    ///
    /// string.extend_from_within(2..);
    /// assert_eq!(string, "abcdecde");
    ///
    /// string.extend_from_within(..2);
    /// assert_eq!(string, "abcdecdeab");
    ///
    /// string.extend_from_within(4..8);
    /// assert_eq!(string, "abcdecdeabecde");
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[unstable(feature = "string_extend_from_within", issue = "none")]
    pub fn extend_from_within<R>(&mut self, src: R)
    where
        R: RangeBounds<usize>,
    {
        let src @ Range { start, end } = slice::range(src, ..self.len());

        assert!(self.is_char_boundary(start));
        assert!(self.is_char_boundary(end));

        self.vec.extend_from_within(src);
    }

    /// Returns this `String`'s capacity, in bytes. <br>返回此字符串的容量 (以字节为单位)。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let s = String::with_capacity(10);
    ///
    /// assert!(s.capacity() >= 10);
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn capacity(&self) -> usize {
        self.vec.capacity()
    }

    /// Reserves capacity for at least `additional` bytes more than the current length. <br>保留至少比当前长度多 `additional` 字节的容量。<br> The allocator may reserve more space to speculatively avoid frequent allocations. <br>分配器可以保留更多空间来推测性地避免频繁分配。<br>
    ///
    /// After calling `reserve`, capacity will be greater than or equal to `self.len() + additional`. <br>调用 `reserve` 后，容量将大于或等于 `self.len() + additional`。<br>
    /// Does nothing if capacity is already sufficient. <br>如果容量已经足够，则不执行任何操作。<br>
    ///
    /// # Panics
    ///
    /// Panics if the new capacity overflows [`usize`]. <br>如果新容量溢出 [`usize`]，就会出现 panics。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::new();
    ///
    /// s.reserve(10);
    ///
    /// assert!(s.capacity() >= 10);
    /// ```
    ///
    /// This might not actually increase the capacity: <br>这实际上可能不会增加容量：<br>
    ///
    /// ```
    /// let mut s = String::with_capacity(10);
    /// s.push('a');
    /// s.push('b');
    ///
    /// // s now has a length of 2 and a capacity of at least 10 <br>s 现在的长度为 2，容量至少为 10<br>
    /// let capacity = s.capacity();
    /// assert_eq!(2, s.len());
    /// assert!(capacity >= 10);
    ///
    /// // Since we already have at least an extra 8 capacity, calling this... <br>由于我们已经至少有 8 个额外的容量，所以调用它...<br>
    /// s.reserve(8);
    ///
    /// // ... doesn't actually increase. <br>实际上并没有增加。<br>
    /// assert_eq!(capacity, s.capacity());
    /// ```
    ///
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn reserve(&mut self, additional: usize) {
        self.vec.reserve(additional)
    }

    /// Reserves the minimum capacity for at least `additional` bytes more than the current length. <br>保留至少比当前长度多 `additional` 字节的最小容量。<br>
    /// Unlike [`reserve`], this will not deliberately over-allocate to speculatively avoid frequent allocations. <br>与 [`reserve`] 不同，这不会故意过度分配以推测性地避免频繁分配。<br>
    ///
    /// After calling `reserve_exact`, capacity will be greater than or equal to `self.len() + additional`. <br>调用 `reserve_exact` 后，容量将大于或等于 `self.len() + additional`。<br>
    /// Does nothing if the capacity is already sufficient. <br>如果容量已经足够，则不执行任何操作。<br>
    ///
    /// [`reserve`]: String::reserve
    ///
    /// # Panics
    ///
    /// Panics if the new capacity overflows [`usize`]. <br>如果新容量溢出 [`usize`]，就会出现 panics。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::new();
    ///
    /// s.reserve_exact(10);
    ///
    /// assert!(s.capacity() >= 10);
    /// ```
    ///
    /// This might not actually increase the capacity: <br>这实际上可能不会增加容量：<br>
    ///
    /// ```
    /// let mut s = String::with_capacity(10);
    /// s.push('a');
    /// s.push('b');
    ///
    /// // s now has a length of 2 and a capacity of at least 10 <br>s 现在的长度为 2，容量至少为 10<br>
    /// let capacity = s.capacity();
    /// assert_eq!(2, s.len());
    /// assert!(capacity >= 10);
    ///
    /// // Since we already have at least an extra 8 capacity, calling this... <br>由于我们已经至少有 8 个额外的容量，所以调用它...<br>
    /// s.reserve_exact(8);
    ///
    /// // ... doesn't actually increase. <br>实际上并没有增加。<br>
    /// assert_eq!(capacity, s.capacity());
    /// ```
    ///
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn reserve_exact(&mut self, additional: usize) {
        self.vec.reserve_exact(additional)
    }

    /// Tries to reserve capacity for at least `additional` bytes more than the current length. <br>尝试为至少比当前长度多 `additional` 字节的容量保留容量。<br>
    /// The allocator may reserve more space to speculatively avoid frequent allocations. <br>分配器可以保留更多空间来推测性地避免频繁分配。<br>
    /// After calling `try_reserve`, capacity will be greater than or equal to `self.len() + additional` if it returns `Ok(())`. <br>调用 `try_reserve` 后，如果返回 `Ok(())`，容量将大于等于 `self.len() + additional`。<br>
    ///
    /// Does nothing if capacity is already sufficient. <br>如果容量已经足够，则不执行任何操作。<br>
    ///
    /// # Errors
    ///
    /// If the capacity overflows, or the allocator reports a failure, then an error is returned. <br>如果容量溢出，或者分配器报告失败，则返回错误。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::TryReserveError;
    ///
    /// fn process_data(data: &str) -> Result<String, TryReserveError> {
    ///     let mut output = String::new();
    ///
    ///     // Pre-reserve the memory, exiting if we can't <br>预先保留内存，如果不能，则退出<br>
    ///     output.try_reserve(data.len())?;
    ///
    ///     // Now we know this can't OOM in the middle of our complex work <br>现在我们知道在我们复杂的工作中这不能 OOM<br>
    ///     output.push_str(data);
    ///
    ///     Ok(output)
    /// }
    /// # process_data("rust").expect("why is the test harness OOMing on 4 bytes?");
    /// ```
    ///
    #[stable(feature = "try_reserve", since = "1.57.0")]
    pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError> {
        self.vec.try_reserve(additional)
    }

    /// Tries to reserve the minimum capacity for at least `additional` bytes more than the current length. <br>尝试为至少比当前长度多 `additional` 字节的最小容量保留。<br>
    /// Unlike [`try_reserve`], this will not deliberately over-allocate to speculatively avoid frequent allocations. <br>与 [`try_reserve`] 不同，这不会故意过度分配以推测性地避免频繁分配。<br>
    /// After calling `try_reserve_exact`, capacity will be greater than or equal to `self.len() + additional` if it returns `Ok(())`. <br>调用 `try_reserve_exact` 后，如果返回 `Ok(())`，则容量将大于或等于 `self.len() + additional`。<br>
    ///
    /// Does nothing if the capacity is already sufficient. <br>如果容量已经足够，则不执行任何操作。<br>
    ///
    /// Note that the allocator may give the collection more space than it requests. <br>请注意，分配器可能会给集合提供比其请求更多的空间。<br>
    /// Therefore, capacity can not be relied upon to be precisely minimal. <br>因此，不能依靠容量来精确地最小化。<br>
    /// Prefer [`try_reserve`] if future insertions are expected. <br>如果希望将来插入，则首选 [`try_reserve`]。<br>
    ///
    /// [`try_reserve`]: String::try_reserve
    ///
    /// # Errors
    ///
    /// If the capacity overflows, or the allocator reports a failure, then an error is returned. <br>如果容量溢出，或者分配器报告失败，则返回错误。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::TryReserveError;
    ///
    /// fn process_data(data: &str) -> Result<String, TryReserveError> {
    ///     let mut output = String::new();
    ///
    ///     // Pre-reserve the memory, exiting if we can't <br>预先保留内存，如果不能，则退出<br>
    ///     output.try_reserve_exact(data.len())?;
    ///
    ///     // Now we know this can't OOM in the middle of our complex work <br>现在我们知道在我们复杂的工作中这不能 OOM<br>
    ///     output.push_str(data);
    ///
    ///     Ok(output)
    /// }
    /// # process_data("rust").expect("why is the test harness OOMing on 4 bytes?");
    /// ```
    ///
    ///
    #[stable(feature = "try_reserve", since = "1.57.0")]
    pub fn try_reserve_exact(&mut self, additional: usize) -> Result<(), TryReserveError> {
        self.vec.try_reserve_exact(additional)
    }

    /// Shrinks the capacity of this `String` to match its length. <br>缩小此 `String` 的容量以使其长度匹配。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("foo");
    ///
    /// s.reserve(100);
    /// assert!(s.capacity() >= 100);
    ///
    /// s.shrink_to_fit();
    /// assert_eq!(3, s.capacity());
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn shrink_to_fit(&mut self) {
        self.vec.shrink_to_fit()
    }

    /// Shrinks the capacity of this `String` with a lower bound. <br>降低 `String` 的容量。<br>
    ///
    /// The capacity will remain at least as large as both the length and the supplied value. <br>容量将至少保持与长度和提供的值一样大。<br>
    ///
    ///
    /// If the current capacity is less than the lower limit, this is a no-op. <br>如果当前容量小于下限，则为无操作。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut s = String::from("foo");
    ///
    /// s.reserve(100);
    /// assert!(s.capacity() >= 100);
    ///
    /// s.shrink_to(10);
    /// assert!(s.capacity() >= 10);
    /// s.shrink_to(0);
    /// assert!(s.capacity() >= 3);
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "shrink_to", since = "1.56.0")]
    pub fn shrink_to(&mut self, min_capacity: usize) {
        self.vec.shrink_to(min_capacity)
    }

    /// Appends the given [`char`] to the end of this `String`. <br>将给定的 [`char`] 追加到该 `String` 的末尾。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("abc");
    ///
    /// s.push('1');
    /// s.push('2');
    /// s.push('3');
    ///
    /// assert_eq!("abc123", s);
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn push(&mut self, ch: char) {
        match ch.len_utf8() {
            1 => self.vec.push(ch as u8),
            _ => self.vec.extend_from_slice(ch.encode_utf8(&mut [0; 4]).as_bytes()),
        }
    }

    /// Returns a byte slice of this `String`'s contents. <br>返回此 String 内容的字节切片。<br>
    ///
    /// The inverse of this method is [`from_utf8`]. <br>与此方法的相反的是 [`from_utf8`]。<br>
    ///
    /// [`from_utf8`]: String::from_utf8
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let s = String::from("hello");
    ///
    /// assert_eq!(&[104, 101, 108, 108, 111], s.as_bytes());
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn as_bytes(&self) -> &[u8] {
        &self.vec
    }

    /// Shortens this `String` to the specified length. <br>将此 `String` 缩短为指定的长度。<br>
    ///
    /// If `new_len` is greater than the string's current length, this has no effect. <br>如果 `new_len` 大于字符串的当前长度，则无效。<br>
    ///
    ///
    /// Note that this method has no effect on the allocated capacity of the string <br>请注意，此方法对字符串的分配容量没有影响<br>
    ///
    /// # Panics
    ///
    /// Panics if `new_len` does not lie on a [`char`] boundary. <br>如果 `new_len` 不位于 [`char`] 边界上，就会出现 panics。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("hello");
    ///
    /// s.truncate(2);
    ///
    /// assert_eq!("he", s);
    /// ```
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn truncate(&mut self, new_len: usize) {
        if new_len <= self.len() {
            assert!(self.is_char_boundary(new_len));
            self.vec.truncate(new_len)
        }
    }

    /// Removes the last character from the string buffer and returns it. <br>从字符串缓冲区中删除最后一个字符并返回它。<br>
    ///
    /// Returns [`None`] if this `String` is empty. <br>如果 `String` 为空，则返回 [`None`]。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("foo");
    ///
    /// assert_eq!(s.pop(), Some('o'));
    /// assert_eq!(s.pop(), Some('o'));
    /// assert_eq!(s.pop(), Some('f'));
    ///
    /// assert_eq!(s.pop(), None);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn pop(&mut self) -> Option<char> {
        let ch = self.chars().rev().next()?;
        let newlen = self.len() - ch.len_utf8();
        unsafe {
            self.vec.set_len(newlen);
        }
        Some(ch)
    }

    /// Removes a [`char`] from this `String` at a byte position and returns it. <br>从该 `String` 的字节位置删除 [`char`] 并将其返回。<br>
    ///
    /// This is an *O*(*n*) operation, as it requires copying every element in the buffer. <br>这是 *O*(*n*) 操作，因为它需要复制缓冲区中的每个元素。<br>
    ///
    /// # Panics
    ///
    /// Panics if `idx` is larger than or equal to the `String`'s length, or if it does not lie on a [`char`] boundary. <br>如果 `idx` 大于或等于 String 的长度，或者它不位于 [`char`] 边界上，就会出现 panics。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("foo");
    ///
    /// assert_eq!(s.remove(0), 'f');
    /// assert_eq!(s.remove(1), 'o');
    /// assert_eq!(s.remove(0), 'o');
    /// ```
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn remove(&mut self, idx: usize) -> char {
        let ch = match self[idx..].chars().next() {
            Some(ch) => ch,
            None => panic!("cannot remove a char from the end of a string"),
        };

        let next = idx + ch.len_utf8();
        let len = self.len();
        unsafe {
            ptr::copy(self.vec.as_ptr().add(next), self.vec.as_mut_ptr().add(idx), len - next);
            self.vec.set_len(len - (next - idx));
        }
        ch
    }

    /// Remove all matches of pattern `pat` in the `String`. <br>删除 `String` 中所有模式 `pat` 的匹配项。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(string_remove_matches)]
    /// let mut s = String::from("Trees are not green, the sky is not blue.");
    /// s.remove_matches("not ");
    /// assert_eq!("Trees are green, the sky is blue.", s);
    /// ```
    ///
    /// Matches will be detected and removed iteratively, so in cases where patterns overlap, only the first pattern will be removed: <br>匹配项将被检测并迭代删除，因此在样式重叠的情况下，仅第一个样式将被删除：<br>
    ///
    ///
    /// ```
    /// #![feature(string_remove_matches)]
    /// let mut s = String::from("banana");
    /// s.remove_matches("ana");
    /// assert_eq!("bna", s);
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[unstable(feature = "string_remove_matches", reason = "new API", issue = "72826")]
    pub fn remove_matches<'a, P>(&'a mut self, pat: P)
    where
        P: for<'x> Pattern<'x>,
    {
        use core::str::pattern::Searcher;

        let rejections = {
            let mut searcher = pat.into_searcher(self);
            // Per Searcher::next:
            //
            // A Match result needs to contain the whole matched pattern, however Reject results may be split up into arbitrary many adjacent fragments. <br>Match 结果需要包含整个匹配的模式，而 Reject 结果可以被分割成任意多个相邻的片段。<br> Both ranges may have zero length. <br>两个范围的长度都可以为零。<br>
            //
            // In practice the implementation of Searcher::next_match tends to be more efficient, so we use it here and do some work to invert matches into rejections since that's what we want to copy below. <br>在实践中，Searcher::next_match 的实现往往更高效，所以我们在这里使用它并做一些工作将匹配转化为拒绝，因为这就是我们想要在下面复制的内容。<br>
            //
            //
            //
            //
            let mut front = 0;
            let rejections: Vec<_> = from_fn(|| {
                let (start, end) = searcher.next_match()?;
                let prev_front = front;
                front = end;
                Some((prev_front, start))
            })
            .collect();
            rejections.into_iter().chain(core::iter::once((front, self.len())))
        };

        let mut len = 0;
        let ptr = self.vec.as_mut_ptr();

        for (start, end) in rejections {
            let count = end - start;
            if start != len {
                // SAFETY: per Searcher::next:
                //
                // The stream of Match and Reject values up to a Done will contain index ranges that are adjacent, non-overlapping, covering the whole haystack, and laying on utf8 boundaries. <br>直到 Done 的 Match 和 Reject 值流将包含相邻、不重叠、覆盖整个 haystack 并位于 utf8 边界上的索引范围。<br>
                //
                //
                //
                unsafe {
                    ptr::copy(ptr.add(start), ptr.add(len), count);
                }
            }
            len += count;
        }

        unsafe {
            self.vec.set_len(len);
        }
    }

    /// Retains only the characters specified by the predicate. <br>仅保留谓词指定的字符。<br>
    ///
    /// In other words, remove all characters `c` such that `f(c)` returns `false`. <br>换句话说，删除所有字符 `c`，以使 `f(c)` 返回 `false`。<br>
    /// This method operates in place, visiting each character exactly once in the original order, and preserves the order of the retained characters. <br>此方法在原地运行，以原始顺序恰好一次访问每个字符，并保留保留字符的顺序。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let mut s = String::from("f_o_ob_ar");
    ///
    /// s.retain(|c| c != '_');
    ///
    /// assert_eq!(s, "foobar");
    /// ```
    ///
    /// Because the elements are visited exactly once in the original order, external state may be used to decide which elements to keep. <br>由于按原始顺序仅对元素进行过一次访问，因此可以使用外部状态来确定要保留哪些元素。<br>
    ///
    /// ```
    /// let mut s = String::from("abcde");
    /// let keep = [false, true, true, false, true];
    /// let mut iter = keep.iter();
    /// s.retain(|_| *iter.next().unwrap());
    /// assert_eq!(s, "bce");
    /// ```
    ///
    #[inline]
    #[stable(feature = "string_retain", since = "1.26.0")]
    pub fn retain<F>(&mut self, mut f: F)
    where
        F: FnMut(char) -> bool,
    {
        struct SetLenOnDrop<'a> {
            s: &'a mut String,
            idx: usize,
            del_bytes: usize,
        }

        impl<'a> Drop for SetLenOnDrop<'a> {
            fn drop(&mut self) {
                let new_len = self.idx - self.del_bytes;
                debug_assert!(new_len <= self.s.len());
                unsafe { self.s.vec.set_len(new_len) };
            }
        }

        let len = self.len();
        let mut guard = SetLenOnDrop { s: self, idx: 0, del_bytes: 0 };

        while guard.idx < len {
            let ch =
                // SAFETY: `guard.idx` is positive-or-zero and less that len so the `get_unchecked` is in bound. <br>`guard.idx` 为正数或零，且小于 len，因此 `get_unchecked` 是有边界的。<br>
                // `self` is valid UTF-8 like string and the returned slice starts at a unicode code point so the `Chars` always return one character. <br>`self` 是像字符串一样的有效 UTF-8，返回的切片从 unicode 代码点开始，因此 `Chars` 始终返回一个字符。<br>
                //
                unsafe { guard.s.get_unchecked(guard.idx..len).chars().next().unwrap_unchecked() };
            let ch_len = ch.len_utf8();

            if !f(ch) {
                guard.del_bytes += ch_len;
            } else if guard.del_bytes > 0 {
                // SAFETY: `guard.idx` is in bound and `guard.del_bytes` represent the number of bytes that are erased from the string so the resulting `guard.idx - guard.del_bytes` always represent a valid unicode code point. <br>`guard.idx` 已绑定，`guard.del_bytes` 表示字符串中 erased 的字节数，因此生成的 `guard.idx - guard.del_bytes` 始终表示有效的 unicode 代码点。<br>
                //
                //
                // `guard.del_bytes` >= `ch.len_utf8()`, so taking a slice with `ch.len_utf8()` len is safe. <br>`guard.del_bytes` >= `ch.len_utf8()`，所以用 `ch.len_utf8()` len 切片是安全的。<br>
                //
                //
                ch.encode_utf8(unsafe {
                    crate::slice::from_raw_parts_mut(
                        guard.s.as_mut_ptr().add(guard.idx - guard.del_bytes),
                        ch.len_utf8(),
                    )
                });
            }

            // Point idx to the next char <br>将 idx 指向下一个字符<br>
            guard.idx += ch_len;
        }

        drop(guard);
    }

    /// Inserts a character into this `String` at a byte position. <br>在此 `String` 的字节位置插入一个字符。<br>
    ///
    /// This is an *O*(*n*) operation as it requires copying every element in the buffer. <br>这是一个 *O*(*n*) 操作，因为它需要复制缓冲区中的每个元素。<br>
    ///
    /// # Panics
    ///
    /// Panics if `idx` is larger than the `String`'s length, or if it does not lie on a [`char`] boundary. <br>如果 `idx` 大于 `String` 的长度，或者它不在 [`char`] 边界上，就会出现 panics。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::with_capacity(3);
    ///
    /// s.insert(0, 'f');
    /// s.insert(1, 'o');
    /// s.insert(2, 'o');
    ///
    /// assert_eq!("foo", s);
    /// ```
    ///
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn insert(&mut self, idx: usize, ch: char) {
        assert!(self.is_char_boundary(idx));
        let mut bits = [0; 4];
        let bits = ch.encode_utf8(&mut bits).as_bytes();

        unsafe {
            self.insert_bytes(idx, bits);
        }
    }

    #[cfg(not(no_global_oom_handling))]
    unsafe fn insert_bytes(&mut self, idx: usize, bytes: &[u8]) {
        let len = self.len();
        let amt = bytes.len();
        self.vec.reserve(amt);

        unsafe {
            ptr::copy(self.vec.as_ptr().add(idx), self.vec.as_mut_ptr().add(idx + amt), len - idx);
            ptr::copy_nonoverlapping(bytes.as_ptr(), self.vec.as_mut_ptr().add(idx), amt);
            self.vec.set_len(len + amt);
        }
    }

    /// Inserts a string slice into this `String` at a byte position. <br>在此 `String` 的字节位置处插入字符串切片。<br>
    ///
    /// This is an *O*(*n*) operation as it requires copying every element in the buffer. <br>这是一个 *O*(*n*) 操作，因为它需要复制缓冲区中的每个元素。<br>
    ///
    /// # Panics
    ///
    /// Panics if `idx` is larger than the `String`'s length, or if it does not lie on a [`char`] boundary. <br>如果 `idx` 大于 `String` 的长度，或者它不在 [`char`] 边界上，就会出现 panics。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("bar");
    ///
    /// s.insert_str(0, "foo");
    ///
    /// assert_eq!("foobar", s);
    /// ```
    ///
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "insert_str", since = "1.16.0")]
    pub fn insert_str(&mut self, idx: usize, string: &str) {
        assert!(self.is_char_boundary(idx));

        unsafe {
            self.insert_bytes(idx, string.as_bytes());
        }
    }

    /// Returns a mutable reference to the contents of this `String`. <br>返回此 `String` 的内容的可变引用。<br>
    ///
    /// # Safety
    ///
    /// This function is unsafe because the returned `&mut Vec` allows writing bytes which are not valid UTF-8. <br>这个函数是不安全的，因为返回的 `&mut Vec` 允许写入无效的 UTF-8 字节。<br>
    /// If this constraint is violated, using the original `String` after dropping the `&mut Vec` may violate memory safety, as the rest of the standard library assumes that `String`s are valid UTF-8. <br>如果违反此约束，则在丢弃 `&mut Vec` 之后使用原始 `String` 可能会违反内存安全，因为标准库的其余部分假定 `String` 是有效的 UTF-8。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("hello");
    ///
    /// unsafe {
    ///     let vec = s.as_mut_vec();
    ///     assert_eq!(&[104, 101, 108, 108, 111][..], &vec[..]);
    ///
    ///     vec.reverse();
    /// }
    /// assert_eq!(s, "olleh");
    /// ```
    ///
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub unsafe fn as_mut_vec(&mut self) -> &mut Vec<u8> {
        &mut self.vec
    }

    /// Returns the length of this `String`, in bytes, not [`char`]s or graphemes. <br>返回此 `String` 的长度，以字节为单位，而不是 [`char`] 或字素。<br>
    /// In other words, it might not be what a human considers the length of the string. <br>换句话说，它可能不是人类认为的字符串长度。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let a = String::from("foo");
    /// assert_eq!(a.len(), 3);
    ///
    /// let fancy_f = String::from("ƒoo");
    /// assert_eq!(fancy_f.len(), 4);
    /// assert_eq!(fancy_f.chars().count(), 3);
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn len(&self) -> usize {
        self.vec.len()
    }

    /// Returns `true` if this `String` has a length of zero, and `false` otherwise. <br>如果此 `String` 的长度为零，则返回 `true`，否则返回 `false`。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut v = String::new();
    /// assert!(v.is_empty());
    ///
    /// v.push('a');
    /// assert!(!v.is_empty());
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn is_empty(&self) -> bool {
        self.len() == 0
    }

    /// Splits the string into two at the given byte index. <br>在给定的字节索引处将字符串拆分为两个。<br>
    ///
    /// Returns a newly allocated `String`. <br>返回新分配的 `String`。<br>
    /// `self` contains bytes `[0, at)`, and the returned `String` contains bytes `[at, len)`. <br>`self` 包含字节 `[0, at)`，返回的 `String` 包含字节 `[at, len)`。<br>
    /// `at` must be on the boundary of a UTF-8 code point. <br>`at` 必须位于 UTF-8 代码点的边界上。<br>
    ///
    /// Note that the capacity of `self` does not change. <br>请注意，`self` 的容量不会改变。<br>
    ///
    /// # Panics
    ///
    /// Panics if `at` is not on a `UTF-8` code point boundary, or if it is beyond the last code point of the string. <br>如果 `at` 不在 `UTF-8` 代码点边界上，或者它超出字符串的最后一个代码点，就会出现 panics。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// # fn main() {
    /// let mut hello = String::from("Hello, World!");
    /// let world = hello.split_off(7);
    /// assert_eq!(hello, "Hello, ");
    /// assert_eq!(world, "World!");
    /// # }
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[inline]
    #[stable(feature = "string_split_off", since = "1.16.0")]
    #[must_use = "use `.truncate()` if you don't need the other half"]
    pub fn split_off(&mut self, at: usize) -> String {
        assert!(self.is_char_boundary(at));
        let other = self.vec.split_off(at);
        unsafe { String::from_utf8_unchecked(other) }
    }

    /// Truncates this `String`, removing all contents. <br>截断此 `String`，删除所有内容。<br>
    ///
    /// While this means the `String` will have a length of zero, it does not touch its capacity. <br>虽然这意味着 `String` 的长度为零，但它并未触及其容量。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("foo");
    ///
    /// s.clear();
    ///
    /// assert!(s.is_empty());
    /// assert_eq!(0, s.len());
    /// assert_eq!(3, s.capacity());
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn clear(&mut self) {
        self.vec.clear()
    }

    /// Removes the specified range from the string in bulk, returning all removed characters as an iterator. <br>从字符串中批量删除指定范围，并以迭代器的形式返回所有删除的字符。<br>
    ///
    /// The returned iterator keeps a mutable borrow on the string to optimize its implementation. <br>返回的迭代器在字符串上保留一个可变借用以优化其实现。<br>
    ///
    /// # Panics
    ///
    /// Panics if the starting point or end point do not lie on a [`char`] boundary, or if they're out of bounds. <br>如果起始点或结束点不在 [`char`] 边界上，或超出边界，就会出现 panic。<br>
    ///
    /// # Leaking
    ///
    /// If the returned iterator goes out of scope without being dropped (due to [`core::mem::forget`], for example), the string may still contain a copy of any drained characters, or may have lost characters arbitrarily, including characters outside the range. <br>如果返回的迭代器离开作用域而没有被丢弃 (例如，由于 [`core::mem::forget`])，则字符串可能仍包含任何耗尽字符的副本，或者可能任意丢失字符，包括范围外的字符。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("α is alpha, β is beta");
    /// let beta_offset = s.find('β').unwrap_or(s.len());
    ///
    /// // Remove the range up until the β from the string <br>删除范围直到字符串中的 β<br>
    /// let t: String = s.drain(..beta_offset).collect();
    /// assert_eq!(t, "α is alpha, ");
    /// assert_eq!(s, "β is beta");
    ///
    /// // A full range clears the string, like `clear()` does <br>全范围清除字符串，就像 `clear()` 一样<br>
    /// s.drain(..);
    /// assert_eq!(s, "");
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "drain", since = "1.6.0")]
    pub fn drain<R>(&mut self, range: R) -> Drain<'_>
    where
        R: RangeBounds<usize>,
    {
        // Memory safety <br>内存安全<br>
        //
        // The String version of Drain does not have the memory safety issues of the vector version. <br>Drain 的字符串版本没有 vector 版本的内存安全问题。<br>
        // The data is just plain bytes. <br>数据只是纯字节。<br>
        // Because the range removal happens in Drop, if the Drain iterator is leaked, the removal will not happen. <br>因为范围移除发生在 Drop 中，所以如果 Drain 迭代器泄漏，则移除不会发生。<br>
        //
        let Range { start, end } = slice::range(range, ..self.len());
        assert!(self.is_char_boundary(start));
        assert!(self.is_char_boundary(end));

        // Take out two simultaneous borrows. <br>同时取出两个借用。<br>
        // The &mut String won't be accessed until iteration is over, in Drop. <br>在 Drop 中，直到迭代结束，才可以访问 &mut 字符串。<br>
        let self_ptr = self as *mut _;
        // SAFETY: `slice::range` and `is_char_boundary` do the appropriate bounds checks. <br>`slice::range` 和 `is_char_boundary` 进行适当的边界检查。<br>
        let chars_iter = unsafe { self.get_unchecked(start..end) }.chars();

        Drain { start, end, iter: chars_iter, string: self_ptr }
    }

    /// Removes the specified range in the string, and replaces it with the given string. <br>删除字符串中的指定范围，并将其替换为给定的字符串。<br>
    /// The given string doesn't need to be the same length as the range. <br>给定的字符串不必与范围相同。<br>
    ///
    /// # Panics
    ///
    /// Panics if the starting point or end point do not lie on a [`char`] boundary, or if they're out of bounds. <br>如果起始点或结束点不在 [`char`] 边界上，或超出边界，就会出现 panic。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let mut s = String::from("α is alpha, β is beta");
    /// let beta_offset = s.find('β').unwrap_or(s.len());
    ///
    /// // Replace the range up until the β from the string <br>替换范围直到字符串中的 β<br>
    /// s.replace_range(..beta_offset, "Α is capital alpha; ");
    /// assert_eq!(s, "Α is capital alpha; β is beta");
    /// ```
    ///
    #[cfg(not(no_global_oom_handling))]
    #[stable(feature = "splice", since = "1.27.0")]
    pub fn replace_range<R>(&mut self, range: R, replace_with: &str)
    where
        R: RangeBounds<usize>,
    {
        // Memory safety <br>内存安全<br>
        //
        // Replace_range does not have the memory safety issues of a vector Splice. <br>Replace_range 没有 vector Splice 的内存安全问题。<br>
        // of the vector version. <br>vector 版本的版本。<br> The data is just plain bytes. <br>数据只是纯字节。<br>

        // WARNING: Inlining this variable would be unsound (#81138) <br>内联此变量将是不正确的 (#81138)<br>
        let start = range.start_bound();
        match start {
            Included(&n) => assert!(self.is_char_boundary(n)),
            Excluded(&n) => assert!(self.is_char_boundary(n + 1)),
            Unbounded => {}
        };
        // WARNING: Inlining this variable would be unsound (#81138) <br>内联此变量将是不正确的 (#81138)<br>
        let end = range.end_bound();
        match end {
            Included(&n) => assert!(self.is_char_boundary(n + 1)),
            Excluded(&n) => assert!(self.is_char_boundary(n)),
            Unbounded => {}
        };

        // Using `range` again would be unsound (#81138) We assume the bounds reported by `range` remain the same, but an adversarial implementation could change between calls <br>再次使用 `range` 是不正确的 (#81138) 我们假设 `range` 所报告的界限保持不变，但是在两次通话之间可能会发生对抗性实现<br>
        //
        //
        unsafe { self.as_mut_vec() }.splice((start, end), replace_with.bytes());
    }

    /// Converts this `String` into a <code>[Box]<[str]></code>. <br>将此 `String` 转换为 <code>[Box]<[str]></code>。<br>
    ///
    /// This will drop any excess capacity. <br>这将丢弃任何多余的容量。<br>
    ///
    /// [str]: prim@str "str"
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let s = String::from("hello");
    ///
    /// let b = s.into_boxed_str();
    /// ```
    #[cfg(not(no_global_oom_handling))]
    #[stable(feature = "box_str", since = "1.4.0")]
    #[must_use = "`self` will be dropped if the result is not used"]
    #[inline]
    pub fn into_boxed_str(self) -> Box<str> {
        let slice = self.vec.into_boxed_slice();
        unsafe { from_boxed_utf8_unchecked(slice) }
    }
}

impl FromUtf8Error {
    /// Returns a slice of [`u8`]s bytes that were attempted to convert to a `String`. <br>返回试图转换为 `String` 的 [u8] 个字节切片。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// // some invalid bytes, in a vector <br>vector 中的一些无效字节<br>
    /// let bytes = vec![0, 159];
    ///
    /// let value = String::from_utf8(bytes);
    ///
    /// assert_eq!(&[0, 159], value.unwrap_err().as_bytes());
    /// ```
    #[must_use]
    #[stable(feature = "from_utf8_error_as_bytes", since = "1.26.0")]
    pub fn as_bytes(&self) -> &[u8] {
        &self.bytes[..]
    }

    /// Returns the bytes that were attempted to convert to a `String`. <br>返回尝试转换为 `String` 的字节。<br>
    ///
    /// This method is carefully constructed to avoid allocation. <br>精心构造此方法以避免分配。<br>
    /// It will consume the error, moving out the bytes, so that a copy of the bytes does not need to be made. <br>它将消耗错误，将字节移出，因此不需要制作字节的副本。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// // some invalid bytes, in a vector <br>vector 中的一些无效字节<br>
    /// let bytes = vec![0, 159];
    ///
    /// let value = String::from_utf8(bytes);
    ///
    /// assert_eq!(vec![0, 159], value.unwrap_err().into_bytes());
    /// ```
    #[must_use = "`self` will be dropped if the result is not used"]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn into_bytes(self) -> Vec<u8> {
        self.bytes
    }

    /// Fetch a `Utf8Error` to get more details about the conversion failure. <br>提取 `Utf8Error` 以获取有关转换失败的更多详细信息。<br>
    ///
    /// The [`Utf8Error`] type provided by [`std::str`] represents an error that may occur when converting a slice of [`u8`]s to a [`&str`]. <br>[`std::str`] 提供的 [`Utf8Error`] 类型表示将 [`u8`] 的切片转换为 [`&str`] 时可能发生的错误。<br>
    /// In this sense, it's an analogue to `FromUtf8Error`. <br>从这个意义上讲，它类似于 `FromUtf8Error`。<br>
    /// See its documentation for more details on using it. <br>有关使用它的更多详细信息，请参见其文档。<br>
    ///
    /// [`std::str`]: core::str "std::str"
    /// [`&str`]: prim@str "&str"
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// // some invalid bytes, in a vector <br>vector 中的一些无效字节<br>
    /// let bytes = vec![0, 159];
    ///
    /// let error = String::from_utf8(bytes).unwrap_err().utf8_error();
    ///
    /// // the first byte is invalid here <br>第一个字节在这里无效<br>
    /// assert_eq!(1, error.valid_up_to());
    /// ```
    ///
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn utf8_error(&self) -> Utf8Error {
        self.error
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Display for FromUtf8Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Display::fmt(&self.error, f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Display for FromUtf16Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Display::fmt("invalid utf-16: lone surrogate found", f)
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl Clone for String {
    fn clone(&self) -> Self {
        String { vec: self.vec.clone() }
    }

    fn clone_from(&mut self, source: &Self) {
        self.vec.clone_from(&source.vec);
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl FromIterator<char> for String {
    fn from_iter<I: IntoIterator<Item = char>>(iter: I) -> String {
        let mut buf = String::new();
        buf.extend(iter);
        buf
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "string_from_iter_by_ref", since = "1.17.0")]
impl<'a> FromIterator<&'a char> for String {
    fn from_iter<I: IntoIterator<Item = &'a char>>(iter: I) -> String {
        let mut buf = String::new();
        buf.extend(iter);
        buf
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> FromIterator<&'a str> for String {
    fn from_iter<I: IntoIterator<Item = &'a str>>(iter: I) -> String {
        let mut buf = String::new();
        buf.extend(iter);
        buf
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "extend_string", since = "1.4.0")]
impl FromIterator<String> for String {
    fn from_iter<I: IntoIterator<Item = String>>(iter: I) -> String {
        let mut iterator = iter.into_iter();

        // Because we're iterating over `String`s, we can avoid at least one allocation by getting the first string from the iterator and appending to it all the subsequent strings. <br>因为我们要在 `String` 上进行迭代，所以可以通过从迭代器获取第一个字符串，并将所有后续字符串追加到该字符串来避免至少一次分配。<br>
        //
        //
        match iterator.next() {
            None => String::new(),
            Some(mut buf) => {
                buf.extend(iterator);
                buf
            }
        }
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "box_str2", since = "1.45.0")]
impl FromIterator<Box<str>> for String {
    fn from_iter<I: IntoIterator<Item = Box<str>>>(iter: I) -> String {
        let mut buf = String::new();
        buf.extend(iter);
        buf
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "herd_cows", since = "1.19.0")]
impl<'a> FromIterator<Cow<'a, str>> for String {
    fn from_iter<I: IntoIterator<Item = Cow<'a, str>>>(iter: I) -> String {
        let mut iterator = iter.into_iter();

        // Because we're iterating over CoWs, we can (potentially) avoid at least one allocation by getting the first item and appending to it all the subsequent items. <br>因为我们要遍历 CoW，所以我们可以 (潜在地) 通过获取第一个项，并向它追加所有后续的项来避免至少一次分配。<br>
        //
        //
        match iterator.next() {
            None => String::new(),
            Some(cow) => {
                let mut buf = cow.into_owned();
                buf.extend(iterator);
                buf
            }
        }
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl Extend<char> for String {
    fn extend<I: IntoIterator<Item = char>>(&mut self, iter: I) {
        let iterator = iter.into_iter();
        let (lower_bound, _) = iterator.size_hint();
        self.reserve(lower_bound);
        iterator.for_each(move |c| self.push(c));
    }

    #[inline]
    fn extend_one(&mut self, c: char) {
        self.push(c);
    }

    #[inline]
    fn extend_reserve(&mut self, additional: usize) {
        self.reserve(additional);
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "extend_ref", since = "1.2.0")]
impl<'a> Extend<&'a char> for String {
    fn extend<I: IntoIterator<Item = &'a char>>(&mut self, iter: I) {
        self.extend(iter.into_iter().cloned());
    }

    #[inline]
    fn extend_one(&mut self, &c: &'a char) {
        self.push(c);
    }

    #[inline]
    fn extend_reserve(&mut self, additional: usize) {
        self.reserve(additional);
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> Extend<&'a str> for String {
    fn extend<I: IntoIterator<Item = &'a str>>(&mut self, iter: I) {
        iter.into_iter().for_each(move |s| self.push_str(s));
    }

    #[inline]
    fn extend_one(&mut self, s: &'a str) {
        self.push_str(s);
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "box_str2", since = "1.45.0")]
impl Extend<Box<str>> for String {
    fn extend<I: IntoIterator<Item = Box<str>>>(&mut self, iter: I) {
        iter.into_iter().for_each(move |s| self.push_str(&s));
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "extend_string", since = "1.4.0")]
impl Extend<String> for String {
    fn extend<I: IntoIterator<Item = String>>(&mut self, iter: I) {
        iter.into_iter().for_each(move |s| self.push_str(&s));
    }

    #[inline]
    fn extend_one(&mut self, s: String) {
        self.push_str(&s);
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "herd_cows", since = "1.19.0")]
impl<'a> Extend<Cow<'a, str>> for String {
    fn extend<I: IntoIterator<Item = Cow<'a, str>>>(&mut self, iter: I) {
        iter.into_iter().for_each(move |s| self.push_str(&s));
    }

    #[inline]
    fn extend_one(&mut self, s: Cow<'a, str>) {
        self.push_str(&s);
    }
}

/// A convenience impl that delegates to the impl for `&str`. <br>一个方便的 impl，委派给 `&str` 的 impl。<br>
///
/// # Examples
///
/// ```
/// assert_eq!(String::from("Hello world").find("world"), Some(6));
/// ```
#[unstable(
    feature = "pattern",
    reason = "API not fully fleshed out and ready to be stabilized",
    issue = "27721"
)]
impl<'a, 'b> Pattern<'a> for &'b String {
    type Searcher = <&'b str as Pattern<'a>>::Searcher;

    fn into_searcher(self, haystack: &'a str) -> <&'b str as Pattern<'a>>::Searcher {
        self[..].into_searcher(haystack)
    }

    #[inline]
    fn is_contained_in(self, haystack: &'a str) -> bool {
        self[..].is_contained_in(haystack)
    }

    #[inline]
    fn is_prefix_of(self, haystack: &'a str) -> bool {
        self[..].is_prefix_of(haystack)
    }

    #[inline]
    fn strip_prefix_of(self, haystack: &'a str) -> Option<&'a str> {
        self[..].strip_prefix_of(haystack)
    }

    #[inline]
    fn is_suffix_of(self, haystack: &'a str) -> bool {
        self[..].is_suffix_of(haystack)
    }

    #[inline]
    fn strip_suffix_of(self, haystack: &'a str) -> Option<&'a str> {
        self[..].strip_suffix_of(haystack)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl PartialEq for String {
    #[inline]
    fn eq(&self, other: &String) -> bool {
        PartialEq::eq(&self[..], &other[..])
    }
    #[inline]
    fn ne(&self, other: &String) -> bool {
        PartialEq::ne(&self[..], &other[..])
    }
}

macro_rules! impl_eq {
    ($lhs:ty, $rhs: ty) => {
        #[stable(feature = "rust1", since = "1.0.0")]
        #[allow(unused_lifetimes)]
        impl<'a, 'b> PartialEq<$rhs> for $lhs {
            #[inline]
            fn eq(&self, other: &$rhs) -> bool {
                PartialEq::eq(&self[..], &other[..])
            }
            #[inline]
            fn ne(&self, other: &$rhs) -> bool {
                PartialEq::ne(&self[..], &other[..])
            }
        }

        #[stable(feature = "rust1", since = "1.0.0")]
        #[allow(unused_lifetimes)]
        impl<'a, 'b> PartialEq<$lhs> for $rhs {
            #[inline]
            fn eq(&self, other: &$lhs) -> bool {
                PartialEq::eq(&self[..], &other[..])
            }
            #[inline]
            fn ne(&self, other: &$lhs) -> bool {
                PartialEq::ne(&self[..], &other[..])
            }
        }
    };
}

impl_eq! { String, str }
impl_eq! { String, &'a str }
#[cfg(not(no_global_oom_handling))]
impl_eq! { Cow<'a, str>, str }
#[cfg(not(no_global_oom_handling))]
impl_eq! { Cow<'a, str>, &'b str }
#[cfg(not(no_global_oom_handling))]
impl_eq! { Cow<'a, str>, String }

#[stable(feature = "rust1", since = "1.0.0")]
#[rustc_const_unstable(feature = "const_default_impls", issue = "87864")]
impl const Default for String {
    /// Creates an empty `String`. <br>创建一个空的 `String`。<br>
    #[inline]
    fn default() -> String {
        String::new()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Display for String {
    #[inline]
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Display::fmt(&**self, f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for String {
    #[inline]
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Debug::fmt(&**self, f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl hash::Hash for String {
    #[inline]
    fn hash<H: hash::Hasher>(&self, hasher: &mut H) {
        (**self).hash(hasher)
    }
}

/// Implements the `+` operator for concatenating two strings. <br>实现 `+` 运算符以连接两个字符串。<br>
///
/// This consumes the `String` on the left-hand side and re-uses its buffer (growing it if necessary). <br>这会消费左侧的 `String`，并重新使用其缓冲区 (如有必要，请增加缓冲区)。<br>
/// This is done to avoid allocating a new `String` and copying the entire contents on every operation, which would lead to *O*(*n*^2) running time when building an *n*-byte string by repeated concatenation. <br>这样做是为了避免分配新的 `String` 并在每个操作上复制整个内容，当通过重复连接构建 *n* 字节的字符串时，这将导致 *O*(*n*^ 2) 运行时间。<br>
///
///
/// The string on the right-hand side is only borrowed; <br>右侧的字符串仅是借用的。<br> its contents are copied into the returned `String`. <br>它的内容被复制到返回的 `String` 中。<br>
///
/// # Examples
///
/// Concatenating two `String`s takes the first by value and borrows the second: <br>将两个 `String` 连接起来，第一个按值取值，第二个借用：<br>
///
/// ```
/// let a = String::from("hello");
/// let b = String::from(" world");
/// let c = a + &b;
/// // `a` is moved and can no longer be used here. <br>`a` 已移动，不能再在此处使用。<br>
/// ```
///
/// If you want to keep using the first `String`, you can clone it and append to the clone instead: <br>如果要继续使用第一个 `String`，则可以对其进行克隆并追加到克隆中：<br>
///
/// ```
/// let a = String::from("hello");
/// let b = String::from(" world");
/// let c = a.clone() + &b;
/// // `a` is still valid here. <br>`a` 在这里仍然有效。<br>
/// ```
///
/// Concatenating `&str` slices can be done by converting the first to a `String`: <br>可以通过将第一个切片转换为 `String` 来完成 `&str` 切片的连接：<br>
///
/// ```
/// let a = "hello";
/// let b = " world";
/// let c = a.to_string() + b;
/// ```
///
///
#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl Add<&str> for String {
    type Output = String;

    #[inline]
    fn add(mut self, other: &str) -> String {
        self.push_str(other);
        self
    }
}

/// Implements the `+=` operator for appending to a `String`. <br>实现用于追加到 `String` 的 `+=` 运算符。<br>
///
/// This has the same behavior as the [`push_str`][String::push_str] method. <br>这与 [`push_str`][String::push_str] 方法具有相同的行为。<br>
#[cfg(not(no_global_oom_handling))]
#[stable(feature = "stringaddassign", since = "1.12.0")]
impl AddAssign<&str> for String {
    #[inline]
    fn add_assign(&mut self, other: &str) {
        self.push_str(other);
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl ops::Index<ops::Range<usize>> for String {
    type Output = str;

    #[inline]
    fn index(&self, index: ops::Range<usize>) -> &str {
        &self[..][index]
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl ops::Index<ops::RangeTo<usize>> for String {
    type Output = str;

    #[inline]
    fn index(&self, index: ops::RangeTo<usize>) -> &str {
        &self[..][index]
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl ops::Index<ops::RangeFrom<usize>> for String {
    type Output = str;

    #[inline]
    fn index(&self, index: ops::RangeFrom<usize>) -> &str {
        &self[..][index]
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl ops::Index<ops::RangeFull> for String {
    type Output = str;

    #[inline]
    fn index(&self, _index: ops::RangeFull) -> &str {
        unsafe { str::from_utf8_unchecked(&self.vec) }
    }
}
#[stable(feature = "inclusive_range", since = "1.26.0")]
impl ops::Index<ops::RangeInclusive<usize>> for String {
    type Output = str;

    #[inline]
    fn index(&self, index: ops::RangeInclusive<usize>) -> &str {
        Index::index(&**self, index)
    }
}
#[stable(feature = "inclusive_range", since = "1.26.0")]
impl ops::Index<ops::RangeToInclusive<usize>> for String {
    type Output = str;

    #[inline]
    fn index(&self, index: ops::RangeToInclusive<usize>) -> &str {
        Index::index(&**self, index)
    }
}

#[stable(feature = "derefmut_for_string", since = "1.3.0")]
impl ops::IndexMut<ops::Range<usize>> for String {
    #[inline]
    fn index_mut(&mut self, index: ops::Range<usize>) -> &mut str {
        &mut self[..][index]
    }
}
#[stable(feature = "derefmut_for_string", since = "1.3.0")]
impl ops::IndexMut<ops::RangeTo<usize>> for String {
    #[inline]
    fn index_mut(&mut self, index: ops::RangeTo<usize>) -> &mut str {
        &mut self[..][index]
    }
}
#[stable(feature = "derefmut_for_string", since = "1.3.0")]
impl ops::IndexMut<ops::RangeFrom<usize>> for String {
    #[inline]
    fn index_mut(&mut self, index: ops::RangeFrom<usize>) -> &mut str {
        &mut self[..][index]
    }
}
#[stable(feature = "derefmut_for_string", since = "1.3.0")]
impl ops::IndexMut<ops::RangeFull> for String {
    #[inline]
    fn index_mut(&mut self, _index: ops::RangeFull) -> &mut str {
        unsafe { str::from_utf8_unchecked_mut(&mut *self.vec) }
    }
}
#[stable(feature = "inclusive_range", since = "1.26.0")]
impl ops::IndexMut<ops::RangeInclusive<usize>> for String {
    #[inline]
    fn index_mut(&mut self, index: ops::RangeInclusive<usize>) -> &mut str {
        IndexMut::index_mut(&mut **self, index)
    }
}
#[stable(feature = "inclusive_range", since = "1.26.0")]
impl ops::IndexMut<ops::RangeToInclusive<usize>> for String {
    #[inline]
    fn index_mut(&mut self, index: ops::RangeToInclusive<usize>) -> &mut str {
        IndexMut::index_mut(&mut **self, index)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl ops::Deref for String {
    type Target = str;

    #[inline]
    fn deref(&self) -> &str {
        unsafe { str::from_utf8_unchecked(&self.vec) }
    }
}

#[stable(feature = "derefmut_for_string", since = "1.3.0")]
impl ops::DerefMut for String {
    #[inline]
    fn deref_mut(&mut self) -> &mut str {
        unsafe { str::from_utf8_unchecked_mut(&mut *self.vec) }
    }
}

/// A type alias for [`Infallible`]. <br>[`Infallible`] 的类型别名。<br>
///
/// This alias exists for backwards compatibility, and may be eventually deprecated. <br>存在此别名是为了向后兼容，并且最终可能会弃用该别名。<br>
///
/// [`Infallible`]: core::convert::Infallible "convert::Infallible"
#[stable(feature = "str_parse_error", since = "1.5.0")]
pub type ParseError = core::convert::Infallible;

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl FromStr for String {
    type Err = core::convert::Infallible;
    #[inline]
    fn from_str(s: &str) -> Result<String, Self::Err> {
        Ok(String::from(s))
    }
}

/// A trait for converting a value to a `String`. <br>一个用于将值转换为 `String` 的 trait。<br>
///
/// This trait is automatically implemented for any type which implements the [`Display`] trait. <br>对于任何实现 [`Display`] trait 的类型，都会自动实现 trait。<br>
/// As such, `ToString` shouldn't be implemented directly: <br>因此，不应直接实现 `ToString`：<br>
/// [`Display`] should be implemented instead, and you get the `ToString` implementation for free. <br>应该实现 [`Display`]，您可以免费获得 `ToString` 实现。<br>
///
///
/// [`Display`]: fmt::Display
#[cfg_attr(not(test), rustc_diagnostic_item = "ToString")]
#[stable(feature = "rust1", since = "1.0.0")]
pub trait ToString {
    /// Converts the given value to a `String`. <br>将给定值转换为 `String`。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let i = 5;
    /// let five = String::from("5");
    ///
    /// assert_eq!(five, i.to_string());
    /// ```
    #[rustc_conversion_suggestion]
    #[stable(feature = "rust1", since = "1.0.0")]
    fn to_string(&self) -> String;
}

/// # Panics
///
/// In this implementation, the `to_string` method panics if the `Display` implementation returns an error. <br>在此实现中，如果 `Display` 实现返回错误，则 `to_string` 方法 panics。<br>
/// This indicates an incorrect `Display` implementation since `fmt::Write for String` never returns an error itself. <br>这表示 `Display` 实现不正确，因为 `fmt::Write for String` 本身从不返回错误。<br>
///
///
#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl<T: fmt::Display + ?Sized> ToString for T {
    // A common guideline is to not inline generic functions. <br>常见的准则是不要内联泛型函数。<br>
    // However, removing `#[inline]` from this method causes non-negligible regressions. <br>但是，从此方法中删除 `#[inline]` 会导致不可忽略的回归。<br>
    // See <https://github.com/rust-lang/rust/pull/74852>, the last attempt to try to remove it. <br>请参见 <https://github.com/rust-lang/rust/pull/74852>，这是尝试将其删除的最后一次尝试。<br>
    //
    #[inline]
    default fn to_string(&self) -> String {
        let mut buf = String::new();
        let mut formatter = core::fmt::Formatter::new(&mut buf);
        // Bypass format_args!() to avoid write_str with zero-length strs <br>绕过 format_args!() 以避免使用零长度字符串 write_str<br>
        fmt::Display::fmt(self, &mut formatter)
            .expect("a Display implementation returned an error unexpectedly");
        buf
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "char_to_string_specialization", since = "1.46.0")]
impl ToString for char {
    #[inline]
    fn to_string(&self) -> String {
        String::from(self.encode_utf8(&mut [0; 4]))
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "u8_to_string_specialization", since = "1.54.0")]
impl ToString for u8 {
    #[inline]
    fn to_string(&self) -> String {
        let mut buf = String::with_capacity(3);
        let mut n = *self;
        if n >= 10 {
            if n >= 100 {
                buf.push((b'0' + n / 100) as char);
                n %= 100;
            }
            buf.push((b'0' + n / 10) as char);
            n %= 10;
        }
        buf.push((b'0' + n) as char);
        buf
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "i8_to_string_specialization", since = "1.54.0")]
impl ToString for i8 {
    #[inline]
    fn to_string(&self) -> String {
        let mut buf = String::with_capacity(4);
        if self.is_negative() {
            buf.push('-');
        }
        let mut n = self.unsigned_abs();
        if n >= 10 {
            if n >= 100 {
                buf.push('1');
                n -= 100;
            }
            buf.push((b'0' + n / 10) as char);
            n %= 10;
        }
        buf.push((b'0' + n) as char);
        buf
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "str_to_string_specialization", since = "1.9.0")]
impl ToString for str {
    #[inline]
    fn to_string(&self) -> String {
        String::from(self)
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "cow_str_to_string_specialization", since = "1.17.0")]
impl ToString for Cow<'_, str> {
    #[inline]
    fn to_string(&self) -> String {
        self[..].to_owned()
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "string_to_string_specialization", since = "1.17.0")]
impl ToString for String {
    #[inline]
    fn to_string(&self) -> String {
        self.to_owned()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<str> for String {
    #[inline]
    fn as_ref(&self) -> &str {
        self
    }
}

#[stable(feature = "string_as_mut", since = "1.43.0")]
impl AsMut<str> for String {
    #[inline]
    fn as_mut(&mut self) -> &mut str {
        self
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<[u8]> for String {
    #[inline]
    fn as_ref(&self) -> &[u8] {
        self.as_bytes()
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl From<&str> for String {
    /// Converts a `&str` into a [`String`]. <br>将 `&str` 转换为 [`String`]。<br>
    ///
    /// The result is allocated on the heap. <br>结果分配在堆上。<br>
    #[inline]
    fn from(s: &str) -> String {
        s.to_owned()
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "from_mut_str_for_string", since = "1.44.0")]
impl From<&mut str> for String {
    /// Converts a `&mut str` into a [`String`]. <br>将 `&mut str` 转换为 [`String`]。<br>
    ///
    /// The result is allocated on the heap. <br>结果分配在堆上。<br>
    #[inline]
    fn from(s: &mut str) -> String {
        s.to_owned()
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "from_ref_string", since = "1.35.0")]
impl From<&String> for String {
    /// Converts a `&String` into a [`String`]. <br>将 `&String` 转换为 [`String`]。<br>
    ///
    /// This clones `s` and returns the clone. <br>这将克隆 `s` 并返回该克隆。<br>
    #[inline]
    fn from(s: &String) -> String {
        s.clone()
    }
}

// note: test pulls in libstd, which causes errors here <br>测试引入 libstd，这会导致错误<br>
#[cfg(not(test))]
#[stable(feature = "string_from_box", since = "1.18.0")]
impl From<Box<str>> for String {
    /// Converts the given boxed `str` slice to a [`String`]. <br>将给定的 boxed `str` 切片转换为 [`String`]。<br>
    /// It is notable that the `str` slice is owned. <br>值得注意的是，`str` 切片是拥有的。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let s1: String = String::from("hello world");
    /// let s2: Box<str> = s1.into_boxed_str();
    /// let s3: String = String::from(s2);
    ///
    /// assert_eq!("hello world", s3)
    /// ```
    fn from(s: Box<str>) -> String {
        s.into_string()
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "box_from_str", since = "1.20.0")]
impl From<String> for Box<str> {
    /// Converts the given [`String`] to a boxed `str` slice that is owned. <br>将给定的 [`String`] 转换为拥有所有权的 boxed `str` 切片。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let s1: String = String::from("hello world");
    /// let s2: Box<str> = Box::from(s1);
    /// let s3: String = String::from(s2);
    ///
    /// assert_eq!("hello world", s3)
    /// ```
    fn from(s: String) -> Box<str> {
        s.into_boxed_str()
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "string_from_cow_str", since = "1.14.0")]
impl<'a> From<Cow<'a, str>> for String {
    /// Converts a clone-on-write string to an owned instance of [`String`]. <br>将写时克隆字符串转换为 [`String`] 的拥有实例。<br>
    ///
    /// This extracts the owned string, clones the string if it is not already owned. <br>这将提取拥有所有权的字符串，如果尚未拥有，则克隆该字符串。<br>
    ///
    ///
    /// # Example
    ///
    /// ```
    /// # use std::borrow::Cow;
    /// // If the string is not owned... <br>如果该字符串不被拥有...<br>
    /// let cow: Cow<str> = Cow::Borrowed("eggplant");
    /// // It will allocate on the heap and copy the string. <br>它将在堆上分配并复制字符串。<br>
    /// let owned: String = String::from(cow);
    /// assert_eq!(&owned[..], "eggplant");
    /// ```
    ///
    fn from(s: Cow<'a, str>) -> String {
        s.into_owned()
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> From<&'a str> for Cow<'a, str> {
    /// Converts a string slice into a [`Borrowed`] variant. <br>将字符串切片转换为 [`Borrowed`] 变体。<br>
    /// No heap allocation is performed, and the string is not copied. <br>不执行堆分配，并且不复制字符串。<br>
    ///
    /// # Example
    ///
    /// ```
    /// # use std::borrow::Cow;
    /// assert_eq!(Cow::from("eggplant"), Cow::Borrowed("eggplant"));
    /// ```
    ///
    /// [`Borrowed`]: crate::borrow::Cow::Borrowed "borrow::Cow::Borrowed"
    ///
    #[inline]
    fn from(s: &'a str) -> Cow<'a, str> {
        Cow::Borrowed(s)
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> From<String> for Cow<'a, str> {
    /// Converts a [`String`] into an [`Owned`] variant. <br>将 [`String`] 转换为 [`Owned`] 变体。<br>
    /// No heap allocation is performed, and the string is not copied. <br>不执行堆分配，并且不复制字符串。<br>
    ///
    ///
    /// # Example
    ///
    /// ```
    /// # use std::borrow::Cow;
    /// let s = "eggplant".to_string();
    /// let s2 = "eggplant".to_string();
    /// assert_eq!(Cow::from(s), Cow::<'static, str>::Owned(s2));
    /// ```
    ///
    /// [`Owned`]: crate::borrow::Cow::Owned "borrow::Cow::Owned"
    #[inline]
    fn from(s: String) -> Cow<'a, str> {
        Cow::Owned(s)
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "cow_from_string_ref", since = "1.28.0")]
impl<'a> From<&'a String> for Cow<'a, str> {
    /// Converts a [`String`] reference into a [`Borrowed`] variant. <br>将 [`String`] 引用转换为 [`Borrowed`] 变体。<br>
    /// No heap allocation is performed, and the string is not copied. <br>不执行堆分配，并且不复制字符串。<br>
    ///
    /// # Example
    ///
    /// ```
    /// # use std::borrow::Cow;
    /// let s = "eggplant".to_string();
    /// assert_eq!(Cow::from(&s), Cow::Borrowed("eggplant"));
    /// ```
    ///
    /// [`Borrowed`]: crate::borrow::Cow::Borrowed "borrow::Cow::Borrowed"
    ///
    #[inline]
    fn from(s: &'a String) -> Cow<'a, str> {
        Cow::Borrowed(s.as_str())
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "cow_str_from_iter", since = "1.12.0")]
impl<'a> FromIterator<char> for Cow<'a, str> {
    fn from_iter<I: IntoIterator<Item = char>>(it: I) -> Cow<'a, str> {
        Cow::Owned(FromIterator::from_iter(it))
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "cow_str_from_iter", since = "1.12.0")]
impl<'a, 'b> FromIterator<&'b str> for Cow<'a, str> {
    fn from_iter<I: IntoIterator<Item = &'b str>>(it: I) -> Cow<'a, str> {
        Cow::Owned(FromIterator::from_iter(it))
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "cow_str_from_iter", since = "1.12.0")]
impl<'a> FromIterator<String> for Cow<'a, str> {
    fn from_iter<I: IntoIterator<Item = String>>(it: I) -> Cow<'a, str> {
        Cow::Owned(FromIterator::from_iter(it))
    }
}

#[stable(feature = "from_string_for_vec_u8", since = "1.14.0")]
impl From<String> for Vec<u8> {
    /// Converts the given [`String`] to a vector [`Vec`] that holds values of type [`u8`]. <br>将给定的 [`String`] 转换为包含 [`u8`] 类型值的 vector [`Vec`]。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let s1 = String::from("hello world");
    /// let v1 = Vec::from(s1);
    ///
    /// for b in v1 {
    ///     println!("{b}");
    /// }
    /// ```
    fn from(string: String) -> Vec<u8> {
        string.into_bytes()
    }
}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Write for String {
    #[inline]
    fn write_str(&mut self, s: &str) -> fmt::Result {
        self.push_str(s);
        Ok(())
    }

    #[inline]
    fn write_char(&mut self, c: char) -> fmt::Result {
        self.push(c);
        Ok(())
    }
}

/// A draining iterator for `String`. <br>`String` 的 draining 迭代器。<br>
///
/// This struct is created by the [`drain`] method on [`String`]. <br>该结构体是通过 [`String`] 上的 [`drain`] 方法创建的。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// [`drain`]: String::drain
#[stable(feature = "drain", since = "1.6.0")]
pub struct Drain<'a> {
    /// Will be used as &'a mut String in the destructor <br>将在析构函数中用作 &'a mut String<br>
    string: *mut String,
    /// Start of part to remove <br>要移除的部分的开始<br>
    start: usize,
    /// End of part to remove <br>要移除的部分的结束<br>
    end: usize,
    /// Current remaining range to remove <br>当前剩余范围要删除<br>
    iter: Chars<'a>,
}

#[stable(feature = "collection_debug", since = "1.17.0")]
impl fmt::Debug for Drain<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple("Drain").field(&self.as_str()).finish()
    }
}

#[stable(feature = "drain", since = "1.6.0")]
unsafe impl Sync for Drain<'_> {}
#[stable(feature = "drain", since = "1.6.0")]
unsafe impl Send for Drain<'_> {}

#[stable(feature = "drain", since = "1.6.0")]
impl Drop for Drain<'_> {
    fn drop(&mut self) {
        unsafe {
            // Use Vec::drain. <br>使用 Vec::drain。<br>
            // "Reaffirm" the bounds checks to avoid panic code being inserted again. <br>"Reaffirm" 边界检查以避免再次插入 panic 代码。<br>
            let self_vec = (*self.string).as_mut_vec();
            if self.start <= self.end && self.end <= self_vec.len() {
                self_vec.drain(self.start..self.end);
            }
        }
    }
}

impl<'a> Drain<'a> {
    /// Returns the remaining (sub)string of this iterator as a slice. <br>返回此迭代器的其余 (子) 字符串作为切片。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut s = String::from("abc");
    /// let mut drain = s.drain(..);
    /// assert_eq!(drain.as_str(), "abc");
    /// let _ = drain.next().unwrap();
    /// assert_eq!(drain.as_str(), "bc");
    /// ```
    #[must_use]
    #[stable(feature = "string_drain_as_str", since = "1.55.0")]
    pub fn as_str(&self) -> &str {
        self.iter.as_str()
    }
}

#[stable(feature = "string_drain_as_str", since = "1.55.0")]
impl<'a> AsRef<str> for Drain<'a> {
    fn as_ref(&self) -> &str {
        self.as_str()
    }
}

#[stable(feature = "string_drain_as_str", since = "1.55.0")]
impl<'a> AsRef<[u8]> for Drain<'a> {
    fn as_ref(&self) -> &[u8] {
        self.as_str().as_bytes()
    }
}

#[stable(feature = "drain", since = "1.6.0")]
impl Iterator for Drain<'_> {
    type Item = char;

    #[inline]
    fn next(&mut self) -> Option<char> {
        self.iter.next()
    }

    fn size_hint(&self) -> (usize, Option<usize>) {
        self.iter.size_hint()
    }

    #[inline]
    fn last(mut self) -> Option<char> {
        self.next_back()
    }
}

#[stable(feature = "drain", since = "1.6.0")]
impl DoubleEndedIterator for Drain<'_> {
    #[inline]
    fn next_back(&mut self) -> Option<char> {
        self.iter.next_back()
    }
}

#[stable(feature = "fused", since = "1.26.0")]
impl FusedIterator for Drain<'_> {}

#[cfg(not(no_global_oom_handling))]
#[stable(feature = "from_char_for_string", since = "1.46.0")]
impl From<char> for String {
    /// Allocates an owned [`String`] from a single character. <br>从单个字符分配一个拥有所有权的 [`String`]。<br>
    ///
    /// # Example
    /// ```rust
    /// let c: char = 'a';
    /// let s: String = String::from(c);
    /// assert_eq!("a", &s[..]);
    /// ```
    #[inline]
    fn from(c: char) -> Self {
        c.to_string()
    }
}
