use crate::borrow::Cow;
use core::iter::FromIterator;

use super::Vec;

#[stable(feature = "cow_from_vec", since = "1.8.0")]
impl<'a, T: Clone> From<&'a [T]> for Cow<'a, [T]> {
    /// Creates a [`Borrowed`] variant of [`Cow`] from a slice. <br>从一个切片创建一个 [`Cow`] 的 [`Borrowed`] 变体。<br>
    ///
    ///
    /// This conversion does not allocate or clone the data. <br>此转换不会分配或克隆数据。<br>
    ///
    /// [`Borrowed`]: crate::borrow::Cow::Borrowed
    fn from(s: &'a [T]) -> Cow<'a, [T]> {
        Cow::Borrowed(s)
    }
}

#[stable(feature = "cow_from_vec", since = "1.8.0")]
impl<'a, T: Clone> From<Vec<T>> for Cow<'a, [T]> {
    /// Creates an [`Owned`] variant of [`Cow`] from an owned instance of [`Vec`]. <br>从拥有所有权的 [`Vec`] 实例创建 [`Cow`] 的 [`Owned`] 变体。<br>
    ///
    ///
    /// This conversion does not allocate or clone the data. <br>此转换不会分配或克隆数据。<br>
    ///
    /// [`Owned`]: crate::borrow::Cow::Owned
    fn from(v: Vec<T>) -> Cow<'a, [T]> {
        Cow::Owned(v)
    }
}

#[stable(feature = "cow_from_vec_ref", since = "1.28.0")]
impl<'a, T: Clone> From<&'a Vec<T>> for Cow<'a, [T]> {
    /// Creates a [`Borrowed`] variant of [`Cow`] from a reference to [`Vec`]. <br>从 [`Vec`] 的引用创建 [`Cow`] 的 [`Borrowed`] 变体。<br>
    ///
    ///
    /// This conversion does not allocate or clone the data. <br>此转换不会分配或克隆数据。<br>
    ///
    /// [`Borrowed`]: crate::borrow::Cow::Borrowed
    fn from(v: &'a Vec<T>) -> Cow<'a, [T]> {
        Cow::Borrowed(v.as_slice())
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, T> FromIterator<T> for Cow<'a, [T]>
where
    T: Clone,
{
    fn from_iter<I: IntoIterator<Item = T>>(it: I) -> Cow<'a, [T]> {
        Cow::Owned(FromIterator::from_iter(it))
    }
}
