use crate::alloc::{Allocator, Global};
use core::ptr::{self};
use core::slice::{self};

use super::{Drain, Vec};

/// A splicing iterator for `Vec`. <br>`Vec` 的拼接迭代器。<br>
///
/// This struct is created by [`Vec::splice()`]. <br>这个结构体是由 [`Vec::splice ()`] 创建的。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// # Example
///
/// ```
/// let mut v = vec![0, 1, 2];
/// let new = [7, 8];
/// let iter: std::vec::Splice<_> = v.splice(1.., new);
/// ```
#[derive(Debug)]
#[stable(feature = "vec_splice", since = "1.21.0")]
pub struct Splice<
    'a,
    I: Iterator + 'a,
    #[unstable(feature = "allocator_api", issue = "32838")] A: Allocator + 'a = Global,
> {
    pub(super) drain: Drain<'a, I::Item, A>,
    pub(super) replace_with: I,
}

#[stable(feature = "vec_splice", since = "1.21.0")]
impl<I: Iterator, A: Allocator> Iterator for Splice<'_, I, A> {
    type Item = I::Item;

    fn next(&mut self) -> Option<Self::Item> {
        self.drain.next()
    }

    fn size_hint(&self) -> (usize, Option<usize>) {
        self.drain.size_hint()
    }
}

#[stable(feature = "vec_splice", since = "1.21.0")]
impl<I: Iterator, A: Allocator> DoubleEndedIterator for Splice<'_, I, A> {
    fn next_back(&mut self) -> Option<Self::Item> {
        self.drain.next_back()
    }
}

#[stable(feature = "vec_splice", since = "1.21.0")]
impl<I: Iterator, A: Allocator> ExactSizeIterator for Splice<'_, I, A> {}

#[stable(feature = "vec_splice", since = "1.21.0")]
impl<I: Iterator, A: Allocator> Drop for Splice<'_, I, A> {
    fn drop(&mut self) {
        self.drain.by_ref().for_each(drop);

        unsafe {
            if self.drain.tail_len == 0 {
                self.drain.vec.as_mut().extend(self.replace_with.by_ref());
                return;
            }

            // First fill the range left by drain(). <br>首先填充 drain() 剩余的范围。<br>
            if !self.drain.fill(&mut self.replace_with) {
                return;
            }

            // There may be more elements. <br>可能还有更多元素。<br> Use the lower bound as an estimate. <br>使用下限作为估计值。<br>
            // FIXME: Is the upper bound a better guess? <br>上限是更好的猜测吗？<br> Or something else? <br>或者是其他东西？<br>
            let (lower_bound, _upper_bound) = self.replace_with.size_hint();
            if lower_bound > 0 {
                self.drain.move_tail(lower_bound);
                if !self.drain.fill(&mut self.replace_with) {
                    return;
                }
            }

            // Collect any remaining elements. <br>收集所有剩余的元素。<br>
            // This is a zero-length vector which does not allocate if `lower_bound` was exact. <br>这是一个零长度的 vector，如果 `lower_bound` 是精确的，则不分配。<br>
            let mut collected = self.replace_with.by_ref().collect::<Vec<I::Item>>().into_iter();
            // Now we have an exact count. <br>现在我们有了一个准确的计数。<br>
            if collected.len() > 0 {
                self.drain.move_tail(collected.len());
                let filled = self.drain.fill(&mut collected);
                debug_assert!(filled);
                debug_assert_eq!(collected.len(), 0);
            }
        }
        // Let `Drain::drop` move the tail back if necessary and restore `vec.len`. <br>如有必要，让 `Drain::drop` 将尾巴向后移，并恢复 `vec.len`。<br>
    }
}

/// Private helper methods for `Splice::drop` <br>`Splice::drop` 的 private 帮助器方法<br>
impl<T, A: Allocator> Drain<'_, T, A> {
    /// The range from `self.vec.len` to `self.tail_start` contains elements that have been moved out. <br>从 `self.vec.len` 到 `self.tail_start` 的范围包含已移出的元素。<br>
    ///
    /// Fill that range as much as possible with new elements from the `replace_with` iterator. <br>尽可能用 `replace_with` 迭代器中的新元素填充该范围。<br>
    /// Returns `true` if we filled the entire range. <br>如果我们填满整个范围，则返回 `true`。<br> (`replace_with.next()` didn’t return `None`.) <br>(`replace_with.next()` 没有返回 `None`。)<br>
    unsafe fn fill<I: Iterator<Item = T>>(&mut self, replace_with: &mut I) -> bool {
        let vec = unsafe { self.vec.as_mut() };
        let range_start = vec.len;
        let range_end = self.tail_start;
        let range_slice = unsafe {
            slice::from_raw_parts_mut(vec.as_mut_ptr().add(range_start), range_end - range_start)
        };

        for place in range_slice {
            if let Some(new_item) = replace_with.next() {
                unsafe { ptr::write(place, new_item) };
                vec.len += 1;
            } else {
                return false;
            }
        }
        true
    }

    /// Makes room for inserting more elements before the tail. <br>为在尾部之前插入更多元素留出空间。<br>
    unsafe fn move_tail(&mut self, additional: usize) {
        let vec = unsafe { self.vec.as_mut() };
        let len = self.tail_start + self.tail_len;
        vec.buf.reserve(len, additional);

        let new_tail_start = self.tail_start + additional;
        unsafe {
            let src = vec.as_ptr().add(self.tail_start);
            let dst = vec.as_mut_ptr().add(new_tail_start);
            ptr::copy(src, dst, self.tail_len);
        }
        self.tail_start = new_tail_start;
    }
}
