//! Operations on ASCII strings and characters. <br>对 ASCII 字符串和字符的操作。<br>
//!
//! Most string operations in Rust act on UTF-8 strings. <br>Rust 中的大多数字符串操作都作用于 UTF-8 字符串。<br>
//! However, at times it makes more sense to only consider the ASCII character set for a specific operation. <br>但是，有时只考虑针对特定操作的 ASCII 字符集更有意义。<br>
//!
//! The [`escape_default`] function provides an iterator over the bytes of an escaped version of the character given. <br>[`escape_default`] 函数在给定字符的转义版本的字节上提供迭代器。<br>
//!
//!

#![stable(feature = "core_ascii", since = "1.26.0")]

use crate::fmt;
use crate::iter::FusedIterator;
use crate::ops::Range;
use crate::str::from_utf8_unchecked;

/// An iterator over the escaped version of a byte. <br>在字节的转义版本上进行迭代的迭代器。<br>
///
/// This `struct` is created by the [`escape_default`] function. <br>该 `struct` 由 [`escape_default`] 函数创建。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Clone)]
pub struct EscapeDefault {
    range: Range<u8>,
    data: [u8; 4],
}

/// Returns an iterator that produces an escaped version of a `u8`. <br>返回一个迭代器，该迭代器生成 `u8` 的转义版本。<br>
///
/// The default is chosen with a bias toward producing literals that are legal in a variety of languages, including C++11 and similar C-family languages. <br>选择默认值时会偏向于生成在多种语言 (包括 C++ 11 和类似的 C 系列语言) 中都合法的字面量。<br>
/// The exact rules are: <br>确切的规则是：<br>
///
/// * Tab is escaped as `\t`. <br>制表符被转义为 `\t`。<br>
/// * Carriage return is escaped as `\r`. <br>回车符被转义为 `\r`。<br>
/// * Line feed is escaped as `\n`. <br>换行符转为 `\n`。<br>
/// * Single quote is escaped as `\'`. <br>单引号转义为 `\'`。<br>
/// * Double quote is escaped as `\"`. <br>双引号转义为 `\"`。<br>
/// * Backslash is escaped as `\\`. <br>反斜杠转义为 `\\`。<br>
/// * Any character in the 'printable ASCII' range `0x20` .. <br>`可打印 ASCII` 范围 `0x20` .. 中的任何字符<br> `0x7e` inclusive is not escaped. <br>`0x7e` (含 `0x7e`) 不会转义。<br>
/// * Any other chars are given hex escapes of the form '\xNN'. <br>任何其他字符都以 '\xNN' 形式给出十六进制转义。<br>
/// * Unicode escapes are never generated by this function. <br>此函数从不生成 Unicode 转义。<br>
///
/// # Examples
///
/// ```
/// use std::ascii;
///
/// let escaped = ascii::escape_default(b'0').next().unwrap();
/// assert_eq!(b'0', escaped);
///
/// let mut escaped = ascii::escape_default(b'\t');
///
/// assert_eq!(b'\\', escaped.next().unwrap());
/// assert_eq!(b't', escaped.next().unwrap());
///
/// let mut escaped = ascii::escape_default(b'\r');
///
/// assert_eq!(b'\\', escaped.next().unwrap());
/// assert_eq!(b'r', escaped.next().unwrap());
///
/// let mut escaped = ascii::escape_default(b'\n');
///
/// assert_eq!(b'\\', escaped.next().unwrap());
/// assert_eq!(b'n', escaped.next().unwrap());
///
/// let mut escaped = ascii::escape_default(b'\'');
///
/// assert_eq!(b'\\', escaped.next().unwrap());
/// assert_eq!(b'\'', escaped.next().unwrap());
///
/// let mut escaped = ascii::escape_default(b'"');
///
/// assert_eq!(b'\\', escaped.next().unwrap());
/// assert_eq!(b'"', escaped.next().unwrap());
///
/// let mut escaped = ascii::escape_default(b'\\');
///
/// assert_eq!(b'\\', escaped.next().unwrap());
/// assert_eq!(b'\\', escaped.next().unwrap());
///
/// let mut escaped = ascii::escape_default(b'\x9d');
///
/// assert_eq!(b'\\', escaped.next().unwrap());
/// assert_eq!(b'x', escaped.next().unwrap());
/// assert_eq!(b'9', escaped.next().unwrap());
/// assert_eq!(b'd', escaped.next().unwrap());
/// ```
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn escape_default(c: u8) -> EscapeDefault {
    let (data, len) = match c {
        b'\t' => ([b'\\', b't', 0, 0], 2),
        b'\r' => ([b'\\', b'r', 0, 0], 2),
        b'\n' => ([b'\\', b'n', 0, 0], 2),
        b'\\' => ([b'\\', b'\\', 0, 0], 2),
        b'\'' => ([b'\\', b'\'', 0, 0], 2),
        b'"' => ([b'\\', b'"', 0, 0], 2),
        b'\x20'..=b'\x7e' => ([c, 0, 0, 0], 1),
        _ => {
            let hex_digits: &[u8; 16] = b"0123456789abcdef";
            ([b'\\', b'x', hex_digits[(c >> 4) as usize], hex_digits[(c & 0xf) as usize]], 4)
        }
    };

    return EscapeDefault { range: 0..len, data };
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Iterator for EscapeDefault {
    type Item = u8;

    #[inline]
    fn next(&mut self) -> Option<u8> {
        self.range.next().map(|i| self.data[i as usize])
    }
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.range.size_hint()
    }
    fn last(mut self) -> Option<u8> {
        self.next_back()
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl DoubleEndedIterator for EscapeDefault {
    fn next_back(&mut self) -> Option<u8> {
        self.range.next_back().map(|i| self.data[i as usize])
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl ExactSizeIterator for EscapeDefault {}
#[stable(feature = "fused", since = "1.26.0")]
impl FusedIterator for EscapeDefault {}

#[stable(feature = "ascii_escape_display", since = "1.39.0")]
impl fmt::Display for EscapeDefault {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        // SAFETY: ok because `escape_default` created only valid utf-8 data <br>好的，因为 `escape_default` 仅创建了有效的 utf-8 数据<br>
        f.write_str(unsafe {
            from_utf8_unchecked(&self.data[(self.range.start as usize)..(self.range.end as usize)])
        })
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for EscapeDefault {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("EscapeDefault").finish_non_exhaustive()
    }
}
