// 包含打印有用的 `assert!` 消息所需的机制。不适合公共使用，甚至不适合夜间用例。
//
// 基于 https://github.com/autoref-specialization。当'specialization' 足够健壮时 (5 年? 10 年? 从不? )，`Capture` 可以专门化为 [Printable]。
//
//
//

#![allow(missing_debug_implementations)]
#![doc(hidden)]
#![unstable(feature = "generic_assert_internals", issue = "44838")]

use crate::{
    fmt::{Debug, Formatter},
    marker::PhantomData,
};

// ***** TryCapture - 泛型 *****

/// [Capture] 使用的标记
#[unstable(feature = "generic_assert_internals", issue = "44838")]
pub struct TryCaptureWithoutDebug;

/// 捕获任意 `E` 并相应地修改 `to`
#[unstable(feature = "generic_assert_internals", issue = "44838")]
pub trait TryCaptureGeneric<E, M> {
    /// 类似于 [TryCapturePrintable] 但泛型到任何 `E`。
    fn try_capture(&self, to: &mut Capture<E, M>);
}

impl<E> TryCaptureGeneric<E, TryCaptureWithoutDebug> for &Wrapper<&E> {
    #[inline]
    fn try_capture(&self, _: &mut Capture<E, TryCaptureWithoutDebug>) {}
}

impl<E> Debug for Capture<E, TryCaptureWithoutDebug> {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), core::fmt::Error> {
        f.write_str("N/A")
    }
}

// ***** TryCapture - 可打印 *****

/// [Capture] 使用的标记
#[unstable(feature = "generic_assert_internals", issue = "44838")]
pub struct TryCaptureWithDebug;

/// 捕获任意 `E: Printable` 并相应地修改 `to`
#[unstable(feature = "generic_assert_internals", issue = "44838")]
pub trait TryCapturePrintable<E, M> {
    /// 与 [TryCaptureGeneric] 类似，但专门用于任何 `E: Printable`。
    fn try_capture(&self, to: &mut Capture<E, M>);
}

impl<E> TryCapturePrintable<E, TryCaptureWithDebug> for Wrapper<&E>
where
    E: Printable,
{
    #[inline]
    fn try_capture(&self, to: &mut Capture<E, TryCaptureWithDebug>) {
        to.elem = Some(*self.0);
    }
}

impl<E> Debug for Capture<E, TryCaptureWithDebug>
where
    E: Printable,
{
    fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), core::fmt::Error> {
        match self.elem {
            None => f.write_str("N/A"),
            Some(ref value) => Debug::fmt(value, f),
        }
    }
}

// ***** 其他 *****

/// 所有可能捕获的 `assert!` 元素
///
/// # Types
///
/// * `E`: 将要显示的元素。
/// * `M`: 用于区分 [Debug] 的 [Capture] 的标记。
#[unstable(feature = "generic_assert_internals", issue = "44838")]
pub struct Capture<E, M> {
    // 如果没有，那么 `E` 没有实现 [Printable] 或 `E` 没有被评估 (`assert!(...
    // )` short-circuited).
    //
    // 如果是 Some，则 `E` 实现 [Printable] 并被评估。
    pub elem: Option<E>,
    phantom: PhantomData<M>,
}

impl<M, T> Capture<M, T> {
    #[inline]
    pub const fn new() -> Self {
        Self { elem: None, phantom: PhantomData }
    }
}

/// `TryCapture*` 的实现所必需的
#[unstable(feature = "generic_assert_internals", issue = "44838")]
pub struct Wrapper<T>(pub T);

/// 告诉可以复制和显示哪些元素
#[unstable(feature = "generic_assert_internals", issue = "44838")]
pub trait Printable: Copy + Debug {}

impl<T> Printable for T where T: Copy + Debug {}
