use crate::ascii;
use crate::cmp::Ordering;
use crate::ffi::c_char;
use crate::fmt::{self, Write};
use crate::ops;
use crate::slice;
use crate::slice::memchr;
use crate::str;

/// Representation of a borrowed C string. <br>借用的 C 字符串的表示形式。<br>
///
/// This type represents a borrowed reference to a nul-terminated array of bytes. <br>此类型表示对以 n 结尾的字节数组的引用。<br>
/// It can be constructed safely from a <code>&[[u8]]</code> slice, or unsafely from a raw `*const c_char`. <br>它可以从一个 <code>&[[u8]]</code> 切片安全地构建，或者从原始 `*const c_char` 不安全地构建。<br>
/// It can then be converted to a Rust <code>&[str]</code> by performing UTF-8 validation, or into an owned `CString`. <br>然后可以通过执行 UTF-8 验证将其转换为 Rust <code>&[str]</code>，或转换为拥有所有权的 `CString`。<br>
///
/// `&CStr` is to `CString` as <code>&[str]</code> is to `String`: the former in each pair are borrowed references; <br>`&CStr` 对 `CString` 就像 <code>&[str]</code> 对 `String` 一样: 每对中的前者都是借用的引用;<br> the latter are owned strings. <br>后者是拥有的字符串。<br>
///
/// Note that this structure is **not** `repr(C)` and is not recommended to be placed in the signatures of FFI functions. <br>请注意，此结构体不是 `repr(C)`，不建议放置在 FFI 函数的签名中。<br>
/// Instead, safe wrappers of FFI functions may leverage the unsafe [`CStr::from_ptr`] constructor to provide a safe interface to other consumers. <br>而是，FFI 函数的安全包装程序可以利用不安全的 [`CStr::from_ptr`] 构造函数为其他使用者提供安全的接口。<br>
///
///
/// # Examples
///
/// Inspecting a foreign C string: <br>检查外部 C 字符串：<br>
///
/// ```ignore (extern-declaration)
/// use std::ffi::CStr;
/// use std::os::raw::c_char;
///
/// extern "C" { fn my_string() -> *const c_char; }
///
/// unsafe {
///     let slice = CStr::from_ptr(my_string());
///     println!("string buffer size without nul terminator: {}", slice.to_bytes().len());
/// }
/// ```
///
/// Passing a Rust-originating C string: <br>传递源自 Rust 的 C 字符串：<br>
///
/// ```ignore (extern-declaration)
/// use std::ffi::{CString, CStr};
/// use std::os::raw::c_char;
///
/// fn work(data: &CStr) {
///     extern "C" { fn work_with(data: *const c_char); }
///
///     unsafe { work_with(data.as_ptr()) }
/// }
///
/// let s = CString::new("data data data data").expect("CString::new failed");
/// work(&s);
/// ```
///
/// Converting a foreign C string into a Rust `String`: <br>将外部 C 字符串转换为 Rust `String`:<br>
///
/// ```ignore (extern-declaration)
/// use std::ffi::CStr;
/// use std::os::raw::c_char;
///
/// extern "C" { fn my_string() -> *const c_char; }
///
/// fn my_string_safe() -> String {
///     unsafe {
///         CStr::from_ptr(my_string()).to_string_lossy().into_owned()
///     }
/// }
///
/// println!("string: {}", my_string_safe());
/// ```
///
/// [str]: prim@str "str"
///
///
///
///
///
#[derive(Hash)]
#[cfg_attr(not(test), rustc_diagnostic_item = "CStr")]
#[unstable(feature = "core_c_str", issue = "94079")]
#[rustc_has_incoherent_inherent_impls]
// FIXME:
// `fn from` in `impl From<&CStr> for Box<CStr>` current implementation relies on `CStr` being layout-compatible with `[u8]`. <br>`impl From<&CStr> for Box<CStr>` 当前实现中的 `fn from` 依赖于 `CStr` 与 `[u8]` 布局兼容。<br>
// When attribute privacy is implemented, `CStr` should be annotated as `#[repr(transparent)]`. <br>实现属性隐私时，应将 `CStr` 注解为 `#[repr(transparent)]`。<br>
// Anyway, `CStr` representation and layout are considered implementation detail, are not documented and must not be relied upon. <br>无论如何，`CStr` 表示形式和布局被视为实现细节，没有文档记录，因此不能依赖。<br>
//
//
pub struct CStr {
    // FIXME: this should not be represented with a DST slice but rather with just a raw `c_char` along with some form of marker to make this an unsized type. <br>这不应该用 DST 切片来表示，而只能用原始 `c_char` 以及某种形式的标记将其表示为未定义大小的类型。<br>
    // Essentially `sizeof(&CStr)` should be the same as `sizeof(&c_char)` but `CStr` should be an unsized type. <br>本质上，`sizeof(&CStr)` 应该与 `sizeof(&c_char)` 相同，但是 `CStr` 应该是未定义大小的类型。<br>
    //
    //
    inner: [c_char],
}

/// An error indicating that a nul byte was not in the expected position. <br>指示 nul 字节不在预期位置中的错误。<br>
///
/// The slice used to create a [`CStr`] must have one and only one nul byte, positioned at the end. <br>用于创建 [`CStr`] 的切片必须位于末尾且只有一个 nul 字节。<br>
///
///
/// This error is created by the [`CStr::from_bytes_with_nul`] method. <br>此错误是由 [`CStr::from_bytes_with_nul`] 方法创建的。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// # Examples
///
/// ```
/// use std::ffi::{CStr, FromBytesWithNulError};
///
/// let _: FromBytesWithNulError = CStr::from_bytes_with_nul(b"f\0oo").unwrap_err();
/// ```
#[derive(Clone, PartialEq, Eq, Debug)]
#[unstable(feature = "core_c_str", issue = "94079")]
pub struct FromBytesWithNulError {
    kind: FromBytesWithNulErrorKind,
}

#[derive(Clone, PartialEq, Eq, Debug)]
enum FromBytesWithNulErrorKind {
    InteriorNul(usize),
    NotNulTerminated,
}

impl FromBytesWithNulError {
    fn interior_nul(pos: usize) -> FromBytesWithNulError {
        FromBytesWithNulError { kind: FromBytesWithNulErrorKind::InteriorNul(pos) }
    }
    fn not_nul_terminated() -> FromBytesWithNulError {
        FromBytesWithNulError { kind: FromBytesWithNulErrorKind::NotNulTerminated }
    }

    #[doc(hidden)]
    #[unstable(feature = "cstr_internals", issue = "none")]
    pub fn __description(&self) -> &str {
        match self.kind {
            FromBytesWithNulErrorKind::InteriorNul(..) => {
                "data provided contains an interior nul byte"
            }
            FromBytesWithNulErrorKind::NotNulTerminated => "data provided is not nul terminated",
        }
    }
}

/// An error indicating that no nul byte was present. <br>指示不存在空字节的错误。<br>
///
/// A slice used to create a [`CStr`] must contain a nul byte somewhere within the slice. <br>用于创建 [`CStr`] 的切片必须在切片内的某处包含一个空字节。<br>
///
///
/// This error is created by the [`CStr::from_bytes_until_nul`] method. <br>此错误是由 [`CStr::from_bytes_until_nul`] 方法创建的。<br>
///
#[derive(Clone, PartialEq, Eq, Debug)]
#[unstable(feature = "cstr_from_bytes_until_nul", issue = "95027")]
pub struct FromBytesUntilNulError(());

#[unstable(feature = "cstr_from_bytes_until_nul", issue = "95027")]
impl fmt::Display for FromBytesUntilNulError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "data provided does not contain a nul")
    }
}

#[stable(feature = "cstr_debug", since = "1.3.0")]
impl fmt::Debug for CStr {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "\"")?;
        for byte in self.to_bytes().iter().flat_map(|&b| ascii::escape_default(b)) {
            f.write_char(byte as char)?;
        }
        write!(f, "\"")
    }
}

#[stable(feature = "cstr_default", since = "1.10.0")]
impl Default for &CStr {
    fn default() -> Self {
        const SLICE: &[c_char] = &[0];
        // SAFETY: `SLICE` is indeed pointing to a valid nul-terminated string. <br>`SLICE` 确实指向一个有效的以 nul 结尾的字符串。<br>
        unsafe { CStr::from_ptr(SLICE.as_ptr()) }
    }
}

#[stable(feature = "frombyteswithnulerror_impls", since = "1.17.0")]
impl fmt::Display for FromBytesWithNulError {
    #[allow(deprecated, deprecated_in_future)]
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str(self.__description())?;
        if let FromBytesWithNulErrorKind::InteriorNul(pos) = self.kind {
            write!(f, " at byte pos {pos}")?;
        }
        Ok(())
    }
}

impl CStr {
    /// Wraps a raw C string with a safe C string wrapper. <br>用安全的 C 字符串包装器包装原始 C 字符串。<br>
    ///
    /// This function will wrap the provided `ptr` with a `CStr` wrapper, which allows inspection and interoperation of non-owned C strings. <br>此函数将使用 `CStr` 包装器包装提供的 `ptr`，从而允许检查和互操作非所有的 C 字符串。<br>
    /// The total size of the raw C string must be smaller than `isize::MAX` **bytes** in memory due to calling the `slice::from_raw_parts` function. <br>由于调用了 `slice::from_raw_parts` 函数，原始 C 字符串的总大小必须小于 `isize::MAX` 字节` 在内存中。<br>
    ///
    /// # Safety
    ///
    /// * The memory pointed to by `ptr` must contain a valid nul terminator at the end of the string. <br>`ptr` 指向的内存必须在字符串末尾包含一个有效的 nul 终止符。<br>
    ///
    /// * `ptr` must be [valid] for reads of bytes up to and including the null terminator. <br>`ptr` 必须是 [valid] 以读取最多包括空终止符的字节。<br>
    ///   This means in particular: <br>这尤其意味着：<br>
    ///
    ///     * The entire memory range of this `CStr` must be contained within a single allocated object! <br>这个 `CStr` 的整个内存范围必须包含在一个分配的对象中!<br>
    ///     * `ptr` must be non-null even for a zero-length cstr. <br>即使对于零长度的 cstr，`ptr` 也必须为非空值。<br>
    ///
    /// * The memory referenced by the returned `CStr` must not be mutated for the duration of lifetime `'a`. <br>返回的 `CStr` 引用的内存在生命周期 `'a` 的持续时间内不能发生可变的。<br>
    ///
    /// > **Note**: This operation is intended to be a 0-cost cast but it is <br>该操作原定为零成本投放，但<br>
    /// > currently implemented with an up-front calculation of the length of <br>目前已通过预先计算长度来实现<br>
    /// > the string. <br>字符串。<br> This is not guaranteed to always be the case. <br>不能保证总是这样。<br>
    ///
    /// # Caveat
    ///
    /// The lifetime for the returned slice is inferred from its usage. <br>从使用中可以推断出返回切片的生命周期。<br>
    /// To prevent accidental misuse, it's suggested to tie the lifetime to whichever source lifetime is safe in the context, such as by providing a helper function taking the lifetime of a host value for the slice, or by explicit annotation. <br>为防止意外滥用，建议将生命周期与生命周期中任何安全的来源联系起来，例如通过提供一个辅助函数，获取切片的宿主值的生命周期，或通过明确的注解法。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```ignore (extern-declaration)
    /// # fn main() {
    /// use std::ffi::CStr;
    /// use std::os::raw::c_char;
    ///
    /// extern "C" {
    ///     fn my_string() -> *const c_char;
    /// }
    ///
    /// unsafe {
    ///     let slice = CStr::from_ptr(my_string());
    ///     println!("string returned: {}", slice.to_str().unwrap());
    /// }
    /// # }
    /// ```
    ///
    /// [valid]: core::ptr#safety
    ///
    ///
    ///
    ///
    ///
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub unsafe fn from_ptr<'a>(ptr: *const c_char) -> &'a CStr {
        // SAFETY: The caller has provided a pointer that points to a valid C string with a NUL terminator of size less than `isize::MAX`, whose content remain valid and doesn't change for the lifetime of the returned `CStr`. <br>调用者提供了一个指针，该指针指向大小小于 `isize::MAX` 的 NUL 终止符的有效 C 字符串，其内容保持有效，并且对于返回的 `CStr` 的生命周期不会更改。<br>
        //
        //
        // Thus computing the length is fine (a NUL byte exists), the call to from_raw_parts is safe because we know the length is at most `isize::MAX`, meaning the call to `from_bytes_with_nul_unchecked` is correct. <br>这样计算长度就可以了 (存在一个 NUL 字节)，对 from_raw_parts 的调用是安全的，因为我们知道长度最多为 `isize::MAX`，这意味着对 `from_bytes_with_nul_unchecked` 的调用是正确的。<br>
        //
        // The cast from c_char to u8 is ok because a c_char is always one byte. <br>从 c_char 到 u8 的转换是可以的，因为 c_char 始终是一个字节。<br>
        //
        //
        //
        //
        unsafe {
            extern "C" {
                /// Provided by libc or compiler_builtins. <br>由 libc 或 compiler_builtins 提供。<br>
                fn strlen(s: *const c_char) -> usize;
            }
            let len = strlen(ptr);
            let ptr = ptr as *const u8;
            CStr::from_bytes_with_nul_unchecked(slice::from_raw_parts(ptr, len as usize + 1))
        }
    }

    /// Creates a C string wrapper from a byte slice. <br>从字节切片创建 C 字符串包装器。<br>
    ///
    /// This method will create a `CStr` from any byte slice that contains at least one nul byte. <br>此方法将从包含至少一个 nul 字节的任何字节切片创建 `CStr`。<br>
    /// The caller does not need to know or specify where the nul byte is located. <br>调用者不需要知道或指定 nul 字节的位置。<br>
    ///
    /// If the first byte is a nul character, this method will return an empty `CStr`. <br>如果第一个字节是一个空字符，这个方法将返回一个空的 `CStr`。<br>
    /// If multiple nul characters are present, the `CStr` will end at the first one. <br>如果存在多个 nul 字符，则 `CStr` 将在第一个字符处结束。<br>
    ///
    /// If the slice only has a single nul byte at the end, this method is equivalent to [`CStr::from_bytes_with_nul`]. <br>如果切片末尾只有一个 nul 字节，则此方法等效于 [`CStr::from_bytes_with_nul`]。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(cstr_from_bytes_until_nul)]
    ///
    /// use std::ffi::CStr;
    ///
    /// let mut buffer = [0u8; 16];
    /// unsafe {
    ///     // Here we might call an unsafe C function that writes a string into the buffer. <br>在这里，我们可能会调用一个不安全的 C 函数，它将字符串写入缓冲区。<br>
    /////
    ///     let buf_ptr = buffer.as_mut_ptr();
    ///     buf_ptr.write_bytes(b'A', 8);
    /// }
    /// // Attempt to extract a C nul-terminated string from the buffer. <br>尝试从缓冲区中提取 C 以 nul 结尾的字符串。<br>
    /// let c_str = CStr::from_bytes_until_nul(&buffer[..]).unwrap();
    /// assert_eq!(c_str.to_str().unwrap(), "AAAAAAAA");
    /// ```
    ///
    ///
    #[unstable(feature = "cstr_from_bytes_until_nul", issue = "95027")]
    pub fn from_bytes_until_nul(bytes: &[u8]) -> Result<&CStr, FromBytesUntilNulError> {
        let nul_pos = memchr::memchr(0, bytes);
        match nul_pos {
            Some(nul_pos) => {
                let subslice = &bytes[..nul_pos + 1];
                // SAFETY: We know there is a nul byte at nul_pos, so this slice (ending at the nul byte) is a well-formed C string. <br>我们知道在 nul_pos 处有一个 nul 字节，所以这个切片 (以 nul 字节结束) 是一个格式良好的 C 字符串。<br>
                //
                Ok(unsafe { CStr::from_bytes_with_nul_unchecked(subslice) })
            }
            None => Err(FromBytesUntilNulError(())),
        }
    }

    /// Creates a C string wrapper from a byte slice. <br>从字节切片创建 C 字符串包装器。<br>
    ///
    /// This function will cast the provided `bytes` to a `CStr` wrapper after ensuring that the byte slice is nul-terminated and does not contain any interior nul bytes. <br>在确保字节切片以 nul 终止并且不包含任何内部 nul 字节之后，此函数会将提供的 `bytes` 强制转换为 `CStr` 包装器。<br>
    ///
    ///
    /// If the nul byte may not be at the end, [`CStr::from_bytes_until_nul`] can be used instead. <br>如果 nul 字节可能不在末尾，则可以使用 [`CStr::from_bytes_until_nul`] 代替。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::ffi::CStr;
    ///
    /// let cstr = CStr::from_bytes_with_nul(b"hello\0");
    /// assert!(cstr.is_ok());
    /// ```
    ///
    /// Creating a `CStr` without a trailing nul terminator is an error: <br>创建没有尾随 nul 终止符的 `CStr` 是错误的：<br>
    ///
    /// ```
    /// use std::ffi::CStr;
    ///
    /// let cstr = CStr::from_bytes_with_nul(b"hello");
    /// assert!(cstr.is_err());
    /// ```
    ///
    /// Creating a `CStr` with an interior nul byte is an error: <br>使用内部 nul 字节创建 `CStr` 是错误的：<br>
    ///
    /// ```
    /// use std::ffi::CStr;
    ///
    /// let cstr = CStr::from_bytes_with_nul(b"he\0llo\0");
    /// assert!(cstr.is_err());
    /// ```
    ///
    ///
    #[stable(feature = "cstr_from_bytes", since = "1.10.0")]
    pub fn from_bytes_with_nul(bytes: &[u8]) -> Result<&Self, FromBytesWithNulError> {
        let nul_pos = memchr::memchr(0, bytes);
        match nul_pos {
            Some(nul_pos) if nul_pos + 1 == bytes.len() => {
                // SAFETY: We know there is only one nul byte, at the end of the byte slice. <br>我们知道在字节切片的末尾只有一个 nul 字节。<br>
                //
                Ok(unsafe { Self::from_bytes_with_nul_unchecked(bytes) })
            }
            Some(nul_pos) => Err(FromBytesWithNulError::interior_nul(nul_pos)),
            None => Err(FromBytesWithNulError::not_nul_terminated()),
        }
    }

    /// Unsafely creates a C string wrapper from a byte slice. <br>从字节切片不安全地创建 C 字符串包装器。<br>
    ///
    /// This function will cast the provided `bytes` to a `CStr` wrapper without performing any sanity checks. <br>此函数会将提供的 `bytes` 强制转换为 `CStr` 包装器，而无需执行任何健全性检查。<br>
    ///
    ///
    /// # Safety
    /// The provided slice **must** be nul-terminated and not contain any interior nul bytes. <br>所提供的切片必须以 nul 结尾，并且不包含任何内部 nul 字节。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::ffi::{CStr, CString};
    ///
    /// unsafe {
    ///     let cstring = CString::new("hello").expect("CString::new failed");
    ///     let cstr = CStr::from_bytes_with_nul_unchecked(cstring.to_bytes_with_nul());
    ///     assert_eq!(cstr, &*cstring);
    /// }
    /// ```
    ///
    #[inline]
    #[must_use]
    #[stable(feature = "cstr_from_bytes", since = "1.10.0")]
    #[rustc_const_stable(feature = "const_cstr_unchecked", since = "1.59.0")]
    pub const unsafe fn from_bytes_with_nul_unchecked(bytes: &[u8]) -> &CStr {
        // We're in a const fn, so this is the best we can do <br>我们在一个 const fn 中，所以这是我们能做的最好的<br>
        debug_assert!(!bytes.is_empty() && bytes[bytes.len() - 1] == 0);
        // SAFETY: Calling an inner function with the same prerequisites. <br>调用具有相同先决条件的内部函数。<br>
        unsafe { Self::_from_bytes_with_nul_unchecked(bytes) }
    }

    #[inline]
    const unsafe fn _from_bytes_with_nul_unchecked(bytes: &[u8]) -> &CStr {
        // SAFETY: Casting to CStr is safe because its internal representation is a [u8] too (safe only inside std). <br>强制转换为 CStr 是安全的，因为其内部表示形式也是 [u8] (仅在 std 内部安全)。<br>
        //
        // Dereferencing the obtained pointer is safe because it comes from a reference. <br>解引用获得的指针是安全的，因为它来自引用。<br>
        // Making a reference is then safe because its lifetime is bound by the lifetime of the given `bytes`. <br>这样，进行引用是安全的，因为其生命周期受给定 `bytes` 的生命周期的约束。<br>
        //
        unsafe { &*(bytes as *const [u8] as *const CStr) }
    }

    /// Returns the inner pointer to this C string. <br>返回此 C 字符串的内部指针。<br>
    ///
    /// The returned pointer will be valid for as long as `self` is, and points to a contiguous region of memory terminated with a 0 byte to represent the end of the string. <br>返回的指针在 `self` 内一直有效，并且指向以 0 字节结尾的连续区域，以表示字符串的结尾。<br>
    ///
    /// **WARNING**
    ///
    /// The returned pointer is read-only; <br>返回的指针是只读的；<br> writing to it (including passing it to C code that writes to it) causes undefined behavior. <br>对其进行写入 (包括将其传递给进行写入的 C 代码) 会导致未定义的行为。<br>
    ///
    /// It is your responsibility to make sure that the underlying memory is not freed too early. <br>您有责任确保底层内存不会过早释放。<br> For example, the following code will cause undefined behavior when `ptr` is used inside the `unsafe` block: <br>例如，当在 `unsafe` 块中使用 `ptr` 时，以下代码将导致未定义的行为：<br>
    ///
    /// ```no_run
    /// # #![allow(unused_must_use)] #![allow(temporary_cstring_as_ptr)]
    /// use std::ffi::CString;
    ///
    /// let ptr = CString::new("Hello").expect("CString::new failed").as_ptr();
    /// unsafe {
    ///     // `ptr` is dangling <br>`ptr` 是悬垂的<br>
    ///     *ptr;
    /// }
    /// ```
    ///
    /// This happens because the pointer returned by `as_ptr` does not carry any lifetime information and the `CString` is deallocated immediately after the `CString::new("Hello").expect("CString::new failed").as_ptr()` expression is evaluated. <br>发生这种情况是因为 `as_ptr` 返回的指针不携带任何生命周期信息，并且 `CString` 在 `CString::new("Hello").expect("CString::new failed").as_ptr()` 表达式计算后立即被释放。<br>
    ///
    /// To fix the problem, bind the `CString` to a local variable: <br>要解决此问题，请将 `CString` 绑定到本地变量：<br>
    ///
    /// ```no_run
    /// # #![allow(unused_must_use)]
    /// use std::ffi::CString;
    ///
    /// let hello = CString::new("Hello").expect("CString::new failed");
    /// let ptr = hello.as_ptr();
    /// unsafe {
    ///     // `ptr` is valid because `hello` is in scope <br>`ptr` 有效，因为 `hello` 在作用域内<br>
    ///     *ptr;
    /// }
    /// ```
    ///
    /// This way, the lifetime of the `CString` in `hello` encompasses the lifetime of `ptr` and the `unsafe` block. <br>这样，`hello` 中 `CString` 的生命周期就包含了 `ptr` 和 `unsafe` 区块的生命周期。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_stable(feature = "const_str_as_ptr", since = "1.32.0")]
    pub const fn as_ptr(&self) -> *const c_char {
        self.inner.as_ptr()
    }

    /// Converts this C string to a byte slice. <br>将此 C 字符串转换为字节片。<br>
    ///
    /// The returned slice will **not** contain the trailing nul terminator that this C string has. <br>返回的切片将不包含此 C 字符串具有的尾随 nul 终止符。<br>
    ///
    ///
    /// > **Note**: This method is currently implemented as a constant-time <br>该方法当前被实现为恒定时间<br>
    /// > cast, but it is planned to alter its definition in the future to <br>强制转换，但计划将其在 future 中的定义更改为<br>
    /// > perform the length calculation whenever this method is called. <br>每当调用此方法时，都要执行长度计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::ffi::CStr;
    ///
    /// let cstr = CStr::from_bytes_with_nul(b"foo\0").expect("CStr::from_bytes_with_nul failed");
    /// assert_eq!(cstr.to_bytes(), b"foo");
    /// ```
    #[inline]
    #[must_use = "this returns the result of the operation, \
                  without modifying the original"]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn to_bytes(&self) -> &[u8] {
        let bytes = self.to_bytes_with_nul();
        // SAFETY: to_bytes_with_nul returns slice with length at least 1 <br>to_bytes_with_nul 返回长度至少为 1 的字节<br>
        unsafe { bytes.get_unchecked(..bytes.len() - 1) }
    }

    /// Converts this C string to a byte slice containing the trailing 0 byte. <br>将此 C 字符串转换为包含尾随 0 字节的字节切片。<br>
    ///
    /// This function is the equivalent of [`CStr::to_bytes`] except that it will retain the trailing nul terminator instead of chopping it off. <br>此函数与 [`CStr::to_bytes`] 等效，除了保留尾随的 nul 终止符而不是将其截断之外。<br>
    ///
    ///
    /// > **Note**: This method is currently implemented as a 0-cost cast, but <br>目前，此方法已实现为零费用强制转换，但是<br>
    /// > it is planned to alter its definition in the future to perform the <br>计划在 future 中更改其定义以执行<br>
    /// > length calculation whenever this method is called. <br>每次调用此方法时的长度计算。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::ffi::CStr;
    ///
    /// let cstr = CStr::from_bytes_with_nul(b"foo\0").expect("CStr::from_bytes_with_nul failed");
    /// assert_eq!(cstr.to_bytes_with_nul(), b"foo\0");
    /// ```
    #[inline]
    #[must_use = "this returns the result of the operation, \
                  without modifying the original"]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn to_bytes_with_nul(&self) -> &[u8] {
        // SAFETY: Transmuting a slice of `c_char`s to a slice of `u8`s is safe on all supported targets. <br>在所有受支持的目标上，将 `c_char`s 切片转换为 `u8`s 切片是安全的。<br>
        //
        unsafe { &*(&self.inner as *const [c_char] as *const [u8]) }
    }

    /// Yields a <code>&[str]</code> slice if the `CStr` contains valid UTF-8. <br>如果 `CStr` 包含有效的 UTF-8，则产生 <code>&[str]</code> 切片。<br>
    ///
    /// If the contents of the `CStr` are valid UTF-8 data, this function will return the corresponding <code>&[str]</code> slice. <br>如果 `CStr` 的内容是有效的 UTF-8 数据，该函数将返回相应的 <code>&[str]</code> 切片。<br>
    ///
    /// Otherwise, it will return an error with details of where UTF-8 validation failed. <br>否则，它将返回错误，并详细说明 UTF-8 验证失败的位置。<br>
    ///
    /// [str]: prim@str "str"
    ///
    /// # Examples
    ///
    /// ```
    /// use std::ffi::CStr;
    ///
    /// let cstr = CStr::from_bytes_with_nul(b"foo\0").expect("CStr::from_bytes_with_nul failed");
    /// assert_eq!(cstr.to_str(), Ok("foo"));
    /// ```
    #[stable(feature = "cstr_to_str", since = "1.4.0")]
    pub fn to_str(&self) -> Result<&str, str::Utf8Error> {
        // N.B., when `CStr` is changed to perform the length check in `.to_bytes()` instead of in `from_ptr()`, it may be worth considering if this should be rewritten to do the UTF-8 check inline with the length calculation instead of doing it afterwards. <br>注意，将 `CStr` 更改为在 `.to_bytes()` 中而不是 `from_ptr()` 中执行长度检查时，可能值得考虑是否应该重写此代码，以便在进行长度计算时内联 UTF-8 检查，而不是随后进行。<br>
        //
        //
        //
        str::from_utf8(self.to_bytes())
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl PartialEq for CStr {
    fn eq(&self, other: &CStr) -> bool {
        self.to_bytes().eq(other.to_bytes())
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Eq for CStr {}
#[stable(feature = "rust1", since = "1.0.0")]
impl PartialOrd for CStr {
    fn partial_cmp(&self, other: &CStr) -> Option<Ordering> {
        self.to_bytes().partial_cmp(&other.to_bytes())
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Ord for CStr {
    fn cmp(&self, other: &CStr) -> Ordering {
        self.to_bytes().cmp(&other.to_bytes())
    }
}

#[stable(feature = "cstr_range_from", since = "1.47.0")]
impl ops::Index<ops::RangeFrom<usize>> for CStr {
    type Output = CStr;

    fn index(&self, index: ops::RangeFrom<usize>) -> &CStr {
        let bytes = self.to_bytes_with_nul();
        // we need to manually check the starting index to account for the null byte, since otherwise we could get an empty string that doesn't end in a null. <br>我们需要手动检查起始索引来说明空字节，因为否则我们会得到一个不以空结尾的空字符串。<br>
        //
        //
        if index.start < bytes.len() {
            // SAFETY: Non-empty tail of a valid `CStr` is still a valid `CStr`. <br>有效 `CStr` 的非空尾仍然是有效 `CStr`。<br>
            unsafe { CStr::from_bytes_with_nul_unchecked(&bytes[index.start..]) }
        } else {
            panic!(
                "index out of bounds: the len is {} but the index is {}",
                bytes.len(),
                index.start
            );
        }
    }
}

#[stable(feature = "cstring_asref", since = "1.7.0")]
impl AsRef<CStr> for CStr {
    #[inline]
    fn as_ref(&self) -> &CStr {
        self
    }
}
