//! Utilities for formatting and printing strings. <br>用于格式化和打印字符串的实用工具。<br>

#![stable(feature = "rust1", since = "1.0.0")]

use crate::cell::{Cell, Ref, RefCell, RefMut, SyncUnsafeCell, UnsafeCell};
use crate::char::EscapeDebugExtArgs;
use crate::marker::PhantomData;
use crate::mem;
use crate::num::fmt as numfmt;
use crate::ops::Deref;
use crate::result;
use crate::str;

mod builders;
#[cfg(not(no_fp_fmt_parse))]
mod float;
#[cfg(no_fp_fmt_parse)]
mod nofloat;
mod num;

#[stable(feature = "fmt_flags_align", since = "1.28.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "Alignment")]
/// Possible alignments returned by `Formatter::align` <br>`Formatter::align` 返回的可能的对齐方式<br>
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
pub enum Alignment {
    #[stable(feature = "fmt_flags_align", since = "1.28.0")]
    /// Indication that contents should be left-aligned. <br>指示内容应左对齐。<br>
    Left,
    #[stable(feature = "fmt_flags_align", since = "1.28.0")]
    /// Indication that contents should be right-aligned. <br>指示内容应右对齐。<br>
    Right,
    #[stable(feature = "fmt_flags_align", since = "1.28.0")]
    /// Indication that contents should be center-aligned. <br>指示内容应居中对齐。<br>
    Center,
}

#[stable(feature = "debug_builders", since = "1.2.0")]
pub use self::builders::{DebugList, DebugMap, DebugSet, DebugStruct, DebugTuple};

#[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
#[doc(hidden)]
pub mod rt {
    pub mod v1;
}

/// The type returned by formatter methods. <br>格式化程序方法返回的类型。<br>
///
/// # Examples
///
/// ```
/// use std::fmt;
///
/// #[derive(Debug)]
/// struct Triangle {
///     a: f32,
///     b: f32,
///     c: f32
/// }
///
/// impl fmt::Display for Triangle {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         write!(f, "({}, {}, {})", self.a, self.b, self.c)
///     }
/// }
///
/// let pythagorean_triple = Triangle { a: 3.0, b: 4.0, c: 5.0 };
///
/// assert_eq!(format!("{pythagorean_triple}"), "(3, 4, 5)");
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub type Result = result::Result<(), Error>;

/// The error type which is returned from formatting a message into a stream. <br>将消息格式化为流后返回的错误类型。<br>
///
/// This type does not support transmission of an error other than that an error occurred. <br>除了发生错误以外，此类型不支持错误的传输。<br>
/// Any extra information must be arranged to be transmitted through some other means. <br>必须安排任何其他信息以通过其他方式进行传输。<br>
///
/// An important thing to remember is that the type `fmt::Error` should not be confused with [`std::io::Error`] or [`std::error::Error`], which you may also have in scope. <br>要记住的重要一点是，不要将 `fmt::Error` 类型与 [`std::io::Error`] 或 [`std::error::Error`] 混淆，在作用域中也可以将它们与 [`std::io::Error`] 或 [`std::error::Error`] 混淆。<br>
///
///
/// [`std::io::Error`]: ../../std/io/struct.Error.html
/// [`std::error::Error`]: ../../std/error/trait.Error.html
///
/// # Examples
///
/// ```rust
/// use std::fmt::{self, write};
///
/// let mut output = String::new();
/// if let Err(fmt::Error) = write(&mut output, format_args!("Hello {}!", "world")) {
///     panic!("An error occurred");
/// }
/// ```
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Copy, Clone, Debug, Default, Eq, Hash, Ord, PartialEq, PartialOrd)]
pub struct Error;

/// A trait for writing or formatting into Unicode-accepting buffers or streams. <br>一个用于写入或格式化为 Unicode 接受的缓冲区或流的 trait。<br>
///
/// This trait only accepts UTF-8–encoded data and is not [flushable]. <br>一个只接受 UTF-8 编码的数据，而不是 [flushable] 的 trait。<br>
/// If you only want to accept Unicode and you don't need flushing, you should implement this trait; <br>如果您只想接受 Unicode 且不需要冲洗，则应实现此 trait; 否则，请执行此操作。<br>
/// otherwise you should implement [`std::io::Write`]. <br>否则，您应该实现 [`std::io::Write`]。<br>
///
/// [`std::io::Write`]: ../../std/io/trait.Write.html
/// [flushable]: ../../std/io/trait.Write.html#tymethod.flush
#[stable(feature = "rust1", since = "1.0.0")]
pub trait Write {
    /// Writes a string slice into this writer, returning whether the write succeeded. <br>将字符串切片写入此 writer，返回写入是否成功。<br>
    ///
    /// This method can only succeed if the entire string slice was successfully written, and this method will not return until all data has been written or an error occurs. <br>仅当成功写入整个字符串切片后，此方法才能成功，并且只有在写入所有数据或发生错误后，该方法才会返回。<br>
    ///
    ///
    /// # Errors
    ///
    /// This function will return an instance of [`Error`] on error. <br>错误时，此函数将返回 [`Error`] 的实例。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt::{Error, Write};
    ///
    /// fn writer<W: Write>(f: &mut W, s: &str) -> Result<(), Error> {
    ///     f.write_str(s)
    /// }
    ///
    /// let mut buf = String::new();
    /// writer(&mut buf, "hola").unwrap();
    /// assert_eq!(&buf, "hola");
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn write_str(&mut self, s: &str) -> Result;

    /// Writes a [`char`] into this writer, returning whether the write succeeded. <br>将 [`char`] 写入此 writer，返回写入是否成功。<br>
    ///
    /// A single [`char`] may be encoded as more than one byte. <br>单个 [`char`] 可以被编码为一个以上的字节。<br>
    /// This method can only succeed if the entire byte sequence was successfully written, and this method will not return until all data has been written or an error occurs. <br>仅当成功写入了整个字节序列后，此方法才能成功，并且直到所有数据都已写入或发生错误后，该方法才会返回。<br>
    ///
    ///
    /// # Errors
    ///
    /// This function will return an instance of [`Error`] on error. <br>错误时，此函数将返回 [`Error`] 的实例。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt::{Error, Write};
    ///
    /// fn writer<W: Write>(f: &mut W, c: char) -> Result<(), Error> {
    ///     f.write_char(c)
    /// }
    ///
    /// let mut buf = String::new();
    /// writer(&mut buf, 'a').unwrap();
    /// writer(&mut buf, 'b').unwrap();
    /// assert_eq!(&buf, "ab");
    /// ```
    ///
    #[stable(feature = "fmt_write_char", since = "1.1.0")]
    fn write_char(&mut self, c: char) -> Result {
        self.write_str(c.encode_utf8(&mut [0; 4]))
    }

    /// Glue for usage of the [`write!`] macro with implementors of this trait. <br>结合使用 [`write!`] 宏和 trait 的实现者。<br>
    ///
    /// This method should generally not be invoked manually, but rather through the [`write!`] macro itself. <br>通常不应手动调用此方法，而应通过 [`write!`] 宏本身来调用。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt::{Error, Write};
    ///
    /// fn writer<W: Write>(f: &mut W, s: &str) -> Result<(), Error> {
    ///     f.write_fmt(format_args!("{s}"))
    /// }
    ///
    /// let mut buf = String::new();
    /// writer(&mut buf, "world").unwrap();
    /// assert_eq!(&buf, "world");
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    fn write_fmt(mut self: &mut Self, args: Arguments<'_>) -> Result {
        write(&mut self, args)
    }
}

#[stable(feature = "fmt_write_blanket_impl", since = "1.4.0")]
impl<W: Write + ?Sized> Write for &mut W {
    fn write_str(&mut self, s: &str) -> Result {
        (**self).write_str(s)
    }

    fn write_char(&mut self, c: char) -> Result {
        (**self).write_char(c)
    }

    fn write_fmt(&mut self, args: Arguments<'_>) -> Result {
        (**self).write_fmt(args)
    }
}

/// Configuration for formatting. <br>格式化配置。<br>
///
/// A `Formatter` represents various options related to formatting. <br>`Formatter` 代表与格式相关的各种选项。<br>
/// Users do not construct `Formatter`s directly; <br>用户不直接构建 `Formatter`s。<br> a mutable reference to one is passed to the `fmt` method of all formatting traits, like [`Debug`] and [`Display`]. <br>将所有格式为 traits 的 `fmt` 方法 (例如 [`Debug`] 和 [`Display`]) 传递给 `fmt` 方法。<br>
///
///
/// To interact with a `Formatter`, you'll call various methods to change the various options related to formatting. <br>要与 `Formatter` 进行交互，您将调用各种方法来更改与格式相关的各种选项。<br>
/// For examples, please see the documentation of the methods defined on `Formatter` below. <br>有关示例，请参见下面在 `Formatter` 上定义的方法的文档。<br>
///
#[allow(missing_debug_implementations)]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Formatter<'a> {
    flags: u32,
    fill: char,
    align: rt::v1::Alignment,
    width: Option<usize>,
    precision: Option<usize>,

    buf: &'a mut (dyn Write + 'a),
}

impl<'a> Formatter<'a> {
    /// Creates a new formatter with default settings. <br>使用默认设置创建一个新的格式化程序。<br>
    ///
    /// This can be used as a micro-optimization in cases where a full `Arguments` structure (as created by `format_args!`) is not necessary; <br>在不需要完整的 `Arguments` 结构 (由 `format_args!` 创建) 的情况下，这可以用作微优化；<br> `Arguments` is a little more expensive to use in simple formatting scenarios. <br>在简单的格式化场景中使用 `Arguments` 的使用成本稍高一些。<br>
    ///
    ///
    /// Currently not intended for use outside of the standard library. <br>目前不打算在标准库之外使用。<br>
    ///
    #[unstable(feature = "fmt_internals", reason = "internal to standard library", issue = "none")]
    #[doc(hidden)]
    pub fn new(buf: &'a mut (dyn Write + 'a)) -> Formatter<'a> {
        Formatter {
            flags: 0,
            fill: ' ',
            align: rt::v1::Alignment::Unknown,
            width: None,
            precision: None,
            buf,
        }
    }
}

// NB.
// Argument is essentially an optimized partially applied formatting function, equivalent to `exists T.(&T, fn(&T, &mut Formatter<'_>) -> Result`. <br>参数本质上是优化的部分应用的格式化函数，等效于 `exists T.(&T, fn(&T, &mut Formatter<'_>) -> Result`。<br>

extern "C" {
    type Opaque;
}

/// This struct represents the generic "argument" which is taken by the Xprintf family of functions. <br>该结构体表示 Xprintf 系列函数采用的泛型 "argument"。<br> It contains a function to format the given value. <br>它包含一个用于格式化给定值的函数。<br>
/// At compile time it is ensured that the function and the value have the correct types, and then this struct is used to canonicalize arguments to one type. <br>在编译时，请确保函数和值具有正确的类型，然后使用此结构体将参数规范化为一种类型。<br>
///
///
#[derive(Copy, Clone)]
#[allow(missing_debug_implementations)]
#[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
#[doc(hidden)]
pub struct ArgumentV1<'a> {
    value: &'a Opaque,
    formatter: fn(&Opaque, &mut Formatter<'_>) -> Result,
}

/// This struct represents the unsafety of constructing an `Arguments`. <br>这个结构体代表了构建 `Arguments` 的不安全性。<br>
/// It exists, rather than an unsafe function, in order to simplify the expansion of `format_args!(..)` and reduce the scope of the `unsafe` block. <br>它的存在，而不是一个不安全的函数，是为了简化 `format_args!(..)` 的扩展，并缩小 `unsafe` 块的作用域。<br>
///
#[allow(missing_debug_implementations)]
#[doc(hidden)]
#[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
pub struct UnsafeArg {
    _private: (),
}

impl UnsafeArg {
    /// See documentation where `UnsafeArg` is required to know when it is safe to create and use `UnsafeArg`. <br>请参见 `UnsafeArg` 的文档，需要知道何时可以安全地创建和使用 `UnsafeArg`。<br>
    ///
    #[doc(hidden)]
    #[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
    #[inline(always)]
    pub unsafe fn new() -> Self {
        Self { _private: () }
    }
}

// This guarantees a single stable value for the function pointer associated with indices/counts in the formatting infrastructure. <br>这样可以确保格式基础结构中与 indices/counts 关联的函数指针具有单个稳定值。<br>
//
// Note that a function defined as such would not be correct as functions are always tagged unnamed_addr with the current lowering to LLVM IR, so their address is not considered important to LLVM and as such the as_usize cast could have been miscompiled. <br>请注意，这样定义的函数将是不正确的，因为该函数始终会被标记为 unnamed_addr，并且当前的状态会降低到 LLVM IR，因此它们的地址对于 LLVM 并不重要，因此 as_usize 强制转换可能会被错误编译。<br>
//
// In practice, we never call as_usize on non-usize containing data (as a matter of static generation of the formatting arguments), so this is merely an additional check. <br>在实践中，我们绝不会在未使用的包含数据上调用 as_usize (作为 formatting 参数的静态生成的问题)，因此，这仅仅是一项额外的检查。<br>
//
// We primarily want to ensure that the function pointer at `USIZE_MARKER` has an address corresponding *only* to functions that also take `&usize` as their first argument. <br>我们主要是要确保 `USIZE_MARKER` 处的函数指针具有对应于 *only* 的地址，该地址也将 `&usize` 作为它的第一个参数。<br>
// The read_volatile here ensures that we can safely ready out a usize from the passed reference and that this address does not point at a non-usize taking function. <br>这里的 read_volatile 确保我们可以安全地从传递的引用中准备出 usize，并且此地址不指向未使用的接受函数。<br>
//
//
//
//
//
//
//
#[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
static USIZE_MARKER: fn(&usize, &mut Formatter<'_>) -> Result = |ptr, _| {
    // SAFETY: ptr is a reference <br>ptr 是引用<br>
    let _v: usize = unsafe { crate::ptr::read_volatile(ptr) };
    loop {}
};

macro_rules! arg_new {
    ($f: ident, $t: ident) => {
        #[doc(hidden)]
        #[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
        #[inline]
        pub fn $f<'b, T: $t>(x: &'b T) -> ArgumentV1<'_> {
            Self::new(x, $t::fmt)
        }
    };
}

#[rustc_diagnostic_item = "ArgumentV1Methods"]
impl<'a> ArgumentV1<'a> {
    #[doc(hidden)]
    #[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
    #[inline]
    pub fn new<'b, T>(x: &'b T, f: fn(&T, &mut Formatter<'_>) -> Result) -> ArgumentV1<'b> {
        // SAFETY: `mem::transmute(x)` is safe because <br>`mem::transmute(x)` 是安全的，因为<br>
        //     1. `&'b T` keeps the lifetime it originated with `'b` (so as to not have an unbounded lifetime) <br>`&'b T` 保持它起源于 `'b` 的生命周期 (以便没有无限的生命周期)<br>
        //     2.
        //     `&'b T` and `&'b Opaque` have the same memory layout (when `T` is `Sized`, as it is here) `mem::transmute(f)` is safe since `fn(&T, &mut Formatter<'_>) -> Result` and `fn(&Opaque, &mut Formatter<'_>) -> Result` have the same ABI (as long as `T` is `Sized`) <br>`&'b T` 和 `&'b Opaque` 具有相同的内存布局 (当 `T` 是 `Sized` 时，就像这里一样) `mem::transmute(f)` 是安全的，因为 `fn(&T, &mut Formatter<'_>) -> Result` 和 `fn(&Opaque, &mut Formatter<'_>) -> Result` 具有相同的 ABI (只要 `T` 是 `Sized`)<br>
        //
        //
        //
        //
        unsafe { ArgumentV1 { formatter: mem::transmute(f), value: mem::transmute(x) } }
    }

    arg_new!(new_display, Display);
    arg_new!(new_debug, Debug);
    arg_new!(new_octal, Octal);
    arg_new!(new_lower_hex, LowerHex);
    arg_new!(new_upper_hex, UpperHex);
    arg_new!(new_pointer, Pointer);
    arg_new!(new_binary, Binary);
    arg_new!(new_lower_exp, LowerExp);
    arg_new!(new_upper_exp, UpperExp);

    #[doc(hidden)]
    #[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
    pub fn from_usize(x: &usize) -> ArgumentV1<'_> {
        ArgumentV1::new(x, USIZE_MARKER)
    }

    fn as_usize(&self) -> Option<usize> {
        // We are type punning a bit here: USIZE_MARKER only takes an &usize but formatter takes an &Opaque. <br>我们在这里有点双关: USIZE_MARKER 只接受一个 &usize，但格式化程序需要接受一个 &Opaque。<br>
        // Rust understandably doesn't think we should compare the function pointers if they don't have the same signature, so we cast to usizes to tell it that we just want to compare addresses. <br>可以理解的是，Rust 认为如果函数指针没有相同的签名，我们就不应该比较它们，所以我们转换成 usizes，告诉它我们只是想比较地址。<br>
        //
        //
        if self.formatter as usize == USIZE_MARKER as usize {
            // SAFETY: The `formatter` field is only set to USIZE_MARKER if the value is a usize, so this is safe <br>仅当值是 usize 时，才将 `formatter` 字段设置为 USIZE_MARKER，因此这是安全的<br>
            //
            Some(unsafe { *(self.value as *const _ as *const usize) })
        } else {
            None
        }
    }
}

// flags available in the v1 format of format_args <br>v1 格式 (format_args) 中可用的标志<br>
#[derive(Copy, Clone)]
enum FlagV1 {
    SignPlus,
    SignMinus,
    Alternate,
    SignAwareZeroPad,
    DebugLowerHex,
    DebugUpperHex,
}

impl<'a> Arguments<'a> {
    /// When using the format_args!() macro, this function is used to generate the Arguments structure. <br>当使用 format_args! () 宏时，此函数用于生成参数结构体。<br>
    ///
    #[doc(hidden)]
    #[inline]
    #[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
    #[rustc_const_unstable(feature = "const_fmt_arguments_new", issue = "none")]
    pub const fn new_v1(pieces: &'a [&'static str], args: &'a [ArgumentV1<'a>]) -> Arguments<'a> {
        if pieces.len() < args.len() || pieces.len() > args.len() + 1 {
            panic!("invalid args");
        }
        Arguments { pieces, fmt: None, args }
    }

    /// This function is used to specify nonstandard formatting parameters. <br>此函数用于指定非标准格式设置参数。<br>
    ///
    /// An `UnsafeArg` is required because the following invariants must be held in order for this function to be safe: <br>`UnsafeArg` 是必需的，因为必须保持以下不变量，此函数才安全：<br>
    ///
    /// 1. The `pieces` slice must be at least as long as `fmt`. <br>`pieces` 必须至少与 `fmt` 一样长。<br>
    /// 2. Every [`rt::v1::Argument::position`] value within `fmt` must be a valid index of `args`. <br>`fmt` 中的每个 [`rt::v1::Argument::position`] 值都必须是 `args` 的有效索引。<br>
    /// 3. Every [`Count::Param`] within `fmt` must contain a valid index of `args`. <br>`fmt` 中的每个 [`Count::Param`] 都必须包含一个有效的 `args` 索引。<br>
    ///
    ///
    #[doc(hidden)]
    #[inline]
    #[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
    #[rustc_const_unstable(feature = "const_fmt_arguments_new", issue = "none")]
    pub const fn new_v1_formatted(
        pieces: &'a [&'static str],
        args: &'a [ArgumentV1<'a>],
        fmt: &'a [rt::v1::Argument],
        _unsafe_arg: UnsafeArg,
    ) -> Arguments<'a> {
        Arguments { pieces, fmt: Some(fmt), args }
    }

    /// Estimates the length of the formatted text. <br>估计格式化文本的长度。<br>
    ///
    /// This is intended to be used for setting initial `String` capacity when using `format!`. <br>当使用 `format!` 时，这旨在用于设置 `String` 的初始容量。<br>
    /// Note: this is neither the lower nor upper bound. <br>这既不是下限也不是上限。<br>
    #[doc(hidden)]
    #[inline]
    #[unstable(feature = "fmt_internals", reason = "internal to format_args!", issue = "none")]
    pub fn estimated_capacity(&self) -> usize {
        let pieces_length: usize = self.pieces.iter().map(|x| x.len()).sum();

        if self.args.is_empty() {
            pieces_length
        } else if !self.pieces.is_empty() && self.pieces[0].is_empty() && pieces_length < 16 {
            // If the format string starts with an argument, don't preallocate anything, unless length of pieces is significant. <br>如果格式字符串以参数开头，则不要预分配任何内容，除非片段长度很大。<br>
            //
            //
            0
        } else {
            // There are some arguments, so any additional push will reallocate the string. <br>有一些参数，因此任何其他推送都会重新分配字符串。<br>
            //
            // To avoid that, we're "pre-doubling" the capacity here. <br>为避免这种情况，我们将此处的容量设置为 "pre-doubling"。<br>
            pieces_length.checked_mul(2).unwrap_or(0)
        }
    }
}

/// This structure represents a safely precompiled version of a format string and its arguments. <br>该结构体表示格式字符串及其参数的安全预编译版本。<br>
/// This cannot be generated at runtime because it cannot safely be done, so no constructors are given and the fields are private to prevent modification. <br>由于无法安全地完成此操作，因此无法在运行时生成该文件，因此未提供任何构造函数，并且该字段为私有字段以防止修改。<br>
///
///
/// The [`format_args!`] macro will safely create an instance of this structure. <br>[`format_args!`] 宏将安全地创建此结构体的实例。<br>
/// The macro validates the format string at compile-time so usage of the [`write()`] and [`format()`] functions can be safely performed. <br>宏在编译时验证格式字符串，因此可以安全地执行 [`write()`] 和 [`format()`] 函数的使用。<br>
///
/// You can use the `Arguments<'a>` that [`format_args!`] returns in `Debug` and `Display` contexts as seen below. <br>您可以在 `Debug` 和 `Display` 上下文中使用 [`format_args!`] 返回的 `Arguments<'a>`，如下所示。<br>
/// The example also shows that `Debug` and `Display` format to the same thing: the interpolated format string in `format_args!`. <br>该示例还显示 `Debug` 和 `Display` 的格式相同: `format_args!` 中的插值格式字符串。<br>
///
/// ```rust
/// let debug = format!("{:?}", format_args!("{} foo {:?}", 1, 2));
/// let display = format!("{}", format_args!("{} foo {:?}", 1, 2));
/// assert_eq!("1 foo 2", display);
/// assert_eq!(display, debug);
/// ```
///
/// [`format()`]: ../../std/fmt/fn.format.html
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "Arguments")]
#[derive(Copy, Clone)]
pub struct Arguments<'a> {
    // Format string pieces to print. <br>格式化要打印的字符串。<br>
    pieces: &'a [&'static str],

    // Placeholder specs, or `None` if all specs are default (as in "{}{}"). <br>占位符规范，如果所有规范均为默认规范，则为 `None` (与 "{}{}" 相同)。<br>
    fmt: Option<&'a [rt::v1::Argument]>,

    // Dynamic arguments for interpolation, to be interleaved with string pieces. <br>用于插值的动态参数，与字符串切片交织。<br>
    // (Every argument is preceded by a string piece.) <br>(每个参数前面都有一个字符串。)<br>
    args: &'a [ArgumentV1<'a>],
}

impl<'a> Arguments<'a> {
    /// Get the formatted string, if it has no arguments to be formatted. <br>获取格式化的字符串 (如果没有要格式化的参数)。<br>
    ///
    /// This can be used to avoid allocations in the most trivial case. <br>在最琐碎的情况下，这可以用来避免分配。<br>
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::fmt::Arguments;
    ///
    /// fn write_str(_: &str) { /* ... */ }
    ///
    /// fn write_fmt(args: &Arguments) {
    ///     if let Some(s) = args.as_str() {
    ///         write_str(s)
    ///     } else {
    ///         write_str(&args.to_string());
    ///     }
    /// }
    /// ```
    ///
    /// ```rust
    /// assert_eq!(format_args!("hello").as_str(), Some("hello"));
    /// assert_eq!(format_args!("").as_str(), Some(""));
    /// assert_eq!(format_args!("{}", 1).as_str(), None);
    /// ```
    #[stable(feature = "fmt_as_str", since = "1.52.0")]
    #[rustc_const_unstable(feature = "const_arguments_as_str", issue = "none")]
    #[must_use]
    #[inline]
    pub const fn as_str(&self) -> Option<&'static str> {
        match (self.pieces, self.args) {
            ([], []) => Some(""),
            ([s], []) => Some(s),
            _ => None,
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Debug for Arguments<'_> {
    fn fmt(&self, fmt: &mut Formatter<'_>) -> Result {
        Display::fmt(self, fmt)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Display for Arguments<'_> {
    fn fmt(&self, fmt: &mut Formatter<'_>) -> Result {
        write(fmt.buf, *self)
    }
}

/// `?` formatting. <br>`?` 格式。<br>
///
/// `Debug` should format the output in a programmer-facing, debugging context. <br>`Debug` 应该在面向程序员的调试上下文中格式化输出。<br>
///
/// Generally speaking, you should just `derive` a `Debug` implementation. <br>一般来说，您应该只将 `derive` 和 `Debug` 实现。<br>
///
/// When used with the alternate format specifier `#?`, the output is pretty-printed. <br>当与备用格式说明符 `#?` 一起使用时，输出将被漂亮地打印。<br>
///
/// For more information on formatters, see [the module-level documentation][module]. <br>有关格式化程序的更多信息，请参见 [模块级文档][module]。<br>
///
/// [module]: ../../std/fmt/index.html
///
/// This trait can be used with `#[derive]` if all fields implement `Debug`. <br>如果所有字段都实现 `Debug`，则此 trait 可以与 `#[derive]` 一起使用。<br> When `derive`d for structs, it will use the name of the `struct`, then `{`, then a comma-separated list of each field's name and `Debug` value, then `}`. <br>当为结构体 `derived' 时，它将使用 `struct` 的名称，然后是 `{`，然后是每个字段名称和 `Debug` 值的逗号分隔列表，然后是 `}`。<br>
/// For `enum`s, it will use the name of the variant and, if applicable, `(`, then the `Debug` values of the fields, then `)`. <br>对于 `enum`，它将使用变体的名称，如果适用，将使用 `(`，然后是字段的 `Debug` 值，然后是 `)`。<br>
///
/// # Stability
///
/// Derived `Debug` formats are not stable, and so may change with future Rust versions. <br>派生的 `Debug` 格式不稳定，因此 future Rust 版本可能会更改。<br> Additionally, `Debug` implementations of types provided by the standard library (`libstd`, `libcore`, `liballoc`, etc.) are not stable, and may also change with future Rust versions. <br>此外，标准库提供的类型 (`libstd`，`libcore`，`liballoc` 等) 的 `Debug` 实现不稳定，并且可能随 future Rust 版本而改变。<br>
///
/// # Examples
///
/// Deriving an implementation: <br>派生实现：<br>
///
/// ```
/// #[derive(Debug)]
/// struct Point {
///     x: i32,
///     y: i32,
/// }
///
/// let origin = Point { x: 0, y: 0 };
///
/// assert_eq!(format!("The origin is: {origin:?}"), "The origin is: Point { x: 0, y: 0 }");
/// ```
///
/// Manually implementing: <br>手动实现：<br>
///
/// ```
/// use std::fmt;
///
/// struct Point {
///     x: i32,
///     y: i32,
/// }
///
/// impl fmt::Debug for Point {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         f.debug_struct("Point")
///          .field("x", &self.x)
///          .field("y", &self.y)
///          .finish()
///     }
/// }
///
/// let origin = Point { x: 0, y: 0 };
///
/// assert_eq!(format!("The origin is: {origin:?}"), "The origin is: Point { x: 0, y: 0 }");
/// ```
///
/// There are a number of helper methods on the [`Formatter`] struct to help you with manual implementations, such as [`debug_struct`]. <br>[`Formatter`] 结构体上有许多辅助方法可以帮助您实现手动实现，例如 [`debug_struct`]。<br>
///
/// [`debug_struct`]: Formatter::debug_struct
///
/// Types that do not wish to use the standard suite of debug representations provided by the `Formatter` trait (`debug_struct`, `debug_tuple`, `debug_list`, `debug_set`, `debug_map`) can do something totally custom by manually writing an arbitrary representation to the `Formatter`. <br>不希望使用 `Formatter` trait 提供的标准调试表示套件 (`debug_struct`、`debug_tuple`、`debug_list`、`debug_set`、`debug_map`) 的类型可以通过手动向 `Formatter` 写入任意表示来完成完全自定义的操作。<br>
///
///
/// ```
/// # use std::fmt;
/// # struct Point {
/// #     x: i32,
/// #     y: i32,
/// # }
/// #
/// impl fmt::Debug for Point {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         write!(f, "Point [{} {}]", self.x, self.y)
///     }
/// }
/// ```
///
/// `Debug` implementations using either `derive` or the debug builder API on [`Formatter`] support pretty-printing using the alternate flag: `{:#?}`. <br>使用 `derive` 或 [`Formatter`] 上的调试构建器 API 的 `Debug` 实现支持使用备用标志 `{:#?}` 的漂亮打印。<br>
///
/// Pretty-printing with `#?`: <br>使用 `#?` 进行漂亮的打印：<br>
///
/// ```
/// #[derive(Debug)]
/// struct Point {
///     x: i32,
///     y: i32,
/// }
///
/// let origin = Point { x: 0, y: 0 };
///
/// assert_eq!(format!("The origin is: {origin:#?}"),
/// "The origin is: Point {
///     x: 0,
///     y: 0,
/// }");
/// ```
///
///
///
///
///
///
///
///
///
///

#[stable(feature = "rust1", since = "1.0.0")]
#[rustc_on_unimplemented(
    on(
        crate_local,
        label = "`{Self}` cannot be formatted using `{{:?}}`",
        note = "add `#[derive(Debug)]` to `{Self}` or manually `impl {Debug} for {Self}`"
    ),
    message = "`{Self}` doesn't implement `{Debug}`",
    label = "`{Self}` cannot be formatted using `{{:?}}` because it doesn't implement `{Debug}`"
)]
#[doc(alias = "{:?}")]
#[rustc_diagnostic_item = "Debug"]
#[rustc_trivial_field_reads]
pub trait Debug {
    /// Formats the value using the given formatter. <br>使用给定的格式化程序格式化该值。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Position {
    ///     longitude: f32,
    ///     latitude: f32,
    /// }
    ///
    /// impl fmt::Debug for Position {
    ///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
    ///         f.debug_tuple("")
    ///          .field(&self.longitude)
    ///          .field(&self.latitude)
    ///          .finish()
    ///     }
    /// }
    ///
    /// let position = Position { longitude: 1.987, latitude: 2.983 };
    /// assert_eq!(format!("{position:?}"), "(1.987, 2.983)");
    ///
    /// assert_eq!(format!("{position:#?}"), "(
    ///     1.987,
    ///     2.983,
    /// )");
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result;
}

// Separate module to reexport the macro `Debug` from prelude without the trait `Debug`. <br>单独的模块，用于从 prelude 重导出 `Debug` 宏，而无需 `Debug` trait。<br>
pub(crate) mod macros {
    /// Derive macro generating an impl of the trait `Debug`. <br>派生宏，生成 `Debug` trait 的 impl。<br>
    #[rustc_builtin_macro]
    #[stable(feature = "builtin_macro_prelude", since = "1.38.0")]
    #[allow_internal_unstable(core_intrinsics)]
    pub macro Debug($item:item) {
        /* compiler built-in */
    }
}
#[stable(feature = "builtin_macro_prelude", since = "1.38.0")]
#[doc(inline)]
pub use macros::Debug;

/// Format trait for an empty format, `{}`. <br>空格式的格式 trait，`{}`。<br>
///
/// `Display` is similar to [`Debug`], but `Display` is for user-facing output, and so cannot be derived. <br>`Display` 与 [`Debug`] 类似，但 `Display` 是面向用户的输出，因此无法导出。<br>
///
///
/// For more information on formatters, see [the module-level documentation][module]. <br>有关格式化程序的更多信息，请参见 [模块级文档][module]。<br>
///
/// [module]: ../../std/fmt/index.html
///
/// # Examples
///
/// Implementing `Display` on a type: <br>在类型上实现 `Display`：<br>
///
/// ```
/// use std::fmt;
///
/// struct Point {
///     x: i32,
///     y: i32,
/// }
///
/// impl fmt::Display for Point {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         write!(f, "({}, {})", self.x, self.y)
///     }
/// }
///
/// let origin = Point { x: 0, y: 0 };
///
/// assert_eq!(format!("The origin is: {origin}"), "The origin is: (0, 0)");
/// ```
#[rustc_on_unimplemented(
    on(
        any(_Self = "std::path::Path", _Self = "std::path::PathBuf"),
        label = "`{Self}` cannot be formatted with the default formatter; call `.display()` on it",
        note = "call `.display()` or `.to_string_lossy()` to safely print paths, \
                as they may contain non-Unicode data"
    ),
    message = "`{Self}` doesn't implement `{Display}`",
    label = "`{Self}` cannot be formatted with the default formatter",
    note = "in format strings you may be able to use `{{:?}}` (or {{:#?}} for pretty-print) instead"
)]
#[doc(alias = "{}")]
#[rustc_diagnostic_item = "Display"]
#[stable(feature = "rust1", since = "1.0.0")]
pub trait Display {
    /// Formats the value using the given formatter. <br>使用给定的格式化程序格式化该值。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Position {
    ///     longitude: f32,
    ///     latitude: f32,
    /// }
    ///
    /// impl fmt::Display for Position {
    ///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
    ///         write!(f, "({}, {})", self.longitude, self.latitude)
    ///     }
    /// }
    ///
    /// assert_eq!("(1.987, 2.983)",
    ///            format!("{}", Position { longitude: 1.987, latitude: 2.983, }));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result;
}

/// `o` formatting. <br>`o` 格式。<br>
///
/// The `Octal` trait should format its output as a number in base-8. <br>`Octal` trait 应该将其输出格式化为 base-8 中的数字。<br>
///
/// For primitive signed integers (`i8` to `i128`, and `isize`), negative values are formatted as the two’s complement representation. <br>对于原始有符号整数 (`i8` 至 `i128` 和 `isize`)，负值的格式设置为二进制补码表示形式。<br>
///
///
/// The alternate flag, `#`, adds a `0o` in front of the output. <br>备用标志 `#` 在输出前面添加 `0o`。<br>
///
/// For more information on formatters, see [the module-level documentation][module]. <br>有关格式化程序的更多信息，请参见 [模块级文档][module]。<br>
///
/// [module]: ../../std/fmt/index.html
///
/// # Examples
///
/// Basic usage with `i32`: <br>`i32` 的基本用法：<br>
///
/// ```
/// let x = 42; // 42 is '52' in octal <br>42 是八进制的 '52'<br>
///
/// assert_eq!(format!("{x:o}"), "52");
/// assert_eq!(format!("{x:#o}"), "0o52");
///
/// assert_eq!(format!("{:o}", -16), "37777777760");
/// ```
///
/// Implementing `Octal` on a type: <br>在类型上实现 `Octal`：<br>
///
/// ```
/// use std::fmt;
///
/// struct Length(i32);
///
/// impl fmt::Octal for Length {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         let val = self.0;
///
///         fmt::Octal::fmt(&val, f) // delegate to i32's implementation <br>委托给 i32 的实现<br>
///     }
/// }
///
/// let l = Length(9);
///
/// assert_eq!(format!("l as octal is: {l:o}"), "l as octal is: 11");
///
/// assert_eq!(format!("l as octal is: {l:#06o}"), "l as octal is: 0o0011");
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub trait Octal {
    /// Formats the value using the given formatter. <br>使用给定的格式化程序格式化该值。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result;
}

/// `b` formatting. <br>`b` 格式。<br>
///
/// The `Binary` trait should format its output as a number in binary. <br>`Binary` trait 应该将其输出格式化为二进制格式的数字。<br>
///
/// For primitive signed integers ([`i8`] to [`i128`], and [`isize`]), negative values are formatted as the two’s complement representation. <br>对于原始有符号整数 ([`i8`] 至 [`i128`] 和 [`isize`])，负值的格式设置为二进制补码表示形式。<br>
///
///
/// The alternate flag, `#`, adds a `0b` in front of the output. <br>备用标志 `#` 在输出前面添加 `0b`。<br>
///
/// For more information on formatters, see [the module-level documentation][module]. <br>有关格式化程序的更多信息，请参见 [模块级文档][module]。<br>
///
/// [module]: ../../std/fmt/index.html
///
/// # Examples
///
/// Basic usage with [`i32`]: <br>[`i32`] 的基本用法：<br>
///
/// ```
/// let x = 42; // 42 is '101010' in binary <br>42 是 '101010' 二进制<br>
///
/// assert_eq!(format!("{x:b}"), "101010");
/// assert_eq!(format!("{x:#b}"), "0b101010");
///
/// assert_eq!(format!("{:b}", -16), "11111111111111111111111111110000");
/// ```
///
/// Implementing `Binary` on a type: <br>在类型上实现 `Binary`：<br>
///
/// ```
/// use std::fmt;
///
/// struct Length(i32);
///
/// impl fmt::Binary for Length {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         let val = self.0;
///
///         fmt::Binary::fmt(&val, f) // delegate to i32's implementation <br>委托给 i32 的实现<br>
///     }
/// }
///
/// let l = Length(107);
///
/// assert_eq!(format!("l as binary is: {l:b}"), "l as binary is: 1101011");
///
/// assert_eq!(
///     format!("l as binary is: {l:#032b}"),
///     "l as binary is: 0b000000000000000000000001101011"
/// );
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub trait Binary {
    /// Formats the value using the given formatter. <br>使用给定的格式化程序格式化该值。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result;
}

/// `x` formatting. <br>`x` 格式。<br>
///
/// The `LowerHex` trait should format its output as a number in hexadecimal, with `a` through `f` in lower case. <br>`LowerHex` trait 应该将其输出格式设置为十六进制数字，其中 `a` 至 `f` 为小写形式。<br>
///
/// For primitive signed integers (`i8` to `i128`, and `isize`), negative values are formatted as the two’s complement representation. <br>对于原始有符号整数 (`i8` 至 `i128` 和 `isize`)，负值的格式设置为二进制补码表示形式。<br>
///
///
/// The alternate flag, `#`, adds a `0x` in front of the output. <br>备用标志 `#` 在输出前面添加 `0x`。<br>
///
/// For more information on formatters, see [the module-level documentation][module]. <br>有关格式化程序的更多信息，请参见 [模块级文档][module]。<br>
///
/// [module]: ../../std/fmt/index.html
///
/// # Examples
///
/// Basic usage with `i32`: <br>`i32` 的基本用法：<br>
///
/// ```
/// let x = 42; // 42 is '2a' in hex <br>42 是十六进制的 '2a'<br>
///
/// assert_eq!(format!("{x:x}"), "2a");
/// assert_eq!(format!("{x:#x}"), "0x2a");
///
/// assert_eq!(format!("{:x}", -16), "fffffff0");
/// ```
///
/// Implementing `LowerHex` on a type: <br>在类型上实现 `LowerHex`：<br>
///
/// ```
/// use std::fmt;
///
/// struct Length(i32);
///
/// impl fmt::LowerHex for Length {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         let val = self.0;
///
///         fmt::LowerHex::fmt(&val, f) // delegate to i32's implementation <br>委托给 i32 的实现<br>
///     }
/// }
///
/// let l = Length(9);
///
/// assert_eq!(format!("l as hex is: {l:x}"), "l as hex is: 9");
///
/// assert_eq!(format!("l as hex is: {l:#010x}"), "l as hex is: 0x00000009");
/// ```
///
#[stable(feature = "rust1", since = "1.0.0")]
pub trait LowerHex {
    /// Formats the value using the given formatter. <br>使用给定的格式化程序格式化该值。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result;
}

/// `X` formatting. <br>`X` 格式。<br>
///
/// The `UpperHex` trait should format its output as a number in hexadecimal, with `A` through `F` in upper case. <br>`UpperHex` trait 应该将其输出格式设置为十六进制数字，其中 `A` 至 `F` 为大写形式。<br>
///
/// For primitive signed integers (`i8` to `i128`, and `isize`), negative values are formatted as the two’s complement representation. <br>对于原始有符号整数 (`i8` 至 `i128` 和 `isize`)，负值的格式设置为二进制补码表示形式。<br>
///
///
/// The alternate flag, `#`, adds a `0x` in front of the output. <br>备用标志 `#` 在输出前面添加 `0x`。<br>
///
/// For more information on formatters, see [the module-level documentation][module]. <br>有关格式化程序的更多信息，请参见 [模块级文档][module]。<br>
///
/// [module]: ../../std/fmt/index.html
///
/// # Examples
///
/// Basic usage with `i32`: <br>`i32` 的基本用法：<br>
///
/// ```
/// let x = 42; // 42 is '2A' in hex <br>42 是十六进制的 '2A'<br>
///
/// assert_eq!(format!("{x:X}"), "2A");
/// assert_eq!(format!("{x:#X}"), "0x2A");
///
/// assert_eq!(format!("{:X}", -16), "FFFFFFF0");
/// ```
///
/// Implementing `UpperHex` on a type: <br>在类型上实现 `UpperHex`：<br>
///
/// ```
/// use std::fmt;
///
/// struct Length(i32);
///
/// impl fmt::UpperHex for Length {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         let val = self.0;
///
///         fmt::UpperHex::fmt(&val, f) // delegate to i32's implementation <br>委托给 i32 的实现<br>
///     }
/// }
///
/// let l = Length(i32::MAX);
///
/// assert_eq!(format!("l as hex is: {l:X}"), "l as hex is: 7FFFFFFF");
///
/// assert_eq!(format!("l as hex is: {l:#010X}"), "l as hex is: 0x7FFFFFFF");
/// ```
///
#[stable(feature = "rust1", since = "1.0.0")]
pub trait UpperHex {
    /// Formats the value using the given formatter. <br>使用给定的格式化程序格式化该值。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result;
}

/// `p` formatting. <br>`p` 格式。<br>
///
/// The `Pointer` trait should format its output as a memory location. <br>`Pointer` trait 应该将其输出格式化为存储位置。<br>
/// This is commonly presented as hexadecimal. <br>通常以十六进制表示。<br>
///
/// For more information on formatters, see [the module-level documentation][module]. <br>有关格式化程序的更多信息，请参见 [模块级文档][module]。<br>
///
/// [module]: ../../std/fmt/index.html
///
/// # Examples
///
/// Basic usage with `&i32`: <br>`&i32` 的基本用法：<br>
///
/// ```
/// let x = &42;
///
/// let address = format!("{x:p}"); // this produces something like '0x7f06092ac6d0' <br>这会产生类似 '0x7f06092ac6d0' 的东西<br>
/// ```
///
/// Implementing `Pointer` on a type: <br>在类型上实现 `Pointer`：<br>
///
/// ```
/// use std::fmt;
///
/// struct Length(i32);
///
/// impl fmt::Pointer for Length {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         // use `as` to convert to a `*const T`, which implements Pointer, which we can use <br>使用 `as` 转换为 `*const T`，该 `*const T` 实现了 Pointer，我们可以使用它<br>
///
///         let ptr = self as *const Self;
///         fmt::Pointer::fmt(&ptr, f)
///     }
/// }
///
/// let l = Length(42);
///
/// println!("l is in memory here: {l:p}");
///
/// let l_ptr = format!("{l:018p}");
/// assert_eq!(l_ptr.len(), 18);
/// assert_eq!(&l_ptr[..2], "0x");
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
#[rustc_diagnostic_item = "Pointer"]
pub trait Pointer {
    /// Formats the value using the given formatter. <br>使用给定的格式化程序格式化该值。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_diagnostic_item = "pointer_trait_fmt"]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result;
}

/// `e` formatting. <br>`e` 格式。<br>
///
/// The `LowerExp` trait should format its output in scientific notation with a lower-case `e`. <br>`LowerExp` trait 应该使用小写的 `e` 以科学计数法格式化其输出。<br>
///
/// For more information on formatters, see [the module-level documentation][module]. <br>有关格式化程序的更多信息，请参见 [模块级文档][module]。<br>
///
/// [module]: ../../std/fmt/index.html
///
/// # Examples
///
/// Basic usage with `f64`: <br>`f64` 的基本用法：<br>
///
/// ```
/// let x = 42.0; // 42.0 is '4.2e1' in scientific notation <br>是 '4.2e1' 的科学计数形式<br>
///
/// assert_eq!(format!("{x:e}"), "4.2e1");
/// ```
///
/// Implementing `LowerExp` on a type: <br>在类型上实现 `LowerExp`：<br>
///
/// ```
/// use std::fmt;
///
/// struct Length(i32);
///
/// impl fmt::LowerExp for Length {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         let val = f64::from(self.0);
///         fmt::LowerExp::fmt(&val, f) // delegate to f64's implementation <br>委托 f64 的实现<br>
///     }
/// }
///
/// let l = Length(100);
///
/// assert_eq!(
///     format!("l in scientific notation is: {l:e}"),
///     "l in scientific notation is: 1e2"
/// );
///
/// assert_eq!(
///     format!("l in scientific notation is: {l:05e}"),
///     "l in scientific notation is: 001e2"
/// );
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub trait LowerExp {
    /// Formats the value using the given formatter. <br>使用给定的格式化程序格式化该值。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result;
}

/// `E` formatting. <br>`E` 格式。<br>
///
/// The `UpperExp` trait should format its output in scientific notation with an upper-case `E`. <br>`UpperExp` trait 应该使用大写的 `E` 以科学计数法格式化其输出。<br>
///
/// For more information on formatters, see [the module-level documentation][module]. <br>有关格式化程序的更多信息，请参见 [模块级文档][module]。<br>
///
/// [module]: ../../std/fmt/index.html
///
/// # Examples
///
/// Basic usage with `f64`: <br>`f64` 的基本用法：<br>
///
/// ```
/// let x = 42.0; // 42.0 is '4.2E1' in scientific notation <br>是 '4.2E1' 的科学计数形式<br>
///
/// assert_eq!(format!("{x:E}"), "4.2E1");
/// ```
///
/// Implementing `UpperExp` on a type: <br>在类型上实现 `UpperExp`：<br>
///
/// ```
/// use std::fmt;
///
/// struct Length(i32);
///
/// impl fmt::UpperExp for Length {
///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
///         let val = f64::from(self.0);
///         fmt::UpperExp::fmt(&val, f) // delegate to f64's implementation <br>委托 f64 的实现<br>
///     }
/// }
///
/// let l = Length(100);
///
/// assert_eq!(
///     format!("l in scientific notation is: {l:E}"),
///     "l in scientific notation is: 1E2"
/// );
///
/// assert_eq!(
///     format!("l in scientific notation is: {l:05E}"),
///     "l in scientific notation is: 001E2"
/// );
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub trait UpperExp {
    /// Formats the value using the given formatter. <br>使用给定的格式化程序格式化该值。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result;
}

/// The `write` function takes an output stream, and an `Arguments` struct that can be precompiled with the `format_args!` macro. <br>`write` 函数接受一个输出流，以及一个可以与 `format_args!` 宏预编译的 `Arguments` 结构体。<br>
///
///
/// The arguments will be formatted according to the specified format string into the output stream provided. <br>参数将根据指定的格式字符串格式化为提供的输出流。<br>
///
/// # Examples
///
/// Basic usage: <br>基本用法：<br>
///
/// ```
/// use std::fmt;
///
/// let mut output = String::new();
/// fmt::write(&mut output, format_args!("Hello {}!", "world"))
///     .expect("Error occurred while trying to write in String");
/// assert_eq!(output, "Hello world!");
/// ```
///
/// Please note that using [`write!`] might be preferable. <br>请注意，使用 [`write!`] 可能更可取。<br> Example:
///
/// ```
/// use std::fmt::Write;
///
/// let mut output = String::new();
/// write!(&mut output, "Hello {}!", "world")
///     .expect("Error occurred while trying to write in String");
/// assert_eq!(output, "Hello world!");
/// ```
///
/// [`write!`]: crate::write!
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn write(output: &mut dyn Write, args: Arguments<'_>) -> Result {
    let mut formatter = Formatter::new(output);
    let mut idx = 0;

    match args.fmt {
        None => {
            // We can use default formatting parameters for all arguments. <br>我们可以对所有参数使用默认格式设置参数。<br>
            for (i, arg) in args.args.iter().enumerate() {
                // SAFETY: args.args and args.pieces come from the same Arguments, which guarantees the indexes are always within bounds. <br>args.args 和 args.pieces 来自同一个参数，保证索引总是在限定范围内。<br>
                //
                let piece = unsafe { args.pieces.get_unchecked(i) };
                if !piece.is_empty() {
                    formatter.buf.write_str(*piece)?;
                }
                (arg.formatter)(arg.value, &mut formatter)?;
                idx += 1;
            }
        }
        Some(fmt) => {
            // Every spec has a corresponding argument that is preceded by a string piece. <br>每个规范都有一个对应的参数，该参数后有一个字符串。<br>
            //
            for (i, arg) in fmt.iter().enumerate() {
                // SAFETY: fmt and args.pieces come from the same Arguments, which guarantees the indexes are always within bounds. <br>fmt 和 args.pieces 来自同一个 参数，保证索引总是在限定范围内。<br>
                //
                let piece = unsafe { args.pieces.get_unchecked(i) };
                if !piece.is_empty() {
                    formatter.buf.write_str(*piece)?;
                }
                // SAFETY: arg and args.args come from the same Arguments, which guarantees the indexes are always within bounds. <br>arg 和 args.args 来自相同的参数，从而确保索引始终在范围之内。<br>
                //
                unsafe { run(&mut formatter, arg, args.args) }?;
                idx += 1;
            }
        }
    }

    // There can be only one trailing string piece left. <br>只能剩下一个尾随的字符串切片。<br>
    if let Some(piece) = args.pieces.get(idx) {
        formatter.buf.write_str(*piece)?;
    }

    Ok(())
}

unsafe fn run(fmt: &mut Formatter<'_>, arg: &rt::v1::Argument, args: &[ArgumentV1<'_>]) -> Result {
    fmt.fill = arg.format.fill;
    fmt.align = arg.format.align;
    fmt.flags = arg.format.flags;
    // SAFETY: arg and args come from the same Arguments, which guarantees the indexes are always within bounds. <br>arg 和 args 来自相同的参数，从而确保索引始终在范围之内。<br>
    //
    unsafe {
        fmt.width = getcount(args, &arg.format.width);
        fmt.precision = getcount(args, &arg.format.precision);
    }

    // Extract the correct argument <br>提取正确的参数<br>
    debug_assert!(arg.position < args.len());
    // SAFETY: arg and args come from the same Arguments, which guarantees its index is always within bounds. <br>arg 和 args 来自相同的参数，从而确保其索引始终在范围之内。<br>
    //
    let value = unsafe { args.get_unchecked(arg.position) };

    // Then actually do some printing <br>然后实际做一些打印<br>
    (value.formatter)(value.value, fmt)
}

unsafe fn getcount(args: &[ArgumentV1<'_>], cnt: &rt::v1::Count) -> Option<usize> {
    match *cnt {
        rt::v1::Count::Is(n) => Some(n),
        rt::v1::Count::Implied => None,
        rt::v1::Count::Param(i) => {
            debug_assert!(i < args.len());
            // SAFETY: cnt and args come from the same Arguments, which guarantees this index is always within bounds. <br>cnt 和 args 来自相同的参数，这确保此索引始终在范围之内。<br>
            //
            unsafe { args.get_unchecked(i).as_usize() }
        }
    }
}

/// Padding after the end of something. <br>结束后填充。<br> Returned by `Formatter::padding`. <br>由 `Formatter::padding` 返回。<br>
#[must_use = "don't forget to write the post padding"]
pub(crate) struct PostPadding {
    fill: char,
    padding: usize,
}

impl PostPadding {
    fn new(fill: char, padding: usize) -> PostPadding {
        PostPadding { fill, padding }
    }

    /// Write this post padding. <br>写这篇文章补充。<br>
    pub(crate) fn write(self, f: &mut Formatter<'_>) -> Result {
        for _ in 0..self.padding {
            f.buf.write_char(self.fill)?;
        }
        Ok(())
    }
}

impl<'a> Formatter<'a> {
    fn wrap_buf<'b, 'c, F>(&'b mut self, wrap: F) -> Formatter<'c>
    where
        'b: 'c,
        F: FnOnce(&'b mut (dyn Write + 'b)) -> &'c mut (dyn Write + 'c),
    {
        Formatter {
            // We want to change this <br>我们要改变这个<br>
            buf: wrap(self.buf),

            // And preserve these <br>并保留这些<br>
            flags: self.flags,
            fill: self.fill,
            align: self.align,
            width: self.width,
            precision: self.precision,
        }
    }

    // Helper methods used for padding and processing formatting arguments that all formatting traits can use. <br>用于填充和处理格式化参数的辅助方法，所有格式化 traits 都可以使用。<br>
    //

    /// Performs the correct padding for an integer which has already been emitted into a str. <br>对已经发出到 str 中的整数执行正确的填充。<br>
    /// The str should *not* contain the sign for the integer, that will be added by this method. <br>str *不应* 包含整数的符号，该符号将通过此方法添加。<br>
    ///
    /// # Arguments
    ///
    /// * is_nonnegative - whether the original integer was either positive or zero. <br>is_nonnegative - 原始整数是正数还是零。<br>
    /// * prefix - if the '#' character (Alternate) is provided, this is the prefix to put in front of the number. <br>prefix - 如果提供了 '#' 字符 (Alternate)，则这是放在数字前面的前缀。<br>
    ///
    /// * buf - the byte array that the number has been formatted into <br>buf - 数字已格式化为的字节数组<br>
    ///
    /// This function will correctly account for the flags provided as well as the minimum width. <br>此函数将正确说明提供的标志以及最小宽度。<br>
    /// It will not take precision into account. <br>它不会考虑精度。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo { nb: i32 }
    ///
    /// impl Foo {
    ///     fn new(nb: i32) -> Foo {
    ///         Foo {
    ///             nb,
    ///         }
    ///     }
    /// }
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         // We need to remove "-" from the number output. <br>我们需要从数字输出中删除 "-"。<br>
    ///         let tmp = self.nb.abs().to_string();
    ///
    ///         formatter.pad_integral(self.nb >= 0, "Foo ", &tmp)
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{}", Foo::new(2)), "2");
    /// assert_eq!(&format!("{}", Foo::new(-1)), "-1");
    /// assert_eq!(&format!("{}", Foo::new(0)), "0");
    /// assert_eq!(&format!("{:#}", Foo::new(-1)), "-Foo 1");
    /// assert_eq!(&format!("{:0>#8}", Foo::new(-1)), "00-Foo 1");
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn pad_integral(&mut self, is_nonnegative: bool, prefix: &str, buf: &str) -> Result {
        let mut width = buf.len();

        let mut sign = None;
        if !is_nonnegative {
            sign = Some('-');
            width += 1;
        } else if self.sign_plus() {
            sign = Some('+');
            width += 1;
        }

        let prefix = if self.alternate() {
            width += prefix.chars().count();
            Some(prefix)
        } else {
            None
        };

        // Writes the sign if it exists, and then the prefix if it was requested <br>写入符号 (如果存在)，然后写入前缀 (如果已请求)<br>
        #[inline(never)]
        fn write_prefix(f: &mut Formatter<'_>, sign: Option<char>, prefix: Option<&str>) -> Result {
            if let Some(c) = sign {
                f.buf.write_char(c)?;
            }
            if let Some(prefix) = prefix { f.buf.write_str(prefix) } else { Ok(()) }
        }

        // The `width` field is more of a `min-width` parameter at this point. <br>此时，`width` 字段更多是 `min-width` 参数。<br>
        match self.width {
            // If there's no minimum length requirements then we can just write the bytes. <br>如果没有最小长度要求，那么我们可以写字节。<br>
            //
            None => {
                write_prefix(self, sign, prefix)?;
                self.buf.write_str(buf)
            }
            // Check if we're over the minimum width, if so then we can also just write the bytes. <br>检查是否超过最小宽度，如果是，那么我们也可以只写字节。<br>
            //
            Some(min) if width >= min => {
                write_prefix(self, sign, prefix)?;
                self.buf.write_str(buf)
            }
            // The sign and prefix goes before the padding if the fill character is zero <br>如果填充字符为零，则符号和前缀位于填充之前<br>
            //
            Some(min) if self.sign_aware_zero_pad() => {
                let old_fill = crate::mem::replace(&mut self.fill, '0');
                let old_align = crate::mem::replace(&mut self.align, rt::v1::Alignment::Right);
                write_prefix(self, sign, prefix)?;
                let post_padding = self.padding(min - width, rt::v1::Alignment::Right)?;
                self.buf.write_str(buf)?;
                post_padding.write(self)?;
                self.fill = old_fill;
                self.align = old_align;
                Ok(())
            }
            // Otherwise, the sign and prefix goes after the padding <br>否则，符号和前缀在填充之后<br>
            Some(min) => {
                let post_padding = self.padding(min - width, rt::v1::Alignment::Right)?;
                write_prefix(self, sign, prefix)?;
                self.buf.write_str(buf)?;
                post_padding.write(self)
            }
        }
    }

    /// This function takes a string slice and emits it to the internal buffer after applying the relevant formatting flags specified. <br>应用指定的相关格式设置标志后，此函数将获取一个字符串切片并将其发送到内部缓冲区。<br>
    /// The flags recognized for generic strings are: <br>泛型字符串可识别的标志为：<br>
    ///
    /// * width - the minimum width of what to emit <br>width - 发射的最小宽度<br>
    /// * fill/align - what to emit and where to emit it if the string provided needs to be padded <br>fill/align - 如果需要填充提供的字符串，要发出什么以及在哪里发出<br>
    /// * precision - the maximum length to emit, the string is truncated if it is longer than this length <br>precision - 发出的最大长度，如果字符串长于该长度，则字符串将被截断<br>
    ///
    /// Notably this function ignores the `flag` parameters. <br>值得注意的是，此函数将忽略 `flag` 参数。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo;
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         formatter.pad("Foo")
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{Foo:<4}"), "Foo ");
    /// assert_eq!(&format!("{Foo:0>4}"), "0Foo");
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn pad(&mut self, s: &str) -> Result {
        // Make sure there's a fast path up front <br>确保前面有一条快速路<br>
        if self.width.is_none() && self.precision.is_none() {
            return self.buf.write_str(s);
        }
        // The `precision` field can be interpreted as a `max-width` for the string being formatted. <br>对于要格式化的字符串，`precision` 字段可以解释为 `max-width`。<br>
        //
        let s = if let Some(max) = self.precision {
            // If our string is longer that the precision, then we must have truncation. <br>如果我们的字符串比精度更长，那么我们必须将其截断。<br>
            // However other flags like `fill`, `width` and `align` must act as always. <br>但是，其他标志 (例如 `fill`，`width` 和 `align`) 必须照常运行。<br>
            //
            if let Some((i, _)) = s.char_indices().nth(max) {
                // LLVM here can't prove that `..i` won't panic `&s[..i]`, but we know that it can't panic. <br>此处的 LLVM 不能证明 `..i` 不会 panic `&s[..i]`，但是我们知道它不能 panic。<br>
                // Use `get` + `unwrap_or` to avoid `unsafe` and otherwise don't emit any panic-related code here. <br>使用 `get` + `unwrap_or` 避免使用 `unsafe`，否则请不要在此处发出任何与 panic 相关的代码。<br>
                //
                //
                s.get(..i).unwrap_or(s)
            } else {
                &s
            }
        } else {
            &s
        };
        // The `width` field is more of a `min-width` parameter at this point. <br>此时，`width` 字段更多是 `min-width` 参数。<br>
        match self.width {
            // If we're under the maximum length, and there's no minimum length requirements, then we can just emit the string <br>如果我们在最大长度以下，并且没有最小长度要求，那么我们就可以发出字符串<br>
            //
            None => self.buf.write_str(s),
            Some(width) => {
                let chars_count = s.chars().count();
                // If we're under the maximum width, check if we're over the minimum width, if so it's as easy as just emitting the string. <br>如果我们在最大宽度之下，请检查我们是否在最小宽度之上，如果是，那么就和发出字符串一样容易。<br>
                //
                if chars_count >= width {
                    self.buf.write_str(s)
                }
                // If we're under both the maximum and the minimum width, then fill up the minimum width with the specified string + some alignment. <br>如果我们同时处于最大和最小宽度之下，则使用指定的字符串 + 某些对齐方式来填充最小宽度。<br>
                //
                else {
                    let align = rt::v1::Alignment::Left;
                    let post_padding = self.padding(width - chars_count, align)?;
                    self.buf.write_str(s)?;
                    post_padding.write(self)
                }
            }
        }
    }

    /// Write the pre-padding and return the unwritten post-padding. <br>编写前填充并返回未写的后填充。<br>
    /// Callers are responsible for ensuring post-padding is written after the thing that is being padded. <br>调用者有责任确保在填充内容之后编写填充后的内容。<br>
    ///
    pub(crate) fn padding(
        &mut self,
        padding: usize,
        default: rt::v1::Alignment,
    ) -> result::Result<PostPadding, Error> {
        let align = match self.align {
            rt::v1::Alignment::Unknown => default,
            _ => self.align,
        };

        let (pre_pad, post_pad) = match align {
            rt::v1::Alignment::Left => (0, padding),
            rt::v1::Alignment::Right | rt::v1::Alignment::Unknown => (padding, 0),
            rt::v1::Alignment::Center => (padding / 2, (padding + 1) / 2),
        };

        for _ in 0..pre_pad {
            self.buf.write_char(self.fill)?;
        }

        Ok(PostPadding::new(self.fill, post_pad))
    }

    /// Takes the formatted parts and applies the padding. <br>取出格式化的部分并应用填充。<br>
    /// Assumes that the caller already has rendered the parts with required precision, so that `self.precision` can be ignored. <br>假定调用者已经以所需的精度渲染了零件，因此可以忽略 `self.precision`。<br>
    ///
    fn pad_formatted_parts(&mut self, formatted: &numfmt::Formatted<'_>) -> Result {
        if let Some(mut width) = self.width {
            // for the sign-aware zero padding, we render the sign first and behave as if we had no sign from the beginning. <br>对于可识别符号的零填充，我们首先渲染符号，然后从一开始就表现为没有符号。<br>
            //
            let mut formatted = formatted.clone();
            let old_fill = self.fill;
            let old_align = self.align;
            let mut align = old_align;
            if self.sign_aware_zero_pad() {
                // a sign always goes first <br>符号总是最先<br>
                let sign = formatted.sign;
                self.buf.write_str(sign)?;

                // remove the sign from the formatted parts <br>从格式化部分中删除符号<br>
                formatted.sign = "";
                width = width.saturating_sub(sign.len());
                align = rt::v1::Alignment::Right;
                self.fill = '0';
                self.align = rt::v1::Alignment::Right;
            }

            // remaining parts go through the ordinary padding process. <br>其余部分则经过普通的填充过程。<br>
            let len = formatted.len();
            let ret = if width <= len {
                // no padding <br>没有填充<br>
                self.write_formatted_parts(&formatted)
            } else {
                let post_padding = self.padding(width - len, align)?;
                self.write_formatted_parts(&formatted)?;
                post_padding.write(self)
            };
            self.fill = old_fill;
            self.align = old_align;
            ret
        } else {
            // this is the common case and we take a shortcut <br>这是常见的情况，我们采取捷径<br>
            self.write_formatted_parts(formatted)
        }
    }

    fn write_formatted_parts(&mut self, formatted: &numfmt::Formatted<'_>) -> Result {
        fn write_bytes(buf: &mut dyn Write, s: &[u8]) -> Result {
            // SAFETY: This is used for `numfmt::Part::Num` and `numfmt::Part::Copy`. <br>这用于 `numfmt::Part::Num` 和 `numfmt::Part::Copy`。<br>
            // It's safe to use for `numfmt::Part::Num` since every char `c` is between `b'0'` and `b'9'`, which means `s` is valid UTF-8. <br>用于 `numfmt::Part::Num` 是安全的，因为每个字符 `c` 都在 `b'0'` 和 `b'9'` 之间，这意味着 `s` 是有效的 UTF-8。<br>
            // It's also probably safe in practice to use for `numfmt::Part::Copy(buf)` since `buf` should be plain ASCII, but it's possible for someone to pass in a bad value for `buf` into `numfmt::to_shortest_str` since it is a public function. <br>在实践中使用 `numfmt::Part::Copy(buf)` 也可能是安全的，因为 `buf` 应该是纯 ASCII，但有人可能会将 `buf` 的错误值传递给 `numfmt::to_shortest_str`，因为它是一个公共函数。<br>
            //
            // FIXME: Determine whether this could result in UB. <br>确定这是否会导致 UB。<br>
            //
            //
            //
            buf.write_str(unsafe { str::from_utf8_unchecked(s) })
        }

        if !formatted.sign.is_empty() {
            self.buf.write_str(formatted.sign)?;
        }
        for part in formatted.parts {
            match *part {
                numfmt::Part::Zero(mut nzeroes) => {
                    const ZEROES: &str = // 64 zeroes <br>64 个零<br>
                        "0000000000000000000000000000000000000000000000000000000000000000";
                    while nzeroes > ZEROES.len() {
                        self.buf.write_str(ZEROES)?;
                        nzeroes -= ZEROES.len();
                    }
                    if nzeroes > 0 {
                        self.buf.write_str(&ZEROES[..nzeroes])?;
                    }
                }
                numfmt::Part::Num(mut v) => {
                    let mut s = [0; 5];
                    let len = part.len();
                    for c in s[..len].iter_mut().rev() {
                        *c = b'0' + (v % 10) as u8;
                        v /= 10;
                    }
                    write_bytes(self.buf, &s[..len])?;
                }
                numfmt::Part::Copy(buf) => {
                    write_bytes(self.buf, buf)?;
                }
            }
        }
        Ok(())
    }

    /// Writes some data to the underlying buffer contained within this formatter. <br>将一些数据写入此格式化程序中包含的底层缓冲区。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo;
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         formatter.write_str("Foo")
    ///         // This is equivalent to: <br>这相当于：<br>
    ///         // write!(formatter, "Foo")
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{Foo}"), "Foo");
    /// assert_eq!(&format!("{Foo:0>8}"), "Foo");
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn write_str(&mut self, data: &str) -> Result {
        self.buf.write_str(data)
    }

    /// Writes some formatted information into this instance. <br>将一些格式化的信息写入此实例。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo(i32);
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         formatter.write_fmt(format_args!("Foo {}", self.0))
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{}", Foo(-1)), "Foo -1");
    /// assert_eq!(&format!("{:0>8}", Foo(2)), "Foo 2");
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn write_fmt(&mut self, fmt: Arguments<'_>) -> Result {
        write(self.buf, fmt)
    }

    /// Flags for formatting <br>格式化标志<br>
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[deprecated(
        since = "1.24.0",
        note = "use the `sign_plus`, `sign_minus`, `alternate`, \
                or `sign_aware_zero_pad` methods instead"
    )]
    pub fn flags(&self) -> u32 {
        self.flags
    }

    /// Character used as 'fill' whenever there is alignment. <br>对齐时用作 'fill' 的字符。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo;
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         let c = formatter.fill();
    ///         if let Some(width) = formatter.width() {
    ///             for _ in 0..width {
    ///                 write!(formatter, "{c}")?;
    ///             }
    ///             Ok(())
    ///         } else {
    ///             write!(formatter, "{c}")
    ///         }
    ///     }
    /// }
    ///
    /// // We set alignment to the right with ">". <br>我们使用 ">" 在右边设置对齐方式。<br>
    /// assert_eq!(&format!("{Foo:G>3}"), "GGG");
    /// assert_eq!(&format!("{Foo:t>6}"), "tttttt");
    /// ```
    #[must_use]
    #[stable(feature = "fmt_flags", since = "1.5.0")]
    pub fn fill(&self) -> char {
        self.fill
    }

    /// Flag indicating what form of alignment was requested. <br>指示请求对齐方式的标志。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// extern crate core;
    ///
    /// use std::fmt::{self, Alignment};
    ///
    /// struct Foo;
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         let s = if let Some(s) = formatter.align() {
    ///             match s {
    ///                 Alignment::Left    => "left",
    ///                 Alignment::Right   => "right",
    ///                 Alignment::Center  => "center",
    ///             }
    ///         } else {
    ///             "into the void"
    ///         };
    ///         write!(formatter, "{s}")
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{Foo:<}"), "left");
    /// assert_eq!(&format!("{Foo:>}"), "right");
    /// assert_eq!(&format!("{Foo:^}"), "center");
    /// assert_eq!(&format!("{Foo}"), "into the void");
    /// ```
    #[must_use]
    #[stable(feature = "fmt_flags_align", since = "1.28.0")]
    pub fn align(&self) -> Option<Alignment> {
        match self.align {
            rt::v1::Alignment::Left => Some(Alignment::Left),
            rt::v1::Alignment::Right => Some(Alignment::Right),
            rt::v1::Alignment::Center => Some(Alignment::Center),
            rt::v1::Alignment::Unknown => None,
        }
    }

    /// Optionally specified integer width that the output should be. <br>(可选) 指定输出应为的整数宽度。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo(i32);
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         if let Some(width) = formatter.width() {
    ///             // If we received a width, we use it <br>如果我们收到一个宽度，我们就用它<br>
    ///             write!(formatter, "{:width$}", &format!("Foo({})", self.0), width = width)
    ///         } else {
    ///             // Otherwise we do nothing special <br>否则我们没什么特别的<br>
    ///             write!(formatter, "Foo({})", self.0)
    ///         }
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{:10}", Foo(23)), "Foo(23)   ");
    /// assert_eq!(&format!("{}", Foo(23)), "Foo(23)");
    /// ```
    #[must_use]
    #[stable(feature = "fmt_flags", since = "1.5.0")]
    pub fn width(&self) -> Option<usize> {
        self.width
    }

    /// Optionally specified precision for numeric types. <br>可选地为数字类型指定精度。<br>
    /// Alternatively, the maximum width for string types. <br>或者，为字符串类型的最大宽度。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo(f32);
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         if let Some(precision) = formatter.precision() {
    ///             // If we received a precision, we use it. <br>如果我们收到了精度，我们就会使用它。<br>
    ///             write!(formatter, "Foo({1:.*})", precision, self.0)
    ///         } else {
    ///             // Otherwise we default to 2. <br>否则，我们默认为 2。<br>
    ///             write!(formatter, "Foo({:.2})", self.0)
    ///         }
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{:.4}", Foo(23.2)), "Foo(23.2000)");
    /// assert_eq!(&format!("{}", Foo(23.2)), "Foo(23.20)");
    /// ```
    #[must_use]
    #[stable(feature = "fmt_flags", since = "1.5.0")]
    pub fn precision(&self) -> Option<usize> {
        self.precision
    }

    /// Determines if the `+` flag was specified. <br>确定是否指定了 `+` 标志。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo(i32);
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         if formatter.sign_plus() {
    ///             write!(formatter,
    ///                    "Foo({}{})",
    ///                    if self.0 < 0 { '-' } else { '+' },
    ///                    self.0)
    ///         } else {
    ///             write!(formatter, "Foo({})", self.0)
    ///         }
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{:+}", Foo(23)), "Foo(+23)");
    /// assert_eq!(&format!("{}", Foo(23)), "Foo(23)");
    /// ```
    #[must_use]
    #[stable(feature = "fmt_flags", since = "1.5.0")]
    pub fn sign_plus(&self) -> bool {
        self.flags & (1 << FlagV1::SignPlus as u32) != 0
    }

    /// Determines if the `-` flag was specified. <br>确定是否指定了 `-` 标志。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo(i32);
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         if formatter.sign_minus() {
    ///             // You want a minus sign? <br>您想要一个减号？<br> Have one! <br>有一个！<br>
    ///             write!(formatter, "-Foo({})", self.0)
    ///         } else {
    ///             write!(formatter, "Foo({})", self.0)
    ///         }
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{:-}", Foo(23)), "-Foo(23)");
    /// assert_eq!(&format!("{}", Foo(23)), "Foo(23)");
    /// ```
    #[must_use]
    #[stable(feature = "fmt_flags", since = "1.5.0")]
    pub fn sign_minus(&self) -> bool {
        self.flags & (1 << FlagV1::SignMinus as u32) != 0
    }

    /// Determines if the `#` flag was specified. <br>确定是否指定了 `#` 标志。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo(i32);
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         if formatter.alternate() {
    ///             write!(formatter, "Foo({})", self.0)
    ///         } else {
    ///             write!(formatter, "{}", self.0)
    ///         }
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{:#}", Foo(23)), "Foo(23)");
    /// assert_eq!(&format!("{}", Foo(23)), "23");
    /// ```
    #[must_use]
    #[stable(feature = "fmt_flags", since = "1.5.0")]
    pub fn alternate(&self) -> bool {
        self.flags & (1 << FlagV1::Alternate as u32) != 0
    }

    /// Determines if the `0` flag was specified. <br>确定是否指定了 `0` 标志。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// struct Foo(i32);
    ///
    /// impl fmt::Display for Foo {
    ///     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
    ///         assert!(formatter.sign_aware_zero_pad());
    ///         assert_eq!(formatter.width(), Some(4));
    ///         // We ignore the formatter's options. <br>我们忽略格式化程序的选项。<br>
    ///         write!(formatter, "{}", self.0)
    ///     }
    /// }
    ///
    /// assert_eq!(&format!("{:04}", Foo(23)), "23");
    /// ```
    #[must_use]
    #[stable(feature = "fmt_flags", since = "1.5.0")]
    pub fn sign_aware_zero_pad(&self) -> bool {
        self.flags & (1 << FlagV1::SignAwareZeroPad as u32) != 0
    }

    // FIXME: Decide what public API we want for these two flags. <br>确定我们要为这两个标志使用的公共 API。<br>
    // https://github.com/rust-lang/rust/issues/48584
    fn debug_lower_hex(&self) -> bool {
        self.flags & (1 << FlagV1::DebugLowerHex as u32) != 0
    }

    fn debug_upper_hex(&self) -> bool {
        self.flags & (1 << FlagV1::DebugUpperHex as u32) != 0
    }

    /// Creates a [`DebugStruct`] builder designed to assist with creation of [`fmt::Debug`] implementations for structs. <br>创建一个 [`DebugStruct`] 构建器，该构建器旨在帮助创建结构体的 [`fmt::Debug`] 实现。<br>
    ///
    ///
    /// [`fmt::Debug`]: self::Debug
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::fmt;
    /// use std::net::Ipv4Addr;
    ///
    /// struct Foo {
    ///     bar: i32,
    ///     baz: String,
    ///     addr: Ipv4Addr,
    /// }
    ///
    /// impl fmt::Debug for Foo {
    ///     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
    ///         fmt.debug_struct("Foo")
    ///             .field("bar", &self.bar)
    ///             .field("baz", &self.baz)
    ///             .field("addr", &format_args!("{}", self.addr))
    ///             .finish()
    ///     }
    /// }
    ///
    /// assert_eq!(
    ///     "Foo { bar: 10, baz: \"Hello World\", addr: 127.0.0.1 }",
    ///     format!("{:?}", Foo {
    ///         bar: 10,
    ///         baz: "Hello World".to_string(),
    ///         addr: Ipv4Addr::new(127, 0, 0, 1),
    ///     })
    /// );
    /// ```
    #[stable(feature = "debug_builders", since = "1.2.0")]
    pub fn debug_struct<'b>(&'b mut self, name: &str) -> DebugStruct<'b, 'a> {
        builders::debug_struct_new(self, name)
    }

    /// Creates a `DebugTuple` builder designed to assist with creation of `fmt::Debug` implementations for tuple structs. <br>创建一个 `DebugTuple` 构建器，该构建器旨在协助创建元组结构体的 `fmt::Debug` 实现。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::fmt;
    /// use std::marker::PhantomData;
    ///
    /// struct Foo<T>(i32, String, PhantomData<T>);
    ///
    /// impl<T> fmt::Debug for Foo<T> {
    ///     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
    ///         fmt.debug_tuple("Foo")
    ///             .field(&self.0)
    ///             .field(&self.1)
    ///             .field(&format_args!("_"))
    ///             .finish()
    ///     }
    /// }
    ///
    /// assert_eq!(
    ///     "Foo(10, \"Hello\", _)",
    ///     format!("{:?}", Foo(10, "Hello".to_string(), PhantomData::<u8>))
    /// );
    /// ```
    #[stable(feature = "debug_builders", since = "1.2.0")]
    pub fn debug_tuple<'b>(&'b mut self, name: &str) -> DebugTuple<'b, 'a> {
        builders::debug_tuple_new(self, name)
    }

    /// Creates a `DebugList` builder designed to assist with creation of `fmt::Debug` implementations for list-like structures. <br>创建一个 `DebugList` 构建器，该构建器旨在帮助为类似列表的结构创建 `fmt::Debug` 实现。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::fmt;
    ///
    /// struct Foo(Vec<i32>);
    ///
    /// impl fmt::Debug for Foo {
    ///     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
    ///         fmt.debug_list().entries(self.0.iter()).finish()
    ///     }
    /// }
    ///
    /// assert_eq!(format!("{:?}", Foo(vec![10, 11])), "[10, 11]");
    /// ```
    #[stable(feature = "debug_builders", since = "1.2.0")]
    pub fn debug_list<'b>(&'b mut self) -> DebugList<'b, 'a> {
        builders::debug_list_new(self)
    }

    /// Creates a `DebugSet` builder designed to assist with creation of `fmt::Debug` implementations for set-like structures. <br>创建一个 `DebugSet` 构建器，该构建器旨在帮助为类似集合的结构创建 `fmt::Debug` 实现。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::fmt;
    ///
    /// struct Foo(Vec<i32>);
    ///
    /// impl fmt::Debug for Foo {
    ///     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
    ///         fmt.debug_set().entries(self.0.iter()).finish()
    ///     }
    /// }
    ///
    /// assert_eq!(format!("{:?}", Foo(vec![10, 11])), "{10, 11}");
    /// ```
    ///
    /// [`format_args!`]: crate::format_args
    ///
    /// In this more complex example, we use [`format_args!`] and `.debug_set()` to build a list of match arms: <br>在这个更复杂的示例中，我们使用 [`format_args!`] 和 `.debug_set()` 来构建匹配分支的列表：<br>
    ///
    /// ```rust
    /// use std::fmt;
    ///
    /// struct Arm<'a, L: 'a, R: 'a>(&'a (L, R));
    /// struct Table<'a, K: 'a, V: 'a>(&'a [(K, V)], V);
    ///
    /// impl<'a, L, R> fmt::Debug for Arm<'a, L, R>
    /// where
    ///     L: 'a + fmt::Debug, R: 'a + fmt::Debug
    /// {
    ///     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
    ///         L::fmt(&(self.0).0, fmt)?;
    ///         fmt.write_str(" => ")?;
    ///         R::fmt(&(self.0).1, fmt)
    ///     }
    /// }
    ///
    /// impl<'a, K, V> fmt::Debug for Table<'a, K, V>
    /// where
    ///     K: 'a + fmt::Debug, V: 'a + fmt::Debug
    /// {
    ///     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
    ///         fmt.debug_set()
    ///         .entries(self.0.iter().map(Arm))
    ///         .entry(&Arm(&(format_args!("_"), &self.1)))
    ///         .finish()
    ///     }
    /// }
    /// ```
    ///
    #[stable(feature = "debug_builders", since = "1.2.0")]
    pub fn debug_set<'b>(&'b mut self) -> DebugSet<'b, 'a> {
        builders::debug_set_new(self)
    }

    /// Creates a `DebugMap` builder designed to assist with creation of `fmt::Debug` implementations for map-like structures. <br>创建一个 `DebugMap` 构建器，该构建器旨在帮助为类似 map 的结构创建 `fmt::Debug` 实现。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::fmt;
    ///
    /// struct Foo(Vec<(String, i32)>);
    ///
    /// impl fmt::Debug for Foo {
    ///     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
    ///         fmt.debug_map().entries(self.0.iter().map(|&(ref k, ref v)| (k, v))).finish()
    ///     }
    /// }
    ///
    /// assert_eq!(
    ///     format!("{:?}",  Foo(vec![("A".to_string(), 10), ("B".to_string(), 11)])),
    ///     r#"{"A": 10, "B": 11}"#
    ///  );
    /// ```
    #[stable(feature = "debug_builders", since = "1.2.0")]
    pub fn debug_map<'b>(&'b mut self) -> DebugMap<'b, 'a> {
        builders::debug_map_new(self)
    }
}

#[stable(since = "1.2.0", feature = "formatter_write")]
impl Write for Formatter<'_> {
    fn write_str(&mut self, s: &str) -> Result {
        self.buf.write_str(s)
    }

    fn write_char(&mut self, c: char) -> Result {
        self.buf.write_char(c)
    }

    fn write_fmt(&mut self, args: Arguments<'_>) -> Result {
        write(self.buf, args)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Display for Error {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Display::fmt("an error occurred when formatting an argument", f)
    }
}

// Implementations of the core formatting traits <br>核心格式 traits 的实现<br>

macro_rules! fmt_refs {
    ($($tr:ident),*) => {
        $(
        #[stable(feature = "rust1", since = "1.0.0")]
        impl<T: ?Sized + $tr> $tr for &T {
            fn fmt(&self, f: &mut Formatter<'_>) -> Result { $tr::fmt(&**self, f) }
        }
        #[stable(feature = "rust1", since = "1.0.0")]
        impl<T: ?Sized + $tr> $tr for &mut T {
            fn fmt(&self, f: &mut Formatter<'_>) -> Result { $tr::fmt(&**self, f) }
        }
        )*
    }
}

fmt_refs! { Debug, Display, Octal, Binary, LowerHex, UpperHex, LowerExp, UpperExp }

#[unstable(feature = "never_type", issue = "35121")]
impl Debug for ! {
    fn fmt(&self, _: &mut Formatter<'_>) -> Result {
        *self
    }
}

#[unstable(feature = "never_type", issue = "35121")]
impl Display for ! {
    fn fmt(&self, _: &mut Formatter<'_>) -> Result {
        *self
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Debug for bool {
    #[inline]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Display::fmt(self, f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Display for bool {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Display::fmt(if *self { "true" } else { "false" }, f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Debug for str {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.write_char('"')?;
        let mut from = 0;
        for (i, c) in self.char_indices() {
            let esc = c.escape_debug_ext(EscapeDebugExtArgs {
                escape_grapheme_extended: true,
                escape_single_quote: false,
                escape_double_quote: true,
            });
            // If char needs escaping, flush backlog so far and write, else skip <br>如果 char 需要转义，请清除到目前为止的积压并编写，否则跳过<br>
            if esc.len() != 1 {
                f.write_str(&self[from..i])?;
                for c in esc {
                    f.write_char(c)?;
                }
                from = i + c.len_utf8();
            }
        }
        f.write_str(&self[from..])?;
        f.write_char('"')
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Display for str {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.pad(self)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Debug for char {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.write_char('\'')?;
        for c in self.escape_debug_ext(EscapeDebugExtArgs {
            escape_grapheme_extended: true,
            escape_single_quote: true,
            escape_double_quote: false,
        }) {
            f.write_char(c)?
        }
        f.write_char('\'')
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Display for char {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        if f.width.is_none() && f.precision.is_none() {
            f.write_char(*self)
        } else {
            f.pad(self.encode_utf8(&mut [0; 4]))
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> Pointer for *const T {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        // Cast is needed here because `.addr()` requires `T: Sized`. <br>这里需要强制转换，因为 `.addr()` 需要 `T: Sized`。<br>
        pointer_fmt_inner((*self as *const ()).addr(), f)
    }
}

/// Since the formatting will be identical for all pointer types, use a non-monomorphized implementation for the actual formatting to reduce the amount of codegen work needed. <br>由于所有指针类型的格式都是相同的，因此对实际格式使用非单态实现以减少所需的 codegen 工作量。<br>
///
///
/// This uses `ptr_addr: usize` and not `ptr: *const ()` to be able to use this for `fn(...) -> ...` without using [problematic] "Oxford Casts". <br>这使用 `ptr_addr: usize` 而不是 `ptr: *const ()` 能够在不使用 [problematic] "Oxford Casts" 的情况下将其用于 `fn(...) -> ...`。<br>
///
/// [problematic]: https://github.com/rust-lang/rust/issues/95489
///
pub(crate) fn pointer_fmt_inner(ptr_addr: usize, f: &mut Formatter<'_>) -> Result {
    let old_width = f.width;
    let old_flags = f.flags;

    // The alternate flag is already treated by LowerHex as being special- it denotes whether to prefix with 0x. <br>LowerHex 已将替代标志视为特殊标志，它表示是否以 0x 作为前缀。<br>
    // We use it to work out whether or not to zero extend, and then unconditionally set it to get the prefix. <br>我们使用它来计算是否为零扩展，然后无条件地将其设置为获取前缀。<br>
    //
    //
    if f.alternate() {
        f.flags |= 1 << (FlagV1::SignAwareZeroPad as u32);

        if f.width.is_none() {
            f.width = Some((usize::BITS / 4) as usize + 2);
        }
    }
    f.flags |= 1 << (FlagV1::Alternate as u32);

    let ret = LowerHex::fmt(&ptr_addr, f);

    f.width = old_width;
    f.flags = old_flags;

    ret
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> Pointer for *mut T {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Pointer::fmt(&(*self as *const T), f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> Pointer for &T {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Pointer::fmt(&(*self as *const T), f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> Pointer for &mut T {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Pointer::fmt(&(&**self as *const T), f)
    }
}

// Implementation of Display/Debug for various core types <br>Display/Debug 在各种核心类型上的实现<br>

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> Debug for *const T {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Pointer::fmt(self, f)
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> Debug for *mut T {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Pointer::fmt(self, f)
    }
}

macro_rules! peel {
    ($name:ident, $($other:ident,)*) => (tuple! { $($other,)* })
}

macro_rules! tuple {
    () => ();
    ( $($name:ident,)+ ) => (
        maybe_tuple_doc! {
            $($name)+ @
            #[stable(feature = "rust1", since = "1.0.0")]
            impl<$($name:Debug),+> Debug for ($($name,)+) where last_type!($($name,)+): ?Sized {
                #[allow(non_snake_case, unused_assignments)]
                fn fmt(&self, f: &mut Formatter<'_>) -> Result {
                    let mut builder = f.debug_tuple("");
                    let ($(ref $name,)+) = *self;
                    $(
                        builder.field(&$name);
                    )+

                    builder.finish()
                }
            }
        }
        peel! { $($name,)+ }
    )
}

macro_rules! maybe_tuple_doc {
    ($a:ident @ #[$meta:meta] $item:item) => {
        #[cfg_attr(not(bootstrap), doc(tuple_variadic))]
        #[doc = "This trait is implemented for tuples up to twelve items long."]
        #[$meta]
        $item
    };
    ($a:ident $($rest_a:ident)+ @ #[$meta:meta] $item:item) => {
        #[doc(hidden)]
        #[$meta]
        $item
    };
}

macro_rules! last_type {
    ($a:ident,) => { $a };
    ($a:ident, $($rest_a:ident,)+) => { last_type!($($rest_a,)+) };
}

tuple! { E, D, C, B, A, Z, Y, X, W, V, U, T, }

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Debug> Debug for [T] {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.debug_list().entries(self.iter()).finish()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Debug for () {
    #[inline]
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.pad("()")
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> Debug for PhantomData<T> {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.debug_struct("PhantomData").finish()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Copy + Debug> Debug for Cell<T> {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.debug_struct("Cell").field("value", &self.get()).finish()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized + Debug> Debug for RefCell<T> {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        match self.try_borrow() {
            Ok(borrow) => f.debug_struct("RefCell").field("value", &borrow).finish(),
            Err(_) => {
                // The RefCell is mutably borrowed so we can't look at its value here. <br>RefCell 是可变地借用的，因此我们在这里无法查看其值。<br>
                // Show a placeholder instead. <br>改为显示一个占位符。<br>
                struct BorrowedPlaceholder;

                impl Debug for BorrowedPlaceholder {
                    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
                        f.write_str("<borrowed>")
                    }
                }

                f.debug_struct("RefCell").field("value", &BorrowedPlaceholder).finish()
            }
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized + Debug> Debug for Ref<'_, T> {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Debug::fmt(&**self, f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized + Debug> Debug for RefMut<'_, T> {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        Debug::fmt(&*(self.deref()), f)
    }
}

#[stable(feature = "core_impl_debug", since = "1.9.0")]
impl<T: ?Sized> Debug for UnsafeCell<T> {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.debug_struct("UnsafeCell").finish_non_exhaustive()
    }
}

#[unstable(feature = "sync_unsafe_cell", issue = "95439")]
impl<T: ?Sized> Debug for SyncUnsafeCell<T> {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.debug_struct("SyncUnsafeCell").finish_non_exhaustive()
    }
}

// If you expected tests to be here, look instead at the core/tests/fmt.rs file, it's a lot easier than creating all of the rt::Piece structures here. <br>如果您希望测试在这里，请查看 core/tests/fmt.rs 文件，这比在此处创建所有 rt::Piece 结构要容易得多。<br>
//
// There are also tests in the alloc crate, for those that need allocations. <br>alloc crate 中也有测试，用于那些需要分配的测试。<br>
