use crate::future::Future;

/// Conversion into a `Future`. <br>转换为 `Future`。<br>
///
/// By implementing `IntoFuture` for a type, you define how it will be converted to a future. <br>通过为类型实现 `IntoFuture`，您可以定义如何将其转换为 future。<br>
///
/// # `.await` desugaring <br>`.await` 脱糖<br>
///
/// The `.await` keyword desugars into a call to `IntoFuture::into_future` first before polling the future to completion. <br>在轮询 future 完成之前，`.await` 关键字首先脱糖到 `IntoFuture::into_future` 的调用中。<br>
/// `IntoFuture` is implemented for all `T: Future` which means the `into_future` method will be available on all futures. <br>`IntoFuture` 适用于所有 `T: Future`，这意味着 `into_future` 方法将适用于所有 futures。<br>
///
/// ```no_run
/// #![feature(into_future)]
///
/// use std::future::IntoFuture;
///
/// # async fn foo() {
/// let v = async { "meow" };
/// let mut fut = v.into_future();
/// assert_eq!("meow", fut.await);
/// # }
/// ```
///
/// # Async builders <br>异步构建器<br>
///
/// When implementing futures manually there will often be a choice between implementing `Future` or `IntoFuture` for a type. <br>当手动实现 futures 时，通常会在为一个类型实现 `Future` 或 `IntoFuture` 之间做出选择。<br>
/// Implementing `Future` is a good choice in most cases. <br>在大多数情况下，实现 `Future` 是一个不错的选择。<br>
/// But implementing `IntoFuture` is most useful when implementing "async builder" types, which allow their values to be modified multiple times before being `.await`ed. <br>但是在实现 "async builder" 类型时实现 `IntoFuture` 是最有用的，它允许在被 `.await` 编辑之前多次修改它们的值。<br>
///
/// ```rust
/// #![feature(into_future)]
///
/// use std::future::{ready, Ready, IntoFuture};
///
/// /// Eventually multiply two numbers <br>最终将两个数字相乘<br>
/// pub struct Multiply {
///     num: u16,
///     factor: u16,
/// }
///
/// impl Multiply {
///     /// Construct a new instance of `Multiply`. <br>创建一个新的 `Multiply` 实例。<br>
///     pub fn new(num: u16, factor: u16) -> Self {
///         Self { num, factor }
///     }
///
///     /// Set the number to multiply by the factor. <br>设置要乘以因子的数字。<br>
///     pub fn number(mut self, num: u16) -> Self {
///         self.num = num;
///         self
///     }
///
///     /// Set the factor to multiply the number with. <br>设置要与数字相乘的因子。<br>
///     pub fn factor(mut self, factor: u16) -> Self {
///         self.factor = factor;
///         self
///     }
/// }
///
/// impl IntoFuture for Multiply {
///     type Output = u16;
///     type IntoFuture = Ready<Self::Output>;
///
///     fn into_future(self) -> Self::IntoFuture {
///         ready(self.num * self.factor)
///     }
/// }
///
/// // NOTE: Rust does not yet have an `async fn main` function, that functionality currently only exists in the ecosystem. <br>Rust 还没有 `async fn main` 函数，该功能目前仅存在于生态系统中。<br>
/////
/// async fn run() {
///     let num = Multiply::new(0, 0)  // initialize the builder to number: 0, factor: 0 <br>将构建器初始化为数字: 0，因子: 0<br>
///         .number(2)                 // change the number to 2 <br>将数字更改为 2<br>
///         .factor(2)                 // change the factor to 2 <br>将因子更改为 2<br>
///         .await;                    // convert to future and .await <br>转换为 future 和 .await<br>
///
///     assert_eq!(num, 4);
/// }
/// ```
///
/// # Usage in trait bounds <br>trait bounds 中的用法<br>
///
/// Using `IntoFuture` in trait bounds allows a function to be generic over both `Future` and `IntoFuture`. <br>在 trait 中使用 `IntoFuture` bounds 允许函数在 `Future` 和 `IntoFuture` 上都是泛型。<br>
/// This is convenient for users of the function, so when they are using it they don't have to make an extra call to `IntoFuture::into_future` to obtain an instance of `Future`: <br>这对函数的用户来说很方便，所以当他们使用它时，他们不必对 `IntoFuture::into_future` 进行额外的调用来获取 `Future` 的实例:<br>
///
///
/// ```rust
/// #![feature(into_future)]
///
/// use std::future::IntoFuture;
///
/// /// Convert the output of a future to a string. <br>将 future 的输出转换为字符串。<br>
/// async fn fut_to_string<Fut>(fut: Fut) -> String
/// where
///     Fut: IntoFuture,
///     Fut::Output: std::fmt::Debug,
/// {
///     format!("{:?}", fut.await)
/// }
/// ```
///
///
///
///
///
///
#[unstable(feature = "into_future", issue = "67644")]
pub trait IntoFuture {
    /// The output that the future will produce on completion. <br>future 完成时将产生的输出。<br>
    #[unstable(feature = "into_future", issue = "67644")]
    type Output;

    /// Which kind of future are we turning this into? <br>我们要把它变成哪种 future?<br>
    #[unstable(feature = "into_future", issue = "67644")]
    type IntoFuture: Future<Output = Self::Output>;

    /// Creates a future from a value. <br>根据一个值创建一个 future。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// #![feature(into_future)]
    ///
    /// use std::future::IntoFuture;
    ///
    /// # async fn foo() {
    /// let v = async { "meow" };
    /// let mut fut = v.into_future();
    /// assert_eq!("meow", fut.await);
    /// # }
    /// ```
    #[unstable(feature = "into_future", issue = "67644")]
    #[lang = "into_future"]
    fn into_future(self) -> Self::IntoFuture;
}

#[unstable(feature = "into_future", issue = "67644")]
impl<F: Future> IntoFuture for F {
    type Output = F::Output;
    type IntoFuture = F;

    fn into_future(self) -> Self::IntoFuture {
        self
    }
}
