use crate::future::Future;
use crate::pin::Pin;
use crate::task::{Context, Poll};

/// A future that is immediately ready with a value. <br>一个立即准备好值的 future。<br>
///
/// This `struct` is created by [`ready()`]. <br>该 `struct` 由 [`ready()`] 创建。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
#[stable(feature = "future_readiness_fns", since = "1.48.0")]
#[derive(Debug, Clone)]
#[must_use = "futures do nothing unless you `.await` or poll them"]
pub struct Ready<T>(Option<T>);

#[stable(feature = "future_readiness_fns", since = "1.48.0")]
impl<T> Unpin for Ready<T> {}

#[stable(feature = "future_readiness_fns", since = "1.48.0")]
impl<T> Future for Ready<T> {
    type Output = T;

    #[inline]
    fn poll(mut self: Pin<&mut Self>, _cx: &mut Context<'_>) -> Poll<T> {
        Poll::Ready(self.0.take().expect("`Ready` polled after completion"))
    }
}

/// Creates a future that is immediately ready with a value. <br>创建一个立即准备好值的 future。<br>
///
/// Futures created through this function are functionally similar to those created through `async {}`. <br>通过此函数创建的 Futures 在功能上类似于通过 `async {}` 创建的 Futures。<br>
/// The main difference is that futures created through this function are named and implement `Unpin`. <br>主要区别在于，通过此函数创建的 futures 被命名并实现 `Unpin`。<br>
///
/// # Examples
///
/// ```
/// use std::future;
///
/// # async fn run() {
/// let a = future::ready(1);
/// assert_eq!(a.await, 1);
/// # }
/// ```
///
#[stable(feature = "future_readiness_fns", since = "1.48.0")]
pub fn ready<T>(t: T) -> Ready<T> {
    Ready(Some(t))
}
