use crate::iter::{FusedIterator, TrustedLen};

/// Creates a new iterator that repeats elements of type `A` endlessly by applying the provided closure, the repeater, `F: FnMut() -> A`. <br>创建一个新的迭代器，通过应用提供的闭包，转发器 `F: FnMut() -> A`，无限地重复 `A` 类型的元素。<br>
///
/// The `repeat_with()` function calls the repeater over and over again. <br>`repeat_with()` 函数一遍又一遍地调用中继器。<br>
///
/// Infinite iterators like `repeat_with()` are often used with adapters like [`Iterator::take()`], in order to make them finite. <br>无限迭代器 (如 `repeat_with()`) 通常与适配器 (如 [`Iterator::take()`]) 一起使用，以使其具有有限性。<br>
///
/// If the element type of the iterator you need implements [`Clone`], and it is OK to keep the source element in memory, you should instead use the [`repeat()`] function. <br>如果您需要的迭代器的元素类型实现 [`Clone`]，并且可以将源元素保留在内存中，则应改用 [`repeat()`] 函数。<br>
///
///
/// An iterator produced by `repeat_with()` is not a [`DoubleEndedIterator`]. <br>`repeat_with()` 产生的迭代器不是 [`DoubleEndedIterator`]。<br>
/// If you need `repeat_with()` to return a [`DoubleEndedIterator`], please open a GitHub issue explaining your use case. <br>如果您需要 `repeat_with()` 来返回 [`DoubleEndedIterator`]，请打开 GitHub 问题，说明您的用例。<br>
///
/// [`repeat()`]: crate::iter::repeat
/// [`DoubleEndedIterator`]: crate::iter::DoubleEndedIterator
///
/// # Examples
///
/// Basic usage: <br>基本用法：<br>
///
/// ```
/// use std::iter;
///
/// // let's assume we have some value of a type that is not `Clone` or which we don't want to have in memory just yet because it is expensive: <br>假设我们有一些不是 `Clone` 类型的值，或者我们还不想在内存中拥有它，因为它很昂贵:<br>
/////
/// #[derive(PartialEq, Debug)]
/// struct Expensive;
///
/// // a particular value forever: <br>永远具有特定的值：<br>
/// let mut things = iter::repeat_with(|| Expensive);
///
/// assert_eq!(Some(Expensive), things.next());
/// assert_eq!(Some(Expensive), things.next());
/// assert_eq!(Some(Expensive), things.next());
/// assert_eq!(Some(Expensive), things.next());
/// assert_eq!(Some(Expensive), things.next());
/// ```
///
/// Using mutation and going finite: <br>使用可变和有限化：<br>
///
/// ```rust
/// use std::iter;
///
/// // From the zeroth to the third power of two: <br>从两个的零次幂到三次幂：<br>
/// let mut curr = 1;
/// let mut pow2 = iter::repeat_with(|| { let tmp = curr; curr *= 2; tmp })
///                     .take(4);
///
/// assert_eq!(Some(1), pow2.next());
/// assert_eq!(Some(2), pow2.next());
/// assert_eq!(Some(4), pow2.next());
/// assert_eq!(Some(8), pow2.next());
///
/// // ... and now we're done <br>现在我们完成了<br>
/// assert_eq!(None, pow2.next());
/// ```
///
///
///
///
#[inline]
#[stable(feature = "iterator_repeat_with", since = "1.28.0")]
pub fn repeat_with<A, F: FnMut() -> A>(repeater: F) -> RepeatWith<F> {
    RepeatWith { repeater }
}

/// An iterator that repeats elements of type `A` endlessly by applying the provided closure `F: FnMut() -> A`. <br>一个迭代器，通过应用提供的闭包 `F: FnMut() -> A`，无限地重复 `A` 类型的元素。<br>
///
///
/// This `struct` is created by the [`repeat_with()`] function. <br>该 `struct` 由 [`repeat_with()`] 函数创建。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
#[derive(Copy, Clone, Debug)]
#[stable(feature = "iterator_repeat_with", since = "1.28.0")]
pub struct RepeatWith<F> {
    repeater: F,
}

#[stable(feature = "iterator_repeat_with", since = "1.28.0")]
impl<A, F: FnMut() -> A> Iterator for RepeatWith<F> {
    type Item = A;

    #[inline]
    fn next(&mut self) -> Option<A> {
        Some((self.repeater)())
    }

    #[inline]
    fn size_hint(&self) -> (usize, Option<usize>) {
        (usize::MAX, None)
    }
}

#[stable(feature = "iterator_repeat_with", since = "1.28.0")]
impl<A, F: FnMut() -> A> FusedIterator for RepeatWith<F> {}

#[unstable(feature = "trusted_len", issue = "37572")]
unsafe impl<A, F: FnMut() -> A> TrustedLen for RepeatWith<F> {}
