use crate::iter;
use crate::num::Wrapping;

/// Trait to represent types that can be created by summing up an iterator. <br>一个表示可以通过对迭代器求和来创建的类型的 trait。<br>
///
/// This trait is used to implement [`Iterator::sum()`]. <br>这个 trait 用于实现 [`Iterator::sum()`]。<br>
/// Types which implement this trait can be generated by using the [`sum()`] method on an iterator. <br>可以通过在迭代器上使用 [`sum()`] 方法生成实现此 trait 的类型。<br>
/// Like [`FromIterator`], this trait should rarely be called directly. <br>和 [`FromIterator`] 一样，这个 trait 应该很少被直接调用。<br>
///
/// [`sum()`]: Iterator::sum
/// [`FromIterator`]: iter::FromIterator
#[stable(feature = "iter_arith_traits", since = "1.12.0")]
pub trait Sum<A = Self>: Sized {
    /// Method which takes an iterator and generates `Self` from the elements by "summing up" the items. <br>使用迭代器并通过 "summing up" 项从元素生成 `Self` 的方法。<br>
    ///
    #[stable(feature = "iter_arith_traits", since = "1.12.0")]
    fn sum<I: Iterator<Item = A>>(iter: I) -> Self;
}

/// Trait to represent types that can be created by multiplying elements of an iterator. <br>一个表示可以通过将迭代器的元素相乘来创建类型的 trait。<br>
///
/// This trait is used to implement [`Iterator::product()`]. <br>这个 trait 用于实现 [`Iterator::product()`]。<br>
/// Types which implement this trait can be generated by using the [`product()`] method on an iterator. <br>可以通过在迭代器上使用 [`product()`] 方法生成实现此 trait 的类型。<br>
///
/// Like [`FromIterator`], this trait should rarely be called directly. <br>和 [`FromIterator`] 一样，这个 trait 应该很少被直接调用。<br>
///
/// [`product()`]: Iterator::product
/// [`FromIterator`]: iter::FromIterator
#[stable(feature = "iter_arith_traits", since = "1.12.0")]
pub trait Product<A = Self>: Sized {
    /// Method which takes an iterator and generates `Self` from the elements by multiplying the items. <br>该方法采用迭代器并通过乘以项从元素生成 `Self`。<br>
    ///
    #[stable(feature = "iter_arith_traits", since = "1.12.0")]
    fn product<I: Iterator<Item = A>>(iter: I) -> Self;
}

macro_rules! integer_sum_product {
    (@impls $zero:expr, $one:expr, #[$attr:meta], $($a:ty)*) => ($(
        #[$attr]
        impl Sum for $a {
            fn sum<I: Iterator<Item=Self>>(iter: I) -> Self {
                iter.fold(
                    $zero,
                    #[rustc_inherit_overflow_checks]
                    |a, b| a + b,
                )
            }
        }

        #[$attr]
        impl Product for $a {
            fn product<I: Iterator<Item=Self>>(iter: I) -> Self {
                iter.fold(
                    $one,
                    #[rustc_inherit_overflow_checks]
                    |a, b| a * b,
                )
            }
        }

        #[$attr]
        impl<'a> Sum<&'a $a> for $a {
            fn sum<I: Iterator<Item=&'a Self>>(iter: I) -> Self {
                iter.fold(
                    $zero,
                    #[rustc_inherit_overflow_checks]
                    |a, b| a + b,
                )
            }
        }

        #[$attr]
        impl<'a> Product<&'a $a> for $a {
            fn product<I: Iterator<Item=&'a Self>>(iter: I) -> Self {
                iter.fold(
                    $one,
                    #[rustc_inherit_overflow_checks]
                    |a, b| a * b,
                )
            }
        }
    )*);
    ($($a:ty)*) => (
        integer_sum_product!(@impls 0, 1,
                #[stable(feature = "iter_arith_traits", since = "1.12.0")],
                $($a)*);
        integer_sum_product!(@impls Wrapping(0), Wrapping(1),
                #[stable(feature = "wrapping_iter_arith", since = "1.14.0")],
                $(Wrapping<$a>)*);
    );
}

macro_rules! float_sum_product {
    ($($a:ident)*) => ($(
        #[stable(feature = "iter_arith_traits", since = "1.12.0")]
        impl Sum for $a {
            fn sum<I: Iterator<Item=Self>>(iter: I) -> Self {
                iter.fold(
                    0.0,
                    #[rustc_inherit_overflow_checks]
                    |a, b| a + b,
                )
            }
        }

        #[stable(feature = "iter_arith_traits", since = "1.12.0")]
        impl Product for $a {
            fn product<I: Iterator<Item=Self>>(iter: I) -> Self {
                iter.fold(
                    1.0,
                    #[rustc_inherit_overflow_checks]
                    |a, b| a * b,
                )
            }
        }

        #[stable(feature = "iter_arith_traits", since = "1.12.0")]
        impl<'a> Sum<&'a $a> for $a {
            fn sum<I: Iterator<Item=&'a Self>>(iter: I) -> Self {
                iter.fold(
                    0.0,
                    #[rustc_inherit_overflow_checks]
                    |a, b| a + b,
                )
            }
        }

        #[stable(feature = "iter_arith_traits", since = "1.12.0")]
        impl<'a> Product<&'a $a> for $a {
            fn product<I: Iterator<Item=&'a Self>>(iter: I) -> Self {
                iter.fold(
                    1.0,
                    #[rustc_inherit_overflow_checks]
                    |a, b| a * b,
                )
            }
        }
    )*)
}

integer_sum_product! { i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize }
float_sum_product! { f32 f64 }

#[stable(feature = "iter_arith_traits_result", since = "1.16.0")]
impl<T, U, E> Sum<Result<U, E>> for Result<T, E>
where
    T: Sum<U>,
{
    /// Takes each element in the [`Iterator`]: if it is an [`Err`], no further elements are taken, and the [`Err`] is returned. <br>接受 [`Iterator`] 中的每个元素：如果它是 [`Err`]，则不再获取其他元素，并返回 [`Err`]。<br>
    /// Should no [`Err`] occur, the sum of all elements is returned. <br>如果没有发生 [`Err`]，则返回所有元素的总和。<br>
    ///
    /// # Examples
    ///
    /// This sums up every integer in a vector, rejecting the sum if a negative element is encountered: <br>这将对 vector 中的每个整数求和，如果遇到负元素，则拒绝求和：<br>
    ///
    /// ```
    /// let v = vec![1, 2];
    /// let res: Result<i32, &'static str> = v.iter().map(|&x: &i32|
    ///     if x < 0 { Err("Negative element found") }
    ///     else { Ok(x) }
    /// ).sum();
    /// assert_eq!(res, Ok(3));
    /// ```
    ///
    ///
    fn sum<I>(iter: I) -> Result<T, E>
    where
        I: Iterator<Item = Result<U, E>>,
    {
        iter::try_process(iter, |i| i.sum())
    }
}

#[stable(feature = "iter_arith_traits_result", since = "1.16.0")]
impl<T, U, E> Product<Result<U, E>> for Result<T, E>
where
    T: Product<U>,
{
    /// Takes each element in the [`Iterator`]: if it is an [`Err`], no further elements are taken, and the [`Err`] is returned. <br>接受 [`Iterator`] 中的每个元素：如果它是 [`Err`]，则不再获取其他元素，并返回 [`Err`]。<br>
    /// Should no [`Err`] occur, the product of all elements is returned. <br>如果没有发生 [`Err`]，则返回所有元素的乘积。<br>
    ///
    fn product<I>(iter: I) -> Result<T, E>
    where
        I: Iterator<Item = Result<U, E>>,
    {
        iter::try_process(iter, |i| i.product())
    }
}

#[stable(feature = "iter_arith_traits_option", since = "1.37.0")]
impl<T, U> Sum<Option<U>> for Option<T>
where
    T: Sum<U>,
{
    /// Takes each element in the [`Iterator`]: if it is a [`None`], no further elements are taken, and the [`None`] is returned. <br>接受 [`Iterator`] 中的每个元素：如果它是 [`None`]，则不再获取其他元素，并返回 [`None`]。<br>
    /// Should no [`None`] occur, the sum of all elements is returned. <br>如果没有发生 [`None`]，则返回所有元素的总和。<br>
    ///
    /// # Examples
    ///
    /// This sums up the position of the character 'a' in a vector of strings, if a word did not have the character 'a' the operation returns `None`: <br>这总结了字符 'a' 在字符串 vector 中的位置，如果单词没有字符 'a'，则该操作返回 `None`：<br>
    ///
    ///
    /// ```
    /// let words = vec!["have", "a", "great", "day"];
    /// let total: Option<usize> = words.iter().map(|w| w.find('a')).sum();
    /// assert_eq!(total, Some(5));
    /// ```
    ///
    fn sum<I>(iter: I) -> Option<T>
    where
        I: Iterator<Item = Option<U>>,
    {
        iter::try_process(iter, |i| i.sum())
    }
}

#[stable(feature = "iter_arith_traits_option", since = "1.37.0")]
impl<T, U> Product<Option<U>> for Option<T>
where
    T: Product<U>,
{
    /// Takes each element in the [`Iterator`]: if it is a [`None`], no further elements are taken, and the [`None`] is returned. <br>接受 [`Iterator`] 中的每个元素：如果它是 [`None`]，则不再获取其他元素，并返回 [`None`]。<br>
    /// Should no [`None`] occur, the product of all elements is returned. <br>如果没有发生 [`None`]，则返回所有元素的乘积。<br>
    ///
    fn product<I>(iter: I) -> Option<T>
    where
        I: Iterator<Item = Option<U>>,
    {
        iter::try_process(iter, |i| i.product())
    }
}
