#[doc = include_str!("panic.md")]
#[macro_export]
#[rustc_builtin_macro(core_panic)]
#[allow_internal_unstable(edition_panic)]
#[stable(feature = "core", since = "1.6.0")]
#[rustc_diagnostic_item = "core_panic_macro"]
macro_rules! panic {
    // Expands to either `$crate::panic::panic_2015` or `$crate::panic::panic_2021` depending on the edition of the caller. <br>根据调用者的版本扩展为 `$crate::panic::panic_2015` 或 `$crate::panic::panic_2021`。<br>
    //
    ($($arg:tt)*) => {
        /* compiler built-in */
    };
}

/// Asserts that two expressions are equal to each other (using [`PartialEq`]). <br>断言两个表达式彼此相等 (使用 [`PartialEq`])。<br>
///
/// On panic, this macro will print the values of the expressions with their debug representations. <br>在 panic 上，此宏将打印表达式的值及其调试表示。<br>
///
///
/// Like [`assert!`], this macro has a second form, where a custom panic message can be provided. <br>像 [`assert!`] 一样，此宏具有第二种形式，可以在其中提供自定义 panic 消息。<br>
///
/// # Examples
///
/// ```
/// let a = 3;
/// let b = 1 + 2;
/// assert_eq!(a, b);
///
/// assert_eq!(a, b, "we are testing addition with {} and {}", a, b);
/// ```
///
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "assert_eq_macro")]
#[allow_internal_unstable(core_panic)]
macro_rules! assert_eq {
    ($left:expr, $right:expr $(,)?) => {
        match (&$left, &$right) {
            (left_val, right_val) => {
                if !(*left_val == *right_val) {
                    let kind = $crate::panicking::AssertKind::Eq;
                    // The reborrows below are intentional. <br>下面的重新借用是有意的。<br>
                    // Without them, the stack slot for the borrow is initialized even before the values are compared, leading to a noticeable slow down. <br>如果没有它们，则即使在比较值之前，也会对借用的栈插槽进行初始化，从而导致速度明显下降。<br>
                    //
                    $crate::panicking::assert_failed(kind, &*left_val, &*right_val, $crate::option::Option::None);
                }
            }
        }
    };
    ($left:expr, $right:expr, $($arg:tt)+) => {
        match (&$left, &$right) {
            (left_val, right_val) => {
                if !(*left_val == *right_val) {
                    let kind = $crate::panicking::AssertKind::Eq;
                    // The reborrows below are intentional. <br>下面的重新借用是有意的。<br>
                    // Without them, the stack slot for the borrow is initialized even before the values are compared, leading to a noticeable slow down. <br>如果没有它们，则即使在比较值之前，也会对借用的栈插槽进行初始化，从而导致速度明显下降。<br>
                    //
                    $crate::panicking::assert_failed(kind, &*left_val, &*right_val, $crate::option::Option::Some($crate::format_args!($($arg)+)));
                }
            }
        }
    };
}

/// Asserts that two expressions are not equal to each other (using [`PartialEq`]). <br>断言两个表达式彼此不相等 (使用 [`PartialEq`])。<br>
///
/// On panic, this macro will print the values of the expressions with their debug representations. <br>在 panic 上，此宏将打印表达式的值及其调试表示。<br>
///
///
/// Like [`assert!`], this macro has a second form, where a custom panic message can be provided. <br>像 [`assert!`] 一样，此宏具有第二种形式，可以在其中提供自定义 panic 消息。<br>
///
/// # Examples
///
/// ```
/// let a = 3;
/// let b = 2;
/// assert_ne!(a, b);
///
/// assert_ne!(a, b, "we are testing that the values are not equal");
/// ```
///
#[macro_export]
#[stable(feature = "assert_ne", since = "1.13.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "assert_ne_macro")]
#[allow_internal_unstable(core_panic)]
macro_rules! assert_ne {
    ($left:expr, $right:expr $(,)?) => {
        match (&$left, &$right) {
            (left_val, right_val) => {
                if *left_val == *right_val {
                    let kind = $crate::panicking::AssertKind::Ne;
                    // The reborrows below are intentional. <br>下面的重新借用是有意的。<br>
                    // Without them, the stack slot for the borrow is initialized even before the values are compared, leading to a noticeable slow down. <br>如果没有它们，则即使在比较值之前，也会对借用的栈插槽进行初始化，从而导致速度明显下降。<br>
                    //
                    $crate::panicking::assert_failed(kind, &*left_val, &*right_val, $crate::option::Option::None);
                }
            }
        }
    };
    ($left:expr, $right:expr, $($arg:tt)+) => {
        match (&($left), &($right)) {
            (left_val, right_val) => {
                if *left_val == *right_val {
                    let kind = $crate::panicking::AssertKind::Ne;
                    // The reborrows below are intentional. <br>下面的重新借用是有意的。<br>
                    // Without them, the stack slot for the borrow is initialized even before the values are compared, leading to a noticeable slow down. <br>如果没有它们，则即使在比较值之前，也会对借用的栈插槽进行初始化，从而导致速度明显下降。<br>
                    //
                    $crate::panicking::assert_failed(kind, &*left_val, &*right_val, $crate::option::Option::Some($crate::format_args!($($arg)+)));
                }
            }
        }
    };
}

/// Asserts that an expression matches any of the given patterns. <br>断言表达式匹配任何给定的模式。<br>
///
/// Like in a `match` expression, the pattern can be optionally followed by `if` and a guard expression that has access to names bound by the pattern. <br>像在 `match` 表达式中一样，可以在模式后跟 `if` 和可以访问由模式绑定的名称的保护表达式。<br>
///
///
/// On panic, this macro will print the value of the expression with its debug representation. <br>在 panic 时，这个宏将打印表达式的值及其调试表示。<br>
///
/// Like [`assert!`], this macro has a second form, where a custom panic message can be provided. <br>像 [`assert!`] 一样，此宏具有第二种形式，可以在其中提供自定义 panic 消息。<br>
///
/// # Examples
///
/// ```
/// #![feature(assert_matches)]
///
/// use std::assert_matches::assert_matches;
///
/// let a = 1u32.checked_add(2);
/// let b = 1u32.checked_sub(2);
/// assert_matches!(a, Some(_));
/// assert_matches!(b, None);
///
/// let c = Ok("abc".to_string());
/// assert_matches!(c, Ok(x) | Err(x) if x.len() < 100);
/// ```
///
///
#[unstable(feature = "assert_matches", issue = "82775")]
#[allow_internal_unstable(core_panic)]
#[rustc_macro_transparency = "semitransparent"]
pub macro assert_matches {
    ($left:expr, $(|)? $( $pattern:pat_param )|+ $( if $guard: expr )? $(,)?) => {
        match $left {
            $( $pattern )|+ $( if $guard )? => {}
            ref left_val => {
                $crate::panicking::assert_matches_failed(
                    left_val,
                    $crate::stringify!($($pattern)|+ $(if $guard)?),
                    $crate::option::Option::None
                );
            }
        }
    },
    ($left:expr, $(|)? $( $pattern:pat_param )|+ $( if $guard: expr )?, $($arg:tt)+) => {
        match $left {
            $( $pattern )|+ $( if $guard )? => {}
            ref left_val => {
                $crate::panicking::assert_matches_failed(
                    left_val,
                    $crate::stringify!($($pattern)|+ $(if $guard)?),
                    $crate::option::Option::Some($crate::format_args!($($arg)+))
                );
            }
        }
    },
}

/// Asserts that a boolean expression is `true` at runtime. <br>声明在运行时布尔表达式为 `true`。<br>
///
/// This will invoke the [`panic!`] macro if the provided expression cannot be evaluated to `true` at runtime. <br>如果提供的表达式在运行时无法评估为 `true`，则将调用 [`panic!`] 宏。<br>
///
/// Like [`assert!`], this macro also has a second version, where a custom panic message can be provided. <br>像 [`assert!`] 一样，此宏还具有第二个版本，可以在其中提供自定义 panic 消息。<br>
///
/// # Uses
///
/// Unlike [`assert!`], `debug_assert!` statements are only enabled in non optimized builds by default. <br>与 [`assert!`] 不同，默认情况下仅在未优化的构建中启用 `debug_assert!` 语句。<br>
/// An optimized build will not execute `debug_assert!` statements unless `-C debug-assertions` is passed to the compiler. <br>除非将 `-C debug-assertions` 传递给编译器，否则优化的构建将不执行 `debug_assert!` 语句。<br>
/// This makes `debug_assert!` useful for checks that are too expensive to be present in a release build but may be helpful during development. <br>这使 `debug_assert!` 对于检查成本太高而无法在发行版本中进行检查，但在开发过程中可能很有用。<br>
/// The result of expanding `debug_assert!` is always type checked. <br>扩展 `debug_assert!` 的结果始终是类型检查的。<br>
///
/// An unchecked assertion allows a program in an inconsistent state to keep running, which might have unexpected consequences but does not introduce unsafety as long as this only happens in safe code. <br>未检查的断言允许处于不一致状态的程序继续运行，这可能会带来意想不到的后果，但不会引入不安全性，只要这种不安全性仅在安全代码中发生即可。<br>
///
/// The performance cost of assertions, however, is not measurable in general. <br>但是，断言的性能成本通常无法衡量。<br>
/// Replacing [`assert!`] with `debug_assert!` is thus only encouraged after thorough profiling, and more importantly, only in safe code! <br>因此，仅在经过全面分析后才鼓励使用 `debug_assert!` 替换 [`assert!`]，更重要的是，仅使用安全代码！<br>
///
/// # Examples
///
/// ```
/// // the panic message for these assertions is the stringified value of the expression given. <br>这些断言的 panic 消息是给定表达式的字符串化值。<br>
/////
/// debug_assert!(true);
///
/// fn some_expensive_computation() -> bool { true } // a very simple function <br>一个非常简单的函数<br>
/// debug_assert!(some_expensive_computation());
///
/// // assert with a custom message <br>使用自定义消息进行断言<br>
/// let x = true;
/// debug_assert!(x, "x wasn't true!");
///
/// let a = 3; let b = 27;
/// debug_assert!(a + b == 30, "a = {}, b = {}", a, b);
/// ```
///
///
///
///
///
///
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[rustc_diagnostic_item = "debug_assert_macro"]
#[allow_internal_unstable(edition_panic)]
macro_rules! debug_assert {
    ($($arg:tt)*) => {
        if $crate::cfg!(debug_assertions) {
            $crate::assert!($($arg)*);
        }
    };
}

/// Asserts that two expressions are equal to each other. <br>断言两个表达式彼此相等。<br>
///
/// On panic, this macro will print the values of the expressions with their debug representations. <br>在 panic 上，此宏将打印表达式的值及其调试表示。<br>
///
/// Unlike [`assert_eq!`], `debug_assert_eq!` statements are only enabled in non optimized builds by default. <br>与 [`assert_eq!`] 不同，默认情况下仅在未优化的构建中启用 `debug_assert_eq!` 语句。<br>
/// An optimized build will not execute `debug_assert_eq!` statements unless `-C debug-assertions` is passed to the compiler. <br>除非将 `-C debug-assertions` 传递给编译器，否则优化的构建将不执行 `debug_assert_eq!` 语句。<br>
/// This makes `debug_assert_eq!` useful for checks that are too expensive to be present in a release build but may be helpful during development. <br>这使 `debug_assert_eq!` 对于检查成本太高而无法在发行版本中进行检查，但在开发过程中可能很有用。<br>
///
/// The result of expanding `debug_assert_eq!` is always type checked. <br>扩展 `debug_assert_eq!` 的结果始终是类型检查的。<br>
///
/// # Examples
///
/// ```
/// let a = 3;
/// let b = 1 + 2;
/// debug_assert_eq!(a, b);
/// ```
///
///
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "debug_assert_eq_macro")]
macro_rules! debug_assert_eq {
    ($($arg:tt)*) => {
        if $crate::cfg!(debug_assertions) {
            $crate::assert_eq!($($arg)*);
        }
    };
}

/// Asserts that two expressions are not equal to each other. <br>断言两个表达式彼此不相等。<br>
///
/// On panic, this macro will print the values of the expressions with their debug representations. <br>在 panic 上，此宏将打印表达式的值及其调试表示。<br>
///
/// Unlike [`assert_ne!`], `debug_assert_ne!` statements are only enabled in non optimized builds by default. <br>与 [`assert_ne!`] 不同，默认情况下仅在未优化的构建中启用 `debug_assert_ne!` 语句。<br>
/// An optimized build will not execute `debug_assert_ne!` statements unless `-C debug-assertions` is passed to the compiler. <br>除非将 `-C debug-assertions` 传递给编译器，否则优化的构建将不执行 `debug_assert_ne!` 语句。<br>
/// This makes `debug_assert_ne!` useful for checks that are too expensive to be present in a release build but may be helpful during development. <br>这使 `debug_assert_ne!` 对于检查成本太高而无法在发行版本中进行检查，但在开发过程中可能很有用。<br>
///
/// The result of expanding `debug_assert_ne!` is always type checked. <br>扩展 `debug_assert_ne!` 的结果始终是类型检查的。<br>
///
/// # Examples
///
/// ```
/// let a = 3;
/// let b = 2;
/// debug_assert_ne!(a, b);
/// ```
///
///
#[macro_export]
#[stable(feature = "assert_ne", since = "1.13.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "debug_assert_ne_macro")]
macro_rules! debug_assert_ne {
    ($($arg:tt)*) => {
        if $crate::cfg!(debug_assertions) {
            $crate::assert_ne!($($arg)*);
        }
    };
}

/// Asserts that an expression matches any of the given patterns. <br>断言表达式匹配任何给定的模式。<br>
///
/// Like in a `match` expression, the pattern can be optionally followed by `if` and a guard expression that has access to names bound by the pattern. <br>像在 `match` 表达式中一样，可以在模式后跟 `if` 和可以访问由模式绑定的名称的保护表达式。<br>
///
/// On panic, this macro will print the value of the expression with its debug representation. <br>在 panic 时，这个宏将打印表达式的值及其调试表示。<br>
///
/// Unlike [`assert_matches!`], `debug_assert_matches!` statements are only enabled in non optimized builds by default. <br>与 [`assert_matches!`] 不同，`debug_assert_matches!` 语句默认仅在非优化构建中启用。<br>
/// An optimized build will not execute `debug_assert_matches!` statements unless `-C debug-assertions` is passed to the compiler. <br>除非将 `-C debug-assertions` 传递给编译器，否则优化的构建将不会执行 `debug_assert_matches!` 语句。<br>
/// This makes `debug_assert_matches!` useful for checks that are too expensive to be present in a release build but may be helpful during development. <br>这使得 `debug_assert_matches!` 可用于在发布版本中出现的检查成本太高，但在开发过程中可能会有所帮助。<br>
///
/// The result of expanding `debug_assert_matches!` is always type checked. <br>扩展 `debug_assert_matches!` 的结果总是经过类型检查。<br>
///
/// # Examples
///
/// ```
/// #![feature(assert_matches)]
///
/// use std::assert_matches::debug_assert_matches;
///
/// let a = 1u32.checked_add(2);
/// let b = 1u32.checked_sub(2);
/// debug_assert_matches!(a, Some(_));
/// debug_assert_matches!(b, None);
///
/// let c = Ok("abc".to_string());
/// debug_assert_matches!(c, Ok(x) | Err(x) if x.len() < 100);
/// ```
///
///
///
///
#[macro_export]
#[unstable(feature = "assert_matches", issue = "82775")]
#[allow_internal_unstable(assert_matches)]
#[rustc_macro_transparency = "semitransparent"]
pub macro debug_assert_matches($($arg:tt)*) {
    if $crate::cfg!(debug_assertions) {
        $crate::assert_matches::assert_matches!($($arg)*);
    }
}

/// Returns whether the given expression matches any of the given patterns. <br>返回给定表达式是否与任何给定模式匹配。<br>
///
/// Like in a `match` expression, the pattern can be optionally followed by `if` and a guard expression that has access to names bound by the pattern. <br>像在 `match` 表达式中一样，可以在模式后跟 `if` 和可以访问由模式绑定的名称的保护表达式。<br>
///
///
/// # Examples
///
/// ```
/// let foo = 'f';
/// assert!(matches!(foo, 'A'..='Z' | 'a'..='z'));
///
/// let bar = Some(4);
/// assert!(matches!(bar, Some(x) if x > 2));
/// ```
#[macro_export]
#[stable(feature = "matches_macro", since = "1.42.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "matches_macro")]
macro_rules! matches {
    ($expression:expr, $(|)? $( $pattern:pat_param )|+ $( if $guard: expr )? $(,)?) => {
        match $expression {
            $( $pattern )|+ $( if $guard )? => true,
            _ => false
        }
    };
}

/// Unwraps a result or propagates its error. <br>解开结果或传播其错误。<br>
///
/// The `?` operator was added to replace `try!` and should be used instead. <br>添加了 `?` 运算符来替换 `try!`，应该用它来代替。<br>
/// Furthermore, `try` is a reserved word in Rust 2018, so if you must use it, you will need to use the [raw-identifier syntax][ris]: `r#try`. <br>此外，`try` 是 Rust 2018 中的保留字，因此如果必须使用它，则需要使用 [raw-identifier syntax][ris]: `r#try`。<br>
///
///
/// [ris]: https://doc.rust-lang.org/nightly/rust-by-example/compatibility/raw_identifiers.html
///
/// `try!` matches the given [`Result`]. <br>`try!` 匹配给定的 [`Result`]。<br> In case of the `Ok` variant, the expression has the value of the wrapped value. <br>对于 `Ok` 变体，表达式具有包装值的值。<br>
///
/// In case of the `Err` variant, it retrieves the inner error. <br>对于 `Err` 变体，它检索内部错误。<br> `try!` then performs conversion using `From`. <br>`try!` 然后使用 `From` 执行转换。<br>
/// This provides automatic conversion between specialized errors and more general ones. <br>这样可以在特殊错误和更常见错误之间进行自动转换。<br>
/// The resulting error is then immediately returned. <br>然后立即返回产生的错误。<br>
///
/// Because of the early return, `try!` can only be used in functions that return [`Result`]. <br>由于提前返回，因此只能在返回 [`Result`] 的函数中使用 `try!`。<br>
///
/// # Examples
///
/// ```
/// use std::io;
/// use std::fs::File;
/// use std::io::prelude::*;
///
/// enum MyError {
///     FileWriteError
/// }
///
/// impl From<io::Error> for MyError {
///     fn from(e: io::Error) -> MyError {
///         MyError::FileWriteError
///     }
/// }
///
/// // The preferred method of quick returning Errors <br>快速返回错误的首选方法<br>
/// fn write_to_file_question() -> Result<(), MyError> {
///     let mut file = File::create("my_best_friends.txt")?;
///     file.write_all(b"This is a list of my best friends.")?;
///     Ok(())
/// }
///
/// // The previous method of quick returning Errors <br>快速返回错误的先前方法<br>
/// fn write_to_file_using_try() -> Result<(), MyError> {
///     let mut file = r#try!(File::create("my_best_friends.txt"));
///     r#try!(file.write_all(b"This is a list of my best friends."));
///     Ok(())
/// }
///
/// // This is equivalent to: <br>这相当于：<br>
/// fn write_to_file_using_match() -> Result<(), MyError> {
///     let mut file = r#try!(File::create("my_best_friends.txt"));
///     match file.write_all(b"This is a list of my best friends.") {
///         Ok(v) => v,
///         Err(e) => return Err(From::from(e)),
///     }
///     Ok(())
/// }
/// ```
///
///
///
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[deprecated(since = "1.39.0", note = "use the `?` operator instead")]
#[doc(alias = "?")]
macro_rules! r#try {
    ($expr:expr $(,)?) => {
        match $expr {
            $crate::result::Result::Ok(val) => val,
            $crate::result::Result::Err(err) => {
                return $crate::result::Result::Err($crate::convert::From::from(err));
            }
        }
    };
}

/// Writes formatted data into a buffer. <br>将格式化的数据写入缓冲区。<br>
///
/// This macro accepts a 'writer', a format string, and a list of arguments. <br>该宏接受 'writer'，格式字符串和参数列表。<br>
/// Arguments will be formatted according to the specified format string and the result will be passed to the writer. <br>参数将根据指定的格式字符串进行格式化，并将结果传递到 writer。<br>
/// The writer may be any value with a `write_fmt` method; <br>使用 `write_fmt` 方法时，writer 可以是任何值；<br> generally this comes from an implementation of either the [`fmt::Write`] or the [`io::Write`] trait. <br>通常，这来自 [`fmt::Write`] 或 [`io::Write`] trait 的实现。<br>
/// The macro returns whatever the `write_fmt` method returns; <br>宏返回 `write_fmt` 方法返回的任何内容；<br> commonly a [`fmt::Result`], or an [`io::Result`]. <br>通常是 [`fmt::Result`] 或 [`io::Result`]。<br>
///
/// See [`std::fmt`] for more information on the format string syntax. <br>有关格式字符串语法的更多信息，请参见 [`std::fmt`]。<br>
///
/// [`std::fmt`]: ../std/fmt/index.html
/// [`fmt::Write`]: crate::fmt::Write
/// [`io::Write`]: ../std/io/trait.Write.html
/// [`fmt::Result`]: crate::fmt::Result
/// [`io::Result`]: ../std/io/type.Result.html
///
/// # Examples
///
/// ```
/// use std::io::Write;
///
/// fn main() -> std::io::Result<()> {
///     let mut w = Vec::new();
///     write!(&mut w, "test")?;
///     write!(&mut w, "formatted {}", "arguments")?;
///
///     assert_eq!(w, b"testformatted arguments");
///     Ok(())
/// }
/// ```
///
/// A module can import both `std::fmt::Write` and `std::io::Write` and call `write!` on objects implementing either, as objects do not typically implement both. <br>模块可以同时在实现两者的对象上导入 `std::fmt::Write` 和 `std::io::Write` 以及调用 `write!`，因为对象通常不会同时实现两者。<br>
///
/// However, the module must import the traits qualified so their names do not conflict: <br>但是，该模块必须导入合格的 traits，以便其名称不会冲突：<br>
///
/// ```
/// use std::fmt::Write as FmtWrite;
/// use std::io::Write as IoWrite;
///
/// fn main() -> Result<(), Box<dyn std::error::Error>> {
///     let mut s = String::new();
///     let mut v = Vec::new();
///
///     write!(&mut s, "{} {}", "abc", 123)?; // uses fmt::Write::write_fmt <br>使用 fmt::Write::write_fmt<br>
///     write!(&mut v, "s = {:?}", s)?; // uses io::Write::write_fmt <br>使用 io::Write::write_fmt<br>
///     assert_eq!(v, b"s = \"abc 123\"");
///     Ok(())
/// }
/// ```
///
/// Note: This macro can be used in `no_std` setups as well. <br>该宏也可以在 `no_std` 设置中使用。<br>
/// In a `no_std` setup you are responsible for the implementation details of the components. <br>在 `no_std` 设置中，您负责组件的实现细节。<br>
///
/// ```no_run
/// # extern crate core;
/// use core::fmt::Write;
///
/// struct Example;
///
/// impl Write for Example {
///     fn write_str(&mut self, _s: &str) -> core::fmt::Result {
///          unimplemented!();
///     }
/// }
///
/// let mut m = Example{};
/// write!(&mut m, "Hello World").expect("Not written");
/// ```
///
///
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "write_macro")]
macro_rules! write {
    ($dst:expr, $($arg:tt)*) => {
        $dst.write_fmt($crate::format_args!($($arg)*))
    };
}

/// Write formatted data into a buffer, with a newline appended. <br>将格式化的数据写入到缓冲区，并追加一个换行符。<br>
///
/// On all platforms, the newline is the LINE FEED character (`\n`/`U+000A`) alone (no additional CARRIAGE RETURN (`\r`/`U+000D`). <br>在所有平台上，新行是一个换行符 (也就是 `\n`/`U+000A`)，并不包含回车符 (也就是 `\r`/`U+000D`)。<br>
///
/// For more information, see [`write!`]. <br>有关更多信息，请参见 [`write!`]。<br> For information on the format string syntax, see [`std::fmt`]. <br>有关格式字符串语法的信息，请参见 [`std::fmt`]。<br>
///
/// [`std::fmt`]: ../std/fmt/index.html
///
/// # Examples
///
/// ```
/// use std::io::{Write, Result};
///
/// fn main() -> Result<()> {
///     let mut w = Vec::new();
///     writeln!(&mut w)?;
///     writeln!(&mut w, "test")?;
///     writeln!(&mut w, "formatted {}", "arguments")?;
///
///     assert_eq!(&w[..], "\ntest\nformatted arguments\n".as_bytes());
///     Ok(())
/// }
/// ```
///
/// A module can import both `std::fmt::Write` and `std::io::Write` and call `write!` on objects implementing either, as objects do not typically implement both. <br>模块可以同时在实现两者的对象上导入 `std::fmt::Write` 和 `std::io::Write` 以及调用 `write!`，因为对象通常不会同时实现两者。<br>
/// However, the module must import the traits qualified so their names do not conflict: <br>但是，该模块必须导入合格的 traits，以便其名称不会冲突：<br>
///
/// ```
/// use std::fmt::Write as FmtWrite;
/// use std::io::Write as IoWrite;
///
/// fn main() -> Result<(), Box<dyn std::error::Error>> {
///     let mut s = String::new();
///     let mut v = Vec::new();
///
///     writeln!(&mut s, "{} {}", "abc", 123)?; // uses fmt::Write::write_fmt <br>使用 fmt::Write::write_fmt<br>
///     writeln!(&mut v, "s = {:?}", s)?; // uses io::Write::write_fmt <br>使用 io::Write::write_fmt<br>
///     assert_eq!(v, b"s = \"abc 123\\n\"\n");
///     Ok(())
/// }
/// ```
///
///
///
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "writeln_macro")]
#[allow_internal_unstable(format_args_nl)]
macro_rules! writeln {
    ($dst:expr $(,)?) => {
        $crate::write!($dst, "\n")
    };
    ($dst:expr, $($arg:tt)*) => {
        $dst.write_fmt($crate::format_args_nl!($($arg)*))
    };
}

/// Indicates unreachable code. <br>表示无法访问的代码。<br>
///
/// This is useful any time that the compiler can't determine that some code is unreachable. <br>每当编译器无法确定某些代码不可访问时，此功能就很有用。<br> For example: <br>例如：<br>
///
/// * Match arms with guard conditions. <br>让分支与守卫条件匹配。<br>
/// * Loops that dynamically terminate. <br>动态终止的循环。<br>
/// * Iterators that dynamically terminate. <br>动态终止的迭代器。<br>
///
/// If the determination that the code is unreachable proves incorrect, the program immediately terminates with a [`panic!`]. <br>如果确定代码不可访问不正确，则程序立即以 [`panic!`] 终止。<br>
///
/// The unsafe counterpart of this macro is the [`unreachable_unchecked`] function, which will cause undefined behavior if the code is reached. <br>[`unreachable_unchecked`] 函数是该宏中不安全的副本，如果到达代码，它将导致未定义的行为。<br>
///
///
/// [`unreachable_unchecked`]: crate::hint::unreachable_unchecked
///
/// # Panics
///
/// This will always [`panic!`] because `unreachable!` is just a shorthand for `panic!` with a fixed, specific message. <br>这将始终为 [`panic!`]，因为 `unreachable!` 只是 `panic!` 的简写，带有固定的、特定的消息。<br>
///
/// Like `panic!`, this macro has a second form for displaying custom values. <br>像 `panic!` 一样，此宏具有用于显示自定义值的第二种形式。<br>
///
/// # Examples
///
/// Match arms: <br>match 分支：<br>
///
/// ```
/// # #[allow(dead_code)]
/// fn foo(x: Option<i32>) {
///     match x {
///         Some(n) if n >= 0 => println!("Some(Non-negative)"),
///         Some(n) if n <  0 => println!("Some(Negative)"),
///         Some(_)           => unreachable!(), // compile error if commented out <br>如果注释掉，就会编译错误<br>
///         None              => println!("None")
///     }
/// }
/// ```
///
/// Iterators:
///
/// ```
/// # #[allow(dead_code)]
/// fn divide_by_three(x: u32) -> u32 { // one of the poorest implementations of x/3 <br>x/3 最差的实现之一<br>
///     for i in 0.. {
///         if 3*i < i { panic!("u32 overflow"); }
///         if x < 3*i { return i-1; }
///     }
///     unreachable!("The loop should always return");
/// }
/// ```
///
///
///
#[macro_export]
#[rustc_builtin_macro(unreachable)]
#[allow_internal_unstable(edition_panic)]
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "unreachable_macro")]
macro_rules! unreachable {
    // Expands to either `$crate::panic::unreachable_2015` or `$crate::panic::unreachable_2021` depending on the edition of the caller. <br>根据调用者的版本扩展到 `$crate::panic::unreachable_2015` 或 `$crate::panic::unreachable_2021`。<br>
    //
    ($($arg:tt)*) => {
        /* compiler built-in */
    };
}

/// Indicates unimplemented code by panicking with a message of "not implemented". <br>通过 panic 并带有 "not implemented" 的消息来指示未实现的代码。<br>
///
/// This allows your code to type-check, which is useful if you are prototyping or implementing a trait that requires multiple methods which you don't plan to use all of. <br>这允许您的代码进行类型检查，如果您正在设计原型或实现需要多个您不打算使用所有方法的特征，这将非常有用。<br>
///
/// The difference between `unimplemented!` and [`todo!`] is that while `todo!` conveys an intent of implementing the functionality later and the message is "not yet implemented", `unimplemented!` makes no such claims. <br>`unimplemented!` 和 [`todo!`] 之间的区别在于，尽管 `todo!` 传达了稍后实现该功能的意图，并且消息为 "not yet implemented"，但 `unimplemented!` 并未提出任何此类声明。<br>
/// Its message is "not implemented". <br>它的消息是 "not implemented"。<br>
/// Also some IDEs will mark `todo!`s. <br>还有一些 IDE 会标记 `todo!`。<br>
///
/// # Panics
///
/// This will always [`panic!`] because `unimplemented!` is just a shorthand for `panic!` with a fixed, specific message. <br>这将始终是 [`panic!`]，因为 `unimplemented!` 只是 `panic!` 的简写，带有固定的特定消息。<br>
///
/// Like `panic!`, this macro has a second form for displaying custom values. <br>像 `panic!` 一样，此宏具有用于显示自定义值的第二种形式。<br>
///
/// [`todo!`]: crate::todo
///
/// # Examples
///
/// Say we have a trait `Foo`: <br>假设我们有一个 `Foo` trait：<br>
///
/// ```
/// trait Foo {
///     fn bar(&self) -> u8;
///     fn baz(&self);
///     fn qux(&self) -> Result<u64, ()>;
/// }
/// ```
///
/// We want to implement `Foo` for 'MyStruct', but for some reason it only makes sense to implement the `bar()` function. <br>我们想为 'MyStruct' 实现 `Foo`，但是由于某些原因，只有实现 `bar()` 函数才有意义。<br>
/// `baz()` and `qux()` will still need to be defined in our implementation of `Foo`, but we can use `unimplemented!` in their definitions to allow our code to compile. <br>`baz()` 和 `qux()` 仍然需要在我们的 `Foo` 实现中定义，但我们可以在它们的定义中使用 `unimplemented!` 来允许我们的代码编译。<br>
///
/// We still want to have our program stop running if the unimplemented methods are reached. <br>如果达到未实现的方法，我们仍然希望程序停止运行。<br>
///
/// ```
/// # trait Foo {
/// #     fn bar(&self) -> u8;
/// #     fn baz(&self);
/// #     fn qux(&self) -> Result<u64, ()>;
/// # }
/// struct MyStruct;
///
/// impl Foo for MyStruct {
///     fn bar(&self) -> u8 {
///         1 + 1
///     }
///
///     fn baz(&self) {
///         // It makes no sense to `baz` a `MyStruct`, so we have no logic here at all. <br>`baz` 和 `MyStruct` 没有任何意义，因此我们完全没有逻辑。<br>
/////
///         // This will display "thread 'main' panicked at 'not implemented'". <br>这将显示 "thread 'main' panicked at 'not implemented'"。<br>
///         unimplemented!();
///     }
///
///     fn qux(&self) -> Result<u64, ()> {
///         // We have some logic here, We can add a message to unimplemented! <br>我们这里有一些逻辑，我们可以向未实现中添加一条消息！<br> to display our omission. <br>显示我们的遗漏。<br>
///         // This will display: "thread 'main' panicked at 'not implemented: MyStruct isn't quxable'". <br>这将显示: "thread 'main' panicked at 'not implemented: MyStruct isn't quxable'"。<br>
/////
/////
///         unimplemented!("MyStruct isn't quxable");
///     }
/// }
///
/// fn main() {
///     let s = MyStruct;
///     s.bar();
/// }
/// ```
///
///
///
///
///
///
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "unimplemented_macro")]
#[allow_internal_unstable(core_panic)]
macro_rules! unimplemented {
    () => {
        $crate::panicking::panic("not implemented")
    };
    ($($arg:tt)+) => {
        $crate::panic!("not implemented: {}", $crate::format_args!($($arg)+))
    };
}

/// Indicates unfinished code. <br>表示未完成的代码。<br>
///
/// This can be useful if you are prototyping and are just looking to have your code typecheck. <br>如果您要进行原型设计并且只是在检查代码类型，这可能会很有用。<br>
///
/// The difference between [`unimplemented!`] and `todo!` is that while `todo!` conveys an intent of implementing the functionality later and the message is "not yet implemented", `unimplemented!` makes no such claims. <br>[`unimplemented!`] 和 `todo!` 之间的区别在于，尽管 `todo!` 传达了稍后实现该功能的意图，并且消息为 "not yet implemented"，但 `unimplemented!` 并未提出任何此类声明。<br>
/// Its message is "not implemented". <br>它的消息是 "not implemented"。<br>
/// Also some IDEs will mark `todo!`s. <br>还有一些 IDE 会标记 `todo!`。<br>
///
/// # Panics
///
/// This will always [`panic!`]. <br>这将始终为 [`panic!`]。<br>
///
/// # Examples
///
/// Here's an example of some in-progress code. <br>这是一些正在进行的代码的示例。<br> We have a trait `Foo`: <br>我们有一个 `Foo` trait：<br>
///
/// ```
/// trait Foo {
///     fn bar(&self);
///     fn baz(&self);
/// }
/// ```
///
/// We want to implement `Foo` on one of our types, but we also want to work on just `bar()` first. <br>我们想在其中一种类型上实现 `Foo`，但我们也想首先仅在 `bar()` 上工作。<br> In order for our code to compile, we need to implement `baz()`, so we can use `todo!`: <br>为了编译我们的代码，我们需要实现 `baz()`，因此我们可以使用 `todo!`：<br>
///
/// ```
/// # trait Foo {
/// #     fn bar(&self);
/// #     fn baz(&self);
/// # }
/// struct MyStruct;
///
/// impl Foo for MyStruct {
///     fn bar(&self) {
///         // implementation goes here <br>实现在这里<br>
///     }
///
///     fn baz(&self) {
///         // let's not worry about implementing baz() for now <br>让我们现在不必担心实现 baz()<br>
///         todo!();
///     }
/// }
///
/// fn main() {
///     let s = MyStruct;
///     s.bar();
///
///     // we aren't even using baz(), so this is fine. <br>我们甚至没有使用 baz()，所以很好。<br>
/// }
/// ```
///
///
///
///
#[macro_export]
#[stable(feature = "todo_macro", since = "1.40.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "todo_macro")]
#[allow_internal_unstable(core_panic)]
macro_rules! todo {
    () => {
        $crate::panicking::panic("not yet implemented")
    };
    ($($arg:tt)+) => {
        $crate::panic!("not yet implemented: {}", $crate::format_args!($($arg)+))
    };
}

/// Definitions of built-in macros. <br>内置宏的定义。<br>
///
/// Most of the macro properties (stability, visibility, etc.) are taken from the source code here, with exception of expansion functions transforming macro inputs into outputs, those functions are provided by the compiler. <br>宏的大多数属性 (稳定性，可见性等) 均来自此处的源代码，除了将宏输入转换为输出的扩展函数外，这些函数由编译器提供。<br>
///
///
pub(crate) mod builtin {

    /// Causes compilation to fail with the given error message when encountered. <br>导致编译失败，并遇到给定的错误消息。<br>
    ///
    /// This macro should be used when a crate uses a conditional compilation strategy to provide better error messages for erroneous conditions. <br>当 crate 使用条件编译策略为错误条件提供更好的错误消息时，应使用此宏。<br>
    ///
    /// It's the compiler-level form of [`panic!`], but emits an error during *compilation* rather than at *runtime*. <br>它是 [`panic!`] 的编译器级别的形式，但在 *编译* 而不是 *运行时* 会发出错误。<br>
    ///
    /// # Examples
    ///
    /// Two such examples are macros and `#[cfg]` environments. <br>宏和 `#[cfg]` 环境就是两个这样的示例。<br>
    ///
    /// Emit a better compiler error if a macro is passed invalid values. <br>如果宏传递了无效值，则发出更好的编译器错误。<br>
    /// Without the final branch, the compiler would still emit an error, but the error's message would not mention the two valid values. <br>没有最终分支，编译器仍然会发出错误，但是错误消息不会提及两个有效值。<br>
    ///
    /// ```compile_fail
    /// macro_rules! give_me_foo_or_bar {
    ///     (foo) => {};
    ///     (bar) => {};
    ///     ($x:ident) => {
    ///         compile_error!("This macro only accepts `foo` or `bar`");
    ///     }
    /// }
    ///
    /// give_me_foo_or_bar!(neither);
    /// // ^ will fail at compile time with message "This macro only accepts `foo` or `bar`" <br>将在编译时失败，并显示消息 "This macro only accepts `foo` or `bar`"<br>
    /// ```
    ///
    /// Emit a compiler error if one of a number of features isn't available. <br>如果许多，特性，之一不可用，则发出编译器错误。<br>
    ///
    /// ```compile_fail
    /// #[cfg(not(any(feature = "foo", feature = "bar")))]
    /// compile_error!("Either feature \"foo\" or \"bar\" must be enabled for this crate.");
    /// ```
    ///
    #[stable(feature = "compile_error_macro", since = "1.20.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "compile_error_macro")]
    macro_rules! compile_error {
        ($msg:expr $(,)?) => {{ /* compiler built-in */ }};
    }

    /// Constructs parameters for the other string-formatting macros. <br>构造其他字符串格式宏的参数。<br>
    ///
    /// This macro functions by taking a formatting string literal containing `{}` for each additional argument passed. <br>此宏函数通过为每个传递的其他参数采用包含 `{}` 的格式字符串字面量来实现。<br>
    /// `format_args!` prepares the additional parameters to ensure the output can be interpreted as a string and canonicalizes the arguments into a single type. <br>`format_args!` 准备附加参数以确保输出可以解释为字符串，并将参数规范化为单一类型。<br>
    /// Any value that implements the [`Display`] trait can be passed to `format_args!`, as can any [`Debug`] implementation be passed to a `{:?}` within the formatting string. <br>可以将实现 [`Display`] trait 的任何值传递给 `format_args!`，也可以将任何 [`Debug`] 实现的形式传递给格式化字符串中的 `{:?}`。<br>
    ///
    ///
    /// This macro produces a value of type [`fmt::Arguments`]. <br>该宏产生 [`fmt::Arguments`] 类型的值。<br> This value can be passed to the macros within [`std::fmt`] for performing useful redirection. <br>可以将该值传递到 [`std::fmt`] 中的宏，以执行有用的重定向。<br>
    /// All other formatting macros ([`format!`], [`write!`], [`println!`], etc) are proxied through this one. <br>所有其他格式化宏 ([`format!`]，[`write!`]，[`println!`] 等) 都通过此代理。<br>
    /// `format_args!`, unlike its derived macros, avoids heap allocations. <br>`format_args!` 与其派生的宏不同，它避免了堆分配。<br>
    ///
    /// You can use the [`fmt::Arguments`] value that `format_args!` returns in `Debug` and `Display` contexts as seen below. <br>您可以使用 `format_args!` 在 `Debug` 和 `Display` 上下文中返回的 [`fmt::Arguments`] 值，如下所示。<br>
    /// The example also shows that `Debug` and `Display` format to the same thing: the interpolated format string in `format_args!`. <br>该示例还显示 `Debug` 和 `Display` 的格式相同: `format_args!` 中的插值格式字符串。<br>
    ///
    /// ```rust
    /// let debug = format!("{:?}", format_args!("{} foo {:?}", 1, 2));
    /// let display = format!("{}", format_args!("{} foo {:?}", 1, 2));
    /// assert_eq!("1 foo 2", display);
    /// assert_eq!(display, debug);
    /// ```
    ///
    /// For more information, see the documentation in [`std::fmt`]. <br>有关更多信息，请参见 [`std::fmt`] 中的文档。<br>
    ///
    /// [`Display`]: crate::fmt::Display
    /// [`Debug`]: crate::fmt::Debug
    /// [`fmt::Arguments`]: crate::fmt::Arguments
    /// [`std::fmt`]: ../std/fmt/index.html
    /// [`format!`]: ../std/macro.format.html
    /// [`println!`]: ../std/macro.println.html
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fmt;
    ///
    /// let s = fmt::format(format_args!("hello {}", "world"));
    /// assert_eq!(s, format!("hello {}", "world"));
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[cfg_attr(not(test), rustc_diagnostic_item = "format_args_macro")]
    #[allow_internal_unsafe]
    #[allow_internal_unstable(fmt_internals)]
    #[rustc_builtin_macro]
    #[macro_export]
    macro_rules! format_args {
        ($fmt:expr) => {{ /* compiler built-in */ }};
        ($fmt:expr, $($args:tt)*) => {{ /* compiler built-in */ }};
    }

    /// Same as [`format_args`], but can be used in some const contexts. <br>与 [`format_args`] 相同，但可以在某些 const 上下文中使用。<br>
    ///
    /// This macro is used by the panic macros for the `const_panic` feature. <br>这个宏被 panic 宏用于 `const_panic` 特性。<br>
    ///
    /// This macro will be removed once `format_args` is allowed in const contexts. <br>一旦在 const 上下文中允许 `format_args`，这个宏将被删除。<br>
    #[unstable(feature = "const_format_args", issue = "none")]
    #[allow_internal_unstable(fmt_internals, const_fmt_arguments_new)]
    #[rustc_builtin_macro]
    #[macro_export]
    macro_rules! const_format_args {
        ($fmt:expr) => {{ /* compiler built-in */ }};
        ($fmt:expr, $($args:tt)*) => {{ /* compiler built-in */ }};
    }

    /// Same as [`format_args`], but adds a newline in the end. <br>与 [`format_args`] 相同，但最后添加了一个换行符。<br>
    #[unstable(
        feature = "format_args_nl",
        issue = "none",
        reason = "`format_args_nl` is only for internal \
                  language use and is subject to change"
    )]
    #[allow_internal_unstable(fmt_internals)]
    #[rustc_builtin_macro]
    #[macro_export]
    macro_rules! format_args_nl {
        ($fmt:expr) => {{ /* compiler built-in */ }};
        ($fmt:expr, $($args:tt)*) => {{ /* compiler built-in */ }};
    }

    /// Inspects an environment variable at compile time. <br>在编译时检查环境变量。<br>
    ///
    /// This macro will expand to the value of the named environment variable at compile time, yielding an expression of type `&'static str`. <br>该宏将在编译时扩展为指定的环境变量的值，从而产生 `&'static str` 类型的表达式。<br>
    ///
    /// Use [`std::env::var`] instead if you want to read the value at runtime. <br>如果要在运行时读取值，请改用 [`std::env::var`]。<br>
    ///
    /// [`std::env::var`]: ../std/env/fn.var.html
    ///
    /// If the environment variable is not defined, then a compilation error will be emitted. <br>如果未定义环境变量，则将发出编译错误。<br>
    /// To not emit a compile error, use the [`option_env!`] macro instead. <br>为了不产生编译错误，请改用 [`option_env!`] 宏。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let path: &'static str = env!("PATH");
    /// println!("the $PATH variable at the time of compiling was: {path}");
    /// ```
    ///
    /// You can customize the error message by passing a string as the second parameter: <br>您可以通过将字符串作为第二个参数传递来自定义错误消息：<br>
    ///
    /// ```compile_fail
    /// let doc: &'static str = env!("documentation", "what's that?!");
    /// ```
    ///
    /// If the `documentation` environment variable is not defined, you'll get the following error: <br>如果未定义 `documentation` 环境变量，则会出现以下错误：<br>
    ///
    /// ```text
    /// error: what's that?!
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "env_macro")]
    macro_rules! env {
        ($name:expr $(,)?) => {{ /* compiler built-in */ }};
        ($name:expr, $error_msg:expr $(,)?) => {{ /* compiler built-in */ }};
    }

    /// Optionally inspects an environment variable at compile time. <br>(可选) 在编译时检查环境变量。<br>
    ///
    /// If the named environment variable is present at compile time, this will expand into an expression of type `Option<&'static str>` whose value is `Some` of the value of the environment variable. <br>如果在编译时存在指定的环境变量，它将扩展为 `Option<&'static str>` 类型的表达式，其值是环境变量的值的 `Some`。<br>
    /// If the environment variable is not present, then this will expand to `None`. <br>如果不存在环境变量，则它将扩展为 `None`。<br>
    /// See [`Option<T>`][Option] for more information on this type. <br>有关此类型的更多信息，请参见 [`Option<T>`][Option]。<br>
    /// Use [`std::env::var`] instead if you want to read the value at runtime. <br>如果要在运行时读取值，请改用 [`std::env::var`]。<br>
    ///
    /// [`std::env::var`]: ../std/env/fn.var.html
    ///
    /// A compile time error is never emitted when using this macro regardless of whether the environment variable is present or not. <br>使用此宏时，无论是否存在环境变量，都不会发出编译时错误。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let key: Option<&'static str> = option_env!("SECRET_KEY");
    /// println!("the secret key might be: {key:?}");
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "option_env_macro")]
    macro_rules! option_env {
        ($name:expr $(,)?) => {{ /* compiler built-in */ }};
    }

    /// Concatenates identifiers into one identifier. <br>将标识符串联为一个标识符。<br>
    ///
    /// This macro takes any number of comma-separated identifiers, and concatenates them all into one, yielding an expression which is a new identifier. <br>该宏采用任意数量的逗号分隔的标识符，并将它们全部连接为一个，从而产生一个表达式，该表达式是一个新的标识符。<br>
    /// Note that hygiene makes it such that this macro cannot capture local variables. <br>请注意，卫生使该宏无法捕获本地变量。<br>
    /// Also, as a general rule, macros are only allowed in item, statement or expression position. <br>同样，作为一般规则，只允许在项，语句或表达式位置使用宏。<br>
    /// That means while you may use this macro for referring to existing variables, functions or modules etc, you cannot define a new one with it. <br>这意味着尽管您可以使用此宏来引用现有的变量，函数或模块等，但是您无法使用它来定义一个新的宏。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(concat_idents)]
    ///
    /// # fn main() {
    /// fn foobar() -> u32 { 23 }
    ///
    /// let f = concat_idents!(foo, bar);
    /// println!("{}", f());
    ///
    /// // fn concat_idents!(new, fun, name) { } // not usable in this way! <br>fn concat_idents!(new, fun, name) { } // 无法以这种方式使用！<br>
    /// # }
    /// ```
    ///
    ///
    ///
    #[unstable(
        feature = "concat_idents",
        issue = "29599",
        reason = "`concat_idents` is not stable enough for use and is subject to change"
    )]
    #[rustc_builtin_macro]
    #[macro_export]
    macro_rules! concat_idents {
        ($($e:ident),+ $(,)?) => {{ /* compiler built-in */ }};
    }

    /// Concatenates literals into a byte slice. <br>将字面量连接成字节切片。<br>
    ///
    /// This macro takes any number of comma-separated literals, and concatenates them all into one, yielding an expression of type `&[u8, _]`, which represents all of the literals concatenated left-to-right. <br>这个宏接受任意数量的逗号分隔的字面量，并将它们全部连接成一个，产生一个类型为 `&[u8, _]` 的表达式，它代表从左到右连接的所有字面量。<br>
    /// The literals passed can be any combination of: <br>传递的字面量可以是以下任意组合：<br>
    ///
    /// - byte literals (`b'r'`) <br>字节字面量 (`b'r'`)<br>
    /// - byte strings (`b"Rust"`) <br>字节字符串 (`b"Rust"`)<br>
    /// - arrays of bytes/numbers (`[b'A', 66, b'C']`) <br>由字节或数字组成的数组 (`[b'A', 66, b'C']`)<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(concat_bytes)]
    ///
    /// # fn main() {
    /// let s: &[u8; 6] = concat_bytes!(b'A', b"BC", [68, b'E', 70]);
    /// assert_eq!(s, b"ABCDEF");
    /// # }
    /// ```
    ///
    #[unstable(feature = "concat_bytes", issue = "87555")]
    #[rustc_builtin_macro]
    #[macro_export]
    macro_rules! concat_bytes {
        ($($e:literal),+ $(,)?) => {{ /* compiler built-in */ }};
    }

    /// Concatenates literals into a static string slice. <br>将字面量串联成一个静态字符串切片。<br>
    ///
    /// This macro takes any number of comma-separated literals, yielding an expression of type `&'static str` which represents all of the literals concatenated left-to-right. <br>该宏采用任意数量的逗号分隔的字面量，产生 `&'static str` 类型的表达式，该表达式表示所有从左到右串联的字面量。<br>
    ///
    ///
    /// Integer and floating point literals are stringified in order to be concatenated. <br>将整数和浮点字面量进行字符串化以将其串联在一起。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let s = concat!("test", 10, 'b', true);
    /// assert_eq!(s, "test10btrue");
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "concat_macro")]
    macro_rules! concat {
        ($($e:expr),* $(,)?) => {{ /* compiler built-in */ }};
    }

    /// Expands to the line number on which it was invoked. <br>扩展为在其上被调用的行号。<br>
    ///
    /// With [`column!`] and [`file!`], these macros provide debugging information for developers about the location within the source. <br>对于 [`column!`] 和 [`file!`]，这些宏为开发人员提供了有关源中位置的调试信息。<br>
    ///
    /// The expanded expression has type `u32` and is 1-based, so the first line in each file evaluates to 1, the second to 2, etc. <br>扩展表达式的类型为 `u32`，基于 1，因此每个文件的第一行求值为 1，第二行求值为 2，依此类推。<br>
    /// This is consistent with error messages by common compilers or popular editors. <br>这与常见编译器或常用编辑器的错误消息一致。<br>
    /// The returned line is *not necessarily* the line of the `line!` invocation itself, but rather the first macro invocation leading up to the invocation of the `line!` macro. <br>返回的行必定是 *not*`line!` 调用本身的行，而是导致 `line!` 宏调用的第一个宏调用。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let current_line = line!();
    /// println!("defined on line: {current_line}");
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "line_macro")]
    macro_rules! line {
        () => {
            /* compiler built-in */
        };
    }

    /// Expands to the column number at which it was invoked. <br>扩展到调用它的列号。<br>
    ///
    /// With [`line!`] and [`file!`], these macros provide debugging information for developers about the location within the source. <br>对于 [`line!`] 和 [`file!`]，这些宏为开发人员提供了有关源中位置的调试信息。<br>
    ///
    /// The expanded expression has type `u32` and is 1-based, so the first column in each line evaluates to 1, the second to 2, etc. <br>扩展表达式的类型为 `u32`，并且基于 1，因此每行的第一列的值为 1，第二列的值为 2，依此类推。<br>
    /// This is consistent with error messages by common compilers or popular editors. <br>这与常见编译器或常用编辑器的错误消息一致。<br>
    /// The returned column is *not necessarily* the line of the `column!` invocation itself, but rather the first macro invocation leading up to the invocation of the `column!` macro. <br>返回的列是 *not 必然*`column!` 调用本身的行，而是导致 `column!` 宏调用的第一个宏调用。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let current_col = column!();
    /// println!("defined on column: {current_col}");
    /// ```
    ///
    /// `column!` counts Unicode code points, not bytes or graphemes. <br>`column!` 计算 Unicode 代码点，而不是字节或字素。<br> As a result, the first two invocations return the same value, but the third does not. <br>作为结果，前两次调用返回相同的值，但第三次调用没有。<br>
    ///
    /// ```
    /// let a = ("foobar", column!()).1;
    /// let b = ("人之初性本善", column!()).1;
    /// let c = ("f̅o̅o̅b̅a̅r̅", column!()).1; // Uses combining overline (U+0305) <br>使用组合上划线 (U+0305)<br>
    ///
    /// assert_eq!(a, b);
    /// assert_ne!(b, c);
    /// ```
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "column_macro")]
    macro_rules! column {
        () => {
            /* compiler built-in */
        };
    }

    /// Expands to the file name in which it was invoked. <br>扩展为调用该文件的文件名。<br>
    ///
    /// With [`line!`] and [`column!`], these macros provide debugging information for developers about the location within the source. <br>对于 [`line!`] 和 [`column!`]，这些宏为开发人员提供了有关源中位置的调试信息。<br>
    ///
    /// The expanded expression has type `&'static str`, and the returned file is not the invocation of the `file!` macro itself, but rather the first macro invocation leading up to the invocation of the `file!` macro. <br>扩展表达式的类型为 `&'static str`，返回的文件不是 `file!` 宏本身的调用，而是导致 `file!` 宏调用的第一个宏调用。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let this_file = file!();
    /// println!("defined in file: {this_file}");
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "file_macro")]
    macro_rules! file {
        () => {
            /* compiler built-in */
        };
    }

    /// Stringifies its arguments. <br>对其参数进行字符串化。<br>
    ///
    /// This macro will yield an expression of type `&'static str` which is the stringification of all the tokens passed to the macro. <br>该宏将产生 `&'static str` 类型的表达式，该表达式是传递给该宏的所有 tokens 的字符串化。<br>
    /// No restrictions are placed on the syntax of the macro invocation itself. <br>宏调用本身的语法没有任何限制。<br>
    ///
    /// Note that the expanded results of the input tokens may change in the future. <br>请注意，输入 tokens 的扩展结果可能会在 future 中发生变化。<br> You should be careful if you rely on the output. <br>如果您依赖输出，则应格外小心。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let one_plus_one = stringify!(1 + 1);
    /// assert_eq!(one_plus_one, "1 + 1");
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "stringify_macro")]
    macro_rules! stringify {
        ($($t:tt)*) => {
            /* compiler built-in */
        };
    }

    /// Includes a UTF-8 encoded file as a string. <br>包含 UTF-8 编码的文件作为字符串。<br>
    ///
    /// The file is located relative to the current file (similarly to how modules are found). <br>该文件相对于当前文件位于 (类似于查找模块的方式)。<br>
    /// The provided path is interpreted in a platform-specific way at compile time. <br>提供的路径在编译时以特定于平台的方式进行解释。<br>
    /// So, for instance, an invocation with a Windows path containing backslashes `\` would not compile correctly on Unix. <br>因此，例如，使用 Windows 路径包含反斜杠 `\` 的调用将无法在 Unix 上正确编译。<br>
    ///
    ///
    /// This macro will yield an expression of type `&'static str` which is the contents of the file. <br>该宏将产生 `&'static str` 类型的表达式，该表达式是文件的内容。<br>
    ///
    /// # Examples
    ///
    /// Assume there are two files in the same directory with the following contents: <br>假设在同一目录中有两个文件，其内容如下：<br>
    ///
    /// File 'spanish.in': <br>文件 'spanish.in'：<br>
    ///
    /// ```text
    /// adiós
    /// ```
    ///
    /// File 'main.rs': <br>文件 'main.rs'：<br>
    ///
    /// ```ignore (cannot-doctest-external-file-dependency)
    /// fn main() {
    ///     let my_str = include_str!("spanish.in");
    ///     assert_eq!(my_str, "adiós\n");
    ///     print!("{my_str}");
    /// }
    /// ```
    ///
    /// Compiling 'main.rs' and running the resulting binary will print "adiós". <br>编译 'main.rs' 并运行生成的二进制文件将打印 "adiós"。<br>
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "include_str_macro")]
    macro_rules! include_str {
        ($file:expr $(,)?) => {{ /* compiler built-in */ }};
    }

    /// Includes a file as a reference to a byte array. <br>包含一个文件作为对字节数组的引用。<br>
    ///
    /// The file is located relative to the current file (similarly to how modules are found). <br>该文件相对于当前文件位于 (类似于查找模块的方式)。<br>
    /// The provided path is interpreted in a platform-specific way at compile time. <br>提供的路径在编译时以特定于平台的方式进行解释。<br>
    /// So, for instance, an invocation with a Windows path containing backslashes `\` would not compile correctly on Unix. <br>因此，例如，使用 Windows 路径包含反斜杠 `\` 的调用将无法在 Unix 上正确编译。<br>
    ///
    ///
    /// This macro will yield an expression of type `&'static [u8; N]` which is the contents of the file. <br>该宏将产生 `&'static [u8; N]` 类型的表达式，该表达式是文件的内容。<br>
    ///
    /// # Examples
    ///
    /// Assume there are two files in the same directory with the following contents: <br>假设在同一目录中有两个文件，其内容如下：<br>
    ///
    /// File 'spanish.in': <br>文件 'spanish.in'：<br>
    ///
    /// ```text
    /// adiós
    /// ```
    ///
    /// File 'main.rs': <br>文件 'main.rs'：<br>
    ///
    /// ```ignore (cannot-doctest-external-file-dependency)
    /// fn main() {
    ///     let bytes = include_bytes!("spanish.in");
    ///     assert_eq!(bytes, b"adi\xc3\xb3s\n");
    ///     print!("{}", String::from_utf8_lossy(bytes));
    /// }
    /// ```
    ///
    /// Compiling 'main.rs' and running the resulting binary will print "adiós". <br>编译 'main.rs' 并运行生成的二进制文件将打印 "adiós"。<br>
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "include_bytes_macro")]
    macro_rules! include_bytes {
        ($file:expr $(,)?) => {{ /* compiler built-in */ }};
    }

    /// Expands to a string that represents the current module path. <br>扩展为代表当前模块路径的字符串。<br>
    ///
    /// The current module path can be thought of as the hierarchy of modules leading back up to the crate root. <br>当前模块路径可以被认为是引回到 crate root 的模块层次结构。<br>
    /// The first component of the path returned is the name of the crate currently being compiled. <br>返回路径的第一部分是当前正在编译的 crate 的名称。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// mod test {
    ///     pub fn foo() {
    ///         assert!(module_path!().ends_with("test"));
    ///     }
    /// }
    ///
    /// test::foo();
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "module_path_macro")]
    macro_rules! module_path {
        () => {
            /* compiler built-in */
        };
    }

    /// Evaluates boolean combinations of configuration flags at compile-time. <br>在编译时评估配置标志的布尔组合。<br>
    ///
    /// In addition to the `#[cfg]` attribute, this macro is provided to allow boolean expression evaluation of configuration flags. <br>除了 `#[cfg]` 属性，还提供了此宏，以允许对配置标志进行布尔表达式评估。<br>
    /// This frequently leads to less duplicated code. <br>这通常会减少重复的代码。<br>
    ///
    /// The syntax given to this macro is the same syntax as the [`cfg`] attribute. <br>该宏的语法与 [`cfg`] 属性的语法相同。<br>
    ///
    /// `cfg!`, unlike `#[cfg]`, does not remove any code and only evaluates to true or false. <br>`cfg!` 与 `#[cfg]` 不同，它不会删除任何代码，只会评估为 true 或 false。<br>
    /// For example, all blocks in an if/else expression need to be valid when `cfg!` is used for the condition, regardless of what `cfg!` is evaluating. <br>例如，当将 `cfg!` 用作条件时，无论 `cfg!` 正在评估什么，if/else 表达式中的所有块都必须有效。<br>
    ///
    ///
    /// [`cfg`]: ../reference/conditional-compilation.html#the-cfg-attribute
    ///
    /// # Examples
    ///
    /// ```
    /// let my_directory = if cfg!(windows) {
    ///     "windows-specific-directory"
    /// } else {
    ///     "unix-directory"
    /// };
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "cfg_macro")]
    macro_rules! cfg {
        ($($cfg:tt)*) => {
            /* compiler built-in */
        };
    }

    /// Parses a file as an expression or an item according to the context. <br>根据上下文将文件解析为表达式或项。<br>
    ///
    /// The file is located relative to the current file (similarly to how modules are found). <br>该文件相对于当前文件位于 (类似于查找模块的方式)。<br> The provided path is interpreted in a platform-specific way at compile time. <br>提供的路径在编译时以特定于平台的方式进行解释。<br>
    /// So, for instance, an invocation with a Windows path containing backslashes `\` would not compile correctly on Unix. <br>因此，例如，使用 Windows 路径包含反斜杠 `\` 的调用将无法在 Unix 上正确编译。<br>
    ///
    /// Using this macro is often a bad idea, because if the file is parsed as an expression, it is going to be placed in the surrounding code unhygienically. <br>使用此宏通常不是一个好主意，因为如果将文件解析为表达式，它将被不卫生地放在周围的代码中。<br>
    /// This could result in variables or functions being different from what the file expected if there are variables or functions that have the same name in the current file. <br>如果当前文件中有同名的变量或函数，这可能导致变量或函数与文件预期的不同。<br>
    ///
    ///
    /// # Examples
    ///
    /// Assume there are two files in the same directory with the following contents: <br>假设在同一目录中有两个文件，其内容如下：<br>
    ///
    /// File 'monkeys.in': <br>文件 'monkeys.in'：<br>
    ///
    /// ```ignore (only-for-syntax-highlight)
    /// ['🙈', '🙊', '🙉']
    ///     .iter()
    ///     .cycle()
    ///     .take(6)
    ///     .collect::<String>()
    /// ```
    ///
    /// File 'main.rs': <br>文件 'main.rs'：<br>
    ///
    /// ```ignore (cannot-doctest-external-file-dependency)
    /// fn main() {
    ///     let my_string = include!("monkeys.in");
    ///     assert_eq!("🙈🙊🙉🙈🙊🙉", my_string);
    ///     println!("{my_string}");
    /// }
    /// ```
    ///
    /// Compiling 'main.rs' and running the resulting binary will print "🙈🙊🙉🙈🙊🙉". <br>编译 'main.rs' 并运行生成的二进制文件将打印 "🙈🙊🙉🙈🙊🙉"。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[cfg_attr(not(test), rustc_diagnostic_item = "include_macro")]
    macro_rules! include {
        ($file:expr $(,)?) => {{ /* compiler built-in */ }};
    }

    /// Asserts that a boolean expression is `true` at runtime. <br>声明在运行时布尔表达式为 `true`。<br>
    ///
    /// This will invoke the [`panic!`] macro if the provided expression cannot be evaluated to `true` at runtime. <br>如果提供的表达式在运行时无法评估为 `true`，则将调用 [`panic!`] 宏。<br>
    ///
    /// # Uses
    ///
    /// Assertions are always checked in both debug and release builds, and cannot be disabled. <br>断言总是在调试和发行版本中被检查，并且不能被禁用。<br>
    /// See [`debug_assert!`] for assertions that are not enabled in release builds by default. <br>有关默认情况下未在发行版中启用的断言，请参见 [`debug_assert!`]。<br>
    ///
    /// Unsafe code may rely on `assert!` to enforce run-time invariants that, if violated could lead to unsafety. <br>不安全的代码可能依赖 `assert!` 来强制执行运行时不变量，如果违反该规定，可能会导致不安全。<br>
    ///
    /// Other use-cases of `assert!` include testing and enforcing run-time invariants in safe code (whose violation cannot result in unsafety). <br>`assert!` 的其他用例包括在安全代码中测试和强制执行运行时不变量 (违反该规则不会导致不安全)。<br>
    ///
    ///
    /// # Custom Messages <br>自定义消息<br>
    ///
    /// This macro has a second form, where a custom panic message can be provided with or without arguments for formatting. <br>此宏具有第二种形式，其中可以提供自定义 panic 消息 (带有或不带有用于格式化的参数)。<br>
    /// See [`std::fmt`] for syntax for this form. <br>有关此格式的语法，请参见 [`std::fmt`]。<br>
    /// Expressions used as format arguments will only be evaluated if the assertion fails. <br>仅当断言失败时，才对用作 format 参数的表达式求值。<br>
    ///
    /// [`std::fmt`]: ../std/fmt/index.html
    ///
    /// # Examples
    ///
    /// ```
    /// // the panic message for these assertions is the stringified value of the expression given. <br>这些断言的 panic 消息是给定表达式的字符串化值。<br>
    /////
    /// assert!(true);
    ///
    /// fn some_computation() -> bool { true } // a very simple function <br>一个非常简单的函数<br>
    ///
    /// assert!(some_computation());
    ///
    /// // assert with a custom message <br>使用自定义消息进行断言<br>
    /// let x = true;
    /// assert!(x, "x wasn't true!");
    ///
    /// let a = 3; let b = 27;
    /// assert!(a + b == 30, "a = {}, b = {}", a, b);
    /// ```
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    #[macro_export]
    #[rustc_diagnostic_item = "assert_macro"]
    #[allow_internal_unstable(core_panic, edition_panic)]
    macro_rules! assert {
        ($cond:expr $(,)?) => {{ /* compiler built-in */ }};
        ($cond:expr, $($arg:tt)+) => {{ /* compiler built-in */ }};
    }

    /// Prints passed tokens into the standard output. <br>将传递的 tokens 打印到标准输出中。<br>
    #[unstable(
        feature = "log_syntax",
        issue = "29598",
        reason = "`log_syntax!` is not stable enough for use and is subject to change"
    )]
    #[rustc_builtin_macro]
    #[macro_export]
    macro_rules! log_syntax {
        ($($arg:tt)*) => {
            /* compiler built-in */
        };
    }

    /// Enables or disables tracing functionality used for debugging other macros. <br>启用或禁用用于调试其他宏的跟踪功能。<br>
    #[unstable(
        feature = "trace_macros",
        issue = "29598",
        reason = "`trace_macros` is not stable enough for use and is subject to change"
    )]
    #[rustc_builtin_macro]
    #[macro_export]
    macro_rules! trace_macros {
        (true) => {{ /* compiler built-in */ }};
        (false) => {{ /* compiler built-in */ }};
    }

    /// Attribute macro used to apply derive macros. <br>用于应用派生宏的属性宏。<br>
    ///
    /// See [the reference] for more info. <br>有关详细信息，请参见 [参考][the reference]。<br>
    ///
    /// [the reference]: ../../../reference/attributes/derive.html
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_builtin_macro]
    pub macro derive($item:item) {
        /* compiler built-in */
    }

    /// Attribute macro applied to a function to turn it into a unit test. <br>将属性宏应用于函数以将其转换为单元测试。<br>
    ///
    /// See [the reference] for more info. <br>有关详细信息，请参见 [参考][the reference]。<br>
    ///
    /// [the reference]: ../../../reference/attributes/testing.html#the-test-attribute
    #[stable(feature = "rust1", since = "1.0.0")]
    #[allow_internal_unstable(test, rustc_attrs)]
    #[rustc_builtin_macro]
    pub macro test($item:item) {
        /* compiler built-in */
    }

    /// Attribute macro applied to a function to turn it into a benchmark test. <br>属性宏应用于函数以将其转换为基准测试。<br>
    #[unstable(
        feature = "test",
        issue = "50297",
        soft,
        reason = "`bench` is a part of custom test frameworks which are unstable"
    )]
    #[allow_internal_unstable(test, rustc_attrs)]
    #[rustc_builtin_macro]
    pub macro bench($item:item) {
        /* compiler built-in */
    }

    /// An implementation detail of the `#[test]` and `#[bench]` macros. <br>`#[test]` 和 `#[bench]` 宏的实现细节。<br>
    #[unstable(
        feature = "custom_test_frameworks",
        issue = "50297",
        reason = "custom test frameworks are an unstable feature"
    )]
    #[allow_internal_unstable(test, rustc_attrs)]
    #[rustc_builtin_macro]
    pub macro test_case($item:item) {
        /* compiler built-in */
    }

    /// Attribute macro applied to a static to register it as a global allocator. <br>将属性宏应用于静态以将其注册为分配器。<br>
    ///
    /// See also [`std::alloc::GlobalAlloc`](../../../std/alloc/trait.GlobalAlloc.html). <br>另请参见 [`std::alloc::GlobalAlloc`](../../../std/alloc/trait.GlobalAlloc.html)。<br>
    #[stable(feature = "global_allocator", since = "1.28.0")]
    #[allow_internal_unstable(rustc_attrs)]
    #[rustc_builtin_macro]
    pub macro global_allocator($item:item) {
        /* compiler built-in */
    }

    /// Keeps the item it's applied to if the passed path is accessible, and removes it otherwise. <br>如果可以访问传递的路径，则保留适用于它的项，否则将其删除。<br>
    #[unstable(
        feature = "cfg_accessible",
        issue = "64797",
        reason = "`cfg_accessible` is not fully implemented"
    )]
    #[rustc_builtin_macro]
    pub macro cfg_accessible($item:item) {
        /* compiler built-in */
    }

    /// Expands all `#[cfg]` and `#[cfg_attr]` attributes in the code fragment it's applied to. <br>扩展其所应用的代码片段中的所有 `#[cfg]` 和 `#[cfg_attr]` 属性。<br>
    #[unstable(
        feature = "cfg_eval",
        issue = "82679",
        reason = "`cfg_eval` is a recently implemented feature"
    )]
    #[rustc_builtin_macro]
    pub macro cfg_eval($($tt:tt)*) {
        /* compiler built-in */
    }

    /// Unstable implementation detail of the `rustc` compiler, do not use. <br>`rustc` 编译器的不稳定实现细节，请勿使用。<br>
    #[rustc_builtin_macro]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[allow_internal_unstable(core_intrinsics, libstd_sys_internals)]
    #[deprecated(since = "1.52.0", note = "rustc-serialize is deprecated and no longer supported")]
    #[doc(hidden)] // While technically stable, using it is unstable, and deprecated. <br>虽然技术上稳定，但使用它是不稳定的，并且已弃用。<br> Hide it. <br>把它隐藏起来。<br>
    pub macro RustcDecodable($item:item) {
        /* compiler built-in */
    }

    /// Unstable implementation detail of the `rustc` compiler, do not use. <br>`rustc` 编译器的不稳定实现细节，请勿使用。<br>
    #[rustc_builtin_macro]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[allow_internal_unstable(core_intrinsics)]
    #[deprecated(since = "1.52.0", note = "rustc-serialize is deprecated and no longer supported")]
    #[doc(hidden)] // While technically stable, using it is unstable, and deprecated. <br>虽然技术上稳定，但使用它是不稳定的，并且已弃用。<br> Hide it. <br>把它隐藏起来。<br>
    pub macro RustcEncodable($item:item) {
        /* compiler built-in */
    }
}
