//! Primitive traits and types representing basic properties of types. <br>代表类型基本属性的原始 traits 和类型。<br>
//!
//! Rust types can be classified in various useful ways according to their intrinsic properties. <br>Rust 类型可以根据其固有属性以各种有用的方式进行分类。<br>
//! These classifications are represented as traits. <br>这些分类表示为 traits。<br>
//!

#![stable(feature = "rust1", since = "1.0.0")]

use crate::cell::UnsafeCell;
use crate::cmp;
use crate::fmt::Debug;
use crate::hash::Hash;
use crate::hash::Hasher;

/// Types that can be transferred across thread boundaries. <br>可以跨线程边界传输的类型。<br>
///
/// This trait is automatically implemented when the compiler determines it's appropriate. <br>当编译器确定适当时，会自动实现此 trait。<br>
///
/// An example of a non-`Send` type is the reference-counting pointer [`rc::Rc`][`Rc`]. <br>非 `Send` 类型的一个例子是引用计数指针 [`rc::Rc`][`Rc`]。<br>
/// If two threads attempt to clone [`Rc`]s that point to the same reference-counted value, they might try to update the reference count at the same time, which is [undefined behavior][ub] because [`Rc`] doesn't use atomic operations. <br>如果两个线程试图克隆指向相同引用计数值的 [`Rc`]，它们可能会同时尝试更新引用计数，这是 [未定义行为][ub] 因为 [`Rc`] 不使用原子操作。<br>
///
/// Its cousin [`sync::Arc`][arc] does use atomic operations (incurring some overhead) and thus is `Send`. <br>它的表亲 [`sync::Arc`][arc] 确实使用原子操作 (产生一些开销)，因此它是 `Send`。<br>
///
/// See [the Nomicon](../../nomicon/send-and-sync.html) for more details. <br>有关更多详细信息，请参见 [the Nomicon](../../nomicon/send-and-sync.html)。<br>
///
/// [`Rc`]: ../../std/rc/struct.Rc.html
/// [arc]: ../../std/sync/struct.Arc.html
/// [ub]: ../../reference/behavior-considered-undefined.html
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "Send")]
#[rustc_on_unimplemented(
    message = "`{Self}` cannot be sent between threads safely",
    label = "`{Self}` cannot be sent between threads safely"
)]
pub unsafe auto trait Send {
    // empty.
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> !Send for *const T {}
#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> !Send for *mut T {}

/// Types with a constant size known at compile time. <br>在编译时已知大小为常量的类型。<br>
///
/// All type parameters have an implicit bound of `Sized`. <br>所有类型参数都有一个 `Sized` 的隐式界限。<br> The special syntax `?Sized` can be used to remove this bound if it's not appropriate. <br>如果不合适，可以使用特殊语法 `?Sized` 删除这个界限。<br>
///
/// ```
/// # #![allow(dead_code)]
/// struct Foo<T>(T);
/// struct Bar<T: ?Sized>(T);
///
/// // struct FooUse(Foo<[i32]>);  // error: Sized is not implemented for [i32] <br>// 错误：没有为 [i32] 实现 Sized<br>
/// struct BarUse(Bar<[i32]>); // OK
/// ```
///
/// The one exception is the implicit `Self` type of a trait. <br>一个例外是 trait 的隐式 `Self` 类型。<br>
/// A trait does not have an implicit `Sized` bound as this is incompatible with [trait object]s where, by definition, the trait needs to work with all possible implementors, and thus could be any size. <br>这个 trait 没有隐式的 `Sized` 界限，因为它与 [trait 对象][trait object] 不兼容，根据定义，trait 需要与所有可能的实现者一起工作，因此可以为任意大小。<br>
///
///
/// Although Rust will let you bind `Sized` to a trait, you won't be able to use it to form a trait object later: <br>尽管 Rust 可以让你将 `Sized` 绑定到一个 trait，但是以后您将无法使用它来生成 trait 对象：<br>
///
/// ```
/// # #![allow(unused_variables)]
/// trait Foo { }
/// trait Bar: Sized { }
///
/// struct Impl;
/// impl Foo for Impl { }
/// impl Bar for Impl { }
///
/// let x: &dyn Foo = &Impl;    // OK
/// // let y: &dyn Bar = &Impl;  // error: the trait `Bar` cannot be made into an object <br>// 错误：无法将 `Bar` trait 制作成对象<br>
/////
/// ```
///
/// [trait object]: ../../book/ch17-02-trait-objects.html
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[lang = "sized"]
#[rustc_on_unimplemented(
    message = "the size for values of type `{Self}` cannot be known at compilation time",
    label = "doesn't have a size known at compile-time"
)]
#[fundamental] // for Default, for example, which requires that `[T]: !Default` be evaluatable <br>例如，对于 Default，它要求 `[T]: !Default` 必须是可评估的<br>
#[rustc_specialization_trait]
pub trait Sized {
    // Empty.
}

/// Types that can be "unsized" to a dynamically-sized type. <br>可以把没有大小的类型改为动态大小的类型。<br>
///
/// For example, the sized array type `[i8; 2]` implements `Unsize<[i8]>` and `Unsize<dyn fmt::Debug>`. <br>例如，按大小排列的数组类型 `[i8; 2]` 实现 `Unsize<[i8]>` 和 `Unsize<dyn fmt::Debug>`。<br>
///
/// All implementations of `Unsize` are provided automatically by the compiler. <br>`Unsize` 的所有实现都是由编译器自动提供的。<br>
/// Those implementations are: <br>这些实现是：<br>
///
/// - Arrays `[T; N]` implement `Unsize<[T]>`.
/// - Types implementing a trait `Trait` also implement `Unsize<dyn Trait>`. <br>实现 `Trait` trait 的类型也实现了 `Unsize<dyn Trait>`。<br>
/// - Structs `Foo<..., T, ...>` implement `Unsize<Foo<..., U, ...>>` if all of these conditions are met: <br>结构体 `Foo<..., T, ...>` 实现了 `Unsize<Foo<..., U, ...>>` 如果所有这些条件都满足：<br>
///   - `T: Unsize<U>`.
///   - Only the last field of `Foo` has a type involving `T`. <br>只有 `Foo` 的最后一个字段具有包含 `T` 的类型。<br>
///   - `Bar<T>: Unsize<Bar<U>>`, where `Bar<T>` stands for the actual type of that last field. <br>`Bar<T>: Unsize<Bar<U>>`，其中 `Bar<T>` 代表最后一个字段的实际类型。<br>
///
/// `Unsize` is used along with [`ops::CoerceUnsized`] to allow "user-defined" containers such as [`Rc`] to contain dynamically-sized types. <br>`Unsize` 与 [`ops::CoerceUnsized`] 一起使用以允许 "user-defined" 容器 (例如 [`Rc`]) 包含动态大小的类型。<br>
/// See the [DST coercion RFC][RFC982] and [the nomicon entry on coercion][nomicon-coerce] for more details. <br>有关更多详细信息，请参见 [DST coercion RFC][RFC982] 和 [the nomicon entry on coercion][nomicon-coerce]。<br>
///
/// [`ops::CoerceUnsized`]: crate::ops::CoerceUnsized
/// [`Rc`]: ../../std/rc/struct.Rc.html
/// [RFC982]: https://github.com/rust-lang/rfcs/blob/master/text/0982-dst-coercion.md
/// [nomicon-coerce]: ../../nomicon/coercions.html
///
///
///
///
#[unstable(feature = "unsize", issue = "27732")]
#[lang = "unsize"]
pub trait Unsize<T: ?Sized> {
    // Empty.
}

/// Required trait for constants used in pattern matches. <br>模式匹配中使用的常量的必需 trait。<br>
///
/// Any type that derives `PartialEq` automatically implements this trait, *regardless* of whether its type-parameters implement `Eq`. <br>不管其类型参数是否实现了 `Eq`，任何派生 `PartialEq` 的类型都会自动实现这个 trait。<br>
///
/// If a `const` item contains some type that does not implement this trait, then that type either (1.) does not implement `PartialEq` (which means the constant will not provide that comparison method, which code generation assumes is available), or (2.) it implements *its own* version of `PartialEq` (which we assume does not conform to a structural-equality comparison). <br>如果 `const` 项包含某种不实现此 trait 的类型，则该类型要么 (1.) 不实现 `PartialEq` (这意味着常量将不提供该比较方法 (代码生成假定可用) )，要么 (2.) 自身实现 *its*`PartialEq` 的版本 (我们认为不符合结构相等性比较)。<br>
///
///
/// In either of the two scenarios above, we reject usage of such a constant in a pattern match. <br>在以上两种情况中的任何一种情况下，我们都拒绝在模式匹配中使用此类常量。<br>
///
/// See also the [structural match RFC][RFC1445], and [issue 63438] which motivated migrating from attribute-based design to this trait. <br>另请参见 [结构匹配 RFC][RFC1445] 和 [issue 63438]，它们促使从基于属性的设计迁移到此 trait。<br>
///
/// [RFC1445]: https://github.com/rust-lang/rfcs/blob/master/text/1445-restrict-constants-in-patterns.md
/// [issue 63438]: https://github.com/rust-lang/rust/issues/63438
///
///
///
///
///
///
///
#[unstable(feature = "structural_match", issue = "31434")]
#[rustc_on_unimplemented(message = "the type `{Self}` does not `#[derive(PartialEq)]`")]
#[lang = "structural_peq"]
pub trait StructuralPartialEq {
    // Empty.
}

/// Required trait for constants used in pattern matches. <br>模式匹配中使用的常量的必需 trait。<br>
///
/// Any type that derives `Eq` automatically implements this trait, *regardless* of whether its type parameters implement `Eq`. <br>派生 `Eq` 的任何类型都会自动实现此 trait，无论其类型参数是否实现 `Eq`。<br>
///
/// This is a hack to work around a limitation in our type system. <br>这是一种解决我们类型系统中的限制的技巧。<br>
///
/// # Background
///
/// We want to require that types of consts used in pattern matches have the attribute `#[derive(PartialEq, Eq)]`. <br>我们要要求模式匹配中使用的 const 类型具有属性 `#[derive(PartialEq, Eq)]`。<br>
///
/// In a more ideal world, we could check that requirement by just checking that the given type implements both the `StructuralPartialEq` trait *and* the `Eq` trait. <br>在更理想的世界中，我们可以通过仅检查给定类型是否同时实现 `StructuralPartialEq` trait 和 `Eq` trait 来检查该要求。<br>
/// However, you can have ADTs that *do* `derive(PartialEq, Eq)`, and be a case that we want the compiler to accept, and yet the constant's type fails to implement `Eq`. <br>但是，您可能拥有 *do*`derive(PartialEq, Eq)` 的 ADT，这是我们希望编译器接受的情况，但是 const 的类型无法实现 `Eq`。<br>
///
/// Namely, a case like this: <br>也就是说，像这样的情况：<br>
///
/// ```rust
/// #[derive(PartialEq, Eq)]
/// struct Wrap<X>(X);
///
/// fn higher_order(_: &()) { }
///
/// const CFN: Wrap<fn(&())> = Wrap(higher_order);
///
/// fn main() {
///     match CFN {
///         CFN => {}
///         _ => {}
///     }
/// }
/// ```
///
/// (The problem in the above code is that `Wrap<fn(&())>` does not implement `PartialEq`, nor `Eq`, because `for<'a> fn(&'a _)` does not implement those traits.) <br>(以上代码中的问题是 `Wrap<fn(&())>` 既不实现 `PartialEq` 也不实现 `Eq`，因为 `for <'a> fn(&'a _)` does not implement those traits.)<br>
///
/// Therefore, we cannot rely on naive check for `StructuralPartialEq` and mere `Eq`. <br>因此，我们不能仅仅依靠 `StructuralPartialEq` 和 `Eq` 的幼稚检查。<br>
///
/// As a hack to work around this, we use two separate traits injected by each of the two derives (`#[derive(PartialEq)]` and `#[derive(Eq)]`) and check that both of them are present as part of structural-match checking. <br>要解决此问题，我们使用两个派生对象 (`#[derive(PartialEq)]` 和 `#[derive(Eq)]`) 中的每个派生注入的两个单独的 traits，并检查它们是否都作为结构匹配检查的一部分出现。<br>
///
///
///
///
///
///
///
///
///
///
#[unstable(feature = "structural_match", issue = "31434")]
#[rustc_on_unimplemented(message = "the type `{Self}` does not `#[derive(Eq)]`")]
#[lang = "structural_teq"]
pub trait StructuralEq {
    // Empty.
}

/// Types whose values can be duplicated simply by copying bits. <br>只需复制位即可复制其值的类型。<br>
///
/// By default, variable bindings have 'move semantics.' In other words: <br>默认情况下，变量绑定具有移动语义。换句话说：<br>
///
/// ```
/// #[derive(Debug)]
/// struct Foo;
///
/// let x = Foo;
///
/// let y = x;
///
/// // `x` has moved into `y`, and so cannot be used <br>`x` 已移入 `y`，因此无法使用<br>
///
/// // println!("{x:?}"); // error: use of moved value
/// ```
///
/// However, if a type implements `Copy`, it instead has 'copy semantics': <br>但是，如果类型实现 `Copy`，则它具有复制语义：<br>
///
/// ```
/// // We can derive a `Copy` implementation. <br>我们可以派生一个 `Copy` 实现。<br>
/// // `Clone` is also required, as it's a supertrait of `Copy`. <br>`Clone` 也是必需的，因为它是 `Copy` 的 super trait。<br>
/// #[derive(Debug, Copy, Clone)]
/// struct Foo;
///
/// let x = Foo;
///
/// let y = x;
///
/// // `y` is a copy of `x` <br>`y` 是 `x` 的副本<br>
///
/// println!("{x:?}"); // A-OK!
/// ```
///
/// It's important to note that in these two examples, the only difference is whether you are allowed to access `x` after the assignment. <br>重要的是要注意，在这两个示例中，唯一的区别是分配后是否允许您访问 `x`。<br>
/// Under the hood, both a copy and a move can result in bits being copied in memory, although this is sometimes optimized away. <br>在后台，复制和移动都可能导致将位复制到内存中，尽管有时会对其进行优化。<br>
///
/// ## How can I implement `Copy`? <br>如何实现 `Copy`？<br>
///
/// There are two ways to implement `Copy` on your type. <br>有两种方法可以在您的类型上实现 `Copy`。<br> The simplest is to use `derive`: <br>最简单的是使用 `derive`：<br>
///
/// ```
/// #[derive(Copy, Clone)]
/// struct MyStruct;
/// ```
///
/// You can also implement `Copy` and `Clone` manually: <br>您还可以手动实现 `Copy` 和 `Clone`：<br>
///
/// ```
/// struct MyStruct;
///
/// impl Copy for MyStruct { }
///
/// impl Clone for MyStruct {
///     fn clone(&self) -> MyStruct {
///         *self
///     }
/// }
/// ```
///
/// There is a small difference between the two: the `derive` strategy will also place a `Copy` bound on type parameters, which isn't always desired. <br>这两者之间有一个小小的区别: `derive` 策略还将 `Copy` 绑定在类型参数上，这并不总是需要的。<br>
///
/// ## What's the difference between `Copy` and `Clone`? <br>`Copy` 和 `Clone` 有什么区别？<br>
///
/// Copies happen implicitly, for example as part of an assignment `y = x`. <br>Copy 是隐式发生的，例如作为赋值 `y = x` 的一部分。<br> The behavior of `Copy` is not overloadable; <br>`Copy` 的行为不可重载；<br> it is always a simple bit-wise copy. <br>它始终是简单的按位复制。<br>
///
/// Cloning is an explicit action, `x.clone()`. <br>Clone 是一个显式操作，`x.clone()`。<br> The implementation of [`Clone`] can provide any type-specific behavior necessary to duplicate values safely. <br>[`Clone`] 的实现可以提供安全复制值所需的任何特定于类型的行为。<br>
/// For example, the implementation of [`Clone`] for [`String`] needs to copy the pointed-to string buffer in the heap. <br>例如，用于 [`String`] 的 [`Clone`] 的实现需要在堆中复制指向字符串的缓冲区。<br>
/// A simple bitwise copy of [`String`] values would merely copy the pointer, leading to a double free down the line. <br>[`String`] 值的简单按位副本将仅复制指针，从而导致该行向下双重释放。<br>
/// For this reason, [`String`] is [`Clone`] but not `Copy`. <br>因此，[`String`] 是 [`Clone`]，但不是 `Copy`。<br>
///
/// [`Clone`] is a supertrait of `Copy`, so everything which is `Copy` must also implement [`Clone`]. <br>[`Clone`] 是 `Copy` 的一个 super trait，所以所有 `Copy` 的东西也必须实现 [`Clone`]。<br>
/// If a type is `Copy` then its [`Clone`] implementation only needs to return `*self` (see the example above). <br>如果类型为 `Copy`，则其 [`Clone`] 实现仅需要返回 `*self` (请参见上面的示例)。<br>
///
/// ## When can my type be `Copy`? <br>什么时候可以输入 `Copy`？<br>
///
/// A type can implement `Copy` if all of its components implement `Copy`. <br>如果类型的所有组件都实现 `Copy`，则它可以实现 `Copy`。<br> For example, this struct can be `Copy`: <br>例如，此结构体可以是 `Copy`：<br>
///
/// ```
/// # #[allow(dead_code)]
/// #[derive(Copy, Clone)]
/// struct Point {
///    x: i32,
///    y: i32,
/// }
/// ```
///
/// A struct can be `Copy`, and [`i32`] is `Copy`, therefore `Point` is eligible to be `Copy`. <br>一个结构体可以是 `Copy`，而 [`i32`] 是 `Copy`，因此 `Point` 有资格成为 `Copy`。<br>
/// By contrast, consider <br>相比之下，考虑<br>
///
/// ```
/// # #![allow(dead_code)]
/// # struct Point;
/// struct PointList {
///     points: Vec<Point>,
/// }
/// ```
///
/// The struct `PointList` cannot implement `Copy`, because [`Vec<T>`] is not `Copy`. <br>结构体 `PointList` 无法实现 `Copy`，因为 [`Vec<T>`] 不是 `Copy`。<br> If we attempt to derive a `Copy` implementation, we'll get an error: <br>如果尝试派生 `Copy` 实现，则会收到错误消息：<br>
///
/// ```text
/// the trait `Copy` may not be implemented for this type; field `points` does not implement `Copy`
/// ```
///
/// Shared references (`&T`) are also `Copy`, so a type can be `Copy`, even when it holds shared references of types `T` that are *not* `Copy`. <br>共享引用 (`&T`) 也是 `Copy`，因此，即使类型中包含 *04* 不是*`Copy` 类型的共享引用 `T`，也可以是 `Copy`。<br>
/// Consider the following struct, which can implement `Copy`, because it only holds a *shared reference* to our non-`Copy` type `PointList` from above: <br>考虑下面的结构体，它可以实现 `Copy`，因为它从上方仅对我们的非 Copy 类型 `PointList` 持有一个 *shared 引用*：<br>
///
/// ```
/// # #![allow(dead_code)]
/// # struct PointList;
/// #[derive(Copy, Clone)]
/// struct PointListWrapper<'a> {
///     point_list_ref: &'a PointList,
/// }
/// ```
///
/// ## When *can't* my type be `Copy`? <br>什么时候我的类型不能为 `Copy`？<br>
///
/// Some types can't be copied safely. <br>某些类型无法安全复制。<br> For example, copying `&mut T` would create an aliased mutable reference. <br>例如，复制 `&mut T` 将创建一个别名可变引用。<br>
/// Copying [`String`] would duplicate responsibility for managing the [`String`]'s buffer, leading to a double free. <br>复制 [`String`] 将重复管理 [`String`] 缓冲区，从而导致双重释放。<br>
///
/// Generalizing the latter case, any type implementing [`Drop`] can't be `Copy`, because it's managing some resource besides its own [`size_of::<T>`] bytes. <br>概括后一种情况，任何实现 [`Drop`] 的类型都不能是 `Copy`，因为它除了管理自己的 [`size_of::<T>`] 字节外还管理一些资源。<br>
///
/// If you try to implement `Copy` on a struct or enum containing non-`Copy` data, you will get the error [E0204]. <br>果您尝试在包含非 `Copy` 数据的结构或枚举上实现 `Copy`，则会收到 [E0204] 错误。<br>
///
/// [E0204]: ../../error-index.html#E0204
///
/// ## When *should* my type be `Copy`? <br>什么时候我的类型应该是 `Copy`？<br>
///
/// Generally speaking, if your type _can_ implement `Copy`, it should. <br>一般来说，如果您的类型可以实现 `Copy`，则应该这样做。<br>
/// Keep in mind, though, that implementing `Copy` is part of the public API of your type. <br>但是请记住，实现 `Copy` 是您类型的公共 API 的一部分。<br>
/// If the type might become non-`Copy` in the future, it could be prudent to omit the `Copy` implementation now, to avoid a breaking API change. <br>如果该类型将来可能变为非 `Copy`，则最好现在省略 `Copy` 实现，以避免 API 发生重大更改。<br>
///
/// ## Additional implementors <br>其他实现者<br>
///
/// In addition to the [implementors listed below][impls], the following types also implement `Copy`: <br>除 [下面列出的实现者][impls] 之外，以下类型也实现了 `Copy`：<br>
///
/// * Function item types (i.e., the distinct types defined for each function) <br>函数项类型 (即，为每个函数定义的不同类型)<br>
/// * Function pointer types (e.g., `fn() -> i32`) <br>函数指针类型 (例如 `fn() -> i32`)<br>
/// * Closure types, if they capture no value from the environment or if all such captured values implement `Copy` themselves. <br>闭包类型，如果它们没有从环境中捕获任何值，或者所有此类捕获的值本身都实现了 `Copy`。<br>
///   Note that variables captured by shared reference always implement `Copy` (even if the referent doesn't), while variables captured by mutable reference never implement `Copy`. <br>请注意，由共享引用捕获的变量始终实现 `Copy` (即使引用对象没有实现)，而由变量引用捕获的变量从不实现 `Copy`。<br>
///
///
/// [`Vec<T>`]: ../../std/vec/struct.Vec.html
/// [`String`]: ../../std/string/struct.String.html
/// [`size_of::<T>`]: crate::mem::size_of
/// [impls]: #implementors
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[lang = "copy"]
// FIXME(matthewjasper) This allows copying a type that doesn't implement `Copy` because of unsatisfied lifetime bounds (copying `A<'_>` when only `A<'static>: Copy` and `A<'_>: Clone`). <br>这允许复制由于不满足生命周期实现边界而没有 `Copy` 的类型 (仅在 `A<'static>: Copy` 和 `A<'_>: Clone` 时复制 `A<'_>`)。<br>
// We have this attribute here for now only because there are quite a few existing specializations on `Copy` that already exist in the standard library, and there's no way to safely have this behavior right now. <br>我们现在在这里具有此属性的原因仅是因为标准库中已经存在 `Copy` 上的许多现有专长，并且目前尚无办法安全地具有此行为。<br>
//
//
//
//
#[rustc_unsafe_specialization_marker]
#[rustc_diagnostic_item = "Copy"]
pub trait Copy: Clone {
    // Empty.
}

/// Derive macro generating an impl of the trait `Copy`. <br>派生宏，生成 `Copy` trait 的 impl。<br>
#[rustc_builtin_macro]
#[stable(feature = "builtin_macro_prelude", since = "1.38.0")]
#[allow_internal_unstable(core_intrinsics, derive_clone_copy)]
pub macro Copy($item:item) {
    /* compiler built-in */
}

/// Types for which it is safe to share references between threads. <br>可以在线程之间安全共享引用的类型。<br>
///
/// This trait is automatically implemented when the compiler determines it's appropriate. <br>当编译器确定适当时，会自动实现此 trait。<br>
///
/// The precise definition is: a type `T` is [`Sync`] if and only if `&T` is [`Send`]. <br>确切的定义是：当且仅当 `&T` 是 [`Send`] 时，类型 `T` 才是 [`Sync`]。<br>
/// In other words, if there is no possibility of [undefined behavior][ub] (including data races) when passing `&T` references between threads. <br>换句话说，如果在线程之间传递 `&T` 引用时没有 [未定义的行为][ub] (包括数据竞争) 的可能性。<br>
///
/// As one would expect, primitive types like [`u8`] and [`f64`] are all [`Sync`], and so are simple aggregate types containing them, like tuples, structs and enums. <br>正如人们所料，像 [`u8`] 和 [`f64`] 这样的原始类型都是 [`Sync`]，包含它们的简单聚合类型也是如此，比如元组、结构体和枚举。<br>
/// More examples of basic [`Sync`] types include "immutable" types like `&T`, and those with simple inherited mutability, such as [`Box<T>`][box], [`Vec<T>`][vec] and most other collection types. <br>基本 [`Sync`] 类型的更多示例包括不可变类型 (例如 `&T`) 以及具有简单继承的可变性的类型，例如 [`Box<T>`][box]，[`Vec<T>`][vec] 和大多数其他集合类型。<br>
///
/// (Generic parameters need to be [`Sync`] for their container to be [`Sync`].) <br>(泛型参数必须为 [`Sync`]，才能使其容器为 [[Sync]]。)<br>
///
/// A somewhat surprising consequence of the definition is that `&mut T` is `Sync` (if `T` is `Sync`) even though it seems like that might provide unsynchronized mutation. <br>该定义的一个令人惊讶的结果是 `&mut T` 是 `Sync` (如果 `T` 是 `Sync`)，即使看起来可能提供了不同步的可变的。<br>
/// The trick is that a mutable reference behind a shared reference (that is, `& &mut T`) becomes read-only, as if it were a `& &T`. <br>诀窍是，共享引用 (即 `& &mut T`) 后面的可变引用将变为只读，就好像它是 `& &T` 一样。<br>
/// Hence there is no risk of a data race. <br>因此，没有数据竞争的风险。<br>
///
/// Types that are not `Sync` are those that have "interior mutability" in a non-thread-safe form, such as [`Cell`][cell] and [`RefCell`][refcell]. <br>不是 `Sync` 的类型是具有非线程安全形式的 "内部可变性" 的类型，例如 [`Cell`][cell] 和 [`RefCell`][refcell]。<br>
/// These types allow for mutation of their contents even through an immutable, shared reference. <br>这些类型甚至允许通过不可变，共享引用来更改其内容。<br>
/// For example the `set` method on [`Cell<T>`][cell] takes `&self`, so it requires only a shared reference [`&Cell<T>`][cell]. <br>例如，[`Cell<T>`][cell] 上的 `set` 方法采用 `&self`，因此它仅需要共享的引用 [`&Cell<T>`][cell]。<br>
/// The method performs no synchronization, thus [`Cell`][cell] cannot be `Sync`. <br>该方法不执行同步，因此 [`Cell`][cell] 不能为 `Sync`。<br>
///
/// Another example of a non-`Sync` type is the reference-counting pointer [`Rc`][rc]. <br>另一个非 `Sync` 类型的例子是引用计数指针 [`Rc`][rc]。<br>
/// Given any reference [`&Rc<T>`][rc], you can clone a new [`Rc<T>`][rc], modifying the reference counts in a non-atomic way. <br>给定任何引用 [`&Rc<T>`][rc]，您可以克隆新的 [`Rc<T>`][rc]，以非原子方式修改引用计数。<br>
///
/// For cases when one does need thread-safe interior mutability, Rust provides [atomic data types], as well as explicit locking via [`sync::Mutex`][mutex] and [`sync::RwLock`][rwlock]. <br>对于确实需要线程安全的内部可变性的情况，Rust 提供 [原子数据类型][atomic data types] 以及通过 [`sync::Mutex`][mutex] 和 [`sync::RwLock`][rwlock] 进行的显式锁定。<br>
/// These types ensure that any mutation cannot cause data races, hence the types are `Sync`. <br>这些类型可确保任何可变的都不会引起数据竞争，因此类型为 `Sync`。<br>
/// Likewise, [`sync::Arc`][arc] provides a thread-safe analogue of [`Rc`][rc]. <br>同样，[`sync::Arc`][arc] 提供了 [`Rc`][rc] 的线程安全模拟。<br>
///
/// Any types with interior mutability must also use the [`cell::UnsafeCell`][unsafecell] wrapper around the value(s) which can be mutated through a shared reference. <br>任何具有内部可变性的类型还必须在值周围使用 [`cell::UnsafeCell`][unsafecell] 包装器，该包装器可以通过共享引用进行转变。<br>
/// Failing to doing this is [undefined behavior][ub]. <br>不这样做是 [未定义的行为][ub]。<br>
/// For example, [`transmute`][transmute]-ing from `&T` to `&mut T` is invalid. <br>例如，从 `&T` 到 `&mut T` 的 [`transmute`][transmute] 无效。<br>
///
/// See [the Nomicon][nomicon-send-and-sync] for more details about `Sync`. <br>有关 `Sync` 的更多详细信息，请参见 [the Nomicon][nomicon-send-and-sync]。<br>
///
/// [box]: ../../std/boxed/struct.Box.html
/// [vec]: ../../std/vec/struct.Vec.html
/// [cell]: crate::cell::Cell
/// [refcell]: crate::cell::RefCell
/// [rc]: ../../std/rc/struct.Rc.html
/// [arc]: ../../std/sync/struct.Arc.html
/// [atomic data types]: crate::sync::atomic
/// [mutex]: ../../std/sync/struct.Mutex.html
/// [rwlock]: ../../std/sync/struct.RwLock.html
/// [unsafecell]: crate::cell::UnsafeCell
/// [ub]: ../../reference/behavior-considered-undefined.html
/// [transmute]: crate::mem::transmute
/// [nomicon-send-and-sync]: ../../nomicon/send-and-sync.html
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "Sync")]
#[lang = "sync"]
#[rustc_on_unimplemented(
    message = "`{Self}` cannot be shared between threads safely",
    label = "`{Self}` cannot be shared between threads safely"
)]
pub unsafe auto trait Sync {
    // FIXME(estebank): once support to add notes in `rustc_on_unimplemented` lands in beta, and it has been extended to check whether a closure is anywhere in the requirement chain, extend it as such (#48534): <br>曾经支持在 `rustc_on_unimplemented` 中添加注释登陆 beta，并且已经扩展以检查闭包是否在需求链中的任何位置，将其扩展为 (#48534)：<br>
    //
    //
    // ```
    // on(
    //     closure,
    //     note="`{Self}` cannot be shared safely, consider marking the closure `move`"
    // ),
    // ```

    // Empty
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> !Sync for *const T {}
#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized> !Sync for *mut T {}

macro_rules! impls {
    ($t: ident) => {
        #[stable(feature = "rust1", since = "1.0.0")]
        impl<T: ?Sized> Hash for $t<T> {
            #[inline]
            fn hash<H: Hasher>(&self, _: &mut H) {}
        }

        #[stable(feature = "rust1", since = "1.0.0")]
        impl<T: ?Sized> cmp::PartialEq for $t<T> {
            fn eq(&self, _other: &$t<T>) -> bool {
                true
            }
        }

        #[stable(feature = "rust1", since = "1.0.0")]
        impl<T: ?Sized> cmp::Eq for $t<T> {}

        #[stable(feature = "rust1", since = "1.0.0")]
        impl<T: ?Sized> cmp::PartialOrd for $t<T> {
            fn partial_cmp(&self, _other: &$t<T>) -> Option<cmp::Ordering> {
                Option::Some(cmp::Ordering::Equal)
            }
        }

        #[stable(feature = "rust1", since = "1.0.0")]
        impl<T: ?Sized> cmp::Ord for $t<T> {
            fn cmp(&self, _other: &$t<T>) -> cmp::Ordering {
                cmp::Ordering::Equal
            }
        }

        #[stable(feature = "rust1", since = "1.0.0")]
        impl<T: ?Sized> Copy for $t<T> {}

        #[stable(feature = "rust1", since = "1.0.0")]
        impl<T: ?Sized> Clone for $t<T> {
            fn clone(&self) -> Self {
                Self
            }
        }

        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_unstable(feature = "const_default_impls", issue = "87864")]
        impl<T: ?Sized> const Default for $t<T> {
            fn default() -> Self {
                Self
            }
        }

        #[unstable(feature = "structural_match", issue = "31434")]
        impl<T: ?Sized> StructuralPartialEq for $t<T> {}

        #[unstable(feature = "structural_match", issue = "31434")]
        impl<T: ?Sized> StructuralEq for $t<T> {}
    };
}

/// Zero-sized type used to mark things that "act like" they own a `T`. <br>零大小的类型用来标记那些行为像它们拥有一个 `T` 的东西。<br>
///
/// Adding a `PhantomData<T>` field to your type tells the compiler that your type acts as though it stores a value of type `T`, even though it doesn't really. <br>向您的类型添加 `PhantomData<T>` 字段将告诉编译器，您的类型的行为就像它存储了 `T` 类型的值一样，即使实际上并非如此。<br>
/// This information is used when computing certain safety properties. <br>在计算某些安全属性时会使用此信息。<br>
///
/// For a more in-depth explanation of how to use `PhantomData<T>`, please see [the Nomicon](../../nomicon/phantom-data.html). <br>有关如何使用 `PhantomData<T>` 的更深入的说明，请参见 [the Nomicon](../../nomicon/phantom-data.html)。<br>
///
/// # A ghastly note <br>一个可怕的笔记<br> 👻👻👻
///
/// Though they both have scary names, `PhantomData` and 'phantom types' are related, but not identical. <br>尽管它们都有可怕的名称，但 `PhantomData` 和 phantom 类型是相关的，但并不完全相同。<br> A phantom type parameter is simply a type parameter which is never used. <br>phantom 类型参数只是从未使用过的类型参数。<br>
/// In Rust, this often causes the compiler to complain, and the solution is to add a "dummy" use by way of `PhantomData`. <br>在 Rust 中，这通常会导致编译器抱怨，而解决方案是通过 `PhantomData` 添加 "dummy" 用途。<br>
///
/// # Examples
///
/// ## Unused lifetime parameters <br>未使用的生命周期参数<br>
///
/// Perhaps the most common use case for `PhantomData` is a struct that has an unused lifetime parameter, typically as part of some unsafe code. <br>`PhantomData` 的最常见用例也许是具有未使用的生命周期参数的结构体，通常将其用作某些不安全代码的一部分。<br>
/// For example, here is a struct `Slice` that has two pointers of type `*const T`, presumably pointing into an array somewhere: <br>例如，这是一个结构体 `Slice`，它具有两个 `*const T` 类型的指针，大概指向某个地方的数组：<br>
///
/// ```compile_fail,E0392
/// struct Slice<'a, T> {
///     start: *const T,
///     end: *const T,
/// }
/// ```
///
/// The intention is that the underlying data is only valid for the lifetime `'a`, so `Slice` should not outlive `'a`. <br>目的是底层数据仅在生命周期 `'a` 内有效，因此 `Slice` 不应超过 `'a`。<br>
/// However, this intent is not expressed in the code, since there are no uses of the lifetime `'a` and hence it is not clear what data it applies to. <br>但是，此意图未在代码中表达，因为没有使用生命周期 `'a`，因此尚不清楚它适用于什么数据。<br>
/// We can correct this by telling the compiler to act *as if* the `Slice` struct contained a reference `&'a T`: <br>我们可以通过告诉编译器如果 `Slice` 结构体包含引用 `&'a T` 来执行 *as 来纠正此问题：<br>
///
/// ```
/// use std::marker::PhantomData;
///
/// # #[allow(dead_code)]
/// struct Slice<'a, T: 'a> {
///     start: *const T,
///     end: *const T,
///     phantom: PhantomData<&'a T>,
/// }
/// ```
///
/// This also in turn requires the annotation `T: 'a`, indicating that any references in `T` are valid over the lifetime `'a`. <br>这也需要注解 `T: 'a`，以指示 `T` 中的所有引用在生命周期 `'a` 上均有效。<br>
///
/// When initializing a `Slice` you simply provide the value `PhantomData` for the field `phantom`: <br>初始化 `Slice` 时，只需为字段 `phantom` 提供值 `PhantomData`：<br>
///
/// ```
/// # #![allow(dead_code)]
/// # use std::marker::PhantomData;
/// # struct Slice<'a, T: 'a> {
/// #     start: *const T,
/// #     end: *const T,
/// #     phantom: PhantomData<&'a T>,
/// # }
/// fn borrow_vec<T>(vec: &Vec<T>) -> Slice<'_, T> {
///     let ptr = vec.as_ptr();
///     Slice {
///         start: ptr,
///         end: unsafe { ptr.add(vec.len()) },
///         phantom: PhantomData,
///     }
/// }
/// ```
///
/// ## Unused type parameters <br>未使用的类型参数<br>
///
/// It sometimes happens that you have unused type parameters which indicate what type of data a struct is "tied" to, even though that data is not actually found in the struct itself. <br>有时可能会发生未使用的类型参数，这些参数指示 "tied" 将结构体数据类型化的数据，即使该数据实际上不是在结构体本身中找到的也是如此。<br>
/// Here is an example where this arises with [FFI]. <br>这是 [FFI] 出现此情况的示例。<br>
/// The foreign interface uses handles of type `*mut ()` to refer to Rust values of different types. <br>外部接口使用 `*mut ()` 类型的句柄来引用不同类型的 Rust 值。<br>
/// We track the Rust type using a phantom type parameter on the struct `ExternalResource` which wraps a handle. <br>我们使用包裹句柄的结构体 `ExternalResource` 上的 `phantom` 类型参数来跟踪 Rust 类型。<br>
///
/// [FFI]: ../../book/ch19-01-unsafe-rust.html#using-extern-functions-to-call-external-code
///
/// ```
/// # #![allow(dead_code)]
/// # trait ResType { }
/// # struct ParamType;
/// # mod foreign_lib {
/// #     pub fn new(_: usize) -> *mut () { 42 as *mut () }
/// #     pub fn do_stuff(_: *mut (), _: usize) {}
/// # }
/// # fn convert_params(_: ParamType) -> usize { 42 }
/// use std::marker::PhantomData;
/// use std::mem;
///
/// struct ExternalResource<R> {
///    resource_handle: *mut (),
///    resource_type: PhantomData<R>,
/// }
///
/// impl<R: ResType> ExternalResource<R> {
///     fn new() -> Self {
///         let size_of_res = mem::size_of::<R>();
///         Self {
///             resource_handle: foreign_lib::new(size_of_res),
///             resource_type: PhantomData,
///         }
///     }
///
///     fn do_stuff(&self, param: ParamType) {
///         let foreign_params = convert_params(param);
///         foreign_lib::do_stuff(self.resource_handle, foreign_params);
///     }
/// }
/// ```
///
/// ## Ownership and the drop check <br>所有权和 drop 检测<br>
///
/// Adding a field of type `PhantomData<T>` indicates that your type owns data of type `T`. <br>添加 `PhantomData<T>` 类型的字段表示您的类型拥有 `T` 类型的数据。<br> This in turn implies that when your type is dropped, it may drop one or more instances of the type `T`. <br>反过来，这意味着丢弃您的类型时，它可能会丢弃一个或多个 `T` 类型的实例。<br>
/// This has bearing on the Rust compiler's [drop check] analysis. <br>这与 Rust 编译器的 [丢弃检查][drop check] 分析有关。<br>
///
/// If your struct does not in fact *own* the data of type `T`, it is better to use a reference type, like `PhantomData<&'a T>` (ideally) or `PhantomData<*const T>` (if no lifetime applies), so as not to indicate ownership. <br>如果您的结构体实际上并不 *拥有*`T` 类型的数据，则最好使用引用类型，例如 `PhantomData<&'a T>` (ideally) 或 `PhantomData<*const T>` (如果没有生命周期适用)，以免表示所有权。<br>
///
///
/// [drop check]: ../../nomicon/dropck.html
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
#[lang = "phantom_data"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct PhantomData<T: ?Sized>;

impls! { PhantomData }

mod impls {
    #[stable(feature = "rust1", since = "1.0.0")]
    unsafe impl<T: Sync + ?Sized> Send for &T {}
    #[stable(feature = "rust1", since = "1.0.0")]
    unsafe impl<T: Send + ?Sized> Send for &mut T {}
}

/// Compiler-internal trait used to indicate the type of enum discriminants. <br>编译器内部的 trait 用于指示枚举判别式的类型。<br>
///
/// This trait is automatically implemented for every type and does not add any guarantees to [`mem::Discriminant`]. <br>这个 trait 是为每种类型自动实现的，并且不会给 [`mem::Discriminant`] 添加任何保证。<br>
/// It is **undefined behavior** to transmute between `DiscriminantKind::Discriminant` and `mem::Discriminant`. <br>在 `DiscriminantKind::Discriminant` 和 `mem::Discriminant` 之间转换是未定义的行为。<br>
///
/// [`mem::Discriminant`]: crate::mem::Discriminant
///
#[unstable(
    feature = "discriminant_kind",
    issue = "none",
    reason = "this trait is unlikely to ever be stabilized, use `mem::discriminant` instead"
)]
#[lang = "discriminant_kind"]
pub trait DiscriminantKind {
    /// The type of the discriminant, which must satisfy the trait bounds required by `mem::Discriminant`. <br>判别类型，必须满足 `mem::Discriminant` 要求的 trait bounds。<br>
    ///
    #[lang = "discriminant_type"]
    type Discriminant: Clone + Copy + Debug + Eq + PartialEq + Hash + Send + Sync + Unpin;
}

/// Compiler-internal trait used to determine whether a type contains any `UnsafeCell` internally, but not through an indirection. <br>编译器内部的 trait 用于确定类型是否在内部包含任何 `UnsafeCell`，但不是通过间接寻址。<br>
///
/// This affects, for example, whether a `static` of that type is placed in read-only static memory or writable static memory. <br>例如，这会影响该类型的 `static` 是否放置在只读静态存储器或可写静态存储器中。<br>
///
#[lang = "freeze"]
pub(crate) unsafe auto trait Freeze {}

impl<T: ?Sized> !Freeze for UnsafeCell<T> {}
unsafe impl<T: ?Sized> Freeze for PhantomData<T> {}
unsafe impl<T: ?Sized> Freeze for *const T {}
unsafe impl<T: ?Sized> Freeze for *mut T {}
unsafe impl<T: ?Sized> Freeze for &T {}
unsafe impl<T: ?Sized> Freeze for &mut T {}

/// Types that can be safely moved after being pinned. <br>固定后可以安全移动的类型。<br>
///
/// Rust itself has no notion of immovable types, and considers moves (e.g., through assignment or [`mem::replace`]) to always be safe. <br>Rust 本身没有固定类型的概念，并认为移动 (例如，通过赋值或 [`mem::replace`]) 始终是安全的。<br>
///
/// The [`Pin`][Pin] type is used instead to prevent moves through the type system. <br>[`Pin`][Pin] 类型代替使用，以防止在类型系统中移动。<br> Pointers `P<T>` wrapped in the [`Pin<P<T>>`][Pin] wrapper can't be moved out of. <br>[`Pin<P<T>>`][Pin] 包装器中包裹的指针 `P<T>` 不能移出。<br>
/// See the [`pin` module] documentation for more information on pinning. <br>有关固定的更多信息，请参见 [`pin` module] 文档。<br>
///
/// Implementing the `Unpin` trait for `T` lifts the restrictions of pinning off the type, which then allows moving `T` out of [`Pin<P<T>>`][Pin] with functions such as [`mem::replace`]. <br>为 `T` 实现 `Unpin` trait 消除了固定该类型的限制，然后允许使用诸如 [`mem::replace`] 之类的功能将 `T` 从 [`Pin<P<T>>`][Pin] 中移出。<br>
///
///
/// `Unpin` has no consequence at all for non-pinned data. <br>`Unpin` 对未固定的数据没有任何影响。<br>
/// In particular, [`mem::replace`] happily moves `!Unpin` data (it works for any `&mut T`, not just when `T: Unpin`). <br>特别是，[`mem::replace`] 可以愉快地移动 `!Unpin` 数据 (它适用于任何 `&mut T`，而不仅限于 `T: Unpin`)。<br>
/// However, you cannot use [`mem::replace`] on data wrapped inside a [`Pin<P<T>>`][Pin] because you cannot get the `&mut T` you need for that, and *that* is what makes this system work. <br>但是，您不能对包装在 [`Pin<P<T>>`][Pin] 内的数据使用 [`mem::replace`]，因为您无法获得所需的 `&mut T`，并且 *that* 是使此系统正常工作的原因。<br>
///
/// So this, for example, can only be done on types implementing `Unpin`: <br>因此，例如，这只能在实现 `Unpin` 的类型上完成：<br>
///
/// ```rust
/// # #![allow(unused_must_use)]
/// use std::mem;
/// use std::pin::Pin;
///
/// let mut string = "this".to_string();
/// let mut pinned_string = Pin::new(&mut string);
///
/// // We need a mutable reference to call `mem::replace`. <br>我们需要一个可变引用来调用 `mem::replace`。<br>
/// // We can obtain such a reference by (implicitly) invoking `Pin::deref_mut`, but that is only possible because `String` implements `Unpin`. <br>我们可以通过 (implicitly) 调用 `Pin::deref_mut` 来获得这样的引用，但这仅是可能的，因为 `String` 实现了 `Unpin`。<br>
/////
/// mem::replace(&mut *pinned_string, "other".to_string());
/// ```
///
/// This trait is automatically implemented for almost every type. <br>几乎所有类型都会自动实现这个 trait。<br>
///
/// [`mem::replace`]: crate::mem::replace
/// [Pin]: crate::pin::Pin
/// [`pin` module]: crate::pin
///
///
///
///
///
///
#[stable(feature = "pin", since = "1.33.0")]
#[rustc_on_unimplemented(
    note = "consider using `Box::pin`",
    message = "`{Self}` cannot be unpinned"
)]
#[lang = "unpin"]
pub auto trait Unpin {}

/// A marker type which does not implement `Unpin`. <br>没有实现 `Unpin` 的标记类型。<br>
///
/// If a type contains a `PhantomPinned`, it will not implement `Unpin` by default. <br>如果类型包含 `PhantomPinned`，则默认情况下将不实现 `Unpin`。<br>
#[stable(feature = "pin", since = "1.33.0")]
#[derive(Debug, Default, Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
pub struct PhantomPinned;

#[stable(feature = "pin", since = "1.33.0")]
impl !Unpin for PhantomPinned {}

#[stable(feature = "pin", since = "1.33.0")]
impl<'a, T: ?Sized + 'a> Unpin for &'a T {}

#[stable(feature = "pin", since = "1.33.0")]
impl<'a, T: ?Sized + 'a> Unpin for &'a mut T {}

#[stable(feature = "pin_raw", since = "1.38.0")]
impl<T: ?Sized> Unpin for *const T {}

#[stable(feature = "pin_raw", since = "1.38.0")]
impl<T: ?Sized> Unpin for *mut T {}

/// A marker for types that can be dropped. <br>可以丢弃的类型的标记。<br>
///
/// This should be used for `~const` bounds, as non-const bounds will always hold for every type. <br>这应该用于 `~const` 边界，因为非常量边界将始终适用于每种类型。<br>
///
#[unstable(feature = "const_trait_impl", issue = "67792")]
#[lang = "destruct"]
#[rustc_on_unimplemented(message = "can't drop `{Self}`", append_const_msg)]
pub trait Destruct {}

/// Implementations of `Copy` for primitive types. <br>原始类型的 `Copy` 实现。<br>
///
/// Implementations that cannot be described in Rust are implemented in `traits::SelectionContext::copy_clone_conditions()` in `rustc_trait_selection`. <br>`rustc_trait_selection` 中的 `traits::SelectionContext::copy_clone_conditions()` 中实现了 Rust 中无法描述的实现。<br>
///
///
mod copy_impls {

    use super::Copy;

    macro_rules! impl_copy {
        ($($t:ty)*) => {
            $(
                #[stable(feature = "rust1", since = "1.0.0")]
                impl Copy for $t {}
            )*
        }
    }

    impl_copy! {
        usize u8 u16 u32 u64 u128
        isize i8 i16 i32 i64 i128
        f32 f64
        bool char
    }

    #[unstable(feature = "never_type", issue = "35121")]
    impl Copy for ! {}

    #[stable(feature = "rust1", since = "1.0.0")]
    impl<T: ?Sized> Copy for *const T {}

    #[stable(feature = "rust1", since = "1.0.0")]
    impl<T: ?Sized> Copy for *mut T {}

    /// Shared references can be copied, but mutable references *cannot*! <br>共享的引用可以复制，但是可变引用 *不能*!<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    impl<T: ?Sized> Copy for &T {}
}
