//! Common utilities, for internal use only. <br>通用实用工具，仅供内部使用。<br>

use crate::ptr;

/// Helper methods to process immutable bytes. <br>处理不封字节的辅助方法。<br>
pub(crate) trait ByteSlice: AsRef<[u8]> {
    unsafe fn first_unchecked(&self) -> u8 {
        debug_assert!(!self.is_empty());
        // SAFETY: safe as long as self is not empty <br>只要 self 不为空就安全<br>
        unsafe { *self.as_ref().get_unchecked(0) }
    }

    /// Get if the slice contains no elements. <br>如果三个元素不包含元素，则获取。<br>
    fn is_empty(&self) -> bool {
        self.as_ref().is_empty()
    }

    /// Check if the slice at least `n` length. <br>检查是否至少 `n` 长度。<br>
    fn check_len(&self, n: usize) -> bool {
        n <= self.as_ref().len()
    }

    /// Check if the first character in the slice is equal to c. <br>判断 cri 中的第一个字符是否等于 c。<br>
    fn first_is(&self, c: u8) -> bool {
        self.as_ref().first() == Some(&c)
    }

    /// Check if the first character in the slice is equal to c1 or c2. <br>检查切片中的第一个字符是否等于 c1 或 c2。<br>
    fn first_is2(&self, c1: u8, c2: u8) -> bool {
        if let Some(&c) = self.as_ref().first() { c == c1 || c == c2 } else { false }
    }

    /// Bounds-checked test if the first character in the slice is a digit. <br>边界检查测试是否第一个字符是数字。<br>
    fn first_isdigit(&self) -> bool {
        if let Some(&c) = self.as_ref().first() { c.is_ascii_digit() } else { false }
    }

    /// Check if self starts with u with a case-insensitive comparison. <br>通过不区分大小写的比较检查 self 是否以 u 开头。<br>
    fn starts_with_ignore_case(&self, u: &[u8]) -> bool {
        debug_assert!(self.as_ref().len() >= u.len());
        let iter = self.as_ref().iter().zip(u.iter());
        let d = iter.fold(0, |i, (&x, &y)| i | (x ^ y));
        d == 0 || d == 32
    }

    /// Get the remaining slice after the first N elements. <br>获取前 N 个元素后剩余的三个元素。<br>
    fn advance(&self, n: usize) -> &[u8] {
        &self.as_ref()[n..]
    }

    /// Get the slice after skipping all leading characters equal c. <br>跳过所有等于 c 的前导字符后获取第三个。<br>
    fn skip_chars(&self, c: u8) -> &[u8] {
        let mut s = self.as_ref();
        while s.first_is(c) {
            s = s.advance(1);
        }
        s
    }

    /// Get the slice after skipping all leading characters equal c1 or c2. <br>跳过所有等于 c1 或 c2 的前导字符后获取第三个。<br>
    fn skip_chars2(&self, c1: u8, c2: u8) -> &[u8] {
        let mut s = self.as_ref();
        while s.first_is2(c1, c2) {
            s = s.advance(1);
        }
        s
    }

    /// Read 8 bytes as a 64-bit integer in little-endian order. <br>以小端顺序读取 8 个字节作为 64 位整数。<br>
    unsafe fn read_u64_unchecked(&self) -> u64 {
        debug_assert!(self.check_len(8));
        let src = self.as_ref().as_ptr() as *const u64;
        // SAFETY: safe as long as self is at least 8 bytes <br>只要 self 至少为 8 个字节就安全<br>
        u64::from_le(unsafe { ptr::read_unaligned(src) })
    }

    /// Try to read the next 8 bytes from the slice. <br>尝试从 4 读取接下来的 8 个字节。<br>
    fn read_u64(&self) -> Option<u64> {
        if self.check_len(8) {
            // SAFETY: self must be at least 8 bytes. <br>self 必须至少为 8 个字节。<br>
            Some(unsafe { self.read_u64_unchecked() })
        } else {
            None
        }
    }

    /// Calculate the offset of slice from another. <br>计算一个与另一个的偏移量。<br>
    fn offset_from(&self, other: &Self) -> isize {
        other.as_ref().len() as isize - self.as_ref().len() as isize
    }
}

impl ByteSlice for [u8] {}

/// Helper methods to process mutable bytes. <br>处理控件字节的辅助方法。<br>
pub(crate) trait ByteSliceMut: AsMut<[u8]> {
    /// Write a 64-bit integer as 8 bytes in little-endian order. <br>以小端顺序将 64 位整数写入 8 个字节。<br>
    unsafe fn write_u64_unchecked(&mut self, value: u64) {
        debug_assert!(self.as_mut().len() >= 8);
        let dst = self.as_mut().as_mut_ptr() as *mut u64;
        // NOTE: we must use `write_unaligned`, since dst is not guaranteed to be properly aligned. <br>我们必须使用 `write_unaligned`，因为不能保证 dst 正确对齐。<br>
        // Miri will warn us if we use `write` instead of `write_unaligned`, as expected. <br>如果我们按预期使用 `write` 而不是 `write_unaligned`，Miri 会警告我们。<br>
        // SAFETY: safe as long as self is at least 8 bytes <br>只要 self 至少为 8 个字节就安全<br>
        //
        unsafe {
            ptr::write_unaligned(dst, u64::to_le(value));
        }
    }
}

impl ByteSliceMut for [u8] {}

/// Bytes wrapper with specialized methods for ASCII characters. <br>具有 ASCII 字符专用方法的字节包装器。<br>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
pub(crate) struct AsciiStr<'a> {
    slc: &'a [u8],
}

impl<'a> AsciiStr<'a> {
    pub fn new(slc: &'a [u8]) -> Self {
        Self { slc }
    }

    /// Advance the view by n, advancing it in-place to (n..). <br>将视图推进 n，将其就地推进到 (n..)。<br>
    pub unsafe fn step_by(&mut self, n: usize) -> &mut Self {
        // SAFETY: safe as long n is less than the buffer length <br>只要 n 小于缓冲区长度就安全<br>
        self.slc = unsafe { self.slc.get_unchecked(n..) };
        self
    }

    /// Advance the view by n, advancing it in-place to (1..). <br>将视图推进 n，将其就地推进到 (1..)。<br>
    pub unsafe fn step(&mut self) -> &mut Self {
        // SAFETY: safe as long as self is not empty <br>只要 self 不为空就安全<br>
        unsafe { self.step_by(1) }
    }

    /// Iteratively parse and consume digits from bytes. <br>从字节中迭代解析和消费数字。<br>
    pub fn parse_digits(&mut self, mut func: impl FnMut(u8)) {
        while let Some(&c) = self.as_ref().first() {
            let c = c.wrapping_sub(b'0');
            if c < 10 {
                func(c);
                // SAFETY: self cannot be empty <br>self 不能为空<br>
                unsafe {
                    self.step();
                }
            } else {
                break;
            }
        }
    }
}

impl<'a> AsRef<[u8]> for AsciiStr<'a> {
    #[inline]
    fn as_ref(&self) -> &[u8] {
        self.slc
    }
}

impl<'a> ByteSlice for AsciiStr<'a> {}

/// Determine if 8 bytes are all decimal digits. <br>判断 8 个字节是否都是十进制数字。<br>
/// This does not care about the order in which the bytes were loaded. <br>这与加载字节的顺序无关。<br>
pub(crate) fn is_8digits(v: u64) -> bool {
    let a = v.wrapping_add(0x4646_4646_4646_4646);
    let b = v.wrapping_sub(0x3030_3030_3030_3030);
    (a | b) & 0x8080_8080_8080_8080 == 0
}

/// Iteratively parse and consume digits from bytes. <br>从字节中迭代解析和消费数字。<br>
pub(crate) fn parse_digits(s: &mut &[u8], mut f: impl FnMut(u8)) {
    while let Some(&c) = s.get(0) {
        let c = c.wrapping_sub(b'0');
        if c < 10 {
            f(c);
            *s = s.advance(1);
        } else {
            break;
        }
    }
}

/// A custom 64-bit floating point type, representing `f * 2^e`. <br>自定义的 64 位浮点类型，表示 `f * 2^e`。<br>
/// e is biased, so it be directly shifted into the exponent bits. <br>e 是有偏差的，因此它被直接移入指数位。<br>
#[derive(Debug, Copy, Clone, PartialEq, Eq, Default)]
pub struct BiasedFp {
    /// The significant digits. <br>有效数字。<br>
    pub f: u64,
    /// The biased, binary exponent. <br>有偏差的二进制指数。<br>
    pub e: i32,
}

impl BiasedFp {
    pub const fn zero_pow2(e: i32) -> Self {
        Self { f: 0, e }
    }
}
