//! Shared utilities used by both float and integer formatting. <br>浮点和整数格式使用的共享实用工具。<br>
#![doc(hidden)]
#![unstable(
    feature = "numfmt",
    reason = "internal routines only exposed for testing",
    issue = "none"
)]

/// Formatted parts. <br>格式化的零件。<br>
#[derive(Copy, Clone, PartialEq, Eq, Debug)]
pub enum Part<'a> {
    /// Given number of zero digits. <br>给定零位数。<br>
    Zero(usize),
    /// A literal number up to 5 digits. <br>字面量数字，最多 5 位。<br>
    Num(u16),
    /// A verbatim copy of given bytes. <br>给定字节的逐字副本。<br>
    Copy(&'a [u8]),
}

impl<'a> Part<'a> {
    /// Returns the exact byte length of given part. <br>返回给定部分的确切字节长度。<br>
    pub fn len(&self) -> usize {
        match *self {
            Part::Zero(nzeroes) => nzeroes,
            Part::Num(v) => {
                if v < 1_000 {
                    if v < 10 {
                        1
                    } else if v < 100 {
                        2
                    } else {
                        3
                    }
                } else {
                    if v < 10_000 { 4 } else { 5 }
                }
            }
            Part::Copy(buf) => buf.len(),
        }
    }

    /// Writes a part into the supplied buffer. <br>将零件写入提供的缓冲区。<br>
    /// Returns the number of written bytes, or `None` if the buffer is not enough. <br>返回写入的字节数，如果缓冲区不足，则返回 `None`。<br>
    /// (It may still leave partially written bytes in the buffer; do not rely on that.) <br>(它可能仍会将部分写入的字节保留在缓冲区中; 不要依赖于此。)<br>
    pub fn write(&self, out: &mut [u8]) -> Option<usize> {
        let len = self.len();
        if out.len() >= len {
            match *self {
                Part::Zero(nzeroes) => {
                    for c in &mut out[..nzeroes] {
                        *c = b'0';
                    }
                }
                Part::Num(mut v) => {
                    for c in out[..len].iter_mut().rev() {
                        *c = b'0' + (v % 10) as u8;
                        v /= 10;
                    }
                }
                Part::Copy(buf) => {
                    out[..buf.len()].copy_from_slice(buf);
                }
            }
            Some(len)
        } else {
            None
        }
    }
}

/// Formatted result containing one or more parts. <br>包含一个或多个部分的格式化结果。<br>
/// This can be written to the byte buffer or converted to the allocated string. <br>可以将其写入字节缓冲区或转换为分配的字符串。<br>
#[allow(missing_debug_implementations)]
#[derive(Clone)]
pub struct Formatted<'a> {
    /// A byte slice representing a sign, either `""`, `"-"` or `"+"`. <br>表示符号 (`""`，`"-"` 或 `"+"`) 的字节切片。<br>
    pub sign: &'static str,
    /// Formatted parts to be rendered after a sign and optional zero padding. <br>在符号和可选的零填充之后要渲染的格式化部分。<br>
    pub parts: &'a [Part<'a>],
}

impl<'a> Formatted<'a> {
    /// Returns the exact byte length of combined formatted result. <br>返回组合格式结果的确切字节长度。<br>
    pub fn len(&self) -> usize {
        let mut len = self.sign.len();
        for part in self.parts {
            len += part.len();
        }
        len
    }

    /// Writes all formatted parts into the supplied buffer. <br>将所有格式化的部分写入提供的缓冲区。<br>
    /// Returns the number of written bytes, or `None` if the buffer is not enough. <br>返回写入的字节数，如果缓冲区不足，则返回 `None`。<br>
    /// (It may still leave partially written bytes in the buffer; do not rely on that.) <br>(它可能仍会将部分写入的字节保留在缓冲区中; 不要依赖于此。)<br>
    pub fn write(&self, out: &mut [u8]) -> Option<usize> {
        if out.len() < self.sign.len() {
            return None;
        }
        out[..self.sign.len()].copy_from_slice(self.sign.as_bytes());

        let mut written = self.sign.len();
        for part in self.parts {
            let len = part.write(&mut out[written..])?;
            written += len;
        }
        Some(written)
    }
}
