//! Optional values. <br>可选值。<br>
//!
//! Type [`Option`] represents an optional value: every [`Option`] is either [`Some`] and contains a value, or [`None`], and does not. <br>类型 [`Option`] 表示一个可选值：每个 [`Option`] 均为 [`Some`] 并包含一个值，或者为 [`None`]，但不包含。<br>
//! [`Option`] types are very common in Rust code, as they have a number of uses: <br>[`Option`] 类型在 Rust 代码中非常常见，因为它们有多种用途：<br>
//!
//! * Initial values <br>初始值<br>
//! * Return values for functions that are not defined over their entire input range (partial functions) <br>未在整个输入范围内定义的函数的返回值 (部分函数)<br>
//! * Return value for otherwise reporting simple errors, where [`None`] is returned on error <br>返回值，用于报告否则将报告简单错误的错误，其中错误返回 [`None`]<br>
//! * Optional struct fields <br>可选的结构体字段<br>
//! * Struct fields that can be loaned or "taken" <br>可借用或 "taken" 的结构体字段<br>
//! * Optional function arguments <br>可选的函数参数<br>
//! * Nullable pointers <br>可空指针<br>
//! * Swapping things out of difficult situations <br>从困难的情况中交换东西<br>
//!
//! [`Option`]s are commonly paired with pattern matching to query the presence of a value and take action, always accounting for the [`None`] case. <br>通常将 [`Option`] 与模式匹配配对，以查询值的存在并采取措施，始终考虑 [`None`] 的情况。<br>
//!
//!
//! ```
//! fn divide(numerator: f64, denominator: f64) -> Option<f64> {
//!     if denominator == 0.0 {
//!         None
//!     } else {
//!         Some(numerator / denominator)
//!     }
//! }
//!
//! // The return value of the function is an option <br>函数的返回值是一个选项<br>
//! let result = divide(2.0, 3.0);
//!
//! // Pattern match to retrieve the value <br>模式匹配以获取值<br>
//! match result {
//!     // The division was valid <br>该划分有效<br>
//!     Some(x) => println!("Result: {x}"),
//!     // The division was invalid <br>划分无效<br>
//!     None    => println!("Cannot divide by 0"),
//! }
//! ```
//!
//!
//!
//!
//!
//
// FIXME: Show how `Option` is used in practice, with lots of methods <br>通过多种方法展示 `Option` 在实践中的使用方式<br>
//
//! # Options and pointers ("nullable" pointers) <br>选项和指针 ("nullable" 指针)<br>
//!
//! Rust's pointer types must always point to a valid location; <br>Rust 的指针类型必须始终指向有效位置。<br> there are no "null" references. <br>没有 "null" 引用。<br> Instead, Rust has *optional* pointers, like the optional owned box, <code>[Option]<[Box\<T>]></code>. <br>相反，Rust 有 *optional* 指针，就像可选的拥有所有权的 box，<code>[Option]<[Box\<T>]></code>。<br>
//!
//! [Box\<T>]: ../../std/boxed/struct.Box.html
//!
//! The following example uses [`Option`] to create an optional box of [`i32`]. <br>以下示例使用 [`Option`] 创建 [`i32`] 的可选 box。<br>
//! Notice that in order to use the inner [`i32`] value, the `check_optional` function first needs to use pattern matching to determine whether the box has a value (i.e., it is [`Some(...)`][`Some`]) or not ([`None`]). <br>注意，为了使用内部的 [`i32`] 值，`check_optional` 函数首先需要使用模式匹配来确定 box 是否有值 (即它是 [`Some(...)`][`Some`]) 或没有 ([`None`])。<br>
//!
//! ```
//! let optional = None;
//! check_optional(optional);
//!
//! let optional = Some(Box::new(9000));
//! check_optional(optional);
//!
//! fn check_optional(optional: Option<Box<i32>>) {
//!     match optional {
//!         Some(p) => println!("has value {p}"),
//!         None => println!("has no value"),
//!     }
//! }
//! ```
//!
//! # Representation
//!
//! Rust guarantees to optimize the following types `T` such that [`Option<T>`] has the same size as `T`: <br>Rust 保证优化以下 `T` 类型，以使 [`Option<T>`] 具有与 `T` 相同的大小：<br>
//!
//! * [`Box<U>`]
//! * `&U`
//! * `&mut U`
//! * `fn`, `extern "C" fn`[^extern_fn]
//! * [`num::NonZero*`]
//! * [`ptr::NonNull<U>`]
//! * `#[repr(transparent)]` struct around one of the types in this list. <br>`#[repr(transparent)]` 结构体围绕此列表中的一种类型。<br>
//!
//! [^extern_fn]: this remains true for any other ABI: `extern "abi" fn` (_e.g._, `extern "system" fn`) <br>[^extern_fn]: 这对于任何其他 ABI 仍然适用: `extern "abi" fn` (例如，`extern "system" fn`)<br>
//!
//! [`Box<U>`]: ../../std/boxed/struct.Box.html
//! [`num::NonZero*`]: crate::num
//! [`ptr::NonNull<U>`]: crate::ptr::NonNull
//!
//! This is called the "null pointer optimization" or NPO. <br>这称为 "空指针优化" 或 NPO。<br>
//!
//! It is further guaranteed that, for the cases above, one can [`mem::transmute`] from all valid values of `T` to `Option<T>` and from `Some::<T>(_)` to `T` (but transmuting `None::<T>` to `T` is undefined behaviour). <br>对于上述情况，可以进一步保证，可以从 `T` 到 `Option<T>` 的所有有效值以及从 `Some::<T>(_)` 到 `T` 的所有有效值 [`mem::transmute`] (但是将 `None::<T>` 转换为 `T` 是未定义的行为)。<br>
//!
//! # Method overview <br>方法概述<br>
//!
//! In addition to working with pattern matching, [`Option`] provides a wide variety of different methods. <br>除了使用模式匹配，[`Option`] 还提供了多种不同的方法。<br>
//!
//! ## Querying the variant <br>查询变体<br>
//!
//! The [`is_some`] and [`is_none`] methods return [`true`] if the [`Option`] is [`Some`] or [`None`], respectively. <br>如果 [`Option`] 分别为 [`Some`] 或 [`None`]，则 [`is_some`] 和 [`is_none`] 方法返回 [`true`]。<br>
//!
//! [`is_none`]: Option::is_none
//! [`is_some`]: Option::is_some
//!
//! ## Adapters for working with references <br>用于处理引用的适配器<br>
//!
//! * [`as_ref`] converts from <code>[&][][Option]\<T></code> to <code>[Option]<[&]T></code> <br>[`as_ref`] 从 <code>[&][][Option]\<T></code> 转换为 <code>[Option]<[&]T></code><br>
//! * [`as_mut`] converts from <code>[&mut] [Option]\<T></code> to <code>[Option]<[&mut] T></code> <br>[`as_mut`] 从 <code>[&mut] [Option]\<T></code> 转换为 <code>[Option]<[&mut] T></code><br>
//! * [`as_deref`] converts from <code>[&][][Option]\<T></code> to <br>[`as_deref`] 从 <code>[&][][Option]\<T></code> 转换为<br>
//!   <code>[Option]<[&]T::[Target]></code>
//! * [`as_deref_mut`] converts from <code>[&mut] [Option]\<T></code> to <br>[`as_deref_mut`] 从 <code>[&mut] [Option]\<T></code> 转换为<br>
//!   <code>[Option]<[&mut] T::[Target]></code>
//! * [`as_pin_ref`] converts from <code>[Pin]<[&][][Option]\<T>></code> to <br>[`as_pin_ref`] 从 <code>[Pin]<[&][][Option]\<T>></code> 转换为<br>
//!   <code>[Option]<[Pin]<[&]T>></code>
//! * [`as_pin_mut`] converts from <code>[Pin]<[&mut] [Option]\<T>></code> to <br>[`as_pin_mut`] 从 <code>[Pin]<[&mut] [Option]\<T>></code> 转换为<br>
//!   <code>[Option]<[Pin]<[&mut] T>></code>
//!
//! [&]: reference "shared reference"
//! [&mut]: reference "mutable reference"
//! [Target]: Deref::Target "ops::Deref::Target"
//! [`as_deref`]: Option::as_deref
//! [`as_deref_mut`]: Option::as_deref_mut
//! [`as_mut`]: Option::as_mut
//! [`as_pin_mut`]: Option::as_pin_mut
//! [`as_pin_ref`]: Option::as_pin_ref
//! [`as_ref`]: Option::as_ref
//!
//! ## Extracting the contained value <br>提取包含的值<br>
//!
//! These methods extract the contained value in an [`Option<T>`] when it is the [`Some`] variant. <br>当它是 [`Some`] 变体时，这些方法提取 [`Option<T>`] 中包含的值。<br> If the [`Option`] is [`None`]: <br>如果 [`Option`] 为 [`None`]：<br>
//!
//! * [`expect`] panics with a provided custom message <br>[`expect`] panics 带有提供的自定义消息<br>
//! * [`unwrap`] panics with a generic message <br>[`unwrap`] panics 带有泛型信息<br>
//! * [`unwrap_or`] returns the provided default value <br>[`unwrap_or`] 返回提供的默认值<br>
//! * [`unwrap_or_default`] returns the default value of the type `T` (which must implement the [`Default`] trait) <br>[`unwrap_or_default`] 返回类型 `T` 的默认值 (必须实现 [`Default`] trait)<br>
//! * [`unwrap_or_else`] returns the result of evaluating the provided function <br>[`unwrap_or_else`] 返回对提供的函数求值的结果<br>
//!
//! [`expect`]: Option::expect
//! [`unwrap`]: Option::unwrap
//! [`unwrap_or`]: Option::unwrap_or
//! [`unwrap_or_default`]: Option::unwrap_or_default
//! [`unwrap_or_else`]: Option::unwrap_or_else
//!
//! ## Transforming contained values <br>转换包含的值<br>
//!
//! These methods transform [`Option`] to [`Result`]: <br>这些方法将 [`Option`] 转换为 [`Result`]：<br>
//!
//! * [`ok_or`] transforms [`Some(v)`] to [`Ok(v)`], and [`None`] to [`Err(err)`] using the provided default `err` value <br>[`ok_or`] 使用提供的默认 `err` 值将 [`Some(v)`] 转换为 [`Ok(v)`]，将 [`None`] 转换为 [`Err(err)`]<br>
//! * [`ok_or_else`] transforms [`Some(v)`] to [`Ok(v)`], and [`None`] to a value of [`Err`] using the provided function <br>[`ok_or_else`] 使用提供的函数将 [`Some(v)`] 转换为 [`Ok(v)`]，并将 [`None`] 转换为 [`Err`] 的值<br>
//! * [`transpose`] transposes an [`Option`] of a [`Result`] into a [`Result`] of an [`Option`]
//!
//! [`Err(err)`]: Err
//! [`Ok(v)`]: Ok
//! [`Some(v)`]: Some
//! [`ok_or`]: Option::ok_or
//! [`ok_or_else`]: Option::ok_or_else
//! [`transpose`]: Option::transpose
//!
//! These methods transform the [`Some`] variant: <br>这些方法转换了 [`Some`] 变体：<br>
//!
//! * [`filter`] calls the provided predicate function on the contained value `t` if the [`Option`] is [`Some(t)`], and returns [`Some(t)`] if the function returns `true`; otherwise, returns [`None`]
//! * [`flatten`] removes one level of nesting from an <br>[`flatten`] 从一个对象中删除一层嵌套<br>
//!   [`Option<Option<T>>`]
//! * [`map`] transforms [`Option<T>`] to [`Option<U>`] by applying the provided function to the contained value of [`Some`] and leaving [`None`] values unchanged <br>[`map`] 通过将提供的函数应用于 [`Some`] 的包含值并保持 [`None`] 值不变，将 [`Option<T>`] 转换为 [`Option<U>`]<br>
//!
//! [`Some(t)`]: Some
//! [`filter`]: Option::filter
//! [`flatten`]: Option::flatten
//! [`map`]: Option::map
//!
//! These methods transform [`Option<T>`] to a value of a possibly different type `U`: <br>这些方法将 [`Option<T>`] 转换为可能不同类型 `U` 的值：<br>
//!
//! * [`map_or`] applies the provided function to the contained value of [`Some`], or returns the provided default value if the [`Option`] is <br>[`map_or`] 将提供的函数应用于 [`Some`] 的包含值，或者如果 [`Option`] 是返回提供的默认值<br>
//!   [`None`]
//! * [`map_or_else`] applies the provided function to the contained value of [`Some`], or returns the result of evaluating the provided fallback function if the [`Option`] is [`None`]
//!
//! [`map_or`]: Option::map_or
//! [`map_or_else`]: Option::map_or_else
//!
//! These methods combine the [`Some`] variants of two [`Option`] values: <br>这些方法结合了两个 [`Option`] 值的 [`Some`] 变体：<br>
//!
//! * [`zip`] returns [`Some((s, o))`] if `self` is [`Some(s)`] and the provided [`Option`] value is [`Some(o)`]; otherwise, returns [`None`]
//! * [`zip_with`] calls the provided function `f` and returns [`Some(f(s, o))`] if `self` is [`Some(s)`] and the provided [`Option`] value is [`Some(o)`]; otherwise, returns [`None`]
//!
//! [`Some(f(s, o))`]: Some
//! [`Some(o)`]: Some
//! [`Some(s)`]: Some
//! [`Some((s, o))`]: Some
//! [`zip`]: Option::zip
//! [`zip_with`]: Option::zip_with
//!
//! ## Boolean operators <br>布尔运算符<br>
//!
//! These methods treat the [`Option`] as a boolean value, where [`Some`] acts like [`true`] and [`None`] acts like [`false`]. <br>这些方法将 [`Option`] 视为布尔值，其中 [`Some`] 的作用类似于 [`true`]，而 [`None`] 的作用类似于 [`false`]。<br> There are two categories of these methods: ones that take an [`Option`] as input, and ones that take a function as input (to be lazily evaluated). <br>这些方法有两类：一类以 [`Option`] 作为输入，一类以函数作为输入 (延迟评估)。<br>
//!
//! The [`and`], [`or`], and [`xor`] methods take another [`Option`] as input, and produce an [`Option`] as output. <br>[`and`]、[`or`] 和 [`xor`] 方法将另一个 [`Option`] 作为输入，并生成一个 [`Option`] 作为输出。<br> Only the [`and`] method can produce an [`Option<U>`] value having a different inner type `U` than [`Option<T>`]. <br>只有 [`and`] 方法可以生成具有与 [`Option<T>`] 不同的内部类型 `U` 的 [`Option<U>`] 值。<br>
//!
//! | method  | self      | input     | output    |
//! |---------|-----------|-----------|-----------|
//! | [`and`] | `None`    | (ignored) | `None`    |
//! | [`and`] | `Some(x)` | `None`    | `None`    |
//! | [`and`] | `Some(x)` | `Some(y)` | `Some(y)` |
//! | [`or`]  | `None`    | `None`    | `None`    |
//! | [`or`]  | `None`    | `Some(y)` | `Some(y)` |
//! | [`or`]  | `Some(x)` | (ignored) | `Some(x)` |
//! | [`xor`] | `None`    | `None`    | `None`    |
//! | [`xor`] | `None`    | `Some(y)` | `Some(y)` |
//! | [`xor`] | `Some(x)` | `None`    | `Some(x)` |
//! | [`xor`] | `Some(x)` | `Some(y)` | `None`    |
//!
//! [`and`]: Option::and
//! [`or`]: Option::or
//! [`xor`]: Option::xor
//!
//! The [`and_then`] and [`or_else`] methods take a function as input, and only evaluate the function when they need to produce a new value. <br>[`and_then`] 和 [`or_else`] 方法将函数作为输入，并且仅在需要产生新值时才评估函数。<br> Only the [`and_then`] method can produce an [`Option<U>`] value having a different inner type `U` than [`Option<T>`]. <br>只有 [`and_then`] 方法可以生成具有与 [`Option<T>`] 不同的内部类型 `U` 的 [`Option<U>`] 值。<br>
//!
//! | method       | self      | function input | function result | output    |
//! |--------------|-----------|----------------|-----------------|-----------|
//! | [`and_then`] | `None`    | (not provided) | (not evaluated) | `None`    |
//! | [`and_then`] | `Some(x)` | `x`            | `None`          | `None`    |
//! | [`and_then`] | `Some(x)` | `x`            | `Some(y)`       | `Some(y)` |
//! | [`or_else`]  | `None`    | (not provided) | `None`          | `None`    |
//! | [`or_else`]  | `None`    | (not provided) | `Some(y)`       | `Some(y)` |
//! | [`or_else`]  | `Some(x)` | (not provided) | (not evaluated) | `Some(x)` |
//!
//! [`and_then`]: Option::and_then
//! [`or_else`]: Option::or_else
//!
//! This is an example of using methods like [`and_then`] and [`or`] in a pipeline of method calls. <br>这是在方法调用管道中使用 [`and_then`] 和 [`or`] 等方法的示例。<br> Early stages of the pipeline pass failure values ([`None`]) through unchanged, and continue processing on success values ([`Some`]). <br>管道的早期阶段通过不变的失败值 ([`None`])，并继续处理成功值 ([`Some`])。<br>
//! Toward the end, [`or`] substitutes an error message if it receives [`None`]. <br>最后，如果 [`or`] 收到 [`None`]，它会替换一条错误消息。<br>
//!
//! ```
//! # use std::collections::BTreeMap;
//! let mut bt = BTreeMap::new();
//! bt.insert(20u8, "foo");
//! bt.insert(42u8, "bar");
//! let res = [0u8, 1, 11, 200, 22]
//!     .into_iter()
//!     .map(|x| {
//!         // `checked_sub()` returns `None` on error <br>`checked_sub()` 出错时返回 `None`<br>
//!         x.checked_sub(1)
//!             // same with `checked_mul()` <br>与 `checked_mul()` 相同<br>
//!             .and_then(|x| x.checked_mul(2))
//!             // `BTreeMap::get` returns `None` on error <br>`BTreeMap::get` 出错时返回 `None`<br>
//!             .and_then(|x| bt.get(&x))
//!             // Substitute an error message if we have `None` so far <br>如果到目前为止我们有 `None`，则替换一条错误消息<br>
//!             .or(Some(&"error!"))
//!             .copied()
//!             // Won't panic because we unconditionally used `Some` above <br>不会 panic 因为我们无条件使用了上面的 `Some`<br>
//!             .unwrap()
//!     })
//!     .collect::<Vec<_>>();
//! assert_eq!(res, ["error!", "error!", "foo", "error!", "bar"]);
//! ```
//!
//! ## Comparison operators <br>比较运算符<br>
//!
//! If `T` implements [`PartialOrd`] then [`Option<T>`] will derive its [`PartialOrd`] implementation. <br>如果 `T` 实现 [`PartialOrd`]，那么 [`Option<T>`] 将派生其 [`PartialOrd`] 实现。<br> With this order, [`None`] compares as less than any [`Some`], and two [`Some`] compare the same way as their contained values would in `T`. <br>使用此顺序，[`None`] 的比较比任何 [`Some`] 都小，两个 [`Some`] 的比较方式与其在 `T` 中包含的值相同。<br>
//! If `T` also implements [`Ord`], then so does [`Option<T>`]. <br>如果 `T` 也实现了 [`Ord`]，那么 [`Option<T>`] 也是如此。<br>
//!
//! ```
//! assert!(None < Some(0));
//! assert!(Some(0) < Some(1));
//! ```
//!
//! ## Iterating over `Option` <br>迭代结束 `Option`<br>
//!
//! An [`Option`] can be iterated over. <br>可以对 [`Option`] 进行迭代。<br> This can be helpful if you need an iterator that is conditionally empty. <br>如果您需要一个条件为空的迭代器，这会很有帮助。<br> The iterator will either produce a single value (when the [`Option`] is [`Some`]), or produce no values (when the [`Option`] is [`None`]). <br>迭代器将产生单个值 (当 [`Option`] 为 [`Some`] 时)，或不产生任何值 (当 [`Option`] 为 [`None`] 时)。<br>
//! For example, [`into_iter`] acts like [`once(v)`] if the [`Option`] is [`Some(v)`], and like [`empty()`] if the [`Option`] is [`None`]. <br>例如，如果 [`Option`] 是 [`Some(v)`]，则 [`into_iter`] 的作用类似于 [`once(v)`]; 如果 [`Option`] 是 [`None`]，则它的作用类似于 [`empty()`]。<br>
//!
//! [`Some(v)`]: Some
//! [`empty()`]: crate::iter::empty
//! [`once(v)`]: crate::iter::once
//!
//! Iterators over [`Option<T>`] come in three types: <br>[`Option<T>`] 上的迭代器分为三种类型：<br>
//!
//! * [`into_iter`] consumes the [`Option`] and produces the contained value <br>[`into_iter`] 消耗 [`Option`] 并产生包含的值<br>
//! * [`iter`] produces an immutable reference of type `&T` to the contained value <br>[`iter`] 对包含的值产生类型为 `&T` 的不可变引用<br>
//! * [`iter_mut`] produces a mutable reference of type `&mut T` to the contained value <br>[`iter_mut`] 产生一个 `&mut T` 类型的可变引用到包含的值<br>
//!
//! [`into_iter`]: Option::into_iter
//! [`iter`]: Option::iter
//! [`iter_mut`]: Option::iter_mut
//!
//! An iterator over [`Option`] can be useful when chaining iterators, for example, to conditionally insert items. <br>[`Option`] 上的迭代器在链接迭代器时很有用，例如，有条件地插入项。<br>
//! (It's not always necessary to explicitly call an iterator constructor: many [`Iterator`] methods that accept other iterators will also accept iterable types that implement [`IntoIterator`], which includes [`Option`].) <br>(并不总是需要显式调用迭代器构造函数：许多接受其他迭代器的 [`Iterator`] 方法也将接受实现 [`IntoIterator`] 的可迭代类型，其中包括 [`Option`]。)<br>
//!
//! ```
//! let yep = Some(42);
//! let nope = None;
//! // chain() already calls into_iter(), so we don't have to do so <br>chain() 已经调用 into_iter()，所以我们不必这样做<br>
//! let nums: Vec<i32> = (0..4).chain(yep).chain(4..8).collect();
//! assert_eq!(nums, [0, 1, 2, 3, 42, 4, 5, 6, 7]);
//! let nums: Vec<i32> = (0..4).chain(nope).chain(4..8).collect();
//! assert_eq!(nums, [0, 1, 2, 3, 4, 5, 6, 7]);
//! ```
//!
//! One reason to chain iterators in this way is that a function returning `impl Iterator` must have all possible return values be of the same concrete type. <br>以这种方式链接迭代器的一个原因是，返回 `impl Iterator` 的函数必须使所有可能的返回值都具有相同的具体类型。<br> Chaining an iterated [`Option`] can help with that. <br>链接一个迭代的 [`Option`] 可以帮助解决这个问题。<br>
//!
//! ```
//! fn make_iter(do_insert: bool) -> impl Iterator<Item = i32> {
//!     // Explicit returns to illustrate return types matching <br>显式返回来说明返回类型匹配<br>
//!     match do_insert {
//!         true => return (0..4).chain(Some(42)).chain(4..8),
//!         false => return (0..4).chain(None).chain(4..8),
//!     }
//! }
//! println!("{:?}", make_iter(true).collect::<Vec<_>>());
//! println!("{:?}", make_iter(false).collect::<Vec<_>>());
//! ```
//!
//! If we try to do the same thing, but using [`once()`] and [`empty()`], we can't return `impl Iterator` anymore because the concrete types of the return values differ. <br>如果我们尝试做同样的事情，但是使用 [`once()`] 和 [`empty()`]，我们就不能再返回 `impl Iterator`，因为返回值的具体类型不同。<br>
//!
//! [`empty()`]: crate::iter::empty
//! [`once()`]: crate::iter::once
//!
//! ```compile_fail,E0308
//! # use std::iter::{empty, once};
//! // This won't compile because all possible returns from the function must have the same concrete type. <br>这不会编译，因为函数的所有可能返回必须具有相同的具体类型。<br>
//! //
//! fn make_iter(do_insert: bool) -> impl Iterator<Item = i32> {
//!     // Explicit returns to illustrate return types not matching <br>显式返回以说明返回类型不匹配<br>
//!     match do_insert {
//!         true => return (0..4).chain(once(42)).chain(4..8),
//!         false => return (0..4).chain(empty()).chain(4..8),
//!     }
//! }
//! ```
//!
//! ## Collecting into `Option` <br>收集到 `Option`<br>
//!
//! [`Option`] implements the [`FromIterator`][impl-FromIterator] trait, which allows an iterator over [`Option`] values to be collected into an [`Option`] of a collection of each contained value of the original [`Option`] values, or [`None`] if any of the elements was [`None`]. <br>[`Option`] 实现了 [`FromIterator`][impl-FromIterator] trait，它允许将 [`Option`] 值上的迭代器收集到原始 [`Option`] 值的每个包含值的集合的 [`Option`] 中，或者如果任何元素是 [`None`]，则为 [`None`]。<br>
//!
//!
//! [impl-FromIterator]: Option#impl-FromIterator%3COption%3CA%3E%3E
//!
//! ```
//! let v = [Some(2), Some(4), None, Some(8)];
//! let res: Option<Vec<_>> = v.into_iter().collect();
//! assert_eq!(res, None);
//! let v = [Some(2), Some(4), Some(8)];
//! let res: Option<Vec<_>> = v.into_iter().collect();
//! assert_eq!(res, Some(vec![2, 4, 8]));
//! ```
//!
//! [`Option`] also implements the [`Product`][impl-Product] and [`Sum`][impl-Sum] traits, allowing an iterator over [`Option`] values to provide the [`product`][Iterator::product] and [`sum`][Iterator::sum] methods. <br>[`Option`] 还实现了 [`Product`][impl-Product] 和 [`Sum`][impl-Sum] traits，允许对 [`Option`] 值的迭代器提供 [`product`][Iterator::product] 和 [`sum`][Iterator::sum] 方法。<br>
//!
//! [impl-Product]: Option#impl-Product%3COption%3CU%3E%3E
//! [impl-Sum]: Option#impl-Sum%3COption%3CU%3E%3E
//!
//! ```
//! let v = [None, Some(1), Some(2), Some(3)];
//! let res: Option<i32> = v.into_iter().sum();
//! assert_eq!(res, None);
//! let v = [Some(1), Some(2), Some(21)];
//! let res: Option<i32> = v.into_iter().product();
//! assert_eq!(res, Some(42));
//! ```
//!
//! ## Modifying an [`Option`] in-place <br>就地修改 [`Option`]<br>
//!
//! These methods return a mutable reference to the contained value of an <br>这些方法返回对包含的值的可变引用<br>
//! [`Option<T>`]:
//!
//! * [`insert`] inserts a value, dropping any old contents <br>[`insert`] 插入一个值，丢弃任何旧内容<br>
//! * [`get_or_insert`] gets the current value, inserting a provided default value if it is [`None`]
//! * [`get_or_insert_default`] gets the current value, inserting the default value of type `T` (which must implement [`Default`]) if it is <br>[`get_or_insert_default`] 获取当前值，如果是，则插入类型 `T` (必须实现 [`Default`]) 的默认值<br>
//!   [`None`]
//! * [`get_or_insert_with`] gets the current value, inserting a default computed by the provided function if it is [`None`]
//!
//! [`get_or_insert`]: Option::get_or_insert
//! [`get_or_insert_default`]: Option::get_or_insert_default
//! [`get_or_insert_with`]: Option::get_or_insert_with
//! [`insert`]: Option::insert
//!
//! These methods transfer ownership of the contained value of an <br>这些方法转移包含的值的所有权<br>
//! [`Option`]:
//!
//! * [`take`] takes ownership of the contained value of an [`Option`], if any, replacing the [`Option`] with [`None`]
//! * [`replace`] takes ownership of the contained value of an [`Option`], if any, replacing the [`Option`] with a [`Some`] containing the provided value <br>[`replace`] 获得 [`Option`] 包含的值的所有权 (如果有)，用包含提供的值的 [`Some`] 替换 [`Option`]<br>
//!
//! [`replace`]: Option::replace
//! [`take`]: Option::take
//!
//! # Examples
//!
//! Basic pattern matching on [`Option`]: <br>[`Option`] 上的基本模式匹配：<br>
//!
//! ```
//! let msg = Some("howdy");
//!
//! // Take a reference to the contained string <br>获取对所包含字符串的引用<br>
//! if let Some(m) = &msg {
//!     println!("{}", *m);
//! }
//!
//! // Remove the contained string, destroying the Option <br>删除包含的字符串，销毁 Option<br>
//! let unwrapped_msg = msg.unwrap_or("default message");
//! ```
//!
//! Initialize a result to [`None`] before a loop: <br>循环前将结果初始化为 [`None`]：<br>
//!
//! ```
//! enum Kingdom { Plant(u32, &'static str), Animal(u32, &'static str) }
//!
//! // A list of data to search through. <br>要搜索的数据列表。<br>
//! let all_the_big_things = [
//!     Kingdom::Plant(250, "redwood"),
//!     Kingdom::Plant(230, "noble fir"),
//!     Kingdom::Plant(229, "sugar pine"),
//!     Kingdom::Animal(25, "blue whale"),
//!     Kingdom::Animal(19, "fin whale"),
//!     Kingdom::Animal(15, "north pacific right whale"),
//! ];
//!
//! // We're going to search for the name of the biggest animal, but to start with we've just got `None`. <br>我们将搜索最大的动物的名称，但首先要获取 `None`。<br>
//! //
//! let mut name_of_biggest_animal = None;
//! let mut size_of_biggest_animal = 0;
//! for big_thing in &all_the_big_things {
//!     match *big_thing {
//!         Kingdom::Animal(size, name) if size > size_of_biggest_animal => {
//!             // Now we've found the name of some big animal <br>现在我们找到了一些大动物的名字<br>
//!             size_of_biggest_animal = size;
//!             name_of_biggest_animal = Some(name);
//!         }
//!         Kingdom::Animal(..) | Kingdom::Plant(..) => ()
//!     }
//! }
//!
//! match name_of_biggest_animal {
//!     Some(name) => println!("the biggest animal is {name}"),
//!     None => println!("there are no animals :("),
//! }
//! ```
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

use crate::iter::{self, FromIterator, FusedIterator, TrustedLen};
use crate::marker::Destruct;
use crate::panicking::{panic, panic_str};
use crate::pin::Pin;
use crate::{
    convert, hint, mem,
    ops::{self, ControlFlow, Deref, DerefMut},
};

/// The `Option` type. <br>`Option` 类型。<br> See [the module level documentation](self) for more. <br>有关更多信息，请参见 [模块级文档](self)。<br>
#[derive(Copy, PartialEq, PartialOrd, Eq, Ord, Debug, Hash)]
#[rustc_diagnostic_item = "Option"]
#[stable(feature = "rust1", since = "1.0.0")]
pub enum Option<T> {
    /// No value. <br>没有值。<br>
    #[lang = "None"]
    #[stable(feature = "rust1", since = "1.0.0")]
    None,
    /// Some value of type `T`. <br>`T` 类型的某些值。<br>
    #[lang = "Some"]
    #[stable(feature = "rust1", since = "1.0.0")]
    Some(#[stable(feature = "rust1", since = "1.0.0")] T),
}

/////////////////////////////////////////////////////////////////////////////
// Type implementation <br>类型实现<br>
/////////////////////////////////////////////////////////////////////////////

impl<T> Option<T> {
    /////////////////////////////////////////////////////////////////////////
    // Querying the contained values <br>查询包含的值<br>
    /////////////////////////////////////////////////////////////////////////

    /// Returns `true` if the option is a [`Some`] value. <br>如果选项是 [`Some`] 值，则返回 `true`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let x: Option<u32> = Some(2);
    /// assert_eq!(x.is_some(), true);
    ///
    /// let x: Option<u32> = None;
    /// assert_eq!(x.is_some(), false);
    /// ```
    #[must_use = "if you intended to assert that this has a value, consider `.unwrap()` instead"]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_stable(feature = "const_option_basics", since = "1.48.0")]
    pub const fn is_some(&self) -> bool {
        matches!(*self, Some(_))
    }

    /// Returns `true` if the option is a [`Some`] and the value inside of it matches a predicate. <br>如果选项是 [`Some`] 并且其中的值与谓词匹配，则返回 `true`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(is_some_with)]
    ///
    /// let x: Option<u32> = Some(2);
    /// assert_eq!(x.is_some_and(|&x| x > 1), true);
    ///
    /// let x: Option<u32> = Some(0);
    /// assert_eq!(x.is_some_and(|&x| x > 1), false);
    ///
    /// let x: Option<u32> = None;
    /// assert_eq!(x.is_some_and(|&x| x > 1), false);
    /// ```
    #[must_use]
    #[inline]
    #[unstable(feature = "is_some_with", issue = "93050")]
    pub fn is_some_and(&self, f: impl FnOnce(&T) -> bool) -> bool {
        matches!(self, Some(x) if f(x))
    }

    /// Returns `true` if the option is a [`None`] value. <br>如果选项是 [`None`] 值，则返回 `true`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let x: Option<u32> = Some(2);
    /// assert_eq!(x.is_none(), false);
    ///
    /// let x: Option<u32> = None;
    /// assert_eq!(x.is_none(), true);
    /// ```
    #[must_use = "if you intended to assert that this doesn't have a value, consider \
                  `.and_then(|_| panic!(\"`Option` had a value when expected `None`\"))` instead"]
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_stable(feature = "const_option_basics", since = "1.48.0")]
    pub const fn is_none(&self) -> bool {
        !self.is_some()
    }

    /////////////////////////////////////////////////////////////////////////
    // Adapter for working with references <br>用于引用的适配器<br>
    /////////////////////////////////////////////////////////////////////////

    /// Converts from `&Option<T>` to `Option<&T>`. <br>从 `&Option<T>` 转换为 `Option<&T>`。<br>
    ///
    /// # Examples
    ///
    /// Converts an <code>Option<[String]></code> into an <code>Option<[usize]></code>, preserving the original. <br>将 <code>Option<[String]></code> 转换为 <code>Option<[usize]></code>，保留原始值。<br>
    /// The [`map`] method takes the `self` argument by value, consuming the original, so this technique uses `as_ref` to first take an `Option` to a reference to the value inside the original. <br>[`map`] 方法按值使用 `self` 参数，从而消耗了原始文件，因此该技术使用 `as_ref` 首先将 `Option` 引用给原始文件中的值。<br>
    ///
    ///
    /// [`map`]: Option::map
    /// [String]: ../../std/string/struct.String.html "String"
    ///
    /// ```
    /// let text: Option<String> = Some("Hello, world!".to_string());
    /// // First, cast `Option<String>` to `Option<&String>` with `as_ref`, then consume *that* with `map`, leaving `text` on the stack. <br>首先，使用 `as_ref` 将 `Option<String>` 转换为 `Option<&String>`，然后在 `map` 上消费它，在栈上留下 `text`。<br>
    /////
    /// let text_length: Option<usize> = text.as_ref().map(|s| s.len());
    /// println!("still can print text: {text:?}");
    /// ```
    ///
    #[inline]
    #[rustc_const_stable(feature = "const_option_basics", since = "1.48.0")]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub const fn as_ref(&self) -> Option<&T> {
        match *self {
            Some(ref x) => Some(x),
            None => None,
        }
    }

    /// Converts from `&mut Option<T>` to `Option<&mut T>`. <br>从 `&mut Option<T>` 转换为 `Option<&mut T>`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut x = Some(2);
    /// match x.as_mut() {
    ///     Some(v) => *v = 42,
    ///     None => {},
    /// }
    /// assert_eq!(x, Some(42));
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option", issue = "67441")]
    pub const fn as_mut(&mut self) -> Option<&mut T> {
        match *self {
            Some(ref mut x) => Some(x),
            None => None,
        }
    }

    /// Converts from <code>[Pin]<[&]Option\<T>></code> to <code>Option<[Pin]<[&]T>></code>. <br>从 <code>[Pin]<[&]Option\<T>></code> 到 <code>Option<[Pin]<[&]T>></code>。<br>
    ///
    /// [&]: reference "shared reference"
    #[inline]
    #[must_use]
    #[stable(feature = "pin", since = "1.33.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn as_pin_ref(self: Pin<&Self>) -> Option<Pin<&T>> {
        match Pin::get_ref(self).as_ref() {
            // SAFETY: `x` is guaranteed to be pinned because it comes from `self` which is pinned. <br>`x` 被固定，因为它来自被固定的 `self`。<br>
            //
            Some(x) => unsafe { Some(Pin::new_unchecked(x)) },
            None => None,
        }
    }

    /// Converts from <code>[Pin]<[&mut] Option\<T>></code> to <code>Option<[Pin]<[&mut] T>></code>. <br>转换自 <code>[Pin]<[&mut] Option\<T>></code> 到 <code>Option<[Pin]<[&mut] T>></code>。<br>
    ///
    /// [&mut]: reference "mutable reference"
    #[inline]
    #[must_use]
    #[stable(feature = "pin", since = "1.33.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn as_pin_mut(self: Pin<&mut Self>) -> Option<Pin<&mut T>> {
        // SAFETY: `get_unchecked_mut` is never used to move the `Option` inside `self`. <br>`get_unchecked_mut` 从未用于在 `self` 内部移动 `Option`。<br>
        // `x` is guaranteed to be pinned because it comes from `self` which is pinned. <br>`x` 被固定，因为它来自被固定的 `self`。<br>
        unsafe {
            match Pin::get_unchecked_mut(self).as_mut() {
                Some(x) => Some(Pin::new_unchecked(x)),
                None => None,
            }
        }
    }

    /////////////////////////////////////////////////////////////////////////
    // Getting to contained values <br>获取包含的值<br>
    /////////////////////////////////////////////////////////////////////////

    /// Returns the contained [`Some`] value, consuming the `self` value. <br>返回包含 `self` 值的包含的 [`Some`] 值。<br>
    ///
    /// # Panics
    ///
    /// Panics if the value is a [`None`] with a custom panic message provided by `msg`. <br>如果值为 [`None`] 并带有由 `msg` 提供的自定义 panic 消息，就会出现 panics。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some("value");
    /// assert_eq!(x.expect("fruits are healthy"), "value");
    /// ```
    ///
    /// ```should_panic
    /// let x: Option<&str> = None;
    /// x.expect("fruits are healthy"); // panics with `fruits are healthy` <br>`fruits are healthy` 的 panics<br>
    /// ```
    ///
    /// # Recommended Message Style <br>推荐的消息样式<br>
    ///
    /// We recommend that `expect` messages are used to describe the reason you _expect_ the `Option` should be `Some`. <br>我们建议使用 `expect` 消息来描述您期望 `Option` 应该是 `Some` 的原因。<br>
    ///
    /// ```should_panic
    /// # let slice: &[u8] = &[];
    /// let item = slice.get(0)
    ///     .expect("slice should not be empty");
    /// ```
    ///
    /// **Hint**: If you're having trouble remembering how to phrase expect error messages remember to focus on the word "should" as in "env variable should be set by blah" or "the given binary should be available and executable by the current user". <br>**提示**: 如果您无法记住如何表达预期错误消息，请记住将注意力集中在 "should" 上，就像在 "env 变量应该由 blah 设置" 或 " 给定的二进制文件应该可由当前用户使用和执行" 中一样。<br>
    ///
    /// For more detail on expect message styles and the reasoning behind our recommendation please refer to the section on ["Common Message Styles"](../../std/error/index.html#common-message-styles) in the [`std::error`](../../std/error/index.html) module docs. <br>有关期望消息样式的更多详细信息以及我们建议背后的原因，请参见 [`std::error`](../../std/error/index.html) 模块文档中有关 ["常见的消息样式"](../../std/error/index.html#common-message-styles) 的部分。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[inline]
    #[track_caller]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option", issue = "67441")]
    pub const fn expect(self, msg: &str) -> T {
        match self {
            Some(val) => val,
            None => expect_failed(msg),
        }
    }

    /// Returns the contained [`Some`] value, consuming the `self` value. <br>返回包含 `self` 值的包含的 [`Some`] 值。<br>
    ///
    /// Because this function may panic, its use is generally discouraged. <br>由于此函数可能为 panic，因此通常不建议使用该函数。<br>
    /// Instead, prefer to use pattern matching and handle the [`None`] case explicitly, or call [`unwrap_or`], [`unwrap_or_else`], or [`unwrap_or_default`]. <br>相反，更喜欢使用模式匹配并显式处理 [`None`] 大小写，或者调用 [`unwrap_or`]，[`unwrap_or_else`] 或 [`unwrap_or_default`]。<br>
    ///
    ///
    /// [`unwrap_or`]: Option::unwrap_or
    /// [`unwrap_or_else`]: Option::unwrap_or_else
    /// [`unwrap_or_default`]: Option::unwrap_or_default
    ///
    /// # Panics
    ///
    /// Panics if the self value equals [`None`]. <br>如果 self 的值等于 [`None`]，就会出现 panics。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some("air");
    /// assert_eq!(x.unwrap(), "air");
    /// ```
    ///
    /// ```should_panic
    /// let x: Option<&str> = None;
    /// assert_eq!(x.unwrap(), "air"); // fails
    /// ```
    ///
    #[inline]
    #[track_caller]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option", issue = "67441")]
    pub const fn unwrap(self) -> T {
        match self {
            Some(val) => val,
            None => panic("called `Option::unwrap()` on a `None` value"),
        }
    }

    /// Returns the contained [`Some`] value or a provided default. <br>返回包含的 [`Some`] 值或提供的默认值。<br>
    ///
    /// Arguments passed to `unwrap_or` are eagerly evaluated; <br>急切地评估传递给 `unwrap_or` 的参数；<br> if you are passing the result of a function call, it is recommended to use [`unwrap_or_else`], which is lazily evaluated. <br>如果要传递函数调用的结果，建议使用 [`unwrap_or_else`]，它是惰性求值的。<br>
    ///
    ///
    /// [`unwrap_or_else`]: Option::unwrap_or_else
    ///
    /// # Examples
    ///
    /// ```
    /// assert_eq!(Some("car").unwrap_or("bike"), "car");
    /// assert_eq!(None.unwrap_or("bike"), "bike");
    /// ```
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn unwrap_or(self, default: T) -> T
    where
        T: ~const Destruct,
    {
        match self {
            Some(x) => x,
            None => default,
        }
    }

    /// Returns the contained [`Some`] value or computes it from a closure. <br>返回包含的 [`Some`] 值或从闭包中计算得出。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let k = 10;
    /// assert_eq!(Some(4).unwrap_or_else(|| 2 * k), 4);
    /// assert_eq!(None.unwrap_or_else(|| 2 * k), 20);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn unwrap_or_else<F>(self, f: F) -> T
    where
        F: ~const FnOnce() -> T,
        F: ~const Destruct,
    {
        match self {
            Some(x) => x,
            None => f(),
        }
    }

    /// Returns the contained [`Some`] value or a default. <br>返回包含的 [`Some`] 值或默认值。<br>
    ///
    /// Consumes the `self` argument then, if [`Some`], returns the contained value, otherwise if [`None`], returns the [default value] for that type. <br>消费 `self` 参数，如果 [`Some`]，则返回所包含的值，否则，如果 [`None`]，则返回该类型的 [默认值][default value]。<br>
    ///
    ///
    /// # Examples
    ///
    /// Converts a string to an integer, turning poorly-formed strings into 0 (the default value for integers). <br>将字符串转换为整数，将格式不正确的字符串转换为 0 (整数的默认值)。<br>
    /// [`parse`] converts a string to any other type that implements [`FromStr`], returning [`None`] on error. <br>[`parse`] 将字符串转换为实现 [`FromStr`] 的任何其他类型，并在出错时返回 [`None`]。<br>
    ///
    /// ```
    /// let good_year_from_input = "1909";
    /// let bad_year_from_input = "190blarg";
    /// let good_year = good_year_from_input.parse().ok().unwrap_or_default();
    /// let bad_year = bad_year_from_input.parse().ok().unwrap_or_default();
    ///
    /// assert_eq!(1909, good_year);
    /// assert_eq!(0, bad_year);
    /// ```
    ///
    /// [default value]: Default::default
    /// [`parse`]: str::parse
    /// [`FromStr`]: crate::str::FromStr
    ///
    ///
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn unwrap_or_default(self) -> T
    where
        T: ~const Default,
    {
        match self {
            Some(x) => x,
            None => Default::default(),
        }
    }

    /// Returns the contained [`Some`] value, consuming the `self` value, without checking that the value is not [`None`]. <br>返回包含 `self` 值的包含的 [`Some`] 值，而不检查该值是否不是 [`None`]。<br>
    ///
    ///
    /// # Safety
    ///
    /// Calling this method on [`None`] is *[undefined behavior]*. <br>在 [`None`] 上调用此方法是 *[undefined behavior]*。<br>
    ///
    /// [undefined behavior]: https://doc.rust-lang.org/reference/behavior-considered-undefined.html
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some("air");
    /// assert_eq!(unsafe { x.unwrap_unchecked() }, "air");
    /// ```
    ///
    /// ```no_run
    /// let x: Option<&str> = None;
    /// assert_eq!(unsafe { x.unwrap_unchecked() }, "air"); // Undefined behavior! <br>未定义的行为！<br>
    /// ```
    #[inline]
    #[track_caller]
    #[stable(feature = "option_result_unwrap_unchecked", since = "1.58.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const unsafe fn unwrap_unchecked(self) -> T {
        debug_assert!(self.is_some());
        match self {
            Some(val) => val,
            // SAFETY: the safety contract must be upheld by the caller. <br>调用者必须坚持安全保证。<br>
            None => unsafe { hint::unreachable_unchecked() },
        }
    }

    /////////////////////////////////////////////////////////////////////////
    // Transforming contained values <br>转换包含的值<br>
    /////////////////////////////////////////////////////////////////////////

    /// Maps an `Option<T>` to `Option<U>` by applying a function to a contained value. <br>通过将函数应用于包含的值，Maps 将 `Option<T>` 转换为 `Option<U>`。<br>
    ///
    /// # Examples
    ///
    /// Converts an <code>Option<[String]></code> into an <code>Option<[usize]></code>, consuming the original: <br>将 <code>Option<[String]></code> 转换为 <code>Option<[usize]></code>，使用原始值：<br>
    ///
    ///
    /// [String]: ../../std/string/struct.String.html "String"
    /// ```
    /// let maybe_some_string = Some(String::from("Hello, World!"));
    /// // `Option::map` takes self *by value*, consuming `maybe_some_string` <br>`Option::map` 按值获取 self，消耗 `maybe_some_string`<br>
    /// let maybe_some_len = maybe_some_string.map(|s| s.len());
    ///
    /// assert_eq!(maybe_some_len, Some(13));
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn map<U, F>(self, f: F) -> Option<U>
    where
        F: ~const FnOnce(T) -> U,
        F: ~const Destruct,
    {
        match self {
            Some(x) => Some(f(x)),
            None => None,
        }
    }

    /// Calls the provided closure with a reference to the contained value (if [`Some`]). <br>使用对包含值的引用调用提供的闭包 (如果 [`Some`])。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(result_option_inspect)]
    ///
    /// let v = vec![1, 2, 3, 4, 5];
    ///
    /// // prints "got: 4" <br>打印 "got: 4"<br>
    /// let x: Option<&usize> = v.get(3).inspect(|x| println!("got: {x}"));
    ///
    /// // prints nothing <br>什么都不打印<br>
    /// let x: Option<&usize> = v.get(5).inspect(|x| println!("got: {x}"));
    /// ```
    #[inline]
    #[unstable(feature = "result_option_inspect", issue = "91345")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn inspect<F>(self, f: F) -> Self
    where
        F: ~const FnOnce(&T),
        F: ~const Destruct,
    {
        if let Some(ref x) = self {
            f(x);
        }

        self
    }

    /// Returns the provided default result (if none), or applies a function to the contained value (if any). <br>返回提供的默认结果 (如果没有)，或将函数应用于包含的值 (如果有)。<br>
    ///
    /// Arguments passed to `map_or` are eagerly evaluated; <br>传递给 `map_or` 的参数会被急切地评估；<br> if you are passing the result of a function call, it is recommended to use [`map_or_else`], which is lazily evaluated. <br>如果要传递函数调用的结果，建议使用 [`map_or_else`]，它是延迟计算的。<br>
    ///
    ///
    /// [`map_or_else`]: Option::map_or_else
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some("foo");
    /// assert_eq!(x.map_or(42, |v| v.len()), 3);
    ///
    /// let x: Option<&str> = None;
    /// assert_eq!(x.map_or(42, |v| v.len()), 42);
    /// ```
    ///
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn map_or<U, F>(self, default: U, f: F) -> U
    where
        F: ~const FnOnce(T) -> U,
        F: ~const Destruct,
        U: ~const Destruct,
    {
        match self {
            Some(t) => f(t),
            None => default,
        }
    }

    /// Computes a default function result (if none), or applies a different function to the contained value (if any). <br>计算 default 函数的结果 (如果没有)，或将不同的函数应用于包含的值 (如果有)。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let k = 21;
    ///
    /// let x = Some("foo");
    /// assert_eq!(x.map_or_else(|| 2 * k, |v| v.len()), 3);
    ///
    /// let x: Option<&str> = None;
    /// assert_eq!(x.map_or_else(|| 2 * k, |v| v.len()), 42);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn map_or_else<U, D, F>(self, default: D, f: F) -> U
    where
        D: ~const FnOnce() -> U,
        D: ~const Destruct,
        F: ~const FnOnce(T) -> U,
        F: ~const Destruct,
    {
        match self {
            Some(t) => f(t),
            None => default(),
        }
    }

    /// Transforms the `Option<T>` into a [`Result<T, E>`], mapping [`Some(v)`] to [`Ok(v)`] and [`None`] to [`Err(err)`]. <br>将 `Option<T>` 转换为 [`Result<T, E>`]，将 [`Some(v)`] 映射到 [`Ok(v)`]，将 [`None`] 映射到 [`Err(err)`]。<br>
    ///
    /// Arguments passed to `ok_or` are eagerly evaluated; <br>急切地评估传递给 `ok_or` 的参数；<br> if you are passing the result of a function call, it is recommended to use [`ok_or_else`], which is lazily evaluated. <br>如果要传递函数调用的结果，建议使用 [`ok_or_else`]，它是延迟计算的。<br>
    ///
    ///
    /// [`Ok(v)`]: Ok
    /// [`Err(err)`]: Err
    /// [`Some(v)`]: Some
    /// [`ok_or_else`]: Option::ok_or_else
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some("foo");
    /// assert_eq!(x.ok_or(0), Ok("foo"));
    ///
    /// let x: Option<&str> = None;
    /// assert_eq!(x.ok_or(0), Err(0));
    /// ```
    ///
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn ok_or<E>(self, err: E) -> Result<T, E>
    where
        E: ~const Destruct,
    {
        match self {
            Some(v) => Ok(v),
            None => Err(err),
        }
    }

    /// Transforms the `Option<T>` into a [`Result<T, E>`], mapping [`Some(v)`] to [`Ok(v)`] and [`None`] to [`Err(err())`]. <br>将 `Option<T>` 转换为 [`Result<T, E>`]，将 [`Some(v)`] 映射到 [`Ok(v)`]，将 [`None`] 映射到 [`Err(err())`]。<br>
    ///
    ///
    /// [`Ok(v)`]: Ok
    /// [`Err(err())`]: Err
    /// [`Some(v)`]: Some
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some("foo");
    /// assert_eq!(x.ok_or_else(|| 0), Ok("foo"));
    ///
    /// let x: Option<&str> = None;
    /// assert_eq!(x.ok_or_else(|| 0), Err(0));
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn ok_or_else<E, F>(self, err: F) -> Result<T, E>
    where
        F: ~const FnOnce() -> E,
        F: ~const Destruct,
    {
        match self {
            Some(v) => Ok(v),
            None => Err(err()),
        }
    }

    /// Converts from `Option<T>` (or `&Option<T>`) to `Option<&T::Target>`. <br>从 `Option<T>` (或 `&Option<T>`) 转换为 `Option<&T::Target>`。<br>
    ///
    /// Leaves the original Option in-place, creating a new one with a reference to the original one, additionally coercing the contents via [`Deref`]. <br>将原始 Option 保留在原位，创建一个带有对原始 Option 的引用的新 Option，并通过 [`Deref`] 强制执行其内容。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let x: Option<String> = Some("hey".to_owned());
    /// assert_eq!(x.as_deref(), Some("hey"));
    ///
    /// let x: Option<String> = None;
    /// assert_eq!(x.as_deref(), None);
    /// ```
    #[stable(feature = "option_deref", since = "1.40.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn as_deref(&self) -> Option<&T::Target>
    where
        T: ~const Deref,
    {
        match self.as_ref() {
            Some(t) => Some(t.deref()),
            None => None,
        }
    }

    /// Converts from `Option<T>` (or `&mut Option<T>`) to `Option<&mut T::Target>`. <br>从 `Option<T>` (或 `&mut Option<T>`) 转换为 `Option<&mut T::Target>`。<br>
    ///
    /// Leaves the original `Option` in-place, creating a new one containing a mutable reference to the inner type's [`Deref::Target`] type. <br>在这里保留原始的 `Option`，创建一个包含对内部类型的 [`Deref::Target`] 类型的可变引用的新的 `Option`。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let mut x: Option<String> = Some("hey".to_owned());
    /// assert_eq!(x.as_deref_mut().map(|x| {
    ///     x.make_ascii_uppercase();
    ///     x
    /// }), Some("HEY".to_owned().as_mut_str()));
    /// ```
    #[stable(feature = "option_deref", since = "1.40.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn as_deref_mut(&mut self) -> Option<&mut T::Target>
    where
        T: ~const DerefMut,
    {
        match self.as_mut() {
            Some(t) => Some(t.deref_mut()),
            None => None,
        }
    }

    /////////////////////////////////////////////////////////////////////////
    // Iterator constructors <br>迭代器构造函数<br>
    /////////////////////////////////////////////////////////////////////////

    /// Returns an iterator over the possibly contained value. <br>返回可能包含的值的迭代器。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some(4);
    /// assert_eq!(x.iter().next(), Some(&4));
    ///
    /// let x: Option<u32> = None;
    /// assert_eq!(x.iter().next(), None);
    /// ```
    #[inline]
    #[rustc_const_unstable(feature = "const_option", issue = "67441")]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub const fn iter(&self) -> Iter<'_, T> {
        Iter { inner: Item { opt: self.as_ref() } }
    }

    /// Returns a mutable iterator over the possibly contained value. <br>返回可能包含的值的可变迭代器。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut x = Some(4);
    /// match x.iter_mut().next() {
    ///     Some(v) => *v = 42,
    ///     None => {},
    /// }
    /// assert_eq!(x, Some(42));
    ///
    /// let mut x: Option<u32> = None;
    /// assert_eq!(x.iter_mut().next(), None);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn iter_mut(&mut self) -> IterMut<'_, T> {
        IterMut { inner: Item { opt: self.as_mut() } }
    }

    /////////////////////////////////////////////////////////////////////////
    // Boolean operations on the values, eager and lazy <br>对值的布尔运算，渴望和懒惰<br>
    /////////////////////////////////////////////////////////////////////////

    /// Returns [`None`] if the option is [`None`], otherwise returns `optb`. <br>如果选项为 [`None`]，则返回 [`None`]; 否则，返回 `optb`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some(2);
    /// let y: Option<&str> = None;
    /// assert_eq!(x.and(y), None);
    ///
    /// let x: Option<u32> = None;
    /// let y = Some("foo");
    /// assert_eq!(x.and(y), None);
    ///
    /// let x = Some(2);
    /// let y = Some("foo");
    /// assert_eq!(x.and(y), Some("foo"));
    ///
    /// let x: Option<u32> = None;
    /// let y: Option<&str> = None;
    /// assert_eq!(x.and(y), None);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn and<U>(self, optb: Option<U>) -> Option<U>
    where
        T: ~const Destruct,
        U: ~const Destruct,
    {
        match self {
            Some(_) => optb,
            None => None,
        }
    }

    /// Returns [`None`] if the option is [`None`], otherwise calls `f` with the wrapped value and returns the result. <br>如果选项为 [`None`]，则返回 [`None`]; 否则，使用包装的值调用 `f`，并返回结果。<br>
    ///
    ///
    /// Some languages call this operation flatmap. <br>一些语言调用此操作平面图。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// fn sq_then_to_string(x: u32) -> Option<String> {
    ///     x.checked_mul(x).map(|sq| sq.to_string())
    /// }
    ///
    /// assert_eq!(Some(2).and_then(sq_then_to_string), Some(4.to_string()));
    /// assert_eq!(Some(1_000_000).and_then(sq_then_to_string), None); // overflowed!
    /// assert_eq!(None.and_then(sq_then_to_string), None);
    /// ```
    ///
    /// Often used to chain fallible operations that may return [`None`]. <br>通常用于链接可能返回 [`None`] 的错误操作。<br>
    ///
    /// ```
    /// let arr_2d = [["A0", "A1"], ["B0", "B1"]];
    ///
    /// let item_0_1 = arr_2d.get(0).and_then(|row| row.get(1));
    /// assert_eq!(item_0_1, Some(&"A1"));
    ///
    /// let item_2_0 = arr_2d.get(2).and_then(|row| row.get(0));
    /// assert_eq!(item_2_0, None);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn and_then<U, F>(self, f: F) -> Option<U>
    where
        F: ~const FnOnce(T) -> Option<U>,
        F: ~const Destruct,
    {
        match self {
            Some(x) => f(x),
            None => None,
        }
    }

    /// Returns [`None`] if the option is [`None`], otherwise calls `predicate` with the wrapped value and returns: <br>如果选项为 [`None`]，则返回 [`None`]; 否则，使用包装的值调用 `predicate` 并返回：<br>
    ///
    ///
    /// - [`Some(t)`] if `predicate` returns `true` (where `t` is the wrapped value), and <br>[`Some(t)`] 如果 `predicate` 返回 `true` (其中 `t` 是包装值)，并且<br>
    /// - [`None`] if `predicate` returns `false`. <br>[`None`] 如果 `predicate` 返回 `false`。<br>
    ///
    /// This function works similar to [`Iterator::filter()`]. <br>该函数的工作方式类似于 [`Iterator::filter()`]。<br>
    /// You can imagine the `Option<T>` being an iterator over one or zero elements. <br>您可以想象 `Option<T>` 是一个或零个元素上的迭代器。<br>
    /// `filter()` lets you decide which elements to keep. <br>`filter()` 让您决定保留哪些元素。<br>
    ///
    /// # Examples
    ///
    /// ```rust
    /// fn is_even(n: &i32) -> bool {
    ///     n % 2 == 0
    /// }
    ///
    /// assert_eq!(None.filter(is_even), None);
    /// assert_eq!(Some(3).filter(is_even), None);
    /// assert_eq!(Some(4).filter(is_even), Some(4));
    /// ```
    ///
    /// [`Some(t)`]: Some
    ///
    #[inline]
    #[stable(feature = "option_filter", since = "1.27.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn filter<P>(self, predicate: P) -> Self
    where
        T: ~const Destruct,
        P: ~const FnOnce(&T) -> bool,
        P: ~const Destruct,
    {
        if let Some(x) = self {
            if predicate(&x) {
                return Some(x);
            }
        }
        None
    }

    /// Returns the option if it contains a value, otherwise returns `optb`. <br>如果包含值，则返回选项，否则返回 `optb`。<br>
    ///
    /// Arguments passed to `or` are eagerly evaluated; <br>传递给 `or` 的参数会被急切地评估；<br> if you are passing the result of a function call, it is recommended to use [`or_else`], which is lazily evaluated. <br>如果要传递函数调用的结果，建议使用 [`or_else`]，它是延迟计算的。<br>
    ///
    ///
    /// [`or_else`]: Option::or_else
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some(2);
    /// let y = None;
    /// assert_eq!(x.or(y), Some(2));
    ///
    /// let x = None;
    /// let y = Some(100);
    /// assert_eq!(x.or(y), Some(100));
    ///
    /// let x = Some(2);
    /// let y = Some(100);
    /// assert_eq!(x.or(y), Some(2));
    ///
    /// let x: Option<u32> = None;
    /// let y = None;
    /// assert_eq!(x.or(y), None);
    /// ```
    ///
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn or(self, optb: Option<T>) -> Option<T>
    where
        T: ~const Destruct,
    {
        match self {
            Some(x) => Some(x),
            None => optb,
        }
    }

    /// Returns the option if it contains a value, otherwise calls `f` and returns the result. <br>如果选项包含值，则返回该选项，否则调用 `f` 并返回结果。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// fn nobody() -> Option<&'static str> { None }
    /// fn vikings() -> Option<&'static str> { Some("vikings") }
    ///
    /// assert_eq!(Some("barbarians").or_else(vikings), Some("barbarians"));
    /// assert_eq!(None.or_else(vikings), Some("vikings"));
    /// assert_eq!(None.or_else(nobody), None);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn or_else<F>(self, f: F) -> Option<T>
    where
        F: ~const FnOnce() -> Option<T>,
        F: ~const Destruct,
    {
        match self {
            Some(x) => Some(x),
            None => f(),
        }
    }

    /// Returns [`Some`] if exactly one of `self`, `optb` is [`Some`], otherwise returns [`None`]. <br>如果 `self`，`optb` 之一恰好是 [`Some`]，则返回 [`Some`]，否则返回 [`None`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some(2);
    /// let y: Option<u32> = None;
    /// assert_eq!(x.xor(y), Some(2));
    ///
    /// let x: Option<u32> = None;
    /// let y = Some(2);
    /// assert_eq!(x.xor(y), Some(2));
    ///
    /// let x = Some(2);
    /// let y = Some(2);
    /// assert_eq!(x.xor(y), None);
    ///
    /// let x: Option<u32> = None;
    /// let y: Option<u32> = None;
    /// assert_eq!(x.xor(y), None);
    /// ```
    #[inline]
    #[stable(feature = "option_xor", since = "1.37.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn xor(self, optb: Option<T>) -> Option<T>
    where
        T: ~const Destruct,
    {
        match (self, optb) {
            (Some(a), None) => Some(a),
            (None, Some(b)) => Some(b),
            _ => None,
        }
    }

    /////////////////////////////////////////////////////////////////////////
    // Entry-like operations to insert a value and return a reference <br>类似条目的操作，插入一个值并返回一个引用<br>
    /////////////////////////////////////////////////////////////////////////

    /// Inserts `value` into the option, then returns a mutable reference to it. <br>将 `value` 插入到选项，然后返回对它的可变引用。<br>
    ///
    /// If the option already contains a value, the old value is dropped. <br>如果该选项已包含值，则将丢弃旧值。<br>
    ///
    /// See also [`Option::get_or_insert`], which doesn't update the value if the option already contains [`Some`]. <br>另请参见 [`Option::get_or_insert`]，如果选项已包含 [`Some`]，则不会更新值。<br>
    ///
    ///
    /// # Example
    ///
    /// ```
    /// let mut opt = None;
    /// let val = opt.insert(1);
    /// assert_eq!(*val, 1);
    /// assert_eq!(opt.unwrap(), 1);
    /// let val = opt.insert(2);
    /// assert_eq!(*val, 2);
    /// *val = 3;
    /// assert_eq!(opt.unwrap(), 3);
    /// ```
    #[must_use = "if you intended to set a value, consider assignment instead"]
    #[inline]
    #[stable(feature = "option_insert", since = "1.53.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn insert(&mut self, value: T) -> &mut T
    where
        T: ~const Destruct,
    {
        *self = Some(value);

        // SAFETY: the code above just filled the option <br>上面的代码刚刚填满了该选项<br>
        unsafe { self.as_mut().unwrap_unchecked() }
    }

    /// Inserts `value` into the option if it is [`None`], then returns a mutable reference to the contained value. <br>如果为 [`None`]，则将 `value` 插入到选项中，然后返回所包含的值的变量引用。<br>
    ///
    ///
    /// See also [`Option::insert`], which updates the value even if the option already contains [`Some`]. <br>另请参见 [`Option::insert`]，即使选项已包含 [`Some`]，它也会更新值。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut x = None;
    ///
    /// {
    ///     let y: &mut u32 = x.get_or_insert(5);
    ///     assert_eq!(y, &5);
    ///
    ///     *y = 7;
    /// }
    ///
    /// assert_eq!(x, Some(7));
    /// ```
    ///
    #[inline]
    #[stable(feature = "option_entry", since = "1.20.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn get_or_insert(&mut self, value: T) -> &mut T
    where
        T: ~const Destruct,
    {
        if let None = *self {
            *self = Some(value);
        }

        // SAFETY: a `None` variant for `self` would have been replaced by a `Some` variant in the code above. <br>在上面的代码中，用于 `self` 的 `None` 变体将被替换为 `Some` 变体。<br>
        //
        unsafe { self.as_mut().unwrap_unchecked() }
    }

    /// Inserts the default value into the option if it is [`None`], then returns a mutable reference to the contained value. <br>如果默认值为 [`None`]，则将其插入选项中，然后将所包含的值返回变量引用。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(option_get_or_insert_default)]
    ///
    /// let mut x = None;
    ///
    /// {
    ///     let y: &mut u32 = x.get_or_insert_default();
    ///     assert_eq!(y, &0);
    ///
    ///     *y = 7;
    /// }
    ///
    /// assert_eq!(x, Some(7));
    /// ```
    #[inline]
    #[unstable(feature = "option_get_or_insert_default", issue = "82901")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn get_or_insert_default(&mut self) -> &mut T
    where
        T: ~const Default,
    {
        const fn default<T: ~const Default>() -> T {
            T::default()
        }

        self.get_or_insert_with(default)
    }

    /// Inserts a value computed from `f` into the option if it is [`None`], then returns a mutable reference to the contained value. <br>如果从 `f` 计算得出的值是 [`None`]，则将其插入选项中，然后将所包含的值返回可变引用。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let mut x = None;
    ///
    /// {
    ///     let y: &mut u32 = x.get_or_insert_with(|| 5);
    ///     assert_eq!(y, &5);
    ///
    ///     *y = 7;
    /// }
    ///
    /// assert_eq!(x, Some(7));
    /// ```
    #[inline]
    #[stable(feature = "option_entry", since = "1.20.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn get_or_insert_with<F>(&mut self, f: F) -> &mut T
    where
        F: ~const FnOnce() -> T,
        F: ~const Destruct,
    {
        if let None = *self {
            // the compiler isn't smart enough to know that we are not dropping a `T` here and wants us to ensure `T` can be dropped at compile time. <br>编译器不够聪明，不知道我们不会在这里丢弃 `T`，而是希望我们确保 `T` 可以在编译时被丢弃。<br>
            //
            mem::forget(mem::replace(self, Some(f())))
        }

        // SAFETY: a `None` variant for `self` would have been replaced by a `Some` variant in the code above. <br>在上面的代码中，用于 `self` 的 `None` 变体将被替换为 `Some` 变体。<br>
        //
        unsafe { self.as_mut().unwrap_unchecked() }
    }

    /////////////////////////////////////////////////////////////////////////
    // Misc
    /////////////////////////////////////////////////////////////////////////

    /// Takes the value out of the option, leaving a [`None`] in its place. <br>从选项中取出值，将 [`None`] 留在其位置。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut x = Some(2);
    /// let y = x.take();
    /// assert_eq!(x, None);
    /// assert_eq!(y, Some(2));
    ///
    /// let mut x: Option<u32> = None;
    /// let y = x.take();
    /// assert_eq!(x, None);
    /// assert_eq!(y, None);
    /// ```
    #[inline]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option", issue = "67441")]
    pub const fn take(&mut self) -> Option<T> {
        // FIXME replace `mem::replace` by `mem::take` when the latter is const ready <br>当后者准备好时，用 `mem::take` 替换 `mem::replace`<br>
        mem::replace(self, None)
    }

    /// Replaces the actual value in the option by the value given in parameter, returning the old value if present, leaving a [`Some`] in its place without deinitializing either one. <br>用参数中给定的值替换选项中的实际值，如果存在则返回旧值，将 [`Some`] 保留在其位置，而不用对其中一个进行初始化。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let mut x = Some(2);
    /// let old = x.replace(5);
    /// assert_eq!(x, Some(5));
    /// assert_eq!(old, Some(2));
    ///
    /// let mut x = None;
    /// let old = x.replace(3);
    /// assert_eq!(x, Some(3));
    /// assert_eq!(old, None);
    /// ```
    ///
    #[inline]
    #[rustc_const_unstable(feature = "const_option", issue = "67441")]
    #[stable(feature = "option_replace", since = "1.31.0")]
    pub const fn replace(&mut self, value: T) -> Option<T> {
        mem::replace(self, Some(value))
    }

    /// Returns `true` if the option is a [`Some`] value containing the given value. <br>如果选项是包含给定值的 [`Some`] 值，则返回 `true`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(option_result_contains)]
    ///
    /// let x: Option<u32> = Some(2);
    /// assert_eq!(x.contains(&2), true);
    ///
    /// let x: Option<u32> = Some(3);
    /// assert_eq!(x.contains(&2), false);
    ///
    /// let x: Option<u32> = None;
    /// assert_eq!(x.contains(&2), false);
    /// ```
    #[must_use]
    #[inline]
    #[unstable(feature = "option_result_contains", issue = "62358")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn contains<U>(&self, x: &U) -> bool
    where
        U: ~const PartialEq<T>,
    {
        match self {
            Some(y) => x.eq(y),
            None => false,
        }
    }

    /// Zips `self` with another `Option`. <br>用另一个 `Option` 压缩 `self`。<br>
    ///
    /// If `self` is `Some(s)` and `other` is `Some(o)`, this method returns `Some((s, o))`. <br>如果 `self` 是 `Some(s)`，而 `other` 是 `Some(o)`，则此方法返回 `Some((s, o))`。<br>
    /// Otherwise, `None` is returned. <br>否则，返回 `None`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some(1);
    /// let y = Some("hi");
    /// let z = None::<u8>;
    ///
    /// assert_eq!(x.zip(y), Some((1, "hi")));
    /// assert_eq!(x.zip(z), None);
    /// ```
    #[stable(feature = "option_zip_option", since = "1.46.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn zip<U>(self, other: Option<U>) -> Option<(T, U)>
    where
        T: ~const Destruct,
        U: ~const Destruct,
    {
        match (self, other) {
            (Some(a), Some(b)) => Some((a, b)),
            _ => None,
        }
    }

    /// Zips `self` and another `Option` with function `f`. <br>使用函数 `f` 压缩 `self` 和另一个 `Option`。<br>
    ///
    /// If `self` is `Some(s)` and `other` is `Some(o)`, this method returns `Some(f(s, o))`. <br>如果 `self` 是 `Some(s)`，而 `other` 是 `Some(o)`，则此方法返回 `Some(f(s, o))`。<br>
    /// Otherwise, `None` is returned. <br>否则，返回 `None`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(option_zip)]
    ///
    /// #[derive(Debug, PartialEq)]
    /// struct Point {
    ///     x: f64,
    ///     y: f64,
    /// }
    ///
    /// impl Point {
    ///     fn new(x: f64, y: f64) -> Self {
    ///         Self { x, y }
    ///     }
    /// }
    ///
    /// let x = Some(17.5);
    /// let y = Some(42.7);
    ///
    /// assert_eq!(x.zip_with(y, Point::new), Some(Point { x: 17.5, y: 42.7 }));
    /// assert_eq!(x.zip_with(None, Point::new), None);
    /// ```
    #[unstable(feature = "option_zip", issue = "70086")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn zip_with<U, F, R>(self, other: Option<U>, f: F) -> Option<R>
    where
        F: ~const FnOnce(T, U) -> R,
        F: ~const Destruct,
        T: ~const Destruct,
        U: ~const Destruct,
    {
        match (self, other) {
            (Some(a), Some(b)) => Some(f(a, b)),
            _ => None,
        }
    }
}

impl<T, U> Option<(T, U)> {
    /// Unzips an option containing a tuple of two options. <br>解压缩包含两个选项的元组的选项。<br>
    ///
    /// If `self` is `Some((a, b))` this method returns `(Some(a), Some(b))`. <br>如果 `self` 是 `Some((a, b))`，则此方法返回 `(Some(a), Some(b))`。<br>
    /// Otherwise, `(None, None)` is returned. <br>否则，返回 `(None, None)`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(unzip_option)]
    ///
    /// let x = Some((1, "hi"));
    /// let y = None::<(u8, u32)>;
    ///
    /// assert_eq!(x.unzip(), (Some(1), Some("hi")));
    /// assert_eq!(y.unzip(), (None, None));
    /// ```
    #[inline]
    #[unstable(feature = "unzip_option", issue = "87800", reason = "recently added")]
    pub const fn unzip(self) -> (Option<T>, Option<U>) {
        match self {
            Some((a, b)) => (Some(a), Some(b)),
            None => (None, None),
        }
    }
}

impl<T> Option<&T> {
    /// Maps an `Option<&T>` to an `Option<T>` by copying the contents of the option. <br>通过复制选项的内容将 `Option<&T>` 的 Maps 转换为 `Option<T>`。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let x = 12;
    /// let opt_x = Some(&x);
    /// assert_eq!(opt_x, Some(&12));
    /// let copied = opt_x.copied();
    /// assert_eq!(copied, Some(12));
    /// ```
    #[must_use = "`self` will be dropped if the result is not used"]
    #[stable(feature = "copied", since = "1.35.0")]
    #[rustc_const_unstable(feature = "const_option", issue = "67441")]
    pub const fn copied(self) -> Option<T>
    where
        T: Copy,
    {
        // FIXME: this implementation, which sidesteps using `Option::map` since it's not const ready yet, should be reverted when possible to avoid code repetition <br>此实现避开使用 `Option::map`，因为它尚未准备好常量，应尽可能还原以避免代码重复<br>
        //
        match self {
            Some(&v) => Some(v),
            None => None,
        }
    }

    /// Maps an `Option<&T>` to an `Option<T>` by cloning the contents of the option. <br>通过克隆选项的内容将 `Option<&T>` Maps 转换为 `Option<T>`。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let x = 12;
    /// let opt_x = Some(&x);
    /// assert_eq!(opt_x, Some(&12));
    /// let cloned = opt_x.cloned();
    /// assert_eq!(cloned, Some(12));
    /// ```
    #[must_use = "`self` will be dropped if the result is not used"]
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_unstable(feature = "const_option_cloned", issue = "91582")]
    pub const fn cloned(self) -> Option<T>
    where
        T: ~const Clone,
    {
        match self {
            Some(t) => Some(t.clone()),
            None => None,
        }
    }
}

impl<T> Option<&mut T> {
    /// Maps an `Option<&mut T>` to an `Option<T>` by copying the contents of the option. <br>通过复制选项的内容将 `Option<&mut T>` 的 Maps 转换为 `Option<T>`。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let mut x = 12;
    /// let opt_x = Some(&mut x);
    /// assert_eq!(opt_x, Some(&mut 12));
    /// let copied = opt_x.copied();
    /// assert_eq!(copied, Some(12));
    /// ```
    #[must_use = "`self` will be dropped if the result is not used"]
    #[stable(feature = "copied", since = "1.35.0")]
    #[rustc_const_unstable(feature = "const_option_ext", issue = "91930")]
    pub const fn copied(self) -> Option<T>
    where
        T: Copy,
    {
        match self {
            Some(&mut t) => Some(t),
            None => None,
        }
    }

    /// Maps an `Option<&mut T>` to an `Option<T>` by cloning the contents of the option. <br>通过克隆选项的内容将 `Option<&mut T>` Maps 转换为 `Option<T>`。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// let mut x = 12;
    /// let opt_x = Some(&mut x);
    /// assert_eq!(opt_x, Some(&mut 12));
    /// let cloned = opt_x.cloned();
    /// assert_eq!(cloned, Some(12));
    /// ```
    #[must_use = "`self` will be dropped if the result is not used"]
    #[stable(since = "1.26.0", feature = "option_ref_mut_cloned")]
    #[rustc_const_unstable(feature = "const_option_cloned", issue = "91582")]
    pub const fn cloned(self) -> Option<T>
    where
        T: ~const Clone,
    {
        match self {
            Some(t) => Some(t.clone()),
            None => None,
        }
    }
}

impl<T, E> Option<Result<T, E>> {
    /// Transposes an `Option` of a [`Result`] into a [`Result`] of an `Option`. <br>将 [`Result`] 的 `Option` 转换为 `Option` 的 [`Result`]。<br>
    ///
    /// [`None`] will be mapped to <code>[Ok]\([None])</code>. <br>[`None`] 将映射到 <code>[Ok]\([None])</code>。<br>
    /// <code>[Some]\([Ok]\(\_))</code> and <code>[Some]\([Err]\(\_))</code> will be mapped to <code>[Ok]\([Some]\(\_))</code> and <code>[Err]\(\_)</code>. <br><code>[Some]\([Ok]\(\_))</code> 和 <code>[Some]\([Err]\(\_))</code> 将映射到 <code>[Ok]\([Some]\(\_))</code> 和 <code>[Err]\(\_)</code>。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #[derive(Debug, Eq, PartialEq)]
    /// struct SomeErr;
    ///
    /// let x: Result<Option<i32>, SomeErr> = Ok(Some(5));
    /// let y: Option<Result<i32, SomeErr>> = Some(Ok(5));
    /// assert_eq!(x, y.transpose());
    /// ```
    #[inline]
    #[stable(feature = "transpose_result", since = "1.33.0")]
    #[rustc_const_unstable(feature = "const_option", issue = "67441")]
    pub const fn transpose(self) -> Result<Option<T>, E> {
        match self {
            Some(Ok(x)) => Ok(Some(x)),
            Some(Err(e)) => Err(e),
            None => Ok(None),
        }
    }
}

// This is a separate function to reduce the code size of .expect() itself. <br>这是一个单独的函数，可以减少 .expect() 本身的代码大小。<br>
#[cfg_attr(not(feature = "panic_immediate_abort"), inline(never))]
#[cfg_attr(feature = "panic_immediate_abort", inline)]
#[cold]
#[track_caller]
#[rustc_const_unstable(feature = "const_option", issue = "67441")]
const fn expect_failed(msg: &str) -> ! {
    panic_str(msg)
}

/////////////////////////////////////////////////////////////////////////////
// Trait implementations <br>trait 实现<br>
/////////////////////////////////////////////////////////////////////////////

#[stable(feature = "rust1", since = "1.0.0")]
#[rustc_const_unstable(feature = "const_clone", issue = "91805")]
impl<T> const Clone for Option<T>
where
    T: ~const Clone + ~const Destruct,
{
    #[inline]
    fn clone(&self) -> Self {
        match self {
            Some(x) => Some(x.clone()),
            None => None,
        }
    }

    #[inline]
    fn clone_from(&mut self, source: &Self) {
        match (self, source) {
            (Some(to), Some(from)) => to.clone_from(from),
            (to, from) => *to = from.clone(),
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
#[rustc_const_unstable(feature = "const_default_impls", issue = "87864")]
impl<T> const Default for Option<T> {
    /// Returns [`None`][Option::None]. <br>返回 [`None`][Option::None]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let opt: Option<u32> = Option::default();
    /// assert!(opt.is_none());
    /// ```
    #[inline]
    fn default() -> Option<T> {
        None
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> IntoIterator for Option<T> {
    type Item = T;
    type IntoIter = IntoIter<T>;

    /// Returns a consuming iterator over the possibly contained value. <br>返回可能包含的值上的消耗迭代器。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let x = Some("string");
    /// let v: Vec<&str> = x.into_iter().collect();
    /// assert_eq!(v, ["string"]);
    ///
    /// let x = None;
    /// let v: Vec<&str> = x.into_iter().collect();
    /// assert!(v.is_empty());
    /// ```
    #[inline]
    fn into_iter(self) -> IntoIter<T> {
        IntoIter { inner: Item { opt: self } }
    }
}

#[stable(since = "1.4.0", feature = "option_iter")]
impl<'a, T> IntoIterator for &'a Option<T> {
    type Item = &'a T;
    type IntoIter = Iter<'a, T>;

    fn into_iter(self) -> Iter<'a, T> {
        self.iter()
    }
}

#[stable(since = "1.4.0", feature = "option_iter")]
impl<'a, T> IntoIterator for &'a mut Option<T> {
    type Item = &'a mut T;
    type IntoIter = IterMut<'a, T>;

    fn into_iter(self) -> IterMut<'a, T> {
        self.iter_mut()
    }
}

#[stable(since = "1.12.0", feature = "option_from")]
#[rustc_const_unstable(feature = "const_convert", issue = "88674")]
impl<T> const From<T> for Option<T> {
    /// Moves `val` into a new [`Some`]. <br>将 `val` 移动到新的 [`Some`] 中。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let o: Option<u8> = Option::from(67);
    ///
    /// assert_eq!(Some(67), o);
    /// ```
    fn from(val: T) -> Option<T> {
        Some(val)
    }
}

#[stable(feature = "option_ref_from_ref_option", since = "1.30.0")]
#[rustc_const_unstable(feature = "const_convert", issue = "88674")]
impl<'a, T> const From<&'a Option<T>> for Option<&'a T> {
    /// Converts from `&Option<T>` to `Option<&T>`. <br>从 `&Option<T>` 转换为 `Option<&T>`。<br>
    ///
    /// # Examples
    ///
    /// Converts an <code>[Option]<[String]></code> into an <code>[Option]<[usize]></code>, preserving the original. <br>将 <code>[Option]<[String]></code> 转换为 <code>[Option]<[usize]></code>，保留原始值。<br>
    /// The [`map`] method takes the `self` argument by value, consuming the original, so this technique uses `from` to first take an [`Option`] to a reference to the value inside the original. <br>[`map`] 方法按值取 `self` 参数，消耗原始值，因此该技术使用 `from` 首先将 [`Option`] 用于对原始值内部值的引用。<br>
    ///
    ///
    /// [`map`]: Option::map
    /// [String]: ../../std/string/struct.String.html "String"
    ///
    /// ```
    /// let s: Option<String> = Some(String::from("Hello, Rustaceans!"));
    /// let o: Option<usize> = Option::from(&s).map(|ss: &String| ss.len());
    ///
    /// println!("Can still print s: {s:?}");
    ///
    /// assert_eq!(o, Some(18));
    /// ```
    ///
    fn from(o: &'a Option<T>) -> Option<&'a T> {
        o.as_ref()
    }
}

#[stable(feature = "option_ref_from_ref_option", since = "1.30.0")]
#[rustc_const_unstable(feature = "const_convert", issue = "88674")]
impl<'a, T> const From<&'a mut Option<T>> for Option<&'a mut T> {
    /// Converts from `&mut Option<T>` to `Option<&mut T>` <br>从 `&mut Option<T>` 转换为 `Option<&mut T>`<br>
    ///
    /// # Examples
    ///
    /// ```
    /// let mut s = Some(String::from("Hello"));
    /// let o: Option<&mut String> = Option::from(&mut s);
    ///
    /// match o {
    ///     Some(t) => *t = String::from("Hello, Rustaceans!"),
    ///     None => (),
    /// }
    ///
    /// assert_eq!(s, Some(String::from("Hello, Rustaceans!")));
    /// ```
    fn from(o: &'a mut Option<T>) -> Option<&'a mut T> {
        o.as_mut()
    }
}

/////////////////////////////////////////////////////////////////////////////
// The Option Iterators <br>Option 迭代器<br>
/////////////////////////////////////////////////////////////////////////////

#[derive(Clone, Debug)]
struct Item<A> {
    opt: Option<A>,
}

impl<A> Iterator for Item<A> {
    type Item = A;

    #[inline]
    fn next(&mut self) -> Option<A> {
        self.opt.take()
    }

    #[inline]
    fn size_hint(&self) -> (usize, Option<usize>) {
        match self.opt {
            Some(_) => (1, Some(1)),
            None => (0, Some(0)),
        }
    }
}

impl<A> DoubleEndedIterator for Item<A> {
    #[inline]
    fn next_back(&mut self) -> Option<A> {
        self.opt.take()
    }
}

impl<A> ExactSizeIterator for Item<A> {}
impl<A> FusedIterator for Item<A> {}
unsafe impl<A> TrustedLen for Item<A> {}

/// An iterator over a reference to the [`Some`] variant of an [`Option`]. <br>对 [`Option`] 的 [`Some`] 变体的引用的迭代器。<br>
///
/// The iterator yields one value if the [`Option`] is a [`Some`], otherwise none. <br>如果 [`Option`] 为 [`Some`]，则迭代器产生一个值，否则为 0。<br>
///
/// This `struct` is created by the [`Option::iter`] function. <br>该 `struct` 由 [`Option::iter`] 函数创建。<br>
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Iter<'a, A: 'a> {
    inner: Item<&'a A>,
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, A> Iterator for Iter<'a, A> {
    type Item = &'a A;

    #[inline]
    fn next(&mut self) -> Option<&'a A> {
        self.inner.next()
    }
    #[inline]
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.inner.size_hint()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, A> DoubleEndedIterator for Iter<'a, A> {
    #[inline]
    fn next_back(&mut self) -> Option<&'a A> {
        self.inner.next_back()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<A> ExactSizeIterator for Iter<'_, A> {}

#[stable(feature = "fused", since = "1.26.0")]
impl<A> FusedIterator for Iter<'_, A> {}

#[unstable(feature = "trusted_len", issue = "37572")]
unsafe impl<A> TrustedLen for Iter<'_, A> {}

#[stable(feature = "rust1", since = "1.0.0")]
impl<A> Clone for Iter<'_, A> {
    #[inline]
    fn clone(&self) -> Self {
        Iter { inner: self.inner.clone() }
    }
}

/// An iterator over a mutable reference to the [`Some`] variant of an [`Option`]. <br>对 [`Option`] 的 [`Some`] 变体的可变引用的迭代器。<br>
///
/// The iterator yields one value if the [`Option`] is a [`Some`], otherwise none. <br>如果 [`Option`] 为 [`Some`]，则迭代器产生一个值，否则为 0。<br>
///
/// This `struct` is created by the [`Option::iter_mut`] function. <br>该 `struct` 由 [`Option::iter_mut`] 函数创建。<br>
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct IterMut<'a, A: 'a> {
    inner: Item<&'a mut A>,
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, A> Iterator for IterMut<'a, A> {
    type Item = &'a mut A;

    #[inline]
    fn next(&mut self) -> Option<&'a mut A> {
        self.inner.next()
    }
    #[inline]
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.inner.size_hint()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, A> DoubleEndedIterator for IterMut<'a, A> {
    #[inline]
    fn next_back(&mut self) -> Option<&'a mut A> {
        self.inner.next_back()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<A> ExactSizeIterator for IterMut<'_, A> {}

#[stable(feature = "fused", since = "1.26.0")]
impl<A> FusedIterator for IterMut<'_, A> {}
#[unstable(feature = "trusted_len", issue = "37572")]
unsafe impl<A> TrustedLen for IterMut<'_, A> {}

/// An iterator over the value in [`Some`] variant of an [`Option`]. <br>对 [`Option`] 的 [`Some`] 变体中的值的迭代器。<br>
///
/// The iterator yields one value if the [`Option`] is a [`Some`], otherwise none. <br>如果 [`Option`] 为 [`Some`]，则迭代器产生一个值，否则为 0。<br>
///
/// This `struct` is created by the [`Option::into_iter`] function. <br>该 `struct` 由 [`Option::into_iter`] 函数创建。<br>
#[derive(Clone, Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct IntoIter<A> {
    inner: Item<A>,
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<A> Iterator for IntoIter<A> {
    type Item = A;

    #[inline]
    fn next(&mut self) -> Option<A> {
        self.inner.next()
    }
    #[inline]
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.inner.size_hint()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<A> DoubleEndedIterator for IntoIter<A> {
    #[inline]
    fn next_back(&mut self) -> Option<A> {
        self.inner.next_back()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<A> ExactSizeIterator for IntoIter<A> {}

#[stable(feature = "fused", since = "1.26.0")]
impl<A> FusedIterator for IntoIter<A> {}

#[unstable(feature = "trusted_len", issue = "37572")]
unsafe impl<A> TrustedLen for IntoIter<A> {}

/////////////////////////////////////////////////////////////////////////////
// FromIterator
/////////////////////////////////////////////////////////////////////////////

#[stable(feature = "rust1", since = "1.0.0")]
impl<A, V: FromIterator<A>> FromIterator<Option<A>> for Option<V> {
    /// Takes each element in the [`Iterator`]: if it is [`None`][Option::None], no further elements are taken, and the [`None`][Option::None] is returned. <br>接受 [`Iterator`] 中的每个元素：如果为 [`None`][Option::None]，则不再获取其他元素，并返回 [`None`][Option::None]。<br>
    /// Should no [`None`][Option::None] occur, a container of type `V` containing the values of each [`Option`] is returned. <br>如果没有出现 [`None`][Option::None]，则返回一个 `V` 类型的容器，其中包含每个 [`Option`] 的值。<br>
    ///
    /// # Examples
    ///
    /// Here is an example which increments every integer in a vector. <br>这是一个使 vector 中的每个整数递增的示例。<br>
    /// We use the checked variant of `add` that returns `None` when the calculation would result in an overflow. <br>当计算将导致溢出时，我们使用 `add` 的检查变体返回 `None`。<br>
    ///
    /// ```
    /// let items = vec![0_u16, 1, 2];
    ///
    /// let res: Option<Vec<u16>> = items
    ///     .iter()
    ///     .map(|x| x.checked_add(1))
    ///     .collect();
    ///
    /// assert_eq!(res, Some(vec![1, 2, 3]));
    /// ```
    ///
    /// As you can see, this will return the expected, valid items. <br>如您所见，这将返回预期的有效项。<br>
    ///
    /// Here is another example that tries to subtract one from another list of integers, this time checking for underflow: <br>这是另一个示例，尝试从另一个整数列表中减去一个，这次检查下溢：<br>
    ///
    /// ```
    /// let items = vec![2_u16, 1, 0];
    ///
    /// let res: Option<Vec<u16>> = items
    ///     .iter()
    ///     .map(|x| x.checked_sub(1))
    ///     .collect();
    ///
    /// assert_eq!(res, None);
    /// ```
    ///
    /// Since the last element is zero, it would underflow. <br>由于最后一个元素为零，因此会下溢。<br> Thus, the resulting value is `None`. <br>因此，结果值为 `None`。<br>
    ///
    /// Here is a variation on the previous example, showing that no further elements are taken from `iter` after the first `None`. <br>这是前一个示例的变体，显示在第一个 `None` 之后不再从 `iter` 提取其他元素。<br>
    ///
    /// ```
    /// let items = vec![3_u16, 2, 1, 10];
    ///
    /// let mut shared = 0;
    ///
    /// let res: Option<Vec<u16>> = items
    ///     .iter()
    ///     .map(|x| { shared += x; x.checked_sub(2) })
    ///     .collect();
    ///
    /// assert_eq!(res, None);
    /// assert_eq!(shared, 6);
    /// ```
    ///
    /// Since the third element caused an underflow, no further elements were taken, so the final value of `shared` is 6 (= `3 + 2 + 1`), not 16. <br>由于第三个元素引起下溢，因此不再使用其他元素，因此 `shared` 的最终值为 6 (= `3 + 2 + 1`)，而不是 16。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[inline]
    fn from_iter<I: IntoIterator<Item = Option<A>>>(iter: I) -> Option<V> {
        // FIXME(#11084): This could be replaced with Iterator::scan when this performance bug is closed. <br>当此性能错误关闭时，这可以替换为 Iterator::scan。<br>
        //

        iter::try_process(iter.into_iter(), |i| i.collect())
    }
}

#[unstable(feature = "try_trait_v2", issue = "84277")]
#[rustc_const_unstable(feature = "const_convert", issue = "88674")]
impl<T> const ops::Try for Option<T> {
    type Output = T;
    type Residual = Option<convert::Infallible>;

    #[inline]
    fn from_output(output: Self::Output) -> Self {
        Some(output)
    }

    #[inline]
    fn branch(self) -> ControlFlow<Self::Residual, Self::Output> {
        match self {
            Some(v) => ControlFlow::Continue(v),
            None => ControlFlow::Break(None),
        }
    }
}

#[unstable(feature = "try_trait_v2", issue = "84277")]
#[rustc_const_unstable(feature = "const_convert", issue = "88674")]
impl<T> const ops::FromResidual for Option<T> {
    #[inline]
    fn from_residual(residual: Option<convert::Infallible>) -> Self {
        match residual {
            None => None,
        }
    }
}

#[unstable(feature = "try_trait_v2_yeet", issue = "96374")]
impl<T> ops::FromResidual<ops::Yeet<()>> for Option<T> {
    #[inline]
    fn from_residual(ops::Yeet(()): ops::Yeet<()>) -> Self {
        None
    }
}

#[unstable(feature = "try_trait_v2_residual", issue = "91285")]
impl<T> ops::Residual<T> for Option<convert::Infallible> {
    type TryType = Option<T>;
}

impl<T> Option<Option<T>> {
    /// Converts from `Option<Option<T>>` to `Option<T>`. <br>从 `Option<Option<T>>` 转换为 `Option<T>`。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// let x: Option<Option<u32>> = Some(Some(6));
    /// assert_eq!(Some(6), x.flatten());
    ///
    /// let x: Option<Option<u32>> = Some(None);
    /// assert_eq!(None, x.flatten());
    ///
    /// let x: Option<Option<u32>> = None;
    /// assert_eq!(None, x.flatten());
    /// ```
    ///
    /// Flattening only removes one level of nesting at a time: <br>展平一次只能删除一层嵌套：<br>
    ///
    /// ```
    /// let x: Option<Option<Option<u32>>> = Some(Some(Some(6)));
    /// assert_eq!(Some(Some(6)), x.flatten());
    /// assert_eq!(Some(6), x.flatten().flatten());
    /// ```
    #[inline]
    #[stable(feature = "option_flattening", since = "1.40.0")]
    #[rustc_const_unstable(feature = "const_option", issue = "67441")]
    pub const fn flatten(self) -> Option<T> {
        match self {
            Some(inner) => inner,
            None => None,
        }
    }
}
