use crate::convert::From;
use crate::fmt;
use crate::marker::{PhantomData, Unsize};
use crate::ops::{CoerceUnsized, DispatchFromDyn};
use crate::ptr::NonNull;

/// A wrapper around a raw non-null `*mut T` that indicates that the possessor of this wrapper owns the referent. <br>原始非空 `*mut T` 周围的包装器，指示该包装器的拥有者拥有引用对象。<br>
/// Useful for building abstractions like `Box<T>`, `Vec<T>`, `String`, and `HashMap<K, V>`. <br>对于构建 `Box<T>`，`Vec<T>`，`String` 和 `HashMap<K, V>` 等抽象很有用。<br>
///
/// Unlike `*mut T`, `Unique<T>` behaves "as if" it were an instance of `T`. <br>与 `*mut T` 不同，`Unique<T>` 的行为与 "as if" 相同，它是 `T` 的实例。<br>
/// It implements `Send`/`Sync` if `T` is `Send`/`Sync`. <br>如果 `T` 为 `Send`/`Sync`，则实现 `Send`/`Sync`。<br>
/// It also implies the kind of strong aliasing guarantees an instance of `T` can expect: <br>这也暗示了 `T` 实例可以期望的那种强别名保证：<br>
/// the referent of the pointer should not be modified without a unique path to its owning Unique. <br>如果没有指向其所属 `unique` 的唯一路径，则不应修改指针的引用对象。<br>
///
/// If you're uncertain of whether it's correct to use `Unique` for your purposes, consider using `NonNull`, which has weaker semantics. <br>如果不确定使用 `Unique` 是否正确，请考虑使用语义较弱的 `NonNull`。<br>
///
///
/// Unlike `*mut T`, the pointer must always be non-null, even if the pointer is never dereferenced. <br>与 `*mut T` 不同，即使从未解引用指针，指针也必须始终为非 null。<br>
/// This is so that enums may use this forbidden value as a discriminant -- `Option<Unique<T>>` has the same size as `Unique<T>`. <br>这样一来，枚举就可以将此禁止值用作判别式 - `Option<Unique<T>>` 与 `Unique<T>` 具有相同的大小。<br>
/// However the pointer may still dangle if it isn't dereferenced. <br>但是，如果指针未解引用，它可能仍会悬垂。<br>
///
/// Unlike `*mut T`, `Unique<T>` is covariant over `T`. <br>与 `*mut T` 不同，`Unique<T>` 在 `T` 上是协变的。<br>
/// This should always be correct for any type which upholds Unique's aliasing requirements. <br>对于任何符合 `Unique` 别名要求的类型，这应该总是正确的。<br>
///
///
///
#[unstable(
    feature = "ptr_internals",
    issue = "none",
    reason = "use `NonNull` instead and consider `PhantomData<T>` \
              (if you also use `#[may_dangle]`), `Send`, and/or `Sync`"
)]
#[doc(hidden)]
#[repr(transparent)]
pub struct Unique<T: ?Sized> {
    pointer: NonNull<T>,
    // NOTE: this marker has no consequences for variance, but is necessary for dropck to understand that we logically own a `T`. <br>此标记不会对差异产生任何影响，但对于 dropck 来说，了解我们在逻辑上拥有 `T` 是必需的。<br>
    //
    //
    // For details, see: <br>有关详细信息，请参见：<br>
    // https://github.com/rust-lang/rfcs/blob/master/text/0769-sound-generic-drop.md#phantom-data
    _marker: PhantomData<T>,
}

/// `Unique` pointers are `Send` if `T` is `Send` because the data they reference is unaliased. <br>如果 `T` 是 `Send`，则 `Unique` 指针是 `Send`，因为它们引用的数据是无别名的。<br>
/// Note that this aliasing invariant is unenforced by the type system; <br>请注意，类型系统不强制使用此别名不变量。<br> the abstraction using the `Unique` must enforce it. <br>使用 `Unique` 的抽象必须强制使用它。<br>
///
///
#[unstable(feature = "ptr_internals", issue = "none")]
unsafe impl<T: Send + ?Sized> Send for Unique<T> {}

/// `Unique` pointers are `Sync` if `T` is `Sync` because the data they reference is unaliased. <br>如果 `T` 是 `Sync`，则 `Unique` 指针是 `Sync`，因为它们引用的数据是无别名的。<br>
/// Note that this aliasing invariant is unenforced by the type system; <br>请注意，类型系统不强制使用此别名不变量。<br> the abstraction using the `Unique` must enforce it. <br>使用 `Unique` 的抽象必须强制使用它。<br>
///
///
#[unstable(feature = "ptr_internals", issue = "none")]
unsafe impl<T: Sync + ?Sized> Sync for Unique<T> {}

#[unstable(feature = "ptr_internals", issue = "none")]
impl<T: Sized> Unique<T> {
    /// Creates a new `Unique` that is dangling, but well-aligned. <br>创建一个悬垂但对齐良好的新 `Unique`。<br>
    ///
    /// This is useful for initializing types which lazily allocate, like `Vec::new` does. <br>与 `Vec::new` 一样，这对于初始化延迟分配的类型很有用。<br>
    ///
    /// Note that the pointer value may potentially represent a valid pointer to a `T`, which means this must not be used as a "not yet initialized" sentinel value. <br>请注意，该指针值可能表示一个指向 `T` 的有效指针，这意味着不得将其用作 "尚未初始化" 标记值。<br>
    /// Types that lazily allocate must track initialization by some other means. <br>延迟分配的类型必须通过其他某种方式来跟踪初始化。<br>
    ///
    ///
    ///
    #[must_use]
    #[inline]
    pub const fn dangling() -> Self {
        Self::from(NonNull::dangling())
    }
}

#[unstable(feature = "ptr_internals", issue = "none")]
impl<T: ?Sized> Unique<T> {
    /// Creates a new `Unique`. <br>创建一个新的 `Unique`。<br>
    ///
    /// # Safety
    ///
    /// `ptr` must be non-null. <br>`ptr` 必须不能为空。<br>
    #[inline]
    pub const unsafe fn new_unchecked(ptr: *mut T) -> Self {
        // SAFETY: the caller must guarantee that `ptr` is non-null. <br>调用者必须保证 `ptr` 不为空。<br>
        unsafe { Unique { pointer: NonNull::new_unchecked(ptr), _marker: PhantomData } }
    }

    /// Creates a new `Unique` if `ptr` is non-null. <br>如果 `ptr` 不为空，则创建一个新的 `Unique`。<br>
    #[inline]
    pub const fn new(ptr: *mut T) -> Option<Self> {
        if let Some(pointer) = NonNull::new(ptr) {
            Some(Unique { pointer, _marker: PhantomData })
        } else {
            None
        }
    }

    /// Acquires the underlying `*mut` pointer. <br>获取底层的 `*mut` 指针。<br>
    #[must_use = "`self` will be dropped if the result is not used"]
    #[inline]
    pub const fn as_ptr(self) -> *mut T {
        self.pointer.as_ptr()
    }

    /// Dereferences the content. <br>解引用内容。<br>
    ///
    /// The resulting lifetime is bound to self so this behaves "as if" it were actually an instance of T that is getting borrowed. <br>最终的生命周期被绑定到 self 上，所以它的行为 "好像" 它实际上是一个被借用的 T 的实例。<br>
    /// If a longer (unbound) lifetime is needed, use `&*my_ptr.as_ptr()`. <br>如果需要更长的 (unbound) 生命周期，请使用 `&*my_ptr.as_ptr()`。<br>
    ///
    #[must_use]
    #[inline]
    pub const unsafe fn as_ref(&self) -> &T {
        // SAFETY: the caller must guarantee that `self` meets all the requirements for a reference. <br>调用者必须保证 `self` 满足引用的所有要求。<br>
        //
        unsafe { self.pointer.as_ref() }
    }

    /// Mutably dereferences the content. <br>相互解引用内容。<br>
    ///
    /// The resulting lifetime is bound to self so this behaves "as if" it were actually an instance of T that is getting borrowed. <br>最终的生命周期被绑定到 self 上，所以它的行为 "好像" 它实际上是一个被借用的 T 的实例。<br>
    /// If a longer (unbound) lifetime is needed, use `&mut *my_ptr.as_ptr()`. <br>如果需要更长的 (unbound) 生命周期，请使用 `&mut *my_ptr.as_ptr()`。<br>
    ///
    #[must_use]
    #[inline]
    pub const unsafe fn as_mut(&mut self) -> &mut T {
        // SAFETY: the caller must guarantee that `self` meets all the requirements for a mutable reference. <br>调用者必须保证 `self` 满足可变引用的所有要求。<br>
        //
        unsafe { self.pointer.as_mut() }
    }

    /// Casts to a pointer of another type. <br>强制转换为另一种类型的指针。<br>
    #[must_use = "`self` will be dropped if the result is not used"]
    #[inline]
    pub const fn cast<U>(self) -> Unique<U> {
        Unique::from(self.pointer.cast())
    }
}

#[unstable(feature = "ptr_internals", issue = "none")]
#[rustc_const_unstable(feature = "const_clone", issue = "91805")]
impl<T: ?Sized> const Clone for Unique<T> {
    #[inline]
    fn clone(&self) -> Self {
        *self
    }
}

#[unstable(feature = "ptr_internals", issue = "none")]
impl<T: ?Sized> Copy for Unique<T> {}

#[unstable(feature = "ptr_internals", issue = "none")]
impl<T: ?Sized, U: ?Sized> CoerceUnsized<Unique<U>> for Unique<T> where T: Unsize<U> {}

#[unstable(feature = "ptr_internals", issue = "none")]
impl<T: ?Sized, U: ?Sized> DispatchFromDyn<Unique<U>> for Unique<T> where T: Unsize<U> {}

#[unstable(feature = "ptr_internals", issue = "none")]
impl<T: ?Sized> fmt::Debug for Unique<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Pointer::fmt(&self.as_ptr(), f)
    }
}

#[unstable(feature = "ptr_internals", issue = "none")]
impl<T: ?Sized> fmt::Pointer for Unique<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Pointer::fmt(&self.as_ptr(), f)
    }
}

#[unstable(feature = "ptr_internals", issue = "none")]
impl<T: ?Sized> const From<&mut T> for Unique<T> {
    /// Converts a `&mut T` to a `Unique<T>`. <br>将 `&mut T` 转换为 `Unique<T>`。<br>
    ///
    /// This conversion is infallible since references cannot be null. <br>这种转换是可靠的，因为引用不能为空。<br>
    #[inline]
    fn from(reference: &mut T) -> Self {
        Self::from(NonNull::from(reference))
    }
}

#[unstable(feature = "ptr_internals", issue = "none")]
impl<T: ?Sized> const From<NonNull<T>> for Unique<T> {
    /// Converts a `NonNull<T>` to a `Unique<T>`. <br>将 `NonNull<T>` 转换为 `Unique<T>`。<br>
    ///
    /// This conversion is infallible since `NonNull` cannot be null. <br>这种转换是可靠的，因为 `NonNull` 不能为空。<br>
    #[inline]
    fn from(pointer: NonNull<T>) -> Self {
        Unique { pointer, _marker: PhantomData }
    }
}
