//! Slice sorting <br>切片分类<br>
//!
//! This module contains a sorting algorithm based on Orson Peters' pattern-defeating quicksort, published at: <https://github.com/orlp/pdqsort> <br>该模块包含基于 Orson Peters 的模式失败快速排序的排序算法，发布于: <https://github.com/orlp/pdqsort><br>
//!
//!
//! Unstable sorting is compatible with libcore because it doesn't allocate memory, unlike our stable sorting implementation. <br>不稳定排序与 libcore 兼容，因为它不分配内存，这与我们稳定的排序实现不同。<br>
//!

use crate::cmp;
use crate::mem::{self, MaybeUninit};
use crate::ptr;

/// When dropped, copies from `src` into `dest`. <br>丢弃时，从 `src` 复制到 `dest`。<br>
struct CopyOnDrop<T> {
    src: *const T,
    dest: *mut T,
}

impl<T> Drop for CopyOnDrop<T> {
    fn drop(&mut self) {
        // SAFETY:  This is a helper class. <br>这是一个帮助程序类。<br>
        //          Please refer to its usage for correctness. <br>请参考其用法以确保正确性。<br>
        //          Namely, one must be sure that `src` and `dst` does not overlap as required by `ptr::copy_nonoverlapping`. <br>即，必须确保 `src` 和 `dst` 不会按照 `ptr::copy_nonoverlapping` 的要求重叠。<br>
        unsafe {
            ptr::copy_nonoverlapping(self.src, self.dest, 1);
        }
    }
}

/// Shifts the first element to the right until it encounters a greater or equal element. <br>将第一个元素向右移动，直到遇到一个更大或相等的元素。<br>
fn shift_head<T, F>(v: &mut [T], is_less: &mut F)
where
    F: FnMut(&T, &T) -> bool,
{
    let len = v.len();
    // SAFETY: The unsafe operations below involves indexing without a bounds check (by offsetting a pointer) and copying memory (`ptr::copy_nonoverlapping`). <br>下面的不安全操作涉及没有边界检查的索引 (通过偏移指针) 和复制内存 (`ptr::copy_nonoverlapping`)。<br>
    //
    //
    // a. Indexing:
    //  1. We checked the size of the array to >=2. <br>我们将数组的大小检查为 >= 2。<br>
    //  2. All the indexing that we will do is always between {0 <= index < len} at most. <br>我们将要进行的所有索引编制最多始终在 {0 <= index < len} 之间。<br>
    //
    // b. Memory copying <br>内存复制<br>
    //  1. We are obtaining pointers to references which are guaranteed to be valid. <br>我们正在获得指向 quot 的指针，这些指针被保证是有效的。<br>
    //  2. They cannot overlap because we obtain pointers to difference indices of the slice. <br>它们不能重叠，因为我们获得了指向切片的不同索引的指针。<br>
    //     Namely, `i` and `i-1`. <br>即 `i` 和 `i-1`。<br>
    //  3. If the slice is properly aligned, the elements are properly aligned. <br>如果切片正确对齐，则元素正确对齐。<br>
    //     It is the caller's responsibility to make sure the slice is properly aligned. <br>确保切片正确对齐是调用者的责任。<br>
    //
    // See comments below for further detail. <br>有关更多详细信息，请参见下面的评论。<br>
    unsafe {
        // If the first two elements are out-of-order... <br>如果前两个元素乱序...<br>
        if len >= 2 && is_less(v.get_unchecked(1), v.get_unchecked(0)) {
            // Read the first element into a stack-allocated variable. <br>将第一个元素读取到栈分配的变量中。<br>
            // If a following comparison operation panics, `hole` will get dropped and automatically write the element back into the slice. <br>如果执行以下比较操作 panics，则 `hole` 将被丢弃并自动将元素写回到切片中。<br>
            //
            let tmp = mem::ManuallyDrop::new(ptr::read(v.get_unchecked(0)));
            let v = v.as_mut_ptr();
            let mut hole = CopyOnDrop { src: &*tmp, dest: v.add(1) };
            ptr::copy_nonoverlapping(v.add(1), v.add(0), 1);

            for i in 2..len {
                if !is_less(&*v.add(i), &*tmp) {
                    break;
                }

                // Move `i`-th element one place to the left, thus shifting the hole to the right. <br>将第 i 个元素向左移动一个位置，从而将 hole 向右移动。<br>
                ptr::copy_nonoverlapping(v.add(i), v.add(i - 1), 1);
                hole.dest = v.add(i);
            }
            // `hole` gets dropped and thus copies `tmp` into the remaining hole in `v`. <br>`hole` 被丢弃，因此将 `tmp` 复制到 `v` 中剩余的 hole 中。<br>
        }
    }
}

/// Shifts the last element to the left until it encounters a smaller or equal element. <br>将最后一个元素向左移动，直到遇到一个较小或相等的元素。<br>
fn shift_tail<T, F>(v: &mut [T], is_less: &mut F)
where
    F: FnMut(&T, &T) -> bool,
{
    let len = v.len();
    // SAFETY: The unsafe operations below involves indexing without a bound check (by offsetting a pointer) and copying memory (`ptr::copy_nonoverlapping`). <br>下面的不安全操作涉及没有边界检查的索引 (通过偏移指针) 和复制内存 (`ptr::copy_nonoverlapping`)。<br>
    //
    //
    // a. Indexing:
    //  1. We checked the size of the array to >= 2. <br>我们检查了数组的大小 >= 2。<br>
    //  2. All the indexing that we will do is always between `0 <= index < len-1` at most. <br>我们将要进行的所有索引编制最多始终在 `0 <= index < len-1` 之间。<br>
    //
    // b. Memory copying <br>内存复制<br>
    //  1. We are obtaining pointers to references which are guaranteed to be valid. <br>我们正在获得指向 quot 的指针，这些指针被保证是有效的。<br>
    //  2. They cannot overlap because we obtain pointers to difference indices of the slice. <br>它们不能重叠，因为我们获得了指向切片的不同索引的指针。<br>
    //     Namely, `i` and `i+1`. <br>即 `i` 和 `i+1`。<br>
    //  3. If the slice is properly aligned, the elements are properly aligned. <br>如果切片正确对齐，则元素正确对齐。<br>
    //     It is the caller's responsibility to make sure the slice is properly aligned. <br>确保切片正确对齐是调用者的责任。<br>
    //
    // See comments below for further detail. <br>有关更多详细信息，请参见下面的评论。<br>
    unsafe {
        // If the last two elements are out-of-order... <br>如果最后两个元素乱序...<br>
        if len >= 2 && is_less(v.get_unchecked(len - 1), v.get_unchecked(len - 2)) {
            // Read the last element into a stack-allocated variable. <br>将最后一个元素读取到栈分配的变量中。<br>
            // If a following comparison operation panics, `hole` will get dropped and automatically write the element back into the slice. <br>如果执行以下比较操作 panics，则 `hole` 将被丢弃并自动将元素写回到切片中。<br>
            //
            let tmp = mem::ManuallyDrop::new(ptr::read(v.get_unchecked(len - 1)));
            let v = v.as_mut_ptr();
            let mut hole = CopyOnDrop { src: &*tmp, dest: v.add(len - 2) };
            ptr::copy_nonoverlapping(v.add(len - 2), v.add(len - 1), 1);

            for i in (0..len - 2).rev() {
                if !is_less(&*tmp, &*v.add(i)) {
                    break;
                }

                // Move `i`-th element one place to the right, thus shifting the hole to the left. <br>将第 i 个元素向右移动一个位置，从而将 hole 向左移动。<br>
                ptr::copy_nonoverlapping(v.add(i), v.add(i + 1), 1);
                hole.dest = v.add(i);
            }
            // `hole` gets dropped and thus copies `tmp` into the remaining hole in `v`. <br>`hole` 被丢弃，因此将 `tmp` 复制到 `v` 中剩余的 hole 中。<br>
        }
    }
}

/// Partially sorts a slice by shifting several out-of-order elements around. <br>通过移动几个乱序的元素来对切片进行部分排序。<br>
///
/// Returns `true` if the slice is sorted at the end. <br>如果切片末尾排序，则返回 `true`。<br> This function is *O*(*n*) worst-case. <br>该函数是 *O*(*n*) 最坏的情况。<br>
#[cold]
fn partial_insertion_sort<T, F>(v: &mut [T], is_less: &mut F) -> bool
where
    F: FnMut(&T, &T) -> bool,
{
    // Maximum number of adjacent out-of-order pairs that will get shifted. <br>将移位的相邻无序对的最大数量。<br>
    const MAX_STEPS: usize = 5;
    // If the slice is shorter than this, don't shift any elements. <br>如果切片短于此，请勿移动任何元素。<br>
    const SHORTEST_SHIFTING: usize = 50;

    let len = v.len();
    let mut i = 1;

    for _ in 0..MAX_STEPS {
        // SAFETY: We already explicitly did the bound checking with `i < len`. <br>我们已经用 `i < len` 明确地进行了边界检查。<br>
        // All our subsequent indexing is only in the range `0 <= index < len` <br>我们所有的后续索引仅在 `0 <= index < len` 范围内<br>
        unsafe {
            // Find the next pair of adjacent out-of-order elements. <br>查找下一对相邻的乱序元素。<br>
            while i < len && !is_less(v.get_unchecked(i), v.get_unchecked(i - 1)) {
                i += 1;
            }
        }

        // Are we done? <br>我们完成了吗？<br>
        if i == len {
            return true;
        }

        // Don't shift elements on short arrays, that has a performance cost. <br>不要在短数组上移动元素，这会降低性能。<br>
        if len < SHORTEST_SHIFTING {
            return false;
        }

        // Swap the found pair of elements. <br>交换找到的一对元素。<br> This puts them in correct order. <br>这使它们处于正确的顺序。<br>
        v.swap(i - 1, i);

        // Shift the smaller element to the left. <br>将较小的元素向左移动。<br>
        shift_tail(&mut v[..i], is_less);
        // Shift the greater element to the right. <br>将较大的元素向右移动。<br>
        shift_head(&mut v[i..], is_less);
    }

    // Didn't manage to sort the slice in the limited number of steps. <br>未能在有限的步骤中对切片进行排序。<br>
    false
}

/// Sorts a slice using insertion sort, which is *O*(*n*^2) worst-case. <br>使用插入排序对切片进行排序，这是 *O*(*n*^ 2) 最坏的情况。<br>
fn insertion_sort<T, F>(v: &mut [T], is_less: &mut F)
where
    F: FnMut(&T, &T) -> bool,
{
    for i in 1..v.len() {
        shift_tail(&mut v[..i + 1], is_less);
    }
}

/// Sorts `v` using heapsort, which guarantees *O*(*n* \* log(*n*)) worst-case. <br>使用堆排序对 `v` 进行排序，这保证了 *O*(*n*\*log(* n*)) 最坏的情况。<br>
#[cold]
#[unstable(feature = "sort_internals", reason = "internal to sort module", issue = "none")]
pub fn heapsort<T, F>(v: &mut [T], mut is_less: F)
where
    F: FnMut(&T, &T) -> bool,
{
    // This binary heap respects the invariant `parent >= child`. <br>该二进制堆遵守不变量 `parent >= child`。<br>
    let mut sift_down = |v: &mut [T], mut node| {
        loop {
            // Children of `node`. <br>`node` 的子节点。<br>
            let mut child = 2 * node + 1;
            if child >= v.len() {
                break;
            }

            // Choose the greater child. <br>选择更大的子节点。<br>
            if child + 1 < v.len() && is_less(&v[child], &v[child + 1]) {
                child += 1;
            }

            // Stop if the invariant holds at `node`. <br>如果不变量位于 `node`，则停止。<br>
            if !is_less(&v[node], &v[child]) {
                break;
            }

            // Swap `node` with the greater child, move one step down, and continue sifting. <br>与更大的子节点交换 `node`，向下移动一步，然后继续筛分。<br>
            v.swap(node, child);
            node = child;
        }
    };

    // Build the heap in linear time. <br>在线性时间内构建堆。<br>
    for i in (0..v.len() / 2).rev() {
        sift_down(v, i);
    }

    // Pop maximal elements from the heap. <br>从堆中弹出最大元素。<br>
    for i in (1..v.len()).rev() {
        v.swap(0, i);
        sift_down(&mut v[..i], 0);
    }
}

/// Partitions `v` into elements smaller than `pivot`, followed by elements greater than or equal to `pivot`. <br>将 `v` 划分为小于 `pivot` 的元素，然后划分为大于或等于 `pivot` 的元素。<br>
///
///
/// Returns the number of elements smaller than `pivot`. <br>返回小于 `pivot` 的元素数。<br>
///
/// Partitioning is performed block-by-block in order to minimize the cost of branching operations. <br>为了最小化分支操作的成本，逐块执行分区。<br>
/// This idea is presented in the [BlockQuicksort][pdf] paper. <br>[块快速排序][pdf] 论文中提出了这个想法。<br>
///
/// [pdf]: https://drops.dagstuhl.de/opus/volltexte/2016/6389/pdf/LIPIcs-ESA-2016-38.pdf
fn partition_in_blocks<T, F>(v: &mut [T], pivot: &T, is_less: &mut F) -> usize
where
    F: FnMut(&T, &T) -> bool,
{
    // Number of elements in a typical block. <br>典型块中的元素数。<br>
    const BLOCK: usize = 128;

    // The partitioning algorithm repeats the following steps until completion: <br>分区算法重复以下步骤，直到完成：<br>
    //
    // 1. Trace a block from the left side to identify elements greater than or equal to the pivot. <br>从左侧跟踪一个块，以识别大于或等于枢轴的元素。<br>
    // 2. Trace a block from the right side to identify elements smaller than the pivot. <br>从右侧跟踪一个块，以识别小于枢轴的元素。<br>
    // 3. Exchange the identified elements between the left and right side. <br>在左侧和右侧之间交换已标识的元素。<br>
    //
    // We keep the following variables for a block of elements: <br>我们为元素块保留以下变量：<br>
    //
    // 1. `block` - Number of elements in the block. <br>`block` - 块中的元素数。<br>
    // 2. `start` - Start pointer into the `offsets` array. <br>`start` - 指向 `offsets` 数组的起始指针。<br>
    // 3. `end` - End pointer into the `offsets` array. <br>`end` - 指向 `offsets` 数组的结束指针。<br>
    // 4. `offsets - Indices of out-of-order elements within the block. <br>偏移量 - 块内乱序元素的索引。<br>

    // The current block on the left side (from `l` to `l.add(block_l)`). <br>左侧的当前块 (从 `l` 到 `l.add(block_l)`)。<br>
    let mut l = v.as_mut_ptr();
    let mut block_l = BLOCK;
    let mut start_l = ptr::null_mut();
    let mut end_l = ptr::null_mut();
    let mut offsets_l = [MaybeUninit::<u8>::uninit(); BLOCK];

    // The current block on the right side (from `r.sub(block_r)` to `r`). <br>右侧的当前块 (从 `r.sub(block_r)` 到 `r`)。<br>
    // SAFETY: The documentation for .add() specifically mention that `vec.as_ptr().add(vec.len())` is always safe` <br>.add() 的文档特别提到 `vec.as_ptr().add(vec.len())` 始终是安全的。<br>
    let mut r = unsafe { l.add(v.len()) };
    let mut block_r = BLOCK;
    let mut start_r = ptr::null_mut();
    let mut end_r = ptr::null_mut();
    let mut offsets_r = [MaybeUninit::<u8>::uninit(); BLOCK];

    // FIXME: When we get VLAs, try creating one array of length `min(v.len(), 2 * BLOCK)` rather than two fixed-size arrays of length `BLOCK`. <br>当我们得到 VLA 时，请尝试创建一个长度为 `min(v.len(), 2 * BLOCK)` 的数组，而不是两个长度为 `BLOCK` 的固定大小的数组。<br>
    // VLAs might be more cache-efficient. <br>VLA 可能会提高缓存效率。<br>

    // Returns the number of elements between pointers `l` (inclusive) and `r` (exclusive). <br>返回指针 `l` (inclusive) 和 `r` (exclusive) 之间的元素数。<br>
    fn width<T>(l: *mut T, r: *mut T) -> usize {
        assert!(mem::size_of::<T>() > 0);
        // FIXME: this should *likely* use `offset_from`, but more investigation is needed (including running tests in miri). <br>这应该可能使用 `offset_from`，但需要进行更多调查 (包括在 miri 中运行测试)。<br>
        //
        (r.addr() - l.addr()) / mem::size_of::<T>()
    }

    loop {
        // We are done with partitioning block-by-block when `l` and `r` get very close. <br>当 `l` 和 `r` 非常接近时，我们将逐块进行分区。<br>
        // Then we do some patch-up work in order to partition the remaining elements in between. <br>然后，我们进行一些修补工作，以便在其余元素之间进行划分。<br>
        let is_done = width(l, r) <= 2 * BLOCK;

        if is_done {
            // Number of remaining elements (still not compared to the pivot). <br>剩余元素数 (仍未与枢轴进行比较)。<br>
            let mut rem = width(l, r);
            if start_l < end_l || start_r < end_r {
                rem -= BLOCK;
            }

            // Adjust block sizes so that the left and right block don't overlap, but get perfectly aligned to cover the whole remaining gap. <br>调整块大小，以使左右块不重叠，但要完全对齐以覆盖整个剩余间隙。<br>
            //
            if start_l < end_l {
                block_r = rem;
            } else if start_r < end_r {
                block_l = rem;
            } else {
                // There were the same number of elements to switch on both blocks during the last iteration, so there are no remaining elements on either block. <br>在上次迭代期间要在两个块上切换的元素数量相同，因此任何一个块上都没有剩余的元素。<br>
                // Cover the remaining items with roughly equally-sized blocks. <br>用大致相同大小的块覆盖剩余的项。<br>
                //
                block_l = rem / 2;
                block_r = rem - block_l;
            }
            debug_assert!(block_l <= BLOCK && block_r <= BLOCK);
            debug_assert!(width(l, r) == block_l + block_r);
        }

        if start_l == end_l {
            // Trace `block_l` elements from the left side. <br>从左侧跟踪 `block_l` 元素。<br>
            start_l = MaybeUninit::slice_as_mut_ptr(&mut offsets_l);
            end_l = start_l;
            let mut elem = l;

            for i in 0..block_l {
                // SAFETY: The unsafety operations below involve the usage of the `offset`. <br>以下不安全操作涉及 `offset` 的使用。<br>
                //         According to the conditions required by the function, we satisfy them because: <br>根据函数所需的条件，我们满足它们，因为：<br>
                //         1. `offsets_l` is stack-allocated, and thus considered separate allocated object. <br>`offsets_l` 是栈分配的，因此被视为单独分配的对象。<br>
                //         2. The function `is_less` returns a `bool`. <br>函数 `is_less` 返回 `bool`。<br>
                //            Casting a `bool` will never overflow `isize`. <br>转换 `bool` 不会使 `isize` 溢出。<br>
                //         3. We have guaranteed that `block_l` will be `<= BLOCK`. <br>我们保证 `block_l` 将是 `<= BLOCK`。<br>
                //            Plus, `end_l` was initially set to the begin pointer of `offsets_` which was declared on the stack. <br>另外，`end_l` 最初设置为在栈上声明的 `offsets_` 的开始指针。<br>
                //            Thus, we know that even in the worst case (all invocations of `is_less` returns false) we will only be at most 1 byte pass the end. <br>因此，我们知道即使在最坏的情况下 (`is_less` 的所有调用都返回 false)，我们最多只能有 1 个字节通过结尾。<br>
                //        Another unsafety operation here is dereferencing `elem`. <br>这里的另一个不安全操作是解引用 `elem`。<br>
                //        However, `elem` was initially the begin pointer to the slice which is always valid. <br>但是，`elem` 最初是指向切片的 begin 指针，始终有效。<br>
                unsafe {
                    // Branchless comparison. <br>无分支比较。<br>
                    *end_l = i as u8;
                    end_l = end_l.offset(!is_less(&*elem, pivot) as isize);
                    elem = elem.offset(1);
                }
            }
        }

        if start_r == end_r {
            // Trace `block_r` elements from the right side. <br>从右侧跟踪 `block_r` 元素。<br>
            start_r = MaybeUninit::slice_as_mut_ptr(&mut offsets_r);
            end_r = start_r;
            let mut elem = r;

            for i in 0..block_r {
                // SAFETY: The unsafety operations below involve the usage of the `offset`. <br>以下不安全操作涉及 `offset` 的使用。<br>
                //         According to the conditions required by the function, we satisfy them because: <br>根据函数所需的条件，我们满足它们，因为：<br>
                //         1. `offsets_r` is stack-allocated, and thus considered separate allocated object. <br>`offsets_r` 是栈分配的，因此被视为单独分配的对象。<br>
                //         2. The function `is_less` returns a `bool`. <br>函数 `is_less` 返回 `bool`。<br>
                //            Casting a `bool` will never overflow `isize`. <br>转换 `bool` 不会使 `isize` 溢出。<br>
                //         3. We have guaranteed that `block_r` will be `<= BLOCK`. <br>我们保证 `block_r` 将是 `<= BLOCK`。<br>
                //            Plus, `end_r` was initially set to the begin pointer of `offsets_` which was declared on the stack. <br>另外，`end_r` 最初设置为在栈上声明的 `offsets_` 的开始指针。<br>
                //            Thus, we know that even in the worst case (all invocations of `is_less` returns true) we will only be at most 1 byte pass the end. <br>因此，我们知道即使在最坏的情况下 (`is_less` 的所有调用都返回 true)，我们最多只能将 1 个字节传递到末尾。<br>
                //        Another unsafety operation here is dereferencing `elem`. <br>这里的另一个不安全操作是解引用 `elem`。<br>
                //        However, `elem` was initially `1 * sizeof(T)` past the end and we decrement it by `1 * sizeof(T)` before accessing it. <br>但是，`elem` 最初是末尾的 `1 *sizeof(T)`，在访问它之前，我们先将其递减 `1* sizeof(T)`。<br>
                //        Plus, `block_r` was asserted to be less than `BLOCK` and `elem` will therefore at most be pointing to the beginning of the slice. <br>另外，断言 `block_r` 小于 `BLOCK`，因此 `elem` 至多将指向切片的开头。<br>
                unsafe {
                    // Branchless comparison. <br>无分支比较。<br>
                    elem = elem.offset(-1);
                    *end_r = i as u8;
                    end_r = end_r.offset(is_less(&*elem, pivot) as isize);
                }
            }
        }

        // Number of out-of-order elements to swap between the left and right side. <br>要在左侧和右侧之间交换的乱序元素的数量。<br>
        let count = cmp::min(width(start_l, end_l), width(start_r, end_r));

        if count > 0 {
            macro_rules! left {
                () => {
                    l.offset(*start_l as isize)
                };
            }
            macro_rules! right {
                () => {
                    r.offset(-(*start_r as isize) - 1)
                };
            }

            // Instead of swapping one pair at the time, it is more efficient to perform a cyclic permutation. <br>与一次交换一对相比，执行循环置换更为有效。<br>
            // This is not strictly equivalent to swapping, but produces a similar result using fewer memory operations. <br>这并不严格等同于交换，但是使用较少的内存操作即可产生类似的结果。<br>
            //

            // SAFETY: The use of `ptr::read` is valid because there is at least one element in both `offsets_l` and `offsets_r`, so `left!` is a valid pointer to read from. <br>`ptr::read` 的使用是有效的，因为在 `offsets_l` 和 `offsets_r` 中至少有一个元素，所以 `left!` 是一个有效的读取指针。<br>
            //
            // The uses of `left!` involve calls to `offset` on `l`, which points to the beginning of `v`. <br>`left!` 的使用涉及在 `l` 上调用 `offset`，它指向 `v` 的开头。<br> All the offsets pointed-to by `start_l` are at most `block_l`, so these `offset` calls are safe as all reads are within the block. <br>`start_l` 指向的所有偏移量最多为 `block_l`，因此这些 `offset` 调用是安全的，因为所有读取都在块内。<br> The same argument applies for the uses of `right!`. <br>相同的参数也适用于 `right!` 的用法。<br>
            //
            // The calls to `start_l.offset` are valid because there are at most `count-1` of them, plus the final one at the end of the unsafe block, where `count` is the minimum number of collected offsets in `offsets_l` and `offsets_r`, so there is no risk of there not being enough elements. <br>对 `start_l.offset` 的调用是有效的，因为它们最多有 `count-1` 个，加上不安全块末尾的最后一个，其中 `count` 是 `offsets_l` 和 `offsets_r` 中收集的最小偏移量，因此不存在不存在的风险足够的元素。<br>
            // The same reasoning applies to the calls to `start_r.offset`. <br>同样的推理适用于对 `start_r.offset` 的调用。<br>
            //
            // The calls to `copy_nonoverlapping` are safe because `left!` and `right!` are guaranteed not to overlap, and are valid because of the reasoning above. <br>对 `copy_nonoverlapping` 的调用是安全的，因为 `left!` 和 `right!` 保证不重叠，并且由于上述推理是有效的。<br>
            //
            //
            //
            //
            //
            //
            //
            unsafe {
                let tmp = ptr::read(left!());
                ptr::copy_nonoverlapping(right!(), left!(), 1);

                for _ in 1..count {
                    start_l = start_l.offset(1);
                    ptr::copy_nonoverlapping(left!(), right!(), 1);
                    start_r = start_r.offset(1);
                    ptr::copy_nonoverlapping(right!(), left!(), 1);
                }

                ptr::copy_nonoverlapping(&tmp, right!(), 1);
                mem::forget(tmp);
                start_l = start_l.offset(1);
                start_r = start_r.offset(1);
            }
        }

        if start_l == end_l {
            // All out-of-order elements in the left block were moved. <br>左侧块中的所有乱序元素均已移动。<br> Move to the next block. <br>移至下一个块。<br>

            // block-width-guarantee
            // SAFETY: if `!is_done` then the slice width is guaranteed to be at least `2*BLOCK` wide. <br>如果 `!is_done` 那么至少保证宽度为 `2*BLOCK` 宽。<br> There are at most `BLOCK` elements in `offsets_l` because of its size, so the `offset` operation is safe. <br>由于 `offsets_l` 的大小，`offsets_l` 中最多有 `BLOCK` 个元素，所以 `offset` 操作是安全的。<br>
            // Otherwise, the debug assertions in the `is_done` case guarantee that `width(l, r) == block_l + block_r`, namely, that the block sizes have been adjusted to account for the smaller number of remaining elements. <br>否则，`is_done` 情况下的调试断言保证 `width(l, r) == block_l + block_r`，即块大小已被调整以考虑较少数量的剩余元素。<br>
            //
            //
            //
            l = unsafe { l.offset(block_l as isize) };
        }

        if start_r == end_r {
            // All out-of-order elements in the right block were moved. <br>右侧块中的所有乱序元素均已移动。<br> Move to the previous block. <br>移至上一个块。<br>

            // SAFETY: Same argument as [block-width-guarantee]. <br>与 [block-width-guarantee] 的参数相同。<br>
            // Either this is a full block `2*BLOCK`-wide, or `block_r` has been adjusted for the last handful of elements. <br>这是一个完整的 `2*BLOCK` 块，或者 `block_r` 已针对最后少数元素进行了调整。<br>
            r = unsafe { r.offset(-(block_r as isize)) };
        }

        if is_done {
            break;
        }
    }

    // All that remains now is at most one block (either the left or the right) with out-of-order elements that need to be moved. <br>现在剩下的全部是至多一个块 (左侧或右侧)，其中需要移动的乱序元素。<br>
    // Such remaining elements can be simply shifted to the end within their block. <br>这些剩余的元素可以简单地移到其块内的末尾。<br>
    //

    if start_l < end_l {
        // The left block remains. <br>剩下的方块仍然保留。<br>
        // Move its remaining out-of-order elements to the far right. <br>将其剩余的乱序元素移到最右边。<br>
        debug_assert_eq!(width(l, r), block_l);
        while start_l < end_l {
            // remaining-elements-safety
            // SAFETY: while the loop condition holds there are still elements in `offsets_l`, so it is safe to point `end_l` to the previous element. <br>当循环条件成立时 `offsets_l` 中仍有元素，因此将 `end_l` 指向前一个元素是安全的。<br>
            //
            // The `ptr::swap` is safe if both its arguments are valid for reads and writes: <br>如果 `ptr::swap` 的参数对读和写都有效，则它是安全的：<br>
            //  - Per the debug assert above, the distance between `l` and `r` is `block_l` elements, so there can be at most `block_l` remaining offsets between `start_l` and `end_l`. <br>根据上面的调试断言，`l` 和 `r` 之间的距离是 `block_l` 元素，因此 `start_l` 和 `end_l` 之间最多可以有 `block_l` 剩余偏移量。<br>
            //  This means `r` will be moved at most `block_l` steps back, which makes the `r.offset` calls valid (at that point `l == r`). <br>这意味着 `r` 最多将向后移动 `block_l` 步，这使得 `r.offset` 调用有效 (在那个时候 `l == r`)。<br>
            //  - `offsets_l` contains valid offsets into `v` collected during the partitioning of the last block, so the `l.offset` calls are valid. <br>`offsets_l` 包含在最后一个块的分区期间收集到的 `v` 的有效偏移量，因此 `l.offset` 调用是有效的。<br>
            //
            //
            //
            //
            unsafe {
                end_l = end_l.offset(-1);
                ptr::swap(l.offset(*end_l as isize), r.offset(-1));
                r = r.offset(-1);
            }
        }
        width(v.as_mut_ptr(), r)
    } else if start_r < end_r {
        // The right block remains. <br>右边的块仍然存在。<br>
        // Move its remaining out-of-order elements to the far left. <br>将其剩余的乱序元素移到最左边。<br>
        debug_assert_eq!(width(l, r), block_r);
        while start_r < end_r {
            // SAFETY: See the reasoning in [remaining-elements-safety]. <br>请参见 [剩余元素安全][remaining-elements-safety] 中的推理。<br>
            unsafe {
                end_r = end_r.offset(-1);
                ptr::swap(l, r.offset(-(*end_r as isize) - 1));
                l = l.offset(1);
            }
        }
        width(v.as_mut_ptr(), l)
    } else {
        // Nothing else to do, we're done. <br>没什么可做的，我们已经完成。<br>
        width(v.as_mut_ptr(), l)
    }
}

/// Partitions `v` into elements smaller than `v[pivot]`, followed by elements greater than or equal to `v[pivot]`. <br>将 `v` 划分为小于 `v[pivot]` 的元素，然后划分为大于或等于 `v[pivot]` 的元素。<br>
///
///
/// Returns a tuple of: <br>返回一个元组：<br>
///
/// 1. Number of elements smaller than `v[pivot]`. <br>小于 `v[pivot]` 的元素数。<br>
/// 2. True if `v` was already partitioned. <br>如果 `v` 已经分区，则为 True。<br>
fn partition<T, F>(v: &mut [T], pivot: usize, is_less: &mut F) -> (usize, bool)
where
    F: FnMut(&T, &T) -> bool,
{
    let (mid, was_partitioned) = {
        // Place the pivot at the beginning of slice. <br>将枢轴放置在切片的开头。<br>
        v.swap(0, pivot);
        let (pivot, v) = v.split_at_mut(1);
        let pivot = &mut pivot[0];

        // Read the pivot into a stack-allocated variable for efficiency. <br>将枢轴读取到栈分配的变量中以提高效率。<br>
        // If a following comparison operation panics, the pivot will be automatically written back into the slice. <br>如果执行以下比较操作 panics，则枢轴将自动写回到切片中。<br>

        // SAFETY: `pivot` is a reference to the first element of `v`, so `ptr::read` is safe. <br>`pivot` 是对 `v` 第一个元素的引用，所以 `ptr::read` 是安全的。<br>
        let tmp = mem::ManuallyDrop::new(unsafe { ptr::read(pivot) });
        let _pivot_guard = CopyOnDrop { src: &*tmp, dest: pivot };
        let pivot = &*tmp;

        // Find the first pair of out-of-order elements. <br>查找第一对乱序元素。<br>
        let mut l = 0;
        let mut r = v.len();

        // SAFETY: The unsafety below involves indexing an array. <br>下面的不安全性涉及对数组进行索引。<br>
        // For the first one: We already do the bounds checking here with `l < r`. <br>对于第一个：我们已经在这里使用 `l < r` 进行了边界检查。<br>
        // For the second one: We initially have `l == 0` and `r == v.len()` and we checked that `l < r` at every indexing operation. <br>对于第二个：我们最初有 `l == 0` 和 `r == v.len()`，我们在每次索引操作中都检查了 `l < r`。<br>
        //                     From here we know that `r` must be at least `r == l` which was shown to be valid from the first one. <br>从这里我们知道 `r` 必须至少是 `r == l`，这从第一个开始就被证明是有效的。<br>
        unsafe {
            // Find the first element greater than or equal to the pivot. <br>找到大于或等于枢轴的第一个元素。<br>
            while l < r && is_less(v.get_unchecked(l), pivot) {
                l += 1;
            }

            // Find the last element smaller that the pivot. <br>找到比枢轴更小的最后一个元素。<br>
            while l < r && !is_less(v.get_unchecked(r - 1), pivot) {
                r -= 1;
            }
        }

        (l + partition_in_blocks(&mut v[l..r], pivot, is_less), l >= r)

        // `_pivot_guard` goes out of scope and writes the pivot (which is a stack-allocated variable) back into the slice where it originally was. <br>`_pivot_guard` 离开作用域并将 pivot (这是一个栈分配的变量) 写回到它原来所在的切片中。<br>
        // This step is critical in ensuring safety! <br>这一步对于确保安全至关重要！<br>
        //
    };

    // Place the pivot between the two partitions. <br>将枢轴放置在两个分区之间。<br>
    v.swap(0, mid);

    (mid, was_partitioned)
}

/// Partitions `v` into elements equal to `v[pivot]` followed by elements greater than `v[pivot]`. <br>将 `v` 划分为等于 `v[pivot]` 的元素，后跟大于 `v[pivot]` 的元素。<br>
///
/// Returns the number of elements equal to the pivot. <br>返回等于枢轴的元素数。<br>
/// It is assumed that `v` does not contain elements smaller than the pivot. <br>假定 `v` 不包含小于枢轴的元素。<br>
fn partition_equal<T, F>(v: &mut [T], pivot: usize, is_less: &mut F) -> usize
where
    F: FnMut(&T, &T) -> bool,
{
    // Place the pivot at the beginning of slice. <br>将枢轴放置在切片的开头。<br>
    v.swap(0, pivot);
    let (pivot, v) = v.split_at_mut(1);
    let pivot = &mut pivot[0];

    // Read the pivot into a stack-allocated variable for efficiency. <br>将枢轴读取到栈分配的变量中以提高效率。<br>
    // If a following comparison operation panics, the pivot will be automatically written back into the slice. <br>如果执行以下比较操作 panics，则枢轴将自动写回到切片中。<br>
    // SAFETY: The pointer here is valid because it is obtained from a reference to a slice. <br>此处的指针是有效的，因为它是从引用到切片获得的。<br>
    let tmp = mem::ManuallyDrop::new(unsafe { ptr::read(pivot) });
    let _pivot_guard = CopyOnDrop { src: &*tmp, dest: pivot };
    let pivot = &*tmp;

    // Now partition the slice. <br>现在对切片进行分区。<br>
    let mut l = 0;
    let mut r = v.len();
    loop {
        // SAFETY: The unsafety below involves indexing an array. <br>下面的不安全性涉及对数组进行索引。<br>
        // For the first one: We already do the bounds checking here with `l < r`. <br>对于第一个：我们已经在这里使用 `l < r` 进行了边界检查。<br>
        // For the second one: We initially have `l == 0` and `r == v.len()` and we checked that `l < r` at every indexing operation. <br>对于第二个：我们最初有 `l == 0` 和 `r == v.len()`，我们在每次索引操作中都检查了 `l < r`。<br>
        //                     From here we know that `r` must be at least `r == l` which was shown to be valid from the first one. <br>从这里我们知道 `r` 必须至少是 `r == l`，这从第一个开始就被证明是有效的。<br>
        unsafe {
            // Find the first element greater than the pivot. <br>找到第一个大于枢轴的元素。<br>
            while l < r && !is_less(pivot, v.get_unchecked(l)) {
                l += 1;
            }

            // Find the last element equal to the pivot. <br>找到等于枢轴的最后一个元素。<br>
            while l < r && is_less(pivot, v.get_unchecked(r - 1)) {
                r -= 1;
            }

            // Are we done? <br>我们完成了吗？<br>
            if l >= r {
                break;
            }

            // Swap the found pair of out-of-order elements. <br>交换找到的一对乱序元素。<br>
            r -= 1;
            let ptr = v.as_mut_ptr();
            ptr::swap(ptr.add(l), ptr.add(r));
            l += 1;
        }
    }

    // We found `l` elements equal to the pivot. <br>我们发现 `l` 元素等于 pivot。<br> Add 1 to account for the pivot itself. <br>为 pivot 本身加 1。<br>
    l + 1

    // `_pivot_guard` goes out of scope and writes the pivot (which is a stack-allocated variable) back into the slice where it originally was. <br>`_pivot_guard` 离开作用域并将 pivot (这是一个栈分配的变量) 写回到它原来所在的切片中。<br>
    // This step is critical in ensuring safety! <br>这一步对于确保安全至关重要！<br>
}

/// Scatters some elements around in an attempt to break patterns that might cause imbalanced partitions in quicksort. <br>散布一些元素，以尝试破坏可能导致快速排序中的分区不平衡的模式。<br>
///
#[cold]
fn break_patterns<T>(v: &mut [T]) {
    let len = v.len();
    if len >= 8 {
        // Pseudorandom number generator from the "Xorshift RNGs" paper by George Marsaglia. <br>George Marsaglia 撰写的 "Xorshift RNGs" 论文中的伪随机数生成器。<br>
        let mut random = len as u32;
        let mut gen_u32 = || {
            random ^= random << 13;
            random ^= random >> 17;
            random ^= random << 5;
            random
        };
        let mut gen_usize = || {
            if usize::BITS <= 32 {
                gen_u32() as usize
            } else {
                (((gen_u32() as u64) << 32) | (gen_u32() as u64)) as usize
            }
        };

        // Take random numbers modulo this number. <br>取该数字取模的随机数。<br>
        // The number fits into `usize` because `len` is not greater than `isize::MAX`. <br>该数字适合 `usize`，因为 `len` 不大于 `isize::MAX`。<br>
        let modulus = len.next_power_of_two();

        // Some pivot candidates will be in the nearby of this index. <br>一些关键候选人将在该指数附近。<br> Let's randomize them. <br>让我们随机化它们。<br>
        let pos = len / 4 * 2;

        for i in 0..3 {
            // Generate a random number modulo `len`. <br>生成一个以 `len` 为模的随机数。<br>
            // However, in order to avoid costly operations we first take it modulo a power of two, and then decrease by `len` until it fits into the range `[0, len - 1]`. <br>但是，为了避免昂贵的操作，我们首先将其取模为 2 的幂，然后减小 `len` 直到它适合 `[0, len - 1]` 范围。<br>
            //
            let mut other = gen_usize() & (modulus - 1);

            // `other` is guaranteed to be less than `2 * len`. <br>`other` 保证小于 `2 * len`。<br>
            if other >= len {
                other -= len;
            }

            v.swap(pos - 1 + i, other);
        }
    }
}

/// Chooses a pivot in `v` and returns the index and `true` if the slice is likely already sorted. <br>在 `v` 中选择一个枢轴，如果切片可能已经排序，则返回索引和 `true`。<br>
///
/// Elements in `v` might be reordered in the process. <br>`v` 中的元素可能会在此过程中重新排序。<br>
fn choose_pivot<T, F>(v: &mut [T], is_less: &mut F) -> (usize, bool)
where
    F: FnMut(&T, &T) -> bool,
{
    // Minimum length to choose the median-of-medians method. <br>选择中位数中位数方法的最小长度。<br>
    // Shorter slices use the simple median-of-three method. <br>较短的切片使用简单的三位数中位数方法。<br>
    const SHORTEST_MEDIAN_OF_MEDIANS: usize = 50;
    // Maximum number of swaps that can be performed in this function. <br>在此函数中可以执行的最大交换次数。<br>
    const MAX_SWAPS: usize = 4 * 3;

    let len = v.len();

    // Three indices near which we are going to choose a pivot. <br>我们将在附近选择一个枢轴的三个指数。<br>
    let mut a = len / 4 * 1;
    let mut b = len / 4 * 2;
    let mut c = len / 4 * 3;

    // Counts the total number of swaps we are about to perform while sorting indices. <br>计算在对索引进行排序时我们将要执行的交换总数。<br>
    let mut swaps = 0;

    if len >= 8 {
        // Swaps indices so that `v[a] <= v[b]`. <br>交换索引，以便 `v[a] <= v[b]`。<br>
        // SAFETY: `len >= 8` so there are at least two elements in the neighborhoods of `a`, `b` and `c`. <br>`len >= 8` 所以在 `a`、`b` 和 `c` 的邻域中至少有两个元素。<br>
        // This means the three calls to `sort_adjacent` result in corresponding calls to `sort3` with valid 3-item neighborhoods around each pointer, which in turn means the calls to `sort2` are done with valid references. <br>这意味着对 `sort_adjacent` 的三个调用导致对 `sort3` 的相应调用以及每个指针周围的有效 3 项邻域，这反过来意味着对 `sort2` 的调用是通过有效的引用完成的。<br>
        //
        // Thus the `v.get_unchecked` calls are safe, as is the `ptr::swap` call. <br>因此 `v.get_unchecked` 调用是安全的，`ptr::swap` 调用也是安全的。<br>
        //
        //
        let mut sort2 = |a: &mut usize, b: &mut usize| unsafe {
            if is_less(v.get_unchecked(*b), v.get_unchecked(*a)) {
                ptr::swap(a, b);
                swaps += 1;
            }
        };

        // Swaps indices so that `v[a] <= v[b] <= v[c]`. <br>交换索引，以便 `v[a] <= v[b] <= v[c]`。<br>
        let mut sort3 = |a: &mut usize, b: &mut usize, c: &mut usize| {
            sort2(a, b);
            sort2(b, c);
            sort2(a, b);
        };

        if len >= SHORTEST_MEDIAN_OF_MEDIANS {
            // Finds the median of `v[a - 1], v[a], v[a + 1]` and stores the index into `a`. <br>查找 `v[a - 1], v[a], v[a + 1]` 的中位数，并将索引存储到 `a` 中。<br>
            let mut sort_adjacent = |a: &mut usize| {
                let tmp = *a;
                sort3(&mut (tmp - 1), a, &mut (tmp + 1));
            };

            // Find medians in the neighborhoods of `a`, `b`, and `c`. <br>查找 `a`，`b` 和 `c` 附近的中位数。<br>
            sort_adjacent(&mut a);
            sort_adjacent(&mut b);
            sort_adjacent(&mut c);
        }

        // Find the median among `a`, `b`, and `c`. <br>在 `a`，`b` 和 `c` 中找到中位数。<br>
        sort3(&mut a, &mut b, &mut c);
    }

    if swaps < MAX_SWAPS {
        (b, swaps == 0)
    } else {
        // The maximum number of swaps was performed. <br>已执行最大交换次数。<br>
        // Chances are the slice is descending or mostly descending, so reversing will probably help sort it faster. <br>切片可能是降序的，或者大多是降序的，因此反转可能有助于更快地对其进行排序。<br>
        v.reverse();
        (len - 1 - b, true)
    }
}

/// Sorts `v` recursively. <br>递归排序 `v`。<br>
///
/// If the slice had a predecessor in the original array, it is specified as `pred`. <br>如果切片在原始数组中具有前身，则将其指定为 `pred`。<br>
///
/// `limit` is the number of allowed imbalanced partitions before switching to `heapsort`. <br>`limit` 是切换到 `heapsort` 之前允许的不平衡分区数。<br>
/// If zero, this function will immediately switch to heapsort. <br>如果为零，则此函数将立即切换到 heapsort。<br>
fn recurse<'a, T, F>(mut v: &'a mut [T], is_less: &mut F, mut pred: Option<&'a T>, mut limit: u32)
where
    F: FnMut(&T, &T) -> bool,
{
    // Slices of up to this length get sorted using insertion sort. <br>长度不超过此长度的切片将使用插入排序进行排序。<br>
    const MAX_INSERTION: usize = 20;

    // True if the last partitioning was reasonably balanced. <br>如果最后一个分区合理平衡，则为 true。<br>
    let mut was_balanced = true;
    // True if the last partitioning didn't shuffle elements (the slice was already partitioned). <br>如果最后一个分区没有重排元素 (切片已分区)，则为 true。<br>
    let mut was_partitioned = true;

    loop {
        let len = v.len();

        // Very short slices get sorted using insertion sort. <br>非常短的切片使用插入排序进行排序。<br>
        if len <= MAX_INSERTION {
            insertion_sort(v, is_less);
            return;
        }

        // If too many bad pivot choices were made, simply fall back to heapsort in order to guarantee `O(n * log(n))` worst-case. <br>如果做出了太多错误的枢轴选择，则只需回退到 heapsort 以确保 `O(n * log(n))` 最坏的情况。<br>
        //
        if limit == 0 {
            heapsort(v, is_less);
            return;
        }

        // If the last partitioning was imbalanced, try breaking patterns in the slice by shuffling some elements around. <br>如果最后一个分区不平衡，请尝试通过改组一些元素来破坏切片中的模式。<br>
        // Hopefully we'll choose a better pivot this time. <br>希望这次我们选择一个更好的支点。<br>
        if !was_balanced {
            break_patterns(v);
            limit -= 1;
        }

        // Choose a pivot and try guessing whether the slice is already sorted. <br>选择一个枢轴，然后尝试猜测切片是否已排序。<br>
        let (pivot, likely_sorted) = choose_pivot(v, is_less);

        // If the last partitioning was decently balanced and didn't shuffle elements, and if pivot selection predicts the slice is likely already sorted... <br>如果最后一个分区达到了合理的平衡并且没有使元素混洗，并且如果枢轴选择预测了切片，则该切片可能已经被排序了...<br>
        //
        if was_balanced && was_partitioned && likely_sorted {
            // Try identifying several out-of-order elements and shifting them to correct positions. <br>尝试识别几个乱序的元素，然后将它们移到正确的位置。<br>
            // If the slice ends up being completely sorted, we're done. <br>如果切片最终被完全排序，那么我们就完成了。<br>
            if partial_insertion_sort(v, is_less) {
                return;
            }
        }

        // If the chosen pivot is equal to the predecessor, then it's the smallest element in the slice. <br>如果选择的枢轴等于前一个枢轴，则它是切片中的最小元素。<br>
        // Partition the slice into elements equal to and elements greater than the pivot. <br>将切片划分为等于和大于枢轴的元素。<br>
        // This case is usually hit when the slice contains many duplicate elements. <br>当切片包含许多重复元素时，通常会遇到这种情况。<br>
        if let Some(p) = pred {
            if !is_less(p, &v[pivot]) {
                let mid = partition_equal(v, pivot, is_less);

                // Continue sorting elements greater than the pivot. <br>继续对大于枢轴的元素进行排序。<br>
                v = &mut v[mid..];
                continue;
            }
        }

        // Partition the slice. <br>对切片进行分区。<br>
        let (mid, was_p) = partition(v, pivot, is_less);
        was_balanced = cmp::min(mid, len - mid) >= len / 8;
        was_partitioned = was_p;

        // Split the slice into `left`, `pivot`, and `right`. <br>将切片分为 `left`，`pivot` 和 `right`。<br>
        let (left, right) = v.split_at_mut(mid);
        let (pivot, right) = right.split_at_mut(1);
        let pivot = &pivot[0];

        // Recurse into the shorter side only in order to minimize the total number of recursive calls and consume less stack space. <br>递归到较短的一侧只是为了最大程度地减少递归调用的总数并占用较少的栈空间。<br>
        // Then just continue with the longer side (this is akin to tail recursion). <br>然后，继续较长的那一面 (这类似于尾部递归)。<br>
        //
        if left.len() < right.len() {
            recurse(left, is_less, pred, limit);
            v = right;
            pred = Some(pivot);
        } else {
            recurse(right, is_less, Some(pivot), limit);
            v = left;
        }
    }
}

/// Sorts `v` using pattern-defeating quicksort, which is *O*(*n* \* log(*n*)) worst-case. <br>使用模式破坏快速排序对 `v` 进行排序，这是 *O*(*n*\*log(* n*)) 最坏的情况。<br>
pub fn quicksort<T, F>(v: &mut [T], mut is_less: F)
where
    F: FnMut(&T, &T) -> bool,
{
    // Sorting has no meaningful behavior on zero-sized types. <br>零大小类型的排序没有有意义的行为。<br>
    if mem::size_of::<T>() == 0 {
        return;
    }

    // Limit the number of imbalanced partitions to `floor(log2(len)) + 1`. <br>将不平衡分区的数量限制为 `floor(log2(len)) + 1`。<br>
    let limit = usize::BITS - v.len().leading_zeros();

    recurse(v, &mut is_less, None, limit);
}

fn partition_at_index_loop<'a, T, F>(
    mut v: &'a mut [T],
    mut index: usize,
    is_less: &mut F,
    mut pred: Option<&'a T>,
) where
    F: FnMut(&T, &T) -> bool,
{
    loop {
        // For slices of up to this length it's probably faster to simply sort them. <br>对于不超过此长度的切片，将其简单排序可能会更快。<br>
        const MAX_INSERTION: usize = 10;
        if v.len() <= MAX_INSERTION {
            insertion_sort(v, is_less);
            return;
        }

        // Choose a pivot <br>选择一个 pivot<br>
        let (pivot, _) = choose_pivot(v, is_less);

        // If the chosen pivot is equal to the predecessor, then it's the smallest element in the slice. <br>如果选择的枢轴等于前一个枢轴，则它是切片中的最小元素。<br>
        // Partition the slice into elements equal to and elements greater than the pivot. <br>将切片划分为等于和大于枢轴的元素。<br>
        // This case is usually hit when the slice contains many duplicate elements. <br>当切片包含许多重复元素时，通常会遇到这种情况。<br>
        if let Some(p) = pred {
            if !is_less(p, &v[pivot]) {
                let mid = partition_equal(v, pivot, is_less);

                // If we've passed our index, then we're good. <br>如果我们通过了索引，那么我们就很好。<br>
                if mid > index {
                    return;
                }

                // Otherwise, continue sorting elements greater than the pivot. <br>否则，继续对大于枢轴的元素进行排序。<br>
                v = &mut v[mid..];
                index = index - mid;
                pred = None;
                continue;
            }
        }

        let (mid, _) = partition(v, pivot, is_less);

        // Split the slice into `left`, `pivot`, and `right`. <br>将切片分为 `left`，`pivot` 和 `right`。<br>
        let (left, right) = v.split_at_mut(mid);
        let (pivot, right) = right.split_at_mut(1);
        let pivot = &pivot[0];

        if mid < index {
            v = right;
            index = index - mid - 1;
            pred = Some(pivot);
        } else if mid > index {
            v = left;
        } else {
            // If mid == index, then we're done, since partition() guaranteed that all elements after mid are greater than or equal to mid. <br>如果 mid == index，那么我们就完成了，因为 partition() 保证 mid 之后的所有元素都大于或等于 mid。<br>
            //
            return;
        }
    }
}

pub fn partition_at_index<T, F>(
    v: &mut [T],
    index: usize,
    mut is_less: F,
) -> (&mut [T], &mut T, &mut [T])
where
    F: FnMut(&T, &T) -> bool,
{
    use cmp::Ordering::Greater;
    use cmp::Ordering::Less;

    if index >= v.len() {
        panic!("partition_at_index index {} greater than length of slice {}", index, v.len());
    }

    if mem::size_of::<T>() == 0 {
        // Sorting has no meaningful behavior on zero-sized types. <br>零大小类型的排序没有有意义的行为。<br> Do nothing. <br>什么也不做。<br>
    } else if index == v.len() - 1 {
        // Find max element and place it in the last position of the array. <br>查找最大元素并将其放置在数组的最后一个位置。<br>
        // We're free to use `unwrap()` here because we know v must not be empty. <br>我们可以在这里自由使用 `unwrap()`，因为我们知道 v 不能为空。<br>
        let (max_index, _) = v
            .iter()
            .enumerate()
            .max_by(|&(_, x), &(_, y)| if is_less(x, y) { Less } else { Greater })
            .unwrap();
        v.swap(max_index, index);
    } else if index == 0 {
        // Find min element and place it in the first position of the array. <br>查找 min 元素并将其放置在数组的第一个位置。<br>
        // We're free to use `unwrap()` here because we know v must not be empty. <br>我们可以在这里自由使用 `unwrap()`，因为我们知道 v 不能为空。<br>
        let (min_index, _) = v
            .iter()
            .enumerate()
            .min_by(|&(_, x), &(_, y)| if is_less(x, y) { Less } else { Greater })
            .unwrap();
        v.swap(min_index, index);
    } else {
        partition_at_index_loop(v, index, &mut is_less, None);
    }

    let (left, right) = v.split_at_mut(index);
    let (pivot, right) = right.split_at_mut(1);
    let pivot = &mut pivot[0];
    (left, pivot, right)
}
