//! Defines utf8 error type. <br>定义 utf8 错误类型。<br>

use crate::fmt;

/// Errors which can occur when attempting to interpret a sequence of [`u8`] as a string. <br>尝试将 [`u8`] 的序列解释为字符串时可能发生的错误。<br>
///
/// As such, the `from_utf8` family of functions and methods for both [`String`]s and [`&str`]s make use of this error, for example. <br>这样，例如 [`String`] 和 [`&str`] 的 `from_utf8` 系列函数和方法都利用了此错误。<br>
///
/// [`String`]: ../../std/string/struct.String.html#method.from_utf8
/// [`&str`]: super::from_utf8
///
/// # Examples
///
/// This error type’s methods can be used to create functionality similar to `String::from_utf8_lossy` without allocating heap memory: <br>此错误类型的方法可用于创建类似于 `String::from_utf8_lossy` 的功能，而无需分配堆内存：<br>
///
///
/// ```
/// fn from_utf8_lossy<F>(mut input: &[u8], mut push: F) where F: FnMut(&str) {
///     loop {
///         match std::str::from_utf8(input) {
///             Ok(valid) => {
///                 push(valid);
///                 break
///             }
///             Err(error) => {
///                 let (valid, after_valid) = input.split_at(error.valid_up_to());
///                 unsafe {
///                     push(std::str::from_utf8_unchecked(valid))
///                 }
///                 push("\u{FFFD}");
///
///                 if let Some(invalid_sequence_length) = error.error_len() {
///                     input = &after_valid[invalid_sequence_length..]
///                 } else {
///                     break
///                 }
///             }
///         }
///     }
/// }
/// ```
///
///
#[derive(Copy, Eq, PartialEq, Clone, Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Utf8Error {
    pub(super) valid_up_to: usize,
    pub(super) error_len: Option<u8>,
}

impl Utf8Error {
    /// Returns the index in the given string up to which valid UTF-8 was verified. <br>返回给定字符串中的索引，直到对其进行有效 UTF-8 验证为止。<br>
    ///
    /// It is the maximum index such that `from_utf8(&input[..index])` would return `Ok(_)`. <br>它是使 `from_utf8(&input[..index])` 返回 `Ok(_)` 的最大索引。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```
    /// use std::str;
    ///
    /// // some invalid bytes, in a vector <br>vector 中的一些无效字节<br>
    /// let sparkle_heart = vec![0, 159, 146, 150];
    ///
    /// // std::str::from_utf8 returns a Utf8Error <br>std::str::from_utf8 返回一个 Utf8Error<br>
    /// let error = str::from_utf8(&sparkle_heart).unwrap_err();
    ///
    /// // the second byte is invalid here <br>第二个字节在这里无效<br>
    /// assert_eq!(1, error.valid_up_to());
    /// ```
    ///
    #[stable(feature = "utf8_error", since = "1.5.0")]
    #[rustc_const_stable(feature = "const_str_from_utf8_shared", since = "1.63.0")]
    #[must_use]
    #[inline]
    pub const fn valid_up_to(&self) -> usize {
        self.valid_up_to
    }

    /// Provides more information about the failure: <br>提供有关失败的更多信息：<br>
    ///
    /// * `None`: the end of the input was reached unexpectedly. <br>`None`: 意外到达输入的结尾。<br>
    ///   `self.valid_up_to()` is 1 to 3 bytes from the end of the input. <br>`self.valid_up_to()` 是从输入的末尾开始的 1 到 3 个字节。<br>
    ///   If a byte stream (such as a file or a network socket) is being decoded incrementally, this could be a valid `char` whose UTF-8 byte sequence is spanning multiple chunks. <br>如果字节流 (例如文件或网络套接字) 正在以增量方式进行解码，则这可能是有效的 `char`，其 UTF-8 字节序列跨越了多个块。<br>
    ///
    ///
    /// * `Some(len)`: an unexpected byte was encountered. <br>`Some(len)`: 遇到意外字节。<br>
    ///   The length provided is that of the invalid byte sequence that starts at the index given by `valid_up_to()`. <br>提供的长度是从 `valid_up_to()` 给定的索引处开始的无效字节序列的长度。<br>
    ///   Decoding should resume after that sequence (after inserting a [`U+FFFD REPLACEMENT CHARACTER`][U+FFFD]) in case of lossy decoding. <br>如果有损解码，则应在该序列之后 (插入 [`U+FFFD REPLACEMENT CHARACTER`][U+FFFD] 之后) 恢复解码。<br>
    ///
    /// [U+FFFD]: ../../std/char/constant.REPLACEMENT_CHARACTER.html
    ///
    ///
    ///
    #[stable(feature = "utf8_error_error_len", since = "1.20.0")]
    #[rustc_const_stable(feature = "const_str_from_utf8_shared", since = "1.63.0")]
    #[must_use]
    #[inline]
    pub const fn error_len(&self) -> Option<usize> {
        // FIXME: This should become `map` again, once it's `const` <br>这应该再次成为 `map`，一旦它是 `const`<br>
        match self.error_len {
            Some(len) => Some(len as usize),
            None => None,
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Display for Utf8Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        if let Some(error_len) = self.error_len {
            write!(
                f,
                "invalid utf-8 sequence of {} bytes from index {}",
                error_len, self.valid_up_to
            )
        } else {
            write!(f, "incomplete utf-8 byte sequence from index {}", self.valid_up_to)
        }
    }
}

/// An error returned when parsing a `bool` using [`from_str`] fails <br>使用 [`from_str`] 解析 `bool` 失败时返回错误<br>
///
/// [`from_str`]: super::FromStr::from_str
#[derive(Debug, Clone, PartialEq, Eq)]
#[non_exhaustive]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct ParseBoolError;

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Display for ParseBoolError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        "provided string was not `true` or `false`".fmt(f)
    }
}
