//! Utilities for parsing DWARF-encoded data streams. <br>解析 DWARF 编码的数据流的实用工具。<br>
//! See <http://www.dwarfstd.org>, DWARF-4 standard, Section 7 - "Data Representation" <br>参见 <http://www.dwarfstd.org>，DWARF-4 标准第 7 节 - "Data Representation"<br>
//!

// This module is used only by x86_64-pc-windows-gnu for now, but we are compiling it everywhere to avoid regressions. <br>目前，此模块仅由 x86_64-pc-windows-gnu 使用，但是我们在各处编译它，以避免出现回归。<br>
//
#![allow(unused)]

#[cfg(test)]
mod tests;

pub mod eh;

use core::mem;

pub struct DwarfReader {
    pub ptr: *const u8,
}

#[repr(C, packed)]
struct Unaligned<T>(T);

impl DwarfReader {
    pub fn new(ptr: *const u8) -> DwarfReader {
        DwarfReader { ptr }
    }

    // DWARF streams are packed, so e.g., a u32 would not necessarily be aligned on a 4-byte boundary. <br>DWARF 流已被包装，因此，例如 u32 不必在 4 字节边界上对齐。<br>
    // This may cause problems on platforms with strict alignment requirements. <br>这可能会在具有严格对齐要求的平台上引起问题。<br>
    // By wrapping data in a "packed" struct, we are telling the backend to generate "misalignment-safe" code. <br>通过将数据包装在 "packed" 结构体中，我们告诉后端生成 "misalignment-safe" 代码。<br>
    //
    pub unsafe fn read<T: Copy>(&mut self) -> T {
        let Unaligned(result) = *(self.ptr as *const Unaligned<T>);
        self.ptr = self.ptr.add(mem::size_of::<T>());
        result
    }

    // ULEB128 and SLEB128 encodings are defined in Section 7.6 - "Variable Length Data". <br>7.6-"可变长度数据" 节中定义了 ULEB128 和 SLEB128 编码。<br>
    //
    pub unsafe fn read_uleb128(&mut self) -> u64 {
        let mut shift: usize = 0;
        let mut result: u64 = 0;
        let mut byte: u8;
        loop {
            byte = self.read::<u8>();
            result |= ((byte & 0x7F) as u64) << shift;
            shift += 7;
            if byte & 0x80 == 0 {
                break;
            }
        }
        result
    }

    pub unsafe fn read_sleb128(&mut self) -> i64 {
        let mut shift: u32 = 0;
        let mut result: u64 = 0;
        let mut byte: u8;
        loop {
            byte = self.read::<u8>();
            result |= ((byte & 0x7F) as u64) << shift;
            shift += 7;
            if byte & 0x80 == 0 {
                break;
            }
        }
        // sign-extend
        if shift < u64::BITS && (byte & 0x40) != 0 {
            result |= (!0 as u64) << shift;
        }
        result as i64
    }
}
