//! Implementation of panics backed by libgcc/libunwind (in some form). <br>由 libgcc/libunwind 支持的 panics 的实现 (以某种形式)。<br>
//!
//! For background on exception handling and stack unwinding please see "Exception Handling in LLVM" (llvm.org/docs/ExceptionHandling.html) and documents linked from it. <br>有关异常处理和栈的背景展开，请参见 "LLVM 中的异常处理" (llvm.org/docs/ExceptionHandling.html) 及其链接的文档。<br>
//! These are also good reads: <br>这些也是不错的读物：<br>
//!  * <https://itanium-cxx-abi.github.io/cxx-abi/abi-eh.html>
//!  * <https://monoinfinito.wordpress.com/series/exception-handling-in-c/>
//!  * <https://www.airs.com/blog/index.php?s=exception+frames>
//!
//! ## A brief summary <br>简要总结<br>
//!
//! Exception handling happens in two phases: a search phase and a cleanup phase. <br>异常处理分为两个阶段：搜索阶段和清理阶段。<br>
//!
//! In both phases the unwinder walks stack frames from top to bottom using information from the stack frame unwind sections of the current process's modules ("module" here refers to an OS module, i.e., an executable or a dynamic library). <br>在这两个阶段中，展开器都使用当前进程模块的栈框架 unwind 部分 (此处的 "module" 指 OS 模块，即可执行文件或动态库) 中的信息从上到下遍历栈框架。<br>
//!
//!
//! For each stack frame, it invokes the associated "personality routine", whose address is also stored in the unwind info section. <br>对于每个栈帧，它调用关联的 "personality routine"，其地址也存储在 unwind info 节中。<br>
//!
//! In the search phase, the job of a personality routine is to examine exception object being thrown, and to decide whether it should be caught at that stack frame. <br>在搜索阶段，personality 例程的工作是检查抛出的异常对象，并确定是否应在该栈帧处捕获该异常对象。<br> Once the handler frame has been identified, cleanup phase begins. <br>一旦确定了处理程序框架，清除阶段就会开始。<br>
//!
//! In the cleanup phase, the unwinder invokes each personality routine again. <br>在清理阶段，展开器再次调用每个 personality 例程。<br>
//! This time it decides which (if any) cleanup code needs to be run for the current stack frame. <br>这次，它决定需要为当前栈帧运行哪些 (如果有的话) 清除代码。<br> If so, the control is transferred to a special branch in the function body, the "landing pad", which invokes destructors, frees memory, etc. <br>如果是这样，则将控件转移到函数体中的特殊分支 "landing pad"，该分支调用析构函数，释放内存等。<br>
//! At the end of the landing pad, control is transferred back to the unwinder and unwinding resumes. <br>在着陆架的尽头，控制权被转移回展开器，并且展开恢复。<br>
//!
//! Once stack has been unwound down to the handler frame level, unwinding stops and the last personality routine transfers control to the catch block. <br>一旦将栈展开到处理程序框架级别，展开将停止，最后一个 personality 例程将控制权转移到 catch 块。<br>
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

use alloc::boxed::Box;
use core::any::Any;

use crate::dwarf::eh::{self, EHAction, EHContext};
use libc::{c_int, uintptr_t};
use unwind as uw;

#[repr(C)]
struct Exception {
    _uwe: uw::_Unwind_Exception,
    cause: Box<dyn Any + Send>,
}

pub unsafe fn panic(data: Box<dyn Any + Send>) -> u32 {
    let exception = Box::new(Exception {
        _uwe: uw::_Unwind_Exception {
            exception_class: rust_exception_class(),
            exception_cleanup,
            private: [0; uw::unwinder_private_data_size],
        },
        cause: data,
    });
    let exception_param = Box::into_raw(exception) as *mut uw::_Unwind_Exception;
    return uw::_Unwind_RaiseException(exception_param) as u32;

    extern "C" fn exception_cleanup(
        _unwind_code: uw::_Unwind_Reason_Code,
        exception: *mut uw::_Unwind_Exception,
    ) {
        unsafe {
            let _: Box<Exception> = Box::from_raw(exception as *mut Exception);
            super::__rust_drop_panic();
        }
    }
}

pub unsafe fn cleanup(ptr: *mut u8) -> Box<dyn Any + Send> {
    let exception = ptr as *mut uw::_Unwind_Exception;
    if (*exception).exception_class != rust_exception_class() {
        uw::_Unwind_DeleteException(exception);
        super::__rust_foreign_exception();
    } else {
        let exception = Box::from_raw(exception as *mut Exception);
        exception.cause
    }
}

// Rust's exception class identifier. <br>Rust 的异常类标识符。<br>
// This is used by personality routines to determine whether the exception was thrown by their own runtime. <br>personality 例程使用它来确定异常是否由其自己的运行时引发。<br>
fn rust_exception_class() -> uw::_Unwind_Exception_Class {
    // M O Z \0  R U S T -- vendor, language <br>MOZ\0 RUST - 供应商，语言<br>
    0x4d4f5a_00_52555354
}

// Register ids were lifted from LLVM's TargetLowering::getExceptionPointerRegister() and TargetLowering::getExceptionSelectorRegister() for each architecture, then mapped to DWARF register numbers via register definition tables (typically <arch>RegisterInfo.td, search for "DwarfRegNum"). <br>对于每种体系结构，从 LLVM 的 TargetLowering::getExceptionPointerRegister () 和 TargetLowering::getExceptionSelectorRegister () 提取寄存器 ID，然后通过寄存器定义表将其映射到 DWARF 寄存器号 (通常是) <arch>RegisterInfo.td，搜索 "DwarfRegNum")。<br>
//
// See also https://llvm.org/docs/WritingAnLLVMBackend.html#defining-a-register. <br>另请参见 https://llvm.org/docs/WritingAnLLVMBackend.html#defining-a-register。<br>
//
//

#[cfg(target_arch = "x86")]
const UNWIND_DATA_REG: (i32, i32) = (0, 2); // EAX, EDX

#[cfg(target_arch = "x86_64")]
const UNWIND_DATA_REG: (i32, i32) = (0, 1); // RAX, RDX

#[cfg(any(target_arch = "arm", target_arch = "aarch64"))]
const UNWIND_DATA_REG: (i32, i32) = (0, 1); // R0, R1 / X0, X1

#[cfg(target_arch = "m68k")]
const UNWIND_DATA_REG: (i32, i32) = (0, 1); // D0, D1

#[cfg(any(target_arch = "mips", target_arch = "mips64"))]
const UNWIND_DATA_REG: (i32, i32) = (4, 5); // A0, A1

#[cfg(any(target_arch = "powerpc", target_arch = "powerpc64"))]
const UNWIND_DATA_REG: (i32, i32) = (3, 4); // R3, R4 / X3, X4

#[cfg(target_arch = "s390x")]
const UNWIND_DATA_REG: (i32, i32) = (6, 7); // R6, R7

#[cfg(any(target_arch = "sparc", target_arch = "sparc64"))]
const UNWIND_DATA_REG: (i32, i32) = (24, 25); // I0, I1

#[cfg(target_arch = "hexagon")]
const UNWIND_DATA_REG: (i32, i32) = (0, 1); // R0, R1

#[cfg(any(target_arch = "riscv64", target_arch = "riscv32"))]
const UNWIND_DATA_REG: (i32, i32) = (10, 11); // x10, x11

// The following code is based on GCC's C and C++ personality routines. <br>以下代码基于 GCC 的 C 和 C++ 的 personality 例程。<br> For reference, see: <br>有关引用，请参见：<br>
// https://github.com/gcc-mirror/gcc/blob/master/libstdc++-v3/libsupc++/eh_personality.cc
// https://github.com/gcc-mirror/gcc/blob/trunk/libgcc/unwind-c.c

cfg_if::cfg_if! {
    if #[cfg(all(target_arch = "arm", not(target_os = "ios"), not(target_os = "netbsd")))] {
        // ARM EHABI personality routine. <br>ARM EHABI 的 personality 例程。<br>
        // https://infocenter.arm.com/help/topic/com.arm.doc.ihi0038b/IHI0038B_ehabi.pdf
        //
        // iOS uses the default routine instead since it uses SjLj unwinding. <br>iOS 使用默认例程，因为它使用 SjLj 展开。<br>
        #[lang = "eh_personality"]
        unsafe extern "C" fn rust_eh_personality(state: uw::_Unwind_State,
                                                 exception_object: *mut uw::_Unwind_Exception,
                                                 context: *mut uw::_Unwind_Context)
                                                 -> uw::_Unwind_Reason_Code {
            let state = state as c_int;
            let action = state & uw::_US_ACTION_MASK as c_int;
            let search_phase = if action == uw::_US_VIRTUAL_UNWIND_FRAME as c_int {
                // Backtraces on ARM will call the personality routine with state == _US_VIRTUAL_UNWIND_FRAME | _US_FORCE_UNWIND. <br>ARM 上的回溯将调用具有 state == _US_VIRTUAL_UNWIND_FRAME | _US_FORCE_UNWIND 的 personality 例程。<br>
                // In those cases we want to continue unwinding the stack, otherwise all our backtraces would end at __rust_try <br>在那种情况下，我们想继续展开栈，否则我们所有的回溯都将以 __rust_try 结尾<br>
                //
                //
                if state & uw::_US_FORCE_UNWIND as c_int != 0 {
                    return continue_unwind(exception_object, context);
                }
                true
            } else if action == uw::_US_UNWIND_FRAME_STARTING as c_int {
                false
            } else if action == uw::_US_UNWIND_FRAME_RESUME as c_int {
                return continue_unwind(exception_object, context);
            } else {
                return uw::_URC_FAILURE;
            };

            // The DWARF unwinder assumes that _Unwind_Context holds things like the function and LSDA pointers, however ARM EHABI places them into the exception object. <br>DWARF 展开器假定 _Unwind_Context 包含函数和 LSDA 指针之类的内容，但是 ARM EHABI 将其放入异常对象中。<br>
            // To preserve signatures of functions like _Unwind_GetLanguageSpecificData(), which take only the context pointer, GCC personality routines stash a pointer to exception_object in the context, using location reserved for ARM's "scratch register" (r12). <br>为了保留仅使用上下文指针的 _Unwind_GetLanguageSpecificData() 之类的函数签名，GCC 的 personality 例程使用为 "暂存寄存器" (r12) 保留的位置在上下文中存储了指向 exception_object 的指针。<br>
            //
            //
            //
            //
            uw::_Unwind_SetGR(context,
                              uw::UNWIND_POINTER_REG,
                              exception_object as uw::_Unwind_Ptr);
            // ...A more principled approach would be to provide the full definition of ARM's _Unwind_Context in our libunwind bindings and fetch the required data from there directly, bypassing DWARF compatibility functions. <br>... 更原则的方法是在我们的 libunwind 绑定中提供分支的 _Unwind_Context 的完整定义，并绕过 DWARF 兼容性函数直接从那里获取所需的数据。<br>
            //
            //

            let eh_action = match find_eh_action(context) {
                Ok(action) => action,
                Err(_) => return uw::_URC_FAILURE,
            };
            if search_phase {
                match eh_action {
                    EHAction::None |
                    EHAction::Cleanup(_) => return continue_unwind(exception_object, context),
                    EHAction::Catch(_) => {
                        // EHABI requires the personality routine to update the SP value in the barrier cache of the exception object. <br>EHABI 要求 personality 例程更新异常对象的屏障缓存中的 SP 值。<br>
                        //
                        (*exception_object).private[5] =
                            uw::_Unwind_GetGR(context, uw::UNWIND_SP_REG);
                        return uw::_URC_HANDLER_FOUND;
                    }
                    EHAction::Terminate => return uw::_URC_FAILURE,
                }
            } else {
                match eh_action {
                    EHAction::None => return continue_unwind(exception_object, context),
                    EHAction::Cleanup(lpad) |
                    EHAction::Catch(lpad) => {
                        uw::_Unwind_SetGR(context, UNWIND_DATA_REG.0,
                                          exception_object as uintptr_t);
                        uw::_Unwind_SetGR(context, UNWIND_DATA_REG.1, 0);
                        uw::_Unwind_SetIP(context, lpad);
                        return uw::_URC_INSTALL_CONTEXT;
                    }
                    EHAction::Terminate => return uw::_URC_FAILURE,
                }
            }

            // On ARM EHABI the personality routine is responsible for actually unwinding a single stack frame before returning (ARM EHABI Sec. <br>在 ARM EHABI 上， personality 例程负责在返回之前实际展开单个栈帧 (ARM EHABI Sec。<br>
            // 6.1).
            unsafe fn continue_unwind(exception_object: *mut uw::_Unwind_Exception,
                                      context: *mut uw::_Unwind_Context)
                                      -> uw::_Unwind_Reason_Code {
                if __gnu_unwind_frame(exception_object, context) == uw::_URC_NO_REASON {
                    uw::_URC_CONTINUE_UNWIND
                } else {
                    uw::_URC_FAILURE
                }
            }
            // defined in libgcc <br>在 libgcc 中定义<br>
            extern "C" {
                fn __gnu_unwind_frame(exception_object: *mut uw::_Unwind_Exception,
                                      context: *mut uw::_Unwind_Context)
                                      -> uw::_Unwind_Reason_Code;
            }
        }
    } else {
        // Default personality routine, which is used directly on most targets and indirectly on Windows x86_64 via SEH. <br>默认 personality 例程，可直接在大多数目标上使用，并通过 SEH 间接在 Windows x86_64 上使用。<br>
        //
        unsafe extern "C" fn rust_eh_personality_impl(version: c_int,
                                                      actions: uw::_Unwind_Action,
                                                      _exception_class: uw::_Unwind_Exception_Class,
                                                      exception_object: *mut uw::_Unwind_Exception,
                                                      context: *mut uw::_Unwind_Context)
                                                      -> uw::_Unwind_Reason_Code {
            if version != 1 {
                return uw::_URC_FATAL_PHASE1_ERROR;
            }
            let eh_action = match find_eh_action(context) {
                Ok(action) => action,
                Err(_) => return uw::_URC_FATAL_PHASE1_ERROR,
            };
            if actions as i32 & uw::_UA_SEARCH_PHASE as i32 != 0 {
                match eh_action {
                    EHAction::None |
                    EHAction::Cleanup(_) => uw::_URC_CONTINUE_UNWIND,
                    EHAction::Catch(_) => uw::_URC_HANDLER_FOUND,
                    EHAction::Terminate => uw::_URC_FATAL_PHASE1_ERROR,
                }
            } else {
                match eh_action {
                    EHAction::None => uw::_URC_CONTINUE_UNWIND,
                    EHAction::Cleanup(lpad) |
                    EHAction::Catch(lpad) => {
                        uw::_Unwind_SetGR(context, UNWIND_DATA_REG.0,
                            exception_object as uintptr_t);
                        uw::_Unwind_SetGR(context, UNWIND_DATA_REG.1, 0);
                        uw::_Unwind_SetIP(context, lpad);
                        uw::_URC_INSTALL_CONTEXT
                    }
                    EHAction::Terminate => uw::_URC_FATAL_PHASE2_ERROR,
                }
            }
        }

        cfg_if::cfg_if! {
            if #[cfg(all(windows, target_arch = "x86_64", target_env = "gnu"))] {
                // On x86_64 MinGW targets, the unwinding mechanism is SEH however the unwind handler data (aka LSDA) uses GCC-compatible encoding. <br>在 x86_64 MinGW 目标上，展开机制为 SEH，但是 unwind 处理程序数据 (aka LSDA) 使用 GCC 兼容的编码。<br>
                //
                #[lang = "eh_personality"]
                #[allow(nonstandard_style)]
                unsafe extern "C" fn rust_eh_personality(exceptionRecord: *mut uw::EXCEPTION_RECORD,
                        establisherFrame: uw::LPVOID,
                        contextRecord: *mut uw::CONTEXT,
                        dispatcherContext: *mut uw::DISPATCHER_CONTEXT)
                        -> uw::EXCEPTION_DISPOSITION {
                    uw::_GCC_specific_handler(exceptionRecord,
                                             establisherFrame,
                                             contextRecord,
                                             dispatcherContext,
                                             rust_eh_personality_impl)
                }
            } else {
                // The personality routine for most of our targets. <br>我们大多数目标的 personality 例程。<br>
                #[lang = "eh_personality"]
                unsafe extern "C" fn rust_eh_personality(version: c_int,
                        actions: uw::_Unwind_Action,
                        exception_class: uw::_Unwind_Exception_Class,
                        exception_object: *mut uw::_Unwind_Exception,
                        context: *mut uw::_Unwind_Context)
                        -> uw::_Unwind_Reason_Code {
                    rust_eh_personality_impl(version,
                                             actions,
                                             exception_class,
                                             exception_object,
                                             context)
                }
            }
        }
    }
}

unsafe fn find_eh_action(context: *mut uw::_Unwind_Context) -> Result<EHAction, ()> {
    let lsda = uw::_Unwind_GetLanguageSpecificData(context) as *const u8;
    let mut ip_before_instr: c_int = 0;
    let ip = uw::_Unwind_GetIPInfo(context, &mut ip_before_instr);
    let eh_context = EHContext {
        // The return address points 1 byte past the call instruction, which could be in the next IP range in LSDA range table. <br>返回地址指向调用指令后的 1 个字节，该地址可能在 LSDA 范围表中的下一个 IP 范围内。<br>
        //
        ip: if ip_before_instr != 0 { ip } else { ip - 1 },
        func_start: uw::_Unwind_GetRegionStart(context),
        get_text_start: &|| uw::_Unwind_GetTextRelBase(context),
        get_data_start: &|| uw::_Unwind_GetDataRelBase(context),
    };
    eh::find_eh_action(lsda, &eh_context)
}

// Frame unwind info registration <br>框架 unwind 信息注册<br>
//
// Each module's image contains a frame unwind info section (usually ".eh_frame"). <br>每个模块的映像都包含一个框架 unwind 信息部分 (通常为 ".eh_frame")。<br> When a module is loaded/unloaded into the process, the unwinder must be informed about the location of this section in memory. <br>当一个模块加载/卸载到进程中时，必须通知展开器这个部分在内存中的位置。<br> The methods of achieving that vary by the platform. <br>实现目标的方法因平台而异。<br>
// On some (e.g., Linux), the unwinder can discover unwind info sections on its own (by dynamically enumerating currently loaded modules via the dl_iterate_phdr() API and finding their ".eh_frame" sections); <br>在某些情况下 (例如 Linux)，展开器可以自己发现展开信息部分 (通过 dl_iterate_phdr() API 动态枚举当前加载的模块并找到它们的 ".eh_frame" 部分)；<br> Others, like Windows, require modules to actively register their unwind info sections via unwinder API. <br>Windows 等其他组件则需要模块通过展开器 API 主动注册其 unwind 信息部分。<br>
//
//
// This module defines two symbols which are referenced and called from rsbegin.rs to register our information with the GCC runtime. <br>该模块定义了两个符号，这些符号从 rsbegin.rs 进行引用和调用，以将我们的信息注册到 GCC 运行时。<br>
// The implementation of stack unwinding is (for now) deferred to libgcc_eh, however Rust crates use these Rust-specific entry points to avoid potential clashes with any GCC runtime. <br>栈扩展的实现 (现在) 推迟到 libgcc_eh 进行，但是 Rust crates 使用这些 Rust 特定的入口点来避免与任何 GCC 运行时潜在的冲突。<br>
//
//
//
//
//
//
//
//
#[cfg(all(target_os = "windows", target_arch = "x86", target_env = "gnu"))]
pub mod eh_frame_registry {
    extern "C" {
        fn __register_frame_info(eh_frame_begin: *const u8, object: *mut u8);
        fn __deregister_frame_info(eh_frame_begin: *const u8, object: *mut u8);
    }

    #[rustc_std_internal_symbol]
    pub unsafe extern "C" fn rust_eh_register_frames(eh_frame_begin: *const u8, object: *mut u8) {
        __register_frame_info(eh_frame_begin, object);
    }

    #[rustc_std_internal_symbol]
    pub unsafe extern "C" fn rust_eh_unregister_frames(eh_frame_begin: *const u8, object: *mut u8) {
        __deregister_frame_info(eh_frame_begin, object);
    }
}
