//! This module contains the LLVM intrinsics bindings that provide the functionality for this crate. <br>该模块包含为此 crate 提供功能的 LLVM 内部函数绑定。<br>
//!
//! The LLVM assembly language is documented here: <https://llvm.org/docs/LangRef.html> <br>LLVM 汇编语言的文档在这里: <https://llvm.org/docs/LangRef.html><br>
//!
//! A quick glossary of jargon that may appear in this module, mostly paraphrasing LLVM's LangRef: <br>本模块中可能出现的行话快速词汇表，主要是对 LLVM 的 LangRef 的解释:<br>
//! - poison: "undefined behavior as a value".  specifically, it is like uninit memory (such as padding bytes). <br>具体来说，它类似于未指定内存 (例如填充字节)。<br> it is "safe" to create poison, BUT poison MUST NOT be observed from safe code, as operations on poison return poison, like NaN. <br>创建 poison 是 "safe" 的，但不能从安全代码中观察 poison，因为对 poison 的操作会返回 poison，如 NaN。<br>
//!
//! unlike NaN, which has defined comparisons, poison is neither true nor false, and LLVM may also convert it to undef (at which point it is both). <br>与定义了比较的 NaN 不同，poison 既不是真也不是假，LLVM 也可以将其转换为 undef (此时两者都是)。<br> so, it can't be conditioned on, either. <br>所以，也不能以此为条件。<br>
//! - undef: "a value that is every value".  functionally like poison, insofar as Rust is concerned. <br>就 Rust 而言，功能上类似于 poison。<br> poison may become this. <br>poison 可能变成这个。<br> note:
//!   this means that division by poison or undef is like division by zero, which means it inflicts... <br>这意味着除以 poison 或 undef 就像除以零一样，这意味着它会造成...<br>
//! - "UB": poison and undef cover most of what people call "UB". <br>"UB": poison 和 undef 覆盖了大多数人们所说的 "UB"。<br> "UB" means this operation immediately invalidates the program: <br>"UB" 表示此操作立即使程序无效:<br>
//!   LLVM is allowed to lower it to `ud2` or other opcodes that may cause an illegal instruction exception, and this is the "good end". <br>LLVM 允许将其降低到 `ud2` 或其他可能导致非法指令异常的操作码，这就是好的结局。<br>
//!   The "bad end" is that LLVM may reverse time to the moment control flow diverged on a path towards undefined behavior, and destroy the other branch, potentially deleting safe code and violating Rust's `unsafe` contract. <br>X 是 LLVM 将时间倒转到在一个未定义的行为上发散的那一刻，并且可能会控制 Z 流向其他分支，可能会删除安全代码并违反 RUST0Z 的 `unsafe` 合同<br>
//!
//! Note that according to LLVM, vectors are not arrays, but they are equivalent when stored to and loaded from memory. <br>请注意，根据 LLVM，vectors 不是数组，但它们在存储到内存和从内存加载时是等效的。<br>
//!
//! Unless stated otherwise, all intrinsics for binary operations require SIMD vectors of equal types and lengths. <br>除非另有说明，所有二进制操作的内部函数都需要类型和长度相同的 SIMD vectors。<br>
//!
//!

// These intrinsics aren't linked directly from LLVM and are mostly undocumented, however they are mostly lowered to the matching LLVM instructions by the compiler in a fairly straightforward manner. <br>这些内部函数没有直接从 LLVM 链接，并且大多没有记录，但是它们大多由编译器以相当简单的方式降低为匹配的 LLVM 指令。<br>
//
// The associated LLVM instruction or intrinsic is documented alongside each Rust intrinsic function. <br>相关的 LLVM 指令或内部函数与每个 Rust 内部函数函数一起记录。<br>
extern "platform-intrinsic" {
    /// add/fadd
    pub(crate) fn simd_add<T>(x: T, y: T) -> T;

    /// sub/fsub
    pub(crate) fn simd_sub<T>(lhs: T, rhs: T) -> T;

    /// mul/fmul
    pub(crate) fn simd_mul<T>(x: T, y: T) -> T;

    /// udiv/sdiv/fdiv
    /// ints and uints: {s,u}div incur UB if division by zero occurs. <br>ints and uints: {s,u} div 如果发生被零除，则产生 UB。<br>
    /// ints: sdiv is UB for int::MIN / -1. <br>ints: sdiv 是 int::MIN/-1 的 UB。<br>
    /// floats: fdiv is never UB, but may create NaNs or infinities. <br>floats: fdiv 永远不会是 UB，但可能会创建 NaN 或无穷大。<br>
    pub(crate) fn simd_div<T>(lhs: T, rhs: T) -> T;

    /// urem/srem/frem
    /// ints and uints: {s,u}rem incur UB if division by zero occurs. <br>ints and uints: {s,u} rem 如果发生被零除，则产生 UB。<br>
    /// ints: srem is UB for int::MIN / -1. <br>ints: srem 是 int::MIN/-1 的 UB。<br>
    /// floats: frem is equivalent to libm::fmod in the "default" floating point environment, sans errno. <br>floats: frem 相当于 "default" 浮点环境中的 libm::fmod，sans errno。<br>
    pub(crate) fn simd_rem<T>(lhs: T, rhs: T) -> T;

    /// shl for (u)ints. <br>shl 用于 (u) 整数。<br>
    /// poison if rhs >= lhs::BITS <br>如果 rhs >= lhs::BITS，则会 poison<br>
    pub(crate) fn simd_shl<T>(lhs: T, rhs: T) -> T;

    /// ints: ashr uints: lshr poison if rhs >= lhs::BITS
    ///
    ///
    pub(crate) fn simd_shr<T>(lhs: T, rhs: T) -> T;

    /// and
    pub(crate) fn simd_and<T>(x: T, y: T) -> T;

    /// or
    pub(crate) fn simd_or<T>(x: T, y: T) -> T;

    /// xor
    pub(crate) fn simd_xor<T>(x: T, y: T) -> T;

    /// getelementptr (without inbounds) <br>getelementptr (无入站)<br>
    pub(crate) fn simd_arith_offset<T, U>(ptrs: T, offsets: U) -> T;

    /// fptoui/fptosi/uitofp/sitofp
    /// casting floats to integers is truncating, so it is safe to convert values like e.g. <br>将浮点数转换为整数会被截断，因此可以安全地转换值，例如<br>
    /// 1.5 but the truncated value must fit in the target type or the result is poison. <br>但截断的值必须适合目标类型，否则结果是 poison 的。<br>
    /// use `simd_as` instead for a cast that performs a saturating conversion. <br>使用 `simd_as` 代替执行饱和转换的强制转换。<br>
    pub(crate) fn simd_cast<T, U>(x: T) -> U;
    /// follows Rust's `T as U` semantics, including saturating float casts which amounts to the same as `simd_cast` for many cases <br>遵循 Rust 的 `T as U` 语义，包括在许多情况下与 `simd_cast` 相同的饱和浮点类型转换<br>
    ///
    pub(crate) fn simd_as<T, U>(x: T) -> U;

    /// neg/fneg
    /// ints: ultimately becomes a call to cg_ssa's BuilderMethods::neg. <br>ints: 最终成为 cg_ssa 的 BuilderMethods::neg 的调用。<br> cg_llvm equates this to `simd_sub(Simd::splat(0), x)`. <br>cg_llvm 将其等同于 `simd_sub(Simd::splat(0), x)`。<br>
    /// floats: LLVM's fneg, which changes the floating point sign bit. <br>floats: LLVM 的 fneg，它改变浮点符号位。<br> Some arches have instructions for it. <br>一些 arches 有它的说明。<br>
    /// Rust panics for Neg::neg(int::MIN) due to overflow, but it is not UB in LLVM without `nsw`. <br>由于溢出，Neg::neg (int::MIN) 的 Rust panic，但它不是 LLVM 中没有 `nsw` 的 UB。<br>
    pub(crate) fn simd_neg<T>(x: T) -> T;

    /// fabs
    pub(crate) fn simd_fabs<T>(x: T) -> T;

    // minnum/maxnum
    pub(crate) fn simd_fmin<T>(x: T, y: T) -> T;
    pub(crate) fn simd_fmax<T>(x: T, y: T) -> T;

    // these return Simd<int, N> with the same BITS size as the inputs <br>这些返回 Simd<int, N>，其 BITS 大小与输入相同<br>
    pub(crate) fn simd_eq<T, U>(x: T, y: T) -> U;
    pub(crate) fn simd_ne<T, U>(x: T, y: T) -> U;
    pub(crate) fn simd_lt<T, U>(x: T, y: T) -> U;
    pub(crate) fn simd_le<T, U>(x: T, y: T) -> U;
    pub(crate) fn simd_gt<T, U>(x: T, y: T) -> U;
    pub(crate) fn simd_ge<T, U>(x: T, y: T) -> U;

    // shufflevector idx: LLVM calls it a "shuffle mask vector constant", a vector of i32s <br>shufflevector idx: LLVM 称之为 "shuffle mask vector constant"，i32s 的 vector<br>
    //
    pub(crate) fn simd_shuffle<T, U, V>(x: T, y: T, idx: U) -> V;

    /// llvm.masked.gather like a loop of pointer reads val: vector of values to select if a lane is masked ptr: vector of pointers to read from mask: a "wide" mask of integers, selects as if simd_select(mask, read(ptr), val) note, the LLVM intrinsic accepts a mask vector of <N x i1> <br>llvm.masked.gather 就像一个指针循环读取 val: vector 的值来选择是否 lane 被屏蔽 ptr: vector 的指针从掩码读取: 整数的 "wide" 掩码，选择就像 simd_select(mask, read(ptr), val) 注意，LLVM 内部函数接受掩码 vector<N x i1><br>
    ///
    /// FIXME: review this if/when we fix up our mask story in general? <br>如果或当我们完成 mask story 的时候，回顾一下这个？<br>
    ///
    ///
    ///
    ///
    pub(crate) fn simd_gather<T, U, V>(val: T, ptr: U, mask: V) -> T;
    /// llvm.masked.scatter like gather, but more spicy, as it writes instead of reads <br>llvm.masked.scatter 类似于 gather，但更具刺激性，因为它写入而不是读取<br>
    ///
    pub(crate) fn simd_scatter<T, U, V>(val: T, ptr: U, mask: V);

    // {s,u}add.sat
    pub(crate) fn simd_saturating_add<T>(x: T, y: T) -> T;

    // {s,u}sub.sat
    pub(crate) fn simd_saturating_sub<T>(lhs: T, rhs: T) -> T;

    // reductions llvm.vector.reduce.{add,fadd}
    //
    pub(crate) fn simd_reduce_add_ordered<T, U>(x: T, y: U) -> U;
    // llvm.vector.reduce.{mul,fmul}
    pub(crate) fn simd_reduce_mul_ordered<T, U>(x: T, y: U) -> U;
    #[allow(unused)]
    pub(crate) fn simd_reduce_all<T>(x: T) -> bool;
    #[allow(unused)]
    pub(crate) fn simd_reduce_any<T>(x: T) -> bool;
    pub(crate) fn simd_reduce_max<T, U>(x: T) -> U;
    pub(crate) fn simd_reduce_min<T, U>(x: T) -> U;
    pub(crate) fn simd_reduce_and<T, U>(x: T) -> U;
    pub(crate) fn simd_reduce_or<T, U>(x: T) -> U;
    pub(crate) fn simd_reduce_xor<T, U>(x: T) -> U;

    // truncate integer vector to bitmask `fn simd_bitmask(vector) -> unsigned integer` takes a vector of integers and returns either an unsigned integer or array of `u8`. <br>将整数 vector 截断为位掩码 `fn simd_bitmask(vector) -> unsigned integer` 采用整数 vector 并返回无符号整数或 `u8` 数组。<br>
    //
    // Every element in the vector becomes a single bit in the returned bitmask. <br>vector 中的每个元素都成为返回的位掩码中的一个位。<br>
    // If the vector has less than 8 lanes, a u8 is returned with zeroed trailing bits. <br>如果 vector 的 lanes 少于 8 个，则返回一个 u8，其尾随位为零。<br>
    // The bit order of the result depends on the byte endianness. <br>结果的位顺序取决于字节字节序。<br> LSB-first for little endian and MSB-first for big endian. <br>LSB 优先用于小端，MSB 优先用于大端。<br>
    //
    // UB if called on a vector with values other than 0 and -1. <br>UB 如果在具有除 0 和 -1 以外的值的 vector 上调用。<br>
    //
    //
    #[allow(unused)]
    pub(crate) fn simd_bitmask<T, U>(x: T) -> U;

    // select first argument is a vector of integers, -1 (all bits 1) is "true" logically equivalent to (yes & m) | (no & (m^-1), but you can use it on floats. <br>选择第一个参数是整数的 vector，-1 (所有位 1) 是 "true" 逻辑上等价于 (yes&m) | (没有 & (m^-1)，但您可以在花车上使用它。<br>
    //
    //
    //
    pub(crate) fn simd_select<M, T>(m: M, yes: T, no: T) -> T;
    #[allow(unused)]
    pub(crate) fn simd_select_bitmask<M, T>(m: M, yes: T, no: T) -> T;
}
