//! 与 vectors 的掩码 lanes 相关的类型和 traits。
//! 代表的类型
#![allow(non_camel_case_types)]

#[cfg_attr(
    not(all(target_arch = "x86_64", target_feature = "avx512f")),
    path = "masks/full_masks.rs"
)]
#[cfg_attr(
    all(target_arch = "x86_64", target_feature = "avx512f"),
    path = "masks/bitmask.rs"
)]
mod mask_impl;

mod to_bitmask;
pub use to_bitmask::ToBitMask;

use crate::simd::{intrinsics, LaneCount, Simd, SimdElement, SupportedLaneCount};
use core::cmp::Ordering;
use core::{fmt, mem};

mod sealed {
    use super::*;

    /// 这不仅封装了 `MaskElement` trait，而且这些函数还防止了其他 trait 进入父边界。
    ///
    /// 例如，可以通过要求 `MaskElement: PartialEq` 来提供 `eq`，但这将阻止我们永远删除该限制，或者阻止我们在将来在非 `PartialEq` 类型上实现 `MaskElement`。
    ///
    ///
    ///
    pub trait Sealed {
        fn valid<const LANES: usize>(values: Simd<Self, LANES>) -> bool
        where
            LaneCount<LANES>: SupportedLaneCount,
            Self: SimdElement;

        fn eq(self, other: Self) -> bool;

        const TRUE: Self;

        const FALSE: Self;
    }
}
use sealed::Sealed;

/// 可用作 SIMD 掩码元素的类型的标记 trait。
///
/// # Safety
/// 类型必须是有符号整数。
pub unsafe trait MaskElement: SimdElement + Sealed {}

macro_rules! impl_element {
    { $ty:ty } => {
        impl Sealed for $ty {
            fn valid<const LANES: usize>(value: Simd<Self, LANES>) -> bool
            where
                LaneCount<LANES>: SupportedLaneCount,
            {
                (value.lanes_eq(Simd::splat(0)) | value.lanes_eq(Simd::splat(-1))).all()
            }

            fn eq(self, other: Self) -> bool { self == other }

            const TRUE: Self = -1;
            const FALSE: Self = 0;
        }

        unsafe impl MaskElement for $ty {}
    }
}

impl_element! { i8 }
impl_element! { i16 }
impl_element! { i32 }
impl_element! { i64 }
impl_element! { isize }

/// `Element` 指定宽度的 `LANES` 元素的 SIMD vector 掩码。
///
/// 这种类型的布局是未指定的。
#[repr(transparent)]
pub struct Mask<T, const LANES: usize>(mask_impl::Mask<T, LANES>)
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount;

impl<T, const LANES: usize> Copy for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
}

impl<T, const LANES: usize> Clone for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    fn clone(&self) -> Self {
        *self
    }
}

impl<T, const LANES: usize> Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    /// 通过将所有 lanes 设置为给定值来构建掩码。
    pub fn splat(value: bool) -> Self {
        Self(mask_impl::Mask::splat(value))
    }

    /// 将布尔数组转换为 SIMD 掩码。
    pub fn from_array(array: [bool; LANES]) -> Self {
        // SAFETY: Rust 的 bool 具有 1 字节 (u8) 的布局，值为 true: 0b_0000_0001 false: 0b_0000_0000 因此，布尔数组也是有效的字节数组: [u8; N] 这作为 "in-place" 转换假设是有效的，但这些是 "dependently-sized" 类型，所以复制省略它是!
        //
        //
        //
        //
        //
        unsafe {
            let bytes: [u8; LANES] = mem::transmute_copy(&array);
            let bools: Simd<i8, LANES> =
                intrinsics::simd_ne(Simd::from_array(bytes), Simd::splat(0u8));
            Mask::from_int_unchecked(intrinsics::simd_cast(bools))
        }
    }

    /// 将 SIMD 掩码转换为布尔数组。
    pub fn to_array(self) -> [bool; LANES] {
        // 这遵循与 from_array 大体相同的逻辑。
        // SAFETY: Rust 的 bool 具有 1 字节 (u8) 的布局，值为 true: 0b_0000_0001 false: 0b_0000_0000 因此，布尔数组也是有效的字节数组: [u8; N] 由于我们的掩码等于设置所有位的整数，我们可以简单地将它们转换为 i8s，然后通过 Rust 的 "true" bool 的位模式对它们进行位和位。
        //
        // 这作为 "in-place" 变形假设是有效的，但这些是 "dependently-sized" 类型，所以它是复制省略!
        //
        //
        //
        //
        //
        //
        unsafe {
            let mut bytes: Simd<i8, LANES> = intrinsics::simd_cast(self.to_int());
            bytes &= Simd::splat(1i8);
            mem::transmute_copy(&bytes)
        }
    }

    /// 将整数 vector 转换为掩码，其中 0 代表 `false`，-1 代表 `true`。
    ///
    ///
    /// # Safety
    /// 所有 lanes 必须为 0 或 -1。
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    pub unsafe fn from_int_unchecked(value: Simd<T, LANES>) -> Self {
        // 安全性: 调用者必须确认这个不，变体
        unsafe { Self(mask_impl::Mask::from_int_unchecked(value)) }
    }

    /// 将整数 vector 转换为掩码，其中 0 代表 `false`，-1 代表 `true`。
    ///
    ///
    /// # Panics
    /// 如果任何 lane 不是 0 或 -1，就会出现 panics。
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    pub fn from_int(value: Simd<T, LANES>) -> Self {
        assert!(T::valid(value), "all values must be either 0 or -1",);
        // 安全性: 有效性已经过检查
        unsafe { Self::from_int_unchecked(value) }
    }

    /// 将掩码转换为整数 vector，其中 0 代表 `false`，-1 代表 `true`。
    ///
    #[inline]
    #[must_use = "method returns a new vector and does not mutate the original value"]
    pub fn to_int(self) -> Simd<T, LANES> {
        self.0.to_int()
    }

    /// 测试指定 lane 的值。
    ///
    /// # Safety
    /// `lane` 必须小于 `LANES`。
    #[inline]
    #[must_use = "method returns a new bool and does not mutate the original value"]
    pub unsafe fn test_unchecked(&self, lane: usize) -> bool {
        // 安全性: 调用者必须确认这个不，变体
        unsafe { self.0.test_unchecked(lane) }
    }

    /// 测试指定 lane 的值。
    ///
    /// # Panics
    /// 如果 `lane` 大于或等于 vector 中的 lane 数，则出现 panics。
    #[inline]
    #[must_use = "method returns a new bool and does not mutate the original value"]
    pub fn test(&self, lane: usize) -> bool {
        assert!(lane < LANES, "lane index out of range");
        // 安全性: 已检查 lane 指数
        unsafe { self.test_unchecked(lane) }
    }

    /// 设置指定 lane 的值。
    ///
    /// # Safety
    /// `lane` 必须小于 `LANES`。
    #[inline]
    pub unsafe fn set_unchecked(&mut self, lane: usize, value: bool) {
        // 安全性: 调用者必须确认这个不，变体
        unsafe {
            self.0.set_unchecked(lane, value);
        }
    }

    /// 设置指定 lane 的值。
    ///
    /// # Panics
    /// 如果 `lane` 大于或等于 vector 中的 lane 数，则出现 panics。
    #[inline]
    pub fn set(&mut self, lane: usize, value: bool) {
        assert!(lane < LANES, "lane index out of range");
        // 安全性: 已检查 lane 指数
        unsafe {
            self.set_unchecked(lane, value);
        }
    }

    /// 如果设置了任何 lane，则返回 true，否则返回 false。
    #[inline]
    #[must_use = "method returns a new bool and does not mutate the original value"]
    pub fn any(self) -> bool {
        self.0.any()
    }

    /// 如果设置了所有 lanes，则返回 true，否则返回 false。
    #[inline]
    #[must_use = "method returns a new bool and does not mutate the original value"]
    pub fn all(self) -> bool {
        self.0.all()
    }
}

// vector 和 array 的转换
impl<T, const LANES: usize> From<[bool; LANES]> for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    fn from(array: [bool; LANES]) -> Self {
        Self::from_array(array)
    }
}

impl<T, const LANES: usize> From<Mask<T, LANES>> for [bool; LANES]
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    fn from(vector: Mask<T, LANES>) -> Self {
        vector.to_array()
    }
}

impl<T, const LANES: usize> Default for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    #[inline]
    #[must_use = "method returns a defaulted mask with all lanes set to false (0)"]
    fn default() -> Self {
        Self::splat(false)
    }
}

impl<T, const LANES: usize> PartialEq for Mask<T, LANES>
where
    T: MaskElement + PartialEq,
    LaneCount<LANES>: SupportedLaneCount,
{
    #[inline]
    #[must_use = "method returns a new bool and does not mutate the original value"]
    fn eq(&self, other: &Self) -> bool {
        self.0 == other.0
    }
}

impl<T, const LANES: usize> PartialOrd for Mask<T, LANES>
where
    T: MaskElement + PartialOrd,
    LaneCount<LANES>: SupportedLaneCount,
{
    #[inline]
    #[must_use = "method returns a new Ordering and does not mutate the original value"]
    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
        self.0.partial_cmp(&other.0)
    }
}

impl<T, const LANES: usize> fmt::Debug for Mask<T, LANES>
where
    T: MaskElement + fmt::Debug,
    LaneCount<LANES>: SupportedLaneCount,
{
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_list()
            .entries((0..LANES).map(|lane| self.test(lane)))
            .finish()
    }
}

impl<T, const LANES: usize> core::ops::BitAnd for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Self;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn bitand(self, rhs: Self) -> Self {
        Self(self.0 & rhs.0)
    }
}

impl<T, const LANES: usize> core::ops::BitAnd<bool> for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Self;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn bitand(self, rhs: bool) -> Self {
        self & Self::splat(rhs)
    }
}

impl<T, const LANES: usize> core::ops::BitAnd<Mask<T, LANES>> for bool
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Mask<T, LANES>;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn bitand(self, rhs: Mask<T, LANES>) -> Mask<T, LANES> {
        Mask::splat(self) & rhs
    }
}

impl<T, const LANES: usize> core::ops::BitOr for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Self;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn bitor(self, rhs: Self) -> Self {
        Self(self.0 | rhs.0)
    }
}

impl<T, const LANES: usize> core::ops::BitOr<bool> for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Self;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn bitor(self, rhs: bool) -> Self {
        self | Self::splat(rhs)
    }
}

impl<T, const LANES: usize> core::ops::BitOr<Mask<T, LANES>> for bool
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Mask<T, LANES>;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn bitor(self, rhs: Mask<T, LANES>) -> Mask<T, LANES> {
        Mask::splat(self) | rhs
    }
}

impl<T, const LANES: usize> core::ops::BitXor for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Self;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn bitxor(self, rhs: Self) -> Self::Output {
        Self(self.0 ^ rhs.0)
    }
}

impl<T, const LANES: usize> core::ops::BitXor<bool> for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Self;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn bitxor(self, rhs: bool) -> Self::Output {
        self ^ Self::splat(rhs)
    }
}

impl<T, const LANES: usize> core::ops::BitXor<Mask<T, LANES>> for bool
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Mask<T, LANES>;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn bitxor(self, rhs: Mask<T, LANES>) -> Self::Output {
        Mask::splat(self) ^ rhs
    }
}

impl<T, const LANES: usize> core::ops::Not for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    type Output = Mask<T, LANES>;
    #[inline]
    #[must_use = "method returns a new mask and does not mutate the original value"]
    fn not(self) -> Self::Output {
        Self(!self.0)
    }
}

impl<T, const LANES: usize> core::ops::BitAndAssign for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    #[inline]
    fn bitand_assign(&mut self, rhs: Self) {
        self.0 = self.0 & rhs.0;
    }
}

impl<T, const LANES: usize> core::ops::BitAndAssign<bool> for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    #[inline]
    fn bitand_assign(&mut self, rhs: bool) {
        *self &= Self::splat(rhs);
    }
}

impl<T, const LANES: usize> core::ops::BitOrAssign for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    #[inline]
    fn bitor_assign(&mut self, rhs: Self) {
        self.0 = self.0 | rhs.0;
    }
}

impl<T, const LANES: usize> core::ops::BitOrAssign<bool> for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    #[inline]
    fn bitor_assign(&mut self, rhs: bool) {
        *self |= Self::splat(rhs);
    }
}

impl<T, const LANES: usize> core::ops::BitXorAssign for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    #[inline]
    fn bitxor_assign(&mut self, rhs: Self) {
        self.0 = self.0 ^ rhs.0;
    }
}

impl<T, const LANES: usize> core::ops::BitXorAssign<bool> for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    #[inline]
    fn bitxor_assign(&mut self, rhs: bool) {
        *self ^= Self::splat(rhs);
    }
}

/// 八个 8 位掩码的 Vector
pub type mask8x8 = Mask<i8, 8>;

/// 16 个 8 位掩码的 Vector
pub type mask8x16 = Mask<i8, 16>;

/// 32 个 8 位掩码的 Vector
pub type mask8x32 = Mask<i8, 32>;

/// 16 个 8 位掩码的 Vector
pub type mask8x64 = Mask<i8, 64>;

/// 四个 16 位掩码的 Vector
pub type mask16x4 = Mask<i16, 4>;

/// 八个 16 位掩码的 Vector
pub type mask16x8 = Mask<i16, 8>;

/// 16 个 16 位掩码的 Vector
pub type mask16x16 = Mask<i16, 16>;

/// 32 个 16 位掩码的 Vector
pub type mask16x32 = Mask<i16, 32>;

/// 两个 32 位掩码的 Vector
pub type mask32x2 = Mask<i32, 2>;

/// 四个 32 位掩码的 Vector
pub type mask32x4 = Mask<i32, 4>;

/// 八个 32 位掩码的 Vector
pub type mask32x8 = Mask<i32, 8>;

/// 16 个 32 位掩码的 Vector
pub type mask32x16 = Mask<i32, 16>;

/// 两个 64 位掩码的 Vector
pub type mask64x2 = Mask<i64, 2>;

/// 四个 64 位掩码的 Vector
pub type mask64x4 = Mask<i64, 4>;

/// 八个 64 位掩码的 Vector
pub type mask64x8 = Mask<i64, 8>;

/// 两个指针宽度掩码的 Vector
pub type masksizex2 = Mask<isize, 2>;

/// 四个指针宽度掩码的 Vector
pub type masksizex4 = Mask<isize, 4>;

/// 八个指针宽度掩码的 Vector
pub type masksizex8 = Mask<isize, 8>;

macro_rules! impl_from {
    { $from:ty  => $($to:ty),* } => {
        $(
        impl<const LANES: usize> From<Mask<$from, LANES>> for Mask<$to, LANES>
        where
            LaneCount<LANES>: SupportedLaneCount,
        {
            fn from(value: Mask<$from, LANES>) -> Self {
                Self(value.0.convert())
            }
        }
        )*
    }
}
impl_from! { i8 => i16, i32, i64, isize }
impl_from! { i16 => i32, i64, isize, i8 }
impl_from! { i32 => i64, isize, i8, i16 }
impl_from! { i64 => isize, i8, i16, i32 }
impl_from! { isize => i8, i16, i32, i64 }
