use super::{mask_impl, Mask, MaskElement};
use crate::simd::{LaneCount, SupportedLaneCount};

mod sealed {
    pub trait Sealed {}
}
pub use sealed::Sealed;

impl<T, const LANES: usize> Sealed for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
}

/// 将掩码转换为整数位掩码和从整数位掩码转换。
///
/// 位掩码的每一位对应一个掩码 lane，从 LSB 开始。
///
/// # Safety
/// 这个 trait 是 `unsafe` 并且是密封的，因为 `BitMask` 类型必须与掩码中 lanes 的数量相匹配。
///
pub unsafe trait ToBitMask: Sealed {
    /// 整数位掩码类型。
    type BitMask;

    /// 将掩码转换为位掩码。
    fn to_bitmask(self) -> Self::BitMask;

    /// 将位掩码转换为掩码。
    fn from_bitmask(bitmask: Self::BitMask) -> Self;
}

macro_rules! impl_integer_intrinsic {
    { $(unsafe impl ToBitMask<BitMask=$int:ty> for Mask<_, $lanes:literal>)* } => {
        $(
        unsafe impl<T: MaskElement> ToBitMask for Mask<T, $lanes> {
            type BitMask = $int;

            fn to_bitmask(self) -> $int {
                self.0.to_bitmask_integer()
            }

            fn from_bitmask(bitmask: $int) -> Self {
                Self(mask_impl::Mask::from_bitmask_integer(bitmask))
            }
        }
        )*
    }
}

impl_integer_intrinsic! {
    unsafe impl ToBitMask<BitMask=u8> for Mask<_, 1>
    unsafe impl ToBitMask<BitMask=u8> for Mask<_, 2>
    unsafe impl ToBitMask<BitMask=u8> for Mask<_, 4>
    unsafe impl ToBitMask<BitMask=u8> for Mask<_, 8>
    unsafe impl ToBitMask<BitMask=u16> for Mask<_, 16>
    unsafe impl ToBitMask<BitMask=u32> for Mask<_, 32>
    unsafe impl ToBitMask<BitMask=u64> for Mask<_, 64>
}
