use super::{mask_impl, Mask, MaskElement};
use crate::simd::{LaneCount, SupportedLaneCount};

mod sealed {
    pub trait Sealed {}
}
pub use sealed::Sealed;

impl<T, const LANES: usize> Sealed for Mask<T, LANES>
where
    T: MaskElement,
    LaneCount<LANES>: SupportedLaneCount,
{
}

/// Converts masks to and from integer bitmasks. <br>将掩码转换为整数位掩码和从整数位掩码转换。<br>
///
/// Each bit of the bitmask corresponds to a mask lane, starting with the LSB. <br>位掩码的每一位对应一个掩码 lane，从 LSB 开始。<br>
///
/// # Safety
/// This trait is `unsafe` and sealed, since the `BitMask` type must match the number of lanes in the mask. <br>这个 trait 是 `unsafe` 并且是密封的，因为 `BitMask` 类型必须与掩码中 lanes 的数量相匹配。<br>
///
pub unsafe trait ToBitMask: Sealed {
    /// The integer bitmask type. <br>整数位掩码类型。<br>
    type BitMask;

    /// Converts a mask to a bitmask. <br>将掩码转换为位掩码。<br>
    fn to_bitmask(self) -> Self::BitMask;

    /// Converts a bitmask to a mask. <br>将位掩码转换为掩码。<br>
    fn from_bitmask(bitmask: Self::BitMask) -> Self;
}

macro_rules! impl_integer_intrinsic {
    { $(unsafe impl ToBitMask<BitMask=$int:ty> for Mask<_, $lanes:literal>)* } => {
        $(
        unsafe impl<T: MaskElement> ToBitMask for Mask<T, $lanes> {
            type BitMask = $int;

            fn to_bitmask(self) -> $int {
                self.0.to_bitmask_integer()
            }

            fn from_bitmask(bitmask: $int) -> Self {
                Self(mask_impl::Mask::from_bitmask_integer(bitmask))
            }
        }
        )*
    }
}

impl_integer_intrinsic! {
    unsafe impl ToBitMask<BitMask=u8> for Mask<_, 1>
    unsafe impl ToBitMask<BitMask=u8> for Mask<_, 2>
    unsafe impl ToBitMask<BitMask=u8> for Mask<_, 4>
    unsafe impl ToBitMask<BitMask=u8> for Mask<_, 8>
    unsafe impl ToBitMask<BitMask=u16> for Mask<_, 16>
    unsafe impl ToBitMask<BitMask=u32> for Mask<_, 32>
    unsafe impl ToBitMask<BitMask=u64> for Mask<_, 64>
}
