use crate::simd::{LaneCount, Simd, SimdElement, SupportedLaneCount};
use core::ops::{Add, Mul};
use core::ops::{BitAnd, BitOr, BitXor};
use core::ops::{Div, Rem, Sub};
use core::ops::{Shl, Shr};

mod assign;
mod deref;
mod unary;

impl<I, T, const LANES: usize> core::ops::Index<I> for Simd<T, LANES>
where
    T: SimdElement,
    LaneCount<LANES>: SupportedLaneCount,
    I: core::slice::SliceIndex<[T]>,
{
    type Output = I::Output;
    fn index(&self, index: I) -> &Self::Output {
        &self.as_array()[index]
    }
}

impl<I, T, const LANES: usize> core::ops::IndexMut<I> for Simd<T, LANES>
where
    T: SimdElement,
    LaneCount<LANES>: SupportedLaneCount,
    I: core::slice::SliceIndex<[T]>,
{
    fn index_mut(&mut self, index: I) -> &mut Self::Output {
        &mut self.as_mut_array()[index]
    }
}

macro_rules! unsafe_base {
    ($lhs:ident, $rhs:ident, {$simd_call:ident}, $($_:tt)*) => {
        unsafe { $crate::simd::intrinsics::$simd_call($lhs, $rhs) }
    };
}

/// SAFETY: This macro should not be used for anything except Shl or Shr, and passed the appropriate shift intrinsic. <br>这个宏不应该用于除 Shl 或 Shr 之外的任何东西，并且通过了适当的 shift 内部函数。<br>
/// It handles performing a bitand in addition to calling the shift operator, so that the result is well-defined: LLVM can return a poison value if you shl, lshr, or ashr if rhs >= <Int>::BITS At worst, this will maybe add another instruction and cycle, at best, it may open up more optimization opportunities, or simply be elided entirely, especially for SIMD ISAs which default to this. <br>除了调用 shift 运算符之外，它还处理执行 bitand，因此结果是明确定义的：如果 shl、lshr 或 ashr 如果 rhs >=，LLVM 可以返回毒值 <Int>::BITS 在最坏的情况下，这可能会添加另一个指令和循环，最好的情况是，它可能会打开更多的优化机会，或者干脆被完全省略，特别是对于默认使用此功能的 SIMD ISA。<br>
///
///
///
///
///
// FIXME: Consider implementing this in cg_llvm instead? <br>考虑在 cg_llvm 中实现这个？<br>
// cg_clif defaults to this, and scalar MIR shifts also default to wrapping <br>cg_clif 默认为这个，标量 MIR 移位也默认为 wrapping<br>
macro_rules! wrap_bitshift {
    ($lhs:ident, $rhs:ident, {$simd_call:ident}, $int:ident) => {
        unsafe {
            $crate::simd::intrinsics::$simd_call(
                $lhs,
                $rhs.bitand(Simd::splat(<$int>::BITS as $int - 1)),
            )
        }
    };
}

/// SAFETY: This macro must only be used to impl Div or Rem and given the matching intrinsic. <br>这个宏只能用于实现 Div 或 Rem 并给出匹配的内部函数。<br>
/// It guards against LLVM's UB conditions for integer div or rem using masks and selects, thus guaranteeing a Rust value returns instead. <br>它使用掩码和选择来防止 LLVM 的整数 div 或 rem 的 UB 条件，从而保证返回 Rust 值。<br>
///
///
/// |                  | LLVM | Rust
/// | :--------------: | :--- | :----------
/// | N {/,%} 0        | UB   | panic!()
/// | <$int>::MIN / -1 | UB   | <$int>::MIN
/// | <$int>::MIN % -1 | UB   | 0
///
macro_rules! int_divrem_guard {
    (   $lhs:ident,
        $rhs:ident,
        {   const PANIC_ZERO: &'static str = $zero:literal;
            $simd_call:ident
        },
        $int:ident ) => {
        if $rhs.lanes_eq(Simd::splat(0)).any() {
            panic!($zero);
        } else {
            // Prevent otherwise-UB overflow on the MIN / -1 case. <br>防止 MIN/-1 情况下的其他 UB 溢出。<br>
            let rhs = if <$int>::MIN != 0 {
                // This should, at worst, optimize to a few branchless logical ops Ideally, this entire conditional should evaporate Fire LLVM and implement those manually if it doesn't get the hint <br>在最坏的情况下，这应该优化到一些无分支的逻辑操作理想情况下，整个条件应该蒸发 Fire LLVM 并在没有得到提示时手动实现它们<br>
                //
                //
                ($lhs.lanes_eq(Simd::splat(<$int>::MIN))
                // type inference can break here, so cut an SInt to size <br>类型推断可能会在此处中断，因此将 SInt 裁剪为合适的大小<br>
                & $rhs.lanes_eq(Simd::splat(-1i64 as _)))
                .select(Simd::splat(1), $rhs)
            } else {
                // Nice base case to make it easy to const-fold away the other branch. <br>很好的终止条件，可以很容易地 const-fold 掉另一个分支。<br>
                $rhs
            };
            unsafe { $crate::simd::intrinsics::$simd_call($lhs, rhs) }
        }
    };
}

macro_rules! for_base_types {
    (   T = ($($scalar:ident),*);
        type Lhs = Simd<T, N>;
        type Rhs = Simd<T, N>;
        type Output = $out:ty;

        impl $op:ident::$call:ident {
            $macro_impl:ident $inner:tt
        }) => {
            $(
                impl<const N: usize> $op<Self> for Simd<$scalar, N>
                where
                    $scalar: SimdElement,
                    LaneCount<N>: SupportedLaneCount,
                {
                    type Output = $out;

                    #[inline]
                    #[must_use = "operator returns a new vector without mutating the inputs"]
                    fn $call(self, rhs: Self) -> Self::Output {
                        $macro_impl!(self, rhs, $inner, $scalar)
                    }
                })*
    }
}

// A "TokenTree muncher": takes a set of scalar types `T = {};` type parameters for the ops it implements, `Op::fn` names, and a macro that expands into an expr, substituting in an intrinsic. <br>"TokenTree muncher": 为其实现的操作采用一组标量类型 `T = {};` 类型参数、`Op::fn` 名称和扩展为 expr 的宏，替换为内部函数。<br>
//
// It passes that to for_base_types, which expands an impl for the types, using the expanded expr in the function, and recurses with itself. <br>它将它传递给 for_base_types，后者使用函数中的扩展 expr 扩展类型的 impl，并与自身递归。<br>
//
// tl;dr impls a set of ops::{Traits} for a set of types <br>tl;dr 为一组类型实现了一组 ops::{Traits}<br>
//
//
macro_rules! for_base_ops {
    (
        T = $types:tt;
        type Lhs = Simd<T, N>;
        type Rhs = Simd<T, N>;
        type Output = $out:ident;
        impl $op:ident::$call:ident
            $inner:tt
        $($rest:tt)*
    ) => {
        for_base_types! {
            T = $types;
            type Lhs = Simd<T, N>;
            type Rhs = Simd<T, N>;
            type Output = $out;
            impl $op::$call
                $inner
        }
        for_base_ops! {
            T = $types;
            type Lhs = Simd<T, N>;
            type Rhs = Simd<T, N>;
            type Output = $out;
            $($rest)*
        }
    };
    ($($done:tt)*) => {
        // Done.
    }
}

// Integers can always accept add, mul, sub, bitand, bitor, and bitxor. <br>整数总是可以接受 add、mul、sub、bitand、bitor 和 bitxor。<br>
// For all of these operations, simd_* intrinsics apply wrapping logic. <br>对于所有这些操作，simd_* 内部函数应用了包装逻辑。<br>
for_base_ops! {
    T = (i8, i16, i32, i64, isize, u8, u16, u32, u64, usize);
    type Lhs = Simd<T, N>;
    type Rhs = Simd<T, N>;
    type Output = Self;

    impl Add::add {
        unsafe_base { simd_add }
    }

    impl Mul::mul {
        unsafe_base { simd_mul }
    }

    impl Sub::sub {
        unsafe_base { simd_sub }
    }

    impl BitAnd::bitand {
        unsafe_base { simd_and }
    }

    impl BitOr::bitor {
        unsafe_base { simd_or }
    }

    impl BitXor::bitxor {
        unsafe_base { simd_xor }
    }

    impl Div::div {
        int_divrem_guard {
            const PANIC_ZERO: &'static str = "attempt to divide by zero";
            simd_div
        }
    }

    impl Rem::rem {
        int_divrem_guard {
            const PANIC_ZERO: &'static str = "attempt to calculate the remainder with a divisor of zero";
            simd_rem
        }
    }

    // The only question is how to handle shifts >= <Int>::BITS? <br>唯一的问题是如何处理 shifts >= <Int>::BITS？<br>
    // Our current solution uses wrapping logic. <br>我们当前的解决方案中使用了包装逻辑。<br>
    impl Shl::shl {
        wrap_bitshift { simd_shl }
    }

    impl Shr::shr {
        wrap_bitshift {
            // This automatically monomorphizes to lshr or ashr, depending, so it's fine to use it for both UInts and SInts. <br>根据情况，它会自动单态化为 lshr 或 ashr，因此对 UInts 和 SInts 都使用它也没问题。<br>
            //
            simd_shr
        }
    }
}

// We don't need any special precautions here: <br>我们在这里不需要任何特殊的预防措施:<br>
// Floats always accept arithmetic ops, but may become NaN. <br>浮点数总是接受算术运算，但可能会变成 NaN。<br>
for_base_ops! {
    T = (f32, f64);
    type Lhs = Simd<T, N>;
    type Rhs = Simd<T, N>;
    type Output = Self;

    impl Add::add {
        unsafe_base { simd_add }
    }

    impl Mul::mul {
        unsafe_base { simd_mul }
    }

    impl Sub::sub {
        unsafe_base { simd_sub }
    }

    impl Div::div {
        unsafe_base { simd_div }
    }

    impl Rem::rem {
        unsafe_base { simd_rem }
    }
}
