use crate::simd::intrinsics::{
    simd_reduce_add_ordered, simd_reduce_and, simd_reduce_max, simd_reduce_min,
    simd_reduce_mul_ordered, simd_reduce_or, simd_reduce_xor,
};
use crate::simd::{LaneCount, Simd, SimdElement, SupportedLaneCount};
use core::ops::{BitAnd, BitOr, BitXor};

macro_rules! impl_integer_reductions {
    { $scalar:ty } => {
        impl<const LANES: usize> Simd<$scalar, LANES>
        where
            LaneCount<LANES>: SupportedLaneCount,
        {
            /// 减少包装添加。返回 vector 的 lane 总和，带包装加法。
            #[inline]
            pub fn reduce_sum(self) -> $scalar {
                // 安全性: `self` 是整数 vector
                unsafe { simd_reduce_add_ordered(self, 0) }
            }

            /// 减少包装乘数。返回 vector 的 lane 的乘积，带包装乘法。
            #[inline]
            pub fn reduce_product(self) -> $scalar {
                // 安全性: `self` 是整数 vector
                unsafe { simd_reduce_mul_ordered(self, 1) }
            }

            /// 减少最大值。返回 vector 中的最大 lane。
            #[inline]
            pub fn reduce_max(self) -> $scalar {
                // 安全性: `self` 是整数 vector
                unsafe { simd_reduce_max(self) }
            }

            /// 减少最小值。返回 vector 中的最小 lane。
            #[inline]
            pub fn reduce_min(self) -> $scalar {
                // 安全性: `self` 是整数 vector
                unsafe { simd_reduce_min(self) }
            }
        }
    }
}

impl_integer_reductions! { i8 }
impl_integer_reductions! { i16 }
impl_integer_reductions! { i32 }
impl_integer_reductions! { i64 }
impl_integer_reductions! { isize }
impl_integer_reductions! { u8 }
impl_integer_reductions! { u16 }
impl_integer_reductions! { u32 }
impl_integer_reductions! { u64 }
impl_integer_reductions! { usize }

macro_rules! impl_float_reductions {
    { $scalar:ty } => {
        impl<const LANES: usize> Simd<$scalar, LANES>
        where
            LaneCount<LANES>: SupportedLaneCount,
        {

            /// 减少加。返回 vector 的 lane 的总和。
            #[inline]
            pub fn reduce_sum(self) -> $scalar {
                // i586 上的 LLVM 总和不准确
                if cfg!(all(target_arch = "x86", not(target_feature = "sse2"))) {
                    self.as_array().iter().sum()
                } else {
                    // 安全性: `self` 是一个浮点数 vector
                    unsafe { simd_reduce_add_ordered(self, 0.) }
                }
            }

            /// 减少乘。返回 vector 的 lanes 的乘积。
            #[inline]
            pub fn reduce_product(self) -> $scalar {
                // LLVM 产品在 i586 上不准确
                if cfg!(all(target_arch = "x86", not(target_feature = "sse2"))) {
                    self.as_array().iter().product()
                } else {
                    // 安全性: `self` 是一个浮点数 vector
                    unsafe { simd_reduce_mul_ordered(self, 1.) }
                }
            }

            /// 减少最大值。返回 vector 中的最大 lane。
            ///
            /// 返回基于相等的值，因此包含 `0.` 和 `-0.` 的 vector 可以返回任意一个值。
            /// 这个函数不会返回 `NaN`，除非所有的 lane 都是 `NaN`。
            #[inline]
            pub fn reduce_max(self) -> $scalar {
                // 安全性: `self` 是一个浮点数 vector
                unsafe { simd_reduce_max(self) }
            }

            /// 减少最小值。返回 vector 中的最小 lane。
            ///
            /// 返回基于相等的值，因此包含 `0.` 和 `-0.` 的 vector 可以返回任意一个值。
            /// 这个函数不会返回 `NaN`，除非所有的 lane 都是 `NaN`。
            #[inline]
            pub fn reduce_min(self) -> $scalar {
                // 安全性: `self` 是一个浮点数 vector
                unsafe { simd_reduce_min(self) }
            }
        }
    }
}

impl_float_reductions! { f32 }
impl_float_reductions! { f64 }

impl<T, const LANES: usize> Simd<T, LANES>
where
    Self: BitAnd<Self, Output = Self>,
    T: SimdElement + BitAnd<T, Output = T>,
    LaneCount<LANES>: SupportedLaneCount,
{
    /// 减少按位与。
    /// 返回跨 vector lane 的累积按位与。
    #[inline]
    pub fn reduce_and(self) -> T {
        unsafe { simd_reduce_and(self) }
    }
}

impl<T, const LANES: usize> Simd<T, LANES>
where
    Self: BitOr<Self, Output = Self>,
    T: SimdElement + BitOr<T, Output = T>,
    LaneCount<LANES>: SupportedLaneCount,
{
    /// 减少按位或。
    /// 返回跨 vector lane 的累积按位或。
    #[inline]
    pub fn reduce_or(self) -> T {
        unsafe { simd_reduce_or(self) }
    }
}

impl<T, const LANES: usize> Simd<T, LANES>
where
    Self: BitXor<Self, Output = Self>,
    T: SimdElement + BitXor<T, Output = T>,
    LaneCount<LANES>: SupportedLaneCount,
{
    /// 减少按位异或。
    /// 返回跨 vector lane 的累积按位异或。
    #[inline]
    pub fn reduce_xor(self) -> T {
        unsafe { simd_reduce_xor(self) }
    }
}
