mod float;
mod int;
mod uint;

pub use float::*;
pub use int::*;
pub use uint::*;

// Vectors 指针目前不公开使用。
pub(crate) mod ptr;

use crate::simd::intrinsics;
use crate::simd::{LaneCount, Mask, MaskElement, SupportedLaneCount};

/// `T` 类型的 `LANES` 元素的 SIMD vector。`Simd<T, N>` 与 [`[`T;N]`](array)，但操作类似于 `T`。
///
/// 按照惯例，相同类型和长度的两个向量将支持 `T` 所支持的运算符 (+、*、等)。
/// 它们在左侧和右侧的每个索引处获取 lane，执行操作，并将结果返回到相同大小的 vector 中的相同 lane 中。
/// 对于给定的运算符，这相当于将两个数组压缩在一起并将运算符映射到每个 lane。
///
/// ```rust
/// # #![feature(array_zip, portable_simd)]
/// # use core::simd::{Simd};
/// let a0: [i32; 4] = [-2, 0, 2, 4];
/// let a1 = [10, 9, 8, 7];
/// let zm_add = a0.zip(a1).map(|(lhs, rhs)| lhs + rhs);
/// let zm_mul = a0.zip(a1).map(|(lhs, rhs)| lhs * rhs);
///
/// // `Simd<T, N>` 实现 `From<[T; 否]>
/// let (v0, v1) = (Simd::from(a0), Simd::from(a1));
/// // 这意味着数组实现了 `Into<Simd<T, N>>`。
/// assert_eq!(v0 + v1, zm_add.into());
/// assert_eq!(v0 * v1, zm_mul.into());
/// ```
///
/// 带有整数的 `Simd` 有一个怪癖，即这些操作本身也是包装的，就好像 `T` 是 [`Wrapping<T>`]。
/// 因此，`Simd` 不实现 `wrapping_add`，因为这是默认行为。
/// 这意味着即使在 "debug" 版本中也不会出现溢出警告。
/// 对于大多数适合 `Simd` 的应用来说，换行就是 "not a bug"，甚至 "debug builds" 也不太可能容忍性能的损失。
/// 如果需要，您可能需要考虑使用显式检查算法。
/// 除以零仍然会导致 panic，因此如果不可接受，您可能需要考虑使用浮点数。
///
/// [`Wrapping<T>`]: core::num::Wrapping
///
/// # Layout
/// `Simd<T, N>` 的布局类似于 `[T; N]` (相同的 "shapes")，但对齐更大。
/// `[T; N]` 与 `T` 对齐，但 `Simd<T, N>` 将基于 `T` 和 `N` 进行对齐。
/// 因此对 [`transmute`] `Simd<T, N>` 对 `[T;
/// N]`，并且通常会优化到零成本，但反向嬗变更有可能需要编译器不能简单地忽略的副本。
///
/// # ABI "Features"
/// 由于 Rust 的安全保证，`Simd<T, N>` 目前是通过内存而不是 SIMD 寄存器在函数中传递和传递，除非作为优化。建议在接受 `Simd<T, N>` 或返回它的函数上使用 `#[inline]` 提示。
/// future 中可能会更正对此的需求。
///
/// # 带有不安全 Rust 的安全 SIMD
///
/// 使用 `Simd` 的操作通常是安全的，但有很多理由希望将 SIMD 与 `unsafe` 代码结合使用。
/// 必须注意尊重 `Simd` 和它可能被转换或派生的其他类型之间的差异。
/// 特别是 `Simd<T, N>` 的布局可能类似于 `[T;
/// N]`，并且可能允许一些变换，但对 `[T; N]` 的引用不能与对 `Simd<T, N>` 的引用互换。
/// 因此，在使用 `unsafe` Rust 通过 [raw pointers] 读写 `Simd<T, N>` 时，最好先尝试使用 [`read_unaligned`] 和 [`write_unaligned`]。这是因为:
/// - [`read`] 和 [`write`] 需要完全对齐 (在这种情况下，`Simd<T, N>` 的对齐方式)
/// - 可能的读取源或写入 `Simd<T, N>` 的目标是 [`[T]`](slice) 和类似类型，与 `T` 对齐
/// - 将这些操作结合起来会违反 `unsafe` 契约，并将程序炸成一堆**未定义的行为**
/// - 如果看到优化，编译器可以隐式调整布局以使未对齐的读取或写入完全对齐
/// - 大多数当代处理器不会因在运行时对齐的 "unaligned" 读取和写入而遭受性能损失
///
/// 通过施加较少的义务，未结盟函数不太可能使程序不健全，并且可能与更严格的替代方案一样快。
/// 在尝试保证对齐时，[`[T]::as_simd`][as_simd] 是将 `[T]` 转换为 `[Simd<T, N>]` 的一个选项，并允许在对齐的 SIMD 主体上进行良好的操作，但在处理标量头部和尾部时可能会花费更多时间。
///
/// 如果这些还不够，那么在使用 `unsafe` Rust 进行读取或写入之前，将数据结构设计为已经与您希望使用的 `Simd<T, N>` 对齐是最理想的。
/// 更传统的补偿这些事实的方法，例如首先将 `Simd` 物化到数组或从数组中取出，由 [`Simd::from_array`] 和 [`Simd::from_slice`] 等安全方法处理。
///
/// [`transmute`]: core::mem::transmute
/// [raw pointers]: pointer
/// [`read_unaligned`]: pointer::read_unaligned
/// [`write_unaligned`]: pointer::write_unaligned
/// [`read`]: pointer::read
/// [`write`]: pointer::write
/// [as_simd]: slice::as_simd
///
///
///
///
///
///
///
#[repr(simd)]
pub struct Simd<T, const LANES: usize>([T; LANES])
where
    T: SimdElement,
    LaneCount<LANES>: SupportedLaneCount;

impl<T, const LANES: usize> Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    /// 此 vector 中 lanes 的数量。
    pub const LANES: usize = LANES;

    /// 获取此 vector 中 lanes 的数量。
    pub const fn lanes(&self) -> usize {
        LANES
    }

    /// 通过将所有 lane 设置为给定值来构建 SIMD vector。
    pub const fn splat(value: T) -> Self {
        Self([value; LANES])
    }

    /// 返回包含整个 SIMD vector 的数组引用。
    pub const fn as_array(&self) -> &[T; LANES] {
        &self.0
    }

    /// 返回一个包含整个 SIMD vector 的可变数组引用。
    pub fn as_mut_array(&mut self) -> &mut [T; LANES] {
        &mut self.0
    }

    /// 将数组转换为 SIMD vector。
    pub const fn from_array(array: [T; LANES]) -> Self {
        Self(array)
    }

    /// 将 SIMD vector 转换为数组。
    pub const fn to_array(self) -> [T; LANES] {
        self.0
    }

    /// 将切片转换为包含 `slice[..LANES]` 的 SIMD vector
    /// # Panics
    /// 如果切片的 `len` 小于 vector 的 `Simd::LANES`，则 `from_slice` 会出现 panic。
    #[must_use]
    pub const fn from_slice(slice: &[T]) -> Self {
        assert!(slice.len() >= LANES, "slice length must be at least the number of lanes");
        let mut array = [slice[0]; LANES];
        let mut i = 0;
        while i < LANES {
            array[i] = slice[i];
            i += 1;
        }
        Self(array)
    }

    /// 执行 SIMD 向量元素到另一个 SIMD 有效类型的 lanewise 转换。
    /// 这遵循 Rust 的 `as` 转换语义，用于将整数转换为无符号整数 (解释为另一种类型，因此 `-1` 到 `MAX`)，以及每个 lane 从浮点数转换为整数 (在极限处截断或饱和)，反之亦然。
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::Simd;
    /// let floats: Simd<f32, 4> = Simd::from_array([1.9, -4.5, f32::INFINITY, f32::NAN]);
    /// let ints = floats.cast::<i32>();
    /// assert_eq!(ints, Simd::from_array([1, -4, i32::MAX, 0]));
    ///
    /// // 形式上是等价的，但 `Simd::cast` 可以优化得更好。
    /// assert_eq!(ints, Simd::from_array(floats.to_array().map(|x| x as i32)));
    ///
    /// // 浮点转换不需要往返。
    /// let floats_again = ints.cast();
    /// assert_ne!(floats, floats_again);
    /// assert_eq!(floats_again, Simd::from_array([1.0, -4.0, 2147483647.0, 0.0]));
    /// ```
    ///
    #[must_use]
    #[inline]
    pub fn cast<U: SimdElement>(self) -> Simd<U, LANES> {
        // 安全性: 输入参数是已知 SIMD 类型的 vector。
        unsafe { intrinsics::simd_as(self) }
    }

    /// 从 `slice` 中可能不连续的索引读取以构建 SIMD vector。
    /// 如果索引越界，则改为从 `or` vector 中选择 lane。
    ///
    /// # Examples
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::Simd;
    /// let vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 5]);
    /// let alt = Simd::from_array([-5, -4, -3, -2]);
    ///
    /// let result = Simd::gather_or(&vec, idxs, alt); // 注意越界的 lane。
    /// assert_eq!(result, Simd::from_array([-5, 13, 10, 15]));
    /// ```
    #[must_use]
    #[inline]
    pub fn gather_or(slice: &[T], idxs: Simd<usize, LANES>, or: Self) -> Self {
        Self::gather_select(slice, Mask::splat(true), idxs, or)
    }

    /// 从 `slice` 中可能不连续的索引读取以构建 SIMD vector。
    /// 如果索引越界，则 lane 将设置为该类型的默认值。
    ///
    /// # Examples
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::Simd;
    /// let vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 5]);
    ///
    /// let result = Simd::gather_or_default(&vec, idxs); // 注意越界的 lane。
    /// assert_eq!(result, Simd::from_array([0, 13, 10, 15]));
    /// ```
    #[must_use]
    #[inline]
    pub fn gather_or_default(slice: &[T], idxs: Simd<usize, LANES>) -> Self
    where
        T: Default,
    {
        Self::gather_or(slice, idxs, Self::splat(T::default()))
    }

    /// 从 `slice` 中可能不连续的索引读取以构建 SIMD vector。
    /// 掩码 `enable` 所有 `true` lane 并禁用所有 `false` lane。
    /// 如果索引被禁用或越界，则从 `or` vector 中选择 lane。
    ///
    /// # Examples
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::{Simd, Mask};
    /// let vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 5]);
    /// let alt = Simd::from_array([-5, -4, -3, -2]);
    /// let enable = Mask::from_array([true, true, true, false]); // 注意最后一个 lane 的掩码。
    ///
    /// let result = Simd::gather_select(&vec, enable, idxs, alt); // 注意越界的 lane。
    /// assert_eq!(result, Simd::from_array([-5, 13, 10, -2]));
    /// ```
    #[must_use]
    #[inline]
    pub fn gather_select(
        slice: &[T],
        enable: Mask<isize, LANES>,
        idxs: Simd<usize, LANES>,
        or: Self,
    ) -> Self {
        let enable: Mask<isize, LANES> = enable & idxs.lanes_lt(Simd::splat(slice.len()));
        // 安全性: 我们已经屏蔽了出界 lanes。
        unsafe { Self::gather_select_unchecked(slice, enable, idxs, or) }
    }

    /// 从 `slice` 中可能不连续的索引读取以构建 SIMD vector。
    /// 掩码 `enable` 所有 `true` lane 并禁用所有 `false` lane。
    /// 如果禁用某个索引，则从 `or` vector 中选择 lane。
    ///
    /// # Safety
    ///
    /// 使用 `enable`d 越界索引调用这个函数是 *[未定义的行为][undefined behavior]*，即使结果值没有被使用。
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::{Simd, Mask};
    /// let vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 5]);
    /// let alt = Simd::from_array([-5, -4, -3, -2]);
    /// let enable = Mask::from_array([true, true, true, false]); // 注意最后的掩码 lane。
    /// // 如果用这个掩码来收集，这是不合理的。让我们解决这个问题。
    /// let enable = enable & idxs.lanes_lt(Simd::splat(vec.len()));
    ///
    /// // 我们已经屏蔽了 OOB lane，所以现在可以安全收集了。
    /// let result = unsafe { Simd::gather_select_unchecked(&vec, enable, idxs, alt) };
    /// assert_eq!(result, Simd::from_array([-5, 13, 10, -2]));
    /// ```
    /// [undefined behavior]: https://doc.rust-lang.org/reference/behavior-considered-undefined.html
    #[must_use]
    #[inline]
    pub unsafe fn gather_select_unchecked(
        slice: &[T],
        enable: Mask<isize, LANES>,
        idxs: Simd<usize, LANES>,
        or: Self,
    ) -> Self {
        let base_ptr = crate::simd::ptr::SimdConstPtr::splat(slice.as_ptr());
        // Ferris 原谅我，我在这里做了指针运算。
        let ptrs = base_ptr.wrapping_add(idxs);
        // 安全性: ptrs 已被屏蔽以防止内存不安全读取 insha'allah
        unsafe { intrinsics::simd_gather(or, ptrs, enable.to_int()) }
    }

    /// 将 SIMD vector 中的值写入 `slice` 中可能不连续的索引。
    /// 如果分散的 vector 中的两个 lanes 将写入相同的索引，则只能保证实际写入最后一个 lane。
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::Simd;
    /// let mut vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 0]);
    /// let vals = Simd::from_array([-27, 82, -41, 124]);
    ///
    /// vals.scatter(&mut vec, idxs); // 索引 0 接收两次写入。
    /// assert_eq!(vec, vec![124, 11, 12, 82, 14, 15, 16, 17, 18]);
    /// ```
    #[inline]
    pub fn scatter(self, slice: &mut [T], idxs: Simd<usize, LANES>) {
        self.scatter_select(slice, Mask::splat(true), idxs)
    }

    /// 将 SIMD vector 中的值写入 `slice` 中的多个可能不连续的索引。
    /// 掩码 `enable` 所有 `true` lane 并禁用所有 `false` lane。
    /// 如果启用的索引越界，则不会写入 lane。
    /// 如果分散的 vector 中的两个启用的 lanes 将写入同一个索引，则只能保证实际写入最后一个 lane。
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::{Simd, Mask};
    /// let mut vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 0]);
    /// let vals = Simd::from_array([-27, 82, -41, 124]);
    /// let enable = Mask::from_array([true, true, true, false]); // 注意最后一个 lane 的掩码。
    ///
    /// vals.scatter_select(&mut vec, enable, idxs); // 索引 0 的第二次写入被屏蔽，因此被省略。
    /// assert_eq!(vec, vec![-41, 11, 12, 82, 14, 15, 16, 17, 18]);
    /// ```
    #[inline]
    pub fn scatter_select(
        self,
        slice: &mut [T],
        enable: Mask<isize, LANES>,
        idxs: Simd<usize, LANES>,
    ) {
        let enable: Mask<isize, LANES> = enable & idxs.lanes_lt(Simd::splat(slice.len()));
        // 安全性: 我们已经屏蔽了出界 lanes。
        unsafe { self.scatter_select_unchecked(slice, enable, idxs) }
    }

    /// 将 SIMD vector 中的值写入 `slice` 中的多个可能不连续的索引。
    /// 掩码 `enable` 所有 `true` lane 并禁用所有 `false` lane。
    /// 如果分散的 vector 中的两个启用的 lanes 将写入同一个索引，则只能保证实际写入最后一个 lane。
    ///
    ///
    /// # Safety
    ///
    /// 使用启用的越界索引调用此函数是 *[undefined 行为]*，并可能导致内存损坏。
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::{Simd, Mask};
    /// let mut vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 0]);
    /// let vals = Simd::from_array([-27, 82, -41, 124]);
    /// let enable = Mask::from_array([true, true, true, false]); // 注意最后一个 lane 的掩码。
    /// // 如果用这个掩码来分散，这是不合理的。让我们解决这个问题。
    /// let enable = enable & idxs.lanes_lt(Simd::splat(vec.len()));
    ///
    /// // 我们已经屏蔽了 OOB lane，所以现在可以安全地分散了。
    /// unsafe { vals.scatter_select_unchecked(&mut vec, enable, idxs); }
    /// // 索引 0 的第二次写入被屏蔽，因此被省略。
    /// assert_eq!(vec, vec![-41, 11, 12, 82, 14, 15, 16, 17, 18]);
    /// ```
    /// [undefined behavior]: https://doc.rust-lang.org/reference/behavior-considered-undefined.html
    #[inline]
    pub unsafe fn scatter_select_unchecked(
        self,
        slice: &mut [T],
        enable: Mask<isize, LANES>,
        idxs: Simd<usize, LANES>,
    ) {
        // 安全性: 此块适用于从 &mut 'a [T] 派生的 *mut T，这意味着它在 Rust 的借用模型中很微妙，大约在 2021 年:
        // &mut 'a [T] 断言唯一性，因此派生 &'a [T] 会使活着的 *mut Ts 无效！
        // 尽管这个块在很大程度上是安全的方法，但它必须完全采用这种方式来防止在原始 ptr 处于活动状态时使其无效。
        //
        // 因此，进入此块需要所有要使用的值都已经准备好：
        // 0. 我们要写入的 idxs，用于构造掩码。
        // 1. 启用，这取决于初始 `&'a [T]` 和 idxs。
        // 2. 要分散 (self) 的实际值。
        // 3. &mut [T] 将成为我们的基础 ptr。
        //
        unsafe {
            // 现在进入 ☢️ `*mut T` 区域
            let base_ptr = crate::simd::ptr::SimdMutPtr::splat(slice.as_mut_ptr());
            // Ferris 原谅我，我在这里做了指针运算。
            let ptrs = base_ptr.wrapping_add(idxs);
            // ptr 已被边界屏蔽以防止内存不安全写入 insha'allah
            intrinsics::simd_scatter(self, ptrs, enable.to_int())
            // 清除 ☢️ `*mut T` 区域
        }
    }
}

impl<T, const LANES: usize> Copy for Simd<T, LANES>
where
    T: SimdElement,
    LaneCount<LANES>: SupportedLaneCount,
{
}

impl<T, const LANES: usize> Clone for Simd<T, LANES>
where
    T: SimdElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    fn clone(&self) -> Self {
        *self
    }
}

impl<T, const LANES: usize> Default for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + Default,
{
    #[inline]
    fn default() -> Self {
        Self::splat(T::default())
    }
}

impl<T, const LANES: usize> PartialEq for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + PartialEq,
{
    #[inline]
    fn eq(&self, other: &Self) -> bool {
        // TODO 使用 SIMD 等式
        self.to_array() == other.to_array()
    }
}

impl<T, const LANES: usize> PartialOrd for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + PartialOrd,
{
    #[inline]
    fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
        // TODO 使用 SIMD 等式
        self.to_array().partial_cmp(other.as_ref())
    }
}

impl<T, const LANES: usize> Eq for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + Eq,
{
}

impl<T, const LANES: usize> Ord for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + Ord,
{
    #[inline]
    fn cmp(&self, other: &Self) -> core::cmp::Ordering {
        // TODO 使用 SIMD 等式
        self.to_array().cmp(other.as_ref())
    }
}

impl<T, const LANES: usize> core::hash::Hash for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + core::hash::Hash,
{
    #[inline]
    fn hash<H>(&self, state: &mut H)
    where
        H: core::hash::Hasher,
    {
        self.as_array().hash(state)
    }
}

// 数组引用
impl<T, const LANES: usize> AsRef<[T; LANES]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    #[inline]
    fn as_ref(&self) -> &[T; LANES] {
        &self.0
    }
}

impl<T, const LANES: usize> AsMut<[T; LANES]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    #[inline]
    fn as_mut(&mut self) -> &mut [T; LANES] {
        &mut self.0
    }
}

// 切片引用
impl<T, const LANES: usize> AsRef<[T]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    #[inline]
    fn as_ref(&self) -> &[T] {
        &self.0
    }
}

impl<T, const LANES: usize> AsMut<[T]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    #[inline]
    fn as_mut(&mut self) -> &mut [T] {
        &mut self.0
    }
}

// vector 和 array 的转换
impl<T, const LANES: usize> From<[T; LANES]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    fn from(array: [T; LANES]) -> Self {
        Self(array)
    }
}

impl<T, const LANES: usize> From<Simd<T, LANES>> for [T; LANES]
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    fn from(vector: Simd<T, LANES>) -> Self {
        vector.to_array()
    }
}

mod sealed {
    pub trait Sealed {}
}
use sealed::Sealed;

/// 可用作 SIMD vector 元素的类型的标记 trait。
///
/// # Safety
/// 这个 trait 在实现时断言编译器可以将标记类型作为元素的 `#[repr(simd)]` 结构体单态化。
/// 严格地说，如果 vector 不会被错误编译，则 impl 是有效的。
/// 实际上，如果 vector 无法编译，那么实现它对用户是不友好的，即使允许用户尝试并没有破坏健全性保证。
///
///
pub unsafe trait SimdElement: Sealed + Copy {
    /// 此元素类型对应的掩码元素类型。
    type Mask: MaskElement;
}

impl Sealed for u8 {}
unsafe impl SimdElement for u8 {
    type Mask = i8;
}

impl Sealed for u16 {}
unsafe impl SimdElement for u16 {
    type Mask = i16;
}

impl Sealed for u32 {}
unsafe impl SimdElement for u32 {
    type Mask = i32;
}

impl Sealed for u64 {}
unsafe impl SimdElement for u64 {
    type Mask = i64;
}

impl Sealed for usize {}
unsafe impl SimdElement for usize {
    type Mask = isize;
}

impl Sealed for i8 {}
unsafe impl SimdElement for i8 {
    type Mask = i8;
}

impl Sealed for i16 {}
unsafe impl SimdElement for i16 {
    type Mask = i16;
}

impl Sealed for i32 {}
unsafe impl SimdElement for i32 {
    type Mask = i32;
}

impl Sealed for i64 {}
unsafe impl SimdElement for i64 {
    type Mask = i64;
}

impl Sealed for isize {}
unsafe impl SimdElement for isize {
    type Mask = isize;
}

impl Sealed for f32 {}
unsafe impl SimdElement for f32 {
    type Mask = i32;
}

impl Sealed for f64 {}
unsafe impl SimdElement for f64 {
    type Mask = i64;
}
