mod float;
mod int;
mod uint;

pub use float::*;
pub use int::*;
pub use uint::*;

// Vectors of pointers are not for public use at the current time. <br>Vectors 指针目前不公开使用。<br>
pub(crate) mod ptr;

use crate::simd::intrinsics;
use crate::simd::{LaneCount, Mask, MaskElement, SupportedLaneCount};

/// A SIMD vector of `LANES` elements of type `T`. <br>`T` 类型的 `LANES` 元素的 SIMD vector。<br> `Simd<T, N>` has the same shape as [`[T; N]`](array), but operates like `T`. <br>`Simd<T, N>` 与 [`[`T;N]`](array)，但操作类似于 `T`。<br>
///
/// Two vectors of the same type and length will, by convention, support the operators (+, *, etc.) that `T` does. <br>按照惯例，相同类型和长度的两个向量将支持 `T` 所支持的运算符 (+、*、等)。<br>
/// These take the lanes at each index on the left-hand side and right-hand side, perform the operation, and return the result in the same lane in a vector of equal size. <br>它们在左侧和右侧的每个索引处获取 lane，执行操作，并将结果返回到相同大小的 vector 中的相同 lane 中。<br>
/// For a given operator, this is equivalent to zipping the two arrays together and mapping the operator over each lane. <br>对于给定的运算符，这相当于将两个数组压缩在一起并将运算符映射到每个 lane。<br>
///
/// ```rust
/// # #![feature(array_zip, portable_simd)]
/// # use core::simd::{Simd};
/// let a0: [i32; 4] = [-2, 0, 2, 4];
/// let a1 = [10, 9, 8, 7];
/// let zm_add = a0.zip(a1).map(|(lhs, rhs)| lhs + rhs);
/// let zm_mul = a0.zip(a1).map(|(lhs, rhs)| lhs * rhs);
///
/// // `Simd<T, N>` implements `From<[T; N]> <br>`Simd<T, N>` 实现 `From<[T; 否]><br>
/// let (v0, v1) = (Simd::from(a0), Simd::from(a1));
/// // Which means arrays implement `Into<Simd<T, N>>`. <br>这意味着数组实现了 `Into<Simd<T, N>>`。<br>
/// assert_eq!(v0 + v1, zm_add.into());
/// assert_eq!(v0 * v1, zm_mul.into());
/// ```
///
/// `Simd` with integers has the quirk that these operations are also inherently wrapping, as if `T` was [`Wrapping<T>`]. <br>带有整数的 `Simd` 有一个怪癖，即这些操作本身也是包装的，就好像 `T` 是 [`Wrapping<T>`]。<br>
/// Thus, `Simd` does not implement `wrapping_add`, because that is the default behavior. <br>因此，`Simd` 不实现 `wrapping_add`，因为这是默认行为。<br>
/// This means there is no warning on overflows, even in "debug" builds. <br>这意味着即使在 "debug" 版本中也不会出现溢出警告。<br>
/// For most applications where `Simd` is appropriate, it is "not a bug" to wrap, and even "debug builds" are unlikely to tolerate the loss of performance. <br>对于大多数适合 `Simd` 的应用来说，换行就是 "not a bug"，甚至 "debug builds" 也不太可能容忍性能的损失。<br>
/// You may want to consider using explicitly checked arithmetic if such is required. <br>如果需要，您可能需要考虑使用显式检查算法。<br>
/// Division by zero still causes a panic, so you may want to consider using floating point numbers if that is unacceptable. <br>除以零仍然会导致 panic，因此如果不可接受，您可能需要考虑使用浮点数。<br>
///
/// [`Wrapping<T>`]: core::num::Wrapping
///
/// # Layout
/// `Simd<T, N>` has a layout similar to `[T; N]` (identical "shapes"), but with a greater alignment. <br>`Simd<T, N>` 的布局类似于 `[T; N]` (相同的 "shapes")，但对齐更大。<br>
/// `[T; N]` is aligned to `T`, but `Simd<T, N>` will have an alignment based on both `T` and `N`. <br>`[T; N]` 与 `T` 对齐，但 `Simd<T, N>` 将基于 `T` 和 `N` 进行对齐。<br>
/// It is thus sound to [`transmute`] `Simd<T, N>` to `[T; <br>因此对 [`transmute`] `Simd<T, N>` 对 `[T;<br>
/// N]`, and will typically optimize to zero cost, but the reverse transmutation is more likely to require a copy the compiler cannot simply elide. <br>N]`，并且通常会优化到零成本，但反向嬗变更有可能需要编译器不能简单地忽略的副本。<br>
///
/// # ABI "Features"
/// Due to Rust's safety guarantees, `Simd<T, N>` is currently passed to and from functions via memory, not SIMD registers, except as an optimization. <br>由于 Rust 的安全保证，`Simd<T, N>` 目前是通过内存而不是 SIMD 寄存器在函数中传递和传递，除非作为优化。<br> `#[inline]` hints are recommended on functions that accept `Simd<T, N>` or return it. <br>建议在接受 `Simd<T, N>` 或返回它的函数上使用 `#[inline]` 提示。<br>
/// The need for this may be corrected in the future. <br>future 中可能会更正对此的需求。<br>
///
/// # Safe SIMD with Unsafe Rust <br>带有不安全 Rust 的安全 SIMD<br>
///
/// Operations with `Simd` are typically safe, but there are many reasons to want to combine SIMD with `unsafe` code. <br>使用 `Simd` 的操作通常是安全的，但有很多理由希望将 SIMD 与 `unsafe` 代码结合使用。<br>
/// Care must be taken to respect differences between `Simd` and other types it may be transformed into or derived from. <br>必须注意尊重 `Simd` 和它可能被转换或派生的其他类型之间的差异。<br>
/// In particular, the layout of `Simd<T, N>` may be similar to `[T; <br>特别是 `Simd<T, N>` 的布局可能类似于 `[T;<br>
/// N]`, and may allow some transmutations, but references to `[T; N]` are not interchangeable with those to `Simd<T, N>`. <br>N]`，并且可能允许一些变换，但对 `[T; N]` 的引用不能与对 `Simd<T, N>` 的引用互换。<br>
/// Thus, when using `unsafe` Rust to read and write `Simd<T, N>` through [raw pointers], it is a good idea to first try with [`read_unaligned`] and [`write_unaligned`]. <br>因此，在使用 `unsafe` Rust 通过 [raw pointers] 读写 `Simd<T, N>` 时，最好先尝试使用 [`read_unaligned`] 和 [`write_unaligned`]。<br> This is because: <br>这是因为:<br>
/// - [`read`] and [`write`] require full alignment (in this case, `Simd<T, N>`'s alignment) <br>[`read`] 和 [`write`] 需要完全对齐 (在这种情况下，`Simd<T, N>` 的对齐方式)<br>
/// - the likely source for reading or destination for writing `Simd<T, N>` is [`[T]`](slice) and similar types, aligned to `T` <br>可能的读取源或写入 `Simd<T, N>` 的目标是 [`[T]`](slice) 和类似类型，与 `T` 对齐<br>
/// - combining these actions would violate the `unsafe` contract and explode the program into a puff of **undefined behavior** <br>将这些操作结合起来会违反 `unsafe` 契约，并将程序炸成一堆**未定义的行为**<br>
/// - the compiler can implicitly adjust layouts to make unaligned reads or writes fully aligned if it sees the optimization <br>如果看到优化，编译器可以隐式调整布局以使未对齐的读取或写入完全对齐<br>
/// - most contemporary processors suffer no performance penalty for "unaligned" reads and writes that are aligned at runtime <br>大多数当代处理器不会因在运行时对齐的 "unaligned" 读取和写入而遭受性能损失<br>
///
/// By imposing less obligations, unaligned functions are less likely to make the program unsound, and may be just as fast as stricter alternatives. <br>通过施加较少的义务，未结盟函数不太可能使程序不健全，并且可能与更严格的替代方案一样快。<br>
/// When trying to guarantee alignment, [`[T]::as_simd`][as_simd] is an option for converting `[T]` to `[Simd<T, N>]`, and allows soundly operating on an aligned SIMD body, but it may cost more time when handling the scalar head and tail. <br>在尝试保证对齐时，[`[T]::as_simd`][as_simd] 是将 `[T]` 转换为 `[Simd<T, N>]` 的一个选项，并允许在对齐的 SIMD 主体上进行良好的操作，但在处理标量头部和尾部时可能会花费更多时间。<br>
///
/// If these are not sufficient, then it is most ideal to design data structures to be already aligned to the `Simd<T, N>` you wish to use before using `unsafe` Rust to read or write. <br>如果这些还不够，那么在使用 `unsafe` Rust 进行读取或写入之前，将数据结构设计为已经与您希望使用的 `Simd<T, N>` 对齐是最理想的。<br>
/// More conventional ways to compensate for these facts, like materializing `Simd` to or from an array first, are handled by safe methods like [`Simd::from_array`] and [`Simd::from_slice`]. <br>更传统的补偿这些事实的方法，例如首先将 `Simd` 物化到数组或从数组中取出，由 [`Simd::from_array`] 和 [`Simd::from_slice`] 等安全方法处理。<br>
///
/// [`transmute`]: core::mem::transmute
/// [raw pointers]: pointer
/// [`read_unaligned`]: pointer::read_unaligned
/// [`write_unaligned`]: pointer::write_unaligned
/// [`read`]: pointer::read
/// [`write`]: pointer::write
/// [as_simd]: slice::as_simd
///
///
///
///
///
///
///
#[repr(simd)]
pub struct Simd<T, const LANES: usize>([T; LANES])
where
    T: SimdElement,
    LaneCount<LANES>: SupportedLaneCount;

impl<T, const LANES: usize> Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    /// Number of lanes in this vector. <br>此 vector 中 lanes 的数量。<br>
    pub const LANES: usize = LANES;

    /// Get the number of lanes in this vector. <br>获取此 vector 中 lanes 的数量。<br>
    pub const fn lanes(&self) -> usize {
        LANES
    }

    /// Construct a SIMD vector by setting all lanes to the given value. <br>通过将所有 lane 设置为给定值来构建 SIMD vector。<br>
    pub const fn splat(value: T) -> Self {
        Self([value; LANES])
    }

    /// Returns an array reference containing the entire SIMD vector. <br>返回包含整个 SIMD vector 的数组引用。<br>
    pub const fn as_array(&self) -> &[T; LANES] {
        &self.0
    }

    /// Returns a mutable array reference containing the entire SIMD vector. <br>返回一个包含整个 SIMD vector 的可变数组引用。<br>
    pub fn as_mut_array(&mut self) -> &mut [T; LANES] {
        &mut self.0
    }

    /// Converts an array to a SIMD vector. <br>将数组转换为 SIMD vector。<br>
    pub const fn from_array(array: [T; LANES]) -> Self {
        Self(array)
    }

    /// Converts a SIMD vector to an array. <br>将 SIMD vector 转换为数组。<br>
    pub const fn to_array(self) -> [T; LANES] {
        self.0
    }

    /// Converts a slice to a SIMD vector containing `slice[..LANES]` <br>将切片转换为包含 `slice[..LANES]` 的 SIMD vector<br>
    /// # Panics
    /// `from_slice` will panic if the slice's `len` is less than the vector's `Simd::LANES`. <br>如果切片的 `len` 小于 vector 的 `Simd::LANES`，则 `from_slice` 会出现 panic。<br>
    #[must_use]
    pub const fn from_slice(slice: &[T]) -> Self {
        assert!(slice.len() >= LANES, "slice length must be at least the number of lanes");
        let mut array = [slice[0]; LANES];
        let mut i = 0;
        while i < LANES {
            array[i] = slice[i];
            i += 1;
        }
        Self(array)
    }

    /// Performs lanewise conversion of a SIMD vector's elements to another SIMD-valid type. <br>执行 SIMD 向量元素到另一个 SIMD 有效类型的 lanewise 转换。<br>
    /// This follows the semantics of Rust's `as` conversion for casting integers to unsigned integers (interpreting as the other type, so `-1` to `MAX`), and from floats to integers (truncating, or saturating at the limits) for each lane, or vice versa. <br>这遵循 Rust 的 `as` 转换语义，用于将整数转换为无符号整数 (解释为另一种类型，因此 `-1` 到 `MAX`)，以及每个 lane 从浮点数转换为整数 (在极限处截断或饱和)，反之亦然。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::Simd;
    /// let floats: Simd<f32, 4> = Simd::from_array([1.9, -4.5, f32::INFINITY, f32::NAN]);
    /// let ints = floats.cast::<i32>();
    /// assert_eq!(ints, Simd::from_array([1, -4, i32::MAX, 0]));
    ///
    /// // Formally equivalent, but `Simd::cast` can optimize better. <br>形式上是等价的，但 `Simd::cast` 可以优化得更好。<br>
    /// assert_eq!(ints, Simd::from_array(floats.to_array().map(|x| x as i32)));
    ///
    /// // The float conversion does not round-trip. <br>浮点转换不需要往返。<br>
    /// let floats_again = ints.cast();
    /// assert_ne!(floats, floats_again);
    /// assert_eq!(floats_again, Simd::from_array([1.0, -4.0, 2147483647.0, 0.0]));
    /// ```
    ///
    #[must_use]
    #[inline]
    pub fn cast<U: SimdElement>(self) -> Simd<U, LANES> {
        // Safety: The input argument is a vector of a known SIMD type. <br>安全性: 输入参数是已知 SIMD 类型的 vector。<br>
        unsafe { intrinsics::simd_as(self) }
    }

    /// Reads from potentially discontiguous indices in `slice` to construct a SIMD vector. <br>从 `slice` 中可能不连续的索引读取以构建 SIMD vector。<br>
    /// If an index is out-of-bounds, the lane is instead selected from the `or` vector. <br>如果索引越界，则改为从 `or` vector 中选择 lane。<br>
    ///
    /// # Examples
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::Simd;
    /// let vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 5]);
    /// let alt = Simd::from_array([-5, -4, -3, -2]);
    ///
    /// let result = Simd::gather_or(&vec, idxs, alt); // Note the lane that is out-of-bounds. <br>注意越界的 lane。<br>
    /// assert_eq!(result, Simd::from_array([-5, 13, 10, 15]));
    /// ```
    #[must_use]
    #[inline]
    pub fn gather_or(slice: &[T], idxs: Simd<usize, LANES>, or: Self) -> Self {
        Self::gather_select(slice, Mask::splat(true), idxs, or)
    }

    /// Reads from potentially discontiguous indices in `slice` to construct a SIMD vector. <br>从 `slice` 中可能不连续的索引读取以构建 SIMD vector。<br>
    /// If an index is out-of-bounds, the lane is set to the default value for the type. <br>如果索引越界，则 lane 将设置为该类型的默认值。<br>
    ///
    /// # Examples
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::Simd;
    /// let vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 5]);
    ///
    /// let result = Simd::gather_or_default(&vec, idxs); // Note the lane that is out-of-bounds. <br>注意越界的 lane。<br>
    /// assert_eq!(result, Simd::from_array([0, 13, 10, 15]));
    /// ```
    #[must_use]
    #[inline]
    pub fn gather_or_default(slice: &[T], idxs: Simd<usize, LANES>) -> Self
    where
        T: Default,
    {
        Self::gather_or(slice, idxs, Self::splat(T::default()))
    }

    /// Reads from potentially discontiguous indices in `slice` to construct a SIMD vector. <br>从 `slice` 中可能不连续的索引读取以构建 SIMD vector。<br>
    /// The mask `enable`s all `true` lanes and disables all `false` lanes. <br>掩码 `enable` 所有 `true` lane 并禁用所有 `false` lane。<br>
    /// If an index is disabled or is out-of-bounds, the lane is selected from the `or` vector. <br>如果索引被禁用或越界，则从 `or` vector 中选择 lane。<br>
    ///
    /// # Examples
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::{Simd, Mask};
    /// let vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 5]);
    /// let alt = Simd::from_array([-5, -4, -3, -2]);
    /// let enable = Mask::from_array([true, true, true, false]); // Note the mask of the last lane. <br>注意最后一个 lane 的掩码。<br>
    ///
    /// let result = Simd::gather_select(&vec, enable, idxs, alt); // Note the lane that is out-of-bounds. <br>注意越界的 lane。<br>
    /// assert_eq!(result, Simd::from_array([-5, 13, 10, -2]));
    /// ```
    #[must_use]
    #[inline]
    pub fn gather_select(
        slice: &[T],
        enable: Mask<isize, LANES>,
        idxs: Simd<usize, LANES>,
        or: Self,
    ) -> Self {
        let enable: Mask<isize, LANES> = enable & idxs.lanes_lt(Simd::splat(slice.len()));
        // Safety: We have masked-off out-of-bounds lanes. <br>安全性: 我们已经屏蔽了出界 lanes。<br>
        unsafe { Self::gather_select_unchecked(slice, enable, idxs, or) }
    }

    /// Reads from potentially discontiguous indices in `slice` to construct a SIMD vector. <br>从 `slice` 中可能不连续的索引读取以构建 SIMD vector。<br>
    /// The mask `enable`s all `true` lanes and disables all `false` lanes. <br>掩码 `enable` 所有 `true` lane 并禁用所有 `false` lane。<br>
    /// If an index is disabled, the lane is selected from the `or` vector. <br>如果禁用某个索引，则从 `or` vector 中选择 lane。<br>
    ///
    /// # Safety
    ///
    /// Calling this function with an `enable`d out-of-bounds index is *[undefined behavior]* even if the resulting value is not used. <br>使用 `enable`d 越界索引调用这个函数是 *[未定义的行为][undefined behavior]*，即使结果值没有被使用。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::{Simd, Mask};
    /// let vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 5]);
    /// let alt = Simd::from_array([-5, -4, -3, -2]);
    /// let enable = Mask::from_array([true, true, true, false]); // Note the final mask lane. <br>注意最后的掩码 lane。<br>
    /// // If this mask was used to gather, it would be unsound. <br>如果用这个掩码来收集，这是不合理的。<br> Let's fix that. <br>让我们解决这个问题。<br>
    /// let enable = enable & idxs.lanes_lt(Simd::splat(vec.len()));
    ///
    /// // We have masked the OOB lane, so it's safe to gather now. <br>我们已经屏蔽了 OOB lane，所以现在可以安全收集了。<br>
    /// let result = unsafe { Simd::gather_select_unchecked(&vec, enable, idxs, alt) };
    /// assert_eq!(result, Simd::from_array([-5, 13, 10, -2]));
    /// ```
    /// [undefined behavior]: https://doc.rust-lang.org/reference/behavior-considered-undefined.html
    #[must_use]
    #[inline]
    pub unsafe fn gather_select_unchecked(
        slice: &[T],
        enable: Mask<isize, LANES>,
        idxs: Simd<usize, LANES>,
        or: Self,
    ) -> Self {
        let base_ptr = crate::simd::ptr::SimdConstPtr::splat(slice.as_ptr());
        // Ferris forgive me, I have done pointer arithmetic here. <br>Ferris 原谅我，我在这里做了指针运算。<br>
        let ptrs = base_ptr.wrapping_add(idxs);
        // Safety: The ptrs have been bounds-masked to prevent memory-unsafe reads insha'allah <br>安全性: ptrs 已被屏蔽以防止内存不安全读取 insha'allah<br>
        unsafe { intrinsics::simd_gather(or, ptrs, enable.to_int()) }
    }

    /// Writes the values in a SIMD vector to potentially discontiguous indices in `slice`. <br>将 SIMD vector 中的值写入 `slice` 中可能不连续的索引。<br>
    /// If two lanes in the scattered vector would write to the same index only the last lane is guaranteed to actually be written. <br>如果分散的 vector 中的两个 lanes 将写入相同的索引，则只能保证实际写入最后一个 lane。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::Simd;
    /// let mut vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 0]);
    /// let vals = Simd::from_array([-27, 82, -41, 124]);
    ///
    /// vals.scatter(&mut vec, idxs); // index 0 receives two writes. <br>索引 0 接收两次写入。<br>
    /// assert_eq!(vec, vec![124, 11, 12, 82, 14, 15, 16, 17, 18]);
    /// ```
    #[inline]
    pub fn scatter(self, slice: &mut [T], idxs: Simd<usize, LANES>) {
        self.scatter_select(slice, Mask::splat(true), idxs)
    }

    /// Writes the values in a SIMD vector to multiple potentially discontiguous indices in `slice`. <br>将 SIMD vector 中的值写入 `slice` 中的多个可能不连续的索引。<br>
    /// The mask `enable`s all `true` lanes and disables all `false` lanes. <br>掩码 `enable` 所有 `true` lane 并禁用所有 `false` lane。<br>
    /// If an enabled index is out-of-bounds, the lane is not written. <br>如果启用的索引越界，则不会写入 lane。<br>
    /// If two enabled lanes in the scattered vector would write to the same index, only the last lane is guaranteed to actually be written. <br>如果分散的 vector 中的两个启用的 lanes 将写入同一个索引，则只能保证实际写入最后一个 lane。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::{Simd, Mask};
    /// let mut vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 0]);
    /// let vals = Simd::from_array([-27, 82, -41, 124]);
    /// let enable = Mask::from_array([true, true, true, false]); // Note the mask of the last lane. <br>注意最后一个 lane 的掩码。<br>
    ///
    /// vals.scatter_select(&mut vec, enable, idxs); // index 0's second write is masked, thus omitted. <br>索引 0 的第二次写入被屏蔽，因此被省略。<br>
    /// assert_eq!(vec, vec![-41, 11, 12, 82, 14, 15, 16, 17, 18]);
    /// ```
    #[inline]
    pub fn scatter_select(
        self,
        slice: &mut [T],
        enable: Mask<isize, LANES>,
        idxs: Simd<usize, LANES>,
    ) {
        let enable: Mask<isize, LANES> = enable & idxs.lanes_lt(Simd::splat(slice.len()));
        // Safety: We have masked-off out-of-bounds lanes. <br>安全性: 我们已经屏蔽了出界 lanes。<br>
        unsafe { self.scatter_select_unchecked(slice, enable, idxs) }
    }

    /// Writes the values in a SIMD vector to multiple potentially discontiguous indices in `slice`. <br>将 SIMD vector 中的值写入 `slice` 中的多个可能不连续的索引。<br>
    /// The mask `enable`s all `true` lanes and disables all `false` lanes. <br>掩码 `enable` 所有 `true` lane 并禁用所有 `false` lane。<br>
    /// If two enabled lanes in the scattered vector would write to the same index, only the last lane is guaranteed to actually be written. <br>如果分散的 vector 中的两个启用的 lanes 将写入同一个索引，则只能保证实际写入最后一个 lane。<br>
    ///
    ///
    /// # Safety
    ///
    /// Calling this function with an enabled out-of-bounds index is *[undefined behavior]*, and may lead to memory corruption. <br>使用启用的越界索引调用此函数是 *[undefined 行为]*，并可能导致内存损坏。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(portable_simd)]
    /// # use core::simd::{Simd, Mask};
    /// let mut vec: Vec<i32> = vec![10, 11, 12, 13, 14, 15, 16, 17, 18];
    /// let idxs = Simd::from_array([9, 3, 0, 0]);
    /// let vals = Simd::from_array([-27, 82, -41, 124]);
    /// let enable = Mask::from_array([true, true, true, false]); // Note the mask of the last lane. <br>注意最后一个 lane 的掩码。<br>
    /// // If this mask was used to scatter, it would be unsound. <br>如果用这个掩码来分散，这是不合理的。<br> Let's fix that. <br>让我们解决这个问题。<br>
    /// let enable = enable & idxs.lanes_lt(Simd::splat(vec.len()));
    ///
    /// // We have masked the OOB lane, so it's safe to scatter now. <br>我们已经屏蔽了 OOB lane，所以现在可以安全地分散了。<br>
    /// unsafe { vals.scatter_select_unchecked(&mut vec, enable, idxs); }
    /// // index 0's second write is masked, thus was omitted. <br>索引 0 的第二次写入被屏蔽，因此被省略。<br>
    /// assert_eq!(vec, vec![-41, 11, 12, 82, 14, 15, 16, 17, 18]);
    /// ```
    /// [undefined behavior]: https://doc.rust-lang.org/reference/behavior-considered-undefined.html
    #[inline]
    pub unsafe fn scatter_select_unchecked(
        self,
        slice: &mut [T],
        enable: Mask<isize, LANES>,
        idxs: Simd<usize, LANES>,
    ) {
        // Safety: This block works with *mut T derived from &mut 'a [T], which means it is delicate in Rust's borrowing model, circa 2021: <br>安全性: 此块适用于从 &mut 'a [T] 派生的 *mut T，这意味着它在 Rust 的借用模型中很微妙，大约在 2021 年:<br>
        // &mut 'a [T] asserts uniqueness, so deriving &'a [T] invalidates live *mut Ts! <br>&mut 'a [T] 断言唯一性，因此派生 &'a [T] 会使活着的 *mut Ts 无效！<br>
        // Even though this block is largely safe methods, it must be exactly this way to prevent invalidating the raw ptrs while they're live. <br>尽管这个块在很大程度上是安全的方法，但它必须完全采用这种方式来防止在原始 ptr 处于活动状态时使其无效。<br>
        //
        // Thus, entering this block requires all values to use being already ready: <br>因此，进入此块需要所有要使用的值都已经准备好：<br>
        // 0. idxs we want to write to, which are used to construct the mask. <br>我们要写入的 idxs，用于构造掩码。<br>
        // 1. enable, which depends on an initial &'a [T] and the idxs. <br>启用，这取决于初始 `&'a [T]` 和 idxs。<br>
        // 2. actual values to scatter (self). <br>要分散 (self) 的实际值。<br>
        // 3. &mut [T] which will become our base ptr. <br>&mut [T] 将成为我们的基础 ptr。<br>
        //
        unsafe {
            // Now Entering ☢️ *mut T Zone <br>现在进入 ☢️ `*mut T` 区域<br>
            let base_ptr = crate::simd::ptr::SimdMutPtr::splat(slice.as_mut_ptr());
            // Ferris forgive me, I have done pointer arithmetic here. <br>Ferris 原谅我，我在这里做了指针运算。<br>
            let ptrs = base_ptr.wrapping_add(idxs);
            // The ptrs have been bounds-masked to prevent memory-unsafe writes insha'allah <br>ptr 已被边界屏蔽以防止内存不安全写入 insha'allah<br>
            intrinsics::simd_scatter(self, ptrs, enable.to_int())
            // Cleared ☢️ *mut T Zone <br>清除 ☢️ `*mut T` 区域<br>
        }
    }
}

impl<T, const LANES: usize> Copy for Simd<T, LANES>
where
    T: SimdElement,
    LaneCount<LANES>: SupportedLaneCount,
{
}

impl<T, const LANES: usize> Clone for Simd<T, LANES>
where
    T: SimdElement,
    LaneCount<LANES>: SupportedLaneCount,
{
    fn clone(&self) -> Self {
        *self
    }
}

impl<T, const LANES: usize> Default for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + Default,
{
    #[inline]
    fn default() -> Self {
        Self::splat(T::default())
    }
}

impl<T, const LANES: usize> PartialEq for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + PartialEq,
{
    #[inline]
    fn eq(&self, other: &Self) -> bool {
        // TODO use SIMD equality <br>使用 SIMD 等式<br>
        self.to_array() == other.to_array()
    }
}

impl<T, const LANES: usize> PartialOrd for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + PartialOrd,
{
    #[inline]
    fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
        // TODO use SIMD equality <br>使用 SIMD 等式<br>
        self.to_array().partial_cmp(other.as_ref())
    }
}

impl<T, const LANES: usize> Eq for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + Eq,
{
}

impl<T, const LANES: usize> Ord for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + Ord,
{
    #[inline]
    fn cmp(&self, other: &Self) -> core::cmp::Ordering {
        // TODO use SIMD equality <br>使用 SIMD 等式<br>
        self.to_array().cmp(other.as_ref())
    }
}

impl<T, const LANES: usize> core::hash::Hash for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement + core::hash::Hash,
{
    #[inline]
    fn hash<H>(&self, state: &mut H)
    where
        H: core::hash::Hasher,
    {
        self.as_array().hash(state)
    }
}

// array references <br>数组引用<br>
impl<T, const LANES: usize> AsRef<[T; LANES]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    #[inline]
    fn as_ref(&self) -> &[T; LANES] {
        &self.0
    }
}

impl<T, const LANES: usize> AsMut<[T; LANES]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    #[inline]
    fn as_mut(&mut self) -> &mut [T; LANES] {
        &mut self.0
    }
}

// slice references <br>切片引用<br>
impl<T, const LANES: usize> AsRef<[T]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    #[inline]
    fn as_ref(&self) -> &[T] {
        &self.0
    }
}

impl<T, const LANES: usize> AsMut<[T]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    #[inline]
    fn as_mut(&mut self) -> &mut [T] {
        &mut self.0
    }
}

// vector/array conversion <br>vector 和 array 的转换<br>
impl<T, const LANES: usize> From<[T; LANES]> for Simd<T, LANES>
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    fn from(array: [T; LANES]) -> Self {
        Self(array)
    }
}

impl<T, const LANES: usize> From<Simd<T, LANES>> for [T; LANES]
where
    LaneCount<LANES>: SupportedLaneCount,
    T: SimdElement,
{
    fn from(vector: Simd<T, LANES>) -> Self {
        vector.to_array()
    }
}

mod sealed {
    pub trait Sealed {}
}
use sealed::Sealed;

/// Marker trait for types that may be used as SIMD vector elements. <br>可用作 SIMD vector 元素的类型的标记 trait。<br>
///
/// # Safety
/// This trait, when implemented, asserts the compiler can monomorphize `#[repr(simd)]` structs with the marked type as an element. <br>这个 trait 在实现时断言编译器可以将标记类型作为元素的 `#[repr(simd)]` 结构体单态化。<br>
/// Strictly, it is valid to impl if the vector will not be miscompiled. <br>严格地说，如果 vector 不会被错误编译，则 impl 是有效的。<br>
/// Practically, it is user-unfriendly to impl it if the vector won't compile, even when no soundness guarantees are broken by allowing the user to try. <br>实际上，如果 vector 无法编译，那么实现它对用户是不友好的，即使允许用户尝试并没有破坏健全性保证。<br>
///
///
pub unsafe trait SimdElement: Sealed + Copy {
    /// The mask element type corresponding to this element type. <br>此元素类型对应的掩码元素类型。<br>
    type Mask: MaskElement;
}

impl Sealed for u8 {}
unsafe impl SimdElement for u8 {
    type Mask = i8;
}

impl Sealed for u16 {}
unsafe impl SimdElement for u16 {
    type Mask = i16;
}

impl Sealed for u32 {}
unsafe impl SimdElement for u32 {
    type Mask = i32;
}

impl Sealed for u64 {}
unsafe impl SimdElement for u64 {
    type Mask = i64;
}

impl Sealed for usize {}
unsafe impl SimdElement for usize {
    type Mask = isize;
}

impl Sealed for i8 {}
unsafe impl SimdElement for i8 {
    type Mask = i8;
}

impl Sealed for i16 {}
unsafe impl SimdElement for i16 {
    type Mask = i16;
}

impl Sealed for i32 {}
unsafe impl SimdElement for i32 {
    type Mask = i32;
}

impl Sealed for i64 {}
unsafe impl SimdElement for i64 {
    type Mask = i64;
}

impl Sealed for isize {}
unsafe impl SimdElement for isize {
    type Mask = isize;
}

impl Sealed for f32 {}
unsafe impl SimdElement for f32 {
    type Mask = i32;
}

impl Sealed for f64 {}
unsafe impl SimdElement for f64 {
    type Mask = i64;
}
