#![allow(non_camel_case_types)]

use crate::simd::intrinsics;
use crate::simd::{LaneCount, Mask, Simd, SupportedLaneCount};

/// 为包含浮点 `$type` 的多个 `$lanes` 的浮点 vector 实现固有方法，该浮点使用 `$bits_ty` 作为它的二进制表示。
///
///
macro_rules! impl_float_vector {
    { $type:ty, $bits_ty:ty, $mask_ty:ty } => {
        impl<const LANES: usize> Simd<$type, LANES>
        where
            LaneCount<LANES>: SupportedLaneCount,
        {
            /// 原始转换为无符号整数 vector 类型，其大小和数量与 lanes 相同。
            ///
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn to_bits(self) -> Simd<$bits_ty, LANES> {
                assert_eq!(core::mem::size_of::<Self>(), core::mem::size_of::<Simd<$bits_ty, LANES>>());
                unsafe { core::mem::transmute_copy(&self) }
            }

            /// 来自具有相同大小和数量的 lanes 的无符号整数 vector 类型的原始变换。
            ///
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn from_bits(bits: Simd<$bits_ty, LANES>) -> Self {
                assert_eq!(core::mem::size_of::<Self>(), core::mem::size_of::<Simd<$bits_ty, LANES>>());
                unsafe { core::mem::transmute_copy(&bits) }
            }

            /// 生成一个 vector，其中每个 lane 都具有 `self` 中等效索引 lane 的绝对值。
            ///
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn abs(self) -> Self {
                unsafe { intrinsics::simd_fabs(self) }
            }

            /// 获取每个 lane 的倒数 (inverse)，`1/x`。
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn recip(self) -> Self {
                Self::splat(1.0) / self
            }

            /// 将每个 lane 从弧度转换为度数。
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn to_degrees(self) -> Self {
                // to_degrees 使用一个特殊的常量来获得更好的精度，所以提取这个常量
                self * Self::splat(<$type>::to_degrees(1.))
            }

            /// 将每个 lane 从度数转换为弧度。
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn to_radians(self) -> Self {
                self * Self::splat(<$type>::to_radians(1.))
            }

            /// 如果每个 lane 具有正号，则为每个 lane 返回 true，包括 `+0.0`、具有正符号位的 `NaN` 和正无穷大。
            ///
            #[inline]
            #[must_use = "method returns a new mask and does not mutate the original value"]
            pub fn is_sign_positive(self) -> Mask<$mask_ty, LANES> {
                !self.is_sign_negative()
            }

            /// 如果每个 lane 具有负号，则为每个 lane 返回 true，包括 `-0.0`、具有负符号位的 `NaN` 和负无穷大。
            ///
            #[inline]
            #[must_use = "method returns a new mask and does not mutate the original value"]
            pub fn is_sign_negative(self) -> Mask<$mask_ty, LANES> {
                let sign_bits = self.to_bits() & Simd::splat((!0 >> 1) + 1);
                sign_bits.lanes_gt(Simd::splat(0))
            }

            /// 如果每个 lane 的值为 `NaN`，则为每个 lane 返回 true。
            #[inline]
            #[must_use = "method returns a new mask and does not mutate the original value"]
            pub fn is_nan(self) -> Mask<$mask_ty, LANES> {
                self.lanes_ne(self)
            }

            /// 如果每个 lane 的值是正无穷大或负无穷大，则为每个 lane 返回 true。
            #[inline]
            #[must_use = "method returns a new mask and does not mutate the original value"]
            pub fn is_infinite(self) -> Mask<$mask_ty, LANES> {
                self.abs().lanes_eq(Self::splat(<$type>::INFINITY))
            }

            /// 如果每个 lane 的值既不是无穷大也不是 `NaN`，则为每个 lane 返回 true。
            #[inline]
            #[must_use = "method returns a new mask and does not mutate the original value"]
            pub fn is_finite(self) -> Mask<$mask_ty, LANES> {
                self.abs().lanes_lt(Self::splat(<$type>::INFINITY))
            }

            /// 如果每个 lane 的值为 subnormal，则为每个 lane 返回 true。
            #[inline]
            #[must_use = "method returns a new mask and does not mutate the original value"]
            pub fn is_subnormal(self) -> Mask<$mask_ty, LANES> {
                self.abs().lanes_ne(Self::splat(0.0)) & (self.to_bits() & Self::splat(<$type>::INFINITY).to_bits()).lanes_eq(Simd::splat(0))
            }

            /// 如果 lane 的值既不是零、无限、subnormal 也不是 `NaN`，则为每个 lane 返回 true。
            ///
            #[inline]
            #[must_use = "method returns a new mask and does not mutate the original value"]
            pub fn is_normal(self) -> Mask<$mask_ty, LANES> {
                !(self.abs().lanes_eq(Self::splat(0.0)) | self.is_nan() | self.is_subnormal() | self.is_infinite())
            }

            /// 用代表其符号的数字替换每个 lane。
            ///
            /// * `1.0` 如果数字是正数，`+0.0` 或 `INFINITY`
            /// * `-1.0` 如果数字是负数，`-0.0` 或 `NEG_INFINITY`
            /// * `NAN` 如果数字是 `NAN`
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn signum(self) -> Self {
                self.is_nan().select(Self::splat(<$type>::NAN), Self::splat(1.0).copysign(self))
            }

            /// 以 `self` 的大小和 `sign` 的符号返回每个 lane。
            ///
            /// 如果任何 lane 是 `NAN`，则返回带有 `sign` 符号的 `NAN`。
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn copysign(self, sign: Self) -> Self {
                let sign_bit = sign.to_bits() & Self::splat(-0.).to_bits();
                let magnitude = self.to_bits() & !Self::splat(-0.).to_bits();
                Self::from_bits(sign_bit | magnitude)
            }

            /// 返回每个 lane 的最小值。
            ///
            /// 如果其中一个值为 `NAN`，则返回另一个值。
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn min(self, other: Self) -> Self {
                unsafe { intrinsics::simd_fmin(self, other) }
            }

            /// 返回每个 lane 的最大值。
            ///
            /// 如果其中一个值为 `NAN`，则返回另一个值。
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn max(self, other: Self) -> Self {
                unsafe { intrinsics::simd_fmax(self, other) }
            }

            /// 将每个 lane 限制在一定的区间内，除非它是 NaN。
            ///
            /// 对于 `self` 中的每个 lane，如果 lane 大于 `max`，则返回 `max` 中对应的 lane，如果 lane 小于 `min`，则返回 `min` 中对应的 lane。
            /// 否则返回 `self` 中的 lane。
            ///
            #[inline]
            #[must_use = "method returns a new vector and does not mutate the original value"]
            pub fn clamp(self, min: Self, max: Self) -> Self {
                assert!(
                    min.lanes_le(max).all(),
                    "each lane in `min` must be less than or equal to the corresponding lane in `max`",
                );
                let mut x = self;
                x = x.lanes_lt(min).select(min, x);
                x = x.lanes_gt(max).select(max, x);
                x
            }
        }
    };
}

impl_float_vector! { f32, u32, i32 }
impl_float_vector! { f64, u64, i64 }

/// 两个 `f32` 值的 Vector
pub type f32x2 = Simd<f32, 2>;

/// 四个 `f32` 值的 Vector
pub type f32x4 = Simd<f32, 4>;

/// 八个 `f32` 值的 Vector
pub type f32x8 = Simd<f32, 8>;

/// 16 个 `f32` 值的 Vector
pub type f32x16 = Simd<f32, 16>;

/// 两个 `f64` 值的 Vector
pub type f64x2 = Simd<f64, 2>;

/// 四个 `f64` 值的 Vector
pub type f64x4 = Simd<f64, 4>;

/// 八个 `f64` 值的 Vector
pub type f64x8 = Simd<f64, 8>;
