#![allow(non_camel_case_types)]

use crate::simd::{LaneCount, Mask, Simd, SupportedLaneCount};

/// 在指定的 vector `$name` 上实现额外的整数 traits (Eq、Ord、Hash)，持有 `$type` 的多个 `$lanes`。
macro_rules! impl_integer_vector {
    { $type:ty } => {
        impl<const LANES: usize> Simd<$type, LANES>
        where
            LaneCount<LANES>: SupportedLaneCount,
        {
            /// 对于每个正 lane 返回真，如果为零或负则返回假。
            #[inline]
            pub fn is_positive(self) -> Mask<$type, LANES> {
                self.lanes_gt(Self::splat(0))
            }

            /// 对于每个负 lane 返回 true，如果为零或正则返回 false。
            #[inline]
            pub fn is_negative(self) -> Mask<$type, LANES> {
                self.lanes_lt(Self::splat(0))
            }

            /// 返回代表每个 lane 符号的数字。
            /// * `0` 如果数字为零
            /// * `1` 如果数字是正数
            /// * `-1` 如果数字是负数
            #[inline]
            pub fn signum(self) -> Self {
                self.is_positive().select(
                    Self::splat(1),
                    self.is_negative().select(Self::splat(-1), Self::splat(0))
                )
            }
        }
    }
}

impl_integer_vector! { isize }
impl_integer_vector! { i16 }
impl_integer_vector! { i32 }
impl_integer_vector! { i64 }
impl_integer_vector! { i8 }

/// 两个 `isize` 值的 Vector
pub type isizex2 = Simd<isize, 2>;

/// 四个 `isize` 值的 Vector
pub type isizex4 = Simd<isize, 4>;

/// 八个 `isize` 值的 Vector
pub type isizex8 = Simd<isize, 8>;

/// 两个 `i16` 值的 Vector
pub type i16x2 = Simd<i16, 2>;

/// 四个 `i16` 值的 Vector
pub type i16x4 = Simd<i16, 4>;

/// 八个 `i16` 值的 Vector
pub type i16x8 = Simd<i16, 8>;

/// 16 个 `i16` 值的 Vector
pub type i16x16 = Simd<i16, 16>;

/// 32 个 `i16` 值的 Vector
pub type i16x32 = Simd<i16, 32>;

/// 两个 `i32` 值的 Vector
pub type i32x2 = Simd<i32, 2>;

/// 四个 `i32` 值的 Vector
pub type i32x4 = Simd<i32, 4>;

/// 八个 `i32` 值的 Vector
pub type i32x8 = Simd<i32, 8>;

/// 16 个 `i32` 值的 Vector
pub type i32x16 = Simd<i32, 16>;

/// 两个 `i64` 值的 Vector
pub type i64x2 = Simd<i64, 2>;

/// 四个 `i64` 值的 Vector
pub type i64x4 = Simd<i64, 4>;

/// 八个 `i64` 值的 Vector
pub type i64x8 = Simd<i64, 8>;

/// 四个 `i8` 值的 Vector
pub type i8x4 = Simd<i8, 4>;

/// 八个 `i8` 值的 Vector
pub type i8x8 = Simd<i8, 8>;

/// 16 个 `i8` 值的 Vector
pub type i8x16 = Simd<i8, 16>;

/// 32 个 `i8` 值的 Vector
pub type i8x32 = Simd<i8, 32>;

/// 64 个 `i8` 值的 Vector
pub type i8x64 = Simd<i8, 64>;
