//! Abstraction for creating `fn` pointers from any callable that *effectively* has the equivalent of implementing `Default`, even if the compiler neither provides `Default` nor allows reifying closures (i.e. creating `fn` pointers) other than those with absolutely no captures. <br>即使编译器既不提供 `Default` 也不允许具体化闭包 (即创建 `fn` 指针)，但绝对没有捕获的任何可调用对象创建 `fn` 指针。<br>
//!
//! More specifically, for a closure-like type to be "effectively `Default`": <br>更具体地说，对于类似于闭包的类型是 "effectively `Default`":<br>
//! * it must be a ZST (zero-sized type): no information contained within, so that `Default`'s return value (if it were implemented) is unambiguous <br>它必须是 ZST (零大小类型) : 其中不包含任何信息，因此 `Default` 的返回值 (如果已实现) 是明确的<br>
//! * it must be `Copy`: no captured "unique ZST tokens" or any other similar types that would make duplicating values at will unsound <br>它必须是 `Copy`: 没有捕获的 "unique ZST tokens" 或任何其他会使随意重复值不合理的类似类型<br>
//!   * combined with the ZST requirement, this confers a kind of "telecopy" ability: similar to `Copy`, but without keeping the value around, and instead "reconstructing" it (a noop given it's a ZST) when needed <br>结合 ZST 要求，这赋予了一种 "telecopy" 能力: 类似于 `Copy`，但不保留周围的值，而是在需要时 "reconstructing" 它 (因为它是 ZST 的 noop)<br>
//! * it must be *provably* inhabited: no captured uninhabited types or any other types that cannot be constructed by the user of this abstraction <br>它必须*可证明*有人居住: 没有捕获的无人居住类型或任何其他无法由此抽象的用户构造的类型<br>
//!   * the proof is a value of the closure-like type itself, in a sense the "seed" for the "telecopy" process made possible by ZST + `Copy` <br>证明是类似闭包的类型本身的值，从某种意义上说，ZST + `Copy` 使 "telecopy" 过程的 "seed" 成为可能<br>
//!   * this requirement is the only reason an abstraction limited to a specific usecase is required: ZST + `Copy` can be checked with *at worst* a panic at the "attempted `::default()` call" time, but that doesn't guarantee that the value can be soundly created, and attempting to use the typical "proof ZST token" approach leads yet again to having a ZST + `Copy` type that is not proof of anything without a value (i.e. <br>此要求是需要限制为特定用例的抽象的唯一原因: ZST + `Copy` 可以在 "attempted `::default()` call" 时间用 *atmost* a panic 进行检查，但这并不能保证可以正确创建该值，并尝试使用典型的 "proof ZST token" 方法再次导致 ZST + `Copy` 类型不能证明任何没有值的东西 (即<br>
//!   isomorphic to a newtype of the type it's trying to prove the inhabitation of) <br>与它试图证明居住的类型的 newtype 同构)<br>
//!
//! A more flexible (and safer) solution to the general problem could exist once `const`-generic parameters can have type parameters in their types: <br>一旦 `const`- 泛型参数可以在其类型中具有类型参数，则可能存在针对一般问题的更灵活 (和更安全) 的解决方案:<br>
//!
//! ```rust,ignore (needs future const-generics)
//! extern "C" fn ffi_wrapper<
//!     A, R,
//!     F: Fn(A) -> R,
//!     const f: F, // <-- this `const`-generic is not yet allowed <br>这个 `const` 泛型还不允许<br>
//! >(arg: A) -> R {
//!     f(arg)
//! }
//! ```
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

use std::mem;

// FIXME(eddyb) this could be `trait` impls except for the `const fn` requirement. <br>除了 `const fn` 要求外，这可能是 `trait` impls。<br>
macro_rules! define_reify_functions {
    ($(
        fn $name:ident $(<$($param:ident),*>)?
            for $(extern $abi:tt)? fn($($arg:ident: $arg_ty:ty),*) -> $ret_ty:ty;
    )+) => {
        $(pub const fn $name<
            $($($param,)*)?
            F: Fn($($arg_ty),*) -> $ret_ty + Copy
        >(f: F) -> $(extern $abi)? fn($($arg_ty),*) -> $ret_ty {
            // FIXME(eddyb) describe the `F` type (e.g. <br>描述 `F` 类型 (例如<br>
            // via `type_name::<F>`) once panic formatting becomes possible in `const fn`. <br>通过 `type_name::<F>`) 一旦在 `const fn` 中 panic 格式化成为可能。<br>
            assert!(mem::size_of::<F>() == 0, "selfless_reify: closure must be zero-sized");

            $(extern $abi)? fn wrapper<
                $($($param,)*)?
                F: Fn($($arg_ty),*) -> $ret_ty + Copy
            >($($arg: $arg_ty),*) -> $ret_ty {
                let f = unsafe {
                    // SAFETY: `F` satisfies all criteria for "out of thin air" reconstructability (see module-level doc comment). <br>`F` 满足 "out of thin air" 可重构性的所有标准 (请参见模块级文档注释)。<br>
                    //
                    mem::MaybeUninit::<F>::uninit().assume_init()
                };
                f($($arg),*)
            }
            let _f_proof = f;
            wrapper::<
                $($($param,)*)?
                F
            >
        })+
    }
}

define_reify_functions! {
    fn _reify_to_extern_c_fn_unary<A, R> for extern "C" fn(arg: A) -> R;

    // HACK(eddyb) this abstraction is used with `for<'a> fn(Bridge<'a>) -> T` but that doesn't work with just `reify_to_extern_c_fn_unary` because of the `fn` pointer type being "higher-ranked" (i.e. <br>此抽象与 `for<'a> fn(Bridge<'a>) -> T` 一起使用，但不适用于仅 `reify_to_extern_c_fn_unary`，因为 `fn` 指针类型为 "higher-ranked" (即<br>
    // the `for<'a>` binder).
    // FIXME(eddyb) try to remove the lifetime from `Bridge`, that'd help. <br>尝试从 `Bridge` 中删除生命周期，这会有所帮助。<br>
    //
    fn reify_to_extern_c_fn_hrt_bridge<R> for extern "C" fn(bridge: super::Bridge<'_>) -> R;
}
