//! A support library for macro authors when defining new macros. <br>定义新宏时为宏作者提供的支持库。<br>
//!
//! This library, provided by the standard distribution, provides the types consumed in the interfaces of procedurally defined macro definitions such as function-like macros `#[proc_macro]`, macro attributes `#[proc_macro_attribute]` and custom derive attributes`#[proc_macro_derive]`. <br>由标准发行版提供的该库提供了在过程定义的宏定义的接口中使用的类型，例如，类似函数的宏 `#[proc_macro]`，宏属性 `#[proc_macro_attribute]` 和自定义派生属性 `#[proc_macro_derive]`。<br>
//!
//!
//! See [the book] for more. <br>有关更多信息，请参见 [这本书][the book]。<br>
//!
//! [the book]: ../book/ch19-06-macros.html#procedural-macros-for-generating-code-from-attributes
//!
//!

#![stable(feature = "proc_macro_lib", since = "1.15.0")]
#![deny(missing_docs)]
#![doc(
    html_playground_url = "https://play.rust-lang.org/",
    issue_tracker_base_url = "https://github.com/rust-lang/rust/issues/",
    test(no_crate_inject, attr(deny(warnings))),
    test(attr(allow(dead_code, deprecated, unused_variables, unused_mut)))
)]
// This library is copied into rust-analyzer to allow loading rustc compiled proc macros. <br>这个库被复制到 rust-analyzer 以允许加载 rustc 编译的 proc 宏。<br>
// Please avoid unstable features where possible to minimize the amount of changes necessary to make it compile with rust-analyzer on stable. <br>请尽可能避免使用 rust-anazer on stable 编译所需的不稳定特性。<br>
//
#![feature(rustc_allow_const_fn_unstable)]
#![feature(staged_api)]
#![feature(allow_internal_unstable)]
#![feature(decl_macro)]
#![feature(negative_impls)]
#![feature(restricted_std)]
#![feature(rustc_attrs)]
#![feature(min_specialization)]
#![recursion_limit = "256"]

#[unstable(feature = "proc_macro_internals", issue = "27812")]
#[doc(hidden)]
pub mod bridge;

mod diagnostic;

#[unstable(feature = "proc_macro_diagnostic", issue = "54140")]
pub use diagnostic::{Diagnostic, Level, MultiSpan};

use std::cmp::Ordering;
use std::ops::RangeBounds;
use std::path::PathBuf;
use std::str::FromStr;
use std::{error, fmt, iter};

/// Determines whether proc_macro has been made accessible to the currently running program. <br>确定是否已使 proc_macro 可被当前正在运行的程序访问。<br>
///
/// The proc_macro crate is only intended for use inside the implementation of procedural macros. <br>proc_macro crate 仅用于在过程宏的实现内部使用。<br> All the functions in this crate panic if invoked from outside of a procedural macro, such as from a build script or unit test or ordinary Rust binary. <br>crate panic 中的所有函数 (如果从程序宏外部调用，例如从构建脚本或单元测试或常规 Rust 二进制文件调用)。<br>
///
/// With consideration for Rust libraries that are designed to support both macro and non-macro use cases, `proc_macro::is_available()` provides a non-panicking way to detect whether the infrastructure required to use the API of proc_macro is presently available. <br>考虑到旨在支持宏和非宏用例的 Rust 库，`proc_macro::is_available()` 提供了一种非 panic 的方式来检测使用 proc_macro API 所需的基础结构是否当前可用。<br>
/// Returns true if invoked from inside of a procedural macro, false if invoked from any other binary. <br>如果从程序宏内部调用，则返回 true; 如果从任何其他二进制文件中调用，则返回 false。<br>
///
///
///
///
///
///
///
#[stable(feature = "proc_macro_is_available", since = "1.57.0")]
pub fn is_available() -> bool {
    bridge::Bridge::is_available()
}

/// The main type provided by this crate, representing an abstract stream of tokens, or, more specifically, a sequence of token trees. <br>此 crate 提供的主要类型，表示 tokens 的抽象流，或更具体地说，表示 token 树的序列。<br>
/// The type provide interfaces for iterating over those token trees and, conversely, collecting a number of token trees into one stream. <br>该类型提供用于迭代这些 token 树的接口，并且相反，将大量 token 树收集到一个流中。<br>
///
///
/// This is both the input and output of `#[proc_macro]`, `#[proc_macro_attribute]` and `#[proc_macro_derive]` definitions. <br>这是 `#[proc_macro]`，`#[proc_macro_attribute]` 和 `#[proc_macro_derive]` 定义的输入和输出。<br>
///
///
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
#[derive(Clone)]
pub struct TokenStream(Option<bridge::client::TokenStream>);

#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl !Send for TokenStream {}
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl !Sync for TokenStream {}

/// Error returned from `TokenStream::from_str`. <br>`TokenStream::from_str` 返回错误。<br>
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
#[non_exhaustive]
#[derive(Debug)]
pub struct LexError;

#[stable(feature = "proc_macro_lexerror_impls", since = "1.44.0")]
impl fmt::Display for LexError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str("cannot parse string into token stream")
    }
}

#[stable(feature = "proc_macro_lexerror_impls", since = "1.44.0")]
impl error::Error for LexError {}

#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl !Send for LexError {}
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl !Sync for LexError {}

/// Error returned from `TokenStream::expand_expr`. <br>从 `TokenStream::expand_expr` 返回的错误。<br>
#[unstable(feature = "proc_macro_expand", issue = "90765")]
#[non_exhaustive]
#[derive(Debug)]
pub struct ExpandError;

#[unstable(feature = "proc_macro_expand", issue = "90765")]
impl fmt::Display for ExpandError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str("macro expansion failed")
    }
}

#[unstable(feature = "proc_macro_expand", issue = "90765")]
impl error::Error for ExpandError {}

#[unstable(feature = "proc_macro_expand", issue = "90765")]
impl !Send for ExpandError {}

#[unstable(feature = "proc_macro_expand", issue = "90765")]
impl !Sync for ExpandError {}

impl TokenStream {
    /// Returns an empty `TokenStream` containing no token trees. <br>返回不包含 token 树的空 `TokenStream`。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn new() -> TokenStream {
        TokenStream(None)
    }

    /// Checks if this `TokenStream` is empty. <br>检查此 `TokenStream` 是否为空。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn is_empty(&self) -> bool {
        self.0.as_ref().map(|h| h.is_empty()).unwrap_or(true)
    }

    /// Parses this `TokenStream` as an expression and attempts to expand any macros within it. <br>将此 `TokenStream` 解析为表达式并尝试扩展其中的任何宏。<br> Returns the expanded `TokenStream`. <br>返回扩展的 `TokenStream`。<br>
    ///
    /// Currently only expressions expanding to literals will succeed, although this may be relaxed in the future. <br>目前只有扩展到字面量的表达式会成功，尽管这在 future 中可能会有所放松。<br>
    ///
    /// NOTE: In error conditions, `expand_expr` may leave macros unexpanded, report an error, failing compilation, and/or return an `Err(..)`. <br>在错误情况下，`expand_expr` 可能会保留未扩展的宏、报告错误、编译失败或者返回 `Err(..)`。<br>
    /// The specific behavior for any error condition, and what conditions are considered errors, is unspecified and may change in the future. <br>任何错误条件的具体行为，以及哪些条件被视为错误，是未指定的，可能会在 future 中发生变化。<br>
    ///
    ///
    ///
    ///
    #[unstable(feature = "proc_macro_expand", issue = "90765")]
    pub fn expand_expr(&self) -> Result<TokenStream, ExpandError> {
        let stream = self.0.as_ref().ok_or(ExpandError)?;
        match bridge::client::TokenStream::expand_expr(stream) {
            Ok(stream) => Ok(TokenStream(Some(stream))),
            Err(_) => Err(ExpandError),
        }
    }
}

/// Attempts to break the string into tokens and parse those tokens into a token stream. <br>尝试将字符串拆分为 tokens 并将那些 tokens 解析为 token 流。<br>
/// May fail for a number of reasons, for example, if the string contains unbalanced delimiters or characters not existing in the language. <br>可能由于多种原因而失败，例如，如果字符串包含不平衡的定界符或该语言中不存在的字符。<br>
///
/// All tokens in the parsed stream get `Span::call_site()` spans. <br>解析的流中的所有 tokens 都获得 `Span::call_site()` 跨度。<br>
///
/// NOTE: some errors may cause panics instead of returning `LexError`. <br>某些错误可能导致 panics 而不是返回 `LexError`。<br> We reserve the right to change these errors into `LexError`s later. <br>我们保留稍后将这些错误更改为 LexError 的权利。<br>
///
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl FromStr for TokenStream {
    type Err = LexError;

    fn from_str(src: &str) -> Result<TokenStream, LexError> {
        Ok(TokenStream(Some(bridge::client::TokenStream::from_str(src))))
    }
}

// N.B., the bridge only provides `to_string`, implement `fmt::Display` based on it (the reverse of the usual relationship between the two). <br>注意，网桥仅提供 `to_string`，基于它实现 `fmt::Display` (两者之间通常的关系相反)。<br>
//
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl ToString for TokenStream {
    fn to_string(&self) -> String {
        self.0.as_ref().map(|t| t.to_string()).unwrap_or_default()
    }
}

/// Prints the token stream as a string that is supposed to be losslessly convertible back into the same token stream (modulo spans), except for possibly `TokenTree::Group`s with `Delimiter::None` delimiters and negative numeric literals. <br>将 token 流打印为应该无损转换为相同 token 流 (模数跨度) 的字符串，但可能带有 `Delimiter::None` 分隔符和负数字字面量的 `TokenTree::Group` 除外。<br>
///
///
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl fmt::Display for TokenStream {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str(&self.to_string())
    }
}

/// Prints token in a form convenient for debugging. <br>以方便调试的形式打印 token。<br>
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl fmt::Debug for TokenStream {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str("TokenStream ")?;
        f.debug_list().entries(self.clone()).finish()
    }
}

#[stable(feature = "proc_macro_token_stream_default", since = "1.45.0")]
impl Default for TokenStream {
    fn default() -> Self {
        TokenStream::new()
    }
}

#[unstable(feature = "proc_macro_quote", issue = "54722")]
pub use quote::{quote, quote_span};

fn tree_to_bridge_tree(
    tree: TokenTree,
) -> bridge::TokenTree<
    bridge::client::Group,
    bridge::client::Punct,
    bridge::client::Ident,
    bridge::client::Literal,
> {
    match tree {
        TokenTree::Group(tt) => bridge::TokenTree::Group(tt.0),
        TokenTree::Punct(tt) => bridge::TokenTree::Punct(tt.0),
        TokenTree::Ident(tt) => bridge::TokenTree::Ident(tt.0),
        TokenTree::Literal(tt) => bridge::TokenTree::Literal(tt.0),
    }
}

/// Creates a token stream containing a single token tree. <br>创建包含单个 token 树的 token 流。<br>
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl From<TokenTree> for TokenStream {
    fn from(tree: TokenTree) -> TokenStream {
        TokenStream(Some(bridge::client::TokenStream::from_token_tree(tree_to_bridge_tree(tree))))
    }
}

/// Non-generic helper for implementing `FromIterator<TokenTree>` and `Extend<TokenTree>` with less monomorphization in calling crates. <br>非泛型助手，用于实现 `FromIterator<TokenTree>` 和 `Extend<TokenTree>`，在调用 crates 时具有较少的单态化。<br>
///
struct ConcatTreesHelper {
    trees: Vec<
        bridge::TokenTree<
            bridge::client::Group,
            bridge::client::Punct,
            bridge::client::Ident,
            bridge::client::Literal,
        >,
    >,
}

impl ConcatTreesHelper {
    fn new(capacity: usize) -> Self {
        ConcatTreesHelper { trees: Vec::with_capacity(capacity) }
    }

    fn push(&mut self, tree: TokenTree) {
        self.trees.push(tree_to_bridge_tree(tree));
    }

    fn build(self) -> TokenStream {
        if self.trees.is_empty() {
            TokenStream(None)
        } else {
            TokenStream(Some(bridge::client::TokenStream::concat_trees(None, self.trees)))
        }
    }

    fn append_to(self, stream: &mut TokenStream) {
        if self.trees.is_empty() {
            return;
        }
        stream.0 = Some(bridge::client::TokenStream::concat_trees(stream.0.take(), self.trees))
    }
}

/// Non-generic helper for implementing `FromIterator<TokenStream>` and `Extend<TokenStream>` with less monomorphization in calling crates. <br>非泛型助手，用于实现 `FromIterator<TokenStream>` 和 `Extend<TokenStream>`，在调用 crates 时具有较少的单态化。<br>
///
struct ConcatStreamsHelper {
    streams: Vec<bridge::client::TokenStream>,
}

impl ConcatStreamsHelper {
    fn new(capacity: usize) -> Self {
        ConcatStreamsHelper { streams: Vec::with_capacity(capacity) }
    }

    fn push(&mut self, stream: TokenStream) {
        if let Some(stream) = stream.0 {
            self.streams.push(stream);
        }
    }

    fn build(mut self) -> TokenStream {
        if self.streams.len() <= 1 {
            TokenStream(self.streams.pop())
        } else {
            TokenStream(Some(bridge::client::TokenStream::concat_streams(None, self.streams)))
        }
    }

    fn append_to(mut self, stream: &mut TokenStream) {
        if self.streams.is_empty() {
            return;
        }
        let base = stream.0.take();
        if base.is_none() && self.streams.len() == 1 {
            stream.0 = self.streams.pop();
        } else {
            stream.0 = Some(bridge::client::TokenStream::concat_streams(base, self.streams));
        }
    }
}

/// Collects a number of token trees into a single stream. <br>将多个 token 树收集到单个流中。<br>
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl iter::FromIterator<TokenTree> for TokenStream {
    fn from_iter<I: IntoIterator<Item = TokenTree>>(trees: I) -> Self {
        let iter = trees.into_iter();
        let mut builder = ConcatTreesHelper::new(iter.size_hint().0);
        iter.for_each(|tree| builder.push(tree));
        builder.build()
    }
}

/// A "flattening" operation on token streams, collects token trees from multiple token streams into a single stream. <br>对 token 流的 "flattening" 操作将来自多个 token 流的 token 树收集到单个流中。<br>
///
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl iter::FromIterator<TokenStream> for TokenStream {
    fn from_iter<I: IntoIterator<Item = TokenStream>>(streams: I) -> Self {
        let iter = streams.into_iter();
        let mut builder = ConcatStreamsHelper::new(iter.size_hint().0);
        iter.for_each(|stream| builder.push(stream));
        builder.build()
    }
}

#[stable(feature = "token_stream_extend", since = "1.30.0")]
impl Extend<TokenTree> for TokenStream {
    fn extend<I: IntoIterator<Item = TokenTree>>(&mut self, trees: I) {
        let iter = trees.into_iter();
        let mut builder = ConcatTreesHelper::new(iter.size_hint().0);
        iter.for_each(|tree| builder.push(tree));
        builder.append_to(self);
    }
}

#[stable(feature = "token_stream_extend", since = "1.30.0")]
impl Extend<TokenStream> for TokenStream {
    fn extend<I: IntoIterator<Item = TokenStream>>(&mut self, streams: I) {
        let iter = streams.into_iter();
        let mut builder = ConcatStreamsHelper::new(iter.size_hint().0);
        iter.for_each(|stream| builder.push(stream));
        builder.append_to(self);
    }
}

/// Public implementation details for the `TokenStream` type, such as iterators. <br>`TokenStream` 类型的公共实现详细信息，例如迭代器。<br>
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
pub mod token_stream {
    use crate::{bridge, Group, Ident, Literal, Punct, TokenStream, TokenTree};

    /// An iterator over `TokenStream`'s `TokenTree`s. <br>遍历 TokenStream 的 TokenTree 的迭代器。<br>
    /// The iteration is "shallow", e.g., the iterator doesn't recurse into delimited groups, and returns whole groups as token trees. <br>迭代是 "shallow"，例如，迭代器不会递归到定界的组中，而是将整个组作为 token 树返回。<br>
    ///
    #[derive(Clone)]
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub struct IntoIter(
        std::vec::IntoIter<
            bridge::TokenTree<
                bridge::client::Group,
                bridge::client::Punct,
                bridge::client::Ident,
                bridge::client::Literal,
            >,
        >,
    );

    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    impl Iterator for IntoIter {
        type Item = TokenTree;

        fn next(&mut self) -> Option<TokenTree> {
            self.0.next().map(|tree| match tree {
                bridge::TokenTree::Group(tt) => TokenTree::Group(Group(tt)),
                bridge::TokenTree::Punct(tt) => TokenTree::Punct(Punct(tt)),
                bridge::TokenTree::Ident(tt) => TokenTree::Ident(Ident(tt)),
                bridge::TokenTree::Literal(tt) => TokenTree::Literal(Literal(tt)),
            })
        }
    }

    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    impl IntoIterator for TokenStream {
        type Item = TokenTree;
        type IntoIter = IntoIter;

        fn into_iter(self) -> IntoIter {
            IntoIter(self.0.map(|v| v.into_trees()).unwrap_or_default().into_iter())
        }
    }
}

/// `quote!(..)` accepts arbitrary tokens and expands into a `TokenStream` describing the input. <br>`quote!(..)` 接受任意的 tokens 并扩展为描述输入的 `TokenStream`。<br>
/// For example, `quote!(a + b)` will produce an expression, that, when evaluated, constructs the `TokenStream` `[Ident("a"), Punct('+', Alone), Ident("b")]`. <br>例如，`quote!(a + b)` 将产生一个表达式，当求值时，它构造 `TokenStream` `[Ident("a"), Punct('+', Alone), Ident("b")]`。<br>
///
///
/// Unquoting is done with `$`, and works by taking the single next ident as the unquoted term. <br>用 `$` 解引用，并通过将单个下一个标识符作为未引用的术语来工作。<br>
/// To quote `$` itself, use `$$`. <br>要引用 `$` 本身，请使用 `$$`。<br>
#[unstable(feature = "proc_macro_quote", issue = "54722")]
#[allow_internal_unstable(proc_macro_def_site, proc_macro_internals)]
#[rustc_builtin_macro]
pub macro quote($($t:tt)*) {
    /* compiler built-in */
}

#[unstable(feature = "proc_macro_internals", issue = "27812")]
#[doc(hidden)]
mod quote;

/// A region of source code, along with macro expansion information. <br>源代码区域以及宏展开信息。<br>
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
#[derive(Copy, Clone)]
pub struct Span(bridge::client::Span);

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl !Send for Span {}
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl !Sync for Span {}

macro_rules! diagnostic_method {
    ($name:ident, $level:expr) => {
        /// Creates a new `Diagnostic` with the given `message` at the span `self`. <br>使用跨度为 `self` 的给定 `message` 创建一个新的 `Diagnostic`。<br>
        ///
        #[unstable(feature = "proc_macro_diagnostic", issue = "54140")]
        pub fn $name<T: Into<String>>(self, message: T) -> Diagnostic {
            Diagnostic::spanned(self, $level, message)
        }
    };
}

impl Span {
    /// A span that resolves at the macro definition site. <br>在宏定义站点解析的跨度。<br>
    #[unstable(feature = "proc_macro_def_site", issue = "54724")]
    pub fn def_site() -> Span {
        Span(bridge::client::Span::def_site())
    }

    /// The span of the invocation of the current procedural macro. <br>当前过程宏的调用范围。<br>
    /// Identifiers created with this span will be resolved as if they were written directly at the macro call location (call-site hygiene) and other code at the macro call site will be able to refer to them as well. <br>以此范围创建的标识符将被解析，就像它们是直接在宏调用位置 (调用站点卫生) 上编写的一样，宏调用站点上的其他代码也将能够引用它们。<br>
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn call_site() -> Span {
        Span(bridge::client::Span::call_site())
    }

    /// A span that represents `macro_rules` hygiene, and sometimes resolves at the macro definition site (local variables, labels, `$crate`) and sometimes at the macro call site (everything else). <br>代表 `macro_rules` 卫生状况的跨度，有时在宏定义站点 (本地变量，标签，`$crate`) 解析，有时在宏调用站点 (其他所有解析) 解析。<br>
    ///
    /// The span location is taken from the call-site. <br>跨度位置取自调用站点。<br>
    ///
    #[stable(feature = "proc_macro_mixed_site", since = "1.45.0")]
    pub fn mixed_site() -> Span {
        Span(bridge::client::Span::mixed_site())
    }

    /// The original source file into which this span points. <br>此跨度指向的原始源文件。<br>
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn source_file(&self) -> SourceFile {
        SourceFile(self.0.source_file())
    }

    /// The `Span` for the tokens in the previous macro expansion from which `self` was generated from, if any. <br>上一个宏的 tokens 的 `Span` (如果有的话) 是从中生成 `self` 的。<br>
    ///
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn parent(&self) -> Option<Span> {
        self.0.parent().map(Span)
    }

    /// The span for the origin source code that `self` was generated from. <br>生成 `self` 的原始源代码的范围。<br>
    /// If this `Span` wasn't generated from other macro expansions then the return value is the same as `*self`. <br>如果此 `Span` 不是由其他宏扩展生成的，则返回值与 `*self` 相同。<br>
    ///
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn source(&self) -> Span {
        Span(self.0.source())
    }

    /// Gets the starting line/column in the source file for this span. <br>在源文件中获取此范围的起始 line/column。<br>
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn start(&self) -> LineColumn {
        self.0.start().add_1_to_column()
    }

    /// Gets the ending line/column in the source file for this span. <br>在源文件中获取此跨度的结尾 line/column。<br>
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn end(&self) -> LineColumn {
        self.0.end().add_1_to_column()
    }

    /// Creates an empty span pointing to directly before this span. <br>创建一个直接指向此跨度之前的空跨度。<br>
    #[unstable(feature = "proc_macro_span_shrink", issue = "87552")]
    pub fn before(&self) -> Span {
        Span(self.0.before())
    }

    /// Creates an empty span pointing to directly after this span. <br>创建一个直接指向此跨度之后的空跨度。<br>
    #[unstable(feature = "proc_macro_span_shrink", issue = "87552")]
    pub fn after(&self) -> Span {
        Span(self.0.after())
    }

    /// Creates a new span encompassing `self` and `other`. <br>创建一个包含 `self` 和 `other` 的新跨度。<br>
    ///
    /// Returns `None` if `self` and `other` are from different files. <br>如果 `self` 和 `other` 来自不同的文件，则返回 `None`。<br>
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn join(&self, other: Span) -> Option<Span> {
        self.0.join(other.0).map(Span)
    }

    /// Creates a new span with the same line/column information as `self` but that resolves symbols as though it were at `other`. <br>创建一个具有与 `self` 相同的 line/column 信息的新跨度，但是可以像在 `other` 一样解析符号。<br>
    ///
    #[stable(feature = "proc_macro_span_resolved_at", since = "1.45.0")]
    pub fn resolved_at(&self, other: Span) -> Span {
        Span(self.0.resolved_at(other.0))
    }

    /// Creates a new span with the same name resolution behavior as `self` but with the line/column information of `other`. <br>创建一个具有与 `self` 相同的名称解析行为但具有 `other` 的 line/column 信息的新跨度。<br>
    ///
    #[stable(feature = "proc_macro_span_located_at", since = "1.45.0")]
    pub fn located_at(&self, other: Span) -> Span {
        other.resolved_at(*self)
    }

    /// Compares to spans to see if they're equal. <br>比较跨度以查看它们是否相等。<br>
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn eq(&self, other: &Span) -> bool {
        self.0 == other.0
    }

    /// Returns the source text behind a span. <br>返回跨度后面的源文本。<br>
    /// This preserves the original source code, including spaces and comments. <br>这将保留原始源代码，包括空格和注释。<br>
    /// It only returns a result if the span corresponds to real source code. <br>仅当范围与实际源代码相对应时，它才返回结果。<br>
    ///
    /// Note: The observable result of a macro should only rely on the tokens and not on this source text. <br>宏的可观察结果应仅依赖于 tokens，而不依赖于此源文本。<br>
    ///
    /// The result of this function is a best effort to be used for diagnostics only. <br>该函数的结果是尽力而为，仅用于诊断。<br>
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn source_text(&self) -> Option<String> {
        self.0.source_text()
    }

    // Used by the implementation of `Span::quote` <br>由 `Span::quote` 的实现使用<br>
    #[doc(hidden)]
    #[unstable(feature = "proc_macro_internals", issue = "27812")]
    pub fn save_span(&self) -> usize {
        self.0.save_span()
    }

    // Used by the implementation of `Span::quote` <br>由 `Span::quote` 的实现使用<br>
    #[doc(hidden)]
    #[unstable(feature = "proc_macro_internals", issue = "27812")]
    pub fn recover_proc_macro_span(id: usize) -> Span {
        Span(bridge::client::Span::recover_proc_macro_span(id))
    }

    diagnostic_method!(error, Level::Error);
    diagnostic_method!(warning, Level::Warning);
    diagnostic_method!(note, Level::Note);
    diagnostic_method!(help, Level::Help);
}

/// Prints a span in a form convenient for debugging. <br>以便于调试的形式打印跨度。<br>
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Debug for Span {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.0.fmt(f)
    }
}

/// A line-column pair representing the start or end of a `Span`. <br>代表 `Span` 开头或结尾的行 - 列对。<br>
#[unstable(feature = "proc_macro_span", issue = "54725")]
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
pub struct LineColumn {
    /// The 1-indexed line in the source file on which the span starts or ends (inclusive). <br>源文件中跨度开始或结束 (inclusive) 的 1 索引行。<br>
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub line: usize,
    /// The 1-indexed column (number of bytes in UTF-8 encoding) in the source file on which the span starts or ends (inclusive). <br>跨度开始或结束 (inclusive) 的源文件中的 1 索引列 (UTF-8 编码中的字节数)。<br>
    ///
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub column: usize,
}

impl LineColumn {
    fn add_1_to_column(self) -> Self {
        LineColumn { line: self.line, column: self.column + 1 }
    }
}

#[unstable(feature = "proc_macro_span", issue = "54725")]
impl !Send for LineColumn {}
#[unstable(feature = "proc_macro_span", issue = "54725")]
impl !Sync for LineColumn {}

#[unstable(feature = "proc_macro_span", issue = "54725")]
impl Ord for LineColumn {
    fn cmp(&self, other: &Self) -> Ordering {
        self.line.cmp(&other.line).then(self.column.cmp(&other.column))
    }
}

#[unstable(feature = "proc_macro_span", issue = "54725")]
impl PartialOrd for LineColumn {
    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
        Some(self.cmp(other))
    }
}

/// The source file of a given `Span`. <br>给定 `Span` 的源文件。<br>
#[unstable(feature = "proc_macro_span", issue = "54725")]
#[derive(Clone)]
pub struct SourceFile(bridge::client::SourceFile);

impl SourceFile {
    /// Gets the path to this source file. <br>获取此源文件的路径。<br>
    ///
    /// ### Note
    /// If the code span associated with this `SourceFile` was generated by an external macro, this macro, this might not be an actual path on the filesystem. <br>如果与此 `SourceFile` 关联的代码跨度是由外部宏生成的，则此宏可能不是文件系统上的实际路径。<br>
    /// Use [`is_real`] to check. <br>使用 [`is_real`] 进行检查。<br>
    ///
    /// Also note that even if `is_real` returns `true`, if `--remap-path-prefix` was passed on the command line, the path as given might not actually be valid. <br>另请注意，即使 `is_real` 返回 `true`，如果 `--remap-path-prefix` 在命令行上传递，给定的路径实际上可能无效。<br>
    ///
    ///
    /// [`is_real`]: Self::is_real
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn path(&self) -> PathBuf {
        PathBuf::from(self.0.path())
    }

    /// Returns `true` if this source file is a real source file, and not generated by an external macro's expansion. <br>如果此源文件是真实的源文件，并且不是由外部宏的扩展生成的，则返回 `true`。<br>
    ///
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn is_real(&self) -> bool {
        // This is a hack until intercrate spans are implemented and we can have real source files for spans generated in external macros. <br>在实现跨度跨度之前，这是一个技巧，我们可以为外部宏中生成的跨度提供真实的源文件。<br>
        //
        // https://github.com/rust-lang/rust/pull/43604#issuecomment-333334368
        self.0.is_real()
    }
}

#[unstable(feature = "proc_macro_span", issue = "54725")]
impl fmt::Debug for SourceFile {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("SourceFile")
            .field("path", &self.path())
            .field("is_real", &self.is_real())
            .finish()
    }
}

#[unstable(feature = "proc_macro_span", issue = "54725")]
impl PartialEq for SourceFile {
    fn eq(&self, other: &Self) -> bool {
        self.0.eq(&other.0)
    }
}

#[unstable(feature = "proc_macro_span", issue = "54725")]
impl Eq for SourceFile {}

/// A single token or a delimited sequence of token trees (e.g., `[1, (), ..]`). <br>单个 token 或 token 树的定界序列 (例如 `[1, (), ..]`)。<br>
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
#[derive(Clone)]
pub enum TokenTree {
    /// A token stream surrounded by bracket delimiters. <br>由括号定界符包围的 token 流。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    Group(#[stable(feature = "proc_macro_lib2", since = "1.29.0")] Group),
    /// An identifier. <br>标识符。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    Ident(#[stable(feature = "proc_macro_lib2", since = "1.29.0")] Ident),
    /// A single punctuation character (`+`, `,`, `$`, etc.). <br>单个标点符号 (`+`，`,`，`$` 等)。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    Punct(#[stable(feature = "proc_macro_lib2", since = "1.29.0")] Punct),
    /// A literal character (`'a'`), string (`"hello"`), number (`2.3`), etc. <br>字面量字符 (`'a'`)，字符串 (`"hello"`)，数字 (`2.3`) 等。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    Literal(#[stable(feature = "proc_macro_lib2", since = "1.29.0")] Literal),
}

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl !Send for TokenTree {}
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl !Sync for TokenTree {}

impl TokenTree {
    /// Returns the span of this tree, delegating to the `span` method of the contained token or a delimited stream. <br>返回此树的范围，委托给包含的 token 或分隔流的 `span` 方法。<br>
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn span(&self) -> Span {
        match *self {
            TokenTree::Group(ref t) => t.span(),
            TokenTree::Ident(ref t) => t.span(),
            TokenTree::Punct(ref t) => t.span(),
            TokenTree::Literal(ref t) => t.span(),
        }
    }

    /// Configures the span for *only this token*. <br>为 *only this token* 配置范围。<br>
    ///
    /// Note that if this token is a `Group` then this method will not configure the span of each of the internal tokens, this will simply delegate to the `set_span` method of each variant. <br>请注意，如果此 token 是 `Group`，则此方法将不会配置每个内部 tokens 的跨度，这将仅委托给每个变体的 `set_span` 方法。<br>
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn set_span(&mut self, span: Span) {
        match *self {
            TokenTree::Group(ref mut t) => t.set_span(span),
            TokenTree::Ident(ref mut t) => t.set_span(span),
            TokenTree::Punct(ref mut t) => t.set_span(span),
            TokenTree::Literal(ref mut t) => t.set_span(span),
        }
    }
}

/// Prints token tree in a form convenient for debugging. <br>以方便调试的形式打印 token 树。<br>
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Debug for TokenTree {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        // Each of these has the name in the struct type in the derived debug, so don't bother with an extra layer of indirection <br>它们中的每一个在派生调试中的结构体类型中都具有名称，因此不必担心额外的间接层<br>
        //
        match *self {
            TokenTree::Group(ref tt) => tt.fmt(f),
            TokenTree::Ident(ref tt) => tt.fmt(f),
            TokenTree::Punct(ref tt) => tt.fmt(f),
            TokenTree::Literal(ref tt) => tt.fmt(f),
        }
    }
}

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl From<Group> for TokenTree {
    fn from(g: Group) -> TokenTree {
        TokenTree::Group(g)
    }
}

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl From<Ident> for TokenTree {
    fn from(g: Ident) -> TokenTree {
        TokenTree::Ident(g)
    }
}

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl From<Punct> for TokenTree {
    fn from(g: Punct) -> TokenTree {
        TokenTree::Punct(g)
    }
}

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl From<Literal> for TokenTree {
    fn from(g: Literal) -> TokenTree {
        TokenTree::Literal(g)
    }
}

// N.B., the bridge only provides `to_string`, implement `fmt::Display` based on it (the reverse of the usual relationship between the two). <br>注意，网桥仅提供 `to_string`，基于它实现 `fmt::Display` (两者之间通常的关系相反)。<br>
//
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl ToString for TokenTree {
    fn to_string(&self) -> String {
        match *self {
            TokenTree::Group(ref t) => t.to_string(),
            TokenTree::Ident(ref t) => t.to_string(),
            TokenTree::Punct(ref t) => t.to_string(),
            TokenTree::Literal(ref t) => t.to_string(),
        }
    }
}

/// Prints the token tree as a string that is supposed to be losslessly convertible back into the same token tree (modulo spans), except for possibly `TokenTree::Group`s with `Delimiter::None` delimiters and negative numeric literals. <br>将 token 树打印为应该无损转换为同一 token 树 (取模跨度) 的字符串，但可能带有 `Delimiter::None` 定界符和负数字字面量的 `TokenTree::Group` 除外。<br>
///
///
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Display for TokenTree {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str(&self.to_string())
    }
}

/// A delimited token stream. <br>分隔的 token 流。<br>
///
/// A `Group` internally contains a `TokenStream` which is surrounded by `Delimiter`s. <br>`Group` 内部包含一个 `TokenStream`，该 `TokenStream` 被 `Delimiter` 包围。<br>
#[derive(Clone)]
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
pub struct Group(bridge::client::Group);

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl !Send for Group {}
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl !Sync for Group {}

/// Describes how a sequence of token trees is delimited. <br>描述如何分隔 token 树的序列。<br>
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
pub enum Delimiter {
    /// `( ... )`
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    Parenthesis,
    /// `{ ... }`
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    Brace,
    /// `[ ... ]`
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    Bracket,
    /// `Ø ... Ø`
    /// An invisible delimiter, that may, for example, appear around tokens coming from a "macro variable" `$var`. <br>一个不可见的分隔符，例如，可能出现在来自 "macro variable" `$var` 的 tokens 周围。<br>
    /// It is important to preserve operator priorities in cases like `$var * 3` where `$var` is `1 + 2`. <br>在诸如 `$var` 为 `1 + 2` 的 `$var * 3` 之类的情况下，保留操作员的优先级很重要。<br>
    /// Invisible delimiters might not survive roundtrip of a token stream through a string. <br>不可见的分隔符可能无法在 token 流通过字符串的往返过程中幸存下来。<br>
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    None,
}

impl Group {
    /// Creates a new `Group` with the given delimiter and token stream. <br>使用给定的定界符和 token 流创建一个新的 `Group`。<br>
    ///
    /// This constructor will set the span for this group to `Span::call_site()`. <br>此构造函数将将此组的跨度设置为 `Span::call_site()`。<br>
    /// To change the span you can use the `set_span` method below. <br>要更改跨度，可以使用下面的 `set_span` 方法。<br>
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn new(delimiter: Delimiter, stream: TokenStream) -> Group {
        Group(bridge::client::Group::new(delimiter, stream.0))
    }

    /// Returns the delimiter of this `Group` <br>返回此 `Group` 的定界符<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn delimiter(&self) -> Delimiter {
        self.0.delimiter()
    }

    /// Returns the `TokenStream` of tokens that are delimited in this `Group`. <br>返回在此 `Group` 中定界的 tokens 的 `TokenStream`。<br>
    ///
    /// Note that the returned token stream does not include the delimiter returned above. <br>请注意，返回的 token 流不包括上面返回的定界符。<br>
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn stream(&self) -> TokenStream {
        TokenStream(Some(self.0.stream()))
    }

    /// Returns the span for the delimiters of this token stream, spanning the entire `Group`. <br>返回此 token 流的定界符的范围，该范围跨越整个 `Group`。<br>
    ///
    ///
    /// ```text
    /// pub fn span(&self) -> Span {
    ///            ^^^^^^^
    /// ```
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn span(&self) -> Span {
        Span(self.0.span())
    }

    /// Returns the span pointing to the opening delimiter of this group. <br>返回指向该组的开始定界符的跨度。<br>
    ///
    /// ```text
    /// pub fn span_open(&self) -> Span {
    ///                 ^
    /// ```
    #[stable(feature = "proc_macro_group_span", since = "1.55.0")]
    pub fn span_open(&self) -> Span {
        Span(self.0.span_open())
    }

    /// Returns the span pointing to the closing delimiter of this group. <br>返回指向该组的结束定界符的跨度。<br>
    ///
    /// ```text
    /// pub fn span_close(&self) -> Span {
    ///                        ^
    /// ```
    #[stable(feature = "proc_macro_group_span", since = "1.55.0")]
    pub fn span_close(&self) -> Span {
        Span(self.0.span_close())
    }

    /// Configures the span for this `Group`'s delimiters, but not its internal tokens. <br>配置此 `Group` 定界符的范围，但不配置其内部 tokens 的范围。<br>
    ///
    /// This method will **not** set the span of all the internal tokens spanned by this group, but rather it will only set the span of the delimiter tokens at the level of the `Group`. <br>此方法将不设置该组所跨越的所有内部 tokens 的跨度，而只会将定界符 tokens 的跨度设置为 `Group` 的水平。<br>
    ///
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn set_span(&mut self, span: Span) {
        self.0.set_span(span.0);
    }
}

// N.B., the bridge only provides `to_string`, implement `fmt::Display` based on it (the reverse of the usual relationship between the two). <br>注意，网桥仅提供 `to_string`，基于它实现 `fmt::Display` (两者之间通常的关系相反)。<br>
//
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl ToString for Group {
    fn to_string(&self) -> String {
        TokenStream::from(TokenTree::from(self.clone())).to_string()
    }
}

/// Prints the group as a string that should be losslessly convertible back into the same group (modulo spans), except for possibly `TokenTree::Group`s with `Delimiter::None` delimiters. <br>将组打印为字符串，该字符串应无损地转换回同一组 (模跨度)，但可能带有 `Delimiter::None` 分隔符的 'TokenTree::Group' 除外。<br>
///
///
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Display for Group {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str(&self.to_string())
    }
}

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Debug for Group {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Group")
            .field("delimiter", &self.delimiter())
            .field("stream", &self.stream())
            .field("span", &self.span())
            .finish()
    }
}

/// A `Punct` is a single punctuation character such as `+`, `-` or `#`. <br>`Punct` 是单个标点符号，例如 `+`、`-` 或 `#`。<br>
///
/// Multi-character operators like `+=` are represented as two instances of `Punct` with different forms of `Spacing` returned. <br>像 `+=` 这样的多字符运算符表示为 `Punct` 的两个实例，它们返回了不同形式的 `Spacing`。<br>
///
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
#[derive(Clone)]
pub struct Punct(bridge::client::Punct);

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl !Send for Punct {}
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl !Sync for Punct {}

/// Describes whether a `Punct` is followed immediately by another `Punct` ([`Spacing::Joint`]) or by a different token or whitespace ([`Spacing::Alone`]). <br>描述 `Punct` 后面是紧跟另一个 `Punct` ([`Spacing::Joint`]) 还是不同的 token 或空格 ([`Spacing::Alone`])。<br>
///
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
pub enum Spacing {
    /// A `Punct` is not immediately followed by another `Punct`. <br>一个 `Punct` 后面不会紧跟另一个 `Punct`。<br>
    /// E.g. `+` is `Alone` in `+ =`, `+ident` and `+()`. <br>例如，`+ =`、`+ident` 和 `+()` 中的 `+` 是 `Alone`。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    Alone,
    /// A `Punct` is immediately followed by another `Punct`. <br>`Punct` 紧随其后的是另一个 `Punct`。<br>
    /// E.g. `+` is `Joint` in `+=` and `++`. <br>例如，`+` 在 `+=` 和 `++` 中是 `Joint`。<br>
    ///
    /// Additionally, single quote `'` can join with identifiers to form lifetimes: `'ident`. <br>此外，单引号 `'` 可以与标识符连接以形成生命周期: `'ident`。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    Joint,
}

impl Punct {
    /// Creates a new `Punct` from the given character and spacing. <br>根据给定的字符和间距创建一个新的 `Punct`。<br>
    /// The `ch` argument must be a valid punctuation character permitted by the language, otherwise the function will panic. <br>`ch` 参数必须是语言允许的有效标点符号，否则函数将为 panic。<br>
    ///
    /// The returned `Punct` will have the default span of `Span::call_site()` which can be further configured with the `set_span` method below. <br>返回的 `Punct` 将具有默认范围 `Span::call_site()`，可以使用下面的 `set_span` 方法对其进行进一步配置。<br>
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn new(ch: char, spacing: Spacing) -> Punct {
        Punct(bridge::client::Punct::new(ch, spacing))
    }

    /// Returns the value of this punctuation character as `char`. <br>将此标点符号的值返回为 `char`。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn as_char(&self) -> char {
        self.0.as_char()
    }

    /// Returns the spacing of this punctuation character, indicating whether it's immediately followed by another `Punct` in the token stream, so they can potentially be combined into a multi-character operator (`Joint`), or it's followed by some other token or whitespace (`Alone`) so the operator has certainly ended. <br>返回此标点符号的间距，指示其是否紧随 token 流中的另一个 `Punct`，以便可以将它们组合为多字符运算符 (`Joint`)，也可以将其与其他 token 或空格 (`Alone`) 组合在一起，因此该运算符肯定具有结束了。<br>
    ///
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn spacing(&self) -> Spacing {
        self.0.spacing()
    }

    /// Returns the span for this punctuation character. <br>返回此标点符号字符的跨度。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn span(&self) -> Span {
        Span(self.0.span())
    }

    /// Configure the span for this punctuation character. <br>配置该标点字符的跨度。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn set_span(&mut self, span: Span) {
        self.0 = self.0.with_span(span.0);
    }
}

// N.B., the bridge only provides `to_string`, implement `fmt::Display` based on it (the reverse of the usual relationship between the two). <br>注意，网桥仅提供 `to_string`，基于它实现 `fmt::Display` (两者之间通常的关系相反)。<br>
//
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl ToString for Punct {
    fn to_string(&self) -> String {
        TokenStream::from(TokenTree::from(self.clone())).to_string()
    }
}

/// Prints the punctuation character as a string that should be losslessly convertible back into the same character. <br>将标点符号打印为字符串，该字符串应可以无损地转换回相同的字符。<br>
///
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Display for Punct {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str(&self.to_string())
    }
}

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Debug for Punct {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Punct")
            .field("ch", &self.as_char())
            .field("spacing", &self.spacing())
            .field("span", &self.span())
            .finish()
    }
}

#[stable(feature = "proc_macro_punct_eq", since = "1.50.0")]
impl PartialEq<char> for Punct {
    fn eq(&self, rhs: &char) -> bool {
        self.as_char() == *rhs
    }
}

#[stable(feature = "proc_macro_punct_eq_flipped", since = "1.52.0")]
impl PartialEq<Punct> for char {
    fn eq(&self, rhs: &Punct) -> bool {
        *self == rhs.as_char()
    }
}

/// An identifier (`ident`). <br>标识符 (`ident`)。<br>
#[derive(Clone)]
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
pub struct Ident(bridge::client::Ident);

impl Ident {
    /// Creates a new `Ident` with the given `string` as well as the specified `span`. <br>使用给定的 `string` 和指定的 `span` 创建新的 `Ident`。<br>
    /// The `string` argument must be a valid identifier permitted by the language (including keywords, e.g. `self` or `fn`). <br>`string` 参数必须是该语言允许的有效标识符 (包括关键字，例如 `self` 或 `fn`)。<br> Otherwise, the function will panic. <br>否则，函数将为 panic。<br>
    ///
    /// Note that `span`, currently in rustc, configures the hygiene information for this identifier. <br>请注意，当前位于 rustc 中的 `span` 为此标识符配置了卫生信息。<br>
    ///
    /// As of this time `Span::call_site()` explicitly opts-in to "call-site" hygiene meaning that identifiers created with this span will be resolved as if they were written directly at the location of the macro call, and other code at the macro call site will be able to refer to them as well. <br>截止到目前，`Span::call_site()` 明确选择采用 "call-site" 卫生标准，这意味着使用该跨度创建的标识符将被解析，就好像它们直接写在宏调用的位置一样，并且宏调用站点上的其他代码将能够引用他们也是。<br>
    ///
    ///
    /// Later spans like `Span::def_site()` will allow to opt-in to "definition-site" hygiene meaning that identifiers created with this span will be resolved at the location of the macro definition and other code at the macro call site will not be able to refer to them. <br>以后的跨度 (例如 `Span::def_site()`) 将允许选择使用 "definition-site" 卫生，这意味着用该跨度创建的标识符将在宏定义的位置解析，而宏调用站点上的其他代码将无法引用它们。<br>
    ///
    /// Due to the current importance of hygiene this constructor, unlike other tokens, requires a `Span` to be specified at construction. <br>由于当前对卫生的重要性，与其他 tokens 不同，该构造函数需要在构造时指定 `Span`。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn new(string: &str, span: Span) -> Ident {
        Ident(bridge::client::Ident::new(string, span.0, false))
    }

    /// Same as `Ident::new`, but creates a raw identifier (`r#ident`). <br>与 `Ident::new` 相同，但创建原始标识符 (`r#ident`)。<br>
    /// The `string` argument be a valid identifier permitted by the language (including keywords, e.g. `fn`). <br>`string` 参数是该语言允许的有效标识符 (包括关键字，例如 `fn`)。<br>
    /// Keywords which are usable in path segments (e.g. <br>在路径段中可用的关键字 (例如，<br>
    /// `self`, `super`) are not supported, and will cause a panic. <br>`self`、`super`) 不受支持，会导致 panic。<br>
    #[stable(feature = "proc_macro_raw_ident", since = "1.47.0")]
    pub fn new_raw(string: &str, span: Span) -> Ident {
        Ident(bridge::client::Ident::new(string, span.0, true))
    }

    /// Returns the span of this `Ident`, encompassing the entire string returned by [`to_string`](Self::to_string). <br>返回此 `Ident` 的范围，包括 [`to_string`](Self::to_string) 返回的整个字符串。<br>
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn span(&self) -> Span {
        Span(self.0.span())
    }

    /// Configures the span of this `Ident`, possibly changing its hygiene context. <br>配置此 `Ident` 的跨度，可能会更改其卫生状况。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn set_span(&mut self, span: Span) {
        self.0 = self.0.with_span(span.0);
    }
}

// N.B., the bridge only provides `to_string`, implement `fmt::Display` based on it (the reverse of the usual relationship between the two). <br>注意，网桥仅提供 `to_string`，基于它实现 `fmt::Display` (两者之间通常的关系相反)。<br>
//
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl ToString for Ident {
    fn to_string(&self) -> String {
        TokenStream::from(TokenTree::from(self.clone())).to_string()
    }
}

/// Prints the identifier as a string that should be losslessly convertible back into the same identifier. <br>将标识符打印为字符串，该字符串应可以无损地转换回相同的标识符。<br>
///
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Display for Ident {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str(&self.to_string())
    }
}

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Debug for Ident {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Ident")
            .field("ident", &self.to_string())
            .field("span", &self.span())
            .finish()
    }
}

/// A literal string (`"hello"`), byte string (`b"hello"`), character (`'a'`), byte character (`b'a'`), an integer or floating point number with or without a suffix (`1`, `1u8`, `2.3`, `2.3f32`). <br>字符串字面量 (`"hello"`)，字节字符串 (`b"hello"`)，字符 (`'a'`)，字节字符 (`b'a'`)，带或不带后缀 ('1'，`1u8`，`2.3`，`2.3f32`) 的整数或浮点数。<br>
///
/// Boolean literals like `true` and `false` do not belong here, they are `Ident`s. <br>`true` 和 `false` 之类的布尔字面量在这里不属于它们，它们是 `Ident`。<br>
///
#[derive(Clone)]
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
pub struct Literal(bridge::client::Literal);

macro_rules! suffixed_int_literals {
    ($($name:ident => $kind:ident,)*) => ($(
        /// Creates a new suffixed integer literal with the specified value. <br>用指定的值创建一个新的后缀整数字面量。<br>
        ///
        /// This function will create an integer like `1u32` where the integer value specified is the first part of the token and the integral is also suffixed at the end. <br>此函数将创建一个类似于 `1u32` 的整数，其中指定的整数值是 token 的第一部分，并且整数也以结尾加后缀。<br>
        /// Literals created from negative numbers might not survive round-trips through `TokenStream` or strings and may be broken into two tokens (`-` and positive literal). <br>从负数创建的字面量可能无法通过 `TokenStream` 或字符串进行往返，并且可能会分解为两个 tokens (`-` 和正字面量)。<br>
        ///
        ///
        /// Literals created through this method have the `Span::call_site()` span by default, which can be configured with the `set_span` method below. <br>通过此方法创建的字面量默认情况下具有 `Span::call_site()` 跨度，可以使用以下 `set_span` 方法进行配置。<br>
        ///
        ///
        ///
        ///
        #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
        pub fn $name(n: $kind) -> Literal {
            Literal(bridge::client::Literal::typed_integer(&n.to_string(), stringify!($kind)))
        }
    )*)
}

macro_rules! unsuffixed_int_literals {
    ($($name:ident => $kind:ident,)*) => ($(
        /// Creates a new unsuffixed integer literal with the specified value. <br>用指定的值创建一个新的无后缀的整数字面量。<br>
        ///
        /// This function will create an integer like `1` where the integer value specified is the first part of the token. <br>此函数将创建一个类似于 `1` 的整数，其中指定的整数值是 token 的第一部分。<br>
        /// No suffix is specified on this token, meaning that invocations like `Literal::i8_unsuffixed(1)` are equivalent to `Literal::u32_unsuffixed(1)`. <br>在此 token 上未指定后缀，这意味着像 `Literal::i8_unsuffixed(1)` 这样的调用等效于 `Literal::u32_unsuffixed(1)`。<br>
        /// Literals created from negative numbers might not survive rountrips through `TokenStream` or strings and may be broken into two tokens (`-` and positive literal). <br>由负数创建的字面量可能无法通过 `TokenStream` 或字符串进行往返，并且可能会分解为两个 tokens (`-` 和正字面量)。<br>
        ///
        ///
        /// Literals created through this method have the `Span::call_site()` span by default, which can be configured with the `set_span` method below. <br>通过此方法创建的字面量默认情况下具有 `Span::call_site()` 跨度，可以使用以下 `set_span` 方法进行配置。<br>
        ///
        ///
        ///
        ///
        ///
        #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
        pub fn $name(n: $kind) -> Literal {
            Literal(bridge::client::Literal::integer(&n.to_string()))
        }
    )*)
}

impl Literal {
    suffixed_int_literals! {
        u8_suffixed => u8,
        u16_suffixed => u16,
        u32_suffixed => u32,
        u64_suffixed => u64,
        u128_suffixed => u128,
        usize_suffixed => usize,
        i8_suffixed => i8,
        i16_suffixed => i16,
        i32_suffixed => i32,
        i64_suffixed => i64,
        i128_suffixed => i128,
        isize_suffixed => isize,
    }

    unsuffixed_int_literals! {
        u8_unsuffixed => u8,
        u16_unsuffixed => u16,
        u32_unsuffixed => u32,
        u64_unsuffixed => u64,
        u128_unsuffixed => u128,
        usize_unsuffixed => usize,
        i8_unsuffixed => i8,
        i16_unsuffixed => i16,
        i32_unsuffixed => i32,
        i64_unsuffixed => i64,
        i128_unsuffixed => i128,
        isize_unsuffixed => isize,
    }

    /// Creates a new unsuffixed floating-point literal. <br>创建一个新的不带后缀的浮点字面量。<br>
    ///
    /// This constructor is similar to those like `Literal::i8_unsuffixed` where the float's value is emitted directly into the token but no suffix is used, so it may be inferred to be a `f64` later in the compiler. <br>此构造函数类似于 `Literal::i8_unsuffixed`，后者将 float 的值直接发出到 token 中，但不使用后缀，因此可以在以后的编译器中推断出它是 `f64`。<br>
    ///
    /// Literals created from negative numbers might not survive rountrips through `TokenStream` or strings and may be broken into two tokens (`-` and positive literal). <br>由负数创建的字面量可能无法通过 `TokenStream` 或字符串进行往返，并且可能会分解为两个 tokens (`-` 和正字面量)。<br>
    ///
    /// # Panics
    ///
    /// This function requires that the specified float is finite, for example if it is infinity or NaN this function will panic. <br>此函数要求指定的浮点数是有限的，例如，如果它是无穷大或 NaN，则此函数将为 panic。<br>
    ///
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn f32_unsuffixed(n: f32) -> Literal {
        if !n.is_finite() {
            panic!("Invalid float literal {n}");
        }
        let mut repr = n.to_string();
        if !repr.contains('.') {
            repr.push_str(".0");
        }
        Literal(bridge::client::Literal::float(&repr))
    }

    /// Creates a new suffixed floating-point literal. <br>创建一个新的后缀浮点字面量。<br>
    ///
    /// This constructor will create a literal like `1.0f32` where the value specified is the preceding part of the token and `f32` is the suffix of the token. <br>该构造函数将创建一个像 `1.0f32` 这样的字面量，其中指定的值是 token 的前一部分，而 `f32` 是 token 的后缀。<br>
    /// This token will always be inferred to be an `f32` in the compiler. <br>token 在编译器中将始终被推断为 `f32`。<br>
    /// Literals created from negative numbers might not survive rountrips through `TokenStream` or strings and may be broken into two tokens (`-` and positive literal). <br>由负数创建的字面量可能无法通过 `TokenStream` 或字符串进行往返，并且可能会分解为两个 tokens (`-` 和正字面量)。<br>
    ///
    ///
    /// # Panics
    ///
    /// This function requires that the specified float is finite, for example if it is infinity or NaN this function will panic. <br>此函数要求指定的浮点数是有限的，例如，如果它是无穷大或 NaN，则此函数将为 panic。<br>
    ///
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn f32_suffixed(n: f32) -> Literal {
        if !n.is_finite() {
            panic!("Invalid float literal {n}");
        }
        Literal(bridge::client::Literal::f32(&n.to_string()))
    }

    /// Creates a new unsuffixed floating-point literal. <br>创建一个新的不带后缀的浮点字面量。<br>
    ///
    /// This constructor is similar to those like `Literal::i8_unsuffixed` where the float's value is emitted directly into the token but no suffix is used, so it may be inferred to be a `f64` later in the compiler. <br>此构造函数类似于 `Literal::i8_unsuffixed`，后者将 float 的值直接发出到 token 中，但不使用后缀，因此可以在以后的编译器中推断出它是 `f64`。<br>
    ///
    /// Literals created from negative numbers might not survive rountrips through `TokenStream` or strings and may be broken into two tokens (`-` and positive literal). <br>由负数创建的字面量可能无法通过 `TokenStream` 或字符串进行往返，并且可能会分解为两个 tokens (`-` 和正字面量)。<br>
    ///
    /// # Panics
    ///
    /// This function requires that the specified float is finite, for example if it is infinity or NaN this function will panic. <br>此函数要求指定的浮点数是有限的，例如，如果它是无穷大或 NaN，则此函数将为 panic。<br>
    ///
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn f64_unsuffixed(n: f64) -> Literal {
        if !n.is_finite() {
            panic!("Invalid float literal {n}");
        }
        let mut repr = n.to_string();
        if !repr.contains('.') {
            repr.push_str(".0");
        }
        Literal(bridge::client::Literal::float(&repr))
    }

    /// Creates a new suffixed floating-point literal. <br>创建一个新的后缀浮点字面量。<br>
    ///
    /// This constructor will create a literal like `1.0f64` where the value specified is the preceding part of the token and `f64` is the suffix of the token. <br>该构造函数将创建一个像 `1.0f64` 这样的字面量，其中指定的值是 token 的前一部分，而 `f64` 是 token 的后缀。<br>
    /// This token will always be inferred to be an `f64` in the compiler. <br>token 在编译器中将始终被推断为 `f64`。<br>
    /// Literals created from negative numbers might not survive rountrips through `TokenStream` or strings and may be broken into two tokens (`-` and positive literal). <br>由负数创建的字面量可能无法通过 `TokenStream` 或字符串进行往返，并且可能会分解为两个 tokens (`-` 和正字面量)。<br>
    ///
    ///
    /// # Panics
    ///
    /// This function requires that the specified float is finite, for example if it is infinity or NaN this function will panic. <br>此函数要求指定的浮点数是有限的，例如，如果它是无穷大或 NaN，则此函数将为 panic。<br>
    ///
    ///
    ///
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn f64_suffixed(n: f64) -> Literal {
        if !n.is_finite() {
            panic!("Invalid float literal {n}");
        }
        Literal(bridge::client::Literal::f64(&n.to_string()))
    }

    /// String literal. <br>字符串字面量。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn string(string: &str) -> Literal {
        Literal(bridge::client::Literal::string(string))
    }

    /// Character literal. <br>字符字面量。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn character(ch: char) -> Literal {
        Literal(bridge::client::Literal::character(ch))
    }

    /// Byte string literal. <br>字节字符串字面量。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn byte_string(bytes: &[u8]) -> Literal {
        Literal(bridge::client::Literal::byte_string(bytes))
    }

    /// Returns the span encompassing this literal. <br>返回包含此字面量的范围。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn span(&self) -> Span {
        Span(self.0.span())
    }

    /// Configures the span associated for this literal. <br>配置与此字面量关联的范围。<br>
    #[stable(feature = "proc_macro_lib2", since = "1.29.0")]
    pub fn set_span(&mut self, span: Span) {
        self.0.set_span(span.0);
    }

    /// Returns a `Span` that is a subset of `self.span()` containing only the source bytes in range `range`. <br>返回 `Span`，它是 `self.span()` 的子集，仅包含范围 `range` 中的源字节。<br>
    /// Returns `None` if the would-be trimmed span is outside the bounds of `self`. <br>如果要修剪的跨度超出 `self` 的边界，则返回 `None`。<br>
    ///
    // FIXME(SergioBenitez): check that the byte range starts and ends at a UTF-8 boundary of the source. <br>检查字节范围是否在源的 UTF-8 边界开始和结束。<br>
    // otherwise, it's likely that a panic will occur elsewhere when the source text is printed. <br>否则，在打印源文本时，panic 可能会出现在其他位置。<br>
    // FIXME(SergioBenitez): there is no way for the user to know what `self.span()` actually maps to, so this method can currently only be called blindly. <br>用户无法知道 `self.span()` 实际映射成什么，因此当前只能盲目调用此方法。<br>
    // For example, `to_string()` for the character 'c' returns "'\u{63}'"; <br>例如，字符 'c' 的 `to_string()` 返回 "'\u{63}'"；<br> there is no way for the user to know whether the source text was 'c' or whether it was '\u{63}'. <br>用户无法知道源文本是 'c' 还是 '\u{63}'。<br>
    //
    //
    //
    //
    #[unstable(feature = "proc_macro_span", issue = "54725")]
    pub fn subspan<R: RangeBounds<usize>>(&self, range: R) -> Option<Span> {
        self.0.subspan(range.start_bound().cloned(), range.end_bound().cloned()).map(Span)
    }
}

/// Parse a single literal from its stringified representation. <br>从字符串化表示中解析单个字面量。<br>
///
/// In order to parse successfully, the input string must not contain anything but the literal token. <br>为了成功解析，输入字符串不能包含除字面量 token 之外的任何内容。<br>
///
/// Specifically, it must not contain whitespace or comments in addition to the literal. <br>具体来说，它不能包含除字面量之外的空格或注释。<br>
///
/// The resulting literal token will have a `Span::call_site()` span. <br>生成的字面量 token 将具有 `Span::call_site()` span。<br>
///
/// NOTE: some errors may cause panics instead of returning `LexError`. <br>某些错误可能导致 panics 而不是返回 `LexError`。<br>
/// We reserve the right to change these errors into `LexError`s later. <br>我们保留稍后将这些错误更改为 LexError 的权利。<br>
#[stable(feature = "proc_macro_literal_parse", since = "1.54.0")]
impl FromStr for Literal {
    type Err = LexError;

    fn from_str(src: &str) -> Result<Self, LexError> {
        match bridge::client::Literal::from_str(src) {
            Ok(literal) => Ok(Literal(literal)),
            Err(()) => Err(LexError),
        }
    }
}

// N.B., the bridge only provides `to_string`, implement `fmt::Display` based on it (the reverse of the usual relationship between the two). <br>注意，网桥仅提供 `to_string`，基于它实现 `fmt::Display` (两者之间通常的关系相反)。<br>
//
#[stable(feature = "proc_macro_lib", since = "1.15.0")]
impl ToString for Literal {
    fn to_string(&self) -> String {
        self.0.to_string()
    }
}

/// Prints the literal as a string that should be losslessly convertible back into the same literal (except for possible rounding for floating point literals). <br>将字面量打印为一个字符串，该字符串应可以无损地转换回相同的字面量 (但浮点型字面量的可能取整除外)。<br>
///
#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Display for Literal {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str(&self.to_string())
    }
}

#[stable(feature = "proc_macro_lib2", since = "1.29.0")]
impl fmt::Debug for Literal {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.0.fmt(f)
    }
}

/// Tracked access to environment variables. <br>跟踪对环境变量的访问。<br>
#[unstable(feature = "proc_macro_tracked_env", issue = "74690")]
pub mod tracked_env {
    use std::env::{self, VarError};
    use std::ffi::OsStr;

    /// Retrieve an environment variable and add it to build dependency info. <br>检索环境变量并将其添加以构建依赖项信息。<br>
    /// Build system executing the compiler will know that the variable was accessed during compilation, and will be able to rerun the build when the value of that variable changes. <br>执行编译器的构建系统将知道在编译期间访问了该变量，并且能够在该变量的值发生变化时重新运行构建。<br>
    ///
    /// Besides the dependency tracking this function should be equivalent to `env::var` from the standard library, except that the argument must be UTF-8. <br>除了依赖项跟踪之外，此函数应与标准库中的 `env::var` 等效，除了参数必须为 UTF-8。<br>
    ///
    #[unstable(feature = "proc_macro_tracked_env", issue = "74690")]
    pub fn var<K: AsRef<OsStr> + AsRef<str>>(key: K) -> Result<String, VarError> {
        let key: &str = key.as_ref();
        let value = env::var(key);
        crate::bridge::client::FreeFunctions::track_env_var(key, value.as_deref().ok());
        value
    }
}

/// Tracked access to additional files. <br>跟踪对其他文件的访问。<br>
#[unstable(feature = "track_path", issue = "73921")]
pub mod tracked_path {

    /// Track a file explicitly. <br>明确跟踪文件。<br>
    ///
    /// Commonly used for tracking asset preprocessing. <br>通常用于跟踪资源预处理。<br>
    #[unstable(feature = "track_path", issue = "73921")]
    pub fn path<P: AsRef<str>>(path: P) {
        let path: &str = path.as_ref();
        crate::bridge::client::FreeFunctions::track_path(path);
    }
}
