//! Collection types. <br>集合类型。<br>
//!
//! Rust's standard collection library provides efficient implementations of the most common general purpose programming data structures. <br>Rust 的标准集合库提供了最常见的通用编程数据结构的有效实现。<br> By using the standard implementations, it should be possible for two libraries to communicate without significant data conversion. <br>通过使用标准实现，两个库在不进行大量数据转换的情况下就可以进行通信。<br>
//!
//! To get this out of the way: you should probably just use [`Vec`] or [`HashMap`]. <br>为了避免这种情况：您可能应该只使用 [`Vec`] 或 [`HashMap`]。<br>
//! These two collections cover most use cases for generic data storage and processing. <br>这两个集合涵盖了泛型数据存储和处理的大多数用例。<br> They are exceptionally good at doing what they do. <br>他们非常擅长于做自己的工作。<br> All the other collections in the standard library have specific use cases where they are the optimal choice, but these cases are borderline *niche* in comparison. <br>标准库中的所有其他集合都具有特定的用例，在这些用例中它们是最佳选择，但相比之下，这些用例是边缘性的。<br>
//! Even when `Vec` and `HashMap` are technically suboptimal, they're probably a good enough choice to get started. <br>即使 `Vec` 和 `HashMap` 在技术上不是最佳选择，它们也可能是入门的足够好选择。<br>
//!
//! Rust's collections can be grouped into four major categories: <br>Rust 的集合可以分为四个主要类别：<br>
//!
//! * Sequences: [`Vec`], [`VecDeque`], [`LinkedList`] <br>序列: [`Vec`]、[`VecDeque`]、[`LinkedList`]<br>
//! * Maps: [`HashMap`], [`BTreeMap`]
//! * Sets: [`HashSet`], [`BTreeSet`] <br>集合: [`HashSet`]、[`BTreeSet`]<br>
//! * Misc: [`BinaryHeap`] <br>混杂: [`BinaryHeap`]<br>
//!
//! # When Should You Use Which Collection? <br>什么时候应该使用哪个集合？<br>
//!
//! These are fairly high-level and quick break-downs of when each collection should be considered. <br>这些是应该何时考虑每个集合的较高层次且快速的细分。<br> Detailed discussions of strengths and weaknesses of individual collections can be found on their own documentation pages. <br>各个集合的优缺点的详细讨论可以在他们自己的文档页面上找到。<br>
//!
//! ### Use a `Vec` when: <br>在以下情况下，请使用 `Vec`：<br>
//! * You want to collect items up to be processed or sent elsewhere later, and don't care about any properties of the actual values being stored. <br>您想要收集项以供以后处理或发送到其他地方，而不必关心所存储的实际值的任何属性。<br>
//! * You want a sequence of elements in a particular order, and will only be appending to (or near) the end. <br>您需要一个按特定顺序排列的元素序列，并且只会追加到末尾 (或接近末尾)。<br>
//! * You want a stack. <br>您想要一个栈。<br>
//! * You want a resizable array. <br>您需要一个可调整大小的数组。<br>
//! * You want a heap-allocated array. <br>您需要一个堆分配的数组。<br>
//!
//! ### Use a `VecDeque` when: <br>在以下情况下，请使用 `VecDeque`：<br>
//! * You want a [`Vec`] that supports efficient insertion at both ends of the sequence. <br>您需要 [`Vec`] 支持序列两端的有效插入。<br>
//! * You want a queue. <br>您想要一个队列。<br>
//! * You want a double-ended queue (deque). <br>您需要一个双端队列 (deque)。<br>
//!
//! ### Use a `LinkedList` when: <br>在以下情况下，请使用 `LinkedList`：<br>
//! * You want a [`Vec`] or [`VecDeque`] of unknown size, and can't tolerate amortization. <br>您需要一个未知大小的 [`Vec`] 或 [`VecDeque`]，并且不能容忍摊销。<br>
//! * You want to efficiently split and append lists. <br>您想有效地拆分和追加列表。<br>
//! * You are *absolutely* certain you *really*, *truly*, want a doubly linked list. <br>您确实，想要一个双向链表。<br>
//!
//! ### Use a `HashMap` when: <br>在以下情况下，请使用 `HashMap`：<br>
//! * You want to associate arbitrary keys with an arbitrary value. <br>您想要将任意键与任意值相关联。<br>
//! * You want a cache. <br>您需要一个缓存。<br>
//! * You want a map, with no extra functionality. <br>您需要一个没有额外功能的 map。<br>
//!
//! ### Use a `BTreeMap` when: <br>在以下情况下，请使用 `BTreeMap`：<br>
//! * You want a map sorted by its keys. <br>您需要一个按其键排序的 map。<br>
//! * You want to be able to get a range of entries on-demand. <br>您希望能够按需获取一系列条目。<br>
//! * You're interested in what the smallest or largest key-value pair is. <br>您对最小或最大键值对是什么感兴趣。<br>
//! * You want to find the largest or smallest key that is smaller or larger than something. <br>您想要找到小于或大于某项的最大或最小键。<br>
//!
//! ### Use the `Set` variant of any of these `Map`s when: <br>在以下情况下，请使用任何这些 `Map` 的 `Set` 变体：<br>
//! * You just want to remember which keys you've seen. <br>您只想记住您所看到的键。<br>
//! * There is no meaningful value to associate with your keys. <br>与您的键关联没有任何有意义的值。<br>
//! * You just want a set. <br>您只想要一个 set。<br>
//!
//! ### Use a `BinaryHeap` when: <br>在以下情况下，请使用 `BinaryHeap`：<br>
//!
//! * You want to store a bunch of elements, but only ever want to process the "biggest" or "most important" one at any given time. <br>您想存储一堆元素，但只想在任何给定时间处理 "biggest" 或 "most important"。<br>
//! * You want a priority queue. <br>您需要一个优先级队列。<br>
//!
//! # Performance
//!
//! Choosing the right collection for the job requires an understanding of what each collection is good at. <br>为工作选择合适的集合需要了解每个集合的特长。<br> Here we briefly summarize the performance of different collections for certain important operations. <br>在这里，我们简要总结了某些重要操作的不同集合的性能。<br>
//! For further details, see each type's documentation, and note that the names of actual methods may differ from the tables below on certain collections. <br>有关更多详细信息，请参见每种类型的文档，并请注意，实际方法的名称可能与某些集合中的下表有所不同。<br>
//!
//! Throughout the documentation, we will follow a few conventions. <br>在整个文档中，我们将遵循一些约定。<br> For all operations, the collection's size is denoted by n. <br>对于所有操作，集合的大小用 n 表示。<br> If another collection is involved in the operation, it contains m elements. <br>如果该操作涉及另一个集合，则它包含 m 个元素。<br> Operations which have an *amortized* cost are suffixed with a `*`. <br>具有 `amortized` 成本的操作以 `*` 为后缀。<br> Operations with an *expected* cost are suffixed with a `~`. <br>具有 `expected` 成本的操作以 `~` 为后缀。<br>
//!
//! All amortized costs are for the potential need to resize when capacity is exhausted. <br>所有摊销成本均用于容量耗尽时可能需要调整的大小。<br> If a resize occurs it will take *O*(*n*) time. <br>如果发生调整大小，则将花费 *O*(*n*) 时间。<br> Our collections never automatically shrink, so removal operations aren't amortized. <br>我们的集合永远不会自动收缩，因此移除操作不会摊销。<br> Over a sufficiently large series of operations, the average cost per operation will deterministically equal the given cost. <br>在足够多的工序系列中，每次工序的平均成本将确定性地等于给定成本。<br>
//!
//! Only [`HashMap`] has expected costs, due to the probabilistic nature of hashing. <br>由于哈希的概率性质，只有 [`HashMap`] 具有预期的成本。<br>
//! It is theoretically possible, though very unlikely, for [`HashMap`] to experience worse performance. <br>从理论上讲，[`HashMap`] 可能会出现性能下降的情况，尽管可能性很小。<br>
//!
//! ## Sequences
//!
//! |                | get(i)                 | insert(i)               | remove(i)              | append    | split_off(i)           |
//! |----------------|------------------------|-------------------------|------------------------|-----------|------------------------|
//! | [`Vec`]        | *O*(1)                 | *O*(*n*-*i*)*           | *O*(*n*-*i*)           | *O*(*m*)* | *O*(*n*-*i*)           |
//! | [`VecDeque`]   | *O*(1)                 | *O*(min(*i*, *n*-*i*))* | *O*(min(*i*, *n*-*i*)) | *O*(*m*)* | *O*(min(*i*, *n*-*i*)) |
//! | [`LinkedList`] | *O*(min(*i*, *n*-*i*)) | *O*(min(*i*, *n*-*i*))  | *O*(min(*i*, *n*-*i*)) | *O*(1)    | *O*(min(*i*, *n*-*i*)) |
//!
//! Note that where ties occur, [`Vec`] is generally going to be faster than [`VecDeque`], and [`VecDeque`] is generally going to be faster than [`LinkedList`]. <br>请注意，在发生联系的地方，[`Vec`] 通常比 [`VecDeque`] 快，而 [`VecDeque`] 通常比 [`LinkedList`] 快。<br>
//!
//! ## Maps
//!
//! For Sets, all operations have the cost of the equivalent Map operation. <br>对于集合，所有操作的成本均等价于 Map 操作。<br>
//!
//! |              | get           | insert        | remove        | range         | append       |
//! |--------------|---------------|---------------|---------------|---------------|--------------|
//! | [`HashMap`]  | *O*(1)~       | *O*(1)~*      | *O*(1)~       | N/A           | N/A          |
//! | [`BTreeMap`] | *O*(log(*n*)) | *O*(log(*n*)) | *O*(log(*n*)) | *O*(log(*n*)) | *O*(*n*+*m*) |
//!
//! # Correct and Efficient Usage of Collections <br>正确有效地使用集合<br>
//!
//! Of course, knowing which collection is the right one for the job doesn't instantly permit you to use it correctly. <br>当然，知道哪种集合最适合该工作并不能立即使您正确使用它。<br> Here are some quick tips for efficient and correct usage of the standard collections in general. <br>以下是一般有效和正确使用标准集合的一些快速技巧。<br> If you're interested in how to use a specific collection in particular, consult its documentation for detailed discussion and code examples. <br>如果您特别对如何使用特定集合感兴趣，请查阅其文档以获取详细的讨论和代码示例。<br>
//!
//! ## Capacity Management <br>容量管理<br>
//!
//! Many collections provide several constructors and methods that refer to "capacity". <br>许多集合提供了一些引用 "capacity" 的构造函数和方法。<br> These collections are generally built on top of an array. <br>这些集合通常构建在数组的顶部。<br>
//! Optimally, this array would be exactly the right size to fit only the elements stored in the collection, but for the collection to do this would be very inefficient. <br>理想情况下，此数组的大小恰好适合仅适合存储在集合中的元素，但是对于集合而言，这样做效率很低。<br> If the backing array was exactly the right size at all times, then every time an element is inserted, the collection would have to grow the array to fit it. <br>如果支持数组始终正确大小，则每次插入元素时，集合都必须增大数组以适合它。<br>
//! Due to the way memory is allocated and managed on most computers, this would almost surely require allocating an entirely new array and copying every single element from the old one into the new one. <br>由于在大多数计算机上分配和管理内存的方式，几乎肯定需要分配一个全新的数组，并将每个元素从旧元素复制到新元素中。<br>
//! Hopefully you can see that this wouldn't be very efficient to do on every operation. <br>希望您能看到这样做并不是在每个操作上都非常有效。<br>
//!
//! Most collections therefore use an *amortized* allocation strategy. <br>因此，大多数集合使用 `amortized` 分配策略。<br> They generally let themselves have a fair amount of unoccupied space so that they only have to grow on occasion. <br>它们通常让自己拥有大量的空闲空间，因此它们仅需要偶尔增长。<br> When they do grow, they allocate a substantially larger array to move the elements into so that it will take a while for another grow to be required. <br>当它们确实增长时，它们会分配一个更大的数组来将元素移入，以便需要一段时间才能进行另一个增长。<br>
//! While this strategy is great in general, it would be even better if the collection *never* had to resize its backing array. <br>尽管此策略总体上不错，但如果必须 * 绝不调整其支持数组的大小，那就更好了。<br>
//! Unfortunately, the collection itself doesn't have enough information to do this itself. <br>不幸的是，集合本身没有足够的信息来执行此操作。<br>
//! Therefore, it is up to us programmers to give it hints. <br>因此，由我们的程序员来提供提示。<br>
//!
//! Any `with_capacity` constructor will instruct the collection to allocate enough space for the specified number of elements. <br>任何 `with_capacity` 构造函数都将指示该集合为指定数量的元素分配足够的空间。<br> Ideally this will be for exactly that many elements, but some implementation details may prevent this. <br>理想情况下，这将恰好适用于许多元素，但是某些实现细节可能会阻止这种情况。<br> See collection-specific documentation for details. <br>有关详细信息，请参见特定于集合的文档。<br> In general, use `with_capacity` when you know exactly how many elements will be inserted, or at least have a reasonable upper-bound on that number. <br>通常，当您确切知道要插入多少个元素，或者至少在该数目上具有合理的上限时，请使用 `with_capacity`。<br>
//!
//! When anticipating a large influx of elements, the `reserve` family of methods can be used to hint to the collection how much room it should make for the coming items. <br>当预期大量元素涌入时，`reserve` 系列方法可用于向集合暗示应该为即将到来的项目腾出多少空间。<br> As with `with_capacity`, the precise behavior of these methods will be specific to the collection of interest. <br>与 `with_capacity` 一样，这些方法的精确行为将特定于感兴趣的集合。<br>
//!
//! For optimal performance, collections will generally avoid shrinking themselves. <br>为了获得最佳性能，集合通常会避免自身收缩。<br> If you believe that a collection will not soon contain any more elements, or just really need the memory, the `shrink_to_fit` method prompts the collection to shrink the backing array to the minimum size capable of holding its elements. <br>如果您认为某个集合将很快不再包含任何其他元素，或者仅真的需要内存，则 `shrink_to_fit` 方法将提示该集合将后备数组缩小到能够容纳其元素的最小大小。<br>
//!
//! Finally, if ever you're interested in what the actual capacity of the collection is, most collections provide a `capacity` method to query this information on demand. <br>最后，如果您对集合的实际容量感兴趣，则大多数集合都会提供 `capacity` 方法来按需查询此信息。<br> This can be useful for debugging purposes, or for use with the `reserve` methods. <br>这对于调试目的或与 `reserve` 方法一起使用可能很有用。<br>
//!
//! ## Iterators
//!
//! Iterators are a powerful and robust mechanism used throughout Rust's standard libraries. <br>迭代器是 Rust 的标准库中使用的一种强大而健壮的机制。<br>
//! Iterators provide a sequence of values in a generic, safe, efficient and convenient way. <br>迭代器以泛型，安全，有效和方便的方式提供一系列值。<br> The contents of an iterator are usually *lazily* evaluated, so that only the values that are actually needed are ever actually produced, and no allocation need be done to temporarily store them. <br>迭代器的内容通常是惰性求值的，因此只有实际需要的值才会实际产生，而不需要进行任何分配来临时存储它们。<br> Iterators are primarily consumed using a `for` loop, although many functions also take iterators where a collection or sequence of values is desired. <br>迭代器主要通过 `for` 循环消耗，尽管许多函数也将迭代器用于需要集合或值序列的地方。<br>
//!
//! All of the standard collections provide several iterators for performing bulk manipulation of their contents. <br>所有标准集合都提供了几个用于对其内容进行批量操作的迭代器。<br> The three primary iterators almost every collection should provide are `iter`, `iter_mut`, and `into_iter`. <br>几乎每个集合应提供的三个主要迭代器是 `iter`，`iter_mut` 和 `into_iter`。<br>
//! Some of these are not provided on collections where it would be unsound or unreasonable to provide them. <br>其中的某些未在集合中提供，因此提供它们是不合理的或不合理的。<br>
//!
//! `iter` provides an iterator of immutable references to all the contents of a collection in the most "natural" order. <br>`iter` 提供了一个迭代器，以最自然的顺序对集合的所有内容进行不可变的引用。<br>
//! For sequence collections like [`Vec`], this means the items will be yielded in increasing order of index starting at 0. <br>对于像 [`Vec`] 这样的序列集合，这意味着该项将以索引从 0 开始的递增顺序产生。<br> For ordered collections like [`BTreeMap`], this means that the items will be yielded in sorted order. <br>对于 [`BTreeMap`] 之类的有序集合，这意味着该项将按排序顺序产生。<br>
//! For unordered collections like [`HashMap`], the items will be yielded in whatever order the internal representation made most convenient. <br>对于 [`HashMap`] 之类的无序集合，将以内部表示最方便的任何顺序产生该项。<br> This is great for reading through all the contents of the collection. <br>这对于通读集合的所有内容非常有用。<br>
//!
//! ```
//! let vec = vec![1, 2, 3, 4];
//! for x in vec.iter() {
//!    println!("vec contained {x:?}");
//! }
//! ```
//!
//! `iter_mut` provides an iterator of *mutable* references in the same order as `iter`. <br>`iter_mut` 提供了一个可变引用的迭代器，其顺序与 `iter` 相同。<br> This is great for mutating all the contents of the collection. <br>这对于更改集合的所有内容非常有用。<br>
//!
//! ```
//! let mut vec = vec![1, 2, 3, 4];
//! for x in vec.iter_mut() {
//!    *x += 1;
//! }
//! ```
//!
//! `into_iter` transforms the actual collection into an iterator over its contents by-value. <br>`into_iter` 将实际集合转换为按值对其内容进行迭代的迭代器。<br> This is great when the collection itself is no longer needed, and the values are needed elsewhere. <br>当不再需要集合本身并且在其他地方需要这些值时，这非常好。<br> Using `extend` with `into_iter` is the main way that contents of one collection are moved into another. <br>将 `extend` 与 `into_iter` 一起使用是将一个集合的内容移入另一个集合的主要方法。<br>
//! `extend` automatically calls `into_iter`, and takes any <code>T: [IntoIterator]</code>. <br>`extend` 自动调用 `into_iter`，并取任何 <code>T: [IntoIterator]</code>。<br>
//! Calling `collect` on an iterator itself is also a great way to convert one collection into another. <br>在迭代器本身上调用 `collect` 也是将一个集合转换为另一个集合的好方法。<br> Both of these methods should internally use the capacity management tools discussed in the previous section to do this as efficiently as possible. <br>这两种方法都应在内部使用上一节中讨论的容量管理工具来尽可能高效地执行此操作。<br>
//!
//! ```
//! let mut vec1 = vec![1, 2, 3, 4];
//! let vec2 = vec![10, 20, 30, 40];
//! vec1.extend(vec2);
//! ```
//!
//! ```
//! use std::collections::VecDeque;
//!
//! let vec = [1, 2, 3, 4];
//! let buf: VecDeque<_> = vec.into_iter().collect();
//! ```
//!
//! Iterators also provide a series of *adapter* methods for performing common threads to sequences. <br>迭代器还提供了一系列 *adapter* 方法，用于对序列执行通用线程。<br>
//! Among the adapters are functional favorites like `map`, `fold`, `skip` and `take`. <br>在适配器中，有功能最喜欢的适配器，例如 `map`，`fold`，`skip` 和 `take`。<br> Of particular interest to collections is the `rev` adapter, which reverses any iterator that supports this operation. <br>集合特别感兴趣的是 `rev` 适配器，它反转支持此操作的任何迭代器。<br> Most collections provide reversible iterators as the way to iterate over them in reverse order. <br>大多数集合都提供可逆迭代器，作为以相反顺序对其进行迭代的方式。<br>
//!
//! ```
//! let vec = vec![1, 2, 3, 4];
//! for x in vec.iter().rev() {
//!    println!("vec contained {x:?}");
//! }
//! ```
//!
//! Several other collection methods also return iterators to yield a sequence of results but avoid allocating an entire collection to store the result in. <br>其他几种集合方法还返回迭代器以产生结果序列，但避免分配整个集合来存储结果。<br>
//! This provides maximum flexibility as `collect` or `extend` can be called to "pipe" the sequence into any collection if desired. <br>这提供了最大的灵活性，因为可以根据需要将 `collect` 或 `extend` 调用到 "pipe" 序列中的任何集合中。<br> Otherwise, the sequence can be looped over with a `for` loop. <br>否则，可以使用 `for` 循环将序列循环。<br> The iterator can also be discarded after partial use, preventing the computation of the unused items. <br>迭代器在部分使用后也可以丢弃，从而避免了未使用项的计算。<br>
//!
//! ## Entries
//!
//! The `entry` API is intended to provide an efficient mechanism for manipulating the contents of a map conditionally on the presence of a key or not. <br>`entry` API 旨在提供一种有效的机制，用于根据键的存在与否来有条件地操作 map 的内容。<br> The primary motivating use case for this is to provide efficient accumulator maps. <br>这样做的主要动机是提供有效的累加器 maps。<br> For instance, if one wishes to maintain a count of the number of times each key has been seen, they will have to perform some conditional logic on whether this is the first time the key has been seen or not. <br>例如，如果希望保持对每个键被看到的次数的计数，则他们将必须执行一些条件逻辑来确定这是否是第一次被看到。<br>
//!
//! Normally, this would require a `find` followed by an `insert`, effectively duplicating the search effort on each insertion. <br>通常，这需要先插入 `find`，然后再插入 `insert`，以有效地重复每次插入时的搜索工作。<br>
//!
//! When a user calls `map.entry(key)`, the map will search for the key and then yield a variant of the `Entry` enum. <br>当用户调用 `map.entry(key)` 时，map 将搜索键，然后生成 `Entry` 枚举的变体。<br>
//!
//! If a `Vacant(entry)` is yielded, then the key *was not* found. <br>如果产生 `Vacant(entry)`，则没有找到键。<br> In this case the only valid operation is to `insert` a value into the entry. <br>在这种情况下，唯一有效的操作是将值 `insert` 放入条目中。<br> When this is done, the vacant entry is consumed and converted into a mutable reference to the value that was inserted. <br>完成此操作后，将消耗空条目并将其转换为所插入值的变量引用。<br> This allows for further manipulation of the value beyond the lifetime of the search itself. <br>这允许对超出搜索本身生命周期的值进行进一步操作。<br>
//! This is useful if complex logic needs to be performed on the value regardless of whether the value was just inserted. <br>如果无论是否插入值，都需要对值执行复杂的逻辑，这将很有用。<br>
//!
//! If an `Occupied(entry)` is yielded, then the key *was* found. <br>如果产生 `Occupied(entry)`，则找到键 *。<br>
//! In this case, the user has several options: they can `get`, `insert` or `remove` the value of the occupied entry. <br>在这种情况下，用户有几个选项：他们可以 `get`，`insert` 或 `remove` 占用的条目的值。<br> Additionally, they can convert the occupied entry into a mutable reference to its value, providing symmetry to the vacant `insert` case. <br>此外，他们可以将占用的条目转换为其值的变量引用，从而为空缺的 `insert` 情况提供对称性。<br>
//!
//! ### Examples
//!
//! Here are the two primary ways in which `entry` is used. <br>这是使用 `entry` 的两种主要方法。<br> First, a simple example where the logic performed on the values is trivial. <br>首先，一个简单的示例，其中对值执行的逻辑很简单。<br>
//!
//! #### Counting the number of times each character in a string occurs <br>计算字符串中每个字符出现的次数<br>
//!
//! ```
//! use std::collections::btree_map::BTreeMap;
//!
//! let mut count = BTreeMap::new();
//! let message = "she sells sea shells by the sea shore";
//!
//! for c in message.chars() {
//!     *count.entry(c).or_insert(0) += 1;
//! }
//!
//! assert_eq!(count.get(&'s'), Some(&8));
//!
//! println!("Number of occurrences of each character");
//! for (char, count) in &count {
//!     println!("{char}: {count}");
//! }
//! ```
//!
//! When the logic to be performed on the value is more complex, we may simply use the `entry` API to ensure that the value is initialized and perform the logic afterwards. <br>当要对该值执行的逻辑比较复杂时，我们可以简单地使用 `entry` API 来确保该值已初始化，然后再执行该逻辑。<br>
//!
//! #### Tracking the inebriation of customers at a bar <br>跟踪酒吧顾客的醉酒情况<br>
//!
//! ```
//! use std::collections::btree_map::BTreeMap;
//!
//! // A client of the bar. <br>酒吧客户端。<br> They have a blood alcohol level. <br>他们有血液中的酒精含量。<br>
//! struct Person { blood_alcohol: f32 }
//!
//! // All the orders made to the bar, by client ID. <br>所有的订单都是按客户 ID 送到酒吧的。<br>
//! let orders = vec![1, 2, 1, 2, 3, 4, 1, 2, 2, 3, 4, 1, 1, 1];
//!
//! // Our clients. <br>我们的客户。<br>
//! let mut blood_alcohol = BTreeMap::new();
//!
//! for id in orders {
//!     // If this is the first time we've seen this customer, initialize them with no blood alcohol. <br>如果这是我们第一次见到此客户，请在不使用酒精的情况下初始化他们。<br>
//!     // Otherwise, just retrieve them. <br>否则，只需检索它们。<br>
//!     let person = blood_alcohol.entry(id).or_insert(Person { blood_alcohol: 0.0 });
//!
//!     // Reduce their blood alcohol level. <br>降低他们的血液酒精含量。<br> It takes time to order and drink a beer! <br>点餐和喝啤酒需要时间！<br>
//!     person.blood_alcohol *= 0.9;
//!
//!     // Check if they're sober enough to have another beer. <br>检查他们是否足够清醒，可以再喝一杯啤酒。<br>
//!     if person.blood_alcohol > 0.3 {
//!         // Too drunk... <br>太醉了...<br> for now. <br>现在。<br>
//!         println!("Sorry {id}, I have to cut you off");
//!     } else {
//!         // Have another! <br>再来一杯！<br>
//!         person.blood_alcohol += 0.1;
//!     }
//! }
//! ```
//!
//! # Insert and complex keys <br>插入复杂的键<br>
//!
//! If we have a more complex key, calls to `insert` will not update the value of the key. <br>如果我们拥有更复杂的键，则对 `insert` 的调用将不会更新键的值。<br> For example: <br>例如：<br>
//!
//! ```
//! use std::cmp::Ordering;
//! use std::collections::BTreeMap;
//! use std::hash::{Hash, Hasher};
//!
//! #[derive(Debug)]
//! struct Foo {
//!     a: u32,
//!     b: &'static str,
//! }
//!
//! // we will compare `Foo`s by their `a` value only. <br>我们将仅通过它们的 `a` 值来比较 `Foo`。<br>
//! impl PartialEq for Foo {
//!     fn eq(&self, other: &Self) -> bool { self.a == other.a }
//! }
//!
//! impl Eq for Foo {}
//!
//! // we will hash `Foo`s by their `a` value only. <br>我们将仅通过 `a` 值对 Foo 进行哈希处理。<br>
//! impl Hash for Foo {
//!     fn hash<H: Hasher>(&self, h: &mut H) { self.a.hash(h); }
//! }
//!
//! impl PartialOrd for Foo {
//!     fn partial_cmp(&self, other: &Self) -> Option<Ordering> { self.a.partial_cmp(&other.a) }
//! }
//!
//! impl Ord for Foo {
//!     fn cmp(&self, other: &Self) -> Ordering { self.a.cmp(&other.a) }
//! }
//!
//! let mut map = BTreeMap::new();
//! map.insert(Foo { a: 1, b: "baz" }, 99);
//!
//! // We already have a Foo with an a of 1, so this will be updating the value. <br>我们已经有一个 a 等于 1 的 Foo，因此这将更新该值。<br>
//! map.insert(Foo { a: 1, b: "xyz" }, 100);
//!
//! // The value has been updated... <br>该值已更新...<br>
//! assert_eq!(map.values().next().unwrap(), &100);
//!
//! // ...but the key hasn't changed. <br>... 但是键没有变。<br> b is still "baz", not "xyz". <br>b 仍然是 "baz"，而不是 "xyz"。<br>
//! assert_eq!(map.keys().next().unwrap().b, "baz");
//! ```
//!
//! [IntoIterator]: crate::iter::IntoIterator "iter::IntoIterator"
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

#[stable(feature = "rust1", since = "1.0.0")]
// FIXME(#82080) The deprecation here is only theoretical, and does not actually produce a warning. <br>这里的弃用只是理论上的，实际上并不会产生警告。<br>
#[deprecated(note = "moved to `std::ops::Bound`", since = "1.26.0")]
#[doc(hidden)]
pub use crate::ops::Bound;

#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::collections::{binary_heap, btree_map, btree_set};
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::collections::{linked_list, vec_deque};
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::collections::{BTreeMap, BTreeSet, BinaryHeap};
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::collections::{LinkedList, VecDeque};

#[stable(feature = "rust1", since = "1.0.0")]
pub use self::hash_map::HashMap;
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::hash_set::HashSet;

#[stable(feature = "try_reserve", since = "1.57.0")]
pub use alloc_crate::collections::TryReserveError;
#[unstable(
    feature = "try_reserve_kind",
    reason = "Uncertain how much info should be exposed",
    issue = "48043"
)]
pub use alloc_crate::collections::TryReserveErrorKind;

mod hash;

#[stable(feature = "rust1", since = "1.0.0")]
pub mod hash_map {
    //! A hash map implemented with quadratic probing and SIMD lookup. <br>通过二次探测和 SIMD 查找实现的哈希 map。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::hash::map::*;
}

#[stable(feature = "rust1", since = "1.0.0")]
pub mod hash_set {
    //! A hash set implemented as a `HashMap` where the value is `()`. <br>实现为 `HashMap` 的哈希集，其中值为 `()`。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::hash::set::*;
}
