//! Inspection and manipulation of the process's environment. <br>检查和操作进程的环境。<br>
//!
//! This module contains functions to inspect various aspects such as environment variables, process arguments, the current directory, and various other important directories. <br>该模块包含用于检查各个方面的函数，例如环境变量，进程参数，当前目录以及各种其他重要目录。<br>
//!
//!
//! There are several functions and structs in this module that have a counterpart ending in `os`. <br>该模块中有几个函数和结构体，它们的对应部分以 `os` 结尾。<br>
//! Those ending in `os` will return an [`OsString`] and those without will return a [`String`]. <br>以 `os` 结尾的那些将返回 [`OsString`]，而没有以 `os` 结尾的那些将返回 [`String`]。<br>
//!
//!

#![stable(feature = "env", since = "1.0.0")]

#[cfg(test)]
mod tests;

use crate::error::Error;
use crate::ffi::{OsStr, OsString};
use crate::fmt;
use crate::io;
use crate::path::{Path, PathBuf};
use crate::sys;
use crate::sys::os as os_imp;

/// Returns the current working directory as a [`PathBuf`]. <br>返回当前的工作目录为 [`PathBuf`]。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function [currently] corresponds to the `getcwd` function on Unix and the `GetCurrentDirectoryW` function on Windows. <br>这个函数 [currently] 对应于 Unix 上的 `getcwd` 函数和 Windows 上的 `GetCurrentDirectoryW` 函数。<br>
///
///
/// [currently]: crate::io#platform-specific-behavior
///
/// # Errors
///
/// Returns an [`Err`] if the current working directory value is invalid. <br>如果当前工作目录值无效，则返回 [`Err`]。<br>
/// Possible cases: <br>可能的情况：<br>
///
/// * Current directory does not exist. <br>当前目录不存在。<br>
/// * There are insufficient permissions to access the current directory. <br>没有足够的权限来访问当前目录。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// fn main() -> std::io::Result<()> {
///     let path = env::current_dir()?;
///     println!("The current directory is {}", path.display());
///     Ok(())
/// }
/// ```
#[doc(alias = "pwd")]
#[doc(alias = "getcwd")]
#[doc(alias = "GetCurrentDirectory")]
#[stable(feature = "env", since = "1.0.0")]
pub fn current_dir() -> io::Result<PathBuf> {
    os_imp::getcwd()
}

/// Changes the current working directory to the specified path. <br>将当前工作目录更改为指定的路径。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function [currently] corresponds to the `chdir` function on Unix and the `SetCurrentDirectoryW` function on Windows. <br>这个函数 [currently] 对应于 Unix 上的 `chdir` 函数和 Windows 上的 `SetCurrentDirectoryW` 函数。<br>
///
///
/// Returns an [`Err`] if the operation fails. <br>如果操作失败，则返回 [`Err`]。<br>
///
/// [currently]: crate::io#platform-specific-behavior
///
/// # Examples
///
/// ```
/// use std::env;
/// use std::path::Path;
///
/// let root = Path::new("/");
/// assert!(env::set_current_dir(&root).is_ok());
/// println!("Successfully changed working directory to {}!", root.display());
/// ```
#[doc(alias = "chdir")]
#[stable(feature = "env", since = "1.0.0")]
pub fn set_current_dir<P: AsRef<Path>>(path: P) -> io::Result<()> {
    os_imp::chdir(path.as_ref())
}

/// An iterator over a snapshot of the environment variables of this process. <br>在此进程的环境变量的快照上进行迭代的迭代器。<br>
///
/// This structure is created by [`env::vars()`]. <br>该结构体由 [`env::vars()`] 创建。<br> See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// [`env::vars()`]: vars
#[stable(feature = "env", since = "1.0.0")]
pub struct Vars {
    inner: VarsOs,
}

/// An iterator over a snapshot of the environment variables of this process. <br>在此进程的环境变量的快照上进行迭代的迭代器。<br>
///
/// This structure is created by [`env::vars_os()`]. <br>该结构体由 [`env::vars_os()`] 创建。<br> See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// [`env::vars_os()`]: vars_os
#[stable(feature = "env", since = "1.0.0")]
pub struct VarsOs {
    inner: os_imp::Env,
}

/// Returns an iterator of (variable, value) pairs of strings, for all the environment variables of the current process. <br>返回当前进程的所有环境变量的字符串 (变量，值) 对的迭代器。<br>
///
///
/// The returned iterator contains a snapshot of the process's environment variables at the time of this invocation. <br>返回的迭代器包含此调用时进程环境变量的快照。<br>
/// Modifications to environment variables afterwards will not be reflected in the returned iterator. <br>之后对环境变量的修改将不会反映在返回的迭代器中。<br>
///
/// # Panics
///
/// While iterating, the returned iterator will panic if any key or value in the environment is not valid unicode. <br>进行迭代时，如果环境中的任何键或值不是有效的 unicode，则返回的迭代器将为 panic。<br>
/// If this is not desired, consider using [`env::vars_os()`]. <br>如果不希望这样做，请考虑使用 [`env::vars_os()`]。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// // We will iterate through the references to the element returned by env::vars(); <br>我们将通过引用迭代到 env::vars () ; 返回的元素。<br>
/////
/// for (key, value) in env::vars() {
///     println!("{key}: {value}");
/// }
/// ```
///
/// [`env::vars_os()`]: vars_os
///
///
#[must_use]
#[stable(feature = "env", since = "1.0.0")]
pub fn vars() -> Vars {
    Vars { inner: vars_os() }
}

/// Returns an iterator of (variable, value) pairs of OS strings, for all the environment variables of the current process. <br>对于当前进程的所有环境变量，返回 OS 字符串对 (变量，值) 对的迭代器。<br>
///
///
/// The returned iterator contains a snapshot of the process's environment variables at the time of this invocation. <br>返回的迭代器包含此调用时进程环境变量的快照。<br>
/// Modifications to environment variables afterwards will not be reflected in the returned iterator. <br>之后对环境变量的修改将不会反映在返回的迭代器中。<br>
///
/// Note that the returned iterator will not check if the environment variables are valid Unicode. <br>请注意，返回的迭代器不会检查环境变量是否为有效的 Unicode。<br>
/// If you want to panic on invalid UTF-8, use the [`vars`] function instead. <br>如果您想在无效的 UTF-8 上使用 panic，请改用 [`vars`] 函数。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// // We will iterate through the references to the element returned by env::vars_os(); <br>我们将通过引用迭代到 env::vars_os () ; 返回的元素。<br>
/////
/// for (key, value) in env::vars_os() {
///     println!("{key:?}: {value:?}");
/// }
/// ```
///
///
#[must_use]
#[stable(feature = "env", since = "1.0.0")]
pub fn vars_os() -> VarsOs {
    VarsOs { inner: os_imp::env() }
}

#[stable(feature = "env", since = "1.0.0")]
impl Iterator for Vars {
    type Item = (String, String);
    fn next(&mut self) -> Option<(String, String)> {
        self.inner.next().map(|(a, b)| (a.into_string().unwrap(), b.into_string().unwrap()))
    }
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.inner.size_hint()
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Vars {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Vars").finish_non_exhaustive()
    }
}

#[stable(feature = "env", since = "1.0.0")]
impl Iterator for VarsOs {
    type Item = (OsString, OsString);
    fn next(&mut self) -> Option<(OsString, OsString)> {
        self.inner.next()
    }
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.inner.size_hint()
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for VarsOs {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("VarOs").finish_non_exhaustive()
    }
}

/// Fetches the environment variable `key` from the current process. <br>从当前进程中获取环境变量 `key`。<br>
///
/// # Errors
///
/// This function will return an error if the environment variable isn't set. <br>如果未设置环境变量，此函数将返回错误。<br>
///
/// This function may return an error if the environment variable's name contains the equal sign character (`=`) or the NUL character. <br>如果环境变量的名称包含等号字符 (`=`) 或 NUL 字符，此函数可能会返回错误。<br>
///
///
/// This function will return an error if the environment variable's value is not valid Unicode. <br>如果环境变量的值不是有效的 Unicode，此函数将返回错误。<br>
/// If this is not desired, consider using [`var_os`]. <br>如果不需要，请考虑使用 [`var_os`]。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// let key = "HOME";
/// match env::var(key) {
///     Ok(val) => println!("{key}: {val:?}"),
///     Err(e) => println!("couldn't interpret {key}: {e}"),
/// }
/// ```
#[stable(feature = "env", since = "1.0.0")]
pub fn var<K: AsRef<OsStr>>(key: K) -> Result<String, VarError> {
    _var(key.as_ref())
}

fn _var(key: &OsStr) -> Result<String, VarError> {
    match var_os(key) {
        Some(s) => s.into_string().map_err(VarError::NotUnicode),
        None => Err(VarError::NotPresent),
    }
}

/// Fetches the environment variable `key` from the current process, returning [`None`] if the variable isn't set or there's another error. <br>从当前进程中获取环境变量 `key`，如果变量未设置或存在另一个错误，则返回 [`None`]。<br>
///
///
/// Note that the method will not check if the environment variable is valid Unicode. <br>请注意，该方法不会检查环境变量是否为有效的 Unicode。<br>
/// If you want to have an error on invalid UTF-8, use the [`var`] function instead. <br>如果您想在无效的 UTF-8 上出错，请改用 [`var`] 函数。<br>
///
/// # Errors
///
/// This function returns an error if the environment variable isn't set. <br>如果未设置环境变量，此函数将返回错误。<br>
///
/// This function may return an error if the environment variable's name contains the equal sign character (`=`) or the NUL character. <br>如果环境变量的名称包含等号字符 (`=`) 或 NUL 字符，此函数可能会返回错误。<br>
///
/// This function may return an error if the environment variable's value contains the NUL character. <br>如果环境变量的值包含 NUL 字符，此函数可能会返回错误。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// let key = "HOME";
/// match env::var_os(key) {
///     Some(val) => println!("{key}: {val:?}"),
///     None => println!("{key} is not defined in the environment.")
/// }
/// ```
///
///
///
#[must_use]
#[stable(feature = "env", since = "1.0.0")]
pub fn var_os<K: AsRef<OsStr>>(key: K) -> Option<OsString> {
    _var_os(key.as_ref())
}

fn _var_os(key: &OsStr) -> Option<OsString> {
    os_imp::getenv(key)
}

/// The error type for operations interacting with environment variables. <br>与环境变量交互的操作的错误类型。<br>
/// Possibly returned from [`env::var()`]. <br>可能是从 [`env::var()`] 返回的。<br>
///
/// [`env::var()`]: var
#[derive(Debug, PartialEq, Eq, Clone)]
#[stable(feature = "env", since = "1.0.0")]
pub enum VarError {
    /// The specified environment variable was not present in the current process's environment. <br>当前进程的环境中不存在指定的环境变量。<br>
    ///
    #[stable(feature = "env", since = "1.0.0")]
    NotPresent,

    /// The specified environment variable was found, but it did not contain valid unicode data. <br>找到了指定的环境变量，但是它不包含有效的 unicode 数据。<br>
    /// The found data is returned as a payload of this variant. <br>找到的数据作为该变体的有效载荷返回。<br>
    ///
    #[stable(feature = "env", since = "1.0.0")]
    NotUnicode(#[stable(feature = "env", since = "1.0.0")] OsString),
}

#[stable(feature = "env", since = "1.0.0")]
impl fmt::Display for VarError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match *self {
            VarError::NotPresent => write!(f, "environment variable not found"),
            VarError::NotUnicode(ref s) => {
                write!(f, "environment variable was not valid unicode: {:?}", s)
            }
        }
    }
}

#[stable(feature = "env", since = "1.0.0")]
impl Error for VarError {
    #[allow(deprecated)]
    fn description(&self) -> &str {
        match *self {
            VarError::NotPresent => "environment variable not found",
            VarError::NotUnicode(..) => "environment variable was not valid unicode",
        }
    }
}

/// Sets the environment variable `key` to the value `value` for the currently running process. <br>将环境变量 `key` 设置为当前正在运行的进程的值 `value`。<br>
///
/// Note that while concurrent access to environment variables is safe in Rust, some platforms only expose inherently unsafe non-threadsafe APIs for inspecting the environment. <br>请注意，虽然在 Rust 中对环境变量进行并发访问是安全的，但某些平台仅公开本质上不安全的非线程安全 API 来检查环境。<br>
/// As a result, extra care needs to be taken when auditing calls to unsafe external FFI functions to ensure that any external environment accesses are properly synchronized with accesses in Rust. <br>因此，在审核对不安全的外部 FFI 函数的调用时，需要格外小心，以确保所有外部环境访问均与 Rust 中的访问正确同步。<br>
///
///
/// Discussion of this unsafety on Unix may be found in: <br>有关 Unix 上这种不安全性的讨论，请参见：<br>
///
///  - [Austin Group Bugzilla](https://austingroupbugs.net/view.php?id=188)
///  - [GNU C library Bugzilla](https://sourceware.org/bugzilla/show_bug.cgi?id=15607#c2)
///
/// # Panics
///
/// This function may panic if `key` is empty, contains an ASCII equals sign `'='` or the NUL character `'\0'`, or when `value` contains the NUL character. <br>如果 `key` 为空，包含 ASCII 等号 `'='` 或 NUL 字符 `'\0'`，或者当 `value` 包含 NUL 字符时，此函数可能 panic。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// let key = "KEY";
/// env::set_var(key, "VALUE");
/// assert_eq!(env::var(key), Ok("VALUE".to_string()));
/// ```
///
///
///
///
#[stable(feature = "env", since = "1.0.0")]
pub fn set_var<K: AsRef<OsStr>, V: AsRef<OsStr>>(key: K, value: V) {
    _set_var(key.as_ref(), value.as_ref())
}

fn _set_var(key: &OsStr, value: &OsStr) {
    os_imp::setenv(key, value).unwrap_or_else(|e| {
        panic!("failed to set environment variable `{key:?}` to `{value:?}`: {e}")
    })
}

/// Removes an environment variable from the environment of the currently running process. <br>从当前正在运行的进程的环境中删除环境变量。<br>
///
/// Note that while concurrent access to environment variables is safe in Rust, some platforms only expose inherently unsafe non-threadsafe APIs for inspecting the environment. <br>请注意，虽然在 Rust 中对环境变量进行并发访问是安全的，但某些平台仅公开本质上不安全的非线程安全 API 来检查环境。<br>
/// As a result extra care needs to be taken when auditing calls to unsafe external FFI functions to ensure that any external environment accesses are properly synchronized with accesses in Rust. <br>因此，在审核对不安全的外部 FFI 函数的调用时，需要格外小心，以确保所有外部环境访问都与 Rust 中的访问正确同步。<br>
///
///
/// Discussion of this unsafety on Unix may be found in: <br>有关 Unix 上这种不安全性的讨论，请参见：<br>
///
///  - [Austin Group Bugzilla](https://austingroupbugs.net/view.php?id=188)
///  - [GNU C library Bugzilla](https://sourceware.org/bugzilla/show_bug.cgi?id=15607#c2)
///
/// # Panics
///
/// This function may panic if `key` is empty, contains an ASCII equals sign `'='` or the NUL character `'\0'`, or when the value contains the NUL character. <br>如果 `key` 为空，包含 ASCII 等于 `'='` 或 NUL 字符 `'\0'` 或值包含 NUL 字符，则此函数可能为 panic。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// let key = "KEY";
/// env::set_var(key, "VALUE");
/// assert_eq!(env::var(key), Ok("VALUE".to_string()));
///
/// env::remove_var(key);
/// assert!(env::var(key).is_err());
/// ```
///
///
///
///
#[stable(feature = "env", since = "1.0.0")]
pub fn remove_var<K: AsRef<OsStr>>(key: K) {
    _remove_var(key.as_ref())
}

fn _remove_var(key: &OsStr) {
    os_imp::unsetenv(key)
        .unwrap_or_else(|e| panic!("failed to remove environment variable `{key:?}`: {e}"))
}

/// An iterator that splits an environment variable into paths according to platform-specific conventions. <br>一个迭代器，该迭代器根据特定于平台的约定将环境变量拆分为路径。<br>
///
///
/// The iterator element type is [`PathBuf`]. <br>迭代器元素类型为 [`PathBuf`]。<br>
///
/// This structure is created by [`env::split_paths()`]. <br>该结构体由 [`env::split_paths()`] 创建。<br> See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// [`env::split_paths()`]: split_paths
///
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "env", since = "1.0.0")]
pub struct SplitPaths<'a> {
    inner: os_imp::SplitPaths<'a>,
}

/// Parses input according to platform conventions for the `PATH` environment variable. <br>根据平台约定对 `PATH` 环境变量解析输入。<br>
///
///
/// Returns an iterator over the paths contained in `unparsed`. <br>返回 `unparsed` 中包含的路径上的迭代器。<br>
/// The iterator element type is [`PathBuf`]. <br>迭代器元素类型为 [`PathBuf`]。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// let key = "PATH";
/// match env::var_os(key) {
///     Some(paths) => {
///         for path in env::split_paths(&paths) {
///             println!("'{}'", path.display());
///         }
///     }
///     None => println!("{key} is not defined in the environment.")
/// }
/// ```
#[stable(feature = "env", since = "1.0.0")]
pub fn split_paths<T: AsRef<OsStr> + ?Sized>(unparsed: &T) -> SplitPaths<'_> {
    SplitPaths { inner: os_imp::split_paths(unparsed.as_ref()) }
}

#[stable(feature = "env", since = "1.0.0")]
impl<'a> Iterator for SplitPaths<'a> {
    type Item = PathBuf;
    fn next(&mut self) -> Option<PathBuf> {
        self.inner.next()
    }
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.inner.size_hint()
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for SplitPaths<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("SplitPaths").finish_non_exhaustive()
    }
}

/// The error type for operations on the `PATH` variable. <br>`PATH` 变量上的操作的错误类型。<br>
/// Possibly returned from [`env::join_paths()`]. <br>可能是从 [`env::join_paths()`] 返回的。<br>
///
/// [`env::join_paths()`]: join_paths
#[derive(Debug)]
#[stable(feature = "env", since = "1.0.0")]
pub struct JoinPathsError {
    inner: os_imp::JoinPathsError,
}

/// Joins a collection of [`Path`]s appropriately for the `PATH` environment variable. <br>为 `PATH` 环境变量适当地加入 [`Path`] 的集合。<br>
///
/// # Errors
///
/// Returns an [`Err`] (containing an error message) if one of the input [`Path`]s contains an invalid character for constructing the `PATH` variable (a double quote on Windows or a colon on Unix). <br>如果输入 [`Path`] 之一包含用于构造 `PATH` 变量的无效字符 (Windows 上的双引号或 Unix 上的冒号)，则返回 [`Err`] (包含错误消息)。<br>
///
///
/// # Examples
///
/// Joining paths on a Unix-like platform: <br>在类似 Unix 的平台上的连接路径：<br>
///
/// ```
/// use std::env;
/// use std::ffi::OsString;
/// use std::path::Path;
///
/// fn main() -> Result<(), env::JoinPathsError> {
/// # if cfg!(unix) {
///     let paths = [Path::new("/bin"), Path::new("/usr/bin")];
///     let path_os_string = env::join_paths(paths.iter())?;
///     assert_eq!(path_os_string, OsString::from("/bin:/usr/bin"));
/// # }
///     Ok(())
/// }
/// ```
///
/// Joining a path containing a colon on a Unix-like platform results in an error: <br>在类似 Unix 的平台上加入包含冒号的路径会导致错误：<br>
///
/// ```
/// # if cfg!(unix) {
/// use std::env;
/// use std::path::Path;
///
/// let paths = [Path::new("/bin"), Path::new("/usr/bi:n")];
/// assert!(env::join_paths(paths.iter()).is_err());
/// # }
/// ```
///
/// Using `env::join_paths()` with [`env::split_paths()`] to append an item to the `PATH` environment variable: <br>将 `env::join_paths()` 与 [`env::split_paths()`] 结合使用，可以将项追加到 `PATH` 环境变量中：<br>
///
/// ```
/// use std::env;
/// use std::path::PathBuf;
///
/// fn main() -> Result<(), env::JoinPathsError> {
///     if let Some(path) = env::var_os("PATH") {
///         let mut paths = env::split_paths(&path).collect::<Vec<_>>();
///         paths.push(PathBuf::from("/home/xyz/bin"));
///         let new_path = env::join_paths(paths)?;
///         env::set_var("PATH", &new_path);
///     }
///
///     Ok(())
/// }
/// ```
///
/// [`env::split_paths()`]: split_paths
///
///
///
///
#[stable(feature = "env", since = "1.0.0")]
pub fn join_paths<I, T>(paths: I) -> Result<OsString, JoinPathsError>
where
    I: IntoIterator<Item = T>,
    T: AsRef<OsStr>,
{
    os_imp::join_paths(paths.into_iter()).map_err(|e| JoinPathsError { inner: e })
}

#[stable(feature = "env", since = "1.0.0")]
impl fmt::Display for JoinPathsError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.inner.fmt(f)
    }
}

#[stable(feature = "env", since = "1.0.0")]
impl Error for JoinPathsError {
    #[allow(deprecated, deprecated_in_future)]
    fn description(&self) -> &str {
        self.inner.description()
    }
}

/// Returns the path of the current user's home directory if known. <br>返回当前用户主目录的路径 (如果已知)。<br>
///
/// # Unix
///
/// - Returns the value of the 'HOME' environment variable if it is set (including to an empty string). <br>返回 'HOME' 环境变量的值 (如果已设置) (包括一个空字符串)。<br>
/// - Otherwise, it tries to determine the home directory by invoking the `getpwuid_r` function using the UID of the current user. <br>否则，它将尝试通过使用当前用户的 UID 调用 `getpwuid_r` 函数来确定主目录。<br>
/// An empty home directory field returned from the `getpwuid_r` function is considered to be a valid value. <br>从 `getpwuid_r` 函数返回的空主目录字段被视为有效值。<br>
/// - Returns `None` if the current user has no entry in the /etc/passwd file. <br>如果当前用户在 /etc/passwd 文件中没有条目，则返回 `None`。<br>
///
/// # Windows
///
/// - Returns the value of the 'HOME' environment variable if it is set (including to an empty string). <br>返回 'HOME' 环境变量的值 (如果已设置) (包括一个空字符串)。<br>
/// - Otherwise, returns the value of the 'USERPROFILE' environment variable if it is set (including to an empty string). <br>否则，返回 'USERPROFILE' 环境变量的值 (如果已设置) (包括一个空字符串)。<br>
/// - If both do not exist, [`GetUserProfileDirectory`][msdn] is used to return the path. <br>如果两者都不存在，则使用 [`GetUserProfileDirectory`][msdn] 返回路径。<br>
///
/// [msdn]: https://docs.microsoft.com/en-us/windows/win32/api/userenv/nf-userenv-getuserprofiledirectorya
///
/// # Examples
///
/// ```
/// use std::env;
///
/// match env::home_dir() {
///     Some(path) => println!("Your home directory, probably: {}", path.display()),
///     None => println!("Impossible to get your home dir!"),
/// }
/// ```
///
///
///
///
#[deprecated(
    since = "1.29.0",
    note = "This function's behavior is unexpected and probably not what you want. \
            Consider using a crate from crates.io instead."
)]
#[must_use]
#[stable(feature = "env", since = "1.0.0")]
pub fn home_dir() -> Option<PathBuf> {
    os_imp::home_dir()
}

/// Returns the path of a temporary directory. <br>返回临时目录的路径。<br>
///
/// The temporary directory may be shared among users, or between processes with different privileges; <br>临时目录可以在用户之间共享，也可以在具有不同权限的进程之间共享；<br> thus, the creation of any files or directories in the temporary directory must use a secure method to create a uniquely named file. <br>因此，在临时目录中创建任何文件或目录都必须使用安全方法来创建唯一命名的文件。<br>
///
/// Creating a file or directory with a fixed or predictable name may result in "insecure temporary file" security vulnerabilities. <br>使用固定或可预测的名称创建文件或目录可能会导致 "不安全的临时文件" 安全漏洞。<br>
/// Consider using a crate that securely creates temporary files or directories. <br>考虑使用安全创建临时文件或目录的 crate。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// On Unix, returns the value of the `TMPDIR` environment variable if it is set, otherwise for non-Android it returns `/tmp`. <br>在 Unix 上，如果设置了 `TMPDIR` 环境变量，则返回它的值，否则，对于非 Android 系统，将返回 `/tmp`。<br>
/// If Android, since there is no global temporary folder (it is usually allocated per-app), it returns `/data/local/tmp`. <br>如果为 Android，则由于没有临时文件夹 (通常是按应用分配的)，它将返回 `/data/local/tmp`。<br>
/// On Windows, the behavior is equivalent to that of [`GetTempPath2`][GetTempPath2] / [`GetTempPath`][GetTempPath], which this function uses internally. <br>在 Windows 上，行为等同于 [`GetTempPath2`][GetTempPath2] / [`GetTempPath`][GetTempPath]，这个函数在内部使用。<br>
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
/// [GetTempPath2]: https://docs.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-gettemppath2a
/// [GetTempPath]: https://docs.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-gettemppatha
///
/// ```no_run
/// use std::env;
///
/// fn main() {
///     let dir = env::temp_dir();
///     println!("Temporary directory: {}", dir.display());
/// }
/// ```
///
///
///
///
///
#[must_use]
#[stable(feature = "env", since = "1.0.0")]
pub fn temp_dir() -> PathBuf {
    os_imp::temp_dir()
}

/// Returns the full filesystem path of the current running executable. <br>返回当前正在运行的可执行文件的完整文件系统路径。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// If the executable was invoked through a symbolic link, some platforms will return the path of the symbolic link and other platforms will return the path of the symbolic link’s target. <br>如果可执行文件是通过符号链接调用的，则某些平台将返回符号链接的路径，而其他平台将返回符号链接目标的路径。<br>
///
///
/// If the executable is renamed while it is running, platforms may return the path at the time it was loaded instead of the new path. <br>如果可执行文件在运行时被重命名，平台可能会返回加载时的路径而不是新路径。<br>
///
/// # Errors
///
/// Acquiring the path of the current executable is a platform-specific operation that can fail for a good number of reasons. <br>获取当前可执行文件的路径是特定于平台的操作，由于多种原因，该操作可能会失败。<br>
/// Some errors can include, but not be limited to, filesystem operations failing or general syscall failures. <br>一些错误可能包括但不限于文件系统操作失败或常规 syscall 错误。<br>
///
/// # Security
///
/// The output of this function should not be trusted for anything that might have security implications. <br>这个函数的输出不应该被任何可能有安全隐患的东西信任。<br>
/// Basically, if users can run the executable, they can change the output arbitrarily. <br>基本上，如果用户可以运行可执行文件，他们可以任意更改输出。<br>
///
/// As an example, you can easily introduce a race condition. <br>例如，您可以很容易地引入一个竞态条件。<br> It goes like this: <br>它是这样的:<br>
///
/// 1. You get the path to the current executable using `current_exe()`, and store it in a variable. <br>您可以使用 `current_exe()` 获取当前可执行文件的路径，并将其存储在一个变量中。<br>
/// 2. Time passes. <br>时间流逝。<br> A malicious actor removes the current executable, and replaces it with a malicious one. <br>恶意行为者删除当前可执行文件，并将其替换为恶意文件。<br>
/// 3. You then use the stored path to re-execute the current executable. <br>然后，您使用存储的路径重新执行当前的可执行文件。<br>
///
/// You expected to safely execute the current executable, but you're instead executing something completely different. <br>您希望安全地执行当前的可执行文件，但您正在执行一些完全不同的东西。<br> The code you just executed run with your privileges. <br>您刚刚执行的代码以您的权限运行。<br>
///
/// This sort of behavior has been known to [lead to privilege escalation] when used incorrectly. <br>如果不正确使用会 [导致特权升级][lead to privilege escalation]，这种行为是已知的。<br>
///
/// [lead to privilege escalation]: https://securityvulns.com/Wdocument183.html
///
/// # Examples
///
/// ```
/// use std::env;
///
/// match env::current_exe() {
///     Ok(exe_path) => println!("Path of this executable is: {}",
///                              exe_path.display()),
///     Err(e) => println!("failed to get current exe path: {e}"),
/// };
/// ```
///
///
///
///
///
///
///
///
///
///
///
#[stable(feature = "env", since = "1.0.0")]
pub fn current_exe() -> io::Result<PathBuf> {
    os_imp::current_exe()
}

/// An iterator over the arguments of a process, yielding a [`String`] value for each argument. <br>进程参数的迭代器，为每个参数产生 [`String`] 值。<br>
///
/// This struct is created by [`env::args()`]. <br>该结构体由 [`env::args()`] 创建。<br> See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// The first element is traditionally the path of the executable, but it can be set to arbitrary text, and might not even exist. <br>第一个元素传统上是可执行文件的路径，但它可以设置为任意文本，甚至可能不存在。<br>
/// This means this property should not be relied upon for security purposes. <br>这意味着出于安全目的，不应依赖此属性。<br>
///
/// [`env::args()`]: args
///
///
///
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "env", since = "1.0.0")]
pub struct Args {
    inner: ArgsOs,
}

/// An iterator over the arguments of a process, yielding an [`OsString`] value for each argument. <br>一个进程参数的迭代器，为每个参数产生 [`OsString`] 值。<br>
///
/// This struct is created by [`env::args_os()`]. <br>该结构体由 [`env::args_os()`] 创建。<br> See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// The first element is traditionally the path of the executable, but it can be set to arbitrary text, and might not even exist. <br>第一个元素传统上是可执行文件的路径，但它可以设置为任意文本，甚至可能不存在。<br>
/// This means this property should not be relied upon for security purposes. <br>这意味着出于安全目的，不应依赖此属性。<br>
///
/// [`env::args_os()`]: args_os
///
///
///
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "env", since = "1.0.0")]
pub struct ArgsOs {
    inner: sys::args::Args,
}

/// Returns the arguments that this program was started with (normally passed via the command line). <br>返回此程序开始的参数 (通常通过命令行传递)。<br>
///
/// The first element is traditionally the path of the executable, but it can be set to arbitrary text, and might not even exist. <br>第一个元素传统上是可执行文件的路径，但它可以设置为任意文本，甚至可能不存在。<br>
///
/// This means this property should not be relied upon for security purposes. <br>这意味着出于安全目的，不应依赖此属性。<br>
///
/// On Unix systems the shell usually expands unquoted arguments with glob patterns (such as `*` and `?`). <br>在 Unix 系统上，shell 通常使用 glob 模式 (例如 `*` 和 `?`) 来扩展不带引号的参数。<br>
/// On Windows this is not done, and such arguments are passed as-is. <br>在 Windows 上，这没有完成，并且此类参数按原样传递。<br>
///
/// On glibc Linux systems, arguments are retrieved by placing a function in `.init_array`. <br>在 glibc Linux 系统上，可以通过在 `.init_array` 中放置一个函数来检索参数。<br>
/// glibc passes `argc`, `argv`, and `envp` to functions in `.init_array`, as a non-standard extension. <br>glibc 将 `argc`、`argv` 和 `envp` 作为非标准扩展传递给 `.init_array` 中的函数。<br>
/// This allows `std::env::args` to work even in a `cdylib` or `staticlib`, as it does on macOS and Windows. <br>这使得 `std::env::args` 甚至可以在 `cdylib` 或 `staticlib` 上工作，就像在 macOS 和 Windows 上一样。<br>
///
/// # Panics
///
/// The returned iterator will panic during iteration if any argument to the process is not valid Unicode. <br>如果进程的任何参数不是有效的 Unicode，则返回的迭代器将在迭代期间 panic。<br>
/// If this is not desired, use the [`args_os`] function instead. <br>如果不希望这样做，请改用 [`args_os`] 函数。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// // Prints each argument on a separate line <br>在单独的行上打印每个参数<br>
/// for argument in env::args() {
///     println!("{argument}");
/// }
/// ```
///
///
///
///
#[stable(feature = "env", since = "1.0.0")]
pub fn args() -> Args {
    Args { inner: args_os() }
}

/// Returns the arguments that this program was started with (normally passed via the command line). <br>返回此程序开始的参数 (通常通过命令行传递)。<br>
///
/// The first element is traditionally the path of the executable, but it can be set to arbitrary text, and might not even exist. <br>第一个元素传统上是可执行文件的路径，但它可以设置为任意文本，甚至可能不存在。<br>
///
/// This means this property should not be relied upon for security purposes. <br>这意味着出于安全目的，不应依赖此属性。<br>
///
/// On Unix systems the shell usually expands unquoted arguments with glob patterns (such as `*` and `?`). <br>在 Unix 系统上，shell 通常使用 glob 模式 (例如 `*` 和 `?`) 来扩展不带引号的参数。<br>
/// On Windows this is not done, and such arguments are passed as-is. <br>在 Windows 上，这没有完成，并且此类参数按原样传递。<br>
///
/// On glibc Linux systems, arguments are retrieved by placing a function in `.init_array`. <br>在 glibc Linux 系统上，可以通过在 `.init_array` 中放置一个函数来检索参数。<br>
/// glibc passes `argc`, `argv`, and `envp` to functions in `.init_array`, as a non-standard extension. <br>glibc 将 `argc`、`argv` 和 `envp` 作为非标准扩展传递给 `.init_array` 中的函数。<br>
/// This allows `std::env::args_os` to work even in a `cdylib` or `staticlib`, as it does on macOS and Windows. <br>这使得 `std::env::args_os` 甚至可以在 `cdylib` 或 `staticlib` 中工作，就像在 macOS 和 Windows 上一样。<br>
///
/// Note that the returned iterator will not check if the arguments to the process are valid Unicode. <br>请注意，返回的迭代器不会检查进程的参数是否为有效的 Unicode。<br>
/// If you want to panic on invalid UTF-8, use the [`args`] function instead. <br>如果您想在无效的 UTF-8 上使用 panic，请改用 [`args`] 函数。<br>
///
/// # Examples
///
/// ```
/// use std::env;
///
/// // Prints each argument on a separate line <br>在单独的行上打印每个参数<br>
/// for argument in env::args_os() {
///     println!("{argument:?}");
/// }
/// ```
///
///
///
///
#[stable(feature = "env", since = "1.0.0")]
pub fn args_os() -> ArgsOs {
    ArgsOs { inner: sys::args::args() }
}

#[stable(feature = "env_unimpl_send_sync", since = "1.26.0")]
impl !Send for Args {}

#[stable(feature = "env_unimpl_send_sync", since = "1.26.0")]
impl !Sync for Args {}

#[stable(feature = "env", since = "1.0.0")]
impl Iterator for Args {
    type Item = String;
    fn next(&mut self) -> Option<String> {
        self.inner.next().map(|s| s.into_string().unwrap())
    }
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.inner.size_hint()
    }
}

#[stable(feature = "env", since = "1.0.0")]
impl ExactSizeIterator for Args {
    fn len(&self) -> usize {
        self.inner.len()
    }
    fn is_empty(&self) -> bool {
        self.inner.is_empty()
    }
}

#[stable(feature = "env_iterators", since = "1.12.0")]
impl DoubleEndedIterator for Args {
    fn next_back(&mut self) -> Option<String> {
        self.inner.next_back().map(|s| s.into_string().unwrap())
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Args {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Args").field("inner", &self.inner.inner).finish()
    }
}

#[stable(feature = "env_unimpl_send_sync", since = "1.26.0")]
impl !Send for ArgsOs {}

#[stable(feature = "env_unimpl_send_sync", since = "1.26.0")]
impl !Sync for ArgsOs {}

#[stable(feature = "env", since = "1.0.0")]
impl Iterator for ArgsOs {
    type Item = OsString;
    fn next(&mut self) -> Option<OsString> {
        self.inner.next()
    }
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.inner.size_hint()
    }
}

#[stable(feature = "env", since = "1.0.0")]
impl ExactSizeIterator for ArgsOs {
    fn len(&self) -> usize {
        self.inner.len()
    }
    fn is_empty(&self) -> bool {
        self.inner.is_empty()
    }
}

#[stable(feature = "env_iterators", since = "1.12.0")]
impl DoubleEndedIterator for ArgsOs {
    fn next_back(&mut self) -> Option<OsString> {
        self.inner.next_back()
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for ArgsOs {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("ArgsOs").field("inner", &self.inner).finish()
    }
}

/// Constants associated with the current target <br>与当前目标关联的常量<br>
#[stable(feature = "env", since = "1.0.0")]
pub mod consts {
    use crate::sys::env::os;

    /// A string describing the architecture of the CPU that is currently in use. <br>一个字符串，描述当前正在使用的 CPU 的体系结构。<br>
    ///
    ///
    /// Some possible values: <br>一些可能的值：<br>
    ///
    /// - x86
    /// - x86_64
    /// - arm
    /// - aarch64
    /// - m68k
    /// - mips
    /// - mips64
    /// - powerpc
    /// - powerpc64
    /// - riscv64
    /// - s390x
    /// - sparc64
    #[stable(feature = "env", since = "1.0.0")]
    pub const ARCH: &str = env!("STD_ENV_ARCH");

    /// The family of the operating system. <br>操作系统的家族。<br> Example value is `unix`. <br>示例值为 `unix`。<br>
    ///
    /// Some possible values: <br>一些可能的值：<br>
    ///
    /// - unix
    /// - windows
    #[stable(feature = "env", since = "1.0.0")]
    pub const FAMILY: &str = os::FAMILY;

    /// A string describing the specific operating system in use. <br>描述正在使用的特定操作系统的字符串。<br>
    /// Example value is `linux`. <br>示例值为 `linux`。<br>
    ///
    /// Some possible values: <br>一些可能的值：<br>
    ///
    /// - linux
    /// - macos
    /// - ios
    /// - freebsd
    /// - dragonfly
    /// - netbsd
    /// - openbsd
    /// - solaris
    /// - android
    /// - windows
    #[stable(feature = "env", since = "1.0.0")]
    pub const OS: &str = os::OS;

    /// Specifies the filename prefix used for shared libraries on this platform. <br>指定用于此平台上的共享库的文件名前缀。<br>
    /// Example value is `lib`. <br>示例值为 `lib`。<br>
    ///
    /// Some possible values: <br>一些可能的值：<br>
    ///
    /// - lib
    /// - `""` (an empty string) <br>`""` (空字符串)<br>
    #[stable(feature = "env", since = "1.0.0")]
    pub const DLL_PREFIX: &str = os::DLL_PREFIX;

    /// Specifies the filename suffix used for shared libraries on this platform. <br>指定此平台上共享库使用的文件名后缀。<br>
    /// Example value is `.so`. <br>示例值为 `.so`。<br>
    ///
    /// Some possible values: <br>一些可能的值：<br>
    ///
    /// - .so
    /// - .dylib
    /// - .dll
    #[stable(feature = "env", since = "1.0.0")]
    pub const DLL_SUFFIX: &str = os::DLL_SUFFIX;

    /// Specifies the file extension used for shared libraries on this platform that goes after the dot. <br>指定此平台上共享库所用的文件扩展名，该扩展名位于点后。<br>
    /// Example value is `so`. <br>示例值为 `so`。<br>
    ///
    /// Some possible values: <br>一些可能的值：<br>
    ///
    /// - so
    /// - dylib
    /// - dll
    #[stable(feature = "env", since = "1.0.0")]
    pub const DLL_EXTENSION: &str = os::DLL_EXTENSION;

    /// Specifies the filename suffix used for executable binaries on this platform. <br>指定用于此平台上的可执行二进制文件的文件名后缀。<br>
    /// Example value is `.exe`. <br>示例值为 `.exe`。<br>
    ///
    /// Some possible values: <br>一些可能的值：<br>
    ///
    /// - .exe
    /// - .nexe
    /// - .pexe
    /// - `""` (an empty string) <br>`""` (空字符串)<br>
    #[stable(feature = "env", since = "1.0.0")]
    pub const EXE_SUFFIX: &str = os::EXE_SUFFIX;

    /// Specifies the file extension, if any, used for executable binaries on this platform. <br>指定用于此平台上的可执行二进制文件的文件扩展名 (如果有)。<br>
    /// Example value is `exe`. <br>示例值为 `exe`。<br>
    ///
    /// Some possible values: <br>一些可能的值：<br>
    ///
    /// - exe
    /// - `""` (an empty string) <br>`""` (空字符串)<br>
    #[stable(feature = "env", since = "1.0.0")]
    pub const EXE_EXTENSION: &str = os::EXE_EXTENSION;
}
