//! Utilities related to FFI bindings. <br>与 FFI 绑定有关的实用工具。<br>
//!
//! This module provides utilities to handle data across non-Rust interfaces, like other programming languages and the underlying operating system. <br>该模块提供了实用工具来处理跨非 Rust 接口的数据，例如其他编程语言和底层操作系统。<br> It is mainly of use for FFI (Foreign Function Interface) bindings and code that needs to exchange C-like strings with other languages. <br>它主要用于 FFI (外部函数接口) 绑定和需要与其他语言交换类 C 字符串的代码。<br>
//!
//! # Overview
//!
//! Rust represents owned strings with the [`String`] type, and borrowed slices of strings with the [`str`] primitive. <br>Rust 代表 [`String`] 类型的拥有的字符串，而借用 [`str`] 原语的字符串切片。<br> Both are always in UTF-8 encoding, and may contain nul bytes in the middle, i.e., if you look at the bytes that make up the string, there may be a `\0` among them. <br>两者始终都是 UTF-8 编码，并且中间可能包含 nul 个字节，即，如果您查看组成字符串的字节，则其中可能有一个 `\0`。<br>
//! Both `String` and `str` store their length explicitly; <br>`String` 和 `str` 都明确存储它们的长度。<br> there are no nul terminators at the end of strings like in C. <br>像 C 中的字符串末尾没有 nul 终止符。<br>
//!
//! C strings are different from Rust strings: <br>C 字符串不同于 Rust 字符串：<br>
//!
//! * **Encodings** - Rust strings are UTF-8, but C strings may use other encodings. <br>**编码**-Rust 字符串是 UTF-8，但是 C 字符串可以使用其他编码。<br> If you are using a string from C, you should check its encoding explicitly, rather than just assuming that it is UTF-8 like you can do in Rust. <br>如果使用的是来自 C 的字符串，则应显式检查其编码，而不是像在 Rust 中那样假定它是 UTF-8。<br>
//!
//! * **Character size** - C strings may use `char` or `wchar_t`-sized characters; <br>**字符大小**-C 字符串可以使用 `char` 或 `wchar_t` 大小的字符；<br> please **note** that C's `char` is different from Rust's. <br>请 **注意** C 的 `char` 与 Rust 的不同。<br>
//! The C standard leaves the actual sizes of those types open to interpretation, but defines different APIs for strings made up of each character type. <br>C 标准使这些类型的实际大小易于解释，但是为由每个字符类型组成的字符串定义了不同的 API。<br> Rust strings are always UTF-8, so different Unicode characters will be encoded in a variable number of bytes each. <br>Rust 字符串始终为 UTF-8，因此每个不同的 Unicode 字符将以可变的字节数进行编码。<br>
//! The Rust type [`char`] represents a '[Unicode scalar value]', which is similar to, but not the same as, a '[Unicode code point]'. <br>Rust 类型 [`char`] 表示 `[Unicode 标量值]`，与 `[Unicode 代码点]` 相似但不相同。<br>
//!
//! * **Nul terminators and implicit string lengths** - Often, C strings are nul-terminated, i.e., they have a `\0` character at the end. <br>**Nul 终止符和隐式字符串长度**-C 字符串通常以 Nul 终止，即，它们的末尾有 `\0` 字符。<br>
//! The length of a string buffer is not stored, but has to be calculated; <br>字符串缓冲区的长度不存储，而是必须计算；<br> to compute the length of a string, C code must manually call a function like `strlen()` for `char`-based strings, or `wcslen()` for `wchar_t`-based ones. <br>要计算字符串的长度，C 代码必须手动调用一个函数，例如 `strlen()` 表示基于 char 的字符串，`wcslen()` 表示基于 wchar_t 的字符串。<br>
//! Those functions return the number of characters in the string excluding the nul terminator, so the buffer length is really `len+1` characters. <br>这些函数返回字符串中不包括 nul 终止符的字符数，因此缓冲区长度实际上是 `len+1` 字符。<br>
//! Rust strings don't have a nul terminator; <br>Rust 字符串没有 nul 终止符；<br> their length is always stored and does not need to be calculated. <br>它们的长度总是存储的，不需要计算。<br>
//! While in Rust accessing a string's length is an *O*(1) operation (because the length is stored); <br>而在 Rust 中，访问字符串的长度是一个 *O*(1) 操作 (因为长度是被存储的) ；<br> in C it is an *O*(*n*) operation because the length needs to be computed by scanning the string for the nul terminator. <br>在 C 中，它是一个 *O*(*n*) 操作，因为需要通过扫描字符串中的 nul 终止符来计算长度。<br>
//!
//! * **Internal nul characters** - When C strings have a nul terminator character, this usually means that they cannot have nul characters in the middle — a nul character would essentially truncate the string. <br>**内部 nul 字符**- 当 C 字符串具有 nul 终止符时，这通常意味着它们中间不能包含 nul 字符 - nul 字符实际上会截断字符串。<br>
//! Rust strings *can* have nul characters in the middle, because nul does not have to mark the end of the string in Rust. <br>Rust 字符串 *可以* 中间有 nul 个字符，因为 nul 不必在 Rust 中标记字符串的结尾。<br>
//!
//! # Representations of non-Rust strings <br>非 Rust 字符串的表示形式<br>
//!
//! [`CString`] and [`CStr`] are useful when you need to transfer UTF-8 strings to and from languages with a C ABI, like Python. <br>[`CString`] 和 [`CStr`] 在您需要将 UTF-8 字符串与带有 C ABI 的语言 (如 Python) 相互传输时很有用。<br>
//!
//! * **From Rust to C:** [`CString`] represents an owned, C-friendly string: it is nul-terminated, and has no internal nul characters. <br>**从 Rust 到 C:**[`CString`] 表示一个拥有的，对 C 友好的字符串：它是 nul 终止的，并且没有内部 nul 字符。<br>
//! Rust code can create a [`CString`] out of a normal string (provided that the string doesn't have nul characters in the middle), and then use a variety of methods to obtain a raw <code>\*mut [u8]</code> that can then be passed as an argument to functions which use the C conventions for strings. <br>Rust 代码可以从一个普通字符串中创建一个 [`CString`] (前提是该字符串中间没有 nul 字符)，然后使用多种方法获得一个原始的 <code>\*mut [u8]</code>，然后可以作为参数传递给使用字符串的 C 约定的函数。<br>
//!
//!
//! * **From C to Rust:** [`CStr`] represents a borrowed C string; <br>**从 C 到 Rust:**[`CStr`] 表示借用的 C 字符串；<br> it is what you would use to wrap a raw <code>\*const [u8]</code> that you got from a C function. <br>它是您用来包装从 C 函数获得的原始 <code>\*const [u8]</code> 的内容。<br> A [`CStr`] is guaranteed to be a nul-terminated array of bytes. <br>[`CStr`] 保证是一个以 nul 结尾的字节数组。<br>
//! Once you have a [`CStr`], you can convert it to a Rust <code>&[str]</code> if it's valid UTF-8, or lossily convert it by adding replacement characters. <br>一旦您有了 [`CStr`]，您可以将它转换为 Rust <code>&[str]</code>，如果它是有效的 UTF-8，或者通过添加替换字符来有损地转换它。<br>
//!
//! [`OsString`] and [`OsStr`] are useful when you need to transfer strings to and from the operating system itself, or when capturing the output of external commands. <br>[`OsString`] 和 [`OsStr`] 在您需要在操作系统本身之间传输字符串时很有用，或者在捕获外部命令的输出时很有用。<br>
//! Conversions between [`OsString`], [`OsStr`] and Rust strings work similarly to those for [`CString`] and [`CStr`]. <br>[`OsString`]，[`OsStr`] 和 Rust 字符串之间的转换与 [`CString`] 和 [`CStr`] 的转换相似。<br>
//!
//! * [`OsString`] losslessly represents an owned platform string. <br>[`OsString`] 无损地表示拥有所有权的平台字符串。<br> However, this representation is not necessarily in a form native to the platform. <br>但是，这种表示不一定采用平台原生的形式。<br>
//! In the Rust standard library, various APIs that transfer strings to/from the operating system use [`OsString`] instead of plain strings. <br>在 Rust 标准库中，用于传输字符串 to/from 的各种 API，操作系统使用 [`OsString`] 代替纯字符串。<br>
//! For example, [`env::var_os()`] is used to query environment variables; <br>例如，[`env::var_os()`] 用于查询环境变量；<br> it returns an <code>[Option]<[OsString]></code>. <br>它返回一个 <code>[Option]<[OsString]></code>。<br> If the environment variable exists you will get a <code>[Some]\(os_string)</code>, which you can *then* try to convert to a Rust string. <br>如果环境变量存在，您将获得一个 <code>[Some]\(os_string)</code>，然后您可以尝试将其转换为 Rust 字符串。<br>
//! This yields a [`Result`], so that your code can detect errors in case the environment variable did not in fact contain valid Unicode data. <br>这将产生一个 [`Result`]，以便在环境变量实际上不包含有效的 Unicode 数据的情况下，您的代码可以检测到错误。<br>
//!
//! * [`OsStr`] losslessly represents a borrowed reference to a platform string. <br>[`OsStr`] 无损地表示对平台字符串的借用引用。<br>
//! However, this representation is not necessarily in a form native to the platform. <br>但是，这种表示不一定采用平台原生的形式。<br>
//! It can be converted into a UTF-8 Rust string slice in a similar way to [`OsString`]. <br>可以用类似 [`OsString`] 的方式转换成 UTF-8 Rust 字符串 小提琴。<br>
//!
//! # Conversions
//!
//! ## On Unix <br>在 Unix 上<br>
//!
//! On Unix, [`OsStr`] implements the <code>std::os::unix::ffi::[OsStrExt][unix.OsStrExt]</code> trait, which augments it with two methods, [`from_bytes`] and [`as_bytes`]. <br>在 Unix 上，[`OsStr`] 实现了 <code>std::os::unix::ffi::[OsStrExt][unix.OsStrExt]</code> trait，它增加了两个方法，[`from_bytes`] 和 [`as_bytes`]。<br>
//! These do inexpensive conversions from and to byte slices. <br>这些在字节片之间进行廉价的转换。<br>
//!
//! Additionally, on Unix [`OsString`] implements the <code>std::os::unix::ffi::[OsStringExt][unix.OsStringExt]</code> trait, which provides [`from_vec`] and [`into_vec`] methods that consume their arguments, and take or produce vectors of [`u8`]. <br>此外，在 Unix 上，[`OsString`] 实现了 <code>std::os::unix::ffi::[OsStringExt][unix.OsStringExt]</code> trait，它提供了 [`from_vec`] 和 [`into_vec`] 方法，这些方法使用它们的参数，并获取或生成 [`u8`] 的 vectors。<br>
//!
//! ## On Windows <br>在 Windows 上<br>
//!
//! An [`OsStr`] can be losslessly converted to a native Windows string. <br>[`OsStr`] 可以无损地转换为原生 Windows 字符串。<br> And a native Windows string can be losslessly converted to an [`OsString`]. <br>并且原生 Windows 字符串可以无损地转换为 [`OsString`]。<br>
//!
//! On Windows, [`OsStr`] implements the <code>std::os::windows::ffi::[OsStrExt][windows.OsStrExt]</code> trait, which provides an [`encode_wide`] method. <br>在 Windows 上，[`OsStr`] 实现了 <code>std::os::windows::ffi::[OsStrExt][windows.OsStrExt]</code> trait，它提供了一个 [`encode_wide`] 方法。<br> This provides an iterator that can be [`collect`]ed into a vector of [`u16`]. <br>这提供了一个迭代器，可以将其迭代到 [`u16`] 的 vector 中。<br>
//! After a nul characters is appended, this is the same as a native Windows string. <br>追加 nul 字符后，这与原生 Windows 字符串相同。<br>
//!
//! Additionally, on Windows [`OsString`] implements the <br>此外，在 Windows 上 [`OsString`] 实现了<br>
//! <code>std::os::windows:ffi::[OsStringExt][windows.OsStringExt]</code>
//! trait, which provides a [`from_wide`] method to convert a native Windows string (without the terminating nul character) to an [`OsString`]. <br>一个 trait，它提供了一个 [`from_wide`] 方法来将原生 Windows 字符串 (没有终止的 nul 字符) 转换为 [`OsString`]。<br>
//!
//! [Unicode scalar value]: https://www.unicode.org/glossary/#unicode_scalar_value
//! [Unicode code point]: https://www.unicode.org/glossary/#code_point
//! [`env::set_var()`]: crate::env::set_var "env::set_var"
//! [`env::var_os()`]: crate::env::var_os "env::var_os"
//! [unix.OsStringExt]: crate::os::unix::ffi::OsStringExt "os::unix::ffi::OsStringExt"
//! [`from_vec`]: crate::os::unix::ffi::OsStringExt::from_vec "os::unix::ffi::OsStringExt::from_vec"
//! [`into_vec`]: crate::os::unix::ffi::OsStringExt::into_vec "os::unix::ffi::OsStringExt::into_vec"
//! [unix.OsStrExt]: crate::os::unix::ffi::OsStrExt "os::unix::ffi::OsStrExt"
//! [`from_bytes`]: crate::os::unix::ffi::OsStrExt::from_bytes "os::unix::ffi::OsStrExt::from_bytes"
//! [`as_bytes`]: crate::os::unix::ffi::OsStrExt::as_bytes "os::unix::ffi::OsStrExt::as_bytes"
//! [`OsStrExt`]: crate::os::unix::ffi::OsStrExt "os::unix::ffi::OsStrExt"
//! [windows.OsStrExt]: crate::os::windows::ffi::OsStrExt "os::windows::ffi::OsStrExt"
//! [`encode_wide`]: crate::os::windows::ffi::OsStrExt::encode_wide "os::windows::ffi::OsStrExt::encode_wide"
//! [`collect`]: crate::iter::Iterator::collect "iter::Iterator::collect"
//! [windows.OsStringExt]: crate::os::windows::ffi::OsStringExt "os::windows::ffi::OsStringExt"
//! [`from_wide`]: crate::os::windows::ffi::OsStringExt::from_wide "os::windows::ffi::OsStringExt::from_wide"
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

/// See [alloc::ffi::FromVecWithNulError]. <br>请参见 [alloc::ffi::FromVecWithNulError]。<br>
#[stable(feature = "cstring_from_vec_with_nul", since = "1.58.0")]
pub type FromVecWithNulError = alloc::ffi::FromVecWithNulError;
/// See [alloc::ffi::CString]. <br>请参见 [alloc::ffi::CString]。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub type CString = alloc::ffi::CString;
/// See [alloc::ffi::IntoStringError]. <br>请参见 [alloc::ffi::IntoStringError]。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub type IntoStringError = alloc::ffi::IntoStringError;
/// See [alloc::ffi::NulError]. <br>请参见 [alloc::ffi::NulError]。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub type NulError = alloc::ffi::NulError;
/// See [core::ffi::CStr]. <br>请参见 [core::ffi::CStr]。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub type CStr = core::ffi::CStr;
/// See [core::ffi::FromBytesWithNulError]. <br>请参见 [core::ffi::FromBytesWithNulError]。<br>
#[stable(feature = "cstr_from_bytes", since = "1.10.0")]
pub type FromBytesWithNulError = core::ffi::FromBytesWithNulError;

#[stable(feature = "rust1", since = "1.0.0")]
pub use self::os_str::{OsStr, OsString};

#[stable(feature = "core_c_void", since = "1.30.0")]
pub use core::ffi::c_void;

#[unstable(
    feature = "c_variadic",
    reason = "the `c_variadic` feature has not been properly tested on \
              all supported platforms",
    issue = "44930"
)]
pub use core::ffi::{VaList, VaListImpl};

mod os_str;
