//! Filesystem manipulation operations. <br>文件系统操作<br>
//!
//! This module contains basic methods to manipulate the contents of the local filesystem. <br>该模块包含一些操作本地文件系统内容的基本方法。<br>
//! All methods in this module represent cross-platform filesystem operations. <br>该模块中的所有方法均表示跨平台文件系统操作。<br>
//! Extra platform-specific functionality can be found in the extension traits of `std::os::$platform`. <br>在 `std::os::$platform` 的扩展名 traits 中可以找到特定于平台的其他功能。<br>
//!

#![stable(feature = "rust1", since = "1.0.0")]
#![deny(unsafe_op_in_unsafe_fn)]

#[cfg(all(test, not(any(target_os = "emscripten", target_env = "sgx"))))]
mod tests;

use crate::ffi::OsString;
use crate::fmt;
use crate::io::{self, IoSlice, IoSliceMut, Read, ReadBuf, Seek, SeekFrom, Write};
use crate::path::{Path, PathBuf};
use crate::sys::fs as fs_imp;
use crate::sys_common::{AsInner, AsInnerMut, FromInner, IntoInner};
use crate::time::SystemTime;

/// A reference to an open file on the filesystem. <br>对文件系统上打开的文件的引用。<br>
///
/// An instance of a `File` can be read and/or written depending on what options it was opened with. <br>可以通过打开 `File` 的选项来读取或者写入 `File` 的实例。<br> Files also implement [`Seek`] to alter the logical cursor that the file contains internally. <br>文件还实现 [`Seek`]，以更改文件内部包含的逻辑游标。<br>
///
/// Files are automatically closed when they go out of scope. <br>文件离开作用域时将自动关闭。<br> Errors detected on closing are ignored by the implementation of `Drop`. <br>`Drop` 的实现将忽略在关闭时检测到的错误。<br> Use the method [`sync_all`] if these errors must be manually handled. <br>如果必须手动处理这些错误，请使用方法 [`sync_all`]。<br>
///
/// # Examples
///
/// Creates a new file and write bytes to it (you can also use [`write()`]): <br>创建一个新文件并向其写入字节 (您也可以使用 [`write()`]) ：<br>
///
/// ```no_run
/// use std::fs::File;
/// use std::io::prelude::*;
///
/// fn main() -> std::io::Result<()> {
///     let mut file = File::create("foo.txt")?;
///     file.write_all(b"Hello, world!")?;
///     Ok(())
/// }
/// ```
///
/// Read the contents of a file into a [`String`] (you can also use [`read`]): <br>将文件内容读入 [`String`] (也可以使用 [`read`]) ：<br>
///
/// ```no_run
/// use std::fs::File;
/// use std::io::prelude::*;
///
/// fn main() -> std::io::Result<()> {
///     let mut file = File::open("foo.txt")?;
///     let mut contents = String::new();
///     file.read_to_string(&mut contents)?;
///     assert_eq!(contents, "Hello, world!");
///     Ok(())
/// }
/// ```
///
/// It can be more efficient to read the contents of a file with a buffered [`Read`]er. This can be accomplished with [`BufReader<R>`]: <br>使用缓冲的 [`Read`] 来读取文件的内容可能会更有效。这可以用 [`BufReader<R>`] 完成：<br>
///
/// ```no_run
/// use std::fs::File;
/// use std::io::BufReader;
/// use std::io::prelude::*;
///
/// fn main() -> std::io::Result<()> {
///     let file = File::open("foo.txt")?;
///     let mut buf_reader = BufReader::new(file);
///     let mut contents = String::new();
///     buf_reader.read_to_string(&mut contents)?;
///     assert_eq!(contents, "Hello, world!");
///     Ok(())
/// }
/// ```
///
/// Note that, although read and write methods require a `&mut File`, because of the interfaces for [`Read`] and [`Write`], the holder of a `&File` can still modify the file, either through methods that take `&File` or by retrieving the underlying OS object and modifying the file that way. <br>请注意，虽然读写方法需要一个 `&mut File`，但由于 [`Read`] 和 [`Write`] 的接口，`&File` 的持有者仍然可以修改文件，可以通过采用 `&File` 的方法，也可以通过检索底层操作系统对象并以这种方式修改文件。<br>
///
/// Additionally, many operating systems allow concurrent modification of files by different processes. <br>另外，许多操作系统允许通过不同的进程并发修改文件。<br> Avoid assuming that holding a `&File` means that the file will not change. <br>避免假定持有 `&File` 意味着文件不会更改。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// On Windows, the implementation of [`Read`] and [`Write`] traits for `File` perform synchronous I/O operations. <br>在 Windows 上，`File` 的 [`Read`] 和 [`Write`] traits 的实现执行同步的 I/O 操作。<br> Therefore the underlying file must not have been opened for asynchronous I/O (e.g. by using `FILE_FLAG_OVERLAPPED`). <br>因此，必须没有为异步 I/O 打开底层文件 (例如使用 `FILE_FLAG_OVERLAPPED`)。<br>
///
/// [`BufReader<R>`]: io::BufReader
/// [`sync_all`]: File::sync_all
///
///
///
///
///
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "File")]
pub struct File {
    inner: fs_imp::File,
}

/// Metadata information about a file. <br>有关文件的元数据信息。<br>
///
/// This structure is returned from the [`metadata`] or [`symlink_metadata`] function or method and represents known metadata about a file such as its permissions, size, modification times, etc. <br>此结构体是从 [`metadata`] 或 [`symlink_metadata`] 函数或方法返回的，表示有关文件的已知元数据，例如其权限，大小，修改时间等。<br>
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Clone)]
pub struct Metadata(fs_imp::FileAttr);

/// Iterator over the entries in a directory. <br>遍历目录中的条目。<br>
///
/// This iterator is returned from the [`read_dir`] function of this module and will yield instances of <code>[io::Result]<[DirEntry]></code>. <br>该迭代器从该模块的 [`read_dir`] 函数返回，将产生一个 <code>[io::Result]<[DirEntry]></code> 实例。<br>
/// Through a [`DirEntry`] information like the entry's path and possibly other metadata can be learned. <br>通过 [`DirEntry`]，可以了解类似条目路径以及可能的其他元数据的信息。<br>
///
/// The order in which this iterator returns entries is platform and filesystem dependent. <br>该迭代器返回条目的顺序取决于平台和文件系统。<br>
///
/// # Errors
///
/// This [`io::Result`] will be an [`Err`] if there's some sort of intermittent IO error during iteration. <br>如果在迭代过程中出现某种间歇性的 IO 错误，则该 [`io::Result`] 将是 [`Err`]。<br>
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct ReadDir(fs_imp::ReadDir);

/// Entries returned by the [`ReadDir`] iterator. <br>[`ReadDir`] 迭代器返回的条目。<br>
///
/// An instance of `DirEntry` represents an entry inside of a directory on the filesystem. <br>`DirEntry` 的实例表示文件系统上目录内的一个条目。<br>
/// Each entry can be inspected via methods to learn about the full path or possibly other metadata through per-platform extension traits. <br>可以通过方法检查每个条目，以通过全平台扩展 traits 了解完整路径或可能的其他元数据。<br>
///
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// On Unix, the `DirEntry` struct contains an internal reference to the open directory. <br>在 Unix 上，`DirEntry` 结构体包含对打开目录的内部引用。<br>
/// Holding `DirEntry` objects will consume a file handle even after the `ReadDir` iterator is dropped. <br>即使在 `ReadDir` 迭代器丢弃之后，持有 `DirEntry` 对象也会消耗文件句柄。<br>
///
/// Note that this [may change in the future][changes]. <br>注意这个 [将来可能会发生变化][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct DirEntry(fs_imp::DirEntry);

/// Options and flags which can be used to configure how a file is opened. <br>可用于配置文件打开方式的选项和标志。<br>
///
/// This builder exposes the ability to configure how a [`File`] is opened and what operations are permitted on the open file. <br>此构建器提供了配置 [`File`] 的打开方式以及打开的文件上允许哪些操作的功能。<br>
/// The [`File::open`] and [`File::create`] methods are aliases for commonly used options using this builder. <br>[`File::open`] 和 [`File::create`] 方法是使用此构建器的常用选项的别名。<br>
///
/// Generally speaking, when using `OpenOptions`, you'll first call [`OpenOptions::new`], then chain calls to methods to set each option, then call [`OpenOptions::open`], passing the path of the file you're trying to open. <br>一般而言，使用 `OpenOptions` 时，首先要调用 [`OpenOptions::new`]，然后链式调用方法以设置每个选项，然后调用 [`OpenOptions::open`]，传递要打开的文件的路径。<br>
///
/// This will give you a [`io::Result`] with a [`File`] inside that you can further operate on. <br>这将为您提供一个内部带有 [`File`] 的 [`io::Result`]，您可以对其进行进一步的操作。<br>
///
/// # Examples
///
/// Opening a file to read: <br>打开一个文件以读取：<br>
///
/// ```no_run
/// use std::fs::OpenOptions;
///
/// let file = OpenOptions::new().read(true).open("foo.txt");
/// ```
///
/// Opening a file for both reading and writing, as well as creating it if it doesn't exist: <br>打开一个文件进行读写，如果不存在则创建一个文件：<br>
///
/// ```no_run
/// use std::fs::OpenOptions;
///
/// let file = OpenOptions::new()
///             .read(true)
///             .write(true)
///             .create(true)
///             .open("foo.txt");
/// ```
///
///
///
///
///
#[derive(Clone, Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct OpenOptions(fs_imp::OpenOptions);

/// Representation of the various permissions on a file. <br>表示文件上的各种权限。<br>
///
/// This module only currently provides one bit of information, [`Permissions::readonly`], which is exposed on all currently supported platforms. <br>该模块当前仅提供一点信息 [`Permissions::readonly`]，该信息在所有当前支持的平台上公开。<br>
/// Unix-specific functionality, such as mode bits, is available through the [`PermissionsExt`] trait. <br>特定于 Unix 的功能 (例如模式位) 可通过 [`PermissionsExt`] trait 获得。<br>
///
/// [`PermissionsExt`]: crate::os::unix::fs::PermissionsExt
///
///
#[derive(Clone, PartialEq, Eq, Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Permissions(fs_imp::FilePermissions);

/// A structure representing a type of file with accessors for each file type. <br>表示文件类型的结构体，每个文件类型都有访问器。<br>
/// It is returned by [`Metadata::file_type`] method. <br>通过 [`Metadata::file_type`] 方法返回。<br>
#[stable(feature = "file_type", since = "1.1.0")]
#[derive(Copy, Clone, PartialEq, Eq, Hash, Debug)]
#[cfg_attr(not(test), rustc_diagnostic_item = "FileType")]
pub struct FileType(fs_imp::FileType);

/// A builder used to create directories in various manners. <br>用于以各种方式创建目录的构建器。<br>
///
/// This builder also supports platform-specific options. <br>该构建器还支持特定于平台的选项。<br>
#[stable(feature = "dir_builder", since = "1.6.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "DirBuilder")]
#[derive(Debug)]
pub struct DirBuilder {
    inner: fs_imp::DirBuilder,
    recursive: bool,
}

/// Read the entire contents of a file into a bytes vector. <br>将文件的全部内容读取为字节 vector。<br>
///
/// This is a convenience function for using [`File::open`] and [`read_to_end`] with fewer imports and without an intermediate variable. <br>这是使用 [`File::open`] 和 [`read_to_end`] 且导入次数较少且没有中间变量的便捷函数。<br>
///
///
/// [`read_to_end`]: Read::read_to_end
///
/// # Errors
///
/// This function will return an error if `path` does not already exist. <br>如果 `path` 还不存在，则此函数将返回错误。<br>
/// Other errors may also be returned according to [`OpenOptions::open`]. <br>根据 [`OpenOptions::open`]，可能还会返回其他错误。<br>
///
/// It will also return an error if it encounters while reading an error of a kind other than [`io::ErrorKind::Interrupted`]. <br>如果在读取 [`io::ErrorKind::Interrupted`] 以外的其他类型的错误时遇到错误，它也会返回错误。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
/// use std::net::SocketAddr;
///
/// fn main() -> Result<(), Box<dyn std::error::Error + 'static>> {
///     let foo: SocketAddr = String::from_utf8_lossy(&fs::read("address.txt")?).parse()?;
///     Ok(())
/// }
/// ```
///
#[stable(feature = "fs_read_write_bytes", since = "1.26.0")]
pub fn read<P: AsRef<Path>>(path: P) -> io::Result<Vec<u8>> {
    fn inner(path: &Path) -> io::Result<Vec<u8>> {
        let mut file = File::open(path)?;
        let mut bytes = Vec::new();
        file.read_to_end(&mut bytes)?;
        Ok(bytes)
    }
    inner(path.as_ref())
}

/// Read the entire contents of a file into a string. <br>将文件的全部内容读取为字符串。<br>
///
/// This is a convenience function for using [`File::open`] and [`read_to_string`] with fewer imports and without an intermediate variable. <br>这是使用 [`File::open`] 和 [`read_to_string`] 且导入次数较少且没有中间变量的便捷函数。<br>
///
/// [`read_to_string`]: Read::read_to_string
///
/// # Errors
///
/// This function will return an error if `path` does not already exist. <br>如果 `path` 还不存在，则此函数将返回错误。<br>
/// Other errors may also be returned according to [`OpenOptions::open`]. <br>根据 [`OpenOptions::open`]，可能还会返回其他错误。<br>
///
/// It will also return an error if it encounters while reading an error of a kind other than [`io::ErrorKind::Interrupted`], or if the contents of the file are not valid UTF-8. <br>如果在读取 [`io::ErrorKind::Interrupted`] 以外的其他错误时遇到错误，或者文件的内容不是有效的 UTF-8，它也会返回错误。<br>
///
///
/// # Examples
///
/// ```no_run
/// use std::fs;
/// use std::net::SocketAddr;
/// use std::error::Error;
///
/// fn main() -> Result<(), Box<dyn Error>> {
///     let foo: SocketAddr = fs::read_to_string("address.txt")?.parse()?;
///     Ok(())
/// }
/// ```
///
///
#[stable(feature = "fs_read_write", since = "1.26.0")]
pub fn read_to_string<P: AsRef<Path>>(path: P) -> io::Result<String> {
    fn inner(path: &Path) -> io::Result<String> {
        let mut file = File::open(path)?;
        let mut string = String::new();
        file.read_to_string(&mut string)?;
        Ok(string)
    }
    inner(path.as_ref())
}

/// Write a slice as the entire contents of a file. <br>写一个切片作为文件的全部内容。<br>
///
/// This function will create a file if it does not exist, and will entirely replace its contents if it does. <br>如果该函数不存在，则此函数将创建一个文件，如果存在，则将完全替换其内容。<br>
///
///
/// This is a convenience function for using [`File::create`] and [`write_all`] with fewer imports. <br>这是使用 [`File::create`] 和 [`write_all`] 且导入次数较少的便捷函数。<br>
///
/// [`write_all`]: Write::write_all
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::write("foo.txt", b"Lorem ipsum")?;
///     fs::write("bar.txt", "dolor sit")?;
///     Ok(())
/// }
/// ```
///
#[stable(feature = "fs_read_write_bytes", since = "1.26.0")]
pub fn write<P: AsRef<Path>, C: AsRef<[u8]>>(path: P, contents: C) -> io::Result<()> {
    fn inner(path: &Path, contents: &[u8]) -> io::Result<()> {
        File::create(path)?.write_all(contents)
    }
    inner(path.as_ref(), contents.as_ref())
}

impl File {
    /// Attempts to open a file in read-only mode. <br>尝试以只读模式打开文件。<br>
    ///
    /// See the [`OpenOptions::open`] method for more details. <br>有关更多详细信息，请参见 [`OpenOptions::open`] 方法。<br>
    ///
    /// # Errors
    ///
    /// This function will return an error if `path` does not already exist. <br>如果 `path` 还不存在，则此函数将返回错误。<br>
    /// Other errors may also be returned according to [`OpenOptions::open`]. <br>根据 [`OpenOptions::open`]，可能还会返回其他错误。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn open<P: AsRef<Path>>(path: P) -> io::Result<File> {
        OpenOptions::new().read(true).open(path.as_ref())
    }

    /// Opens a file in write-only mode. <br>以只写模式打开文件。<br>
    ///
    /// This function will create a file if it does not exist, and will truncate it if it does. <br>如果该函数不存在，则此函数将创建一个文件，如果存在则将截断该文件。<br>
    ///
    ///
    /// See the [`OpenOptions::open`] function for more details. <br>有关更多详细信息，请参见 [`OpenOptions::open`] 函数。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut f = File::create("foo.txt")?;
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn create<P: AsRef<Path>>(path: P) -> io::Result<File> {
        OpenOptions::new().write(true).create(true).truncate(true).open(path.as_ref())
    }

    /// Returns a new OpenOptions object. <br>返回一个新的 OpenOptions 对象。<br>
    ///
    /// This function returns a new OpenOptions object that you can use to open or create a file with specific options if `open()` or `create()` are not appropriate. <br>如果不适合使用 `open()` 或 `create()`，则此函数返回一个新的 OpenOptions 对象，可用于打开或创建具有特定选项的文件。<br>
    ///
    ///
    /// It is equivalent to `OpenOptions::new()`, but allows you to write more readable code. <br>它相当于 `OpenOptions::new()`，但允许您编写更具可读性的代码。<br>
    /// Instead of `OpenOptions::new().append(true).open("example.log")`, you can write `File::options().append(true).open("example.log")`. <br>您可以写 `File::options().append(true).open("example.log")`，而不是 `OpenOptions::new().append(true).open("example.log")`。<br>
    /// This also avoids the need to import `OpenOptions`. <br>这也避免了导入 `OpenOptions` 的需要。<br>
    ///
    /// See the [`OpenOptions::new`] function for more details. <br>有关更多详细信息，请参见 [`OpenOptions::new`] 函数。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut f = File::options().append(true).open("example.log")?;
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    #[must_use]
    #[stable(feature = "with_options", since = "1.58.0")]
    pub fn options() -> OpenOptions {
        OpenOptions::new()
    }

    /// Attempts to sync all OS-internal metadata to disk. <br>尝试将所有操作系统内部元数据同步到磁盘。<br>
    ///
    /// This function will attempt to ensure that all in-memory data reaches the filesystem before returning. <br>此函数将尝试确保所有内存数据在返回之前都已到达文件系统。<br>
    ///
    ///
    /// This can be used to handle errors that would otherwise only be caught when the `File` is closed. <br>这可用于处理错误，否则这些错误仅在 `File` 关闭时才会被捕获。<br>
    /// Dropping a file will ignore errors in synchronizing this in-memory data. <br>丢弃文件将忽略同步此内存中数据的错误。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::File;
    /// use std::io::prelude::*;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut f = File::create("foo.txt")?;
    ///     f.write_all(b"Hello, world!")?;
    ///
    ///     f.sync_all()?;
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn sync_all(&self) -> io::Result<()> {
        self.inner.fsync()
    }

    /// This function is similar to [`sync_all`], except that it might not synchronize file metadata to the filesystem. <br>该函数与 [`sync_all`] 类似，不同之处在于它可能不会将文件元数据同步到文件系统。<br>
    ///
    ///
    /// This is intended for use cases that must synchronize content, but don't need the metadata on disk. <br>这适用于必须同步内容但不需要磁盘上元数据的用例。<br>
    /// The goal of this method is to reduce disk operations. <br>此方法的目标是减少磁盘操作。<br>
    ///
    /// Note that some platforms may simply implement this in terms of [`sync_all`]. <br>请注意，某些平台可能只是根据 [`sync_all`] 来实现此目的。<br>
    ///
    /// [`sync_all`]: File::sync_all
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::File;
    /// use std::io::prelude::*;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut f = File::create("foo.txt")?;
    ///     f.write_all(b"Hello, world!")?;
    ///
    ///     f.sync_data()?;
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn sync_data(&self) -> io::Result<()> {
        self.inner.datasync()
    }

    /// Truncates or extends the underlying file, updating the size of this file to become `size`. <br>截断或扩展底层文件，将此文件的大小更新为 `size`。<br>
    ///
    /// If the `size` is less than the current file's size, then the file will be shrunk. <br>如果 `size` 小于当前文件的大小，则文件将被缩小。<br>
    /// If it is greater than the current file's size, then the file will be extended to `size` and have all of the intermediate data filled in with 0s. <br>如果它大于当前文件的大小，则文件将扩展到 `size`，并且所有中间数据都用 0 填充。<br>
    ///
    ///
    /// The file's cursor isn't changed. <br>文件的游标未更改。<br>
    /// In particular, if the cursor was at the end and the file is shrunk using this operation, the cursor will now be past the end. <br>特别是，如果游标位于末尾，并且使用此操作将文件缩小了，那么游标现在将超过末尾。<br>
    ///
    /// # Errors
    ///
    /// This function will return an error if the file is not opened for writing. <br>如果未打开文件进行写入，则此函数将返回错误。<br>
    /// Also, std::io::ErrorKind::InvalidInput will be returned if the desired length would cause an overflow due to the implementation specifics. <br>同样，如果期望的长度由于实现细节而导致溢出，则将返回 std::io::ErrorKind::InvalidInput。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut f = File::create("foo.txt")?;
    ///     f.set_len(10)?;
    ///     Ok(())
    /// }
    /// ```
    ///
    /// Note that this method alters the content of the underlying file, even though it takes `&self` rather than `&mut self`. <br>请注意，即使使用 `&self` 而不是 `&mut self`，此方法也会更改底层文件的内容。<br>
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn set_len(&self, size: u64) -> io::Result<()> {
        self.inner.truncate(size)
    }

    /// Queries metadata about the underlying file. <br>查询有关底层文件的元数据。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///     let metadata = f.metadata()?;
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn metadata(&self) -> io::Result<Metadata> {
        self.inner.file_attr().map(Metadata)
    }

    /// Creates a new `File` instance that shares the same underlying file handle as the existing `File` instance. <br>创建一个新的 `File` 实例，该实例与现有 `File` 实例共享相同的底层文件句柄。<br>
    /// Reads, writes, and seeks will affect both `File` instances simultaneously. <br>读取，写入和查找将同时影响两个 `File` 实例。<br>
    ///
    /// # Examples
    ///
    /// Creates two handles for a file named `foo.txt`: <br>为名为 `foo.txt` 的文件创建两个句柄：<br>
    ///
    /// ```no_run
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut file = File::open("foo.txt")?;
    ///     let file_copy = file.try_clone()?;
    ///     Ok(())
    /// }
    /// ```
    ///
    /// Assuming there’s a file named `foo.txt` with contents `abcdef\n`, create two handles, seek one of them, and read the remaining bytes from the other handle: <br>假设有一个名为 `foo.txt` 的文件，其内容为 `abcdef\n`，创建两个句柄，查找其中一个，然后从另一个句柄读取剩余的字节：<br>
    ///
    ///
    /// ```no_run
    /// use std::fs::File;
    /// use std::io::SeekFrom;
    /// use std::io::prelude::*;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut file = File::open("foo.txt")?;
    ///     let mut file_copy = file.try_clone()?;
    ///
    ///     file.seek(SeekFrom::Start(3))?;
    ///
    ///     let mut contents = vec![];
    ///     file_copy.read_to_end(&mut contents)?;
    ///     assert_eq!(contents, b"def\n");
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "file_try_clone", since = "1.9.0")]
    pub fn try_clone(&self) -> io::Result<File> {
        Ok(File { inner: self.inner.duplicate()? })
    }

    /// Changes the permissions on the underlying file. <br>更改底层文件的权限。<br>
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// This function currently corresponds to the `fchmod` function on Unix and the `SetFileInformationByHandle` function on Windows. <br>该函数当前对应于 Unix 上的 `fchmod` 函数和 Windows 上的 `SetFileInformationByHandle` 函数。<br>
    ///
    /// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
    ///
    /// [changes]: io#platform-specific-behavior
    ///
    /// # Errors
    ///
    /// This function will return an error if the user lacks permission change attributes on the underlying file. <br>如果用户缺少底层文件的权限更改属性，则此函数将返回错误。<br>
    /// It may also return an error in other os-specific unspecified cases. <br>在其他特定于操作系统的未指定情况下，它也可能返回错误。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// fn main() -> std::io::Result<()> {
    ///     use std::fs::File;
    ///
    ///     let file = File::open("foo.txt")?;
    ///     let mut perms = file.metadata()?.permissions();
    ///     perms.set_readonly(true);
    ///     file.set_permissions(perms)?;
    ///     Ok(())
    /// }
    /// ```
    ///
    /// Note that this method alters the permissions of the underlying file, even though it takes `&self` rather than `&mut self`. <br>请注意，即使使用 `&self` 而不是 `&mut self`，此方法也会更改底层文件的权限。<br>
    ///
    ///
    #[stable(feature = "set_permissions_atomic", since = "1.16.0")]
    pub fn set_permissions(&self, perm: Permissions) -> io::Result<()> {
        self.inner.set_permissions(perm.0)
    }
}

// In addition to the `impl`s here, `File` also has `impl`s for `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` and `AsRawFd`/`IntoRawFd`/`FromRawFd`, on Unix and WASI, and `AsHandle`/`From<OwnedHandle>`/`Into<OwnedHandle>` and `AsRawHandle`/`IntoRawHandle`/`FromRawHandle` on Windows. <br>除了这里的 impl 之外，`File` 还有 `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` 和 `AsRawFd`/`IntoRawFd`/`FromRawFd`、Unix 和 WASI 以及 Windows 上的 `AsHandle`/`From<OwnedHandle>`/`Into<OwnedHandle>` 和 `AsRawHandle`/`IntoRawHandle`/`FromRawHandle` 的 impl。<br>
//
//
//
//

impl AsInner<fs_imp::File> for File {
    fn as_inner(&self) -> &fs_imp::File {
        &self.inner
    }
}
impl FromInner<fs_imp::File> for File {
    fn from_inner(f: fs_imp::File) -> File {
        File { inner: f }
    }
}
impl IntoInner<fs_imp::File> for File {
    fn into_inner(self) -> fs_imp::File {
        self.inner
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for File {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.inner.fmt(f)
    }
}

/// Indicates how much extra capacity is needed to read the rest of the file. <br>指示读取文件的其余部分需要多少额外容量。<br>
fn buffer_capacity_required(mut file: &File) -> usize {
    let size = file.metadata().map(|m| m.len()).unwrap_or(0);
    let pos = file.stream_position().unwrap_or(0);
    // Don't worry about `usize` overflow because reading will fail regardless in that case. <br>不必担心 `usize` 溢出，因为无论那种情况，读取都会失败。<br>
    //
    size.saturating_sub(pos) as usize
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Read for File {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.inner.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.inner.read_vectored(bufs)
    }

    fn read_buf(&mut self, buf: &mut ReadBuf<'_>) -> io::Result<()> {
        self.inner.read_buf(buf)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.inner.is_read_vectored()
    }

    // Reserves space in the buffer based on the file size when available. <br>根据可用的文件大小在缓冲区中保留空间。<br>
    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
        buf.reserve(buffer_capacity_required(self));
        io::default_read_to_end(self, buf)
    }

    // Reserves space in the buffer based on the file size when available. <br>根据可用的文件大小在缓冲区中保留空间。<br>
    fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
        buf.reserve(buffer_capacity_required(self));
        io::default_read_to_string(self, buf)
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Write for File {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.inner.write(buf)
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.inner.write_vectored(bufs)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.inner.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        self.inner.flush()
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Seek for File {
    fn seek(&mut self, pos: SeekFrom) -> io::Result<u64> {
        self.inner.seek(pos)
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Read for &File {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.inner.read(buf)
    }

    fn read_buf(&mut self, buf: &mut ReadBuf<'_>) -> io::Result<()> {
        self.inner.read_buf(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.inner.read_vectored(bufs)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.inner.is_read_vectored()
    }

    // Reserves space in the buffer based on the file size when available. <br>根据可用的文件大小在缓冲区中保留空间。<br>
    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
        buf.reserve(buffer_capacity_required(self));
        io::default_read_to_end(self, buf)
    }

    // Reserves space in the buffer based on the file size when available. <br>根据可用的文件大小在缓冲区中保留空间。<br>
    fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
        buf.reserve(buffer_capacity_required(self));
        io::default_read_to_string(self, buf)
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Write for &File {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.inner.write(buf)
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.inner.write_vectored(bufs)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.inner.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        self.inner.flush()
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Seek for &File {
    fn seek(&mut self, pos: SeekFrom) -> io::Result<u64> {
        self.inner.seek(pos)
    }
}

impl OpenOptions {
    /// Creates a blank new set of options ready for configuration. <br>创建一组可供配置的空白新选项。<br>
    ///
    /// All options are initially set to `false`. <br>所有选项最初都设置为 `false`。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::OpenOptions;
    ///
    /// let mut options = OpenOptions::new();
    /// let file = options.read(true).open("foo.txt");
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn new() -> Self {
        OpenOptions(fs_imp::OpenOptions::new())
    }

    /// Sets the option for read access. <br>设置读取访问权限的选项。<br>
    ///
    /// This option, when true, will indicate that the file should be `read`-able if opened. <br>该选项为 true 时，则表示打开的文件应该是可读的。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::OpenOptions;
    ///
    /// let file = OpenOptions::new().read(true).open("foo.txt");
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn read(&mut self, read: bool) -> &mut Self {
        self.0.read(read);
        self
    }

    /// Sets the option for write access. <br>设置写访问权限的选项。<br>
    ///
    /// This option, when true, will indicate that the file should be `write`-able if opened. <br>此选项为 true 时，则表示打开的文件应该是可写的。<br>
    ///
    /// If the file already exists, any write calls on it will overwrite its contents, without truncating it. <br>如果该文件已经存在，则对该文件的任何写调用都将覆盖其内容，而不会将其截断。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::OpenOptions;
    ///
    /// let file = OpenOptions::new().write(true).open("foo.txt");
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn write(&mut self, write: bool) -> &mut Self {
        self.0.write(write);
        self
    }

    /// Sets the option for the append mode. <br>设置追加模式的选项。<br>
    ///
    /// This option, when true, means that writes will append to a file instead of overwriting previous contents. <br>此选项为 true 时，表示写入将追加到文件中，而不是覆盖以前的内容。<br>
    /// Note that setting `.write(true).append(true)` has the same effect as setting only `.append(true)`. <br>请注意，设置 `.write(true).append(true)` 与仅设置 `.append(true)` 具有相同的效果。<br>
    ///
    /// For most filesystems, the operating system guarantees that all writes are atomic: no writes get mangled because another process writes at the same time. <br>对于大多数文件系统，操作系统保证所有写操作都是原子的：不会浪费任何写操作，因为另一个进程会同时进行写操作。<br>
    ///
    /// One maybe obvious note when using append-mode: make sure that all data that belongs together is written to the file in one operation. <br>使用追加模式时，可能有一个明显的注意事项：确保一次完成将所有在一起的数据写入文件。<br>
    /// This can be done by concatenating strings before passing them to [`write()`], or using a buffered writer (with a buffer of adequate size), and calling [`flush()`] when the message is complete. <br>这可以通过在将字符串传递给 [`write()`] 之前串联字符串，或使用缓冲的 writer (具有足够大小的缓冲区) 并在消息完成后调用 [`flush()`] 来完成。<br>
    ///
    ///
    /// If a file is opened with both read and append access, beware that after opening, and after every write, the position for reading may be set at the end of the file. <br>如果同时使用读取和追加的访问权限打开文件，请注意，在打开之后以及每次写入之后，读取位置可能设置在文件末尾。<br>
    /// So, before writing, save the current position (using <code>[seek]\([SeekFrom]::[Current]\(0))</code>), and restore it before the next read. <br>所以，在写入之前，保存当前位置 (使用 <code>[seek]\([SeekFrom]::[Current]\(0))</code>)，并在下次读取之前恢复它。<br>
    ///
    /// ## Note
    ///
    /// This function doesn't create the file if it doesn't exist. <br>如果该函数不存在，则该函数不会创建该文件。<br> Use the [`OpenOptions::create`] method to do so. <br>使用 [`OpenOptions::create`] 方法来执行此操作。<br>
    ///
    /// [`write()`]: Write::write "io::Write::write"
    /// [`flush()`]: Write::flush "io::Write::flush"
    /// [seek]: Seek::seek "io::Seek::seek"
    /// [Current]: SeekFrom::Current "io::SeekFrom::Current"
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::OpenOptions;
    ///
    /// let file = OpenOptions::new().append(true).open("foo.txt");
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn append(&mut self, append: bool) -> &mut Self {
        self.0.append(append);
        self
    }

    /// Sets the option for truncating a previous file. <br>设置截断上一个文件的选项。<br>
    ///
    /// If a file is successfully opened with this option set it will truncate the file to 0 length if it already exists. <br>如果使用此选项设置成功打开了文件，则如果文件已经存在，它将把文件的长度截断为 0。<br>
    ///
    ///
    /// The file must be opened with write access for truncate to work. <br>该文件必须具有写访问权限才能打开，才能进行截断。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::OpenOptions;
    ///
    /// let file = OpenOptions::new().write(true).truncate(true).open("foo.txt");
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn truncate(&mut self, truncate: bool) -> &mut Self {
        self.0.truncate(truncate);
        self
    }

    /// Sets the option to create a new file, or open it if it already exists. <br>设置选项以创建一个新文件，或者如果已经存在则将其打开。<br>
    ///
    /// In order for the file to be created, [`OpenOptions::write`] or [`OpenOptions::append`] access must be used. <br>为了创建文件，必须使用 [`OpenOptions::write`] 或 [`OpenOptions::append`] 访问。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::OpenOptions;
    ///
    /// let file = OpenOptions::new().write(true).create(true).open("foo.txt");
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn create(&mut self, create: bool) -> &mut Self {
        self.0.create(create);
        self
    }

    /// Sets the option to create a new file, failing if it already exists. <br>设置创建新文件的选项，如果该文件已经存在则失败。<br>
    ///
    /// No file is allowed to exist at the target location, also no (dangling) symlink. <br>目标位置不允许存在任何文件，(dangling) 符号链接也不允许存在。<br> In this way, if the call succeeds, the file returned is guaranteed to be new. <br>这样，如果调用成功，则保证返回的文件是新文件。<br>
    ///
    /// This option is useful because it is atomic. <br>此选项很有用，因为它是原子的。<br>
    /// Otherwise between checking whether a file exists and creating a new one, the file may have been created by another process (a TOCTOU race condition / attack). <br>否则，在检查文件是否存在与创建新文件之间，文件可能是由另一个进程创建的 (TOCTOU 竞态条件 / 攻击)。<br>
    ///
    ///
    /// If `.create_new(true)` is set, [`.create()`] and [`.truncate()`] are ignored. <br>如果设置了 `.create_new(true)`，则忽略 [`.create()`] 和 [`.truncate()`]。<br>
    ///
    /// The file must be opened with write or append access in order to create a new file. <br>必须使用写或追加访问权限打开文件才能创建新文件。<br>
    ///
    /// [`.create()`]: OpenOptions::create
    /// [`.truncate()`]: OpenOptions::truncate
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::OpenOptions;
    ///
    /// let file = OpenOptions::new().write(true)
    ///                              .create_new(true)
    ///                              .open("foo.txt");
    /// ```
    ///
    ///
    ///
    #[stable(feature = "expand_open_options2", since = "1.9.0")]
    pub fn create_new(&mut self, create_new: bool) -> &mut Self {
        self.0.create_new(create_new);
        self
    }

    /// Opens a file at `path` with the options specified by `self`. <br>使用 `self` 指定的选项在 `path` 打开文件。<br>
    ///
    /// # Errors
    ///
    /// This function will return an error under a number of different circumstances. <br>在许多不同的情况下，此函数将返回错误。<br> Some of these error conditions are listed here, together with their [`io::ErrorKind`]. <br>其中列出了一些错误条件及其 [`io::ErrorKind`]。<br>
    /// The mapping to [`io::ErrorKind`]s is not part of the compatibility contract of the function. <br>映射到 [`io::ErrorKind`] 不是函数兼容性契约的一部分。<br>
    ///
    /// * [`NotFound`]: The specified file does not exist and neither `create` or `create_new` is set. <br>[`NotFound`]: 指定的文件不存在，并且未设置 `create` 或 `create_new`。<br>
    /// * [`NotFound`]: One of the directory components of the file path does not exist. <br>[`NotFound`]: 文件路径的目录组件之一不存在。<br>
    /// * [`PermissionDenied`]: The user lacks permission to get the specified access rights for the file. <br>[`PermissionDenied`]: 用户缺乏获取文件指定访问权限的权限。<br>
    /// * [`PermissionDenied`]: The user lacks permission to open one of the directory components of the specified path. <br>[`PermissionDenied`]: 用户没有权限打开指定路径的某个目录组件。<br>
    /// * [`AlreadyExists`]: `create_new` was specified and the file already exists. <br>[`AlreadyExists`]: 指定了 `create_new` 并且文件已经存在。<br>
    /// * [`InvalidInput`]: Invalid combinations of open options (truncate without write access, no access mode set, etc.). <br>[`InvalidInput`]: 打开选项无效组合 (在没有写访问、没有访问模式设置等情况下截断)。<br>
    ///
    /// The following errors don't match any existing [`io::ErrorKind`] at the moment: <br>以下错误目前与任何现有的 [`io::ErrorKind`] 都不匹配：<br>
    /// * One of the directory components of the specified file path was not, in fact, a directory. <br>实际上，指定文件路径的目录组件之一不是目录。<br>
    /// * Filesystem-level errors: full disk, write permission requested on a read-only file system, exceeded disk quota, too many open files, too long filename, too many symbolic links in the specified path (Unix-like systems only), etc. <br>文件系统级错误：已满磁盘，对只读文件系统请求的写许可权，超出磁盘配额，打开的文件过多，文件名太长，指定路径中的符号链接太多 (仅适用于 Unix 系统)，等等。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::OpenOptions;
    ///
    /// let file = OpenOptions::new().read(true).open("foo.txt");
    /// ```
    ///
    /// [`AlreadyExists`]: io::ErrorKind::AlreadyExists
    /// [`InvalidInput`]: io::ErrorKind::InvalidInput
    /// [`NotFound`]: io::ErrorKind::NotFound
    /// [`PermissionDenied`]: io::ErrorKind::PermissionDenied
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn open<P: AsRef<Path>>(&self, path: P) -> io::Result<File> {
        self._open(path.as_ref())
    }

    fn _open(&self, path: &Path) -> io::Result<File> {
        fs_imp::File::open(path, &self.0).map(|inner| File { inner })
    }
}

impl AsInner<fs_imp::OpenOptions> for OpenOptions {
    fn as_inner(&self) -> &fs_imp::OpenOptions {
        &self.0
    }
}

impl AsInnerMut<fs_imp::OpenOptions> for OpenOptions {
    fn as_inner_mut(&mut self) -> &mut fs_imp::OpenOptions {
        &mut self.0
    }
}

impl Metadata {
    /// Returns the file type for this metadata. <br>返回此元数据的文件类型。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// fn main() -> std::io::Result<()> {
    ///     use std::fs;
    ///
    ///     let metadata = fs::metadata("foo.txt")?;
    ///
    ///     println!("{:?}", metadata.file_type());
    ///     Ok(())
    /// }
    /// ```
    #[must_use]
    #[stable(feature = "file_type", since = "1.1.0")]
    pub fn file_type(&self) -> FileType {
        FileType(self.0.file_type())
    }

    /// Returns `true` if this metadata is for a directory. <br>如果此元数据用于目录，则返回 `true`。<br>
    /// The result is mutually exclusive to the result of [`Metadata::is_file`], and will be false for symlink metadata obtained from [`symlink_metadata`]. <br>结果与 [`Metadata::is_file`] 的结果互斥，并且对于从 [`symlink_metadata`] 获得的符号链接元数据将为 false。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// fn main() -> std::io::Result<()> {
    ///     use std::fs;
    ///
    ///     let metadata = fs::metadata("foo.txt")?;
    ///
    ///     assert!(!metadata.is_dir());
    ///     Ok(())
    /// }
    /// ```
    ///
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn is_dir(&self) -> bool {
        self.file_type().is_dir()
    }

    /// Returns `true` if this metadata is for a regular file. <br>如果此元数据用于常规文件，则返回 `true`。<br>
    /// The result is mutually exclusive to the result of [`Metadata::is_dir`], and will be false for symlink metadata obtained from [`symlink_metadata`]. <br>结果与 [`Metadata::is_dir`] 的结果互斥，并且对于从 [`symlink_metadata`] 获得的符号链接元数据将为 false。<br>
    ///
    ///
    /// When the goal is simply to read from (or write to) the source, the most reliable way to test the source can be read (or written to) is to open it. <br>当目标只是读取 (或写入) 源时，可以读取 (或写入) 最可靠的测试源方法是打开它。<br>
    /// Only using `is_file` can break workflows like `diff <( prog_a )` on a Unix-like system for example. <br>例如，仅使用 `is_file` 才能中断类似 Unix 的系统上的工作流，例如 `diff <( prog_a )`。<br>
    /// See [`File::open`] or [`OpenOptions::open`] for more information. <br>有关更多信息，请参见 [`File::open`] 或 [`OpenOptions::open`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let metadata = fs::metadata("foo.txt")?;
    ///
    ///     assert!(metadata.is_file());
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn is_file(&self) -> bool {
        self.file_type().is_file()
    }

    /// Returns `true` if this metadata is for a symbolic link. <br>如果此元数据用于符号链接，则返回 `true`。<br>
    ///
    /// # Examples
    ///
    #[cfg_attr(unix, doc = "```no_run")]
    #[cfg_attr(not(unix), doc = "```ignore")]
    /// use std::fs;
    /// use std::path::Path;
    /// use std::os::unix::fs::symlink;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let link_path = Path::new("link");
    ///     symlink("/origin_does_not_exist/", link_path)?;
    ///
    ///     let metadata = fs::symlink_metadata(link_path)?;
    ///
    ///     assert!(metadata.is_symlink());
    ///     Ok(())
    /// }
    /// ```
    #[must_use]
    #[stable(feature = "is_symlink", since = "1.58.0")]
    pub fn is_symlink(&self) -> bool {
        self.file_type().is_symlink()
    }

    /// Returns the size of the file, in bytes, this metadata is for. <br>返回此元数据用于的文件大小 (以字节为单位)。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let metadata = fs::metadata("foo.txt")?;
    ///
    ///     assert_eq!(0, metadata.len());
    ///     Ok(())
    /// }
    /// ```
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn len(&self) -> u64 {
        self.0.size()
    }

    /// Returns the permissions of the file this metadata is for. <br>返回此元数据所针对的文件的权限。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let metadata = fs::metadata("foo.txt")?;
    ///
    ///     assert!(!metadata.permissions().readonly());
    ///     Ok(())
    /// }
    /// ```
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn permissions(&self) -> Permissions {
        Permissions(self.0.perm())
    }

    /// Returns the last modification time listed in this metadata. <br>返回此元数据中列出的最后修改时间。<br>
    ///
    /// The returned value corresponds to the `mtime` field of `stat` on Unix platforms and the `ftLastWriteTime` field on Windows platforms. <br>返回的值对应于 Unix 平台上的 `stat` 的 `mtime` 字段和 Windows 平台上的 `ftLastWriteTime` 字段。<br>
    ///
    ///
    /// # Errors
    ///
    /// This field might not be available on all platforms, and will return an `Err` on platforms where it is not available. <br>此字段可能并非在所有平台上都可用，并且在它不可用的平台上将返回 `Err`。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let metadata = fs::metadata("foo.txt")?;
    ///
    ///     if let Ok(time) = metadata.modified() {
    ///         println!("{time:?}");
    ///     } else {
    ///         println!("Not supported on this platform");
    ///     }
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "fs_time", since = "1.10.0")]
    pub fn modified(&self) -> io::Result<SystemTime> {
        self.0.modified().map(FromInner::from_inner)
    }

    /// Returns the last access time of this metadata. <br>返回此元数据的最后访问时间。<br>
    ///
    /// The returned value corresponds to the `atime` field of `stat` on Unix platforms and the `ftLastAccessTime` field on Windows platforms. <br>返回的值对应于 Unix 平台上的 `stat` 的 `atime` 字段和 Windows 平台上的 `ftLastAccessTime` 字段。<br>
    ///
    /// Note that not all platforms will keep this field update in a file's metadata, for example Windows has an option to disable updating this time when files are accessed and Linux similarly has `noatime`. <br>请注意，并非所有平台都会在文件的元数据中保留此字段的更新，例如，Windows 可以选择在访问文件时禁用此更新，而 Linux 同样具有 `noatime`。<br>
    ///
    ///
    /// # Errors
    ///
    /// This field might not be available on all platforms, and will return an `Err` on platforms where it is not available. <br>此字段可能并非在所有平台上都可用，并且在它不可用的平台上将返回 `Err`。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let metadata = fs::metadata("foo.txt")?;
    ///
    ///     if let Ok(time) = metadata.accessed() {
    ///         println!("{time:?}");
    ///     } else {
    ///         println!("Not supported on this platform");
    ///     }
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    #[stable(feature = "fs_time", since = "1.10.0")]
    pub fn accessed(&self) -> io::Result<SystemTime> {
        self.0.accessed().map(FromInner::from_inner)
    }

    /// Returns the creation time listed in this metadata. <br>返回此元数据中列出的创建时间。<br>
    ///
    /// The returned value corresponds to the `btime` field of `statx` on Linux kernel starting from to 4.11, the `birthtime` field of `stat` on other Unix platforms, and the `ftCreationTime` field on Windows platforms. <br>返回的值与从 4.11 开始的 Linux 内核上的 `statx` 的 `btime` 字段，在其他 Unix 平台上的 `stat` 的 `birthtime` 字段以及在 Windows 平台上的 `ftCreationTime` 字段相对应。<br>
    ///
    ///
    /// # Errors
    ///
    /// This field might not be available on all platforms, and will return an `Err` on platforms or filesystems where it is not available. <br>此字段可能并非在所有平台上都可用，并且会在它不可用的平台或文件系统上返回 `Err`。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let metadata = fs::metadata("foo.txt")?;
    ///
    ///     if let Ok(time) = metadata.created() {
    ///         println!("{time:?}");
    ///     } else {
    ///         println!("Not supported on this platform or filesystem");
    ///     }
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "fs_time", since = "1.10.0")]
    pub fn created(&self) -> io::Result<SystemTime> {
        self.0.created().map(FromInner::from_inner)
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Metadata {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Metadata")
            .field("file_type", &self.file_type())
            .field("is_dir", &self.is_dir())
            .field("is_file", &self.is_file())
            .field("permissions", &self.permissions())
            .field("modified", &self.modified())
            .field("accessed", &self.accessed())
            .field("created", &self.created())
            .finish_non_exhaustive()
    }
}

impl AsInner<fs_imp::FileAttr> for Metadata {
    fn as_inner(&self) -> &fs_imp::FileAttr {
        &self.0
    }
}

impl FromInner<fs_imp::FileAttr> for Metadata {
    fn from_inner(attr: fs_imp::FileAttr) -> Metadata {
        Metadata(attr)
    }
}

impl Permissions {
    /// Returns `true` if these permissions describe a readonly (unwritable) file. <br>如果这些权限描述了只读 (unwritable) 文件，则返回 `true`。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut f = File::create("foo.txt")?;
    ///     let metadata = f.metadata()?;
    ///
    ///     assert_eq!(false, metadata.permissions().readonly());
    ///     Ok(())
    /// }
    /// ```
    #[must_use = "call `set_readonly` to modify the readonly flag"]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn readonly(&self) -> bool {
        self.0.readonly()
    }

    /// Modifies the readonly flag for this set of permissions. <br>修改此权限集的只读标志。<br> If the `readonly` argument is `true`, using the resulting `Permission` will update file permissions to forbid writing. <br>如果 `readonly` 参数是 `true`，则使用生成的 `Permission` 将更新文件权限以禁止写入。<br>
    ///
    /// Conversely, if it's `false`, using the resulting `Permission` will update file permissions to allow writing. <br>相反，如果是 `false`，则使用生成的 `Permission` 将更新文件权限以允许写入。<br>
    ///
    /// This operation does **not** modify the filesystem. <br>此操作不修改文件系统。<br> To modify the filesystem use the [`set_permissions`] function. <br>要修改文件系统，请使用 [`set_permissions`] 函数。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let f = File::create("foo.txt")?;
    ///     let metadata = f.metadata()?;
    ///     let mut permissions = metadata.permissions();
    ///
    ///     permissions.set_readonly(true);
    ///
    ///     // filesystem doesn't change <br>文件系统不会改变<br>
    ///     assert_eq!(false, metadata.permissions().readonly());
    ///
    ///     // just this particular `permissions`. <br>只是这个特殊的 `permissions`。<br>
    ///     assert_eq!(true, permissions.readonly());
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn set_readonly(&mut self, readonly: bool) {
        self.0.set_readonly(readonly)
    }
}

impl FileType {
    /// Tests whether this file type represents a directory. <br>测试此文件类型是否代表目录。<br>
    /// The result is mutually exclusive to the results of [`is_file`] and [`is_symlink`]; <br>结果与 [`is_file`] 和 [`is_symlink`] 的结果互斥；<br> only zero or one of these tests may pass. <br>这些测试只能通过零或其中一项。<br>
    ///
    ///
    /// [`is_file`]: FileType::is_file
    /// [`is_symlink`]: FileType::is_symlink
    ///
    /// # Examples
    ///
    /// ```no_run
    /// fn main() -> std::io::Result<()> {
    ///     use std::fs;
    ///
    ///     let metadata = fs::metadata("foo.txt")?;
    ///     let file_type = metadata.file_type();
    ///
    ///     assert_eq!(file_type.is_dir(), false);
    ///     Ok(())
    /// }
    /// ```
    ///
    #[must_use]
    #[stable(feature = "file_type", since = "1.1.0")]
    pub fn is_dir(&self) -> bool {
        self.0.is_dir()
    }

    /// Tests whether this file type represents a regular file. <br>测试此文件类型是否代表常规文件。<br>
    /// The result is  mutually exclusive to the results of [`is_dir`] and [`is_symlink`]; <br>结果与 [`is_dir`] 和 [`is_symlink`] 的结果互斥；<br> only zero or one of these tests may pass. <br>这些测试只能通过零或其中一项。<br>
    ///
    /// When the goal is simply to read from (or write to) the source, the most reliable way to test the source can be read (or written to) is to open it. <br>当目标只是读取 (或写入) 源时，可以读取 (或写入) 最可靠的测试源方法是打开它。<br>
    /// Only using `is_file` can break workflows like `diff <( prog_a )` on a Unix-like system for example. <br>例如，仅使用 `is_file` 才能中断类似 Unix 的系统上的工作流，例如 `diff <( prog_a )`。<br>
    /// See [`File::open`] or [`OpenOptions::open`] for more information. <br>有关更多信息，请参见 [`File::open`] 或 [`OpenOptions::open`]。<br>
    ///
    /// [`is_dir`]: FileType::is_dir
    /// [`is_symlink`]: FileType::is_symlink
    ///
    /// # Examples
    ///
    /// ```no_run
    /// fn main() -> std::io::Result<()> {
    ///     use std::fs;
    ///
    ///     let metadata = fs::metadata("foo.txt")?;
    ///     let file_type = metadata.file_type();
    ///
    ///     assert_eq!(file_type.is_file(), true);
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    ///
    #[must_use]
    #[stable(feature = "file_type", since = "1.1.0")]
    pub fn is_file(&self) -> bool {
        self.0.is_file()
    }

    /// Tests whether this file type represents a symbolic link. <br>测试此文件类型是否代表符号链接。<br>
    /// The result is mutually exclusive to the results of [`is_dir`] and [`is_file`]; <br>结果与 [`is_dir`] 和 [`is_file`] 的结果互斥；<br> only zero or one of these tests may pass. <br>这些测试只能通过零或其中一项。<br>
    ///
    /// The underlying [`Metadata`] struct needs to be retrieved with the [`fs::symlink_metadata`] function and not the [`fs::metadata`] function. <br>需要使用 [`fs::symlink_metadata`] 函数而不是 [`fs::metadata`] 函数来检索底层 [`Metadata`] 结构。<br>
    /// The [`fs::metadata`] function follows symbolic links, so [`is_symlink`] would always return `false` for the target file. <br>[`fs::metadata`] 函数遵循符号链接，因此 [`is_symlink`] 将始终为目标文件返回 `false`。<br>
    ///
    /// [`fs::metadata`]: metadata
    /// [`fs::symlink_metadata`]: symlink_metadata
    /// [`is_dir`]: FileType::is_dir
    /// [`is_file`]: FileType::is_file
    /// [`is_symlink`]: FileType::is_symlink
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let metadata = fs::symlink_metadata("foo.txt")?;
    ///     let file_type = metadata.file_type();
    ///
    ///     assert_eq!(file_type.is_symlink(), false);
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[must_use]
    #[stable(feature = "file_type", since = "1.1.0")]
    pub fn is_symlink(&self) -> bool {
        self.0.is_symlink()
    }
}

impl AsInner<fs_imp::FileType> for FileType {
    fn as_inner(&self) -> &fs_imp::FileType {
        &self.0
    }
}

impl FromInner<fs_imp::FilePermissions> for Permissions {
    fn from_inner(f: fs_imp::FilePermissions) -> Permissions {
        Permissions(f)
    }
}

impl AsInner<fs_imp::FilePermissions> for Permissions {
    fn as_inner(&self) -> &fs_imp::FilePermissions {
        &self.0
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Iterator for ReadDir {
    type Item = io::Result<DirEntry>;

    fn next(&mut self) -> Option<io::Result<DirEntry>> {
        self.0.next().map(|entry| entry.map(DirEntry))
    }
}

impl DirEntry {
    /// Returns the full path to the file that this entry represents. <br>返回此条目表示的文件的完整路径。<br>
    ///
    /// The full path is created by joining the original path to `read_dir` with the filename of this entry. <br>通过将 `read_dir` 的原始路径与该条目的文件名连接起来，可以创建完整路径。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     for entry in fs::read_dir(".")? {
    ///         let dir = entry?;
    ///         println!("{:?}", dir.path());
    ///     }
    ///     Ok(())
    /// }
    /// ```
    ///
    /// This prints output like: <br>打印输出如下：<br>
    ///
    /// ```text
    /// "./whatever.txt"
    /// "./foo.html"
    /// "./hello_world.rs"
    /// ```
    ///
    /// The exact text, of course, depends on what files you have in `.`. <br>当然，确切的文本取决于 `.` 中包含的文件。<br>
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn path(&self) -> PathBuf {
        self.0.path()
    }

    /// Returns the metadata for the file that this entry points at. <br>返回此条目指向的文件的元数据。<br>
    ///
    /// This function will not traverse symlinks if this entry points at a symlink. <br>如果此函数指向符号链接，则该函数将不会遍历符号链接。<br> To traverse symlinks use [`fs::metadata`] or [`fs::File::metadata`]. <br>要遍历符号链接，请使用 [`fs::metadata`] 或 [`fs::File::metadata`]。<br>
    ///
    /// [`fs::metadata`]: metadata
    /// [`fs::File::metadata`]: File::metadata
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// On Windows this function is cheap to call (no extra system calls needed), but on Unix platforms this function is the equivalent of calling `symlink_metadata` on the path. <br>在 Windows 上，此函数的调用很便宜 (不需要额外的系统调用)，但是在 Unix 平台上，此函数等效于在路径上调用 `symlink_metadata`。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fs;
    ///
    /// if let Ok(entries) = fs::read_dir(".") {
    ///     for entry in entries {
    ///         if let Ok(entry) = entry {
    ///             // Here, `entry` is a `DirEntry`. <br>在此，`entry` 是 `DirEntry`。<br>
    ///             if let Ok(metadata) = entry.metadata() {
    ///                 // Now let's show our entry's permissions! <br>现在，让我们显示条目的权限！<br>
    ///                 println!("{:?}: {:?}", entry.path(), metadata.permissions());
    ///             } else {
    ///                 println!("Couldn't get metadata for {:?}", entry.path());
    ///             }
    ///         }
    ///     }
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "dir_entry_ext", since = "1.1.0")]
    pub fn metadata(&self) -> io::Result<Metadata> {
        self.0.metadata().map(Metadata)
    }

    /// Returns the file type for the file that this entry points at. <br>返回此条目指向的文件的文件类型。<br>
    ///
    /// This function will not traverse symlinks if this entry points at a symlink. <br>如果此函数指向符号链接，则该函数将不会遍历符号链接。<br>
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// On Windows and most Unix platforms this function is free (no extra system calls needed), but some Unix platforms may require the equivalent call to `symlink_metadata` to learn about the target file type. <br>在 Windows 和大多数 Unix 平台上，此函数是免费的 (不需要额外的系统调用)，但是某些 Unix 平台可能需要与 `symlink_metadata` 等效的调用才能了解目标文件类型。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fs;
    ///
    /// if let Ok(entries) = fs::read_dir(".") {
    ///     for entry in entries {
    ///         if let Ok(entry) = entry {
    ///             // Here, `entry` is a `DirEntry`. <br>在此，`entry` 是 `DirEntry`。<br>
    ///             if let Ok(file_type) = entry.file_type() {
    ///                 // Now let's show our entry's file type! <br>现在，让我们显示条目的文件类型！<br>
    ///                 println!("{:?}: {:?}", entry.path(), file_type);
    ///             } else {
    ///                 println!("Couldn't get file type for {:?}", entry.path());
    ///             }
    ///         }
    ///     }
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "dir_entry_ext", since = "1.1.0")]
    pub fn file_type(&self) -> io::Result<FileType> {
        self.0.file_type().map(FileType)
    }

    /// Returns the bare file name of this directory entry without any other leading path component. <br>返回此目录条目的裸文件名，不包含任何其他前导路径组件。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fs;
    ///
    /// if let Ok(entries) = fs::read_dir(".") {
    ///     for entry in entries {
    ///         if let Ok(entry) = entry {
    ///             // Here, `entry` is a `DirEntry`. <br>在此，`entry` 是 `DirEntry`。<br>
    ///             println!("{:?}", entry.file_name());
    ///         }
    ///     }
    /// }
    /// ```
    #[must_use]
    #[stable(feature = "dir_entry_ext", since = "1.1.0")]
    pub fn file_name(&self) -> OsString {
        self.0.file_name()
    }
}

#[stable(feature = "dir_entry_debug", since = "1.13.0")]
impl fmt::Debug for DirEntry {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_tuple("DirEntry").field(&self.path()).finish()
    }
}

impl AsInner<fs_imp::DirEntry> for DirEntry {
    fn as_inner(&self) -> &fs_imp::DirEntry {
        &self.0
    }
}

/// Removes a file from the filesystem. <br>从文件系统中删除文件。<br>
///
/// Note that there is no guarantee that the file is immediately deleted (e.g., depending on platform, other open file descriptors may prevent immediate removal). <br>请注意，不能保证立即删除文件 (例如，取决于平台，其他打开的文件描述符可能会阻止立即删除)。<br>
///
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `unlink` function on Unix and the `DeleteFile` function on Windows. <br>该函数当前对应于 Unix 上的 `unlink` 函数和 Windows 上的 `DeleteFile` 函数。<br>
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * `path` points to a directory. <br>`path` 指向一个目录。<br>
/// * The file doesn't exist. <br>该文件不存在。<br>
/// * The user lacks permissions to remove the file. <br>用户没有删除文件的权限。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::remove_file("a.txt")?;
///     Ok(())
/// }
/// ```
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn remove_file<P: AsRef<Path>>(path: P) -> io::Result<()> {
    fs_imp::unlink(path.as_ref())
}

/// Given a path, query the file system to get information about a file, directory, etc. <br>给定路径，查询文件系统以获取有关文件，目录等的信息。<br>
///
/// This function will traverse symbolic links to query information about the destination file. <br>该函数将遍历符号链接以查询有关目标文件的信息。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `stat` function on Unix and the `GetFileInformationByHandle` function on Windows. <br>该函数目前对应于 Unix 上的 `stat` 函数和 Windows 上的 `GetFileInformationByHandle` 函数。<br>
///
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * The user lacks permissions to perform `metadata` call on `path`. <br>用户没有权限在 `path` 上执行 `metadata` 调用。<br>
/// * `path` does not exist. <br>`path` 不存在。<br>
///
/// # Examples
///
/// ```rust,no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     let attr = fs::metadata("/some/file/path.txt")?;
///     // inspect attr ... <br>检查属性...<br>
///     Ok(())
/// }
/// ```
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn metadata<P: AsRef<Path>>(path: P) -> io::Result<Metadata> {
    fs_imp::stat(path.as_ref()).map(Metadata)
}

/// Query the metadata about a file without following symlinks. <br>查询有关文件的元数据，而无需遵循符号链接。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `lstat` function on Unix and the `GetFileInformationByHandle` function on Windows. <br>该函数目前对应于 Unix 上的 `lstat` 函数和 Windows 上的 `GetFileInformationByHandle` 函数。<br>
///
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * The user lacks permissions to perform `metadata` call on `path`. <br>用户没有权限在 `path` 上执行 `metadata` 调用。<br>
/// * `path` does not exist. <br>`path` 不存在。<br>
///
/// # Examples
///
/// ```rust,no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     let attr = fs::symlink_metadata("/some/file/path.txt")?;
///     // inspect attr ... <br>检查属性...<br>
///     Ok(())
/// }
/// ```
///
#[stable(feature = "symlink_metadata", since = "1.1.0")]
pub fn symlink_metadata<P: AsRef<Path>>(path: P) -> io::Result<Metadata> {
    fs_imp::lstat(path.as_ref()).map(Metadata)
}

/// Rename a file or directory to a new name, replacing the original file if `to` already exists. <br>将文件或目录重命名为新名称，如果 `to` 已经存在，则替换原始文件。<br>
///
/// This will not work if the new name is on a different mount point. <br>如果新名称在其他安装点上，则将无法使用。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `rename` function on Unix and the `MoveFileEx` function with the `MOVEFILE_REPLACE_EXISTING` flag on Windows. <br>该函数当前对应于 Unix 上的 `rename` 函数和 Windows 上带有 `MOVEFILE_REPLACE_EXISTING` 标志的 `MoveFileEx` 函数。<br>
///
/// Because of this, the behavior when both `from` and `to` exist differs. <br>因此，`from` 和 `to` 都存在时的行为是不同的。<br> On Unix, if `from` is a directory, `to` must also be an (empty) directory. <br>在 Unix 上，如果 `from` 是目录，则 `to` 也必须是 (empty) 目录。<br> If `from` is not a directory, `to` must also be not a directory. <br>如果 `from` 不是目录，则 `to` 也必须不是目录。<br>
///
/// In contrast, on Windows, `from` can be anything, but `to` must *not* be a directory. <br>相比之下，在 Windows 上，`from` 可以是任何东西，但是 `to` 一定不是目录。<br>
///
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * `from` does not exist. <br>`from` 不存在。<br>
/// * The user lacks permissions to view contents. <br>用户没有查看内容的权限。<br>
/// * `from` and `to` are on separate filesystems. <br>`from` 和 `to` 在不同的文件系统上。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::rename("a.txt", "b.txt")?; // Rename a.txt to b.txt <br>将 a.txt 重命名为 b.txt<br>
///     Ok(())
/// }
/// ```
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn rename<P: AsRef<Path>, Q: AsRef<Path>>(from: P, to: Q) -> io::Result<()> {
    fs_imp::rename(from.as_ref(), to.as_ref())
}

/// Copies the contents of one file to another. <br>将一个文件的内容复制到另一个文件。<br> This function will also copy the permission bits of the original file to the destination file. <br>此函数还将复制原始文件的权限位到目标文件。<br>
///
/// This function will **overwrite** the contents of `to`. <br>该函数将覆盖 `to` 的内容。<br>
///
/// Note that if `from` and `to` both point to the same file, then the file will likely get truncated by this operation. <br>请注意，如果 `from` 和 `to` 都指向同一个文件，则此操作可能会截断该文件。<br>
///
/// On success, the total number of bytes copied is returned and it is equal to the length of the `to` file as reported by `metadata`. <br>成功后，将返回复制的字节总数，该总数等于 `metadata` 报告的 `to` 文件的长度。<br>
///
/// If you’re wanting to copy the contents of one file to another and you’re working with [`File`]s, see the [`io::copy()`] function. <br>如果您想将一个文件的内容复制到另一个文件，并且正在使用 [`File`]，请参见 [`io::copy()`] 函数。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `open` function in Unix with `O_RDONLY` for `from` and `O_WRONLY`, `O_CREAT`, and `O_TRUNC` for `to`. <br>此函数当前与 Unix 中的 `open` 函数相对应，其中 `from` 的 `O_RDONLY` 和 `to` 的 `O_WRONLY`，`O_CREAT` 和 `O_TRUNC`。<br>
///
/// `O_CLOEXEC` is set for returned file descriptors. <br>`O_CLOEXEC` 是为返回的文件描述符设置的。<br>
///
/// On Linux (including Android), this function attempts to use `copy_file_range(2)`, and falls back to reading and writing if that is not possible. <br>在 Linux (包括 Android) 上，该函数尝试使用 `copy_file_range(2)`，如果不可能，则回退到读取和写入。<br>
///
/// On Windows, this function currently corresponds to `CopyFileEx`. <br>在 Windows 上，此函数当前对应于 `CopyFileEx`。<br>
/// Alternate NTFS streams are copied but only the size of the main stream is returned by this function. <br>复制备用 NTFS 流，但此函数仅返回主流的大小。<br>
///
/// On MacOS, this function corresponds to `fclonefileat` and `fcopyfile`. <br>在 MacOS 上，此函数对应于 `fclonefileat` 和 `fcopyfile`。<br>
///
/// Note that platform-specific behavior [may change in the future][changes]. <br>请注意，特定于平台的行为 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * `from` is neither a regular file nor a symlink to a regular file. <br>`from` 既不是普通文件，也不是普通文件的符号链接。<br>
/// * `from` does not exist. <br>`from` 不存在。<br>
/// * The current process does not have the permission rights to read `from` or write `to`. <br>当前进程没有读取 `from` 或写入 `to` 的权限。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::copy("foo.txt", "bar.txt")?;  // Copy foo.txt to bar.txt <br>将 foo.txt 复制到 bar.txt<br>
///     Ok(())
/// }
/// ```
///
///
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn copy<P: AsRef<Path>, Q: AsRef<Path>>(from: P, to: Q) -> io::Result<u64> {
    fs_imp::copy(from.as_ref(), to.as_ref())
}

/// Creates a new hard link on the filesystem. <br>在文件系统上创建一个新的硬链接。<br>
///
/// The `link` path will be a link pointing to the `original` path. <br>`link` 路径将是指向 `original` 路径的链接。<br> Note that systems often require these two paths to both be located on the same filesystem. <br>请注意，系统通常要求这两个路径都位于同一文件系统上。<br>
///
/// If `original` names a symbolic link, it is platform-specific whether the symbolic link is followed. <br>如果 `original` 命名符号链接，则是否遵循符号链接是特定于平台的。<br>
/// On platforms where it's possible to not follow it, it is not followed, and the created hard link points to the symbolic link itself. <br>在可能不遵循它的平台上，它不会被遵循，并且创建的硬链接指向符号链接本身。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds the `CreateHardLink` function on Windows. <br>该函数目前对应于 Windows 上的 `CreateHardLink` 函数。<br>
/// On most Unix systems, it corresponds to the `linkat` function with no flags. <br>在大多数 Unix 系统上，它对应于没有标志的 `linkat` 函数。<br>
/// On Android, VxWorks, and Redox, it instead corresponds to the `link` function. <br>在 Android、VxWorks 和 Redox 上，它对应于 `link` 函数。<br>
/// On MacOS, it uses the `linkat` function if it is available, but on very old systems where `linkat` is not available, `link` is selected at runtime instead. <br>在 MacOS 上，它使用 `linkat` 函数 (如果可用)，但在 `linkat` 不可用的非常旧的系统上，`link` 在运行时被选择。<br>
///
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * The `original` path is not a file or doesn't exist. <br>`original` 路径不是文件或不存在。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::hard_link("a.txt", "b.txt")?; // Hard link a.txt to b.txt <br>硬链接 a.txt 到 b.txt<br>
///     Ok(())
/// }
/// ```
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn hard_link<P: AsRef<Path>, Q: AsRef<Path>>(original: P, link: Q) -> io::Result<()> {
    fs_imp::link(original.as_ref(), link.as_ref())
}

/// Creates a new symbolic link on the filesystem. <br>在文件系统上创建一个新的符号链接。<br>
///
/// The `link` path will be a symbolic link pointing to the `original` path. <br>`link` 路径将是指向 `original` 路径的符号链接。<br>
/// On Windows, this will be a file symlink, not a directory symlink; <br>在 Windows 上，这将是文件符号链接，而不是目录符号链接。<br>
/// for this reason, the platform-specific [`std::os::unix::fs::symlink`] and [`std::os::windows::fs::symlink_file`] or [`symlink_dir`] should be used instead to make the intent explicit. <br>因此，应该使用平台特定的 [`std::os::unix::fs::symlink`] 和 [`std::os::windows::fs::symlink_file`] 或 [`symlink_dir`] 来明确意图。<br>
///
///
/// [`std::os::unix::fs::symlink`]: crate::os::unix::fs::symlink
/// [`std::os::windows::fs::symlink_file`]: crate::os::windows::fs::symlink_file
/// [`symlink_dir`]: crate::os::windows::fs::symlink_dir
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::soft_link("a.txt", "b.txt")?;
///     Ok(())
/// }
/// ```
///
#[stable(feature = "rust1", since = "1.0.0")]
#[deprecated(
    since = "1.1.0",
    note = "replaced with std::os::unix::fs::symlink and \
            std::os::windows::fs::{symlink_file, symlink_dir}"
)]
pub fn soft_link<P: AsRef<Path>, Q: AsRef<Path>>(original: P, link: Q) -> io::Result<()> {
    fs_imp::symlink(original.as_ref(), link.as_ref())
}

/// Reads a symbolic link, returning the file that the link points to. <br>读取符号链接，返回链接指向的文件。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `readlink` function on Unix and the `CreateFile` function with `FILE_FLAG_OPEN_REPARSE_POINT` and `FILE_FLAG_BACKUP_SEMANTICS` flags on Windows. <br>该函数当前对应于 Unix 上的 `readlink` 函数，以及 Windows 上带有 `FILE_FLAG_OPEN_REPARSE_POINT` 和 `FILE_FLAG_BACKUP_SEMANTICS` 标志的 `CreateFile` 函数。<br>
///
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * `path` is not a symbolic link. <br>`path` 不是符号链接。<br>
/// * `path` does not exist. <br>`path` 不存在。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     let path = fs::read_link("a.txt")?;
///     Ok(())
/// }
/// ```
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn read_link<P: AsRef<Path>>(path: P) -> io::Result<PathBuf> {
    fs_imp::readlink(path.as_ref())
}

/// Returns the canonical, absolute form of a path with all intermediate components normalized and symbolic links resolved. <br>返回路径的规范，绝对形式，所有中间组件均已规范化且符号链接已解析。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `realpath` function on Unix and the `CreateFile` and `GetFinalPathNameByHandle` functions on Windows. <br>该函数当前对应于 Unix 上的 `realpath` 函数以及 Windows 上的 `CreateFile` 和 `GetFinalPathNameByHandle` 函数。<br>
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// On Windows, this converts the path to use [extended length path][path] syntax, which allows your program to use longer path names, but means you can only join backslash-delimited paths to it, and it may be incompatible with other applications (if passed to the application on the command-line, or written to a file another application may read). <br>在 Windows 上，这会将路径转换为使用 [扩展长度路径][path] 语法，这允许您的程序使用更长的路径名，但是意味着您只能将反斜杠分隔的路径连接到该路径，并且它可能与其他应用程序不兼容 (如果传递给该应用程序，命令行，或写入另一个应用程序可以读取的文件)。<br>
///
///
/// [changes]: io#platform-specific-behavior
/// [path]: https://docs.microsoft.com/en-us/windows/win32/fileio/naming-a-file
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * `path` does not exist. <br>`path` 不存在。<br>
/// * A non-final component in path is not a directory. <br>path 中的非最终组件不是目录。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     let path = fs::canonicalize("../a/../foo.txt")?;
///     Ok(())
/// }
/// ```
///
///
///
///
///
///
#[doc(alias = "realpath")]
#[doc(alias = "GetFinalPathNameByHandle")]
#[stable(feature = "fs_canonicalize", since = "1.5.0")]
pub fn canonicalize<P: AsRef<Path>>(path: P) -> io::Result<PathBuf> {
    fs_imp::canonicalize(path.as_ref())
}

/// Creates a new, empty directory at the provided path <br>在提供的路径中创建一个新的空目录<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `mkdir` function on Unix and the `CreateDirectory` function on Windows. <br>该函数当前对应于 Unix 上的 `mkdir` 函数和 Windows 上的 `CreateDirectory` 函数。<br>
///
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// **NOTE**: If a parent of the given path doesn't exist, this function will return an error. <br>如果给定路径的父项不存在，则此函数将返回错误。<br>
/// To create a directory and all its missing parents at the same time, use the [`create_dir_all`] function. <br>要同时创建目录及其所有丢失的父目录，请使用 [`create_dir_all`] 函数。<br>
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * User lacks permissions to create directory at `path`. <br>用户没有权限在 `path` 上创建目录。<br>
/// * A parent of the given path doesn't exist. <br>给定路径的父级不存在。<br>
/// (To create a directory and all its missing parents at the same time, use the [`create_dir_all`] function.) <br>(要同时创建目录及其所有丢失的父目录，请使用 [`create_dir_all`] 函数。)<br>
/// * `path` already exists. <br>`path` 已经存在。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::create_dir("/some/dir")?;
///     Ok(())
/// }
/// ```
///
///
///
#[doc(alias = "mkdir")]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn create_dir<P: AsRef<Path>>(path: P) -> io::Result<()> {
    DirBuilder::new().create(path.as_ref())
}

/// Recursively create a directory and all of its parent components if they are missing. <br>递归创建目录及其所有父组件 (如果缺少)。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `mkdir` function on Unix and the `CreateDirectory` function on Windows. <br>该函数当前对应于 Unix 上的 `mkdir` 函数和 Windows 上的 `CreateDirectory` 函数。<br>
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * If any directory in the path specified by `path` does not already exist and it could not be created otherwise. <br>如果 `path` 指定的路径中的任何目录都不存在，否则无法创建。<br>
/// The specific error conditions for when a directory is being created (after it is determined to not exist) are outlined by [`fs::create_dir`]. <br>[`fs::create_dir`] 概述了创建目录时 (确定目录不存在后) 的特定错误条件。<br>
///
/// Notable exception is made for situations where any of the directories specified in the `path` could not be created as it was being created concurrently. <br>对于在 `path` 中指定的任何目录无法同时创建的情况下，将创建一个明显的例外。<br>
///
/// Such cases are considered to be successful. <br>这种情况被认为是成功的。<br>
/// That is, calling `create_dir_all` concurrently from multiple threads or processes is guaranteed not to fail due to a race condition with itself. <br>即，保证了从多个线程或进程并发调用 `create_dir_all` 不会由于自身的竞争态而失败。<br>
///
/// [`fs::create_dir`]: create_dir
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::create_dir_all("/some/dir")?;
///     Ok(())
/// }
/// ```
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn create_dir_all<P: AsRef<Path>>(path: P) -> io::Result<()> {
    DirBuilder::new().recursive(true).create(path.as_ref())
}

/// Removes an empty directory. <br>删除一个空目录。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `rmdir` function on Unix and the `RemoveDirectory` function on Windows. <br>该函数当前对应于 Unix 上的 `rmdir` 函数和 Windows 上的 `RemoveDirectory` 函数。<br>
///
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * `path` doesn't exist. <br>`path` 不存在。<br>
/// * `path` isn't a directory. <br>`path` 不是目录。<br>
/// * The user lacks permissions to remove the directory at the provided `path`. <br>用户没有权限删除提供的 `path` 上的目录。<br>
/// * The directory isn't empty. <br>目录不为空。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::remove_dir("/some/dir")?;
///     Ok(())
/// }
/// ```
///
#[doc(alias = "rmdir")]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn remove_dir<P: AsRef<Path>>(path: P) -> io::Result<()> {
    fs_imp::rmdir(path.as_ref())
}

/// Removes a directory at this path, after removing all its contents. <br>删除目录中的所有内容后，将在此路径中删除该目录。<br> Use carefully! <br>小心使用！<br>
///
/// This function does **not** follow symbolic links and it will simply remove the symbolic link itself. <br>此函数不跟随符号链接，它会简单地删除符号链接本身。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to `openat`, `fdopendir`, `unlinkat` and `lstat` functions on Unix (except for macOS before version 10.10 and REDOX) and the `CreateFileW`, `GetFileInformationByHandleEx`, `SetFileInformationByHandle`, and `NtCreateFile` functions on Windows. <br>该函数目前对应于 Unix 上的 `openat`、`fdopendir`、`unlinkat` 和 `lstat` 函数 (10.10 和 REDOX 之前版本的 macOS 除外) 和 Windows 上的 `CreateFileW`、`GetFileInformationByHandleEx`、`SetFileInformationByHandle` 和 `NtCreateFile` 函数。<br>
///
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// On macOS before version 10.10 and REDOX, as well as when running in Miri for any target, this function is not protected against time-of-check to time-of-use (TOCTOU) race conditions, and should not be used in security-sensitive code on those platforms. <br>在 10.10 和 REDOX 之前版本的 macOS 上，以及在 Miri 中针对任何目标运行时，此函数不受 time-of-check to time-of-use (TOCTOU) 竞争状态的保护，不应在安全敏感代码中使用在那些平台上。<br>
/// All other platforms are protected. <br>所有其他平台都受到保护。<br>
///
/// # Errors
///
/// See [`fs::remove_file`] and [`fs::remove_dir`]. <br>请参见 [`fs::remove_file`] 和 [`fs::remove_dir`]。<br>
///
/// [`fs::remove_file`]: remove_file
/// [`fs::remove_dir`]: remove_dir
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     fs::remove_dir_all("/some/dir")?;
///     Ok(())
/// }
/// ```
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn remove_dir_all<P: AsRef<Path>>(path: P) -> io::Result<()> {
    fs_imp::remove_dir_all(path.as_ref())
}

/// Returns an iterator over the entries within a directory. <br>返回目录中条目的迭代器。<br>
///
/// The iterator will yield instances of <code>[io::Result]<[DirEntry]></code>. <br>迭代器将产生 <code>[io::Result]<[DirEntry]></code> 实例。<br>
/// New errors may be encountered after an iterator is initially constructed. <br>最初构造迭代器后，可能会遇到新的错误。<br>
/// Entries for the current and parent directories (typically `.` and `..`) are skipped. <br>当前目录和父目录 (通常为 `.` 和 `..`) 的条目将被跳过。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `opendir` function on Unix and the `FindFirstFile` function on Windows. <br>该函数当前对应于 Unix 上的 `opendir` 函数和 Windows 上的 `FindFirstFile` 函数。<br>
/// Advancing the iterator currently corresponds to `readdir` on Unix and `FindNextFile` on Windows. <br>推进迭代器当前对应于 Unix 上的 `readdir` 和 Windows 上的 `FindNextFile`。<br>
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// The order in which this iterator returns entries is platform and filesystem dependent. <br>该迭代器返回条目的顺序取决于平台和文件系统。<br>
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * The provided `path` doesn't exist. <br>提供的 `path` 不存在。<br>
/// * The process lacks permissions to view the contents. <br>该进程没有查看内容的权限。<br>
/// * The `path` points at a non-directory file. <br>`path` 指向非目录文件。<br>
///
/// # Examples
///
/// ```
/// use std::io;
/// use std::fs::{self, DirEntry};
/// use std::path::Path;
///
/// // one possible implementation of walking a directory only visiting files <br>一种仅通过访问文件来遍历目录的可能实现方式<br>
/// fn visit_dirs(dir: &Path, cb: &dyn Fn(&DirEntry)) -> io::Result<()> {
///     if dir.is_dir() {
///         for entry in fs::read_dir(dir)? {
///             let entry = entry?;
///             let path = entry.path();
///             if path.is_dir() {
///                 visit_dirs(&path, cb)?;
///             } else {
///                 cb(&entry);
///             }
///         }
///     }
///     Ok(())
/// }
/// ```
///
/// ```rust,no_run
/// use std::{fs, io};
///
/// fn main() -> io::Result<()> {
///     let mut entries = fs::read_dir(".")?
///         .map(|res| res.map(|e| e.path()))
///         .collect::<Result<Vec<_>, io::Error>>()?;
///
///     // The order in which `read_dir` returns entries is not guaranteed. <br>不保证 `read_dir` 返回条目的顺序。<br>
///     // If reproducible ordering is required the entries should be explicitly sorted. <br>如果需要可重复的排序，则应对条目进行显式排序。<br>
///
///     entries.sort();
///
///     // The entries have now been sorted by their path. <br>现在，条目已经按照路径进行了排序。<br>
///
///     Ok(())
/// }
/// ```
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn read_dir<P: AsRef<Path>>(path: P) -> io::Result<ReadDir> {
    fs_imp::readdir(path.as_ref()).map(ReadDir)
}

/// Changes the permissions found on a file or a directory. <br>更改在文件或目录上找到的权限。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// This function currently corresponds to the `chmod` function on Unix and the `SetFileAttributes` function on Windows. <br>该函数当前对应于 Unix 上的 `chmod` 函数和 Windows 上的 `SetFileAttributes` 函数。<br>
///
/// Note that, this [may change in the future][changes]. <br>注意，这个 [将来可能会改变][changes]。<br>
///
/// [changes]: io#platform-specific-behavior
///
/// # Errors
///
/// This function will return an error in the following situations, but is not limited to just these cases: <br>在以下情况下，此函数将返回错误，但不仅限于这些情况：<br>
///
/// * `path` does not exist. <br>`path` 不存在。<br>
/// * The user lacks the permission to change attributes of the file. <br>用户没有更改文件属性的权限。<br>
///
/// # Examples
///
/// ```no_run
/// use std::fs;
///
/// fn main() -> std::io::Result<()> {
///     let mut perms = fs::metadata("foo.txt")?.permissions();
///     perms.set_readonly(true);
///     fs::set_permissions("foo.txt", perms)?;
///     Ok(())
/// }
/// ```
///
#[stable(feature = "set_permissions", since = "1.1.0")]
pub fn set_permissions<P: AsRef<Path>>(path: P, perm: Permissions) -> io::Result<()> {
    fs_imp::set_perm(path.as_ref(), perm.0)
}

impl DirBuilder {
    /// Creates a new set of options with default mode/security settings for all platforms and also non-recursive. <br>使用所有平台的默认 mode/security 设置创建一组新选项，并且这些选项也是非递归的。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fs::DirBuilder;
    ///
    /// let builder = DirBuilder::new();
    /// ```
    #[stable(feature = "dir_builder", since = "1.6.0")]
    #[must_use]
    pub fn new() -> DirBuilder {
        DirBuilder { inner: fs_imp::DirBuilder::new(), recursive: false }
    }

    /// Indicates that directories should be created recursively, creating all parent directories. <br>指示应递归创建目录，并创建所有父目录。<br>
    /// Parents that do not exist are created with the same security and permissions settings. <br>使用相同的安全性和权限设置创建不存在的父级。<br>
    ///
    /// This option defaults to `false`. <br>此选项默认为 `false`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::fs::DirBuilder;
    ///
    /// let mut builder = DirBuilder::new();
    /// builder.recursive(true);
    /// ```
    ///
    #[stable(feature = "dir_builder", since = "1.6.0")]
    pub fn recursive(&mut self, recursive: bool) -> &mut Self {
        self.recursive = recursive;
        self
    }

    /// Creates the specified directory with the options configured in this builder. <br>使用在此构建器中配置的选项来创建指定的目录。<br>
    ///
    /// It is considered an error if the directory already exists unless recursive mode is enabled. <br>如果目录已经存在，除非启用了递归模式，否则将被视为错误。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::fs::{self, DirBuilder};
    ///
    /// let path = "/tmp/foo/bar/baz";
    /// DirBuilder::new()
    ///     .recursive(true)
    ///     .create(path).unwrap();
    ///
    /// assert!(fs::metadata(path).unwrap().is_dir());
    /// ```
    ///
    #[stable(feature = "dir_builder", since = "1.6.0")]
    pub fn create<P: AsRef<Path>>(&self, path: P) -> io::Result<()> {
        self._create(path.as_ref())
    }

    fn _create(&self, path: &Path) -> io::Result<()> {
        if self.recursive { self.create_dir_all(path) } else { self.inner.mkdir(path) }
    }

    fn create_dir_all(&self, path: &Path) -> io::Result<()> {
        if path == Path::new("") {
            return Ok(());
        }

        match self.inner.mkdir(path) {
            Ok(()) => return Ok(()),
            Err(ref e) if e.kind() == io::ErrorKind::NotFound => {}
            Err(_) if path.is_dir() => return Ok(()),
            Err(e) => return Err(e),
        }
        match path.parent() {
            Some(p) => self.create_dir_all(p)?,
            None => {
                return Err(io::const_io_error!(
                    io::ErrorKind::Uncategorized,
                    "failed to create whole tree",
                ));
            }
        }
        match self.inner.mkdir(path) {
            Ok(()) => Ok(()),
            Err(_) if path.is_dir() => Ok(()),
            Err(e) => Err(e),
        }
    }
}

impl AsInnerMut<fs_imp::DirBuilder> for DirBuilder {
    fn as_inner_mut(&mut self) -> &mut fs_imp::DirBuilder {
        &mut self.inner
    }
}

/// Returns `Ok(true)` if the path points at an existing entity. <br>如果路径指向现有实体，则返回 `Ok(true)`。<br>
///
/// This function will traverse symbolic links to query information about the destination file. <br>该函数将遍历符号链接以查询有关目标文件的信息。<br>
/// In case of broken symbolic links this will return `Ok(false)`. <br>如果符号链接断开，则将返回 `Ok(false)`。<br>
///
/// As opposed to the [`Path::exists`] method, this one doesn't silently ignore errors unrelated to the path not existing. <br>与 [`Path::exists`] 方法相反，此方法不会默默地忽略与不存在的路径无关的错误。<br> (E.g.
/// it will return `Err(_)` in case of permission denied on some of the parent directories.) <br>如果某些父目录的权限被拒绝，它将返回 `Err(_)`。)<br>
///
/// Note that while this avoids some pitfalls of the `exists()` method, it still can not prevent time-of-check to time-of-use (TOCTOU) bugs. <br>请注意，虽然这避免了 `exists()` 方法的一些缺陷，但它仍然无法防止时间检查到使用时间 (TOCTOU) 错误。<br>
///
/// You should only use it in scenarios where those bugs are not an issue. <br>您应该只在这些错误不是问题的情况下使用它。<br>
///
/// # Examples
///
/// ```no_run
/// #![feature(fs_try_exists)]
/// use std::fs;
///
/// assert!(!fs::try_exists("does_not_exist.txt").expect("Can't check existence of file does_not_exist.txt"));
/// assert!(fs::try_exists("/root/secret_file.txt").is_err());
/// ```
///
/// [`Path::exists`]: crate::path::Path::exists
///
// FIXME: stabilization should modify documentation of `exists()` to recommend this method instead. <br>稳定性应修改 `exists()` 的文档以推荐此方法。<br>
//
#[unstable(feature = "fs_try_exists", issue = "83186")]
#[inline]
pub fn try_exists<P: AsRef<Path>>(path: P) -> io::Result<bool> {
    fs_imp::try_exists(path.as_ref())
}
