use crate::cmp;
use crate::fmt;
use crate::io::{
    self, BufRead, IoSliceMut, Read, ReadBuf, Seek, SeekFrom, SizeHint, DEFAULT_BUF_SIZE,
};
use crate::mem::MaybeUninit;

/// The `BufReader<R>` struct adds buffering to any reader. <br>`BufReader<R>` 结构体将缓冲添加到任何 reader。<br>
///
/// It can be excessively inefficient to work directly with a [`Read`] instance. <br>直接使用 [`Read`] 实例可能会非常低效。<br>
/// For example, every call to [`read`][`TcpStream::read`] on [`TcpStream`] results in a system call. <br>例如，对 [`TcpStream`] 上 [`read`][`TcpStream::read`] 的每次调用都会导致系统调用。<br>
/// A `BufReader<R>` performs large, infrequent reads on the underlying [`Read`] and maintains an in-memory buffer of the results. <br>`BufReader<R>` 对底层 [`Read`] 进行大批量的不频繁读取，并维护结果的内存缓冲区。<br>
///
/// `BufReader<R>` can improve the speed of programs that make *small* and *repeated* read calls to the same file or network socket. <br>`BufReader<R>` 可以提高对同一文件或网络套接字进行小规模重复读取调用的程序的速度。<br>
///
/// It does not help when reading very large amounts at once, or reading just one or a few times. <br>一次读取非常多的内容，或者仅读取一次或几次，则无济于事。<br>
/// It also provides no advantage when reading from a source that is already in memory, like a <code>[Vec]\<u8></code>. <br>从已经在内存中的源读取时，它也没有任何优势，例如 <code>[Vec]\<u8></code>.<br>
///
/// When the `BufReader<R>` is dropped, the contents of its buffer will be discarded. <br>当 `BufReader<R>` 被丢弃时，其缓冲区的内容将被丢弃。<br>
/// Creating multiple instances of a `BufReader<R>` on the same stream can cause data loss. <br>在同一流上创建 `BufReader<R>` 的多个实例可能会导致数据丢失。<br>
/// Reading from the underlying reader after unwrapping the `BufReader<R>` with [`BufReader::into_inner`] can also cause data loss. <br>将 `BufReader<R>` 与 [`BufReader::into_inner`] 展开包装后，从底层 reader 进行读取也会导致数据丢失。<br>
///
///
///
///
///
// HACK(#78696): can't use `crate` for associated items <br>不能将 `crate` 用于关联的项<br>
/// [`TcpStream::read`]: super::super::super::net::TcpStream::read
/// [`TcpStream`]: crate::net::TcpStream
///
/// # Examples
///
/// ```no_run
/// use std::io::prelude::*;
/// use std::io::BufReader;
/// use std::fs::File;
///
/// fn main() -> std::io::Result<()> {
///     let f = File::open("log.txt")?;
///     let mut reader = BufReader::new(f);
///
///     let mut line = String::new();
///     let len = reader.read_line(&mut line)?;
///     println!("First line is {len} bytes long");
///     Ok(())
/// }
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub struct BufReader<R> {
    inner: R,
    buf: Box<[MaybeUninit<u8>]>,
    pos: usize,
    cap: usize,
    init: usize,
}

impl<R: Read> BufReader<R> {
    /// Creates a new `BufReader<R>` with a default buffer capacity. <br>创建一个具有默认缓冲区容量的新 `BufReader<R>`。<br>
    /// The default is currently 8 KB, but may change in the future. <br>当前默认值为 8 KB，但可能会在 future 中进行更改。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::BufReader;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let f = File::open("log.txt")?;
    ///     let reader = BufReader::new(f);
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn new(inner: R) -> BufReader<R> {
        BufReader::with_capacity(DEFAULT_BUF_SIZE, inner)
    }

    /// Creates a new `BufReader<R>` with the specified buffer capacity. <br>用指定的缓冲区容量创建一个新的 `BufReader<R>`。<br>
    ///
    /// # Examples
    ///
    /// Creating a buffer with ten bytes of capacity: <br>创建一个具有十个字节容量的缓冲区：<br>
    ///
    /// ```no_run
    /// use std::io::BufReader;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let f = File::open("log.txt")?;
    ///     let reader = BufReader::with_capacity(10, f);
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn with_capacity(capacity: usize, inner: R) -> BufReader<R> {
        let buf = Box::new_uninit_slice(capacity);
        BufReader { inner, buf, pos: 0, cap: 0, init: 0 }
    }
}

impl<R> BufReader<R> {
    /// Gets a reference to the underlying reader. <br>获取对底层 reader 的引用。<br>
    ///
    /// It is inadvisable to directly read from the underlying reader. <br>直接从底层 reader 读取是不可取的。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::BufReader;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let f1 = File::open("log.txt")?;
    ///     let reader = BufReader::new(f1);
    ///
    ///     let f2 = reader.get_ref();
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn get_ref(&self) -> &R {
        &self.inner
    }

    /// Gets a mutable reference to the underlying reader. <br>获取底层 reader 的可变引用。<br>
    ///
    /// It is inadvisable to directly read from the underlying reader. <br>直接从底层 reader 读取是不可取的。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::BufReader;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let f1 = File::open("log.txt")?;
    ///     let mut reader = BufReader::new(f1);
    ///
    ///     let f2 = reader.get_mut();
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn get_mut(&mut self) -> &mut R {
        &mut self.inner
    }

    /// Returns a reference to the internally buffered data. <br>返回对内部缓冲数据的引用。<br>
    ///
    /// Unlike [`fill_buf`], this will not attempt to fill the buffer if it is empty. <br>与 [`fill_buf`] 不同，如果缓冲区为空，它将不会尝试填充缓冲区。<br>
    ///
    /// [`fill_buf`]: BufRead::fill_buf
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::{BufReader, BufRead};
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let f = File::open("log.txt")?;
    ///     let mut reader = BufReader::new(f);
    ///     assert!(reader.buffer().is_empty());
    ///
    ///     if reader.fill_buf()?.len() > 0 {
    ///         assert!(!reader.buffer().is_empty());
    ///     }
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "bufreader_buffer", since = "1.37.0")]
    pub fn buffer(&self) -> &[u8] {
        // SAFETY: self.cap is always <= self.init, so self.buf[self.pos..self.cap] is always init <br>self.cap 总是 <= self.init，所以 self.buf[self.pos..self.cap] 总是 init<br>
        unsafe { MaybeUninit::slice_assume_init_ref(&self.buf[self.pos..self.cap]) }
    }

    /// Returns the number of bytes the internal buffer can hold at once. <br>返回内部缓冲区可以一次保存的字节数。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::{BufReader, BufRead};
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let f = File::open("log.txt")?;
    ///     let mut reader = BufReader::new(f);
    ///
    ///     let capacity = reader.capacity();
    ///     let buffer = reader.fill_buf()?;
    ///     assert!(buffer.len() <= capacity);
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "buffered_io_capacity", since = "1.46.0")]
    pub fn capacity(&self) -> usize {
        self.buf.len()
    }

    /// Unwraps this `BufReader<R>`, returning the underlying reader. <br>解包此 `BufReader<R>`，返回底层 reader。<br>
    ///
    /// Note that any leftover data in the internal buffer is lost. <br>请注意，内部缓冲区中的所有剩余数据都会丢失。<br>
    /// Therefore, a following read from the underlying reader may lead to data loss. <br>因此，从底层 reader 进行后续读取可能会导致数据丢失。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::BufReader;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let f1 = File::open("log.txt")?;
    ///     let reader = BufReader::new(f1);
    ///
    ///     let f2 = reader.into_inner();
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn into_inner(self) -> R {
        self.inner
    }

    /// Invalidates all data in the internal buffer. <br>使内部缓冲区中的所有数据无效。<br>
    #[inline]
    fn discard_buffer(&mut self) {
        self.pos = 0;
        self.cap = 0;
    }
}

impl<R: Seek> BufReader<R> {
    /// Seeks relative to the current position. <br>相对于当前位置寻找。<br>
    /// If the new position lies within the buffer, the buffer will not be flushed, allowing for more efficient seeks. <br>如果新位置位于缓冲区内，则不会刷新缓冲区，从而实现更有效的查找。<br>
    /// This method does not return the location of the underlying reader, so the caller must track this information themselves if it is required. <br>此方法不返回底层 reader 的位置，因此如果需要，调用者必须自己跟踪此信息。<br>
    ///
    #[stable(feature = "bufreader_seek_relative", since = "1.53.0")]
    pub fn seek_relative(&mut self, offset: i64) -> io::Result<()> {
        let pos = self.pos as u64;
        if offset < 0 {
            if let Some(new_pos) = pos.checked_sub((-offset) as u64) {
                self.pos = new_pos as usize;
                return Ok(());
            }
        } else if let Some(new_pos) = pos.checked_add(offset as u64) {
            if new_pos <= self.cap as u64 {
                self.pos = new_pos as usize;
                return Ok(());
            }
        }

        self.seek(SeekFrom::Current(offset)).map(drop)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<R: Read> Read for BufReader<R> {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        // If we don't have any buffered data and we're doing a massive read (larger than our internal buffer), bypass our internal buffer entirely. <br>如果我们没有任何缓冲数据，并且正在进行大量读取 (大于内部缓冲区)，请完全绕过内部缓冲区。<br>
        //
        //
        if self.pos == self.cap && buf.len() >= self.buf.len() {
            self.discard_buffer();
            return self.inner.read(buf);
        }
        let nread = {
            let mut rem = self.fill_buf()?;
            rem.read(buf)?
        };
        self.consume(nread);
        Ok(nread)
    }

    fn read_buf(&mut self, buf: &mut ReadBuf<'_>) -> io::Result<()> {
        // If we don't have any buffered data and we're doing a massive read (larger than our internal buffer), bypass our internal buffer entirely. <br>如果我们没有任何缓冲数据，并且正在进行大量读取 (大于内部缓冲区)，请完全绕过内部缓冲区。<br>
        //
        //
        if self.pos == self.cap && buf.remaining() >= self.buf.len() {
            self.discard_buffer();
            return self.inner.read_buf(buf);
        }

        let prev = buf.filled_len();

        let mut rem = self.fill_buf()?;
        rem.read_buf(buf)?;

        self.consume(buf.filled_len() - prev); // slice impl of read_buf known to never unfill buf <br>已知 read_buf 的切片实现从不填充 buf<br>

        Ok(())
    }

    // Small read_exacts from a BufReader are extremely common when used with a deserializer. <br>与反序列化器一起使用时，来自 BufReader 的小 read_exacts 非常普遍。<br>
    // The default implementation calls read in a loop, which results in surprisingly poor code generation for the common path where the buffer has enough bytes to fill the passed-in buffer. <br>默认实现在循环中调用 read，这会导致在公共路径中缓冲区具有足够的字节来填充传入缓冲区的公共路径，导致代码生成异常差。<br>
    //
    //
    fn read_exact(&mut self, buf: &mut [u8]) -> io::Result<()> {
        if self.buffer().len() >= buf.len() {
            buf.copy_from_slice(&self.buffer()[..buf.len()]);
            self.consume(buf.len());
            return Ok(());
        }

        crate::io::default_read_exact(self, buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        let total_len = bufs.iter().map(|b| b.len()).sum::<usize>();
        if self.pos == self.cap && total_len >= self.buf.len() {
            self.discard_buffer();
            return self.inner.read_vectored(bufs);
        }
        let nread = {
            let mut rem = self.fill_buf()?;
            rem.read_vectored(bufs)?
        };
        self.consume(nread);
        Ok(nread)
    }

    fn is_read_vectored(&self) -> bool {
        self.inner.is_read_vectored()
    }

    // The inner reader might have an optimized `read_to_end`. <br>内部 reader 可能具有优化的 `read_to_end`。<br>
    // Drain our buffer and then delegate to the inner implementation. <br>耗尽我们的缓冲区，然后委托给内部实现。<br>
    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
        let nread = self.cap - self.pos;
        buf.extend_from_slice(&self.buffer());
        self.discard_buffer();
        Ok(nread + self.inner.read_to_end(buf)?)
    }

    // The inner reader might have an optimized `read_to_end`. <br>内部 reader 可能具有优化的 `read_to_end`。<br>
    // Drain our buffer and then delegate to the inner implementation. <br>耗尽我们的缓冲区，然后委托给内部实现。<br>
    fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
        // In the general `else` case below we must read bytes into a side buffer, check that they are valid UTF-8, and then append them to `buf`. <br>在下面的一般 `else` 情况下，我们必须将字节读入一个 side 缓冲区，检查它们是否是有效的 UTF-8，然后将它们追加到 `buf`。<br>
        // This requires a potentially large memcpy. <br>这需要一个潜在的大型 memcpy。<br>
        //
        // If `buf` is empty--the most common case--we can leverage `append_to_string` to read directly into `buf`'s internal byte buffer, saving an allocation and a memcpy. <br>如果 `buf` 为空 -- 最常见的情况 -- 我们可以利用 `append_to_string` 直接读入 `buf` 的内部字节缓冲区，从而保存分配和 memcpy。<br>
        //
        //
        //
        if buf.is_empty() {
            // `append_to_string`'s safety relies on the buffer only being appended to since it only checks the UTF-8 validity of new data. <br>`append_to_string` 的安全性仅依赖于被追加到的缓冲区，因为它只检查新数据的 UTF-8 有效性。<br> If there were existing content in `buf` then an untrustworthy reader (i.e. `self.inner`) could not only append bytes but also modify existing bytes and render them invalid. <br>如果 `buf` 中存在现有内容，那么不可信的 reader (即 `self.inner`) 不仅可以追加字节，还可以修改现有字节并使它们无效。<br>
            //
            // On the other hand, if `buf` is empty then by definition any writes must be appends and `append_to_string` will validate all of the new bytes. <br>另一方面，如果 `buf` 为空，则根据定义，任何写入都必须追加，并且 `append_to_string` 将验证所有新字节。<br>
            //
            //
            //
            unsafe { crate::io::append_to_string(buf, |b| self.read_to_end(b)) }
        } else {
            // We cannot append our byte buffer directly onto the `buf` String as there could be an incomplete UTF-8 sequence that has only been partially read. <br>我们不能将字节缓冲区直接追加到 `buf` 字符串上，因为可能存在仅部分读取的不完整 UTF-8 序列。<br>
            // We must read everything into a side buffer first and then call `from_utf8` on the complete buffer. <br>我们必须首先将所有内容读入 side 缓冲区，然后在整个缓冲区上调用 `from_utf8`。<br>
            //
            //
            let mut bytes = Vec::new();
            self.read_to_end(&mut bytes)?;
            let string = crate::str::from_utf8(&bytes).map_err(|_| {
                io::const_io_error!(
                    io::ErrorKind::InvalidData,
                    "stream did not contain valid UTF-8",
                )
            })?;
            *buf += string;
            Ok(string.len())
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<R: Read> BufRead for BufReader<R> {
    fn fill_buf(&mut self) -> io::Result<&[u8]> {
        // If we've reached the end of our internal buffer then we need to fetch some more data from the underlying reader. <br>如果我们已经到达内部缓冲区的末尾，那么我们需要从底层 reader 中获取更多数据。<br>
        //
        // Branch using `>=` instead of the more correct `==` to tell the compiler that the pos..cap slice is always valid. <br>使用 `>=` 而不是更正确的 `==` 进行分支，以告知编译器 pos..cap 切片始终有效。<br>
        //
        if self.pos >= self.cap {
            debug_assert!(self.pos == self.cap);

            let mut readbuf = ReadBuf::uninit(&mut self.buf);

            // SAFETY: `self.init` is either 0 or set to `readbuf.initialized_len()` from the last time this function was called <br>`self.init` 为 0，或从上次调用此函数时设置为 `readbuf.initialized_len()`<br>
            //
            unsafe {
                readbuf.assume_init(self.init);
            }

            self.inner.read_buf(&mut readbuf)?;

            self.cap = readbuf.filled_len();
            self.init = readbuf.initialized_len();

            self.pos = 0;
        }
        Ok(self.buffer())
    }

    fn consume(&mut self, amt: usize) {
        self.pos = cmp::min(self.pos + amt, self.cap);
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<R> fmt::Debug for BufReader<R>
where
    R: fmt::Debug,
{
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt.debug_struct("BufReader")
            .field("reader", &self.inner)
            .field("buffer", &format_args!("{}/{}", self.cap - self.pos, self.buf.len()))
            .finish()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<R: Seek> Seek for BufReader<R> {
    /// Seek to an offset, in bytes, in the underlying reader. <br>在底层 reader 中查找偏移量 (以字节为单位)。<br>
    ///
    /// The position used for seeking with <code>[SeekFrom::Current]\(_)</code> is the position the underlying reader would be at if the `BufReader<R>` had no internal buffer. <br>用于使用 <code>[SeekFrom::Current]\(_)</code> 查找的位置是底层 reader 所在的位置，如果 `BufReader<R>` 没有内部缓冲区。<br>
    ///
    /// Seeking always discards the internal buffer, even if the seek position would otherwise fall within it. <br>查找总是会丢弃内部缓冲区，即使查找位置本来会落在内部缓冲区内。<br>
    /// This guarantees that calling [`BufReader::into_inner()`] immediately after a seek yields the underlying reader at the same position. <br>这保证了在查找之后立即调用 [`BufReader::into_inner()`] 会在相同位置产生底层 reader。<br>
    ///
    /// To seek without discarding the internal buffer, use [`BufReader::seek_relative`]. <br>要在不丢弃内部缓冲区的情况下进行查找，请使用 [`BufReader::seek_relative`]。<br>
    ///
    /// See [`std::io::Seek`] for more details. <br>有关更多详细信息，请参见 [`std::io::Seek`]。<br>
    ///
    /// Note: In the edge case where you're seeking with <code>[SeekFrom::Current]\(n)</code> where `n` minus the internal buffer length overflows an `i64`, two seeks will be performed instead of one. <br>在 edge 情况下，您使用 <code>[SeekFrom::Current]\(n)</code> 进行查找，其中 `n` 减去内部缓冲区长度会溢出 `i64`，将执行两次查找而不是一次查找。<br>
    ///
    /// If the second seek returns [`Err`], the underlying reader will be left at the same position it would have if you called `seek` with <code>[SeekFrom::Current]\(0)</code>. <br>如果第二个 seek 返回 [`Err`]，则底层 reader 将保留在与使用 <code>[SeekFrom::Current]\(0)</code> 调用 `seek` 时相同的位置。<br>
    ///
    /// [`std::io::Seek`]: Seek
    ///
    ///
    ///
    ///
    ///
    ///
    fn seek(&mut self, pos: SeekFrom) -> io::Result<u64> {
        let result: u64;
        if let SeekFrom::Current(n) = pos {
            let remainder = (self.cap - self.pos) as i64;
            // it should be safe to assume that remainder fits within an i64 as the alternative means we managed to allocate 8 exbibytes and that's absurd. <br>可以放心地认为其余部分适合 i64，因为这意味着我们设法分配了 8 EB，这是荒谬的。<br>
            // But it's not out of the realm of possibility for some weird underlying reader to support seeking by i64::MIN so we need to handle underflow when subtracting remainder. <br>但是，对于一些奇怪的底层 reader 来说，支持 i64::MIN 的搜索并非不可能，所以我们需要在减去余数时处理下溢。<br>
            //
            //
            //
            if let Some(offset) = n.checked_sub(remainder) {
                result = self.inner.seek(SeekFrom::Current(offset))?;
            } else {
                // seek backwards by our remainder, and then by the offset <br>向后搜索我们的余数，然后通过偏移量<br>
                self.inner.seek(SeekFrom::Current(-remainder))?;
                self.discard_buffer();
                result = self.inner.seek(SeekFrom::Current(n))?;
            }
        } else {
            // Seeking with Start/End doesn't care about our buffer length. <br>使用 Start/End 查找并不关心我们的缓冲区长度。<br>
            result = self.inner.seek(pos)?;
        }
        self.discard_buffer();
        Ok(result)
    }

    /// Returns the current seek position from the start of the stream. <br>从流的开头返回当前查找位置。<br>
    ///
    /// The value returned is equivalent to `self.seek(SeekFrom::Current(0))` but does not flush the internal buffer. <br>返回的值等效于 `self.seek(SeekFrom::Current(0))`，但不刷新内部缓冲区。<br>
    /// Due to this optimization the function does not guarantee that calling `.into_inner()` immediately afterwards will yield the underlying reader at the same position. <br>由于进行了这种优化，该函数不能保证在此之后立即调用 `.into_inner()` 将在同一位置产生底层 reader。<br>
    /// Use [`BufReader::seek`] instead if you require that guarantee. <br>如果需要该保证，请改用 [`BufReader::seek`]。<br>
    ///
    /// # Panics
    ///
    /// This function will panic if the position of the inner reader is smaller than the amount of buffered data. <br>如果内部 reader 的位置小于缓冲数据量，则此函数将为 panic。<br>
    /// That can happen if the inner reader has an incorrect implementation of [`Seek::stream_position`], or if the position has gone out of sync due to calling [`Seek::seek`] directly on the underlying reader. <br>如果内部 reader 的 [`Seek::stream_position`] 实现不正确，或者由于直接在底层 reader 上调用 [`Seek::seek`] 而导致位置不同步，则可能发生这种情况。<br>
    ///
    ///
    /// # Example
    ///
    /// ```no_run
    /// use std::{
    ///     io::{self, BufRead, BufReader, Seek},
    ///     fs::File,
    /// };
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = BufReader::new(File::open("foo.txt")?);
    ///
    ///     let before = f.stream_position()?;
    ///     f.read_line(&mut String::new())?;
    ///     let after = f.stream_position()?;
    ///
    ///     println!("The first line was {} bytes long", after - before);
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    ///
    fn stream_position(&mut self) -> io::Result<u64> {
        let remainder = (self.cap - self.pos) as u64;
        self.inner.stream_position().map(|pos| {
            pos.checked_sub(remainder).expect(
                "overflow when subtracting remaining buffer size from inner stream position",
            )
        })
    }
}

impl<T> SizeHint for BufReader<T> {
    #[inline]
    fn lower_bound(&self) -> usize {
        SizeHint::lower_bound(self.get_ref()) + self.buffer().len()
    }

    #[inline]
    fn upper_bound(&self) -> Option<usize> {
        SizeHint::upper_bound(self.get_ref()).and_then(|up| self.buffer().len().checked_add(up))
    }
}
