//! Buffering wrappers for I/O traits <br>I/O traits 的缓冲包装器<br>

mod bufreader;
mod bufwriter;
mod linewriter;
mod linewritershim;

#[cfg(test)]
mod tests;

use crate::error;
use crate::fmt;
use crate::io::Error;

#[stable(feature = "rust1", since = "1.0.0")]
pub use self::{bufreader::BufReader, bufwriter::BufWriter, linewriter::LineWriter};
use linewritershim::LineWriterShim;

#[stable(feature = "bufwriter_into_parts", since = "1.56.0")]
pub use bufwriter::WriterPanicked;

/// An error returned by [`BufWriter::into_inner`] which combines an error that happened while writing out the buffer, and the buffered writer object which may be used to recover from the condition. <br>[`BufWriter::into_inner`] 返回的错误，将写出缓冲区时发生的错误与缓冲的 writer 对象结合在一起，可用于从条件中恢复。<br>
///
///
/// # Examples
///
/// ```no_run
/// use std::io::BufWriter;
/// use std::net::TcpStream;
///
/// let mut stream = BufWriter::new(TcpStream::connect("127.0.0.1:34254").unwrap());
///
/// // do stuff with the stream <br>用流做东西<br>
///
/// // we want to get our `TcpStream` back, so let's try: <br>我们想取回 `TcpStream`，所以让我们尝试：<br>
///
/// let stream = match stream.into_inner() {
///     Ok(s) => s,
///     Err(e) => {
///         // Here, e is an IntoInnerError <br>在这里，e 是一个 IntoInnerError<br>
///         panic!("An error occurred");
///     }
/// };
/// ```
///
#[derive(Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct IntoInnerError<W>(W, Error);

impl<W> IntoInnerError<W> {
    /// Construct a new IntoInnerError <br>创建一个新的 IntoInnerError<br>
    fn new(writer: W, error: Error) -> Self {
        Self(writer, error)
    }

    /// Helper to construct a new IntoInnerError; <br>帮助程序创建一个新的 IntoInnerError；<br> intended to help with adapters that wrap other adapters <br>旨在帮助包装其他适配器的适配器<br>
    ///
    fn new_wrapped<W2>(self, f: impl FnOnce(W) -> W2) -> IntoInnerError<W2> {
        let Self(writer, error) = self;
        IntoInnerError::new(f(writer), error)
    }

    /// Returns the error which caused the call to [`BufWriter::into_inner()`] to fail. <br>返回导致 [`BufWriter::into_inner()`] 调用失败的错误。<br>
    ///
    ///
    /// This error was returned when attempting to write the internal buffer. <br>尝试写入内部缓冲区时返回此错误。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::BufWriter;
    /// use std::net::TcpStream;
    ///
    /// let mut stream = BufWriter::new(TcpStream::connect("127.0.0.1:34254").unwrap());
    ///
    /// // do stuff with the stream <br>用流做东西<br>
    ///
    /// // we want to get our `TcpStream` back, so let's try: <br>我们想取回 `TcpStream`，所以让我们尝试：<br>
    ///
    /// let stream = match stream.into_inner() {
    ///     Ok(s) => s,
    ///     Err(e) => {
    ///         // Here, e is an IntoInnerError, let's log the inner error. <br>在这里，e 是一个 IntoInnerError，让我们记录内部错误。<br>
    /////
    ///         // We'll just 'log' to stdout for this example. <br>在此示例中，我们仅将 'log' 更改为 stdout。<br>
    ///         println!("{}", e.error());
    ///
    ///         panic!("An unexpected error occurred.");
    ///     }
    /// };
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn error(&self) -> &Error {
        &self.1
    }

    /// Returns the buffered writer instance which generated the error. <br>返回产生错误的缓冲 writer 实例。<br>
    ///
    /// The returned object can be used for error recovery, such as re-inspecting the buffer. <br>返回的对象可用于错误恢复，例如重新检查缓冲区。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::BufWriter;
    /// use std::net::TcpStream;
    ///
    /// let mut stream = BufWriter::new(TcpStream::connect("127.0.0.1:34254").unwrap());
    ///
    /// // do stuff with the stream <br>用流做东西<br>
    ///
    /// // we want to get our `TcpStream` back, so let's try: <br>我们想取回 `TcpStream`，所以让我们尝试：<br>
    ///
    /// let stream = match stream.into_inner() {
    ///     Ok(s) => s,
    ///     Err(e) => {
    ///         // Here, e is an IntoInnerError, let's re-examine the buffer: <br>在这里，e 是一个 IntoInnerError，让我们重新检查缓冲区：<br>
    ///         let buffer = e.into_inner();
    ///
    ///         // do stuff to try to recover <br>做一些事情来尝试恢复<br>
    ///
    ///         // afterwards, let's just return the stream <br>之后，让我们返回流<br>
    ///         buffer.into_inner().unwrap()
    ///     }
    /// };
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn into_inner(self) -> W {
        self.0
    }

    /// Consumes the [`IntoInnerError`] and returns the error which caused the call to [`BufWriter::into_inner()`] to fail. <br>消耗 [`IntoInnerError`] 并返回导致 [`BufWriter::into_inner()`] 调用失败的错误。<br>
    /// Unlike `error`, this can be used to obtain ownership of the underlying error. <br>与 `error` 不同，它可用于获取基本错误的所有权。<br>
    ///
    /// # Example
    ///
    /// ```
    /// use std::io::{BufWriter, ErrorKind, Write};
    ///
    /// let mut not_enough_space = [0u8; 10];
    /// let mut stream = BufWriter::new(not_enough_space.as_mut());
    /// write!(stream, "this cannot be actually written").unwrap();
    /// let into_inner_err = stream.into_inner().expect_err("now we discover it's too small");
    /// let err = into_inner_err.into_error();
    /// assert_eq!(err.kind(), ErrorKind::WriteZero);
    /// ```
    #[stable(feature = "io_into_inner_error_parts", since = "1.55.0")]
    pub fn into_error(self) -> Error {
        self.1
    }

    /// Consumes the [`IntoInnerError`] and returns the error which caused the call to [`BufWriter::into_inner()`] to fail, and the underlying writer. <br>消耗 [`IntoInnerError`] 并返回导致 [`BufWriter::into_inner()`] 调用失败的错误，以及返回的 writer。<br>
    ///
    ///
    /// This can be used to simply obtain ownership of the underlying error; <br>这可以用来简单地获取潜在错误的所有权。<br> it can also be used for advanced error recovery. <br>它也可以用于高级错误恢复。<br>
    ///
    /// # Example
    ///
    /// ```
    /// use std::io::{BufWriter, ErrorKind, Write};
    ///
    /// let mut not_enough_space = [0u8; 10];
    /// let mut stream = BufWriter::new(not_enough_space.as_mut());
    /// write!(stream, "this cannot be actually written").unwrap();
    /// let into_inner_err = stream.into_inner().expect_err("now we discover it's too small");
    /// let (err, recovered_writer) = into_inner_err.into_parts();
    /// assert_eq!(err.kind(), ErrorKind::WriteZero);
    /// assert_eq!(recovered_writer.buffer(), b"t be actually written");
    /// ```
    #[stable(feature = "io_into_inner_error_parts", since = "1.55.0")]
    pub fn into_parts(self) -> (Error, W) {
        (self.1, self.0)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<W> From<IntoInnerError<W>> for Error {
    fn from(iie: IntoInnerError<W>) -> Error {
        iie.1
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<W: Send + fmt::Debug> error::Error for IntoInnerError<W> {
    #[allow(deprecated, deprecated_in_future)]
    fn description(&self) -> &str {
        error::Error::description(self.error())
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<W> fmt::Display for IntoInnerError<W> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.error().fmt(f)
    }
}
