use super::{BufWriter, ErrorKind, Read, ReadBuf, Result, Write, DEFAULT_BUF_SIZE};
use crate::mem::MaybeUninit;

/// Copies the entire contents of a reader into a writer. <br>将 reader 的全部内容复制到 writer 中。<br>
///
/// This function will continuously read data from `reader` and then write it into `writer` in a streaming fashion until `reader` returns EOF. <br>此函数将连续从 `reader` 读取数据，然后以流方式将其写入 `writer`，直到 `reader` 返回 EOF。<br>
///
/// On success, the total number of bytes that were copied from `reader` to `writer` is returned. <br>成功后，将返回从 `reader` 复制到 `writer` 的字节总数。<br>
///
/// If you’re wanting to copy the contents of one file to another and you’re working with filesystem paths, see the [`fs::copy`] function. <br>如果要将一个文件的内容复制到另一个文件，并且正在使用文件系统路径，请参见 [`fs::copy`] 函数。<br>
///
/// [`fs::copy`]: crate::fs::copy
///
/// # Errors
///
/// This function will return an error immediately if any call to [`read`] or [`write`] returns an error. <br>如果对 [`read`] 或 [`write`] 的任何调用返回错误，则此函数将立即返回错误。<br>
/// All instances of [`ErrorKind::Interrupted`] are handled by this function and the underlying operation is retried. <br>此函数将处理 [`ErrorKind::Interrupted`] 的所有实例，并重试底层操作。<br>
///
/// [`read`]: Read::read
/// [`write`]: Write::write
///
/// # Examples
///
/// ```
/// use std::io;
///
/// fn main() -> io::Result<()> {
///     let mut reader: &[u8] = b"hello";
///     let mut writer: Vec<u8> = vec![];
///
///     io::copy(&mut reader, &mut writer)?;
///
///     assert_eq!(&b"hello"[..], &writer[..]);
///     Ok(())
/// }
/// ```
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// On Linux (including Android), this function uses `copy_file_range(2)`, `sendfile(2)` or `splice(2)` syscalls to move data directly between file descriptors if possible. <br>在 Linux (包括 Android) 上，如果可能的话，此函数使用 `copy_file_range(2)`、`sendfile(2)` 或 `splice(2)` 系统调用在文件描述符之间直接移动数据。<br>
///
///
/// Note that platform-specific behavior [may change in the future][changes]. <br>请注意，特定于平台的行为 [将来可能会改变][changes]。<br>
///
/// [changes]: crate::io#platform-specific-behavior
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn copy<R: ?Sized, W: ?Sized>(reader: &mut R, writer: &mut W) -> Result<u64>
where
    R: Read,
    W: Write,
{
    cfg_if::cfg_if! {
        if #[cfg(any(target_os = "linux", target_os = "android"))] {
            crate::sys::kernel_copy::copy_spec(reader, writer)
        } else {
            generic_copy(reader, writer)
        }
    }
}

/// The userspace read-write-loop implementation of `io::copy` that is used when OS-specific specializations for copy offloading are not available or not applicable. <br>`io::copy` 的用户空间读写循环实现，当特定于操作系统的专门用于复制卸载的专业化不可用或不适用时，将使用该 `io::copy`。<br>
///
pub(crate) fn generic_copy<R: ?Sized, W: ?Sized>(reader: &mut R, writer: &mut W) -> Result<u64>
where
    R: Read,
    W: Write,
{
    BufferedCopySpec::copy_to(reader, writer)
}

/// Specialization of the read-write loop that either uses a stack buffer or reuses the internal buffer of a BufWriter <br>使用栈缓冲区或重用 BufWriter 的内部缓冲区的读写循环的特殊化<br>
///
trait BufferedCopySpec: Write {
    fn copy_to<R: Read + ?Sized>(reader: &mut R, writer: &mut Self) -> Result<u64>;
}

impl<W: Write + ?Sized> BufferedCopySpec for W {
    default fn copy_to<R: Read + ?Sized>(reader: &mut R, writer: &mut Self) -> Result<u64> {
        stack_buffer_copy(reader, writer)
    }
}

impl<I: Write> BufferedCopySpec for BufWriter<I> {
    fn copy_to<R: Read + ?Sized>(reader: &mut R, writer: &mut Self) -> Result<u64> {
        if writer.capacity() < DEFAULT_BUF_SIZE {
            return stack_buffer_copy(reader, writer);
        }

        let mut len = 0;
        let mut init = 0;

        loop {
            let buf = writer.buffer_mut();
            let mut read_buf = ReadBuf::uninit(buf.spare_capacity_mut());

            // SAFETY: init is either 0 or the initialized_len of the previous iteration <br>init 要么是 0，要么是上一次迭代的 initialized_len<br>
            unsafe {
                read_buf.assume_init(init);
            }

            if read_buf.capacity() >= DEFAULT_BUF_SIZE {
                match reader.read_buf(&mut read_buf) {
                    Ok(()) => {
                        let bytes_read = read_buf.filled_len();

                        if bytes_read == 0 {
                            return Ok(len);
                        }

                        init = read_buf.initialized_len() - bytes_read;

                        // SAFETY: ReadBuf guarantees all of its filled bytes are init <br>ReadBuf 保证所有填充的字节都是初始化的<br>
                        unsafe { buf.set_len(buf.len() + bytes_read) };
                        len += bytes_read as u64;
                        // Read again if the buffer still has enough capacity, as BufWriter itself would do This will occur if the reader returns short reads <br>如果缓冲区仍然有足够的容量，则重新读取，如 BufWriter 本身那样。如果 reader 返回短读，则会发生这种情况<br>
                        //
                        continue;
                    }
                    Err(ref e) if e.kind() == ErrorKind::Interrupted => continue,
                    Err(e) => return Err(e),
                }
            }

            writer.flush_buf()?;
        }
    }
}

fn stack_buffer_copy<R: Read + ?Sized, W: Write + ?Sized>(
    reader: &mut R,
    writer: &mut W,
) -> Result<u64> {
    let mut buf = [MaybeUninit::uninit(); DEFAULT_BUF_SIZE];
    let mut buf = ReadBuf::uninit(&mut buf);

    let mut len = 0;

    loop {
        match reader.read_buf(&mut buf) {
            Ok(()) => {}
            Err(e) if e.kind() == ErrorKind::Interrupted => continue,
            Err(e) => return Err(e),
        };

        if buf.filled().is_empty() {
            break;
        }

        len += buf.filled().len() as u64;
        writer.write_all(buf.filled())?;
        buf.clear();
    }

    Ok(len)
}
