#[cfg(test)]
mod tests;

#[cfg(target_pointer_width = "64")]
mod repr_bitpacked;
#[cfg(target_pointer_width = "64")]
use repr_bitpacked::Repr;

#[cfg(not(target_pointer_width = "64"))]
mod repr_unpacked;
#[cfg(not(target_pointer_width = "64"))]
use repr_unpacked::Repr;

use crate::convert::From;
use crate::error;
use crate::fmt;
use crate::result;
use crate::sys;

/// A specialized [`Result`] type for I/O operations. <br>I/O 操作的专用 [`Result`] 类型。<br>
///
/// This type is broadly used across [`std::io`] for any operation which may produce an error. <br>[`std::io`] 广泛使用此类型进行可能产生错误的任何操作。<br>
///
/// This typedef is generally used to avoid writing out [`io::Error`] directly and is otherwise a direct mapping to [`Result`]. <br>通常使用这种 typedef 来避免直接写出 [`io::Error`]，否则直接映射到 [`Result`]。<br>
///
/// While usual Rust style is to import types directly, aliases of [`Result`] often are not, to make it easier to distinguish between them. <br>通常的 Rust 样式是直接导入类型，而 [`Result`] 的别名通常不是，以便于区分它们。<br>
/// [`Result`] is generally assumed to be [`std::result::Result`][`Result`], and so users of this alias will generally use `io::Result` instead of shadowing the [prelude]'s import of [`std::result::Result`][`Result`]. <br>[`Result`] 通常被假定为 [`std::result::Result`][`Result`]，因此这个别名的用户通常会使用 `io::Result` 而不是隐藏 [prelude] 对 [`std::result::Result`][`Result`] 的导入。<br>
///
///
/// [`std::io`]: crate::io
/// [`io::Error`]: Error
/// [`Result`]: crate::result::Result
/// [prelude]: crate::prelude
///
/// # Examples
///
/// A convenience function that bubbles an `io::Result` to its caller: <br>一个方便的函数，将 `io::Result` 冒泡给其调用者：<br>
///
/// ```
/// use std::io;
///
/// fn get_string() -> io::Result<String> {
///     let mut buffer = String::new();
///
///     io::stdin().read_line(&mut buffer)?;
///
///     Ok(buffer)
/// }
/// ```
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub type Result<T> = result::Result<T, Error>;

/// The error type for I/O operations of the [`Read`], [`Write`], [`Seek`], and associated traits. <br>[`Read`]，[`Write`]，[`Seek`] 和关联的 traits 的 I/O 操作的错误类型。<br>
///
/// Errors mostly originate from the underlying OS, but custom instances of `Error` can be created with crafted error messages and a particular value of [`ErrorKind`]. <br>错误主要来自底层操作系统，但可以使用精心制作的错误消息和特定的 [`ErrorKind`] 值来创建 `Error` 的自定义实例。<br>
///
///
/// [`Read`]: crate::io::Read
/// [`Write`]: crate::io::Write
/// [`Seek`]: crate::io::Seek
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Error {
    repr: Repr,
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Debug::fmt(&self.repr, f)
    }
}

// Only derive debug in tests, to make sure it doesn't accidentally get printed. <br>仅在测试中派生调试，以确保它不会意外打印。<br>
//
#[cfg_attr(test, derive(Debug))]
enum ErrorData<C> {
    Os(i32),
    Simple(ErrorKind),
    SimpleMessage(&'static SimpleMessage),
    Custom(C),
}

// `#[repr(align(4))]` is probably redundant, it should have that value or higher already. <br>`#[repr(align(4))]` 可能是多余的，它应该已经具有该值或更高的值。<br> We include it just because repr_bitpacked.rs's encoding requires an alignment >= 4 (note that `#[repr(align)]` will not reduce the alignment required by the struct, only increase it). <br>我们包含它只是因为 repr_bitpacked.rs 的编码需要对齐 >= 4 (注意 `#[repr(align)]` 不会减少结构体所需的对齐，只会增加它)。<br>
//
// If we add more variants to ErrorData, this can be increased to 8, but it should probably be behind `#[cfg_attr(target_pointer_width = "64", ...)]` or whatever cfg we're using to enable the `repr_bitpacked` code, since only the that version needs the alignment, and 8 is higher than the alignment we'll have on 32 bit platforms. <br>如果我们向 ErrorData 添加更多变体，这可以增加到 8，但它可能应该在 `#[cfg_attr(target_pointer_width = "64", ...)]` 或我们用来启用 `repr_bitpacked` 代码的任何 cfg 之后，因为只有那个版本需要对齐，并且 8 高于我们将在 32 位平台上进行对齐。<br>
//
//
// (For the sake of being explicit: the alignment requirement here only matters if `error/repr_bitpacked.rs` is in use — for the unpacked repr it doesn't matter at all) <br>(为了明确起见：这里的对齐要求只有在使用 `error/repr_bitpacked.rs` 时才重要 -- 对于解包的 repr 来说根本不重要)<br>
//
//
//
//
//
//
//
//
#[repr(align(4))]
#[derive(Debug)]
pub(crate) struct SimpleMessage {
    kind: ErrorKind,
    message: &'static str,
}

impl SimpleMessage {
    pub(crate) const fn new(kind: ErrorKind, message: &'static str) -> Self {
        Self { kind, message }
    }
}

/// Create and return an `io::Error` for a given `ErrorKind` and constant message. <br>为给定的 `ErrorKind` 和常量消息创建并返回 `io::Error`。<br>
/// This doesn't allocate. <br>这个不分配。<br>
pub(crate) macro const_io_error($kind:expr, $message:expr $(,)?) {
    $crate::io::error::Error::from_static_message({
        const MESSAGE_DATA: $crate::io::error::SimpleMessage =
            $crate::io::error::SimpleMessage::new($kind, $message);
        &MESSAGE_DATA
    })
}

// As with `SimpleMessage`: `#[repr(align(4))]` here is just because repr_bitpacked's encoding requires it. <br>与 `SimpleMessage` 一样：这里的 `#[repr(align(4))]` 只是因为 repr_bitpacked 的编码需要它。<br>
// In practice it almost certainly be already be this high or higher. <br>在实践中，几乎可以肯定，它已经达到或超过了这个水平。<br>
//
#[derive(Debug)]
#[repr(align(4))]
struct Custom {
    kind: ErrorKind,
    error: Box<dyn error::Error + Send + Sync>,
}

/// A list specifying general categories of I/O error. <br>一个列表，指定 I/O 错误的常规类别。<br>
///
/// This list is intended to grow over time and it is not recommended to exhaustively match against it. <br>此列表旨在随着时间的增长而增长，不建议对其进行详尽的匹配。<br>
///
/// It is used with the [`io::Error`] type. <br>与 [`io::Error`] 类型一起使用。<br>
///
/// [`io::Error`]: Error
///
/// # Handling errors and matching on `ErrorKind` <br>在 `ErrorKind` 上处理错误和匹配<br>
///
/// In application code, use `match` for the `ErrorKind` values you are expecting; <br>在应用程序代码中，使用 `match` 作为您期望的 `ErrorKind` 值;<br> use `_` to match "all other errors". <br>使用 `_` 匹配 "所有其他错误"。<br>
///
/// In comprehensive and thorough tests that want to verify that a test doesn't return any known incorrect error kind, you may want to cut-and-paste the current full list of errors from here into your test code, and then match `_` as the correct case. <br>在想要验证测试不会返回任何已知的不正确错误类型的全面而彻底的测试中，您可能需要将当前完整的错误列表从此处剪切并粘贴到您的测试代码中，然后将 `_` 匹配为正确的情况。<br>
/// This seems counterintuitive, but it will make your tests more robust. <br>这似乎违反直觉，但它会使您的测试更加健壮。<br>
/// In particular, if you want to verify that your code does produce an unrecognized error kind, the robust solution is to check for all the recognized error kinds and fail in those cases. <br>特别是，如果您想验证您的代码确实产生了无法识别的错误类型，那么可靠的解决方案是检查所有已识别的错误类型并在这些情况下失败。<br>
///
///
///
///
///
///
#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated)]
#[non_exhaustive]
pub enum ErrorKind {
    /// An entity was not found, often a file. <br>找不到实体，通常是文件。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    NotFound,
    /// The operation lacked the necessary privileges to complete. <br>该操作缺少完成操作所需的权限。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    PermissionDenied,
    /// The connection was refused by the remote server. <br>远程服务器拒绝了连接。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    ConnectionRefused,
    /// The connection was reset by the remote server. <br>连接已由远程服务器重置。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    ConnectionReset,
    /// The remote host is not reachable. <br>远程主机不可访问。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    HostUnreachable,
    /// The network containing the remote host is not reachable. <br>无法访问包含远程主机的网络。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    NetworkUnreachable,
    /// The connection was aborted (terminated) by the remote server. <br>(terminated) 连接被远程服务器中止。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    ConnectionAborted,
    /// The network operation failed because it was not connected yet. <br>网络操作失败，因为尚未连接。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    NotConnected,
    /// A socket address could not be bound because the address is already in use elsewhere. <br>无法绑定套接字地址，因为该地址已在其他地方使用。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    AddrInUse,
    /// A nonexistent interface was requested or the requested address was not local. <br>请求了不存在的接口，或者请求的地址不是本地的。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    AddrNotAvailable,
    /// The system's networking is down. <br>系统的网络已关闭。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    NetworkDown,
    /// The operation failed because a pipe was closed. <br>操作失败，因为管道已关闭。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    BrokenPipe,
    /// An entity already exists, often a file. <br>一个实体已经存在，通常是一个文件。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    AlreadyExists,
    /// The operation needs to block to complete, but the blocking operation was requested to not occur. <br>该操作需要阻止才能完成，但是请求阻止操作不会发生。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    WouldBlock,
    /// A filesystem object is, unexpectedly, not a directory. <br>出乎意料的是，文件系统对象不是目录。<br>
    ///
    /// For example, a filesystem path was specified where one of the intermediate directory components was, in fact, a plain file. <br>例如，指定了一个文件系统路径，其中一个中间目录组件实际上是一个普通文件。<br>
    ///
    #[unstable(feature = "io_error_more", issue = "86442")]
    NotADirectory,
    /// The filesystem object is, unexpectedly, a directory. <br>出乎意料的是，文件系统对象是一个目录。<br>
    ///
    /// A directory was specified when a non-directory was expected. <br>当需要一个非目录时指定了一个目录。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    IsADirectory,
    /// A non-empty directory was specified where an empty directory was expected. <br>在需要空目录的地方指定了一个非空目录。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    DirectoryNotEmpty,
    /// The filesystem or storage medium is read-only, but a write operation was attempted. <br>文件系统或存储介质是只读的，但尝试了写入操作。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    ReadOnlyFilesystem,
    /// Loop in the filesystem or IO subsystem; <br>在文件系统或 IO 子系统中循环；<br> often, too many levels of symbolic links. <br>通常，太多级别的符号链接。<br>
    ///
    /// There was a loop (or excessively long chain) resolving a filesystem object or file IO object. <br>有一个循环 (或过长的链) 解析文件系统对象或文件 IO 对象。<br>
    ///
    /// On Unix this is usually the result of a symbolic link loop; <br>在 Unix 上，这通常是符号链接循环的结果；<br> or, of exceeding the system-specific limit on the depth of symlink traversal. <br>或者，超过系统特定的符号链接遍历深度限制。<br>
    ///
    ///
    #[unstable(feature = "io_error_more", issue = "86442")]
    FilesystemLoop,
    /// Stale network file handle. <br>陈旧的网络文件句柄。<br>
    ///
    /// With some network filesystems, notably NFS, an open file (or directory) can be invalidated by problems with the network or server. <br>对于某些网络文件系统，尤其是 NFS，打开的文件 (或目录) 可能会因网络或服务器问题而失效。<br>
    ///
    #[unstable(feature = "io_error_more", issue = "86442")]
    StaleNetworkFileHandle,
    /// A parameter was incorrect. <br>参数不正确。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    InvalidInput,
    /// Data not valid for the operation were encountered. <br>遇到对该操作无效的数据。<br>
    ///
    /// Unlike [`InvalidInput`], this typically means that the operation parameters were valid, however the error was caused by malformed input data. <br>与 [`InvalidInput`] 不同，这通常意味着操作参数有效，但是该错误是由格式错误的输入数据引起的。<br>
    ///
    ///
    /// For example, a function that reads a file into a string will error with `InvalidData` if the file's contents are not valid UTF-8. <br>例如，如果文件的内容无效 UTF-8，则将文件读入字符串的函数将出现 `InvalidData` 错误。<br>
    ///
    /// [`InvalidInput`]: ErrorKind::InvalidInput
    ///
    ///
    #[stable(feature = "io_invalid_data", since = "1.2.0")]
    InvalidData,
    /// The I/O operation's timeout expired, causing it to be canceled. <br>I/O 操作的超时已到期，导致其被取消。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    TimedOut,
    /// An error returned when an operation could not be completed because a call to [`write`] returned [`Ok(0)`]. <br>由于调用 [`write`] 返回 [`Ok(0)`] 而无法完成操作时，返回错误。<br>
    ///
    /// This typically means that an operation could only succeed if it wrote a particular number of bytes but only a smaller number of bytes could be written. <br>这通常意味着操作只有在写入特定数量的字节后才能成功，但是只能写入较少的字节数才能成功。<br>
    ///
    ///
    /// [`write`]: crate::io::Write::write
    /// [`Ok(0)`]: Ok
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    WriteZero,
    /// The underlying storage (typically, a filesystem) is full. <br>底层存储 (通常是文件系统) 已满。<br>
    ///
    /// This does not include out of quota errors. <br>这不包括超出配额的错误。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    StorageFull,
    /// Seek on unseekable file. <br>Seek 在不可搜索的文件上。<br>
    ///
    /// Seeking was attempted on an open file handle which is not suitable for seeking - for example, on Unix, a named pipe opened with `File::open`. <br>尝试在不适合搜索的打开文件句柄上进行搜索 - 例如，在 Unix 上，使用 `File::open` 打开的命名管道。<br>
    ///
    #[unstable(feature = "io_error_more", issue = "86442")]
    NotSeekable,
    /// Filesystem quota was exceeded. <br>超出了文件系统配额。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    FilesystemQuotaExceeded,
    /// File larger than allowed or supported. <br>文件大于允许或支持。<br>
    ///
    /// This might arise from a hard limit of the underlying filesystem or file access API, or from an administratively imposed resource limitation. <br>这可能源于底层文件系统或文件访问 API 的硬限制，或源于管理强加的资源限制。<br>
    /// Simple disk full, and out of quota, have their own errors. <br>简单的磁盘已满，超出配额，都有自己的错误。<br>
    ///
    #[unstable(feature = "io_error_more", issue = "86442")]
    FileTooLarge,
    /// Resource is busy. <br>资源繁忙。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    ResourceBusy,
    /// Executable file is busy. <br>可执行文件正忙。<br>
    ///
    /// An attempt was made to write to a file which is also in use as a running program. <br>试图写入一个文件，该文件也用作正在运行的程序。<br>
    /// (Not all operating systems detect this situation.) <br>(并非所有操作系统都检测到这种情况。)<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    ExecutableFileBusy,
    /// Deadlock (avoided). <br>死锁 (avoided)。<br>
    ///
    /// A file locking operation would result in deadlock. <br>文件锁定操作会导致死锁。<br>
    /// This situation is typically detected, if at all, on a best-effort basis. <br>这种情况通常是在尽力而为的基础上检测到的。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    Deadlock,
    /// Cross-device or cross-filesystem (hard) link or rename. <br>跨设备或跨文件系统 (hard) 链接或重命名。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    CrossesDevices,
    /// Too many (hard) links to the same filesystem object. <br>太多 (hard) 链接到同一个文件系统对象。<br>
    ///
    /// The filesystem does not support making so many hardlinks to the same file. <br>文件系统不支持对同一个文件建立如此多的硬链接。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    TooManyLinks,
    /// A filename was invalid. <br>文件名无效。<br>
    ///
    /// This error can also cause if it exceeded the filename length limit. <br>如果超出文件名长度限制，也会导致此错误。<br>
    #[unstable(feature = "io_error_more", issue = "86442")]
    InvalidFilename,
    /// Program argument list too long. <br>程序参数列表太长。<br>
    ///
    /// When trying to run an external program, a system or process limit on the size of the arguments would have been exceeded. <br>尝试运行外部程序时，会超出系统或进程对参数大小的限制。<br>
    ///
    #[unstable(feature = "io_error_more", issue = "86442")]
    ArgumentListTooLong,
    /// This operation was interrupted. <br>该操作被中断。<br>
    ///
    /// Interrupted operations can typically be retried. <br>通常可以重试中断的操作。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    Interrupted,

    /// This operation is unsupported on this platform. <br>此平台不支持此操作。<br>
    ///
    /// This means that the operation can never succeed. <br>这意味着操作永远不会成功。<br>
    #[stable(feature = "unsupported_error", since = "1.53.0")]
    Unsupported,

    // ErrorKinds which are primarily categorisations for OS error codes should be added above. <br>ErrorKinds 主要是 OS 错误代码的分类应在上面添加。<br>
    //
    //
    /// An error returned when an operation could not be completed because an "end of file" was reached prematurely. <br>由于过早到达 "文件结尾" 而无法完成操作时，返回错误。<br>
    ///
    /// This typically means that an operation could only succeed if it read a particular number of bytes but only a smaller number of bytes could be read. <br>通常，这意味着操作只有在读取特定数量的字节后才能成功，但是只能读取较少的字节。<br>
    ///
    ///
    ///
    #[stable(feature = "read_exact", since = "1.6.0")]
    UnexpectedEof,

    /// An operation could not be completed, because it failed to allocate enough memory. <br>操作无法完成，因为它未能分配足够的内存。<br>
    ///
    #[stable(feature = "out_of_memory_error", since = "1.54.0")]
    OutOfMemory,

    // "Unusual" error kinds which do not correspond simply to (sets of) OS error codes, should be added just above this comment. <br>"Unusual" 错误类型不只对应于 (组) 操作系统错误代码，应添加在这个注释的正上方。<br>
    //
    // `Other` and `Uncategorised` should remain at the end: <br>`Other` 和 `Uncategorised` 应保留在末尾：<br>
    //
    /// A custom error that does not fall under any other I/O error kind. <br>不属于任何其他 I/O 错误类型的自定义错误。<br>
    ///
    /// This can be used to construct your own [`Error`]s that do not match any [`ErrorKind`]. <br>这可用于构建您自己的不匹配任何 [`ErrorKind`] 的 [`Error`]。<br>
    ///
    /// This [`ErrorKind`] is not used by the standard library. <br>标准库不使用此 [`ErrorKind`]。<br>
    ///
    /// Errors from the standard library that do not fall under any of the I/O error kinds cannot be `match`ed on, and will only match a wildcard (`_`) pattern. <br>标准库中不属于任何 I/O 错误类型的错误无法 `匹配`，并且只会匹配通配符 (`_`) 模式。<br>
    ///
    /// New [`ErrorKind`]s might be added in the future for some of those. <br>可能会在 future 中为其中一些添加新的 [`ErrorKind`]。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    Other,

    /// Any I/O error from the standard library that's not part of this list. <br>标准库中不属于此列表的任何 I/O 错误。<br>
    ///
    /// Errors that are `Uncategorized` now may move to a different or a new [`ErrorKind`] variant in the future. <br>现在 `Uncategorized` 的错误将来可能会转移到不同的或新的 [`ErrorKind`] 变体。<br>
    /// It is not recommended to match an error against `Uncategorized`; <br>不建议针对 `Uncategorized` 匹配错误；<br> use a wildcard match (`_`) instead. <br>改用通配符匹配 (`_`)。<br>
    ///
    #[unstable(feature = "io_error_uncategorized", issue = "none")]
    #[doc(hidden)]
    Uncategorized,
}

impl ErrorKind {
    pub(crate) fn as_str(&self) -> &'static str {
        use ErrorKind::*;
        // Strictly alphabetical, please. <br>请严格按字母顺序排列。<br> (Sadly rustfmt cannot do this yet.) <br>(遗憾的是 rustfmt 还不能做到这一点。)<br>
        match *self {
            AddrInUse => "address in use",
            AddrNotAvailable => "address not available",
            AlreadyExists => "entity already exists",
            ArgumentListTooLong => "argument list too long",
            BrokenPipe => "broken pipe",
            ConnectionAborted => "connection aborted",
            ConnectionRefused => "connection refused",
            ConnectionReset => "connection reset",
            CrossesDevices => "cross-device link or rename",
            Deadlock => "deadlock",
            DirectoryNotEmpty => "directory not empty",
            ExecutableFileBusy => "executable file busy",
            FileTooLarge => "file too large",
            FilesystemLoop => "filesystem loop or indirection limit (e.g. symlink loop)",
            FilesystemQuotaExceeded => "filesystem quota exceeded",
            HostUnreachable => "host unreachable",
            Interrupted => "operation interrupted",
            InvalidData => "invalid data",
            InvalidFilename => "invalid filename",
            InvalidInput => "invalid input parameter",
            IsADirectory => "is a directory",
            NetworkDown => "network down",
            NetworkUnreachable => "network unreachable",
            NotADirectory => "not a directory",
            NotConnected => "not connected",
            NotFound => "entity not found",
            NotSeekable => "seek on unseekable file",
            Other => "other error",
            OutOfMemory => "out of memory",
            PermissionDenied => "permission denied",
            ReadOnlyFilesystem => "read-only filesystem or storage medium",
            ResourceBusy => "resource busy",
            StaleNetworkFileHandle => "stale network file handle",
            StorageFull => "no storage space",
            TimedOut => "timed out",
            TooManyLinks => "too many links",
            Uncategorized => "uncategorized error",
            UnexpectedEof => "unexpected end of file",
            Unsupported => "unsupported",
            WouldBlock => "operation would block",
            WriteZero => "write zero",
        }
    }
}

#[stable(feature = "io_errorkind_display", since = "1.60.0")]
impl fmt::Display for ErrorKind {
    /// Shows a human-readable description of the `ErrorKind`. <br>显示了对 `ErrorKind` 的可读描述。<br>
    ///
    /// This is similar to `impl Display for Error`, but doesn't require first converting to Error. <br>这类似于 `impl Display for Error`，但不需要先转换为 Error。<br>
    ///
    /// # Examples
    /// ```
    /// use std::io::ErrorKind;
    /// assert_eq!("entity not found", ErrorKind::NotFound.to_string());
    /// ```
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt.write_str(self.as_str())
    }
}

/// Intended for use for errors not exposed to the user, where allocating onto the heap (for normal construction via Error::new) is too costly. <br>旨在用于未暴露给用户的错误，因为分配到堆上 (通过 Error::new 进行常规构建) 的代价太高了。<br>
///
#[stable(feature = "io_error_from_errorkind", since = "1.14.0")]
impl From<ErrorKind> for Error {
    /// Converts an [`ErrorKind`] into an [`Error`]. <br>将 [`ErrorKind`] 转换为 [`Error`]。<br>
    ///
    /// This conversion creates a new error with a simple representation of error kind. <br>这种转换会创建一个带有错误类型的简单表示的新错误。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::{Error, ErrorKind};
    ///
    /// let not_found = ErrorKind::NotFound;
    /// let error = Error::from(not_found);
    /// assert_eq!("entity not found", format!("{error}"));
    /// ```
    #[inline]
    fn from(kind: ErrorKind) -> Error {
        Error { repr: Repr::new_simple(kind) }
    }
}

impl Error {
    /// Creates a new I/O error from a known kind of error as well as an arbitrary error payload. <br>根据已知的错误以及任意的错误有效载荷创建新的 I/O 错误。<br>
    ///
    /// This function is used to generically create I/O errors which do not originate from the OS itself. <br>此函数通常用于创建 I/O 错误，这些错误并非源于操作系统本身。<br>
    /// The `error` argument is an arbitrary payload which will be contained in this [`Error`]. <br>`error` 参数是将包含在此 [`Error`] 中的任意有效载荷。<br>
    ///
    /// If no extra payload is required, use the `From` conversion from `ErrorKind`. <br>如果不需要额外的有效载荷，请使用来自 `ErrorKind` 的 `From` 转换。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::{Error, ErrorKind};
    ///
    /// // errors can be created from strings <br>可以从字符串创建错误<br>
    /// let custom_error = Error::new(ErrorKind::Other, "oh no!");
    ///
    /// // errors can also be created from other errors <br>错误也可以从其他错误中创建<br>
    /// let custom_error2 = Error::new(ErrorKind::Interrupted, custom_error);
    ///
    /// // creating an error without payload <br>在没有有效载荷的情况下创建错误<br>
    /// let eof_error = Error::from(ErrorKind::UnexpectedEof);
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn new<E>(kind: ErrorKind, error: E) -> Error
    where
        E: Into<Box<dyn error::Error + Send + Sync>>,
    {
        Self::_new(kind, error.into())
    }

    /// Creates a new I/O error from an arbitrary error payload. <br>从任意错误有效载荷创建新的 I/O 错误。<br>
    ///
    /// This function is used to generically create I/O errors which do not originate from the OS itself. <br>此函数通常用于创建 I/O 错误，这些错误并非源于操作系统本身。<br>
    /// It is a shortcut for [`Error::new`] with [`ErrorKind::Other`]. <br>它是 [`Error::new`] 与 [`ErrorKind::Other`] 的快捷方式。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(io_error_other)]
    ///
    /// use std::io::Error;
    ///
    /// // errors can be created from strings <br>可以从字符串创建错误<br>
    /// let custom_error = Error::other("oh no!");
    ///
    /// // errors can also be created from other errors <br>错误也可以从其他错误中创建<br>
    /// let custom_error2 = Error::other(custom_error);
    /// ```
    ///
    #[unstable(feature = "io_error_other", issue = "91946")]
    pub fn other<E>(error: E) -> Error
    where
        E: Into<Box<dyn error::Error + Send + Sync>>,
    {
        Self::_new(ErrorKind::Other, error.into())
    }

    fn _new(kind: ErrorKind, error: Box<dyn error::Error + Send + Sync>) -> Error {
        Error { repr: Repr::new_custom(Box::new(Custom { kind, error })) }
    }

    /// Creates a new I/O error from a known kind of error as well as a constant message. <br>从已知类型的错误以及恒定消息创建一个新的 I/O 错误。<br>
    ///
    /// This function does not allocate. <br>这个函数不分配。<br>
    ///
    /// You should not use this directly, and instead use the `const_io_error!` macro: `io::const_io_error!(ErrorKind::Something, "some_message")`. <br>您不应该直接使用它，而应使用 `const_io_error!` 宏: `io::const_io_error!(ErrorKind::Something, "some_message")`。<br>
    ///
    /// This function should maybe change to `from_static_message<const MSG: &'static str>(kind: ErrorKind)` in the future, when const generics allow that. <br>将来，当 const 泛型允许时，这个函数可能会更改为 `from_static_message<const MSG: &'static str>(kind: ErrorKind)`。<br>
    ///
    ///
    ///
    #[inline]
    pub(crate) const fn from_static_message(msg: &'static SimpleMessage) -> Error {
        Self { repr: Repr::new_simple_message(msg) }
    }

    /// Returns an error representing the last OS error which occurred. <br>返回代表最近发生的操作系统错误的错误。<br>
    ///
    /// This function reads the value of `errno` for the target platform (e.g. <br>该函数读取目标平台的 `errno` 值 (例如，<br>
    /// `GetLastError` on Windows) and will return a corresponding instance of [`Error`] for the error code. <br>Windows 上的 `GetLastError`) 并将为错误代码返回相应的 [`Error`] 实例。<br>
    ///
    /// This should be called immediately after a call to a platform function, otherwise the state of the error value is indeterminate. <br>这应该在调用到平台函数之后立即调用，否则错误值的状态是不确定的。<br>
    /// In particular, other standard library functions may call platform functions that may (or may not) reset the error value even if they succeed. <br>特别是，其他标准库函数可能会调用平台函数，即使它们成功也可能 (或可能不会) 重置错误值。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::Error;
    ///
    /// let os_error = Error::last_os_error();
    /// println!("last OS error: {os_error:?}");
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn last_os_error() -> Error {
        Error::from_raw_os_error(sys::os::errno() as i32)
    }

    /// Creates a new instance of an [`Error`] from a particular OS error code. <br>根据特定的操作系统错误代码创建 [`Error`] 的新实例。<br>
    ///
    /// # Examples
    ///
    /// On Linux: <br>在 Linux 上：<br>
    ///
    /// ```
    /// # if cfg!(target_os = "linux") {
    /// use std::io;
    ///
    /// let error = io::Error::from_raw_os_error(22);
    /// assert_eq!(error.kind(), io::ErrorKind::InvalidInput);
    /// # }
    /// ```
    ///
    /// On Windows: <br>在 Windows 上：<br>
    ///
    /// ```
    /// # if cfg!(windows) {
    /// use std::io;
    ///
    /// let error = io::Error::from_raw_os_error(10022);
    /// assert_eq!(error.kind(), io::ErrorKind::InvalidInput);
    /// # }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn from_raw_os_error(code: i32) -> Error {
        Error { repr: Repr::new_os(code) }
    }

    /// Returns the OS error that this error represents (if any). <br>返回此错误表示的操作系统错误 (如果有)。<br>
    ///
    /// If this [`Error`] was constructed via [`last_os_error`] or [`from_raw_os_error`], then this function will return [`Some`], otherwise it will return [`None`]. <br>如果此 [`Error`] 是通过 [`last_os_error`] 或 [`from_raw_os_error`] 构造的，则此函数将返回 [`Some`]，否则它将返回 [`None`]。<br>
    ///
    ///
    /// [`last_os_error`]: Error::last_os_error
    /// [`from_raw_os_error`]: Error::from_raw_os_error
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::{Error, ErrorKind};
    ///
    /// fn print_os_error(err: &Error) {
    ///     if let Some(raw_os_err) = err.raw_os_error() {
    ///         println!("raw OS error: {raw_os_err:?}");
    ///     } else {
    ///         println!("Not an OS error");
    ///     }
    /// }
    ///
    /// fn main() {
    ///     // Will print "raw OS error: ...". <br>将打印 "raw OS error: ..."。<br>
    ///     print_os_error(&Error::last_os_error());
    ///     // Will print "Not an OS error". <br>将打印 "Not an OS error"。<br>
    ///     print_os_error(&Error::new(ErrorKind::Other, "oh no!"));
    /// }
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn raw_os_error(&self) -> Option<i32> {
        match self.repr.data() {
            ErrorData::Os(i) => Some(i),
            ErrorData::Custom(..) => None,
            ErrorData::Simple(..) => None,
            ErrorData::SimpleMessage(..) => None,
        }
    }

    /// Returns a reference to the inner error wrapped by this error (if any). <br>返回对此错误包装的内部错误 (如果有) 的引用。<br>
    ///
    /// If this [`Error`] was constructed via [`new`] then this function will return [`Some`], otherwise it will return [`None`]. <br>如果此 [`Error`] 是通过 [`new`] 构造的，则此函数将返回 [`Some`]，否则它将返回 [`None`]。<br>
    ///
    ///
    /// [`new`]: Error::new
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::{Error, ErrorKind};
    ///
    /// fn print_error(err: &Error) {
    ///     if let Some(inner_err) = err.get_ref() {
    ///         println!("Inner error: {inner_err:?}");
    ///     } else {
    ///         println!("No inner error");
    ///     }
    /// }
    ///
    /// fn main() {
    ///     // Will print "No inner error". <br>将打印 "No inner error"。<br>
    ///     print_error(&Error::last_os_error());
    ///     // Will print "Inner error: ...". <br>将打印 "Inner error: ..."。<br>
    ///     print_error(&Error::new(ErrorKind::Other, "oh no!"));
    /// }
    /// ```
    #[stable(feature = "io_error_inner", since = "1.3.0")]
    #[must_use]
    #[inline]
    pub fn get_ref(&self) -> Option<&(dyn error::Error + Send + Sync + 'static)> {
        match self.repr.data() {
            ErrorData::Os(..) => None,
            ErrorData::Simple(..) => None,
            ErrorData::SimpleMessage(..) => None,
            ErrorData::Custom(c) => Some(&*c.error),
        }
    }

    /// Returns a mutable reference to the inner error wrapped by this error (if any). <br>返回对此错误包装的内部错误的可变引用 (如果有)。<br>
    ///
    /// If this [`Error`] was constructed via [`new`] then this function will return [`Some`], otherwise it will return [`None`]. <br>如果此 [`Error`] 是通过 [`new`] 构造的，则此函数将返回 [`Some`]，否则它将返回 [`None`]。<br>
    ///
    ///
    /// [`new`]: Error::new
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::{Error, ErrorKind};
    /// use std::{error, fmt};
    /// use std::fmt::Display;
    ///
    /// #[derive(Debug)]
    /// struct MyError {
    ///     v: String,
    /// }
    ///
    /// impl MyError {
    ///     fn new() -> MyError {
    ///         MyError {
    ///             v: "oh no!".to_string()
    ///         }
    ///     }
    ///
    ///     fn change_message(&mut self, new_message: &str) {
    ///         self.v = new_message.to_string();
    ///     }
    /// }
    ///
    /// impl error::Error for MyError {}
    ///
    /// impl Display for MyError {
    ///     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
    ///         write!(f, "MyError: {}", &self.v)
    ///     }
    /// }
    ///
    /// fn change_error(mut err: Error) -> Error {
    ///     if let Some(inner_err) = err.get_mut() {
    ///         inner_err.downcast_mut::<MyError>().unwrap().change_message("I've been changed!");
    ///     }
    ///     err
    /// }
    ///
    /// fn print_error(err: &Error) {
    ///     if let Some(inner_err) = err.get_ref() {
    ///         println!("Inner error: {inner_err}");
    ///     } else {
    ///         println!("No inner error");
    ///     }
    /// }
    ///
    /// fn main() {
    ///     // Will print "No inner error". <br>将打印 "No inner error"。<br>
    ///     print_error(&change_error(Error::last_os_error()));
    ///     // Will print "Inner error: ...". <br>将打印 "Inner error: ..."。<br>
    ///     print_error(&change_error(Error::new(ErrorKind::Other, MyError::new())));
    /// }
    /// ```
    ///
    #[stable(feature = "io_error_inner", since = "1.3.0")]
    #[must_use]
    #[inline]
    pub fn get_mut(&mut self) -> Option<&mut (dyn error::Error + Send + Sync + 'static)> {
        match self.repr.data_mut() {
            ErrorData::Os(..) => None,
            ErrorData::Simple(..) => None,
            ErrorData::SimpleMessage(..) => None,
            ErrorData::Custom(c) => Some(&mut *c.error),
        }
    }

    /// Consumes the `Error`, returning its inner error (if any). <br>消耗 `Error`，并返回其内部错误 (如果有)。<br>
    ///
    /// If this [`Error`] was constructed via [`new`] then this function will return [`Some`], otherwise it will return [`None`]. <br>如果此 [`Error`] 是通过 [`new`] 构造的，则此函数将返回 [`Some`]，否则它将返回 [`None`]。<br>
    ///
    ///
    /// [`new`]: Error::new
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::{Error, ErrorKind};
    ///
    /// fn print_error(err: Error) {
    ///     if let Some(inner_err) = err.into_inner() {
    ///         println!("Inner error: {inner_err}");
    ///     } else {
    ///         println!("No inner error");
    ///     }
    /// }
    ///
    /// fn main() {
    ///     // Will print "No inner error". <br>将打印 "No inner error"。<br>
    ///     print_error(Error::last_os_error());
    ///     // Will print "Inner error: ...". <br>将打印 "Inner error: ..."。<br>
    ///     print_error(Error::new(ErrorKind::Other, "oh no!"));
    /// }
    /// ```
    #[stable(feature = "io_error_inner", since = "1.3.0")]
    #[must_use = "`self` will be dropped if the result is not used"]
    #[inline]
    pub fn into_inner(self) -> Option<Box<dyn error::Error + Send + Sync>> {
        match self.repr.into_data() {
            ErrorData::Os(..) => None,
            ErrorData::Simple(..) => None,
            ErrorData::SimpleMessage(..) => None,
            ErrorData::Custom(c) => Some(c.error),
        }
    }

    /// Returns the corresponding [`ErrorKind`] for this error. <br>返回与此错误对应的 [`ErrorKind`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::{Error, ErrorKind};
    ///
    /// fn print_error(err: Error) {
    ///     println!("{:?}", err.kind());
    /// }
    ///
    /// fn main() {
    ///     // Will print "Uncategorized". <br>将打印 "Uncategorized"。<br>
    ///     print_error(Error::last_os_error());
    ///     // Will print "AddrInUse". <br>将打印 "AddrInUse"。<br>
    ///     print_error(Error::new(ErrorKind::AddrInUse, "oh no!"));
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn kind(&self) -> ErrorKind {
        match self.repr.data() {
            ErrorData::Os(code) => sys::decode_error_kind(code),
            ErrorData::Custom(c) => c.kind,
            ErrorData::Simple(kind) => kind,
            ErrorData::SimpleMessage(m) => m.kind,
        }
    }
}

impl fmt::Debug for Repr {
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self.data() {
            ErrorData::Os(code) => fmt
                .debug_struct("Os")
                .field("code", &code)
                .field("kind", &sys::decode_error_kind(code))
                .field("message", &sys::os::error_string(code))
                .finish(),
            ErrorData::Custom(c) => fmt::Debug::fmt(&c, fmt),
            ErrorData::Simple(kind) => fmt.debug_tuple("Kind").field(&kind).finish(),
            ErrorData::SimpleMessage(msg) => fmt
                .debug_struct("Error")
                .field("kind", &msg.kind)
                .field("message", &msg.message)
                .finish(),
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Display for Error {
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self.repr.data() {
            ErrorData::Os(code) => {
                let detail = sys::os::error_string(code);
                write!(fmt, "{detail} (os error {code})")
            }
            ErrorData::Custom(ref c) => c.error.fmt(fmt),
            ErrorData::Simple(kind) => write!(fmt, "{}", kind.as_str()),
            ErrorData::SimpleMessage(msg) => msg.message.fmt(fmt),
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl error::Error for Error {
    #[allow(deprecated, deprecated_in_future)]
    fn description(&self) -> &str {
        match self.repr.data() {
            ErrorData::Os(..) | ErrorData::Simple(..) => self.kind().as_str(),
            ErrorData::SimpleMessage(msg) => msg.message,
            ErrorData::Custom(c) => c.error.description(),
        }
    }

    #[allow(deprecated)]
    fn cause(&self) -> Option<&dyn error::Error> {
        match self.repr.data() {
            ErrorData::Os(..) => None,
            ErrorData::Simple(..) => None,
            ErrorData::SimpleMessage(..) => None,
            ErrorData::Custom(c) => c.error.cause(),
        }
    }

    fn source(&self) -> Option<&(dyn error::Error + 'static)> {
        match self.repr.data() {
            ErrorData::Os(..) => None,
            ErrorData::Simple(..) => None,
            ErrorData::SimpleMessage(..) => None,
            ErrorData::Custom(c) => c.error.source(),
        }
    }
}

fn _assert_error_is_sync_send() {
    fn _is_sync_send<T: Sync + Send>() {}
    _is_sync_send::<Error>();
}
