//! This is a densely packed error representation which is used on targets with <br>这是一个密集包装的错误表示，用于具有<br>
//! 64-bit pointers. <br>64 位指针。<br>
//!
//! (Note that `bitpacked` vs `unpacked` here has no relationship to `#[repr(packed)]`, it just refers to attempting to use any available bits in a more clever manner than `rustc`'s default layout algorithm would). <br>(请注意，这里的 `bitpacked` 与 `unpacked` 与 `#[repr(packed)]` 没有关系，它只是指尝试以比 `rustc` 的默认布局算法更聪明的方式使用任何可用位)。<br>
//!
//! Conceptually, it stores the same data as the "unpacked" equivalent we use on other targets. <br>从概念上讲，它存储的数据与我们在其他目标上使用的 "unpacked" 等效数据相同。<br> Specifically, you can imagine it as an optimized version of the following enum (which is roughly equivalent to what's stored by `repr_unpacked::Repr`, e.g. <br>具体来说，您可以将其想象为以下枚举的优化版本 (大致相当于 `repr_unpacked::Repr` 存储的内容，例如<br>
//! `super::ErrorData<Box<Custom>>`):
//!
//! ```ignore (exposition-only)
//! enum ErrorData {
//!    Os(i32),
//!    Simple(ErrorKind),
//!    SimpleMessage(&'static SimpleMessage),
//!    Custom(Box<Custom>),
//! }
//! ```
//!
//! However, it packs this data into a 64bit non-zero value. <br>但是，它将这些数据打包成一个 64 位非零值。<br>
//!
//! This optimization not only allows `io::Error` to occupy a single pointer, but improves `io::Result` as well, especially for situations like `io::Result<()>` (which is now 64 bits) or `io::Result<u64>` (which is now <br>这种优化不仅允许 `io::Error` 占用单个指针，而且还改进了 `io::Result`，尤其是对于像 `io::Result<()>` (现在是 64 位) 或 `io::Result<u64>` (现在<br>
//! 128 bits), which are quite common. <br>128 位)，这是非常常见的。<br>
//!
//! # Layout
//! Tagged values are 64 bits, with the 2 least significant bits used for the tag. <br>标记值为 64 位，其中 2 个最低有效位用于标记。<br> This means there are there are 4 "variants": <br>这意味着有 4 个变体：<br>
//!
//! - **Tag 0b00**: The first variant is equivalent to `ErrorData::SimpleMessage`, and holds a `&'static SimpleMessage` directly. <br>**标记 0b00**: 第一个变体相当于 `ErrorData::SimpleMessage`，直接持有一个 `&'static SimpleMessage`。<br>
//!
//!   `SimpleMessage` has an alignment >= 4 (which is requested with `#[repr(align)]` and checked statically at the bottom of this file), which means every `&'static SimpleMessage` should have the both tag bits as 0, meaning its tagged and untagged representation are equivalent. <br>`SimpleMessage` 具有 >= 4 的对齐方式 (这是通过 `#[repr(align)]` 请求的，并在此文件底部进行静态检查)，这意味着每个 `&'static SimpleMessage` 的两个标记位都应为 0，这意味着它的标记和未标记表示是等效的。<br>
//!
//!
//!   This means we can skip tagging it, which is necessary as this variant can be constructed from a `const fn`, which probably cannot tag pointers (or at least it would be difficult). <br>这意味着我们可以跳过标记它，这是必要的，因为这个变体可以从 `const fn` 构造，它可能无法标记指针 (或者至少它会很困难)。<br>
//!
//! - **Tag 0b01**: The other pointer variant holds the data for `ErrorData::Custom` and the remaining 62 bits are used to store a `Box<Custom>`. <br>**标记 0b01**: 另一个指针，变体，保存 `ErrorData::Custom` 的数据，其余 62 位用于存储 `Box<Custom>`。<br> `Custom` also has alignment >= 4, so the bottom two bits are free to use for the tag. <br>`Custom` 也有对齐 >= 4，因此底部两位可自由用于标记。<br>
//!
//!   The only important thing to note is that `ptr::wrapping_add` and `ptr::wrapping_sub` are used to tag the pointer, rather than bitwise operations. <br>唯一需要注意的是 `ptr::wrapping_add` 和 `ptr::wrapping_sub` 用于标记指针，而不是按位操作。<br> This should preserve the pointer's provenance, which would otherwise be lost. <br>这应该保留指针的出处，否则会丢失。<br>
//!
//! - **Tag 0b10**: Holds the data for `ErrorData::Os(i32)`. <br>**标记 0b10**: 保存 `ErrorData::Os(i32)` 的数据。<br> We store the `i32` in the pointer's most significant 32 bits, and don't use the bits `2..32` for anything. <br>我们将 `i32` 存储在指针的最高有效 32 位中，并且不使用 `2..32` 位做任何事情。<br> Using the top 32 bits is just to let us easily recover the `i32` code with the correct sign. <br>使用前 32 位只是为了让我们轻松恢复具有正确符号的 `i32` 代码。<br>
//!
//! - **Tag 0b11**: Holds the data for `ErrorData::Simple(ErrorKind)`. <br>**标记 0b11**: 保存 `ErrorData::Simple(ErrorKind)` 的数据。<br> This stores the `ErrorKind` in the top 32 bits as well, although it doesn't occupy nearly that many. <br>这也将 `ErrorKind` 存储在前 32 位中，尽管它几乎没有占用那么多。<br> Most of the bits are unused here, but it's not like we need them for anything else yet. <br>大多数位在这里都没有使用，但我们还不需要它们来做其他事情。<br>
//!
//! # Use of `NonNull<()>` <br>使用 `NonNull<()>`<br>
//!
//! Everything is stored in a `NonNull<()>`, which is odd, but actually serves a purpose. <br>一切都存储在 `NonNull<()>` 中，这很奇怪，但实际上是有目的的。<br>
//!
//! Conceptually you might think of this more like: <br>从概念上讲，您可能会认为这更像是：<br>
//!
//! ```ignore (exposition-only)
//! union Repr {
//!     // holds integer (Simple/Os) variants, and provides access to the tag bits. <br>保存整数 (Simple/Os) 变体，并提供对标记位的访问。<br>
//!     //
//!     bits: NonZeroU64,
//!     // Tag is 0, so this is stored untagged. <br>标记为 0，因此未标记存储。<br>
//!     msg: &'static SimpleMessage,
//!     // Tagged (offset) `Box<Custom>` pointer. <br>标记 (偏移) `Box<Custom>` 指针。<br>
//!     tagged_custom: NonNull<()>,
//! }
//! ```
//!
//! But there are a few problems with this: <br>但是这样做有几个问题:<br>
//!
//! 1. Union access is equivalent to a transmute, so this representation would require we transmute between integers and pointers in at least one direction, which may be UB (and even if not, it is likely harder for a compiler to reason about than explicit ptr->int operations). <br>Union 访问等效于转换，因此这种表示需要我们在整数和指针之间至少在一个方向上转换，这可能是 UB (即使不是，编译器可能比显式 ptr -> 更难推理整数运算)。<br>
//!
//! 2.
//! Even if all fields of a union have a niche, the union itself doesn't, although this may change in the future. <br>即使 union 的所有字段都有 niche，union 本身也没有，尽管这在未来可能会改变。<br> This would make things like `io::Result<()>` and `io::Result<usize>` larger, which defeats part of the motivation of this bitpacking. <br>这会使 `io::Result<()>` 和 `io::Result<usize>` 之类的东西变得更大，从而破坏了这种 bitpacking 的部分动机。<br>
//!
//! Storing everything in a `NonZeroUsize` (or some other integer) would be a bit more traditional for pointer tagging, but it would lose provenance information, couldn't be constructed from a `const fn`, and would probably run into other issues as well. <br>将所有内容存储在 `NonZeroUsize` (或其他整数) 中对于指针标记来说会更传统一些，但它会丢失出处信息，无法从 `const fn` 构造，并且可能还会遇到其他问题。<br>
//!
//! The `NonNull<()>` seems like the only alternative, even if it's fairly odd to use a pointer type to store something that may hold an integer, some of the time. <br>`NonNull<()>` 似乎是唯一的选择，即使有时使用指针类型来存储可能包含整数的东西是相当奇怪的。<br>
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

use super::{Custom, ErrorData, ErrorKind, SimpleMessage};
use alloc::boxed::Box;
use core::marker::PhantomData;
use core::mem::{align_of, size_of};
use core::ptr::{self, NonNull};

// The 2 least-significant bits are used as tag. <br>2 个最低有效位用作标记。<br>
const TAG_MASK: usize = 0b11;
const TAG_SIMPLE_MESSAGE: usize = 0b00;
const TAG_CUSTOM: usize = 0b01;
const TAG_OS: usize = 0b10;
const TAG_SIMPLE: usize = 0b11;

/// The internal representation. <br>内部表示。<br>
///
/// See the module docs for more, this is just a way to hack in a check that we indeed are not unwind-safe. <br>有关更多信息，请参见模块文档，这只是一种侵入检查的方式，我们实际上并不安全。<br>
///
///
/// ```compile_fail,E0277
/// fn is_unwind_safe<T: core::panic::UnwindSafe>() {}
/// is_unwind_safe::<std::io::Error>();
/// ```
#[repr(transparent)]
pub(super) struct Repr(NonNull<()>, PhantomData<ErrorData<Box<Custom>>>);

// All the types `Repr` stores internally are Send + Sync, and so is it. <br>`Repr` 内部存储的所有类型都是 Send + Sync，它也是如此。<br>
unsafe impl Send for Repr {}
unsafe impl Sync for Repr {}

impl Repr {
    pub(super) fn new_custom(b: Box<Custom>) -> Self {
        let p = Box::into_raw(b).cast::<u8>();
        // Should only be possible if an allocator handed out a pointer with wrong alignment. <br>只有当分配器分发了一个错误对齐的指针时才可能。<br>
        //
        debug_assert_eq!(p.addr() & TAG_MASK, 0);
        // Note: We know `TAG_CUSTOM <= size_of::<Custom>()` (static_assert at end of file), and both the start and end of the expression must be valid without address space wraparound due to `Box`'s semantics. <br>我们知道 `TAG_CUSTOM <= size_of::<Custom>()` (文件末尾的 static_assert)，由于 `Box` 的语义，表达式的开头和结尾都必须在没有地址空间环绕的情况下有效。<br>
        //
        //
        // This means it would be correct to implement this using `ptr::add` (rather than `ptr::wrapping_add`), but it's unclear this would give any benefit, so we just use `wrapping_add` instead. <br>这意味着使用 `ptr::add` (而不是 `ptr::wrapping_add`) 实现它是正确的，但不清楚这会带来什么好处，所以我们只使用 `wrapping_add`。<br>
        //
        //
        //
        let tagged = p.wrapping_add(TAG_CUSTOM).cast::<()>();
        // Safety: `TAG_CUSTOM + p` is the same as `TAG_CUSTOM | p`, because `p`'s alignment means it isn't allowed to have any of the `TAG_BITS` set (you can verify that addition and bitwise-or are the same when the operands have no bits in common using a truth table). <br>安全性: `TAG_CUSTOM + p` 与 `TAG_CUSTOM | p` 相同，因为 `p` 的对齐方式意味着不允许设置任何 `TAG_BITS` (当操作数没有共同位时，您可以验证加法和按位或相同) 使用真值表)。<br>
        //
        //
        // Then, `TAG_CUSTOM | p` is not zero, as that would require `TAG_CUSTOM` and `p` both be zero, and neither is (as `p` came from a box, and `TAG_CUSTOM` just... <br>然后，`TAG_CUSTOM | p` 不为零，因为这要求 `TAG_CUSTOM` 和 `p` 都为零，而且两者都不是 (因为 `p` 来自 box，而 `TAG_CUSTOM` 只是......<br>
        // isn't zero -- it's `0b01`). <br>不是零 -- 它是 `0b01`)。<br>
        // Therefore, `TAG_CUSTOM + p` isn't zero and so `tagged` can't be, and the `new_unchecked` is safe. <br>因此，`TAG_CUSTOM + p` 不为零，所以 `tagged` 不能为零，`new_unchecked` 是安全的。<br>
        //
        //
        //
        //
        let res = Self(unsafe { NonNull::new_unchecked(tagged) }, PhantomData);
        // quickly smoke-check we encoded the right thing (This generally will only run in libstd's tests, unless the user uses -Zbuild-std) <br>快速冒烟检查我们编码了正确的东西 (这通常只会在 libstd 的测试中运行，除非用户使用 - Zbuild-std)<br>
        //
        debug_assert!(matches!(res.data(), ErrorData::Custom(_)), "repr(custom) encoding failed");
        res
    }

    #[inline]
    pub(super) fn new_os(code: i32) -> Self {
        let utagged = ((code as usize) << 32) | TAG_OS;
        // Safety: `TAG_OS` is not zero, so the result of the `|` is not 0. <br>安全性: `TAG_OS` 不为零，因此 `|` 的结果不为 0。<br>
        let res = Self(unsafe { NonNull::new_unchecked(ptr::invalid_mut(utagged)) }, PhantomData);
        // quickly smoke-check we encoded the right thing (This generally will only run in libstd's tests, unless the user uses -Zbuild-std) <br>快速冒烟检查我们编码了正确的东西 (这通常只会在 libstd 的测试中运行，除非用户使用 - Zbuild-std)<br>
        //
        debug_assert!(
            matches!(res.data(), ErrorData::Os(c) if c == code),
            "repr(os) encoding failed for {code}"
        );
        res
    }

    #[inline]
    pub(super) fn new_simple(kind: ErrorKind) -> Self {
        let utagged = ((kind as usize) << 32) | TAG_SIMPLE;
        // Safety: `TAG_SIMPLE` is not zero, so the result of the `|` is not 0. <br>安全性: `TAG_SIMPLE` 不为零，因此 `|` 的结果不为 0。<br>
        let res = Self(unsafe { NonNull::new_unchecked(ptr::invalid_mut(utagged)) }, PhantomData);
        // quickly smoke-check we encoded the right thing (This generally will only run in libstd's tests, unless the user uses -Zbuild-std) <br>快速冒烟检查我们编码了正确的东西 (这通常只会在 libstd 的测试中运行，除非用户使用 - Zbuild-std)<br>
        //
        debug_assert!(
            matches!(res.data(), ErrorData::Simple(k) if k == kind),
            "repr(simple) encoding failed {:?}",
            kind,
        );
        res
    }

    #[inline]
    pub(super) const fn new_simple_message(m: &'static SimpleMessage) -> Self {
        // Safety: References are never null. <br>安全性：引用永远不会为空。<br>
        Self(unsafe { NonNull::new_unchecked(m as *const _ as *mut ()) }, PhantomData)
    }

    #[inline]
    pub(super) fn data(&self) -> ErrorData<&Custom> {
        // Safety: We're a Repr, decode_repr is fine. <br>安全：我们是 Repr，decode_repr 是好的。<br>
        unsafe { decode_repr(self.0, |c| &*c) }
    }

    #[inline]
    pub(super) fn data_mut(&mut self) -> ErrorData<&mut Custom> {
        // Safety: We're a Repr, decode_repr is fine. <br>安全：我们是 Repr，decode_repr 是好的。<br>
        unsafe { decode_repr(self.0, |c| &mut *c) }
    }

    #[inline]
    pub(super) fn into_data(self) -> ErrorData<Box<Custom>> {
        let this = core::mem::ManuallyDrop::new(self);
        // Safety: We're a Repr, decode_repr is fine. <br>安全：我们是 Repr，decode_repr 是好的。<br>
        // The `Box::from_raw` is safe because we prevent double-drop using `ManuallyDrop`. <br>`Box::from_raw` 是安全的，因为我们使用 `ManuallyDrop` 防止丢弃。<br>
        unsafe { decode_repr(this.0, |p| Box::from_raw(p)) }
    }
}

impl Drop for Repr {
    #[inline]
    fn drop(&mut self) {
        // Safety: We're a Repr, decode_repr is fine. <br>安全：我们是 Repr，decode_repr 是好的。<br>
        // The `Box::from_raw` is safe because we're being dropped. <br>`Box::from_raw` 是安全的，因为我们正在被丢弃。<br>
        unsafe {
            let _ = decode_repr(self.0, |p| Box::<Custom>::from_raw(p));
        }
    }
}

// Shared helper to decode a `Repr`'s internal pointer into an ErrorData. <br>将 `Repr` 的内部指针解码为 ErrorData 的共享助手。<br>
//
// Safety: `ptr`'s bits should be encoded as described in the document at the top (it should `some_repr.0`) <br>安全性: `ptr` 的位应该按照顶部文档中的描述进行编码 (应该是 `some_repr.0`)<br>
//
#[inline]
unsafe fn decode_repr<C, F>(ptr: NonNull<()>, make_custom: F) -> ErrorData<C>
where
    F: FnOnce(*mut Custom) -> C,
{
    let bits = ptr.as_ptr().addr();
    match bits & TAG_MASK {
        TAG_OS => {
            let code = ((bits as i64) >> 32) as i32;
            ErrorData::Os(code)
        }
        TAG_SIMPLE => {
            let kind_bits = (bits >> 32) as u32;
            let kind = kind_from_prim(kind_bits).unwrap_or_else(|| {
                debug_assert!(false, "Invalid io::error::Repr bits: `Repr({:#018x})`", bits);
                // This means the `ptr` passed in was not valid, which violates the unsafe contract of `decode_repr`. <br>这意味着传入的 `ptr` 无效，违反了 `decode_repr` 的不安全契约。<br>
                //
                // Using this rather than unwrap meaningfully improves the code for callers which only care about one variant (usually `Custom`) <br>使用它而不是展开有意义地改进了只关心一个变体 (通常是 `Custom`) 的调用者的代码<br>
                //
                //
                //
                core::hint::unreachable_unchecked();
            });
            ErrorData::Simple(kind)
        }
        TAG_SIMPLE_MESSAGE => ErrorData::SimpleMessage(&*ptr.cast::<SimpleMessage>().as_ptr()),
        TAG_CUSTOM => {
            // It would be correct for us to use `ptr::sub` here (see the comment above the `wrapping_add` call in `new_custom` for why), but it isn't clear that it makes a difference, so we don't. <br>我们在这里使用 `ptr::sub` 是正确的 (请参见 `new_custom` 中 `wrapping_add` 调用上方的注释了解原因)，但不清楚它是否有区别，所以我们没有。<br>
            //
            //
            let custom = ptr.as_ptr().cast::<u8>().wrapping_sub(TAG_CUSTOM).cast::<Custom>();
            ErrorData::Custom(make_custom(custom))
        }
        _ => {
            // Can't happen, and compiler can tell <br>这是不可能发生的，编译器可以判断<br>
            unreachable!();
        }
    }
}

// This compiles to the same code as the check+transmute, but doesn't require unsafe, or to hard-code max ErrorKind or its size in a way the compiler couldn't verify. <br>这将编译为与 check+transmute 相同的代码，但不需要 unsafe，或者以编译器无法验证的方式硬编码 max ErrorKind 或其大小。<br>
//
//
#[inline]
fn kind_from_prim(ek: u32) -> Option<ErrorKind> {
    macro_rules! from_prim {
        ($prim:expr => $Enum:ident { $($Variant:ident),* $(,)? }) => {{
            // Force a compile error if the list gets out of date. <br>如果列表过期，则强制编译错误。<br>
            const _: fn(e: $Enum) = |e: $Enum| match e {
                $($Enum::$Variant => ()),*
            };
            match $prim {
                $(v if v == ($Enum::$Variant as _) => Some($Enum::$Variant),)*
                _ => None,
            }
        }}
    }
    from_prim!(ek => ErrorKind {
        NotFound,
        PermissionDenied,
        ConnectionRefused,
        ConnectionReset,
        HostUnreachable,
        NetworkUnreachable,
        ConnectionAborted,
        NotConnected,
        AddrInUse,
        AddrNotAvailable,
        NetworkDown,
        BrokenPipe,
        AlreadyExists,
        WouldBlock,
        NotADirectory,
        IsADirectory,
        DirectoryNotEmpty,
        ReadOnlyFilesystem,
        FilesystemLoop,
        StaleNetworkFileHandle,
        InvalidInput,
        InvalidData,
        TimedOut,
        WriteZero,
        StorageFull,
        NotSeekable,
        FilesystemQuotaExceeded,
        FileTooLarge,
        ResourceBusy,
        ExecutableFileBusy,
        Deadlock,
        CrossesDevices,
        TooManyLinks,
        InvalidFilename,
        ArgumentListTooLong,
        Interrupted,
        Other,
        UnexpectedEof,
        Unsupported,
        OutOfMemory,
        Uncategorized,
    })
}

// Some static checking to alert us if a change breaks any of the assumptions that our encoding relies on for correctness and soundness. <br>一些静态检查提醒我们一个变化是否打破了我们编码所依赖的正确性和可靠性的任何假设。<br>
// (Some of these are a bit overly thorough/cautious, admittedly) <br>(诚然，其中一些有点过于彻底/谨慎)<br>
//
// If any of these are hit on a platform that libstd supports, we should likely just use `repr_unpacked.rs` there instead (unless the fix is easy). <br>如果其中任何一个在 libstd 支持的平台上运行，我们可能应该在那里使用 `repr_unpacked.rs` 代替 (除非修复很容易)。<br>
//
//
macro_rules! static_assert {
    ($condition:expr) => {
        const _: () = assert!($condition);
    };
    (@usize_eq: $lhs:expr, $rhs:expr) => {
        const _: [(); $lhs] = [(); $rhs];
    };
}

// The bitpacking we use requires pointers be exactly 64 bits. <br>我们使用的位打包要求指针正好是 64 位。<br>
static_assert!(@usize_eq: size_of::<NonNull<()>>(), 8);

// We also require pointers and usize be the same size. <br>我们还要求指针和 usize 的大小相同。<br>
static_assert!(@usize_eq: size_of::<NonNull<()>>(), size_of::<usize>());

// `Custom` and `SimpleMessage` need to be thin pointers. <br>`Custom` 和 `SimpleMessage` 需要是瘦指针。<br>
static_assert!(@usize_eq: size_of::<&'static SimpleMessage>(), 8);
static_assert!(@usize_eq: size_of::<Box<Custom>>(), 8);

static_assert!((TAG_MASK + 1).is_power_of_two());
// And they must have sufficient alignment. <br>它们必须有足够的对齐。<br>
static_assert!(align_of::<SimpleMessage>() >= TAG_MASK + 1);
static_assert!(align_of::<Custom>() >= TAG_MASK + 1);

static_assert!(@usize_eq: (TAG_MASK & TAG_SIMPLE_MESSAGE), TAG_SIMPLE_MESSAGE);
static_assert!(@usize_eq: (TAG_MASK & TAG_CUSTOM), TAG_CUSTOM);
static_assert!(@usize_eq: (TAG_MASK & TAG_OS), TAG_OS);
static_assert!(@usize_eq: (TAG_MASK & TAG_SIMPLE), TAG_SIMPLE);

// This is obviously true (`TAG_CUSTOM` is `0b01`), but in `Repr::new_custom` we offset a pointer by this value, and expect it to both be within the same object, and to not wrap around the address space. <br>这显然是正确的 (`TAG_CUSTOM` 是 `0b01`)，但是在 `Repr::new_custom` 中，我们将指针偏移了这个值，并期望它都在同一个 object 内，并且不会环绕地址空间。<br>
// See the comment in that function for further details. <br>有关更多详细信息，请参见该函数中的注释。<br>
//
// Actually, at the moment we use `ptr::wrapping_add`, not `ptr::add`, so this check isn't needed for that one, although the assertion that we don't actually wrap around in that wrapping_add does simplify the safety reasoning elsewhere considerably. <br>实际上，目前我们使用的是 `ptr::wrapping_add`，而不是 `ptr::add`，因此该检查不需要此检查，尽管我们实际上并未在 wrapping_add 中进行回绕的断言确实大大简化了其他地方的安全推理。<br>
//
//
//
//
//
static_assert!(size_of::<Custom>() >= TAG_CUSTOM);

// These two store a payload which is allowed to be zero, so they must be non-zero to preserve the `NonNull`'s range invariant. <br>这两个函数存储的有效载荷允许为零，因此它们必须为非零以保持 `NonNull` 的范围不变。<br>
//
static_assert!(TAG_OS != 0);
static_assert!(TAG_SIMPLE != 0);
// We can't tag `SimpleMessage`s, the tag must be 0. <br>我们不能标记 `SimpleMessage`，标记必须为 0。<br>
static_assert!(@usize_eq: TAG_SIMPLE_MESSAGE, 0);

// Check that the point of all of this still holds. <br>检查所有这一切的要点是否仍然成立。<br>
//
// We'd check against `io::Error`, but *technically* it's allowed to vary, as it's not `#[repr(transparent)]`/`#[repr(C)]`. <br>我们会检查 `io::Error`，但技术上它是允许变化的，因为它不是 `#[repr(transparent)]`/`#[repr(C)]`。<br>
// We could add that, but the `#[repr()]` would show up in rustdoc, which might be seen as a stable commitment. <br>我们可以添加它，但 `#[repr()]` 会出现在 rustdoc 中，这可能被视为一个稳定的承诺。<br>
//
//
static_assert!(@usize_eq: size_of::<Repr>(), 8);
static_assert!(@usize_eq: size_of::<Option<Repr>>(), 8);
static_assert!(@usize_eq: size_of::<Result<(), Repr>>(), 8);
static_assert!(@usize_eq: size_of::<Result<usize, Repr>>(), 16);
