//! Traits, helpers, and type definitions for core I/O functionality. <br>核心 I/O 功能的 Traits，帮助程序和类型定义。<br>
//!
//! The `std::io` module contains a number of common things you'll need when doing input and output. <br>`std::io` 模块包含许多在执行输入和输出时需要的常见操作。<br>
//! The most core part of this module is the [`Read`] and [`Write`] traits, which provide the most general interface for reading and writing input and output. <br>该模块中最核心的部分是 [`Read`] 和 [`Write`] traits，它们提供用于读取和写入输入和输出的最通用接口。<br>
//!
//! # Read and Write <br>读和写<br>
//!
//! Because they are traits, [`Read`] and [`Write`] are implemented by a number of other types, and you can implement them for your types too. <br>因为它们是 traits，所以 [`Read`] 和 [`Write`] 由许多其他类型实现，您也可以为您的类型实现它们。<br>
//! As such, you'll see a few different types of I/O throughout the documentation in this module: [`File`]s, [`TcpStream`]s, and sometimes even [`Vec<T>`]s. <br>这样，您将在此模块的整个文档中看到几种不同类型的 I/O: [`File`]，[`TcpStream`]，有时甚至是 [`Vec<T>`]。<br>
//! For example, [`Read`] adds a [`read`][`Read::read`] method, which we can use on [`File`]s: <br>例如，[`Read`] 添加了 [`read`][`Read::read`] 方法，我们可以在 [`File`] 上使用该方法：<br>
//!
//! ```no_run
//! use std::io;
//! use std::io::prelude::*;
//! use std::fs::File;
//!
//! fn main() -> io::Result<()> {
//!     let mut f = File::open("foo.txt")?;
//!     let mut buffer = [0; 10];
//!
//!     // read up to 10 bytes <br>最多读取 10 个字节<br>
//!     let n = f.read(&mut buffer)?;
//!
//!     println!("The bytes: {:?}", &buffer[..n]);
//!     Ok(())
//! }
//! ```
//!
//! [`Read`] and [`Write`] are so important, implementors of the two traits have a nickname: readers and writers. <br>[`Read`] 和 [`Write`] 非常重要，两个 traits 的实现者有一个昵称: readers 和 writers。<br>
//! So you'll sometimes see 'a reader' instead of 'a type that implements the [`Read`] trait'. <br>因此，有时您会看到 `reader` 而不是 `实现 [`Read`] trait 的类型`。<br>
//! Much easier! <br>容易得多！<br>
//!
//! ## Seek and BufRead <br>Seek 和 BufRead<br>
//!
//! Beyond that, there are two important traits that are provided: [`Seek`] and [`BufRead`]. <br>除此之外，还提供了两个重要的 traits: [`Seek`] 和 [`BufRead`]。<br> Both of these build on top of a reader to control how the reading happens. <br>两者都建立在 reader 的顶部，以控制读取的方式。<br>
//! [`Seek`] lets you control where the next byte is coming from: <br>[`Seek`] 可让您控制下一个字节的来源：<br>
//!
//! ```no_run
//! use std::io;
//! use std::io::prelude::*;
//! use std::io::SeekFrom;
//! use std::fs::File;
//!
//! fn main() -> io::Result<()> {
//!     let mut f = File::open("foo.txt")?;
//!     let mut buffer = [0; 10];
//!
//!     // skip to the last 10 bytes of the file <br>跳到文件的最后 10 个字节<br>
//!     f.seek(SeekFrom::End(-10))?;
//!
//!     // read up to 10 bytes <br>最多读取 10 个字节<br>
//!     let n = f.read(&mut buffer)?;
//!
//!     println!("The bytes: {:?}", &buffer[..n]);
//!     Ok(())
//! }
//! ```
//!
//! [`BufRead`] uses an internal buffer to provide a number of other ways to read, but to show it off, we'll need to talk about buffers in general. <br>[`BufRead`] 使用内部缓冲区来提供许多其他读取方式，但为了展示它，我们需要一般地讨论缓冲区。<br> Keep reading! <br>继续阅读！<br>
//!
//! ## BufReader and BufWriter <br>BufReader 和 BufWriter<br>
//!
//! Byte-based interfaces are unwieldy and can be inefficient, as we'd need to be making near-constant calls to the operating system. <br>基于字节的接口笨拙且效率低下，因为我们需要对操作系统进行近乎恒定的调用。<br>
//! To help with this, `std::io` comes with two structs, [`BufReader`] and [`BufWriter`], which wrap readers and writers. <br>为了解决这个问题，`std::io` 带有两个结构体 [`BufReader`] 和 [`BufWriter`]，它们包装了 readers 和 writers。<br>
//! The wrapper uses a buffer, reducing the number of calls and providing nicer methods for accessing exactly what you want. <br>包装器使用缓冲区，从而减少了调用次数，并提供了更好的方法来访问所需的内容。<br>
//!
//! For example, [`BufReader`] works with the [`BufRead`] trait to add extra methods to any reader: <br>例如，[`BufReader`] 与 [`BufRead`] trait 一起为任何 reader 添加额外的方法：<br>
//!
//! ```no_run
//! use std::io;
//! use std::io::prelude::*;
//! use std::io::BufReader;
//! use std::fs::File;
//!
//! fn main() -> io::Result<()> {
//!     let f = File::open("foo.txt")?;
//!     let mut reader = BufReader::new(f);
//!     let mut buffer = String::new();
//!
//!     // read a line into buffer <br>将一行读入缓冲区<br>
//!     reader.read_line(&mut buffer)?;
//!
//!     println!("{buffer}");
//!     Ok(())
//! }
//! ```
//!
//! [`BufWriter`] doesn't add any new ways of writing; <br>[`BufWriter`] 没有添加任何新的写入方式；<br> it just buffers every call to [`write`][`Write::write`]: <br>它只是缓冲每个调用到 [`write`][`Write::write`]：<br>
//!
//! ```no_run
//! use std::io;
//! use std::io::prelude::*;
//! use std::io::BufWriter;
//! use std::fs::File;
//!
//! fn main() -> io::Result<()> {
//!     let f = File::create("foo.txt")?;
//!     {
//!         let mut writer = BufWriter::new(f);
//!
//!         // write a byte to the buffer <br>向缓冲区写入一个字节<br>
//!         writer.write(&[42])?;
//!
//!     } // the buffer is flushed once writer goes out of scope <br>writer 离开作用域后，将刷新缓冲区<br>
//!
//!     Ok(())
//! }
//! ```
//!
//! ## Standard input and output <br>标准输入输出<br>
//!
//! A very common source of input is standard input: <br>输入的一个非常常见的来源是标准输入：<br>
//!
//! ```no_run
//! use std::io;
//!
//! fn main() -> io::Result<()> {
//!     let mut input = String::new();
//!
//!     io::stdin().read_line(&mut input)?;
//!
//!     println!("You typed: {}", input.trim());
//!     Ok(())
//! }
//! ```
//!
//! Note that you cannot use the [`?` operator] in functions that do not return a [`Result<T, E>`][`Result`]. <br>请注意，不能在不返回 [`Result<T, E>`][`Result`] 的函数中使用 [`?` operator]。<br>
//! Instead, you can call [`.unwrap()`] or `match` on the return value to catch any possible errors: <br>相反，您可以在返回值上调用 [`.unwrap()`] 或 `match` 来捕获任何可能的错误：<br>
//!
//! ```no_run
//! use std::io;
//!
//! let mut input = String::new();
//!
//! io::stdin().read_line(&mut input).unwrap();
//! ```
//!
//! And a very common source of output is standard output: <br>标准输出是一个非常常见的输出源：<br>
//!
//! ```no_run
//! use std::io;
//! use std::io::prelude::*;
//!
//! fn main() -> io::Result<()> {
//!     io::stdout().write(&[42])?;
//!     Ok(())
//! }
//! ```
//!
//! Of course, using [`io::stdout`] directly is less common than something like [`println!`]. <br>当然，直接使用 [`io::stdout`] 比使用 [`println!`] 少见。<br>
//!
//! ## Iterator types <br>迭代器类型<br>
//!
//! A large number of the structures provided by `std::io` are for various ways of iterating over I/O. <br>`std::io` 提供的大量结构用于在 I/O 上进行迭代的各种方式。<br> For example, [`Lines`] is used to split over lines: <br>例如，[`Lines`] 用于拆分多行：<br>
//!
//! ```no_run
//! use std::io;
//! use std::io::prelude::*;
//! use std::io::BufReader;
//! use std::fs::File;
//!
//! fn main() -> io::Result<()> {
//!     let f = File::open("foo.txt")?;
//!     let reader = BufReader::new(f);
//!
//!     for line in reader.lines() {
//!         println!("{}", line?);
//!     }
//!     Ok(())
//! }
//! ```
//!
//! ## Functions
//!
//! There are a number of [functions][functions-list] that offer access to various features. <br>有许多 [函数][functions-list] 提供对各种特性的访问。<br>
//! For example, we can use three of these functions to copy everything from standard input to standard output: <br>例如，我们可以使用以下三个函数将所有内容从标准输入复制到标准输出：<br>
//!
//! ```no_run
//! use std::io;
//!
//! fn main() -> io::Result<()> {
//!     io::copy(&mut io::stdin(), &mut io::stdout())?;
//!     Ok(())
//! }
//! ```
//!
//! [functions-list]: #functions-1
//!
//! ## io::Result
//!
//! Last, but certainly not least, is [`io::Result`]. <br>最后但并非最不重要的是 [`io::Result`]。<br>
//! This type is used as the return type of many `std::io` functions that can cause an error, and can be returned from your own functions as well. <br>此类型用作许多 `std::io` 函数的返回类型，它们可能导致错误，也可以从您自己的函数中返回。<br>
//! Many of the examples in this module use the [`?` operator]: <br>该模块中的许多示例都使用 [`?` operator]：<br>
//!
//! ```
//! use std::io;
//!
//! fn read_input() -> io::Result<()> {
//!     let mut input = String::new();
//!
//!     io::stdin().read_line(&mut input)?;
//!
//!     println!("You typed: {}", input.trim());
//!
//!     Ok(())
//! }
//! ```
//!
//! The return type of `read_input()`, [`io::Result<()>`][`io::Result`], is a very common type for functions which don't have a 'real' return value, but do want to return errors if they happen. <br>`read_input()` 的返回类型 [`io::Result<()>`][`io::Result`] 是函数的一种非常常见的类型，它没有 'real' 返回值，但是希望在发生错误时返回错误。<br>
//!
//! In this case, the only purpose of this function is to read the line and print it, so we use `()`. <br>在这种情况下，此函数的唯一目的是读取并打印该行，因此我们使用 `()`。<br>
//!
//! ## Platform-specific behavior <br>特定于平台的行为<br>
//!
//! Many I/O functions throughout the standard library are documented to indicate what various library or syscalls they are delegated to. <br>记录了整个标准库中的许多 I/O 函数，以指示将它们委派给的各种库或 syscall。<br>
//! This is done to help applications both understand what's happening under the hood as well as investigate any possibly unclear semantics. <br>这样做是为了帮助应用程序了解幕后的情况以及调查任何可能不清楚的语义。<br>
//! Note, however, that this is informative, not a binding contract. <br>但是请注意，这只是提供信息，而不是有约束力的契约。<br>
//! The implementation of many of these functions are subject to change over time and may call fewer or more syscalls/library functions. <br>其中许多函数的实现会随时间而变化，并且可能调用更少或更多的 syscalls/library 函数。<br>
//!
//! [`File`]: crate::fs::File
//! [`TcpStream`]: crate::net::TcpStream
//! [`io::stdout`]: stdout
//! [`io::Result`]: self::Result
//! [`?` operator]: ../../book/appendix-02-operators.html
//! [`Result`]: crate::result::Result
//! [`.unwrap()`]: crate::result::Result::unwrap
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

#[cfg(test)]
mod tests;

use crate::cmp;
use crate::fmt;
use crate::mem::replace;
use crate::ops::{Deref, DerefMut};
use crate::slice;
use crate::str;
use crate::sys;
use crate::sys_common::memchr;

#[stable(feature = "bufwriter_into_parts", since = "1.56.0")]
pub use self::buffered::WriterPanicked;
#[unstable(feature = "internal_output_capture", issue = "none")]
#[doc(no_inline, hidden)]
pub use self::stdio::set_output_capture;
#[unstable(feature = "print_internals", issue = "none")]
pub use self::stdio::{_eprint, _print};
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::{
    buffered::{BufReader, BufWriter, IntoInnerError, LineWriter},
    copy::copy,
    cursor::Cursor,
    error::{Error, ErrorKind, Result},
    stdio::{stderr, stdin, stdout, Stderr, StderrLock, Stdin, StdinLock, Stdout, StdoutLock},
    util::{empty, repeat, sink, Empty, Repeat, Sink},
};

#[unstable(feature = "read_buf", issue = "78485")]
pub use self::readbuf::ReadBuf;
pub(crate) use error::const_io_error;

mod buffered;
pub(crate) mod copy;
mod cursor;
mod error;
mod impls;
pub mod prelude;
mod readbuf;
mod stdio;
mod util;

const DEFAULT_BUF_SIZE: usize = crate::sys_common::io::DEFAULT_BUF_SIZE;

pub(crate) use stdio::cleanup;

struct Guard<'a> {
    buf: &'a mut Vec<u8>,
    len: usize,
}

impl Drop for Guard<'_> {
    fn drop(&mut self) {
        unsafe {
            self.buf.set_len(self.len);
        }
    }
}

// Several `read_to_string` and `read_line` methods in the standard library will append data into a `String` buffer, but we need to be pretty careful when doing this. <br>标准库中的几个 `read_to_string` 和 `read_line` 方法会将数据追加到 `String` 缓冲区中，但我们在执行此操作时需要非常小心。<br>
// The implementation will just call `.as_mut_vec()` and then delegate to a byte-oriented reading method, but we must ensure that when returning we never leave `buf` in a state such that it contains invalid UTF-8 in its bounds. <br>该实现将只是调用 `.as_mut_vec()`，然后委派给一个面向字节的读取方法，但我们必须确保在返回时，切勿将 `buf` 保持在其边界内包含无效 UTF-8 的状态。<br>
//
//
// To this end, we use an RAII guard (to protect against panics) which updates the length of the string when it is dropped. <br>为此，我们使用了 RAII 保护器 (以防止 panics)，该保护器会在丢弃字符串时更新字符串的长度。<br>
// This guard initially truncates the string to the prior length and only after we've validated that the new contents are valid UTF-8 do we allow it to set a longer length. <br>此保护程序最初将字符串截断为先前的长度，只有在我们确认新内容有效 UTF-8 之后，才允许它设置更长的长度。<br>
//
// The unsafety in this function is twofold: <br>此函数的不安全性是双重的：<br>
//
// 1. We're looking at the raw bytes of `buf`, so we take on the burden of UTF-8 checks. <br>我们正在查看 `buf` 的原始字节，因此我们承担了 UTF-8 检查的负担。<br>
// 2. We're passing a raw buffer to the function `f`, and it is expected that the function only *appends* bytes to the buffer. We'll get undefined behavior if existing bytes are overwritten to have non-UTF-8 data. <br>我们正在将原始缓冲区传递给函数 `f`，并且期望该函数仅追加字节到缓冲区。如果现有字节被覆盖以具有非 UTF-8 数据，我们将得到未定义的行为。<br>
//
//
//
//
//
//
//
//
pub(crate) unsafe fn append_to_string<F>(buf: &mut String, f: F) -> Result<usize>
where
    F: FnOnce(&mut Vec<u8>) -> Result<usize>,
{
    let mut g = Guard { len: buf.len(), buf: buf.as_mut_vec() };
    let ret = f(g.buf);
    if str::from_utf8(&g.buf[g.len..]).is_err() {
        ret.and_then(|_| {
            Err(error::const_io_error!(
                ErrorKind::InvalidData,
                "stream did not contain valid UTF-8"
            ))
        })
    } else {
        g.len = g.buf.len();
        ret
    }
}

// This uses an adaptive system to extend the vector when it fills. <br>当填充 vector 时，它将使用自适应系统来对其进行扩展。<br>
// We want to avoid paying to allocate and zero a huge chunk of memory if the reader only has 4 bytes while still making large reads if the reader does have a ton of data to return. <br>如果 reader 只有 4 个字节，而 reader 确实有大量数据要返回，那么我们要避免花大钱分配零内存。<br>
// Simply tacking on an extra DEFAULT_BUF_SIZE space every time is 4,500 times (!) slower than a default reservation size of 32 if the reader has a very small amount of data to return. <br>如果 reader 返回的数据量非常小，则每次简单地增加 DEFAULT_BUF_SIZE 的额外空间就比默认保留大小 32 慢 X4X 的 4,500 倍。<br>
//
//
//
pub(crate) fn default_read_to_end<R: Read + ?Sized>(r: &mut R, buf: &mut Vec<u8>) -> Result<usize> {
    let start_len = buf.len();
    let start_cap = buf.capacity();

    let mut initialized = 0; // Extra initialized bytes from previous loop iteration <br>来自先前循环迭代的额外初始化字节<br>
    loop {
        if buf.len() == buf.capacity() {
            buf.reserve(32); // buf is full, need more space <br>buf 已满，需要更多空间<br>
        }

        let mut read_buf = ReadBuf::uninit(buf.spare_capacity_mut());

        // SAFETY: These bytes were initialized but not filled in the previous loop <br>这些字节已初始化，但未在上一个循环中填充<br>
        unsafe {
            read_buf.assume_init(initialized);
        }

        match r.read_buf(&mut read_buf) {
            Ok(()) => {}
            Err(e) if e.kind() == ErrorKind::Interrupted => continue,
            Err(e) => return Err(e),
        }

        if read_buf.filled_len() == 0 {
            return Ok(buf.len() - start_len);
        }

        // store how much was initialized but not filled <br>存储多少已初始化但未填充<br>
        initialized = read_buf.initialized_len() - read_buf.filled_len();
        let new_len = read_buf.filled_len() + buf.len();

        // SAFETY: ReadBuf's invariants mean this much memory is init <br>ReadBuf 的不变量意味着这么多内存是初始化的<br>
        unsafe {
            buf.set_len(new_len);
        }

        if buf.len() == buf.capacity() && buf.capacity() == start_cap {
            // The buffer might be an exact fit. <br>缓冲区可能是完全合适的。<br>
            // Let's read into a probe buffer and see if it returns `Ok(0)`. <br>让我们读入一个探测缓冲区，看看它是否返回 `Ok(0)`。<br>
            // If so, we've avoided an unnecessary doubling of the capacity. <br>如果是这样，我们就避免了不必要的容量翻倍。<br>
            // But if not, append the probe buffer to the primary buffer and let its capacity grow. <br>但如果没有，请将探测缓冲区追加到主缓冲区，并增加它的容量。<br>
            let mut probe = [0u8; 32];

            loop {
                match r.read(&mut probe) {
                    Ok(0) => return Ok(buf.len() - start_len),
                    Ok(n) => {
                        buf.extend_from_slice(&probe[..n]);
                        break;
                    }
                    Err(ref e) if e.kind() == ErrorKind::Interrupted => continue,
                    Err(e) => return Err(e),
                }
            }
        }
    }
}

pub(crate) fn default_read_to_string<R: Read + ?Sized>(
    r: &mut R,
    buf: &mut String,
) -> Result<usize> {
    // Note that we do *not* call `r.read_to_end()` here. <br>请注意，我们在这里不调用 `r.read_to_end()`。<br> We are passing `&mut Vec<u8>` (the raw contents of `buf`) into the `read_to_end` method to fill it up. <br>我们将 `&mut Vec<u8>` (`buf` 的原始内容) 传递给 `read_to_end` 方法以填充它。<br>
    // An arbitrary implementation could overwrite the entire contents of the vector, not just append to it (which is what we are expecting). <br>任意实现都可以覆盖 vector 的全部内容，而不仅仅是追加到 vector (这是我们所期望的)。<br>
    //
    // To prevent extraneously checking the UTF-8-ness of the entire buffer we pass it to our hardcoded `default_read_to_end` implementation which we know is guaranteed to only read data into the end of the buffer. <br>为了防止额外检查整个缓冲区的 UTF-8-ness，我们将它传递给我们的硬编码 `default_read_to_end` 实现，我们知道它保证只将数据读入缓冲区的末尾。<br>
    //
    //
    //
    //
    //
    unsafe { append_to_string(buf, |b| default_read_to_end(r, b)) }
}

pub(crate) fn default_read_vectored<F>(read: F, bufs: &mut [IoSliceMut<'_>]) -> Result<usize>
where
    F: FnOnce(&mut [u8]) -> Result<usize>,
{
    let buf = bufs.iter_mut().find(|b| !b.is_empty()).map_or(&mut [][..], |b| &mut **b);
    read(buf)
}

pub(crate) fn default_write_vectored<F>(write: F, bufs: &[IoSlice<'_>]) -> Result<usize>
where
    F: FnOnce(&[u8]) -> Result<usize>,
{
    let buf = bufs.iter().find(|b| !b.is_empty()).map_or(&[][..], |b| &**b);
    write(buf)
}

pub(crate) fn default_read_exact<R: Read + ?Sized>(this: &mut R, mut buf: &mut [u8]) -> Result<()> {
    while !buf.is_empty() {
        match this.read(buf) {
            Ok(0) => break,
            Ok(n) => {
                let tmp = buf;
                buf = &mut tmp[n..];
            }
            Err(ref e) if e.kind() == ErrorKind::Interrupted => {}
            Err(e) => return Err(e),
        }
    }
    if !buf.is_empty() {
        Err(error::const_io_error!(ErrorKind::UnexpectedEof, "failed to fill whole buffer"))
    } else {
        Ok(())
    }
}

pub(crate) fn default_read_buf<F>(read: F, buf: &mut ReadBuf<'_>) -> Result<()>
where
    F: FnOnce(&mut [u8]) -> Result<usize>,
{
    let n = read(buf.initialize_unfilled())?;
    buf.add_filled(n);
    Ok(())
}

/// The `Read` trait allows for reading bytes from a source. <br>`Read` trait 允许从源读取字节。<br>
///
/// Implementors of the `Read` trait are called 'readers'. <br>`Read` trait 的实现者称为 'readers'。<br>
///
/// Readers are defined by one required method, [`read()`]. <br>Readers 由一种必需的方法 [`read()`] 定义。<br> Each call to [`read()`] will attempt to pull bytes from this source into a provided buffer. <br>对 [`read()`] 的每次调用都会尝试将字节从此源拉入提供的缓冲区。<br>
/// A number of other methods are implemented in terms of [`read()`], giving implementors a number of ways to read bytes while only needing to implement a single method. <br>[`read()`] 还实现了许多其他方法，从而为实现者提供了多种读取字节的方式，而只需要实现一种方法即可。<br>
///
/// Readers are intended to be composable with one another. <br>Readers 旨在彼此组成。<br> Many implementors throughout [`std::io`] take and provide types which implement the `Read` trait. <br>[`std::io`] 上的许多实现器都采用并提供实现 `Read` trait 的类型。<br>
///
/// Please note that each call to [`read()`] may involve a system call, and therefore, using something that implements [`BufRead`], such as [`BufReader`], will be more efficient. <br>请注意，对 [`read()`] 的每次调用都可能涉及一个系统调用，因此，使用实现 [`BufRead`] 的东西 (例如 [`BufReader`]) 会更加有效。<br>
///
///
/// # Examples
///
/// [`File`]s implement `Read`: <br>[`File`] 的工具 `Read`：<br>
///
/// ```no_run
/// use std::io;
/// use std::io::prelude::*;
/// use std::fs::File;
///
/// fn main() -> io::Result<()> {
///     let mut f = File::open("foo.txt")?;
///     let mut buffer = [0; 10];
///
///     // read up to 10 bytes <br>最多读取 10 个字节<br>
///     f.read(&mut buffer)?;
///
///     let mut buffer = Vec::new();
///     // read the whole file <br>读取整个文件<br>
///     f.read_to_end(&mut buffer)?;
///
///     // read into a String, so that you don't need to do the conversion. <br>读入一个字符串，这样就不需要进行转换。<br>
///     let mut buffer = String::new();
///     f.read_to_string(&mut buffer)?;
///
///     // and more! <br>和更多！<br> See the other methods for more details. <br>有关更多详细信息，请参见其他方法。<br>
///     Ok(())
/// }
/// ```
///
/// Read from [`&str`] because [`&[u8]`][prim@slice] implements `Read`: <br>从 [`&str`] 读取，因为 [`&[u8]`][prim@slice] 实现了 `Read`：<br>
///
/// ```no_run
/// # use std::io;
/// use std::io::prelude::*;
///
/// fn main() -> io::Result<()> {
///     let mut b = "This string will be read".as_bytes();
///     let mut buffer = [0; 10];
///
///     // read up to 10 bytes <br>最多读取 10 个字节<br>
///     b.read(&mut buffer)?;
///
///     // etc... it works exactly as a File does! <br>等等... 它的工作原理与文件一样！<br>
///     Ok(())
/// }
/// ```
///
/// [`read()`]: Read::read
/// [`&str`]: prim@str
/// [`std::io`]: self
/// [`File`]: crate::fs::File
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[doc(notable_trait)]
#[cfg_attr(not(test), rustc_diagnostic_item = "IoRead")]
pub trait Read {
    /// Pull some bytes from this source into the specified buffer, returning how many bytes were read. <br>从该源中提取一些字节到指定的缓冲区中，返回读取的字节数。<br>
    ///
    /// This function does not provide any guarantees about whether it blocks waiting for data, but if an object needs to block for a read and cannot, it will typically signal this via an [`Err`] return value. <br>该函数不提供有关是否阻塞等待数据的任何保证，但是如果对象需要阻塞读取而不能阻塞，则通常会通过 [`Err`] 返回值来发出信号。<br>
    ///
    ///
    /// If the return value of this method is [`Ok(n)`], then implementations must guarantee that `0 <= n <= buf.len()`. <br>如果此方法的返回值为 [`Ok(n)`]，则实现必须保证 `0 <= n <= buf.len()`。<br>
    /// A nonzero `n` value indicates that the buffer `buf` has been filled in with `n` bytes of data from this source. <br>`n` 值非零表示缓冲区 `buf` 已填充有来自该源的 `n` 字节的数据。<br>
    /// If `n` is `0`, then it can indicate one of two scenarios: <br>如果 `n` 是 `0`，则它可以指示以下两种情况之一：<br>
    ///
    /// 1. This reader has reached its "end of file" and will likely no longer be able to produce bytes. <br>reader 已到达其 "文件结尾"，可能不再能够产生字节。<br> Note that this does not mean that the reader will *always* no longer be able to produce bytes. <br>请注意，这并不意味着 reader 总是会不再能够生成字节。<br>
    /// As an example, on Linux, this method will call the `recv` syscall for a [`TcpStream`], where returning zero indicates the connection was shut down correctly. <br>例如，在 Linux 上，此方法将调用 [`TcpStream`] 的 `recv` 系统调用，其中返回零表示连接已正确关闭。<br>
    /// While for [`File`], it is possible to reach the end of file and get zero as result, but if more data is appended to the file, future calls to `read` will return more data. <br>而对于 [`File`]，有可能到达文件末尾并得到零作为结果，但如果向文件追加更多数据，那么将来对 `read` 的调用将返回更多数据。<br>
    /// 2. The buffer specified was 0 bytes in length. <br>指定的缓冲区的长度为 0 个字节。<br>
    ///
    /// It is not an error if the returned value `n` is smaller than the buffer size, even when the reader is not at the end of the stream yet. <br>如果返回值 `n` 小于缓冲区大小，即使 reader 不在流的末尾，也不会出错。<br>
    /// This may happen for example because fewer bytes are actually available right now (e. g. being close to end-of-file) or because read() was interrupted by a signal. <br>例如，这可能是因为现在实际可用的字节较少 (例如，接近文件末尾) 或 read() 被信号中断。<br>
    ///
    /// As this trait is safe to implement, callers cannot rely on `n <= buf.len()` for safety. <br>由于此 trait 可以安全实现，因此调用者不能依靠 `n <= buf.len()` 来保证安全。<br>
    /// Extra care needs to be taken when `unsafe` functions are used to access the read bytes. <br>当使用 `unsafe` 函数访问读取的字节时，需要格外小心。<br>
    /// Callers have to ensure that no unchecked out-of-bounds accesses are possible even if `n > buf.len()`. <br>调用者必须确保即使 `n > buf.len()` 也不能进行未经检查的越界访问。<br>
    ///
    /// No guarantees are provided about the contents of `buf` when this function is called, implementations cannot rely on any property of the contents of `buf` being true. <br>调用此函数时，不能保证 `buf` 的内容，实现不能依赖 `buf` 内容的任何属性为 true。<br>
    /// It is recommended that *implementations* only write data to `buf` instead of reading its contents. <br>建议 `implementations` 仅将数据写入 `buf`，而不要读取其内容。<br>
    ///
    /// Correspondingly, however, *callers* of this method must not assume any guarantees about how the implementation uses `buf`. <br>然而，相应地，此方法的*调用者*不得对实现如何使用 `buf` 做出任何保证。<br>
    /// The trait is safe to implement, so it is possible that the code that's supposed to write to the buffer might also read from it. <br>这个 trait 可以安全地实现，所以应该写入缓冲区的代码也有可能从中读取。<br>
    /// It is your responsibility to make sure that `buf` is initialized before calling `read`. <br>您有责任在调用 `read` 之前确保 `buf` 已初始化。<br>
    /// Calling `read` with an uninitialized `buf` (of the kind one obtains via [`MaybeUninit<T>`]) is not safe, and can lead to undefined behavior. <br>用未初始化的 `buf` (通过 [`MaybeUninit<T>`] 获得的那种) 来调用 `read` 是不安全的，并且可能导致未定义的行为。<br>
    ///
    /// [`MaybeUninit<T>`]: crate::mem::MaybeUninit
    ///
    /// # Errors
    ///
    /// If this function encounters any form of I/O or other error, an error variant will be returned. <br>如果此函数遇到任何形式的 I/O 或其他错误，将返回一个错误变体。<br> If an error is returned then it must be guaranteed that no bytes were read. <br>如果返回错误，则必须保证未读取任何字节。<br>
    ///
    /// An error of the [`ErrorKind::Interrupted`] kind is non-fatal and the read operation should be retried if there is nothing else to do. <br>[`ErrorKind::Interrupted`] 类型的错误是非致命错误，如果没有其他事情可做，则应重试读取操作。<br>
    ///
    /// # Examples
    ///
    /// [`File`]s implement `Read`: <br>[`File`] 的工具 `Read`：<br>
    ///
    /// [`Ok(n)`]: Ok
    /// [`File`]: crate::fs::File
    /// [`TcpStream`]: crate::net::TcpStream
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///     let mut buffer = [0; 10];
    ///
    ///     // read up to 10 bytes <br>最多读取 10 个字节<br>
    ///     let n = f.read(&mut buffer[..])?;
    ///
    ///     println!("The bytes: {:?}", &buffer[..n]);
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn read(&mut self, buf: &mut [u8]) -> Result<usize>;

    /// Like `read`, except that it reads into a slice of buffers. <br>与 `read` 相似，不同之处在于它读入缓冲区的一部分。<br>
    ///
    /// Data is copied to fill each buffer in order, with the final buffer written to possibly being only partially filled. <br>复制数据以按顺序填充每个缓冲区，而写入的最终缓冲区可能仅被部分填充。<br>
    /// This method must behave equivalently to a single call to `read` with concatenated buffers. <br>此方法必须等效于使用级联缓冲区对 `read` 的单个调用。<br>
    ///
    /// The default implementation calls `read` with either the first nonempty buffer provided, or an empty one if none exists. <br>默认实现使用提供的第一个非空缓冲区调用 `read`，如果不存在，则为空。<br>
    ///
    ///
    ///
    #[stable(feature = "iovec", since = "1.36.0")]
    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> Result<usize> {
        default_read_vectored(|b| self.read(b), bufs)
    }

    /// Determines if this `Read`er has an efficient `read_vectored` implementation. <br>确定此 `Read` 是否具有有效的 `read_vectored` 实现。<br>
    ///
    /// If a `Read`er does not override the default `read_vectored` implementation, code using it may want to avoid the method all together and coalesce writes into a single buffer for higher performance. <br>如果 `Read` 没有覆盖默认的 `read_vectored` 实现，则使用它的代码可能希望完全避免使用该方法，并合并写入单个缓冲区以提高性能。<br>
    ///
    ///
    /// The default implementation returns `false`. <br>默认实现返回 `false`。<br>
    ///
    ///
    #[unstable(feature = "can_vector", issue = "69941")]
    fn is_read_vectored(&self) -> bool {
        false
    }

    /// Read all bytes until EOF in this source, placing them into `buf`. <br>读取所有字节，直到此源中的 EOF 为止，然后将它们放入 `buf`。<br>
    ///
    /// All bytes read from this source will be appended to the specified buffer `buf`. <br>从该源读取的所有字节都将追加到指定的缓冲区 `buf`。<br>
    /// This function will continuously call [`read()`] to append more data to `buf` until [`read()`] returns either [`Ok(0)`] or an error of non-[`ErrorKind::Interrupted`] kind. <br>该函数将不断调用 [`read()`] 来向 `buf` 追加更多的数据，直到 [`read()`] 返回 [`Ok(0)`] 或非 [`ErrorKind::Interrupted`] 类型的错误为止。<br>
    ///
    ///
    /// If successful, this function will return the total number of bytes read. <br>如果成功，此函数将返回读取的字节总数。<br>
    ///
    /// # Errors
    ///
    /// If this function encounters an error of the kind [`ErrorKind::Interrupted`] then the error is ignored and the operation will continue. <br>如果此函数遇到 [`ErrorKind::Interrupted`] 类型的错误，则该错误将被忽略，并且操作将继续。<br>
    ///
    /// If any other read error is encountered then this function immediately returns. <br>如果遇到任何其他读取错误，则此函数立即返回。<br> Any bytes which have already been read will be appended to `buf`. <br>任何已经读取的字节都将被追加到 `buf`。<br>
    ///
    /// # Examples
    ///
    /// [`File`]s implement `Read`: <br>[`File`] 的工具 `Read`：<br>
    ///
    /// [`read()`]: Read::read
    /// [`Ok(0)`]: Ok
    /// [`File`]: crate::fs::File
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///     let mut buffer = Vec::new();
    ///
    ///     // read the whole file <br>读取整个文件<br>
    ///     f.read_to_end(&mut buffer)?;
    ///     Ok(())
    /// }
    /// ```
    ///
    /// (See also the [`std::fs::read`] convenience function for reading from a file.) <br>(有关读取文件的信息，另请参见 [`std::fs::read`] 便利函数。)<br>
    ///
    /// [`std::fs::read`]: crate::fs::read
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> Result<usize> {
        default_read_to_end(self, buf)
    }

    /// Read all bytes until EOF in this source, appending them to `buf`. <br>读取这个源中的所有字节，直到 EOF 为止，然后将它们追加到 `buf`。<br>
    ///
    /// If successful, this function returns the number of bytes which were read and appended to `buf`. <br>如果成功，则此函数返回已读取并追加到 `buf` 的字节数。<br>
    ///
    /// # Errors
    ///
    /// If the data in this stream is *not* valid UTF-8 then an error is returned and `buf` is unchanged. <br>如果此流中的数据 *不是* 有效的 UTF-8，则返回错误，并且 `buf` 不变。<br>
    ///
    ///
    /// See [`read_to_end`] for other error semantics. <br>有关其他错误语义，请参见 [`read_to_end`]。<br>
    ///
    /// [`read_to_end`]: Read::read_to_end
    ///
    /// # Examples
    ///
    /// [`File`]s implement `Read`: <br>[`File`] 的工具 `Read`：<br>
    ///
    /// [`File`]: crate::fs::File
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///     let mut buffer = String::new();
    ///
    ///     f.read_to_string(&mut buffer)?;
    ///     Ok(())
    /// }
    /// ```
    ///
    /// (See also the [`std::fs::read_to_string`] convenience function for reading from a file.) <br>(有关读取文件的信息，另请参见 [`std::fs::read_to_string`] 便利函数。)<br>
    ///
    /// [`std::fs::read_to_string`]: crate::fs::read_to_string
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn read_to_string(&mut self, buf: &mut String) -> Result<usize> {
        default_read_to_string(self, buf)
    }

    /// Read the exact number of bytes required to fill `buf`. <br>读取填充 `buf` 所需的确切字节数。<br>
    ///
    /// This function reads as many bytes as necessary to completely fill the specified buffer `buf`. <br>该函数读取所需的字节数以完全填充指定的缓冲区 `buf`。<br>
    ///
    /// No guarantees are provided about the contents of `buf` when this function is called, implementations cannot rely on any property of the contents of `buf` being true. <br>调用此函数时，不能保证 `buf` 的内容，实现不能依赖 `buf` 内容的任何属性为 true。<br>
    /// It is recommended that implementations only write data to `buf` instead of reading its contents. <br>建议实现仅将数据写入 `buf`，而不读取其内容。<br>
    /// The documentation on [`read`] has a more detailed explanation on this subject. <br>[`read`] 上的文档对此主题有更详细的说明。<br>
    ///
    /// # Errors
    ///
    /// If this function encounters an error of the kind [`ErrorKind::Interrupted`] then the error is ignored and the operation will continue. <br>如果此函数遇到 [`ErrorKind::Interrupted`] 类型的错误，则该错误将被忽略，并且操作将继续。<br>
    ///
    /// If this function encounters an "end of file" before completely filling the buffer, it returns an error of the kind [`ErrorKind::UnexpectedEof`]. <br>如果此函数在完全填充缓冲区之前遇到 "文件结尾"，它将返回 [`ErrorKind::UnexpectedEof`] 类型的错误。<br>
    /// The contents of `buf` are unspecified in this case. <br>在这种情况下，`buf` 的内容未指定。<br>
    ///
    /// If any other read error is encountered then this function immediately returns. <br>如果遇到任何其他读取错误，则此函数立即返回。<br> The contents of `buf` are unspecified in this case. <br>在这种情况下，`buf` 的内容未指定。<br>
    ///
    /// If this function returns an error, it is unspecified how many bytes it has read, but it will never read more than would be necessary to completely fill the buffer. <br>如果此函数返回错误，则无法确定已读取多少字节，但读取的字节数永远不会超过完全填充缓冲区所需的字节数。<br>
    ///
    /// # Examples
    ///
    /// [`File`]s implement `Read`: <br>[`File`] 的工具 `Read`：<br>
    ///
    /// [`read`]: Read::read
    /// [`File`]: crate::fs::File
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///     let mut buffer = [0; 10];
    ///
    ///     // read exactly 10 bytes <br>精确读取 10 个字节<br>
    ///     f.read_exact(&mut buffer)?;
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "read_exact", since = "1.6.0")]
    fn read_exact(&mut self, buf: &mut [u8]) -> Result<()> {
        default_read_exact(self, buf)
    }

    /// Pull some bytes from this source into the specified buffer. <br>从此源中提取一些字节到指定的缓冲区中。<br>
    ///
    /// This is equivalent to the [`read`](Read::read) method, except that it is passed a [`ReadBuf`] rather than `[u8]` to allow use with uninitialized buffers. <br>这等效于 [`read`](Read::read) 方法，除了它传递的是 [`ReadBuf`] 而不是 `[u8]` 以允许使用未初始化的缓冲区。<br>
    /// The new data will be appended to any existing contents of `buf`. <br>新的数据将被追加到 `buf` 的任何现有内容中。<br>
    ///
    /// The default implementation delegates to `read`. <br>默认实现委托给 `read`。<br>
    #[unstable(feature = "read_buf", issue = "78485")]
    fn read_buf(&mut self, buf: &mut ReadBuf<'_>) -> Result<()> {
        default_read_buf(|b| self.read(b), buf)
    }

    /// Read the exact number of bytes required to fill `buf`. <br>读取填充 `buf` 所需的确切字节数。<br>
    ///
    /// This is equivalent to the [`read_exact`](Read::read_exact) method, except that it is passed a [`ReadBuf`] rather than `[u8]` to allow use with uninitialized buffers. <br>这等效于 [`read_exact`](Read::read_exact) 方法，除了它传递的是 [`ReadBuf`] 而不是 `[u8]` 以允许使用未初始化的缓冲区。<br>
    ///
    #[unstable(feature = "read_buf", issue = "78485")]
    fn read_buf_exact(&mut self, buf: &mut ReadBuf<'_>) -> Result<()> {
        while buf.remaining() > 0 {
            let prev_filled = buf.filled().len();
            match self.read_buf(buf) {
                Ok(()) => {}
                Err(e) if e.kind() == ErrorKind::Interrupted => continue,
                Err(e) => return Err(e),
            }

            if buf.filled().len() == prev_filled {
                return Err(Error::new(ErrorKind::UnexpectedEof, "failed to fill buffer"));
            }
        }

        Ok(())
    }

    /// Creates a "by reference" adaptor for this instance of `Read`. <br>为这个 `Read` 实例创建一个 "by reference" 适配器。<br>
    ///
    /// The returned adapter also implements `Read` and will simply borrow this current reader. <br>返回的适配器也实现了 `Read`，并且将简单地借用当前的 reader。<br>
    ///
    ///
    /// # Examples
    ///
    /// [`File`]s implement `Read`: <br>[`File`] 的工具 `Read`：<br>
    ///
    /// [`File`]: crate::fs::File
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::Read;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///     let mut buffer = Vec::new();
    ///     let mut other_buffer = Vec::new();
    ///
    ///     {
    ///         let reference = f.by_ref();
    ///
    ///         // read at most 5 bytes <br>最多读取 5 个字节<br>
    ///         reference.take(5).read_to_end(&mut buffer)?;
    ///
    ///     } // drop our &mut reference so we can use f again <br>丢弃 &mut 引用，以便我们可以再次使用 f<br>
    ///
    ///     // original file still usable, read the rest <br>原始文件仍然可用，请读取其余内容<br>
    ///     f.read_to_end(&mut other_buffer)?;
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    fn by_ref(&mut self) -> &mut Self
    where
        Self: Sized,
    {
        self
    }

    /// Transforms this `Read` instance to an [`Iterator`] over its bytes. <br>将此 `Read` 实例的字节数转换为 [`Iterator`]。<br>
    ///
    /// The returned type implements [`Iterator`] where the [`Item`] is <code>[Result]<[u8], [io::Error]></code>. <br>返回的类型实现 [`Iterator`]，其中 [`Item`] 是 <code>[Result]<[u8], [io::Error]></code>。<br>
    ///
    /// The yielded item is [`Ok`] if a byte was successfully read and [`Err`] otherwise. <br>如果成功读取了一个字节，则产生的项为 [`Ok`]，否则为 [`Err`]。<br>
    /// EOF is mapped to returning [`None`] from this iterator. <br>EOF 映射为从此迭代器返回 [`None`]。<br>
    ///
    /// # Examples
    ///
    /// [`File`]s implement `Read`: <br>[`File`] 的工具 `Read`：<br>
    ///
    /// [`Item`]: Iterator::Item
    /// [`File`]: crate::fs::File "fs::File"
    /// [Result]: crate::result::Result "Result"
    /// [io::Error]: self::Error "io::Error"
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///
    ///     for byte in f.bytes() {
    ///         println!("{}", byte.unwrap());
    ///     }
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    fn bytes(self) -> Bytes<Self>
    where
        Self: Sized,
    {
        Bytes { inner: self }
    }

    /// Creates an adapter which will chain this stream with another. <br>创建一个适配器，将这个流与另一个链接起来。<br>
    ///
    /// The returned `Read` instance will first read all bytes from this object until EOF is encountered. <br>返回的 `Read` 实例将首先从该对象读取所有字节，直到遇到 EOF。<br>
    /// Afterwards the output is equivalent to the output of `next`. <br>之后，输出等同于 `next` 的输出。<br>
    ///
    /// # Examples
    ///
    /// [`File`]s implement `Read`: <br>[`File`] 的工具 `Read`：<br>
    ///
    /// [`File`]: crate::fs::File
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f1 = File::open("foo.txt")?;
    ///     let mut f2 = File::open("bar.txt")?;
    ///
    ///     let mut handle = f1.chain(f2);
    ///     let mut buffer = String::new();
    ///
    ///     // read the value into a String. <br>将值读入字符串。<br>
    ///     // We could use any Read method here, this is just one example. <br>我们可以在这里使用任何 Read 方法，这只是一个示例。<br>
    ///     handle.read_to_string(&mut buffer)?;
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn chain<R: Read>(self, next: R) -> Chain<Self, R>
    where
        Self: Sized,
    {
        Chain { first: self, second: next, done_first: false }
    }

    /// Creates an adapter which will read at most `limit` bytes from it. <br>创建一个适配器，最多从中读取 `limit` 个字节。<br>
    ///
    /// This function returns a new instance of `Read` which will read at most `limit` bytes, after which it will always return EOF ([`Ok(0)`]). <br>此函数返回 `Read` 的新实例，该实例最多读取 `limit` 字节，此后它将始终返回 EOF ([`Ok(0)`])。<br>
    /// Any read errors will not count towards the number of bytes read and future calls to [`read()`] may succeed. <br>任何读取错误都不会计入读取的字节数，并且 [`read()`] 的 future 调用可能会成功。<br>
    ///
    /// # Examples
    ///
    /// [`File`]s implement `Read`: <br>[`File`] 的工具 `Read`：<br>
    ///
    /// [`File`]: crate::fs::File
    /// [`Ok(0)`]: Ok
    /// [`read()`]: Read::read
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///     let mut buffer = [0; 5];
    ///
    ///     // read at most five bytes <br>最多读取五个字节<br>
    ///     let mut handle = f.take(5);
    ///
    ///     handle.read(&mut buffer)?;
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn take(self, limit: u64) -> Take<Self>
    where
        Self: Sized,
    {
        Take { inner: self, limit }
    }
}

/// Read all bytes from a [reader][Read] into a new [`String`]. <br>将 [读取][Read] 的所有字节读入新的 [`String`]。<br>
///
/// This is a convenience function for [`Read::read_to_string`]. <br>这是 [`Read::read_to_string`] 的便捷函数。<br>
/// Using this function avoids having to create a variable first and provides more type safety since you can only get the buffer out if there were no errors. <br>使用此函数避免了必须先创建变量，并且提供了更多的类型安全性，因为只有在没有错误的情况下才可以取出缓冲区。<br>
/// (If you use [`Read::read_to_string`] you have to remember to check whether the read succeeded because otherwise your buffer will be empty or only partially full.) <br>(如果使用 [`Read::read_to_string`]，则必须记住检查读取是否成功，否则缓冲区将为空或仅部分充满。)<br>
///
///
/// # Performance
///
/// The downside of this function's increased ease of use and type safety is that it gives you less control over performance. <br>该函数提高了易用性和类型安全性的缺点是，它使您对性能的控制较少。<br>
/// For example, you can't pre-allocate memory like you can using [`String::with_capacity`] and [`Read::read_to_string`]. <br>例如，您不能像使用 [`String::with_capacity`] 和 [`Read::read_to_string`] 一样预先分配内存。<br>
/// Also, you can't re-use the buffer if an error occurs while reading. <br>另外，如果读取时发生错误，则无法重新使用缓冲区。<br>
///
/// In many cases, this function's performance will be adequate and the ease of use and type safety tradeoffs will be worth it. <br>在许多情况下，此函数的性能将是适当的，并且易于使用和类型安全性的折衷是值得的。<br>
/// However, there are cases where you need more control over performance, and in those cases you should definitely use [`Read::read_to_string`] directly. <br>但是，在某些情况下，您需要对性能进行更多控制，在这些情况下，您绝对应该直接使用 [`Read::read_to_string`]。<br>
///
/// Note that in some special cases, such as when reading files, this function will pre-allocate memory based on the size of the input it is reading. <br>请注意，在某些特殊情况下，例如读取文件时，该函数会根据正在读取的输入的大小预先分配内存。<br>
/// In those cases, the performance should be as good as if you had used [`Read::read_to_string`] with a manually pre-allocated buffer. <br>在这些情况下，性能应该与您使用手动预分配缓冲区的 [`Read::read_to_string`] 一样好。<br>
///
/// # Errors
///
/// This function forces you to handle errors because the output (the `String`) is wrapped in a [`Result`]. <br>此函数迫使您处理错误，因为输出 (`String`) 包装在 [`Result`] 中。<br>
/// See [`Read::read_to_string`] for the errors that can occur. <br>有关可能发生的错误，请参见 [`Read::read_to_string`]。<br>
/// If any error occurs, you will get an [`Err`], so you don't have to worry about your buffer being empty or partially full. <br>如果发生任何错误，您将得到 [`Err`]，因此您不必担心缓冲区为空或部分已满。<br>
///
/// # Examples
///
/// ```no_run
/// #![feature(io_read_to_string)]
///
/// # use std::io;
/// fn main() -> io::Result<()> {
///     let stdin = io::read_to_string(io::stdin())?;
///     println!("Stdin was:");
///     println!("{stdin}");
///     Ok(())
/// }
/// ```
///
///
///
///
///
///
///
///
#[unstable(feature = "io_read_to_string", issue = "80218")]
pub fn read_to_string<R: Read>(mut reader: R) -> Result<String> {
    let mut buf = String::new();
    reader.read_to_string(&mut buf)?;
    Ok(buf)
}

/// A buffer type used with `Read::read_vectored`. <br>`Read::read_vectored` 使用的缓冲区类型。<br>
///
/// It is semantically a wrapper around an `&mut [u8]`, but is guaranteed to be ABI compatible with the `iovec` type on Unix platforms and `WSABUF` on Windows. <br>从语义上来说，它是 `&mut [u8]` 的包装，但是可以保证与 Unix 平台上的 `iovec` 类型和 Windows 上的 `WSABUF` 具有 ABI 兼容性。<br>
///
///
#[stable(feature = "iovec", since = "1.36.0")]
#[repr(transparent)]
pub struct IoSliceMut<'a>(sys::io::IoSliceMut<'a>);

#[stable(feature = "iovec-send-sync", since = "1.44.0")]
unsafe impl<'a> Send for IoSliceMut<'a> {}

#[stable(feature = "iovec-send-sync", since = "1.44.0")]
unsafe impl<'a> Sync for IoSliceMut<'a> {}

#[stable(feature = "iovec", since = "1.36.0")]
impl<'a> fmt::Debug for IoSliceMut<'a> {
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Debug::fmt(self.0.as_slice(), fmt)
    }
}

impl<'a> IoSliceMut<'a> {
    /// Creates a new `IoSliceMut` wrapping a byte slice. <br>创建一个新的 `IoSliceMut`，包装一个字节切片。<br>
    ///
    /// # Panics
    ///
    /// Panics on Windows if the slice is larger than 4GB. <br>如果切片大于 4GB，则在 Windows 上使用 panics。<br>
    #[stable(feature = "iovec", since = "1.36.0")]
    #[inline]
    pub fn new(buf: &'a mut [u8]) -> IoSliceMut<'a> {
        IoSliceMut(sys::io::IoSliceMut::new(buf))
    }

    /// Advance the internal cursor of the slice. <br>前进切片的内部游标。<br>
    ///
    /// Also see [`IoSliceMut::advance_slices`] to advance the cursors of multiple buffers. <br>另请参见 [`IoSliceMut::advance_slices`] 以推进多个缓冲区的游标。<br>
    ///
    ///
    /// # Panics
    ///
    /// Panics when trying to advance beyond the end of the slice. <br>试图超越切片末端时出现 panic。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(io_slice_advance)]
    ///
    /// use std::io::IoSliceMut;
    /// use std::ops::Deref;
    ///
    /// let mut data = [1; 8];
    /// let mut buf = IoSliceMut::new(&mut data);
    ///
    /// // Mark 3 bytes as read. <br>将 3 个字节标记为已读。<br>
    /// buf.advance(3);
    /// assert_eq!(buf.deref(), [1; 5].as_ref());
    /// ```
    #[unstable(feature = "io_slice_advance", issue = "62726")]
    #[inline]
    pub fn advance(&mut self, n: usize) {
        self.0.advance(n)
    }

    /// Advance a slice of slices. <br>在 slices 中推进一个切片。<br>
    ///
    /// Shrinks the slice to remove any `IoSliceMut`s that are fully advanced over. <br>缩小切片以删除任何完全超前的 `IoSliceMut`。<br>
    /// If the cursor ends up in the middle of an `IoSliceMut`, it is modified to start at that cursor. <br>如果游标最终位于 `IoSliceMut` 的中间，则将其修改为从该游标处开始。<br>
    ///
    /// For example, if we have a slice of two 8-byte `IoSliceMut`s, and we advance by 10 bytes, the result will only include the second `IoSliceMut`, advanced by 2 bytes. <br>例如，如果我们有两个 8 字节的 `IoSliceMut` 的切片，并且我们提前 10 个字节，结果将只包括第二个 `IoSliceMut`，提前 2 个字节。<br>
    ///
    ///
    /// # Panics
    ///
    /// Panics when trying to advance beyond the end of the slices. <br>试图超越切片末端时出现 panic。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(io_slice_advance)]
    ///
    /// use std::io::IoSliceMut;
    /// use std::ops::Deref;
    ///
    /// let mut buf1 = [1; 8];
    /// let mut buf2 = [2; 16];
    /// let mut buf3 = [3; 8];
    /// let mut bufs = &mut [
    ///     IoSliceMut::new(&mut buf1),
    ///     IoSliceMut::new(&mut buf2),
    ///     IoSliceMut::new(&mut buf3),
    /// ][..];
    ///
    /// // Mark 10 bytes as read. <br>将 10 个字节标记为已读。<br>
    /// IoSliceMut::advance_slices(&mut bufs, 10);
    /// assert_eq!(bufs[0].deref(), [2; 14].as_ref());
    /// assert_eq!(bufs[1].deref(), [3; 8].as_ref());
    /// ```
    ///
    #[unstable(feature = "io_slice_advance", issue = "62726")]
    #[inline]
    pub fn advance_slices(bufs: &mut &mut [IoSliceMut<'a>], n: usize) {
        // Number of buffers to remove. <br>要删除的缓冲区数。<br>
        let mut remove = 0;
        // Total length of all the to be removed buffers. <br>所有要删除的缓冲区的总长度。<br>
        let mut accumulated_len = 0;
        for buf in bufs.iter() {
            if accumulated_len + buf.len() > n {
                break;
            } else {
                accumulated_len += buf.len();
                remove += 1;
            }
        }

        *bufs = &mut replace(bufs, &mut [])[remove..];
        if bufs.is_empty() {
            assert!(n == accumulated_len, "advancing io slices beyond their length");
        } else {
            bufs[0].advance(n - accumulated_len)
        }
    }
}

#[stable(feature = "iovec", since = "1.36.0")]
impl<'a> Deref for IoSliceMut<'a> {
    type Target = [u8];

    #[inline]
    fn deref(&self) -> &[u8] {
        self.0.as_slice()
    }
}

#[stable(feature = "iovec", since = "1.36.0")]
impl<'a> DerefMut for IoSliceMut<'a> {
    #[inline]
    fn deref_mut(&mut self) -> &mut [u8] {
        self.0.as_mut_slice()
    }
}

/// A buffer type used with `Write::write_vectored`. <br>`Write::write_vectored` 使用的缓冲区类型。<br>
///
/// It is semantically a wrapper around a `&[u8]`, but is guaranteed to be ABI compatible with the `iovec` type on Unix platforms and `WSABUF` on Windows. <br>它在语义上是 `&[u8]` 的包装器，但保证与 Unix 平台上的 `iovec` 类型和 Windows 上的 `WSABUF` 类型的 ABI 兼容。<br>
///
///
#[stable(feature = "iovec", since = "1.36.0")]
#[derive(Copy, Clone)]
#[repr(transparent)]
pub struct IoSlice<'a>(sys::io::IoSlice<'a>);

#[stable(feature = "iovec-send-sync", since = "1.44.0")]
unsafe impl<'a> Send for IoSlice<'a> {}

#[stable(feature = "iovec-send-sync", since = "1.44.0")]
unsafe impl<'a> Sync for IoSlice<'a> {}

#[stable(feature = "iovec", since = "1.36.0")]
impl<'a> fmt::Debug for IoSlice<'a> {
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Debug::fmt(self.0.as_slice(), fmt)
    }
}

impl<'a> IoSlice<'a> {
    /// Creates a new `IoSlice` wrapping a byte slice. <br>创建一个新的 `IoSlice`，包装一个字节切片。<br>
    ///
    /// # Panics
    ///
    /// Panics on Windows if the slice is larger than 4GB. <br>如果切片大于 4GB，则在 Windows 上使用 panics。<br>
    #[stable(feature = "iovec", since = "1.36.0")]
    #[must_use]
    #[inline]
    pub fn new(buf: &'a [u8]) -> IoSlice<'a> {
        IoSlice(sys::io::IoSlice::new(buf))
    }

    /// Advance the internal cursor of the slice. <br>前进切片的内部游标。<br>
    ///
    /// Also see [`IoSlice::advance_slices`] to advance the cursors of multiple buffers. <br>另请参见 [`IoSlice::advance_slices`] 以推进多个缓冲区的游标。<br>
    ///
    ///
    /// # Panics
    ///
    /// Panics when trying to advance beyond the end of the slice. <br>试图超越切片末端时出现 panic。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(io_slice_advance)]
    ///
    /// use std::io::IoSlice;
    /// use std::ops::Deref;
    ///
    /// let data = [1; 8];
    /// let mut buf = IoSlice::new(&data);
    ///
    /// // Mark 3 bytes as read. <br>将 3 个字节标记为已读。<br>
    /// buf.advance(3);
    /// assert_eq!(buf.deref(), [1; 5].as_ref());
    /// ```
    #[unstable(feature = "io_slice_advance", issue = "62726")]
    #[inline]
    pub fn advance(&mut self, n: usize) {
        self.0.advance(n)
    }

    /// Advance a slice of slices. <br>在 slices 中推进一个切片。<br>
    ///
    /// Shrinks the slice to remove any `IoSlice`s that are fully advanced over. <br>缩小切片以删除任何完全超前的 `IoSlice`。<br>
    /// If the cursor ends up in the middle of an `IoSlice`, it is modified to start at that cursor. <br>如果游标最终位于 `IoSlice` 的中间，则将其修改为从该游标处开始。<br>
    ///
    /// For example, if we have a slice of two 8-byte `IoSlice`s, and we advance by 10 bytes, the result will only include the second `IoSlice`, advanced by 2 bytes. <br>例如，如果我们有两个 8 字节的 `IoSlice` 的切片，并且我们提前 10 个字节，结果将只包括第二个 `IoSlice`，提前 2 个字节。<br>
    ///
    ///
    /// # Panics
    ///
    /// Panics when trying to advance beyond the end of the slices. <br>试图超越切片末端时出现 panic。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(io_slice_advance)]
    ///
    /// use std::io::IoSlice;
    /// use std::ops::Deref;
    ///
    /// let buf1 = [1; 8];
    /// let buf2 = [2; 16];
    /// let buf3 = [3; 8];
    /// let mut bufs = &mut [
    ///     IoSlice::new(&buf1),
    ///     IoSlice::new(&buf2),
    ///     IoSlice::new(&buf3),
    /// ][..];
    ///
    /// // Mark 10 bytes as written. <br>将 10 个字节标记为已写入。<br>
    /// IoSlice::advance_slices(&mut bufs, 10);
    /// assert_eq!(bufs[0].deref(), [2; 14].as_ref());
    /// assert_eq!(bufs[1].deref(), [3; 8].as_ref());
    ///
    #[unstable(feature = "io_slice_advance", issue = "62726")]
    #[inline]
    pub fn advance_slices(bufs: &mut &mut [IoSlice<'a>], n: usize) {
        // Number of buffers to remove. <br>要删除的缓冲区数。<br>
        let mut remove = 0;
        // Total length of all the to be removed buffers. <br>所有要删除的缓冲区的总长度。<br>
        let mut accumulated_len = 0;
        for buf in bufs.iter() {
            if accumulated_len + buf.len() > n {
                break;
            } else {
                accumulated_len += buf.len();
                remove += 1;
            }
        }

        *bufs = &mut replace(bufs, &mut [])[remove..];
        if bufs.is_empty() {
            assert!(n == accumulated_len, "advancing io slices beyond their length");
        } else {
            bufs[0].advance(n - accumulated_len)
        }
    }
}

#[stable(feature = "iovec", since = "1.36.0")]
impl<'a> Deref for IoSlice<'a> {
    type Target = [u8];

    #[inline]
    fn deref(&self) -> &[u8] {
        self.0.as_slice()
    }
}

/// A trait for objects which are byte-oriented sinks. <br>面向字节的接收器对象的 trait。<br>
///
/// Implementors of the `Write` trait are sometimes called 'writers'. <br>`Write` trait 的实现者有时称为 'writers'。<br>
///
/// Writers are defined by two required methods, [`write`] and [`flush`]: <br>Writers 由两种必需的方法 [`write`] 和 [`flush`] 定义：<br>
///
/// * The [`write`] method will attempt to write some data into the object, returning how many bytes were successfully written. <br>[`write`] 方法将尝试将一些数据写入对象，返回成功写入的字节数。<br>
///
/// * The [`flush`] method is useful for adapters and explicit buffers themselves for ensuring that all buffered data has been pushed out to the 'true sink'. <br>[`flush`] 方法对于适配器和显式缓冲区本身很有用，以确保所有缓冲数据都已被推送到 `真正的 sink`。<br>
///
///
/// Writers are intended to be composable with one another. <br>Writers 旨在彼此组成。<br>
/// Many implementors throughout [`std::io`] take and provide types which implement the `Write` trait. <br>[`std::io`] 上的许多实现器都采用并提供实现 `Write` trait 的类型。<br>
///
/// [`write`]: Write::write
/// [`flush`]: Write::flush
/// [`std::io`]: self
///
/// # Examples
///
/// ```no_run
/// use std::io::prelude::*;
/// use std::fs::File;
///
/// fn main() -> std::io::Result<()> {
///     let data = b"some bytes";
///
///     let mut pos = 0;
///     let mut buffer = File::create("foo.txt")?;
///
///     while pos < data.len() {
///         let bytes_written = buffer.write(&data[pos..])?;
///         pos += bytes_written;
///     }
///     Ok(())
/// }
/// ```
///
/// The trait also provides convenience methods like [`write_all`], which calls `write` in a loop until its entire input has been written. <br>这个 trait 还提供了便捷的方法，例如 [`write_all`]，它在循环中调用 `write`，直到其整个输入都被写入为止。<br>
///
/// [`write_all`]: Write::write_all
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[doc(notable_trait)]
#[cfg_attr(not(test), rustc_diagnostic_item = "IoWrite")]
pub trait Write {
    /// Write a buffer into this writer, returning how many bytes were written. <br>在此 writer 中写入一个缓冲区，返回写入的字节数。<br>
    ///
    /// This function will attempt to write the entire contents of `buf`, but the entire write might not succeed, or the write may also generate an error. <br>这个函数会尝试写入 `buf` 的全部内容，但是整个写入可能不会成功，或者写入也会产生错误。<br>
    /// A call to `write` represents *at most one* attempt to write to any wrapped object. <br>对 `write` 的调用表示 *at most 对任何包装的对象进行写操作的尝试。<br>
    ///
    /// Calls to `write` are not guaranteed to block waiting for data to be written, and a write which would otherwise block can be indicated through an [`Err`] variant. <br>不能保证对 `write` 的调用会阻塞等待数据的写入，否则可以通过 [`Err`] 变体指示被阻塞的写入。<br>
    ///
    /// If the return value is [`Ok(n)`] then it must be guaranteed that `n <= buf.len()`. <br>如果返回值为 [`Ok(n)`]，则必须保证 `n <= buf.len()`。<br>
    /// A return value of `0` typically means that the underlying object is no longer able to accept bytes and will likely not be able to in the future as well, or that the buffer provided is empty. <br>返回值 `0` 通常意味着底层对象不再能够接受字节，并且将来也可能无法接受，或者提供的缓冲区为空。<br>
    ///
    ///
    /// # Errors
    ///
    /// Each call to `write` may generate an I/O error indicating that the operation could not be completed. <br>对 `write` 的每次调用都可能产生 I/O 错误，指示操作无法完成。<br> If an error is returned then no bytes in the buffer were written to this writer. <br>如果返回错误，则缓冲区中没有字节写入此 writer。<br>
    ///
    /// It is **not** considered an error if the entire buffer could not be written to this writer. <br>如果无法将整个缓冲区写入此 writer，则不认为是错误。<br>
    ///
    /// An error of the [`ErrorKind::Interrupted`] kind is non-fatal and the write operation should be retried if there is nothing else to do. <br>[`ErrorKind::Interrupted`] 类型的错误是非致命错误，如果没有其他事情可做，则应重试写入操作。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut buffer = File::create("foo.txt")?;
    ///
    ///     // Writes some prefix of the byte string, not necessarily all of it. <br>写入字节字符串的某些前缀，不一定要全部。<br>
    ///     buffer.write(b"some bytes")?;
    ///     Ok(())
    /// }
    /// ```
    ///
    /// [`Ok(n)`]: Ok
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn write(&mut self, buf: &[u8]) -> Result<usize>;

    /// Like [`write`], except that it writes from a slice of buffers. <br>类似于 [`write`]，不同之处在于它是从缓冲区切片中写入数据的。<br>
    ///
    /// Data is copied from each buffer in order, with the final buffer read from possibly being only partially consumed. <br>数据是按顺序从每个缓冲区复制的，从中读取的最终缓冲区可能仅被部分消耗。<br>
    /// This method must behave as a call to [`write`] with the buffers concatenated would. <br>此方法必须与串联的缓冲区对 [`write`] 的调用相同。<br>
    ///
    /// The default implementation calls [`write`] with either the first nonempty buffer provided, or an empty one if none exists. <br>默认实现使用提供的第一个非空缓冲区调用 [`write`]，如果不存在，则为空。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::IoSlice;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let data1 = [1; 8];
    ///     let data2 = [15; 8];
    ///     let io_slice1 = IoSlice::new(&data1);
    ///     let io_slice2 = IoSlice::new(&data2);
    ///
    ///     let mut buffer = File::create("foo.txt")?;
    ///
    ///     // Writes some prefix of the byte string, not necessarily all of it. <br>写入字节字符串的某些前缀，不一定要全部。<br>
    ///     buffer.write_vectored(&[io_slice1, io_slice2])?;
    ///     Ok(())
    /// }
    /// ```
    ///
    /// [`write`]: Write::write
    ///
    #[stable(feature = "iovec", since = "1.36.0")]
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> Result<usize> {
        default_write_vectored(|b| self.write(b), bufs)
    }

    /// Determines if this `Write`r has an efficient [`write_vectored`] implementation. <br>确定此 `Write`r 是否具有有效的 [`write_vectored`] 实现。<br>
    ///
    /// If a `Write`r does not override the default [`write_vectored`] implementation, code using it may want to avoid the method all together and coalesce writes into a single buffer for higher performance. <br>如果 `Write`r 没有覆盖默认的 [`write_vectored`] 实现，则使用它的代码可能希望完全避免使用该方法，并合并写入单个缓冲区以提高性能。<br>
    ///
    ///
    /// The default implementation returns `false`. <br>默认实现返回 `false`。<br>
    ///
    /// [`write_vectored`]: Write::write_vectored
    ///
    ///
    #[unstable(feature = "can_vector", issue = "69941")]
    fn is_write_vectored(&self) -> bool {
        false
    }

    /// Flush this output stream, ensuring that all intermediately buffered contents reach their destination. <br>刷新此输出流，确保所有中间缓冲的内容均到达其目的地。<br>
    ///
    ///
    /// # Errors
    ///
    /// It is considered an error if not all bytes could be written due to I/O errors or EOF being reached. <br>如果由于 I/O 错误或达到 EOF 而无法写入所有字节，则认为是错误。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::prelude::*;
    /// use std::io::BufWriter;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut buffer = BufWriter::new(File::create("foo.txt")?);
    ///
    ///     buffer.write_all(b"some bytes")?;
    ///     buffer.flush()?;
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn flush(&mut self) -> Result<()>;

    /// Attempts to write an entire buffer into this writer. <br>尝试将整个缓冲区写入此 writer。<br>
    ///
    /// This method will continuously call [`write`] until there is no more data to be written or an error of non-[`ErrorKind::Interrupted`] kind is returned. <br>此方法将连续调用 [`write`]，直到没有更多数据要写入或返回非 [`ErrorKind::Interrupted`] 类型的错误为止。<br>
    /// This method will not return until the entire buffer has been successfully written or such an error occurs. <br>在成功写入整个缓冲区或发生此类错误之前，此方法将不会返回。<br>
    /// The first error that is not of [`ErrorKind::Interrupted`] kind generated from this method will be returned. <br>从此方法生成的不是 [`ErrorKind::Interrupted`] 类型的第一个错误将被返回。<br>
    ///
    /// If the buffer contains no data, this will never call [`write`]. <br>如果缓冲区不包含任何数据，则永远不会调用 [`write`]。<br>
    ///
    /// # Errors
    ///
    /// This function will return the first error of non-[`ErrorKind::Interrupted`] kind that [`write`] returns. <br>该函数将返回 [`write`] 返回的第一个非 -[`ErrorKind::Interrupted`] 类型的错误。<br>
    ///
    /// [`write`]: Write::write
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut buffer = File::create("foo.txt")?;
    ///
    ///     buffer.write_all(b"some bytes")?;
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn write_all(&mut self, mut buf: &[u8]) -> Result<()> {
        while !buf.is_empty() {
            match self.write(buf) {
                Ok(0) => {
                    return Err(error::const_io_error!(
                        ErrorKind::WriteZero,
                        "failed to write whole buffer",
                    ));
                }
                Ok(n) => buf = &buf[n..],
                Err(ref e) if e.kind() == ErrorKind::Interrupted => {}
                Err(e) => return Err(e),
            }
        }
        Ok(())
    }

    /// Attempts to write multiple buffers into this writer. <br>尝试将多个缓冲区写入此 writer。<br>
    ///
    /// This method will continuously call [`write_vectored`] until there is no more data to be written or an error of non-[`ErrorKind::Interrupted`] kind is returned. <br>此方法将连续调用 [`write_vectored`]，直到没有更多数据要写入或返回非 [`ErrorKind::Interrupted`] 类型的错误为止。<br>
    ///
    /// This method will not return until all buffers have been successfully written or such an error occurs. <br>在成功写入所有缓冲区或发生此类错误之前，此方法将不会返回。<br>
    /// The first error that is not of [`ErrorKind::Interrupted`] kind generated from this method will be returned. <br>从此方法生成的不是 [`ErrorKind::Interrupted`] 类型的第一个错误将被返回。<br>
    ///
    /// If the buffer contains no data, this will never call [`write_vectored`]. <br>如果缓冲区不包含任何数据，则永远不会调用 [`write_vectored`]。<br>
    ///
    /// # Notes
    ///
    /// Unlike [`write_vectored`], this takes a *mutable* reference to a slice of [`IoSlice`]s, not an immutable one. <br>与 [`write_vectored`] 不同，这需要对 [`IoSlice`] 的切片进行 *可变* 引用，而不是不可变。<br>
    /// That's because we need to modify the slice to keep track of the bytes already written. <br>那是因为我们需要修改切片以跟踪已写入的字节。<br>
    ///
    /// Once this function returns, the contents of `bufs` are unspecified, as this depends on how many calls to [`write_vectored`] were necessary. <br>此函数返回后，将不指定 `bufs` 的内容，这取决于需要对 [`write_vectored`] 进行多少次调用。<br>
    /// It is best to understand this function as taking ownership of `bufs` and to not use `bufs` afterwards. <br>最好将此函数理解为拥有 `bufs` 的所有权，并且此后不使用 `bufs`。<br>
    /// The underlying buffers, to which the [`IoSlice`]s point (but not the [`IoSlice`]s themselves), are unchanged and can be reused. <br>[`IoSlice`] 指向的底层缓冲区 (而不是 [`IoSlice`] 本身) 是不变的，可以重用。<br>
    ///
    /// [`write_vectored`]: Write::write_vectored
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(write_all_vectored)]
    /// # fn main() -> std::io::Result<()> {
    ///
    /// use std::io::{Write, IoSlice};
    ///
    /// let mut writer = Vec::new();
    /// let bufs = &mut [
    ///     IoSlice::new(&[1]),
    ///     IoSlice::new(&[2, 3]),
    ///     IoSlice::new(&[4, 5, 6]),
    /// ];
    ///
    /// writer.write_all_vectored(bufs)?;
    /// // Note: the contents of `bufs` is now undefined, see the Notes section. <br>`bufs` 的内容现在是不确定的，请参见说明部分。<br>
    ///
    /// assert_eq!(writer, &[1, 2, 3, 4, 5, 6]);
    /// # Ok(()) }
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    #[unstable(feature = "write_all_vectored", issue = "70436")]
    fn write_all_vectored(&mut self, mut bufs: &mut [IoSlice<'_>]) -> Result<()> {
        // Guarantee that bufs is empty if it contains no data, to avoid calling write_vectored if there is no data to be written. <br>如果 bufs 不包含任何数据，请确保它为空，以避免在没有要写入的数据时调用 write_vectored。<br>
        //
        IoSlice::advance_slices(&mut bufs, 0);
        while !bufs.is_empty() {
            match self.write_vectored(bufs) {
                Ok(0) => {
                    return Err(error::const_io_error!(
                        ErrorKind::WriteZero,
                        "failed to write whole buffer",
                    ));
                }
                Ok(n) => IoSlice::advance_slices(&mut bufs, n),
                Err(ref e) if e.kind() == ErrorKind::Interrupted => {}
                Err(e) => return Err(e),
            }
        }
        Ok(())
    }

    /// Writes a formatted string into this writer, returning any error encountered. <br>将格式化的字符串写入此 writer，返回遇到的任何错误。<br>
    ///
    /// This method is primarily used to interface with the [`format_args!()`] macro, and it is rare that this should explicitly be called. <br>此方法主要用于与 [`format_args!()`] 宏接口，很少需要显式调用。<br>
    /// The [`write!()`] macro should be favored to invoke this method instead. <br>应优先使用 [`write!()`] 宏来调用此方法。<br>
    ///
    /// This function internally uses the [`write_all`] method on this trait and hence will continuously write data so long as no errors are received. <br>此函数内部在此 trait 上使用 [`write_all`] 方法，因此只要没有收到错误，就将连续写入数据。<br>
    ///
    /// This also means that partial writes are not indicated in this signature. <br>这也意味着此签名中未指示部分写入。<br>
    ///
    /// [`write_all`]: Write::write_all
    ///
    /// # Errors
    ///
    /// This function will return any I/O error reported while formatting. <br>此函数将返回格式化时报告的任何 I/O 错误。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut buffer = File::create("foo.txt")?;
    ///
    ///     // this call <br>这个<br>
    ///     write!(buffer, "{:.*}", 2, 1.234567)?;
    ///     // turns into this: <br>变成这样：<br>
    ///     buffer.write_fmt(format_args!("{:.*}", 2, 1.234567))?;
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn write_fmt(&mut self, fmt: fmt::Arguments<'_>) -> Result<()> {
        // Create a shim which translates a Write to a fmt::Write and saves off I/O errors. <br>创建一个垫片，该垫片将 Write 转换为 fmt::Write 并避免 I/O 错误。<br>
        // instead of discarding them <br>而不是丢弃它们<br>
        struct Adapter<'a, T: ?Sized + 'a> {
            inner: &'a mut T,
            error: Result<()>,
        }

        impl<T: Write + ?Sized> fmt::Write for Adapter<'_, T> {
            fn write_str(&mut self, s: &str) -> fmt::Result {
                match self.inner.write_all(s.as_bytes()) {
                    Ok(()) => Ok(()),
                    Err(e) => {
                        self.error = Err(e);
                        Err(fmt::Error)
                    }
                }
            }
        }

        let mut output = Adapter { inner: self, error: Ok(()) };
        match fmt::write(&mut output, fmt) {
            Ok(()) => Ok(()),
            Err(..) => {
                // check if the error came from the underlying `Write` or not <br>检查错误是否来自底层 `Write`<br>
                if output.error.is_err() {
                    output.error
                } else {
                    Err(error::const_io_error!(ErrorKind::Uncategorized, "formatter error"))
                }
            }
        }
    }

    /// Creates a "by reference" adapter for this instance of `Write`. <br>为这个 `Write` 实例创建一个 "by reference" 适配器。<br>
    ///
    /// The returned adapter also implements `Write` and will simply borrow this current writer. <br>返回的适配器也实现了 `Write`，并将简单地借用当前的 writer。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::Write;
    /// use std::fs::File;
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let mut buffer = File::create("foo.txt")?;
    ///
    ///     let reference = buffer.by_ref();
    ///
    ///     // we can use reference just like our original buffer <br>我们可以像使用原始缓冲区一样使用引用<br>
    ///     reference.write_all(b"some bytes")?;
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    fn by_ref(&mut self) -> &mut Self
    where
        Self: Sized,
    {
        self
    }
}

/// The `Seek` trait provides a cursor which can be moved within a stream of bytes. <br>`Seek` trait 提供了一个游标，可以在字节流中移动它。<br>
///
/// The stream typically has a fixed size, allowing seeking relative to either end or the current offset. <br>流通常具有固定的大小，允许相对于端点或当前偏移量进行搜索。<br>
///
///
/// # Examples
///
/// [`File`]s implement `Seek`: <br>[`File`] 的工具 `Seek`：<br>
///
/// [`File`]: crate::fs::File
///
/// ```no_run
/// use std::io;
/// use std::io::prelude::*;
/// use std::fs::File;
/// use std::io::SeekFrom;
///
/// fn main() -> io::Result<()> {
///     let mut f = File::open("foo.txt")?;
///
///     // move the cursor 42 bytes from the start of the file <br>从文件的开头将游标移动 42 个字节<br>
///     f.seek(SeekFrom::Start(42))?;
///     Ok(())
/// }
/// ```
///
#[stable(feature = "rust1", since = "1.0.0")]
pub trait Seek {
    /// Seek to an offset, in bytes, in a stream. <br>在流中寻找以字节为单位的偏移量。<br>
    ///
    /// A seek beyond the end of a stream is allowed, but behavior is defined by the implementation. <br>允许在流的末尾进行查找，但是行为由实现定义。<br>
    ///
    /// If the seek operation completed successfully, this method returns the new position from the start of the stream. <br>如果查找操作成功完成，则此方法从流的开头返回新位置。<br>
    ///
    /// That position can be used later with [`SeekFrom::Start`]. <br>该位置以后可以与 [`SeekFrom::Start`] 一起使用。<br>
    ///
    /// # Errors
    ///
    /// Seeking can fail, for example because it might involve flushing a buffer. <br>查找可能会失败，例如因为它可能涉及刷新缓冲区。<br>
    ///
    /// Seeking to a negative offset is considered an error. <br>寻求负偏移被认为是错误。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn seek(&mut self, pos: SeekFrom) -> Result<u64>;

    /// Rewind to the beginning of a stream. <br>返回到流的开头。<br>
    ///
    /// This is a convenience method, equivalent to `seek(SeekFrom::Start(0))`. <br>这是一个方便的方法，相当于 `seek(SeekFrom::Start(0))`。<br>
    ///
    /// # Errors
    ///
    /// Rewinding can fail, for example because it might involve flushing a buffer. <br>返回可能会失败，例如因为它可能涉及刷新缓冲区。<br>
    ///
    /// # Example
    ///
    /// ```no_run
    /// use std::io::{Read, Seek, Write};
    /// use std::fs::OpenOptions;
    ///
    /// let mut f = OpenOptions::new()
    ///     .write(true)
    ///     .read(true)
    ///     .create(true)
    ///     .open("foo.txt").unwrap();
    ///
    /// let hello = "Hello!\n";
    /// write!(f, "{hello}").unwrap();
    /// f.rewind().unwrap();
    ///
    /// let mut buf = String::new();
    /// f.read_to_string(&mut buf).unwrap();
    /// assert_eq!(&buf, hello);
    /// ```
    #[stable(feature = "seek_rewind", since = "1.55.0")]
    fn rewind(&mut self) -> Result<()> {
        self.seek(SeekFrom::Start(0))?;
        Ok(())
    }

    /// Returns the length of this stream (in bytes). <br>返回此流的长度 (以字节为单位)。<br>
    ///
    /// This method is implemented using up to three seek operations. <br>此方法最多使用三个查找操作来实现。<br> If this method returns successfully, the seek position is unchanged (i.e. the position before calling this method is the same as afterwards). <br>如果此方法成功返回，则搜索位置不变 (即，调用此方法之前的位置与之后的位置相同)。<br>
    /// However, if this method returns an error, the seek position is unspecified. <br>但是，如果此方法返回错误，则未指定搜索位置。<br>
    ///
    /// If you need to obtain the length of *many* streams and you don't care about the seek position afterwards, you can reduce the number of seek operations by simply calling `seek(SeekFrom::End(0))` and using its return value (it is also the stream length). <br>如果您需要获取 *多个* 流的长度，并且以后不再关心查找位置，则可以通过简单地调用 `seek(SeekFrom::End(0))` 并使用其返回值 (它也是流的长度) 来减少查找操作的次数。<br>
    ///
    ///
    /// Note that length of a stream can change over time (for example, when data is appended to a file). <br>请注意，流的长度可能会随时间变化 (例如，将数据追加到文件时)。<br> So calling this method multiple times does not necessarily return the same length each time. <br>因此，多次调用此方法不一定每次都返回相同的长度。<br>
    ///
    /// # Example
    ///
    /// ```no_run
    /// #![feature(seek_stream_len)]
    /// use std::{
    ///     io::{self, Seek},
    ///     fs::File,
    /// };
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = File::open("foo.txt")?;
    ///
    ///     let len = f.stream_len()?;
    ///     println!("The file is currently {len} bytes long");
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[unstable(feature = "seek_stream_len", issue = "59359")]
    fn stream_len(&mut self) -> Result<u64> {
        let old_pos = self.stream_position()?;
        let len = self.seek(SeekFrom::End(0))?;

        // Avoid seeking a third time when we were already at the end of the stream. <br>当我们已经走到尽头时，请避免再寻求第三次机会。<br>
        // The branch is usually way cheaper than a seek operation. <br>分支通常比查找操作便宜。<br>
        if old_pos != len {
            self.seek(SeekFrom::Start(old_pos))?;
        }

        Ok(len)
    }

    /// Returns the current seek position from the start of the stream. <br>从流的开头返回当前查找位置。<br>
    ///
    /// This is equivalent to `self.seek(SeekFrom::Current(0))`. <br>这等效于 `self.seek(SeekFrom::Current(0))`。<br>
    ///
    /// # Example
    ///
    /// ```no_run
    /// use std::{
    ///     io::{self, BufRead, BufReader, Seek},
    ///     fs::File,
    /// };
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut f = BufReader::new(File::open("foo.txt")?);
    ///
    ///     let before = f.stream_position()?;
    ///     f.read_line(&mut String::new())?;
    ///     let after = f.stream_position()?;
    ///
    ///     println!("The first line was {} bytes long", after - before);
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "seek_convenience", since = "1.51.0")]
    fn stream_position(&mut self) -> Result<u64> {
        self.seek(SeekFrom::Current(0))
    }
}

/// Enumeration of possible methods to seek within an I/O object. <br>列举可能在 I/O 对象中进行搜索的方法。<br>
///
/// It is used by the [`Seek`] trait. <br>[`Seek`] trait 使用它。<br>
#[derive(Copy, PartialEq, Eq, Clone, Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub enum SeekFrom {
    /// Sets the offset to the provided number of bytes. <br>将偏移量设置为提供的字节数。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    Start(#[stable(feature = "rust1", since = "1.0.0")] u64),

    /// Sets the offset to the size of this object plus the specified number of bytes. <br>将偏移量设置为此对象的大小加上指定的字节数。<br>
    ///
    /// It is possible to seek beyond the end of an object, but it's an error to seek before byte 0. <br>可以在对象的末尾进行查找，但是在字节 0 之前进行查找是错误的。<br>
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    End(#[stable(feature = "rust1", since = "1.0.0")] i64),

    /// Sets the offset to the current position plus the specified number of bytes. <br>将偏移量设置为当前位置加上指定的字节数。<br>
    ///
    /// It is possible to seek beyond the end of an object, but it's an error to seek before byte 0. <br>可以在对象的末尾进行查找，但是在字节 0 之前进行查找是错误的。<br>
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    Current(#[stable(feature = "rust1", since = "1.0.0")] i64),
}

fn read_until<R: BufRead + ?Sized>(r: &mut R, delim: u8, buf: &mut Vec<u8>) -> Result<usize> {
    let mut read = 0;
    loop {
        let (done, used) = {
            let available = match r.fill_buf() {
                Ok(n) => n,
                Err(ref e) if e.kind() == ErrorKind::Interrupted => continue,
                Err(e) => return Err(e),
            };
            match memchr::memchr(delim, available) {
                Some(i) => {
                    buf.extend_from_slice(&available[..=i]);
                    (true, i + 1)
                }
                None => {
                    buf.extend_from_slice(available);
                    (false, available.len())
                }
            }
        };
        r.consume(used);
        read += used;
        if done || used == 0 {
            return Ok(read);
        }
    }
}

/// A `BufRead` is a type of `Read`er which has an internal buffer, allowing it to perform extra ways of reading. <br>`BufRead` 是带有内部缓冲区的 `Read` 类型，它可以执行其他读取方式。<br>
///
/// For example, reading line-by-line is inefficient without using a buffer, so if you want to read by line, you'll need `BufRead`, which includes a [`read_line`] method as well as a [`lines`] iterator. <br>例如，在不使用缓冲区的情况下，逐行读取效率很低，因此，如果要逐行读取，则需要 `BufRead`，其中包括 [`read_line`] 方法和 [`lines`] 迭代器。<br>
///
///
/// # Examples
///
/// A locked standard input implements `BufRead`: <br>锁定的标准输入实现 `BufRead`：<br>
///
/// ```no_run
/// use std::io;
/// use std::io::prelude::*;
///
/// let stdin = io::stdin();
/// for line in stdin.lock().lines() {
///     println!("{}", line.unwrap());
/// }
/// ```
///
/// If you have something that implements [`Read`], you can use the [`BufReader` type][`BufReader`] to turn it into a `BufRead`. <br>如果您具有实现 [`Read`] 的功能，则可以使用 [`BufReader` type][`BufReader`] 将其转换为 `BufRead`。<br>
///
/// For example, [`File`] implements [`Read`], but not `BufRead`. <br>例如，[`File`] 实现 [`Read`]，但不实现 `BufRead`。<br>
/// [`BufReader`] to the rescue! <br>[`BufReader`] 来救援！<br>
///
/// [`File`]: crate::fs::File
/// [`read_line`]: BufRead::read_line
/// [`lines`]: BufRead::lines
///
/// ```no_run
/// use std::io::{self, BufReader};
/// use std::io::prelude::*;
/// use std::fs::File;
///
/// fn main() -> io::Result<()> {
///     let f = File::open("foo.txt")?;
///     let f = BufReader::new(f);
///
///     for line in f.lines() {
///         println!("{}", line.unwrap());
///     }
///
///     Ok(())
/// }
/// ```
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub trait BufRead: Read {
    /// Returns the contents of the internal buffer, filling it with more data from the inner reader if it is empty. <br>返回内部缓冲区的内容，如果内部缓冲区为空，则使用内部 reader 中的更多数据填充内部缓冲区。<br>
    ///
    /// This function is a lower-level call. <br>此函数是较低级别的调用。<br> It needs to be paired with the [`consume`] method to function properly. <br>它需要与 [`consume`] 方法配对才能正确执行功能。<br>
    /// When calling this method, none of the contents will be "read" in the sense that later calling `read` may return the same contents. <br>当调用此方法时，任何内容都不是 "read"，因为稍后调用 `read` 可能会返回相同的内容。<br>
    /// As such, [`consume`] must be called with the number of bytes that are consumed from this buffer to ensure that the bytes are never returned twice. <br>因此，必须使用此缓冲区消耗的字节数来调用 [`consume`]，以确保字节永远不会返回两次。<br>
    ///
    ///
    /// [`consume`]: BufRead::consume
    ///
    /// An empty buffer returned indicates that the stream has reached EOF. <br>返回的空缓冲区表示流已达到 EOF。<br>
    ///
    /// # Errors
    ///
    /// This function will return an I/O error if the underlying reader was read, but returned an error. <br>如果读取了底层 reader，但返回了错误，则此函数将返回 I/O 错误。<br>
    ///
    /// # Examples
    ///
    /// A locked standard input implements `BufRead`: <br>锁定的标准输入实现 `BufRead`：<br>
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    ///
    /// let stdin = io::stdin();
    /// let mut stdin = stdin.lock();
    ///
    /// let buffer = stdin.fill_buf().unwrap();
    ///
    /// // work with buffer <br>使用缓冲区<br>
    /// println!("{buffer:?}");
    ///
    /// // ensure the bytes we worked with aren't returned again later <br>确保我们处理过的字节以后不再返回<br>
    /// let length = buffer.len();
    /// stdin.consume(length);
    /// ```
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn fill_buf(&mut self) -> Result<&[u8]>;

    /// Tells this buffer that `amt` bytes have been consumed from the buffer, so they should no longer be returned in calls to `read`. <br>告诉此缓冲区 `amt` 字节已从缓冲区中消耗掉，因此在调用 `read` 时不再应返回它们。<br>
    ///
    /// This function is a lower-level call. <br>此函数是较低级别的调用。<br> It needs to be paired with the [`fill_buf`] method to function properly. <br>它需要与 [`fill_buf`] 方法配合才能正常使用。<br>
    /// This function does not perform any I/O, it simply informs this object that some amount of its buffer, returned from [`fill_buf`], has been consumed and should no longer be returned. <br>该函数不执行任何 I/O，它只是通知对象从 [`fill_buf`] 返回的某些缓冲区已被消耗，不应再返回。<br>
    ///
    /// As such, this function may do odd things if [`fill_buf`] isn't called before calling it. <br>因此，如果在调用 [`fill_buf`] 之前未对其进行调用，则此函数可能会做一些奇怪的事情。<br>
    ///
    /// The `amt` must be `<=` the number of bytes in the buffer returned by [`fill_buf`]. <br>`amt` 必须为 `<=`，即 [`fill_buf`] 返回的缓冲区中的字节数。<br>
    ///
    /// # Examples
    ///
    /// Since `consume()` is meant to be used with [`fill_buf`], that method's example includes an example of `consume()`. <br>由于 `consume()` 旨在与 [`fill_buf`] 一起使用，因此该方法的示例包括 `consume()` 的示例。<br>
    ///
    /// [`fill_buf`]: BufRead::fill_buf
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn consume(&mut self, amt: usize);

    /// Check if the underlying `Read` has any data left to be read. <br>检查底层 `Read` 是否有任何数据可供读取。<br>
    ///
    /// This function may fill the buffer to check for data, so this functions returns `Result<bool>`, not `bool`. <br>这个函数可能会填充缓冲区来检查数据，所以这个函数返回的是 `Result<bool>`，而不是 `bool`。<br>
    ///
    /// Default implementation calls `fill_buf` and checks that returned slice is empty (which means that there is no data left, since EOF is reached). <br>默认实现调用 `fill_buf` 并检查返回的 4 为空 (这意味着没有数据剩余，因为达到了 EOF)。<br>
    ///
    ///
    /// Examples
    ///
    /// ```
    /// #![feature(buf_read_has_data_left)]
    /// use std::io;
    /// use std::io::prelude::*;
    ///
    /// let stdin = io::stdin();
    /// let mut stdin = stdin.lock();
    ///
    /// while stdin.has_data_left().unwrap() {
    ///     let mut line = String::new();
    ///     stdin.read_line(&mut line).unwrap();
    ///     // work with line <br>与行一起工作<br>
    ///     println!("{line:?}");
    /// }
    /// ```
    ///
    ///
    #[unstable(feature = "buf_read_has_data_left", reason = "recently added", issue = "86423")]
    fn has_data_left(&mut self) -> Result<bool> {
        self.fill_buf().map(|b| !b.is_empty())
    }

    /// Read all bytes into `buf` until the delimiter `byte` or EOF is reached. <br>将所有字节读入 `buf`，直到到达定界符 `byte` 或 EOF。<br>
    ///
    /// This function will read bytes from the underlying stream until the delimiter or EOF is found. <br>该函数将从底层流中读取字节，直到找到定界符或 EOF。<br>
    /// Once found, all bytes up to, and including, the delimiter (if found) will be appended to `buf`. <br>一旦找到，直到并包括分隔符 (如果找到) 的所有字节都将被追加到 `buf`。<br>
    ///
    /// If successful, this function will return the total number of bytes read. <br>如果成功，此函数将返回读取的字节总数。<br>
    ///
    /// This function is blocking and should be used carefully: it is possible for an attacker to continuously send bytes without ever sending the delimiter or EOF. <br>该函数正在阻塞，应谨慎使用：攻击者有可能连续发送字节而无需发送定界符或 EOF。<br>
    ///
    ///
    /// # Errors
    ///
    /// This function will ignore all instances of [`ErrorKind::Interrupted`] and will otherwise return any errors returned by [`fill_buf`]. <br>该函数将忽略 [`ErrorKind::Interrupted`] 的所有实例，否则将返回 [`fill_buf`] 返回的任何错误。<br>
    ///
    /// If an I/O error is encountered then all bytes read so far will be present in `buf` and its length will have been adjusted appropriately. <br>如果遇到 I/O 错误，则 `buf` 中将存在到目前为止已读取的所有字节，并且已对其长度进行了适当的调整。<br>
    ///
    /// [`fill_buf`]: BufRead::fill_buf
    ///
    /// # Examples
    ///
    /// [`std::io::Cursor`][`Cursor`] is a type that implements `BufRead`. <br>[`std::io::Cursor`][`Cursor`] 是一种实现了 `BufRead` 的类型。<br>
    /// In this example, we use [`Cursor`] to read all the bytes in a byte slice in hyphen delimited segments: <br>在此示例中，我们使用 [`Cursor`] 读取以连字符分隔的段中的字节片中的所有字节：<br>
    ///
    /// ```
    /// use std::io::{self, BufRead};
    ///
    /// let mut cursor = io::Cursor::new(b"lorem-ipsum");
    /// let mut buf = vec![];
    ///
    /// // cursor is at 'l' <br>游标在 'l'<br>
    /// let num_bytes = cursor.read_until(b'-', &mut buf)
    ///     .expect("reading from cursor won't fail");
    /// assert_eq!(num_bytes, 6);
    /// assert_eq!(buf, b"lorem-");
    /// buf.clear();
    ///
    /// // cursor is at 'i' <br>游标在 'i'<br>
    /// let num_bytes = cursor.read_until(b'-', &mut buf)
    ///     .expect("reading from cursor won't fail");
    /// assert_eq!(num_bytes, 5);
    /// assert_eq!(buf, b"ipsum");
    /// buf.clear();
    ///
    /// // cursor is at EOF <br>游标在 EOF 处<br>
    /// let num_bytes = cursor.read_until(b'-', &mut buf)
    ///     .expect("reading from cursor won't fail");
    /// assert_eq!(num_bytes, 0);
    /// assert_eq!(buf, b"");
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn read_until(&mut self, byte: u8, buf: &mut Vec<u8>) -> Result<usize> {
        read_until(self, byte, buf)
    }

    /// Read all bytes until a newline (the `0xA` byte) is reached, and append them to the provided buffer. <br>读取所有字节，直到到达换行符 (`0xA` 字节)，然后将它们追加到提供的缓冲区中。<br>
    /// You do not need to clear the buffer before appending. <br>您不需要在追加之前清除缓冲区。<br>
    ///
    /// This function will read bytes from the underlying stream until the newline delimiter (the `0xA` byte) or EOF is found. <br>该函数将从底层流中读取字节，直到找到换行符 (`0xA` 字节) 或 EOF。<br>
    /// Once found, all bytes up to, and including, the delimiter (if found) will be appended to `buf`. <br>一旦找到，直到并包括分隔符 (如果找到) 的所有字节都将被追加到 `buf`。<br>
    ///
    /// If successful, this function will return the total number of bytes read. <br>如果成功，此函数将返回读取的字节总数。<br>
    ///
    /// If this function returns [`Ok(0)`], the stream has reached EOF. <br>如果此函数返回 [`Ok(0)`]，则流已到达 EOF。<br>
    ///
    /// This function is blocking and should be used carefully: it is possible for an attacker to continuously send bytes without ever sending a newline or EOF. <br>该函数正在阻塞，应谨慎使用：攻击者有可能连续发送字节而无需发送换行符或 EOF。<br>
    ///
    ///
    /// [`Ok(0)`]: Ok
    ///
    /// # Errors
    ///
    /// This function has the same error semantics as [`read_until`] and will also return an error if the read bytes are not valid UTF-8. <br>该函数与 [`read_until`] 具有相同的错误语义，如果读取的字节无效，则还将返回错误。<br>
    /// If an I/O error is encountered then `buf` may contain some bytes already read in the event that all data read so far was valid UTF-8. <br>如果遇到 I/O 错误，则 `buf` 可能包含一些已读取的字节，如果到目前为止读取的所有数据都是有效的 UTF-8。<br>
    ///
    /// [`read_until`]: BufRead::read_until
    ///
    /// # Examples
    ///
    /// [`std::io::Cursor`][`Cursor`] is a type that implements `BufRead`. <br>[`std::io::Cursor`][`Cursor`] 是一种实现了 `BufRead` 的类型。<br> In this example, we use [`Cursor`] to read all the lines in a byte slice: <br>在这个例子中，我们使用 [`Cursor`] 读取一个字节切片中的所有行：<br>
    ///
    /// ```
    /// use std::io::{self, BufRead};
    ///
    /// let mut cursor = io::Cursor::new(b"foo\nbar");
    /// let mut buf = String::new();
    ///
    /// // cursor is at 'f' <br>游标在 'f'<br>
    /// let num_bytes = cursor.read_line(&mut buf)
    ///     .expect("reading from cursor won't fail");
    /// assert_eq!(num_bytes, 4);
    /// assert_eq!(buf, "foo\n");
    /// buf.clear();
    ///
    /// // cursor is at 'b' <br>游标在 'b'<br>
    /// let num_bytes = cursor.read_line(&mut buf)
    ///     .expect("reading from cursor won't fail");
    /// assert_eq!(num_bytes, 3);
    /// assert_eq!(buf, "bar");
    /// buf.clear();
    ///
    /// // cursor is at EOF <br>游标在 EOF 处<br>
    /// let num_bytes = cursor.read_line(&mut buf)
    ///     .expect("reading from cursor won't fail");
    /// assert_eq!(num_bytes, 0);
    /// assert_eq!(buf, "");
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn read_line(&mut self, buf: &mut String) -> Result<usize> {
        // Note that we are not calling the `.read_until` method here, but rather our hardcoded implementation. <br>请注意，我们不是在这里调用 `.read_until` 方法，而是我们的硬编码实现。<br>
        // For more details as to why, see the comments in `read_to_end`. <br>有关原因的更多详细信息，请参见 `read_to_end` 中的注释。<br>
        //
        unsafe { append_to_string(buf, |b| read_until(self, b'\n', b)) }
    }

    /// Returns an iterator over the contents of this reader split on the byte `byte`. <br>返回对该字节 `byte` 上的 reader 拆分内容的迭代器。<br>
    ///
    /// The iterator returned from this function will return instances of <code>[io::Result]<[Vec]\<u8>></code>. <br>这个函数返回的迭代器将返回 <code>[io::Result]<[Vec]\<u8>></code> 的实例。<br>
    ///
    /// Each vector returned will *not* have the delimiter byte at the end. <br>返回的每个 vector 都不会在末尾有定界符字节。<br>
    ///
    /// This function will yield errors whenever [`read_until`] would have also yielded an error. <br>只要 [`read_until`] 也产生错误，此函数就会产生错误。<br>
    ///
    /// [io::Result]: self::Result "io::Result"
    /// [`read_until`]: BufRead::read_until
    ///
    /// # Examples
    ///
    /// [`std::io::Cursor`][`Cursor`] is a type that implements `BufRead`. <br>[`std::io::Cursor`][`Cursor`] 是一种实现了 `BufRead` 的类型。<br>
    /// In this example, we use [`Cursor`] to iterate over all hyphen delimited segments in a byte slice <br>在此示例中，我们使用 [`Cursor`] 遍历字节切片中的所有连字符分隔的段<br>
    ///
    /// ```
    /// use std::io::{self, BufRead};
    ///
    /// let cursor = io::Cursor::new(b"lorem-ipsum-dolor");
    ///
    /// let mut split_iter = cursor.split(b'-').map(|l| l.unwrap());
    /// assert_eq!(split_iter.next(), Some(b"lorem".to_vec()));
    /// assert_eq!(split_iter.next(), Some(b"ipsum".to_vec()));
    /// assert_eq!(split_iter.next(), Some(b"dolor".to_vec()));
    /// assert_eq!(split_iter.next(), None);
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn split(self, byte: u8) -> Split<Self>
    where
        Self: Sized,
    {
        Split { buf: self, delim: byte }
    }

    /// Returns an iterator over the lines of this reader. <br>返回此 reader 的各行上的迭代器。<br>
    ///
    /// The iterator returned from this function will yield instances of <code>[io::Result]<[String]></code>. <br>从这个函数返回的迭代器将产生 <code>[io::Result]<[String]></code> 的实例。<br>
    /// Each string returned will *not* have a newline byte (the `0xA` byte) or `CRLF` (`0xD`, `0xA` bytes) at the end. <br>返回的每个字符串末尾都不会有换行符字节 (`0xA` 字节) 或 `CRLF` (`0xD`，`0xA` 字节)。<br>
    ///
    ///
    /// [io::Result]: self::Result "io::Result"
    ///
    /// # Examples
    ///
    /// [`std::io::Cursor`][`Cursor`] is a type that implements `BufRead`. <br>[`std::io::Cursor`][`Cursor`] 是一种实现了 `BufRead` 的类型。<br>
    /// In this example, we use [`Cursor`] to iterate over all the lines in a byte slice. <br>在这个例子中，我们使用 [`Cursor`] 来遍历字节切片中的所有行。<br>
    ///
    /// ```
    /// use std::io::{self, BufRead};
    ///
    /// let cursor = io::Cursor::new(b"lorem\nipsum\r\ndolor");
    ///
    /// let mut lines_iter = cursor.lines().map(|l| l.unwrap());
    /// assert_eq!(lines_iter.next(), Some(String::from("lorem")));
    /// assert_eq!(lines_iter.next(), Some(String::from("ipsum")));
    /// assert_eq!(lines_iter.next(), Some(String::from("dolor")));
    /// assert_eq!(lines_iter.next(), None);
    /// ```
    ///
    /// # Errors
    ///
    /// Each line of the iterator has the same error semantics as [`BufRead::read_line`]. <br>迭代器的每一行都具有与 [`BufRead::read_line`] 相同的错误语义。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn lines(self) -> Lines<Self>
    where
        Self: Sized,
    {
        Lines { buf: self }
    }
}

/// Adapter to chain together two readers. <br>将两个 readers 链接在一起的适配器。<br>
///
/// This struct is generally created by calling [`chain`] on a reader. <br>这个结构体通常是通过在 reader 上调用 [`chain`] 来创建的。<br>
/// Please see the documentation of [`chain`] for more details. <br>有关更多详细信息，请参见 [`chain`] 的文档。<br>
///
/// [`chain`]: Read::chain
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Chain<T, U> {
    first: T,
    second: U,
    done_first: bool,
}

impl<T, U> Chain<T, U> {
    /// Consumes the `Chain`, returning the wrapped readers. <br>消耗 `Chain`，返回包装的 readers。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut foo_file = File::open("foo.txt")?;
    ///     let mut bar_file = File::open("bar.txt")?;
    ///
    ///     let chain = foo_file.chain(bar_file);
    ///     let (foo_file, bar_file) = chain.into_inner();
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "more_io_inner_methods", since = "1.20.0")]
    pub fn into_inner(self) -> (T, U) {
        (self.first, self.second)
    }

    /// Gets references to the underlying readers in this `Chain`. <br>获取此 `Chain` 中的底层 readers 的引用。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut foo_file = File::open("foo.txt")?;
    ///     let mut bar_file = File::open("bar.txt")?;
    ///
    ///     let chain = foo_file.chain(bar_file);
    ///     let (foo_file, bar_file) = chain.get_ref();
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "more_io_inner_methods", since = "1.20.0")]
    pub fn get_ref(&self) -> (&T, &U) {
        (&self.first, &self.second)
    }

    /// Gets mutable references to the underlying readers in this `Chain`. <br>获取此 `Chain` 中的底层 readers 的可变引用。<br>
    ///
    /// Care should be taken to avoid modifying the internal I/O state of the underlying readers as doing so may corrupt the internal state of this `Chain`. <br>应注意避免修改底层 readers 的内部 I/O 状态，因为这样做可能会破坏该 `Chain` 的内部状态。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut foo_file = File::open("foo.txt")?;
    ///     let mut bar_file = File::open("bar.txt")?;
    ///
    ///     let mut chain = foo_file.chain(bar_file);
    ///     let (foo_file, bar_file) = chain.get_mut();
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "more_io_inner_methods", since = "1.20.0")]
    pub fn get_mut(&mut self) -> (&mut T, &mut U) {
        (&mut self.first, &mut self.second)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Read, U: Read> Read for Chain<T, U> {
    fn read(&mut self, buf: &mut [u8]) -> Result<usize> {
        if !self.done_first {
            match self.first.read(buf)? {
                0 if !buf.is_empty() => self.done_first = true,
                n => return Ok(n),
            }
        }
        self.second.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> Result<usize> {
        if !self.done_first {
            match self.first.read_vectored(bufs)? {
                0 if bufs.iter().any(|b| !b.is_empty()) => self.done_first = true,
                n => return Ok(n),
            }
        }
        self.second.read_vectored(bufs)
    }
}

#[stable(feature = "chain_bufread", since = "1.9.0")]
impl<T: BufRead, U: BufRead> BufRead for Chain<T, U> {
    fn fill_buf(&mut self) -> Result<&[u8]> {
        if !self.done_first {
            match self.first.fill_buf()? {
                buf if buf.is_empty() => {
                    self.done_first = true;
                }
                buf => return Ok(buf),
            }
        }
        self.second.fill_buf()
    }

    fn consume(&mut self, amt: usize) {
        if !self.done_first { self.first.consume(amt) } else { self.second.consume(amt) }
    }
}

impl<T, U> SizeHint for Chain<T, U> {
    #[inline]
    fn lower_bound(&self) -> usize {
        SizeHint::lower_bound(&self.first) + SizeHint::lower_bound(&self.second)
    }

    #[inline]
    fn upper_bound(&self) -> Option<usize> {
        match (SizeHint::upper_bound(&self.first), SizeHint::upper_bound(&self.second)) {
            (Some(first), Some(second)) => first.checked_add(second),
            _ => None,
        }
    }
}

/// Reader adapter which limits the bytes read from an underlying reader. <br>Reader 适配器，用于限制从底层 reader 读取的字节。<br>
///
/// This struct is generally created by calling [`take`] on a reader. <br>这个结构体通常是通过在 reader 上调用 [`take`] 来创建的。<br>
/// Please see the documentation of [`take`] for more details. <br>有关更多详细信息，请参见 [`take`] 的文档。<br>
///
/// [`take`]: Read::take
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Take<T> {
    inner: T,
    limit: u64,
}

impl<T> Take<T> {
    /// Returns the number of bytes that can be read before this instance will return EOF. <br>返回在此实例返回 EOF 之前可以读取的字节数。<br>
    ///
    /// # Note
    ///
    /// This instance may reach `EOF` after reading fewer bytes than indicated by this method if the underlying [`Read`] instance reaches EOF. <br>如果底层 [`Read`] 实例达到 EOF，则在读取的字节数少于此方法指示的字节数之后，此实例可能到达 `EOF`。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let f = File::open("foo.txt")?;
    ///
    ///     // read at most five bytes <br>最多读取五个字节<br>
    ///     let handle = f.take(5);
    ///
    ///     println!("limit: {}", handle.limit());
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn limit(&self) -> u64 {
        self.limit
    }

    /// Sets the number of bytes that can be read before this instance will return EOF. <br>设置在此实例返回 EOF 之前可以读取的字节数。<br>
    /// This is the same as constructing a new `Take` instance, so the amount of bytes read and the previous limit value don't matter when calling this method. <br>这与构造新的 `Take` 实例相同，因此在调用此方法时，读取的字节数和先前的限制值无关紧要。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let f = File::open("foo.txt")?;
    ///
    ///     // read at most five bytes <br>最多读取五个字节<br>
    ///     let mut handle = f.take(5);
    ///     handle.set_limit(10);
    ///
    ///     assert_eq!(handle.limit(), 10);
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "take_set_limit", since = "1.27.0")]
    pub fn set_limit(&mut self, limit: u64) {
        self.limit = limit;
    }

    /// Consumes the `Take`, returning the wrapped reader. <br>消耗 `Take`，返回包装的 reader。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut file = File::open("foo.txt")?;
    ///
    ///     let mut buffer = [0; 5];
    ///     let mut handle = file.take(5);
    ///     handle.read(&mut buffer)?;
    ///
    ///     let file = handle.into_inner();
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "io_take_into_inner", since = "1.15.0")]
    pub fn into_inner(self) -> T {
        self.inner
    }

    /// Gets a reference to the underlying reader. <br>获取对底层 reader 的引用。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut file = File::open("foo.txt")?;
    ///
    ///     let mut buffer = [0; 5];
    ///     let mut handle = file.take(5);
    ///     handle.read(&mut buffer)?;
    ///
    ///     let file = handle.get_ref();
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "more_io_inner_methods", since = "1.20.0")]
    pub fn get_ref(&self) -> &T {
        &self.inner
    }

    /// Gets a mutable reference to the underlying reader. <br>获取底层 reader 的可变引用。<br>
    ///
    /// Care should be taken to avoid modifying the internal I/O state of the underlying reader as doing so may corrupt the internal limit of this `Take`. <br>应注意避免修改底层 reader 的内部 I/O 状态，因为这样做可能会破坏该 `Take` 的内部限制。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    /// use std::io::prelude::*;
    /// use std::fs::File;
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut file = File::open("foo.txt")?;
    ///
    ///     let mut buffer = [0; 5];
    ///     let mut handle = file.take(5);
    ///     handle.read(&mut buffer)?;
    ///
    ///     let file = handle.get_mut();
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "more_io_inner_methods", since = "1.20.0")]
    pub fn get_mut(&mut self) -> &mut T {
        &mut self.inner
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Read> Read for Take<T> {
    fn read(&mut self, buf: &mut [u8]) -> Result<usize> {
        // Don't call into inner reader at all at EOF because it may still block <br>在 EOF 根本不要调用内部 reader，因为它可能仍然会阻塞<br>
        if self.limit == 0 {
            return Ok(0);
        }

        let max = cmp::min(buf.len() as u64, self.limit) as usize;
        let n = self.inner.read(&mut buf[..max])?;
        self.limit -= n as u64;
        Ok(n)
    }

    fn read_buf(&mut self, buf: &mut ReadBuf<'_>) -> Result<()> {
        // Don't call into inner reader at all at EOF because it may still block <br>在 EOF 根本不要调用内部 reader，因为它可能仍然会阻塞<br>
        if self.limit == 0 {
            return Ok(());
        }

        let prev_filled = buf.filled_len();

        if self.limit <= buf.remaining() as u64 {
            // if we just use an as cast to convert, limit may wrap around on a 32 bit target <br>如果我们只是使用 as cast 进行转换，limit 可能会环绕在 32 位目标上<br>
            let limit = cmp::min(self.limit, usize::MAX as u64) as usize;

            let extra_init = cmp::min(limit as usize, buf.initialized_len() - buf.filled_len());

            // SAFETY: no uninit data is written to ibuf <br>没有将 uninit 数据写入到 ibuf<br>
            let ibuf = unsafe { &mut buf.unfilled_mut()[..limit] };

            let mut sliced_buf = ReadBuf::uninit(ibuf);

            // SAFETY: extra_init bytes of ibuf are known to be initialized <br>已知 ibuf 的 extra_init 字节已被初始化<br>
            unsafe {
                sliced_buf.assume_init(extra_init);
            }

            self.inner.read_buf(&mut sliced_buf)?;

            let new_init = sliced_buf.initialized_len();
            let filled = sliced_buf.filled_len();

            // sliced_buf / ibuf must drop here <br>sliced_bu / ibuf 必须在此处丢弃<br>

            // SAFETY: new_init bytes of buf's unfilled buffer have been initialized <br>buf 的未填充缓冲区的 new_init 字节已被初始化<br>
            unsafe {
                buf.assume_init(new_init);
            }

            buf.add_filled(filled);

            self.limit -= filled as u64;
        } else {
            self.inner.read_buf(buf)?;

            // inner may unfill <br>内部可能未填充<br>
            self.limit -= buf.filled_len().saturating_sub(prev_filled) as u64;
        }

        Ok(())
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: BufRead> BufRead for Take<T> {
    fn fill_buf(&mut self) -> Result<&[u8]> {
        // Don't call into inner reader at all at EOF because it may still block <br>在 EOF 根本不要调用内部 reader，因为它可能仍然会阻塞<br>
        if self.limit == 0 {
            return Ok(&[]);
        }

        let buf = self.inner.fill_buf()?;
        let cap = cmp::min(buf.len() as u64, self.limit) as usize;
        Ok(&buf[..cap])
    }

    fn consume(&mut self, amt: usize) {
        // Don't let callers reset the limit by passing an overlarge value <br>不要让调用者通过传递超大值来重置限制<br>
        let amt = cmp::min(amt as u64, self.limit) as usize;
        self.limit -= amt as u64;
        self.inner.consume(amt);
    }
}

impl<T> SizeHint for Take<T> {
    #[inline]
    fn lower_bound(&self) -> usize {
        cmp::min(SizeHint::lower_bound(&self.inner) as u64, self.limit) as usize
    }

    #[inline]
    fn upper_bound(&self) -> Option<usize> {
        match SizeHint::upper_bound(&self.inner) {
            Some(upper_bound) => Some(cmp::min(upper_bound as u64, self.limit) as usize),
            None => self.limit.try_into().ok(),
        }
    }
}

/// An iterator over `u8` values of a reader. <br>reader 的 `u8` 值上的迭代器。<br>
///
/// This struct is generally created by calling [`bytes`] on a reader. <br>通常通过在 reader 上调用 [`bytes`] 来创建此结构体。<br>
/// Please see the documentation of [`bytes`] for more details. <br>请参见 [`bytes`] 的文档以获取更多详细信息。<br>
///
/// [`bytes`]: Read::bytes
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Bytes<R> {
    inner: R,
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<R: Read> Iterator for Bytes<R> {
    type Item = Result<u8>;

    fn next(&mut self) -> Option<Result<u8>> {
        let mut byte = 0;
        loop {
            return match self.inner.read(slice::from_mut(&mut byte)) {
                Ok(0) => None,
                Ok(..) => Some(Ok(byte)),
                Err(ref e) if e.kind() == ErrorKind::Interrupted => continue,
                Err(e) => Some(Err(e)),
            };
        }
    }

    fn size_hint(&self) -> (usize, Option<usize>) {
        SizeHint::size_hint(&self.inner)
    }
}

trait SizeHint {
    fn lower_bound(&self) -> usize;

    fn upper_bound(&self) -> Option<usize>;

    fn size_hint(&self) -> (usize, Option<usize>) {
        (self.lower_bound(), self.upper_bound())
    }
}

impl<T> SizeHint for T {
    #[inline]
    default fn lower_bound(&self) -> usize {
        0
    }

    #[inline]
    default fn upper_bound(&self) -> Option<usize> {
        None
    }
}

impl<T> SizeHint for &mut T {
    #[inline]
    fn lower_bound(&self) -> usize {
        SizeHint::lower_bound(*self)
    }

    #[inline]
    fn upper_bound(&self) -> Option<usize> {
        SizeHint::upper_bound(*self)
    }
}

impl<T> SizeHint for Box<T> {
    #[inline]
    fn lower_bound(&self) -> usize {
        SizeHint::lower_bound(&**self)
    }

    #[inline]
    fn upper_bound(&self) -> Option<usize> {
        SizeHint::upper_bound(&**self)
    }
}

impl SizeHint for &[u8] {
    #[inline]
    fn lower_bound(&self) -> usize {
        self.len()
    }

    #[inline]
    fn upper_bound(&self) -> Option<usize> {
        Some(self.len())
    }
}

/// An iterator over the contents of an instance of `BufRead` split on a particular byte. <br>对 `BufRead` 实例的内容进行迭代的迭代器，该实例在特定字节上拆分。<br>
///
///
/// This struct is generally created by calling [`split`] on a `BufRead`. <br>通常通过在 `BufRead` 上调用 [`split`] 来创建此结构体。<br>
/// Please see the documentation of [`split`] for more details. <br>请参见 [`split`] 的文档以获取更多详细信息。<br>
///
/// [`split`]: BufRead::split
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Split<B> {
    buf: B,
    delim: u8,
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<B: BufRead> Iterator for Split<B> {
    type Item = Result<Vec<u8>>;

    fn next(&mut self) -> Option<Result<Vec<u8>>> {
        let mut buf = Vec::new();
        match self.buf.read_until(self.delim, &mut buf) {
            Ok(0) => None,
            Ok(_n) => {
                if buf[buf.len() - 1] == self.delim {
                    buf.pop();
                }
                Some(Ok(buf))
            }
            Err(e) => Some(Err(e)),
        }
    }
}

/// An iterator over the lines of an instance of `BufRead`. <br>`BufRead` 实例的行上的迭代器。<br>
///
/// This struct is generally created by calling [`lines`] on a `BufRead`. <br>通常通过在 `BufRead` 上调用 [`lines`] 来创建此结构体。<br>
/// Please see the documentation of [`lines`] for more details. <br>请参见 [`lines`] 的文档以获取更多详细信息。<br>
///
/// [`lines`]: BufRead::lines
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Lines<B> {
    buf: B,
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<B: BufRead> Iterator for Lines<B> {
    type Item = Result<String>;

    fn next(&mut self) -> Option<Result<String>> {
        let mut buf = String::new();
        match self.buf.read_line(&mut buf) {
            Ok(0) => None,
            Ok(_n) => {
                if buf.ends_with('\n') {
                    buf.pop();
                    if buf.ends_with('\r') {
                        buf.pop();
                    }
                }
                Some(Ok(buf))
            }
            Err(e) => Some(Err(e)),
        }
    }
}
