#![unstable(feature = "read_buf", issue = "78485")]

#[cfg(test)]
mod tests;

use crate::cmp;
use crate::fmt::{self, Debug, Formatter};
use crate::mem::MaybeUninit;

/// A wrapper around a byte buffer that is incrementally filled and initialized. <br>字节缓冲区的包装器，它被增量填充和初始化。<br>
///
/// This type is a sort of "double cursor". <br>这种类型是一种 "double cursor"。<br>
/// It tracks three regions in the buffer: a region at the beginning of the buffer that has been logically filled with data, a region that has been initialized at some point but not yet logically filled, and a region at the end that is fully uninitialized. <br>它跟踪缓冲区中的三个区域：缓冲区开头的区域已被逻辑填充数据，已在某个时刻初始化但尚未逻辑填充的区域，以及完全未初始化的末尾区域。<br>
/// The filled region is guaranteed to be a subset of the initialized region. <br>填充区域保证是初始化区域的子集。<br>
///
/// In summary, the contents of the buffer can be visualized as: <br>总之，缓冲区的内容可以可视化为：<br>
///
/// ```not_rust
/// [             capacity              ]
/// [ filled |         unfilled         ]
/// [    initialized    | uninitialized ]
/// ```
pub struct ReadBuf<'a> {
    buf: &'a mut [MaybeUninit<u8>],
    filled: usize,
    initialized: usize,
}

impl Debug for ReadBuf<'_> {
    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
        f.debug_struct("ReadBuf")
            .field("init", &self.initialized())
            .field("filled", &self.filled)
            .field("capacity", &self.capacity())
            .finish()
    }
}

impl<'a> ReadBuf<'a> {
    /// Creates a new `ReadBuf` from a fully initialized buffer. <br>从完全初始化的缓冲区创建一个新的 `ReadBuf`。<br>
    #[inline]
    pub fn new(buf: &'a mut [u8]) -> ReadBuf<'a> {
        let len = buf.len();

        ReadBuf {
            // SAFETY: initialized data never becoming uninitialized is an invariant of ReadBuf <br>初始化数据永远不会变为未初始化是 ReadBuf 的不变量<br>
            buf: unsafe { (buf as *mut [u8]).as_uninit_slice_mut().unwrap() },
            filled: 0,
            initialized: len,
        }
    }

    /// Creates a new `ReadBuf` from a fully uninitialized buffer. <br>从完全未初始化的缓冲区创建一个新的 `ReadBuf`。<br>
    ///
    /// Use `assume_init` if part of the buffer is known to be already initialized. <br>如果已知缓冲区的一部分已经初始化，请使用 `assume_init`。<br>
    #[inline]
    pub fn uninit(buf: &'a mut [MaybeUninit<u8>]) -> ReadBuf<'a> {
        ReadBuf { buf, filled: 0, initialized: 0 }
    }

    /// Returns the total capacity of the buffer. <br>返回缓冲区的总容量。<br>
    #[inline]
    pub fn capacity(&self) -> usize {
        self.buf.len()
    }

    /// Returns a shared reference to the filled portion of the buffer. <br>返回对缓冲区填充部分的共享引用。<br>
    #[inline]
    pub fn filled(&self) -> &[u8] {
        // SAFETY: We only slice the filled part of the buffer, which is always valid <br>我们只对缓冲区的填充部分进行切片，这总是有效的<br>
        unsafe { MaybeUninit::slice_assume_init_ref(&self.buf[0..self.filled]) }
    }

    /// Returns a mutable reference to the filled portion of the buffer. <br>返回对缓冲区已填充部分的可变引用。<br>
    #[inline]
    pub fn filled_mut(&mut self) -> &mut [u8] {
        // SAFETY: We only slice the filled part of the buffer, which is always valid <br>我们只对缓冲区的填充部分进行切片，这总是有效的<br>
        unsafe { MaybeUninit::slice_assume_init_mut(&mut self.buf[0..self.filled]) }
    }

    /// Returns a shared reference to the initialized portion of the buffer. <br>返回对缓冲区初始化部分的共享引用。<br>
    ///
    /// This includes the filled portion. <br>这包括填充的部分。<br>
    #[inline]
    pub fn initialized(&self) -> &[u8] {
        // SAFETY: We only slice the initialized part of the buffer, which is always valid <br>我们只对缓冲区的初始化部分进行切片，这始终是有效的<br>
        unsafe { MaybeUninit::slice_assume_init_ref(&self.buf[0..self.initialized]) }
    }

    /// Returns a mutable reference to the initialized portion of the buffer. <br>返回对缓冲区初始化部分的可变引用。<br>
    ///
    /// This includes the filled portion. <br>这包括填充的部分。<br>
    #[inline]
    pub fn initialized_mut(&mut self) -> &mut [u8] {
        // SAFETY: We only slice the initialized part of the buffer, which is always valid <br>我们只对缓冲区的初始化部分进行切片，这始终是有效的<br>
        unsafe { MaybeUninit::slice_assume_init_mut(&mut self.buf[0..self.initialized]) }
    }

    /// Returns a mutable reference to the unfilled part of the buffer without ensuring that it has been fully initialized. <br>返回对缓冲区未填充部分的可变引用，而不确保缓冲区已完全初始化。<br>
    ///
    ///
    /// # Safety
    ///
    /// The caller must not de-initialize portions of the buffer that have already been initialized. <br>调用者不得取消已初始化的缓冲区部分的初始化。<br>
    #[inline]
    pub unsafe fn unfilled_mut(&mut self) -> &mut [MaybeUninit<u8>] {
        &mut self.buf[self.filled..]
    }

    /// Returns a mutable reference to the uninitialized part of the buffer. <br>返回对缓冲区未初始化部分的可变引用。<br>
    ///
    /// It is safe to uninitialize any of these bytes. <br>取消初始化这些字节中的任何一个都是安全的。<br>
    #[inline]
    pub fn uninitialized_mut(&mut self) -> &mut [MaybeUninit<u8>] {
        &mut self.buf[self.initialized..]
    }

    /// Returns a mutable reference to the unfilled part of the buffer, ensuring it is fully initialized. <br>返回对缓冲区未填充部分的可变引用，确保它已完全初始化。<br>
    ///
    /// Since `ReadBuf` tracks the region of the buffer that has been initialized, this is effectively "free" after the first use. <br>由于 `ReadBuf` 跟踪已初始化的缓冲区区域，因此在第一次使用后实际上是 "free"。<br>
    ///
    #[inline]
    pub fn initialize_unfilled(&mut self) -> &mut [u8] {
        // should optimize out the assertion <br>应该优化断言<br>
        self.initialize_unfilled_to(self.remaining())
    }

    /// Returns a mutable reference to the first `n` bytes of the unfilled part of the buffer, ensuring it is fully initialized. <br>返回对缓冲区未填充部分的第一个 `n` 字节的可变引用，确保它已完全初始化。<br>
    ///
    ///
    /// # Panics
    ///
    /// Panics if `self.remaining()` is less than `n`. <br>如果 `self.remaining()` 小于 `n`，就会出现 panics。<br>
    #[inline]
    pub fn initialize_unfilled_to(&mut self, n: usize) -> &mut [u8] {
        assert!(self.remaining() >= n);

        let extra_init = self.initialized - self.filled;
        // If we don't have enough initialized, do zeroing <br>如果我们没有足够的初始化，就归零<br>
        if n > extra_init {
            let uninit = n - extra_init;
            let unfilled = &mut self.uninitialized_mut()[0..uninit];

            for byte in unfilled.iter_mut() {
                byte.write(0);
            }

            // SAFETY: we just initialized uninit bytes, and the previous bytes were already init <br>我们刚刚初始化了 uninit 字节，前面的字节已经初始化了<br>
            unsafe {
                self.assume_init(n);
            }
        }

        let filled = self.filled;

        &mut self.initialized_mut()[filled..filled + n]
    }

    /// Returns the number of bytes at the end of the slice that have not yet been filled. <br>返回切片末尾尚未填充的字节数。<br>
    #[inline]
    pub fn remaining(&self) -> usize {
        self.capacity() - self.filled
    }

    /// Clears the buffer, resetting the filled region to empty. <br>清除缓冲区，将填充区域重置为空。<br>
    ///
    /// The number of initialized bytes is not changed, and the contents of the buffer are not modified. <br>初始化的字节数不变，缓冲区的内容也不变。<br>
    #[inline]
    pub fn clear(&mut self) -> &mut Self {
        self.set_filled(0) // The assertion in `set_filled` is optimized out <br>`set_filled` 中的断言优化掉了<br>
    }

    /// Increases the size of the filled region of the buffer. <br>增加缓冲区填充区域的大小。<br>
    ///
    /// The number of initialized bytes is not changed. <br>初始化字节数不变。<br>
    ///
    /// # Panics
    ///
    /// Panics if the filled region of the buffer would become larger than the initialized region. <br>如果缓冲区的填充区域大于初始化区域，就会出现 panics。<br>
    #[inline]
    pub fn add_filled(&mut self, n: usize) -> &mut Self {
        self.set_filled(self.filled + n)
    }

    /// Sets the size of the filled region of the buffer. <br>设置缓冲区填充区域的大小。<br>
    ///
    /// The number of initialized bytes is not changed. <br>初始化字节数不变。<br>
    ///
    /// Note that this can be used to *shrink* the filled region of the buffer in addition to growing it (for example, by a `Read` implementation that compresses data in-place). <br>请注意，除了增加缓冲区的填充区域 (例如，通过就地压缩数据的 `Read` 实现) 之外，这还可用于*缩小*缓冲区的填充区域。<br>
    ///
    ///
    /// # Panics
    ///
    /// Panics if the filled region of the buffer would become larger than the initialized region. <br>如果缓冲区的填充区域大于初始化区域，就会出现 panics。<br>
    #[inline]
    pub fn set_filled(&mut self, n: usize) -> &mut Self {
        assert!(n <= self.initialized);

        self.filled = n;
        self
    }

    /// Asserts that the first `n` unfilled bytes of the buffer are initialized. <br>断言缓冲区的前 `n` 个未填充字节已初始化。<br>
    ///
    /// `ReadBuf` assumes that bytes are never de-initialized, so this method does nothing when called with fewer bytes than are already known to be initialized. <br>`ReadBuf` 假定字节永远不会被反初始化，因此当使用少于已知已初始化字节的字节调用此方法时，不会执行任何操作。<br>
    ///
    ///
    /// # Safety
    ///
    /// The caller must ensure that the first `n` unfilled bytes of the buffer have already been initialized. <br>调用者必须确保缓冲区的前 `n` 个未填充字节已经初始化。<br>
    #[inline]
    pub unsafe fn assume_init(&mut self, n: usize) -> &mut Self {
        self.initialized = cmp::max(self.initialized, self.filled + n);
        self
    }

    /// Appends data to the buffer, advancing the written position and possibly also the initialized position. <br>将数据追加到缓冲区，推进写入位置，也可能推进初始化位置。<br>
    ///
    /// # Panics
    ///
    /// Panics if `self.remaining()` is less than `buf.len()`. <br>如果 `self.remaining()` 小于 `buf.len()`，就会出现 panics。<br>
    #[inline]
    pub fn append(&mut self, buf: &[u8]) {
        assert!(self.remaining() >= buf.len());

        // SAFETY: we do not de-initialize any of the elements of the slice <br>我们不会取消初始化切片的任何元素<br>
        unsafe {
            MaybeUninit::write_slice(&mut self.unfilled_mut()[..buf.len()], buf);
        }

        // SAFETY: We just added the entire contents of buf to the filled section. <br>我们只是将 buf 的全部内容添加到了填充部分。<br>
        unsafe {
            self.assume_init(buf.len());
        }
        self.add_filled(buf.len());
    }

    /// Returns the amount of bytes that have been filled. <br>返回已填充的字节数。<br>
    #[inline]
    pub fn filled_len(&self) -> usize {
        self.filled
    }

    /// Returns the amount of bytes that have been initialized. <br>返回已初始化的字节数。<br>
    #[inline]
    pub fn initialized_len(&self) -> usize {
        self.initialized
    }
}
