#![cfg_attr(test, allow(unused))]

#[cfg(test)]
mod tests;

use crate::io::prelude::*;

use crate::cell::{Cell, RefCell};
use crate::fmt;
use crate::io::{self, BufReader, IoSlice, IoSliceMut, LineWriter, Lines};
use crate::pin::Pin;
use crate::sync::atomic::{AtomicBool, Ordering};
use crate::sync::{Arc, Mutex, MutexGuard, OnceLock};
use crate::sys::stdio;
use crate::sys_common::remutex::{ReentrantMutex, ReentrantMutexGuard};

type LocalStream = Arc<Mutex<Vec<u8>>>;

thread_local! {
    /// 测试 crate 用于捕获打印宏和 panics 的输出。
    static OUTPUT_CAPTURE: Cell<Option<LocalStream>> = {
        Cell::new(None)
    }
}

/// 指示使用 OUTPUT_CAPTURE 的标志。
///
/// 如果它为 None 并且从未在任何线程上设置，则此标志设置为 false，并且可以安全地在所有线程上忽略 OUTPUT_CAPTURE，从而节省了时间和内存，以在本地注册未使用的线程。
///
/// 有关内存排序的注意事项：它包含有关是否可能正在使用线程局部变量的信息。
/// 尽管这是一个 γ 标志，但线程之间的内存顺序无关紧要：我们只希望此标志在 set_output_capture 和 print_to *within the 相同线程* 之间具有一致的顺序。
///
/// 在同一线程中，事物始终具有完全一致的顺序。
/// 所以 Ordering::Relaxed 很好。
///
///
///
static OUTPUT_CAPTURE_USED: AtomicBool = AtomicBool::new(false);

/// 此进程的标准输入流的原始实例的句柄。
///
/// 此句柄不以任何方式同步或缓冲。
/// 通过 `std::io::stdio::stdin_raw` 函数构造。
struct StdinRaw(stdio::Stdin);

/// 此进程的标准输出流的原始实例的句柄。
///
/// 此句柄不以任何方式同步或缓冲。
/// 通过 `std::io::stdio::stdout_raw` 函数构造。
struct StdoutRaw(stdio::Stdout);

/// 此进程的标准输出流的原始实例的句柄。
///
/// 此句柄不以任何方式同步或缓冲。
/// 通过 `std::io::stdio::stderr_raw` 函数构造。
struct StderrRaw(stdio::Stderr);

/// 为这个进程的标准输入创建一个新的原始句柄。
///
/// 返回的句柄不与任何其他创建的句柄交互，也不与 `std::io::stdin` 返回的句柄交互。
/// `std::io::stdin` 句柄缓冲的数据对于从此函数返回的原始句柄不可用。
///
/// 返回的句柄没有外部同步或缓冲。
///
#[unstable(feature = "libstd_sys_internals", issue = "none")]
const fn stdin_raw() -> StdinRaw {
    StdinRaw(stdio::Stdin::new())
}

/// 为该进程的标准输出流创建一个新的原始句柄。
///
/// 返回的句柄不与任何其他创建的句柄交互，也不与 `std::io::stdout` 返回的句柄交互。
/// 请注意，数据由 `std::io::stdout` 句柄缓冲，因此通过此原始句柄发生的写操作可能会在之前的写操作之前出现。
///
///
/// 返回的句柄顶部没有分层的外部同步或缓冲。
///
///
#[unstable(feature = "libstd_sys_internals", issue = "none")]
const fn stdout_raw() -> StdoutRaw {
    StdoutRaw(stdio::Stdout::new())
}

/// 为该进程的标准错误流创建一个新的原始句柄。
///
/// 返回的句柄不与任何其他创建的句柄交互，也不与 `std::io::stderr` 返回的句柄交互。
///
///
/// 返回的句柄顶部没有分层的外部同步或缓冲。
///
#[unstable(feature = "libstd_sys_internals", issue = "none")]
const fn stderr_raw() -> StderrRaw {
    StderrRaw(stdio::Stderr::new())
}

impl Read for StdinRaw {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        handle_ebadf(self.0.read(buf), 0)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        handle_ebadf(self.0.read_vectored(bufs), 0)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.0.is_read_vectored()
    }

    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
        handle_ebadf(self.0.read_to_end(buf), 0)
    }

    fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
        handle_ebadf(self.0.read_to_string(buf), 0)
    }
}

impl Write for StdoutRaw {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        handle_ebadf(self.0.write(buf), buf.len())
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        let total = bufs.iter().map(|b| b.len()).sum();
        handle_ebadf(self.0.write_vectored(bufs), total)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.0.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        handle_ebadf(self.0.flush(), ())
    }

    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        handle_ebadf(self.0.write_all(buf), ())
    }

    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        handle_ebadf(self.0.write_all_vectored(bufs), ())
    }

    fn write_fmt(&mut self, fmt: fmt::Arguments<'_>) -> io::Result<()> {
        handle_ebadf(self.0.write_fmt(fmt), ())
    }
}

impl Write for StderrRaw {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        handle_ebadf(self.0.write(buf), buf.len())
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        let total = bufs.iter().map(|b| b.len()).sum();
        handle_ebadf(self.0.write_vectored(bufs), total)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.0.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        handle_ebadf(self.0.flush(), ())
    }

    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        handle_ebadf(self.0.write_all(buf), ())
    }

    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        handle_ebadf(self.0.write_all_vectored(bufs), ())
    }

    fn write_fmt(&mut self, fmt: fmt::Arguments<'_>) -> io::Result<()> {
        handle_ebadf(self.0.write_fmt(fmt), ())
    }
}

fn handle_ebadf<T>(r: io::Result<T>, default: T) -> io::Result<T> {
    match r {
        Err(ref e) if stdio::is_ebadf(e) => Ok(default),
        r => r,
    }
}

/// 进程的标准输入流的句柄。
///
/// 每个句柄都是对该进程输入数据的全局缓冲区的共享引用。可以对句柄进行 `lock`，以获取对 [`BufRead`] 方法 (例如 `.lines()`) 的完全访问权限。
/// 否则，将针对其他读取锁定对此句柄的读取。
///
/// 该句柄实现了 `Read` trait，但请注意，必须谨慎执行 `Stdin` 的并发读取。
///
/// 由 [`io::stdin`] 方法创建。
///
/// [`io::stdin`]: stdin
///
/// ### Note: Windows 可移植性注意事项
///
/// 在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。
/// 尝试读取无效的 UTF-8 字节将返回错误。
///
/// 在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。
///
/// 在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。
/// 通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。
///
/// # Examples
///
/// ```no_run
/// use std::io;
///
/// fn main() -> io::Result<()> {
///     let mut buffer = String::new();
///     let mut stdin = io::stdin(); // 我们在这里得到 `Stdin`。
///     stdin.read_line(&mut buffer)?;
///     Ok(())
/// }
/// ```
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Stdin {
    inner: &'static Mutex<BufReader<StdinRaw>>,
}

/// [`Stdin`] 句柄的锁定引用。
///
/// 该句柄同时实现 [`Read`] 和 [`BufRead`] traits，并通过 [`Stdin::lock`] 方法构造。
///
/// ### Note: Windows 可移植性注意事项
///
/// 在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。
/// 尝试读取无效的 UTF-8 字节将返回错误。
///
/// 在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。
///
/// 在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。
/// 通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。
///
/// # Examples
///
/// ```no_run
/// use std::io::{self, BufRead};
///
/// fn main() -> io::Result<()> {
///     let mut buffer = String::new();
///     let stdin = io::stdin(); // 我们在这里得到 `Stdin`。
///     {
///         let mut handle = stdin.lock(); // 我们在这里得到 `StdinLock`。
///         handle.read_line(&mut buffer)?;
///     } // `StdinLock` 被丢弃在这里。
///     Ok(())
/// }
/// ```
///
///
///
#[must_use = "if unused stdin will immediately unlock"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct StdinLock<'a> {
    inner: MutexGuard<'a, BufReader<StdinRaw>>,
}

/// 为当前进程的标准输入创建一个新的句柄。
///
/// 返回的每个句柄都是对共享缓冲区的引用，该缓冲区的访问通过互斥锁进行同步。
/// 如果需要对锁定进行更明确的控制，请参见 [`Stdin::lock`] 方法。
///
/// ### Note: Windows 可移植性注意事项
///
/// 在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。
/// 尝试读取无效的 UTF-8 字节将返回错误。
///
/// 在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。
///
/// 在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。
/// 通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。
///
/// # Examples
///
/// 使用隐式同步：
///
/// ```no_run
/// use std::io;
///
/// fn main() -> io::Result<()> {
///     let mut buffer = String::new();
///     io::stdin().read_line(&mut buffer)?;
///     Ok(())
/// }
/// ```
///
/// 使用显式同步：
///
/// ```no_run
/// use std::io::{self, BufRead};
///
/// fn main() -> io::Result<()> {
///     let mut buffer = String::new();
///     let stdin = io::stdin();
///     let mut handle = stdin.lock();
///
///     handle.read_line(&mut buffer)?;
///     Ok(())
/// }
/// ```
///
///
///
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn stdin() -> Stdin {
    static INSTANCE: OnceLock<Mutex<BufReader<StdinRaw>>> = OnceLock::new();
    Stdin {
        inner: INSTANCE.get_or_init(|| {
            Mutex::new(BufReader::with_capacity(stdio::STDIN_BUF_SIZE, stdin_raw()))
        }),
    }
}

impl Stdin {
    /// 将此句柄锁定到标准输入流，返回可读的保护。
    ///
    /// 当返回的锁离开作用域时，将释放该锁。
    /// 返回的防护还实现了用于访问底层数据的 [`Read`] 和 [`BufRead`] traits。
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::{self, BufRead};
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut buffer = String::new();
    ///     let stdin = io::stdin();
    ///     let mut handle = stdin.lock();
    ///
    ///     handle.read_line(&mut buffer)?;
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn lock(&self) -> StdinLock<'static> {
        // 用 'static 生命周期锁定这个句柄。
        // 这取决于底层 `Mutex` 是静态的实现细节。
        StdinLock { inner: self.inner.lock().unwrap_or_else(|e| e.into_inner()) }
    }

    /// 锁定此句柄并读取输入行，并将其添加到指定的缓冲区。
    ///
    /// 有关此方法的详细语义，请参见 [`BufRead::read_line`] 上的文档。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    ///
    /// let mut input = String::new();
    /// match io::stdin().read_line(&mut input) {
    ///     Ok(n) => {
    ///         println!("{n} bytes read");
    ///         println!("{input}");
    ///     }
    ///     Err(error) => println!("error: {error}"),
    /// }
    /// ```
    ///
    /// 您可以通过以下两种方式之一运行示例：
    ///
    /// - 通过管道将一些文本传递给它，例如 `printf foo | path/to/executable`
    /// - 通过直接运行可执行文件以交互方式输入文本，在这种情况下，它将等待按下 Enter 键，然后继续
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn read_line(&self, buf: &mut String) -> io::Result<usize> {
        self.lock().read_line(buf)
    }

    /// 消费这个句柄并在输入行上返回一个迭代器。
    ///
    /// 有关此方法的详细语义，请参见 [`BufRead::lines`] 上的文档。
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    ///
    /// let lines = io::stdin().lines();
    /// for line in lines {
    ///     println!("got a line: {}", line.unwrap());
    /// }
    /// ```
    #[must_use = "`self` will be dropped if the result is not used"]
    #[stable(feature = "stdin_forwarders", since = "1.62.0")]
    pub fn lines(self) -> Lines<StdinLock<'static>> {
        self.lock().lines()
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Stdin {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Stdin").finish_non_exhaustive()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Read for Stdin {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.lock().read(buf)
    }
    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.lock().read_vectored(bufs)
    }
    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.lock().is_read_vectored()
    }
    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
        self.lock().read_to_end(buf)
    }
    fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
        self.lock().read_to_string(buf)
    }
    fn read_exact(&mut self, buf: &mut [u8]) -> io::Result<()> {
        self.lock().read_exact(buf)
    }
}

// 仅由依赖于平台的 io::copy 专业使用，即在某些平台上未使用
#[cfg(any(target_os = "linux", target_os = "android"))]
impl StdinLock<'_> {
    pub(crate) fn as_mut_buf(&mut self) -> &mut BufReader<impl Read> {
        &mut self.inner
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Read for StdinLock<'_> {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.inner.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.inner.read_vectored(bufs)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.inner.is_read_vectored()
    }

    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
        self.inner.read_to_end(buf)
    }

    fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
        self.inner.read_to_string(buf)
    }

    fn read_exact(&mut self, buf: &mut [u8]) -> io::Result<()> {
        self.inner.read_exact(buf)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl BufRead for StdinLock<'_> {
    fn fill_buf(&mut self) -> io::Result<&[u8]> {
        self.inner.fill_buf()
    }

    fn consume(&mut self, n: usize) {
        self.inner.consume(n)
    }

    fn read_until(&mut self, byte: u8, buf: &mut Vec<u8>) -> io::Result<usize> {
        self.inner.read_until(byte, buf)
    }

    fn read_line(&mut self, buf: &mut String) -> io::Result<usize> {
        self.inner.read_line(buf)
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for StdinLock<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("StdinLock").finish_non_exhaustive()
    }
}

/// 当前进程的标准输出流的句柄。
///
/// 每个句柄共享一个待写入标准输出流的数据缓冲区。
/// 访问也可以通过锁定来同步，并且可以通过 [`lock`] 方法对锁定进行显式控制。
///
/// 由 [`io::stdout`] 方法创建。
///
/// ### Note: Windows 可移植性注意事项
///
/// 在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。
/// 尝试写入无效的 UTF-8 字节将返回错误。
///
/// 在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。
///
/// 在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。
/// 通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。
///
/// [`lock`]: Stdout::lock
/// [`io::stdout`]: stdout
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Stdout {
    // FIXME: 根据 stdout 的状态 (是否为 tty)，它应该是 LineWriter 或 BufWriter。
    // 请注意，如果这不是行缓冲的，则还应该在 panic 上进行刷新或在终止时进行某种形式的刷新。
    //
    inner: Pin<&'static ReentrantMutex<RefCell<LineWriter<StdoutRaw>>>>,
}

/// [`Stdout`] 句柄的锁定引用。
///
/// 该句柄实现 [`Write`] trait，并通过 [`Stdout::lock`] 方法构造。有关更多信息，请参见其文档。
///
/// ### Note: Windows 可移植性注意事项
///
/// 在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。
/// 尝试写入无效的 UTF-8 字节将返回错误。
///
/// 在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。
///
/// 在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。
/// 通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。
///
///
///
#[must_use = "if unused stdout will immediately unlock"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct StdoutLock<'a> {
    inner: ReentrantMutexGuard<'a, RefCell<LineWriter<StdoutRaw>>>,
}

static STDOUT: OnceLock<ReentrantMutex<RefCell<LineWriter<StdoutRaw>>>> = OnceLock::new();

/// 为当前进程的标准输出创建一个新的句柄。
///
/// 返回的每个句柄都是对共享缓冲区的引用，该缓冲区的访问通过互斥锁进行同步。
/// 如果需要对锁定进行更明确的控制，请参见 [`Stdout::lock`] 方法。
///
/// ### Note: Windows 可移植性注意事项
///
/// 在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。
/// 尝试写入无效的 UTF-8 字节将返回错误。
///
/// 在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。
///
/// 在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。
/// 通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。
///
/// # Examples
///
/// 使用隐式同步：
///
/// ```no_run
/// use std::io::{self, Write};
///
/// fn main() -> io::Result<()> {
///     io::stdout().write_all(b"hello world")?;
///
///     Ok(())
/// }
/// ```
///
/// 使用显式同步：
///
/// ```no_run
/// use std::io::{self, Write};
///
/// fn main() -> io::Result<()> {
///     let stdout = io::stdout();
///     let mut handle = stdout.lock();
///
///     handle.write_all(b"hello world")?;
///
///     Ok(())
/// }
/// ```
///
///
///
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn stdout() -> Stdout {
    Stdout {
        inner: Pin::static_ref(&STDOUT).get_or_init_pin(
            || unsafe { ReentrantMutex::new(RefCell::new(LineWriter::new(stdout_raw()))) },
            |mutex| unsafe { mutex.init() },
        ),
    }
}

pub fn cleanup() {
    if let Some(instance) = STDOUT.get() {
        // 通过用零缓冲容量将行 writer 替换为 1，刷新数据并在关闭期间禁用缓冲。
        // 我们使用 try_lock() 代替 lock()，因为有人可能泄漏了 StdoutLock，否则将在此处导致死锁。
        //
        //
        //
        //
        if let Some(lock) = Pin::static_ref(instance).try_lock() {
            *lock.borrow_mut() = LineWriter::with_capacity(0, stdout_raw());
        }
    }
}

impl Stdout {
    /// 将此句柄锁定到标准输出流，返回可写防护。
    ///
    ///
    /// 当返回的锁离开作用域时，将释放该锁。
    /// 返回的防护还实现 `Write` trait 来写入数据。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::{self, Write};
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut stdout = io::stdout().lock();
    ///
    ///     stdout.write_all(b"hello world")?;
    ///
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn lock(&self) -> StdoutLock<'static> {
        // 用 'static 生命周期锁定这个句柄。
        // 这取决于底层 `ReentrantMutex` 是静态的实现细节。
        //
        StdoutLock { inner: self.inner.lock() }
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Stdout {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Stdout").finish_non_exhaustive()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Write for Stdout {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        (&*self).write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        (&*self).write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        io::Write::is_write_vectored(&&*self)
    }
    fn flush(&mut self) -> io::Result<()> {
        (&*self).flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        (&*self).write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        (&*self).write_all_vectored(bufs)
    }
    fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
        (&*self).write_fmt(args)
    }
}

#[stable(feature = "write_mt", since = "1.48.0")]
impl Write for &Stdout {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.lock().write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.lock().write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.lock().is_write_vectored()
    }
    fn flush(&mut self) -> io::Result<()> {
        self.lock().flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        self.lock().write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        self.lock().write_all_vectored(bufs)
    }
    fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
        self.lock().write_fmt(args)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Write for StdoutLock<'_> {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.inner.borrow_mut().write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.inner.borrow_mut().write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.inner.borrow_mut().is_write_vectored()
    }
    fn flush(&mut self) -> io::Result<()> {
        self.inner.borrow_mut().flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        self.inner.borrow_mut().write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        self.inner.borrow_mut().write_all_vectored(bufs)
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for StdoutLock<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("StdoutLock").finish_non_exhaustive()
    }
}

/// 进程的标准错误流的句柄。
///
/// 有关更多信息，请参见 [`io::stderr`] 方法。
///
/// [`io::stderr`]: stderr
///
/// ### Note: Windows 可移植性注意事项
///
/// 在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。
/// 尝试写入无效的 UTF-8 字节将返回错误。
///
/// 在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。
///
/// 在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。
/// 通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Stderr {
    inner: Pin<&'static ReentrantMutex<RefCell<StderrRaw>>>,
}

/// [`Stderr`] 句柄的锁定引用。
///
/// 该句柄实现 [`Write`] trait，并通过 [`Stderr::lock`] 方法构造。有关更多信息，请参见其文档。
///
/// ### Note: Windows 可移植性注意事项
///
/// 在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。
/// 尝试写入无效的 UTF-8 字节将返回错误。
///
/// 在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。
///
/// 在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。
/// 通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。
///
///
///
#[must_use = "if unused stderr will immediately unlock"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct StderrLock<'a> {
    inner: ReentrantMutexGuard<'a, RefCell<StderrRaw>>,
}

/// 为当前进程的标准错误创建一个新的句柄。
///
/// 此句柄未缓冲。
///
/// ### Note: Windows 可移植性注意事项
///
/// 在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。
/// 尝试写入无效的 UTF-8 字节将返回错误。
///
/// 在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。
///
/// 在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。
/// 通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。
///
/// # Examples
///
/// 使用隐式同步：
///
/// ```no_run
/// use std::io::{self, Write};
///
/// fn main() -> io::Result<()> {
///     io::stderr().write_all(b"hello world")?;
///
///     Ok(())
/// }
/// ```
///
/// 使用显式同步：
///
/// ```no_run
/// use std::io::{self, Write};
///
/// fn main() -> io::Result<()> {
///     let stderr = io::stderr();
///     let mut handle = stderr.lock();
///
///     handle.write_all(b"hello world")?;
///
///     Ok(())
/// }
/// ```
///
///
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn stderr() -> Stderr {
    // 请注意，与 `stdout()` 不同，我们在这里不使用 `at_exit` 注册析构函数。
    // stderr 未缓冲，因此无需运行析构函数来刷新缓冲区
    //
    static INSTANCE: OnceLock<ReentrantMutex<RefCell<StderrRaw>>> = OnceLock::new();

    Stderr {
        inner: Pin::static_ref(&INSTANCE).get_or_init_pin(
            || unsafe { ReentrantMutex::new(RefCell::new(stderr_raw())) },
            |mutex| unsafe { mutex.init() },
        ),
    }
}

impl Stderr {
    /// 将此句柄锁定到标准错误流，返回可写防护。
    ///
    ///
    /// 当返回的锁离开作用域时，将释放该锁。
    /// 返回的防护还实现 [`Write`] trait 来写入数据。
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::{self, Write};
    ///
    /// fn foo() -> io::Result<()> {
    ///     let stderr = io::stderr();
    ///     let mut handle = stderr.lock();
    ///
    ///     handle.write_all(b"hello world")?;
    ///
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn lock(&self) -> StderrLock<'static> {
        // 用 'static 生命周期锁定这个句柄。
        // 这取决于底层 `ReentrantMutex` 是静态的实现细节。
        //
        StderrLock { inner: self.inner.lock() }
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Stderr {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Stderr").finish_non_exhaustive()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Write for Stderr {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        (&*self).write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        (&*self).write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        io::Write::is_write_vectored(&&*self)
    }
    fn flush(&mut self) -> io::Result<()> {
        (&*self).flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        (&*self).write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        (&*self).write_all_vectored(bufs)
    }
    fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
        (&*self).write_fmt(args)
    }
}

#[stable(feature = "write_mt", since = "1.48.0")]
impl Write for &Stderr {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.lock().write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.lock().write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.lock().is_write_vectored()
    }
    fn flush(&mut self) -> io::Result<()> {
        self.lock().flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        self.lock().write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        self.lock().write_all_vectored(bufs)
    }
    fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
        self.lock().write_fmt(args)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Write for StderrLock<'_> {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.inner.borrow_mut().write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.inner.borrow_mut().write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.inner.borrow_mut().is_write_vectored()
    }
    fn flush(&mut self) -> io::Result<()> {
        self.inner.borrow_mut().flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        self.inner.borrow_mut().write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        self.inner.borrow_mut().write_all_vectored(bufs)
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for StderrLock<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("StderrLock").finish_non_exhaustive()
    }
}

/// 设置线程本地输出捕获缓冲区并返回旧的缓冲区。
#[unstable(
    feature = "internal_output_capture",
    reason = "this function is meant for use in the test crate \
        and may disappear in the future",
    issue = "none"
)]
#[doc(hidden)]
pub fn set_output_capture(sink: Option<LocalStream>) -> Option<LocalStream> {
    if sink.is_none() && !OUTPUT_CAPTURE_USED.load(Ordering::Relaxed) {
        // OUTPUT_CAPTURE 绝对为 None，因为 OUTPUT_CAPTURE_USED 为 false。
        return None;
    }
    OUTPUT_CAPTURE_USED.store(true, Ordering::Relaxed);
    OUTPUT_CAPTURE.with(move |slot| slot.replace(sink))
}

/// 将 `args` 写入捕获缓冲区 (如果已启用且可能的话)，否则将其写入 `global_s`。`label` 在 panic 消息中标识流。
///
/// 此函数用于打印错误消息，因此在 `local_s` 不可用时要格外小心，以免引起 panic。
/// 例如，如果本地流的 TLS 密钥已被破坏，或者如果本地流已被另一个线程锁定，它就会回退到全局流。
///
///
/// 但是，如果实际的 I/O 导致错误，则此函数执行 panic。
///
///
///
fn print_to<T>(args: fmt::Arguments<'_>, global_s: fn() -> T, label: &str)
where
    T: Write,
{
    if OUTPUT_CAPTURE_USED.load(Ordering::Relaxed)
        && OUTPUT_CAPTURE.try_with(|s| {
            // 请注意，如果我们以递归方式打印 panics/prints，我们将完全删除要写入的本地 sink，因此，递归 panic/print 会转到 sink，而不是我们的本地 sink。
            //
            //
            s.take().map(|w| {
                let _ = w.lock().unwrap_or_else(|e| e.into_inner()).write_fmt(args);
                s.set(Some(w));
            })
        }) == Ok(Some(()))
    {
        // 已成功写入捕获缓冲区。
        return;
    }

    if let Err(e) = global_s().write_fmt(args) {
        panic!("failed printing to {label}: {e}");
    }
}

#[unstable(
    feature = "print_internals",
    reason = "implementation detail which may disappear or be replaced at any time",
    issue = "none"
)]
#[doc(hidden)]
#[cfg(not(test))]
pub fn _print(args: fmt::Arguments<'_>) {
    print_to(args, stdout, "stdout");
}

#[unstable(
    feature = "print_internals",
    reason = "implementation detail which may disappear or be replaced at any time",
    issue = "none"
)]
#[doc(hidden)]
#[cfg(not(test))]
pub fn _eprint(args: fmt::Arguments<'_>) {
    print_to(args, stderr, "stderr");
}

#[cfg(test)]
pub use realstd::io::{_eprint, _print};
