#![cfg_attr(test, allow(unused))]

#[cfg(test)]
mod tests;

use crate::io::prelude::*;

use crate::cell::{Cell, RefCell};
use crate::fmt;
use crate::io::{self, BufReader, IoSlice, IoSliceMut, LineWriter, Lines};
use crate::pin::Pin;
use crate::sync::atomic::{AtomicBool, Ordering};
use crate::sync::{Arc, Mutex, MutexGuard, OnceLock};
use crate::sys::stdio;
use crate::sys_common::remutex::{ReentrantMutex, ReentrantMutexGuard};

type LocalStream = Arc<Mutex<Vec<u8>>>;

thread_local! {
    /// Used by the test crate to capture the output of the print macros and panics. <br>测试 crate 用于捕获打印宏和 panics 的输出。<br>
    static OUTPUT_CAPTURE: Cell<Option<LocalStream>> = {
        Cell::new(None)
    }
}

/// Flag to indicate OUTPUT_CAPTURE is used. <br>指示使用 OUTPUT_CAPTURE 的标志。<br>
///
/// If it is None and was never set on any thread, this flag is set to false, and OUTPUT_CAPTURE can be safely ignored on all threads, saving some time and memory registering an unused thread local. <br>如果它为 None 并且从未在任何线程上设置，则此标志设置为 false，并且可以安全地在所有线程上忽略 OUTPUT_CAPTURE，从而节省了时间和内存，以在本地注册未使用的线程。<br>
///
/// Note about memory ordering: This contains information about whether a thread local variable might be in use. <br>有关内存排序的注意事项：它包含有关是否可能正在使用线程局部变量的信息。<br>
/// Although this is a global flag, the memory ordering between threads does not matter: we only want this flag to have a consistent order between set_output_capture and print_to *within the same thread*. <br>尽管这是一个 γ 标志，但线程之间的内存顺序无关紧要：我们只希望此标志在 set_output_capture 和 print_to *within the 相同线程* 之间具有一致的顺序。<br>
///
/// Within the same thread, things always have a perfectly consistent order. <br>在同一线程中，事物始终具有完全一致的顺序。<br>
/// So Ordering::Relaxed is fine. <br>所以 Ordering::Relaxed 很好。<br>
///
///
///
static OUTPUT_CAPTURE_USED: AtomicBool = AtomicBool::new(false);

/// A handle to a raw instance of the standard input stream of this process. <br>此进程的标准输入流的原始实例的句柄。<br>
///
/// This handle is not synchronized or buffered in any fashion. <br>此句柄不以任何方式同步或缓冲。<br>
/// Constructed via the `std::io::stdio::stdin_raw` function. <br>通过 `std::io::stdio::stdin_raw` 函数构造。<br>
struct StdinRaw(stdio::Stdin);

/// A handle to a raw instance of the standard output stream of this process. <br>此进程的标准输出流的原始实例的句柄。<br>
///
/// This handle is not synchronized or buffered in any fashion. <br>此句柄不以任何方式同步或缓冲。<br>
/// Constructed via the `std::io::stdio::stdout_raw` function. <br>通过 `std::io::stdio::stdout_raw` 函数构造。<br>
struct StdoutRaw(stdio::Stdout);

/// A handle to a raw instance of the standard output stream of this process. <br>此进程的标准输出流的原始实例的句柄。<br>
///
/// This handle is not synchronized or buffered in any fashion. <br>此句柄不以任何方式同步或缓冲。<br>
/// Constructed via the `std::io::stdio::stderr_raw` function. <br>通过 `std::io::stdio::stderr_raw` 函数构造。<br>
struct StderrRaw(stdio::Stderr);

/// Constructs a new raw handle to the standard input of this process. <br>为这个进程的标准输入创建一个新的原始句柄。<br>
///
/// The returned handle does not interact with any other handles created nor handles returned by `std::io::stdin`. <br>返回的句柄不与任何其他创建的句柄交互，也不与 `std::io::stdin` 返回的句柄交互。<br>
/// Data buffered by the `std::io::stdin` handles is **not** available to raw handles returned from this function. <br>`std::io::stdin` 句柄缓冲的数据对于从此函数返回的原始句柄不可用。<br>
///
/// The returned handle has no external synchronization or buffering. <br>返回的句柄没有外部同步或缓冲。<br>
///
#[unstable(feature = "libstd_sys_internals", issue = "none")]
const fn stdin_raw() -> StdinRaw {
    StdinRaw(stdio::Stdin::new())
}

/// Constructs a new raw handle to the standard output stream of this process. <br>为该进程的标准输出流创建一个新的原始句柄。<br>
///
/// The returned handle does not interact with any other handles created nor handles returned by `std::io::stdout`. <br>返回的句柄不与任何其他创建的句柄交互，也不与 `std::io::stdout` 返回的句柄交互。<br>
/// Note that data is buffered by the `std::io::stdout` handles so writes which happen via this raw handle may appear before previous writes. <br>请注意，数据由 `std::io::stdout` 句柄缓冲，因此通过此原始句柄发生的写操作可能会在之前的写操作之前出现。<br>
///
///
/// The returned handle has no external synchronization or buffering layered on top. <br>返回的句柄顶部没有分层的外部同步或缓冲。<br>
///
///
#[unstable(feature = "libstd_sys_internals", issue = "none")]
const fn stdout_raw() -> StdoutRaw {
    StdoutRaw(stdio::Stdout::new())
}

/// Constructs a new raw handle to the standard error stream of this process. <br>为该进程的标准错误流创建一个新的原始句柄。<br>
///
/// The returned handle does not interact with any other handles created nor handles returned by `std::io::stderr`. <br>返回的句柄不与任何其他创建的句柄交互，也不与 `std::io::stderr` 返回的句柄交互。<br>
///
///
/// The returned handle has no external synchronization or buffering layered on top. <br>返回的句柄顶部没有分层的外部同步或缓冲。<br>
///
#[unstable(feature = "libstd_sys_internals", issue = "none")]
const fn stderr_raw() -> StderrRaw {
    StderrRaw(stdio::Stderr::new())
}

impl Read for StdinRaw {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        handle_ebadf(self.0.read(buf), 0)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        handle_ebadf(self.0.read_vectored(bufs), 0)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.0.is_read_vectored()
    }

    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
        handle_ebadf(self.0.read_to_end(buf), 0)
    }

    fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
        handle_ebadf(self.0.read_to_string(buf), 0)
    }
}

impl Write for StdoutRaw {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        handle_ebadf(self.0.write(buf), buf.len())
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        let total = bufs.iter().map(|b| b.len()).sum();
        handle_ebadf(self.0.write_vectored(bufs), total)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.0.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        handle_ebadf(self.0.flush(), ())
    }

    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        handle_ebadf(self.0.write_all(buf), ())
    }

    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        handle_ebadf(self.0.write_all_vectored(bufs), ())
    }

    fn write_fmt(&mut self, fmt: fmt::Arguments<'_>) -> io::Result<()> {
        handle_ebadf(self.0.write_fmt(fmt), ())
    }
}

impl Write for StderrRaw {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        handle_ebadf(self.0.write(buf), buf.len())
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        let total = bufs.iter().map(|b| b.len()).sum();
        handle_ebadf(self.0.write_vectored(bufs), total)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.0.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        handle_ebadf(self.0.flush(), ())
    }

    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        handle_ebadf(self.0.write_all(buf), ())
    }

    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        handle_ebadf(self.0.write_all_vectored(bufs), ())
    }

    fn write_fmt(&mut self, fmt: fmt::Arguments<'_>) -> io::Result<()> {
        handle_ebadf(self.0.write_fmt(fmt), ())
    }
}

fn handle_ebadf<T>(r: io::Result<T>, default: T) -> io::Result<T> {
    match r {
        Err(ref e) if stdio::is_ebadf(e) => Ok(default),
        r => r,
    }
}

/// A handle to the standard input stream of a process. <br>进程的标准输入流的句柄。<br>
///
/// Each handle is a shared reference to a global buffer of input data to this process. <br>每个句柄都是对该进程输入数据的全局缓冲区的共享引用。<br> A handle can be `lock`'d to gain full access to [`BufRead`] methods (e.g., `.lines()`). <br>可以对句柄进行 `lock`，以获取对 [`BufRead`] 方法 (例如 `.lines()`) 的完全访问权限。<br>
/// Reads to this handle are otherwise locked with respect to other reads. <br>否则，将针对其他读取锁定对此句柄的读取。<br>
///
/// This handle implements the `Read` trait, but beware that concurrent reads of `Stdin` must be executed with care. <br>该句柄实现了 `Read` trait，但请注意，必须谨慎执行 `Stdin` 的并发读取。<br>
///
/// Created by the [`io::stdin`] method. <br>由 [`io::stdin`] 方法创建。<br>
///
/// [`io::stdin`]: stdin
///
/// ### Note: Windows Portability Considerations <br>Windows 可移植性注意事项<br>
///
/// When operating in a console, the Windows implementation of this stream does not support non-UTF-8 byte sequences. <br>在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。<br>
/// Attempting to read bytes that are not valid UTF-8 will return an error. <br>尝试读取无效的 UTF-8 字节将返回错误。<br>
///
/// In a process with a detached console, such as one using `#![windows_subsystem = "windows"]`, or in a child process spawned from such a process, the contained handle will be null. <br>在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。<br>
///
/// In such cases, the standard library's `Read` and `Write` will do nothing and silently succeed. <br>在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。<br>
/// All other I/O operations, via the standard library or via raw Windows API calls, will fail. <br>通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。<br>
///
/// # Examples
///
/// ```no_run
/// use std::io;
///
/// fn main() -> io::Result<()> {
///     let mut buffer = String::new();
///     let mut stdin = io::stdin(); // We get `Stdin` here. <br>我们在这里得到 `Stdin`。<br>
///     stdin.read_line(&mut buffer)?;
///     Ok(())
/// }
/// ```
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Stdin {
    inner: &'static Mutex<BufReader<StdinRaw>>,
}

/// A locked reference to the [`Stdin`] handle. <br>[`Stdin`] 句柄的锁定引用。<br>
///
/// This handle implements both the [`Read`] and [`BufRead`] traits, and is constructed via the [`Stdin::lock`] method. <br>该句柄同时实现 [`Read`] 和 [`BufRead`] traits，并通过 [`Stdin::lock`] 方法构造。<br>
///
/// ### Note: Windows Portability Considerations <br>Windows 可移植性注意事项<br>
///
/// When operating in a console, the Windows implementation of this stream does not support non-UTF-8 byte sequences. <br>在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。<br>
/// Attempting to read bytes that are not valid UTF-8 will return an error. <br>尝试读取无效的 UTF-8 字节将返回错误。<br>
///
/// In a process with a detached console, such as one using `#![windows_subsystem = "windows"]`, or in a child process spawned from such a process, the contained handle will be null. <br>在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。<br>
///
/// In such cases, the standard library's `Read` and `Write` will do nothing and silently succeed. <br>在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。<br>
/// All other I/O operations, via the standard library or via raw Windows API calls, will fail. <br>通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。<br>
///
/// # Examples
///
/// ```no_run
/// use std::io::{self, BufRead};
///
/// fn main() -> io::Result<()> {
///     let mut buffer = String::new();
///     let stdin = io::stdin(); // We get `Stdin` here. <br>我们在这里得到 `Stdin`。<br>
///     {
///         let mut handle = stdin.lock(); // We get `StdinLock` here. <br>我们在这里得到 `StdinLock`。<br>
///         handle.read_line(&mut buffer)?;
///     } // `StdinLock` is dropped here. <br>`StdinLock` 被丢弃在这里。<br>
///     Ok(())
/// }
/// ```
///
///
///
#[must_use = "if unused stdin will immediately unlock"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct StdinLock<'a> {
    inner: MutexGuard<'a, BufReader<StdinRaw>>,
}

/// Constructs a new handle to the standard input of the current process. <br>为当前进程的标准输入创建一个新的句柄。<br>
///
/// Each handle returned is a reference to a shared global buffer whose access is synchronized via a mutex. <br>返回的每个句柄都是对共享缓冲区的引用，该缓冲区的访问通过互斥锁进行同步。<br>
/// If you need more explicit control over locking, see the [`Stdin::lock`] method. <br>如果需要对锁定进行更明确的控制，请参见 [`Stdin::lock`] 方法。<br>
///
/// ### Note: Windows Portability Considerations <br>Windows 可移植性注意事项<br>
///
/// When operating in a console, the Windows implementation of this stream does not support non-UTF-8 byte sequences. <br>在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。<br>
/// Attempting to read bytes that are not valid UTF-8 will return an error. <br>尝试读取无效的 UTF-8 字节将返回错误。<br>
///
/// In a process with a detached console, such as one using `#![windows_subsystem = "windows"]`, or in a child process spawned from such a process, the contained handle will be null. <br>在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。<br>
///
/// In such cases, the standard library's `Read` and `Write` will do nothing and silently succeed. <br>在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。<br>
/// All other I/O operations, via the standard library or via raw Windows API calls, will fail. <br>通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。<br>
///
/// # Examples
///
/// Using implicit synchronization: <br>使用隐式同步：<br>
///
/// ```no_run
/// use std::io;
///
/// fn main() -> io::Result<()> {
///     let mut buffer = String::new();
///     io::stdin().read_line(&mut buffer)?;
///     Ok(())
/// }
/// ```
///
/// Using explicit synchronization: <br>使用显式同步：<br>
///
/// ```no_run
/// use std::io::{self, BufRead};
///
/// fn main() -> io::Result<()> {
///     let mut buffer = String::new();
///     let stdin = io::stdin();
///     let mut handle = stdin.lock();
///
///     handle.read_line(&mut buffer)?;
///     Ok(())
/// }
/// ```
///
///
///
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn stdin() -> Stdin {
    static INSTANCE: OnceLock<Mutex<BufReader<StdinRaw>>> = OnceLock::new();
    Stdin {
        inner: INSTANCE.get_or_init(|| {
            Mutex::new(BufReader::with_capacity(stdio::STDIN_BUF_SIZE, stdin_raw()))
        }),
    }
}

impl Stdin {
    /// Locks this handle to the standard input stream, returning a readable guard. <br>将此句柄锁定到标准输入流，返回可读的保护。<br>
    ///
    /// The lock is released when the returned lock goes out of scope. <br>当返回的锁离开作用域时，将释放该锁。<br>
    /// The returned guard also implements the [`Read`] and [`BufRead`] traits for accessing the underlying data. <br>返回的防护还实现了用于访问底层数据的 [`Read`] 和 [`BufRead`] traits。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::{self, BufRead};
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut buffer = String::new();
    ///     let stdin = io::stdin();
    ///     let mut handle = stdin.lock();
    ///
    ///     handle.read_line(&mut buffer)?;
    ///     Ok(())
    /// }
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn lock(&self) -> StdinLock<'static> {
        // Locks this handle with 'static lifetime. <br>用 'static 生命周期锁定这个句柄。<br>
        // This depends on the implementation detail that the underlying `Mutex` is static. <br>这取决于底层 `Mutex` 是静态的实现细节。<br>
        StdinLock { inner: self.inner.lock().unwrap_or_else(|e| e.into_inner()) }
    }

    /// Locks this handle and reads a line of input, appending it to the specified buffer. <br>锁定此句柄并读取输入行，并将其添加到指定的缓冲区。<br>
    ///
    /// For detailed semantics of this method, see the documentation on [`BufRead::read_line`]. <br>有关此方法的详细语义，请参见 [`BufRead::read_line`] 上的文档。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    ///
    /// let mut input = String::new();
    /// match io::stdin().read_line(&mut input) {
    ///     Ok(n) => {
    ///         println!("{n} bytes read");
    ///         println!("{input}");
    ///     }
    ///     Err(error) => println!("error: {error}"),
    /// }
    /// ```
    ///
    /// You can run the example one of two ways: <br>您可以通过以下两种方式之一运行示例：<br>
    ///
    /// - Pipe some text to it, e.g., `printf foo | path/to/executable` <br>通过管道将一些文本传递给它，例如 `printf foo | path/to/executable`<br>
    /// - Give it text interactively by running the executable directly, in which case it will wait for the Enter key to be pressed before continuing <br>通过直接运行可执行文件以交互方式输入文本，在这种情况下，它将等待按下 Enter 键，然后继续<br>
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn read_line(&self, buf: &mut String) -> io::Result<usize> {
        self.lock().read_line(buf)
    }

    /// Consumes this handle and returns an iterator over input lines. <br>消费这个句柄并在输入行上返回一个迭代器。<br>
    ///
    /// For detailed semantics of this method, see the documentation on [`BufRead::lines`]. <br>有关此方法的详细语义，请参见 [`BufRead::lines`] 上的文档。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io;
    ///
    /// let lines = io::stdin().lines();
    /// for line in lines {
    ///     println!("got a line: {}", line.unwrap());
    /// }
    /// ```
    #[must_use = "`self` will be dropped if the result is not used"]
    #[stable(feature = "stdin_forwarders", since = "1.62.0")]
    pub fn lines(self) -> Lines<StdinLock<'static>> {
        self.lock().lines()
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Stdin {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Stdin").finish_non_exhaustive()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Read for Stdin {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.lock().read(buf)
    }
    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.lock().read_vectored(bufs)
    }
    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.lock().is_read_vectored()
    }
    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
        self.lock().read_to_end(buf)
    }
    fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
        self.lock().read_to_string(buf)
    }
    fn read_exact(&mut self, buf: &mut [u8]) -> io::Result<()> {
        self.lock().read_exact(buf)
    }
}

// only used by platform-dependent io::copy specializations, i.e. unused on some platforms <br>仅由依赖于平台的 io::copy 专业使用，即在某些平台上未使用<br>
#[cfg(any(target_os = "linux", target_os = "android"))]
impl StdinLock<'_> {
    pub(crate) fn as_mut_buf(&mut self) -> &mut BufReader<impl Read> {
        &mut self.inner
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Read for StdinLock<'_> {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.inner.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.inner.read_vectored(bufs)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.inner.is_read_vectored()
    }

    fn read_to_end(&mut self, buf: &mut Vec<u8>) -> io::Result<usize> {
        self.inner.read_to_end(buf)
    }

    fn read_to_string(&mut self, buf: &mut String) -> io::Result<usize> {
        self.inner.read_to_string(buf)
    }

    fn read_exact(&mut self, buf: &mut [u8]) -> io::Result<()> {
        self.inner.read_exact(buf)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl BufRead for StdinLock<'_> {
    fn fill_buf(&mut self) -> io::Result<&[u8]> {
        self.inner.fill_buf()
    }

    fn consume(&mut self, n: usize) {
        self.inner.consume(n)
    }

    fn read_until(&mut self, byte: u8, buf: &mut Vec<u8>) -> io::Result<usize> {
        self.inner.read_until(byte, buf)
    }

    fn read_line(&mut self, buf: &mut String) -> io::Result<usize> {
        self.inner.read_line(buf)
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for StdinLock<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("StdinLock").finish_non_exhaustive()
    }
}

/// A handle to the global standard output stream of the current process. <br>当前进程的标准输出流的句柄。<br>
///
/// Each handle shares a global buffer of data to be written to the standard output stream. <br>每个句柄共享一个待写入标准输出流的数据缓冲区。<br>
/// Access is also synchronized via a lock and explicit control over locking is available via the [`lock`] method. <br>访问也可以通过锁定来同步，并且可以通过 [`lock`] 方法对锁定进行显式控制。<br>
///
/// Created by the [`io::stdout`] method. <br>由 [`io::stdout`] 方法创建。<br>
///
/// ### Note: Windows Portability Considerations <br>Windows 可移植性注意事项<br>
///
/// When operating in a console, the Windows implementation of this stream does not support non-UTF-8 byte sequences. <br>在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。<br>
/// Attempting to write bytes that are not valid UTF-8 will return an error. <br>尝试写入无效的 UTF-8 字节将返回错误。<br>
///
/// In a process with a detached console, such as one using `#![windows_subsystem = "windows"]`, or in a child process spawned from such a process, the contained handle will be null. <br>在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。<br>
///
/// In such cases, the standard library's `Read` and `Write` will do nothing and silently succeed. <br>在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。<br>
/// All other I/O operations, via the standard library or via raw Windows API calls, will fail. <br>通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。<br>
///
/// [`lock`]: Stdout::lock
/// [`io::stdout`]: stdout
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Stdout {
    // FIXME: this should be LineWriter or BufWriter depending on the state of stdout (tty or not). <br>根据 stdout 的状态 (是否为 tty)，它应该是 LineWriter 或 BufWriter。<br>
    // Note that if this is not line buffered it should also flush-on-panic or some form of flush-on-abort. <br>请注意，如果这不是行缓冲的，则还应该在 panic 上进行刷新或在终止时进行某种形式的刷新。<br>
    //
    inner: Pin<&'static ReentrantMutex<RefCell<LineWriter<StdoutRaw>>>>,
}

/// A locked reference to the [`Stdout`] handle. <br>[`Stdout`] 句柄的锁定引用。<br>
///
/// This handle implements the [`Write`] trait, and is constructed via the [`Stdout::lock`] method. <br>该句柄实现 [`Write`] trait，并通过 [`Stdout::lock`] 方法构造。<br> See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// ### Note: Windows Portability Considerations <br>Windows 可移植性注意事项<br>
///
/// When operating in a console, the Windows implementation of this stream does not support non-UTF-8 byte sequences. <br>在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。<br>
/// Attempting to write bytes that are not valid UTF-8 will return an error. <br>尝试写入无效的 UTF-8 字节将返回错误。<br>
///
/// In a process with a detached console, such as one using `#![windows_subsystem = "windows"]`, or in a child process spawned from such a process, the contained handle will be null. <br>在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。<br>
///
/// In such cases, the standard library's `Read` and `Write` will do nothing and silently succeed. <br>在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。<br>
/// All other I/O operations, via the standard library or via raw Windows API calls, will fail. <br>通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。<br>
///
///
///
#[must_use = "if unused stdout will immediately unlock"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct StdoutLock<'a> {
    inner: ReentrantMutexGuard<'a, RefCell<LineWriter<StdoutRaw>>>,
}

static STDOUT: OnceLock<ReentrantMutex<RefCell<LineWriter<StdoutRaw>>>> = OnceLock::new();

/// Constructs a new handle to the standard output of the current process. <br>为当前进程的标准输出创建一个新的句柄。<br>
///
/// Each handle returned is a reference to a shared global buffer whose access is synchronized via a mutex. <br>返回的每个句柄都是对共享缓冲区的引用，该缓冲区的访问通过互斥锁进行同步。<br>
/// If you need more explicit control over locking, see the [`Stdout::lock`] method. <br>如果需要对锁定进行更明确的控制，请参见 [`Stdout::lock`] 方法。<br>
///
/// ### Note: Windows Portability Considerations <br>Windows 可移植性注意事项<br>
///
/// When operating in a console, the Windows implementation of this stream does not support non-UTF-8 byte sequences. <br>在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。<br>
/// Attempting to write bytes that are not valid UTF-8 will return an error. <br>尝试写入无效的 UTF-8 字节将返回错误。<br>
///
/// In a process with a detached console, such as one using `#![windows_subsystem = "windows"]`, or in a child process spawned from such a process, the contained handle will be null. <br>在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。<br>
///
/// In such cases, the standard library's `Read` and `Write` will do nothing and silently succeed. <br>在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。<br>
/// All other I/O operations, via the standard library or via raw Windows API calls, will fail. <br>通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。<br>
///
/// # Examples
///
/// Using implicit synchronization: <br>使用隐式同步：<br>
///
/// ```no_run
/// use std::io::{self, Write};
///
/// fn main() -> io::Result<()> {
///     io::stdout().write_all(b"hello world")?;
///
///     Ok(())
/// }
/// ```
///
/// Using explicit synchronization: <br>使用显式同步：<br>
///
/// ```no_run
/// use std::io::{self, Write};
///
/// fn main() -> io::Result<()> {
///     let stdout = io::stdout();
///     let mut handle = stdout.lock();
///
///     handle.write_all(b"hello world")?;
///
///     Ok(())
/// }
/// ```
///
///
///
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn stdout() -> Stdout {
    Stdout {
        inner: Pin::static_ref(&STDOUT).get_or_init_pin(
            || unsafe { ReentrantMutex::new(RefCell::new(LineWriter::new(stdout_raw()))) },
            |mutex| unsafe { mutex.init() },
        ),
    }
}

pub fn cleanup() {
    if let Some(instance) = STDOUT.get() {
        // Flush the data and disable buffering during shutdown by replacing the line writer by one with zero buffering capacity. <br>通过用零缓冲容量将行 writer 替换为 1，刷新数据并在关闭期间禁用缓冲。<br>
        // We use try_lock() instead of lock(), because someone might have leaked a StdoutLock, which would otherwise cause a deadlock here. <br>我们使用 try_lock() 代替 lock()，因为有人可能泄漏了 StdoutLock，否则将在此处导致死锁。<br>
        //
        //
        //
        //
        if let Some(lock) = Pin::static_ref(instance).try_lock() {
            *lock.borrow_mut() = LineWriter::with_capacity(0, stdout_raw());
        }
    }
}

impl Stdout {
    /// Locks this handle to the standard output stream, returning a writable guard. <br>将此句柄锁定到标准输出流，返回可写防护。<br>
    ///
    ///
    /// The lock is released when the returned lock goes out of scope. <br>当返回的锁离开作用域时，将释放该锁。<br>
    /// The returned guard also implements the `Write` trait for writing data. <br>返回的防护还实现 `Write` trait 来写入数据。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::io::{self, Write};
    ///
    /// fn main() -> io::Result<()> {
    ///     let mut stdout = io::stdout().lock();
    ///
    ///     stdout.write_all(b"hello world")?;
    ///
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn lock(&self) -> StdoutLock<'static> {
        // Locks this handle with 'static lifetime. <br>用 'static 生命周期锁定这个句柄。<br>
        // This depends on the implementation detail that the underlying `ReentrantMutex` is static. <br>这取决于底层 `ReentrantMutex` 是静态的实现细节。<br>
        //
        StdoutLock { inner: self.inner.lock() }
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Stdout {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Stdout").finish_non_exhaustive()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Write for Stdout {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        (&*self).write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        (&*self).write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        io::Write::is_write_vectored(&&*self)
    }
    fn flush(&mut self) -> io::Result<()> {
        (&*self).flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        (&*self).write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        (&*self).write_all_vectored(bufs)
    }
    fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
        (&*self).write_fmt(args)
    }
}

#[stable(feature = "write_mt", since = "1.48.0")]
impl Write for &Stdout {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.lock().write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.lock().write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.lock().is_write_vectored()
    }
    fn flush(&mut self) -> io::Result<()> {
        self.lock().flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        self.lock().write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        self.lock().write_all_vectored(bufs)
    }
    fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
        self.lock().write_fmt(args)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Write for StdoutLock<'_> {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.inner.borrow_mut().write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.inner.borrow_mut().write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.inner.borrow_mut().is_write_vectored()
    }
    fn flush(&mut self) -> io::Result<()> {
        self.inner.borrow_mut().flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        self.inner.borrow_mut().write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        self.inner.borrow_mut().write_all_vectored(bufs)
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for StdoutLock<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("StdoutLock").finish_non_exhaustive()
    }
}

/// A handle to the standard error stream of a process. <br>进程的标准错误流的句柄。<br>
///
/// For more information, see the [`io::stderr`] method. <br>有关更多信息，请参见 [`io::stderr`] 方法。<br>
///
/// [`io::stderr`]: stderr
///
/// ### Note: Windows Portability Considerations <br>Windows 可移植性注意事项<br>
///
/// When operating in a console, the Windows implementation of this stream does not support non-UTF-8 byte sequences. <br>在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。<br>
/// Attempting to write bytes that are not valid UTF-8 will return an error. <br>尝试写入无效的 UTF-8 字节将返回错误。<br>
///
/// In a process with a detached console, such as one using `#![windows_subsystem = "windows"]`, or in a child process spawned from such a process, the contained handle will be null. <br>在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。<br>
///
/// In such cases, the standard library's `Read` and `Write` will do nothing and silently succeed. <br>在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。<br>
/// All other I/O operations, via the standard library or via raw Windows API calls, will fail. <br>通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。<br>
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Stderr {
    inner: Pin<&'static ReentrantMutex<RefCell<StderrRaw>>>,
}

/// A locked reference to the [`Stderr`] handle. <br>[`Stderr`] 句柄的锁定引用。<br>
///
/// This handle implements the [`Write`] trait and is constructed via the [`Stderr::lock`] method. <br>该句柄实现 [`Write`] trait，并通过 [`Stderr::lock`] 方法构造。<br> See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// ### Note: Windows Portability Considerations <br>Windows 可移植性注意事项<br>
///
/// When operating in a console, the Windows implementation of this stream does not support non-UTF-8 byte sequences. <br>在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。<br>
/// Attempting to write bytes that are not valid UTF-8 will return an error. <br>尝试写入无效的 UTF-8 字节将返回错误。<br>
///
/// In a process with a detached console, such as one using `#![windows_subsystem = "windows"]`, or in a child process spawned from such a process, the contained handle will be null. <br>在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。<br>
///
/// In such cases, the standard library's `Read` and `Write` will do nothing and silently succeed. <br>在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。<br>
/// All other I/O operations, via the standard library or via raw Windows API calls, will fail. <br>通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。<br>
///
///
///
#[must_use = "if unused stderr will immediately unlock"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct StderrLock<'a> {
    inner: ReentrantMutexGuard<'a, RefCell<StderrRaw>>,
}

/// Constructs a new handle to the standard error of the current process. <br>为当前进程的标准错误创建一个新的句柄。<br>
///
/// This handle is not buffered. <br>此句柄未缓冲。<br>
///
/// ### Note: Windows Portability Considerations <br>Windows 可移植性注意事项<br>
///
/// When operating in a console, the Windows implementation of this stream does not support non-UTF-8 byte sequences. <br>在控制台中操作时，此流的 Windows 实现不支持非 UTF-8 字节序列。<br>
/// Attempting to write bytes that are not valid UTF-8 will return an error. <br>尝试写入无效的 UTF-8 字节将返回错误。<br>
///
/// In a process with a detached console, such as one using `#![windows_subsystem = "windows"]`, or in a child process spawned from such a process, the contained handle will be null. <br>在具有分离控制台的进程中，例如使用 `#![windows_subsystem = "windows"]` 的进程，或在从此类进程派生的子进程中，包含的句柄将为空。<br>
///
/// In such cases, the standard library's `Read` and `Write` will do nothing and silently succeed. <br>在这种情况下，标准库的 `Read` 和 `Write` 将什么都不做，默默地成功。<br>
/// All other I/O operations, via the standard library or via raw Windows API calls, will fail. <br>通过标准库或通过原始 Windows API 调用的所有其他 I/O 操作都将失败。<br>
///
/// # Examples
///
/// Using implicit synchronization: <br>使用隐式同步：<br>
///
/// ```no_run
/// use std::io::{self, Write};
///
/// fn main() -> io::Result<()> {
///     io::stderr().write_all(b"hello world")?;
///
///     Ok(())
/// }
/// ```
///
/// Using explicit synchronization: <br>使用显式同步：<br>
///
/// ```no_run
/// use std::io::{self, Write};
///
/// fn main() -> io::Result<()> {
///     let stderr = io::stderr();
///     let mut handle = stderr.lock();
///
///     handle.write_all(b"hello world")?;
///
///     Ok(())
/// }
/// ```
///
///
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn stderr() -> Stderr {
    // Note that unlike `stdout()` we don't use `at_exit` here to register a destructor. <br>请注意，与 `stdout()` 不同，我们在这里不使用 `at_exit` 注册析构函数。<br>
    // Stderr is not buffered , so there's no need to run a destructor for flushing the buffer <br>stderr 未缓冲，因此无需运行析构函数来刷新缓冲区<br>
    //
    static INSTANCE: OnceLock<ReentrantMutex<RefCell<StderrRaw>>> = OnceLock::new();

    Stderr {
        inner: Pin::static_ref(&INSTANCE).get_or_init_pin(
            || unsafe { ReentrantMutex::new(RefCell::new(stderr_raw())) },
            |mutex| unsafe { mutex.init() },
        ),
    }
}

impl Stderr {
    /// Locks this handle to the standard error stream, returning a writable guard. <br>将此句柄锁定到标准错误流，返回可写防护。<br>
    ///
    ///
    /// The lock is released when the returned lock goes out of scope. <br>当返回的锁离开作用域时，将释放该锁。<br>
    /// The returned guard also implements the [`Write`] trait for writing data. <br>返回的防护还实现 [`Write`] trait 来写入数据。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::io::{self, Write};
    ///
    /// fn foo() -> io::Result<()> {
    ///     let stderr = io::stderr();
    ///     let mut handle = stderr.lock();
    ///
    ///     handle.write_all(b"hello world")?;
    ///
    ///     Ok(())
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn lock(&self) -> StderrLock<'static> {
        // Locks this handle with 'static lifetime. <br>用 'static 生命周期锁定这个句柄。<br>
        // This depends on the implementation detail that the underlying `ReentrantMutex` is static. <br>这取决于底层 `ReentrantMutex` 是静态的实现细节。<br>
        //
        StderrLock { inner: self.inner.lock() }
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Stderr {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Stderr").finish_non_exhaustive()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Write for Stderr {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        (&*self).write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        (&*self).write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        io::Write::is_write_vectored(&&*self)
    }
    fn flush(&mut self) -> io::Result<()> {
        (&*self).flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        (&*self).write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        (&*self).write_all_vectored(bufs)
    }
    fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
        (&*self).write_fmt(args)
    }
}

#[stable(feature = "write_mt", since = "1.48.0")]
impl Write for &Stderr {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.lock().write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.lock().write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.lock().is_write_vectored()
    }
    fn flush(&mut self) -> io::Result<()> {
        self.lock().flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        self.lock().write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        self.lock().write_all_vectored(bufs)
    }
    fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
        self.lock().write_fmt(args)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Write for StderrLock<'_> {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.inner.borrow_mut().write(buf)
    }
    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.inner.borrow_mut().write_vectored(bufs)
    }
    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.inner.borrow_mut().is_write_vectored()
    }
    fn flush(&mut self) -> io::Result<()> {
        self.inner.borrow_mut().flush()
    }
    fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        self.inner.borrow_mut().write_all(buf)
    }
    fn write_all_vectored(&mut self, bufs: &mut [IoSlice<'_>]) -> io::Result<()> {
        self.inner.borrow_mut().write_all_vectored(bufs)
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for StderrLock<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("StderrLock").finish_non_exhaustive()
    }
}

/// Sets the thread-local output capture buffer and returns the old one. <br>设置线程本地输出捕获缓冲区并返回旧的缓冲区。<br>
#[unstable(
    feature = "internal_output_capture",
    reason = "this function is meant for use in the test crate \
        and may disappear in the future",
    issue = "none"
)]
#[doc(hidden)]
pub fn set_output_capture(sink: Option<LocalStream>) -> Option<LocalStream> {
    if sink.is_none() && !OUTPUT_CAPTURE_USED.load(Ordering::Relaxed) {
        // OUTPUT_CAPTURE is definitely None since OUTPUT_CAPTURE_USED is false. <br>OUTPUT_CAPTURE 绝对为 None，因为 OUTPUT_CAPTURE_USED 为 false。<br>
        return None;
    }
    OUTPUT_CAPTURE_USED.store(true, Ordering::Relaxed);
    OUTPUT_CAPTURE.with(move |slot| slot.replace(sink))
}

/// Write `args` to the capture buffer if enabled and possible, or `global_s` otherwise. <br>将 `args` 写入捕获缓冲区 (如果已启用且可能的话)，否则将其写入 `global_s`。<br> `label` identifies the stream in a panic message. <br>`label` 在 panic 消息中标识流。<br>
///
/// This function is used to print error messages, so it takes extra care to avoid causing a panic when `local_s` is unusable. <br>此函数用于打印错误消息，因此在 `local_s` 不可用时要格外小心，以免引起 panic。<br>
/// For instance, if the TLS key for the local stream is already destroyed, or if the local stream is locked by another thread, it will just fall back to the global stream. <br>例如，如果本地流的 TLS 密钥已被破坏，或者如果本地流已被另一个线程锁定，它就会回退到全局流。<br>
///
///
/// However, if the actual I/O causes an error, this function does panic. <br>但是，如果实际的 I/O 导致错误，则此函数执行 panic。<br>
///
///
///
fn print_to<T>(args: fmt::Arguments<'_>, global_s: fn() -> T, label: &str)
where
    T: Write,
{
    if OUTPUT_CAPTURE_USED.load(Ordering::Relaxed)
        && OUTPUT_CAPTURE.try_with(|s| {
            // Note that we completely remove a local sink to write to in case our printing recursively panics/prints, so the recursive panic/print goes to the global sink instead of our local sink. <br>请注意，如果我们以递归方式打印 panics/prints，我们将完全删除要写入的本地 sink，因此，递归 panic/print 会转到 sink，而不是我们的本地 sink。<br>
            //
            //
            s.take().map(|w| {
                let _ = w.lock().unwrap_or_else(|e| e.into_inner()).write_fmt(args);
                s.set(Some(w));
            })
        }) == Ok(Some(()))
    {
        // Successfully wrote to capture buffer. <br>已成功写入捕获缓冲区。<br>
        return;
    }

    if let Err(e) = global_s().write_fmt(args) {
        panic!("failed printing to {label}: {e}");
    }
}

#[unstable(
    feature = "print_internals",
    reason = "implementation detail which may disappear or be replaced at any time",
    issue = "none"
)]
#[doc(hidden)]
#[cfg(not(test))]
pub fn _print(args: fmt::Arguments<'_>) {
    print_to(args, stdout, "stdout");
}

#[unstable(
    feature = "print_internals",
    reason = "implementation detail which may disappear or be replaced at any time",
    issue = "none"
)]
#[doc(hidden)]
#[cfg(not(test))]
pub fn _eprint(args: fmt::Arguments<'_>) {
    print_to(args, stderr, "stderr");
}

#[cfg(test)]
pub use realstd::io::{_eprint, _print};
