//! # The Rust Standard Library <br>Rust 标准库<br>
//!
//! The Rust Standard Library is the foundation of portable Rust software, a set of minimal and battle-tested shared abstractions for the [broader Rust ecosystem][crates.io]. <br>Rust 标准库是可移植 Rust 软件的基础，这是一组针对 [更广泛的 Rust 生态系统][crates.io] 的最小且经过实战测试的共享抽象。<br>
//! It offers core types, like [`Vec<T>`] and [`Option<T>`], library-defined [operations on language primitives](#primitives), [standard macros](#macros), [I/O] and [multithreading], among [many other things][other]. <br>它提供了核心类型，例如 [`Vec<T>`] 和 [`Option<T>`]，库定义的对 [语言原语](#primitives) 的操作，[标准库宏](#macros)，[I/O] 和 [多线程][multithreading]，以及许多 [其他][other] 东西。<br>
//!
//! `std` is available to all Rust crates by default. <br>默认情况下，`std` 可用于所有 Rust crates。<br> Therefore, the standard library can be accessed in [`use`] statements through the path `std`, as in [`use std::env`]. <br>因此，可以通过 [`use`] 语句使用路径 `std` 来访问标准库，就像在 [`use std::env`] 中一样。<br>
//!
//! # How to read this documentation <br>如何阅读本文档<br>
//!
//! If you already know the name of what you are looking for, the fastest way to find it is to use the <a href="#" onclick="focusSearchBar();">search bar</a> at the top of the page. <br>如果您已经知道要查找的内容的名称，最快的查找方法是使用页面顶部的 <a href="#" onclick="focusSearchBar();">搜索栏</a>。<br>
//!
//! Otherwise, you may want to jump to one of these useful sections: <br>否则，您可能想跳转到以下有用的部分之一：<br>
//!
//! * [`std::*` modules](#modules)
//! * [Primitive types](#primitives)
//! * [Standard macros](#macros)
//! * [The Rust Prelude]
//!
//! If this is your first time, the documentation for the standard library is written to be casually perused. <br>如果这是您第一次来，那么标准库的文档可以被随意的阅读。<br> Clicking on interesting things should generally lead you to interesting places. <br>点击有趣的东西通常会把您带到有趣的地方。<br>
//! Still, there are important bits you don't want to miss, so read on for a tour of the standard library and its documentation! <br>尽管如此，您还是不想错过一些重要的内容，因此请继续阅读标准库及其文档！<br>
//!
//! Once you are familiar with the contents of the standard library you may begin to find the verbosity of the prose distracting. <br>一旦您熟悉了标准库的内容，您可能会发现冗长的描述会使人分心。<br> At this stage in your development you may want to press the `[-]` button near the top of the page to collapse it into a more skimmable view. <br>在开发阶段，您可能需要按页面顶部附近的 `[-]` 按钮，将其折叠为更易于阅读的视图。<br>
//!
//! While you are looking at that `[-]` button also notice the `source` link. <br>当您查看 `[-]` 按钮时，还请注意 `source` 链接。<br> Rust's API documentation comes with the source code and you are encouraged to read it. <br>Rust 的 API 文档附带了源代码，我们鼓励您阅读它。<br>
//! The standard library source is generally high quality and a peek behind the curtains is often enlightening. <br>标准库的资源通常是高质量的，通常可以启发人们对幕后的了解。<br>
//!
//! # What is in the standard library documentation? <br>标准库文档中有什么？<br>
//!
//! First of all, The Rust Standard Library is divided into a number of focused modules, [all listed further down this page](#modules). <br>首先，Rust 标准库分为多个重点 [模块](#modules)，所有的这些模块都会在本页下方列出。<br> These modules are the bedrock upon which all of Rust is forged, and they have mighty names like [`std::slice`] and [`std::cmp`]. <br>这些模块是所有 Rust 锻造的基础，它们具有强大的名称，如 [`std::slice`] 和 [`std::cmp`]。<br>
//! Modules' documentation typically includes an overview of the module along with examples, and are a smart place to start familiarizing yourself with the library. <br>模块的文档通常包括模块的概述和示例，是开始熟悉库的好地方。<br>
//!
//! Second, implicit methods on [primitive types] are documented here. <br>其次，此处记录了 [原始类型][primitive types] 上的隐式方法。<br> This can be a source of confusion for two reasons: <br>造成混淆的原因有两个：<br>
//!
//! 1. While primitives are implemented by the compiler, the standard library implements methods directly on the primitive types (and it is the only library that does so), which are [documented in the section on primitives](#primitives). <br>虽然原语是由编译器实现的，但标准库是直接在原始类型上实现方法 (而且它是唯一一个这样做的库)。在 [原语](#primitives) 部分中对此进行了说明。<br>
//! 2. The standard library exports many modules *with the same name as primitive types*. <br>标准库导出了许多模块，这些模块的名称和原始类型的名称相同。<br> These define additional items related to the primitive type, but not the all-important methods. <br>它们定义了与原始类型有关的其他项，但没有定义所有重要的方法。<br>
//!
//! So for example there is a [page for the primitive type `i32`](primitive::i32) that lists all the methods that can be called on <br>例如，有一个 [原始类型为 `i32`](primitive::i32) 的页面列出了可以调用的所有方法<br>
//! 32-bit integers (very useful), and there is a [page for the module `std::i32`] that documents the constant values [`MIN`] and [`MAX`] (rarely useful). <br>32 位整数 (非常有用)，并且有一个 [`std::i32` 模块的页面][page for the module `std::i32`] 记录了常量值 [`MIN`] 和 [`MAX`] (很少有用)。<br>
//!
//! Note the documentation for the primitives [`str`] and [`[T]`][prim@slice] (also called 'slice'). <br>请注意原始 [`str`] 和 [`[T]`][prim@slice] (也称为 'slice') 的文档。<br> Many method calls on [`String`] and [`Vec<T>`] are actually calls to methods on [`str`] and [`[T]`][prim@slice] respectively, via [deref coercions][deref-coercions]. <br>[`String`] 和 [`Vec<T>`] 上许多方法的调用实际上都是通过 [解引用强制多态][deref-coercions] 分别对 [`str`] 和 [`[T]`][prim@slice] 上的方法的调用。<br>
//!
//! Third, the standard library defines [The Rust Prelude], a small collection of items - mostly traits - that are imported into every module of every crate. <br>第三，标准库还定义了 Rust [Prelude][The Rust Prelude]，这是一小部分项目 - 主要是 traits - 导入到每个 crate 的每个模块中。<br>
//! The traits in the prelude are pervasive, making the prelude documentation a good entry point to learning about the library. <br>prelude 中的 traits 无处不在，这使 prelude 文档成为了解该库的一个很好的切入点。<br>
//!
//! And finally, the standard library exports a number of standard macros, and [lists them on this page](#macros) (technically, not all of the standard macros are defined by the standard library - some are defined by the compiler - but they are documented here the same). <br>最后，标准库导出了许多标准宏，并且在 [此页面](#macros) 上列出了它们 (从技术上讲，并不是所有的标准宏都由标准库定义的 - 有些是由编译器定义的 - 但它们在这里的文档是相同的)。<br>
//!
//! Like the prelude, the standard macros are imported by default into all crates. <br>与 prelude 一样，默认情况下会将标准宏导入到所有 crates 中。<br>
//!
//! # Contributing changes to the documentation <br>对文档的更改做出贡献<br>
//!
//! Check out the rust contribution guidelines [here](https://rustc-dev-guide.rust-lang.org/contributing.html#writing-documentation). <br>在 [这里](https://rustc-dev-guide.rust-lang.org/contributing.html#writing-documentation) 查看 Rust 贡献指南。<br>
//! The source for this documentation can be found on [GitHub](https://github.com/rust-lang/rust). <br>该文档的源代码可以在 [GitHub](https://github.com/rust-lang/rust) 上找到。<br>
//! To contribute changes, make sure you read the guidelines first, then submit pull-requests for your suggested changes. <br>要贡献更改，请确保您先阅读指南，然后为您建议的更改提交拉取请求。<br>
//!
//! Contributions are appreciated! <br>感谢您的贡献！<br> If you see a part of the docs that can be improved, submit a PR, or chat with us first on [Discord][rust-discord] #docs. <br>如果您看到可以改进的部分文档，请提交 PR，或者先在 [Discord][rust-discord] #docs 上与我们聊天。<br>
//!
//! # A Tour of The Rust Standard Library <br>Rust 标准库之旅<br>
//!
//! The rest of this crate documentation is dedicated to pointing out notable features of The Rust Standard Library. <br>crate 文档的其余部分致力于指出 Rust 标准库的显著特性。<br>
//!
//! ## Containers and collections <br>容器和集合<br>
//!
//! The [`option`] and [`result`] modules define optional and error-handling types, [`Option<T>`] and [`Result<T, E>`]. <br>[`option`] 和 [`result`] 模块定义了可选和错误处理类型 [`Option<T>`] 和 [`Result<T, E>`]。<br> The [`iter`] module defines Rust's iterator trait, [`Iterator`], which works with the [`for`] loop to access collections. <br>[`iter`] 模块定义了 Rust 的迭代器 [`Iterator`] trait，它与 [`for`] 循环一起工作来访问集合。<br>
//!
//! The standard library exposes three common ways to deal with contiguous regions of memory: <br>标准库公开了三种处理连续内存区域的常用方法：<br>
//!
//! * [`Vec<T>`] - A heap-allocated *vector* that is resizable at runtime. <br>[`Vec<T>`] - 在运行时可调整大小的堆分配的 *vector*。<br>
//! * [`[T; N]`][prim@array] - An inline *array* with a fixed size at compile time. <br>[`[T; N]`][prim@array] - 在编译时具有固定大小的内联数组。<br>
//! * [`[T]`][prim@slice] - A dynamically sized *slice* into any other kind of contiguous storage, whether heap-allocated or not. <br>[`[T]`][prim@slice] - 一个动态大小的*切片*放入到任何其他类型的连续存储中，无论是否为堆分配。<br>
//!
//! Slices can only be handled through some kind of *pointer*, and as such come in many flavors such as: <br>切片只能通过某种 *指针* 来处理，因此具有多种形式，例如：<br>
//!
//! * `&[T]` - *shared slice* <br>`&[T]`- *共享切片*<br>
//! * `&mut [T]` - *mutable slice* <br>`&mut [T]`- *可变切片*<br>
//! * [`Box<[T]>`][owned slice] - *owned slice* <br>[`Box<[T]>`][owned slice] - *拥有所有权的切片*<br>
//!
//! [`str`], a UTF-8 string slice, is a primitive type, and the standard library defines many methods for it. <br>[`str`]，一个 UTF-8 字符串切片，是一种原始类型，标准库为它定义了很多方法。<br> Rust [`str`]s are typically accessed as immutable references: `&str`. <br>Rust [`str`] 通常作为不可变引用来访问: `&str`。<br> Use the owned [`String`] for building and mutating strings. <br>使用拥有所有权的 [`String`] 来创建和修改字符串。<br>
//!
//! For converting to strings use the [`format!`] macro, and for converting from strings use the [`FromStr`] trait. <br>要转换为字符串，请使用 [`format!`] 宏; 要从字符串转换，请使用 [`FromStr`] trait。<br>
//!
//! Data may be shared by placing it in a reference-counted box or the [`Rc`] type, and if further contained in a [`Cell`] or [`RefCell`], may be mutated as well as shared. <br>可以通过将数据放在引用计数的 Box 或 [`Rc`] 类型中来共享数据，并且，如果进一步包含在 [`Cell`] 或 [`RefCell`] 中，则可以对其进行可变的和共享。<br>
//! Likewise, in a concurrent setting it is common to pair an atomically-reference-counted box, [`Arc`], with a [`Mutex`] to get the same effect. <br>同样，在并发设置中，通常将原子引用计数的 Box [`Arc`] 与 [`Mutex`] 配合以获得相同的效果。<br>
//!
//! The [`collections`] module defines maps, sets, linked lists and other typical collection types, including the common [`HashMap<K, V>`]. <br>[`collections`] 模块定义了 Map，Set，链表和其他典型的集合类型，包括常见的 [`HashMap<K, V>`]。<br>
//!
//! ## Platform abstractions and I/O <br>平台抽象和 I/O<br>
//!
//! Besides basic data types, the standard library is largely concerned with abstracting over differences in common platforms, most notably Windows and Unix derivatives. <br>除了基本的数据类型外，标准库还主要关注对通用平台差异的抽象 (尤其是 Windows 和 Unix 派生平台)。<br>
//!
//! Common types of I/O, including [files], [TCP], [UDP], are defined in the [`io`], [`fs`], and [`net`] modules. <br>常见的 I/O 类型包括 [files]，[TCP] 和 [UDP]。它们被定义在 [`io`]，[`fs`] 和 [`net`] 模块中。<br>
//!
//! The [`thread`] module contains Rust's threading abstractions. <br>[`thread`] 模块包含了 Rust 的线程抽象。<br> [`sync`] contains further primitive shared memory types, including [`atomic`] and [`mpsc`], which contains the channel types for message passing. <br>[`sync`] 包含更多的原始共享内存类型，包括 [`atomic`] 和 [`mpsc`]，其中包含用于消息传递的通道类型。<br>
//!
//! [I/O]: io
//! [`MIN`]: i32::MIN
//! [`MAX`]: i32::MAX
//! [page for the module `std::i32`]: crate::i32
//! [TCP]: net::TcpStream
//! [The Rust Prelude]: prelude
//! [UDP]: net::UdpSocket
//! [`Arc`]: sync::Arc
//! [owned slice]: boxed
//! [`Cell`]: cell::Cell
//! [`FromStr`]: str::FromStr
//! [`HashMap<K, V>`]: collections::HashMap
//! [`Mutex`]: sync::Mutex
//! [`Option<T>`]: option::Option
//! [`Rc`]: rc::Rc
//! [`RefCell`]: cell::RefCell
//! [`Result<T, E>`]: result::Result
//! [`Vec<T>`]: vec::Vec
//! [`atomic`]: sync::atomic
//! [`for`]: ../book/ch03-05-control-flow.html#looping-through-a-collection-with-for
//! [`str`]: prim@str
//! [`mpsc`]: sync::mpsc
//! [`std::cmp`]: cmp
//! [`std::slice`]: mod@slice
//! [`use std::env`]: env/index.html
//! [`use`]: ../book/ch07-02-defining-modules-to-control-scope-and-privacy.html
//! [crates.io]: https://crates.io
//! [deref-coercions]: ../book/ch15-02-deref.html#implicit-deref-coercions-with-functions-and-methods
//! [files]: fs::File
//! [multithreading]: thread
//! [other]: #what-is-in-the-standard-library-documentation
//! [primitive types]: ../book/ch03-02-data-types.html
//! [rust-discord]: https://discord.gg/rust-lang
//! [array]: prim@array
//! [slice]: prim@slice
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
#![cfg_attr(not(feature = "restricted-std"), stable(feature = "rust1", since = "1.0.0"))]
#![cfg_attr(feature = "restricted-std", unstable(feature = "restricted_std", issue = "none"))]
#![doc(
    html_playground_url = "https://play.rust-lang.org/",
    issue_tracker_base_url = "https://github.com/rust-lang/rust/issues/",
    test(no_crate_inject, attr(deny(warnings))),
    test(attr(allow(dead_code, deprecated, unused_variables, unused_mut)))
)]
#![doc(cfg_hide(
    not(test),
    not(any(test, bootstrap)),
    no_global_oom_handling,
    not(no_global_oom_handling)
))]
// Don't link to std. <br>不要链接到 std。<br> We are std. <br>我们是 std。<br>
#![no_std]
#![warn(deprecated_in_future)]
#![warn(missing_docs)]
#![warn(missing_debug_implementations)]
#![allow(explicit_outlives_requirements)]
#![allow(unused_lifetimes)]
// Tell the compiler to link to either panic_abort or panic_unwind <br>告诉编译器链接到 panic_abort 或 panic_unwind<br>
#![needs_panic_runtime]
// std may use features in a platform-specific way <br>std 可能以特定于平台的方式使用特性<br>
#![allow(unused_features)]
#![cfg_attr(test, feature(internal_output_capture, print_internals, update_panic_count))]
#![cfg_attr(
    all(target_vendor = "fortanix", target_env = "sgx"),
    feature(slice_index_methods, coerce_unsized, sgx_platform)
)]
#![deny(rustc::existing_doc_keyword)]
//
// Language features: <br>语言特性：<br>
#![feature(alloc_error_handler)]
#![feature(allocator_internals)]
#![feature(allow_internal_unsafe)]
#![feature(allow_internal_unstable)]
#![feature(box_syntax)]
#![feature(c_unwind)]
#![feature(cfg_target_thread_local)]
#![feature(concat_idents)]
#![feature(const_mut_refs)]
#![feature(const_trait_impl)]
#![feature(decl_macro)]
#![feature(deprecated_suggestion)]
#![feature(doc_cfg)]
#![feature(doc_cfg_hide)]
#![feature(doc_masked)]
#![feature(doc_notable_trait)]
#![feature(dropck_eyepatch)]
#![feature(exhaustive_patterns)]
#![feature(intra_doc_pointers)]
#![feature(label_break_value)]
#![feature(lang_items)]
#![feature(let_chains)]
#![feature(let_else)]
#![feature(linkage)]
#![feature(min_specialization)]
#![feature(must_not_suspend)]
#![feature(needs_panic_runtime)]
#![feature(negative_impls)]
#![feature(never_type)]
#![cfg_attr(bootstrap, feature(nll))]
#![feature(platform_intrinsics)]
#![feature(prelude_import)]
#![feature(rustc_attrs)]
#![feature(rustdoc_internals)]
#![feature(staged_api)]
#![feature(thread_local)]
#![feature(try_blocks)]
//
// Library features (core): <br>库特性 (core):<br>
#![feature(array_error_internals)]
#![feature(atomic_mut_ptr)]
#![feature(char_error_internals)]
#![feature(char_internals)]
#![feature(core_c_str)]
#![feature(core_intrinsics)]
#![feature(cstr_from_bytes_until_nul)]
#![feature(cstr_internals)]
#![feature(duration_checked_float)]
#![feature(duration_constants)]
#![feature(exact_size_is_empty)]
#![feature(extend_one)]
#![feature(float_minimum_maximum)]
#![feature(hasher_prefixfree_extras)]
#![feature(hashmap_internals)]
#![feature(int_error_internals)]
#![feature(maybe_uninit_slice)]
#![feature(maybe_uninit_write_slice)]
#![feature(mixed_integer_ops)]
#![feature(nonnull_slice_from_raw_parts)]
#![feature(panic_can_unwind)]
#![feature(panic_info_message)]
#![feature(panic_internals)]
#![feature(portable_simd)]
#![feature(prelude_2024)]
#![feature(ptr_as_uninit)]
#![feature(raw_os_nonzero)]
#![feature(slice_internals)]
#![feature(slice_ptr_get)]
#![feature(std_internals)]
#![feature(str_internals)]
#![feature(strict_provenance)]
//
// Library features (alloc): <br>库特性 (alloc):<br>
#![feature(alloc_layout_extra)]
#![feature(alloc_c_string)]
#![feature(allocator_api)]
#![feature(get_mut_unchecked)]
#![feature(map_try_insert)]
#![feature(new_uninit)]
#![feature(thin_box)]
#![feature(try_reserve_kind)]
#![feature(vec_into_raw_parts)]
#![feature(slice_concat_trait)]
//
// Library features (unwind): <br>库特性 (unwind):<br>
#![feature(panic_unwind)]
//
// Only for re-exporting: <br>仅适用于重导出:<br>
#![feature(assert_matches)]
#![feature(async_iterator)]
#![feature(c_variadic)]
#![feature(cfg_accessible)]
#![feature(cfg_eval)]
#![feature(concat_bytes)]
#![feature(const_format_args)]
#![feature(core_ffi_c)]
#![feature(core_panic)]
#![feature(custom_test_frameworks)]
#![feature(edition_panic)]
#![feature(format_args_nl)]
#![feature(log_syntax)]
#![feature(once_cell)]
#![feature(saturating_int_impl)]
#![feature(stdsimd)]
#![feature(test)]
#![feature(trace_macros)]
//
// Only used in tests/benchmarks: <br>仅用于测试/基准测试:<br>
#![feature(bench_black_box)]
//
// Only for const-ness: <br>仅适用于 const-ness:<br>
#![feature(const_io_structs)]
#![feature(const_ip)]
#![feature(const_ipv4)]
#![feature(const_ipv6)]
#![feature(const_option)]
#![feature(const_socketaddr)]
#![feature(thread_local_internals)]
//
#![default_lib_allocator]

// Explicitly import the prelude. <br>显式导入 prelude。<br>
// The compiler uses this same unstable attribute to import the prelude implicitly when building crates that depend on std. <br>在构建依赖于 std 的 crates 时，编译器使用此不稳定属性来隐式导入 prelude。<br>
#[prelude_import]
#[allow(unused)]
use prelude::rust_2021::*;

// Access to Bencher, etc. <br>访问 Bencher 等<br>
#[cfg(test)]
extern crate test;

#[allow(unused_imports)] // macros from `alloc` are not used on all platforms <br>并非所有平台都使用来自 `alloc` 的宏<br>
#[macro_use]
extern crate alloc as alloc_crate;
#[doc(masked)]
#[allow(unused_extern_crates)]
extern crate libc;

// We always need an unwinder currently for backtraces <br>当前，我们总是需要一个 unwinder 来回溯<br>
#[doc(masked)]
#[allow(unused_extern_crates)]
extern crate unwind;

#[doc(masked)]
#[allow(unused_extern_crates)]
#[cfg(feature = "miniz_oxide")]
extern crate miniz_oxide;

// During testing, this crate is not actually the "real" std library, but rather it links to the real std library, which was compiled from this same source code. <br>在测试期间，此 crate 实际上不是 "真正的" std 库，而是链接到实际的 std 库，该库是从相同的源代码编译而成的。<br>
// So any lang items std defines are conditionally excluded (or else they would generate duplicate lang item errors), and any globals it defines are _not_ the globals used by "real" std. <br>因此，std 定义的任何 lang 项都被有条件地排除 (否则它们将生成重复的 lang 项错误)，并且它定义的任何全局变量都不是真实 std 中使用的全局变量。<br>
//
// So this import, defined only during testing gives test-std access to real-std lang items and globals. <br>所以这个仅在测试期间定义的导入使 test-std 可以访问 real-std lang 项和全局变量。<br>
// See #2912 <br>请参见 #2912<br>
//
#[cfg(test)]
extern crate std as realstd;

// The standard macros that are not built-in to the compiler. <br>编译器未内置的标准宏。<br>
#[macro_use]
mod macros;

// The runtime entry point and a few unstable public functions used by the compiler <br>运行时入口点和编译器使用的一些不稳定的公共函数<br>
//
#[macro_use]
pub mod rt;

// The Rust prelude <br>Rust prelude<br>
pub mod prelude;

// Public module declarations and re-exports <br>公共模块声明和重导出<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::borrow;
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::boxed;
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::fmt;
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::format;
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::rc;
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::slice;
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::str;
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::string;
#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::vec;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::any;
#[stable(feature = "core_array", since = "1.36.0")]
pub use core::array;
#[unstable(feature = "async_iterator", issue = "79024")]
pub use core::async_iter;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::cell;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::char;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::clone;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::cmp;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::convert;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::default;
#[stable(feature = "futures_api", since = "1.36.0")]
pub use core::future;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::hash;
#[stable(feature = "core_hint", since = "1.27.0")]
pub use core::hint;
#[stable(feature = "i128", since = "1.26.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::i128;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::i16;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::i32;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::i64;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::i8;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::intrinsics;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::isize;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::iter;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::marker;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::mem;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::ops;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::option;
#[stable(feature = "pin", since = "1.33.0")]
pub use core::pin;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::ptr;
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::result;
#[stable(feature = "i128", since = "1.26.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::u128;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::u16;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::u32;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::u64;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::u8;
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::usize;

pub mod f32;
pub mod f64;

#[macro_use]
pub mod thread;
pub mod ascii;
pub mod backtrace;
pub mod collections;
pub mod env;
pub mod error;
pub mod ffi;
pub mod fs;
pub mod io;
pub mod net;
pub mod num;
pub mod os;
pub mod panic;
pub mod path;
pub mod process;
pub mod sync;
pub mod time;

#[unstable(feature = "once_cell", issue = "74465")]
pub mod lazy;

// Pull in `std_float` crate  into libstd. <br>将 `std_float` crate 拉取到 libstd。<br>
// The contents of `std_float` are in a different repository: rust-lang/portable-simd. <br>`std_float` 的内容在另一个仓库: rust-lang/portable-simd。<br>
#[path = "../../portable-simd/crates/std_float/src/lib.rs"]
#[allow(missing_debug_implementations, dead_code, unsafe_op_in_unsafe_fn, unused_unsafe)]
#[allow(rustdoc::bare_urls)]
#[unstable(feature = "portable_simd", issue = "86656")]
mod std_float;

#[doc = include_str!("../../portable-simd/crates/core_simd/src/core_simd_docs.md")]
#[unstable(feature = "portable_simd", issue = "86656")]
pub mod simd {
    #[doc(inline)]
    pub use crate::std_float::StdFloat;
    #[doc(inline)]
    pub use core::simd::*;
}

#[stable(feature = "futures_api", since = "1.36.0")]
pub mod task {
    //! Types and Traits for working with asynchronous tasks. <br>类型和 Traits 用于处理异步任务。<br>

    #[doc(inline)]
    #[stable(feature = "futures_api", since = "1.36.0")]
    pub use core::task::*;

    #[doc(inline)]
    #[stable(feature = "wake_trait", since = "1.51.0")]
    pub use alloc::task::*;
}

#[doc = include_str!("../../stdarch/crates/core_arch/src/core_arch_docs.md")]
#[stable(feature = "simd_arch", since = "1.27.0")]
pub mod arch {
    #[stable(feature = "simd_arch", since = "1.27.0")]
    // The `no_inline`-attribute is required to make the documentation of all targets available. <br>必须提供 `no_inline` 属性，以提供所有目标的文档。<br>
    // See https://github.com/rust-lang/rust/pull/57808#issuecomment-457390549formoreinformation.
    //
    //
    #[doc(no_inline)] // Note (#82861): required for correct documentation <br>需要正确的文档<br>
    pub use core::arch::*;

    #[stable(feature = "simd_aarch64", since = "1.60.0")]
    pub use std_detect::is_aarch64_feature_detected;
    #[stable(feature = "simd_x86", since = "1.27.0")]
    pub use std_detect::is_x86_feature_detected;
    #[unstable(feature = "stdsimd", issue = "48556")]
    pub use std_detect::{
        is_arm_feature_detected, is_mips64_feature_detected, is_mips_feature_detected,
        is_powerpc64_feature_detected, is_powerpc_feature_detected, is_riscv_feature_detected,
    };
}

// This was stabilized in the crate root so we have to keep it there. <br>这在 crate root 中已稳定，因此我们必须将其保留在那里。<br>
#[stable(feature = "simd_x86", since = "1.27.0")]
pub use std_detect::is_x86_feature_detected;

// Platform-abstraction modules <br>平台抽象模块<br>
mod sys;
mod sys_common;

pub mod alloc;

// Private support modules <br>private 支持模块<br>
mod panicking;

#[path = "../../backtrace/src/lib.rs"]
#[allow(dead_code, unused_attributes)]
mod backtrace_rs;

// Re-export macros defined in libcore. <br>重导出 libcore 中定义的宏。<br>
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated, deprecated_in_future)]
pub use core::{
    assert_eq, assert_ne, debug_assert, debug_assert_eq, debug_assert_ne, matches, r#try, todo,
    unimplemented, unreachable, write, writeln,
};

// Re-export built-in macros defined through libcore. <br>重导出通过 libcore 定义的内置宏。<br>
#[stable(feature = "builtin_macro_prelude", since = "1.38.0")]
#[allow(deprecated)]
pub use core::{
    assert, assert_matches, cfg, column, compile_error, concat, concat_idents, const_format_args,
    env, file, format_args, format_args_nl, include, include_bytes, include_str, line, log_syntax,
    module_path, option_env, stringify, trace_macros,
};

#[unstable(
    feature = "concat_bytes",
    issue = "87555",
    reason = "`concat_bytes` is not stable enough for use and is subject to change"
)]
pub use core::concat_bytes;

#[stable(feature = "core_primitive", since = "1.43.0")]
pub use core::primitive;

// Include a number of private modules that exist solely to provide the rustdoc documentation for primitive types. <br>包括许多私有模块，这些模块仅用于提供原始类型的 rustdoc 文档。<br>
// Using `include!` because rustdoc only looks for these modules at the crate level. <br>使用 `include!` 是因为 rustdoc 只在 crate 级别上查找这些模块。<br>
//
include!("primitive_docs.rs");

// Include a number of private modules that exist solely to provide the rustdoc documentation for the existing keywords. <br>包括许多私有模块，这些模块仅用于为现有关键字提供 rustdoc 文档。<br>
// Using `include!` because rustdoc only looks for these modules at the crate level. <br>使用 `include!` 是因为 rustdoc 只在 crate 级别上查找这些模块。<br>
//
include!("keyword_docs.rs");

// This is required to avoid an unstable error when `restricted-std` is not enabled. <br>这是必需的，以避免在未启用 `restricted-std` 时产生不稳定的错误。<br>
// The use of #![feature(restricted_std)] in rustc-std-workspace-std is unconditional, so the unstable feature needs to be defined somewhere. <br>#![feature(restricted_std)] 在 rustc-std-workspace-std 中的使用是无条件的，所以不稳定的特性需要在某处定义。<br>
//
#[unstable(feature = "restricted_std", issue = "none")]
mod __restricted_std_workaround {}

mod sealed {
    /// This trait being unreachable from outside the crate prevents outside implementations of our extension traits. <br>这个 trait 从 crate 外部无法访问，这阻止了我们扩展 trait 的外部实现。<br>
    ///
    /// This allows adding more trait methods in the future. <br>这允许在未来添加更多 trait 方法。<br>
    #[unstable(feature = "sealed", issue = "none")]
    pub trait Sealed {}
}
