//! Standard library macros <br>标准库宏<br>
//!
//! This module contains a set of macros which are exported from the standard library. <br>该模块包含一组从标准库导出的宏。<br>
//! Each macro is available for use when linking against the standard library. <br>当链接到标准库时，每个宏都可以使用。<br>
//!

#[doc = include_str!("../../core/src/macros/panic.md")]
#[macro_export]
#[rustc_builtin_macro(std_panic)]
#[stable(feature = "rust1", since = "1.0.0")]
#[allow_internal_unstable(edition_panic)]
#[cfg_attr(not(test), rustc_diagnostic_item = "std_panic_macro")]
macro_rules! panic {
    // Expands to either `$crate::panic::panic_2015` or `$crate::panic::panic_2021` depending on the edition of the caller. <br>根据调用者的版本扩展为 `$crate::panic::panic_2015` 或 `$crate::panic::panic_2021`。<br>
    //
    ($($arg:tt)*) => {
        /* compiler built-in */
    };
}

/// Prints to the standard output. <br>打印到标准输出。<br>
///
/// Equivalent to the [`println!`] macro except that a newline is not printed at the end of the message. <br>等效于 [`println!`] 宏，只是在消息末尾不打印换行符。<br>
///
/// Note that stdout is frequently line-buffered by default so it may be necessary to use [`io::stdout().flush()`][flush] to ensure the output is emitted immediately. <br>注意，默认情况下，stdout 通常是行缓冲的，因此可能有必要使用 [`io::stdout().flush()`][flush] 以确保立即发出输出。<br>
///
///
/// Use `print!` only for the primary output of your program. <br>`print!` 仅用于程序的主要输出。<br> Use [`eprint!`] instead to print error and progress messages. <br>请改用 [`eprint!`] 打印错误和进度消息。<br>
///
/// [flush]: crate::io::Write::flush
/// [`println!`]: crate::println
/// [`eprint!`]: crate::eprint
///
/// # Panics
///
/// Panics if writing to `io::stdout()` fails. <br>如果写入 `io::stdout()` 失败，就会出现 panics。<br>
///
/// # Examples
///
/// ```
/// use std::io::{self, Write};
///
/// print!("this ");
/// print!("will ");
/// print!("be ");
/// print!("on ");
/// print!("the ");
/// print!("same ");
/// print!("line ");
///
/// io::stdout().flush().unwrap();
///
/// print!("this string has a newline, why not choose println! instead?\n");
///
/// io::stdout().flush().unwrap();
/// ```
///
///
///
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "print_macro")]
#[allow_internal_unstable(print_internals)]
macro_rules! print {
    ($($arg:tt)*) => {{
        $crate::io::_print($crate::format_args!($($arg)*));
    }};
}

/// Prints to the standard output, with a newline. <br>使用换行符打印到标准输出。<br>
///
/// On all platforms, the newline is the LINE FEED character (`\n`/`U+000A`) alone (no additional CARRIAGE RETURN (`\r`/`U+000D`)). <br>在所有平台上，新行是一个换行符 (也就是 `\n`/`U+000A`)，并不包含回车符 (也就是 `\r`/`U+000D`)。<br>
///
///
/// This macro uses the same syntax as [`format!`], but writes to the standard output instead. <br>此宏使用与 [`format!`] 相同的语法，但改为写入标准输出。<br>
/// See [`std::fmt`] for more information. <br>有关详细信息，请参见 [`std::fmt`]。<br>
///
/// Use `println!` only for the primary output of your program. <br>`println!` 仅用于程序的主要输出。<br> Use [`eprintln!`] instead to print error and progress messages. <br>请改用 [`eprintln!`] 打印错误和进度消息。<br>
///
/// [`std::fmt`]: crate::fmt
/// [`eprintln!`]: crate::eprintln
///
/// # Panics
///
/// Panics if writing to [`io::stdout`] fails. <br>如果写入 [`io::stdout`] 失败，就会出现 panics。<br>
///
/// [`io::stdout`]: crate::io::stdout
///
/// # Examples
///
/// ```
/// println!(); // prints just a newline <br>只打印换行符<br>
/// println!("hello there!");
/// println!("format {} arguments", "some");
/// ```
///
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "println_macro")]
#[allow_internal_unstable(print_internals, format_args_nl)]
macro_rules! println {
    () => {
        $crate::print!("\n")
    };
    ($($arg:tt)*) => {{
        $crate::io::_print($crate::format_args_nl!($($arg)*));
    }};
}

/// Prints to the standard error. <br>打印到标准错误。<br>
///
/// Equivalent to the [`print!`] macro, except that output goes to [`io::stderr`] instead of [`io::stdout`]. <br>等效于 [`print!`] 宏，除了输出转到 [`io::stderr`] 而不是 [`io::stdout`]。<br>
///
/// See [`print!`] for example usage. <br>有关用法示例，请参见 [`print!`]。<br>
///
/// Use `eprint!` only for error and progress messages. <br>仅将 `eprint!` 用于错误和进度消息。<br>
/// Use `print!` instead for the primary output of your program. <br>改用 `print!` 作为程序的主要输出。<br>
///
/// [`io::stderr`]: crate::io::stderr
/// [`io::stdout`]: crate::io::stdout
///
/// # Panics
///
/// Panics if writing to `io::stderr` fails. <br>如果写入 `io::stderr` 失败，就会出现 panics。<br>
///
/// # Examples
///
/// ```
/// eprint!("Error: Could not complete task");
/// ```
#[macro_export]
#[stable(feature = "eprint", since = "1.19.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "eprint_macro")]
#[allow_internal_unstable(print_internals)]
macro_rules! eprint {
    ($($arg:tt)*) => {{
        $crate::io::_eprint($crate::format_args!($($arg)*));
    }};
}

/// Prints to the standard error, with a newline. <br>使用换行符打印到标准错误。<br>
///
/// Equivalent to the [`println!`] macro, except that output goes to [`io::stderr`] instead of [`io::stdout`]. <br>等效于 [`println!`] 宏，除了输出转到 [`io::stderr`] 而不是 [`io::stdout`]。<br>
///
/// See [`println!`] for example usage. <br>有关用法示例，请参见 [`println!`]。<br>
///
/// Use `eprintln!` only for error and progress messages. <br>仅将 `eprintln!` 用于错误和进度消息。<br>
/// Use `println!` instead for the primary output of your program. <br>改用 `println!` 作为程序的主要输出。<br>
///
/// [`io::stderr`]: crate::io::stderr
/// [`io::stdout`]: crate::io::stdout
/// [`println!`]: crate::println
///
/// # Panics
///
/// Panics if writing to `io::stderr` fails. <br>如果写入 `io::stderr` 失败，就会出现 panics。<br>
///
/// # Examples
///
/// ```
/// eprintln!("Error: Could not complete task");
/// ```
#[macro_export]
#[stable(feature = "eprint", since = "1.19.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "eprintln_macro")]
#[allow_internal_unstable(print_internals, format_args_nl)]
macro_rules! eprintln {
    () => {
        $crate::eprint!("\n")
    };
    ($($arg:tt)*) => {{
        $crate::io::_eprint($crate::format_args_nl!($($arg)*));
    }};
}

/// Prints and returns the value of a given expression for quick and dirty debugging. <br>打印并返回给定表达式的值，以进行快速而肮脏的调试。<br>
///
/// An example: <br>一个例子：<br>
///
/// ```rust
/// let a = 2;
/// let b = dbg!(a * 2) + 1;
/// // ^-- prints: [src/main.rs:2] a * 2 = 4 <br>打印: [src/main.rs:2] a * 2 = 4<br>
/// assert_eq!(b, 5);
/// ```
///
/// The macro works by using the `Debug` implementation of the type of the given expression to print the value to [stderr] along with the source location of the macro invocation as well as the source code of the expression. <br>宏通过使用给定表达式的类型的 `Debug` 实现将值与宏调用的源位置以及表达式的源代码一起打印到 [标准错误][stderr] 来工作。<br>
///
///
/// Invoking the macro on an expression moves and takes ownership of it before returning the evaluated expression unchanged. <br>调用表达式上的宏会移动并获取它的所有权，然后再返回不变的求值表达式。<br>
/// If the type of the expression does not implement `Copy` and you don't want to give up ownership, you can instead borrow with `dbg!(&expr)` for some expression `expr`. <br>如果表达式的类型未实现 `Copy`，并且您不想放弃所有权，则可以改用 `dbg!(&expr)` 借用某些表达式 `expr`。<br>
///
/// The `dbg!` macro works exactly the same in release builds. <br>`dbg!` 宏在发行版中的工作原理完全相同。<br>
/// This is useful when debugging issues that only occur in release builds or when debugging in release mode is significantly faster. <br>当仅在发行版本中发生的调试问题或在发行模式下进行的调试明显更快时，此功能很有用。<br>
///
/// Note that the macro is intended as a debugging tool and therefore you should avoid having uses of it in version control for long periods (other than in tests and similar). <br>请注意，宏的目的是作为调试工具，因此您应该避免在版本控制中长时间使用它 (测试和类似的情况除外)。<br>
/// Debug output from production code is better done with other facilities such as the [`debug!`] macro from the [`log`] crate. <br>使用其他工具 (例如 [`log`] crate 的 [`debug!`] 宏) 可以更好地完成生产代码的调试输出。<br>
///
/// # Stability
///
/// The exact output printed by this macro should not be relied upon and is subject to future changes. <br>不应依赖此宏打印的确切输出，并且可能会受到 future 的更改。<br>
///
/// # Panics
///
/// Panics if writing to `io::stderr` fails. <br>如果写入 `io::stderr` 失败，就会出现 panics。<br>
///
/// # Further examples <br>进一步的例子<br>
///
/// With a method call: <br>用一种方法：<br>
///
/// ```rust
/// fn foo(n: usize) {
///     if let Some(_) = dbg!(n.checked_sub(4)) {
///         // ...
///     }
/// }
///
/// foo(3)
/// ```
///
/// This prints to [stderr]: <br>打印到 [stderr]：<br>
///
/// ```text,ignore
/// [src/main.rs:4] n.checked_sub(4) = None
/// ```
///
/// Naive factorial implementation: <br>单纯的析因实现：<br>
///
/// ```rust
/// fn factorial(n: u32) -> u32 {
///     if dbg!(n <= 1) {
///         dbg!(1)
///     } else {
///         dbg!(n * factorial(n - 1))
///     }
/// }
///
/// dbg!(factorial(4));
/// ```
///
/// This prints to [stderr]: <br>打印到 [stderr]：<br>
///
/// ```text,ignore
/// [src/main.rs:3] n <= 1 = false
/// [src/main.rs:3] n <= 1 = false
/// [src/main.rs:3] n <= 1 = false
/// [src/main.rs:3] n <= 1 = true
/// [src/main.rs:4] 1 = 1
/// [src/main.rs:5] n * factorial(n - 1) = 2
/// [src/main.rs:5] n * factorial(n - 1) = 6
/// [src/main.rs:5] n * factorial(n - 1) = 24
/// [src/main.rs:11] factorial(4) = 24
/// ```
///
/// The `dbg!(..)` macro moves the input: <br>`dbg!(..)` 宏移动输入：<br>
///
/// ```compile_fail
/// /// A wrapper around `usize` which importantly is not Copyable. <br>`usize` 的包装器，重要的是它不可复制。<br>
/// #[derive(Debug)]
/// struct NoCopy(usize);
///
/// let a = NoCopy(42);
/// let _ = dbg!(a); // <-- `a` is moved here. <br>`a` 被移到了这里。<br>
/// let _ = dbg!(a); // <-- `a` is moved again; <br>`a` 被再次移动；<br> error!
/// ```
///
/// You can also use `dbg!()` without a value to just print the file and line whenever it's reached. <br>您也可以使用不带任何值的 `dbg!()` 来仅在到达文件和行时打印它。<br>
///
/// Finally, if you want to `dbg!(..)` multiple values, it will treat them as a tuple (and return it, too): <br>最后，如果要 `dbg!(..)` 多个值，它将把它们视为一个元组 (并返回它) ：<br>
///
/// ```
/// assert_eq!(dbg!(1usize, 2u32), (1, 2));
/// ```
///
/// However, a single argument with a trailing comma will still not be treated as a tuple, following the convention of ignoring trailing commas in macro invocations. <br>但是，遵循在宏调用中忽略尾部逗号的约定，带有尾部逗号的单个参数仍不会被视为元组。<br> You can use a 1-tuple directly if you need one: <br>如果需要一个，可以直接使用一个 1 元组：<br>
///
/// ```
/// assert_eq!(1, dbg!(1u32,)); // trailing comma ignored <br>尾部逗号被忽略<br>
/// assert_eq!((1,), dbg!((1u32,))); // 1-tuple
/// ```
///
/// [stderr]: https://en.wikipedia.org/wiki/Standard_streams#Standard_error_(stderr)
/// [`debug!`]: https://docs.rs/log/*/log/macro.debug.html
/// [`log`]: https://crates.io/crates/log
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
#[macro_export]
#[cfg_attr(not(test), rustc_diagnostic_item = "dbg_macro")]
#[stable(feature = "dbg_macro", since = "1.32.0")]
macro_rules! dbg {
    // NOTE: We cannot use `concat!` to make a static string as a format argument of `eprintln!` because `file!` could contain a `{` or `$val` expression could be a block (`{ .. <br>我们不能使用 `concat!` 作为 `eprintln!` 的格式参数来生成静态字符串，因为 `file!` 可能包含 `{` 或 `$val` 表达式可能是一个块 (`{。<br>
    // }`), in which case the `eprintln!` will be malformed. <br>}`)，在这种情况下 `eprintln!` 的格式将不正确。<br>
    //
    //
    () => {
        $crate::eprintln!("[{}:{}]", $crate::file!(), $crate::line!())
    };
    ($val:expr $(,)?) => {
        // Use of `match` here is intentional because it affects the lifetimes of temporaries - https://stackoverflow.com/a/48732525/1063961 <br>在此使用 `match` 是有意的，因为它会影响临时人员的生命周期 - https://stackoverflow.com/a/48732525/1063961<br>
        //
        match $val {
            tmp => {
                $crate::eprintln!("[{}:{}] {} = {:#?}",
                    $crate::file!(), $crate::line!(), $crate::stringify!($val), &tmp);
                tmp
            }
        }
    };
    ($($val:expr),+ $(,)?) => {
        ($($crate::dbg!($val)),+,)
    };
}

#[cfg(test)]
macro_rules! assert_approx_eq {
    ($a:expr, $b:expr) => {{
        let (a, b) = (&$a, &$b);
        assert!((*a - *b).abs() < 1.0e-6, "{} is not approximately equal to {}", *a, *b);
    }};
}
