//! Networking primitives for TCP/UDP communication. <br>TCP/UDP 通信的网络原语。<br>
//!
//! This module provides networking functionality for the Transmission Control and User Datagram Protocols, as well as types for IP and socket addresses. <br>该模块提供了传输控制和用户数据报协议的网络功能，以及 IP 和套接字地址的类型。<br>
//!
//! # Organization
//!
//! * [`TcpListener`] and [`TcpStream`] provide functionality for communication over TCP <br>[`TcpListener`] 和 [`TcpStream`] 提供了通过 TCP 进行通信的功能<br>
//! * [`UdpSocket`] provides functionality for communication over UDP <br>[`UdpSocket`] 提供通过 UDP 进行通信的功能<br>
//! * [`IpAddr`] represents IP addresses of either IPv4 or IPv6; <br>[`IpAddr`] 代表 IPv4 或 IPv6 的 IP 地址；<br> [`Ipv4Addr`] and [`Ipv6Addr`] are respectively IPv4 and IPv6 addresses <br>[`Ipv4Addr`] 和 [`Ipv6Addr`] 分别是 IPv4 和 IPv6 地址<br>
//! * [`SocketAddr`] represents socket addresses of either IPv4 or IPv6; <br>[`SocketAddr`] 代表 IPv4 或 IPv6 的套接字地址；<br> [`SocketAddrV4`] and [`SocketAddrV6`] are respectively IPv4 and IPv6 socket addresses <br>[`SocketAddrV4`] 和 [`SocketAddrV6`] 分别是 IPv4 和 IPv6 套接字地址<br>
//! * [`ToSocketAddrs`] is a trait that used for generic address resolution when interacting with networking objects like [`TcpListener`], [`TcpStream`] or [`UdpSocket`]
//!
//! * Other types are return or parameter types for various methods in this module <br>其他类型是此模块中各种方法的返回值或参数类型<br>
//!
//! Rust disables inheritance of socket objects to child processes by default when possible. <br>Rust 在可能的情况下默认禁用套接字对象对子进程的继承。<br>
//! For example, through the use of the `CLOEXEC` flag in UNIX systems or the `HANDLE_FLAG_INHERIT` flag on Windows. <br>例如，通过在 UNIX 系统中使用 `CLOEXEC` 标志或在 Windows 上使用 `HANDLE_FLAG_INHERIT` 标志。<br>
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

use crate::io::{self, ErrorKind};

#[stable(feature = "rust1", since = "1.0.0")]
pub use self::addr::{SocketAddr, SocketAddrV4, SocketAddrV6, ToSocketAddrs};
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::ip::{IpAddr, Ipv4Addr, Ipv6Addr, Ipv6MulticastScope};
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::parser::AddrParseError;
#[unstable(feature = "tcplistener_into_incoming", issue = "88339")]
pub use self::tcp::IntoIncoming;
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::tcp::{Incoming, TcpListener, TcpStream};
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::udp::UdpSocket;

mod addr;
mod ip;
mod parser;
mod tcp;
#[cfg(test)]
mod test;
mod udp;

/// Possible values which can be passed to the [`TcpStream::shutdown`] method. <br>可以传递给 [`TcpStream::shutdown`] 方法的可能值。<br>
#[derive(Copy, Clone, PartialEq, Eq, Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub enum Shutdown {
    /// The reading portion of the [`TcpStream`] should be shut down. <br>[`TcpStream`] 的读取部分应关闭。<br>
    ///
    /// All currently blocked and future [reads] will return <code>[Ok]\(0)</code>. <br>所有当前被阻止的和未来的 [reads] 将返回 <code>[Ok]\(0)</code>。<br>
    ///
    /// [reads]: crate::io::Read "io::Read"
    #[stable(feature = "rust1", since = "1.0.0")]
    Read,
    /// The writing portion of the [`TcpStream`] should be shut down. <br>[`TcpStream`] 的写入部分应关闭。<br>
    ///
    /// All currently blocked and future [writes] will return an error. <br>所有当前被阻止和未来的 [写入][writes] 将返回错误。<br>
    ///
    /// [writes]: crate::io::Write "io::Write"
    #[stable(feature = "rust1", since = "1.0.0")]
    Write,
    /// Both the reading and the writing portions of the [`TcpStream`] should be shut down. <br>[`TcpStream`] 的读取和写入部分均应关闭。<br>
    ///
    /// See [`Shutdown::Read`] and [`Shutdown::Write`] for more information. <br>有关更多信息，请参见 [`Shutdown::Read`] 和 [`Shutdown::Write`]。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    Both,
}

#[inline]
const fn htons(i: u16) -> u16 {
    i.to_be()
}
#[inline]
const fn ntohs(i: u16) -> u16 {
    u16::from_be(i)
}

fn each_addr<A: ToSocketAddrs, F, T>(addr: A, mut f: F) -> io::Result<T>
where
    F: FnMut(io::Result<&SocketAddr>) -> io::Result<T>,
{
    let addrs = match addr.to_socket_addrs() {
        Ok(addrs) => addrs,
        Err(e) => return f(Err(e)),
    };
    let mut last_err = None;
    for addr in addrs {
        match f(Ok(&addr)) {
            Ok(l) => return Ok(l),
            Err(e) => last_err = Some(e),
        }
    }
    Err(last_err.unwrap_or_else(|| {
        io::const_io_error!(ErrorKind::InvalidInput, "could not resolve to any addresses")
    }))
}
