#![deny(unsafe_op_in_unsafe_fn)]

#[cfg(all(test, not(target_os = "emscripten")))]
mod tests;

use crate::io::prelude::*;

use crate::fmt;
use crate::io::{self, IoSlice, IoSliceMut};
use crate::net::{Shutdown, SocketAddr, ToSocketAddrs};
use crate::sys_common::net as net_imp;
use crate::sys_common::{AsInner, FromInner, IntoInner};
use crate::time::Duration;

/// 本地套接字和远程套接字之间的 TCP 流。
///
/// 在通过 [`connect`] 到远程主机或 [`accept`] 在 [`TcpListener`] 上创建连接来创建 `TcpStream` 后，数据可以由 [读取][reading] 和 [写入][writing] 传输到该 `TcpStream`。
///
///
/// 丢弃该值时，连接将关闭。也可以使用 [`shutdown`] 方法单独关闭连接的读取和写入部分。
///
/// 传输控制协议在 [IETF RFC 793] 中指定。
///
/// [`accept`]: TcpListener::accept
/// [`connect`]: TcpStream::connect
/// [IETF RFC 793]: https://tools.ietf.org/html/rfc793
/// [reading]: Read
/// [`shutdown`]: TcpStream::shutdown
/// [writing]: Write
///
/// # Examples
///
/// ```no_run
/// use std::io::prelude::*;
/// use std::net::TcpStream;
///
/// fn main() -> std::io::Result<()> {
///     let mut stream = TcpStream::connect("127.0.0.1:34254")?;
///
///     stream.write(&[1])?;
///     stream.read(&mut [0; 128])?;
///     Ok(())
/// } // 流在这里关闭
/// ```
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct TcpStream(net_imp::TcpStream);

/// TCP 套接字服务器，侦听连接。
///
/// 通过将 `TcpListener` 绑定到套接字地址来创建 `TcpListener` 之后，它会侦听传入的 TCP 连接。
/// 可以通过调用 [`accept`] 或在 [`incoming`][`TcpListener::incoming`] 返回的 [`Incoming`] 迭代器上进行迭代来接受它们。
///
///
/// 丢弃该值时，套接字将关闭。
///
/// 传输控制协议在 [IETF RFC 793] 中指定。
///
/// [`accept`]: TcpListener::accept
/// [`bind`]: TcpListener::bind
/// [IETF RFC 793]: https://tools.ietf.org/html/rfc793
///
/// # Examples
///
/// ```no_run
/// use std::net::{TcpListener, TcpStream};
///
/// fn handle_client(stream: TcpStream) {
///     // ...
/// }
///
/// fn main() -> std::io::Result<()> {
///     let listener = TcpListener::bind("127.0.0.1:80")?;
///
///     // 接受连接并顺序处理它们
///     for stream in listener.incoming() {
///         handle_client(stream?);
///     }
///     Ok(())
/// }
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub struct TcpListener(net_imp::TcpListener);

/// 一个在 [`TcpListener`] 上无限 [`accept`] 连接的迭代器。
///
/// 该 `struct` 是通过 [`TcpListener::incoming`] 方法创建的。
/// 有关更多信息，请参见其文档。
///
/// [`accept`]: TcpListener::accept
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Incoming<'a> {
    listener: &'a TcpListener,
}

/// 一个在 [`TcpListener`] 上无限 [`accept`] 连接的迭代器。
///
/// 这个 `struct` 是由 [`TcpListener::into_incoming`] 方法创建的。
/// 有关更多信息，请参见其文档。
///
/// [`accept`]: TcpListener::accept
#[derive(Debug)]
#[unstable(feature = "tcplistener_into_incoming", issue = "88339")]
pub struct IntoIncoming {
    listener: TcpListener,
}

impl TcpStream {
    /// 打开到远程主机的 TCP 连接。
    ///
    /// `addr` 是远程主机的地址。
    /// 可以提供任何实现 [`ToSocketAddrs`] trait 的地址。有关具体示例，请参见此 trait 文档。
    ///
    /// 如果 `addr` 产生多个地址，则将使用每个地址尝试 `connect`，直到连接成功。
    /// 如果没有一个地址导致连接成功，则返回从上一次连接尝试 (最后一个地址) 返回的错误。
    ///
    ///
    /// # Examples
    ///
    /// 打开到 `127.0.0.1:8080` 的 TCP 连接：
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// if let Ok(stream) = TcpStream::connect("127.0.0.1:8080") {
    ///     println!("Connected to the server!");
    /// } else {
    ///     println!("Couldn't connect to server...");
    /// }
    /// ```
    ///
    /// 打开到 `127.0.0.1:8080` 的 TCP 连接。如果连接失败，请打开与 `127.0.0.1:8081` 的 TCP 连接：
    ///
    /// ```no_run
    /// use std::net::{SocketAddr, TcpStream};
    ///
    /// let addrs = [
    ///     SocketAddr::from(([127, 0, 0, 1], 8080)),
    ///     SocketAddr::from(([127, 0, 0, 1], 8081)),
    /// ];
    /// if let Ok(stream) = TcpStream::connect(&addrs[..]) {
    ///     println!("Connected to the server!");
    /// } else {
    ///     println!("Couldn't connect to server...");
    /// }
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn connect<A: ToSocketAddrs>(addr: A) -> io::Result<TcpStream> {
        super::each_addr(addr, net_imp::TcpStream::connect).map(TcpStream)
    }

    /// 超时打开与远程主机的 TCP 连接。
    ///
    /// 与 `connect` 不同，由于必须将超时应用于单个地址，因此 `connect_timeout` 仅占用一个 [`SocketAddr`]。
    ///
    /// 向此函数传递零 `Duration` 是错误的。
    ///
    /// 与 `TcpStream` 上的其他方法不同，这并不对应于单个系统调用。
    /// 相反，它以非阻塞模式调用 `connect`，然后使用特定于操作系统的机制来等待连接请求的完成。
    ///
    ///
    ///
    #[stable(feature = "tcpstream_connect_timeout", since = "1.21.0")]
    pub fn connect_timeout(addr: &SocketAddr, timeout: Duration) -> io::Result<TcpStream> {
        net_imp::TcpStream::connect_timeout(addr, timeout).map(TcpStream)
    }

    /// 返回此 TCP 连接的远程对等方的套接字地址。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{Ipv4Addr, SocketAddr, SocketAddrV4, TcpStream};
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// assert_eq!(stream.peer_addr().unwrap(),
    ///            SocketAddr::V4(SocketAddrV4::new(Ipv4Addr::new(127, 0, 0, 1), 8080)));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn peer_addr(&self) -> io::Result<SocketAddr> {
        self.0.peer_addr()
    }

    /// 返回此 TCP 连接本地一半的套接字地址。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{IpAddr, Ipv4Addr, TcpStream};
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// assert_eq!(stream.local_addr().unwrap().ip(),
    ///            IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn local_addr(&self) -> io::Result<SocketAddr> {
        self.0.socket_addr()
    }

    /// 关闭此连接的读取，写入或两半。
    ///
    /// 此函数将导致指定部分上的所有未决和 future I/O 立即以适当的值返回 (请参见 [`Shutdown`] 的文档)。
    ///
    ///
    /// # 特定于平台的行为
    ///
    /// 取决于操作系统，多次调用此函数可能会导致不同的行为。
    /// 在 Linux 上，第二个调用将返回 `Ok(())`，但在 macOS 上，它将返回 `ErrorKind::NotConnected`。
    /// future 可能会改变。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{Shutdown, TcpStream};
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.shutdown(Shutdown::Both).expect("shutdown call failed");
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn shutdown(&self, how: Shutdown) -> io::Result<()> {
        self.0.shutdown(how)
    }

    /// 为底层套接字创建一个新的独立的拥有所有权的句柄。
    ///
    /// 返回的 `TcpStream` 是与此对象引用相同的流的引用。
    /// 两个句柄将读取和写入相同的数据流，并且在一个流上设置的选项将传播到另一流。
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// let stream_clone = stream.try_clone().expect("clone failed...");
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn try_clone(&self) -> io::Result<TcpStream> {
        self.0.duplicate().map(TcpStream)
    }

    /// 将读取超时设置为指定的超时。
    ///
    /// 如果指定的值为 [`None`]，则 [`read`] 调用将无限期阻塞。
    /// 如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]。
    ///
    /// # 特定于平台的行为
    ///
    /// 由于设置此选项而导致读取超时时，平台可能会返回不同的错误代码。
    /// 例如，Unix 通常返回类型为 [`WouldBlock`] 的错误，但是 Windows 可能返回 [`TimedOut`]。
    ///
    ///
    /// [`read`]: Read::read
    /// [`WouldBlock`]: io::ErrorKind::WouldBlock
    /// [`TimedOut`]: io::ErrorKind::TimedOut
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_read_timeout(None).expect("set_read_timeout call failed");
    /// ```
    ///
    /// 如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]：
    ///
    /// ```no_run
    /// use std::io;
    /// use std::net::TcpStream;
    /// use std::time::Duration;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080").unwrap();
    /// let result = stream.set_read_timeout(Some(Duration::new(0, 0)));
    /// let err = result.unwrap_err();
    /// assert_eq!(err.kind(), io::ErrorKind::InvalidInput)
    /// ```
    ///
    ///
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn set_read_timeout(&self, dur: Option<Duration>) -> io::Result<()> {
        self.0.set_read_timeout(dur)
    }

    /// 将写超时设置为指定的超时。
    ///
    /// 如果指定的值为 [`None`]，则 [`write`] 调用将无限期阻塞。
    /// 如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]。
    ///
    /// # 特定于平台的行为
    ///
    /// 由于设置此选项而导致写超时时，平台可能会返回不同的错误代码。
    /// 例如，Unix 通常返回类型为 [`WouldBlock`] 的错误，但是 Windows 可能返回 [`TimedOut`]。
    ///
    ///
    /// [`write`]: Write::write
    /// [`WouldBlock`]: io::ErrorKind::WouldBlock
    /// [`TimedOut`]: io::ErrorKind::TimedOut
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_write_timeout(None).expect("set_write_timeout call failed");
    /// ```
    ///
    /// 如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]：
    ///
    /// ```no_run
    /// use std::io;
    /// use std::net::TcpStream;
    /// use std::time::Duration;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080").unwrap();
    /// let result = stream.set_write_timeout(Some(Duration::new(0, 0)));
    /// let err = result.unwrap_err();
    /// assert_eq!(err.kind(), io::ErrorKind::InvalidInput)
    /// ```
    ///
    ///
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn set_write_timeout(&self, dur: Option<Duration>) -> io::Result<()> {
        self.0.set_write_timeout(dur)
    }

    /// 返回此套接字的读取超时。
    ///
    /// 如果超时为 [`None`]，则 [`read`] 调用将无限期阻塞。
    ///
    /// # 特定于平台的行为
    ///
    /// 某些平台不提供对当前超时的访问。
    ///
    /// [`read`]: Read::read
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_read_timeout(None).expect("set_read_timeout call failed");
    /// assert_eq!(stream.read_timeout().unwrap(), None);
    /// ```
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn read_timeout(&self) -> io::Result<Option<Duration>> {
        self.0.read_timeout()
    }

    /// 返回此套接字的写入超时。
    ///
    /// 如果超时为 [`None`]，则 [`write`] 调用将无限期阻塞。
    ///
    /// # 特定于平台的行为
    ///
    /// 某些平台不提供对当前超时的访问。
    ///
    /// [`write`]: Write::write
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_write_timeout(None).expect("set_write_timeout call failed");
    /// assert_eq!(stream.write_timeout().unwrap(), None);
    /// ```
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn write_timeout(&self) -> io::Result<Option<Duration>> {
        self.0.write_timeout()
    }

    /// 从套接字所连接的远程地址接收套接字上的数据，而无需从队列中删除该数据。
    ///
    /// 成功时，返回偷看的字节数。
    ///
    /// 连续调用返回相同的数据。
    /// 这是通过将 `MSG_PEEK` 作为标志传递给底层的 `recv` 系统调用来实现的。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8000")
    ///                        .expect("Couldn't connect to the server...");
    /// let mut buf = [0; 10];
    /// let len = stream.peek(&mut buf).expect("peek failed");
    /// ```
    #[stable(feature = "peek", since = "1.18.0")]
    pub fn peek(&self, buf: &mut [u8]) -> io::Result<usize> {
        self.0.peek(buf)
    }

    /// 设置此套接字上 `SO_LINGER` 选项的值。
    ///
    /// 此值控制当数据仍有待发送时如何关闭套接字。
    /// 如果设置了 `SO_LINGER`，则当系统尝试发送挂起的数据时，套接字将在指定的持续时间内保持打开状态。
    ///
    /// 否则，系统可能会立即关闭套接字，或等待默认超时。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// #![feature(tcp_linger)]
    ///
    /// use std::net::TcpStream;
    /// use std::time::Duration;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_linger(Some(Duration::from_secs(0))).expect("set_linger call failed");
    /// ```
    ///
    #[unstable(feature = "tcp_linger", issue = "88494")]
    pub fn set_linger(&self, linger: Option<Duration>) -> io::Result<()> {
        self.0.set_linger(linger)
    }

    /// 获取此套接字上 `SO_LINGER` 选项的值。
    ///
    /// 有关此选项的更多信息，请参见 [`TcpStream::set_linger`]。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// #![feature(tcp_linger)]
    ///
    /// use std::net::TcpStream;
    /// use std::time::Duration;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_linger(Some(Duration::from_secs(0))).expect("set_linger call failed");
    /// assert_eq!(stream.linger().unwrap(), Some(Duration::from_secs(0)));
    /// ```
    #[unstable(feature = "tcp_linger", issue = "88494")]
    pub fn linger(&self) -> io::Result<Option<Duration>> {
        self.0.linger()
    }

    /// 设置此套接字上 `TCP_NODELAY` 选项的值。
    ///
    /// 如果设置，则此选项禁用 Nagle 算法。
    /// 这意味着即使只有少量数据，也总是尽快发送段。
    /// 如果未设置，则对数据进行缓冲，直到有足够的数据量可以发送出去，从而避免了频繁发送小数据包。
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_nodelay(true).expect("set_nodelay call failed");
    /// ```
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_nodelay(&self, nodelay: bool) -> io::Result<()> {
        self.0.set_nodelay(nodelay)
    }

    /// 获取此套接字上 `TCP_NODELAY` 选项的值。
    ///
    /// 有关此选项的更多信息，请参见 [`TcpStream::set_nodelay`]。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_nodelay(true).expect("set_nodelay call failed");
    /// assert_eq!(stream.nodelay().unwrap_or(false), true);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn nodelay(&self) -> io::Result<bool> {
        self.0.nodelay()
    }

    /// 设置此套接字上 `IP_TTL` 选项的值。
    ///
    /// 此值设置从该套接字发送的每个数据包中使用的生存时间字段。
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_ttl(100).expect("set_ttl call failed");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_ttl(&self, ttl: u32) -> io::Result<()> {
        self.0.set_ttl(ttl)
    }

    /// 获取此套接字的 `IP_TTL` 选项的值。
    ///
    /// 有关此选项的更多信息，请参见 [`TcpStream::set_ttl`]。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_ttl(100).expect("set_ttl call failed");
    /// assert_eq!(stream.ttl().unwrap_or(0), 100);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn ttl(&self) -> io::Result<u32> {
        self.0.ttl()
    }

    /// 获取此套接字上 `SO_ERROR` 选项的值。
    ///
    /// 这将检索底层套接字中存储的错误，从而清除进程中的字段。
    /// 这对于检查两次调用之间的错误很有用。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.take_error().expect("No error was expected...");
    /// ```
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn take_error(&self) -> io::Result<Option<io::Error>> {
        self.0.take_error()
    }

    /// 将此 TCP 流移入或移出非阻塞模式。
    ///
    /// 这将导致 `read`，`write`，`recv` 和 `send` 操作变为非阻塞，即立即从其调用中返回。
    ///
    /// 如果 IO 操作成功，则返回 `Ok`，并且不需要进一步的操作。
    /// 如果 IO 操作无法完成，需要重试，则返回类型为 [`io::ErrorKind::WouldBlock`] 的错误。
    ///
    /// 在 Unix 平台上，调用此方法相当于调用 `fcntl` `FIONBIO`。
    /// 在 Windows 上，调用此方法对应于调用 `ioctlsocket` `FIONBIO`。
    ///
    /// # Examples
    ///
    /// 在非阻塞模式下从 TCP 流读取字节：
    ///
    /// ```no_run
    /// use std::io::{self, Read};
    /// use std::net::TcpStream;
    ///
    /// let mut stream = TcpStream::connect("127.0.0.1:7878")
    ///     .expect("Couldn't connect to the server...");
    /// stream.set_nonblocking(true).expect("set_nonblocking call failed");
    ///
    /// # fn wait_for_fd() { unimplemented!() }
    /// let mut buf = vec![];
    /// loop {
    ///     match stream.read_to_end(&mut buf) {
    ///         Ok(_) => break,
    ///         Err(ref e) if e.kind() == io::ErrorKind::WouldBlock => {
    ///             // 等待网络套接字就绪，通常通过平台特定的 API (例如 epoll 或 IOCP) 实现
    /////
    ///             wait_for_fd();
    ///         }
    ///         Err(e) => panic!("encountered IO error: {e}"),
    ///     };
    /// };
    /// println!("bytes: {buf:?}");
    /// ```
    ///
    ///
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
        self.0.set_nonblocking(nonblocking)
    }
}

// 除了这里的 impl 之外，`TcpStream` 还有 `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` 和 `AsRawFd`/`IntoRawFd`/`FromRawFd`、Unix 和 WASI 以及 Windows 上的 `AsSocket`/`From<OwnedSocket>`/`Into<OwnedSocket>` 和 `AsRawSocket`/`IntoRawSocket`/`FromRawSocket` 的 impl。
//
//
//
//

#[stable(feature = "rust1", since = "1.0.0")]
impl Read for TcpStream {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.0.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.0.read_vectored(bufs)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.0.is_read_vectored()
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Write for TcpStream {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.0.write(buf)
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.0.write_vectored(bufs)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.0.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        Ok(())
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Read for &TcpStream {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.0.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.0.read_vectored(bufs)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.0.is_read_vectored()
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Write for &TcpStream {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.0.write(buf)
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.0.write_vectored(bufs)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.0.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        Ok(())
    }
}

impl AsInner<net_imp::TcpStream> for TcpStream {
    fn as_inner(&self) -> &net_imp::TcpStream {
        &self.0
    }
}

impl FromInner<net_imp::TcpStream> for TcpStream {
    fn from_inner(inner: net_imp::TcpStream) -> TcpStream {
        TcpStream(inner)
    }
}

impl IntoInner<net_imp::TcpStream> for TcpStream {
    fn into_inner(self) -> net_imp::TcpStream {
        self.0
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for TcpStream {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.0.fmt(f)
    }
}

impl TcpListener {
    /// 创建一个新的 `TcpListener`，它将绑定到指定的地址。
    ///
    /// 返回的侦听器已准备好接受连接。
    ///
    /// 端口号为 0 的绑定将要求 OS 为该侦听器分配端口。
    /// 可以通过 [`TcpListener::local_addr`] 方法查询分配的端口。
    ///
    /// 地址类型可以是 [`ToSocketAddrs`] trait 的任何实现者。有关具体的例子，请参见其文档。
    ///
    /// 如果 `addr` 产生多个地址，则将对每个地址尝试 `bind`，直到一个成功并返回侦听器为止。
    ///
    /// 如果没有一个地址成功创建侦听器，则返回从上次尝试 (最后一个地址) 返回的错误。
    ///
    /// # Examples
    ///
    /// 创建绑定到 `127.0.0.1:80` 的 TCP 侦听器：
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    /// ```
    ///
    /// 创建绑定到 `127.0.0.1:80` 的 TCP 侦听器。如果失败，请创建绑定到 `127.0.0.1:443` 的 TCP 侦听器：
    ///
    /// ```no_run
    /// use std::net::{SocketAddr, TcpListener};
    ///
    /// let addrs = [
    ///     SocketAddr::from(([127, 0, 0, 1], 80)),
    ///     SocketAddr::from(([127, 0, 0, 1], 443)),
    /// ];
    /// let listener = TcpListener::bind(&addrs[..]).unwrap();
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn bind<A: ToSocketAddrs>(addr: A) -> io::Result<TcpListener> {
        super::each_addr(addr, net_imp::TcpListener::bind).map(TcpListener)
    }

    /// 返回此侦听器的本地套接字地址。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{Ipv4Addr, SocketAddr, SocketAddrV4, TcpListener};
    ///
    /// let listener = TcpListener::bind("127.0.0.1:8080").unwrap();
    /// assert_eq!(listener.local_addr().unwrap(),
    ///            SocketAddr::V4(SocketAddrV4::new(Ipv4Addr::new(127, 0, 0, 1), 8080)));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn local_addr(&self) -> io::Result<SocketAddr> {
        self.0.socket_addr()
    }

    /// 为底层套接字创建一个新的独立的拥有所有权的句柄。
    ///
    /// 返回的 [`TcpListener`] 是与此对象引用相同的套接字的引用。
    /// 这两个句柄均可用于接受传入连接，并且在一个侦听器上设置的选项将影响另一个。
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:8080").unwrap();
    /// let listener_clone = listener.try_clone().unwrap();
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn try_clone(&self) -> io::Result<TcpListener> {
        self.0.duplicate().map(TcpListener)
    }

    /// 接受来自此侦听器的新传入连接。
    ///
    /// 该函数将阻塞调用线程，直到建立新的 TCP 连接为止。
    /// 建立后，将返回相应的 [`TcpStream`] 和远程对等方的地址。
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:8080").unwrap();
    /// match listener.accept() {
    ///     Ok((_socket, addr)) => println!("new client: {addr:?}"),
    ///     Err(e) => println!("couldn't get client: {e:?}"),
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn accept(&self) -> io::Result<(TcpStream, SocketAddr)> {
        // 在 WASM 上，`TcpStream` 是无人居住的 (因为它不受支持)，因此这里的 `a` 变量在技术上是未使用的。
        //
        #[cfg_attr(target_arch = "wasm32", allow(unused_variables))]
        self.0.accept().map(|(a, b)| (TcpStream(a), b))
    }

    /// 返回在此侦听器上接收到的连接上的迭代器。
    ///
    /// 返回的迭代器将永远不会返回 [`None`]，也不会产生对等方的 [`SocketAddr`] 结构体。
    /// 对其进行迭代等效于在循环中调用 [`TcpListener::accept`]。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{TcpListener, TcpStream};
    ///
    /// fn handle_connection(stream: TcpStream) {
    ///    // ...
    /// }
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    ///
    ///     for stream in listener.incoming() {
    ///         match stream {
    ///             Ok(stream) => {
    ///                 handle_connection(stream);
    ///             }
    ///             Err(e) => { /* connection failed */ }
    ///         }
    ///     }
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn incoming(&self) -> Incoming<'_> {
        Incoming { listener: self }
    }

    /// 在此侦听器上接收到的连接上将其转换为迭代器。
    ///
    /// 返回的迭代器将永远不会返回 [`None`]，也不会产生对等方的 [`SocketAddr`] 结构体。
    /// 对其进行迭代等效于在循环中调用 [`TcpListener::accept`]。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// #![feature(tcplistener_into_incoming)]
    /// use std::net::{TcpListener, TcpStream};
    ///
    /// fn listen_on(port: u16) -> impl Iterator<Item = TcpStream> {
    ///     let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    ///     listener.into_incoming()
    ///         .filter_map(Result::ok) /* Ignore failed connections */
    /// }
    ///
    /// fn main() -> std::io::Result<()> {
    ///     for stream in listen_on(80) {
    ///         /* handle the connection here */
    ///     }
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    #[must_use = "`self` will be dropped if the result is not used"]
    #[unstable(feature = "tcplistener_into_incoming", issue = "88339")]
    pub fn into_incoming(self) -> IntoIncoming {
        IntoIncoming { listener: self }
    }

    /// 设置此套接字上 `IP_TTL` 选项的值。
    ///
    /// 此值设置从该套接字发送的每个数据包中使用的生存时间字段。
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    /// listener.set_ttl(100).expect("could not set TTL");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_ttl(&self, ttl: u32) -> io::Result<()> {
        self.0.set_ttl(ttl)
    }

    /// 获取此套接字的 `IP_TTL` 选项的值。
    ///
    /// 有关此选项的更多信息，请参见 [`TcpListener::set_ttl`]。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    /// listener.set_ttl(100).expect("could not set TTL");
    /// assert_eq!(listener.ttl().unwrap_or(0), 100);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn ttl(&self) -> io::Result<u32> {
        self.0.ttl()
    }

    #[stable(feature = "net2_mutators", since = "1.9.0")]
    #[deprecated(since = "1.16.0", note = "this option can only be set before the socket is bound")]
    #[allow(missing_docs)]
    pub fn set_only_v6(&self, only_v6: bool) -> io::Result<()> {
        self.0.set_only_v6(only_v6)
    }

    #[stable(feature = "net2_mutators", since = "1.9.0")]
    #[deprecated(since = "1.16.0", note = "this option can only be set before the socket is bound")]
    #[allow(missing_docs)]
    pub fn only_v6(&self) -> io::Result<bool> {
        self.0.only_v6()
    }

    /// 获取此套接字上 `SO_ERROR` 选项的值。
    ///
    /// 这将检索底层套接字中存储的错误，从而清除进程中的字段。
    /// 这对于检查两次调用之间的错误很有用。
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    /// listener.take_error().expect("No error was expected");
    /// ```
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn take_error(&self) -> io::Result<Option<io::Error>> {
        self.0.take_error()
    }

    /// 将此 TCP 流移入或移出非阻塞模式。
    ///
    /// 这将导致 `accept` 操作变为非阻塞，即立即从其调用中返回。
    /// 如果 IO 操作成功，则返回 `Ok`，并且不需要进一步的操作。
    /// 如果 IO 操作无法完成，需要重试，则返回类型为 [`io::ErrorKind::WouldBlock`] 的错误。
    ///
    ///
    /// 在 Unix 平台上，调用此方法相当于调用 `fcntl` `FIONBIO`。
    /// 在 Windows 上，调用此方法对应于调用 `ioctlsocket` `FIONBIO`。
    ///
    /// # Examples
    ///
    /// 将 TCP 侦听器绑定到地址，侦听连接，并以非阻塞模式读取字节：
    ///
    /// ```no_run
    /// use std::io;
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:7878").unwrap();
    /// listener.set_nonblocking(true).expect("Cannot set non-blocking");
    ///
    /// # fn wait_for_fd() { unimplemented!() }
    /// # fn handle_connection(stream: std::net::TcpStream) { unimplemented!() }
    /// for stream in listener.incoming() {
    ///     match stream {
    ///         Ok(s) => {
    ///             // 用 TcpStream 做某事
    ///             handle_connection(s);
    ///         }
    ///         Err(ref e) if e.kind() == io::ErrorKind::WouldBlock => {
    ///             // 等待网络套接字就绪，通常通过平台特定的 API (例如 epoll 或 IOCP) 实现
    /////
    ///             wait_for_fd();
    ///             continue;
    ///         }
    ///         Err(e) => panic!("encountered IO error: {e}"),
    ///     }
    /// }
    /// ```
    ///
    ///
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
        self.0.set_nonblocking(nonblocking)
    }
}

// 除了这里的 impl 之外，`TcpListener` 还有 `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` 和 `AsRawFd`/`IntoRawFd`/`FromRawFd`、Unix 和 WASI 以及 Windows 上的 `AsSocket`/`From<OwnedSocket>`/`Into<OwnedSocket>` 和 `AsRawSocket`/`IntoRawSocket`/`FromRawSocket` 的 impl。
//
//
//
//

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> Iterator for Incoming<'a> {
    type Item = io::Result<TcpStream>;
    fn next(&mut self) -> Option<io::Result<TcpStream>> {
        Some(self.listener.accept().map(|p| p.0))
    }
}

#[unstable(feature = "tcplistener_into_incoming", issue = "88339")]
impl Iterator for IntoIncoming {
    type Item = io::Result<TcpStream>;
    fn next(&mut self) -> Option<io::Result<TcpStream>> {
        Some(self.listener.accept().map(|p| p.0))
    }
}

impl AsInner<net_imp::TcpListener> for TcpListener {
    fn as_inner(&self) -> &net_imp::TcpListener {
        &self.0
    }
}

impl FromInner<net_imp::TcpListener> for TcpListener {
    fn from_inner(inner: net_imp::TcpListener) -> TcpListener {
        TcpListener(inner)
    }
}

impl IntoInner<net_imp::TcpListener> for TcpListener {
    fn into_inner(self) -> net_imp::TcpListener {
        self.0
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for TcpListener {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.0.fmt(f)
    }
}
