#![deny(unsafe_op_in_unsafe_fn)]

#[cfg(all(test, not(target_os = "emscripten")))]
mod tests;

use crate::io::prelude::*;

use crate::fmt;
use crate::io::{self, IoSlice, IoSliceMut};
use crate::net::{Shutdown, SocketAddr, ToSocketAddrs};
use crate::sys_common::net as net_imp;
use crate::sys_common::{AsInner, FromInner, IntoInner};
use crate::time::Duration;

/// A TCP stream between a local and a remote socket. <br>本地套接字和远程套接字之间的 TCP 流。<br>
///
/// After creating a `TcpStream` by either [`connect`]ing to a remote host or [`accept`]ing a connection on a [`TcpListener`], data can be transmitted by [reading] and [writing] to it. <br>在通过 [`connect`] 到远程主机或 [`accept`] 在 [`TcpListener`] 上创建连接来创建 `TcpStream` 后，数据可以由 [读取][reading] 和 [写入][writing] 传输到该 `TcpStream`。<br>
///
///
/// The connection will be closed when the value is dropped. <br>丢弃该值时，连接将关闭。<br> The reading and writing portions of the connection can also be shut down individually with the [`shutdown`] method. <br>也可以使用 [`shutdown`] 方法单独关闭连接的读取和写入部分。<br>
///
/// The Transmission Control Protocol is specified in [IETF RFC 793]. <br>传输控制协议在 [IETF RFC 793] 中指定。<br>
///
/// [`accept`]: TcpListener::accept
/// [`connect`]: TcpStream::connect
/// [IETF RFC 793]: https://tools.ietf.org/html/rfc793
/// [reading]: Read
/// [`shutdown`]: TcpStream::shutdown
/// [writing]: Write
///
/// # Examples
///
/// ```no_run
/// use std::io::prelude::*;
/// use std::net::TcpStream;
///
/// fn main() -> std::io::Result<()> {
///     let mut stream = TcpStream::connect("127.0.0.1:34254")?;
///
///     stream.write(&[1])?;
///     stream.read(&mut [0; 128])?;
///     Ok(())
/// } // the stream is closed here <br>流在这里关闭<br>
/// ```
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct TcpStream(net_imp::TcpStream);

/// A TCP socket server, listening for connections. <br>TCP 套接字服务器，侦听连接。<br>
///
/// After creating a `TcpListener` by [`bind`]ing it to a socket address, it listens for incoming TCP connections. <br>通过将 `TcpListener` 绑定到套接字地址来创建 `TcpListener` 之后，它会侦听传入的 TCP 连接。<br>
/// These can be accepted by calling [`accept`] or by iterating over the [`Incoming`] iterator returned by [`incoming`][`TcpListener::incoming`]. <br>可以通过调用 [`accept`] 或在 [`incoming`][`TcpListener::incoming`] 返回的 [`Incoming`] 迭代器上进行迭代来接受它们。<br>
///
///
/// The socket will be closed when the value is dropped. <br>丢弃该值时，套接字将关闭。<br>
///
/// The Transmission Control Protocol is specified in [IETF RFC 793]. <br>传输控制协议在 [IETF RFC 793] 中指定。<br>
///
/// [`accept`]: TcpListener::accept
/// [`bind`]: TcpListener::bind
/// [IETF RFC 793]: https://tools.ietf.org/html/rfc793
///
/// # Examples
///
/// ```no_run
/// use std::net::{TcpListener, TcpStream};
///
/// fn handle_client(stream: TcpStream) {
///     // ...
/// }
///
/// fn main() -> std::io::Result<()> {
///     let listener = TcpListener::bind("127.0.0.1:80")?;
///
///     // accept connections and process them serially <br>接受连接并顺序处理它们<br>
///     for stream in listener.incoming() {
///         handle_client(stream?);
///     }
///     Ok(())
/// }
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub struct TcpListener(net_imp::TcpListener);

/// An iterator that infinitely [`accept`]s connections on a [`TcpListener`]. <br>一个在 [`TcpListener`] 上无限 [`accept`] 连接的迭代器。<br>
///
/// This `struct` is created by the [`TcpListener::incoming`] method. <br>该 `struct` 是通过 [`TcpListener::incoming`] 方法创建的。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// [`accept`]: TcpListener::accept
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Incoming<'a> {
    listener: &'a TcpListener,
}

/// An iterator that infinitely [`accept`]s connections on a [`TcpListener`]. <br>一个在 [`TcpListener`] 上无限 [`accept`] 连接的迭代器。<br>
///
/// This `struct` is created by the [`TcpListener::into_incoming`] method. <br>这个 `struct` 是由 [`TcpListener::into_incoming`] 方法创建的。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// [`accept`]: TcpListener::accept
#[derive(Debug)]
#[unstable(feature = "tcplistener_into_incoming", issue = "88339")]
pub struct IntoIncoming {
    listener: TcpListener,
}

impl TcpStream {
    /// Opens a TCP connection to a remote host. <br>打开到远程主机的 TCP 连接。<br>
    ///
    /// `addr` is an address of the remote host. <br>`addr` 是远程主机的地址。<br>
    /// Anything which implements [`ToSocketAddrs`] trait can be supplied for the address; <br>可以提供任何实现 [`ToSocketAddrs`] trait 的地址。<br> see this trait documentation for concrete examples. <br>有关具体示例，请参见此 trait 文档。<br>
    ///
    /// If `addr` yields multiple addresses, `connect` will be attempted with each of the addresses until a connection is successful. <br>如果 `addr` 产生多个地址，则将使用每个地址尝试 `connect`，直到连接成功。<br>
    /// If none of the addresses result in a successful connection, the error returned from the last connection attempt (the last address) is returned. <br>如果没有一个地址导致连接成功，则返回从上一次连接尝试 (最后一个地址) 返回的错误。<br>
    ///
    ///
    /// # Examples
    ///
    /// Open a TCP connection to `127.0.0.1:8080`: <br>打开到 `127.0.0.1:8080` 的 TCP 连接：<br>
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// if let Ok(stream) = TcpStream::connect("127.0.0.1:8080") {
    ///     println!("Connected to the server!");
    /// } else {
    ///     println!("Couldn't connect to server...");
    /// }
    /// ```
    ///
    /// Open a TCP connection to `127.0.0.1:8080`. <br>打开到 `127.0.0.1:8080` 的 TCP 连接。<br> If the connection fails, open a TCP connection to `127.0.0.1:8081`: <br>如果连接失败，请打开与 `127.0.0.1:8081` 的 TCP 连接：<br>
    ///
    /// ```no_run
    /// use std::net::{SocketAddr, TcpStream};
    ///
    /// let addrs = [
    ///     SocketAddr::from(([127, 0, 0, 1], 8080)),
    ///     SocketAddr::from(([127, 0, 0, 1], 8081)),
    /// ];
    /// if let Ok(stream) = TcpStream::connect(&addrs[..]) {
    ///     println!("Connected to the server!");
    /// } else {
    ///     println!("Couldn't connect to server...");
    /// }
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn connect<A: ToSocketAddrs>(addr: A) -> io::Result<TcpStream> {
        super::each_addr(addr, net_imp::TcpStream::connect).map(TcpStream)
    }

    /// Opens a TCP connection to a remote host with a timeout. <br>超时打开与远程主机的 TCP 连接。<br>
    ///
    /// Unlike `connect`, `connect_timeout` takes a single [`SocketAddr`] since timeout must be applied to individual addresses. <br>与 `connect` 不同，由于必须将超时应用于单个地址，因此 `connect_timeout` 仅占用一个 [`SocketAddr`]。<br>
    ///
    /// It is an error to pass a zero `Duration` to this function. <br>向此函数传递零 `Duration` 是错误的。<br>
    ///
    /// Unlike other methods on `TcpStream`, this does not correspond to a single system call. <br>与 `TcpStream` 上的其他方法不同，这并不对应于单个系统调用。<br>
    /// It instead calls `connect` in nonblocking mode and then uses an OS-specific mechanism to await the completion of the connection request. <br>相反，它以非阻塞模式调用 `connect`，然后使用特定于操作系统的机制来等待连接请求的完成。<br>
    ///
    ///
    ///
    #[stable(feature = "tcpstream_connect_timeout", since = "1.21.0")]
    pub fn connect_timeout(addr: &SocketAddr, timeout: Duration) -> io::Result<TcpStream> {
        net_imp::TcpStream::connect_timeout(addr, timeout).map(TcpStream)
    }

    /// Returns the socket address of the remote peer of this TCP connection. <br>返回此 TCP 连接的远程对等方的套接字地址。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{Ipv4Addr, SocketAddr, SocketAddrV4, TcpStream};
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// assert_eq!(stream.peer_addr().unwrap(),
    ///            SocketAddr::V4(SocketAddrV4::new(Ipv4Addr::new(127, 0, 0, 1), 8080)));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn peer_addr(&self) -> io::Result<SocketAddr> {
        self.0.peer_addr()
    }

    /// Returns the socket address of the local half of this TCP connection. <br>返回此 TCP 连接本地一半的套接字地址。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{IpAddr, Ipv4Addr, TcpStream};
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// assert_eq!(stream.local_addr().unwrap().ip(),
    ///            IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn local_addr(&self) -> io::Result<SocketAddr> {
        self.0.socket_addr()
    }

    /// Shuts down the read, write, or both halves of this connection. <br>关闭此连接的读取，写入或两半。<br>
    ///
    /// This function will cause all pending and future I/O on the specified portions to return immediately with an appropriate value (see the documentation of [`Shutdown`]). <br>此函数将导致指定部分上的所有未决和 future I/O 立即以适当的值返回 (请参见 [`Shutdown`] 的文档)。<br>
    ///
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// Calling this function multiple times may result in different behavior, depending on the operating system. <br>取决于操作系统，多次调用此函数可能会导致不同的行为。<br>
    /// On Linux, the second call will return `Ok(())`, but on macOS, it will return `ErrorKind::NotConnected`. <br>在 Linux 上，第二个调用将返回 `Ok(())`，但在 macOS 上，它将返回 `ErrorKind::NotConnected`。<br>
    /// This may change in the future. <br>future 可能会改变。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{Shutdown, TcpStream};
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.shutdown(Shutdown::Both).expect("shutdown call failed");
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn shutdown(&self, how: Shutdown) -> io::Result<()> {
        self.0.shutdown(how)
    }

    /// Creates a new independently owned handle to the underlying socket. <br>为底层套接字创建一个新的独立的拥有所有权的句柄。<br>
    ///
    /// The returned `TcpStream` is a reference to the same stream that this object references. <br>返回的 `TcpStream` 是与此对象引用相同的流的引用。<br>
    /// Both handles will read and write the same stream of data, and options set on one stream will be propagated to the other stream. <br>两个句柄将读取和写入相同的数据流，并且在一个流上设置的选项将传播到另一流。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// let stream_clone = stream.try_clone().expect("clone failed...");
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn try_clone(&self) -> io::Result<TcpStream> {
        self.0.duplicate().map(TcpStream)
    }

    /// Sets the read timeout to the timeout specified. <br>将读取超时设置为指定的超时。<br>
    ///
    /// If the value specified is [`None`], then [`read`] calls will block indefinitely. <br>如果指定的值为 [`None`]，则 [`read`] 调用将无限期阻塞。<br>
    /// An [`Err`] is returned if the zero [`Duration`] is passed to this method. <br>如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]。<br>
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// Platforms may return a different error code whenever a read times out as a result of setting this option. <br>由于设置此选项而导致读取超时时，平台可能会返回不同的错误代码。<br>
    /// For example Unix typically returns an error of the kind [`WouldBlock`], but Windows may return [`TimedOut`]. <br>例如，Unix 通常返回类型为 [`WouldBlock`] 的错误，但是 Windows 可能返回 [`TimedOut`]。<br>
    ///
    ///
    /// [`read`]: Read::read
    /// [`WouldBlock`]: io::ErrorKind::WouldBlock
    /// [`TimedOut`]: io::ErrorKind::TimedOut
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_read_timeout(None).expect("set_read_timeout call failed");
    /// ```
    ///
    /// An [`Err`] is returned if the zero [`Duration`] is passed to this method: <br>如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]：<br>
    ///
    /// ```no_run
    /// use std::io;
    /// use std::net::TcpStream;
    /// use std::time::Duration;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080").unwrap();
    /// let result = stream.set_read_timeout(Some(Duration::new(0, 0)));
    /// let err = result.unwrap_err();
    /// assert_eq!(err.kind(), io::ErrorKind::InvalidInput)
    /// ```
    ///
    ///
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn set_read_timeout(&self, dur: Option<Duration>) -> io::Result<()> {
        self.0.set_read_timeout(dur)
    }

    /// Sets the write timeout to the timeout specified. <br>将写超时设置为指定的超时。<br>
    ///
    /// If the value specified is [`None`], then [`write`] calls will block indefinitely. <br>如果指定的值为 [`None`]，则 [`write`] 调用将无限期阻塞。<br>
    /// An [`Err`] is returned if the zero [`Duration`] is passed to this method. <br>如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]。<br>
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// Platforms may return a different error code whenever a write times out as a result of setting this option. <br>由于设置此选项而导致写超时时，平台可能会返回不同的错误代码。<br>
    /// For example Unix typically returns an error of the kind [`WouldBlock`], but Windows may return [`TimedOut`]. <br>例如，Unix 通常返回类型为 [`WouldBlock`] 的错误，但是 Windows 可能返回 [`TimedOut`]。<br>
    ///
    ///
    /// [`write`]: Write::write
    /// [`WouldBlock`]: io::ErrorKind::WouldBlock
    /// [`TimedOut`]: io::ErrorKind::TimedOut
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_write_timeout(None).expect("set_write_timeout call failed");
    /// ```
    ///
    /// An [`Err`] is returned if the zero [`Duration`] is passed to this method: <br>如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]：<br>
    ///
    /// ```no_run
    /// use std::io;
    /// use std::net::TcpStream;
    /// use std::time::Duration;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080").unwrap();
    /// let result = stream.set_write_timeout(Some(Duration::new(0, 0)));
    /// let err = result.unwrap_err();
    /// assert_eq!(err.kind(), io::ErrorKind::InvalidInput)
    /// ```
    ///
    ///
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn set_write_timeout(&self, dur: Option<Duration>) -> io::Result<()> {
        self.0.set_write_timeout(dur)
    }

    /// Returns the read timeout of this socket. <br>返回此套接字的读取超时。<br>
    ///
    /// If the timeout is [`None`], then [`read`] calls will block indefinitely. <br>如果超时为 [`None`]，则 [`read`] 调用将无限期阻塞。<br>
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// Some platforms do not provide access to the current timeout. <br>某些平台不提供对当前超时的访问。<br>
    ///
    /// [`read`]: Read::read
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_read_timeout(None).expect("set_read_timeout call failed");
    /// assert_eq!(stream.read_timeout().unwrap(), None);
    /// ```
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn read_timeout(&self) -> io::Result<Option<Duration>> {
        self.0.read_timeout()
    }

    /// Returns the write timeout of this socket. <br>返回此套接字的写入超时。<br>
    ///
    /// If the timeout is [`None`], then [`write`] calls will block indefinitely. <br>如果超时为 [`None`]，则 [`write`] 调用将无限期阻塞。<br>
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// Some platforms do not provide access to the current timeout. <br>某些平台不提供对当前超时的访问。<br>
    ///
    /// [`write`]: Write::write
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_write_timeout(None).expect("set_write_timeout call failed");
    /// assert_eq!(stream.write_timeout().unwrap(), None);
    /// ```
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn write_timeout(&self) -> io::Result<Option<Duration>> {
        self.0.write_timeout()
    }

    /// Receives data on the socket from the remote address to which it is connected, without removing that data from the queue. <br>从套接字所连接的远程地址接收套接字上的数据，而无需从队列中删除该数据。<br>
    ///
    /// On success, returns the number of bytes peeked. <br>成功时，返回偷看的字节数。<br>
    ///
    /// Successive calls return the same data. <br>连续调用返回相同的数据。<br>
    /// This is accomplished by passing `MSG_PEEK` as a flag to the underlying `recv` system call. <br>这是通过将 `MSG_PEEK` 作为标志传递给底层的 `recv` 系统调用来实现的。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8000")
    ///                        .expect("Couldn't connect to the server...");
    /// let mut buf = [0; 10];
    /// let len = stream.peek(&mut buf).expect("peek failed");
    /// ```
    #[stable(feature = "peek", since = "1.18.0")]
    pub fn peek(&self, buf: &mut [u8]) -> io::Result<usize> {
        self.0.peek(buf)
    }

    /// Sets the value of the `SO_LINGER` option on this socket. <br>设置此套接字上 `SO_LINGER` 选项的值。<br>
    ///
    /// This value controls how the socket is closed when data remains to be sent. <br>此值控制当数据仍有待发送时如何关闭套接字。<br>
    /// If `SO_LINGER` is set, the socket will remain open for the specified duration as the system attempts to send pending data. <br>如果设置了 `SO_LINGER`，则当系统尝试发送挂起的数据时，套接字将在指定的持续时间内保持打开状态。<br>
    ///
    /// Otherwise, the system may close the socket immediately, or wait for a default timeout. <br>否则，系统可能会立即关闭套接字，或等待默认超时。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// #![feature(tcp_linger)]
    ///
    /// use std::net::TcpStream;
    /// use std::time::Duration;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_linger(Some(Duration::from_secs(0))).expect("set_linger call failed");
    /// ```
    ///
    #[unstable(feature = "tcp_linger", issue = "88494")]
    pub fn set_linger(&self, linger: Option<Duration>) -> io::Result<()> {
        self.0.set_linger(linger)
    }

    /// Gets the value of the `SO_LINGER` option on this socket. <br>获取此套接字上 `SO_LINGER` 选项的值。<br>
    ///
    /// For more information about this option, see [`TcpStream::set_linger`]. <br>有关此选项的更多信息，请参见 [`TcpStream::set_linger`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// #![feature(tcp_linger)]
    ///
    /// use std::net::TcpStream;
    /// use std::time::Duration;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_linger(Some(Duration::from_secs(0))).expect("set_linger call failed");
    /// assert_eq!(stream.linger().unwrap(), Some(Duration::from_secs(0)));
    /// ```
    #[unstable(feature = "tcp_linger", issue = "88494")]
    pub fn linger(&self) -> io::Result<Option<Duration>> {
        self.0.linger()
    }

    /// Sets the value of the `TCP_NODELAY` option on this socket. <br>设置此套接字上 `TCP_NODELAY` 选项的值。<br>
    ///
    /// If set, this option disables the Nagle algorithm. <br>如果设置，则此选项禁用 Nagle 算法。<br>
    /// This means that segments are always sent as soon as possible, even if there is only a small amount of data. <br>这意味着即使只有少量数据，也总是尽快发送段。<br>
    /// When not set, data is buffered until there is a sufficient amount to send out, thereby avoiding the frequent sending of small packets. <br>如果未设置，则对数据进行缓冲，直到有足够的数据量可以发送出去，从而避免了频繁发送小数据包。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_nodelay(true).expect("set_nodelay call failed");
    /// ```
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_nodelay(&self, nodelay: bool) -> io::Result<()> {
        self.0.set_nodelay(nodelay)
    }

    /// Gets the value of the `TCP_NODELAY` option on this socket. <br>获取此套接字上 `TCP_NODELAY` 选项的值。<br>
    ///
    /// For more information about this option, see [`TcpStream::set_nodelay`]. <br>有关此选项的更多信息，请参见 [`TcpStream::set_nodelay`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_nodelay(true).expect("set_nodelay call failed");
    /// assert_eq!(stream.nodelay().unwrap_or(false), true);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn nodelay(&self) -> io::Result<bool> {
        self.0.nodelay()
    }

    /// Sets the value for the `IP_TTL` option on this socket. <br>设置此套接字上 `IP_TTL` 选项的值。<br>
    ///
    /// This value sets the time-to-live field that is used in every packet sent from this socket. <br>此值设置从该套接字发送的每个数据包中使用的生存时间字段。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_ttl(100).expect("set_ttl call failed");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_ttl(&self, ttl: u32) -> io::Result<()> {
        self.0.set_ttl(ttl)
    }

    /// Gets the value of the `IP_TTL` option for this socket. <br>获取此套接字的 `IP_TTL` 选项的值。<br>
    ///
    /// For more information about this option, see [`TcpStream::set_ttl`]. <br>有关此选项的更多信息，请参见 [`TcpStream::set_ttl`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.set_ttl(100).expect("set_ttl call failed");
    /// assert_eq!(stream.ttl().unwrap_or(0), 100);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn ttl(&self) -> io::Result<u32> {
        self.0.ttl()
    }

    /// Gets the value of the `SO_ERROR` option on this socket. <br>获取此套接字上 `SO_ERROR` 选项的值。<br>
    ///
    /// This will retrieve the stored error in the underlying socket, clearing the field in the process. <br>这将检索底层套接字中存储的错误，从而清除进程中的字段。<br>
    /// This can be useful for checking errors between calls. <br>这对于检查两次调用之间的错误很有用。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpStream;
    ///
    /// let stream = TcpStream::connect("127.0.0.1:8080")
    ///                        .expect("Couldn't connect to the server...");
    /// stream.take_error().expect("No error was expected...");
    /// ```
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn take_error(&self) -> io::Result<Option<io::Error>> {
        self.0.take_error()
    }

    /// Moves this TCP stream into or out of nonblocking mode. <br>将此 TCP 流移入或移出非阻塞模式。<br>
    ///
    /// This will result in `read`, `write`, `recv` and `send` operations becoming nonblocking, i.e., immediately returning from their calls. <br>这将导致 `read`，`write`，`recv` 和 `send` 操作变为非阻塞，即立即从其调用中返回。<br>
    ///
    /// If the IO operation is successful, `Ok` is returned and no further action is required. <br>如果 IO 操作成功，则返回 `Ok`，并且不需要进一步的操作。<br>
    /// If the IO operation could not be completed and needs to be retried, an error with kind [`io::ErrorKind::WouldBlock`] is returned. <br>如果 IO 操作无法完成，需要重试，则返回类型为 [`io::ErrorKind::WouldBlock`] 的错误。<br>
    ///
    /// On Unix platforms, calling this method corresponds to calling `fcntl` `FIONBIO`. <br>在 Unix 平台上，调用此方法相当于调用 `fcntl` `FIONBIO`。<br>
    /// On Windows calling this method corresponds to calling `ioctlsocket` `FIONBIO`. <br>在 Windows 上，调用此方法对应于调用 `ioctlsocket` `FIONBIO`。<br>
    ///
    /// # Examples
    ///
    /// Reading bytes from a TCP stream in non-blocking mode: <br>在非阻塞模式下从 TCP 流读取字节：<br>
    ///
    /// ```no_run
    /// use std::io::{self, Read};
    /// use std::net::TcpStream;
    ///
    /// let mut stream = TcpStream::connect("127.0.0.1:7878")
    ///     .expect("Couldn't connect to the server...");
    /// stream.set_nonblocking(true).expect("set_nonblocking call failed");
    ///
    /// # fn wait_for_fd() { unimplemented!() }
    /// let mut buf = vec![];
    /// loop {
    ///     match stream.read_to_end(&mut buf) {
    ///         Ok(_) => break,
    ///         Err(ref e) if e.kind() == io::ErrorKind::WouldBlock => {
    ///             // wait until network socket is ready, typically implemented via platform-specific APIs such as epoll or IOCP <br>等待网络套接字就绪，通常通过平台特定的 API (例如 epoll 或 IOCP) 实现<br>
    /////
    ///             wait_for_fd();
    ///         }
    ///         Err(e) => panic!("encountered IO error: {e}"),
    ///     };
    /// };
    /// println!("bytes: {buf:?}");
    /// ```
    ///
    ///
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
        self.0.set_nonblocking(nonblocking)
    }
}

// In addition to the `impl`s here, `TcpStream` also has `impl`s for `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` and `AsRawFd`/`IntoRawFd`/`FromRawFd`, on Unix and WASI, and `AsSocket`/`From<OwnedSocket>`/`Into<OwnedSocket>` and `AsRawSocket`/`IntoRawSocket`/`FromRawSocket` on Windows. <br>除了这里的 impl 之外，`TcpStream` 还有 `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` 和 `AsRawFd`/`IntoRawFd`/`FromRawFd`、Unix 和 WASI 以及 Windows 上的 `AsSocket`/`From<OwnedSocket>`/`Into<OwnedSocket>` 和 `AsRawSocket`/`IntoRawSocket`/`FromRawSocket` 的 impl。<br>
//
//
//
//

#[stable(feature = "rust1", since = "1.0.0")]
impl Read for TcpStream {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.0.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.0.read_vectored(bufs)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.0.is_read_vectored()
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Write for TcpStream {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.0.write(buf)
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.0.write_vectored(bufs)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.0.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        Ok(())
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Read for &TcpStream {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.0.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.0.read_vectored(bufs)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.0.is_read_vectored()
    }
}
#[stable(feature = "rust1", since = "1.0.0")]
impl Write for &TcpStream {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.0.write(buf)
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.0.write_vectored(bufs)
    }

    #[inline]
    fn is_write_vectored(&self) -> bool {
        self.0.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        Ok(())
    }
}

impl AsInner<net_imp::TcpStream> for TcpStream {
    fn as_inner(&self) -> &net_imp::TcpStream {
        &self.0
    }
}

impl FromInner<net_imp::TcpStream> for TcpStream {
    fn from_inner(inner: net_imp::TcpStream) -> TcpStream {
        TcpStream(inner)
    }
}

impl IntoInner<net_imp::TcpStream> for TcpStream {
    fn into_inner(self) -> net_imp::TcpStream {
        self.0
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for TcpStream {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.0.fmt(f)
    }
}

impl TcpListener {
    /// Creates a new `TcpListener` which will be bound to the specified address. <br>创建一个新的 `TcpListener`，它将绑定到指定的地址。<br>
    ///
    /// The returned listener is ready for accepting connections. <br>返回的侦听器已准备好接受连接。<br>
    ///
    /// Binding with a port number of 0 will request that the OS assigns a port to this listener. <br>端口号为 0 的绑定将要求 OS 为该侦听器分配端口。<br>
    /// The port allocated can be queried via the [`TcpListener::local_addr`] method. <br>可以通过 [`TcpListener::local_addr`] 方法查询分配的端口。<br>
    ///
    /// The address type can be any implementor of [`ToSocketAddrs`] trait. <br>地址类型可以是 [`ToSocketAddrs`] trait 的任何实现者。<br> See its documentation for concrete examples. <br>有关具体的例子，请参见其文档。<br>
    ///
    /// If `addr` yields multiple addresses, `bind` will be attempted with each of the addresses until one succeeds and returns the listener. <br>如果 `addr` 产生多个地址，则将对每个地址尝试 `bind`，直到一个成功并返回侦听器为止。<br>
    ///
    /// If none of the addresses succeed in creating a listener, the error returned from the last attempt (the last address) is returned. <br>如果没有一个地址成功创建侦听器，则返回从上次尝试 (最后一个地址) 返回的错误。<br>
    ///
    /// # Examples
    ///
    /// Creates a TCP listener bound to `127.0.0.1:80`: <br>创建绑定到 `127.0.0.1:80` 的 TCP 侦听器：<br>
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    /// ```
    ///
    /// Creates a TCP listener bound to `127.0.0.1:80`. <br>创建绑定到 `127.0.0.1:80` 的 TCP 侦听器。<br> If that fails, create a TCP listener bound to `127.0.0.1:443`: <br>如果失败，请创建绑定到 `127.0.0.1:443` 的 TCP 侦听器：<br>
    ///
    /// ```no_run
    /// use std::net::{SocketAddr, TcpListener};
    ///
    /// let addrs = [
    ///     SocketAddr::from(([127, 0, 0, 1], 80)),
    ///     SocketAddr::from(([127, 0, 0, 1], 443)),
    /// ];
    /// let listener = TcpListener::bind(&addrs[..]).unwrap();
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn bind<A: ToSocketAddrs>(addr: A) -> io::Result<TcpListener> {
        super::each_addr(addr, net_imp::TcpListener::bind).map(TcpListener)
    }

    /// Returns the local socket address of this listener. <br>返回此侦听器的本地套接字地址。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{Ipv4Addr, SocketAddr, SocketAddrV4, TcpListener};
    ///
    /// let listener = TcpListener::bind("127.0.0.1:8080").unwrap();
    /// assert_eq!(listener.local_addr().unwrap(),
    ///            SocketAddr::V4(SocketAddrV4::new(Ipv4Addr::new(127, 0, 0, 1), 8080)));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn local_addr(&self) -> io::Result<SocketAddr> {
        self.0.socket_addr()
    }

    /// Creates a new independently owned handle to the underlying socket. <br>为底层套接字创建一个新的独立的拥有所有权的句柄。<br>
    ///
    /// The returned [`TcpListener`] is a reference to the same socket that this object references. <br>返回的 [`TcpListener`] 是与此对象引用相同的套接字的引用。<br>
    /// Both handles can be used to accept incoming connections and options set on one listener will affect the other. <br>这两个句柄均可用于接受传入连接，并且在一个侦听器上设置的选项将影响另一个。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:8080").unwrap();
    /// let listener_clone = listener.try_clone().unwrap();
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn try_clone(&self) -> io::Result<TcpListener> {
        self.0.duplicate().map(TcpListener)
    }

    /// Accept a new incoming connection from this listener. <br>接受来自此侦听器的新传入连接。<br>
    ///
    /// This function will block the calling thread until a new TCP connection is established. <br>该函数将阻塞调用线程，直到建立新的 TCP 连接为止。<br>
    /// When established, the corresponding [`TcpStream`] and the remote peer's address will be returned. <br>建立后，将返回相应的 [`TcpStream`] 和远程对等方的地址。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:8080").unwrap();
    /// match listener.accept() {
    ///     Ok((_socket, addr)) => println!("new client: {addr:?}"),
    ///     Err(e) => println!("couldn't get client: {e:?}"),
    /// }
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn accept(&self) -> io::Result<(TcpStream, SocketAddr)> {
        // On WASM, `TcpStream` is uninhabited (as it's unsupported) and so the `a` variable here is technically unused. <br>在 WASM 上，`TcpStream` 是无人居住的 (因为它不受支持)，因此这里的 `a` 变量在技术上是未使用的。<br>
        //
        #[cfg_attr(target_arch = "wasm32", allow(unused_variables))]
        self.0.accept().map(|(a, b)| (TcpStream(a), b))
    }

    /// Returns an iterator over the connections being received on this listener. <br>返回在此侦听器上接收到的连接上的迭代器。<br>
    ///
    /// The returned iterator will never return [`None`] and will also not yield the peer's [`SocketAddr`] structure. <br>返回的迭代器将永远不会返回 [`None`]，也不会产生对等方的 [`SocketAddr`] 结构体。<br>
    /// Iterating over it is equivalent to calling [`TcpListener::accept`] in a loop. <br>对其进行迭代等效于在循环中调用 [`TcpListener::accept`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{TcpListener, TcpStream};
    ///
    /// fn handle_connection(stream: TcpStream) {
    ///    // ...
    /// }
    ///
    /// fn main() -> std::io::Result<()> {
    ///     let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    ///
    ///     for stream in listener.incoming() {
    ///         match stream {
    ///             Ok(stream) => {
    ///                 handle_connection(stream);
    ///             }
    ///             Err(e) => { /* connection failed */ }
    ///         }
    ///     }
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn incoming(&self) -> Incoming<'_> {
        Incoming { listener: self }
    }

    /// Turn this into an iterator over the connections being received on this listener. <br>在此侦听器上接收到的连接上将其转换为迭代器。<br>
    ///
    /// The returned iterator will never return [`None`] and will also not yield the peer's [`SocketAddr`] structure. <br>返回的迭代器将永远不会返回 [`None`]，也不会产生对等方的 [`SocketAddr`] 结构体。<br>
    /// Iterating over it is equivalent to calling [`TcpListener::accept`] in a loop. <br>对其进行迭代等效于在循环中调用 [`TcpListener::accept`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// #![feature(tcplistener_into_incoming)]
    /// use std::net::{TcpListener, TcpStream};
    ///
    /// fn listen_on(port: u16) -> impl Iterator<Item = TcpStream> {
    ///     let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    ///     listener.into_incoming()
    ///         .filter_map(Result::ok) /* Ignore failed connections */
    /// }
    ///
    /// fn main() -> std::io::Result<()> {
    ///     for stream in listen_on(80) {
    ///         /* handle the connection here */
    ///     }
    ///     Ok(())
    /// }
    /// ```
    ///
    ///
    #[must_use = "`self` will be dropped if the result is not used"]
    #[unstable(feature = "tcplistener_into_incoming", issue = "88339")]
    pub fn into_incoming(self) -> IntoIncoming {
        IntoIncoming { listener: self }
    }

    /// Sets the value for the `IP_TTL` option on this socket. <br>设置此套接字上 `IP_TTL` 选项的值。<br>
    ///
    /// This value sets the time-to-live field that is used in every packet sent from this socket. <br>此值设置从该套接字发送的每个数据包中使用的生存时间字段。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    /// listener.set_ttl(100).expect("could not set TTL");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_ttl(&self, ttl: u32) -> io::Result<()> {
        self.0.set_ttl(ttl)
    }

    /// Gets the value of the `IP_TTL` option for this socket. <br>获取此套接字的 `IP_TTL` 选项的值。<br>
    ///
    /// For more information about this option, see [`TcpListener::set_ttl`]. <br>有关此选项的更多信息，请参见 [`TcpListener::set_ttl`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    /// listener.set_ttl(100).expect("could not set TTL");
    /// assert_eq!(listener.ttl().unwrap_or(0), 100);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn ttl(&self) -> io::Result<u32> {
        self.0.ttl()
    }

    #[stable(feature = "net2_mutators", since = "1.9.0")]
    #[deprecated(since = "1.16.0", note = "this option can only be set before the socket is bound")]
    #[allow(missing_docs)]
    pub fn set_only_v6(&self, only_v6: bool) -> io::Result<()> {
        self.0.set_only_v6(only_v6)
    }

    #[stable(feature = "net2_mutators", since = "1.9.0")]
    #[deprecated(since = "1.16.0", note = "this option can only be set before the socket is bound")]
    #[allow(missing_docs)]
    pub fn only_v6(&self) -> io::Result<bool> {
        self.0.only_v6()
    }

    /// Gets the value of the `SO_ERROR` option on this socket. <br>获取此套接字上 `SO_ERROR` 选项的值。<br>
    ///
    /// This will retrieve the stored error in the underlying socket, clearing the field in the process. <br>这将检索底层套接字中存储的错误，从而清除进程中的字段。<br>
    /// This can be useful for checking errors between calls. <br>这对于检查两次调用之间的错误很有用。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:80").unwrap();
    /// listener.take_error().expect("No error was expected");
    /// ```
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn take_error(&self) -> io::Result<Option<io::Error>> {
        self.0.take_error()
    }

    /// Moves this TCP stream into or out of nonblocking mode. <br>将此 TCP 流移入或移出非阻塞模式。<br>
    ///
    /// This will result in the `accept` operation becoming nonblocking, i.e., immediately returning from their calls. <br>这将导致 `accept` 操作变为非阻塞，即立即从其调用中返回。<br>
    /// If the IO operation is successful, `Ok` is returned and no further action is required. <br>如果 IO 操作成功，则返回 `Ok`，并且不需要进一步的操作。<br>
    /// If the IO operation could not be completed and needs to be retried, an error with kind [`io::ErrorKind::WouldBlock`] is returned. <br>如果 IO 操作无法完成，需要重试，则返回类型为 [`io::ErrorKind::WouldBlock`] 的错误。<br>
    ///
    ///
    /// On Unix platforms, calling this method corresponds to calling `fcntl` `FIONBIO`. <br>在 Unix 平台上，调用此方法相当于调用 `fcntl` `FIONBIO`。<br>
    /// On Windows calling this method corresponds to calling `ioctlsocket` `FIONBIO`. <br>在 Windows 上，调用此方法对应于调用 `ioctlsocket` `FIONBIO`。<br>
    ///
    /// # Examples
    ///
    /// Bind a TCP listener to an address, listen for connections, and read bytes in nonblocking mode: <br>将 TCP 侦听器绑定到地址，侦听连接，并以非阻塞模式读取字节：<br>
    ///
    /// ```no_run
    /// use std::io;
    /// use std::net::TcpListener;
    ///
    /// let listener = TcpListener::bind("127.0.0.1:7878").unwrap();
    /// listener.set_nonblocking(true).expect("Cannot set non-blocking");
    ///
    /// # fn wait_for_fd() { unimplemented!() }
    /// # fn handle_connection(stream: std::net::TcpStream) { unimplemented!() }
    /// for stream in listener.incoming() {
    ///     match stream {
    ///         Ok(s) => {
    ///             // do something with the TcpStream <br>用 TcpStream 做某事<br>
    ///             handle_connection(s);
    ///         }
    ///         Err(ref e) if e.kind() == io::ErrorKind::WouldBlock => {
    ///             // wait until network socket is ready, typically implemented via platform-specific APIs such as epoll or IOCP <br>等待网络套接字就绪，通常通过平台特定的 API (例如 epoll 或 IOCP) 实现<br>
    /////
    ///             wait_for_fd();
    ///             continue;
    ///         }
    ///         Err(e) => panic!("encountered IO error: {e}"),
    ///     }
    /// }
    /// ```
    ///
    ///
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
        self.0.set_nonblocking(nonblocking)
    }
}

// In addition to the `impl`s here, `TcpListener` also has `impl`s for `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` and `AsRawFd`/`IntoRawFd`/`FromRawFd`, on Unix and WASI, and `AsSocket`/`From<OwnedSocket>`/`Into<OwnedSocket>` and `AsRawSocket`/`IntoRawSocket`/`FromRawSocket` on Windows. <br>除了这里的 impl 之外，`TcpListener` 还有 `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` 和 `AsRawFd`/`IntoRawFd`/`FromRawFd`、Unix 和 WASI 以及 Windows 上的 `AsSocket`/`From<OwnedSocket>`/`Into<OwnedSocket>` 和 `AsRawSocket`/`IntoRawSocket`/`FromRawSocket` 的 impl。<br>
//
//
//
//

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> Iterator for Incoming<'a> {
    type Item = io::Result<TcpStream>;
    fn next(&mut self) -> Option<io::Result<TcpStream>> {
        Some(self.listener.accept().map(|p| p.0))
    }
}

#[unstable(feature = "tcplistener_into_incoming", issue = "88339")]
impl Iterator for IntoIncoming {
    type Item = io::Result<TcpStream>;
    fn next(&mut self) -> Option<io::Result<TcpStream>> {
        Some(self.listener.accept().map(|p| p.0))
    }
}

impl AsInner<net_imp::TcpListener> for TcpListener {
    fn as_inner(&self) -> &net_imp::TcpListener {
        &self.0
    }
}

impl FromInner<net_imp::TcpListener> for TcpListener {
    fn from_inner(inner: net_imp::TcpListener) -> TcpListener {
        TcpListener(inner)
    }
}

impl IntoInner<net_imp::TcpListener> for TcpListener {
    fn into_inner(self) -> net_imp::TcpListener {
        self.0
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for TcpListener {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.0.fmt(f)
    }
}
