#[cfg(all(test, not(any(target_os = "emscripten", target_env = "sgx"))))]
mod tests;

use crate::fmt;
use crate::io::{self, ErrorKind};
use crate::net::{Ipv4Addr, Ipv6Addr, SocketAddr, ToSocketAddrs};
use crate::sys_common::net as net_imp;
use crate::sys_common::{AsInner, FromInner, IntoInner};
use crate::time::Duration;

/// A UDP socket. <br>UDP 套接字。<br>
///
/// After creating a `UdpSocket` by [`bind`]ing it to a socket address, data can be [sent to] and [received from] any other socket address. <br>通过将 `UdpSocket` 绑定到一个套接字地址来创建 `UdpSocket` 之后，数据可以是 [发送到][sent to] 和 [从哪里接收][received from] 任何其他套接字地址。<br>
///
/// Although UDP is a connectionless protocol, this implementation provides an interface to set an address where data should be sent and received from. <br>尽管 UDP 是无连接协议，但是此实现提供了一个接口，用于设置一个地址，在该地址处应发送和接收数据。<br>
/// After setting a remote address with [`connect`], data can be sent to and received from that address with [`send`] and [`recv`]. <br>使用 [`connect`] 设置远程地址后，可以使用 [`send`] 和 [`recv`] 向该地址发送数据和从该地址接收数据。<br>
///
/// As stated in the User Datagram Protocol's specification in [IETF RFC 768], UDP is an unordered, unreliable protocol; <br>如 [IETF RFC 768] 中的用户数据报协议规范中所述，UDP 是无序，不可靠的协议。<br> refer to [`TcpListener`] and [`TcpStream`] for TCP primitives. <br>有关 TCP 原语，请参考 [`TcpListener`] 和 [`TcpStream`]。<br>
///
///
/// [`bind`]: UdpSocket::bind
/// [`connect`]: UdpSocket::connect
/// [IETF RFC 768]: https://tools.ietf.org/html/rfc768
/// [`recv`]: UdpSocket::recv
/// [received from]: UdpSocket::recv_from
/// [`send`]: UdpSocket::send
/// [sent to]: UdpSocket::send_to
/// [`TcpListener`]: crate::net::TcpListener
/// [`TcpStream`]: crate::net::TcpStream
///
/// # Examples
///
/// ```no_run
/// use std::net::UdpSocket;
///
/// fn main() -> std::io::Result<()> {
///     {
///         let socket = UdpSocket::bind("127.0.0.1:34254")?;
///
///         // Receives a single datagram message on the socket. <br>在套接字上接收单个数据报消息。<br>
///         // If `buf` is too small to hold the message, it will be cut off. <br>如果 `buf` 太小而无法容纳该消息，它将被切断。<br>
///         let mut buf = [0; 10];
///         let (amt, src) = socket.recv_from(&mut buf)?;
///
///         // Redeclare `buf` as slice of the received data and send reverse data back to origin. <br>将 `buf` 声明为接收数据的切片，并将反向数据发送回原点。<br>
///         let buf = &mut buf[..amt];
///         buf.reverse();
///         socket.send_to(buf, &src)?;
///     } // the socket is closed here <br>套接字在这里关闭<br>
///     Ok(())
/// }
/// ```
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct UdpSocket(net_imp::UdpSocket);

impl UdpSocket {
    /// Creates a UDP socket from the given address. <br>从给定的地址创建一个 UDP 套接字。<br>
    ///
    /// The address type can be any implementor of [`ToSocketAddrs`] trait. <br>地址类型可以是 [`ToSocketAddrs`] trait 的任何实现者。<br> See its documentation for concrete examples. <br>有关具体的例子，请参见其文档。<br>
    ///
    /// If `addr` yields multiple addresses, `bind` will be attempted with each of the addresses until one succeeds and returns the socket. <br>如果 `addr` 产生多个地址，则将使用每个地址尝试 `bind`，直到一个成功并返回套接字为止。<br>
    ///
    /// If none of the addresses succeed in creating a socket, the error returned from the last attempt (the last address) is returned. <br>如果没有一个地址成功创建套接字，则返回上一次尝试返回的错误 (最后一个地址)。<br>
    ///
    /// # Examples
    ///
    /// Creates a UDP socket bound to `127.0.0.1:3400`: <br>创建绑定到 `127.0.0.1:3400` 的 UDP 套接字：<br>
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:3400").expect("couldn't bind to address");
    /// ```
    ///
    /// Creates a UDP socket bound to `127.0.0.1:3400`. <br>创建绑定到 `127.0.0.1:3400` 的 UDP 套接字。<br>
    /// If the socket cannot be bound to that address, create a UDP socket bound to `127.0.0.1:3401`: <br>如果套接字无法绑定到该地址，请创建绑定到 `127.0.0.1:3401` 的 UDP 套接字：<br>
    ///
    /// ```no_run
    /// use std::net::{SocketAddr, UdpSocket};
    ///
    /// let addrs = [
    ///     SocketAddr::from(([127, 0, 0, 1], 3400)),
    ///     SocketAddr::from(([127, 0, 0, 1], 3401)),
    /// ];
    /// let socket = UdpSocket::bind(&addrs[..]).expect("couldn't bind to address");
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn bind<A: ToSocketAddrs>(addr: A) -> io::Result<UdpSocket> {
        super::each_addr(addr, net_imp::UdpSocket::bind).map(UdpSocket)
    }

    /// Receives a single datagram message on the socket. <br>在套接字上接收单个数据报消息。<br>
    /// On success, returns the number of bytes read and the origin. <br>成功时，返回读取的字节数和源。<br>
    ///
    /// The function must be called with valid byte array `buf` of sufficient size to hold the message bytes. <br>必须使用足够大的有效字节数组 `buf` 来调用函数，以容纳消息字节。<br>
    ///
    /// If a message is too long to fit in the supplied buffer, excess bytes may be discarded. <br>如果消息太长而无法容纳在提供的缓冲区中，则多余的字节可能会被丢弃。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// let mut buf = [0; 10];
    /// let (number_of_bytes, src_addr) = socket.recv_from(&mut buf)
    ///                                         .expect("Didn't receive data");
    /// let filled_buf = &mut buf[..number_of_bytes];
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn recv_from(&self, buf: &mut [u8]) -> io::Result<(usize, SocketAddr)> {
        self.0.recv_from(buf)
    }

    /// Receives a single datagram message on the socket, without removing it from the queue. <br>在套接字上接收单个数据报消息，而无需将其从队列中删除。<br>
    /// On success, returns the number of bytes read and the origin. <br>成功时，返回读取的字节数和源。<br>
    ///
    /// The function must be called with valid byte array `buf` of sufficient size to hold the message bytes. <br>必须使用足够大的有效字节数组 `buf` 来调用函数，以容纳消息字节。<br>
    /// If a message is too long to fit in the supplied buffer, excess bytes may be discarded. <br>如果消息太长而无法容纳在提供的缓冲区中，则多余的字节可能会被丢弃。<br>
    ///
    /// Successive calls return the same data. <br>连续调用返回相同的数据。<br>
    /// This is accomplished by passing `MSG_PEEK` as a flag to the underlying `recvfrom` system call. <br>这是通过将 `MSG_PEEK` 作为标志传递到底层的 `recvfrom` 系统调用来实现的。<br>
    ///
    /// Do not use this function to implement busy waiting, instead use `libc::poll` to synchronize IO events on one or more sockets. <br>不要使用此函数来实现繁忙等待，而应使用 `libc::poll` 来同步一个或多个套接字上的 IO 事件。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// let mut buf = [0; 10];
    /// let (number_of_bytes, src_addr) = socket.peek_from(&mut buf)
    ///                                         .expect("Didn't receive data");
    /// let filled_buf = &mut buf[..number_of_bytes];
    /// ```
    ///
    #[stable(feature = "peek", since = "1.18.0")]
    pub fn peek_from(&self, buf: &mut [u8]) -> io::Result<(usize, SocketAddr)> {
        self.0.peek_from(buf)
    }

    /// Sends data on the socket to the given address. <br>将套接字上的数据发送到给定的地址。<br> On success, returns the number of bytes written. <br>成功时，返回写入的字节数。<br>
    ///
    /// Address type can be any implementor of [`ToSocketAddrs`] trait. <br>地址类型可以是 [`ToSocketAddrs`] trait 的任何实现者。<br> See its documentation for concrete examples. <br>有关具体的例子，请参见其文档。<br>
    ///
    /// It is possible for `addr` to yield multiple addresses, but `send_to` will only send data to the first address yielded by `addr`. <br>`addr` 可以产生多个地址，但是 `send_to` 只会将数据发送到 `addr` 产生的第一个地址。<br>
    ///
    ///
    /// This will return an error when the IP version of the local socket does not match that returned from [`ToSocketAddrs`]. <br>当本地套接字的 IP 版本与 [`ToSocketAddrs`] 返回的 IP 版本不匹配时，这将返回错误。<br>
    ///
    /// See [Issue #34202] for more details. <br>有关更多详细信息，请参见 [Issue #34202]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.send_to(&[0; 10], "127.0.0.1:4242").expect("couldn't send data");
    /// ```
    ///
    /// [Issue #34202]: https://github.com/rust-lang/rust/issues/34202
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn send_to<A: ToSocketAddrs>(&self, buf: &[u8], addr: A) -> io::Result<usize> {
        match addr.to_socket_addrs()?.next() {
            Some(addr) => self.0.send_to(buf, &addr),
            None => {
                Err(io::const_io_error!(ErrorKind::InvalidInput, "no addresses to send data to"))
            }
        }
    }

    /// Returns the socket address of the remote peer this socket was connected to. <br>返回此套接字连接到的远程对等方的套接字地址。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{Ipv4Addr, SocketAddr, SocketAddrV4, UdpSocket};
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.connect("192.168.0.1:41203").expect("couldn't connect to address");
    /// assert_eq!(socket.peer_addr().unwrap(),
    ///            SocketAddr::V4(SocketAddrV4::new(Ipv4Addr::new(192, 168, 0, 1), 41203)));
    /// ```
    ///
    /// If the socket isn't connected, it will return a [`NotConnected`] error. <br>如果未连接套接字，它将返回 [`NotConnected`] 错误。<br>
    ///
    /// [`NotConnected`]: io::ErrorKind::NotConnected
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// assert_eq!(socket.peer_addr().unwrap_err().kind(),
    ///            std::io::ErrorKind::NotConnected);
    /// ```
    #[stable(feature = "udp_peer_addr", since = "1.40.0")]
    pub fn peer_addr(&self) -> io::Result<SocketAddr> {
        self.0.peer_addr()
    }

    /// Returns the socket address that this socket was created from. <br>返回创建此套接字的套接字地址。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::{Ipv4Addr, SocketAddr, SocketAddrV4, UdpSocket};
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// assert_eq!(socket.local_addr().unwrap(),
    ///            SocketAddr::V4(SocketAddrV4::new(Ipv4Addr::new(127, 0, 0, 1), 34254)));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn local_addr(&self) -> io::Result<SocketAddr> {
        self.0.socket_addr()
    }

    /// Creates a new independently owned handle to the underlying socket. <br>为底层套接字创建一个新的独立的拥有所有权的句柄。<br>
    ///
    /// The returned `UdpSocket` is a reference to the same socket that this object references. <br>返回的 `UdpSocket` 是与此对象引用相同的套接字的引用。<br>
    /// Both handles will read and write the same port, and options set on one socket will be propagated to the other. <br>两个句柄都将读取和写入相同的端口，并且在一个套接字上设置的选项将传播到另一个套接字。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// let socket_clone = socket.try_clone().expect("couldn't clone the socket");
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn try_clone(&self) -> io::Result<UdpSocket> {
        self.0.duplicate().map(UdpSocket)
    }

    /// Sets the read timeout to the timeout specified. <br>将读取超时设置为指定的超时。<br>
    ///
    /// If the value specified is [`None`], then [`read`] calls will block indefinitely. <br>如果指定的值为 [`None`]，则 [`read`] 调用将无限期阻塞。<br>
    /// An [`Err`] is returned if the zero [`Duration`] is passed to this method. <br>如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]。<br>
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// Platforms may return a different error code whenever a read times out as a result of setting this option. <br>由于设置此选项而导致读取超时时，平台可能会返回不同的错误代码。<br>
    /// For example Unix typically returns an error of the kind [`WouldBlock`], but Windows may return [`TimedOut`]. <br>例如，Unix 通常返回类型为 [`WouldBlock`] 的错误，但是 Windows 可能返回 [`TimedOut`]。<br>
    ///
    ///
    /// [`read`]: io::Read::read
    /// [`WouldBlock`]: io::ErrorKind::WouldBlock
    /// [`TimedOut`]: io::ErrorKind::TimedOut
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_read_timeout(None).expect("set_read_timeout call failed");
    /// ```
    ///
    /// An [`Err`] is returned if the zero [`Duration`] is passed to this method: <br>如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]：<br>
    ///
    /// ```no_run
    /// use std::io;
    /// use std::net::UdpSocket;
    /// use std::time::Duration;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").unwrap();
    /// let result = socket.set_read_timeout(Some(Duration::new(0, 0)));
    /// let err = result.unwrap_err();
    /// assert_eq!(err.kind(), io::ErrorKind::InvalidInput)
    /// ```
    ///
    ///
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn set_read_timeout(&self, dur: Option<Duration>) -> io::Result<()> {
        self.0.set_read_timeout(dur)
    }

    /// Sets the write timeout to the timeout specified. <br>将写超时设置为指定的超时。<br>
    ///
    /// If the value specified is [`None`], then [`write`] calls will block indefinitely. <br>如果指定的值为 [`None`]，则 [`write`] 调用将无限期阻塞。<br>
    /// An [`Err`] is returned if the zero [`Duration`] is passed to this method. <br>如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]。<br>
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// Platforms may return a different error code whenever a write times out as a result of setting this option. <br>由于设置此选项而导致写超时时，平台可能会返回不同的错误代码。<br>
    /// For example Unix typically returns an error of the kind [`WouldBlock`], but Windows may return [`TimedOut`]. <br>例如，Unix 通常返回类型为 [`WouldBlock`] 的错误，但是 Windows 可能返回 [`TimedOut`]。<br>
    ///
    ///
    /// [`write`]: io::Write::write
    /// [`WouldBlock`]: io::ErrorKind::WouldBlock
    /// [`TimedOut`]: io::ErrorKind::TimedOut
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_write_timeout(None).expect("set_write_timeout call failed");
    /// ```
    ///
    /// An [`Err`] is returned if the zero [`Duration`] is passed to this method: <br>如果将零 [`Duration`] 传递给此方法，则返回 [`Err`]：<br>
    ///
    /// ```no_run
    /// use std::io;
    /// use std::net::UdpSocket;
    /// use std::time::Duration;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").unwrap();
    /// let result = socket.set_write_timeout(Some(Duration::new(0, 0)));
    /// let err = result.unwrap_err();
    /// assert_eq!(err.kind(), io::ErrorKind::InvalidInput)
    /// ```
    ///
    ///
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn set_write_timeout(&self, dur: Option<Duration>) -> io::Result<()> {
        self.0.set_write_timeout(dur)
    }

    /// Returns the read timeout of this socket. <br>返回此套接字的读取超时。<br>
    ///
    /// If the timeout is [`None`], then [`read`] calls will block indefinitely. <br>如果超时为 [`None`]，则 [`read`] 调用将无限期阻塞。<br>
    ///
    /// [`read`]: io::Read::read
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_read_timeout(None).expect("set_read_timeout call failed");
    /// assert_eq!(socket.read_timeout().unwrap(), None);
    /// ```
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn read_timeout(&self) -> io::Result<Option<Duration>> {
        self.0.read_timeout()
    }

    /// Returns the write timeout of this socket. <br>返回此套接字的写入超时。<br>
    ///
    /// If the timeout is [`None`], then [`write`] calls will block indefinitely. <br>如果超时为 [`None`]，则 [`write`] 调用将无限期阻塞。<br>
    ///
    /// [`write`]: io::Write::write
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_write_timeout(None).expect("set_write_timeout call failed");
    /// assert_eq!(socket.write_timeout().unwrap(), None);
    /// ```
    #[stable(feature = "socket_timeout", since = "1.4.0")]
    pub fn write_timeout(&self) -> io::Result<Option<Duration>> {
        self.0.write_timeout()
    }

    /// Sets the value of the `SO_BROADCAST` option for this socket. <br>设置此套接字的 `SO_BROADCAST` 选项的值。<br>
    ///
    /// When enabled, this socket is allowed to send packets to a broadcast address. <br>启用后，允许此套接字将数据包发送到广播地址。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_broadcast(false).expect("set_broadcast call failed");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_broadcast(&self, broadcast: bool) -> io::Result<()> {
        self.0.set_broadcast(broadcast)
    }

    /// Gets the value of the `SO_BROADCAST` option for this socket. <br>获取此套接字的 `SO_BROADCAST` 选项的值。<br>
    ///
    /// For more information about this option, see [`UdpSocket::set_broadcast`]. <br>有关此选项的更多信息，请参见 [`UdpSocket::set_broadcast`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_broadcast(false).expect("set_broadcast call failed");
    /// assert_eq!(socket.broadcast().unwrap(), false);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn broadcast(&self) -> io::Result<bool> {
        self.0.broadcast()
    }

    /// Sets the value of the `IP_MULTICAST_LOOP` option for this socket. <br>设置此套接字的 `IP_MULTICAST_LOOP` 选项的值。<br>
    ///
    /// If enabled, multicast packets will be looped back to the local socket. <br>如果启用，则多播数据包将循环回到本地套接字。<br>
    /// Note that this might not have any effect on IPv6 sockets. <br>请注意，这可能对 IPv6 套接字没有任何影响。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_multicast_loop_v4(false).expect("set_multicast_loop_v4 call failed");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_multicast_loop_v4(&self, multicast_loop_v4: bool) -> io::Result<()> {
        self.0.set_multicast_loop_v4(multicast_loop_v4)
    }

    /// Gets the value of the `IP_MULTICAST_LOOP` option for this socket. <br>获取此套接字的 `IP_MULTICAST_LOOP` 选项的值。<br>
    ///
    /// For more information about this option, see [`UdpSocket::set_multicast_loop_v4`]. <br>有关此选项的更多信息，请参见 [`UdpSocket::set_multicast_loop_v4`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_multicast_loop_v4(false).expect("set_multicast_loop_v4 call failed");
    /// assert_eq!(socket.multicast_loop_v4().unwrap(), false);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn multicast_loop_v4(&self) -> io::Result<bool> {
        self.0.multicast_loop_v4()
    }

    /// Sets the value of the `IP_MULTICAST_TTL` option for this socket. <br>设置此套接字的 `IP_MULTICAST_TTL` 选项的值。<br>
    ///
    /// Indicates the time-to-live value of outgoing multicast packets for this socket. <br>指示此套接字的传出多播数据包的生存时间值。<br>
    /// The default value is 1 which means that multicast packets don't leave the local network unless explicitly requested. <br>默认值为 1，这意味着除非明确请求，否则多播数据包不会离开本地网络。<br>
    ///
    ///
    /// Note that this might not have any effect on IPv6 sockets. <br>请注意，这可能对 IPv6 套接字没有任何影响。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_multicast_ttl_v4(42).expect("set_multicast_ttl_v4 call failed");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_multicast_ttl_v4(&self, multicast_ttl_v4: u32) -> io::Result<()> {
        self.0.set_multicast_ttl_v4(multicast_ttl_v4)
    }

    /// Gets the value of the `IP_MULTICAST_TTL` option for this socket. <br>获取此套接字的 `IP_MULTICAST_TTL` 选项的值。<br>
    ///
    /// For more information about this option, see [`UdpSocket::set_multicast_ttl_v4`]. <br>有关此选项的更多信息，请参见 [`UdpSocket::set_multicast_ttl_v4`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_multicast_ttl_v4(42).expect("set_multicast_ttl_v4 call failed");
    /// assert_eq!(socket.multicast_ttl_v4().unwrap(), 42);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn multicast_ttl_v4(&self) -> io::Result<u32> {
        self.0.multicast_ttl_v4()
    }

    /// Sets the value of the `IPV6_MULTICAST_LOOP` option for this socket. <br>设置此套接字的 `IPV6_MULTICAST_LOOP` 选项的值。<br>
    ///
    /// Controls whether this socket sees the multicast packets it sends itself. <br>控制此套接字是否看到自己发送的多播数据包。<br>
    /// Note that this might not have any affect on IPv4 sockets. <br>请注意，这可能不会对 IPv4 套接字产生任何影响。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_multicast_loop_v6(false).expect("set_multicast_loop_v6 call failed");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_multicast_loop_v6(&self, multicast_loop_v6: bool) -> io::Result<()> {
        self.0.set_multicast_loop_v6(multicast_loop_v6)
    }

    /// Gets the value of the `IPV6_MULTICAST_LOOP` option for this socket. <br>获取此套接字的 `IPV6_MULTICAST_LOOP` 选项的值。<br>
    ///
    /// For more information about this option, see [`UdpSocket::set_multicast_loop_v6`]. <br>有关此选项的更多信息，请参见 [`UdpSocket::set_multicast_loop_v6`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_multicast_loop_v6(false).expect("set_multicast_loop_v6 call failed");
    /// assert_eq!(socket.multicast_loop_v6().unwrap(), false);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn multicast_loop_v6(&self) -> io::Result<bool> {
        self.0.multicast_loop_v6()
    }

    /// Sets the value for the `IP_TTL` option on this socket. <br>设置此套接字上 `IP_TTL` 选项的值。<br>
    ///
    /// This value sets the time-to-live field that is used in every packet sent from this socket. <br>此值设置从该套接字发送的每个数据包中使用的生存时间字段。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_ttl(42).expect("set_ttl call failed");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_ttl(&self, ttl: u32) -> io::Result<()> {
        self.0.set_ttl(ttl)
    }

    /// Gets the value of the `IP_TTL` option for this socket. <br>获取此套接字的 `IP_TTL` 选项的值。<br>
    ///
    /// For more information about this option, see [`UdpSocket::set_ttl`]. <br>有关此选项的更多信息，请参见 [`UdpSocket::set_ttl`]。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.set_ttl(42).expect("set_ttl call failed");
    /// assert_eq!(socket.ttl().unwrap(), 42);
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn ttl(&self) -> io::Result<u32> {
        self.0.ttl()
    }

    /// Executes an operation of the `IP_ADD_MEMBERSHIP` type. <br>执行 `IP_ADD_MEMBERSHIP` 类型的操作。<br>
    ///
    /// This function specifies a new multicast group for this socket to join. <br>此函数为此套接字指定一个新的多播组。<br>
    /// The address must be a valid multicast address, and `interface` is the address of the local interface with which the system should join the multicast group. <br>该地址必须是有效的多播地址，而 `interface` 是系统应加入多播组的本地接口的地址。<br>
    /// If it's equal to `INADDR_ANY` then an appropriate interface is chosen by the system. <br>如果等于 `INADDR_ANY`，则系统选择适当的接口。<br>
    ///
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn join_multicast_v4(&self, multiaddr: &Ipv4Addr, interface: &Ipv4Addr) -> io::Result<()> {
        self.0.join_multicast_v4(multiaddr, interface)
    }

    /// Executes an operation of the `IPV6_ADD_MEMBERSHIP` type. <br>执行 `IPV6_ADD_MEMBERSHIP` 类型的操作。<br>
    ///
    /// This function specifies a new multicast group for this socket to join. <br>此函数为此套接字指定一个新的多播组。<br>
    /// The address must be a valid multicast address, and `interface` is the index of the interface to join/leave (or 0 to indicate any interface). <br>该地址必须是有效的多播地址，并且 `interface` 是接口到 join/leave 的索引 (或 0 表示任何接口)。<br>
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn join_multicast_v6(&self, multiaddr: &Ipv6Addr, interface: u32) -> io::Result<()> {
        self.0.join_multicast_v6(multiaddr, interface)
    }

    /// Executes an operation of the `IP_DROP_MEMBERSHIP` type. <br>执行 `IP_DROP_MEMBERSHIP` 类型的操作。<br>
    ///
    /// For more information about this option, see [`UdpSocket::join_multicast_v4`]. <br>有关此选项的更多信息，请参见 [`UdpSocket::join_multicast_v4`]。<br>
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn leave_multicast_v4(&self, multiaddr: &Ipv4Addr, interface: &Ipv4Addr) -> io::Result<()> {
        self.0.leave_multicast_v4(multiaddr, interface)
    }

    /// Executes an operation of the `IPV6_DROP_MEMBERSHIP` type. <br>执行 `IPV6_DROP_MEMBERSHIP` 类型的操作。<br>
    ///
    /// For more information about this option, see [`UdpSocket::join_multicast_v6`]. <br>有关此选项的更多信息，请参见 [`UdpSocket::join_multicast_v6`]。<br>
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn leave_multicast_v6(&self, multiaddr: &Ipv6Addr, interface: u32) -> io::Result<()> {
        self.0.leave_multicast_v6(multiaddr, interface)
    }

    /// Gets the value of the `SO_ERROR` option on this socket. <br>获取此套接字上 `SO_ERROR` 选项的值。<br>
    ///
    /// This will retrieve the stored error in the underlying socket, clearing the field in the process. <br>这将检索底层套接字中存储的错误，从而清除进程中的字段。<br>
    /// This can be useful for checking errors between calls. <br>这对于检查两次调用之间的错误很有用。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// match socket.take_error() {
    ///     Ok(Some(error)) => println!("UdpSocket error: {error:?}"),
    ///     Ok(None) => println!("No error"),
    ///     Err(error) => println!("UdpSocket.take_error failed: {error:?}"),
    /// }
    /// ```
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn take_error(&self) -> io::Result<Option<io::Error>> {
        self.0.take_error()
    }

    /// Connects this UDP socket to a remote address, allowing the `send` and `recv` syscalls to be used to send data and also applies filters to only receive data from the specified address. <br>将此 UDP 套接字连接到远程地址，从而允许使用 `send` 和 `recv` 系统调用来发送数据，并且还应用过滤器以仅接收来自指定地址的数据。<br>
    ///
    /// If `addr` yields multiple addresses, `connect` will be attempted with each of the addresses until the underlying OS function returns no error. <br>如果 `addr` 产生多个地址，则 `connect` 将尝试使用每个地址，直到底层 OS 函数没有返回错误为止。<br>
    /// Note that usually, a successful `connect` call does not specify that there is a remote server listening on the port, rather, such an error would only be detected after the first send. <br>请注意，通常，成功的 `connect` 调用不会指定有远程服务器在端口上侦听，而是仅在第一次发送后才检测到此错误。<br>
    /// If the OS returns an error for each of the specified addresses, the error returned from the last connection attempt (the last address) is returned. <br>如果操作系统为每个指定的地址返回错误，则返回从上次连接尝试 (最后一个地址) 返回的错误。<br>
    ///
    /// # Examples
    ///
    /// Creates a UDP socket bound to `127.0.0.1:3400` and connect the socket to <br>创建绑定到 `127.0.0.1:3400` 的 UDP 套接字并将该套接字连接到<br>
    /// `127.0.0.1:8080`:
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:3400").expect("couldn't bind to address");
    /// socket.connect("127.0.0.1:8080").expect("connect function failed");
    /// ```
    ///
    /// Unlike in the TCP case, passing an array of addresses to the `connect` function of a UDP socket is not a useful thing to do: The OS will be unable to determine whether something is listening on the remote address without the application sending data. <br>与 TCP 情况不同，将地址数组传递给 UDP 套接字的 `connect` 函数不是一件有用的事情：在没有应用程序发送数据的情况下，操作系统将无法确定是否有人在监听远程地址。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn connect<A: ToSocketAddrs>(&self, addr: A) -> io::Result<()> {
        super::each_addr(addr, |addr| self.0.connect(addr))
    }

    /// Sends data on the socket to the remote address to which it is connected. <br>将套接字上的数据发送到它所连接的远程地址。<br>
    ///
    /// [`UdpSocket::connect`] will connect this socket to a remote address. <br>[`UdpSocket::connect`] 将把这个套接字连接到一个远程地址。<br>
    /// This method will fail if the socket is not connected. <br>如果未连接套接字，则此方法将失败。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.connect("127.0.0.1:8080").expect("connect function failed");
    /// socket.send(&[0, 1, 2]).expect("couldn't send message");
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn send(&self, buf: &[u8]) -> io::Result<usize> {
        self.0.send(buf)
    }

    /// Receives a single datagram message on the socket from the remote address to which it is connected. <br>从套接字连接到的远程地址在套接字上接收单个数据报消息。<br>
    /// On success, returns the number of bytes read. <br>成功时，返回读取的字节数。<br>
    ///
    /// The function must be called with valid byte array `buf` of sufficient size to hold the message bytes. <br>必须使用足够大的有效字节数组 `buf` 来调用函数，以容纳消息字节。<br>
    ///
    /// If a message is too long to fit in the supplied buffer, excess bytes may be discarded. <br>如果消息太长而无法容纳在提供的缓冲区中，则多余的字节可能会被丢弃。<br>
    ///
    /// [`UdpSocket::connect`] will connect this socket to a remote address. <br>[`UdpSocket::connect`] 将把这个套接字连接到一个远程地址。<br>
    /// This method will fail if the socket is not connected. <br>如果未连接套接字，则此方法将失败。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.connect("127.0.0.1:8080").expect("connect function failed");
    /// let mut buf = [0; 10];
    /// match socket.recv(&mut buf) {
    ///     Ok(received) => println!("received {received} bytes {:?}", &buf[..received]),
    ///     Err(e) => println!("recv function failed: {e:?}"),
    /// }
    /// ```
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn recv(&self, buf: &mut [u8]) -> io::Result<usize> {
        self.0.recv(buf)
    }

    /// Receives single datagram on the socket from the remote address to which it is connected, without removing the message from input queue. <br>从套接字所连接的远程地址接收套接字上的单个数据报，而不会从输入队列中删除消息。<br>
    /// On success, returns the number of bytes peeked. <br>成功时，返回偷看的字节数。<br>
    ///
    /// The function must be called with valid byte array `buf` of sufficient size to hold the message bytes. <br>必须使用足够大的有效字节数组 `buf` 来调用函数，以容纳消息字节。<br>
    /// If a message is too long to fit in the supplied buffer, excess bytes may be discarded. <br>如果消息太长而无法容纳在提供的缓冲区中，则多余的字节可能会被丢弃。<br>
    ///
    /// Successive calls return the same data. <br>连续调用返回相同的数据。<br> This is accomplished by passing `MSG_PEEK` as a flag to the underlying `recv` system call. <br>这是通过将 `MSG_PEEK` 作为标志传递给底层的 `recv` 系统调用来实现的。<br>
    ///
    /// Do not use this function to implement busy waiting, instead use `libc::poll` to synchronize IO events on one or more sockets. <br>不要使用此函数来实现繁忙等待，而应使用 `libc::poll` 来同步一个或多个套接字上的 IO 事件。<br>
    ///
    /// [`UdpSocket::connect`] will connect this socket to a remote address. <br>[`UdpSocket::connect`] 将把这个套接字连接到一个远程地址。<br> This method will fail if the socket is not connected. <br>如果未连接套接字，则此方法将失败。<br>
    ///
    /// # Errors
    ///
    /// This method will fail if the socket is not connected. <br>如果未连接套接字，则此方法将失败。<br> The `connect` method will connect this socket to a remote address. <br>`connect` 方法会将此套接字连接到远程地址。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
    /// socket.connect("127.0.0.1:8080").expect("connect function failed");
    /// let mut buf = [0; 10];
    /// match socket.peek(&mut buf) {
    ///     Ok(received) => println!("received {received} bytes"),
    ///     Err(e) => println!("peek function failed: {e:?}"),
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "peek", since = "1.18.0")]
    pub fn peek(&self, buf: &mut [u8]) -> io::Result<usize> {
        self.0.peek(buf)
    }

    /// Moves this UDP socket into or out of nonblocking mode. <br>将此 UDP 套接字移入或移出非阻塞模式。<br>
    ///
    /// This will result in `recv`, `recv_from`, `send`, and `send_to` operations becoming nonblocking, i.e., immediately returning from their calls. <br>这将导致 `recv`，`recv_from`，`send` 和 `send_to` 操作变为非阻塞，即立即从其调用中返回。<br>
    ///
    /// If the IO operation is successful, `Ok` is returned and no further action is required. <br>如果 IO 操作成功，则返回 `Ok`，并且不需要进一步的操作。<br>
    /// If the IO operation could not be completed and needs to be retried, an error with kind [`io::ErrorKind::WouldBlock`] is returned. <br>如果 IO 操作无法完成，需要重试，则返回类型为 [`io::ErrorKind::WouldBlock`] 的错误。<br>
    ///
    /// On Unix platforms, calling this method corresponds to calling `fcntl` `FIONBIO`. <br>在 Unix 平台上，调用此方法相当于调用 `fcntl` `FIONBIO`。<br>
    /// On Windows calling this method corresponds to calling `ioctlsocket` `FIONBIO`. <br>在 Windows 上，调用此方法对应于调用 `ioctlsocket` `FIONBIO`。<br>
    ///
    /// # Examples
    ///
    /// Creates a UDP socket bound to `127.0.0.1:7878` and read bytes in nonblocking mode: <br>创建绑定到 `127.0.0.1:7878` 的 UDP 套接字，并以非阻塞模式读取字节：<br>
    ///
    /// ```no_run
    /// use std::io;
    /// use std::net::UdpSocket;
    ///
    /// let socket = UdpSocket::bind("127.0.0.1:7878").unwrap();
    /// socket.set_nonblocking(true).unwrap();
    ///
    /// # fn wait_for_fd() { unimplemented!() }
    /// let mut buf = [0; 10];
    /// let (num_bytes_read, _) = loop {
    ///     match socket.recv_from(&mut buf) {
    ///         Ok(n) => break n,
    ///         Err(ref e) if e.kind() == io::ErrorKind::WouldBlock => {
    ///             // wait until network socket is ready, typically implemented via platform-specific APIs such as epoll or IOCP <br>等待网络套接字就绪，通常通过平台特定的 API (例如 epoll 或 IOCP) 实现<br>
    /////
    ///             wait_for_fd();
    ///         }
    ///         Err(e) => panic!("encountered IO error: {e}"),
    ///     }
    /// };
    /// println!("bytes: {:?}", &buf[..num_bytes_read]);
    /// ```
    ///
    ///
    ///
    ///
    #[stable(feature = "net2_mutators", since = "1.9.0")]
    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
        self.0.set_nonblocking(nonblocking)
    }
}

// In addition to the `impl`s here, `UdpSocket` also has `impl`s for `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` and `AsRawFd`/`IntoRawFd`/`FromRawFd`, on Unix and WASI, and `AsSocket`/`From<OwnedSocket>`/`Into<OwnedSocket>` and `AsRawSocket`/`IntoRawSocket`/`FromRawSocket` on Windows. <br>除了这里的 impl 之外，`UdpSocket` 还有 `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` 和 `AsRawFd`/`IntoRawFd`/`FromRawFd`、Unix 和 WASI 以及 Windows 上的 `AsSocket`/`From<OwnedSocket>`/`Into<OwnedSocket>` 和 `AsRawSocket`/`IntoRawSocket`/`FromRawSocket` 的 impl。<br>
//
//
//
//

impl AsInner<net_imp::UdpSocket> for UdpSocket {
    fn as_inner(&self) -> &net_imp::UdpSocket {
        &self.0
    }
}

impl FromInner<net_imp::UdpSocket> for UdpSocket {
    fn from_inner(inner: net_imp::UdpSocket) -> UdpSocket {
        UdpSocket(inner)
    }
}

impl IntoInner<net_imp::UdpSocket> for UdpSocket {
    fn into_inner(self) -> net_imp::UdpSocket {
        self.0
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for UdpSocket {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.0.fmt(f)
    }
}
