//! Raw Unix-like file descriptors. <br>原始的类 Unix 文件描述符。<br>

#![stable(feature = "rust1", since = "1.0.0")]

use crate::fs;
use crate::io;
use crate::os::raw;
#[cfg(all(doc, not(target_arch = "wasm32")))]
use crate::os::unix::io::AsFd;
#[cfg(unix)]
use crate::os::unix::io::OwnedFd;
#[cfg(target_os = "wasi")]
use crate::os::wasi::io::OwnedFd;
use crate::sys_common::{AsInner, IntoInner};

/// Raw file descriptors. <br>原始文件描述符。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub type RawFd = raw::c_int;

/// A trait to extract the raw file descriptor from an underlying object. <br>一个从底层对象中提取原始文件描述符的 trait。<br>
///
/// This is only available on unix and WASI platforms and must be imported in order to call the method. <br>这仅在 unix 和 WASI 平台上可用，必须导入才能调用该方法。<br>
/// Windows platforms have a corresponding `AsRawHandle` and `AsRawSocket` set of traits. <br>Windows 平台有相应的 `AsRawHandle` 和 `AsRawSocket` 的 traits 集合。<br>
///
#[stable(feature = "rust1", since = "1.0.0")]
pub trait AsRawFd {
    /// Extracts the raw file descriptor. <br>提取原始文件描述符。<br>
    ///
    /// This function is typically used to **borrow** an owned file descriptor. <br>此函数通常用于**借用**拥有所有权的文件描述符。<br>
    /// When used in this way, this method does **not** pass ownership of the raw file descriptor to the caller, and the file descriptor is only guaranteed to be valid while the original object has not yet been destroyed. <br>以这种方式使用时，该方法**不**传递原始文件描述符的所有权，并且文件描述符仅在原始 object 尚未被调用者被调用时才保证有效。<br>
    ///
    ///
    /// However, borrowing is not strictly required. <br>但是，借用并不是严格要求的。<br> See [`AsFd::as_fd`] for an API which strictly borrows a file descriptor. <br>有关严格借用文件描述符的 API，请参见 [`AsFd::as_fd`]。<br>
    ///
    /// # Example
    ///
    /// ```no_run
    /// use std::fs::File;
    /// # use std::io;
    /// #[cfg(unix)]
    /// use std::os::unix::io::{AsRawFd, RawFd};
    /// #[cfg(target_os = "wasi")]
    /// use std::os::wasi::io::{AsRawFd, RawFd};
    ///
    /// let mut f = File::open("foo.txt")?;
    /// // Note that `raw_fd` is only valid as long as `f` exists. <br>注意，`raw_fd` 仅在存在 `f` 时才有效。<br>
    /// #[cfg(any(unix, target_os = "wasi"))]
    /// let raw_fd: RawFd = f.as_raw_fd();
    /// # Ok::<(), io::Error>(())
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn as_raw_fd(&self) -> RawFd;
}

/// A trait to express the ability to construct an object from a raw file descriptor. <br>一个表示从原始文件描述符构造对象的能力的 trait。<br>
///
#[stable(feature = "from_raw_os", since = "1.1.0")]
pub trait FromRawFd {
    /// Constructs a new instance of `Self` from the given raw file descriptor. <br>根据给定的原始文件描述符构造 `Self` 的新实例。<br>
    ///
    /// This function is typically used to **consume ownership** of the specified file descriptor. <br>此函数通常用于**消耗指定文件描述符的所有权**。<br>
    /// When used in this way, the returned object will take responsibility for closing it when the object goes out of scope. <br>以这种方式使用时，返回的 object 将负责在 object 离开角色域时关闭它。<br>
    ///
    ///
    /// However, consuming ownership is not strictly required. <br>但是，并不严格要求使用所有权。<br>
    /// Use a [`From<OwnedFd>::from`] implementation for an API which strictly consumes ownership. <br>对严格消耗所有权的 API 使用 [`From<OwnedFd>::from`] 实现。<br>
    ///
    /// # Safety
    ///
    /// The `fd` passed in must be a valid an open file descriptor. <br>传入的 `fd` 必须是有效的打开文件描述符。<br>
    ///
    /// # Example
    ///
    /// ```no_run
    /// use std::fs::File;
    /// # use std::io;
    /// #[cfg(unix)]
    /// use std::os::unix::io::{FromRawFd, IntoRawFd, RawFd};
    /// #[cfg(target_os = "wasi")]
    /// use std::os::wasi::io::{FromRawFd, IntoRawFd, RawFd};
    ///
    /// let f = File::open("foo.txt")?;
    /// # #[cfg(any(unix, target_os = "wasi"))]
    /// let raw_fd: RawFd = f.into_raw_fd();
    /// // SAFETY: no other functions should call `from_raw_fd`, so there is only one owner for the file descriptor. <br>没有其他函数可以调用 `from_raw_fd`，因此文件描述符只有一个所有者。<br>
    /////
    /// # #[cfg(any(unix, target_os = "wasi"))]
    /// let f = unsafe { File::from_raw_fd(raw_fd) };
    /// # Ok::<(), io::Error>(())
    /// ```
    ///
    ///
    ///
    #[stable(feature = "from_raw_os", since = "1.1.0")]
    unsafe fn from_raw_fd(fd: RawFd) -> Self;
}

/// A trait to express the ability to consume an object and acquire ownership of its raw file descriptor. <br>一个 trait，表示消费对象并获得其原始文件描述符所有权的能力。<br>
///
#[stable(feature = "into_raw_os", since = "1.4.0")]
pub trait IntoRawFd {
    /// Consumes this object, returning the raw underlying file descriptor. <br>消费这个对象，返回原始的底层文件描述符。<br>
    ///
    /// This function is typically used to **transfer ownership** of the underlying file descriptor to the caller. <br>此函数通常用于将底层文件描述符的所有权 ** 转移给调用者。<br>
    /// When used in this way, callers are then the unique owners of the file descriptor and must close it once it's no longer needed. <br>当以这种方式使用时，调用者是文件描述符的唯一所有者，一旦不再需要它就必须关闭它。<br>
    ///
    ///
    /// However, transferring ownership is not strictly required. <br>但是，转让所有权并不是严格要求的。<br>
    /// Use a [`Into<OwnedFd>::into`] implementation for an API which strictly transfers ownership. <br>对严格转移所有权的 API 使用 [`Into<OwnedFd>::into`] 实现。<br>
    ///
    /// # Example
    ///
    /// ```no_run
    /// use std::fs::File;
    /// # use std::io;
    /// #[cfg(unix)]
    /// use std::os::unix::io::{IntoRawFd, RawFd};
    /// #[cfg(target_os = "wasi")]
    /// use std::os::wasi::io::{IntoRawFd, RawFd};
    ///
    /// let f = File::open("foo.txt")?;
    /// #[cfg(any(unix, target_os = "wasi"))]
    /// let raw_fd: RawFd = f.into_raw_fd();
    /// # Ok::<(), io::Error>(())
    /// ```
    ///
    #[stable(feature = "into_raw_os", since = "1.4.0")]
    fn into_raw_fd(self) -> RawFd;
}

#[stable(feature = "raw_fd_reflexive_traits", since = "1.48.0")]
impl AsRawFd for RawFd {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        *self
    }
}
#[stable(feature = "raw_fd_reflexive_traits", since = "1.48.0")]
impl IntoRawFd for RawFd {
    #[inline]
    fn into_raw_fd(self) -> RawFd {
        self
    }
}
#[stable(feature = "raw_fd_reflexive_traits", since = "1.48.0")]
impl FromRawFd for RawFd {
    #[inline]
    unsafe fn from_raw_fd(fd: RawFd) -> RawFd {
        fd
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRawFd for fs::File {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        self.as_inner().as_raw_fd()
    }
}
#[stable(feature = "from_raw_os", since = "1.1.0")]
impl FromRawFd for fs::File {
    #[inline]
    unsafe fn from_raw_fd(fd: RawFd) -> fs::File {
        unsafe { fs::File::from(OwnedFd::from_raw_fd(fd)) }
    }
}
#[stable(feature = "into_raw_os", since = "1.4.0")]
impl IntoRawFd for fs::File {
    #[inline]
    fn into_raw_fd(self) -> RawFd {
        self.into_inner().into_inner().into_raw_fd()
    }
}

#[stable(feature = "asraw_stdio", since = "1.21.0")]
impl AsRawFd for io::Stdin {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        libc::STDIN_FILENO
    }
}

#[stable(feature = "asraw_stdio", since = "1.21.0")]
impl AsRawFd for io::Stdout {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        libc::STDOUT_FILENO
    }
}

#[stable(feature = "asraw_stdio", since = "1.21.0")]
impl AsRawFd for io::Stderr {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        libc::STDERR_FILENO
    }
}

#[stable(feature = "asraw_stdio_locks", since = "1.35.0")]
impl<'a> AsRawFd for io::StdinLock<'a> {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        libc::STDIN_FILENO
    }
}

#[stable(feature = "asraw_stdio_locks", since = "1.35.0")]
impl<'a> AsRawFd for io::StdoutLock<'a> {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        libc::STDOUT_FILENO
    }
}

#[stable(feature = "asraw_stdio_locks", since = "1.35.0")]
impl<'a> AsRawFd for io::StderrLock<'a> {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        libc::STDERR_FILENO
    }
}
