//! 特定于 SGX 的通用 I/O 原语扩展
//!
//! SGX 文件描述符的行为与 Unix 文件描述符不同。
//! 有关更多详细信息，请参见 [`TryIntoRawFd`] 的描述。
#![unstable(feature = "sgx_platform", issue = "56975")]

use crate::net;
pub use crate::sys::abi::usercalls::raw::Fd as RawFd;
use crate::sys::{self, AsInner, FromInner, IntoInner, TryIntoInner};

/// 一个从底层对象中提取原始 SGX 文件描述符的 trait。
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub trait AsRawFd {
    /// 提取原始文件描述符。
    ///
    /// 此方法不会将原始文件描述符的所有权传递给调用者。
    /// 仅当原始对象尚未销毁时，才保证描述符是有效的。
    ///
    #[unstable(feature = "sgx_platform", issue = "56975")]
    fn as_raw_fd(&self) -> RawFd;
}

/// 一个表示从原始文件描述符构造对象的能力的 trait。
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub trait FromRawFd {
    /// 关联类型，其中包含 `Self` 的相关元数据。
    type Metadata: Default;

    /// 根据给定的原始文件描述符和元数据构造 `Self` 的新实例。
    ///
    /// 这个函数会消耗指定文件描述符的所有权。
    /// 当对象离开作用域时，返回的对象将负责关闭它。
    ///
    /// 此函数也是不安全的，因为当前返回的原语具有它们是包装的文件描述符的唯一所有者的约定。
    ///
    /// 该函数的使用可能会意外地允许违反该契约，这可能会导致依赖该契约的代码的内存不安全。
    ///
    ///
    ///
    ///
    #[unstable(feature = "sgx_platform", issue = "56975")]
    unsafe fn from_raw_fd(fd: RawFd, metadata: Self::Metadata) -> Self;
}

/// 一个 trait，表示消费对象并获得其原始文件描述符所有权的能力。
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub trait TryIntoRawFd: Sized {
    /// 消耗此对象，如果这个对象没有被克隆，就返回原始底层文件描述符。
    ///
    /// 该函数将底层文件描述符的所有权转移给调用者。
    /// 这样，调用者便是文件描述符的唯一所有者，一旦不再需要描述符，就必须关闭该描述符。
    ///
    ///
    /// 与其他平台不同，在 SGX 上，文件描述符在对象的所有克隆之间共享。
    /// 为了避免竞争状态，此函数仅在最终克隆上调用时才返回 `Ok`。
    ///
    ///
    #[unstable(feature = "sgx_platform", issue = "56975")]
    fn try_into_raw_fd(self) -> Result<RawFd, Self>;
}

impl AsRawFd for net::TcpStream {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        *self.as_inner().as_inner().as_inner().as_inner()
    }
}

impl AsRawFd for net::TcpListener {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        *self.as_inner().as_inner().as_inner().as_inner()
    }
}

/// `TcpStream` 的元数据。
#[derive(Debug, Clone, Default)]
#[unstable(feature = "sgx_platform", issue = "56975")]
pub struct TcpStreamMetadata {
    /// TCP 流的本地地址
    pub local_addr: Option<String>,
    /// TCP 流的对端地址
    pub peer_addr: Option<String>,
}

impl FromRawFd for net::TcpStream {
    type Metadata = TcpStreamMetadata;

    #[inline]
    unsafe fn from_raw_fd(fd: RawFd, metadata: Self::Metadata) -> net::TcpStream {
        let fd = sys::fd::FileDesc::from_inner(fd);
        let socket = sys::net::Socket::from_inner((fd, metadata.local_addr));
        net::TcpStream::from_inner(sys::net::TcpStream::from_inner((socket, metadata.peer_addr)))
    }
}

/// `TcpListener` 的元数据。
#[derive(Debug, Clone, Default)]
#[unstable(feature = "sgx_platform", issue = "56975")]
pub struct TcpListenerMetadata {
    /// TCP 侦听器的本地地址
    pub local_addr: Option<String>,
}

impl FromRawFd for net::TcpListener {
    type Metadata = TcpListenerMetadata;

    #[inline]
    unsafe fn from_raw_fd(fd: RawFd, metadata: Self::Metadata) -> net::TcpListener {
        let fd = sys::fd::FileDesc::from_inner(fd);
        let socket = sys::net::Socket::from_inner((fd, metadata.local_addr));
        net::TcpListener::from_inner(sys::net::TcpListener::from_inner(socket))
    }
}

impl TryIntoRawFd for net::TcpStream {
    #[inline]
    fn try_into_raw_fd(self) -> Result<RawFd, Self> {
        let (socket, peer_addr) = self.into_inner().into_inner();
        match socket.try_into_inner() {
            Ok(fd) => Ok(fd.into_inner()),
            Err(socket) => {
                let sys = sys::net::TcpStream::from_inner((socket, peer_addr));
                Err(net::TcpStream::from_inner(sys))
            }
        }
    }
}

impl TryIntoRawFd for net::TcpListener {
    #[inline]
    fn try_into_raw_fd(self) -> Result<RawFd, Self> {
        match self.into_inner().into_inner().try_into_inner() {
            Ok(fd) => Ok(fd.into_inner()),
            Err(socket) => {
                let sys = sys::net::TcpListener::from_inner(socket);
                Err(net::TcpListener::from_inner(sys))
            }
        }
    }
}
