//! SGX-specific extensions to general I/O primitives <br>特定于 SGX 的通用 I/O 原语扩展<br>
//!
//! SGX file descriptors behave differently from Unix file descriptors. <br>SGX 文件描述符的行为与 Unix 文件描述符不同。<br>
//! See the description of [`TryIntoRawFd`] for more details. <br>有关更多详细信息，请参见 [`TryIntoRawFd`] 的描述。<br>
#![unstable(feature = "sgx_platform", issue = "56975")]

use crate::net;
pub use crate::sys::abi::usercalls::raw::Fd as RawFd;
use crate::sys::{self, AsInner, FromInner, IntoInner, TryIntoInner};

/// A trait to extract the raw SGX file descriptor from an underlying object. <br>一个从底层对象中提取原始 SGX 文件描述符的 trait。<br>
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub trait AsRawFd {
    /// Extracts the raw file descriptor. <br>提取原始文件描述符。<br>
    ///
    /// This method does **not** pass ownership of the raw file descriptor to the caller. <br>此方法不会将原始文件描述符的所有权传递给调用者。<br>
    /// The descriptor is only guaranteed to be valid while the original object has not yet been destroyed. <br>仅当原始对象尚未销毁时，才保证描述符是有效的。<br>
    ///
    #[unstable(feature = "sgx_platform", issue = "56975")]
    fn as_raw_fd(&self) -> RawFd;
}

/// A trait to express the ability to construct an object from a raw file descriptor. <br>一个表示从原始文件描述符构造对象的能力的 trait。<br>
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub trait FromRawFd {
    /// An associated type that contains relevant metadata for `Self`. <br>关联类型，其中包含 `Self` 的相关元数据。<br>
    type Metadata: Default;

    /// Constructs a new instance of `Self` from the given raw file descriptor and metadata. <br>根据给定的原始文件描述符和元数据构造 `Self` 的新实例。<br>
    ///
    /// This function **consumes ownership** of the specified file descriptor. <br>这个函数会消耗指定文件描述符的所有权。<br>
    /// The returned object will take responsibility for closing it when the object goes out of scope. <br>当对象离开作用域时，返回的对象将负责关闭它。<br>
    ///
    /// This function is also unsafe as the primitives currently returned have the contract that they are the sole owner of the file descriptor they are wrapping. <br>此函数也是不安全的，因为当前返回的原语具有它们是包装的文件描述符的唯一所有者的约定。<br>
    ///
    /// Usage of this function could accidentally allow violating this contract which can cause memory unsafety in code that relies on it being true. <br>该函数的使用可能会意外地允许违反该契约，这可能会导致依赖该契约的代码的内存不安全。<br>
    ///
    ///
    ///
    ///
    #[unstable(feature = "sgx_platform", issue = "56975")]
    unsafe fn from_raw_fd(fd: RawFd, metadata: Self::Metadata) -> Self;
}

/// A trait to express the ability to consume an object and acquire ownership of its raw file descriptor. <br>一个 trait，表示消费对象并获得其原始文件描述符所有权的能力。<br>
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub trait TryIntoRawFd: Sized {
    /// Consumes this object, returning the raw underlying file descriptor, if this object is not cloned. <br>消耗此对象，如果这个对象没有被克隆，就返回原始底层文件描述符。<br>
    ///
    /// This function **transfers ownership** of the underlying file descriptor to the caller. <br>该函数将底层文件描述符的所有权转移给调用者。<br>
    /// Callers are then the unique owners of the file descriptor and must close the descriptor once it's no longer needed. <br>这样，调用者便是文件描述符的唯一所有者，一旦不再需要描述符，就必须关闭该描述符。<br>
    ///
    ///
    /// Unlike other platforms, on SGX, the file descriptor is shared between all clones of an object. <br>与其他平台不同，在 SGX 上，文件描述符在对象的所有克隆之间共享。<br>
    /// To avoid race conditions, this function will only return `Ok` when called on the final clone. <br>为了避免竞争状态，此函数仅在最终克隆上调用时才返回 `Ok`。<br>
    ///
    ///
    #[unstable(feature = "sgx_platform", issue = "56975")]
    fn try_into_raw_fd(self) -> Result<RawFd, Self>;
}

impl AsRawFd for net::TcpStream {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        *self.as_inner().as_inner().as_inner().as_inner()
    }
}

impl AsRawFd for net::TcpListener {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        *self.as_inner().as_inner().as_inner().as_inner()
    }
}

/// Metadata for `TcpStream`. <br>`TcpStream` 的元数据。<br>
#[derive(Debug, Clone, Default)]
#[unstable(feature = "sgx_platform", issue = "56975")]
pub struct TcpStreamMetadata {
    /// Local address of the TCP stream <br>TCP 流的本地地址<br>
    pub local_addr: Option<String>,
    /// Peer address of the TCP stream <br>TCP 流的对端地址<br>
    pub peer_addr: Option<String>,
}

impl FromRawFd for net::TcpStream {
    type Metadata = TcpStreamMetadata;

    #[inline]
    unsafe fn from_raw_fd(fd: RawFd, metadata: Self::Metadata) -> net::TcpStream {
        let fd = sys::fd::FileDesc::from_inner(fd);
        let socket = sys::net::Socket::from_inner((fd, metadata.local_addr));
        net::TcpStream::from_inner(sys::net::TcpStream::from_inner((socket, metadata.peer_addr)))
    }
}

/// Metadata for `TcpListener`. <br>`TcpListener` 的元数据。<br>
#[derive(Debug, Clone, Default)]
#[unstable(feature = "sgx_platform", issue = "56975")]
pub struct TcpListenerMetadata {
    /// Local address of the TCP listener <br>TCP 侦听器的本地地址<br>
    pub local_addr: Option<String>,
}

impl FromRawFd for net::TcpListener {
    type Metadata = TcpListenerMetadata;

    #[inline]
    unsafe fn from_raw_fd(fd: RawFd, metadata: Self::Metadata) -> net::TcpListener {
        let fd = sys::fd::FileDesc::from_inner(fd);
        let socket = sys::net::Socket::from_inner((fd, metadata.local_addr));
        net::TcpListener::from_inner(sys::net::TcpListener::from_inner(socket))
    }
}

impl TryIntoRawFd for net::TcpStream {
    #[inline]
    fn try_into_raw_fd(self) -> Result<RawFd, Self> {
        let (socket, peer_addr) = self.into_inner().into_inner();
        match socket.try_into_inner() {
            Ok(fd) => Ok(fd.into_inner()),
            Err(socket) => {
                let sys = sys::net::TcpStream::from_inner((socket, peer_addr));
                Err(net::TcpStream::from_inner(sys))
            }
        }
    }
}

impl TryIntoRawFd for net::TcpListener {
    #[inline]
    fn try_into_raw_fd(self) -> Result<RawFd, Self> {
        match self.into_inner().into_inner().try_into_inner() {
            Ok(fd) => Ok(fd.into_inner()),
            Err(socket) => {
                let sys = sys::net::TcpListener::from_inner(socket);
                Err(net::TcpListener::from_inner(sys))
            }
        }
    }
}
