//! SOLID-specific extensions to general I/O primitives <br>特定于 SOLID 的通用 I/O 原语扩展<br>

#![deny(unsafe_op_in_unsafe_fn)]
#![unstable(feature = "solid_ext", issue = "none")]

use crate::net;
use crate::sys;
use crate::sys_common::{self, AsInner, FromInner, IntoInner};

/// Raw file descriptors. <br>原始文件描述符。<br>
pub type RawFd = i32;

/// A trait to extract the raw SOLID Sockets file descriptor from an underlying object. <br>从底层对象中提取原始 SOLID 套接字文件描述符的 trait。<br>
///
pub trait AsRawFd {
    /// Extracts the raw file descriptor. <br>提取原始文件描述符。<br>
    ///
    /// This method does **not** pass ownership of the raw file descriptor to the caller. <br>此方法不会将原始文件描述符的所有权传递给调用者。<br>
    /// The descriptor is only guaranteed to be valid while the original object has not yet been destroyed. <br>仅当原始对象尚未销毁时，才保证描述符是有效的。<br>
    ///
    fn as_raw_fd(&self) -> RawFd;
}

/// A trait to express the ability to construct an object from a raw file descriptor. <br>一个表示从原始文件描述符构造对象的能力的 trait。<br>
///
pub trait FromRawFd {
    /// Constructs a new instance of `Self` from the given raw file descriptor. <br>根据给定的原始文件描述符构造 `Self` 的新实例。<br>
    ///
    /// This function **consumes ownership** of the specified file descriptor. <br>这个函数会消耗指定文件描述符的所有权。<br>
    /// The returned object will take responsibility for closing it when the object goes out of scope. <br>当对象离开作用域时，返回的对象将负责关闭它。<br>
    ///
    /// This function is also unsafe as the primitives currently returned have the contract that they are the sole owner of the file descriptor they are wrapping. <br>此函数也是不安全的，因为当前返回的原语具有它们是包装的文件描述符的唯一所有者的约定。<br>
    ///
    /// Usage of this function could accidentally allow violating this contract which can cause memory unsafety in code that relies on it being true. <br>该函数的使用可能会意外地允许违反该契约，这可能会导致依赖该契约的代码的内存不安全。<br>
    ///
    ///
    ///
    ///
    unsafe fn from_raw_fd(fd: RawFd) -> Self;
}

/// A trait to express the ability to consume an object and acquire ownership of its raw file descriptor. <br>一个 trait，表示消费对象并获得其原始文件描述符所有权的能力。<br>
///
pub trait IntoRawFd {
    /// Consumes this object, returning the raw underlying file descriptor. <br>消费这个对象，返回原始的底层文件描述符。<br>
    ///
    /// This function **transfers ownership** of the underlying file descriptor to the caller. <br>该函数将底层文件描述符的所有权转移给调用者。<br>
    /// Callers are then the unique owners of the file descriptor and must close the descriptor once it's no longer needed. <br>这样，调用者便是文件描述符的唯一所有者，一旦不再需要描述符，就必须关闭该描述符。<br>
    ///
    fn into_raw_fd(self) -> RawFd;
}

#[stable(feature = "raw_fd_reflexive_traits", since = "1.48.0")]
impl AsRawFd for RawFd {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        *self
    }
}
#[stable(feature = "raw_fd_reflexive_traits", since = "1.48.0")]
impl IntoRawFd for RawFd {
    #[inline]
    fn into_raw_fd(self) -> RawFd {
        self
    }
}
#[stable(feature = "raw_fd_reflexive_traits", since = "1.48.0")]
impl FromRawFd for RawFd {
    #[inline]
    unsafe fn from_raw_fd(fd: RawFd) -> RawFd {
        fd
    }
}

macro_rules! impl_as_raw_fd {
    ($($t:ident)*) => {$(
        #[stable(feature = "rust1", since = "1.0.0")]
        impl AsRawFd for net::$t {
            #[inline]
            fn as_raw_fd(&self) -> RawFd {
                *self.as_inner().socket().as_inner()
            }
        }
    )*};
}
impl_as_raw_fd! { TcpStream TcpListener UdpSocket }

macro_rules! impl_from_raw_fd {
    ($($t:ident)*) => {$(
        #[stable(feature = "from_raw_os", since = "1.1.0")]
        impl FromRawFd for net::$t {
            #[inline]
            unsafe fn from_raw_fd(fd: RawFd) -> net::$t {
                let socket = sys::net::Socket::from_inner(fd);
                net::$t::from_inner(sys_common::net::$t::from_inner(socket))
            }
        }
    )*};
}
impl_from_raw_fd! { TcpStream TcpListener UdpSocket }

macro_rules! impl_into_raw_fd {
    ($($t:ident)*) => {$(
        #[stable(feature = "into_raw_os", since = "1.4.0")]
        impl IntoRawFd for net::$t {
            #[inline]
            fn into_raw_fd(self) -> RawFd {
                self.into_inner().into_socket().into_inner()
            }
        }
    )*};
}
impl_into_raw_fd! { TcpStream TcpListener UdpSocket }
