//! 特定于 SOLID 的通用 I/O 原语扩展

#![deny(unsafe_op_in_unsafe_fn)]
#![unstable(feature = "solid_ext", issue = "none")]

use crate::net;
use crate::sys;
use crate::sys_common::{self, AsInner, FromInner, IntoInner};

/// 原始文件描述符。
pub type RawFd = i32;

/// 从底层对象中提取原始 SOLID 套接字文件描述符的 trait。
///
pub trait AsRawFd {
    /// 提取原始文件描述符。
    ///
    /// 此方法不会将原始文件描述符的所有权传递给调用者。
    /// 仅当原始对象尚未销毁时，才保证描述符是有效的。
    ///
    fn as_raw_fd(&self) -> RawFd;
}

/// 一个表示从原始文件描述符构造对象的能力的 trait。
///
pub trait FromRawFd {
    /// 根据给定的原始文件描述符构造 `Self` 的新实例。
    ///
    /// 这个函数会消耗指定文件描述符的所有权。
    /// 当对象离开作用域时，返回的对象将负责关闭它。
    ///
    /// 此函数也是不安全的，因为当前返回的原语具有它们是包装的文件描述符的唯一所有者的约定。
    ///
    /// 该函数的使用可能会意外地允许违反该契约，这可能会导致依赖该契约的代码的内存不安全。
    ///
    ///
    ///
    ///
    unsafe fn from_raw_fd(fd: RawFd) -> Self;
}

/// 一个 trait，表示消费对象并获得其原始文件描述符所有权的能力。
///
pub trait IntoRawFd {
    /// 消费这个对象，返回原始的底层文件描述符。
    ///
    /// 该函数将底层文件描述符的所有权转移给调用者。
    /// 这样，调用者便是文件描述符的唯一所有者，一旦不再需要描述符，就必须关闭该描述符。
    ///
    fn into_raw_fd(self) -> RawFd;
}

#[stable(feature = "raw_fd_reflexive_traits", since = "1.48.0")]
impl AsRawFd for RawFd {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        *self
    }
}
#[stable(feature = "raw_fd_reflexive_traits", since = "1.48.0")]
impl IntoRawFd for RawFd {
    #[inline]
    fn into_raw_fd(self) -> RawFd {
        self
    }
}
#[stable(feature = "raw_fd_reflexive_traits", since = "1.48.0")]
impl FromRawFd for RawFd {
    #[inline]
    unsafe fn from_raw_fd(fd: RawFd) -> RawFd {
        fd
    }
}

macro_rules! impl_as_raw_fd {
    ($($t:ident)*) => {$(
        #[stable(feature = "rust1", since = "1.0.0")]
        impl AsRawFd for net::$t {
            #[inline]
            fn as_raw_fd(&self) -> RawFd {
                *self.as_inner().socket().as_inner()
            }
        }
    )*};
}
impl_as_raw_fd! { TcpStream TcpListener UdpSocket }

macro_rules! impl_from_raw_fd {
    ($($t:ident)*) => {$(
        #[stable(feature = "from_raw_os", since = "1.1.0")]
        impl FromRawFd for net::$t {
            #[inline]
            unsafe fn from_raw_fd(fd: RawFd) -> net::$t {
                let socket = sys::net::Socket::from_inner(fd);
                net::$t::from_inner(sys_common::net::$t::from_inner(socket))
            }
        }
    )*};
}
impl_from_raw_fd! { TcpStream TcpListener UdpSocket }

macro_rules! impl_into_raw_fd {
    ($($t:ident)*) => {$(
        #[stable(feature = "into_raw_os", since = "1.4.0")]
        impl IntoRawFd for net::$t {
            #[inline]
            fn into_raw_fd(self) -> RawFd {
                self.into_inner().into_socket().into_inner()
            }
        }
    )*};
}
impl_into_raw_fd! { TcpStream TcpListener UdpSocket }
