//! Unix-specific extensions to general I/O primitives. <br>特定于 Unix 的通用 I/O 原语扩展。<br>
//!
//! Just like raw pointers, raw file descriptors point to resources with dynamic lifetimes, and they can dangle if they outlive their resources or be forged if they're created from invalid values. <br>就像裸指针一样，原始文件描述符指向具有动态生命周期的资源，如果它们比它们的资源生命周期更长，它们可以悬垂，如果它们是由无效值创建的，则它们可以被伪造。<br>
//!
//! This module provides three types for representing file descriptors, with different ownership properties: raw, borrowed, and owned, which are analogous to types used for representing pointers: <br>该模块提供了三种用于表示文件描述符的类型，具有不同的所有权属性：原始、借用和拥有，类似于用于表示指针的类型：<br>
//!
//! | Type               | Analogous to |
//! | ------------------ | ------------ |
//! | [`RawFd`]          | `*const _`   |
//! | [`BorrowedFd<'a>`] | `&'a _`      |
//! | [`OwnedFd`]        | `Box<_>`     |
//!
//! Like raw pointers, `RawFd` values are primitive values. <br>和裸指针一样，`RawFd` 值是原始值。<br>
//! And in new code, they should be considered unsafe to do I/O on (analogous to dereferencing them). <br>并且在新代码中，他们应该被认为是不安全的 (类似于解引用他们)。<br> Rust did not always provide this guidance, so existing code in the Rust ecosystem often doesn't mark `RawFd` usage as unsafe. <br>Rust 并不总是提供此指导，因此 Rust 生态系统中的现有代码通常不会将 `RawFd` 的使用标记为不安全。<br>
//! Once the `io_safety` feature is stable, libraries will be encouraged to migrate, either by adding `unsafe` to APIs that dereference `RawFd` values, or by using to `BorrowedFd` or `OwnedFd` instead. <br>一旦 `io_safety` 特性稳定，将鼓励库迁移，通过将 `unsafe` 添加到解引用 `RawFd` 值的 API，或者使用 `BorrowedFd` 或 `OwnedFd` 代替。<br>
//!
//! Like references, `BorrowedFd` values are tied to a lifetime, to ensure that they don't outlive the resource they point to. <br>与引用一样，`BorrowedFd` 值与生命周期相关联，以确保它们的生命周期不会超过它们指向的资源。<br> These are safe to use. <br>这些可以安全使用。<br>
//! `BorrowedFd` values may be used in APIs which provide safe access to any system call except for: <br>`BorrowedFd` 值可用于提供对任何系统调用的安全访问的 API 中，除了：<br>
//!
//!  - `close`, because that would end the dynamic lifetime of the resource without ending the lifetime of the file descriptor. <br>`close`，因为这将结束资源的动态生命周期，而不结束文件描述符的生命周期。<br>
//!
//!  - `dup2`/`dup3`, in the second argument, because this argument is closed and assigned a new resource, which may break the assumptions other code using that file descriptor. <br>`dup2`/`dup3`，在第二个参数中，因为这个参数被关闭并分配了一个新的资源，这可能会破坏其他代码使用该文件描述符的假设。<br>
//!
//! `BorrowedFd` values may be used in APIs which provide safe access to `dup` system calls, so types implementing `AsFd` or `From<OwnedFd>` should not assume they always have exclusive access to the underlying file description. <br>`BorrowedFd` 值可用于提供对 `dup` 系统调用的安全访问的 API，因此实现 `AsFd` 或 `From<OwnedFd>` 的类型不应假定它们始终具有对底层文件描述的独占访问权。<br>
//!
//! `BorrowedFd` values may also be used with `mmap`, since `mmap` uses the provided file descriptor in a manner similar to `dup` and does not require the `BorrowedFd` passed to it to live for the lifetime of the resulting mapping. <br>`BorrowedFd` 值也可以与 `mmap` 一起使用，因为 `mmap` 以类似于 `dup` 的方式使用提供的文件描述符，并且不需要传递给它的 `BorrowedFd` 在生成的映射的生命周期中存在。<br>
//!
//! That said, `mmap` is unsafe for other reasons: it operates on raw pointers, and it can have undefined behavior if the underlying storage is mutated. <br>也就是说，`mmap` 是不安全的还有其他原因：它在裸体疗法上运行，如果底层存储发生可变，它可能会有未定义的行为。<br>
//! Mutations may come from other processes, or from the same process if the API provides `BorrowedFd` access, since as mentioned earlier, `BorrowedFd` values may be used in APIs which provide safe access to any system call. <br>可变的可能来自其他进程，或者来自同一进程，如果 API 提供 `BorrowedFd` 访问，因为如前所述，`BorrowedFd` 值可能用于提供对任何系统调用的安全访问的 API。<br>
//! Consequently, code using `mmap` and presenting a safe API must take full responsibility for ensuring that safe Rust code cannot evoke undefined behavior through it. <br>因此，使用 `mmap` 并提供安全 API 的代码必须全权负责确保安全的 Rust 代码不会通过它引起未定义的行为。<br>
//!
//! Like boxes, `OwnedFd` values conceptually own the resource they point to, and free (close) it when they are dropped. <br>与 box 一样，`OwnedFd` 在概念上拥有它们指向的资源，并在它们被丢弃时释放 X1X。<br>
//!
//! ## `/proc/self/mem` and similar OS features <br>`/proc/self/mem` 和类似的操作系统特性<br>
//!
//! Some platforms have special files, such as `/proc/self/mem`, which provide read and write access to the process's memory. <br>某些平台具有特殊文件，例如 `/proc/self/mem`，它们提供对进程内存的读写访问。<br>
//! Such reads and writes happen outside the control of the Rust compiler, so they do not uphold Rust's memory safety guarantees. <br>此类读取和写入发生在 Rust 编译器的控制之外，因此它们不支持 Rust 的内存安全保证。<br>
//!
//! This does not mean that all APIs that might allow `/proc/self/mem` to be opened and read from or written must be `unsafe`. <br>这并不意味着所有可能允许打开和读取或写入 `/proc/self/mem` 的 API 都必须是 `unsafe`。<br>
//! Rust's safety guarantees only cover what the program itself can do, and not what entities outside the program can do to it. <br>Rust 的安全保证只涵盖程序本身可以做的事情，而不包括程序外的实体可以对它做的事情。<br>
//! `/proc/self/mem` is considered to be such an external entity, along with debugging interfaces, and people with physical access to the hardware. <br>`/proc/self/mem` 被认为是这样一个外部实体，连同调试接口，以及对硬件有物理访问权限的人。<br>
//! This is true even in cases where the program is controlling the external entity. <br>即使在程序控制外部实体的情况下也是如此。<br>
//!
//! If you desire to comprehensively prevent programs from reaching out and causing external entities to reach back in and violate memory safety, it's necessary to use *sandboxing*, which is outside the scope of `std`. <br>如果您希望全面防止程序伸出并导致外部实体再次进入并违反内存安全，则有必要使用 *sandboxing*，它不在 `std` 的作用域范围内。<br>
//!
//! [`BorrowedFd<'a>`]: crate::os::unix::io::BorrowedFd
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

mod fd;
mod raw;

#[stable(feature = "io_safety", since = "1.63.0")]
pub use fd::*;
#[stable(feature = "rust1", since = "1.0.0")]
pub use raw::*;
