//! Unix-specific extensions to primitives in the [`std::process`] module. <br>特定于 Unix 的 [`std::process`] 模块中的原语扩展。<br>
//!
//! [`std::process`]: crate::process

#![stable(feature = "rust1", since = "1.0.0")]

use crate::ffi::OsStr;
use crate::io;
use crate::os::unix::io::{AsFd, AsRawFd, BorrowedFd, FromRawFd, IntoRawFd, OwnedFd, RawFd};
use crate::process;
use crate::sealed::Sealed;
use crate::sys;
use crate::sys_common::{AsInner, AsInnerMut, FromInner, IntoInner};

#[cfg(not(any(target_os = "vxworks", target_os = "espidf", target_os = "horizon")))]
type UserId = u32;
#[cfg(not(any(target_os = "vxworks", target_os = "espidf", target_os = "horizon")))]
type GroupId = u32;

#[cfg(any(target_os = "vxworks", target_os = "espidf", target_os = "horizon"))]
type UserId = u16;
#[cfg(any(target_os = "vxworks", target_os = "espidf", target_os = "horizon"))]
type GroupId = u16;

/// Unix-specific extensions to the [`process::Command`] builder. <br>特定于 Unix 的 [`process::Command`] 构建器扩展。<br>
///
/// This trait is sealed: it cannot be implemented outside the standard library. <br>这个 trait 是封闭的：它不能在标准库之外实现。<br>
/// This is so that future additional methods are not breaking changes. <br>这是为了将来的附加方法不会破坏更改。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub trait CommandExt: Sealed {
    /// Sets the child process's user ID. <br>设置子进程的用户 ID。<br>
    /// This translates to a `setuid` call in the child process. <br>这将转换为子进程中的 `setuid` 调用。<br>
    /// Failure in the `setuid` call will cause the spawn to fail. <br>`setuid` 调用失败将导致 spawn 失败。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    fn uid(&mut self, id: UserId) -> &mut process::Command;

    /// Similar to `uid`, but sets the group ID of the child process. <br>与 `uid` 相似，但是设置子进程的组 ID。<br>
    /// This has the same semantics as the `uid` field. <br>这具有与 `uid` 字段相同的语义。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    fn gid(&mut self, id: GroupId) -> &mut process::Command;

    /// Sets the supplementary group IDs for the calling process. <br>设置调用进程的补充组 ID。<br>
    /// Translates to a `setgroups` call in the child process. <br>在子进程中转换为 `setgroups` 调用。<br>
    #[unstable(feature = "setgroups", issue = "90747")]
    fn groups(&mut self, groups: &[GroupId]) -> &mut process::Command;

    /// Schedules a closure to be run just before the `exec` function is invoked. <br>计划在 `exec` 函数被调用之前运行一个闭包。<br>
    ///
    /// The closure is allowed to return an I/O error whose OS error code will be communicated back to the parent and returned as an error from when the spawn was requested. <br>允许闭包返回 I/O 错误，该错误的 OS 错误代码将被传达回父级，并从请求 spawn 开始作为错误返回。<br>
    ///
    /// Multiple closures can be registered and they will be called in order of their registration. <br>可以注册多个闭包，并且将按照注册顺序对其进行调用。<br> If a closure returns `Err` then no further closures will be called and the spawn operation will immediately return with a failure. <br>如果闭包返回 `Err`，则不会再调用任何闭包，并且 spawn 操作将立即失败返回。<br>
    ///
    /// # Notes and Safety <br>注意和安全<br>
    ///
    /// This closure will be run in the context of the child process after a `fork`. <br>`fork` 之后，此闭包将在子进程的上下文中运行。<br> This primarily means that any modifications made to memory on behalf of this closure will **not** be visible to the parent process. <br>这主要意味着，代表此闭包对内存所做的任何修改对于父进程都是不可见的。<br>
    /// This is often a very constrained environment where normal operations like `malloc`, accessing environment variables through [`std::env`] or acquiring a mutex are not guaranteed to work (due to other threads perhaps still running when the `fork` was run). <br>这通常是一个非常受限制的环境，无法保证正常操作 (如 `malloc`，通过 [`std::env`] 访问环境变量或获取互斥锁) (由于运行 `fork` 时可能仍在运行其他线程)。<br>
    ///
    /// For further details refer to the [POSIX fork() specification] and the equivalent documentation for any targeted platform, especially the requirements around *async-signal-safety*. <br>有关更多详细信息，请参见 [POSIX fork() 规范][POSIX fork() specification] 和任何目标平台的等效文档，尤其是有关 *async-signal-safety* 的要求。<br>
    ///
    /// This also means that all resources such as file descriptors and memory-mapped regions got duplicated. <br>这也意味着所有资源 (例如文件描述符和内存映射的区域) 都被复制了。<br> It is your responsibility to make sure that the closure does not violate library invariants by making invalid use of these duplicates. <br>您有责任确保闭包不会因为无效使用这些副本而违反库不变量。<br>
    ///
    /// Panicking in the closure is safe only if all the format arguments for the panic message can be safely formatted; <br>只有当 panic 消息的所有格式参数都可以安全格式化时，闭包中的 panic 才是安全的；<br> this is because although `Command` calls [`std::panic::always_abort`](crate::panic::always_abort) before calling the pre_exec hook, panic will still try to format the panic message. <br>这是因为尽管 `Command` 在调用 pre_exec 钩子之前调用了 [`std::panic::always_abort`](crate::panic::always_abort)，但 panic 仍会尝试格式化 panic 消息。<br>
    ///
    ///
    /// When this closure is run, aspects such as the stdio file descriptors and working directory have successfully been changed, so output to these locations might not appear where intended. <br>运行此闭包时，标准输入输出文件描述符和工作目录等方面已成功更改，因此这些位置的输出可能不会出现在预期的位置。<br>
    ///
    /// [POSIX fork() specification]:
    ///     https://pubs.opengroup.org/onlinepubs/9699919799/functions/fork.html
    /// [`std::env`]: mod@crate::env
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "process_pre_exec", since = "1.34.0")]
    unsafe fn pre_exec<F>(&mut self, f: F) -> &mut process::Command
    where
        F: FnMut() -> io::Result<()> + Send + Sync + 'static;

    /// Schedules a closure to be run just before the `exec` function is invoked. <br>计划在 `exec` 函数被调用之前运行一个闭包。<br>
    ///
    ///
    /// This method is stable and usable, but it should be unsafe. <br>此方法稳定且可用，但不安全。<br>
    /// To fix that, it got deprecated in favor of the unsafe [`pre_exec`]. <br>为了解决这个问题，它不赞成使用不安全的 [`pre_exec`]。<br>
    ///
    /// [`pre_exec`]: CommandExt::pre_exec
    #[stable(feature = "process_exec", since = "1.15.0")]
    #[deprecated(since = "1.37.0", note = "should be unsafe, use `pre_exec` instead")]
    fn before_exec<F>(&mut self, f: F) -> &mut process::Command
    where
        F: FnMut() -> io::Result<()> + Send + Sync + 'static,
    {
        unsafe { self.pre_exec(f) }
    }

    /// Performs all the required setup by this `Command`, followed by calling the `execvp` syscall. <br>通过此 `Command` 执行所有必需的设置，然后调用 `execvp` syscall。<br>
    ///
    /// On success this function will not return, and otherwise it will return an error indicating why the exec (or another part of the setup of the `Command`) failed. <br>成功后，该函数将不会返回，否则它将返回错误，指示 exec (或 `Command` 的另一部分安装) 失败的原因。<br>
    ///
    /// `exec` not returning has the same implications as calling [`process::exit`] – no destructors on the current stack or any other thread’s stack will be run. <br>`exec` 不返回与调用 [`process::exit`] 具有相同的含义 -- 当前栈或任何其他线程栈上的析构函数都不会运行。<br> Therefore, it is recommended to only call `exec` at a point where it is fine to not run any destructors. <br>因此，建议仅在不运行任何析构函数的时候调用 `exec`。<br>
    ///
    /// Note, that the `execvp` syscall independently guarantees that all memory is freed and all file descriptors with the `CLOEXEC` option (set by default on all file descriptors opened by the standard library) are closed. <br>请注意，`execvp` 系统调用独立保证释放所有内存，并关闭所有带有 `CLOEXEC` 选项的文件描述符 (默认情况下在标准库打开的所有文件描述符上设置)。<br>
    ///
    /// This function, unlike `spawn`, will **not** `fork` the process to create a new child. <br>与 `spawn` 不同，此函数不会使用 `fork` 来创建一个新的子进程。<br> Like spawn, however, the default behavior for the stdio descriptors will be to inherited from the current process. <br>但是，与 spawn 一样，标准输入输出描述符的默认行为将继承自当前进程。<br>
    ///
    /// # Notes
    ///
    /// The process may be in a "broken state" if this function returns in error. <br>如果此函数返回错误，则该进程可能在 "broken state" 中。<br> For example the working directory, environment variables, signal handling settings, various user/group information, or aspects of stdio file descriptors may have changed. <br>例如，工作目录，环境变量，信号处理设置，各种 user/group 信息或标准输入输出文件描述符的各个方面可能已更改。<br>
    /// If a "transactional spawn" is required to gracefully handle errors it is recommended to use the cross-platform `spawn` instead. <br>如果需要 "事务性生成" 来妥善处理错误，建议改用跨平台 `spawn`。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "process_exec2", since = "1.9.0")]
    fn exec(&mut self) -> io::Error;

    /// Set executable argument <br>设置可执行参数<br>
    ///
    /// Set the first process argument, `argv[0]`, to something other than the default executable path. <br>将第一个进程参数 `argv[0]` 设置为默认可执行路径以外的其他值。<br>
    ///
    #[stable(feature = "process_set_argv0", since = "1.45.0")]
    fn arg0<S>(&mut self, arg: S) -> &mut process::Command
    where
        S: AsRef<OsStr>;

    /// Sets the process group ID of the child process. <br>设置子进程的进程组 ID。<br>
    /// Translates to a `setpgid` call in the child process. <br>转换为子进程中的 `setpgid` 调用。<br>
    #[unstable(feature = "process_set_process_group", issue = "93857")]
    fn process_group(&mut self, pgroup: i32) -> &mut process::Command;
}

#[stable(feature = "rust1", since = "1.0.0")]
impl CommandExt for process::Command {
    fn uid(&mut self, id: UserId) -> &mut process::Command {
        self.as_inner_mut().uid(id);
        self
    }

    fn gid(&mut self, id: GroupId) -> &mut process::Command {
        self.as_inner_mut().gid(id);
        self
    }

    fn groups(&mut self, groups: &[GroupId]) -> &mut process::Command {
        self.as_inner_mut().groups(groups);
        self
    }

    unsafe fn pre_exec<F>(&mut self, f: F) -> &mut process::Command
    where
        F: FnMut() -> io::Result<()> + Send + Sync + 'static,
    {
        self.as_inner_mut().pre_exec(Box::new(f));
        self
    }

    fn exec(&mut self) -> io::Error {
        // NOTE: This may *not* be safe to call after `libc::fork`, because it may allocate. <br>在 `libc::fork` 之后调用此方法可能不是 * 安全的，因为它可能会分配。<br>
        // That may be worth fixing at some point in the future. <br>在 future 中的某个位置可能值得解决。<br>
        self.as_inner_mut().exec(sys::process::Stdio::Inherit)
    }

    fn arg0<S>(&mut self, arg: S) -> &mut process::Command
    where
        S: AsRef<OsStr>,
    {
        self.as_inner_mut().set_arg_0(arg.as_ref());
        self
    }

    fn process_group(&mut self, pgroup: i32) -> &mut process::Command {
        self.as_inner_mut().pgroup(pgroup);
        self
    }
}

/// Unix-specific extensions to [`process::ExitStatus`] and [`ExitStatusError`](process::ExitStatusError). <br>特定于 Unix 的 [`process::ExitStatus`] 和 [`ExitStatusError`](process::ExitStatusError) 的扩展。<br>
///
/// On Unix, `ExitStatus` **does not necessarily represent an exit status**, as passed to the `_exit` system call or returned by [`ExitStatus::code()`](crate::process::ExitStatus::code). <br>在 Unix 上，`ExitStatus`**不一定代表退出状态**，因为传递给 `_exit` 系统调用或由 [`ExitStatus::code()`](crate::process::ExitStatus::code) 返回。<br>
/// It represents **any wait status** as returned by one of the `wait` family of system calls. <br>它表示由 `wait` 系列系统调用之一返回的**任何等待状态**。<br>
///
/// A Unix wait status (a Rust `ExitStatus`) can represent a Unix exit status, but can also represent other kinds of process event. <br>Unix 等待状态 (Rust `ExitStatus`) 可以代表 Unix 退出状态，但也可以代表其他类型的进程事件。<br>
///
/// This trait is sealed: it cannot be implemented outside the standard library. <br>这个 trait 是封闭的：它不能在标准库之外实现。<br>
/// This is so that future additional methods are not breaking changes. <br>这是为了将来的附加方法不会破坏更改。<br>
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub trait ExitStatusExt: Sealed {
    /// Creates a new `ExitStatus` or `ExitStatusError` from the raw underlying integer status value from `wait` <br>从 `wait` 的原始底层整数状态值创建一个新的 `ExitStatus` 或 `ExitStatusError`<br>
    ///
    ///
    /// The value should be a **wait status, not an exit status**. <br>该值应该是等待状态，而不是退出状态。<br>
    ///
    /// # Panics
    ///
    /// Panics on an attempt to make an `ExitStatusError` from a wait status of `0`. <br>尝试从 `0` 的等待状态生成 `ExitStatusError` 时出现 panic。<br>
    ///
    /// Making an `ExitStatus` always succeeds and never panics. <br>创建 `ExitStatus` 总是会成功的，并且从不 panics。<br>
    #[stable(feature = "exit_status_from", since = "1.12.0")]
    fn from_raw(raw: i32) -> Self;

    /// If the process was terminated by a signal, returns that signal. <br>如果进程被一个信号终止，则返回该信号。<br>
    ///
    /// In other words, if `WIFSIGNALED`, this returns `WTERMSIG`. <br>换句话说，如果为 `WIFSIGNALED`，则返回 `WTERMSIG`。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    fn signal(&self) -> Option<i32>;

    /// If the process was terminated by a signal, says whether it dumped core. <br>如果进程被一个信号终止，说明它是否丢弃了核心。<br>
    #[stable(feature = "unix_process_wait_more", since = "1.58.0")]
    fn core_dumped(&self) -> bool;

    /// If the process was stopped by a signal, returns that signal. <br>如果该进程被信号停止，则返回该信号。<br>
    ///
    /// In other words, if `WIFSTOPPED`, this returns `WSTOPSIG`. <br>换句话说，如果为 `WIFSTOPPED`，则返回 `WSTOPSIG`。<br>
    /// This is only possible if the status came from a `wait` system call which was passed `WUNTRACED`, and was then converted into an `ExitStatus`. <br>这仅在状态来自 `wait` 系统调用时才可能，该系统调用通过 `WUNTRACED`，然后转换为 `ExitStatus`。<br>
    #[stable(feature = "unix_process_wait_more", since = "1.58.0")]
    fn stopped_signal(&self) -> Option<i32>;

    /// Whether the process was continued from a stopped status. <br>进程是否从停止状态继续。<br>
    ///
    /// Ie, `WIFCONTINUED`. <br>即，`WIFCONTINUED`。<br>
    /// This is only possible if the status came from a `wait` system call which was passed `WCONTINUED`, and was then converted into an `ExitStatus`. <br>这只有在状态来自 `wait` 系统调用时才有可能，该系统调用通过 `WCONTINUED`，然后转换为 `ExitStatus`。<br>
    #[stable(feature = "unix_process_wait_more", since = "1.58.0")]
    fn continued(&self) -> bool;

    /// Returns the underlying raw `wait` status. <br>返回底层的原始 `wait` 状态。<br>
    ///
    /// The returned integer is a **wait status, not an exit status**. <br>返回的整数是等待状态，而不是退出状态。<br>
    #[stable(feature = "unix_process_wait_more", since = "1.58.0")]
    fn into_raw(self) -> i32;
}

#[stable(feature = "rust1", since = "1.0.0")]
impl ExitStatusExt for process::ExitStatus {
    fn from_raw(raw: i32) -> Self {
        process::ExitStatus::from_inner(From::from(raw))
    }

    fn signal(&self) -> Option<i32> {
        self.as_inner().signal()
    }

    fn core_dumped(&self) -> bool {
        self.as_inner().core_dumped()
    }

    fn stopped_signal(&self) -> Option<i32> {
        self.as_inner().stopped_signal()
    }

    fn continued(&self) -> bool {
        self.as_inner().continued()
    }

    fn into_raw(self) -> i32 {
        self.as_inner().into_raw().into()
    }
}

#[unstable(feature = "exit_status_error", issue = "84908")]
impl ExitStatusExt for process::ExitStatusError {
    fn from_raw(raw: i32) -> Self {
        process::ExitStatus::from_raw(raw)
            .exit_ok()
            .expect_err("<ExitStatusError as ExitStatusExt>::from_raw(0) but zero is not an error")
    }

    fn signal(&self) -> Option<i32> {
        self.into_status().signal()
    }

    fn core_dumped(&self) -> bool {
        self.into_status().core_dumped()
    }

    fn stopped_signal(&self) -> Option<i32> {
        self.into_status().stopped_signal()
    }

    fn continued(&self) -> bool {
        self.into_status().continued()
    }

    fn into_raw(self) -> i32 {
        self.into_status().into_raw()
    }
}

#[stable(feature = "process_extensions", since = "1.2.0")]
impl FromRawFd for process::Stdio {
    #[inline]
    unsafe fn from_raw_fd(fd: RawFd) -> process::Stdio {
        let fd = sys::fd::FileDesc::from_raw_fd(fd);
        let io = sys::process::Stdio::Fd(fd);
        process::Stdio::from_inner(io)
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<OwnedFd> for process::Stdio {
    #[inline]
    fn from(fd: OwnedFd) -> process::Stdio {
        let fd = sys::fd::FileDesc::from_inner(fd);
        let io = sys::process::Stdio::Fd(fd);
        process::Stdio::from_inner(io)
    }
}

#[stable(feature = "process_extensions", since = "1.2.0")]
impl AsRawFd for process::ChildStdin {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        self.as_inner().as_raw_fd()
    }
}

#[stable(feature = "process_extensions", since = "1.2.0")]
impl AsRawFd for process::ChildStdout {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        self.as_inner().as_raw_fd()
    }
}

#[stable(feature = "process_extensions", since = "1.2.0")]
impl AsRawFd for process::ChildStderr {
    #[inline]
    fn as_raw_fd(&self) -> RawFd {
        self.as_inner().as_raw_fd()
    }
}

#[stable(feature = "into_raw_os", since = "1.4.0")]
impl IntoRawFd for process::ChildStdin {
    #[inline]
    fn into_raw_fd(self) -> RawFd {
        self.into_inner().into_inner().into_raw_fd()
    }
}

#[stable(feature = "into_raw_os", since = "1.4.0")]
impl IntoRawFd for process::ChildStdout {
    #[inline]
    fn into_raw_fd(self) -> RawFd {
        self.into_inner().into_inner().into_raw_fd()
    }
}

#[stable(feature = "into_raw_os", since = "1.4.0")]
impl IntoRawFd for process::ChildStderr {
    #[inline]
    fn into_raw_fd(self) -> RawFd {
        self.into_inner().into_inner().into_raw_fd()
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsFd for crate::process::ChildStdin {
    #[inline]
    fn as_fd(&self) -> BorrowedFd<'_> {
        self.as_inner().as_fd()
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<crate::process::ChildStdin> for OwnedFd {
    #[inline]
    fn from(child_stdin: crate::process::ChildStdin) -> OwnedFd {
        child_stdin.into_inner().into_inner().into_inner()
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsFd for crate::process::ChildStdout {
    #[inline]
    fn as_fd(&self) -> BorrowedFd<'_> {
        self.as_inner().as_fd()
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<crate::process::ChildStdout> for OwnedFd {
    #[inline]
    fn from(child_stdout: crate::process::ChildStdout) -> OwnedFd {
        child_stdout.into_inner().into_inner().into_inner()
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsFd for crate::process::ChildStderr {
    #[inline]
    fn as_fd(&self) -> BorrowedFd<'_> {
        self.as_inner().as_fd()
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<crate::process::ChildStderr> for OwnedFd {
    #[inline]
    fn from(child_stderr: crate::process::ChildStderr) -> OwnedFd {
        child_stderr.into_inner().into_inner().into_inner()
    }
}

/// Returns the OS-assigned process identifier associated with this process's parent. <br>返回与此进程的父级关联的操作系统分配的进程标识符。<br>
#[must_use]
#[stable(feature = "unix_ppid", since = "1.27.0")]
pub fn parent_id() -> u32 {
    crate::sys::os::getppid()
}
