//! Unix peer credentials. <br>Unix 对等凭据。<br>

// NOTE: Code in this file is heavily based on work done in PR 13 from the tokio-uds repository on GitHub. <br>该文件中的代码很大程度上基于 GitHub 上 tokio-uds 仓库在 PR 13 中所做的工作。<br>
//
//
//       For reference, the link is here: https://github.com/tokio-rs/tokio-uds/pull/13 <br>以下链接供参考: https://github.com/tokio-rs/tokio-uds/pull/13<br>
//       Credit to Martin Habovštiak (GitHub username Kixunil) and contributors for this work. <br>感谢 MartinHabovštiak (GitHub 用户名 Kixunil) 和这项工作的贡献者。<br>

use libc::{gid_t, pid_t, uid_t};

/// Credentials for a UNIX process for credentials passing. <br>UNIX 进程的凭据以用于凭据传递。<br>
#[unstable(feature = "peer_credentials_unix_socket", issue = "42839", reason = "unstable")]
#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
pub struct UCred {
    /// The UID part of the peer credential. <br>对等凭证的 UID 部分。<br>
    /// This is the effective UID of the process at the domain socket's endpoint. <br>这是域套接字端点处进程的有效 UID。<br>
    pub uid: uid_t,
    /// The GID part of the peer credential. <br>对等凭证的 GID 部分。<br>
    /// This is the effective GID of the process at the domain socket's endpoint. <br>这是域套接字端点处进程的有效 GID。<br>
    pub gid: gid_t,
    /// The PID part of the peer credential. <br>对等凭证的 PID 部分。<br> This field is optional because the PID part of the peer credentials is not supported on every platform. <br>该字段是可选的，因为并非每个平台都支持对等凭据的 PID 部分。<br>
    /// On platforms where the mechanism to discover the PID exists, this field will be populated to the PID of the process at the domain socket's endpoint. <br>在存在发现 PID 的机制的平台上，此字段将填充到域套接字端点处的进程的 PID。<br>
    ///
    /// Otherwise, it will be set to None. <br>否则，它将设置为 `None`。<br>
    pub pid: Option<pid_t>,
}

#[cfg(any(target_os = "android", target_os = "linux"))]
pub use self::impl_linux::peer_cred;

#[cfg(any(
    target_os = "dragonfly",
    target_os = "freebsd",
    target_os = "openbsd",
    target_os = "netbsd"
))]
pub use self::impl_bsd::peer_cred;

#[cfg(any(target_os = "macos", target_os = "ios",))]
pub use self::impl_mac::peer_cred;

#[cfg(any(target_os = "linux", target_os = "android"))]
pub mod impl_linux {
    use super::UCred;
    use crate::os::unix::io::AsRawFd;
    use crate::os::unix::net::UnixStream;
    use crate::{io, mem};
    use libc::{c_void, getsockopt, socklen_t, ucred, SOL_SOCKET, SO_PEERCRED};

    pub fn peer_cred(socket: &UnixStream) -> io::Result<UCred> {
        let ucred_size = mem::size_of::<ucred>();

        // Trivial sanity checks. <br>微不足道的检查。<br>
        assert!(mem::size_of::<u32>() <= mem::size_of::<usize>());
        assert!(ucred_size <= u32::MAX as usize);

        let mut ucred_size = ucred_size as socklen_t;
        let mut ucred: ucred = ucred { pid: 1, uid: 1, gid: 1 };

        unsafe {
            let ret = getsockopt(
                socket.as_raw_fd(),
                SOL_SOCKET,
                SO_PEERCRED,
                &mut ucred as *mut ucred as *mut c_void,
                &mut ucred_size,
            );

            if ret == 0 && ucred_size as usize == mem::size_of::<ucred>() {
                Ok(UCred { uid: ucred.uid, gid: ucred.gid, pid: Some(ucred.pid) })
            } else {
                Err(io::Error::last_os_error())
            }
        }
    }
}

#[cfg(any(
    target_os = "dragonfly",
    target_os = "freebsd",
    target_os = "openbsd",
    target_os = "netbsd"
))]
pub mod impl_bsd {
    use super::UCred;
    use crate::io;
    use crate::os::unix::io::AsRawFd;
    use crate::os::unix::net::UnixStream;

    pub fn peer_cred(socket: &UnixStream) -> io::Result<UCred> {
        let mut cred = UCred { uid: 1, gid: 1, pid: None };
        unsafe {
            let ret = libc::getpeereid(socket.as_raw_fd(), &mut cred.uid, &mut cred.gid);

            if ret == 0 { Ok(cred) } else { Err(io::Error::last_os_error()) }
        }
    }
}

#[cfg(any(target_os = "macos", target_os = "ios",))]
pub mod impl_mac {
    use super::UCred;
    use crate::os::unix::io::AsRawFd;
    use crate::os::unix::net::UnixStream;
    use crate::{io, mem};
    use libc::{c_void, getpeereid, getsockopt, pid_t, socklen_t, LOCAL_PEERPID, SOL_LOCAL};

    pub fn peer_cred(socket: &UnixStream) -> io::Result<UCred> {
        let mut cred = UCred { uid: 1, gid: 1, pid: None };
        unsafe {
            let ret = getpeereid(socket.as_raw_fd(), &mut cred.uid, &mut cred.gid);

            if ret != 0 {
                return Err(io::Error::last_os_error());
            }

            let mut pid: pid_t = 1;
            let mut pid_size = mem::size_of::<pid_t>() as socklen_t;

            let ret = getsockopt(
                socket.as_raw_fd(),
                SOL_LOCAL,
                LOCAL_PEERPID,
                &mut pid as *mut pid_t as *mut c_void,
                &mut pid_size,
            );

            if ret == 0 && pid_size as usize == mem::size_of::<pid_t>() {
                cred.pid = Some(pid);
                Ok(cred)
            } else {
                Err(io::Error::last_os_error())
            }
        }
    }
}
