//! WASI-specific extensions to primitives in the [`std::fs`] module. <br>特定于 WASI 的 [`std::fs`] 模块中原语的扩展。<br>
//!
//! [`std::fs`]: crate::fs

#![deny(unsafe_op_in_unsafe_fn)]
#![unstable(feature = "wasi_ext", issue = "71213")]

use crate::ffi::OsStr;
use crate::fs::{self, File, Metadata, OpenOptions};
use crate::io::{self, IoSlice, IoSliceMut};
use crate::path::{Path, PathBuf};
use crate::sys_common::{AsInner, AsInnerMut, FromInner};
// Used for `File::read` on intra-doc links <br>用于文档内链接上的 `File::read`<br>
#[allow(unused_imports)]
use io::{Read, Write};

/// WASI-specific extensions to [`File`]. <br>特定于 WASI 的 [`File`] 的扩展。<br>
pub trait FileExt {
    /// Reads a number of bytes starting from a given offset. <br>从给定的偏移量开始读取多个字节。<br>
    ///
    /// Returns the number of bytes read. <br>返回读取的字节数。<br>
    ///
    /// The offset is relative to the start of the file and thus independent from the current cursor. <br>偏移量相对于文件的开始，因此独立于当前游标。<br>
    ///
    ///
    /// The current file cursor is not affected by this function. <br>当前文件游标不受此函数影响。<br>
    ///
    /// Note that similar to [`File::read`], it is not an error to return with a short read. <br>请注意，类似于 [`File::read`]，短读返回不会出错。<br>
    ///
    fn read_at(&self, buf: &mut [u8], offset: u64) -> io::Result<usize> {
        let bufs = &mut [IoSliceMut::new(buf)];
        self.read_vectored_at(bufs, offset)
    }

    /// Reads a number of bytes starting from a given offset. <br>从给定的偏移量开始读取多个字节。<br>
    ///
    /// Returns the number of bytes read. <br>返回读取的字节数。<br>
    ///
    /// The offset is relative to the start of the file and thus independent from the current cursor. <br>偏移量相对于文件的开始，因此独立于当前游标。<br>
    ///
    ///
    /// The current file cursor is not affected by this function. <br>当前文件游标不受此函数影响。<br>
    ///
    /// Note that similar to [`File::read_vectored`], it is not an error to return with a short read. <br>请注意，类似于 [`File::read_vectored`]，短读返回不会出错。<br>
    ///
    fn read_vectored_at(&self, bufs: &mut [IoSliceMut<'_>], offset: u64) -> io::Result<usize>;

    /// Reads the exact number of byte required to fill `buf` from the given offset. <br>从给定的偏移量读取填充 `buf` 所需的确切字节数。<br>
    ///
    /// The offset is relative to the start of the file and thus independent from the current cursor. <br>偏移量相对于文件的开始，因此独立于当前游标。<br>
    ///
    /// The current file cursor is not affected by this function. <br>当前文件游标不受此函数影响。<br>
    ///
    /// Similar to [`Read::read_exact`] but uses [`read_at`] instead of `read`. <br>与 [`Read::read_exact`] 相似，但使用 [`read_at`] 代替 `read`。<br>
    ///
    /// [`read_at`]: FileExt::read_at
    ///
    /// # Errors
    ///
    /// If this function encounters an error of the kind [`io::ErrorKind::Interrupted`] then the error is ignored and the operation will continue. <br>如果此函数遇到 [`io::ErrorKind::Interrupted`] 类型的错误，则该错误将被忽略，并且操作将继续。<br>
    ///
    /// If this function encounters an "end of file" before completely filling the buffer, it returns an error of the kind [`io::ErrorKind::UnexpectedEof`]. <br>如果此函数在完全填充缓冲区之前遇到 "end of file"，它将返回 [`io::ErrorKind::UnexpectedEof`] 类型的错误。<br>
    /// The contents of `buf` are unspecified in this case. <br>在这种情况下，`buf` 的内容未指定。<br>
    ///
    /// If any other read error is encountered then this function immediately returns. <br>如果遇到任何其他读取错误，则此函数立即返回。<br> The contents of `buf` are unspecified in this case. <br>在这种情况下，`buf` 的内容未指定。<br>
    ///
    /// If this function returns an error, it is unspecified how many bytes it has read, but it will never read more than would be necessary to completely fill the buffer. <br>如果此函数返回错误，则无法确定已读取多少字节，但读取的字节数永远不会超过完全填充缓冲区所需的字节数。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rw_exact_all_at", since = "1.33.0")]
    fn read_exact_at(&self, mut buf: &mut [u8], mut offset: u64) -> io::Result<()> {
        while !buf.is_empty() {
            match self.read_at(buf, offset) {
                Ok(0) => break,
                Ok(n) => {
                    let tmp = buf;
                    buf = &mut tmp[n..];
                    offset += n as u64;
                }
                Err(ref e) if e.kind() == io::ErrorKind::Interrupted => {}
                Err(e) => return Err(e),
            }
        }
        if !buf.is_empty() {
            Err(io::const_io_error!(io::ErrorKind::UnexpectedEof, "failed to fill whole buffer"))
        } else {
            Ok(())
        }
    }

    /// Writes a number of bytes starting from a given offset. <br>从给定的偏移量开始写入多个字节。<br>
    ///
    /// Returns the number of bytes written. <br>返回写入的字节数。<br>
    ///
    /// The offset is relative to the start of the file and thus independent from the current cursor. <br>偏移量相对于文件的开始，因此独立于当前游标。<br>
    ///
    /// The current file cursor is not affected by this function. <br>当前文件游标不受此函数影响。<br>
    ///
    /// When writing beyond the end of the file, the file is appropriately extended and the intermediate bytes are initialized with the value 0. <br>当写入超出文件末尾时，将适当扩展文件，并将中间字节初始化为值 0。<br>
    ///
    ///
    /// Note that similar to [`File::write`], it is not an error to return a short write. <br>请注意，与 [`File::write`] 相似，返回短写也不是错误。<br>
    ///
    ///
    fn write_at(&self, buf: &[u8], offset: u64) -> io::Result<usize> {
        let bufs = &[IoSlice::new(buf)];
        self.write_vectored_at(bufs, offset)
    }

    /// Writes a number of bytes starting from a given offset. <br>从给定的偏移量开始写入多个字节。<br>
    ///
    /// Returns the number of bytes written. <br>返回写入的字节数。<br>
    ///
    /// The offset is relative to the start of the file and thus independent from the current cursor. <br>偏移量相对于文件的开始，因此独立于当前游标。<br>
    ///
    /// The current file cursor is not affected by this function. <br>当前文件游标不受此函数影响。<br>
    ///
    /// When writing beyond the end of the file, the file is appropriately extended and the intermediate bytes are initialized with the value 0. <br>当写入超出文件末尾时，将适当扩展文件，并将中间字节初始化为值 0。<br>
    ///
    ///
    /// Note that similar to [`File::write_vectored`], it is not an error to return a short write. <br>请注意，与 [`File::write_vectored`] 相似，返回短写也不是错误。<br>
    ///
    ///
    fn write_vectored_at(&self, bufs: &[IoSlice<'_>], offset: u64) -> io::Result<usize>;

    /// Attempts to write an entire buffer starting from a given offset. <br>尝试从给定的偏移量开始写入整个缓冲区。<br>
    ///
    /// The offset is relative to the start of the file and thus independent from the current cursor. <br>偏移量相对于文件的开始，因此独立于当前游标。<br>
    ///
    /// The current file cursor is not affected by this function. <br>当前文件游标不受此函数影响。<br>
    ///
    /// This method will continuously call [`write_at`] until there is no more data to be written or an error of non-[`io::ErrorKind::Interrupted`] kind is returned. <br>此方法将不断地调用 [`write_at`]，直到没有更多数据要写入或返回非 [`io::ErrorKind::Interrupted`] 类型的错误为止。<br>
    /// This method will not return until the entire buffer has been successfully written or such an error occurs. <br>在成功写入整个缓冲区或发生此类错误之前，此方法将不会返回。<br>
    /// The first error that is not of [`io::ErrorKind::Interrupted`] kind generated from this method will be returned. <br>将返回第一个不是由该方法生成的 [`io::ErrorKind::Interrupted`] 类型的错误。<br>
    ///
    /// # Errors
    ///
    /// This function will return the first error of non-[`io::ErrorKind::Interrupted`] kind that [`write_at`] returns. <br>该函数将返回 [`write_at`] 返回的第一个非 [`io::ErrorKind::Interrupted`] 类型的错误。<br>
    ///
    /// [`write_at`]: FileExt::write_at
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rw_exact_all_at", since = "1.33.0")]
    fn write_all_at(&self, mut buf: &[u8], mut offset: u64) -> io::Result<()> {
        while !buf.is_empty() {
            match self.write_at(buf, offset) {
                Ok(0) => {
                    return Err(io::const_io_error!(
                        io::ErrorKind::WriteZero,
                        "failed to write whole buffer",
                    ));
                }
                Ok(n) => {
                    buf = &buf[n..];
                    offset += n as u64
                }
                Err(ref e) if e.kind() == io::ErrorKind::Interrupted => {}
                Err(e) => return Err(e),
            }
        }
        Ok(())
    }

    /// Returns the current position within the file. <br>返回文件中的当前位置。<br>
    ///
    /// This corresponds to the `fd_tell` syscall and is similar to `seek` where you offset 0 bytes from the current position. <br>这对应于 `fd_tell` syscall，并且类似于 `seek`，其中您从当前位置偏移了 0 个字节。<br>
    ///
    fn tell(&self) -> io::Result<u64>;

    /// Adjust the flags associated with this file. <br>调整与此文件关联的标志。<br>
    ///
    /// This corresponds to the `fd_fdstat_set_flags` syscall. <br>这对应于 `fd_fdstat_set_flags` syscall。<br>
    fn fdstat_set_flags(&self, flags: u16) -> io::Result<()>;

    /// Adjust the rights associated with this file. <br>调整与此文件关联的权限。<br>
    ///
    /// This corresponds to the `fd_fdstat_set_rights` syscall. <br>这对应于 `fd_fdstat_set_rights` syscall。<br>
    fn fdstat_set_rights(&self, rights: u64, inheriting: u64) -> io::Result<()>;

    /// Provide file advisory information on a file descriptor. <br>提供有关文件描述符的文件咨询信息。<br>
    ///
    /// This corresponds to the `fd_advise` syscall. <br>这对应于 `fd_advise` syscall。<br>
    fn advise(&self, offset: u64, len: u64, advice: u8) -> io::Result<()>;

    /// Force the allocation of space in a file. <br>强制在文件中分配空间。<br>
    ///
    /// This corresponds to the `fd_allocate` syscall. <br>这对应于 `fd_allocate` syscall。<br>
    fn allocate(&self, offset: u64, len: u64) -> io::Result<()>;

    /// Create a directory. <br>创建一个目录。<br>
    ///
    /// This corresponds to the `path_create_directory` syscall. <br>这对应于 `path_create_directory` syscall。<br>
    fn create_directory<P: AsRef<Path>>(&self, dir: P) -> io::Result<()>;

    /// Read the contents of a symbolic link. <br>读取符号链接的内容。<br>
    ///
    /// This corresponds to the `path_readlink` syscall. <br>这对应于 `path_readlink` syscall。<br>
    fn read_link<P: AsRef<Path>>(&self, path: P) -> io::Result<PathBuf>;

    /// Return the attributes of a file or directory. <br>返回文件或目录的属性。<br>
    ///
    /// This corresponds to the `path_filestat_get` syscall. <br>这对应于 `path_filestat_get` syscall。<br>
    fn metadata_at<P: AsRef<Path>>(&self, lookup_flags: u32, path: P) -> io::Result<Metadata>;

    /// Unlink a file. <br>取消链接文件。<br>
    ///
    /// This corresponds to the `path_unlink_file` syscall. <br>这对应于 `path_unlink_file` syscall。<br>
    fn remove_file<P: AsRef<Path>>(&self, path: P) -> io::Result<()>;

    /// Remove a directory. <br>删除目录。<br>
    ///
    /// This corresponds to the `path_remove_directory` syscall. <br>这对应于 `path_remove_directory` syscall。<br>
    fn remove_directory<P: AsRef<Path>>(&self, path: P) -> io::Result<()>;
}

// FIXME: bind fd_fdstat_get - need to define a custom return type <br>绑定 fd_fdstat_get - 需要定义一个自定义返回类型<br>
// FIXME: bind fd_readdir - can't return `ReadDir` since we only have entry name <br>绑定 fd_readdir - 无法返回 `ReadDir`，因为我们只有条目名<br>
// FIXME: bind fd_filestat_set_times maybe? <br>绑定 fd_filestat_set_times 也许？<br> - on crates.io for unix <br>- 在 crates.io 上用于 unix<br>
// FIXME: bind path_filestat_set_times maybe? <br>绑定 path_filestat_set_times 也许？<br> - on crates.io for unix <br>- 在 crates.io 上用于 unix<br>
// FIXME: bind poll_oneoff maybe? <br>绑定 poll_oneoff 也许？<br> - probably should wait for I/O to settle <br>- 可能应该等待 I/O 稳定下来<br>
// FIXME: bind random_get maybe? <br>绑定 random_get 也许？<br> - on crates.io for unix <br>- 在 crates.io 上用于 unix<br>

impl FileExt for fs::File {
    fn read_vectored_at(&self, bufs: &mut [IoSliceMut<'_>], offset: u64) -> io::Result<usize> {
        self.as_inner().as_inner().pread(bufs, offset)
    }

    fn write_vectored_at(&self, bufs: &[IoSlice<'_>], offset: u64) -> io::Result<usize> {
        self.as_inner().as_inner().pwrite(bufs, offset)
    }

    fn tell(&self) -> io::Result<u64> {
        self.as_inner().as_inner().tell()
    }

    fn fdstat_set_flags(&self, flags: u16) -> io::Result<()> {
        self.as_inner().as_inner().set_flags(flags)
    }

    fn fdstat_set_rights(&self, rights: u64, inheriting: u64) -> io::Result<()> {
        self.as_inner().as_inner().set_rights(rights, inheriting)
    }

    fn advise(&self, offset: u64, len: u64, advice: u8) -> io::Result<()> {
        let advice = match advice {
            a if a == wasi::ADVICE_NORMAL.raw() => wasi::ADVICE_NORMAL,
            a if a == wasi::ADVICE_SEQUENTIAL.raw() => wasi::ADVICE_SEQUENTIAL,
            a if a == wasi::ADVICE_RANDOM.raw() => wasi::ADVICE_RANDOM,
            a if a == wasi::ADVICE_WILLNEED.raw() => wasi::ADVICE_WILLNEED,
            a if a == wasi::ADVICE_DONTNEED.raw() => wasi::ADVICE_DONTNEED,
            a if a == wasi::ADVICE_NOREUSE.raw() => wasi::ADVICE_NOREUSE,
            _ => {
                return Err(io::const_io_error!(
                    io::ErrorKind::InvalidInput,
                    "invalid parameter 'advice'",
                ));
            }
        };

        self.as_inner().as_inner().advise(offset, len, advice)
    }

    fn allocate(&self, offset: u64, len: u64) -> io::Result<()> {
        self.as_inner().as_inner().allocate(offset, len)
    }

    fn create_directory<P: AsRef<Path>>(&self, dir: P) -> io::Result<()> {
        self.as_inner().as_inner().create_directory(osstr2str(dir.as_ref().as_ref())?)
    }

    fn read_link<P: AsRef<Path>>(&self, path: P) -> io::Result<PathBuf> {
        self.as_inner().read_link(path.as_ref())
    }

    fn metadata_at<P: AsRef<Path>>(&self, lookup_flags: u32, path: P) -> io::Result<Metadata> {
        let m = self.as_inner().metadata_at(lookup_flags, path.as_ref())?;
        Ok(FromInner::from_inner(m))
    }

    fn remove_file<P: AsRef<Path>>(&self, path: P) -> io::Result<()> {
        self.as_inner().as_inner().unlink_file(osstr2str(path.as_ref().as_ref())?)
    }

    fn remove_directory<P: AsRef<Path>>(&self, path: P) -> io::Result<()> {
        self.as_inner().as_inner().remove_directory(osstr2str(path.as_ref().as_ref())?)
    }
}

/// WASI-specific extensions to [`fs::OpenOptions`]. <br>特定于 WASI 的 [`fs::OpenOptions`] 的扩展。<br>
pub trait OpenOptionsExt {
    /// Pass custom `dirflags` argument to `path_open`. <br>将自定义 `dirflags` 参数传递给 `path_open`。<br>
    ///
    /// This option configures the `dirflags` argument to the `path_open` syscall which `OpenOptions` will eventually call. <br>此选项将 `dirflags` 参数配置为 `OpenOptions` 最终将调用的 `path_open` 系统调用。<br>
    /// The `dirflags` argument configures how the file is looked up, currently primarily affecting whether symlinks are followed or not. <br>`dirflags` 参数配置如何查找文件，当前主要影响是否遵循符号链接。<br>
    ///
    ///
    /// By default this value is `__WASI_LOOKUP_SYMLINK_FOLLOW`, or symlinks are followed. <br>默认情况下，此值为 `__WASI_LOOKUP_SYMLINK_FOLLOW`，否则将遵循符号链接。<br>
    /// You can call this method with 0 to disable following symlinks <br>您可以使用 0 调用此方法以禁用以下符号链接<br>
    ///
    fn lookup_flags(&mut self, flags: u32) -> &mut Self;

    /// Indicates whether `OpenOptions` must open a directory or not. <br>指示 `OpenOptions` 是否必须打开目录。<br>
    ///
    /// This method will configure whether the `__WASI_O_DIRECTORY` flag is passed when opening a file. <br>此方法将配置打开文件时是否传递 `__WASI_O_DIRECTORY` 标志。<br>
    /// When passed it will require that the opened path is a directory. <br>传递时，将要求打开的路径是目录。<br>
    ///
    /// This option is by default `false` <br>默认情况下，此选项为 `false`<br>
    ///
    fn directory(&mut self, dir: bool) -> &mut Self;

    /// Indicates whether `__WASI_FDFLAG_DSYNC` is passed in the `fs_flags` field of `path_open`. <br>指示是否在 `path_open` 的 `fs_flags` 字段中传递 `__WASI_FDFLAG_DSYNC`。<br>
    ///
    ///
    /// This option is by default `false` <br>默认情况下，此选项为 `false`<br>
    fn dsync(&mut self, dsync: bool) -> &mut Self;

    /// Indicates whether `__WASI_FDFLAG_NONBLOCK` is passed in the `fs_flags` field of `path_open`. <br>指示是否在 `path_open` 的 `fs_flags` 字段中传递 `__WASI_FDFLAG_NONBLOCK`。<br>
    ///
    ///
    /// This option is by default `false` <br>默认情况下，此选项为 `false`<br>
    fn nonblock(&mut self, nonblock: bool) -> &mut Self;

    /// Indicates whether `__WASI_FDFLAG_RSYNC` is passed in the `fs_flags` field of `path_open`. <br>指示是否在 `path_open` 的 `fs_flags` 字段中传递 `__WASI_FDFLAG_RSYNC`。<br>
    ///
    ///
    /// This option is by default `false` <br>默认情况下，此选项为 `false`<br>
    fn rsync(&mut self, rsync: bool) -> &mut Self;

    /// Indicates whether `__WASI_FDFLAG_SYNC` is passed in the `fs_flags` field of `path_open`. <br>指示是否在 `path_open` 的 `fs_flags` 字段中传递 `__WASI_FDFLAG_SYNC`。<br>
    ///
    ///
    /// This option is by default `false` <br>默认情况下，此选项为 `false`<br>
    fn sync(&mut self, sync: bool) -> &mut Self;

    /// Indicates the value that should be passed in for the `fs_rights_base` parameter of `path_open`. <br>指示应为 `path_open` 的 `fs_rights_base` 参数传递的值。<br>
    ///
    /// This option defaults based on the `read` and `write` configuration of this `OpenOptions` builder. <br>默认情况下，此选项基于此 `OpenOptions` 构建器的 `read` 和 `write` 配置。<br>
    /// If this method is called, however, the exact mask passed in will be used instead. <br>但是，如果调用此方法，则将使用传入的确切掩码。<br>
    ///
    ///
    fn fs_rights_base(&mut self, rights: u64) -> &mut Self;

    /// Indicates the value that should be passed in for the `fs_rights_inheriting` parameter of `path_open`. <br>指示应为 `path_open` 的 `fs_rights_inheriting` 参数传递的值。<br>
    ///
    /// The default for this option is the same value as what will be passed for the `fs_rights_base` parameter but if this method is called then the specified value will be used instead. <br>此选项的默认值与将为 `fs_rights_base` 参数传递的值相同，但是如果调用此方法，则将使用指定的值代替。<br>
    ///
    ///
    ///
    fn fs_rights_inheriting(&mut self, rights: u64) -> &mut Self;

    /// Open a file or directory. <br>打开文件或目录。<br>
    ///
    /// This corresponds to the `path_open` syscall. <br>这对应于 `path_open` syscall。<br>
    fn open_at<P: AsRef<Path>>(&self, file: &File, path: P) -> io::Result<File>;
}

impl OpenOptionsExt for OpenOptions {
    fn lookup_flags(&mut self, flags: u32) -> &mut OpenOptions {
        self.as_inner_mut().lookup_flags(flags);
        self
    }

    fn directory(&mut self, dir: bool) -> &mut OpenOptions {
        self.as_inner_mut().directory(dir);
        self
    }

    fn dsync(&mut self, enabled: bool) -> &mut OpenOptions {
        self.as_inner_mut().dsync(enabled);
        self
    }

    fn nonblock(&mut self, enabled: bool) -> &mut OpenOptions {
        self.as_inner_mut().nonblock(enabled);
        self
    }

    fn rsync(&mut self, enabled: bool) -> &mut OpenOptions {
        self.as_inner_mut().rsync(enabled);
        self
    }

    fn sync(&mut self, enabled: bool) -> &mut OpenOptions {
        self.as_inner_mut().sync(enabled);
        self
    }

    fn fs_rights_base(&mut self, rights: u64) -> &mut OpenOptions {
        self.as_inner_mut().fs_rights_base(rights);
        self
    }

    fn fs_rights_inheriting(&mut self, rights: u64) -> &mut OpenOptions {
        self.as_inner_mut().fs_rights_inheriting(rights);
        self
    }

    fn open_at<P: AsRef<Path>>(&self, file: &File, path: P) -> io::Result<File> {
        let inner = file.as_inner().open_at(path.as_ref(), self.as_inner())?;
        Ok(File::from_inner(inner))
    }
}

/// WASI-specific extensions to [`fs::Metadata`]. <br>特定于 WASI 的 [`fs::Metadata`] 的扩展。<br>
pub trait MetadataExt {
    /// Returns the `st_dev` field of the internal `filestat_t` <br>返回内部 `filestat_t` 的 `st_dev` 字段<br>
    fn dev(&self) -> u64;
    /// Returns the `st_ino` field of the internal `filestat_t` <br>返回内部 `filestat_t` 的 `st_ino` 字段<br>
    fn ino(&self) -> u64;
    /// Returns the `st_nlink` field of the internal `filestat_t` <br>返回内部 `filestat_t` 的 `st_nlink` 字段<br>
    fn nlink(&self) -> u64;
    /// Returns the `st_size` field of the internal `filestat_t` <br>返回内部 `filestat_t` 的 `st_size` 字段<br>
    fn size(&self) -> u64;
    /// Returns the `st_atim` field of the internal `filestat_t` <br>返回内部 `filestat_t` 的 `st_atim` 字段<br>
    fn atim(&self) -> u64;
    /// Returns the `st_mtim` field of the internal `filestat_t` <br>返回内部 `filestat_t` 的 `st_mtim` 字段<br>
    fn mtim(&self) -> u64;
    /// Returns the `st_ctim` field of the internal `filestat_t` <br>返回内部 `filestat_t` 的 `st_ctim` 字段<br>
    fn ctim(&self) -> u64;
}

impl MetadataExt for fs::Metadata {
    fn dev(&self) -> u64 {
        self.as_inner().as_wasi().dev
    }
    fn ino(&self) -> u64 {
        self.as_inner().as_wasi().ino
    }
    fn nlink(&self) -> u64 {
        self.as_inner().as_wasi().nlink
    }
    fn size(&self) -> u64 {
        self.as_inner().as_wasi().size
    }
    fn atim(&self) -> u64 {
        self.as_inner().as_wasi().atim
    }
    fn mtim(&self) -> u64 {
        self.as_inner().as_wasi().mtim
    }
    fn ctim(&self) -> u64 {
        self.as_inner().as_wasi().ctim
    }
}

/// WASI-specific extensions for [`fs::FileType`]. <br>特定于 WASI 的 [`fs::FileType`] 的扩展。<br>
///
/// Adds support for special WASI file types such as block/character devices, pipes, and sockets. <br>增加了对特殊 WASI 文件类型的支持，例如 block/character 设备，管道和套接字。<br>
///
pub trait FileTypeExt {
    /// Returns `true` if this file type is a block device. <br>如果此文件类型是块设备，则返回 `true`。<br>
    fn is_block_device(&self) -> bool;
    /// Returns `true` if this file type is a character device. <br>如果此文件类型是字符设备，则返回 `true`。<br>
    fn is_char_device(&self) -> bool;
    /// Returns `true` if this file type is a socket datagram. <br>如果此文件类型是套接字数据报，则返回 `true`。<br>
    fn is_socket_dgram(&self) -> bool;
    /// Returns `true` if this file type is a socket stream. <br>如果此文件类型是套接字流，则返回 `true`。<br>
    fn is_socket_stream(&self) -> bool;
    /// Returns `true` if this file type is any type of socket. <br>如果此文件类型是任何类型的套接字，则返回 `true`。<br>
    fn is_socket(&self) -> bool {
        self.is_socket_stream() || self.is_socket_dgram()
    }
}

impl FileTypeExt for fs::FileType {
    fn is_block_device(&self) -> bool {
        self.as_inner().bits() == wasi::FILETYPE_BLOCK_DEVICE
    }
    fn is_char_device(&self) -> bool {
        self.as_inner().bits() == wasi::FILETYPE_CHARACTER_DEVICE
    }
    fn is_socket_dgram(&self) -> bool {
        self.as_inner().bits() == wasi::FILETYPE_SOCKET_DGRAM
    }
    fn is_socket_stream(&self) -> bool {
        self.as_inner().bits() == wasi::FILETYPE_SOCKET_STREAM
    }
}

/// WASI-specific extension methods for [`fs::DirEntry`]. <br>特定于 WASI 的 [`fs::DirEntry`] 的扩展方法。<br>
pub trait DirEntryExt {
    /// Returns the underlying `d_ino` field of the `dirent_t` <br>返回 `dirent_t` 的底层 `d_ino` 字段<br>
    fn ino(&self) -> u64;
}

impl DirEntryExt for fs::DirEntry {
    fn ino(&self) -> u64 {
        self.as_inner().ino()
    }
}

/// Create a hard link. <br>创建一个硬链接。<br>
///
/// This corresponds to the `path_link` syscall. <br>这对应于 `path_link` syscall。<br>
pub fn link<P: AsRef<Path>, U: AsRef<Path>>(
    old_fd: &File,
    old_flags: u32,
    old_path: P,
    new_fd: &File,
    new_path: U,
) -> io::Result<()> {
    old_fd.as_inner().as_inner().link(
        old_flags,
        osstr2str(old_path.as_ref().as_ref())?,
        new_fd.as_inner().as_inner(),
        osstr2str(new_path.as_ref().as_ref())?,
    )
}

/// Rename a file or directory. <br>重命名文件或目录。<br>
///
/// This corresponds to the `path_rename` syscall. <br>这对应于 `path_rename` syscall。<br>
pub fn rename<P: AsRef<Path>, U: AsRef<Path>>(
    old_fd: &File,
    old_path: P,
    new_fd: &File,
    new_path: U,
) -> io::Result<()> {
    old_fd.as_inner().as_inner().rename(
        osstr2str(old_path.as_ref().as_ref())?,
        new_fd.as_inner().as_inner(),
        osstr2str(new_path.as_ref().as_ref())?,
    )
}

/// Create a symbolic link. <br>创建一个符号链接。<br>
///
/// This corresponds to the `path_symlink` syscall. <br>这对应于 `path_symlink` syscall。<br>
pub fn symlink<P: AsRef<Path>, U: AsRef<Path>>(
    old_path: P,
    fd: &File,
    new_path: U,
) -> io::Result<()> {
    fd.as_inner()
        .as_inner()
        .symlink(osstr2str(old_path.as_ref().as_ref())?, osstr2str(new_path.as_ref().as_ref())?)
}

/// Create a symbolic link. <br>创建一个符号链接。<br>
///
/// This is a convenience API similar to `std::os::unix::fs::symlink` and `std::os::windows::fs::symlink_file` and `std::os::windows::fs::symlink_dir`. <br>这是一种便捷的 API，类似于 `std::os::unix::fs::symlink`，`std::os::windows::fs::symlink_file` 和 `std::os::windows::fs::symlink_dir`。<br>
///
pub fn symlink_path<P: AsRef<Path>, U: AsRef<Path>>(old_path: P, new_path: U) -> io::Result<()> {
    crate::sys::fs::symlink(old_path.as_ref(), new_path.as_ref())
}

fn osstr2str(f: &OsStr) -> io::Result<&str> {
    f.to_str()
        .ok_or_else(|| io::const_io_error!(io::ErrorKind::Uncategorized, "input must be utf-8"))
}
