//! Platform-specific extensions to `std` for the WebAssembly System Interface (WASI). <br>特定于平台的用于 WebAssembly 系统接口 (WASI) 的 `std` 的扩展。<br>
//!
//! Provides access to platform-level information on WASI, and exposes WASI-specific functions that would otherwise be inappropriate as part of the core `std` library. <br>提供对 WASI 上平台级信息的访问，并公开特定于 WASI 的函数，否则该函数不适合作为核心 `std` 库的一部分。<br>
//!
//! It exposes more ways to deal with platform-specific strings (`OsStr`, `OsString`), allows to set permissions more granularly, extract low-level file descriptors from files and sockets, and has platform-specific helpers for spawning processes. <br>它提供了更多方式来处理特定于平台的字符串 (`OsStr`，`OsString`)，允许更精细地设置权限，从文件和套接字中提取托管文件描述符，并具有用于生成程序的特定于平台的帮助程序。<br>
//!
//!
//! # Examples
//!
//! ```no_run
//! use std::fs::File;
//! use std::os::wasi::prelude::*;
//!
//! fn main() -> std::io::Result<()> {
//!     let f = File::create("foo.txt")?;
//!     let fd = f.as_raw_fd();
//!
//!     // use fd with native WASI bindings <br>将 fd 与原生 WASI 绑定一起使用<br>
//!
//!     Ok(())
//! }
//! ```
//!
//! [`OsStr`]: crate::ffi::OsStr
//! [`OsString`]: crate::ffi::OsString
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]
#![deny(unsafe_op_in_unsafe_fn)]
#![doc(cfg(target_os = "wasi"))]

pub mod ffi;
pub mod fs;
pub mod io;
pub mod net;

/// A prelude for conveniently writing platform-specific code. <br>prelude，用于方便地编写平台特定的代码。<br>
///
/// Includes all extension traits, and some important type definitions. <br>包括所有扩展名 traits 和一些重要的类型定义。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub mod prelude {
    #[doc(no_inline)]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::ffi::{OsStrExt, OsStringExt};
    #[doc(no_inline)]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::fs::FileTypeExt;
    #[doc(no_inline)]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::fs::{DirEntryExt, FileExt, MetadataExt, OpenOptionsExt};
    #[doc(no_inline)]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::io::{AsFd, AsRawFd, BorrowedFd, FromRawFd, IntoRawFd, OwnedFd, RawFd};
}
