//! Windows-specific extensions to primitives in the [`std::ffi`] module. <br>特定于 Windows 的 [`std::ffi`] 模块中原语的扩展。<br>
//!
//! # Overview
//!
//! For historical reasons, the Windows API uses a form of potentially ill-formed UTF-16 encoding for strings. <br>由于历史原因，Windows API 对字符串使用了一种格式可能不正确的 UTF-16 编码。<br> Specifically, the 16-bit code units in Windows strings may contain [isolated surrogate code points which are not paired together][ill-formed-utf-16]. <br>具体来说，Windows 字符串中的 16 位代码单元可能包含 [未配对在一起的孤立代理代码点][ill-formed-utf-16]。<br>
//! The Unicode standard requires that surrogate code points (those in the range U+D800 to U+DFFF) always be *paired*, because in the UTF-16 encoding a *surrogate code unit pair* is used to encode a single character. <br>Unicode 标准要求代理代码点 (在 U+D800 到 U+DFFF 范围内的代码点) 总是成对的，因为在 UTF-16 编码中，*surrogate code 单元对 `*` 用于编码单个字符。<br>
//! For compatibility with code that does not enforce these pairings, Windows does not enforce them, either. <br>为了与不强制执行这些配对的代码兼容，Windows 也不强制执行它们。<br>
//!
//! While it is not always possible to convert such a string losslessly into a valid UTF-16 string (or even UTF-8), it is often desirable to be able to round-trip such a string from and to Windows APIs losslessly. <br>虽然不一定总是可以无损地将这样的字符串转换为有效的 UTF-16 字符串 (甚至 UTF-8)，但是通常希望能够无损地将这样的字符串往返于 Windows API 之间。<br>
//! For example, some Rust code may be "bridging" some Windows APIs together, just passing `WCHAR` strings among those APIs without ever really looking into the strings. <br>例如，某些 Rust 代码可能与某些 Windows API 一起成为 "bridging"，仅在这些 API 之间传递 `WCHAR` 字符串，而无需真正研究这些字符串。<br>
//!
//! If Rust code *does* need to look into those strings, it can convert them to valid UTF-8, possibly lossily, by substituting invalid sequences with [`U+FFFD REPLACEMENT CHARACTER`][U+FFFD], as is conventionally done in other Rust APIs that deal with string encodings. <br>如果确实需要 Rust 代码查看这些字符串，则可以通过用 [`U+FFFD REPLACEMENT CHARACTER`][U+FFFD] 替换无效的序列来将它们转换为有效的 UTF-8，这可能是有损的，就像在其他处理字符串编码的 Rust API 中所做的那样。<br>
//!
//!
//! # `OsStringExt` and `OsStrExt` <br>`OsStringExt` 和 `OsStrExt`<br>
//!
//! [`OsString`] is the Rust wrapper for owned strings in the preferred representation of the operating system. <br>[`OsString`] 是 Rust 包装器，用于操作系统首选表示中的拥有所有权的字符串。<br>
//! On Windows, this struct gets augmented with an implementation of the [`OsStringExt`] trait, which has an [`OsStringExt::from_wide`] method. <br>在 Windows 上，此结构体通过 [`OsStringExt`] trait 的实现得到了增强，该实现具有 [`OsStringExt::from_wide`] 方法。<br> This lets you create an [`OsString`] from a `&[u16]` slice; <br>这使您可以从 `&[u16]` 切片创建 [`OsString`]。<br> presumably you get such a slice out of a `WCHAR` Windows API. <br>大概是从 `WCHAR` Windows API 中得到的切片。<br>
//!
//! Similarly, [`OsStr`] is the Rust wrapper for borrowed strings from preferred representation of the operating system. <br>同样，[`OsStr`] 是 Rust 包装器，用于从操作系统的首选表示形式中借用字符串。<br> On Windows, the [`OsStrExt`] trait provides the [`OsStrExt::encode_wide`] method, which outputs an [`EncodeWide`] iterator. <br>在 Windows 上，[`OsStrExt`] trait 提供了 [`OsStrExt::encode_wide`] 方法，该方法输出 [`EncodeWide`] 迭代器。<br>
//! You can [`collect`] this iterator, for example, to obtain a `Vec<u16>`; <br>例如，您可以 [`collect`] 这个迭代器，以获得一个 `Vec<u16>`；<br> you can later get a pointer to this vector's contents and feed it to Windows APIs. <br>您以后可以获取指向此 vector 内容的指针，并将其提供给 Windows API。<br>
//!
//! These traits, along with [`OsString`] and [`OsStr`], work in conjunction so that it is possible to **round-trip** strings from Windows and back, with no loss of data, even if the strings are ill-formed UTF-16. <br>这些 traits 与 [`OsString`] 和 [`OsStr`] 一起工作，这样就可以在 Windows 和后面往返字符串，而不会丢失数据，即使字符串是格式错误的 UTF-16。<br>
//!
//! [ill-formed-utf-16]: https://simonsapin.github.io/wtf-8/#ill-formed-utf-16
//! [`collect`]: crate::iter::Iterator::collect
//! [U+FFFD]: crate::char::REPLACEMENT_CHARACTER
//! [`std::ffi`]: crate::ffi
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

use crate::ffi::{OsStr, OsString};
use crate::sealed::Sealed;
use crate::sys::os_str::Buf;
use crate::sys_common::wtf8::Wtf8Buf;
use crate::sys_common::{AsInner, FromInner};

#[stable(feature = "rust1", since = "1.0.0")]
pub use crate::sys_common::wtf8::EncodeWide;

/// Windows-specific extensions to [`OsString`]. <br>特定于 Windows 的 [`OsString`] 扩展。<br>
///
/// This trait is sealed: it cannot be implemented outside the standard library. <br>这个 trait 是封闭的：它不能在标准库之外实现。<br>
/// This is so that future additional methods are not breaking changes. <br>这是为了将来的附加方法不会破坏更改。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub trait OsStringExt: Sealed {
    /// Creates an `OsString` from a potentially ill-formed UTF-16 slice of <br>从可能是格式不正确的 UTF-16 切片创建 `OsString`<br>
    /// 16-bit code units. <br>16 位代码单元。<br>
    ///
    /// This is lossless: calling [`OsStrExt::encode_wide`] on the resulting string will always return the original code units. <br>这是无损的：在结果字符串上调用 [`OsStrExt::encode_wide`] 将始终返回原始代码单元。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::ffi::OsString;
    /// use std::os::windows::prelude::*;
    ///
    /// // UTF-16 encoding for "Unicode". <br>"Unicode" 的 UTF-16 编码。<br>
    /// let source = [0x0055, 0x006E, 0x0069, 0x0063, 0x006F, 0x0064, 0x0065];
    ///
    /// let string = OsString::from_wide(&source[..]);
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    fn from_wide(wide: &[u16]) -> Self;
}

#[stable(feature = "rust1", since = "1.0.0")]
impl OsStringExt for OsString {
    fn from_wide(wide: &[u16]) -> OsString {
        FromInner::from_inner(Buf { inner: Wtf8Buf::from_wide(wide) })
    }
}

/// Windows-specific extensions to [`OsStr`]. <br>特定于 Windows 的 [`OsStr`] 扩展。<br>
///
/// This trait is sealed: it cannot be implemented outside the standard library. <br>这个 trait 是封闭的：它不能在标准库之外实现。<br>
/// This is so that future additional methods are not breaking changes. <br>这是为了将来的附加方法不会破坏更改。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub trait OsStrExt: Sealed {
    /// Re-encodes an `OsStr` as a wide character sequence, i.e., potentially ill-formed UTF-16. <br>将 `OsStr` 重新编码为宽字符序列，即可能格式不正确的 UTF-16。<br>
    ///
    /// This is lossless: calling [`OsStringExt::from_wide`] and then `encode_wide` on the result will yield the original code units. <br>这是无损的：在结果上调用 [`OsStringExt::from_wide`]，然后调用 `encode_wide`，将产生原始代码单元。<br>
    ///
    /// Note that the encoding does not add a final null terminator. <br>请注意，编码不会添加最终的空终止符。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::ffi::OsString;
    /// use std::os::windows::prelude::*;
    ///
    /// // UTF-16 encoding for "Unicode". <br>"Unicode" 的 UTF-16 编码。<br>
    /// let source = [0x0055, 0x006E, 0x0069, 0x0063, 0x006F, 0x0064, 0x0065];
    ///
    /// let string = OsString::from_wide(&source[..]);
    ///
    /// let result: Vec<u16> = string.encode_wide().collect();
    /// assert_eq!(&source[..], &result[..]);
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    fn encode_wide(&self) -> EncodeWide<'_>;
}

#[stable(feature = "rust1", since = "1.0.0")]
impl OsStrExt for OsStr {
    fn encode_wide(&self) -> EncodeWide<'_> {
        self.as_inner().inner.encode_wide()
    }
}
