//! Owned and borrowed OS handles. <br>拥有和借用的操作系统句柄。<br>

#![stable(feature = "io_safety", since = "1.63.0")]

use super::raw::{AsRawHandle, FromRawHandle, IntoRawHandle, RawHandle};
use crate::fmt;
use crate::fs;
use crate::io;
use crate::marker::PhantomData;
use crate::mem::forget;
use crate::ptr;
use crate::sys::c;
use crate::sys::cvt;
use crate::sys_common::{AsInner, FromInner, IntoInner};

/// A borrowed handle. <br>借来的句柄。<br>
///
/// This has a lifetime parameter to tie it to the lifetime of something that owns the handle. <br>它有一个生命周期参数，将它与拥有句柄的事物的生命周期联系起来。<br>
///
/// This uses `repr(transparent)` and has the representation of a host handle, so it can be used in FFI in places where a handle is passed as an argument, it is not captured or consumed. <br>这使用 `repr(transparent)` 并具有主机句柄的表示形式，因此它可以在 FFI 中将句柄作为参数传递的地方使用，它不会被捕获或消耗。<br>
///
/// Note that it *may* have the value `-1`, which in `BorrowedHandle` always represents a valid handle value, such as [the current process handle], and not `INVALID_HANDLE_VALUE`, despite the two having the same value. <br>请注意，它*可能*具有值 `-1`，它在 `BorrowedHandle` 中始终表示有效的句柄值，例如 [the current process handle]，而不是 `INVALID_HANDLE_VALUE`，尽管两者具有相同的值。<br>
/// See [here] for the full story. <br>有关完整故事，请参见 [这个][here]。<br>
///
/// And, it *may* have the value `NULL` (0), which can occur when consoles are detached from processes, or when `windows_subsystem` is used. <br>并且，它*可能*具有值 `NULL` (0)，当控制台与进程分离或使用 `windows_subsystem` 时，可能会发生这种情况。<br>
///
/// This type's `.to_owned()` implementation returns another `BorrowedHandle` rather than an `OwnedHandle`. <br>这种类型的 `.to_owned()` 实现返回另一个 `BorrowedHandle` 而不是 `OwnedHandle`。<br> It just makes a trivial copy of the raw handle, which is then borrowed under the same lifetime. <br>它只是简单地复制原始句柄，然后在相同的生命周期下借用。<br>
///
/// [here]: https://devblogs.microsoft.com/oldnewthing/20040302-00/?p=40443
/// [the current process handle]: https://docs.microsoft.com/en-us/windows/win32/api/processthreadsapi/nf-processthreadsapi-getcurrentprocess#remarks
///
///
///
///
///
///
///
///
#[derive(Copy, Clone)]
#[repr(transparent)]
#[stable(feature = "io_safety", since = "1.63.0")]
pub struct BorrowedHandle<'handle> {
    handle: RawHandle,
    _phantom: PhantomData<&'handle OwnedHandle>,
}

/// An owned handle. <br>拥有所有权的句柄。<br>
///
/// This closes the handle on drop. <br>这将会丢弃时关闭句柄。<br>
///
/// Note that it *may* have the value `-1`, which in `OwnedHandle` always represents a valid handle value, such as [the current process handle], and not `INVALID_HANDLE_VALUE`, despite the two having the same value. <br>请注意，它*可能*具有值 `-1`，它在 `OwnedHandle` 中始终表示有效的句柄值，例如 [the current process handle]，而不是 `INVALID_HANDLE_VALUE`，尽管两者具有相同的值。<br>
/// See [here] for the full story. <br>有关完整故事，请参见 [这个][here]。<br>
///
/// And, it *may* have the value `NULL` (0), which can occur when consoles are detached from processes, or when `windows_subsystem` is used. <br>并且，它*可能*具有值 `NULL` (0)，当控制台与进程分离或使用 `windows_subsystem` 时，可能会发生这种情况。<br>
///
/// `OwnedHandle` uses [`CloseHandle`] to close its handle on drop. <br>`OwnedHandle` 使用 [`CloseHandle`] 在关闭时丢弃其句柄。<br>
/// As such, it must not be used with handles to open registry keys which need to be closed with [`RegCloseKey`] instead. <br>因此，它不能与句柄一起使用来打开需要用 [`RegCloseKey`] 关闭的注册表项。<br>
///
/// [`CloseHandle`]: https://docs.microsoft.com/en-us/windows/win32/api/handleapi/nf-handleapi-closehandle
/// [`RegCloseKey`]: https://docs.microsoft.com/en-us/windows/win32/api/winreg/nf-winreg-regclosekey
///
/// [here]: https://devblogs.microsoft.com/oldnewthing/20040302-00/?p=40443
/// [the current process handle]: https://docs.microsoft.com/en-us/windows/win32/api/processthreadsapi/nf-processthreadsapi-getcurrentprocess#remarks
///
///
///
///
#[repr(transparent)]
#[stable(feature = "io_safety", since = "1.63.0")]
pub struct OwnedHandle {
    handle: RawHandle,
}

/// FFI type for handles in return values or out parameters, where `NULL` is used as a sentry value to indicate errors, such as in the return value of `CreateThread`. <br>返回值或输出参数句柄的 FFI 类型，其中 `NULL` 用作指示错误的哨兵值，例如在 `CreateThread` 的返回值中。<br>
/// This uses `repr(transparent)` and has the representation of a host handle, so that it can be used in such FFI declarations. <br>这使用 `repr(transparent)` 并具有主机句柄的表示，因此它可以用于此类 FFI 声明。<br>
///
/// The only thing you can usefully do with a `HandleOrNull` is to convert it into an `OwnedHandle` using its [`TryFrom`] implementation; <br>您可以对 `HandleOrNull` 做的唯一有用的事情是使用其 [`TryFrom`] 实现将其转换成一个 `OwnedHandle`；<br> this conversion takes care of the check for `NULL`. <br>此转换负责检查 `NULL`。<br>
///
/// This ensures that such FFI calls cannot start using the handle without checking for `NULL` first. <br>这可确保此类 FFI 调用在未先检查 `NULL` 的情况下无法开始使用句柄。<br>
///
/// This type may hold any handle value that [`OwnedHandle`] may hold. <br>此类型可以保存 [`OwnedHandle`] 可能保存的任何句柄值。<br>
/// As with `OwnedHandle`, when it holds `-1`, that value is interpreted as a valid handle value, such as [the current process handle], and not `INVALID_HANDLE_VALUE`. <br>与 `OwnedHandle` 一样，当它包含 `-1` 时，该值被解释为有效的句柄值，例如 [the current process handle]，而不是 `INVALID_HANDLE_VALUE`。<br>
///
/// If this holds a non-null handle, it will close the handle on drop. <br>如果它持有一个非空句柄，它将在丢弃时关闭句柄。<br>
///
/// [the current process handle]: https://docs.microsoft.com/en-us/windows/win32/api/processthreadsapi/nf-processthreadsapi-getcurrentprocess#remarks
///
///
///
///
#[repr(transparent)]
#[stable(feature = "io_safety", since = "1.63.0")]
#[derive(Debug)]
pub struct HandleOrNull(OwnedHandle);

/// FFI type for handles in return values or out parameters, where `INVALID_HANDLE_VALUE` is used as a sentry value to indicate errors, such as in the return value of `CreateFileW`. <br>返回值或输出参数句柄的 FFI 类型，其中 `INVALID_HANDLE_VALUE` 用作哨兵值以指示错误，例如在 `CreateFileW` 的返回值中。<br>
/// This uses `repr(transparent)` and has the representation of a host handle, so that it can be used in such FFI declarations. <br>这使用 `repr(transparent)` 并具有主机句柄的表示，因此它可以用于此类 FFI 声明。<br>
///
/// The only thing you can usefully do with a `HandleOrInvalid` is to convert it into an `OwnedHandle` using its [`TryFrom`] implementation; <br>您可以对 `HandleOrInvalid` 做的唯一有用的事情是使用其 [`TryFrom`] 实现将其转换为 `OwnedHandle`；<br> this conversion takes care of the check for `INVALID_HANDLE_VALUE`. <br>此转换负责检查 `INVALID_HANDLE_VALUE`。<br>
///
/// This ensures that such FFI calls cannot start using the handle without checking for `INVALID_HANDLE_VALUE` first. <br>这可确保此类 FFI 调用在未先检查 `INVALID_HANDLE_VALUE` 的情况下无法开始使用句柄。<br>
///
/// This type may hold any handle value that [`OwnedHandle`] may hold, except that when it holds `-1`, that value is interpreted to mean `INVALID_HANDLE_VALUE`. <br>此类型可以保存 [`OwnedHandle`] 可能保存的任何句柄值，但当它保存 `-1` 时，该值被解释为表示 `INVALID_HANDLE_VALUE`。<br>
///
/// If holds a handle other than `INVALID_HANDLE_VALUE`, it will close the handle on drop. <br>如果持有 `INVALID_HANDLE_VALUE` 以外的句柄，它将在丢弃时关闭句柄。<br>
///
///
///
///
#[repr(transparent)]
#[stable(feature = "io_safety", since = "1.63.0")]
#[derive(Debug)]
pub struct HandleOrInvalid(OwnedHandle);

// The Windows [`HANDLE`] type may be transferred across and shared between thread boundaries (despite containing a `*mut void`, which in general isn't `Send` or `Sync`). <br>Windows [`HANDLE`] 类型可以跨线程边界传输和共享 (尽管包含 `*mut void`，它通常不是 `Send` 或 `Sync`)。<br>
//
//
// [`HANDLE`]: std::os::windows::raw::HANDLE
//
#[stable(feature = "io_safety", since = "1.63.0")]
unsafe impl Send for OwnedHandle {}
#[stable(feature = "io_safety", since = "1.63.0")]
unsafe impl Send for HandleOrNull {}
#[stable(feature = "io_safety", since = "1.63.0")]
unsafe impl Send for HandleOrInvalid {}
#[stable(feature = "io_safety", since = "1.63.0")]
unsafe impl Send for BorrowedHandle<'_> {}
#[stable(feature = "io_safety", since = "1.63.0")]
unsafe impl Sync for OwnedHandle {}
#[stable(feature = "io_safety", since = "1.63.0")]
unsafe impl Sync for HandleOrNull {}
#[stable(feature = "io_safety", since = "1.63.0")]
unsafe impl Sync for HandleOrInvalid {}
#[stable(feature = "io_safety", since = "1.63.0")]
unsafe impl Sync for BorrowedHandle<'_> {}

impl BorrowedHandle<'_> {
    /// Return a `BorrowedHandle` holding the given raw handle. <br>返回一个 `BorrowedHandle` 持有给定的原始句柄。<br>
    ///
    /// # Safety
    ///
    /// The resource pointed to by `handle` must be a valid open handle, it must remain open for the duration of the returned `BorrowedHandle`. <br>`handle` 指向的资源必须是有效的打开句柄，它必须在返回的 `BorrowedHandle` 期间保持打开状态。<br>
    ///
    /// Note that it *may* have the value `INVALID_HANDLE_VALUE` (-1), which is sometimes a valid handle value. <br>请注意，它*可能*具有值 `INVALID_HANDLE_VALUE` (-1)，有时这是一个有效的句柄值。<br> See [here] for the full story. <br>有关完整故事，请参见 [这个][here]。<br>
    ///
    /// And, it *may* have the value `NULL` (0), which can occur when consoles are detached from processes, or when `windows_subsystem` is used. <br>并且，它*可能*具有值 `NULL` (0)，当控制台与进程分离或使用 `windows_subsystem` 时，可能会发生这种情况。<br>
    ///
    ///
    /// [here]: https://devblogs.microsoft.com/oldnewthing/20040302-00/?p=40443
    ///
    ///
    #[inline]
    #[rustc_const_stable(feature = "io_safety", since = "1.63.0")]
    #[stable(feature = "io_safety", since = "1.63.0")]
    pub const unsafe fn borrow_raw(handle: RawHandle) -> Self {
        Self { handle, _phantom: PhantomData }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl TryFrom<HandleOrNull> for OwnedHandle {
    type Error = NullHandleError;

    #[inline]
    fn try_from(handle_or_null: HandleOrNull) -> Result<Self, NullHandleError> {
        let owned_handle = handle_or_null.0;
        if owned_handle.handle.is_null() {
            // Don't call `CloseHandle`; <br>不要调用 `CloseHandle`;<br> it'd be harmless, except that it could overwrite the `GetLastError` error. <br>它是无害的，只是它可能会覆盖 `GetLastError` 错误。<br>
            //
            forget(owned_handle);

            Err(NullHandleError(()))
        } else {
            Ok(owned_handle)
        }
    }
}

impl OwnedHandle {
    /// Creates a new `OwnedHandle` instance that shares the same underlying object as the existing `OwnedHandle` instance. <br>创建一个新的 `OwnedHandle` 实例，该实例与现有 `OwnedHandle` 实例共享相同的底层 object。<br>
    ///
    #[stable(feature = "io_safety", since = "1.63.0")]
    pub fn try_clone(&self) -> crate::io::Result<Self> {
        self.as_handle().try_clone_to_owned()
    }
}

impl BorrowedHandle<'_> {
    /// Creates a new `OwnedHandle` instance that shares the same underlying object as the existing `BorrowedHandle` instance. <br>创建一个新的 `OwnedHandle` 实例，该实例与现有 `BorrowedHandle` 实例共享相同的底层 object。<br>
    ///
    #[stable(feature = "io_safety", since = "1.63.0")]
    pub fn try_clone_to_owned(&self) -> crate::io::Result<OwnedHandle> {
        self.duplicate(0, false, c::DUPLICATE_SAME_ACCESS)
    }

    pub(crate) fn duplicate(
        &self,
        access: c::DWORD,
        inherit: bool,
        options: c::DWORD,
    ) -> io::Result<OwnedHandle> {
        let handle = self.as_raw_handle();

        // `Stdin`, `Stdout`, and `Stderr` can all hold null handles, such as in a process with a detached console. <br>`Stdin`、`Stdout` 和 `Stderr` 都可以持有空句柄，例如在具有分离控制台的进程中。<br>
        // `DuplicateHandle` would fail if we passed it a null handle, but we can treat null as a valid handle which doesn't do any I/O, and allow it to be duplicated. <br>如果我们向 `DuplicateHandle` 传递一个空句柄，`DuplicateHandle` 将失败，但我们可以将 null 视为不执行任何 I/O 的有效句柄，并允许它被复制。<br>
        //
        //
        if handle.is_null() {
            return unsafe { Ok(OwnedHandle::from_raw_handle(handle)) };
        }

        let mut ret = ptr::null_mut();
        cvt(unsafe {
            let cur_proc = c::GetCurrentProcess();
            c::DuplicateHandle(
                cur_proc,
                handle,
                cur_proc,
                &mut ret,
                access,
                inherit as c::BOOL,
                options,
            )
        })?;
        unsafe { Ok(OwnedHandle::from_raw_handle(ret)) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl TryFrom<HandleOrInvalid> for OwnedHandle {
    type Error = InvalidHandleError;

    #[inline]
    fn try_from(handle_or_invalid: HandleOrInvalid) -> Result<Self, InvalidHandleError> {
        let owned_handle = handle_or_invalid.0;
        if owned_handle.handle == c::INVALID_HANDLE_VALUE {
            // Don't call `CloseHandle`; <br>不要调用 `CloseHandle`;<br> it'd be harmless, except that it could overwrite the `GetLastError` error. <br>它是无害的，只是它可能会覆盖 `GetLastError` 错误。<br>
            //
            forget(owned_handle);

            Err(InvalidHandleError(()))
        } else {
            Ok(owned_handle)
        }
    }
}

/// This is the error type used by [`HandleOrNull`] when attempting to convert into a handle, to indicate that the value is null. <br>这是 [`HandleOrNull`] 在尝试转换为句柄时使用的错误类型，以指示该值为空。<br>
///
// The empty field prevents constructing this, and allows extending it in the future. <br>空字段阻止构建它，并允许在 future 中扩展它。<br>
#[stable(feature = "io_safety", since = "1.63.0")]
#[derive(Debug, Clone, PartialEq, Eq)]
pub struct NullHandleError(());

#[stable(feature = "io_safety", since = "1.63.0")]
impl fmt::Display for NullHandleError {
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        "A HandleOrNull could not be converted to a handle because it was null".fmt(fmt)
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl crate::error::Error for NullHandleError {}

/// This is the error type used by [`HandleOrInvalid`] when attempting to convert into a handle, to indicate that the value is `INVALID_HANDLE_VALUE`. <br>这是 [`HandleOrInvalid`] 在尝试转换为句柄时使用的错误类型，以指示该值为 `INVALID_HANDLE_VALUE`。<br>
///
///
// The empty field prevents constructing this, and allows extending it in the future. <br>空字段阻止构建它，并允许在 future 中扩展它。<br>
#[stable(feature = "io_safety", since = "1.63.0")]
#[derive(Debug, Clone, PartialEq, Eq)]
pub struct InvalidHandleError(());

#[stable(feature = "io_safety", since = "1.63.0")]
impl fmt::Display for InvalidHandleError {
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        "A HandleOrInvalid could not be converted to a handle because it was INVALID_HANDLE_VALUE"
            .fmt(fmt)
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl crate::error::Error for InvalidHandleError {}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsRawHandle for BorrowedHandle<'_> {
    #[inline]
    fn as_raw_handle(&self) -> RawHandle {
        self.handle
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsRawHandle for OwnedHandle {
    #[inline]
    fn as_raw_handle(&self) -> RawHandle {
        self.handle
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl IntoRawHandle for OwnedHandle {
    #[inline]
    fn into_raw_handle(self) -> RawHandle {
        let handle = self.handle;
        forget(self);
        handle
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl FromRawHandle for OwnedHandle {
    #[inline]
    unsafe fn from_raw_handle(handle: RawHandle) -> Self {
        Self { handle }
    }
}

impl HandleOrNull {
    /// Constructs a new instance of `Self` from the given `RawHandle` returned from a Windows API that uses null to indicate failure, such as `CreateThread`. <br>根据从 Windows API 返回的给定 `RawHandle` 构造一个新的 `Self` 实例，该 API 使用 null 表示失败，例如 `CreateThread`。<br>
    ///
    ///
    /// Use `HandleOrInvalid` instead of `HandleOrNull` for APIs that use `INVALID_HANDLE_VALUE` to indicate failure. <br>对于使用 `INVALID_HANDLE_VALUE` 指示失败的 API，使用 `HandleOrInvalid` 而不是 `HandleOrNull`。<br>
    ///
    /// # Safety
    ///
    /// The passed `handle` value must either satisfy the safety requirements of [`FromRawHandle::from_raw_handle`], or be null. <br>传递的 `handle` 值必须满足 [`FromRawHandle::from_raw_handle`] 的安全要求，或者为空。<br>
    /// Note that not all Windows APIs use null for errors; <br>请注意，并非所有 Windows API 都使用 null 表示错误；<br> see [here] for the full story. <br>有关完整故事，请参见 [这个][here]。<br>
    ///
    /// [here]: https://devblogs.microsoft.com/oldnewthing/20040302-00/?p=40443
    ///
    ///
    ///
    #[stable(feature = "io_safety", since = "1.63.0")]
    #[inline]
    pub unsafe fn from_raw_handle(handle: RawHandle) -> Self {
        Self(OwnedHandle::from_raw_handle(handle))
    }
}

impl HandleOrInvalid {
    /// Constructs a new instance of `Self` from the given `RawHandle` returned from a Windows API that uses `INVALID_HANDLE_VALUE` to indicate failure, such as `CreateFileW`. <br>根据从 Windows API 返回的给定 `RawHandle` 构造 `Self` 的新实例，该 API 使用 `INVALID_HANDLE_VALUE` 指示失败，例如 `CreateFileW`。<br>
    ///
    ///
    /// Use `HandleOrNull` instead of `HandleOrInvalid` for APIs that use null to indicate failure. <br>对于使用 null 表示失败的 API，使用 `HandleOrNull` 而不是 `HandleOrInvalid`。<br>
    ///
    /// # Safety
    ///
    /// The passed `handle` value must either satisfy the safety requirements of [`FromRawHandle::from_raw_handle`], or be `INVALID_HANDLE_VALUE` (-1). <br>传递的 `handle` 值必须满足 [`FromRawHandle::from_raw_handle`] 的安全要求，或者是 `INVALID_HANDLE_VALUE` (-1)。<br>
    /// Note that not all Windows APIs use `INVALID_HANDLE_VALUE` for errors; <br>请注意，并非所有 Windows API 都使用 `INVALID_HANDLE_VALUE` 来处理错误；<br> see [here] for the full story. <br>有关完整故事，请参见 [这个][here]。<br>
    ///
    /// [here]: https://devblogs.microsoft.com/oldnewthing/20040302-00/?p=40443
    ///
    ///
    ///
    ///
    #[stable(feature = "io_safety", since = "1.63.0")]
    #[inline]
    pub unsafe fn from_raw_handle(handle: RawHandle) -> Self {
        Self(OwnedHandle::from_raw_handle(handle))
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl Drop for OwnedHandle {
    #[inline]
    fn drop(&mut self) {
        unsafe {
            let _ = c::CloseHandle(self.handle);
        }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl fmt::Debug for BorrowedHandle<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("BorrowedHandle").field("handle", &self.handle).finish()
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl fmt::Debug for OwnedHandle {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("OwnedHandle").field("handle", &self.handle).finish()
    }
}

/// A trait to borrow the handle from an underlying object. <br>从底层对象借用句柄的 trait。<br>
#[stable(feature = "io_safety", since = "1.63.0")]
pub trait AsHandle {
    /// Borrows the handle. <br>借用句柄。<br>
    ///
    /// # Example
    ///
    /// ```rust,no_run
    /// use std::fs::File;
    /// # use std::io;
    /// use std::os::windows::io::{AsHandle, BorrowedHandle};
    ///
    /// let mut f = File::open("foo.txt")?;
    /// let borrowed_handle: BorrowedHandle<'_> = f.as_handle();
    /// # Ok::<(), io::Error>(())
    /// ```
    #[stable(feature = "io_safety", since = "1.63.0")]
    fn as_handle(&self) -> BorrowedHandle<'_>;
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl<T: AsHandle> AsHandle for &T {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        T::as_handle(self)
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl<T: AsHandle> AsHandle for &mut T {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        T::as_handle(self)
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for BorrowedHandle<'_> {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        *self
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for OwnedHandle {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        // Safety: `OwnedHandle` and `BorrowedHandle` have the same validity invariants, and the `BorrowdHandle` is bounded by the lifetime of `&self`. <br>安全性: `OwnedHandle` 和 `BorrowedHandle` 具有相同的有效性不变量，`BorrowdHandle` 受 `&self` 的生命周期限制。<br>
        //
        //
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for fs::File {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        self.as_inner().as_handle()
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<fs::File> for OwnedHandle {
    #[inline]
    fn from(file: fs::File) -> OwnedHandle {
        file.into_inner().into_inner().into_inner().into()
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<OwnedHandle> for fs::File {
    #[inline]
    fn from(owned: OwnedHandle) -> Self {
        Self::from_inner(FromInner::from_inner(FromInner::from_inner(owned)))
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for crate::io::Stdin {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl<'a> AsHandle for crate::io::StdinLock<'a> {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for crate::io::Stdout {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl<'a> AsHandle for crate::io::StdoutLock<'a> {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for crate::io::Stderr {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl<'a> AsHandle for crate::io::StderrLock<'a> {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for crate::process::ChildStdin {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<crate::process::ChildStdin> for OwnedHandle {
    #[inline]
    fn from(child_stdin: crate::process::ChildStdin) -> OwnedHandle {
        unsafe { OwnedHandle::from_raw_handle(child_stdin.into_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for crate::process::ChildStdout {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<crate::process::ChildStdout> for OwnedHandle {
    #[inline]
    fn from(child_stdout: crate::process::ChildStdout) -> OwnedHandle {
        unsafe { OwnedHandle::from_raw_handle(child_stdout.into_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for crate::process::ChildStderr {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<crate::process::ChildStderr> for OwnedHandle {
    #[inline]
    fn from(child_stderr: crate::process::ChildStderr) -> OwnedHandle {
        unsafe { OwnedHandle::from_raw_handle(child_stderr.into_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl<T> AsHandle for crate::thread::JoinHandle<T> {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        unsafe { BorrowedHandle::borrow_raw(self.as_raw_handle()) }
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl<T> From<crate::thread::JoinHandle<T>> for OwnedHandle {
    #[inline]
    fn from(join_handle: crate::thread::JoinHandle<T>) -> OwnedHandle {
        join_handle.into_inner().into_handle().into_inner()
    }
}
