//! Platform-specific extensions to `std` for Windows. <br>特定于平台的 Windows 的 `std` 扩展。<br>
//!
//! Provides access to platform-level information for Windows, and exposes Windows-specific idioms that would otherwise be inappropriate as part the core `std` library. <br>提供对 Windows 的平台级信息的访问，并公开 Windows 特定的习惯用法，否则这些习惯用法作为核心 `std` 库的一部分将是不合适的。<br>
//! These extensions allow developers to use `std` types and idioms with Windows in a way that the normal platform-agnostic idioms would not normally support. <br>这些扩展使开发人员可以将 `std` 类型和惯用语与 Windows 一起使用，而通常与平台无关的惯用语通常不支持这种方式。<br>
//!
//! # Examples
//!
//! ```no_run
//! use std::fs::File;
//! use std::os::windows::prelude::*;
//!
//! fn main() -> std::io::Result<()> {
//!     let f = File::create("foo.txt")?;
//!     let handle = f.as_raw_handle();
//!
//!     // use handle with native windows bindings <br>将句柄与本地 windows 绑定一起使用<br>
//!
//!     Ok(())
//! }
//! ```
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]
#![doc(cfg(windows))]

pub mod ffi;
pub mod fs;
pub mod io;
pub mod process;
pub mod raw;
pub mod thread;

/// A prelude for conveniently writing platform-specific code. <br>prelude，用于方便地编写平台特定的代码。<br>
///
/// Includes all extension traits, and some important type definitions. <br>包括所有扩展名 traits 和一些重要的类型定义。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub mod prelude {
    #[doc(no_inline)]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::ffi::{OsStrExt, OsStringExt};
    #[doc(no_inline)]
    #[stable(feature = "file_offset", since = "1.15.0")]
    pub use super::fs::FileExt;
    #[doc(no_inline)]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::fs::{MetadataExt, OpenOptionsExt};
    #[doc(no_inline)]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::io::{
        AsHandle, AsSocket, BorrowedHandle, BorrowedSocket, FromRawHandle, FromRawSocket,
        HandleOrInvalid, IntoRawHandle, IntoRawSocket, OwnedHandle, OwnedSocket,
    };
    #[doc(no_inline)]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub use super::io::{AsRawHandle, AsRawSocket, RawHandle, RawSocket};
}
