//! Windows-specific extensions to primitives in the [`std::process`] module. <br>特定于 Windows 的 [`std::process`] 模块中原语的扩展。<br>
//!
//! [`std::process`]: crate::process

#![stable(feature = "process_extensions", since = "1.2.0")]

use crate::ffi::OsStr;
use crate::os::windows::io::{
    AsHandle, AsRawHandle, BorrowedHandle, FromRawHandle, IntoRawHandle, OwnedHandle, RawHandle,
};
use crate::process;
use crate::sealed::Sealed;
use crate::sys;
use crate::sys_common::{AsInner, AsInnerMut, FromInner, IntoInner};

#[stable(feature = "process_extensions", since = "1.2.0")]
impl FromRawHandle for process::Stdio {
    unsafe fn from_raw_handle(handle: RawHandle) -> process::Stdio {
        let handle = sys::handle::Handle::from_raw_handle(handle as *mut _);
        let io = sys::process::Stdio::Handle(handle);
        process::Stdio::from_inner(io)
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<OwnedHandle> for process::Stdio {
    fn from(handle: OwnedHandle) -> process::Stdio {
        let handle = sys::handle::Handle::from_inner(handle);
        let io = sys::process::Stdio::Handle(handle);
        process::Stdio::from_inner(io)
    }
}

#[stable(feature = "process_extensions", since = "1.2.0")]
impl AsRawHandle for process::Child {
    #[inline]
    fn as_raw_handle(&self) -> RawHandle {
        self.as_inner().handle().as_raw_handle() as *mut _
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl AsHandle for process::Child {
    #[inline]
    fn as_handle(&self) -> BorrowedHandle<'_> {
        self.as_inner().handle().as_handle()
    }
}

#[stable(feature = "into_raw_os", since = "1.4.0")]
impl IntoRawHandle for process::Child {
    fn into_raw_handle(self) -> RawHandle {
        self.into_inner().into_handle().into_raw_handle() as *mut _
    }
}

#[stable(feature = "io_safety", since = "1.63.0")]
impl From<process::Child> for OwnedHandle {
    fn from(child: process::Child) -> OwnedHandle {
        child.into_inner().into_handle().into_inner()
    }
}

#[stable(feature = "process_extensions", since = "1.2.0")]
impl AsRawHandle for process::ChildStdin {
    #[inline]
    fn as_raw_handle(&self) -> RawHandle {
        self.as_inner().handle().as_raw_handle() as *mut _
    }
}

#[stable(feature = "process_extensions", since = "1.2.0")]
impl AsRawHandle for process::ChildStdout {
    #[inline]
    fn as_raw_handle(&self) -> RawHandle {
        self.as_inner().handle().as_raw_handle() as *mut _
    }
}

#[stable(feature = "process_extensions", since = "1.2.0")]
impl AsRawHandle for process::ChildStderr {
    #[inline]
    fn as_raw_handle(&self) -> RawHandle {
        self.as_inner().handle().as_raw_handle() as *mut _
    }
}

#[stable(feature = "into_raw_os", since = "1.4.0")]
impl IntoRawHandle for process::ChildStdin {
    fn into_raw_handle(self) -> RawHandle {
        self.into_inner().into_handle().into_raw_handle() as *mut _
    }
}

#[stable(feature = "into_raw_os", since = "1.4.0")]
impl IntoRawHandle for process::ChildStdout {
    fn into_raw_handle(self) -> RawHandle {
        self.into_inner().into_handle().into_raw_handle() as *mut _
    }
}

#[stable(feature = "into_raw_os", since = "1.4.0")]
impl IntoRawHandle for process::ChildStderr {
    fn into_raw_handle(self) -> RawHandle {
        self.into_inner().into_handle().into_raw_handle() as *mut _
    }
}

/// Windows-specific extensions to [`process::ExitStatus`]. <br>特定于 Windows 的 [`process::ExitStatus`] 扩展。<br>
///
/// This trait is sealed: it cannot be implemented outside the standard library. <br>这个 trait 是封闭的：它不能在标准库之外实现。<br>
/// This is so that future additional methods are not breaking changes. <br>这是为了将来的附加方法不会破坏更改。<br>
#[stable(feature = "exit_status_from", since = "1.12.0")]
pub trait ExitStatusExt: Sealed {
    /// Creates a new `ExitStatus` from the raw underlying `u32` return value of a process. <br>根据进程的原始底层 `u32` 返回值创建新的 `ExitStatus`。<br>
    ///
    #[stable(feature = "exit_status_from", since = "1.12.0")]
    fn from_raw(raw: u32) -> Self;
}

#[stable(feature = "exit_status_from", since = "1.12.0")]
impl ExitStatusExt for process::ExitStatus {
    fn from_raw(raw: u32) -> Self {
        process::ExitStatus::from_inner(From::from(raw))
    }
}

/// Windows-specific extensions to the [`process::Command`] builder. <br>特定于 Windows 的 [`process::Command`] 构建器扩展。<br>
///
/// This trait is sealed: it cannot be implemented outside the standard library. <br>这个 trait 是封闭的：它不能在标准库之外实现。<br>
/// This is so that future additional methods are not breaking changes. <br>这是为了将来的附加方法不会破坏更改。<br>
#[stable(feature = "windows_process_extensions", since = "1.16.0")]
pub trait CommandExt: Sealed {
    /// Sets the [process creation flags][1] to be passed to `CreateProcess`. <br>设置要传递给 `CreateProcess` 的 [进程创建标志][1]。<br>
    ///
    /// These will always be ORed with `CREATE_UNICODE_ENVIRONMENT`. <br>这些将始终与 `CREATE_UNICODE_ENVIRONMENT` 进行或运算。<br>
    ///
    /// [1]: https://docs.microsoft.com/en-us/windows/win32/procthread/process-creation-flags
    #[stable(feature = "windows_process_extensions", since = "1.16.0")]
    fn creation_flags(&mut self, flags: u32) -> &mut process::Command;

    /// Forces all arguments to be wrapped in quote (`"`) characters. <br>强制所有参数用 (`"`) 引号括起来。<br>
    ///
    /// This is useful for passing arguments to [MSYS2/Cygwin][1] based executables: these programs will expand unquoted arguments containing wildcard characters (`?` and `*`) by searching for any file paths matching the wildcard pattern. <br>这对于将参数传递给基于 [MSYS2/Cygwin][1] 的可执行文件很有用：这些程序将通过搜索与通配符模式匹配的任何文件路径来扩展包含通配符 (`?` 和 `*`) 的不带引号的参数。<br>
    ///
    ///
    /// Adding quotes has no effect when passing arguments to programs that use [msvcrt][2]. <br>将参数传递给使用 [msvcrt][2] 的程序时，添加引号无效。<br> This includes programs built with both MinGW and MSVC. <br>这包括使用 MinGW 和 MSVC 构建的程序。<br>
    ///
    /// [1]: <https://github.com/msys2/MSYS2-packages/issues/2176>
    /// [2]: <https://msdn.microsoft.com/en-us/library/17w5ykft.aspx>
    ///
    ///
    ///
    ///
    #[unstable(feature = "windows_process_extensions_force_quotes", issue = "82227")]
    fn force_quotes(&mut self, enabled: bool) -> &mut process::Command;

    /// Append literal text to the command line without any quoting or escaping. <br>将字面量文本追加到命令行，无需任何引用或转义。<br>
    ///
    /// This is useful for passing arguments to `cmd.exe /c`, which doesn't follow `CommandLineToArgvW` escaping rules. <br>这对于将参数传递给 `cmd.exe /c` 很有用，它不遵循 `CommandLineToArgvW` 转义规则。<br>
    ///
    #[stable(feature = "windows_process_extensions_raw_arg", since = "1.62.0")]
    fn raw_arg<S: AsRef<OsStr>>(&mut self, text_to_append_as_is: S) -> &mut process::Command;

    /// When [`process::Command`] creates pipes, request that our side is always async. <br>当 [`process::Command`] 创建管道时，请求我们这边总是异步的。<br>
    ///
    /// By default [`process::Command`] may choose to use pipes where both ends are opened for synchronous read or write operations. <br>默认情况下，[`process::Command`] 可以选择使用两端都打开的管道进行同步读取或写入操作。<br>
    /// By using `async_pipes(true)`, this behavior is overridden so that our side is always async. <br>通过使用 `async_pipes(true)`，这种行为被覆盖，所以我们这边总是异步的。<br>
    ///
    /// This is important because if doing async I/O a pipe or a file has to be opened for async access. <br>这很重要，因为如果执行异步 I/O，则必须打开管道或文件以进行异步访问。<br>
    ///
    /// The end of the pipe sent to the child process will always be synchronous regardless of this option. <br>不管这个选项如何，发送到子进程的管道末端总是同步的。<br>
    ///
    /// # Example
    ///
    /// ```
    /// #![feature(windows_process_extensions_async_pipes)]
    /// use std::os::windows::process::CommandExt;
    /// use std::process::{Command, Stdio};
    ///
    /// # let program = "";
    ///
    /// Command::new(program)
    ///     .async_pipes(true)
    ///     .stdin(Stdio::piped())
    ///     .stdout(Stdio::piped())
    ///     .stderr(Stdio::piped());
    /// ```
    ///
    ///
    ///
    ///
    #[unstable(feature = "windows_process_extensions_async_pipes", issue = "98289")]
    fn async_pipes(&mut self, always_async: bool) -> &mut process::Command;
}

#[stable(feature = "windows_process_extensions", since = "1.16.0")]
impl CommandExt for process::Command {
    fn creation_flags(&mut self, flags: u32) -> &mut process::Command {
        self.as_inner_mut().creation_flags(flags);
        self
    }

    fn force_quotes(&mut self, enabled: bool) -> &mut process::Command {
        self.as_inner_mut().force_quotes(enabled);
        self
    }

    fn raw_arg<S: AsRef<OsStr>>(&mut self, raw_text: S) -> &mut process::Command {
        self.as_inner_mut().raw_arg(raw_text.as_ref());
        self
    }

    fn async_pipes(&mut self, always_async: bool) -> &mut process::Command {
        // FIXME: This currently has an intentional no-op implementation. <br>这目前有一个有意的无操作实现。<br>
        // For the time being our side of the pipes will always be async. <br>目前，我们的管道将始终是异步的。<br>
        // Once the ecosystem has adjusted, we may then be able to start making use of synchronous pipes within the standard library. <br>一旦生态系统调整完毕，我们就可以开始使用标准库中的同步管道。<br>
        //
        let _ = always_async;
        self
    }
}

#[unstable(feature = "windows_process_extensions_main_thread_handle", issue = "96723")]
pub trait ChildExt: Sealed {
    /// Extracts the main thread raw handle, without taking ownership <br>提取主线程原始句柄，不获取所有权<br>
    #[unstable(feature = "windows_process_extensions_main_thread_handle", issue = "96723")]
    fn main_thread_handle(&self) -> BorrowedHandle<'_>;
}

#[unstable(feature = "windows_process_extensions_main_thread_handle", issue = "96723")]
impl ChildExt for process::Child {
    fn main_thread_handle(&self) -> BorrowedHandle<'_> {
        self.handle.main_thread_handle()
    }
}
