//! Panic support in the standard library. <br>标准库中的 Panic 支持。<br>

#![stable(feature = "std_panic", since = "1.9.0")]

use crate::any::Any;
use crate::collections;
use crate::panicking;
use crate::sync::atomic::{AtomicUsize, Ordering};
use crate::sync::{Mutex, RwLock};
use crate::thread::Result;

#[doc(hidden)]
#[unstable(feature = "edition_panic", issue = "none", reason = "use panic!() instead")]
#[allow_internal_unstable(libstd_sys_internals, const_format_args, core_panic)]
#[cfg_attr(not(test), rustc_diagnostic_item = "std_panic_2015_macro")]
#[rustc_macro_transparency = "semitransparent"]
pub macro panic_2015 {
    () => ({
        $crate::rt::begin_panic("explicit panic")
    }),
    ($msg:expr $(,)?) => ({
        $crate::rt::begin_panic($msg)
    }),
    // Special-case the single-argument case for const_panic. <br>特殊情况下 const_panic 的单个参数情况。<br>
    ("{}", $arg:expr $(,)?) => ({
        $crate::rt::panic_display(&$arg)
    }),
    ($fmt:expr, $($arg:tt)+) => ({
        $crate::rt::panic_fmt($crate::const_format_args!($fmt, $($arg)+))
    }),
}

#[doc(hidden)]
#[unstable(feature = "edition_panic", issue = "none", reason = "use panic!() instead")]
pub use core::panic::panic_2021;

#[stable(feature = "panic_hooks", since = "1.10.0")]
pub use crate::panicking::{set_hook, take_hook};

#[unstable(feature = "panic_update_hook", issue = "92649")]
pub use crate::panicking::update_hook;

#[stable(feature = "panic_hooks", since = "1.10.0")]
pub use core::panic::{Location, PanicInfo};

#[stable(feature = "catch_unwind", since = "1.9.0")]
pub use core::panic::{AssertUnwindSafe, RefUnwindSafe, UnwindSafe};

/// Panic the current thread with the given message as the panic payload. <br>使用给定的消息作为 panic 有效载荷，使当前线程 panic。<br>
///
/// The message can be of any (`Any + Send`) type, not just strings. <br>该消息可以是任何 (`Any + Send`) 类型，而不仅仅是字符串。<br>
///
/// The message is wrapped in a `Box<'static + Any + Send>`, which can be accessed later using [`PanicInfo::payload`]. <br>该消息包装在一个 `Box<'static + Any + Send>` 中，以后可以使用 [`PanicInfo::payload`] 访问它。<br>
///
///
/// See the [`panic!`] macro for more information about panicking. <br>有关 panic 的更多信息，请参见 [`panic!`] 宏。<br>
#[stable(feature = "panic_any", since = "1.51.0")]
#[inline]
#[track_caller]
pub fn panic_any<M: 'static + Any + Send>(msg: M) -> ! {
    crate::panicking::begin_panic(msg);
}

#[stable(feature = "catch_unwind", since = "1.9.0")]
impl<T: ?Sized> UnwindSafe for Mutex<T> {}
#[stable(feature = "catch_unwind", since = "1.9.0")]
impl<T: ?Sized> UnwindSafe for RwLock<T> {}

#[stable(feature = "unwind_safe_lock_refs", since = "1.12.0")]
impl<T: ?Sized> RefUnwindSafe for Mutex<T> {}
#[stable(feature = "unwind_safe_lock_refs", since = "1.12.0")]
impl<T: ?Sized> RefUnwindSafe for RwLock<T> {}

// https://github.com/rust-lang/rust/issues/62301
#[stable(feature = "hashbrown", since = "1.36.0")]
impl<K, V, S> UnwindSafe for collections::HashMap<K, V, S>
where
    K: UnwindSafe,
    V: UnwindSafe,
    S: UnwindSafe,
{
}

/// Invokes a closure, capturing the cause of an unwinding panic if one occurs. <br>调用一个闭包，如果发生，则捕获展开 panic 的原因。<br>
///
/// This function will return `Ok` with the closure's result if the closure does not panic, and will return `Err(cause)` if the closure panics. <br>如果闭包不是 panic，则此函数将返回 `Ok`，并返回闭包的结果; 如果闭包 panics，则此函数将返回 `Err(cause)`。<br>
/// The `cause` returned is the object with which panic was originally invoked. <br>返回的 `cause` 是最初调用 panic 的对象。<br>
///
/// It is currently undefined behavior to unwind from Rust code into foreign code, so this function is particularly useful when Rust is called from another language (normally C). <br>unwind 当前从 Rust 代码转换为外来代码是未定义的行为，因此当从另一种语言 (通常为 C) 调用 Rust 时，此函数特别有用。<br>
/// This can run arbitrary Rust code, capturing a panic and allowing a graceful handling of the error. <br>这可以运行任意的 Rust 代码，捕获 panic 并允许对错误进行适当的处理。<br>
///
/// It is **not** recommended to use this function for a general try/catch mechanism. <br>不建议将此函数用于一般的 try/catch 机制。<br> The [`Result`] type is more appropriate to use for functions that can fail on a regular basis. <br>[`Result`] 类型更适合用于经常失败的函数。<br>
/// Additionally, this function is not guaranteed to catch all panics, see the "Notes" section below. <br>此外，不能保证此函数可以捕获所有 panics，请参见下面的 "Notes" 部分。<br>
///
/// The closure provided is required to adhere to the [`UnwindSafe`] trait to ensure that all captured variables are safe to cross this boundary. <br>提供的闭包必须遵守 [`UnwindSafe`] trait，以确保所有捕获的变量都可以安全越过此边界。<br>
/// The purpose of this bound is to encode the concept of [exception safety][rfc] in the type system. <br>此绑定的目的是在类型系统中对 [异常安全][rfc] 的概念进行编码。<br>
/// Most usage of this function should not need to worry about this bound as programs are naturally unwind safe without `unsafe` code. <br>此函数的大多数用法都不必担心此绑定，因为没有 `unsafe` 代码的程序自然是 unwind 安全的。<br>
/// If it becomes a problem the [`AssertUnwindSafe`] wrapper struct can be used to quickly assert that the usage here is indeed unwind safe. <br>如果出现问题，可以使用 [`AssertUnwindSafe`] 包装器结构体快速断言此处的使用确实是 unwind 安全的。<br>
///
/// [rfc]: https://github.com/rust-lang/rfcs/blob/master/text/1236-stabilize-catch-panic.md
///
/// # Notes
///
/// Note that this function **might not catch all panics** in Rust. <br>请注意，此函数**可能无法捕获 Rust 中的所有 panics**。<br> A panic in Rust is not always implemented via unwinding, but can be implemented by aborting the process as well. <br>Rust 中的 panic 并不总是通过展开来实现，但是也可以通过中止进程来实现。<br>
///
/// This function *only* catches unwinding panics, not those that abort the process. <br>该函数 *仅* 捕获展开 panics，而不是终止进程的 panics。<br>
///
/// Also note that unwinding into Rust code with a foreign exception (e.g. <br>另请注意，使用外部异常 (例如，展开为 Rust 代码)<br>
/// an exception thrown from C++ code) is undefined behavior. <br>从 C++ 代码引发的异常) 是未定义的行为。<br>
///
/// # Examples
///
/// ```
/// use std::panic;
///
/// let result = panic::catch_unwind(|| {
///     println!("hello!");
/// });
/// assert!(result.is_ok());
///
/// let result = panic::catch_unwind(|| {
///     panic!("oh no!");
/// });
/// assert!(result.is_err());
/// ```
///
///
///
///
///
///
///
///
///
#[stable(feature = "catch_unwind", since = "1.9.0")]
pub fn catch_unwind<F: FnOnce() -> R + UnwindSafe, R>(f: F) -> Result<R> {
    unsafe { panicking::r#try(f) }
}

/// Triggers a panic without invoking the panic hook. <br>在不调用 panic 钩子的情况下触发 panic。<br>
///
/// This is designed to be used in conjunction with [`catch_unwind`] to, for example, carry a panic across a layer of C code. <br>它被设计为与 [`catch_unwind`] 结合使用，例如，跨 C 代码层携带 panic。<br>
///
/// # Notes
///
/// Note that panics in Rust are not always implemented via unwinding, but they may be implemented by aborting the process. <br>请注意，Rust 中的 panics 并不总是通过展开来实现，但是可以通过中止进程来实现。<br>
/// If this function is called when panics are implemented this way then this function will abort the process, not trigger an unwind. <br>如果以这种方式实现 panics 时调用了此函数，则此函数将终止进程，而不触发 unwind。<br>
///
///
/// # Examples
///
/// ```should_panic
/// use std::panic;
///
/// let result = panic::catch_unwind(|| {
///     panic!("oh no!");
/// });
///
/// if let Err(err) = result {
///     panic::resume_unwind(err);
/// }
/// ```
///
///
#[stable(feature = "resume_unwind", since = "1.9.0")]
pub fn resume_unwind(payload: Box<dyn Any + Send>) -> ! {
    panicking::rust_panic_without_hook(payload)
}

/// Make all future panics abort directly without running the panic hook or unwinding. <br>使所有未来的 panic 直接中止，而不运行 panic 钩子或展开。<br>
///
/// There is no way to undo this; <br>没有办法撤销这个；<br> the effect lasts until the process exits or execs (or the equivalent). <br>效果持续到进程退出或执行 (或等效)。<br>
///
/// # Use after fork <br>fork 后使用<br>
///
/// This function is particularly useful for calling after `libc::fork`. <br>这个函数在 `libc::fork` 之后调用特别有用。<br> After `fork`, in a multithreaded program it is (on many platforms) not safe to call the allocator. <br>在 `fork` 之后，在多线程程序中 (在许多平台上) 调用分配器是不安全的。<br>
/// It is also generally highly undesirable for an unwind to unwind past the `fork`, because that results in the unwind propagating to code that was only ever expecting to run in the parent. <br>`fork` 之后的 unwind 到 unwind 通常也是非常不可取的，因为这会导致 unwind 传播到只希望在父级中运行的代码。<br>
///
///
/// `panic::always_abort()` helps avoid both of these. <br>`panic::always_abort()` 有助于避免这两种情况。<br>
/// It directly avoids any further unwinding, and if there is a panic, the abort will occur without allocating provided that the arguments to panic can be formatted without allocating. <br>它直接避免了任何进一步的展开，如果存在 panic，则无需分配即可终止终止，前提是 panic 的参数可以在不分配的情况下进行格式化。<br>
///
/// Examples
///
/// ```no_run
/// #![feature(panic_always_abort)]
/// use std::panic;
///
/// panic::always_abort();
///
/// let _ = panic::catch_unwind(|| {
///     panic!("inside the catch");
/// });
///
/// // We will have aborted already, due to the panic. <br>由于 panic，我们已经中止了。<br>
/// unreachable!();
/// ```
///
///
///
#[unstable(feature = "panic_always_abort", issue = "84438")]
pub fn always_abort() {
    crate::panicking::panic_count::set_always_abort();
}

/// The configuration for whether and how the default panic hook will capture and display the backtrace. <br>默认 panic 钩子，是否以及如何捕获和显示回溯的配置。<br>
///
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
#[unstable(feature = "panic_backtrace_config", issue = "93346")]
#[non_exhaustive]
pub enum BacktraceStyle {
    /// Prints a terser backtrace which ideally only contains relevant information. <br>打印一个简洁的回溯，理想情况下只包含相关信息。<br>
    ///
    Short,
    /// Prints a backtrace with all possible information. <br>打印包含所有可能信息的回溯。<br>
    Full,
    /// Disable collecting and displaying backtraces. <br>禁用收集和显示回溯。<br>
    Off,
}

impl BacktraceStyle {
    pub(crate) fn full() -> Option<Self> {
        if cfg!(feature = "backtrace") { Some(BacktraceStyle::Full) } else { None }
    }

    fn as_usize(self) -> usize {
        match self {
            BacktraceStyle::Short => 1,
            BacktraceStyle::Full => 2,
            BacktraceStyle::Off => 3,
        }
    }

    fn from_usize(s: usize) -> Option<Self> {
        Some(match s {
            0 => return None,
            1 => BacktraceStyle::Short,
            2 => BacktraceStyle::Full,
            3 => BacktraceStyle::Off,
            _ => unreachable!(),
        })
    }
}

// Tracks whether we should/can capture a backtrace, and how we should display that backtrace. <br>跟踪我们是否应该或能够捕获回溯，以及我们应该如何显示回溯。<br>
//
//
// Internally stores equivalent of an Option<BacktraceStyle>. <br>内部存储相当于 Option<BacktraceStyle>。<br>
static SHOULD_CAPTURE: AtomicUsize = AtomicUsize::new(0);

/// Configure whether the default panic hook will capture and display a backtrace. <br>配置默认的 panic 钩子，是否将捕获并显示回溯。<br>
///
/// The default value for this setting may be set by the `RUST_BACKTRACE` environment variable; <br>此设置的默认值可以由 `RUST_BACKTRACE` 环境变量设置;<br> see the details in [`get_backtrace_style`]. <br>请参见 [`get_backtrace_style`] 中的详细信息。<br>
///
///
#[unstable(feature = "panic_backtrace_config", issue = "93346")]
pub fn set_backtrace_style(style: BacktraceStyle) {
    if !cfg!(feature = "backtrace") {
        // If the `backtrace` feature of this crate isn't enabled, skip setting. <br>如果此 crate 的 `backtrace` 特性没有被启用，请跳过设置。<br>
        return;
    }
    SHOULD_CAPTURE.store(style.as_usize(), Ordering::Release);
}

/// Checks whether the standard library's panic hook will capture and print a backtrace. <br>检查标准库的 panic 钩子是否会捕获并打印回溯。<br>
///
/// This function will, if a backtrace style has not been set via [`set_backtrace_style`], read the environment variable `RUST_BACKTRACE` to determine a default value for the backtrace formatting: <br>如果没有通过 [`set_backtrace_style`] 设置回溯样式，此函数将读取环境变量 `RUST_BACKTRACE` 以确定回溯格式的默认值：<br>
///
///
/// The first call to `get_backtrace_style` may read the `RUST_BACKTRACE` environment variable if `set_backtrace_style` has not been called to override the default value. <br>如果没有调用 `set_backtrace_style` 来覆盖默认值，则对 `get_backtrace_style` 的第一个调用可能会读取 `RUST_BACKTRACE` 环境变量。<br>
/// After a call to `set_backtrace_style` or `get_backtrace_style`, any changes to `RUST_BACKTRACE` will have no effect. <br>调用 `set_backtrace_style` 或 `get_backtrace_style` 后，对 `RUST_BACKTRACE` 的任何更改都将无效。<br>
///
/// `RUST_BACKTRACE` is read according to these rules: <br>`RUST_BACKTRACE` 根据以下规则读取:<br>
///
/// * `0` for `BacktraceStyle::Off` <br>`0` 代表 `BacktraceStyle::Off`<br>
/// * `full` for `BacktraceStyle::Full` <br>`full` 代表 `BacktraceStyle::Full`<br>
/// * `1` for `BacktraceStyle::Short` <br>`1` 代表 `BacktraceStyle::Short`<br>
/// * Other values are currently `BacktraceStyle::Short`, but this may change in the future <br>其他值目前是 `BacktraceStyle::Short`，但这可能会在未来改变<br>
///
/// Returns `None` if backtraces aren't currently supported. <br>如果当前不支持回溯，则返回 `None`。<br>
///
///
///
///
///
#[unstable(feature = "panic_backtrace_config", issue = "93346")]
pub fn get_backtrace_style() -> Option<BacktraceStyle> {
    if !cfg!(feature = "backtrace") {
        // If the `backtrace` feature of this crate isn't enabled quickly return `Unsupported` so this can be constant propagated all over the place to optimize away callers. <br>如果未启用此 crate 的 `backtrace` 特性，请快速返回 `Unsupported`，这样就可以在整个地方不断传播，以优化离开的调用者。<br>
        //
        //
        return None;
    }
    if let Some(style) = BacktraceStyle::from_usize(SHOULD_CAPTURE.load(Ordering::Acquire)) {
        return Some(style);
    }

    // Setting environment variables for Fuchsia components isn't a standard or easily supported workflow. <br>为 Fuchsia 组件设置环境变量不是标准的或易于支持的工作流程。<br>
    // For now, display backtraces by default. <br>目前，默认情况下显示回溯。<br>
    let format = if cfg!(target_os = "fuchsia") {
        BacktraceStyle::Full
    } else {
        crate::env::var_os("RUST_BACKTRACE")
            .map(|x| {
                if &x == "0" {
                    BacktraceStyle::Off
                } else if &x == "full" {
                    BacktraceStyle::Full
                } else {
                    BacktraceStyle::Short
                }
            })
            .unwrap_or(BacktraceStyle::Off)
    };
    set_backtrace_style(format);
    Some(format)
}

#[cfg(test)]
mod tests;
