//! Cross-platform path manipulation. <br>跨平台路径操作。<br>
//!
//! This module provides two types, [`PathBuf`] and [`Path`] (akin to [`String`] and [`str`]), for working with paths abstractly. <br>该模块提供两种类型，即 [`PathBuf`] 和 [`Path`] (类似于 [`String`] 和 [`str`])，用于抽象地处理路径。<br>
//! These types are thin wrappers around [`OsString`] and [`OsStr`] respectively, meaning that they work directly on strings according to the local platform's path syntax. <br>这些类型分别是围绕 [`OsString`] 和 [`OsStr`] 的薄包装器，这意味着它们根据本地平台的路径语法直接在字符串上工作。<br>
//!
//! Paths can be parsed into [`Component`]s by iterating over the structure returned by the [`components`] method on [`Path`]. <br>通过遍历 [`Path`] 上 [`components`] 方法返回的结构体，可以将路径解析为 [`Component`]。<br> [`Component`]s roughly correspond to the substrings between path separators (`/` or `\`). <br>[`Component`] 大致对应于路径分隔符 (`/` 或 `\`) 之间的子字符串。<br>
//! You can reconstruct an equivalent path from components with the [`push`] method on [`PathBuf`]; <br>您可以使用 [`PathBuf`] 上的 [`push`] 方法从组件重建等效路径；<br> note that the paths may differ syntactically by the normalization described in the documentation for the [`components`] method. <br>请注意，根据 [`components`] 方法文档中描述的规范化，路径可能在语法上有所不同。<br>
//!
//!
//! ## Case sensitivity <br>区分大小写<br>
//!
//! Unless otherwise indicated path methods that do not access the filesystem, such as [`Path::starts_with`] and [`Path::ends_with`], are case sensitive no matter the platform or filesystem. <br>除非另有说明，否则不访问文件系统的路径方法，例如 [`Path::starts_with`] 和 [`Path::ends_with`]，无论平台或文件系统如何都区分大小写。<br>
//! An exception to this is made for Windows drive letters. <br>Windows 驱动器盘符例外。<br>
//!
//! ## Simple usage <br>使用简单<br>
//!
//! Path manipulation includes both parsing components from slices and building new owned paths. <br>路径操作既包括从切片中解析组件，也包括构建新的拥有的路径。<br>
//!
//! To parse a path, you can create a [`Path`] slice from a [`str`] slice and start asking questions: <br>要解析路径，您可以从 [`str`] 切片创建 [`Path`] 切片并开始提出问题：<br>
//!
//! ```
//! use std::path::Path;
//! use std::ffi::OsStr;
//!
//! let path = Path::new("/tmp/foo/bar.txt");
//!
//! let parent = path.parent();
//! assert_eq!(parent, Some(Path::new("/tmp/foo")));
//!
//! let file_stem = path.file_stem();
//! assert_eq!(file_stem, Some(OsStr::new("bar")));
//!
//! let extension = path.extension();
//! assert_eq!(extension, Some(OsStr::new("txt")));
//! ```
//!
//! To build or modify paths, use [`PathBuf`]: <br>要构建或修改路径，请使用 [`PathBuf`]：<br>
//!
//! ```
//! use std::path::PathBuf;
//!
//! // This way works... <br>这种方式有效...<br>
//! let mut path = PathBuf::from("c:\\");
//!
//! path.push("windows");
//! path.push("system32");
//!
//! path.set_extension("dll");
//!
//! // ... but push is best used if you don't know everything up front. <br>但是如果您不了解所有内容，则最好使用推送。<br>
//! // If you do, this way is better: <br>如果您这样做，则这种方法更好：<br>
//! let path: PathBuf = ["c:\\", "windows", "system32.dll"].iter().collect();
//! ```
//!
//! [`components`]: Path::components
//! [`push`]: PathBuf::push
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]
#![deny(unsafe_op_in_unsafe_fn)]

#[cfg(test)]
mod tests;

use crate::borrow::{Borrow, Cow};
use crate::cmp;
use crate::collections::TryReserveError;
use crate::error::Error;
use crate::fmt;
use crate::fs;
use crate::hash::{Hash, Hasher};
use crate::io;
use crate::iter::{self, FusedIterator};
use crate::ops::{self, Deref};
use crate::rc::Rc;
use crate::str::FromStr;
use crate::sync::Arc;

use crate::ffi::{OsStr, OsString};
use crate::sys;
use crate::sys::path::{is_sep_byte, is_verbatim_sep, parse_prefix, MAIN_SEP_STR};

////////////////////////////////////////////////////////////////////////////////
// GENERAL NOTES <br>一般注意事项<br>
////////////////////////////////////////////////////////////////////////////////
// Parsing in this module is done by directly transmuting OsStr to [u8] slices, taking advantage of the fact that OsStr always encodes ASCII characters as-is. <br>利用 OsStr 始终按原样编码 ASCII 字符这一事实，可以通过将 OsStr 直接转换为 [u8] slice 来完成此模块中的解析。<br>
// Eventually, this transmutation should be replaced by direct uses of OsStr APIs for parsing, but it will take a while for those to become available. <br>最终，应使用直接使用 OsStr API 进行解析来代替这种转换，但是要使这些转换可用需要一些时间。<br>
//
//
//
//

////////////////////////////////////////////////////////////////////////////////
// Windows Prefixes <br>Windows 前缀<br>
////////////////////////////////////////////////////////////////////////////////

/// Windows path prefixes, e.g., `C:` or `\\server\share`. <br>Windows 路径前缀，例如 `C:` 或 `\\server\share`。<br>
///
/// Windows uses a variety of path prefix styles, including references to drive volumes (like `C:`), network shared folders (like `\\server\share`), and others. <br>Windows 使用多种路径前缀样式，包括引用驱动卷 (如 `C:`)、网络共享文件夹 (如 `\\server\share`) 等。<br>
/// In addition, some path prefixes are "verbatim" (i.e., prefixed with `\\?\`), in which case `/` is *not* treated as a separator and essentially no normalization is performed. <br>另外，某些路径前缀是 "verbatim" (即以 `\\?\` 前缀)，在这种情况下，*不* 将 `/` 视为分隔符，并且基本上不执行规范化。<br>
///
///
/// # Examples
///
/// ```
/// use std::path::{Component, Path, Prefix};
/// use std::path::Prefix::*;
/// use std::ffi::OsStr;
///
/// fn get_path_prefix(s: &str) -> Prefix {
///     let path = Path::new(s);
///     match path.components().next().unwrap() {
///         Component::Prefix(prefix_component) => prefix_component.kind(),
///         _ => panic!(),
///     }
/// }
///
/// # if cfg!(windows) {
/// assert_eq!(Verbatim(OsStr::new("pictures")),
///            get_path_prefix(r"\\?\pictures\kittens"));
/// assert_eq!(VerbatimUNC(OsStr::new("server"), OsStr::new("share")),
///            get_path_prefix(r"\\?\UNC\server\share"));
/// assert_eq!(VerbatimDisk(b'C'), get_path_prefix(r"\\?\c:\"));
/// assert_eq!(DeviceNS(OsStr::new("BrainInterface")),
///            get_path_prefix(r"\\.\BrainInterface"));
/// assert_eq!(UNC(OsStr::new("server"), OsStr::new("share")),
///            get_path_prefix(r"\\server\share"));
/// assert_eq!(Disk(b'C'), get_path_prefix(r"C:\Users\Rust\Pictures\Ferris"));
/// # }
/// ```
///
///
#[derive(Copy, Clone, Debug, Hash, PartialOrd, Ord, PartialEq, Eq)]
#[stable(feature = "rust1", since = "1.0.0")]
pub enum Prefix<'a> {
    /// Verbatim prefix, e.g., `\\?\cat_pics`. <br>逐字前缀，例如，`\\?\cat_pics`。<br>
    ///
    /// Verbatim prefixes consist of `\\?\` immediately followed by the given component. <br>逐字前缀由 `\\?\` 组成，紧随其后是给定的组件。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    Verbatim(#[stable(feature = "rust1", since = "1.0.0")] &'a OsStr),

    /// Verbatim prefix using Windows' _**U**niform **N**aming **C**onvention_, e.g., `\\?\UNC\server\share`. <br>逐字前缀使用 Windows 的 _**U**niform **N**aming **C**onvention_，例如 `\\?\UNC\server\share`。<br>
    ///
    /// Verbatim UNC prefixes consist of `\\?\UNC\` immediately followed by the server's hostname and a share name. <br>Verbatim UNC 前缀由 `\\?\UNC\` 组成，其后紧跟服务器的主机名和共享名。<br>
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    VerbatimUNC(
        #[stable(feature = "rust1", since = "1.0.0")] &'a OsStr,
        #[stable(feature = "rust1", since = "1.0.0")] &'a OsStr,
    ),

    /// Verbatim disk prefix, e.g., `\\?\C:`. <br>逐字磁盘前缀，例如 `\\?\C:`。<br>
    ///
    /// Verbatim disk prefixes consist of `\\?\` immediately followed by the drive letter and `:`. <br>逐字磁盘前缀由 `\\?\` 紧随其后的驱动器号和 `:` 组成。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    VerbatimDisk(#[stable(feature = "rust1", since = "1.0.0")] u8),

    /// Device namespace prefix, e.g., `\\.\COM42`. <br>设备命名空间前缀，例如 `\\.\COM42`。<br>
    ///
    /// Device namespace prefixes consist of `\\.\` (possibly using `/` instead of `\`), immediately followed by the device name. <br>设备名称空间前缀由 `\\.\` (可能使用 `/` 而不是 `\`) 组成，后跟设备名称。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    DeviceNS(#[stable(feature = "rust1", since = "1.0.0")] &'a OsStr),

    /// Prefix using Windows' _**U**niform **N**aming **C**onvention_, e.g. <br>使用 Windows 的 _**U**niform **N**aming **C**onvention_ 的前缀，例如<br>
    /// `\\server\share`.
    ///
    /// UNC prefixes consist of the server's hostname and a share name. <br>UNC 前缀由服务器的主机名和共享名组成。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    UNC(
        #[stable(feature = "rust1", since = "1.0.0")] &'a OsStr,
        #[stable(feature = "rust1", since = "1.0.0")] &'a OsStr,
    ),

    /// Prefix `C:` for the given disk drive. <br>给定磁盘驱动器的前缀 `C:`。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    Disk(#[stable(feature = "rust1", since = "1.0.0")] u8),
}

impl<'a> Prefix<'a> {
    #[inline]
    fn len(&self) -> usize {
        use self::Prefix::*;
        fn os_str_len(s: &OsStr) -> usize {
            s.bytes().len()
        }
        match *self {
            Verbatim(x) => 4 + os_str_len(x),
            VerbatimUNC(x, y) => {
                8 + os_str_len(x) + if os_str_len(y) > 0 { 1 + os_str_len(y) } else { 0 }
            }
            VerbatimDisk(_) => 6,
            UNC(x, y) => 2 + os_str_len(x) + if os_str_len(y) > 0 { 1 + os_str_len(y) } else { 0 },
            DeviceNS(x) => 4 + os_str_len(x),
            Disk(_) => 2,
        }
    }

    /// Determines if the prefix is verbatim, i.e., begins with `\\?\`. <br>确定前缀是否为逐字形式，即以 `\\?\` 开头。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Prefix::*;
    /// use std::ffi::OsStr;
    ///
    /// assert!(Verbatim(OsStr::new("pictures")).is_verbatim());
    /// assert!(VerbatimUNC(OsStr::new("server"), OsStr::new("share")).is_verbatim());
    /// assert!(VerbatimDisk(b'C').is_verbatim());
    /// assert!(!DeviceNS(OsStr::new("BrainInterface")).is_verbatim());
    /// assert!(!UNC(OsStr::new("server"), OsStr::new("share")).is_verbatim());
    /// assert!(!Disk(b'C').is_verbatim());
    /// ```
    #[inline]
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn is_verbatim(&self) -> bool {
        use self::Prefix::*;
        matches!(*self, Verbatim(_) | VerbatimDisk(_) | VerbatimUNC(..))
    }

    #[inline]
    fn is_drive(&self) -> bool {
        matches!(*self, Prefix::Disk(_))
    }

    #[inline]
    fn has_implicit_root(&self) -> bool {
        !self.is_drive()
    }
}

////////////////////////////////////////////////////////////////////////////////
// Exposed parsing helpers <br>公开的解析帮助程序<br>
////////////////////////////////////////////////////////////////////////////////

/// Determines whether the character is one of the permitted path separators for the current platform. <br>确定字符是否为当前平台允许的路径分隔符之一。<br>
///
///
/// # Examples
///
/// ```
/// use std::path;
///
/// assert!(path::is_separator('/')); // '/' works for both Unix and Windows <br>'/' 适用于 Unix 和 Windows<br>
/// assert!(!path::is_separator('❤'));
/// ```
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn is_separator(c: char) -> bool {
    c.is_ascii() && is_sep_byte(c as u8)
}

/// The primary separator of path components for the current platform. <br>当前平台的路径组件的主要分隔符。<br>
///
/// For example, `/` on Unix and `\` on Windows. <br>例如，Unix 上的 `/` 和 Windows 上的 `\`。<br>
#[stable(feature = "rust1", since = "1.0.0")]
pub const MAIN_SEPARATOR: char = crate::sys::path::MAIN_SEP;

/// The primary separator of path components for the current platform. <br>当前平台的路径组件的主要分隔符。<br>
///
/// For example, `/` on Unix and `\` on Windows. <br>例如，Unix 上的 `/` 和 Windows 上的 `\`。<br>
#[unstable(feature = "main_separator_str", issue = "94071")]
pub const MAIN_SEPARATOR_STR: &str = crate::sys::path::MAIN_SEP_STR;

////////////////////////////////////////////////////////////////////////////////
// Misc helpers <br>杂项帮手<br>
////////////////////////////////////////////////////////////////////////////////

// Iterate through `iter` while it matches `prefix`; <br>与 `prefix` 匹配时遍历 `iter`；<br> return `None` if `prefix` is not a prefix of `iter`, otherwise return `Some(iter_after_prefix)` giving `iter` after having exhausted `prefix`. <br>如果 `prefix` 不是 `iter` 的前缀，则返回 `None`; 否则，在用完 `prefix` 后返回 `Some(iter_after_prefix)`，得到 `iter`。<br>
//
//
fn iter_after<'a, 'b, I, J>(mut iter: I, mut prefix: J) -> Option<I>
where
    I: Iterator<Item = Component<'a>> + Clone,
    J: Iterator<Item = Component<'b>>,
{
    loop {
        let mut iter_next = iter.clone();
        match (iter_next.next(), prefix.next()) {
            (Some(ref x), Some(ref y)) if x == y => (),
            (Some(_), Some(_)) => return None,
            (Some(_), None) => return Some(iter),
            (None, None) => return Some(iter),
            (None, Some(_)) => return None,
        }
        iter = iter_next;
    }
}

unsafe fn u8_slice_as_os_str(s: &[u8]) -> &OsStr {
    // SAFETY: See note at the top of this module to understand why this and `OsStr::bytes` are used: <br>请参见本模块顶部的注释以了解使用此和 `OsStr::bytes` 的原因:<br>
    //
    // This casts are safe as OsStr is internally a wrapper around [u8] on all platforms. <br>这种强制转换是安全的，因为 OsStr 在内部是所有平台上 [u8] 的包装器。<br>
    //
    // Note that currently this relies on the special knowledge that libstd has; <br>请注意，当前这依赖于 libstd 拥有的特殊知识。<br>
    // these types are single-element structs but are not marked repr(transparent) or repr(C) which would make these casts not allowable outside std. <br>这些类型是单一元素结构体，但没有标记为 repr(transparent) 或 repr(C)，这将使这些类型转换在 std 之外是不允许的。<br>
    //
    //
    //
    //
    unsafe { &*(s as *const [u8] as *const OsStr) }
}

// Detect scheme on Redox <br>`Redox` 检测方案<br>
fn has_redox_scheme(s: &[u8]) -> bool {
    cfg!(target_os = "redox") && s.contains(&b':')
}

////////////////////////////////////////////////////////////////////////////////
// Cross-platform, iterator-independent parsing <br>跨平台，独立于迭代器的解析<br>
////////////////////////////////////////////////////////////////////////////////

/// Says whether the first byte after the prefix is a separator. <br>说前缀之后的第一个字节是否为分隔符。<br>
fn has_physical_root(s: &[u8], prefix: Option<Prefix<'_>>) -> bool {
    let path = if let Some(p) = prefix { &s[p.len()..] } else { s };
    !path.is_empty() && is_sep_byte(path[0])
}

// basic workhorse for splitting stem and extension <br>分裂茎和伸展的基本力量<br>
fn rsplit_file_at_dot(file: &OsStr) -> (Option<&OsStr>, Option<&OsStr>) {
    if file.bytes() == b".." {
        return (Some(file), None);
    }

    // The unsafety here stems from converting between &OsStr and &[u8] and back. <br>这里的不安全性源于 &OsStr 和 &[u8] and 之间的转换。<br>
    // This is safe to do because (1) we only look at ASCII contents of the encoding and (2) new &OsStr values are produced only from ASCII-bounded slices of existing &OsStr values. <br>这样做是安全的，因为 (1) 我们仅查看编码的 ASCII 内容，而 (2) 新的 &OsStr 值仅从现有 &OsStr values 的 ASCII 限制切片中产生。<br>
    //
    //
    let mut iter = file.bytes().rsplitn(2, |b| *b == b'.');
    let after = iter.next();
    let before = iter.next();
    if before == Some(b"") {
        (Some(file), None)
    } else {
        unsafe { (before.map(|s| u8_slice_as_os_str(s)), after.map(|s| u8_slice_as_os_str(s))) }
    }
}

fn split_file_at_dot(file: &OsStr) -> (&OsStr, Option<&OsStr>) {
    let slice = file.bytes();
    if slice == b".." {
        return (file, None);
    }

    // The unsafety here stems from converting between &OsStr and &[u8] and back. <br>这里的不安全性源于 &OsStr 和 &[u8] and 之间的转换。<br>
    // This is safe to do because (1) we only look at ASCII contents of the encoding and (2) new &OsStr values are produced only from ASCII-bounded slices of existing &OsStr values. <br>这样做是安全的，因为 (1) 我们仅查看编码的 ASCII 内容，而 (2) 新的 &OsStr 值仅从现有 &OsStr values 的 ASCII 限制切片中产生。<br>
    //
    //
    let i = match slice[1..].iter().position(|b| *b == b'.') {
        Some(i) => i + 1,
        None => return (file, None),
    };
    let before = &slice[..i];
    let after = &slice[i + 1..];
    unsafe { (u8_slice_as_os_str(before), Some(u8_slice_as_os_str(after))) }
}

////////////////////////////////////////////////////////////////////////////////
// The core iterators <br>核心迭代器<br>
////////////////////////////////////////////////////////////////////////////////

/// Component parsing works by a double-ended state machine; <br>组件解析由双端状态机完成；<br> the cursors at the front and back of the path each keep track of what parts of the path have been consumed so far. <br>路径前面和后面的游标都记录了到目前为止路径的哪些部分被消耗了。<br>
///
///
/// Going front to back, a path is made up of a prefix, a starting directory component, and a body (of normal components) <br>从前到后，路径由前缀，起始目录组件和主体组成 (由常规组件组成)<br>
///
///
#[derive(Copy, Clone, PartialEq, PartialOrd, Debug)]
enum State {
    Prefix = 0,   // c:
    StartDir = 1, // / or . or nothing <br>或者。或者什么都没有<br>
    Body = 2,     // foo/bar/baz
    Done = 3,
}

/// A structure wrapping a Windows path prefix as well as its unparsed string representation. <br>包装 Windows 路径前缀及其未解析的字符串表示形式的结构体。<br>
///
/// In addition to the parsed [`Prefix`] information returned by [`kind`], `PrefixComponent` also holds the raw and unparsed [`OsStr`] slice, returned by [`as_os_str`]. <br>除了由 [`kind`] 返回的已解析 [`Prefix`] 信息外，`PrefixComponent` 还保存由 [`as_os_str`] 返回的原始和未解析的 [`OsStr`] 切片。<br>
///
///
/// Instances of this `struct` can be obtained by matching against the [`Prefix` variant] on [`Component`]. <br>可以通过与 [`Component`] 上的 [`Prefix` variant] 匹配来获得此 `struct` 的实例。<br>
///
/// Does not occur on Unix. <br>在 Unix 上不会发生。<br>
///
/// # Examples
///
/// ```
/// # if cfg!(windows) {
/// use std::path::{Component, Path, Prefix};
/// use std::ffi::OsStr;
///
/// let path = Path::new(r"c:\you\later\");
/// match path.components().next().unwrap() {
///     Component::Prefix(prefix_component) => {
///         assert_eq!(Prefix::Disk(b'C'), prefix_component.kind());
///         assert_eq!(OsStr::new("c:"), prefix_component.as_os_str());
///     }
///     _ => unreachable!(),
/// }
/// # }
/// ```
///
/// [`as_os_str`]: PrefixComponent::as_os_str
/// [`kind`]: PrefixComponent::kind
/// [`Prefix` variant]: Component::Prefix
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Copy, Clone, Eq, Debug)]
pub struct PrefixComponent<'a> {
    /// The prefix as an unparsed `OsStr` slice. <br>前缀为未解析的 `OsStr` 切片。<br>
    raw: &'a OsStr,

    /// The parsed prefix data. <br>解析的前缀数据。<br>
    parsed: Prefix<'a>,
}

impl<'a> PrefixComponent<'a> {
    /// Returns the parsed prefix data. <br>返回已解析的前缀数据。<br>
    ///
    /// See [`Prefix`]'s documentation for more information on the different kinds of prefixes. <br>有关不同种类的前缀的更多信息，请参见 [`Prefix`] 的文档。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn kind(&self) -> Prefix<'a> {
        self.parsed
    }

    /// Returns the raw [`OsStr`] slice for this prefix. <br>返回此前缀的原始 [`OsStr`] 切片。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn as_os_str(&self) -> &'a OsStr {
        self.raw
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> cmp::PartialEq for PrefixComponent<'a> {
    #[inline]
    fn eq(&self, other: &PrefixComponent<'a>) -> bool {
        cmp::PartialEq::eq(&self.parsed, &other.parsed)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> cmp::PartialOrd for PrefixComponent<'a> {
    #[inline]
    fn partial_cmp(&self, other: &PrefixComponent<'a>) -> Option<cmp::Ordering> {
        cmp::PartialOrd::partial_cmp(&self.parsed, &other.parsed)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::Ord for PrefixComponent<'_> {
    #[inline]
    fn cmp(&self, other: &Self) -> cmp::Ordering {
        cmp::Ord::cmp(&self.parsed, &other.parsed)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Hash for PrefixComponent<'_> {
    fn hash<H: Hasher>(&self, h: &mut H) {
        self.parsed.hash(h);
    }
}

/// A single component of a path. <br>路径的单个组成部分。<br>
///
/// A `Component` roughly corresponds to a substring between path separators (`/` or `\`). <br>`Component` 大致对应于路径分隔符 (`/` 或 `\`) 之间的子字符串。<br>
///
/// This `enum` is created by iterating over [`Components`], which in turn is created by the [`components`](Path::components) method on [`Path`]. <br>该 `enum` 是通过迭代 [`Components`] 来创建的，而 [`Components`] 又是通过 [`Path`] 上的 [`components`](Path::components) 方法创建的。<br>
///
///
/// # Examples
///
/// ```rust
/// use std::path::{Component, Path};
///
/// let path = Path::new("/tmp/foo/bar.txt");
/// let components = path.components().collect::<Vec<_>>();
/// assert_eq!(&components, &[
///     Component::RootDir,
///     Component::Normal("tmp".as_ref()),
///     Component::Normal("foo".as_ref()),
///     Component::Normal("bar.txt".as_ref()),
/// ]);
/// ```
///
#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub enum Component<'a> {
    /// A Windows path prefix, e.g., `C:` or `\\server\share`. <br>Windows 路径前缀，例如 `C:` 或 `\\server\share`。<br>
    ///
    /// There is a large variety of prefix types, see [`Prefix`]'s documentation for more. <br>前缀类型种类繁多，有关更多信息，请参见 [`Prefix`] 的文档。<br>
    ///
    ///
    /// Does not occur on Unix. <br>在 Unix 上不会发生。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    Prefix(#[stable(feature = "rust1", since = "1.0.0")] PrefixComponent<'a>),

    /// The root directory component, appears after any prefix and before anything else. <br>根目录组件出现在任何前缀之后和其他任何内容之前。<br>
    ///
    /// It represents a separator that designates that a path starts from root. <br>它代表一个分隔符，它指定路径从根开始。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    RootDir,

    /// A reference to the current directory, i.e., `.`. <br>对当前目录的引用，即 `.`。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    CurDir,

    /// A reference to the parent directory, i.e., `..`. <br>对父目录的引用，即 `..`。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    ParentDir,

    /// A normal component, e.g., `a` and `b` in `a/b`. <br>正常组件，例如 `a/b` 中的 `a` 和 `b`。<br>
    ///
    /// This variant is the most common one, it represents references to files or directories. <br>这种变体是最常见的一种，它代表对文件或目录的引用。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    Normal(#[stable(feature = "rust1", since = "1.0.0")] &'a OsStr),
}

impl<'a> Component<'a> {
    /// Extracts the underlying [`OsStr`] slice. <br>提取底层的 [`OsStr`] 切片。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let path = Path::new("./tmp/foo/bar.txt");
    /// let components: Vec<_> = path.components().map(|comp| comp.as_os_str()).collect();
    /// assert_eq!(&components, &[".", "tmp", "foo", "bar.txt"]);
    /// ```
    #[must_use = "`self` will be dropped if the result is not used"]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn as_os_str(self) -> &'a OsStr {
        match self {
            Component::Prefix(p) => p.as_os_str(),
            Component::RootDir => OsStr::new(MAIN_SEP_STR),
            Component::CurDir => OsStr::new("."),
            Component::ParentDir => OsStr::new(".."),
            Component::Normal(path) => path,
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<OsStr> for Component<'_> {
    #[inline]
    fn as_ref(&self) -> &OsStr {
        self.as_os_str()
    }
}

#[stable(feature = "path_component_asref", since = "1.25.0")]
impl AsRef<Path> for Component<'_> {
    #[inline]
    fn as_ref(&self) -> &Path {
        self.as_os_str().as_ref()
    }
}

/// An iterator over the [`Component`]s of a [`Path`]. <br>[`Path`] 的 [`Component`] 上的迭代器。<br>
///
/// This `struct` is created by the [`components`] method on [`Path`]. <br>该 `struct` 是通过 [`Path`] 上的 [`components`] 方法创建的。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// # Examples
///
/// ```
/// use std::path::Path;
///
/// let path = Path::new("/tmp/foo/bar.txt");
///
/// for component in path.components() {
///     println!("{component:?}");
/// }
/// ```
///
/// [`components`]: Path::components
#[derive(Clone)]
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Components<'a> {
    // The path left to parse components from <br>解析组件所剩下的路径<br>
    path: &'a [u8],

    // The prefix as it was originally parsed, if any <br>最初解析的前缀 (如果有)<br>
    prefix: Option<Prefix<'a>>,

    // true if path *physically* has a root separator; <br>如果 path *物理* 具有根分隔符，则为 true; 否则为 true。<br> for most Windows prefixes, it may have a "logical" root separator for the purposes of normalization, e.g.,  \\server\share == \\server\share\. <br>对于大多数 Windows 前缀，为了规范化的目的，它可能有一个 "logical" 根分隔符，例如，\\server\share == \\server\share\。<br>
    //
    //
    has_physical_root: bool,

    // The iterator is double-ended, and these two states keep track of what has been produced from either end <br>迭代器是双端的，这两个状态跟踪从两端产生的结果<br>
    //
    front: State,
    back: State,
}

/// An iterator over the [`Component`]s of a [`Path`], as [`OsStr`] slices. <br>[`Path`] 的 [`Component`] 上的迭代器，作为 [`OsStr`] 切片。<br>
///
/// This `struct` is created by the [`iter`] method on [`Path`]. <br>该 `struct` 是通过 [`Path`] 上的 [`iter`] 方法创建的。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// [`iter`]: Path::iter
#[derive(Clone)]
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Iter<'a> {
    inner: Components<'a>,
}

#[stable(feature = "path_components_debug", since = "1.13.0")]
impl fmt::Debug for Components<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        struct DebugHelper<'a>(&'a Path);

        impl fmt::Debug for DebugHelper<'_> {
            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
                f.debug_list().entries(self.0.components()).finish()
            }
        }

        f.debug_tuple("Components").field(&DebugHelper(self.as_path())).finish()
    }
}

impl<'a> Components<'a> {
    // how long is the prefix, if any? <br>前缀多长时间 (如果有) ?<br>
    #[inline]
    fn prefix_len(&self) -> usize {
        self.prefix.as_ref().map(Prefix::len).unwrap_or(0)
    }

    #[inline]
    fn prefix_verbatim(&self) -> bool {
        self.prefix.as_ref().map(Prefix::is_verbatim).unwrap_or(false)
    }

    /// how much of the prefix is left from the point of view of iteration? <br>从迭代的角度来看，还剩下多少前缀？<br>
    #[inline]
    fn prefix_remaining(&self) -> usize {
        if self.front == State::Prefix { self.prefix_len() } else { 0 }
    }

    // Given the iteration so far, how much of the pre-State::Body path is left? <br>给定到目前为止的迭代，还剩下多少 pre-State::Body 路径？<br>
    #[inline]
    fn len_before_body(&self) -> usize {
        let root = if self.front <= State::StartDir && self.has_physical_root { 1 } else { 0 };
        let cur_dir = if self.front <= State::StartDir && self.include_cur_dir() { 1 } else { 0 };
        self.prefix_remaining() + root + cur_dir
    }

    // is the iteration complete? <br>迭代完成了吗？<br>
    #[inline]
    fn finished(&self) -> bool {
        self.front == State::Done || self.back == State::Done || self.front > self.back
    }

    #[inline]
    fn is_sep_byte(&self, b: u8) -> bool {
        if self.prefix_verbatim() { is_verbatim_sep(b) } else { is_sep_byte(b) }
    }

    /// Extracts a slice corresponding to the portion of the path remaining for iteration. <br>提取与迭代剩余路径部分相对应的切片。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let mut components = Path::new("/tmp/foo/bar.txt").components();
    /// components.next();
    /// components.next();
    ///
    /// assert_eq!(Path::new("foo/bar.txt"), components.as_path());
    /// ```
    #[must_use]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn as_path(&self) -> &'a Path {
        let mut comps = self.clone();
        if comps.front == State::Body {
            comps.trim_left();
        }
        if comps.back == State::Body {
            comps.trim_right();
        }
        unsafe { Path::from_u8_slice(comps.path) }
    }

    /// Is the *original* path rooted? <br>*原始* 路径是否有根？<br>
    fn has_root(&self) -> bool {
        if self.has_physical_root {
            return true;
        }
        if let Some(p) = self.prefix {
            if p.has_implicit_root() {
                return true;
            }
        }
        false
    }

    /// Should the normalized path include a leading . ? <br>规范化路径是否应包含前导。?<br>
    fn include_cur_dir(&self) -> bool {
        if self.has_root() {
            return false;
        }
        let mut iter = self.path[self.prefix_remaining()..].iter();
        match (iter.next(), iter.next()) {
            (Some(&b'.'), None) => true,
            (Some(&b'.'), Some(&b)) => self.is_sep_byte(b),
            _ => false,
        }
    }

    // parse a given byte sequence into the corresponding path component <br>将给定的字节序列解析为相应的路径组件<br>
    fn parse_single_component<'b>(&self, comp: &'b [u8]) -> Option<Component<'b>> {
        match comp {
            b"." if self.prefix_verbatim() => Some(Component::CurDir),
            b"." => None, // . components are normalized away, except at <br>组件被标准化后，除了在<br>
            // the beginning of a path, which is treated separately via `include_cur_dir` <br>路径的开始，通过 `include_cur_dir` 进行单独处理<br>
            //
            b".." => Some(Component::ParentDir),
            b"" => None,
            _ => Some(Component::Normal(unsafe { u8_slice_as_os_str(comp) })),
        }
    }

    // parse a component from the left, saying how many bytes to consume to remove the component <br>从左侧解析一个组件，说出删除该组件要消耗多少字节<br>
    //
    fn parse_next_component(&self) -> (usize, Option<Component<'a>>) {
        debug_assert!(self.front == State::Body);
        let (extra, comp) = match self.path.iter().position(|b| self.is_sep_byte(*b)) {
            None => (0, self.path),
            Some(i) => (1, &self.path[..i]),
        };
        (comp.len() + extra, self.parse_single_component(comp))
    }

    // parse a component from the right, saying how many bytes to consume to remove the component <br>从右边解析一个组件，说出删除该组件要消耗多少字节<br>
    //
    fn parse_next_component_back(&self) -> (usize, Option<Component<'a>>) {
        debug_assert!(self.back == State::Body);
        let start = self.len_before_body();
        let (extra, comp) = match self.path[start..].iter().rposition(|b| self.is_sep_byte(*b)) {
            None => (0, &self.path[start..]),
            Some(i) => (1, &self.path[start + i + 1..]),
        };
        (comp.len() + extra, self.parse_single_component(comp))
    }

    // trim away repeated separators (i.e., empty components) on the left <br>在左侧修剪掉重复的分隔符 (即空组件)<br>
    fn trim_left(&mut self) {
        while !self.path.is_empty() {
            let (size, comp) = self.parse_next_component();
            if comp.is_some() {
                return;
            } else {
                self.path = &self.path[size..];
            }
        }
    }

    // trim away repeated separators (i.e., empty components) on the right <br>在右侧修剪掉重复的分隔符 (即空组件)<br>
    fn trim_right(&mut self) {
        while self.path.len() > self.len_before_body() {
            let (size, comp) = self.parse_next_component_back();
            if comp.is_some() {
                return;
            } else {
                self.path = &self.path[..self.path.len() - size];
            }
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<Path> for Components<'_> {
    #[inline]
    fn as_ref(&self) -> &Path {
        self.as_path()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<OsStr> for Components<'_> {
    #[inline]
    fn as_ref(&self) -> &OsStr {
        self.as_path().as_os_str()
    }
}

#[stable(feature = "path_iter_debug", since = "1.13.0")]
impl fmt::Debug for Iter<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        struct DebugHelper<'a>(&'a Path);

        impl fmt::Debug for DebugHelper<'_> {
            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
                f.debug_list().entries(self.0.iter()).finish()
            }
        }

        f.debug_tuple("Iter").field(&DebugHelper(self.as_path())).finish()
    }
}

impl<'a> Iter<'a> {
    /// Extracts a slice corresponding to the portion of the path remaining for iteration. <br>提取与迭代剩余路径部分相对应的切片。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let mut iter = Path::new("/tmp/foo/bar.txt").iter();
    /// iter.next();
    /// iter.next();
    ///
    /// assert_eq!(Path::new("foo/bar.txt"), iter.as_path());
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn as_path(&self) -> &'a Path {
        self.inner.as_path()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<Path> for Iter<'_> {
    #[inline]
    fn as_ref(&self) -> &Path {
        self.as_path()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<OsStr> for Iter<'_> {
    #[inline]
    fn as_ref(&self) -> &OsStr {
        self.as_path().as_os_str()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> Iterator for Iter<'a> {
    type Item = &'a OsStr;

    #[inline]
    fn next(&mut self) -> Option<&'a OsStr> {
        self.inner.next().map(Component::as_os_str)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> DoubleEndedIterator for Iter<'a> {
    #[inline]
    fn next_back(&mut self) -> Option<&'a OsStr> {
        self.inner.next_back().map(Component::as_os_str)
    }
}

#[stable(feature = "fused", since = "1.26.0")]
impl FusedIterator for Iter<'_> {}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> Iterator for Components<'a> {
    type Item = Component<'a>;

    fn next(&mut self) -> Option<Component<'a>> {
        while !self.finished() {
            match self.front {
                State::Prefix if self.prefix_len() > 0 => {
                    self.front = State::StartDir;
                    debug_assert!(self.prefix_len() <= self.path.len());
                    let raw = &self.path[..self.prefix_len()];
                    self.path = &self.path[self.prefix_len()..];
                    return Some(Component::Prefix(PrefixComponent {
                        raw: unsafe { u8_slice_as_os_str(raw) },
                        parsed: self.prefix.unwrap(),
                    }));
                }
                State::Prefix => {
                    self.front = State::StartDir;
                }
                State::StartDir => {
                    self.front = State::Body;
                    if self.has_physical_root {
                        debug_assert!(!self.path.is_empty());
                        self.path = &self.path[1..];
                        return Some(Component::RootDir);
                    } else if let Some(p) = self.prefix {
                        if p.has_implicit_root() && !p.is_verbatim() {
                            return Some(Component::RootDir);
                        }
                    } else if self.include_cur_dir() {
                        debug_assert!(!self.path.is_empty());
                        self.path = &self.path[1..];
                        return Some(Component::CurDir);
                    }
                }
                State::Body if !self.path.is_empty() => {
                    let (size, comp) = self.parse_next_component();
                    self.path = &self.path[size..];
                    if comp.is_some() {
                        return comp;
                    }
                }
                State::Body => {
                    self.front = State::Done;
                }
                State::Done => unreachable!(),
            }
        }
        None
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> DoubleEndedIterator for Components<'a> {
    fn next_back(&mut self) -> Option<Component<'a>> {
        while !self.finished() {
            match self.back {
                State::Body if self.path.len() > self.len_before_body() => {
                    let (size, comp) = self.parse_next_component_back();
                    self.path = &self.path[..self.path.len() - size];
                    if comp.is_some() {
                        return comp;
                    }
                }
                State::Body => {
                    self.back = State::StartDir;
                }
                State::StartDir => {
                    self.back = State::Prefix;
                    if self.has_physical_root {
                        self.path = &self.path[..self.path.len() - 1];
                        return Some(Component::RootDir);
                    } else if let Some(p) = self.prefix {
                        if p.has_implicit_root() && !p.is_verbatim() {
                            return Some(Component::RootDir);
                        }
                    } else if self.include_cur_dir() {
                        self.path = &self.path[..self.path.len() - 1];
                        return Some(Component::CurDir);
                    }
                }
                State::Prefix if self.prefix_len() > 0 => {
                    self.back = State::Done;
                    return Some(Component::Prefix(PrefixComponent {
                        raw: unsafe { u8_slice_as_os_str(self.path) },
                        parsed: self.prefix.unwrap(),
                    }));
                }
                State::Prefix => {
                    self.back = State::Done;
                    return None;
                }
                State::Done => unreachable!(),
            }
        }
        None
    }
}

#[stable(feature = "fused", since = "1.26.0")]
impl FusedIterator for Components<'_> {}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> cmp::PartialEq for Components<'a> {
    #[inline]
    fn eq(&self, other: &Components<'a>) -> bool {
        let Components { path: _, front: _, back: _, has_physical_root: _, prefix: _ } = self;

        // Fast path for exact matches, e.g. for hashmap lookups. <br>精确匹配的快速路径，例如 hashmap 查找。<br>
        // Don't explicitly compare the prefix or has_physical_root fields since they'll either be covered by the `path` buffer or are only relevant for `prefix_verbatim()`. <br>不要明确比较前缀或 has_physical_root 字段，因为它们要么被 `path` 缓冲区覆盖，要么只与 `prefix_verbatim()` 相关。<br>
        //
        if self.path.len() == other.path.len()
            && self.front == other.front
            && self.back == State::Body
            && other.back == State::Body
            && self.prefix_verbatim() == other.prefix_verbatim()
        {
            // possible future improvement: this could bail out earlier if there were a reverse memcmp/bcmp comparing back to front <br>未来可能的改进：如果有反向 memcmp/bcmp 比较前后对比，这可以更早地摆脱<br>
            //
            if self.path == other.path {
                return true;
            }
        }

        // compare back to front since absolute paths often share long prefixes <br>前后比较，因为绝对路径通常共享长前缀<br>
        Iterator::eq(self.clone().rev(), other.clone().rev())
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::Eq for Components<'_> {}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a> cmp::PartialOrd for Components<'a> {
    #[inline]
    fn partial_cmp(&self, other: &Components<'a>) -> Option<cmp::Ordering> {
        Some(compare_components(self.clone(), other.clone()))
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::Ord for Components<'_> {
    #[inline]
    fn cmp(&self, other: &Self) -> cmp::Ordering {
        compare_components(self.clone(), other.clone())
    }
}

fn compare_components(mut left: Components<'_>, mut right: Components<'_>) -> cmp::Ordering {
    // Fast path for long shared prefixes <br>长共享前缀的快速路径<br>
    //
    // - compare raw bytes to find first mismatch <br>比较原始字节以找到第一个不匹配<br>
    // - backtrack to find separator before mismatch to avoid ambiguous parsings of '.' or '..' characters <br>回溯以在不匹配之前找到分隔符，以避免 '.' 或 '..' 字符的歧义解析<br>
    // - if found update state to only do a component-wise comparison on the remainder, otherwise do it on the full path <br>如果发现更新状态只对剩余部分进行组件比较，否则在完整路径上进行<br>
    //
    //
    // The fast path isn't taken for paths with a PrefixComponent to avoid backtracking into the middle of one <br>对于带有 PrefixComponent 的路径，不采用快速路径以避免回溯到一个路径的中间<br>
    //
    if left.prefix.is_none() && right.prefix.is_none() && left.front == right.front {
        // possible future improvement: a [u8]::first_mismatch simd implementation <br>未来可能的改进: [u8]::first_mismatch simd 实现<br>
        let first_difference = match left.path.iter().zip(right.path).position(|(&a, &b)| a != b) {
            None if left.path.len() == right.path.len() => return cmp::Ordering::Equal,
            None => left.path.len().min(right.path.len()),
            Some(diff) => diff,
        };

        if let Some(previous_sep) =
            left.path[..first_difference].iter().rposition(|&b| left.is_sep_byte(b))
        {
            let mismatched_component_start = previous_sep + 1;
            left.path = &left.path[mismatched_component_start..];
            left.front = State::Body;
            right.path = &right.path[mismatched_component_start..];
            right.front = State::Body;
        }
    }

    Iterator::cmp(left, right)
}

/// An iterator over [`Path`] and its ancestors. <br>[`Path`] 及其祖先上的迭代器。<br>
///
/// This `struct` is created by the [`ancestors`] method on [`Path`]. <br>该 `struct` 是通过 [`Path`] 上的 [`ancestors`] 方法创建的。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// # Examples
///
/// ```
/// use std::path::Path;
///
/// let path = Path::new("/foo/bar");
///
/// for ancestor in path.ancestors() {
///     println!("{}", ancestor.display());
/// }
/// ```
///
/// [`ancestors`]: Path::ancestors
#[derive(Copy, Clone, Debug)]
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "path_ancestors", since = "1.28.0")]
pub struct Ancestors<'a> {
    next: Option<&'a Path>,
}

#[stable(feature = "path_ancestors", since = "1.28.0")]
impl<'a> Iterator for Ancestors<'a> {
    type Item = &'a Path;

    #[inline]
    fn next(&mut self) -> Option<Self::Item> {
        let next = self.next;
        self.next = next.and_then(Path::parent);
        next
    }
}

#[stable(feature = "path_ancestors", since = "1.28.0")]
impl FusedIterator for Ancestors<'_> {}

////////////////////////////////////////////////////////////////////////////////
// Basic types and traits <br>原始类型和 traits<br>
////////////////////////////////////////////////////////////////////////////////

/// An owned, mutable path (akin to [`String`]). <br>拥有的可变路径 (类似于 [`String`])。<br>
///
/// This type provides methods like [`push`] and [`set_extension`] that mutate the path in place. <br>这种类型提供了 [`push`] 和 [`set_extension`] 之类的方法，这些方法会改变路径。<br>
/// It also implements [`Deref`] to [`Path`], meaning that all methods on [`Path`] slices are available on `PathBuf` values as well. <br>它还实现了 [`Deref`] 到 [`Path`]，这意味着 [`Path`] 切片上的所有方法也可以在 `PathBuf` 值上使用。<br>
///
///
/// [`push`]: PathBuf::push
/// [`set_extension`]: PathBuf::set_extension
///
/// More details about the overall approach can be found in the [module documentation](self). <br>有关整体方法的更多详细信息，请参见 [模块级文档](self)。<br>
///
/// # Examples
///
/// You can use [`push`] to build up a `PathBuf` from components: <br>您可以使用 [`push`] 从组件构建 `PathBuf`：<br>
///
/// ```
/// use std::path::PathBuf;
///
/// let mut path = PathBuf::new();
///
/// path.push(r"C:\");
/// path.push("windows");
/// path.push("system32");
///
/// path.set_extension("dll");
/// ```
///
/// However, [`push`] is best used for dynamic situations. <br>但是，[`push`] 最适合用于动态情况。<br>
/// This is a better way to do this when you know all of the components ahead of time: <br>当您提前了解所有组件时，这是一种更好的方法：<br>
///
/// ```
/// use std::path::PathBuf;
///
/// let path: PathBuf = [r"C:\", "windows", "system32.dll"].iter().collect();
/// ```
///
/// We can still do better than this! <br>我们仍然可以做得更好！<br> Since these are all strings, we can use <br>由于这些都是字符串，因此我们可以使用<br>
/// `From::from`:
///
/// ```
/// use std::path::PathBuf;
///
/// let path = PathBuf::from(r"C:\windows\system32.dll");
/// ```
///
/// Which method works best depends on what kind of situation you're in. <br>哪种方法最有效取决于您所处的情况。<br>
///
///
#[cfg_attr(not(test), rustc_diagnostic_item = "PathBuf")]
#[stable(feature = "rust1", since = "1.0.0")]
// FIXME:
// `PathBuf::as_mut_vec` current implementation relies on `PathBuf` being layout-compatible with `Vec<u8>`. <br>`PathBuf::as_mut_vec` 当前的实现依赖于 `PathBuf` 与 `Vec<u8>` 的布局兼容。<br>
// When attribute privacy is implemented, `PathBuf` should be annotated as `#[repr(transparent)]`. <br>实现属性隐私时，应将 `PathBuf` 注解为 `#[repr(transparent)]`。<br>
// Anyway, `PathBuf` representation and layout are considered implementation detail, are not documented and must not be relied upon. <br>无论如何，`PathBuf` 表示形式和布局被视为实现细节，没有文档记录，因此不能依赖。<br>
//
//
pub struct PathBuf {
    inner: OsString,
}

impl PathBuf {
    #[inline]
    fn as_mut_vec(&mut self) -> &mut Vec<u8> {
        unsafe { &mut *(self as *mut PathBuf as *mut Vec<u8>) }
    }

    /// Allocates an empty `PathBuf`. <br>分配一个空的 `PathBuf`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::PathBuf;
    ///
    /// let path = PathBuf::new();
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn new() -> PathBuf {
        PathBuf { inner: OsString::new() }
    }

    /// Creates a new `PathBuf` with a given capacity used to create the internal [`OsString`]. <br>创建具有给定容量的新 `PathBuf`，用于创建内部 [`OsString`]。<br>
    /// See [`with_capacity`] defined on [`OsString`]. <br>请参见在 [`OsString`] 上定义的 [`with_capacity`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::PathBuf;
    ///
    /// let mut path = PathBuf::with_capacity(10);
    /// let capacity = path.capacity();
    ///
    /// // This push is done without reallocating <br>无需重新分配即可完成此推送<br>
    /// path.push(r"C:\");
    ///
    /// assert_eq!(capacity, path.capacity());
    /// ```
    ///
    /// [`with_capacity`]: OsString::with_capacity
    #[stable(feature = "path_buf_capacity", since = "1.44.0")]
    #[must_use]
    #[inline]
    pub fn with_capacity(capacity: usize) -> PathBuf {
        PathBuf { inner: OsString::with_capacity(capacity) }
    }

    /// Coerces to a [`Path`] slice. <br>强制转换为 [`Path`] 切片。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::{Path, PathBuf};
    ///
    /// let p = PathBuf::from("/test");
    /// assert_eq!(Path::new("/test"), p.as_path());
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn as_path(&self) -> &Path {
        self
    }

    /// Extends `self` with `path`. <br>用 `path` 扩展 `self`。<br>
    ///
    /// If `path` is absolute, it replaces the current path. <br>如果 `path` 是绝对路径，它将替换当前路径。<br>
    ///
    /// On Windows: <br>在 Windows 上：<br>
    ///
    /// * if `path` has a root but no prefix (e.g., `\windows`), it replaces everything except for the prefix (if any) of `self`. <br>如果 `path` 具有根但没有前缀 (例如 `\windows`)，它将替换 `self` 的前缀 (如果有) 之外的所有内容。<br>
    ///
    /// * if `path` has a prefix but no root, it replaces `self`. <br>如果 `path` 有前缀但没有根，它将替换 `self`。<br>
    /// * if `self` has a verbatim prefix (e.g. <br>如果 `self` 有 verbatim 前缀 (例如<br>
    /// `\\?\C:\windows`) and `path` is not empty, the new path is normalized: all references to `.` and `..` are removed. <br>`\\?\C:\windows`) 并且 `path` 不为空，新路径被规范化：所有对 `.` 和 `..` 的引用都被删除。<br>
    ///
    /// # Examples
    ///
    /// Pushing a relative path extends the existing path: <br>推动相对路径可扩展现有路径：<br>
    ///
    /// ```
    /// use std::path::PathBuf;
    ///
    /// let mut path = PathBuf::from("/tmp");
    /// path.push("file.bk");
    /// assert_eq!(path, PathBuf::from("/tmp/file.bk"));
    /// ```
    ///
    /// Pushing an absolute path replaces the existing path: <br>推送绝对路径将替换现有路径：<br>
    ///
    /// ```
    /// use std::path::PathBuf;
    ///
    /// let mut path = PathBuf::from("/tmp");
    /// path.push("/etc");
    /// assert_eq!(path, PathBuf::from("/etc"));
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn push<P: AsRef<Path>>(&mut self, path: P) {
        self._push(path.as_ref())
    }

    fn _push(&mut self, path: &Path) {
        // in general, a separator is needed if the rightmost byte is not a separator <br>通常，如果最右边的字节不是分隔符，则需要分隔符<br>
        let mut need_sep = self.as_mut_vec().last().map(|c| !is_sep_byte(*c)).unwrap_or(false);

        // in the special case of `C:` on Windows, do *not* add a separator <br>在 Windows 上的 `C:` 的特殊情况下，请 *不要* 添加分隔符<br>
        let comps = self.components();

        if comps.prefix_len() > 0
            && comps.prefix_len() == comps.path.len()
            && comps.prefix.unwrap().is_drive()
        {
            need_sep = false
        }

        // absolute `path` replaces `self` <br>绝对 `path` 替代 `self`<br>
        if path.is_absolute() || path.prefix().is_some() {
            self.as_mut_vec().truncate(0);

        // verbatim paths need . and .. <br>verbatim 路径需要 . 和 ..<br> removed
        } else if comps.prefix_verbatim() && !path.inner.is_empty() {
            let mut buf: Vec<_> = comps.collect();
            for c in path.components() {
                match c {
                    Component::RootDir => {
                        buf.truncate(1);
                        buf.push(c);
                    }
                    Component::CurDir => (),
                    Component::ParentDir => {
                        if let Some(Component::Normal(_)) = buf.last() {
                            buf.pop();
                        }
                    }
                    _ => buf.push(c),
                }
            }

            let mut res = OsString::new();
            let mut need_sep = false;

            for c in buf {
                if need_sep && c != Component::RootDir {
                    res.push(MAIN_SEP_STR);
                }
                res.push(c.as_os_str());

                need_sep = match c {
                    Component::RootDir => false,
                    Component::Prefix(prefix) => {
                        !prefix.parsed.is_drive() && prefix.parsed.len() > 0
                    }
                    _ => true,
                }
            }

            self.inner = res;
            return;

        // `path` has a root but no prefix, e.g., `\windows` (Windows only) <br>`path` 有一个根但没有前缀，例如，`\windows` (仅 Windows)<br>
        } else if path.has_root() {
            let prefix_len = self.components().prefix_remaining();
            self.as_mut_vec().truncate(prefix_len);

        // `path` is a pure relative path <br>`path` 是一个纯粹的相对路径<br>
        } else if need_sep {
            self.inner.push(MAIN_SEP_STR);
        }

        self.inner.push(path);
    }

    /// Truncates `self` to [`self.parent`]. <br>将 `self` 截断为 [`self.parent`]。<br>
    ///
    /// Returns `false` and does nothing if [`self.parent`] is [`None`]. <br>返回 `false`，如果 [`self.parent`] 为 [`None`]，则不执行任何操作。<br>
    /// Otherwise, returns `true`. <br>否则，返回 `true`。<br>
    ///
    /// [`self.parent`]: Path::parent
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::{Path, PathBuf};
    ///
    /// let mut p = PathBuf::from("/spirited/away.rs");
    ///
    /// p.pop();
    /// assert_eq!(Path::new("/spirited"), p);
    /// p.pop();
    /// assert_eq!(Path::new("/"), p);
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn pop(&mut self) -> bool {
        match self.parent().map(|p| p.as_u8_slice().len()) {
            Some(len) => {
                self.as_mut_vec().truncate(len);
                true
            }
            None => false,
        }
    }

    /// Updates [`self.file_name`] to `file_name`. <br>将 [`self.file_name`] 更新为 `file_name`。<br>
    ///
    /// If [`self.file_name`] was [`None`], this is equivalent to pushing `file_name`. <br>如果 [`self.file_name`] 是 [`None`]，则等效于按下 `file_name`。<br>
    ///
    ///
    /// Otherwise it is equivalent to calling [`pop`] and then pushing `file_name`. <br>否则，这等效于调用 [`pop`]，然后按 `file_name`。<br>
    /// The new path will be a sibling of the original path. <br>新路径将是原始路径的同级。<br>
    /// (That is, it will have the same parent.) <br>(也就是说，它将具有相同的父对象。)<br>
    ///
    /// [`self.file_name`]: Path::file_name
    /// [`pop`]: PathBuf::pop
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::PathBuf;
    ///
    /// let mut buf = PathBuf::from("/");
    /// assert!(buf.file_name() == None);
    /// buf.set_file_name("bar");
    /// assert!(buf == PathBuf::from("/bar"));
    /// assert!(buf.file_name().is_some());
    /// buf.set_file_name("baz.txt");
    /// assert!(buf == PathBuf::from("/baz.txt"));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn set_file_name<S: AsRef<OsStr>>(&mut self, file_name: S) {
        self._set_file_name(file_name.as_ref())
    }

    fn _set_file_name(&mut self, file_name: &OsStr) {
        if self.file_name().is_some() {
            let popped = self.pop();
            debug_assert!(popped);
        }
        self.push(file_name);
    }

    /// Updates [`self.extension`] to `extension`. <br>将 [`self.extension`] 更新为 `extension`。<br>
    ///
    /// Returns `false` and does nothing if [`self.file_name`] is [`None`], returns `true` and updates the extension otherwise. <br>返回 `false`，如果 [`self.file_name`] 为 [`None`]，则不执行任何操作，否则返回 `true`，并更新扩展名。<br>
    ///
    ///
    /// If [`self.extension`] is [`None`], the extension is added; <br>如果 [`self.extension`] 为 [`None`]，则添加扩展名；<br> otherwise it is replaced. <br>否则将被替换。<br>
    ///
    /// [`self.file_name`]: Path::file_name
    /// [`self.extension`]: Path::extension
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::{Path, PathBuf};
    ///
    /// let mut p = PathBuf::from("/feel/the");
    ///
    /// p.set_extension("force");
    /// assert_eq!(Path::new("/feel/the.force"), p.as_path());
    ///
    /// p.set_extension("dark_side");
    /// assert_eq!(Path::new("/feel/the.dark_side"), p.as_path());
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn set_extension<S: AsRef<OsStr>>(&mut self, extension: S) -> bool {
        self._set_extension(extension.as_ref())
    }

    fn _set_extension(&mut self, extension: &OsStr) -> bool {
        let file_stem = match self.file_stem() {
            None => return false,
            Some(f) => f.bytes(),
        };

        // truncate until right after the file stem <br>截断直到文件干之后<br>
        let end_file_stem = file_stem[file_stem.len()..].as_ptr().addr();
        let start = self.inner.bytes().as_ptr().addr();
        let v = self.as_mut_vec();
        v.truncate(end_file_stem.wrapping_sub(start));

        // add the new extension, if any <br>添加新的扩展名 (如果有)<br>
        let new = extension.bytes();
        if !new.is_empty() {
            v.reserve_exact(new.len() + 1);
            v.push(b'.');
            v.extend_from_slice(new);
        }

        true
    }

    /// Consumes the `PathBuf`, yielding its internal [`OsString`] storage. <br>消耗 `PathBuf`，产生其内部 [`OsString`] 存储。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::PathBuf;
    ///
    /// let p = PathBuf::from("/the/head");
    /// let os_str = p.into_os_string();
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use = "`self` will be dropped if the result is not used"]
    #[inline]
    pub fn into_os_string(self) -> OsString {
        self.inner
    }

    /// Converts this `PathBuf` into a [boxed](Box) [`Path`]. <br>将此 `PathBuf` 转换为 [boxed](Box) [`Path`]。<br>
    #[stable(feature = "into_boxed_path", since = "1.20.0")]
    #[must_use = "`self` will be dropped if the result is not used"]
    #[inline]
    pub fn into_boxed_path(self) -> Box<Path> {
        let rw = Box::into_raw(self.inner.into_boxed_os_str()) as *mut Path;
        unsafe { Box::from_raw(rw) }
    }

    /// Invokes [`capacity`] on the underlying instance of [`OsString`]. <br>在 [`OsString`] 的底层实例上调用 [`capacity`]。<br>
    ///
    /// [`capacity`]: OsString::capacity
    #[stable(feature = "path_buf_capacity", since = "1.44.0")]
    #[must_use]
    #[inline]
    pub fn capacity(&self) -> usize {
        self.inner.capacity()
    }

    /// Invokes [`clear`] on the underlying instance of [`OsString`]. <br>在 [`OsString`] 的底层实例上调用 [`clear`]。<br>
    ///
    /// [`clear`]: OsString::clear
    #[stable(feature = "path_buf_capacity", since = "1.44.0")]
    #[inline]
    pub fn clear(&mut self) {
        self.inner.clear()
    }

    /// Invokes [`reserve`] on the underlying instance of [`OsString`]. <br>在 [`OsString`] 的底层实例上调用 [`reserve`]。<br>
    ///
    /// [`reserve`]: OsString::reserve
    #[stable(feature = "path_buf_capacity", since = "1.44.0")]
    #[inline]
    pub fn reserve(&mut self, additional: usize) {
        self.inner.reserve(additional)
    }

    /// Invokes [`try_reserve`] on the underlying instance of [`OsString`]. <br>在 [`OsString`] 的底层实例上调用 [`try_reserve`]。<br>
    ///
    /// [`try_reserve`]: OsString::try_reserve
    #[stable(feature = "try_reserve_2", since = "1.63.0")]
    #[inline]
    pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError> {
        self.inner.try_reserve(additional)
    }

    /// Invokes [`reserve_exact`] on the underlying instance of [`OsString`]. <br>在 [`OsString`] 的底层实例上调用 [`reserve_exact`]。<br>
    ///
    /// [`reserve_exact`]: OsString::reserve_exact
    #[stable(feature = "path_buf_capacity", since = "1.44.0")]
    #[inline]
    pub fn reserve_exact(&mut self, additional: usize) {
        self.inner.reserve_exact(additional)
    }

    /// Invokes [`try_reserve_exact`] on the underlying instance of [`OsString`]. <br>在 [`OsString`] 的底层实例上调用 [`try_reserve_exact`]。<br>
    ///
    /// [`try_reserve_exact`]: OsString::try_reserve_exact
    #[stable(feature = "try_reserve_2", since = "1.63.0")]
    #[inline]
    pub fn try_reserve_exact(&mut self, additional: usize) -> Result<(), TryReserveError> {
        self.inner.try_reserve_exact(additional)
    }

    /// Invokes [`shrink_to_fit`] on the underlying instance of [`OsString`]. <br>在 [`OsString`] 的底层实例上调用 [`shrink_to_fit`]。<br>
    ///
    /// [`shrink_to_fit`]: OsString::shrink_to_fit
    #[stable(feature = "path_buf_capacity", since = "1.44.0")]
    #[inline]
    pub fn shrink_to_fit(&mut self) {
        self.inner.shrink_to_fit()
    }

    /// Invokes [`shrink_to`] on the underlying instance of [`OsString`]. <br>在 [`OsString`] 的底层实例上调用 [`shrink_to`]。<br>
    ///
    /// [`shrink_to`]: OsString::shrink_to
    #[stable(feature = "shrink_to", since = "1.56.0")]
    #[inline]
    pub fn shrink_to(&mut self, min_capacity: usize) {
        self.inner.shrink_to(min_capacity)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Clone for PathBuf {
    #[inline]
    fn clone(&self) -> Self {
        PathBuf { inner: self.inner.clone() }
    }

    #[inline]
    fn clone_from(&mut self, source: &Self) {
        self.inner.clone_from(&source.inner)
    }
}

#[stable(feature = "box_from_path", since = "1.17.0")]
impl From<&Path> for Box<Path> {
    /// Creates a boxed [`Path`] from a reference. <br>从引用创建一个 boxed [`Path`]。<br>
    ///
    /// This will allocate and clone `path` to it. <br>这将为它分配和克隆 `path`。<br>
    fn from(path: &Path) -> Box<Path> {
        let boxed: Box<OsStr> = path.inner.into();
        let rw = Box::into_raw(boxed) as *mut Path;
        unsafe { Box::from_raw(rw) }
    }
}

#[stable(feature = "box_from_cow", since = "1.45.0")]
impl From<Cow<'_, Path>> for Box<Path> {
    /// Creates a boxed [`Path`] from a clone-on-write pointer. <br>从写时克隆指针创建一个 boxed [`Path`]。<br>
    ///
    /// Converting from a `Cow::Owned` does not clone or allocate. <br>从 `Cow::Owned` 转换不会克隆或分配。<br>
    #[inline]
    fn from(cow: Cow<'_, Path>) -> Box<Path> {
        match cow {
            Cow::Borrowed(path) => Box::from(path),
            Cow::Owned(path) => Box::from(path),
        }
    }
}

#[stable(feature = "path_buf_from_box", since = "1.18.0")]
impl From<Box<Path>> for PathBuf {
    /// Converts a <code>[Box]<[Path]></code> into a [`PathBuf`]. <br>将 <code>[Box]<[Path]></code> 转换为 [`PathBuf`]。<br>
    ///
    /// This conversion does not allocate or copy memory. <br>此转换不会分配或复制内存。<br>
    #[inline]
    fn from(boxed: Box<Path>) -> PathBuf {
        boxed.into_path_buf()
    }
}

#[stable(feature = "box_from_path_buf", since = "1.20.0")]
impl From<PathBuf> for Box<Path> {
    /// Converts a [`PathBuf`] into a <code>[Box]<[Path]></code>. <br>将 [`PathBuf`] 转换为 <code>[Box]<[Path]></code>。<br>
    ///
    /// This conversion currently should not allocate memory, but this behavior is not guaranteed on all platforms or in all future versions. <br>此转换当前不应该分配内存，但是不能在所有平台上或所有 future 版本中都保证此行为。<br>
    ///
    #[inline]
    fn from(p: PathBuf) -> Box<Path> {
        p.into_boxed_path()
    }
}

#[stable(feature = "more_box_slice_clone", since = "1.29.0")]
impl Clone for Box<Path> {
    #[inline]
    fn clone(&self) -> Self {
        self.to_path_buf().into_boxed_path()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: ?Sized + AsRef<OsStr>> From<&T> for PathBuf {
    /// Converts a borrowed [`OsStr`] to a [`PathBuf`]. <br>将借用的 [`OsStr`] 转换为 [`PathBuf`]。<br>
    ///
    /// Allocates a [`PathBuf`] and copies the data into it. <br>分配一个 [`PathBuf`] 并将数据复制到其中。<br>
    #[inline]
    fn from(s: &T) -> PathBuf {
        PathBuf::from(s.as_ref().to_os_string())
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl From<OsString> for PathBuf {
    /// Converts an [`OsString`] into a [`PathBuf`] <br>将 [`OsString`] 转换为 [`PathBuf`]<br>
    ///
    /// This conversion does not allocate or copy memory. <br>此转换不会分配或复制内存。<br>
    #[inline]
    fn from(s: OsString) -> PathBuf {
        PathBuf { inner: s }
    }
}

#[stable(feature = "from_path_buf_for_os_string", since = "1.14.0")]
impl From<PathBuf> for OsString {
    /// Converts a [`PathBuf`] into an [`OsString`] <br>将 [`PathBuf`] 转换为 [`OsString`]<br>
    ///
    /// This conversion does not allocate or copy memory. <br>此转换不会分配或复制内存。<br>
    #[inline]
    fn from(path_buf: PathBuf) -> OsString {
        path_buf.inner
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl From<String> for PathBuf {
    /// Converts a [`String`] into a [`PathBuf`] <br>将 [`String`] 转换为 [`PathBuf`]<br>
    ///
    /// This conversion does not allocate or copy memory. <br>此转换不会分配或复制内存。<br>
    #[inline]
    fn from(s: String) -> PathBuf {
        PathBuf::from(OsString::from(s))
    }
}

#[stable(feature = "path_from_str", since = "1.32.0")]
impl FromStr for PathBuf {
    type Err = core::convert::Infallible;

    #[inline]
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        Ok(PathBuf::from(s))
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<P: AsRef<Path>> iter::FromIterator<P> for PathBuf {
    fn from_iter<I: IntoIterator<Item = P>>(iter: I) -> PathBuf {
        let mut buf = PathBuf::new();
        buf.extend(iter);
        buf
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<P: AsRef<Path>> iter::Extend<P> for PathBuf {
    fn extend<I: IntoIterator<Item = P>>(&mut self, iter: I) {
        iter.into_iter().for_each(move |p| self.push(p.as_ref()));
    }

    #[inline]
    fn extend_one(&mut self, p: P) {
        self.push(p.as_ref());
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for PathBuf {
    fn fmt(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Debug::fmt(&**self, formatter)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl ops::Deref for PathBuf {
    type Target = Path;
    #[inline]
    fn deref(&self) -> &Path {
        Path::new(&self.inner)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Borrow<Path> for PathBuf {
    #[inline]
    fn borrow(&self) -> &Path {
        self.deref()
    }
}

#[stable(feature = "default_for_pathbuf", since = "1.17.0")]
impl Default for PathBuf {
    #[inline]
    fn default() -> Self {
        PathBuf::new()
    }
}

#[stable(feature = "cow_from_path", since = "1.6.0")]
impl<'a> From<&'a Path> for Cow<'a, Path> {
    /// Creates a clone-on-write pointer from a reference to [`Path`]. <br>创建一个从引用到 [`Path`] 的写时克隆指针。<br>
    ///
    ///
    /// This conversion does not clone or allocate. <br>此转换不会克隆或分配。<br>
    #[inline]
    fn from(s: &'a Path) -> Cow<'a, Path> {
        Cow::Borrowed(s)
    }
}

#[stable(feature = "cow_from_path", since = "1.6.0")]
impl<'a> From<PathBuf> for Cow<'a, Path> {
    /// Creates a clone-on-write pointer from an owned instance of [`PathBuf`]. <br>从 [`PathBuf`] 的拥有实例创建一个写时克隆指针。<br>
    ///
    ///
    /// This conversion does not clone or allocate. <br>此转换不会克隆或分配。<br>
    #[inline]
    fn from(s: PathBuf) -> Cow<'a, Path> {
        Cow::Owned(s)
    }
}

#[stable(feature = "cow_from_pathbuf_ref", since = "1.28.0")]
impl<'a> From<&'a PathBuf> for Cow<'a, Path> {
    /// Creates a clone-on-write pointer from a reference to [`PathBuf`]. <br>创建一个从引用到 [`PathBuf`] 的写时克隆指针。<br>
    ///
    ///
    /// This conversion does not clone or allocate. <br>此转换不会克隆或分配。<br>
    #[inline]
    fn from(p: &'a PathBuf) -> Cow<'a, Path> {
        Cow::Borrowed(p.as_path())
    }
}

#[stable(feature = "pathbuf_from_cow_path", since = "1.28.0")]
impl<'a> From<Cow<'a, Path>> for PathBuf {
    /// Converts a clone-on-write pointer to an owned path. <br>将写时克隆指针转换为拥有所有权的路径。<br>
    ///
    /// Converting from a `Cow::Owned` does not clone or allocate. <br>从 `Cow::Owned` 转换不会克隆或分配。<br>
    #[inline]
    fn from(p: Cow<'a, Path>) -> Self {
        p.into_owned()
    }
}

#[stable(feature = "shared_from_slice2", since = "1.24.0")]
impl From<PathBuf> for Arc<Path> {
    /// Converts a [`PathBuf`] into an <code>[Arc]<[Path]></code> by moving the [`PathBuf`] data into a new [`Arc`] buffer. <br>通过将 [`PathBuf`] 数据移动到新的 [`Arc`] 缓冲区中，将 [`PathBuf`] 转换为 <code>[Arc]<[Path]></code>。<br>
    ///
    #[inline]
    fn from(s: PathBuf) -> Arc<Path> {
        let arc: Arc<OsStr> = Arc::from(s.into_os_string());
        unsafe { Arc::from_raw(Arc::into_raw(arc) as *const Path) }
    }
}

#[stable(feature = "shared_from_slice2", since = "1.24.0")]
impl From<&Path> for Arc<Path> {
    /// Converts a [`Path`] into an [`Arc`] by copying the [`Path`] data into a new [`Arc`] buffer. <br>通过将 [`Path`] 数据复制到新的 [`Arc`] 缓冲区，将 [`Path`] 转换为 [`Arc`]。<br>
    #[inline]
    fn from(s: &Path) -> Arc<Path> {
        let arc: Arc<OsStr> = Arc::from(s.as_os_str());
        unsafe { Arc::from_raw(Arc::into_raw(arc) as *const Path) }
    }
}

#[stable(feature = "shared_from_slice2", since = "1.24.0")]
impl From<PathBuf> for Rc<Path> {
    /// Converts a [`PathBuf`] into an <code>[Rc]<[Path]></code> by moving the [`PathBuf`] data into a new [`Rc`] buffer. <br>通过将 [`PathBuf`] 数据移动到新的 [`Rc`] 缓冲区中，将 [`PathBuf`] 转换为 <code>[Rc]<[Path]></code>。<br>
    ///
    #[inline]
    fn from(s: PathBuf) -> Rc<Path> {
        let rc: Rc<OsStr> = Rc::from(s.into_os_string());
        unsafe { Rc::from_raw(Rc::into_raw(rc) as *const Path) }
    }
}

#[stable(feature = "shared_from_slice2", since = "1.24.0")]
impl From<&Path> for Rc<Path> {
    /// Converts a [`Path`] into an [`Rc`] by copying the [`Path`] data into a new [`Rc`] buffer. <br>通过将 [`Path`] 数据复制到新的 [`Rc`] 缓冲区中，将 [`Path`] 转换为 [`Rc`]。<br>
    #[inline]
    fn from(s: &Path) -> Rc<Path> {
        let rc: Rc<OsStr> = Rc::from(s.as_os_str());
        unsafe { Rc::from_raw(Rc::into_raw(rc) as *const Path) }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl ToOwned for Path {
    type Owned = PathBuf;
    #[inline]
    fn to_owned(&self) -> PathBuf {
        self.to_path_buf()
    }
    #[inline]
    fn clone_into(&self, target: &mut PathBuf) {
        self.inner.clone_into(&mut target.inner);
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::PartialEq for PathBuf {
    #[inline]
    fn eq(&self, other: &PathBuf) -> bool {
        self.components() == other.components()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Hash for PathBuf {
    fn hash<H: Hasher>(&self, h: &mut H) {
        self.as_path().hash(h)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::Eq for PathBuf {}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::PartialOrd for PathBuf {
    #[inline]
    fn partial_cmp(&self, other: &PathBuf) -> Option<cmp::Ordering> {
        Some(compare_components(self.components(), other.components()))
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::Ord for PathBuf {
    #[inline]
    fn cmp(&self, other: &PathBuf) -> cmp::Ordering {
        compare_components(self.components(), other.components())
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<OsStr> for PathBuf {
    #[inline]
    fn as_ref(&self) -> &OsStr {
        &self.inner[..]
    }
}

/// A slice of a path (akin to [`str`]). <br>路径的切片 (类似于 [`str`])。<br>
///
/// This type supports a number of operations for inspecting a path, including breaking the path into its components (separated by `/` on Unix and by either `/` or `\` on Windows), extracting the file name, determining whether the path is absolute, and so on. <br>此类型支持许多检查路径的操作，包括将路径分为其各个组成部分 (由 Unix 上的 `/` 和 Windows 上的 `/` 或 `\` 分隔)，提取文件名，确定路径是否为绝对路径，等等。。<br>
///
///
/// This is an *unsized* type, meaning that it must always be used behind a pointer like `&` or [`Box`]. <br>这是未定义大小的类型，表示必须始终在 `&` 或 [`Box`] 之类的指针后面使用它。<br>
/// For an owned version of this type, see [`PathBuf`]. <br>有关此类型的拥有版本，请参见 [`PathBuf`]。<br>
///
/// More details about the overall approach can be found in the [module documentation](self). <br>有关整体方法的更多详细信息，请参见 [模块级文档](self)。<br>
///
/// # Examples
///
/// ```
/// use std::path::Path;
/// use std::ffi::OsStr;
///
/// // Note: this example does work on Windows <br>这个例子可以在 Windows 上使用<br>
/// let path = Path::new("./foo/bar.txt");
///
/// let parent = path.parent();
/// assert_eq!(parent, Some(Path::new("./foo")));
///
/// let file_stem = path.file_stem();
/// assert_eq!(file_stem, Some(OsStr::new("bar")));
///
/// let extension = path.extension();
/// assert_eq!(extension, Some(OsStr::new("txt")));
/// ```
///
///
///
///
#[cfg_attr(not(test), rustc_diagnostic_item = "Path")]
#[stable(feature = "rust1", since = "1.0.0")]
// FIXME:
// `Path::new` current implementation relies on `Path` being layout-compatible with `OsStr`. <br>`Path::new` 当前实现依赖于 `Path` 与 `OsStr` 布局兼容。<br>
// When attribute privacy is implemented, `Path` should be annotated as `#[repr(transparent)]`. <br>实现属性隐私时，应将 `Path` 注解为 `#[repr(transparent)]`。<br>
// Anyway, `Path` representation and layout are considered implementation detail, are not documented and must not be relied upon. <br>无论如何，`Path` 表示形式和布局被视为实现细节，没有文档记录，因此不能依赖。<br>
//
//
pub struct Path {
    inner: OsStr,
}

/// An error returned from [`Path::strip_prefix`] if the prefix was not found. <br>如果找不到前缀，则从 [`Path::strip_prefix`] 返回错误。<br>
///
/// This `struct` is created by the [`strip_prefix`] method on [`Path`]. <br>该 `struct` 是通过 [`Path`] 上的 [`strip_prefix`] 方法创建的。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
///
/// [`strip_prefix`]: Path::strip_prefix
#[derive(Debug, Clone, PartialEq, Eq)]
#[stable(since = "1.7.0", feature = "strip_prefix")]
pub struct StripPrefixError(());

impl Path {
    // The following (private!) function allows construction of a path from a u8 slice, which is only safe when it is known to follow the OsStr encoding. <br>以下 (private!) 函数允许从 u8 切片构造路径，仅当已知遵循 OsStr 编码时，此路径才是安全的。<br>
    //
    unsafe fn from_u8_slice(s: &[u8]) -> &Path {
        unsafe { Path::new(u8_slice_as_os_str(s)) }
    }
    // The following (private!) function reveals the byte encoding used for OsStr. <br>以下 (private!) 函数揭示了用于 OsStr 的字节编码。<br>
    fn as_u8_slice(&self) -> &[u8] {
        self.inner.bytes()
    }

    /// Directly wraps a string slice as a `Path` slice. <br>将字符串切片直接包装为 `Path` 切片。<br>
    ///
    /// This is a cost-free conversion. <br>这是一个免费的转换。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// Path::new("foo.txt");
    /// ```
    ///
    /// You can create `Path`s from `String`s, or even other `Path`s: <br>您可以从 `String` 甚至其他 `Path`s 创建 `Path`s：<br>
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let string = String::from("foo.txt");
    /// let from_string = Path::new(&string);
    /// let from_path = Path::new(&from_string);
    /// assert_eq!(from_string, from_path);
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn new<S: AsRef<OsStr> + ?Sized>(s: &S) -> &Path {
        unsafe { &*(s.as_ref() as *const OsStr as *const Path) }
    }

    /// Yields the underlying [`OsStr`] slice. <br>产生底层的 [`OsStr`] 切片。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let os_str = Path::new("foo.txt").as_os_str();
    /// assert_eq!(os_str, std::ffi::OsStr::new("foo.txt"));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn as_os_str(&self) -> &OsStr {
        &self.inner
    }

    /// Yields a [`&str`] slice if the `Path` is valid unicode. <br>如果 `Path` 是有效的 unicode，则产生 [`&str`] 切片。<br>
    ///
    /// This conversion may entail doing a check for UTF-8 validity. <br>此转换可能需要检查 UTF-8 有效性。<br>
    /// Note that validation is performed because non-UTF-8 strings are perfectly valid for some OS. <br>请注意，执行验证是因为非 UTF-8 字符串对于某些 OS 完全有效。<br>
    ///
    ///
    /// [`&str`]: str
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let path = Path::new("foo.txt");
    /// assert_eq!(path.to_str(), Some("foo.txt"));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use = "this returns the result of the operation, \
                  without modifying the original"]
    #[inline]
    pub fn to_str(&self) -> Option<&str> {
        self.inner.to_str()
    }

    /// Converts a `Path` to a [`Cow<str>`]. <br>将 `Path` 转换为 [`Cow<str>`]。<br>
    ///
    /// Any non-Unicode sequences are replaced with [`U+FFFD REPLACEMENT CHARACTER`][U+FFFD]. <br>任何非 Unicode 序列都将替换为 [`U+FFFD REPLACEMENT CHARACTER`][U+FFFD]。<br>
    ///
    /// [U+FFFD]: super::char::REPLACEMENT_CHARACTER
    ///
    /// # Examples
    ///
    /// Calling `to_string_lossy` on a `Path` with valid unicode: <br>使用有效的 Unicode 在 `Path` 上调用 `to_string_lossy`：<br>
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let path = Path::new("foo.txt");
    /// assert_eq!(path.to_string_lossy(), "foo.txt");
    /// ```
    ///
    /// Had `path` contained invalid unicode, the `to_string_lossy` call might have returned `"fo.txt"`. <br>如果 `path` 包含无效的 unicode，则 `to_string_lossy` 调用可能已返回 `"fo.txt"`。<br>
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use = "this returns the result of the operation, \
                  without modifying the original"]
    #[inline]
    pub fn to_string_lossy(&self) -> Cow<'_, str> {
        self.inner.to_string_lossy()
    }

    /// Converts a `Path` to an owned [`PathBuf`]. <br>将 `Path` 转换为拥有的 [`PathBuf`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let path_buf = Path::new("foo.txt").to_path_buf();
    /// assert_eq!(path_buf, std::path::PathBuf::from("foo.txt"));
    /// ```
    #[rustc_conversion_suggestion]
    #[must_use = "this returns the result of the operation, \
                  without modifying the original"]
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn to_path_buf(&self) -> PathBuf {
        PathBuf::from(self.inner.to_os_string())
    }

    /// Returns `true` if the `Path` is absolute, i.e., if it is independent of the current directory. <br>如果 `Path` 是绝对的，即独立于当前目录，则返回 `true`。<br>
    ///
    /// * On Unix, a path is absolute if it starts with the root, so `is_absolute` and [`has_root`] are equivalent. <br>在 Unix 上，如果路径以根开头，则它是绝对的，因此 `is_absolute` 和 [`has_root`] 是等效的。<br>
    ///
    /// * On Windows, a path is absolute if it has a prefix and starts with the root: `c:\windows` is absolute, while `c:temp` and `\temp` are not. <br>在 Windows 上，如果路径具有前缀并以根开头，则它是绝对路径: `c:\windows` 是绝对路径，而 `c:temp` 和 `\temp` 不是。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// assert!(!Path::new("foo.txt").is_absolute());
    /// ```
    ///
    /// [`has_root`]: Path::has_root
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[allow(deprecated)]
    pub fn is_absolute(&self) -> bool {
        if cfg!(target_os = "redox") {
            // FIXME: Allow Redox prefixes <br>允许 `Redox` 前缀<br>
            self.has_root() || has_redox_scheme(self.as_u8_slice())
        } else {
            self.has_root() && (cfg!(any(unix, target_os = "wasi")) || self.prefix().is_some())
        }
    }

    /// Returns `true` if the `Path` is relative, i.e., not absolute. <br>如果 `Path` 是相对的，即不是绝对的，则返回 `true`。<br>
    ///
    /// See [`is_absolute`]'s documentation for more details. <br>有关更多详细信息，请参见 [`is_absolute`] 的文档。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// assert!(Path::new("foo.txt").is_relative());
    /// ```
    ///
    /// [`is_absolute`]: Path::is_absolute
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn is_relative(&self) -> bool {
        !self.is_absolute()
    }

    fn prefix(&self) -> Option<Prefix<'_>> {
        self.components().prefix
    }

    /// Returns `true` if the `Path` has a root. <br>如果 `Path` 具有根，则返回 `true`。<br>
    ///
    /// * On Unix, a path has a root if it begins with `/`. <br>在 Unix 上，如果路径以 `/` 开头，则该路径具有根。<br>
    ///
    /// * On Windows, a path has a root if it: <br>在 Windows 上，如果路径具有根，则它是：<br>
    ///     * has no prefix and begins with a separator, e.g., `\windows` <br>没有前缀并以分隔符开头，例如 `\windows`<br>
    ///     * has a prefix followed by a separator, e.g., `c:\windows` but not `c:windows` <br>有一个前缀，后跟一个分隔符，例如 `c:\windows`，但没有 `c:windows`<br>
    ///     * has any non-disk prefix, e.g., `\\server\share` <br>具有任何非磁盘前缀，例如 `\\server\share`<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// assert!(Path::new("/etc/passwd").has_root());
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    #[inline]
    pub fn has_root(&self) -> bool {
        self.components().has_root()
    }

    /// Returns the `Path` without its final component, if there is one. <br>如果没有 `Path`，则返回不包含其最终组成部分的 `Path`。<br>
    ///
    /// Returns [`None`] if the path terminates in a root or prefix. <br>如果路径以根或前缀结尾，则返回 [`None`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let path = Path::new("/foo/bar");
    /// let parent = path.parent().unwrap();
    /// assert_eq!(parent, Path::new("/foo"));
    ///
    /// let grand_parent = parent.parent().unwrap();
    /// assert_eq!(grand_parent, Path::new("/"));
    /// assert_eq!(grand_parent.parent(), None);
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn parent(&self) -> Option<&Path> {
        let mut comps = self.components();
        let comp = comps.next_back();
        comp.and_then(|p| match p {
            Component::Normal(_) | Component::CurDir | Component::ParentDir => {
                Some(comps.as_path())
            }
            _ => None,
        })
    }

    /// Produces an iterator over `Path` and its ancestors. <br>在 `Path` 及其祖先上生成一个迭代器。<br>
    ///
    /// The iterator will yield the `Path` that is returned if the [`parent`] method is used zero or more times. <br>如果 [`parent`] 方法使用了 0 次或多次，则迭代器将产生返回的 `Path`。<br>
    /// That means, the iterator will yield `&self`, `&self.parent().unwrap()`, `&self.parent().unwrap().parent().unwrap()` and so on. <br>这意味着，迭代器将产生 `&self`，`&self.parent().unwrap()`，`&self.parent().unwrap().parent().unwrap()` 等。<br> If the [`parent`] method returns [`None`], the iterator will do likewise. <br>如果 [`parent`] 方法返回 [`None`]，则迭代器也将这样做。<br>
    ///
    /// The iterator will always yield at least one value, namely `&self`. <br>迭代器将始终产生至少一个值，即 `&self`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let mut ancestors = Path::new("/foo/bar").ancestors();
    /// assert_eq!(ancestors.next(), Some(Path::new("/foo/bar")));
    /// assert_eq!(ancestors.next(), Some(Path::new("/foo")));
    /// assert_eq!(ancestors.next(), Some(Path::new("/")));
    /// assert_eq!(ancestors.next(), None);
    ///
    /// let mut ancestors = Path::new("../foo/bar").ancestors();
    /// assert_eq!(ancestors.next(), Some(Path::new("../foo/bar")));
    /// assert_eq!(ancestors.next(), Some(Path::new("../foo")));
    /// assert_eq!(ancestors.next(), Some(Path::new("..")));
    /// assert_eq!(ancestors.next(), Some(Path::new("")));
    /// assert_eq!(ancestors.next(), None);
    /// ```
    ///
    /// [`parent`]: Path::parent
    ///
    #[stable(feature = "path_ancestors", since = "1.28.0")]
    #[inline]
    pub fn ancestors(&self) -> Ancestors<'_> {
        Ancestors { next: Some(&self) }
    }

    /// Returns the final component of the `Path`, if there is one. <br>返回 `Path` 的最后一个组件 (如果有)。<br>
    ///
    /// If the path is a normal file, this is the file name. <br>如果路径是普通文件，则为文件名。<br>
    /// If it's the path of a directory, this is the directory name. <br>如果是目录路径，则为目录名称。<br>
    ///
    /// Returns [`None`] if the path terminates in `..`. <br>如果路径以 `..` 结尾，则返回 [`None`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    /// use std::ffi::OsStr;
    ///
    /// assert_eq!(Some(OsStr::new("bin")), Path::new("/usr/bin/").file_name());
    /// assert_eq!(Some(OsStr::new("foo.txt")), Path::new("tmp/foo.txt").file_name());
    /// assert_eq!(Some(OsStr::new("foo.txt")), Path::new("foo.txt/.").file_name());
    /// assert_eq!(Some(OsStr::new("foo.txt")), Path::new("foo.txt/.//").file_name());
    /// assert_eq!(None, Path::new("foo.txt/..").file_name());
    /// assert_eq!(None, Path::new("/").file_name());
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn file_name(&self) -> Option<&OsStr> {
        self.components().next_back().and_then(|p| match p {
            Component::Normal(p) => Some(p),
            _ => None,
        })
    }

    /// Returns a path that, when joined onto `base`, yields `self`. <br>返回连接到 `base` 时产生 `self` 的路径。<br>
    ///
    /// # Errors
    ///
    /// If `base` is not a prefix of `self` (i.e., [`starts_with`] returns `false`), returns [`Err`]. <br>如果 `base` 不是 `self` 的前缀 (即 [`starts_with`] 返回 `false`)，则返回 [`Err`]。<br>
    ///
    ///
    /// [`starts_with`]: Path::starts_with
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::{Path, PathBuf};
    ///
    /// let path = Path::new("/test/haha/foo.txt");
    ///
    /// assert_eq!(path.strip_prefix("/"), Ok(Path::new("test/haha/foo.txt")));
    /// assert_eq!(path.strip_prefix("/test"), Ok(Path::new("haha/foo.txt")));
    /// assert_eq!(path.strip_prefix("/test/"), Ok(Path::new("haha/foo.txt")));
    /// assert_eq!(path.strip_prefix("/test/haha/foo.txt"), Ok(Path::new("")));
    /// assert_eq!(path.strip_prefix("/test/haha/foo.txt/"), Ok(Path::new("")));
    ///
    /// assert!(path.strip_prefix("test").is_err());
    /// assert!(path.strip_prefix("/haha").is_err());
    ///
    /// let prefix = PathBuf::from("/test/");
    /// assert_eq!(path.strip_prefix(prefix), Ok(Path::new("haha/foo.txt")));
    /// ```
    #[stable(since = "1.7.0", feature = "path_strip_prefix")]
    pub fn strip_prefix<P>(&self, base: P) -> Result<&Path, StripPrefixError>
    where
        P: AsRef<Path>,
    {
        self._strip_prefix(base.as_ref())
    }

    fn _strip_prefix(&self, base: &Path) -> Result<&Path, StripPrefixError> {
        iter_after(self.components(), base.components())
            .map(|c| c.as_path())
            .ok_or(StripPrefixError(()))
    }

    /// Determines whether `base` is a prefix of `self`. <br>确定 `base` 是否为 `self` 的前缀。<br>
    ///
    /// Only considers whole path components to match. <br>仅考虑整个路径组件匹配。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let path = Path::new("/etc/passwd");
    ///
    /// assert!(path.starts_with("/etc"));
    /// assert!(path.starts_with("/etc/"));
    /// assert!(path.starts_with("/etc/passwd"));
    /// assert!(path.starts_with("/etc/passwd/")); // extra slash is okay <br>额外的斜线是可以的<br>
    /// assert!(path.starts_with("/etc/passwd///")); // multiple extra slashes are okay
    ///
    /// assert!(!path.starts_with("/e"));
    /// assert!(!path.starts_with("/etc/passwd.txt"));
    ///
    /// assert!(!Path::new("/etc/foo.rs").starts_with("/etc/foo"));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn starts_with<P: AsRef<Path>>(&self, base: P) -> bool {
        self._starts_with(base.as_ref())
    }

    fn _starts_with(&self, base: &Path) -> bool {
        iter_after(self.components(), base.components()).is_some()
    }

    /// Determines whether `child` is a suffix of `self`. <br>确定 `child` 是否为 `self` 的后缀。<br>
    ///
    /// Only considers whole path components to match. <br>仅考虑整个路径组件匹配。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let path = Path::new("/etc/resolv.conf");
    ///
    /// assert!(path.ends_with("resolv.conf"));
    /// assert!(path.ends_with("etc/resolv.conf"));
    /// assert!(path.ends_with("/etc/resolv.conf"));
    ///
    /// assert!(!path.ends_with("/resolv.conf"));
    /// assert!(!path.ends_with("conf")); // use .extension() instead <br>改用 .extension()<br>
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn ends_with<P: AsRef<Path>>(&self, child: P) -> bool {
        self._ends_with(child.as_ref())
    }

    fn _ends_with(&self, child: &Path) -> bool {
        iter_after(self.components().rev(), child.components().rev()).is_some()
    }

    /// Extracts the stem (non-extension) portion of [`self.file_name`]. <br>提取 [`self.file_name`] 的茎 (non-extension) 部分。<br>
    ///
    /// [`self.file_name`]: Path::file_name
    ///
    /// The stem is: <br>词干为：<br>
    ///
    /// * [`None`], if there is no file name; <br>[`None`]，如果没有文件名；<br>
    /// * The entire file name if there is no embedded `.`; <br>如果没有嵌入式 `.`，则为整个文件名; 否则为 0。<br>
    /// * The entire file name if the file name begins with `.` and has no other `.`s within; <br>如果文件名以 `.` 开头且内部没有其他 `.`，则为整个文件名；<br>
    /// * Otherwise, the portion of the file name before the final `.` <br>否则，文件名中最后 `.` 之前的部分<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// assert_eq!("foo", Path::new("foo.rs").file_stem().unwrap());
    /// assert_eq!("foo.tar", Path::new("foo.tar.gz").file_stem().unwrap());
    /// ```
    ///
    /// # See Also <br>也可以看看<br>
    /// This method is similar to [`Path::file_prefix`], which extracts the portion of the file name before the *first* `.` <br>这个方法类似于 [`Path::file_prefix`]，提取 *first* `.` 之前的文件名部分<br>
    ///
    ///
    /// [`Path::file_prefix`]: Path::file_prefix
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn file_stem(&self) -> Option<&OsStr> {
        self.file_name().map(rsplit_file_at_dot).and_then(|(before, after)| before.or(after))
    }

    /// Extracts the prefix of [`self.file_name`]. <br>提取 [`self.file_name`] 的前缀。<br>
    ///
    /// The prefix is: <br>前缀是：<br>
    ///
    /// * [`None`], if there is no file name; <br>[`None`]，如果没有文件名；<br>
    /// * The entire file name if there is no embedded `.`; <br>如果没有嵌入式 `.`，则为整个文件名; 否则为 0。<br>
    /// * The portion of the file name before the first non-beginning `.`; <br>文件名中第一个非开头 `.` 之前的部分；<br>
    /// * The entire file name if the file name begins with `.` and has no other `.`s within; <br>如果文件名以 `.` 开头且内部没有其他 `.`，则为整个文件名；<br>
    /// * The portion of the file name before the second `.` if the file name begins with `.` <br>如果文件名以 `.` 开头，则为第二个 `.` 之前的文件名部分<br>
    ///
    /// [`self.file_name`]: Path::file_name
    ///
    /// # Examples
    ///
    /// ```
    /// # #![feature(path_file_prefix)]
    /// use std::path::Path;
    ///
    /// assert_eq!("foo", Path::new("foo.rs").file_prefix().unwrap());
    /// assert_eq!("foo", Path::new("foo.tar.gz").file_prefix().unwrap());
    /// ```
    ///
    /// # See Also <br>也可以看看<br>
    /// This method is similar to [`Path::file_stem`], which extracts the portion of the file name before the *last* `.` <br>此方法类似于 [`Path::file_stem`]，提取文件名中 *last* `.` 之前的部分<br>
    ///
    ///
    /// [`Path::file_stem`]: Path::file_stem
    ///
    #[unstable(feature = "path_file_prefix", issue = "86319")]
    #[must_use]
    pub fn file_prefix(&self) -> Option<&OsStr> {
        self.file_name().map(split_file_at_dot).and_then(|(before, _after)| Some(before))
    }

    /// Extracts the extension of [`self.file_name`], if possible. <br>如果可能，提取 [`self.file_name`] 的扩展名。<br>
    ///
    /// The extension is: <br>扩展名是：<br>
    ///
    /// * [`None`], if there is no file name; <br>[`None`]，如果没有文件名；<br>
    /// * [`None`], if there is no embedded `.`; <br>[`None`]，如果没有嵌入的 `.`；<br>
    /// * [`None`], if the file name begins with `.` and has no other `.`s within; <br>[`None`]，如果文件名以 `.` 开头，并且里面没有其他的 `.`；<br>
    /// * Otherwise, the portion of the file name after the final `.` <br>否则，文件名中最后一个 `.` 之后的部分<br>
    ///
    /// [`self.file_name`]: Path::file_name
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// assert_eq!("rs", Path::new("foo.rs").extension().unwrap());
    /// assert_eq!("gz", Path::new("foo.tar.gz").extension().unwrap());
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn extension(&self) -> Option<&OsStr> {
        self.file_name().map(rsplit_file_at_dot).and_then(|(before, after)| before.and(after))
    }

    /// Creates an owned [`PathBuf`] with `path` adjoined to `self`. <br>创建一个拥有的 [`PathBuf`]，并将 `path` 附加到 `self`。<br>
    ///
    /// See [`PathBuf::push`] for more details on what it means to adjoin a path. <br>有关连接路径的含义的更多详细信息，请参见 [`PathBuf::push`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::{Path, PathBuf};
    ///
    /// assert_eq!(Path::new("/etc").join("passwd"), PathBuf::from("/etc/passwd"));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn join<P: AsRef<Path>>(&self, path: P) -> PathBuf {
        self._join(path.as_ref())
    }

    fn _join(&self, path: &Path) -> PathBuf {
        let mut buf = self.to_path_buf();
        buf.push(path);
        buf
    }

    /// Creates an owned [`PathBuf`] like `self` but with the given file name. <br>创建一个拥有的 [`PathBuf`]，例如 `self`，但具有给定的文件名。<br>
    ///
    /// See [`PathBuf::set_file_name`] for more details. <br>有关更多详细信息，请参见 [`PathBuf::set_file_name`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::{Path, PathBuf};
    ///
    /// let path = Path::new("/tmp/foo.txt");
    /// assert_eq!(path.with_file_name("bar.txt"), PathBuf::from("/tmp/bar.txt"));
    ///
    /// let path = Path::new("/tmp");
    /// assert_eq!(path.with_file_name("var"), PathBuf::from("/var"));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn with_file_name<S: AsRef<OsStr>>(&self, file_name: S) -> PathBuf {
        self._with_file_name(file_name.as_ref())
    }

    fn _with_file_name(&self, file_name: &OsStr) -> PathBuf {
        let mut buf = self.to_path_buf();
        buf.set_file_name(file_name);
        buf
    }

    /// Creates an owned [`PathBuf`] like `self` but with the given extension. <br>创建一个拥有的 [`PathBuf`]，例如 `self`，但具有给定的扩展名。<br>
    ///
    /// See [`PathBuf::set_extension`] for more details. <br>有关更多详细信息，请参见 [`PathBuf::set_extension`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::{Path, PathBuf};
    ///
    /// let path = Path::new("foo.rs");
    /// assert_eq!(path.with_extension("txt"), PathBuf::from("foo.txt"));
    ///
    /// let path = Path::new("foo.tar.gz");
    /// assert_eq!(path.with_extension(""), PathBuf::from("foo.tar"));
    /// assert_eq!(path.with_extension("xz"), PathBuf::from("foo.tar.xz"));
    /// assert_eq!(path.with_extension("").with_extension("txt"), PathBuf::from("foo.txt"));
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn with_extension<S: AsRef<OsStr>>(&self, extension: S) -> PathBuf {
        self._with_extension(extension.as_ref())
    }

    fn _with_extension(&self, extension: &OsStr) -> PathBuf {
        let mut buf = self.to_path_buf();
        buf.set_extension(extension);
        buf
    }

    /// Produces an iterator over the [`Component`]s of the path. <br>生成路径的 [`Component`] 上的迭代器。<br>
    ///
    /// When parsing the path, there is a small amount of normalization: <br>解析路径时，需要进行少量标准化：<br>
    ///
    /// * Repeated separators are ignored, so `a/b` and `a//b` both have `a` and `b` as components. <br>重复的分隔符将被忽略，因此 `a/b` 和 `a//b` 都具有 `a` 和 `b` 作为组件。<br>
    ///
    /// * Occurrences of `.` are normalized away, except if they are at the beginning of the path. <br>`.` 的出现被归一化，除非它们位于路径的开头。<br>
    /// For example, `a/./b`, `a/b/`, `a/b/.` and `a/b` all have `a` and `b` as components, but `./a/b` starts with an additional [`CurDir`] component. <br>例如，`a/./b`，`a/b/`，`a/b/.` 和 `a/b` 都具有 `a` 和 `b` 作为组件，但是 `./a/b` 以附加的 [`CurDir`] 组件开头。<br>
    ///
    /// * A trailing slash is normalized away, `/a/b` and `/a/b/` are equivalent. <br>尾部的斜杠已标准化，`/a/b` 和 `/a/b/` 是等效的。<br>
    ///
    /// Note that no other normalization takes place; <br>请注意，没有其他标准化发生。<br> in particular, `a/c` and `a/b/../c` are distinct, to account for the possibility that `b` is a symbolic link (so its parent isn't `a`). <br>特别是，`a/c` 和 `a/b/../c` 是不同的，以考虑到 `b` 是符号链接 (因此其父代不是 `a`) 的可能性。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::{Path, Component};
    /// use std::ffi::OsStr;
    ///
    /// let mut components = Path::new("/tmp/foo.txt").components();
    ///
    /// assert_eq!(components.next(), Some(Component::RootDir));
    /// assert_eq!(components.next(), Some(Component::Normal(OsStr::new("tmp"))));
    /// assert_eq!(components.next(), Some(Component::Normal(OsStr::new("foo.txt"))));
    /// assert_eq!(components.next(), None)
    /// ```
    ///
    /// [`CurDir`]: Component::CurDir
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn components(&self) -> Components<'_> {
        let prefix = parse_prefix(self.as_os_str());
        Components {
            path: self.as_u8_slice(),
            prefix,
            has_physical_root: has_physical_root(self.as_u8_slice(), prefix)
                || has_redox_scheme(self.as_u8_slice()),
            front: State::Prefix,
            back: State::Body,
        }
    }

    /// Produces an iterator over the path's components viewed as [`OsStr`] slices. <br>在视为 [`OsStr`] slice 的路径的组件上生成迭代器。<br>
    ///
    /// For more information about the particulars of how the path is separated into components, see [`components`]. <br>有关如何将路径分成多个组件的详细信息，请参见 [`components`]。<br>
    ///
    ///
    /// [`components`]: Path::components
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::{self, Path};
    /// use std::ffi::OsStr;
    ///
    /// let mut it = Path::new("/tmp/foo.txt").iter();
    /// assert_eq!(it.next(), Some(OsStr::new(&path::MAIN_SEPARATOR.to_string())));
    /// assert_eq!(it.next(), Some(OsStr::new("tmp")));
    /// assert_eq!(it.next(), Some(OsStr::new("foo.txt")));
    /// assert_eq!(it.next(), None)
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[inline]
    pub fn iter(&self) -> Iter<'_> {
        Iter { inner: self.components() }
    }

    /// Returns an object that implements [`Display`] for safely printing paths that may contain non-Unicode data. <br>返回实现 [`Display`] 的对象，以安全地打印可能包含非 Unicode 数据的路径。<br>
    ///
    /// This may perform lossy conversion, depending on the platform. <br>根据平台的不同，这可能会执行有损转换。<br>
    /// If you would like an implementation which escapes the path please use [`Debug`] instead. <br>如果您想要一个转义路径的实现，请改用 [`Debug`]。<br>
    ///
    /// [`Display`]: fmt::Display
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    ///
    /// let path = Path::new("/tmp/foo.rs");
    ///
    /// println!("{}", path.display());
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use = "this does not display the path, \
                  it returns an object that can be displayed"]
    #[inline]
    pub fn display(&self) -> Display<'_> {
        Display { path: self }
    }

    /// Queries the file system to get information about a file, directory, etc. <br>查询文件系统以获取有关文件，目录等的信息。<br>
    ///
    /// This function will traverse symbolic links to query information about the destination file. <br>该函数将遍历符号链接以查询有关目标文件的信息。<br>
    ///
    ///
    /// This is an alias to [`fs::metadata`]. <br>这是 [`fs::metadata`] 的别名。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::path::Path;
    ///
    /// let path = Path::new("/Minas/tirith");
    /// let metadata = path.metadata().expect("metadata call failed");
    /// println!("{:?}", metadata.file_type());
    /// ```
    #[stable(feature = "path_ext", since = "1.5.0")]
    #[inline]
    pub fn metadata(&self) -> io::Result<fs::Metadata> {
        fs::metadata(self)
    }

    /// Queries the metadata about a file without following symlinks. <br>查询有关文件的元数据，而无需遵循符号链接。<br>
    ///
    /// This is an alias to [`fs::symlink_metadata`]. <br>这是 [`fs::symlink_metadata`] 的别名。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::path::Path;
    ///
    /// let path = Path::new("/Minas/tirith");
    /// let metadata = path.symlink_metadata().expect("symlink_metadata call failed");
    /// println!("{:?}", metadata.file_type());
    /// ```
    #[stable(feature = "path_ext", since = "1.5.0")]
    #[inline]
    pub fn symlink_metadata(&self) -> io::Result<fs::Metadata> {
        fs::symlink_metadata(self)
    }

    /// Returns the canonical, absolute form of the path with all intermediate components normalized and symbolic links resolved. <br>返回路径的规范，绝对形式，所有中间组件均已标准化，符号链接已解析。<br>
    ///
    ///
    /// This is an alias to [`fs::canonicalize`]. <br>这是 [`fs::canonicalize`] 的别名。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::path::{Path, PathBuf};
    ///
    /// let path = Path::new("/foo/test/../test/bar.rs");
    /// assert_eq!(path.canonicalize().unwrap(), PathBuf::from("/foo/test/bar.rs"));
    /// ```
    #[stable(feature = "path_ext", since = "1.5.0")]
    #[inline]
    pub fn canonicalize(&self) -> io::Result<PathBuf> {
        fs::canonicalize(self)
    }

    /// Reads a symbolic link, returning the file that the link points to. <br>读取符号链接，返回链接指向的文件。<br>
    ///
    /// This is an alias to [`fs::read_link`]. <br>这是 [`fs::read_link`] 的别名。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::path::Path;
    ///
    /// let path = Path::new("/laputa/sky_castle.rs");
    /// let path_link = path.read_link().expect("read_link call failed");
    /// ```
    #[stable(feature = "path_ext", since = "1.5.0")]
    #[inline]
    pub fn read_link(&self) -> io::Result<PathBuf> {
        fs::read_link(self)
    }

    /// Returns an iterator over the entries within a directory. <br>返回目录中条目的迭代器。<br>
    ///
    /// The iterator will yield instances of <code>[io::Result]<[fs::DirEntry]></code>. <br>迭代器将产生一个 <code>[io::Result]<[fs::DirEntry]></code> 的实例。<br>
    /// New errors may be encountered after an iterator is initially constructed. <br>最初构造迭代器后，可能会遇到新的错误。<br>
    ///
    /// This is an alias to [`fs::read_dir`]. <br>这是 [`fs::read_dir`] 的别名。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::path::Path;
    ///
    /// let path = Path::new("/laputa");
    /// for entry in path.read_dir().expect("read_dir call failed") {
    ///     if let Ok(entry) = entry {
    ///         println!("{:?}", entry.path());
    ///     }
    /// }
    /// ```
    #[stable(feature = "path_ext", since = "1.5.0")]
    #[inline]
    pub fn read_dir(&self) -> io::Result<fs::ReadDir> {
        fs::read_dir(self)
    }

    /// Returns `true` if the path points at an existing entity. <br>如果路径指向现有实体，则返回 `true`。<br>
    ///
    /// Warning: this method may be error-prone, consider using [`try_exists()`] instead! <br>警告: 此方法可能容易出错，请考虑改用 [`try_exists()`]!<br>
    /// It also has a risk of introducing time-of-check to time-of-use (TOCTOU) bugs. <br>它还存在将时间检查引入使用时间 (TOCTOU) 错误的风险。<br>
    ///
    /// This function will traverse symbolic links to query information about the destination file. <br>该函数将遍历符号链接以查询有关目标文件的信息。<br>
    ///
    ///
    /// If you cannot access the metadata of the file, e.g. <br>如果您无法访问文件的元数据，例如<br>
    /// because of a permission error or broken symbolic links, this will return `false`. <br>由于权限错误或损坏的符号链接，这将返回 `false`。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::path::Path;
    /// assert!(!Path::new("does_not_exist.txt").exists());
    /// ```
    ///
    /// # See Also <br>也可以看看<br>
    ///
    /// This is a convenience function that coerces errors to false. <br>这是一个方便的函数，可将错误强制为 false。<br>
    /// If you want to check errors, call [`Path::try_exists`]. <br>如果要检查错误，调用 [`Path::try_exists`]。<br>
    ///
    /// [`try_exists()`]: Self::try_exists
    #[stable(feature = "path_ext", since = "1.5.0")]
    #[must_use]
    #[inline]
    pub fn exists(&self) -> bool {
        fs::metadata(self).is_ok()
    }

    /// Returns `Ok(true)` if the path points at an existing entity. <br>如果路径指向现有实体，则返回 `Ok(true)`。<br>
    ///
    /// This function will traverse symbolic links to query information about the destination file. <br>该函数将遍历符号链接以查询有关目标文件的信息。<br>
    /// In case of broken symbolic links this will return `Ok(false)`. <br>如果符号链接断开，则将返回 `Ok(false)`。<br>
    ///
    /// As opposed to the [`exists()`] method, this one doesn't silently ignore errors unrelated to the path not existing. <br>与 [`exists()`] 方法相反，此方法不会默默地忽略与不存在的路径无关的错误。<br> (E.g.
    /// it will return `Err(_)` in case of permission denied on some of the parent directories.) <br>如果某些父目录的权限被拒绝，它将返回 `Err(_)`。)<br>
    ///
    /// Note that while this avoids some pitfalls of the `exists()` method, it still can not prevent time-of-check to time-of-use (TOCTOU) bugs. <br>请注意，虽然这避免了 `exists()` 方法的一些缺陷，但它仍然无法防止时间检查到使用时间 (TOCTOU) 错误。<br>
    ///
    /// You should only use it in scenarios where those bugs are not an issue. <br>您应该只在这些错误不是问题的情况下使用它。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::path::Path;
    /// assert!(!Path::new("does_not_exist.txt").try_exists().expect("Can't check existence of file does_not_exist.txt"));
    /// assert!(Path::new("/root/secret_file.txt").try_exists().is_err());
    /// ```
    ///
    /// [`exists()`]: Self::exists
    ///
    #[stable(feature = "path_try_exists", since = "1.63.0")]
    #[inline]
    pub fn try_exists(&self) -> io::Result<bool> {
        fs::try_exists(self)
    }

    /// Returns `true` if the path exists on disk and is pointing at a regular file. <br>如果路径在磁盘上并且指向常规文件，则返回 `true`。<br>
    ///
    /// This function will traverse symbolic links to query information about the destination file. <br>该函数将遍历符号链接以查询有关目标文件的信息。<br>
    ///
    /// If you cannot access the metadata of the file, e.g. because of a permission error or broken symbolic links, this will return `false`. <br>如果您无法访问文件的元数据，例如由于权限错误或损坏的符号链接，这将返回 `false`。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::path::Path;
    /// assert_eq!(Path::new("./is_a_directory/").is_file(), false);
    /// assert_eq!(Path::new("a_file.txt").is_file(), true);
    /// ```
    ///
    /// # See Also <br>也可以看看<br>
    ///
    /// This is a convenience function that coerces errors to false. <br>这是一个方便的函数，可将错误强制为 false。<br> If you want to check errors, call [`fs::metadata`] and handle its [`Result`]. <br>如果要检查错误，请调用 [`fs::metadata`] 并处理其 [`Result`]。<br>
    /// Then call [`fs::Metadata::is_file`] if it was [`Ok`]. <br>如果是 [`Ok`]，则调用 [`fs::Metadata::is_file`]。<br>
    ///
    /// When the goal is simply to read from (or write to) the source, the most reliable way to test the source can be read (or written to) is to open it. <br>当目标只是读取 (或写入) 源时，可以读取 (或写入) 最可靠的测试源方法是打开它。<br>
    ///
    /// Only using `is_file` can break workflows like `diff <( prog_a )` on a Unix-like system for example. <br>例如，仅使用 `is_file` 才能中断类似 Unix 的系统上的工作流，例如 `diff <( prog_a )`。<br>
    /// See [`fs::File::open`] or [`fs::OpenOptions::open`] for more information. <br>有关更多信息，请参见 [`fs::File::open`] 或 [`fs::OpenOptions::open`]。<br>
    ///
    ///
    ///
    ///
    #[stable(feature = "path_ext", since = "1.5.0")]
    #[must_use]
    pub fn is_file(&self) -> bool {
        fs::metadata(self).map(|m| m.is_file()).unwrap_or(false)
    }

    /// Returns `true` if the path exists on disk and is pointing at a directory. <br>如果路径在磁盘上并且指向目录，则返回 `true`。<br>
    ///
    /// This function will traverse symbolic links to query information about the destination file. <br>该函数将遍历符号链接以查询有关目标文件的信息。<br>
    ///
    ///
    /// If you cannot access the metadata of the file, e.g. <br>如果您无法访问文件的元数据，例如<br>
    /// because of a permission error or broken symbolic links, this will return `false`. <br>由于权限错误或损坏的符号链接，这将返回 `false`。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::path::Path;
    /// assert_eq!(Path::new("./is_a_directory/").is_dir(), true);
    /// assert_eq!(Path::new("a_file.txt").is_dir(), false);
    /// ```
    ///
    /// # See Also <br>也可以看看<br>
    ///
    /// This is a convenience function that coerces errors to false. <br>这是一个方便的函数，可将错误强制为 false。<br>
    /// If you want to check errors, call [`fs::metadata`] and handle its [`Result`]. <br>如果要检查错误，请调用 [`fs::metadata`] 并处理其 [`Result`]。<br>
    /// Then call [`fs::Metadata::is_dir`] if it was [`Ok`]. <br>如果是 [`Ok`]，则调用 [`fs::Metadata::is_dir`]。<br>
    #[stable(feature = "path_ext", since = "1.5.0")]
    #[must_use]
    pub fn is_dir(&self) -> bool {
        fs::metadata(self).map(|m| m.is_dir()).unwrap_or(false)
    }

    /// Returns `true` if the path exists on disk and is pointing at a symbolic link. <br>如果路径存在于磁盘上并且指向符号链接，则返回 `true`。<br>
    ///
    /// This function will not traverse symbolic links. <br>这个函数不会遍历符号链接。<br>
    /// In case of a broken symbolic link this will also return true. <br>如果符号链接损坏，这也将返回 true。<br>
    ///
    /// If you cannot access the directory containing the file, e.g., because of a permission error, this will return false. <br>如果您无法访问包含该文件的目录，例如，由于权限错误，这将返回 false。<br>
    ///
    ///
    /// # Examples
    ///
    #[cfg_attr(unix, doc = "```no_run")]
    #[cfg_attr(not(unix), doc = "```ignore")]
    /// use std::path::Path;
    /// use std::os::unix::fs::symlink;
    ///
    /// let link_path = Path::new("link");
    /// symlink("/origin_does_not_exist/", link_path).unwrap();
    /// assert_eq!(link_path.is_symlink(), true);
    /// assert_eq!(link_path.exists(), false);
    /// ```
    ///
    /// # See Also <br>也可以看看<br>
    ///
    /// This is a convenience function that coerces errors to false. <br>这是一个方便的函数，可将错误强制为 false。<br>
    /// If you want to check errors, call [`fs::symlink_metadata`] and handle its [`Result`]. <br>如果要检查错误，请调用 [`fs::symlink_metadata`] 并处理其 [`Result`]。<br>
    /// Then call [`fs::Metadata::is_symlink`] if it was [`Ok`]. <br>如果是 [`Ok`]，则调用 [`fs::Metadata::is_symlink`]。<br>
    #[must_use]
    #[stable(feature = "is_symlink", since = "1.58.0")]
    pub fn is_symlink(&self) -> bool {
        fs::symlink_metadata(self).map(|m| m.is_symlink()).unwrap_or(false)
    }

    /// Converts a [`Box<Path>`](Box) into a [`PathBuf`] without copying or allocating. <br>无需复制或分配即可将 [`Box<Path>`](Box) 转换为 [`PathBuf`]。<br>
    ///
    #[stable(feature = "into_boxed_path", since = "1.20.0")]
    #[must_use = "`self` will be dropped if the result is not used"]
    pub fn into_path_buf(self: Box<Path>) -> PathBuf {
        let rw = Box::into_raw(self) as *mut OsStr;
        let inner = unsafe { Box::from_raw(rw) };
        PathBuf { inner: OsString::from(inner) }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<OsStr> for Path {
    #[inline]
    fn as_ref(&self) -> &OsStr {
        &self.inner
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for Path {
    fn fmt(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Debug::fmt(&self.inner, formatter)
    }
}

/// Helper struct for safely printing paths with [`format!`] and `{}`. <br>Helper 结构体，用于使用 [`format!`] 和 `{}` 安全地打印路径。<br>
///
/// A [`Path`] might contain non-Unicode data. <br>[`Path`] 可能包含非 Unicode 数据。<br>
/// This `struct` implements the [`Display`] trait in a way that mitigates that. <br>该 `struct` 可以减轻 [`Display`] trait 的负担。<br>
/// It is created by the [`display`](Path::display) method on [`Path`]. <br>它是通过 [`Path`] 上的 [`display`](Path::display) 方法创建的。<br>
/// This may perform lossy conversion, depending on the platform. <br>根据平台的不同，这可能会执行有损转换。<br>
/// If you would like an implementation which escapes the path please use [`Debug`] instead. <br>如果您想要一个转义路径的实现，请改用 [`Debug`]。<br>
///
/// # Examples
///
/// ```
/// use std::path::Path;
///
/// let path = Path::new("/tmp/foo.rs");
///
/// println!("{}", path.display());
/// ```
///
/// [`Display`]: fmt::Display
/// [`format!`]: crate::format
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Display<'a> {
    path: &'a Path,
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for Display<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Debug::fmt(&self.path, f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Display for Display<'_> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.path.inner.display(f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::PartialEq for Path {
    #[inline]
    fn eq(&self, other: &Path) -> bool {
        self.components() == other.components()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl Hash for Path {
    fn hash<H: Hasher>(&self, h: &mut H) {
        let bytes = self.as_u8_slice();
        let (prefix_len, verbatim) = match parse_prefix(&self.inner) {
            Some(prefix) => {
                prefix.hash(h);
                (prefix.len(), prefix.is_verbatim())
            }
            None => (0, false),
        };
        let bytes = &bytes[prefix_len..];

        let mut component_start = 0;
        let mut bytes_hashed = 0;

        for i in 0..bytes.len() {
            let is_sep = if verbatim { is_verbatim_sep(bytes[i]) } else { is_sep_byte(bytes[i]) };
            if is_sep {
                if i > component_start {
                    let to_hash = &bytes[component_start..i];
                    h.write(to_hash);
                    bytes_hashed += to_hash.len();
                }

                // skip over separator and optionally a following CurDir item since components() would normalize these away. <br>跳过分隔符和可选的后续 CurDir 项，因为 components() 会将它们标准化。<br>
                //
                component_start = i + 1;

                let tail = &bytes[component_start..];

                if !verbatim {
                    component_start += match tail {
                        [b'.'] => 1,
                        [b'.', sep @ _, ..] if is_sep_byte(*sep) => 1,
                        _ => 0,
                    };
                }
            }
        }

        if component_start < bytes.len() {
            let to_hash = &bytes[component_start..];
            h.write(to_hash);
            bytes_hashed += to_hash.len();
        }

        h.write_usize(bytes_hashed);
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::Eq for Path {}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::PartialOrd for Path {
    #[inline]
    fn partial_cmp(&self, other: &Path) -> Option<cmp::Ordering> {
        Some(compare_components(self.components(), other.components()))
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl cmp::Ord for Path {
    #[inline]
    fn cmp(&self, other: &Path) -> cmp::Ordering {
        compare_components(self.components(), other.components())
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<Path> for Path {
    #[inline]
    fn as_ref(&self) -> &Path {
        self
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<Path> for OsStr {
    #[inline]
    fn as_ref(&self) -> &Path {
        Path::new(self)
    }
}

#[stable(feature = "cow_os_str_as_ref_path", since = "1.8.0")]
impl AsRef<Path> for Cow<'_, OsStr> {
    #[inline]
    fn as_ref(&self) -> &Path {
        Path::new(self)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<Path> for OsString {
    #[inline]
    fn as_ref(&self) -> &Path {
        Path::new(self)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<Path> for str {
    #[inline]
    fn as_ref(&self) -> &Path {
        Path::new(self)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<Path> for String {
    #[inline]
    fn as_ref(&self) -> &Path {
        Path::new(self)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl AsRef<Path> for PathBuf {
    #[inline]
    fn as_ref(&self) -> &Path {
        self
    }
}

#[stable(feature = "path_into_iter", since = "1.6.0")]
impl<'a> IntoIterator for &'a PathBuf {
    type Item = &'a OsStr;
    type IntoIter = Iter<'a>;
    #[inline]
    fn into_iter(self) -> Iter<'a> {
        self.iter()
    }
}

#[stable(feature = "path_into_iter", since = "1.6.0")]
impl<'a> IntoIterator for &'a Path {
    type Item = &'a OsStr;
    type IntoIter = Iter<'a>;
    #[inline]
    fn into_iter(self) -> Iter<'a> {
        self.iter()
    }
}

macro_rules! impl_cmp {
    ($lhs:ty, $rhs: ty) => {
        #[stable(feature = "partialeq_path", since = "1.6.0")]
        impl<'a, 'b> PartialEq<$rhs> for $lhs {
            #[inline]
            fn eq(&self, other: &$rhs) -> bool {
                <Path as PartialEq>::eq(self, other)
            }
        }

        #[stable(feature = "partialeq_path", since = "1.6.0")]
        impl<'a, 'b> PartialEq<$lhs> for $rhs {
            #[inline]
            fn eq(&self, other: &$lhs) -> bool {
                <Path as PartialEq>::eq(self, other)
            }
        }

        #[stable(feature = "cmp_path", since = "1.8.0")]
        impl<'a, 'b> PartialOrd<$rhs> for $lhs {
            #[inline]
            fn partial_cmp(&self, other: &$rhs) -> Option<cmp::Ordering> {
                <Path as PartialOrd>::partial_cmp(self, other)
            }
        }

        #[stable(feature = "cmp_path", since = "1.8.0")]
        impl<'a, 'b> PartialOrd<$lhs> for $rhs {
            #[inline]
            fn partial_cmp(&self, other: &$lhs) -> Option<cmp::Ordering> {
                <Path as PartialOrd>::partial_cmp(self, other)
            }
        }
    };
}

impl_cmp!(PathBuf, Path);
impl_cmp!(PathBuf, &'a Path);
impl_cmp!(Cow<'a, Path>, Path);
impl_cmp!(Cow<'a, Path>, &'b Path);
impl_cmp!(Cow<'a, Path>, PathBuf);

macro_rules! impl_cmp_os_str {
    ($lhs:ty, $rhs: ty) => {
        #[stable(feature = "cmp_path", since = "1.8.0")]
        impl<'a, 'b> PartialEq<$rhs> for $lhs {
            #[inline]
            fn eq(&self, other: &$rhs) -> bool {
                <Path as PartialEq>::eq(self, other.as_ref())
            }
        }

        #[stable(feature = "cmp_path", since = "1.8.0")]
        impl<'a, 'b> PartialEq<$lhs> for $rhs {
            #[inline]
            fn eq(&self, other: &$lhs) -> bool {
                <Path as PartialEq>::eq(self.as_ref(), other)
            }
        }

        #[stable(feature = "cmp_path", since = "1.8.0")]
        impl<'a, 'b> PartialOrd<$rhs> for $lhs {
            #[inline]
            fn partial_cmp(&self, other: &$rhs) -> Option<cmp::Ordering> {
                <Path as PartialOrd>::partial_cmp(self, other.as_ref())
            }
        }

        #[stable(feature = "cmp_path", since = "1.8.0")]
        impl<'a, 'b> PartialOrd<$lhs> for $rhs {
            #[inline]
            fn partial_cmp(&self, other: &$lhs) -> Option<cmp::Ordering> {
                <Path as PartialOrd>::partial_cmp(self.as_ref(), other)
            }
        }
    };
}

impl_cmp_os_str!(PathBuf, OsStr);
impl_cmp_os_str!(PathBuf, &'a OsStr);
impl_cmp_os_str!(PathBuf, Cow<'a, OsStr>);
impl_cmp_os_str!(PathBuf, OsString);
impl_cmp_os_str!(Path, OsStr);
impl_cmp_os_str!(Path, &'a OsStr);
impl_cmp_os_str!(Path, Cow<'a, OsStr>);
impl_cmp_os_str!(Path, OsString);
impl_cmp_os_str!(&'a Path, OsStr);
impl_cmp_os_str!(&'a Path, Cow<'b, OsStr>);
impl_cmp_os_str!(&'a Path, OsString);
impl_cmp_os_str!(Cow<'a, Path>, OsStr);
impl_cmp_os_str!(Cow<'a, Path>, &'b OsStr);
impl_cmp_os_str!(Cow<'a, Path>, OsString);

#[stable(since = "1.7.0", feature = "strip_prefix")]
impl fmt::Display for StripPrefixError {
    #[allow(deprecated, deprecated_in_future)]
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.description().fmt(f)
    }
}

#[stable(since = "1.7.0", feature = "strip_prefix")]
impl Error for StripPrefixError {
    #[allow(deprecated)]
    fn description(&self) -> &str {
        "prefix not found"
    }
}

/// Makes the path absolute without accessing the filesystem. <br>在不访问文件系统的情况下使路径成为绝对路径。<br>
///
/// If the path is relative, the current directory is used as the base directory. <br>如果路径是相对的，则将当前目录用作基本目录。<br>
/// All intermediate components will be resolved according to platforms-specific rules but unlike [`canonicalize`][crate::fs::canonicalize] this does not resolve symlinks and may succeed even if the path does not exist. <br>所有中间组件都将根据特定于平台的规则进行解析，但与 [`canonicalize`][crate::fs::canonicalize] 不同，它不会解析符号链接，即使路径不存在也可能成功。<br>
///
///
/// If the `path` is empty or getting the [current directory][crate::env::current_dir] fails then an error will be returned. <br>如果 `path` 为空或获取 [current directory][crate::env::current_dir] 失败，则返回错误。<br>
///
/// # Examples
///
/// ## Posix paths <br>Posix 路径<br>
///
/// ```
/// #![feature(absolute_path)]
/// # #[cfg(unix)]
/// fn main() -> std::io::Result<()> {
///   use std::path::{self, Path};
///
///   // Relative to absolute <br>相对于绝对<br>
///   let absolute = path::absolute("foo/./bar")?;
///   assert!(absolute.ends_with("foo/bar"));
///
///   // Absolute to absolute <br>绝对到绝对<br>
///   let absolute = path::absolute("/foo//test/.././bar.rs")?;
///   assert_eq!(absolute, Path::new("/foo/test/../bar.rs"));
///   Ok(())
/// }
/// # #[cfg(not(unix))]
/// # fn main() {}
/// ```
///
/// The path is resolved using [POSIX semantics][posix-semantics] except that it stops short of resolving symlinks. <br>该路径是使用 [POSIX semantics][posix-semantics] 解析的，只是它没有解析符号链接。<br> This means it will keep `..` components and trailing slashes. <br>这意味着它将保留 `..` 组件和尾部斜杠。<br>
///
/// ## Windows paths <br>Windows 路径<br>
///
/// ```
/// #![feature(absolute_path)]
/// # #[cfg(windows)]
/// fn main() -> std::io::Result<()> {
///   use std::path::{self, Path};
///
///   // Relative to absolute <br>相对于绝对<br>
///   let absolute = path::absolute("foo/./bar")?;
///   assert!(absolute.ends_with(r"foo\bar"));
///
///   // Absolute to absolute <br>绝对到绝对<br>
///   let absolute = path::absolute(r"C:\foo//test\..\./bar.rs")?;
///
///   assert_eq!(absolute, Path::new(r"C:\foo\bar.rs"));
///   Ok(())
/// }
/// # #[cfg(not(windows))]
/// # fn main() {}
/// ```
///
/// For verbatim paths this will simply return the path as given. <br>对于逐字路径，这将简单地返回给定的路径。<br> For other paths this is currently equivalent to calling [`GetFullPathNameW`][windows-path] This may change in the future. <br>对于其他路径，这目前相当于调用 [`GetFullPathNameW`][windows-path] 这可能会在 future 中发生变化。<br>
///
/// [posix-semantics]: https://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap04.html#tag_04_13
/// [windows-path]: https://docs.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-getfullpathnamew
///
///
///
///
///
///
///
#[unstable(feature = "absolute_path", issue = "92750")]
pub fn absolute<P: AsRef<Path>>(path: P) -> io::Result<PathBuf> {
    let path = path.as_ref();
    if path.as_os_str().is_empty() {
        Err(io::const_io_error!(io::ErrorKind::InvalidInput, "cannot make an empty path absolute",))
    } else {
        sys::path::absolute(path)
    }
}
