//! A module for working with processes. <br>用于处理进程的模块。<br>
//!
//! This module is mostly concerned with spawning and interacting with child processes, but it also provides [`abort`] and [`exit`] for terminating the current process. <br>该模块主要与产生和与子进程交互有关，但是它也提供了 [`abort`] 和 [`exit`] 来终止当前进程。<br>
//!
//! # Spawning a process <br>产生一个进程<br>
//!
//! The [`Command`] struct is used to configure and spawn processes: <br>[`Command`] 结构体用于配置和 spawn 进程：<br>
//!
//! ```no_run
//! use std::process::Command;
//!
//! let output = Command::new("echo")
//!                      .arg("Hello world")
//!                      .output()
//!                      .expect("Failed to execute command");
//!
//! assert_eq!(b"Hello world\n", output.stdout.as_slice());
//! ```
//!
//! Several methods on [`Command`], such as [`spawn`] or [`output`], can be used to spawn a process. <br>[`Command`] 上的几种方法 (例如 [`spawn`] 或 [`output`]) 可用于 spawn 进程。<br>
//! In particular, [`output`] spawns the child process and waits until the process terminates, while [`spawn`] will return a [`Child`] that represents the spawned child process. <br>特别是，[`output`] 生成子进程并等待直到该进程终止，而 [`spawn`] 将返回代表生成的子进程的 [`Child`]。<br>
//!
//!
//! # Handling I/O <br>处理 I/O<br>
//!
//! The [`stdout`], [`stdin`], and [`stderr`] of a child process can be configured by passing an [`Stdio`] to the corresponding method on [`Command`]. <br>可以通过将 [`Stdio`] 传递给 [`Command`] 上的相应方法来配置子进程的 [`stdout`]，[`stdin`] 和 [`stderr`]。<br>
//! Once spawned, they can be accessed from the [`Child`]. <br>生成后，可以从 [`Child`] 访问它们。<br>
//! For example, piping output from one command into another command can be done like so: <br>例如，可以将一个命令的输出管道输送到另一命令，如下所示：<br>
//!
//! ```no_run
//! use std::process::{Command, Stdio};
//!
//! // stdout must be configured with `Stdio::piped` in order to use `echo_child.stdout` <br>stdout 必须配置 `Stdio::piped` 才能使用 `echo_child.stdout`<br>
//! //
//! let echo_child = Command::new("echo")
//!     .arg("Oh no, a tpyo!")
//!     .stdout(Stdio::piped())
//!     .spawn()
//!     .expect("Failed to start echo process");
//!
//! // Note that `echo_child` is moved here, but we won't be needing `echo_child` anymore <br>请注意，`echo_child` 已移到此处，但我们不再需要 `echo_child`<br>
//! //
//! let echo_out = echo_child.stdout.expect("Failed to open echo stdout");
//!
//! let mut sed_child = Command::new("sed")
//!     .arg("s/tpyo/typo/")
//!     .stdin(Stdio::from(echo_out))
//!     .stdout(Stdio::piped())
//!     .spawn()
//!     .expect("Failed to start sed process");
//!
//! let output = sed_child.wait_with_output().expect("Failed to wait on sed");
//! assert_eq!(b"Oh no, a typo!\n", output.stdout.as_slice());
//! ```
//!
//! Note that [`ChildStderr`] and [`ChildStdout`] implement [`Read`] and [`ChildStdin`] implements [`Write`]: <br>请注意，[`ChildStderr`] 和 [`ChildStdout`] 实现 [`Read`]，而 [`ChildStdin`] 实现 [`Write`]：<br>
//!
//! ```no_run
//! use std::process::{Command, Stdio};
//! use std::io::Write;
//!
//! let mut child = Command::new("/bin/cat")
//!     .stdin(Stdio::piped())
//!     .stdout(Stdio::piped())
//!     .spawn()
//!     .expect("failed to execute child");
//!
//! // If the child process fills its stdout buffer, it may end up waiting until the parent reads the stdout, and not be able to read stdin in the meantime, causing a deadlock. <br>如果子进程填充了其 stdout 缓冲区，则它可能最终会等待，直到父进程读取 stdout，并且在此期间无法读取 stdin，从而导致死锁。<br>
//! //
//! // Writing from another thread ensures that stdout is being read at the same time, avoiding the problem. <br>从另一个线程进行写入可确保同时读取 stdout，从而避免了该问题。<br>
//! //
//! //
//! let mut stdin = child.stdin.take().expect("failed to get stdin");
//! std::thread::spawn(move || {
//!     stdin.write_all(b"test").expect("failed to write to stdin");
//! });
//!
//! let output = child
//!     .wait_with_output()
//!     .expect("failed to wait on child");
//!
//! assert_eq!(b"test", output.stdout.as_slice());
//! ```
//!
//! [`spawn`]: Command::spawn
//! [`output`]: Command::output
//!
//! [`stdout`]: Command::stdout
//! [`stdin`]: Command::stdin
//! [`stderr`]: Command::stderr
//!
//! [`Write`]: io::Write
//! [`Read`]: io::Read
//!
//!
//!
//!
//!
//!

#![stable(feature = "process", since = "1.0.0")]
#![deny(unsafe_op_in_unsafe_fn)]

#[cfg(all(test, not(any(target_os = "emscripten", target_env = "sgx"))))]
mod tests;

use crate::io::prelude::*;

use crate::convert::Infallible;
use crate::ffi::OsStr;
use crate::fmt;
use crate::fs;
use crate::io::{self, IoSlice, IoSliceMut};
use crate::num::NonZeroI32;
use crate::path::Path;
use crate::str;
use crate::sys::pipe::{read2, AnonPipe};
use crate::sys::process as imp;
#[stable(feature = "command_access", since = "1.57.0")]
pub use crate::sys_common::process::CommandEnvs;
use crate::sys_common::{AsInner, AsInnerMut, FromInner, IntoInner};

/// Representation of a running or exited child process. <br>表示正在运行或退出的子进程。<br>
///
/// This structure is used to represent and manage child processes. <br>该结构体用于表示和管理子进程。<br>
/// A child process is created via the [`Command`] struct, which configures the spawning process and can itself be constructed using a builder-style interface. <br>子进程是通过 [`Command`] 结构体创建的，该子进程配置了生成进程，并且可以使用生成器样式的接口本身来创建子进程。<br>
///
/// There is no implementation of [`Drop`] for child processes, so if you do not ensure the `Child` has exited then it will continue to run, even after the `Child` handle to the child process has gone out of scope. <br>子进程没有 [`Drop`] 的实现，所以如果您不确保 `Child` 已经退出，那么它会继续运行，即使在子进程的 `Child` 句柄已经离开作用域之后。<br>
///
///
/// Calling [`wait`] (or other functions that wrap around it) will make the parent process wait until the child has actually exited before continuing. <br>调用 [`wait`] (或其他环绕它的函数) 将使父进程等待直到子进程实际退出后再继续。<br>
///
/// # Warning
///
/// On some systems, calling [`wait`] or similar is necessary for the OS to release resources. <br>在某些系统上，操作系统释放资源必须调用 [`wait`] 或类似方法。<br> A process that terminated but has not been waited on is still around as a "zombie". <br>终止但尚未等待的进程仍然是 "zombie"。<br>
/// Leaving too many zombies around may exhaust global resources (for example process IDs). <br>留下太多的僵尸可能会耗尽整个资源 (例如，进程 ID)。<br>
///
/// The standard library does *not* automatically wait on child processes (not even if the `Child` is dropped), it is up to the application developer to do so. <br>标准库不会自动等待子进程 (即使 `Child` 被丢弃也不会)，这取决于应用程序开发人员。<br>
/// As a consequence, dropping `Child` handles without waiting on them first is not recommended in long-running applications. <br>因此，在长时间运行的应用程序中，不建议先丢弃 `Child` 句柄而不先等待它们。<br>
///
/// # Examples
///
/// ```should_panic
/// use std::process::Command;
///
/// let mut child = Command::new("/bin/cat")
///                         .arg("file.txt")
///                         .spawn()
///                         .expect("failed to execute child");
///
/// let ecode = child.wait()
///                  .expect("failed to wait on child");
///
/// assert!(ecode.success());
/// ```
///
/// [`wait`]: Child::wait
///
///
///
///
///
///
///
///
///
///
#[stable(feature = "process", since = "1.0.0")]
pub struct Child {
    pub(crate) handle: imp::Process,

    /// The handle for writing to the child's standard input (stdin), if it has been captured. <br>写入子节点标准输入 (stdin) 的句柄 (如果已捕获)。<br>
    /// To avoid partially moving the `child` and thus blocking yourself from calling functions on `child` while using `stdin`, you might find it helpful: <br>为避免部分移动 `child` 并因此阻止您自己在使用 `stdin` 时在 `child` 上调用函数，您可能会发现它很有用：<br>
    ///
    ///
    /// ```compile_fail,E0425
    /// let stdin = child.stdin.take().unwrap();
    /// ```
    ///
    ///
    #[stable(feature = "process", since = "1.0.0")]
    pub stdin: Option<ChildStdin>,

    /// The handle for reading from the child's standard output (stdout), if it has been captured. <br>从子节点的标准输出 (stdout) 读取的句柄 (如果已捕获)。<br>
    /// You might find it helpful to do <br>您可能会发现这样做很有帮助<br>
    ///
    /// ```compile_fail,E0425
    /// let stdout = child.stdout.take().unwrap();
    /// ```
    ///
    /// to avoid partially moving the `child` and thus blocking yourself from calling functions on `child` while using `stdout`. <br>为了避免部分移动 `child`，从而在使用 `stdout` 时阻止自己在 `child` 上调用函数。<br>
    ///
    #[stable(feature = "process", since = "1.0.0")]
    pub stdout: Option<ChildStdout>,

    /// The handle for reading from the child's standard error (stderr), if it has been captured. <br>从子节点的标准错误 (stderr) 读取的句柄 (如果已捕获)。<br>
    /// You might find it helpful to do <br>您可能会发现这样做很有帮助<br>
    ///
    /// ```compile_fail,E0425
    /// let stderr = child.stderr.take().unwrap();
    /// ```
    ///
    /// to avoid partially moving the `child` and thus blocking yourself from calling functions on `child` while using `stderr`. <br>为了避免部分移动 `child`，从而在使用 `stderr` 时阻止自己在 `child` 上调用函数。<br>
    ///
    #[stable(feature = "process", since = "1.0.0")]
    pub stderr: Option<ChildStderr>,
}

/// Allows extension traits within `std`. <br>允许在 `std` 内扩展 traits。<br>
#[unstable(feature = "sealed", issue = "none")]
impl crate::sealed::Sealed for Child {}

impl AsInner<imp::Process> for Child {
    fn as_inner(&self) -> &imp::Process {
        &self.handle
    }
}

impl FromInner<(imp::Process, imp::StdioPipes)> for Child {
    fn from_inner((handle, io): (imp::Process, imp::StdioPipes)) -> Child {
        Child {
            handle,
            stdin: io.stdin.map(ChildStdin::from_inner),
            stdout: io.stdout.map(ChildStdout::from_inner),
            stderr: io.stderr.map(ChildStderr::from_inner),
        }
    }
}

impl IntoInner<imp::Process> for Child {
    fn into_inner(self) -> imp::Process {
        self.handle
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Child {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Child")
            .field("stdin", &self.stdin)
            .field("stdout", &self.stdout)
            .field("stderr", &self.stderr)
            .finish_non_exhaustive()
    }
}

/// A handle to a child process's standard input (stdin). <br>子进程的标准输入 (stdin) 的句柄。<br>
///
/// This struct is used in the [`stdin`] field on [`Child`]. <br>该结构体用于 [`Child`] 的 [`stdin`] 字段中。<br>
///
/// When an instance of `ChildStdin` is [dropped], the `ChildStdin`'s underlying file handle will be closed. <br>当 `ChildStdin` 的实例被 [丢弃][dropped] 时，`ChildStdin` 的底层文件句柄将关闭。<br>
/// If the child process was blocked on input prior to being dropped, it will become unblocked after dropping. <br>如果子进程在丢弃之前已在输入上被阻止，则子进程将在丢弃后变为未阻止状态。<br>
///
///
/// [`stdin`]: Child::stdin
/// [dropped]: Drop
#[stable(feature = "process", since = "1.0.0")]
pub struct ChildStdin {
    inner: AnonPipe,
}

// In addition to the `impl`s here, `ChildStdin` also has `impl`s for `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` and `AsRawFd`/`IntoRawFd`/`FromRawFd`, on Unix and WASI, and `AsHandle`/`From<OwnedHandle>`/`Into<OwnedHandle>` and `AsRawHandle`/`IntoRawHandle`/`FromRawHandle` on Windows. <br>除了这里的 impl 之外，`ChildStdin` 还有 `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` 和 `AsRawFd`/`IntoRawFd`/`FromRawFd`、Unix 和 WASI 以及 Windows 上的 `AsHandle`/`From<OwnedHandle>`/`Into<OwnedHandle>` 和 `AsRawHandle`/`IntoRawHandle`/`FromRawHandle` 的 impl。<br>
//
//
//
//

#[stable(feature = "process", since = "1.0.0")]
impl Write for ChildStdin {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        (&*self).write(buf)
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        (&*self).write_vectored(bufs)
    }

    fn is_write_vectored(&self) -> bool {
        io::Write::is_write_vectored(&&*self)
    }

    fn flush(&mut self) -> io::Result<()> {
        (&*self).flush()
    }
}

#[stable(feature = "write_mt", since = "1.48.0")]
impl Write for &ChildStdin {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.inner.write(buf)
    }

    fn write_vectored(&mut self, bufs: &[IoSlice<'_>]) -> io::Result<usize> {
        self.inner.write_vectored(bufs)
    }

    fn is_write_vectored(&self) -> bool {
        self.inner.is_write_vectored()
    }

    fn flush(&mut self) -> io::Result<()> {
        Ok(())
    }
}

impl AsInner<AnonPipe> for ChildStdin {
    fn as_inner(&self) -> &AnonPipe {
        &self.inner
    }
}

impl IntoInner<AnonPipe> for ChildStdin {
    fn into_inner(self) -> AnonPipe {
        self.inner
    }
}

impl FromInner<AnonPipe> for ChildStdin {
    fn from_inner(pipe: AnonPipe) -> ChildStdin {
        ChildStdin { inner: pipe }
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for ChildStdin {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("ChildStdin").finish_non_exhaustive()
    }
}

/// A handle to a child process's standard output (stdout). <br>子进程的标准输出 (stdout) 的句柄。<br>
///
/// This struct is used in the [`stdout`] field on [`Child`]. <br>该结构体用于 [`Child`] 的 [`stdout`] 字段中。<br>
///
/// When an instance of `ChildStdout` is [dropped], the `ChildStdout`'s underlying file handle will be closed. <br>当 `ChildStdout` 的实例被 [丢弃][dropped] 时，`ChildStdout` 的底层文件句柄将关闭。<br>
///
///
/// [`stdout`]: Child::stdout
/// [dropped]: Drop
#[stable(feature = "process", since = "1.0.0")]
pub struct ChildStdout {
    inner: AnonPipe,
}

// In addition to the `impl`s here, `ChildStdout` also has `impl`s for `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` and `AsRawFd`/`IntoRawFd`/`FromRawFd`, on Unix and WASI, and `AsHandle`/`From<OwnedHandle>`/`Into<OwnedHandle>` and `AsRawHandle`/`IntoRawHandle`/`FromRawHandle` on Windows. <br>除了这里的 impl 之外，`ChildStdout` 还有 `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` 和 `AsRawFd`/`IntoRawFd`/`FromRawFd`、Unix 和 WASI 以及 Windows 上的 `AsHandle`/`From<OwnedHandle>`/`Into<OwnedHandle>` 和 `AsRawHandle`/`IntoRawHandle`/`FromRawHandle` 的 impl。<br>
//
//
//
//

#[stable(feature = "process", since = "1.0.0")]
impl Read for ChildStdout {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.inner.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.inner.read_vectored(bufs)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.inner.is_read_vectored()
    }
}

impl AsInner<AnonPipe> for ChildStdout {
    fn as_inner(&self) -> &AnonPipe {
        &self.inner
    }
}

impl IntoInner<AnonPipe> for ChildStdout {
    fn into_inner(self) -> AnonPipe {
        self.inner
    }
}

impl FromInner<AnonPipe> for ChildStdout {
    fn from_inner(pipe: AnonPipe) -> ChildStdout {
        ChildStdout { inner: pipe }
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for ChildStdout {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("ChildStdout").finish_non_exhaustive()
    }
}

/// A handle to a child process's stderr. <br>子进程的 stderr 的句柄。<br>
///
/// This struct is used in the [`stderr`] field on [`Child`]. <br>该结构体用于 [`Child`] 的 [`stderr`] 字段中。<br>
///
/// When an instance of `ChildStderr` is [dropped], the `ChildStderr`'s underlying file handle will be closed. <br>当 `ChildStderr` 的实例被 [丢弃][dropped] 时，`ChildStderr` 的底层文件句柄将关闭。<br>
///
///
/// [`stderr`]: Child::stderr
/// [dropped]: Drop
#[stable(feature = "process", since = "1.0.0")]
pub struct ChildStderr {
    inner: AnonPipe,
}

// In addition to the `impl`s here, `ChildStderr` also has `impl`s for `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` and `AsRawFd`/`IntoRawFd`/`FromRawFd`, on Unix and WASI, and `AsHandle`/`From<OwnedHandle>`/`Into<OwnedHandle>` and `AsRawHandle`/`IntoRawHandle`/`FromRawHandle` on Windows. <br>除了这里的 impl 之外，`ChildStderr` 还有 `AsFd`/`From<OwnedFd>`/`Into<OwnedFd>` 和 `AsRawFd`/`IntoRawFd`/`FromRawFd`、Unix 和 WASI 以及 Windows 上的 `AsHandle`/`From<OwnedHandle>`/`Into<OwnedHandle>` 和 `AsRawHandle`/`IntoRawHandle`/`FromRawHandle` 的 impl。<br>
//
//
//
//

#[stable(feature = "process", since = "1.0.0")]
impl Read for ChildStderr {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        self.inner.read(buf)
    }

    fn read_vectored(&mut self, bufs: &mut [IoSliceMut<'_>]) -> io::Result<usize> {
        self.inner.read_vectored(bufs)
    }

    #[inline]
    fn is_read_vectored(&self) -> bool {
        self.inner.is_read_vectored()
    }
}

impl AsInner<AnonPipe> for ChildStderr {
    fn as_inner(&self) -> &AnonPipe {
        &self.inner
    }
}

impl IntoInner<AnonPipe> for ChildStderr {
    fn into_inner(self) -> AnonPipe {
        self.inner
    }
}

impl FromInner<AnonPipe> for ChildStderr {
    fn from_inner(pipe: AnonPipe) -> ChildStderr {
        ChildStderr { inner: pipe }
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for ChildStderr {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("ChildStderr").finish_non_exhaustive()
    }
}

/// A process builder, providing fine-grained control over how a new process should be spawned. <br>进程生成器，提供对如何生成新进程的细粒度控制。<br>
///
/// A default configuration can be generated using `Command::new(program)`, where `program` gives a path to the program to be executed. <br>可以使用 `Command::new(program)` 生成默认配置，其中 `program` 提供了要执行的程序的路径。<br>
///
/// Additional builder methods allow the configuration to be changed (for example, by adding arguments) prior to spawning: <br>其他生成器方法允许在生成之前更改配置 (例如，通过添加参数) ：<br>
///
/// ```
/// use std::process::Command;
///
/// let output = if cfg!(target_os = "windows") {
///     Command::new("cmd")
///             .args(["/C", "echo hello"])
///             .output()
///             .expect("failed to execute process")
/// } else {
///     Command::new("sh")
///             .arg("-c")
///             .arg("echo hello")
///             .output()
///             .expect("failed to execute process")
/// };
///
/// let hello = output.stdout;
/// ```
///
/// `Command` can be reused to spawn multiple processes. <br>`Command` 可以重用以生成多个进程。<br>
/// The builder methods change the command without needing to immediately spawn the process. <br>构建器方法无需立即使进程 spawn 即可更改命令。<br>
///
/// ```no_run
/// use std::process::Command;
///
/// let mut echo_hello = Command::new("sh");
/// echo_hello.arg("-c")
///           .arg("echo hello");
/// let hello_1 = echo_hello.output().expect("failed to execute process");
/// let hello_2 = echo_hello.output().expect("failed to execute process");
/// ```
///
/// Similarly, you can call builder methods after spawning a process and then spawn a new process with the modified settings. <br>同样，您可以在生成进程之后调用构建器方法，然后使用修改后的设置 spawn 新建一个进程。<br>
///
/// ```no_run
/// use std::process::Command;
///
/// let mut list_dir = Command::new("ls");
///
/// // Execute `ls` in the current directory of the program. <br>在程序的当前目录中执行 `ls`。<br>
/// list_dir.status().expect("process failed to execute");
///
/// println!();
///
/// // Change `ls` to execute in the root directory. <br>更改 `ls` 以在根目录中执行。<br>
/// list_dir.current_dir("/");
///
/// // And then execute `ls` again but in the root directory. <br>然后再次在根目录中执行 `ls`。<br>
/// list_dir.status().expect("process failed to execute");
/// ```
///
///
///
#[stable(feature = "process", since = "1.0.0")]
pub struct Command {
    inner: imp::Command,
}

/// Allows extension traits within `std`. <br>允许在 `std` 内扩展 traits。<br>
#[unstable(feature = "sealed", issue = "none")]
impl crate::sealed::Sealed for Command {}

impl Command {
    /// Constructs a new `Command` for launching the program at path `program`, with the following default configuration: <br>使用以下默认配置创建一个新的 `Command`，以在路径 `program` 处启动该程序：<br>
    ///
    /// * No arguments to the program <br>程序无参数<br>
    /// * Inherit the current process's environment <br>继承当前进程的环境<br>
    /// * Inherit the current process's working directory <br>继承当前进程的工作目录<br>
    /// * Inherit stdin/stdout/stderr for [`spawn`] or [`status`], but create pipes for [`output`] <br>为 [`spawn`] 或 [`status`] 继承 stdin/stdout/stderr，但为 [`output`] 创建管道<br>
    ///
    /// [`spawn`]: Self::spawn
    /// [`status`]: Self::status
    /// [`output`]: Self::output
    ///
    /// Builder methods are provided to change these defaults and otherwise configure the process. <br>提供了生成器方法来更改这些默认值，并以其他方式配置该进程。<br>
    ///
    /// If `program` is not an absolute path, the `PATH` will be searched in an OS-defined way. <br>如果 `program` 不是绝对路径，则将以 OS 定义的方式搜索 `PATH`。<br>
    ///
    /// The search path to be used may be controlled by setting the `PATH` environment variable on the Command, but this has some implementation limitations on Windows (see issue #37519). <br>可以通过在 Command 上设置 `PATH` 环境变量来控制要使用的搜索路径，但这在 Windows 上有一些实现上的限制 (请参见 issue #37519)。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// Command::new("sh")
    ///         .spawn()
    ///         .expect("sh command failed to start");
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "process", since = "1.0.0")]
    pub fn new<S: AsRef<OsStr>>(program: S) -> Command {
        Command { inner: imp::Command::new(program.as_ref()) }
    }

    /// Adds an argument to pass to the program. <br>添加参数以传递给程序。<br>
    ///
    /// Only one argument can be passed per use. <br>每次使用只能传递一个参数。<br> So instead of: <br>因此，而不是：<br>
    ///
    /// ```no_run
    /// # std::process::Command::new("sh")
    /// .arg("-C /path/to/repo")
    /// # ;
    /// ```
    ///
    /// usage would be: <br>用法是：<br>
    ///
    /// ```no_run
    /// # std::process::Command::new("sh")
    /// .arg("-C")
    /// .arg("/path/to/repo")
    /// # ;
    /// ```
    ///
    /// To pass multiple arguments see [`args`]. <br>要传递多个参数，请参见 [`args`]。<br>
    ///
    /// [`args`]: Command::args
    ///
    /// Note that the argument is not passed through a shell, but given literally to the program. <br>注意，该参数不是通过 shell 传递的，而是按字面意义提供给程序的。<br>
    /// This means that shell syntax like quotes, escaped characters, word splitting, glob patterns, substitution, etc. <br>这意味着 shell 语法，例如引号，转义字符，单词拆分，全局模式，替换等。<br>
    ///
    /// have no effect. <br>没有效果。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// Command::new("ls")
    ///         .arg("-l")
    ///         .arg("-a")
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn arg<S: AsRef<OsStr>>(&mut self, arg: S) -> &mut Command {
        self.inner.arg(arg.as_ref());
        self
    }

    /// Adds multiple arguments to pass to the program. <br>添加多个参数以传递给程序。<br>
    ///
    /// To pass a single argument see [`arg`]. <br>要传递单个参数，请参见 [`arg`]。<br>
    ///
    /// [`arg`]: Command::arg
    ///
    /// Note that the arguments are not passed through a shell, but given literally to the program. <br>请注意，该参数不是通过 shell 传递的，而是按字面意义提供给程序的。<br>
    /// This means that shell syntax like quotes, escaped characters, word splitting, glob patterns, substitution, etc. <br>这意味着 shell 语法，例如引号，转义字符，单词拆分，全局模式，替换等。<br>
    ///
    /// have no effect. <br>没有效果。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// Command::new("ls")
    ///         .args(["-l", "-a"])
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn args<I, S>(&mut self, args: I) -> &mut Command
    where
        I: IntoIterator<Item = S>,
        S: AsRef<OsStr>,
    {
        for arg in args {
            self.arg(arg.as_ref());
        }
        self
    }

    /// Inserts or updates an environment variable mapping. <br>插入或更新环境变量映射。<br>
    ///
    /// Note that environment variable names are case-insensitive (but case-preserving) on Windows, and case-sensitive on all other platforms. <br>请注意，环境变量名称在 Windows 上不区分大小写 (但保留大小写)，在所有其他平台上则区分大小写。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// Command::new("ls")
    ///         .env("PATH", "/bin")
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn env<K, V>(&mut self, key: K, val: V) -> &mut Command
    where
        K: AsRef<OsStr>,
        V: AsRef<OsStr>,
    {
        self.inner.env_mut().set(key.as_ref(), val.as_ref());
        self
    }

    /// Adds or updates multiple environment variable mappings. <br>添加或更新多个环境变量映射。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::{Command, Stdio};
    /// use std::env;
    /// use std::collections::HashMap;
    ///
    /// let filtered_env : HashMap<String, String> =
    ///     env::vars().filter(|&(ref k, _)|
    ///         k == "TERM" || k == "TZ" || k == "LANG" || k == "PATH"
    ///     ).collect();
    ///
    /// Command::new("printenv")
    ///         .stdin(Stdio::null())
    ///         .stdout(Stdio::inherit())
    ///         .env_clear()
    ///         .envs(&filtered_env)
    ///         .spawn()
    ///         .expect("printenv failed to start");
    /// ```
    #[stable(feature = "command_envs", since = "1.19.0")]
    pub fn envs<I, K, V>(&mut self, vars: I) -> &mut Command
    where
        I: IntoIterator<Item = (K, V)>,
        K: AsRef<OsStr>,
        V: AsRef<OsStr>,
    {
        for (ref key, ref val) in vars {
            self.inner.env_mut().set(key.as_ref(), val.as_ref());
        }
        self
    }

    /// Removes an environment variable mapping. <br>删除环境变量映射。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// Command::new("ls")
    ///         .env_remove("PATH")
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn env_remove<K: AsRef<OsStr>>(&mut self, key: K) -> &mut Command {
        self.inner.env_mut().remove(key.as_ref());
        self
    }

    /// Clears the entire environment map for the child process. <br>清除子进程的整个环境 map。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// Command::new("ls")
    ///         .env_clear()
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn env_clear(&mut self) -> &mut Command {
        self.inner.env_mut().clear();
        self
    }

    /// Sets the working directory for the child process. <br>设置子进程的工作目录。<br>
    ///
    /// # Platform-specific behavior <br>特定于平台的行为<br>
    ///
    /// If the program path is relative (e.g., `"./script.sh"`), it's ambiguous whether it should be interpreted relative to the parent's working directory or relative to `current_dir`. <br>如果程序路径是相对路径 (例如 `"./script.sh"`)，则是相对于父级工作目录还是相对于 `current_dir` 来解释路径。<br>
    /// The behavior in this case is platform specific and unstable, and it's recommended to use [`canonicalize`] to get an absolute program path instead. <br>这种情况下的行为是特定于平台且不稳定的，建议使用 [`canonicalize`] 来获取绝对程序路径。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// Command::new("ls")
    ///         .current_dir("/bin")
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    ///
    /// [`canonicalize`]: crate::fs::canonicalize
    ///
    ///
    ///
    #[stable(feature = "process", since = "1.0.0")]
    pub fn current_dir<P: AsRef<Path>>(&mut self, dir: P) -> &mut Command {
        self.inner.cwd(dir.as_ref().as_ref());
        self
    }

    /// Configuration for the child process's standard input (stdin) handle. <br>子进程的标准输入 (stdin) 句柄的配置。<br>
    ///
    /// Defaults to [`inherit`] when used with [`spawn`] or [`status`], and defaults to [`piped`] when used with [`output`]. <br>与 [`spawn`] 或 [`status`] 一起使用时，默认为 [`inherit`]，与 [`output`] 一起使用时，默认为 [`piped`]。<br>
    ///
    ///
    /// [`inherit`]: Stdio::inherit
    /// [`piped`]: Stdio::piped
    /// [`spawn`]: Self::spawn
    /// [`status`]: Self::status
    /// [`output`]: Self::output
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::{Command, Stdio};
    ///
    /// Command::new("ls")
    ///         .stdin(Stdio::null())
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn stdin<T: Into<Stdio>>(&mut self, cfg: T) -> &mut Command {
        self.inner.stdin(cfg.into().0);
        self
    }

    /// Configuration for the child process's standard output (stdout) handle. <br>子进程的标准输出 (stdout) 句柄的配置。<br>
    ///
    /// Defaults to [`inherit`] when used with [`spawn`] or [`status`], and defaults to [`piped`] when used with [`output`]. <br>与 [`spawn`] 或 [`status`] 一起使用时，默认为 [`inherit`]，与 [`output`] 一起使用时，默认为 [`piped`]。<br>
    ///
    ///
    /// [`inherit`]: Stdio::inherit
    /// [`piped`]: Stdio::piped
    /// [`spawn`]: Self::spawn
    /// [`status`]: Self::status
    /// [`output`]: Self::output
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::{Command, Stdio};
    ///
    /// Command::new("ls")
    ///         .stdout(Stdio::null())
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn stdout<T: Into<Stdio>>(&mut self, cfg: T) -> &mut Command {
        self.inner.stdout(cfg.into().0);
        self
    }

    /// Configuration for the child process's standard error (stderr) handle. <br>子进程的标准错误 (stderr) 句柄的配置。<br>
    ///
    /// Defaults to [`inherit`] when used with [`spawn`] or [`status`], and defaults to [`piped`] when used with [`output`]. <br>与 [`spawn`] 或 [`status`] 一起使用时，默认为 [`inherit`]，与 [`output`] 一起使用时，默认为 [`piped`]。<br>
    ///
    ///
    /// [`inherit`]: Stdio::inherit
    /// [`piped`]: Stdio::piped
    /// [`spawn`]: Self::spawn
    /// [`status`]: Self::status
    /// [`output`]: Self::output
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::{Command, Stdio};
    ///
    /// Command::new("ls")
    ///         .stderr(Stdio::null())
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn stderr<T: Into<Stdio>>(&mut self, cfg: T) -> &mut Command {
        self.inner.stderr(cfg.into().0);
        self
    }

    /// Executes the command as a child process, returning a handle to it. <br>将命令作为子进程执行，并返回其句柄。<br>
    ///
    /// By default, stdin, stdout and stderr are inherited from the parent. <br>默认情况下，stdin、stdout 和 stderr 都是从父级继承的。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// Command::new("ls")
    ///         .spawn()
    ///         .expect("ls command failed to start");
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn spawn(&mut self) -> io::Result<Child> {
        self.inner.spawn(imp::Stdio::Inherit, true).map(Child::from_inner)
    }

    /// Executes the command as a child process, waiting for it to finish and collecting all of its output. <br>将命令作为子进程执行，等待其完成并收集所有输出。<br>
    ///
    /// By default, stdout and stderr are captured (and used to provide the resulting output). <br>默认情况下，将捕获 stdout 和 stderr (并用于提供结果输出)。<br>
    /// Stdin is not inherited from the parent and any attempt by the child process to read from the stdin stream will result in the stream immediately closing. <br>Stdin 不是从父级继承的，子进程尝试从 stdin 流中进行读取的任何尝试都将导致该流立即关闭。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```should_panic
    /// use std::process::Command;
    /// use std::io::{self, Write};
    /// let output = Command::new("/bin/cat")
    ///                      .arg("file.txt")
    ///                      .output()
    ///                      .expect("failed to execute process");
    ///
    /// println!("status: {}", output.status);
    /// io::stdout().write_all(&output.stdout).unwrap();
    /// io::stderr().write_all(&output.stderr).unwrap();
    ///
    /// assert!(output.status.success());
    /// ```
    ///
    ///
    #[stable(feature = "process", since = "1.0.0")]
    pub fn output(&mut self) -> io::Result<Output> {
        self.inner
            .spawn(imp::Stdio::MakePipe, false)
            .map(Child::from_inner)
            .and_then(|p| p.wait_with_output())
    }

    /// Executes a command as a child process, waiting for it to finish and collecting its status. <br>将命令作为子进程执行，等待其完成并收集其状态。<br>
    ///
    ///
    /// By default, stdin, stdout and stderr are inherited from the parent. <br>默认情况下，stdin、stdout 和 stderr 都是从父级继承的。<br>
    ///
    /// # Examples
    ///
    /// ```should_panic
    /// use std::process::Command;
    ///
    /// let status = Command::new("/bin/cat")
    ///                      .arg("file.txt")
    ///                      .status()
    ///                      .expect("failed to execute process");
    ///
    /// println!("process finished with: {status}");
    ///
    /// assert!(status.success());
    /// ```
    #[stable(feature = "process", since = "1.0.0")]
    pub fn status(&mut self) -> io::Result<ExitStatus> {
        self.inner
            .spawn(imp::Stdio::Inherit, true)
            .map(Child::from_inner)
            .and_then(|mut p| p.wait())
    }

    /// Returns the path to the program that was given to [`Command::new`]. <br>返回给 [`Command::new`] 的程序的路径。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::process::Command;
    ///
    /// let cmd = Command::new("echo");
    /// assert_eq!(cmd.get_program(), "echo");
    /// ```
    #[must_use]
    #[stable(feature = "command_access", since = "1.57.0")]
    pub fn get_program(&self) -> &OsStr {
        self.inner.get_program()
    }

    /// Returns an iterator of the arguments that will be passed to the program. <br>返回将传递给程序的参数的迭代器。<br>
    ///
    /// This does not include the path to the program as the first argument; <br>这不包括程序的路径作为第一个参数；<br>
    /// it only includes the arguments specified with [`Command::arg`] and [`Command::args`]. <br>它仅包含 [`Command::arg`] 和 [`Command::args`] 指定的参数。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::ffi::OsStr;
    /// use std::process::Command;
    ///
    /// let mut cmd = Command::new("echo");
    /// cmd.arg("first").arg("second");
    /// let args: Vec<&OsStr> = cmd.get_args().collect();
    /// assert_eq!(args, &["first", "second"]);
    /// ```
    #[stable(feature = "command_access", since = "1.57.0")]
    pub fn get_args(&self) -> CommandArgs<'_> {
        CommandArgs { inner: self.inner.get_args() }
    }

    /// Returns an iterator of the environment variables that will be set when the process is spawned. <br>返回将在生成进程时设置的环境变量的迭代器。<br>
    ///
    /// Each element is a tuple `(&OsStr, Option<&OsStr>)`, where the first value is the key, and the second is the value, which is [`None`] if the environment variable is to be explicitly removed. <br>每个元素都是一个元组 `(&OsStr, Option<&OsStr>)`，其中第一个值为键，第二个为值，如果要显式删除环境变量，则为 [`None`]。<br>
    ///
    ///
    /// This only includes environment variables explicitly set with [`Command::env`], [`Command::envs`], and [`Command::env_remove`]. <br>这仅包括用 [`Command::env`]，[`Command::envs`] 和 [`Command::env_remove`] 显式设置的环境变量。<br>
    /// It does not include environment variables that will be inherited by the child process. <br>它不包括子进程将继承的环境变量。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::ffi::OsStr;
    /// use std::process::Command;
    ///
    /// let mut cmd = Command::new("ls");
    /// cmd.env("TERM", "dumb").env_remove("TZ");
    /// let envs: Vec<(&OsStr, Option<&OsStr>)> = cmd.get_envs().collect();
    /// assert_eq!(envs, &[
    ///     (OsStr::new("TERM"), Some(OsStr::new("dumb"))),
    ///     (OsStr::new("TZ"), None)
    /// ]);
    /// ```
    ///
    ///
    ///
    ///
    #[stable(feature = "command_access", since = "1.57.0")]
    pub fn get_envs(&self) -> CommandEnvs<'_> {
        self.inner.get_envs()
    }

    /// Returns the working directory for the child process. <br>返回子进程的工作目录。<br>
    ///
    /// This returns [`None`] if the working directory will not be changed. <br>如果不更改工作目录，则返回 [`None`]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::path::Path;
    /// use std::process::Command;
    ///
    /// let mut cmd = Command::new("ls");
    /// assert_eq!(cmd.get_current_dir(), None);
    /// cmd.current_dir("/bin");
    /// assert_eq!(cmd.get_current_dir(), Some(Path::new("/bin")));
    /// ```
    #[must_use]
    #[stable(feature = "command_access", since = "1.57.0")]
    pub fn get_current_dir(&self) -> Option<&Path> {
        self.inner.get_current_dir()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for Command {
    /// Format the program and arguments of a Command for display. <br>格式化命令的程序和参数以进行显示。<br>
    /// Any non-utf8 data is lossily converted using the utf8 replacement character. <br>使用 utf8 替换字符有损地转换所有非 utf8 数据。<br>
    ///
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.inner.fmt(f)
    }
}

impl AsInner<imp::Command> for Command {
    fn as_inner(&self) -> &imp::Command {
        &self.inner
    }
}

impl AsInnerMut<imp::Command> for Command {
    fn as_inner_mut(&mut self) -> &mut imp::Command {
        &mut self.inner
    }
}

/// An iterator over the command arguments. <br>命令的迭代器。<br>
///
/// This struct is created by [`Command::get_args`]. <br>该结构体由 [`Command::get_args`] 创建。<br>
/// See its documentation for more. <br>有关更多信息，请参见其文档。<br>
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[stable(feature = "command_access", since = "1.57.0")]
#[derive(Debug)]
pub struct CommandArgs<'a> {
    inner: imp::CommandArgs<'a>,
}

#[stable(feature = "command_access", since = "1.57.0")]
impl<'a> Iterator for CommandArgs<'a> {
    type Item = &'a OsStr;
    fn next(&mut self) -> Option<&'a OsStr> {
        self.inner.next()
    }
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.inner.size_hint()
    }
}

#[stable(feature = "command_access", since = "1.57.0")]
impl<'a> ExactSizeIterator for CommandArgs<'a> {
    fn len(&self) -> usize {
        self.inner.len()
    }
    fn is_empty(&self) -> bool {
        self.inner.is_empty()
    }
}

/// The output of a finished process. <br>完成的进程的输出。<br>
///
/// This is returned in a Result by either the [`output`] method of a [`Command`], or the [`wait_with_output`] method of a [`Child`] process. <br>[`Command`] 的 [`output`] 方法或 [`Child`] 进程的 [`wait_with_output`] 方法在结果中返回该值。<br>
///
///
/// [`output`]: Command::output
/// [`wait_with_output`]: Child::wait_with_output
///
#[derive(PartialEq, Eq, Clone)]
#[stable(feature = "process", since = "1.0.0")]
pub struct Output {
    /// The status (exit code) of the process. <br>进程的状态 (退出代码)。<br>
    #[stable(feature = "process", since = "1.0.0")]
    pub status: ExitStatus,
    /// The data that the process wrote to stdout. <br>进程写入 stdout 的数据。<br>
    #[stable(feature = "process", since = "1.0.0")]
    pub stdout: Vec<u8>,
    /// The data that the process wrote to stderr. <br>进程写入 stderr 的数据。<br>
    #[stable(feature = "process", since = "1.0.0")]
    pub stderr: Vec<u8>,
}

// If either stderr or stdout are valid utf8 strings it prints the valid strings, otherwise it prints the byte sequence instead <br>如果 stderr 或 stdout 是有效的 utf8 字符串，它将打印有效的字符串，否则将打印字节序列。<br>
//
#[stable(feature = "process_output_debug", since = "1.7.0")]
impl fmt::Debug for Output {
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        let stdout_utf8 = str::from_utf8(&self.stdout);
        let stdout_debug: &dyn fmt::Debug = match stdout_utf8 {
            Ok(ref str) => str,
            Err(_) => &self.stdout,
        };

        let stderr_utf8 = str::from_utf8(&self.stderr);
        let stderr_debug: &dyn fmt::Debug = match stderr_utf8 {
            Ok(ref str) => str,
            Err(_) => &self.stderr,
        };

        fmt.debug_struct("Output")
            .field("status", &self.status)
            .field("stdout", stdout_debug)
            .field("stderr", stderr_debug)
            .finish()
    }
}

/// Describes what to do with a standard I/O stream for a child process when passed to the [`stdin`], [`stdout`], and [`stderr`] methods of [`Command`]. <br>描述当传递给 [`Command`] 的 [`stdin`]，[`stdout`] 和 [`stderr`] 方法时，如何对子进程使用标准 I/O 流。<br>
///
///
/// [`stdin`]: Command::stdin
/// [`stdout`]: Command::stdout
/// [`stderr`]: Command::stderr
#[stable(feature = "process", since = "1.0.0")]
pub struct Stdio(imp::Stdio);

impl Stdio {
    /// A new pipe should be arranged to connect the parent and child processes. <br>应该安排一个新管道来连接父进程和子进程。<br>
    ///
    /// # Examples
    ///
    /// With stdout: <br>使用 stdout：<br>
    ///
    /// ```no_run
    /// use std::process::{Command, Stdio};
    ///
    /// let output = Command::new("echo")
    ///     .arg("Hello, world!")
    ///     .stdout(Stdio::piped())
    ///     .output()
    ///     .expect("Failed to execute command");
    ///
    /// assert_eq!(String::from_utf8_lossy(&output.stdout), "Hello, world!\n");
    /// // Nothing echoed to console <br>控制台没有回显<br>
    /// ```
    ///
    /// With stdin: <br>使用 stdin：<br>
    ///
    /// ```no_run
    /// use std::io::Write;
    /// use std::process::{Command, Stdio};
    ///
    /// let mut child = Command::new("rev")
    ///     .stdin(Stdio::piped())
    ///     .stdout(Stdio::piped())
    ///     .spawn()
    ///     .expect("Failed to spawn child process");
    ///
    /// let mut stdin = child.stdin.take().expect("Failed to open stdin");
    /// std::thread::spawn(move || {
    ///     stdin.write_all("Hello, world!".as_bytes()).expect("Failed to write to stdin");
    /// });
    ///
    /// let output = child.wait_with_output().expect("Failed to read stdout");
    /// assert_eq!(String::from_utf8_lossy(&output.stdout), "!dlrow ,olleH");
    /// ```
    ///
    /// Writing more than a pipe buffer's worth of input to stdin without also reading stdout and stderr at the same time may cause a deadlock. <br>在不同时读取 stdout 和 stderr 的情况下，向 stdin 写入超过管道缓冲区的输入值可能会导致死锁。<br>
    /// This is an issue when running any program that doesn't guarantee that it reads its entire stdin before writing more than a pipe buffer's worth of output. <br>这在运行任何不能保证在写入超过管道缓冲区的输出值之前不能保证读取其整个 stdin 的程序时就是一个问题。<br>
    ///
    /// The size of a pipe buffer varies on different targets. <br>管道缓冲区的大小在不同的目标上有所不同。<br>
    ///
    ///
    #[must_use]
    #[stable(feature = "process", since = "1.0.0")]
    pub fn piped() -> Stdio {
        Stdio(imp::Stdio::MakePipe)
    }

    /// The child inherits from the corresponding parent descriptor. <br>子级从相应的父级描述符继承。<br>
    ///
    /// # Examples
    ///
    /// With stdout: <br>使用 stdout：<br>
    ///
    /// ```no_run
    /// use std::process::{Command, Stdio};
    ///
    /// let output = Command::new("echo")
    ///     .arg("Hello, world!")
    ///     .stdout(Stdio::inherit())
    ///     .output()
    ///     .expect("Failed to execute command");
    ///
    /// assert_eq!(String::from_utf8_lossy(&output.stdout), "");
    /// // "Hello, world!" echoed to console <br>"Hello, world!" 回显到控制台<br>
    /// ```
    ///
    /// With stdin: <br>使用 stdin：<br>
    ///
    /// ```no_run
    /// use std::process::{Command, Stdio};
    /// use std::io::{self, Write};
    ///
    /// let output = Command::new("rev")
    ///     .stdin(Stdio::inherit())
    ///     .stdout(Stdio::piped())
    ///     .output()
    ///     .expect("Failed to execute command");
    ///
    /// print!("You piped in the reverse of: ");
    /// io::stdout().write_all(&output.stdout).unwrap();
    /// ```
    #[must_use]
    #[stable(feature = "process", since = "1.0.0")]
    pub fn inherit() -> Stdio {
        Stdio(imp::Stdio::Inherit)
    }

    /// This stream will be ignored. <br>此流将被忽略。<br>
    /// This is the equivalent of attaching the stream to `/dev/null`. <br>这等效于将流附加到 `/dev/null`。<br>
    ///
    /// # Examples
    ///
    /// With stdout: <br>使用 stdout：<br>
    ///
    /// ```no_run
    /// use std::process::{Command, Stdio};
    ///
    /// let output = Command::new("echo")
    ///     .arg("Hello, world!")
    ///     .stdout(Stdio::null())
    ///     .output()
    ///     .expect("Failed to execute command");
    ///
    /// assert_eq!(String::from_utf8_lossy(&output.stdout), "");
    /// // Nothing echoed to console <br>控制台没有回显<br>
    /// ```
    ///
    /// With stdin: <br>使用 stdin：<br>
    ///
    /// ```no_run
    /// use std::process::{Command, Stdio};
    ///
    /// let output = Command::new("rev")
    ///     .stdin(Stdio::null())
    ///     .stdout(Stdio::piped())
    ///     .output()
    ///     .expect("Failed to execute command");
    ///
    /// assert_eq!(String::from_utf8_lossy(&output.stdout), "");
    /// // Ignores any piped-in input <br>忽略任何管道输入<br>
    /// ```
    #[must_use]
    #[stable(feature = "process", since = "1.0.0")]
    pub fn null() -> Stdio {
        Stdio(imp::Stdio::Null)
    }

    /// Returns `true` if this requires [`Command`] to create a new pipe. <br>如果这需要 [`Command`] 创建新管道，则返回 `true`。<br>
    ///
    /// # Example
    ///
    /// ```
    /// #![feature(stdio_makes_pipe)]
    /// use std::process::Stdio;
    ///
    /// let io = Stdio::piped();
    /// assert_eq!(io.makes_pipe(), true);
    /// ```
    #[unstable(feature = "stdio_makes_pipe", issue = "98288")]
    pub fn makes_pipe(&self) -> bool {
        matches!(self.0, imp::Stdio::MakePipe)
    }
}

impl FromInner<imp::Stdio> for Stdio {
    fn from_inner(inner: imp::Stdio) -> Stdio {
        Stdio(inner)
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Stdio {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Stdio").finish_non_exhaustive()
    }
}

#[stable(feature = "stdio_from", since = "1.20.0")]
impl From<ChildStdin> for Stdio {
    /// Converts a [`ChildStdin`] into a [`Stdio`]. <br>将 [`ChildStdin`] 转换为 [`Stdio`]。<br>
    ///
    /// # Examples
    ///
    /// `ChildStdin` will be converted to `Stdio` using `Stdio::from` under the hood. <br>`ChildStdin` 将在引擎盖下使用 `Stdio::from` 转换为 `Stdio`。<br>
    ///
    /// ```rust,no_run
    /// use std::process::{Command, Stdio};
    ///
    /// let reverse = Command::new("rev")
    ///     .stdin(Stdio::piped())
    ///     .spawn()
    ///     .expect("failed reverse command");
    ///
    /// let _echo = Command::new("echo")
    ///     .arg("Hello, world!")
    ///     .stdout(reverse.stdin.unwrap()) // Converted into a Stdio here <br>在此处转换为 Stdio<br>
    ///     .output()
    ///     .expect("failed echo command");
    ///
    /// // "!dlrow ,olleH" echoed to console <br>"!dlrow ,olleH" 回显到控制台<br>
    /// ```
    fn from(child: ChildStdin) -> Stdio {
        Stdio::from_inner(child.into_inner().into())
    }
}

#[stable(feature = "stdio_from", since = "1.20.0")]
impl From<ChildStdout> for Stdio {
    /// Converts a [`ChildStdout`] into a [`Stdio`]. <br>将 [`ChildStdout`] 转换为 [`Stdio`]。<br>
    ///
    /// # Examples
    ///
    /// `ChildStdout` will be converted to `Stdio` using `Stdio::from` under the hood. <br>`ChildStdout` 将在引擎盖下使用 `Stdio::from` 转换为 `Stdio`。<br>
    ///
    /// ```rust,no_run
    /// use std::process::{Command, Stdio};
    ///
    /// let hello = Command::new("echo")
    ///     .arg("Hello, world!")
    ///     .stdout(Stdio::piped())
    ///     .spawn()
    ///     .expect("failed echo command");
    ///
    /// let reverse = Command::new("rev")
    ///     .stdin(hello.stdout.unwrap())  // Converted into a Stdio here <br>在此处转换为 Stdio<br>
    ///     .output()
    ///     .expect("failed reverse command");
    ///
    /// assert_eq!(reverse.stdout, b"!dlrow ,olleH\n");
    /// ```
    fn from(child: ChildStdout) -> Stdio {
        Stdio::from_inner(child.into_inner().into())
    }
}

#[stable(feature = "stdio_from", since = "1.20.0")]
impl From<ChildStderr> for Stdio {
    /// Converts a [`ChildStderr`] into a [`Stdio`]. <br>将 [`ChildStderr`] 转换为 [`Stdio`]。<br>
    ///
    /// # Examples
    ///
    /// ```rust,no_run
    /// use std::process::{Command, Stdio};
    ///
    /// let reverse = Command::new("rev")
    ///     .arg("non_existing_file.txt")
    ///     .stderr(Stdio::piped())
    ///     .spawn()
    ///     .expect("failed reverse command");
    ///
    /// let cat = Command::new("cat")
    ///     .arg("-")
    ///     .stdin(reverse.stderr.unwrap()) // Converted into a Stdio here <br>在此处转换为 Stdio<br>
    ///     .output()
    ///     .expect("failed echo command");
    ///
    /// assert_eq!(
    ///     String::from_utf8_lossy(&cat.stdout),
    ///     "rev: cannot open non_existing_file.txt: No such file or directory\n"
    /// );
    /// ```
    fn from(child: ChildStderr) -> Stdio {
        Stdio::from_inner(child.into_inner().into())
    }
}

#[stable(feature = "stdio_from", since = "1.20.0")]
impl From<fs::File> for Stdio {
    /// Converts a [`File`](fs::File) into a [`Stdio`]. <br>将 [`File`](fs::File) 转换为 [`Stdio`]。<br>
    ///
    /// # Examples
    ///
    /// `File` will be converted to `Stdio` using `Stdio::from` under the hood. <br>`File` 将在引擎盖下使用 `Stdio::from` 转换为 `Stdio`。<br>
    ///
    /// ```rust,no_run
    /// use std::fs::File;
    /// use std::process::Command;
    ///
    /// // With the `foo.txt` file containing `Hello, world!" <br>使用包含 `Hello，world! ` 的 `foo.txt` 文件。<br>
    /// let file = File::open("foo.txt").unwrap();
    ///
    /// let reverse = Command::new("rev")
    ///     .stdin(file)  // Implicit File conversion into a Stdio <br>隐式文件转换为 Stdio<br>
    ///     .output()
    ///     .expect("failed reverse command");
    ///
    /// assert_eq!(reverse.stdout, b"!dlrow ,olleH");
    /// ```
    fn from(file: fs::File) -> Stdio {
        Stdio::from_inner(file.into_inner().into())
    }
}

/// Describes the result of a process after it has terminated. <br>描述进程终止后的结果。<br>
///
/// This `struct` is used to represent the exit status or other termination of a child process. <br>`struct` 用于表示子进程的退出状态或其他终止。<br>
/// Child processes are created via the [`Command`] struct and their exit status is exposed through the [`status`] method, or the [`wait`] method of a [`Child`] process. <br>子进程是通过 [`Command`] 结构体创建的，其退出状态通过 [`status`] 方法或 [`Child`] 进程的 [`wait`] 方法公开。<br>
///
/// An `ExitStatus` represents every possible disposition of a process. <br>`ExitStatus` 表示进程的每一种可能的配置。<br> On Unix this is the **wait status**. <br>在 Unix 上，这是等待状态。<br> It is *not* simply an *exit status* (a value passed to `exit`). <br>*不是* 仅仅是 *exit 状态*(传递给 `exit` 的值)。<br>
///
/// For proper error reporting of failed processes, print the value of `ExitStatus` or `ExitStatusError` using their implementations of [`Display`](crate::fmt::Display). <br>为了正确报告失败进程的错误，请使用 [`Display`](crate::fmt::Display) 的实现打印 `ExitStatus` 或 `ExitStatusError` 的值。<br>
///
/// # Differences from `ExitCode` <br>与 `ExitCode` 的区别<br>
///
/// [`ExitCode`] is intended for terminating the currently running process, via the `Termination` trait, in contrast to `ExitStatus`, which represents the termination of a child process. <br>[`ExitCode`] 旨在通过 `Termination` trait 终止当前正在运行的进程，而 `ExitStatus` 则表示终止子进程。<br>
/// These APIs are separate due to platform compatibility differences and their expected usage; <br>由于平台兼容性差异及其预期用途，这些 API 是分开的;<br> it is not generally possible to exactly reproduce an `ExitStatus` from a child for the current process after the fact. <br>事后通常不可能从子节点那里准确地复制当前进程的 `ExitStatus`。<br>
///
///
/// [`status`]: Command::status
/// [`wait`]: Child::wait
///
///
///
///
///
///
///
// We speak slightly loosely (here and in various other places in the stdlib docs) about `exit` vs `_exit`. <br>我们对 `exit` 和 `_exit` 的讨论有点松散 (这里和 stdlib 文档中的其他地方)。<br>
// Naming of Unix system calls is not standardised across Unices, so terminology is a matter of convention and tradition. <br>Unix 系统调用的命名在 Unices 中没有标准化，因此术语是约定俗成的问题。<br>
//
// For clarity we usually speak of `exit`, even when we might mean an underlying system call such as `_exit`. <br>为清楚起见，我们通常使用 `exit`，即使我们可能指的是底层系统调用，例如 `_exit`。<br>
//
#[derive(PartialEq, Eq, Clone, Copy, Debug)]
#[stable(feature = "process", since = "1.0.0")]
pub struct ExitStatus(imp::ExitStatus);

/// Allows extension traits within `std`. <br>允许在 `std` 内扩展 traits。<br>
#[unstable(feature = "sealed", issue = "none")]
impl crate::sealed::Sealed for ExitStatus {}

impl ExitStatus {
    /// Was termination successful? <br>终止成功了吗？<br>  Returns a `Result`. <br>返回 `Result`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(exit_status_error)]
    /// # if cfg!(unix) {
    /// use std::process::Command;
    ///
    /// let status = Command::new("ls")
    ///                      .arg("/dev/nonexistent")
    ///                      .status()
    ///                      .expect("ls could not be executed");
    ///
    /// println!("ls: {status}");
    /// status.exit_ok().expect_err("/dev/nonexistent could be listed!");
    /// # } // cfg!(unix)
    /// ```
    #[unstable(feature = "exit_status_error", issue = "84908")]
    pub fn exit_ok(&self) -> Result<(), ExitStatusError> {
        self.0.exit_ok().map_err(ExitStatusError)
    }

    /// Was termination successful? <br>终止成功了吗？<br> Signal termination is not considered a success, and success is defined as a zero exit status. <br>信号终止不被视为成功，并且成功被定义为零退出状态。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```rust,no_run
    /// use std::process::Command;
    ///
    /// let status = Command::new("mkdir")
    ///                      .arg("projects")
    ///                      .status()
    ///                      .expect("failed to execute mkdir");
    ///
    /// if status.success() {
    ///     println!("'projects/' directory created");
    /// } else {
    ///     println!("failed to create 'projects/' directory: {status}");
    /// }
    /// ```
    #[must_use]
    #[stable(feature = "process", since = "1.0.0")]
    pub fn success(&self) -> bool {
        self.0.exit_ok().is_ok()
    }

    /// Returns the exit code of the process, if any. <br>返回进程的退出代码 (如果有)。<br>
    ///
    /// In Unix terms the return value is the **exit status**: the value passed to `exit`, if the process finished by calling `exit`. <br>用 Unix 的术语来说，返回值是退出状态：如果进程通过调用 `exit` 完成，则传递给 `exit` 的值。<br>
    /// Note that on Unix the exit status is truncated to 8 bits, and that values that didn't come from a program's call to `exit` may be invented by the runtime system (often, for example, 255, 254, 127 or 126). <br>请注意，在 Unix 上，退出状态被截断为 8 位，并且不是来自程序调用到 `exit` 的值可能是由运行时系统发明的 (通常，例如，255、254、127 或 126)。<br>
    ///
    ///
    /// On Unix, this will return `None` if the process was terminated by a signal. <br>在 Unix 上，如果进程被信号终止，它将返回 `None`。<br>
    /// [`ExitStatusExt`](crate::os::unix::process::ExitStatusExt) is an extension trait for extracting any such signal, and other details, from the `ExitStatus`. <br>[`ExitStatusExt`](crate::os::unix::process::ExitStatusExt) 是 trait 的扩展，用于从 `ExitStatus` 中提取任何此类信号和其他细节。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// let status = Command::new("mkdir")
    ///                      .arg("projects")
    ///                      .status()
    ///                      .expect("failed to execute mkdir");
    ///
    /// match status.code() {
    ///     Some(code) => println!("Exited with status code: {code}"),
    ///     None       => println!("Process terminated by signal")
    /// }
    /// ```
    ///
    ///
    #[must_use]
    #[stable(feature = "process", since = "1.0.0")]
    pub fn code(&self) -> Option<i32> {
        self.0.code()
    }
}

impl AsInner<imp::ExitStatus> for ExitStatus {
    fn as_inner(&self) -> &imp::ExitStatus {
        &self.0
    }
}

impl FromInner<imp::ExitStatus> for ExitStatus {
    fn from_inner(s: imp::ExitStatus) -> ExitStatus {
        ExitStatus(s)
    }
}

#[stable(feature = "process", since = "1.0.0")]
impl fmt::Display for ExitStatus {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.0.fmt(f)
    }
}

/// Allows extension traits within `std`. <br>允许在 `std` 内扩展 traits。<br>
#[unstable(feature = "sealed", issue = "none")]
impl crate::sealed::Sealed for ExitStatusError {}

/// Describes the result of a process after it has failed <br>描述进程失败后的结果<br>
///
/// Produced by the [`.exit_ok`](ExitStatus::exit_ok) method on [`ExitStatus`]. <br>通过 [`ExitStatus`] 上的 [`.exit_ok`](ExitStatus::exit_ok) 方法生成。<br>
///
/// # Examples
///
/// ```
/// #![feature(exit_status_error)]
/// # if cfg!(unix) {
/// use std::process::{Command, ExitStatusError};
///
/// fn run(cmd: &str) -> Result<(),ExitStatusError> {
///     Command::new(cmd).status().unwrap().exit_ok()?;
///     Ok(())
/// }
///
/// run("true").unwrap();
/// run("false").unwrap_err();
/// # } // cfg!(unix)
/// ```
#[derive(PartialEq, Eq, Clone, Copy, Debug)]
#[unstable(feature = "exit_status_error", issue = "84908")]
// The definition of imp::ExitStatusError should ideally be such that Result<(), imp::ExitStatusError> has an identical representation to imp::ExitStatus. <br>理想情况下，imp::ExitStatusError 的定义应该是 Result<(), imp::ExitStatusError> 与 imp::ExitStatus 具有相同的表示。<br>
//
pub struct ExitStatusError(imp::ExitStatusError);

#[unstable(feature = "exit_status_error", issue = "84908")]
impl ExitStatusError {
    /// Reports the exit code, if applicable, from an `ExitStatusError`. <br>从 `ExitStatusError` 报告退出代码 (如果适用)。<br>
    ///
    /// In Unix terms the return value is the **exit status**: the value passed to `exit`, if the process finished by calling `exit`. <br>用 Unix 的术语来说，返回值是退出状态：如果进程通过调用 `exit` 完成，则传递给 `exit` 的值。<br>
    /// Note that on Unix the exit status is truncated to 8 bits, and that values that didn't come from a program's call to `exit` may be invented by the runtime system (often, for example, 255, 254, 127 or 126). <br>请注意，在 Unix 上，退出状态被截断为 8 位，并且不是来自程序调用到 `exit` 的值可能是由运行时系统发明的 (通常，例如，255、254、127 或 126)。<br>
    ///
    ///
    /// On Unix, this will return `None` if the process was terminated by a signal. <br>在 Unix 上，如果进程被信号终止，它将返回 `None`。<br>
    /// If you want to handle such situations specially, consider using methods from [`ExitStatusExt`](crate::os::unix::process::ExitStatusExt). <br>如果您想专门处理这种情况，请考虑使用 [`ExitStatusExt`](crate::os::unix::process::ExitStatusExt) 中的方法。<br>
    ///
    /// If the process finished by calling `exit` with a nonzero value, this will return that exit status. <br>如果进程通过使用非零值调用 `exit` 完成，这将返回该退出状态。<br>
    ///
    /// If the error was something else, it will return `None`. <br>如果错误是其他原因，它将返回 `None`。<br>
    ///
    /// If the process exited successfully (ie, by calling `exit(0)`), there is no `ExitStatusError`. <br>如果进程成功退出 (即通过调用 `exit(0)`)，则没有 `ExitStatusError`。<br> So the return value from `ExitStatusError::code()` is always nonzero. <br>所以 `ExitStatusError::code()` 的返回值总是非零。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(exit_status_error)]
    /// # #[cfg(unix)] {
    /// use std::process::Command;
    ///
    /// let bad = Command::new("false").status().unwrap().exit_ok().unwrap_err();
    /// assert_eq!(bad.code(), Some(1));
    /// # } // #[cfg(unix)]
    /// ```
    ///
    ///
    ///
    ///
    #[must_use]
    pub fn code(&self) -> Option<i32> {
        self.code_nonzero().map(Into::into)
    }

    /// Reports the exit code, if applicable, from an `ExitStatusError`, as a `NonZero` <br>将 `ExitStatusError` 的退出代码 (如果适用) 报告为 `NonZero`<br>
    ///
    /// This is exactly like [`code()`](Self::code), except that it returns a `NonZeroI32`. <br>这与 [`code()`](Self::code) 完全一样，只是它返回一个 `NonZeroI32`。<br>
    ///
    /// Plain `code`, returning a plain integer, is provided because is is often more convenient. <br>提供普通 `code`，返回一个普通整数，因为它通常更方便。<br>
    /// The returned value from `code()` is indeed also nonzero; <br>`code()` 的返回值确实也是非零的；<br> use `code_nonzero()` when you want a type-level guarantee of nonzeroness. <br>当您想要非零的类型级别保证时，请使用 `code_nonzero()`。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(exit_status_error)]
    /// # if cfg!(unix) {
    /// use std::num::NonZeroI32;
    /// use std::process::Command;
    ///
    /// let bad = Command::new("false").status().unwrap().exit_ok().unwrap_err();
    /// assert_eq!(bad.code_nonzero().unwrap(), NonZeroI32::try_from(1).unwrap());
    /// # } // cfg!(unix)
    /// ```
    #[must_use]
    pub fn code_nonzero(&self) -> Option<NonZeroI32> {
        self.0.code()
    }

    /// Converts an `ExitStatusError` (back) to an `ExitStatus`. <br>将 `ExitStatusError` (back) 转换为 `ExitStatus`。<br>
    #[must_use]
    pub fn into_status(&self) -> ExitStatus {
        ExitStatus(self.0.into())
    }
}

#[unstable(feature = "exit_status_error", issue = "84908")]
impl Into<ExitStatus> for ExitStatusError {
    fn into(self) -> ExitStatus {
        ExitStatus(self.0.into())
    }
}

#[unstable(feature = "exit_status_error", issue = "84908")]
impl fmt::Display for ExitStatusError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "process exited unsuccessfully: {}", self.into_status())
    }
}

#[unstable(feature = "exit_status_error", issue = "84908")]
impl crate::error::Error for ExitStatusError {}

/// This type represents the status code the current process can return to its parent under normal termination. <br>该类型表示当前进程在正常终止下可以返回其父进程的状态码。<br>
///
/// `ExitCode` is intended to be consumed only by the standard library (via [`Termination::report()`]), and intentionally does not provide accessors like `PartialEq`, `Eq`, or `Hash`. <br>`ExitCode` 旨在仅由标准库 (通过 [`Termination::report()`]) 使用，并且有意不提供 `PartialEq`、`Eq` 或 `Hash` 等访问器。<br>
/// Instead the standard library provides the canonical `SUCCESS` and `FAILURE` exit codes as well as `From<u8> for ExitCode` for constructing other arbitrary exit codes. <br>相反，标准库提供了规范的 `SUCCESS` 和 `FAILURE` 退出代码以及用于构造其他任意退出代码的 `From<u8> for ExitCode`。<br>
///
/// # Portability
///
/// Numeric values used in this type don't have portable meanings, and different platforms may mask different amounts of them. <br>此类型中使用的数字值没有可移植的含义，并且不同的平台可能会掩盖不同数量的含义。<br>
///
/// For the platform's canonical successful and unsuccessful codes, see the [`SUCCESS`] and [`FAILURE`] associated items. <br>有关平台的规范成功和失败代码，请参见 [`SUCCESS`] 和 [`FAILURE`] 关联项。<br>
///
/// [`SUCCESS`]: ExitCode::SUCCESS
/// [`FAILURE`]: ExitCode::FAILURE
///
/// # Differences from `ExitStatus` <br>与 `ExitStatus` 的区别<br>
///
/// `ExitCode` is intended for terminating the currently running process, via the `Termination` trait, in contrast to [`ExitStatus`], which represents the termination of a child process. <br>`ExitCode` 旨在通过 `Termination` trait 终止当前正在运行的进程，而 [`ExitStatus`] 则表示终止子进程。<br>
/// These APIs are separate due to platform compatibility differences and their expected usage; <br>由于平台兼容性差异及其预期用途，这些 API 是分开的;<br> it is not generally possible to exactly reproduce an `ExitStatus` from a child for the current process after the fact. <br>事后通常不可能从子节点那里准确地复制当前进程的 `ExitStatus`。<br>
///
///
/// # Examples
///
/// `ExitCode` can be returned from the `main` function of a crate, as it implements <br>`ExitCode` 可以从 crate 的 `main` 函数返回，因为它实现了<br>
/// [`Termination`]:
///
/// ```
/// use std::process::ExitCode;
/// # fn check_foo() -> bool { true }
///
/// fn main() -> ExitCode {
///     if !check_foo() {
///         return ExitCode::from(42);
///     }
///
///     ExitCode::SUCCESS
/// }
/// ```
///
///
///
///
///
///
///
///
///
#[derive(Clone, Copy, Debug)]
#[stable(feature = "process_exitcode", since = "1.61.0")]
pub struct ExitCode(imp::ExitCode);

#[stable(feature = "process_exitcode", since = "1.61.0")]
impl ExitCode {
    /// The canonical `ExitCode` for successful termination on this platform. <br>在此平台上成功终止的规范 `ExitCode`。<br>
    ///
    /// Note that a `()`-returning `main` implicitly results in a successful termination, so there's no need to return this from `main` unless you're also returning other possible codes. <br>请注意，返回 `main` 的 ` () ` 隐式导致成功终止，因此除非您还返回了其他可能的代码，否则无需从 `main` 返回此值。<br>
    ///
    ///
    #[stable(feature = "process_exitcode", since = "1.61.0")]
    pub const SUCCESS: ExitCode = ExitCode(imp::ExitCode::SUCCESS);

    /// The canonical `ExitCode` for unsuccessful termination on this platform. <br>此平台上未成功终止的规范 `ExitCode`。<br>
    ///
    /// If you're only returning this and `SUCCESS` from `main`, consider instead returning `Err(_)` and `Ok(())` respectively, which will return the same codes (but will also `eprintln!` the error). <br>如果仅从 `main` 返回此代码和 `SUCCESS`，则考虑分别返回 `Err(_)` 和 `Ok(())`，这将返回相同的代码 (但也会返回 `eprintln!` 错误)。<br>
    ///
    ///
    #[stable(feature = "process_exitcode", since = "1.61.0")]
    pub const FAILURE: ExitCode = ExitCode(imp::ExitCode::FAILURE);

    /// Exit the current process with the given `ExitCode`. <br>使用给定的 `ExitCode` 退出当前进程。<br>
    ///
    /// Note that this has the same caveats as [`process::exit()`][exit], namely that this function terminates the process immediately, so no destructors on the current stack or any other thread's stack will be run. <br>请注意，这与 [`process::exit()`][exit] 具有相同的注意事项，即此函数立即终止进程，因此当前栈或任何其他线程栈上的析构函数都不会运行。<br>
    /// If a clean shutdown is needed, it is recommended to simply return this ExitCode from the `main` function, as demonstrated in the [type documentation](#examples). <br>如果需要彻底关闭，建议简单地从 `main` 函数返回此 ExitCode，如 [类型文档](#examples) 中所示。<br>
    ///
    /// # Differences from `process::exit()` <br>与 `process::exit()` 的区别<br>
    ///
    /// `process::exit()` accepts any `i32` value as the exit code for the process; <br>`process::exit()` 接受任何 `i32` 值作为进程的退出代码;<br> however, there are platforms that only use a subset of that value (see [`process::exit` platform-specific behavior][exit#platform-specific-behavior]). <br>但是，有些平台只使用该值的一个子集 (参见 [`process::exit` platform-specific behavior][exit#platform-specific-behavior])。<br>
    ///
    /// `ExitCode` exists because of this; <br>`ExitCode` 就是因为这个而存在的;<br> only `ExitCode`s that are supported by a majority of our platforms can be created, so those problems don't exist (as much) with this method. <br>只能创建我们大多数平台支持的 `ExitCode`，因此使用此方法不存在 (尽可能多) 这些问题。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(exitcode_exit_method)]
    /// # use std::process::ExitCode;
    /// # use std::fmt;
    /// # enum UhOhError { GenericProblem, Specific, WithCode { exit_code: ExitCode, _x: () } }
    /// # impl fmt::Display for UhOhError {
    /// #     fn fmt(&self, _: &mut fmt::Formatter) -> fmt::Result { unimplemented!() }
    /// # }
    /// // there's no way to gracefully recover from an UhOhError, so we just print a message and exit <br>没有办法优雅地从 UhOhError 中恢复，所以我们只打印一条消息并退出<br>
    /////
    /// fn handle_unrecoverable_error(err: UhOhError) -> ! {
    ///     eprintln!("UH OH! {err}");
    ///     let code = match err {
    ///         UhOhError::GenericProblem => ExitCode::FAILURE,
    ///         UhOhError::Specific => ExitCode::from(3),
    ///         UhOhError::WithCode { exit_code, .. } => exit_code,
    ///     };
    ///     code.exit_process()
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[unstable(feature = "exitcode_exit_method", issue = "97100")]
    pub fn exit_process(self) -> ! {
        exit(self.to_i32())
    }
}

impl ExitCode {
    // This is private/perma-unstable because ExitCode is opaque; <br>这是私有的/永不稳定的，因为 ExitCode 是不透明的;<br> we don't know that i32 will serve all usecases, for example windows seems to use u32, unix uses the 8-15th bits of an i32, we likely want to isolate users anything that could restrict the platform specific representation of an ExitCode <br>我们不知道 i32 是否适用于所有用例，例如 windows 似乎使用 u32，unix 使用 i32 的第 8-15 位，我们可能希望隔离用户任何可能限制 ExitCode 特定平台表示的内容<br>
    //
    //
    // More info: https://internals.rust-lang.org/t/mini-pre-rfc-redesigning-process-exitstatus/5426 <br>有关更多信息请访问: https://internals.rust-lang.org/t/mini-pre-rfc-redesigning-process-exitstatus/5426<br>
    //
    //
    /// Convert an `ExitCode` into an i32 <br>将 `ExitCode` 转换为 i32<br>
    #[unstable(
        feature = "process_exitcode_internals",
        reason = "exposed only for libstd",
        issue = "none"
    )]
    #[inline]
    #[doc(hidden)]
    pub fn to_i32(self) -> i32 {
        self.0.as_i32()
    }
}

#[stable(feature = "process_exitcode", since = "1.61.0")]
impl From<u8> for ExitCode {
    /// Construct an `ExitCode` from an arbitrary u8 value. <br>从任意 u8 值构造 `ExitCode`。<br>
    fn from(code: u8) -> Self {
        ExitCode(imp::ExitCode::from(code))
    }
}

impl Child {
    /// Forces the child process to exit. <br>强制子进程退出。<br>
    /// If the child has already exited, an [`InvalidInput`] error is returned. <br>如果子节点已经退出，则返回 [`InvalidInput`] 错误。<br>
    ///
    /// The mapping to [`ErrorKind`]s is not part of the compatibility contract of the function. <br>到 [`ErrorKind`] 的映射不是函数的兼容性契约的一部分。<br>
    ///
    /// This is equivalent to sending a SIGKILL on Unix platforms. <br>这等效于在 Unix 平台上发送 SIGKILL。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// let mut command = Command::new("yes");
    /// if let Ok(mut child) = command.spawn() {
    ///     child.kill().expect("command wasn't running");
    /// } else {
    ///     println!("yes command didn't start");
    /// }
    /// ```
    ///
    /// [`ErrorKind`]: io::ErrorKind
    /// [`InvalidInput`]: io::ErrorKind::InvalidInput
    #[stable(feature = "process", since = "1.0.0")]
    pub fn kill(&mut self) -> io::Result<()> {
        self.handle.kill()
    }

    /// Returns the OS-assigned process identifier associated with this child. <br>返回与此子级关联的操作系统分配的进程标识符。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// let mut command = Command::new("ls");
    /// if let Ok(child) = command.spawn() {
    ///     println!("Child's ID is {}", child.id());
    /// } else {
    ///     println!("ls command didn't start");
    /// }
    /// ```
    #[must_use]
    #[stable(feature = "process_id", since = "1.3.0")]
    pub fn id(&self) -> u32 {
        self.handle.id()
    }

    /// Waits for the child to exit completely, returning the status that it exited with. <br>等待子节点完全退出，并返回退出时的状态。<br>
    /// This function will continue to have the same return value after it has been called at least once. <br>至少调用一次之后，该函数将继续具有相同的返回值。<br>
    ///
    /// The stdin handle to the child process, if any, will be closed before waiting. <br>子进程的 stdin 句柄 (如果有) 将在等待之前关闭。<br>
    /// This helps avoid deadlock: it ensures that the child does not block waiting for input from the parent, while the parent waits for the child to exit. <br>这有助于避免死锁：它确保子进程在父进程等待子进程退出时不会阻止其等待父进程的输入。<br>
    ///
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// let mut command = Command::new("ls");
    /// if let Ok(mut child) = command.spawn() {
    ///     child.wait().expect("command wasn't running");
    ///     println!("Child has finished its execution!");
    /// } else {
    ///     println!("ls command didn't start");
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "process", since = "1.0.0")]
    pub fn wait(&mut self) -> io::Result<ExitStatus> {
        drop(self.stdin.take());
        self.handle.wait().map(ExitStatus)
    }

    /// Attempts to collect the exit status of the child if it has already exited. <br>如果子节点已经退出，则尝试收集其退出状态。<br>
    ///
    /// This function will not block the calling thread and will only check to see if the child process has exited or not. <br>这个函数不会阻塞调用线程，只会检查子进程是否退出。<br>
    ///
    /// If the child has exited then on Unix the process ID is reaped. <br>如果子节点退出了，则在 Unix 上获取进程 ID。<br>
    /// This function is guaranteed to repeatedly return a successful exit status so long as the child has already exited. <br>只要子节点已经退出，就可以保证该函数重复返回成功的退出状态。<br>
    ///
    /// If the child has exited, then `Ok(Some(status))` is returned. <br>如果子节点已经退出，则返回 `Ok(Some(status))`。<br>
    /// If the exit status is not available at this time then `Ok(None)` is returned. <br>如果此时退出状态不可用，则返回 `Ok(None)`。<br>
    /// If an error occurs, then that error is returned. <br>如果发生错误，则返回该错误。<br>
    ///
    /// Note that unlike `wait`, this function will not attempt to drop stdin. <br>请注意，与 `wait` 不同，此函数不会尝试丢弃 stdin。<br>
    ///
    /// # Examples
    ///
    /// Basic usage: <br>基本用法：<br>
    ///
    /// ```no_run
    /// use std::process::Command;
    ///
    /// let mut child = Command::new("ls").spawn().unwrap();
    ///
    /// match child.try_wait() {
    ///     Ok(Some(status)) => println!("exited with: {status}"),
    ///     Ok(None) => {
    ///         println!("status not ready yet, let's really wait");
    ///         let res = child.wait();
    ///         println!("result: {res:?}");
    ///     }
    ///     Err(e) => println!("error attempting to wait: {e}"),
    /// }
    /// ```
    ///
    ///
    #[stable(feature = "process_try_wait", since = "1.18.0")]
    pub fn try_wait(&mut self) -> io::Result<Option<ExitStatus>> {
        Ok(self.handle.try_wait()?.map(ExitStatus))
    }

    /// Simultaneously waits for the child to exit and collect all remaining output on the stdout/stderr handles, returning an `Output` instance. <br>同时等待子节点退出并收集 stdout/stderr 句柄上的所有剩余输出，并返回 `Output` 实例。<br>
    ///
    /// The stdin handle to the child process, if any, will be closed before waiting. <br>子进程的 stdin 句柄 (如果有) 将在等待之前关闭。<br>
    /// This helps avoid deadlock: it ensures that the child does not block waiting for input from the parent, while the parent waits for the child to exit. <br>这有助于避免死锁：它确保子进程在父进程等待子进程退出时不会阻止其等待父进程的输入。<br>
    ///
    ///
    /// By default, stdin, stdout and stderr are inherited from the parent. <br>默认情况下，stdin、stdout 和 stderr 都是从父级继承的。<br>
    /// In order to capture the output into this `Result<Output>` it is necessary to create new pipes between parent and child. <br>为了将输出捕获到此 `Result<Output>` 中，必须在父级和子级之间创建新管道。<br>
    /// Use `stdout(Stdio::piped())` or `stderr(Stdio::piped())`, respectively. <br>分别使用 `stdout(Stdio::piped())` 或 `stderr(Stdio::piped())`。<br>
    ///
    /// # Examples
    ///
    /// ```should_panic
    /// use std::process::{Command, Stdio};
    ///
    /// let child = Command::new("/bin/cat")
    ///     .arg("file.txt")
    ///     .stdout(Stdio::piped())
    ///     .spawn()
    ///     .expect("failed to execute child");
    ///
    /// let output = child
    ///     .wait_with_output()
    ///     .expect("failed to wait on child");
    ///
    /// assert!(output.status.success());
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "process", since = "1.0.0")]
    pub fn wait_with_output(mut self) -> io::Result<Output> {
        drop(self.stdin.take());

        let (mut stdout, mut stderr) = (Vec::new(), Vec::new());
        match (self.stdout.take(), self.stderr.take()) {
            (None, None) => {}
            (Some(mut out), None) => {
                let res = out.read_to_end(&mut stdout);
                res.unwrap();
            }
            (None, Some(mut err)) => {
                let res = err.read_to_end(&mut stderr);
                res.unwrap();
            }
            (Some(out), Some(err)) => {
                let res = read2(out.inner, &mut stdout, err.inner, &mut stderr);
                res.unwrap();
            }
        }

        let status = self.wait()?;
        Ok(Output { status, stdout, stderr })
    }
}

/// Terminates the current process with the specified exit code. <br>使用指定的退出代码终止当前进程。<br>
///
/// This function will never return and will immediately terminate the current process. <br>该函数将永远不会返回，并会立即终止当前进程。<br> The exit code is passed through to the underlying OS and will be available for consumption by another process. <br>退出代码将传递到底层操作系统，并且可供其他进程使用。<br>
///
/// Note that because this function never returns, and that it terminates the process, no destructors on the current stack or any other thread's stack will be run. <br>请注意，由于此函数从不返回，并且终止了该进程，因此将不运行当前栈或任何其他线程的栈上的析构函数。<br>
/// If a clean shutdown is needed it is recommended to only call this function at a known point where there are no more destructors left to run; <br>如果需要彻底关闭，建议仅在没有更多析构函数要运行的已知点调用此函数;<br> or, preferably, simply return a type implementing [`Termination`] (such as [`ExitCode`] or `Result`) from the `main` function and avoid this function altogether: <br>或者，最好简单地从 `main` 函数中返回一个实现 [`Termination`] 的类型 (例如 [`ExitCode`] 或 `Result`) 并完全避免这个函数:<br>
///
///
/// ```
/// # use std::io::Error as MyError;
/// fn main() -> Result<(), MyError> {
///     // ...
///     Ok(())
/// }
/// ```
///
/// ## Platform-specific behavior <br>特定于平台的行为<br>
///
/// **Unix**: On Unix-like platforms, it is unlikely that all 32 bits of `exit` will be visible to a parent process inspecting the exit code. <br>**Unix**: 在类似 Unix 的平台上，检查父级退出代码的父进程不太可能看到 `exit` 的所有 32 位。<br> On most Unix-like platforms, only the eight least-significant bits are considered. <br>在大多数类 Unix 平台上，仅考虑八个最低有效位。<br>
///
/// For example, the exit code for this example will be `0` on Linux, but `256` on Windows: <br>例如，此示例的退出代码将是 Linux 上的 `0`，但 Windows 上的 `256`:<br>
///
/// ```no_run
/// use std::process;
///
/// process::exit(0x0100);
/// ```
///
///
///
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn exit(code: i32) -> ! {
    crate::rt::cleanup();
    crate::sys::os::exit(code)
}

/// Terminates the process in an abnormal fashion. <br>以异常方式终止进程。<br>
///
/// The function will never return and will immediately terminate the current process in a platform specific "abnormal" manner. <br>该函数将永远不会返回，并会以特定于平台的 "abnormal" 方式立即终止当前进程。<br>
///
/// Note that because this function never returns, and that it terminates the process, no destructors on the current stack or any other thread's stack will be run. <br>请注意，由于此函数从不返回，并且终止了该进程，因此将不运行当前栈或任何其他线程的栈上的析构函数。<br>
///
///
/// Rust IO buffers (eg, from `BufWriter`) will not be flushed. <br>Rust IO 缓冲区 (例如，来自 `BufWriter`) 不会被刷新。<br>
/// Likewise, C stdio buffers will (on most platforms) not be flushed. <br>同样，C 标准输入输出缓冲器 (在大多数平台上) 不会被刷新。<br>
///
/// This is in contrast to the default behaviour of [`panic!`] which unwinds the current thread's stack and calls all destructors. <br>这与 [`panic!`] 的默认行为相反，后者默认情况下会展开当前线程的栈并调用所有析构函数。<br>
/// When `panic="abort"` is set, either as an argument to `rustc` or in a crate's Cargo.toml, [`panic!`] and `abort` are similar. <br>设置 `panic="abort"` 时，无论是 `rustc` 的参数还是 crate 的 Cargo.toml，[`panic!`] 和 `abort` 都是相似的。<br>
/// However, [`panic!`] will still call the [panic hook] while `abort` will not. <br>但是，[`panic!`] 仍然会调用 [panic 钩子][panic hook]，而 `abort` 不会。<br>
///
/// If a clean shutdown is needed it is recommended to only call this function at a known point where there are no more destructors left to run. <br>如果需要彻底关闭，建议仅在没有更多析构函数可以运行的已知点上调用此函数。<br>
///
/// The process's termination will be similar to that from the C `abort()` function. <br>该进程的终止将类似于 C `abort()` 函数的终止。<br>
/// On Unix, the process will terminate with signal `SIGABRT`, which typically means that the shell prints "Aborted". <br>在 Unix 上，进程将以信号 `SIGABRT` 终止，这通常意味着 shell 打印 "Aborted"。<br>
///
/// # Examples
///
/// ```no_run
/// use std::process;
///
/// fn main() {
///     println!("aborting");
///
///     process::abort();
///
///     // execution never gets here <br>执行永远不会到这里<br>
/// }
/// ```
///
/// The `abort` function terminates the process, so the destructor will not get run on the example below: <br>`abort` 函数会终止该进程，因此析构函数将不会在以下示例上运行：<br>
///
/// ```no_run
/// use std::process;
///
/// struct HasDrop;
///
/// impl Drop for HasDrop {
///     fn drop(&mut self) {
///         println!("This will never be printed!");
///     }
/// }
///
/// fn main() {
///     let _x = HasDrop;
///     process::abort();
///     // the destructor implemented for HasDrop will never get run <br>为 HasDrop 实现的析构函数永远不会运行<br>
/// }
/// ```
///
/// [panic hook]: crate::panic::set_hook
///
///
///
///
///
///
///
///
#[stable(feature = "process_abort", since = "1.17.0")]
#[cold]
pub fn abort() -> ! {
    crate::sys::abort_internal();
}

/// Returns the OS-assigned process identifier associated with this process. <br>返回与此进程关联的操作系统分配的进程标识符。<br>
///
/// # Examples
///
/// Basic usage: <br>基本用法：<br>
///
/// ```no_run
/// use std::process;
///
/// println!("My pid is {}", process::id());
/// ```
///
///
#[must_use]
#[stable(feature = "getpid", since = "1.26.0")]
pub fn id() -> u32 {
    crate::sys::os::getpid()
}

/// A trait for implementing arbitrary return types in the `main` function. <br>一个 trait，用于在 `main` 函数中实现任意的返回类型。<br>
///
/// The C-main function only supports returning integers. <br>C-main 函数只支持返回整数。<br>
/// So, every type implementing the `Termination` trait has to be converted to an integer. <br>因此，实现 `Termination` trait 的每种类型都必须转换为整数。<br>
///
/// The default implementations are returning `libc::EXIT_SUCCESS` to indicate a successful execution. <br>默认实现将返回 `libc::EXIT_SUCCESS` 以指示成功执行。<br> In case of a failure, `libc::EXIT_FAILURE` is returned. <br>如果发生故障，则返回 `libc::EXIT_FAILURE`。<br>
///
/// Because different runtimes have different specifications on the return value of the `main` function, this trait is likely to be available only on standard library's runtime for convenience. <br>因为不同的运行时对 `main` 函数的返回值有不同的规范，为了方便起见，这个 trait 很可能只在标准库的运行时可用。<br>
/// Other runtimes are not required to provide similar functionality. <br>其他运行时不需要提供类似的功能。<br>
///
///
///
///
#[cfg_attr(not(test), lang = "termination")]
#[stable(feature = "termination_trait_lib", since = "1.61.0")]
#[rustc_on_unimplemented(
    message = "`main` has invalid return type `{Self}`",
    label = "`main` can only return types that implement `{Termination}`"
)]
pub trait Termination {
    /// Is called to get the representation of the value as status code. <br>被调用以获取值的表示形式作为状态码。<br>
    /// This status code is returned to the operating system. <br>此状态代码返回到操作系统。<br>
    #[stable(feature = "termination_trait_lib", since = "1.61.0")]
    fn report(self) -> ExitCode;
}

#[stable(feature = "termination_trait_lib", since = "1.61.0")]
impl Termination for () {
    #[inline]
    fn report(self) -> ExitCode {
        ExitCode::SUCCESS
    }
}

#[stable(feature = "termination_trait_lib", since = "1.61.0")]
impl Termination for ! {
    fn report(self) -> ExitCode {
        self
    }
}

#[stable(feature = "termination_trait_lib", since = "1.61.0")]
impl Termination for Infallible {
    fn report(self) -> ExitCode {
        match self {}
    }
}

#[stable(feature = "termination_trait_lib", since = "1.61.0")]
impl Termination for ExitCode {
    #[inline]
    fn report(self) -> ExitCode {
        self
    }
}

#[stable(feature = "termination_trait_lib", since = "1.61.0")]
impl<T: Termination, E: fmt::Debug> Termination for Result<T, E> {
    fn report(self) -> ExitCode {
        match self {
            Ok(val) => val.report(),
            Err(err) => {
                // Ignore error if the write fails, for example because stderr is already closed. <br>如果写入失败，则忽略错误，例如因为标准错误已经关闭。<br>
                // There is not much point panicking at this point. <br>在这一点上，panic 的话没有太大的意义。<br>
                let _ = writeln!(io::stderr(), "Error: {err:?}");
                ExitCode::FAILURE
            }
        }
    }
}
