//! Runtime services <br>运行时服务<br>
//!
//! The `rt` module provides a narrow set of runtime services, including the global heap (exported in `heap`) and unwinding and backtrace support. <br>`rt` 模块提供了一组 narrow 的运行时服务，包括全局堆 (在 `heap` 中导出) 以及展开和回溯支持。<br>
//!
//! The APIs in this module are highly unstable, and should be considered as private implementation details for the time being. <br>该模块中的 API 高度不稳定，因此暂时应将其视为私有实现细节。<br>
//!
//!

#![unstable(
    feature = "rt",
    reason = "this public module should not exist and is highly likely \
              to disappear",
    issue = "none"
)]
#![doc(hidden)]
#![deny(unsafe_op_in_unsafe_fn)]
#![allow(unused_macros)]

use crate::ffi::CString;

// Re-export some of our utilities which are expected by other crates. <br>重推出其他 crates 期望的一些实用工具。<br>
pub use crate::panicking::{begin_panic, panic_count};
pub use core::panicking::{panic_display, panic_fmt};

use crate::sync::Once;
use crate::sys;
use crate::sys_common::thread_info;
use crate::thread::Thread;

// Prints to the "panic output", depending on the platform this may be: <br>打印到 "panic output"，取决于平台，这可能是：<br>
// - the standard error output <br>标准错误输出<br>
// - some dedicated platform specific output <br>一些专用平台特定的输出<br>
// - nothing (so this macro is a no-op) <br>什么都没有 (所以这个宏是一个空操作)<br>
macro_rules! rtprintpanic {
    ($($t:tt)*) => {
        if let Some(mut out) = crate::sys::stdio::panic_output() {
            let _ = crate::io::Write::write_fmt(&mut out, format_args!($($t)*));
        }
    }
}

macro_rules! rtabort {
    ($($t:tt)*) => {
        {
            rtprintpanic!("fatal runtime error: {}\n", format_args!($($t)*));
            crate::sys::abort_internal();
        }
    }
}

macro_rules! rtassert {
    ($e:expr) => {
        if !$e {
            rtabort!(concat!("assertion failed: ", stringify!($e)));
        }
    };
}

macro_rules! rtunwrap {
    ($ok:ident, $e:expr) => {
        match $e {
            $ok(v) => v,
            ref err => {
                let err = err.as_ref().map(drop); // map Ok/Some which might not be Debug <br>映射 Ok/Some，可能不是 Debug<br>
                rtabort!(concat!("unwrap failed: ", stringify!($e), " = {:?}"), err)
            }
        }
    };
}

// One-time runtime initialization. <br>一次性运行时初始化。<br>
// Runs before `main`. <br>在 `main` 之前运行。<br>
// SAFETY: must be called only once during runtime initialization. <br>在运行时初始化期间只能调用一次。<br>
// NOTE: this is not guaranteed to run, for example when Rust code is called externally. <br>这不能保证运行，例如在外部调用 Rust 代码时。<br>
#[cfg_attr(test, allow(dead_code))]
unsafe fn init(argc: isize, argv: *const *const u8) {
    unsafe {
        sys::init(argc, argv);

        let main_guard = sys::thread::guard::init();
        // Next, set up the current Thread with the guard information we just created. <br>接下来，使用我们刚刚创建的保护信息设置当前线程。<br>
        // Note that this isn't necessary in general for new threads, but we just do this to name the main thread and to give it correct info about the stack bounds. <br>请注意，通常对于新线程来说这不是必需的，但是我们只是这样做以命名主线程并为它提供有关栈边界的正确信息。<br>
        //
        //
        let thread = Thread::new(Some(rtunwrap!(Ok, CString::new("main"))));
        thread_info::set(main_guard, thread);
    }
}

// One-time runtime cleanup. <br>一次性运行时清理。<br>
// Runs after `main` or at program exit. <br>在 `main` 之后或在程序退出时运行。<br>
// NOTE: this is not guaranteed to run, for example when the program aborts. <br>这不能保证运行，例如当程序中止时。<br>
pub(crate) fn cleanup() {
    static CLEANUP: Once = Once::new();
    CLEANUP.call_once(|| unsafe {
        // Flush stdout and disable buffering. <br>刷新 stdout 并禁用缓冲。<br>
        crate::io::cleanup();
        // SAFETY: Only called once during runtime cleanup. <br>仅在运行时清理期间调用一次。<br>
        sys::cleanup();
    });
}

// To reduce the generated code of the new `lang_start`, this function is doing the real work. <br>为了减少新 `lang_start` 的生成代码，此函数正在做实际工作。<br>
//
#[cfg(not(test))]
fn lang_start_internal(
    main: &(dyn Fn() -> i32 + Sync + crate::panic::RefUnwindSafe),
    argc: isize,
    argv: *const *const u8,
) -> Result<isize, !> {
    use crate::{mem, panic};
    let rt_abort = move |e| {
        mem::forget(e);
        rtabort!("initialization or cleanup bug");
    };
    // Guard against the code called by this function from unwinding outside of the Rust-controlled code, which is UB. <br>在 Rust 控制的代码之外，要防范这个函数从展开中调用的代码，也就是 UB。<br>
    // This is a requirement imposed by a combination of how the `#[lang="start"]` attribute is implemented as well as by the implementation of the panicking mechanism itself. <br>这是由 `#[lang="start"]` 属性的实现方式以及 panicking 机制本身的实现所强加的要求。<br>
    //
    //
    // There are a couple of instances where unwinding can begin. <br>有几种情况可以开始展开。<br> First is inside of the `rt::init`, `rt::cleanup` and similar functions controlled by libstd. <br>首先是 libstd 控制的 `rt::init`、`rt::cleanup` 等类似函数的内部。<br>
    // In those instances a panic is a libstd implementation bug. <br>在这些情况下，panic 是 libstd 实现错误。<br>
    // A quite likely one too, as there isn't any way to prevent libstd from accidentally introducing a panic to these functions. <br>也很有可能，因为没有任何方法可以防止 libstd 意外地将 panic 引入这些函数。<br>
    // Another is from user code from `main` or, more nefariously, as described in e.g. <br>另一个来自 `main` 的用户代码，或者更邪恶的是，如<br>
    // issue #86030.
    // SAFETY: Only called once during runtime initialization. <br>在运行时初始化期间只调用一次。<br>
    //
    panic::catch_unwind(move || unsafe { init(argc, argv) }).map_err(rt_abort)?;
    let ret_code = panic::catch_unwind(move || panic::catch_unwind(main).unwrap_or(101) as isize)
        .map_err(move |e| {
            mem::forget(e);
            rtabort!("drop of the panic payload panicked");
        });
    panic::catch_unwind(cleanup).map_err(rt_abort)?;
    ret_code
}

#[cfg(not(test))]
#[lang = "start"]
fn lang_start<T: crate::process::Termination + 'static>(
    main: fn() -> T,
    argc: isize,
    argv: *const *const u8,
) -> isize {
    let Ok(v) = lang_start_internal(
        &move || crate::sys_common::backtrace::__rust_begin_short_backtrace(main).report().to_i32(),
        argc,
        argv,
    );
    v
}
