#[cfg(test)]
mod tests;

use crate::fmt;
use crate::sync::{mutex, poison, LockResult, MutexGuard, PoisonError};
use crate::sys_common::condvar as sys;
use crate::time::{Duration, Instant};

/// A type indicating whether a timed wait on a condition variable returned due to a time out or not. <br>指示是否因超时而返回的条件变量的定时等待的类型。<br>
///
///
/// It is returned by the [`wait_timeout`] method. <br>它由 [`wait_timeout`] 方法返回。<br>
///
/// [`wait_timeout`]: Condvar::wait_timeout
#[derive(Debug, PartialEq, Eq, Copy, Clone)]
#[stable(feature = "wait_timeout", since = "1.5.0")]
pub struct WaitTimeoutResult(bool);

impl WaitTimeoutResult {
    /// Returns `true` if the wait was known to have timed out. <br>如果已知等待超时，则返回 `true`。<br>
    ///
    /// # Examples
    ///
    /// This example spawns a thread which will update the boolean value and then wait 100 milliseconds before notifying the condvar. <br>本示例生成一个线程，该线程将更新布尔值，然后等待 100 毫秒再通知 condvar。<br>
    ///
    ///
    /// The main thread will wait with a timeout on the condvar and then leave once the boolean has been updated and notified. <br>主线程将在 condvar 上等待超时，然后在布尔值已更新并通知后离开。<br>
    ///
    /// ```
    /// use std::sync::{Arc, Condvar, Mutex};
    /// use std::thread;
    /// use std::time::Duration;
    ///
    /// let pair = Arc::new((Mutex::new(false), Condvar::new()));
    /// let pair2 = Arc::clone(&pair);
    ///
    /// thread::spawn(move || {
    ///     let (lock, cvar) = &*pair2;
    ///
    ///     // Let's wait 20 milliseconds before notifying the condvar. <br>让我们等待 20 毫秒，然后再通知 condvar。<br>
    ///     thread::sleep(Duration::from_millis(20));
    ///
    ///     let mut started = lock.lock().unwrap();
    ///     // We update the boolean value. <br>我们更新布尔值。<br>
    ///     *started = true;
    ///     cvar.notify_one();
    /// });
    ///
    /// // Wait for the thread to start up. <br>等待线程启动。<br>
    /// let (lock, cvar) = &*pair;
    /// let mut started = lock.lock().unwrap();
    /// loop {
    ///     // Let's put a timeout on the condvar's wait. <br>让我们在 condvar 的等待上设置超时。<br>
    ///     let result = cvar.wait_timeout(started, Duration::from_millis(10)).unwrap();
    ///     // 10 milliseconds have passed, or maybe the value changed! <br>10 毫秒已过去，或者值已更改！<br>
    ///     started = result.0;
    ///     if *started == true {
    ///         // We received the notification and the value has been updated, we can leave. <br>我们已收到通知，并且值已更新，我们可以离开。<br>
    ///         break
    ///     }
    /// }
    /// ```
    ///
    #[must_use]
    #[stable(feature = "wait_timeout", since = "1.5.0")]
    pub fn timed_out(&self) -> bool {
        self.0
    }
}

/// A Condition Variable <br>条件变量<br>
///
/// Condition variables represent the ability to block a thread such that it consumes no CPU time while waiting for an event to occur. <br>条件变量表示阻塞线程的能力，以使线程在等待事件发生时不占用任何 CPU 时间。<br>
/// Condition variables are typically associated with a boolean predicate (a condition) and a mutex. <br>条件变量通常与布尔谓词 (条件) 和互斥锁相关联。<br>
/// The predicate is always verified inside of the mutex before determining that a thread must block. <br>在确定线程必须阻塞之前，始终在互斥锁内部验证该谓词。<br>
///
/// Functions in this module will block the current **thread** of execution. <br>此模块中的函数将阻止当前的执行线程。<br>
/// Note that any attempt to use multiple mutexes on the same condition variable may result in a runtime panic. <br>请注意，对同一条件变量使用多个互斥锁的任何尝试都可能导致运行时 panic。<br>
///
/// # Examples
///
/// ```
/// use std::sync::{Arc, Mutex, Condvar};
/// use std::thread;
///
/// let pair = Arc::new((Mutex::new(false), Condvar::new()));
/// let pair2 = Arc::clone(&pair);
///
/// // Inside of our lock, spawn a new thread, and then wait for it to start. <br>在我们的锁内部，spawn 有一个新线程，然后等待它启动。<br>
/// thread::spawn(move|| {
///     let (lock, cvar) = &*pair2;
///     let mut started = lock.lock().unwrap();
///     *started = true;
///     // We notify the condvar that the value has changed. <br>我们通知 condvar 值已更改。<br>
///     cvar.notify_one();
/// });
///
/// // Wait for the thread to start up. <br>等待线程启动。<br>
/// let (lock, cvar) = &*pair;
/// let mut started = lock.lock().unwrap();
/// while !*started {
///     started = cvar.wait(started).unwrap();
/// }
/// ```
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Condvar {
    inner: sys::Condvar,
}

impl Condvar {
    /// Creates a new condition variable which is ready to be waited on and notified. <br>创建一个新的条件变量，可以随时等待它并通知它。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::Condvar;
    ///
    /// let condvar = Condvar::new();
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[rustc_const_stable(feature = "const_locks", since = "1.63.0")]
    #[must_use]
    #[inline]
    pub const fn new() -> Condvar {
        Condvar { inner: sys::Condvar::new() }
    }

    /// Blocks the current thread until this condition variable receives a notification. <br>阻塞当前线程，直到此条件变量收到通知为止。<br>
    ///
    /// This function will atomically unlock the mutex specified (represented by `guard`) and block the current thread. <br>该函数将自动解锁指定的互斥锁 (由 `guard` 表示) 并阻塞当前线程。<br>
    /// This means that any calls to [`notify_one`] or [`notify_all`] which happen logically after the mutex is unlocked are candidates to wake this thread up. <br>这意味着在互斥锁解锁后逻辑上发生的任何对 [`notify_one`] 或 [`notify_all`] 的调用都可以唤醒该线程。<br>
    /// When this function call returns, the lock specified will have been re-acquired. <br>当此函数调用返回时，将重新获得指定的锁。<br>
    ///
    /// Note that this function is susceptible to spurious wakeups. <br>请注意，此函数易受虚假唤醒的影响。<br>
    /// Condition variables normally have a boolean predicate associated with them, and the predicate must always be checked each time this function returns to protect against spurious wakeups. <br>条件变量通常具有与之关联的布尔谓词，并且每次此函数返回时都必须始终检查谓词，以防止虚假唤醒。<br>
    ///
    ///
    /// # Errors
    ///
    /// This function will return an error if the mutex being waited on is poisoned when this thread re-acquires the lock. <br>如果正在等待的互斥锁在此线程重新获取锁时中毒，则此函数将返回错误。<br>
    /// For more information, see information about [poisoning] on the [`Mutex`] type. <br>有关更多信息，请参见有关 [`Mutex`] 类型的 [中毒][poisoning] 的信息。<br>
    ///
    /// # Panics
    ///
    /// This function may [`panic!`] if it is used with more than one mutex over time. <br>如果长时间使用此函数与多个互斥锁，则该函数可能为 [`panic!`]。<br>
    ///
    /// [`notify_one`]: Self::notify_one
    /// [`notify_all`]: Self::notify_all
    /// [poisoning]: super::Mutex#poisoning
    /// [`Mutex`]: super::Mutex
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::{Arc, Mutex, Condvar};
    /// use std::thread;
    ///
    /// let pair = Arc::new((Mutex::new(false), Condvar::new()));
    /// let pair2 = Arc::clone(&pair);
    ///
    /// thread::spawn(move|| {
    ///     let (lock, cvar) = &*pair2;
    ///     let mut started = lock.lock().unwrap();
    ///     *started = true;
    ///     // We notify the condvar that the value has changed. <br>我们通知 condvar 值已更改。<br>
    ///     cvar.notify_one();
    /// });
    ///
    /// // Wait for the thread to start up. <br>等待线程启动。<br>
    /// let (lock, cvar) = &*pair;
    /// let mut started = lock.lock().unwrap();
    /// // As long as the value inside the `Mutex<bool>` is `false`, we wait. <br>只要 `Mutex<bool>` 内部的值为 `false`，我们就等待。<br>
    /// while !*started {
    ///     started = cvar.wait(started).unwrap();
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn wait<'a, T>(&self, guard: MutexGuard<'a, T>) -> LockResult<MutexGuard<'a, T>> {
        let poisoned = unsafe {
            let lock = mutex::guard_lock(&guard);
            self.inner.wait(lock);
            mutex::guard_poison(&guard).get()
        };
        if poisoned { Err(PoisonError::new(guard)) } else { Ok(guard) }
    }

    /// Blocks the current thread until this condition variable receives a notification and the provided condition is false. <br>阻止当前线程，直到此条件变量接收到通知并且所提供的条件为 false 为止。<br>
    ///
    /// This function will atomically unlock the mutex specified (represented by `guard`) and block the current thread. <br>该函数将自动解锁指定的互斥锁 (由 `guard` 表示) 并阻塞当前线程。<br>
    /// This means that any calls to [`notify_one`] or [`notify_all`] which happen logically after the mutex is unlocked are candidates to wake this thread up. <br>这意味着在互斥锁解锁后逻辑上发生的任何对 [`notify_one`] 或 [`notify_all`] 的调用都可以唤醒该线程。<br>
    ///
    /// When this function call returns, the lock specified will have been re-acquired. <br>当此函数调用返回时，将重新获得指定的锁。<br>
    ///
    /// # Errors
    ///
    /// This function will return an error if the mutex being waited on is poisoned when this thread re-acquires the lock. <br>如果正在等待的互斥锁在此线程重新获取锁时中毒，则此函数将返回错误。<br>
    /// For more information, see information about [poisoning] on the [`Mutex`] type. <br>有关更多信息，请参见有关 [`Mutex`] 类型的 [中毒][poisoning] 的信息。<br>
    ///
    /// [`notify_one`]: Self::notify_one
    /// [`notify_all`]: Self::notify_all
    /// [poisoning]: super::Mutex#poisoning
    /// [`Mutex`]: super::Mutex
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::{Arc, Mutex, Condvar};
    /// use std::thread;
    ///
    /// let pair = Arc::new((Mutex::new(true), Condvar::new()));
    /// let pair2 = Arc::clone(&pair);
    ///
    /// thread::spawn(move|| {
    ///     let (lock, cvar) = &*pair2;
    ///     let mut pending = lock.lock().unwrap();
    ///     *pending = false;
    ///     // We notify the condvar that the value has changed. <br>我们通知 condvar 值已更改。<br>
    ///     cvar.notify_one();
    /// });
    ///
    /// // Wait for the thread to start up. <br>等待线程启动。<br>
    /// let (lock, cvar) = &*pair;
    /// // As long as the value inside the `Mutex<bool>` is `true`, we wait. <br>只要 `Mutex<bool>` 内部的值为 `true`，我们就等待。<br>
    /// let _guard = cvar.wait_while(lock.lock().unwrap(), |pending| { *pending }).unwrap();
    /// ```
    ///
    ///
    ///
    #[stable(feature = "wait_until", since = "1.42.0")]
    pub fn wait_while<'a, T, F>(
        &self,
        mut guard: MutexGuard<'a, T>,
        mut condition: F,
    ) -> LockResult<MutexGuard<'a, T>>
    where
        F: FnMut(&mut T) -> bool,
    {
        while condition(&mut *guard) {
            guard = self.wait(guard)?;
        }
        Ok(guard)
    }

    /// Waits on this condition variable for a notification, timing out after a specified duration. <br>等待此条件变量以获取通知，并在指定的持续时间后超时。<br>
    ///
    /// The semantics of this function are equivalent to [`wait`] except that the thread will be blocked for roughly no longer than `ms` milliseconds. <br>该函数的语义与 [`wait`] 等效，除了线程被阻塞的时间大约不超过 `ms` 毫秒。<br>
    /// This method should not be used for precise timing due to anomalies such as preemption or platform differences that might not cause the maximum amount of time waited to be precisely `ms`. <br>由于抢占或平台差异等异常情况可能不会导致最大等待时间精确到 `ms`，因此不应将此方法用于精确计时。<br>
    ///
    ///
    /// Note that the best effort is made to ensure that the time waited is measured with a monotonic clock, and not affected by the changes made to the system time. <br>请注意，已尽最大努力确保使用单调时钟来测量等待的时间，并且不受系统时间更改的影响。<br>
    ///
    /// The returned boolean is `false` only if the timeout is known to have elapsed. <br>仅当已知超时已经过去时，返回的布尔值为 `false`。<br>
    ///
    /// Like [`wait`], the lock specified will be re-acquired when this function returns, regardless of whether the timeout elapsed or not. <br>与 [`wait`] 一样，无论是否经过超时，都将在此函数返回时重新获取指定的锁。<br>
    ///
    /// [`wait`]: Self::wait
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::{Arc, Mutex, Condvar};
    /// use std::thread;
    ///
    /// let pair = Arc::new((Mutex::new(false), Condvar::new()));
    /// let pair2 = Arc::clone(&pair);
    ///
    /// thread::spawn(move|| {
    ///     let (lock, cvar) = &*pair2;
    ///     let mut started = lock.lock().unwrap();
    ///     *started = true;
    ///     // We notify the condvar that the value has changed. <br>我们通知 condvar 值已更改。<br>
    ///     cvar.notify_one();
    /// });
    ///
    /// // Wait for the thread to start up. <br>等待线程启动。<br>
    /// let (lock, cvar) = &*pair;
    /// let mut started = lock.lock().unwrap();
    /// // As long as the value inside the `Mutex<bool>` is `false`, we wait. <br>只要 `Mutex<bool>` 内部的值为 `false`，我们就等待。<br>
    /// loop {
    ///     let result = cvar.wait_timeout_ms(started, 10).unwrap();
    ///     // 10 milliseconds have passed, or maybe the value changed! <br>10 毫秒已过去，或者值已更改！<br>
    ///     started = result.0;
    ///     if *started == true {
    ///         // We received the notification and the value has been updated, we can leave. <br>我们已收到通知，并且值已更新，我们可以离开。<br>
    ///         break
    ///     }
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[deprecated(since = "1.6.0", note = "replaced by `std::sync::Condvar::wait_timeout`")]
    pub fn wait_timeout_ms<'a, T>(
        &self,
        guard: MutexGuard<'a, T>,
        ms: u32,
    ) -> LockResult<(MutexGuard<'a, T>, bool)> {
        let res = self.wait_timeout(guard, Duration::from_millis(ms as u64));
        poison::map_result(res, |(a, b)| (a, !b.timed_out()))
    }

    /// Waits on this condition variable for a notification, timing out after a specified duration. <br>等待此条件变量以获取通知，并在指定的持续时间后超时。<br>
    ///
    /// The semantics of this function are equivalent to [`wait`] except that the thread will be blocked for roughly no longer than `dur`. <br>该函数的语义与 [`wait`] 等效，除了线程被阻塞的时间不超过 `dur`。<br>
    /// This method should not be used for precise timing due to anomalies such as preemption or platform differences that might not cause the maximum amount of time waited to be precisely `dur`. <br>由于抢占或平台差异等异常情况可能不会导致最大等待时间精确到 `dur`，因此不应将此方法用于精确计时。<br>
    ///
    ///
    /// Note that the best effort is made to ensure that the time waited is measured with a monotonic clock, and not affected by the changes made to the system time. <br>请注意，已尽最大努力确保使用单调时钟来测量等待的时间，并且不受系统时间更改的影响。<br>
    /// This function is susceptible to spurious wakeups. <br>此函数易受虚假唤醒的影响。<br>
    /// Condition variables normally have a boolean predicate associated with them, and the predicate must always be checked each time this function returns to protect against spurious wakeups. <br>条件变量通常具有与之关联的布尔谓词，并且每次此函数返回时都必须始终检查谓词，以防止虚假唤醒。<br>
    /// Additionally, it is typically desirable for the timeout to not exceed some duration in spite of spurious wakes, thus the sleep-duration is decremented by the amount slept. <br>另外，尽管有虚假的唤醒，但通常还是希望超时不超过某个持续时间，因此睡眠时间会减少睡眠量。<br>
    /// Alternatively, use the `wait_timeout_while` method to wait with a timeout while a predicate is true. <br>或者，在谓词为 true 时，使用 `wait_timeout_while` 方法等待超时。<br>
    ///
    /// The returned [`WaitTimeoutResult`] value indicates if the timeout is known to have elapsed. <br>返回的 [`WaitTimeoutResult`] 值指示是否已知超时。<br>
    ///
    /// Like [`wait`], the lock specified will be re-acquired when this function returns, regardless of whether the timeout elapsed or not. <br>与 [`wait`] 一样，无论是否经过超时，都将在此函数返回时重新获取指定的锁。<br>
    ///
    /// [`wait`]: Self::wait
    /// [`wait_timeout_while`]: Self::wait_timeout_while
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::{Arc, Mutex, Condvar};
    /// use std::thread;
    /// use std::time::Duration;
    ///
    /// let pair = Arc::new((Mutex::new(false), Condvar::new()));
    /// let pair2 = Arc::clone(&pair);
    ///
    /// thread::spawn(move|| {
    ///     let (lock, cvar) = &*pair2;
    ///     let mut started = lock.lock().unwrap();
    ///     *started = true;
    ///     // We notify the condvar that the value has changed. <br>我们通知 condvar 值已更改。<br>
    ///     cvar.notify_one();
    /// });
    ///
    /// // wait for the thread to start up <br>等待线程启动<br>
    /// let (lock, cvar) = &*pair;
    /// let mut started = lock.lock().unwrap();
    /// // as long as the value inside the `Mutex<bool>` is `false`, we wait <br>只要 `Mutex<bool>` 内的值为 `false`，我们就等待<br>
    /// loop {
    ///     let result = cvar.wait_timeout(started, Duration::from_millis(10)).unwrap();
    ///     // 10 milliseconds have passed, or maybe the value changed! <br>10 毫秒已过去，或者值已更改！<br>
    ///     started = result.0;
    ///     if *started == true {
    ///         // We received the notification and the value has been updated, we can leave. <br>我们已收到通知，并且值已更新，我们可以离开。<br>
    ///         break
    ///     }
    /// }
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "wait_timeout", since = "1.5.0")]
    pub fn wait_timeout<'a, T>(
        &self,
        guard: MutexGuard<'a, T>,
        dur: Duration,
    ) -> LockResult<(MutexGuard<'a, T>, WaitTimeoutResult)> {
        let (poisoned, result) = unsafe {
            let lock = mutex::guard_lock(&guard);
            let success = self.inner.wait_timeout(lock, dur);
            (mutex::guard_poison(&guard).get(), WaitTimeoutResult(!success))
        };
        if poisoned { Err(PoisonError::new((guard, result))) } else { Ok((guard, result)) }
    }

    /// Waits on this condition variable for a notification, timing out after a specified duration. <br>等待此条件变量以获取通知，并在指定的持续时间后超时。<br>
    ///
    /// The semantics of this function are equivalent to [`wait_while`] except that the thread will be blocked for roughly no longer than `dur`. <br>该函数的语义与 [`wait_while`] 等效，除了线程被阻塞的时间不超过 `dur`。<br>
    /// This method should not be used for precise timing due to anomalies such as preemption or platform differences that might not cause the maximum amount of time waited to be precisely `dur`. <br>由于抢占或平台差异等异常情况可能不会导致最大等待时间精确到 `dur`，因此不应将此方法用于精确计时。<br>
    ///
    ///
    /// Note that the best effort is made to ensure that the time waited is measured with a monotonic clock, and not affected by the changes made to the system time. <br>请注意，已尽最大努力确保使用单调时钟来测量等待的时间，并且不受系统时间更改的影响。<br>
    ///
    /// The returned [`WaitTimeoutResult`] value indicates if the timeout is known to have elapsed without the condition being met. <br>返回的 [`WaitTimeoutResult`] 值指示是否已知超时已经过去，而没有满足条件。<br>
    ///
    /// Like [`wait_while`], the lock specified will be re-acquired when this function returns, regardless of whether the timeout elapsed or not. <br>与 [`wait_while`] 一样，无论是否经过超时，都将在此函数返回时重新获取指定的锁。<br>
    ///
    /// [`wait_while`]: Self::wait_while
    /// [`wait_timeout`]: Self::wait_timeout
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::{Arc, Mutex, Condvar};
    /// use std::thread;
    /// use std::time::Duration;
    ///
    /// let pair = Arc::new((Mutex::new(true), Condvar::new()));
    /// let pair2 = Arc::clone(&pair);
    ///
    /// thread::spawn(move|| {
    ///     let (lock, cvar) = &*pair2;
    ///     let mut pending = lock.lock().unwrap();
    ///     *pending = false;
    ///     // We notify the condvar that the value has changed. <br>我们通知 condvar 值已更改。<br>
    ///     cvar.notify_one();
    /// });
    ///
    /// // wait for the thread to start up <br>等待线程启动<br>
    /// let (lock, cvar) = &*pair;
    /// let result = cvar.wait_timeout_while(
    ///     lock.lock().unwrap(),
    ///     Duration::from_millis(100),
    ///     |&mut pending| pending,
    /// ).unwrap();
    /// if result.1.timed_out() {
    ///     // timed-out without the condition ever evaluating to false. <br>超时，条件永远不会为 false。<br>
    /// }
    /// // access the locked mutex via result.0 <br>通过 result.0 访问锁定的互斥锁<br>
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "wait_timeout_until", since = "1.42.0")]
    pub fn wait_timeout_while<'a, T, F>(
        &self,
        mut guard: MutexGuard<'a, T>,
        dur: Duration,
        mut condition: F,
    ) -> LockResult<(MutexGuard<'a, T>, WaitTimeoutResult)>
    where
        F: FnMut(&mut T) -> bool,
    {
        let start = Instant::now();
        loop {
            if !condition(&mut *guard) {
                return Ok((guard, WaitTimeoutResult(false)));
            }
            let timeout = match dur.checked_sub(start.elapsed()) {
                Some(timeout) => timeout,
                None => return Ok((guard, WaitTimeoutResult(true))),
            };
            guard = self.wait_timeout(guard, timeout)?.0;
        }
    }

    /// Wakes up one blocked thread on this condvar. <br>在此 condvar 上唤醒一个阻塞的线程。<br>
    ///
    /// If there is a blocked thread on this condition variable, then it will be woken up from its call to [`wait`] or [`wait_timeout`]. <br>如果此条件变量上有阻塞的线程，则它将从其调用中唤醒到 [`wait`] 或 [`wait_timeout`]。<br>
    /// Calls to `notify_one` are not buffered in any way. <br>不会以任何方式缓冲对 `notify_one` 的调用。<br>
    ///
    /// To wake up all threads, see [`notify_all`]. <br>要唤醒所有线程，请参见 [`notify_all`]。<br>
    ///
    /// [`wait`]: Self::wait
    /// [`wait_timeout`]: Self::wait_timeout
    /// [`notify_all`]: Self::notify_all
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::{Arc, Mutex, Condvar};
    /// use std::thread;
    ///
    /// let pair = Arc::new((Mutex::new(false), Condvar::new()));
    /// let pair2 = Arc::clone(&pair);
    ///
    /// thread::spawn(move|| {
    ///     let (lock, cvar) = &*pair2;
    ///     let mut started = lock.lock().unwrap();
    ///     *started = true;
    ///     // We notify the condvar that the value has changed. <br>我们通知 condvar 值已更改。<br>
    ///     cvar.notify_one();
    /// });
    ///
    /// // Wait for the thread to start up. <br>等待线程启动。<br>
    /// let (lock, cvar) = &*pair;
    /// let mut started = lock.lock().unwrap();
    /// // As long as the value inside the `Mutex<bool>` is `false`, we wait. <br>只要 `Mutex<bool>` 内部的值为 `false`，我们就等待。<br>
    /// while !*started {
    ///     started = cvar.wait(started).unwrap();
    /// }
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn notify_one(&self) {
        self.inner.notify_one()
    }

    /// Wakes up all blocked threads on this condvar. <br>唤醒此 condvar 上的所有阻塞线程。<br>
    ///
    /// This method will ensure that any current waiters on the condition variable are awoken. <br>此方法将确保唤醒条件变量上的所有当前侍者。<br>
    /// Calls to `notify_all()` are not buffered in any way. <br>不会以任何方式缓冲对 `notify_all()` 的调用。<br>
    ///
    /// To wake up only one thread, see [`notify_one`]. <br>要仅唤醒一个线程，请参见 [`notify_one`]。<br>
    ///
    /// [`notify_one`]: Self::notify_one
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::{Arc, Mutex, Condvar};
    /// use std::thread;
    ///
    /// let pair = Arc::new((Mutex::new(false), Condvar::new()));
    /// let pair2 = Arc::clone(&pair);
    ///
    /// thread::spawn(move|| {
    ///     let (lock, cvar) = &*pair2;
    ///     let mut started = lock.lock().unwrap();
    ///     *started = true;
    ///     // We notify the condvar that the value has changed. <br>我们通知 condvar 值已更改。<br>
    ///     cvar.notify_all();
    /// });
    ///
    /// // Wait for the thread to start up. <br>等待线程启动。<br>
    /// let (lock, cvar) = &*pair;
    /// let mut started = lock.lock().unwrap();
    /// // As long as the value inside the `Mutex<bool>` is `false`, we wait. <br>只要 `Mutex<bool>` 内部的值为 `false`，我们就等待。<br>
    /// while !*started {
    ///     started = cvar.wait(started).unwrap();
    /// }
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn notify_all(&self) {
        self.inner.notify_all()
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Condvar {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Condvar").finish_non_exhaustive()
    }
}

#[stable(feature = "condvar_default", since = "1.10.0")]
impl Default for Condvar {
    /// Creates a `Condvar` which is ready to be waited on and notified. <br>创建一个 `Condvar`，可以等待它并通知它。<br>
    fn default() -> Condvar {
        Condvar::new()
    }
}
