//! Useful synchronization primitives. <br>有用的同步原语。<br>
//!
//! ## The need for synchronization <br>需要同步<br>
//!
//! Conceptually, a Rust program is a series of operations which will be executed on a computer. <br>从概念上讲，Rust 程序是将在计算机上执行的一系列操作。<br> The timeline of events happening in the program is consistent with the order of the operations in the code. <br>程序中发生的事件的时间线与代码中操作的顺序一致。<br>
//!
//! Consider the following code, operating on some global static variables: <br>考虑下面的代码，对某些静态变量进行操作：<br>
//!
//! ```rust
//! static mut A: u32 = 0;
//! static mut B: u32 = 0;
//! static mut C: u32 = 0;
//!
//! fn main() {
//!     unsafe {
//!         A = 3;
//!         B = 4;
//!         A = A + B;
//!         C = B;
//!         println!("{A} {B} {C}");
//!         C = A;
//!     }
//! }
//! ```
//!
//! It appears as if some variables stored in memory are changed, an addition is performed, result is stored in `A` and the variable `C` is modified twice. <br>似乎存储在存储器中的某些变量已更改，执行了加法运算，结果存储在 `A` 中，并且变量 `C` 被修改了两次。<br>
//!
//! When only a single thread is involved, the results are as expected: <br>当仅涉及单个线程时，结果如预期的那样：<br>
//! the line `7 4 4` gets printed. <br>`7 4 4` 行被打印。<br>
//!
//! As for what happens behind the scenes, when optimizations are enabled the final generated machine code might look very different from the code: <br>至于幕后发生的情况，启用优化后，最终生成的机器代码可能看起来与代码有很大不同：<br>
//!
//! - The first store to `C` might be moved before the store to `A` or `B`, _as if_ we had written `C = 4; A = 3; B = 4`. <br>`C` 的第一个存储区可能在存储区移至 `A` 或 `B` 之前已被移至 _as if_。<br>
//!
//! - Assignment of `A + B` to `A` might be removed, since the sum can be stored in a temporary location until it gets printed, with the global variable never getting updated. <br>可以删除 `A + B` 到 `A` 的分配，因为总和可以存储在一个临时位置，直到打印出来为止，并且变量永远不会更新。<br>
//!
//! - The final result could be determined just by looking at the code at compile time, so [constant folding] might turn the whole block into a simple `println!("7 4 4")`. <br>最终结果可以仅通过在编译时查看代码来确定，因此 [常量合并][constant folding] 可能会将整个块变成一个简单的 `println!("7 4 4")`。<br>
//!
//! The compiler is allowed to perform any combination of these optimizations, as long as the final optimized code, when executed, produces the same results as the one without optimizations. <br>只要最终的优化代码在执行时产生的结果与没有优化的结果相同，编译器就可以执行这些优化的任何组合。<br>
//!
//! Due to the [concurrency] involved in modern computers, assumptions about the program's execution order are often wrong. <br>由于现代计算机中使用了 [并发][concurrency]，因此有关程序执行顺序的假设通常是错误的。<br>
//! Access to global variables can lead to nondeterministic results, **even if** compiler optimizations are disabled, and it is **still possible** to introduce synchronization bugs. <br>访问全局变量会导致不确定的结果，即使编译器优化被禁用，仍然可能引入同步错误。<br>
//!
//! Note that thanks to Rust's safety guarantees, accessing global (static) variables requires `unsafe` code, assuming we don't use any of the synchronization primitives in this module. <br>请注意，由于 Rust 的安全保证，假设我们在此模块中未使用任何同步原语，则访问 (static) 变量需要 `unsafe` 代码。<br>
//!
//! [constant folding]: https://en.wikipedia.org/wiki/Constant_folding
//! [concurrency]: https://en.wikipedia.org/wiki/Concurrency_(computer_science)
//!
//! ## Out-of-order execution <br>乱序执行<br>
//!
//! Instructions can execute in a different order from the one we define, due to various reasons: <br>由于多种原因，指令的执行顺序与我们定义的顺序可以不同：<br>
//!
//! - The **compiler** reordering instructions: If the compiler can issue an instruction at an earlier point, it will try to do so. <br>**编译器** 重新排序指令：如果编译器可以在较早的时候发出指令，它将尝试这样做。<br>
//! For example, it might hoist memory loads at the top of a code block, so that the CPU can start [prefetching] the values from memory. <br>例如，它可能会在代码块的顶部提升内存负载，以便 CPU 可以从内存中启动 [预取][prefetching] 值。<br>
//!
//!   In single-threaded scenarios, this can cause issues when writing signal handlers or certain kinds of low-level code. <br>在单线程方案中，这可能会在编写信号处理程序或某些类型的代码时引起问题。<br>
//!   Use [compiler fences] to prevent this reordering. <br>使用 [编译器栅栏][compiler fences] 来防止这种重新排序。<br>
//!
//! - A **single processor** executing instructions [out-of-order]: <br>一个单处理器执行指令 [乱序][out-of-order]：<br>
//!   Modern CPUs are capable of [superscalar] execution, i.e., multiple instructions might be executing at the same time, even though the machine code describes a sequential process. <br>现代的 CPU 能够执行 [超标量][superscalar]，也就是说，多个指令可能同时执行，即使机器代码描述的是一个顺序过程。<br>
//!
//!   This kind of reordering is handled transparently by the CPU. <br>这种重新排序由 CPU 透明地处理。<br>
//!
//! - A **multiprocessor** system executing multiple hardware threads at the same time: In multi-threaded scenarios, you can use two kinds of primitives to deal with synchronization: <br>同时执行多个硬件线程的 **多处理器** 系统：在多线程方案中，可以使用两种原语来处理同步：<br>
//!   - [memory fences] to ensure memory accesses are made visible to other CPUs in the right order. <br>[内存栅栏][memory fences] 以确保以正确的顺序对其他 CPU 进行内存访问。<br>
//!   - [atomic operations] to ensure simultaneous access to the same memory location doesn't lead to undefined behavior. <br>[原子操作][atomic operations] 以确保同时访问同一内存位置不会导致未定义的行为。<br>
//!
//! [prefetching]: https://en.wikipedia.org/wiki/Cache_prefetching
//! [compiler fences]: crate::sync::atomic::compiler_fence
//! [out-of-order]: https://en.wikipedia.org/wiki/Out-of-order_execution
//! [superscalar]: https://en.wikipedia.org/wiki/Superscalar_processor
//! [memory fences]: crate::sync::atomic::fence
//! [atomic operations]: crate::sync::atomic
//!
//! ## Higher-level synchronization objects <br>更高级别的同步对象<br>
//!
//! Most of the low-level synchronization primitives are quite error-prone and inconvenient to use, which is why the standard library also exposes some higher-level synchronization objects. <br>大多数同步原语都非常容易出错，使用起来也不方便，这就是为什么标准库还公开了一些更高级别的同步对象的原因。<br>
//!
//! These abstractions can be built out of lower-level primitives. <br>这些抽象可以在较低级别的原语基础上构建。<br>
//! For efficiency, the sync objects in the standard library are usually implemented with help from the operating system's kernel, which is able to reschedule the threads while they are blocked on acquiring a lock. <br>为了提高效率，标准库中的同步对象通常是在操作系统内核的帮助下实现的，该内核可以在线程被锁定而被阻塞时重新安排线程的时间。<br>
//!
//!
//! The following is an overview of the available synchronization objects: <br>以下是可用的同步对象的概述：<br>
//!
//! - [`Arc`]: Atomically Reference-Counted pointer, which can be used in multithreaded environments to prolong the lifetime of some data until all the threads have finished using it. <br>[`Arc`]: 原子引用计数指针，可以在多线程环境中使用，延长某些数据的生命周期，直到所有线程都执行完为止。<br>
//!
//! - [`Barrier`]: Ensures multiple threads will wait for each other to reach a point in the program, before continuing execution all together. <br>[`Barrier`]: 确保多个线程将等待彼此到达程序中的某个点，然后再一起继续执行。<br>
//!
//! - [`Condvar`]: Condition Variable, providing the ability to block a thread while waiting for an event to occur. <br>[`Condvar`]: 条件变量，提供在等待事件发生时阻塞线程的能力。<br>
//!
//! - [`mpsc`]: Multi-producer, single-consumer queues, used for message-based communication. <br>[`mpsc`]: 多生产者、单消费者队列，用于基于消息的通信。<br> Can provide a lightweight inter-thread synchronisation mechanism, at the cost of some extra memory. <br>可以提供轻量级的线程间同步机制，但要花一些额外的内存。<br>
//!
//! - [`Mutex`]: Mutual Exclusion mechanism, which ensures that at most one thread at a time is able to access some data. <br>[`Mutex`]: 互斥机制，保证一次最多有一个线程可以访问一些数据。<br>
//!
//! - [`Once`]: Used for thread-safe, one-time initialization of a global variable. <br>[`Once`]: 用于线程安全的一次性初始化全局变量。<br>
//!
//! - [`RwLock`]: Provides a mutual exclusion mechanism which allows multiple readers at the same time, while allowing only one writer at a time. <br>[`RwLock`]: 提供一种互斥机制，允许同时有多个 readers，但一次只允许一个 writer。<br> In some cases, this can be more efficient than a mutex. <br>在某些情况下，这可能比互斥锁更有效。<br>
//!
//! [`Arc`]: crate::sync::Arc
//! [`Barrier`]: crate::sync::Barrier
//! [`Condvar`]: crate::sync::Condvar
//! [`mpsc`]: crate::sync::mpsc
//! [`Mutex`]: crate::sync::Mutex
//! [`Once`]: crate::sync::Once
//! [`RwLock`]: crate::sync::RwLock
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

#[stable(feature = "rust1", since = "1.0.0")]
pub use alloc_crate::sync::{Arc, Weak};
#[stable(feature = "rust1", since = "1.0.0")]
pub use core::sync::atomic;

#[stable(feature = "rust1", since = "1.0.0")]
pub use self::barrier::{Barrier, BarrierWaitResult};
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::condvar::{Condvar, WaitTimeoutResult};
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::mutex::{Mutex, MutexGuard};
#[stable(feature = "rust1", since = "1.0.0")]
#[allow(deprecated)]
pub use self::once::{Once, OnceState, ONCE_INIT};
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::poison::{LockResult, PoisonError, TryLockError, TryLockResult};
#[stable(feature = "rust1", since = "1.0.0")]
pub use self::rwlock::{RwLock, RwLockReadGuard, RwLockWriteGuard};

#[unstable(feature = "once_cell", issue = "74465")]
pub use self::lazy_lock::LazyLock;
#[unstable(feature = "once_cell", issue = "74465")]
pub use self::once_lock::OnceLock;

pub mod mpsc;

mod barrier;
mod condvar;
mod lazy_lock;
mod mutex;
mod once;
mod once_lock;
mod poison;
mod rwlock;
