//! Multi-producer, single-consumer FIFO queue communication primitives. <br>多生产者，单消费者 FIFO 队列通信原语。<br>
//!
//! This module provides message-based communication over channels, concretely defined among three types: <br>该模块通过通道提供基于消息的通信，具体定义为以下三种类型：<br>
//!
//! * [`Sender`]
//! * [`SyncSender`]
//! * [`Receiver`]
//!
//! A [`Sender`] or [`SyncSender`] is used to send data to a [`Receiver`]. <br>[`Sender`] 或 [`SyncSender`] 用于将数据发送到 [`Receiver`]。<br>
//! Both senders are clone-able (multi-producer) such that many threads can send simultaneously to one receiver (single-consumer). <br>两个发送者都是可克隆的 (multi-producer)，因此许多线程可以同时发送到一个接收者 (single-consumer)。<br>
//!
//! These channels come in two flavors: <br>这些通道有两种口味：<br>
//!
//! 1. An asynchronous, infinitely buffered channel. <br>异步，无限缓冲的通道。<br>
//! The [`channel`] function will return a `(Sender, Receiver)` tuple where all sends will be **asynchronous** (they never block). <br>[`channel`] 函数将返回 `(Sender, Receiver)` 元组，其中所有发送都是异步的 (它们从不阻塞)。<br>
//! The channel conceptually has an infinite buffer. <br>通道在概念上具有无限的缓冲区。<br>
//!
//! 2. A synchronous, bounded channel. <br>同步的有界通道。<br>
//! The [`sync_channel`] function will return a `(SyncSender, Receiver)` tuple where the storage for pending messages is a pre-allocated buffer of a fixed size. <br>[`sync_channel`] 函数将返回 `(SyncSender, Receiver)` 元组，其中待处理消息的存储是固定大小的预分配缓冲区。<br>
//! All sends will be **synchronous** by blocking until there is buffer space available. <br>所有的发送都将被阻塞，直到有可用的缓冲区空间。<br>
//! Note that a bound of 0 is allowed, causing the channel to become a "rendezvous" channel where each sender atomically hands off a message to a receiver. <br>请注意，允许的界限为 0，从而使通道成为 "rendezvous" 通道，每个发送者在该通道上原子地将消息传递给接收者。<br>
//!
//! [`send`]: Sender::send
//!
//! ## Disconnection
//!
//! The send and receive operations on channels will all return a [`Result`] indicating whether the operation succeeded or not. <br>通道上的发送和接收操作都将返回 [`Result`]，指示该操作是否成功。<br>
//! An unsuccessful operation is normally indicative of the other half of a channel having "hung up" by being dropped in its corresponding thread. <br>不成功的操作通常通过将其丢弃在相应线程中来指示具有 "hung up" 的通道的另一半。<br>
//!
//! Once half of a channel has been deallocated, most operations can no longer continue to make progress, so [`Err`] will be returned. <br>释放通道的一半后，大多数操作将不再继续进行，因此将返回 [`Err`]。<br>
//! Many applications will continue to [`unwrap`] the results returned from this module, instigating a propagation of failure among threads if one unexpectedly dies. <br>许多应用程序将继续对该模块返回的结果进行 [`unwrap`] 处理，如果一个线程意外死亡，则会导致线程之间传播失败。<br>
//!
//!
//! [`unwrap`]: Result::unwrap
//!
//! # Examples
//!
//! Simple usage: <br>简单用法：<br>
//!
//! ```
//! use std::thread;
//! use std::sync::mpsc::channel;
//!
//! // Create a simple streaming channel <br>创建一个简单的流通道<br>
//! let (tx, rx) = channel();
//! thread::spawn(move|| {
//!     tx.send(10).unwrap();
//! });
//! assert_eq!(rx.recv().unwrap(), 10);
//! ```
//!
//! Shared usage: <br>共享用法：<br>
//!
//! ```
//! use std::thread;
//! use std::sync::mpsc::channel;
//!
//! // Create a shared channel that can be sent along from many threads where tx is the sending half (tx for transmission), and rx is the receiving half (rx for receiving). <br>创建一个可以从许多线程一起发送的共享通道，其中 tx 是发送一半 (用于传输的 tx)，rx 是接收一半 (用于接收的 rx)。<br>
//! //
//! //
//! let (tx, rx) = channel();
//! for i in 0..10 {
//!     let tx = tx.clone();
//!     thread::spawn(move|| {
//!         tx.send(i).unwrap();
//!     });
//! }
//!
//! for _ in 0..10 {
//!     let j = rx.recv().unwrap();
//!     assert!(0 <= j && j < 10);
//! }
//! ```
//!
//! Propagating panics: <br>传播 panics：<br>
//!
//! ```
//! use std::sync::mpsc::channel;
//!
//! // The call to recv() will return an error because the channel has already hung up (or been deallocated) <br>调用 recv() 将返回错误，因为通道已挂起 (或已释放)<br>
//! //
//! let (tx, rx) = channel::<i32>();
//! drop(tx);
//! assert!(rx.recv().is_err());
//! ```
//!
//! Synchronous channels: <br>同步通道：<br>
//!
//! ```
//! use std::thread;
//! use std::sync::mpsc::sync_channel;
//!
//! let (tx, rx) = sync_channel::<i32>(0);
//! thread::spawn(move|| {
//!     // This will wait for the parent thread to start receiving <br>这将等待父线程开始接收<br>
//!     tx.send(53).unwrap();
//! });
//! rx.recv().unwrap();
//! ```
//!
//! Unbounded receive loop: <br>无限接收循环：<br>
//!
//! ```
//! use std::sync::mpsc::sync_channel;
//! use std::thread;
//!
//! let (tx, rx) = sync_channel(3);
//!
//! for _ in 0..3 {
//!     // It would be the same without thread and clone here since there will still be one `tx` left. <br>这里没有线程和克隆也是一样的，因为仍然会剩下一个 `tx`。<br>
//!     //
//!     let tx = tx.clone();
//!     // cloned tx dropped within thread <br>克隆的 tx 丢弃在线程中<br>
//!     thread::spawn(move || tx.send("ok").unwrap());
//! }
//!
//! // Drop the last sender to stop `rx` waiting for message. <br>删除最后一个发送者停止 `rx` 等待消息。<br>
//! // The program will not complete if we comment this out. <br>如果我们将其注释掉，程序将无法完成。<br>
//! // **All** `tx` needs to be dropped for `rx` to have `Err`. <br>所有需要为 `rx` 排除 `tx` 才能拥有 `Err`。<br>
//! drop(tx);
//!
//! // Unbounded receiver waiting for all senders to complete. <br>无限接收者等待所有发送者完成。<br>
//! while let Ok(msg) = rx.recv() {
//!     println!("{msg}");
//! }
//!
//! println!("completed");
//! ```
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

#[cfg(all(test, not(target_os = "emscripten")))]
mod tests;

#[cfg(all(test, not(target_os = "emscripten")))]
mod sync_tests;

// A description of how Rust's channel implementation works <br>Rust 的通道实现的工作方式的描述<br>
//
// Channels are supposed to be the basic building block for all other concurrent primitives that are used in Rust. <br>通道应该是 Rust 中使用的所有其他并发原语的基本构建块。<br> As a result, the channel type needs to be highly optimized, flexible, and broad enough for use everywhere. <br>因此，通道类型需要高度优化，灵活并且足够广泛以在任何地方使用。<br>
//
// The choice of implementation of all channels is to be built on lock-free data structures. <br>所有通道的实现选择都将基于无锁数据结构。<br> The channels themselves are then consequently also lock-free data structures. <br>因此，通道本身也是无锁数据结构。<br>
// As always with lock-free code, this is a very "here be dragons" territory, especially because I'm unaware of any academic papers that have gone into great length about channels of these flavors. <br>与使用无锁代码一样，这是一个非常 "here be dragons" 的领域，特别是因为我不知道有任何关于这些风格的通道的学术论文。<br>
//
// ## Flavors of channels <br>风格的通道<br>
//
// From the perspective of a consumer of this library, there is only one flavor of channel. <br>从该库的使用者的角度来看，只有一种通道。<br> This channel can be used as a stream and cloned to allow multiple senders. <br>此通道可以用作流，并可以克隆以允许多个发送者。<br>
// Under the hood, however, there are actually three flavors of channels in play. <br>然而，在幕后，实际上有三种风格的通道在起作用。<br>
//
// * Flavor::Oneshots - these channels are highly optimized for the one-send use case. <br>Flavor::Oneshots - 这些通道针对单次发送用例进行了高度优化。<br> They contain as few atomics as possible and involve one and exactly one allocation. <br>它们包含尽可能少的原子，并且涉及一个且恰好是一个分配。<br>
// * Streams - these channels are optimized for the non-shared use case. <br>流 - 这些通道针对非共享用例进行了优化。<br> They use a different concurrent queue that is more tailored for this use case. <br>他们使用了一个更适合此用例的不同并发队列。<br> The initial allocation of this flavor of channel is not optimized. <br>这种风格的通道初始分配没有得到优化。<br>
// * Shared - this is the most general form of channel that this module offers, a channel with multiple senders. <br>共享 - 这是此模块提供的最通用的通道形式，即具有多个发送者的通道。<br> This type is as optimized as it can be, but the previous two types mentioned are much faster for their use-cases. <br>这种类型已尽可能地进行了优化，但是前面提到的两种类型在其用例方面要快得多。<br>
//
// ## Concurrent queues <br>并发队列<br>
//
// The basic idea of Rust's Sender/Receiver types is that send() never blocks, but recv() obviously blocks. <br>Rust 的 Sender/Receiver 类型的基本思想是 send() 永远不会阻塞，但 recv() 显然会阻塞。<br> This means that under the hood there must be some shared and concurrent queue holding all of the actual data. <br>这意味着在后台必须有一些共享和并发队列保存所有实际数据。<br>
//
// With two flavors of channels, two flavors of queues are also used. <br>对于两种通道，还使用两种队列。<br>
// We have chosen to use queues from a well-known author that are abbreviated as SPSC and MPSC (single producer, single consumer and multiple producer, single consumer). <br>我们选择使用一位著名作者的队列 (简称为 SPSC 和 MPSC) 的队列 (单生产者、单消费者和多生产者、单消费者)。<br>
// SPSC queues are used for streams while MPSC queues are used for shared channels. <br>SPSC 队列用于流，而 MPSC 队列用于共享通道。<br>
//
// ### SPSC optimizations <br>SPSC 优化<br>
//
// The SPSC queue found online is essentially a linked list of nodes where one half of the nodes are the "queue of data" and the other half of nodes are a cache of unused nodes. <br>在线找到的 SPSC 队列实质上是节点的链表，其中一半节点是 "数据队列"，另一半节点是未使用节点的缓存。<br>
// The unused nodes are used such that an allocation is not required on every push() and a free doesn't need to happen on every pop(). <br>使用未使用的节点，这样就不需要在每个 push() 上进行分配，并且不需要在每个 pop() 上进行分配。<br>
//
// As found online, however, the cache of nodes is of an infinite size. <br>然而，正如在网上发现的那样，节点的缓存是无限大的。<br> This means that if a channel at one point in its life had 50k items in the queue, then the queue will always have the capacity for 50k items. <br>这意味着，如果某个通道在其生命周期的某一点上有 50k 项，那么该队列将始终具有 50k 项的容量。<br>
// I believed that this was an unnecessary limitation of the implementation, so I have altered the queue to optionally have a bound on the cache size. <br>我认为这是对实现的不必要限制，因此我更改了队列以有选择地对缓存大小进行限制。<br>
//
// By default, streams will have an unbounded SPSC queue with a small-ish cache size. <br>默认情况下，流将具有无限制的 SPSC 队列，且缓存大小较小。<br>
// The hope is that the cache is still large enough to have very fast send() operations while not too large such that millions of channels can coexist at once. <br>希望缓存仍然足够大，可以进行非常快速的 send() 操作，而又不要太大，以至于数以百万计的通道可以一次共存。<br>
//
// ### MPSC optimizations <br>MPSC 优化<br>
//
// Right now the MPSC queue has not been optimized. <br>目前，MPSC 队列尚未优化。<br> Like the SPSC queue, it uses a linked list under the hood to earn its unboundedness, but I have not put forth much effort into having a cache of nodes similar to the SPSC queue. <br>像 SPSC 队列一样，它使用引擎盖下的链表来获取其无限制的内容，但是我并没有花太多精力来使节点的缓存类似于 SPSC 队列。<br>
//
// For now, I believe that this is "ok" because shared channels are not the most common type, but soon we may wish to revisit this queue choice and determine another candidate for backend storage of shared channels. <br>现在，我认为这是 "ok"，因为共享通道不是最常见的类型，但是不久之后，我们不妨重新访问此队列选择，并确定另一个用于共享通道的后端存储的候选对象。<br>
//
// ## Overview of the Implementation <br>实现概述<br>
//
// Now that there's a little background on the concurrent queues used, it's worth going into much more detail about the channels themselves. <br>既然使用的并发队列有一些背景知识，那么有必要进一步详细介绍通道本身。<br> The basic pseudocode for a send/recv are: <br>send/recv 的基本伪代码为：<br>
//
//      send(t)                             recv() queue.push(t)                       return if queue.pop() if increment() == -1                deschedule { wakeup()                            if decrement() > 0
//                                                cancel_deschedule()
//                                            }
//                                            queue.pop()
//
// As mentioned before, there are no locks in this implementation, only atomic instructions are used. <br>如前所述，此实现中没有锁，仅使用原子指令。<br>
//
// ### The internal atomic counter <br>内部原子计数器<br>
//
// Every channel has a shared counter with each half to keep track of the size of the queue. <br>每个通道都有一个共享的计数器，每个通道都有一个计数器，以跟踪队列的大小。<br> This counter is used to abort descheduling by the receiver and to know when to wake up on the sending side. <br>该计数器用于接收方终止调度，并知道何时在发送方唤醒。<br>
//
// As seen in the pseudocode, senders will increment this count and receivers will decrement the count. <br>从伪代码中可以看出，发送者将增加该计数，接收者将减少该计数。<br> The theory behind this is that if a sender sees a <br>其背后的理论是，如果发送者看到<br>
// -1 count, it will wake up the receiver, and if the receiver sees a 1+ count, then it doesn't need to block. <br>-1 计数，它将唤醒接收器，如果接收器看到 1+ 计数，那么它就不需要阻塞。<br>
//
// The recv() method has a beginning call to pop(), and if successful, it needs to decrement the count. <br>recv() 方法从 pop() 开始调用，如果成功，则需要减少计数。<br> It is a crucial implementation detail that this decrement does *not* happen to the shared counter. <br>这是一个至关重要的实现细节，此减量不会 * 发生在共享计数器上。<br>
// If this were the case, then it would be possible for the counter to be very negative when there were no receivers waiting, in which case the senders would have to determine when it was actually appropriate to wake up a receiver. <br>如果是这样，那么在没有等待接收者的情况下，计数器可能会变得非常负数，在这种情况下，发送者将不得不确定何时真正适合唤醒接收者。<br>
//
//
// Instead, the "steal count" is kept track of separately (not atomically because it's only used by receivers), and then the decrement() call when descheduling will lump in all of the recent steals into one large decrement. <br>取而代之的是，"steal count" 被分开跟踪 (不是原子地，因为它仅由接收者使用)，然后在调度时将 decrement() 调用将最近的所有窃取都归为一个大的递减。<br>
//
// The implication of this is that if a sender sees a -1 count, then there's guaranteed to be a waiter waiting! <br>这意味着如果发送方看到 -1 计数，那么肯定有一个等待者在等！<br>
//
// ## Native Implementation <br>原生实现<br>
//
// A major goal of these channels is to work seamlessly on and off the runtime. <br>这些通道的主要目标是在运行时上上下下无缝工作。<br>
// All of the previous race conditions have been worded in terms of scheduler-isms (which is obviously not available without the runtime). <br>以前的所有竞争状态都用调度程序表述 (显然，没有运行时就无法使用)。<br>
//
// For now, native usage of channels (off the runtime) will fall back onto mutexes/cond vars for descheduling/atomic decisions. <br>目前，通道的原生用法 (在运行时关闭) 将回退到 mutexes/cond 变量上进行 descheduling/atomic 决策。<br>
// The no-contention path is still entirely lock-free, the "deschedule" blocks above are surrounded by a mutex and the "wakeup" blocks involve grabbing a mutex and signaling on a condition variable. <br>无争用路径仍然是完全无锁的，上面的 "deschedule" 块被互斥锁包围，而 "wakeup" 块涉及获取互斥锁并在条件变量上发出信号。<br>
//
// ## Select
//
// Being able to support selection over channels has greatly influenced this design, and not only does selection need to work inside the runtime, but also outside the runtime. <br>能够支持通过通道进行选择极大地影响了该设计，选择不仅需要在运行时内部进行，而且还需要在运行时外部进行。<br>
//
// The implementation is fairly straightforward. <br>该实现非常简单。<br> The goal of select() is not to return some data, but only to return which channel can receive data without blocking. <br>select() 的目标不是返回某些数据，而只是返回哪个通道可以无阻塞地接收数据。<br>
// The implementation is essentially the entire blocking procedure followed by an increment as soon as its woken up. <br>该实现实质上是整个阻塞进程，随后在其唤醒后立即进行递增。<br>
// The cancellation procedure involves an increment and swapping out of to_wake to acquire ownership of the thread to unblock. <br>取消进程涉及一个增量，并交换出 to_wake 来获取要解锁的线程的所有权。<br>
//
// Sadly this current implementation requires multiple allocations, so I have seen the throughput of select() be much worse than it should be. <br>可悲的是，当前的实现需要多个分配，因此我已经看到 select() 的吞吐量比应该的要差得多。<br>
// I do not believe that there is anything fundamental that needs to change about these channels, however, in order to support a more efficient select(). <br>我认为，为了支持更高效的 select()，不需要对这些通道进行任何根本性的更改。<br>
//
// FIXME: Select is now removed, so these factors are ready to be cleaned up! <br>Select 现在已删除，因此可以清除这些因素！<br>
//
// # Conclusion
//
// And now that you've seen all the races that I found and attempted to fix, here's the code for you to find some more! <br>现在，您已经看到了我发现并尝试解决的所有问题，下面是为您找到更多代码的代码！<br>
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//

use crate::cell::UnsafeCell;
use crate::error;
use crate::fmt;
use crate::mem;
use crate::sync::Arc;
use crate::time::{Duration, Instant};

mod blocking;
mod mpsc_queue;
mod oneshot;
mod shared;
mod spsc_queue;
mod stream;
mod sync;

mod cache_aligned;

/// The receiving half of Rust's [`channel`] (or [`sync_channel`]) type. <br>Rust 的 [`channel`] (或 [`sync_channel`]) 类型的接收一半。<br>
/// This half can only be owned by one thread. <br>这一半只能由一个线程拥有。<br>
///
/// Messages sent to the channel can be retrieved using [`recv`]. <br>可以使用 [`recv`] 检索发送到通道的消息。<br>
///
/// [`recv`]: Receiver::recv
///
/// # Examples
///
/// ```rust
/// use std::sync::mpsc::channel;
/// use std::thread;
/// use std::time::Duration;
///
/// let (send, recv) = channel();
///
/// thread::spawn(move || {
///     send.send("Hello world!").unwrap();
///     thread::sleep(Duration::from_secs(2)); // block for two seconds <br>阻塞两秒钟<br>
///     send.send("Delayed for 2 seconds").unwrap();
/// });
///
/// println!("{}", recv.recv().unwrap()); // Received immediately <br>立即收到<br>
/// println!("Waiting...");
/// println!("{}", recv.recv().unwrap()); // Received after 2 seconds <br>2 秒后收到<br>
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "Receiver")]
pub struct Receiver<T> {
    inner: UnsafeCell<Flavor<T>>,
}

// The receiver port can be sent from place to place, so long as it is not used to receive non-sendable things. <br>接收者端口可以在任何地方发送，只要它不用于接收不可发送的东西即可。<br>
//
#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<T: Send> Send for Receiver<T> {}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> !Sync for Receiver<T> {}

/// An iterator over messages on a [`Receiver`], created by [`iter`]. <br>[`iter`] 在 [`Receiver`] 上创建的消息上的迭代器。<br>
///
/// This iterator will block whenever [`next`] is called, waiting for a new message, and [`None`] will be returned when the corresponding channel has hung up. <br>每当调用 [`next`]，等待新消息时，此迭代器将阻塞，并且在相应的通道挂起时，将返回 [`None`]。<br>
///
///
/// [`iter`]: Receiver::iter
/// [`next`]: Iterator::next
///
/// # Examples
///
/// ```rust
/// use std::sync::mpsc::channel;
/// use std::thread;
///
/// let (send, recv) = channel();
///
/// thread::spawn(move || {
///     send.send(1u8).unwrap();
///     send.send(2u8).unwrap();
///     send.send(3u8).unwrap();
/// });
///
/// for x in recv.iter() {
///     println!("Got: {x}");
/// }
/// ```
///
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Iter<'a, T: 'a> {
    rx: &'a Receiver<T>,
}

/// An iterator that attempts to yield all pending values for a [`Receiver`], created by [`try_iter`]. <br>尝试产生 [`try_iter`] 创建的 [`Receiver`] 的所有挂起值的迭代器。<br>
///
/// [`None`] will be returned when there are no pending values remaining or if the corresponding channel has hung up. <br>当没有剩余的挂起的值或对应的通道已挂断时，将返回 [`None`]。<br>
///
///
/// This iterator will never block the caller in order to wait for data to become available. <br>该迭代器永远不会阻塞调用者，以等待数据可用。<br>
/// Instead, it will return [`None`]. <br>相反，它将返回 [`None`]。<br>
///
/// [`try_iter`]: Receiver::try_iter
///
/// # Examples
///
/// ```rust
/// use std::sync::mpsc::channel;
/// use std::thread;
/// use std::time::Duration;
///
/// let (sender, receiver) = channel();
///
/// // Nothing is in the buffer yet <br>缓冲区什么都没有了<br>
/// assert!(receiver.try_iter().next().is_none());
/// println!("Nothing in the buffer...");
///
/// thread::spawn(move || {
///     sender.send(1).unwrap();
///     sender.send(2).unwrap();
///     sender.send(3).unwrap();
/// });
///
/// println!("Going to sleep...");
/// thread::sleep(Duration::from_secs(2)); // block for two seconds <br>阻塞两秒钟<br>
///
/// for x in receiver.try_iter() {
///     println!("Got: {x}");
/// }
/// ```
///
#[stable(feature = "receiver_try_iter", since = "1.15.0")]
#[derive(Debug)]
pub struct TryIter<'a, T: 'a> {
    rx: &'a Receiver<T>,
}

/// An owning iterator over messages on a [`Receiver`], created by [`into_iter`]. <br>[`Receiver`] 上消息的拥有所有权的迭代器，由 [`into_iter`] 创建。<br>
///
/// This iterator will block whenever [`next`] is called, waiting for a new message, and [`None`] will be returned if the corresponding channel has hung up. <br>每当调用 [`next`]，等待新消息时，此迭代器将阻塞，如果相应的通道已挂起，则将返回 [`None`]。<br>
///
///
/// [`into_iter`]: Receiver::into_iter
/// [`next`]: Iterator::next
///
/// # Examples
///
/// ```rust
/// use std::sync::mpsc::channel;
/// use std::thread;
///
/// let (send, recv) = channel();
///
/// thread::spawn(move || {
///     send.send(1u8).unwrap();
///     send.send(2u8).unwrap();
///     send.send(3u8).unwrap();
/// });
///
/// for x in recv.into_iter() {
///     println!("Got: {x}");
/// }
/// ```
///
///
#[stable(feature = "receiver_into_iter", since = "1.1.0")]
#[derive(Debug)]
pub struct IntoIter<T> {
    rx: Receiver<T>,
}

/// The sending-half of Rust's asynchronous [`channel`] type. <br>Rust 的异步 [`channel`] 类型的发送一半。<br>
/// This half can only be owned by one thread, but it can be cloned to send to other threads. <br>这一半只能由一个线程拥有，但可以克隆以发送给其他线程。<br>
///
/// Messages can be sent through this channel with [`send`]. <br>可以使用 [`send`] 通过此通道发送消息。<br>
///
/// Note: all senders (the original and the clones) need to be dropped for the receiver to stop blocking to receive messages with [`Receiver::recv`]. <br>所有发送者 (原始和克隆) 都需要被接收者去除，以停止阻止接收带有 [`Receiver::recv`] 的消息。<br>
///
///
/// [`send`]: Sender::send
///
/// # Examples
///
/// ```rust
/// use std::sync::mpsc::channel;
/// use std::thread;
///
/// let (sender, receiver) = channel();
/// let sender2 = sender.clone();
///
/// // First thread owns sender <br>第一个线程拥有发送者<br>
/// thread::spawn(move || {
///     sender.send(1).unwrap();
/// });
///
/// // Second thread owns sender2 <br>第二个线程拥有 sender2<br>
/// thread::spawn(move || {
///     sender2.send(2).unwrap();
/// });
///
/// let msg = receiver.recv().unwrap();
/// let msg2 = receiver.recv().unwrap();
///
/// assert_eq!(3, msg + msg2);
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Sender<T> {
    inner: UnsafeCell<Flavor<T>>,
}

// The send port can be sent from place to place, so long as it is not used to send non-sendable things. <br>只要不用于发送不可发送的东西，发送端口就可以从一个地方发送到另一个地方。<br>
//
#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<T: Send> Send for Sender<T> {}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> !Sync for Sender<T> {}

/// The sending-half of Rust's synchronous [`sync_channel`] type. <br>Rust 的同步 [`sync_channel`] 类型的发送一半。<br>
///
/// Messages can be sent through this channel with [`send`] or [`try_send`]. <br>可以使用 [`send`] 或 [`try_send`] 通过此通道发送消息。<br>
///
/// [`send`] will block if there is no space in the internal buffer. <br>如果内部缓冲区没有空间，[`send`] 将阻塞。<br>
///
/// [`send`]: SyncSender::send
/// [`try_send`]: SyncSender::try_send
///
/// # Examples
///
/// ```rust
/// use std::sync::mpsc::sync_channel;
/// use std::thread;
///
/// // Create a sync_channel with buffer size 2 <br>创建一个缓冲区大小为 2 的 sync_channel<br>
/// let (sync_sender, receiver) = sync_channel(2);
/// let sync_sender2 = sync_sender.clone();
///
/// // First thread owns sync_sender <br>第一个线程拥有 sync_sender<br>
/// thread::spawn(move || {
///     sync_sender.send(1).unwrap();
///     sync_sender.send(2).unwrap();
/// });
///
/// // Second thread owns sync_sender2 <br>第二个线程拥有 sync_sender2<br>
/// thread::spawn(move || {
///     sync_sender2.send(3).unwrap();
///     // thread will now block since the buffer is full <br>由于缓冲区已满，线程现在将阻塞<br>
///     println!("Thread unblocked!");
/// });
///
/// let mut msg;
///
/// msg = receiver.recv().unwrap();
/// println!("message {msg} received");
///
/// // "Thread unblocked!" will be printed now <br>现在将打印 "Thread unblocked!"<br>
///
/// msg = receiver.recv().unwrap();
/// println!("message {msg} received");
///
/// msg = receiver.recv().unwrap();
///
/// println!("message {msg} received");
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub struct SyncSender<T> {
    inner: Arc<sync::Packet<T>>,
}

#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl<T: Send> Send for SyncSender<T> {}

/// An error returned from the [`Sender::send`] or [`SyncSender::send`] function on **channel**s. <br>从 **通道** 上的 [`Sender::send`] 或 [`SyncSender::send`] 函数返回错误。<br>
///
/// A **send** operation can only fail if the receiving end of a channel is disconnected, implying that the data could never be received. <br>仅当通道的接收端断开连接时，发送操作才会失败，这意味着永远无法接收到数据。<br>
/// The error contains the data being sent as a payload so it can be recovered. <br>该错误包含作为有效载荷发送的数据，因此可以对其进行恢复。<br>
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(PartialEq, Eq, Clone, Copy)]
pub struct SendError<T>(#[stable(feature = "rust1", since = "1.0.0")] pub T);

/// An error returned from the [`recv`] function on a [`Receiver`]. <br>[`Receiver`] 上的 [`recv`] 函数返回错误。<br>
///
/// The [`recv`] operation can only fail if the sending half of a [`channel`] (or [`sync_channel`]) is disconnected, implying that no further messages will ever be received. <br>仅当 [`channel`] (或 [`sync_channel`]) 的发送半部断开连接时，[`recv`] 操作才会失败，这意味着将再也不会收到任何消息。<br>
///
///
/// [`recv`]: Receiver::recv
///
#[derive(PartialEq, Eq, Clone, Copy, Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub struct RecvError;

/// This enumeration is the list of the possible reasons that [`try_recv`] could not return data when called. <br>该枚举是 [`try_recv`] 在调用时无法返回数据的可能原因的列表。<br>
/// This can occur with both a [`channel`] and a [`sync_channel`]. <br>[`channel`] 和 [`sync_channel`] 都可能发生这种情况。<br>
///
/// [`try_recv`]: Receiver::try_recv
///
#[derive(PartialEq, Eq, Clone, Copy, Debug)]
#[stable(feature = "rust1", since = "1.0.0")]
pub enum TryRecvError {
    /// This **channel** is currently empty, but the **Sender**(s) have not yet disconnected, so data may yet become available. <br>该 **通道** 当前为空，但 **发送者** 尚未断开连接，因此数据可能仍可用。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    Empty,

    /// The **channel**'s sending half has become disconnected, and there will never be any more data received on it. <br>**通道** 的发送一半已断开，并且永远不会再收到任何数据。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    Disconnected,
}

/// This enumeration is the list of possible errors that made [`recv_timeout`] unable to return data when called. <br>该枚举是导致 [`recv_timeout`] 在调用时无法返回数据的可能错误的列表。<br>
/// This can occur with both a [`channel`] and a [`sync_channel`]. <br>[`channel`] 和 [`sync_channel`] 都可能发生这种情况。<br>
///
/// [`recv_timeout`]: Receiver::recv_timeout
///
#[derive(PartialEq, Eq, Clone, Copy, Debug)]
#[stable(feature = "mpsc_recv_timeout", since = "1.12.0")]
pub enum RecvTimeoutError {
    /// This **channel** is currently empty, but the **Sender**(s) have not yet disconnected, so data may yet become available. <br>该 **通道** 当前为空，但 **发送者** 尚未断开连接，因此数据可能仍可用。<br>
    ///
    #[stable(feature = "mpsc_recv_timeout", since = "1.12.0")]
    Timeout,
    /// The **channel**'s sending half has become disconnected, and there will never be any more data received on it. <br>**通道** 的发送一半已断开，并且永远不会再收到任何数据。<br>
    ///
    #[stable(feature = "mpsc_recv_timeout", since = "1.12.0")]
    Disconnected,
}

/// This enumeration is the list of the possible error outcomes for the [`try_send`] method. <br>此枚举是 [`try_send`] 方法可能的错误结果的列表。<br>
///
///
/// [`try_send`]: SyncSender::try_send
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(PartialEq, Eq, Clone, Copy)]
pub enum TrySendError<T> {
    /// The data could not be sent on the [`sync_channel`] because it would require that the callee block to send the data. <br>无法在 [`sync_channel`] 上发送数据，因为它需要被调用者阻止发送数据。<br>
    ///
    ///
    /// If this is a buffered channel, then the buffer is full at this time. <br>如果这是一个缓冲的通道，则此时缓冲区已满。<br>
    /// If this is not a buffered channel, then there is no [`Receiver`] available to acquire the data. <br>如果这不是缓冲的通道，则没有 [`Receiver`] 可用于获取数据。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    Full(#[stable(feature = "rust1", since = "1.0.0")] T),

    /// This [`sync_channel`]'s receiving half has disconnected, so the data could not be sent. <br>此 [`sync_channel`] 的接收部分已断开连接，因此无法发送数据。<br>
    /// The data is returned back to the callee in this case. <br>在这种情况下，数据将返回给被调用者。<br>
    #[stable(feature = "rust1", since = "1.0.0")]
    Disconnected(#[stable(feature = "rust1", since = "1.0.0")] T),
}

enum Flavor<T> {
    Oneshot(Arc<oneshot::Packet<T>>),
    Stream(Arc<stream::Packet<T>>),
    Shared(Arc<shared::Packet<T>>),
    Sync(Arc<sync::Packet<T>>),
}

#[doc(hidden)]
trait UnsafeFlavor<T> {
    fn inner_unsafe(&self) -> &UnsafeCell<Flavor<T>>;
    unsafe fn inner_mut(&self) -> &mut Flavor<T> {
        &mut *self.inner_unsafe().get()
    }
    unsafe fn inner(&self) -> &Flavor<T> {
        &*self.inner_unsafe().get()
    }
}
impl<T> UnsafeFlavor<T> for Sender<T> {
    fn inner_unsafe(&self) -> &UnsafeCell<Flavor<T>> {
        &self.inner
    }
}
impl<T> UnsafeFlavor<T> for Receiver<T> {
    fn inner_unsafe(&self) -> &UnsafeCell<Flavor<T>> {
        &self.inner
    }
}

/// Creates a new asynchronous channel, returning the sender/receiver halves. <br>创建一个新的异步通道，返回 sender/receiver 一半。<br>
/// All data sent on the [`Sender`] will become available on the [`Receiver`] in the same order as it was sent, and no [`send`] will block the calling thread (this channel has an "infinite buffer", unlike [`sync_channel`], which will block after its buffer limit is reached). <br>在 [`Sender`] 上发送的所有数据将以与发送时相同的顺序在 [`Receiver`] 上可用，并且没有 [`send`] 将阻塞调用线程 (此通道具有一个 "无限缓冲区"，与 [`sync_channel`] 不同，它将在达到其缓冲区限制后阻塞)。<br>
/// [`recv`] will block until a message is available while there is at least one [`Sender`] alive (including clones). <br>[`recv`] 将阻塞直到消息可用，同时至少有一个 [`Sender`] 活着 (包括克隆)。<br>
///
/// The [`Sender`] can be cloned to [`send`] to the same channel multiple times, but only one [`Receiver`] is supported. <br>[`Sender`] 可以多次克隆到 [`send`] 到同一通道，但是仅支持一个 [`Receiver`]。<br>
///
/// If the [`Receiver`] is disconnected while trying to [`send`] with the [`Sender`], the [`send`] method will return a [`SendError`]. <br>如果在尝试使用 [`Sender`] 进行 [`send`] 时断开 [`Receiver`] 的连接，则 [`send`] 方法将返回 [`SendError`]。<br> Similarly, if the [`Sender`] is disconnected while trying to [`recv`], the [`recv`] method will return a [`RecvError`]. <br>同样，如果在尝试 [`recv`] 时断开 [`Sender`] 的连接，则 [`recv`] 方法将返回 [`RecvError`]。<br>
///
/// [`send`]: Sender::send
/// [`recv`]: Receiver::recv
///
/// # Examples
///
/// ```
/// use std::sync::mpsc::channel;
/// use std::thread;
///
/// let (sender, receiver) = channel();
///
/// // Spawn off an expensive computation <br>Spawn 关闭昂贵的计算<br>
/// thread::spawn(move|| {
/// #   fn expensive_computation() {}
///     sender.send(expensive_computation()).unwrap();
/// });
///
/// // Do some useful work for awhile <br>暂时做一些有用的工作<br>
///
/// // Let's see what that answer was <br>让我们看看答案是什么<br>
/// println!("{:?}", receiver.recv().unwrap());
/// ```
///
///
///
///
///
///
///
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn channel<T>() -> (Sender<T>, Receiver<T>) {
    let a = Arc::new(oneshot::Packet::new());
    (Sender::new(Flavor::Oneshot(a.clone())), Receiver::new(Flavor::Oneshot(a)))
}

/// Creates a new synchronous, bounded channel. <br>创建一个新的同步有界通道。<br>
/// All data sent on the [`SyncSender`] will become available on the [`Receiver`] in the same order as it was sent. <br>[`SyncSender`] 上发送的所有数据将以与发送相同的顺序在 [`Receiver`] 上可用。<br>
/// Like asynchronous [`channel`]s, the [`Receiver`] will block until a message becomes available. <br>像异步 [`channel`] 一样，[`Receiver`] 将阻塞直到消息可用为止。<br>
/// `sync_channel` differs greatly in the semantics of the sender, however. <br>然而，`sync_channel` 在发送者的语义上有很大不同。<br>
///
/// This channel has an internal buffer on which messages will be queued. <br>该通道具有内部缓冲区，消息将在该缓冲区上排队。<br>
/// `bound` specifies the buffer size. <br>`bound` 指定缓冲区大小。<br>
/// When the internal buffer becomes full, future sends will *block* waiting for the buffer to open up. <br>当内部缓冲区已满时，future 将发送 *block*，等待缓冲区打开。<br>
/// Note that a buffer size of 0 is valid, in which case this becomes "rendezvous channel" where each [`send`] will not return until a [`recv`] is paired with it. <br>请注意，缓冲区大小为 0 是有效的，在这种情况下，它变为 "会合通道"，其中每个 [`send`] 在与 [`recv`] 配对之前不会返回。<br>
///
/// The [`SyncSender`] can be cloned to [`send`] to the same channel multiple times, but only one [`Receiver`] is supported. <br>[`SyncSender`] 可以多次克隆到 [`send`] 到同一通道，但是仅支持一个 [`Receiver`]。<br>
///
/// Like asynchronous channels, if the [`Receiver`] is disconnected while trying to [`send`] with the [`SyncSender`], the [`send`] method will return a [`SendError`]. <br>与异步通道一样，如果在尝试使用 [`SyncSender`] 进行 [`send`] 时断开 [`Receiver`]，则 [`send`] 方法将返回 [`SendError`]。<br>
///
/// Similarly, If the [`SyncSender`] is disconnected while trying to [`recv`], the [`recv`] method will return a [`RecvError`]. <br>同样，如果在尝试 [`recv`] 时断开 [`SyncSender`] 的连接，则 [`recv`] 方法将返回 [`RecvError`]。<br>
///
/// [`send`]: SyncSender::send
/// [`recv`]: Receiver::recv
///
/// # Examples
///
/// ```
/// use std::sync::mpsc::sync_channel;
/// use std::thread;
///
/// let (sender, receiver) = sync_channel(1);
///
/// // this returns immediately <br>这立即返回<br>
/// sender.send(1).unwrap();
///
/// thread::spawn(move|| {
///     // this will block until the previous message has been received <br>这将阻塞，直到收到上一条消息为止<br>
///     sender.send(2).unwrap();
/// });
///
/// assert_eq!(receiver.recv().unwrap(), 1);
/// assert_eq!(receiver.recv().unwrap(), 2);
/// ```
///
///
///
///
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn sync_channel<T>(bound: usize) -> (SyncSender<T>, Receiver<T>) {
    let a = Arc::new(sync::Packet::new(bound));
    (SyncSender::new(a.clone()), Receiver::new(Flavor::Sync(a)))
}

////////////////////////////////////////////////////////////////////////////////
// Sender
////////////////////////////////////////////////////////////////////////////////

impl<T> Sender<T> {
    fn new(inner: Flavor<T>) -> Sender<T> {
        Sender { inner: UnsafeCell::new(inner) }
    }

    /// Attempts to send a value on this channel, returning it back if it could not be sent. <br>尝试在此通道上发送值，如果无法发送，则将其返回。<br>
    ///
    /// A successful send occurs when it is determined that the other end of the channel has not hung up already. <br>当确定通道的另一端尚未挂断时，发送成功。<br>
    /// An unsuccessful send would be one where the corresponding receiver has already been deallocated. <br>不成功的发送将是相应的接收者已被重新分配的发送。<br>
    /// Note that a return value of [`Err`] means that the data will never be received, but a return value of [`Ok`] does *not* mean that the data will be received. <br>请注意，返回值 [`Err`] 表示将永远不会接收到数据，但是 [`Ok`] 的返回值 *不是* 意味着将接收到数据。<br>
    ///
    /// It is possible for the corresponding receiver to hang up immediately after this function returns [`Ok`]. <br>此函数返回 [`Ok`] 之后，相应的接收者有可能立即挂断。<br>
    ///
    /// This method will never block the current thread. <br>此方法永远不会阻塞当前线程。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::mpsc::channel;
    ///
    /// let (tx, rx) = channel();
    ///
    /// // This send is always successful <br>此发送始终成功<br>
    /// tx.send(1).unwrap();
    ///
    /// // This send will fail because the receiver is gone <br>由于接收者不见了，因此发送失败<br>
    /// drop(rx);
    /// assert_eq!(tx.send(1).unwrap_err().0, 1);
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn send(&self, t: T) -> Result<(), SendError<T>> {
        let (new_inner, ret) = match *unsafe { self.inner() } {
            Flavor::Oneshot(ref p) => {
                if !p.sent() {
                    return p.send(t).map_err(SendError);
                } else {
                    let a = Arc::new(stream::Packet::new());
                    let rx = Receiver::new(Flavor::Stream(a.clone()));
                    match p.upgrade(rx) {
                        oneshot::UpSuccess => {
                            let ret = a.send(t);
                            (a, ret)
                        }
                        oneshot::UpDisconnected => (a, Err(t)),
                        oneshot::UpWoke(token) => {
                            // This send cannot panic because the thread is asleep (we're looking at it), so the receiver can't go away. <br>此发送不能 panic，因为线程处于睡眠状态 (我们正在查看它)，因此接收者无法消失。<br>
                            //
                            //
                            a.send(t).ok().unwrap();
                            token.signal();
                            (a, Ok(()))
                        }
                    }
                }
            }
            Flavor::Stream(ref p) => return p.send(t).map_err(SendError),
            Flavor::Shared(ref p) => return p.send(t).map_err(SendError),
            Flavor::Sync(..) => unreachable!(),
        };

        unsafe {
            let tmp = Sender::new(Flavor::Stream(new_inner));
            mem::swap(self.inner_mut(), tmp.inner_mut());
        }
        ret.map_err(SendError)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Clone for Sender<T> {
    /// Clone a sender to send to other threads. <br>克隆发送者以发送到其他线程。<br>
    ///
    /// Note, be aware of the lifetime of the sender because all senders (including the original) need to be dropped in order for [`Receiver::recv`] to stop blocking. <br>请注意，请注意发送方的生命周期，因为所有发送方 (包括原始发送方) 都需要丢弃，以便 [`Receiver::recv`] 停止阻塞。<br>
    ///
    ///
    fn clone(&self) -> Sender<T> {
        let packet = match *unsafe { self.inner() } {
            Flavor::Oneshot(ref p) => {
                let a = Arc::new(shared::Packet::new());
                {
                    let guard = a.postinit_lock();
                    let rx = Receiver::new(Flavor::Shared(a.clone()));
                    let sleeper = match p.upgrade(rx) {
                        oneshot::UpSuccess | oneshot::UpDisconnected => None,
                        oneshot::UpWoke(task) => Some(task),
                    };
                    a.inherit_blocker(sleeper, guard);
                }
                a
            }
            Flavor::Stream(ref p) => {
                let a = Arc::new(shared::Packet::new());
                {
                    let guard = a.postinit_lock();
                    let rx = Receiver::new(Flavor::Shared(a.clone()));
                    let sleeper = match p.upgrade(rx) {
                        stream::UpSuccess | stream::UpDisconnected => None,
                        stream::UpWoke(task) => Some(task),
                    };
                    a.inherit_blocker(sleeper, guard);
                }
                a
            }
            Flavor::Shared(ref p) => {
                p.clone_chan();
                return Sender::new(Flavor::Shared(p.clone()));
            }
            Flavor::Sync(..) => unreachable!(),
        };

        unsafe {
            let tmp = Sender::new(Flavor::Shared(packet.clone()));
            mem::swap(self.inner_mut(), tmp.inner_mut());
        }
        Sender::new(Flavor::Shared(packet))
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Drop for Sender<T> {
    fn drop(&mut self) {
        match *unsafe { self.inner() } {
            Flavor::Oneshot(ref p) => p.drop_chan(),
            Flavor::Stream(ref p) => p.drop_chan(),
            Flavor::Shared(ref p) => p.drop_chan(),
            Flavor::Sync(..) => unreachable!(),
        }
    }
}

#[stable(feature = "mpsc_debug", since = "1.8.0")]
impl<T> fmt::Debug for Sender<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Sender").finish_non_exhaustive()
    }
}

////////////////////////////////////////////////////////////////////////////////
// SyncSender
////////////////////////////////////////////////////////////////////////////////

impl<T> SyncSender<T> {
    fn new(inner: Arc<sync::Packet<T>>) -> SyncSender<T> {
        SyncSender { inner }
    }

    /// Sends a value on this synchronous channel. <br>在此同步通道上发送一个值。<br>
    ///
    /// This function will *block* until space in the internal buffer becomes available or a receiver is available to hand off the message to. <br>该函数将阻塞，直到内部缓冲区中的空间可用或接收者可以将消息传递给它为止。<br>
    ///
    /// Note that a successful send does *not* guarantee that the receiver will ever see the data if there is a buffer on this channel. <br>请注意，如果此通道上有缓冲区，则成功发送并不能保证接收者会看到数据。<br>
    /// Items may be enqueued in the internal buffer for the receiver to receive at a later time. <br>该项可能会排队在内部缓冲区中，以供接收者在以后的时间接收。<br>
    /// If the buffer size is 0, however, the channel becomes a rendezvous channel and it guarantees that the receiver has indeed received the data if this function returns success. <br>但是，如果缓冲区大小为 0，则通道成为集合通道，并且如果此函数返回成功，则它保证接收者确实已接收到数据。<br>
    ///
    ///
    /// This function will never panic, but it may return [`Err`] if the [`Receiver`] has disconnected and is no longer able to receive information. <br>此函数永远不会 panic，但是如果 [`Receiver`] 已断开连接并且不再能够接收信息，则它可能返回 [`Err`]。<br>
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::sync::mpsc::sync_channel;
    /// use std::thread;
    ///
    /// // Create a rendezvous sync_channel with buffer size 0 <br>创建一个缓冲区大小为 0 的集合点 sync_channel<br>
    /// let (sync_sender, receiver) = sync_channel(0);
    ///
    /// thread::spawn(move || {
    ///    println!("sending message...");
    ///    sync_sender.send(1).unwrap();
    ///    // Thread is now blocked until the message is received <br>现在线程被阻塞，直到收到消息为止<br>
    ///
    ///    println!("...message received!");
    /// });
    ///
    /// let msg = receiver.recv().unwrap();
    /// assert_eq!(1, msg);
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn send(&self, t: T) -> Result<(), SendError<T>> {
        self.inner.send(t).map_err(SendError)
    }

    /// Attempts to send a value on this channel without blocking. <br>尝试在此通道上发送值而不会阻塞。<br>
    ///
    /// This method differs from [`send`] by returning immediately if the channel's buffer is full or no receiver is waiting to acquire some data. <br>此方法不同于 [`send`]，如果通道的缓冲区已满或没有接收者正在等待获取某些数据，则立即返回。<br>
    /// Compared with [`send`], this function has two failure cases instead of one (one for disconnection, one for a full buffer). <br>与 [`send`] 相比，此函数有两种故障情况，而不是一种情况 (一种情况是断开连接，一种情况是完整的缓冲区)。<br>
    ///
    /// See [`send`] for notes about guarantees of whether the receiver has received the data or not if this function is successful. <br>请参见 [`send`]，以获取有关确保此函数成功后接收方是否已接收到数据的说明。<br>
    ///
    /// [`send`]: Self::send
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::sync::mpsc::sync_channel;
    /// use std::thread;
    ///
    /// // Create a sync_channel with buffer size 1 <br>创建一个缓冲区大小为 1 的 sync_channel<br>
    /// let (sync_sender, receiver) = sync_channel(1);
    /// let sync_sender2 = sync_sender.clone();
    ///
    /// // First thread owns sync_sender <br>第一个线程拥有 sync_sender<br>
    /// thread::spawn(move || {
    ///     sync_sender.send(1).unwrap();
    ///     sync_sender.send(2).unwrap();
    ///     // Thread blocked <br>线程被阻塞<br>
    /// });
    ///
    /// // Second thread owns sync_sender2 <br>第二个线程拥有 sync_sender2<br>
    /// thread::spawn(move || {
    ///     // This will return an error and send no message if the buffer is full <br>如果缓冲区已满，这将返回错误并且不发送任何消息<br>
    /////
    ///     let _ = sync_sender2.try_send(3);
    /// });
    ///
    /// let mut msg;
    /// msg = receiver.recv().unwrap();
    /// println!("message {msg} received");
    ///
    /// msg = receiver.recv().unwrap();
    /// println!("message {msg} received");
    ///
    /// // Third message may have never been sent <br>第三条消息可能从未发送过<br>
    /// match receiver.try_recv() {
    ///     Ok(msg) => println!("message {msg} received"),
    ///     Err(_) => println!("the third message was never sent"),
    /// }
    /// ```
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn try_send(&self, t: T) -> Result<(), TrySendError<T>> {
        self.inner.try_send(t)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Clone for SyncSender<T> {
    fn clone(&self) -> SyncSender<T> {
        self.inner.clone_chan();
        SyncSender::new(self.inner.clone())
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Drop for SyncSender<T> {
    fn drop(&mut self) {
        self.inner.drop_chan();
    }
}

#[stable(feature = "mpsc_debug", since = "1.8.0")]
impl<T> fmt::Debug for SyncSender<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("SyncSender").finish_non_exhaustive()
    }
}

////////////////////////////////////////////////////////////////////////////////
// Receiver
////////////////////////////////////////////////////////////////////////////////

impl<T> Receiver<T> {
    fn new(inner: Flavor<T>) -> Receiver<T> {
        Receiver { inner: UnsafeCell::new(inner) }
    }

    /// Attempts to return a pending value on this receiver without blocking. <br>尝试在不阻塞的情况下在此接收者上返回挂起的值。<br>
    ///
    /// This method will never block the caller in order to wait for data to become available. <br>此方法决不会阻塞调用者，以等待数据可用。<br>
    /// Instead, this will always return immediately with a possible option of pending data on the channel. <br>取而代之的是，它总是立即返回，并可能在通道上保留未决数据。<br>
    ///
    /// This is useful for a flavor of "optimistic check" before deciding to block on a receiver. <br>在决定阻塞接收者之前，这对于 "optimistic check" 很有用。<br>
    ///
    /// Compared with [`recv`], this function has two failure cases instead of one (one for disconnection, one for an empty buffer). <br>与 [`recv`] 相比，此函数有两种故障情况，而不是一种情况 (一种情况是断开连接，一种情况是空缓冲区)。<br>
    ///
    ///
    /// [`recv`]: Self::recv
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::sync::mpsc::{Receiver, channel};
    ///
    /// let (_, receiver): (_, Receiver<i32>) = channel();
    ///
    /// assert!(receiver.try_recv().is_err());
    /// ```
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn try_recv(&self) -> Result<T, TryRecvError> {
        loop {
            let new_port = match *unsafe { self.inner() } {
                Flavor::Oneshot(ref p) => match p.try_recv() {
                    Ok(t) => return Ok(t),
                    Err(oneshot::Empty) => return Err(TryRecvError::Empty),
                    Err(oneshot::Disconnected) => return Err(TryRecvError::Disconnected),
                    Err(oneshot::Upgraded(rx)) => rx,
                },
                Flavor::Stream(ref p) => match p.try_recv() {
                    Ok(t) => return Ok(t),
                    Err(stream::Empty) => return Err(TryRecvError::Empty),
                    Err(stream::Disconnected) => return Err(TryRecvError::Disconnected),
                    Err(stream::Upgraded(rx)) => rx,
                },
                Flavor::Shared(ref p) => match p.try_recv() {
                    Ok(t) => return Ok(t),
                    Err(shared::Empty) => return Err(TryRecvError::Empty),
                    Err(shared::Disconnected) => return Err(TryRecvError::Disconnected),
                },
                Flavor::Sync(ref p) => match p.try_recv() {
                    Ok(t) => return Ok(t),
                    Err(sync::Empty) => return Err(TryRecvError::Empty),
                    Err(sync::Disconnected) => return Err(TryRecvError::Disconnected),
                },
            };
            unsafe {
                mem::swap(self.inner_mut(), new_port.inner_mut());
            }
        }
    }

    /// Attempts to wait for a value on this receiver, returning an error if the corresponding channel has hung up. <br>尝试等待此接收者上的值，如果相应的通道已挂起，则返回错误。<br>
    ///
    /// This function will always block the current thread if there is no data available and it's possible for more data to be sent (at least one sender still exists). <br>如果没有可用数据并且有可能发送更多数据 (至少一个发送者仍然存在)，此函数将始终阻塞当前线程。<br>
    /// Once a message is sent to the corresponding [`Sender`] (or [`SyncSender`]), this receiver will wake up and return that message. <br>一旦消息发送到相应的 [`Sender`] (或 [`SyncSender`])，该接收者将唤醒并返回该消息。<br>
    ///
    /// If the corresponding [`Sender`] has disconnected, or it disconnects while this call is blocking, this call will wake up and return [`Err`] to indicate that no more messages can ever be received on this channel. <br>如果相应的 [`Sender`] 断开连接，或者在此调用阻塞时断开连接，则此调用将唤醒并返回 [`Err`]，以指示该通道上再也不会收到任何消息。<br>
    ///
    /// However, since channels are buffered, messages sent before the disconnect will still be properly received. <br>但是，由于缓冲了通道，因此在断开连接之前发送的消息仍将被正确接收。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::mpsc;
    /// use std::thread;
    ///
    /// let (send, recv) = mpsc::channel();
    /// let handle = thread::spawn(move || {
    ///     send.send(1u8).unwrap();
    /// });
    ///
    /// handle.join().unwrap();
    ///
    /// assert_eq!(Ok(1), recv.recv());
    /// ```
    ///
    /// Buffering behavior: <br>缓冲行为：<br>
    ///
    /// ```
    /// use std::sync::mpsc;
    /// use std::thread;
    /// use std::sync::mpsc::RecvError;
    ///
    /// let (send, recv) = mpsc::channel();
    /// let handle = thread::spawn(move || {
    ///     send.send(1u8).unwrap();
    ///     send.send(2).unwrap();
    ///     send.send(3).unwrap();
    ///     drop(send);
    /// });
    ///
    /// // wait for the thread to join so we ensure the sender is dropped <br>等待线程加入，因此我们确保发送者已被丢弃<br>
    /// handle.join().unwrap();
    ///
    /// assert_eq!(Ok(1), recv.recv());
    /// assert_eq!(Ok(2), recv.recv());
    /// assert_eq!(Ok(3), recv.recv());
    /// assert_eq!(Err(RecvError), recv.recv());
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn recv(&self) -> Result<T, RecvError> {
        loop {
            let new_port = match *unsafe { self.inner() } {
                Flavor::Oneshot(ref p) => match p.recv(None) {
                    Ok(t) => return Ok(t),
                    Err(oneshot::Disconnected) => return Err(RecvError),
                    Err(oneshot::Upgraded(rx)) => rx,
                    Err(oneshot::Empty) => unreachable!(),
                },
                Flavor::Stream(ref p) => match p.recv(None) {
                    Ok(t) => return Ok(t),
                    Err(stream::Disconnected) => return Err(RecvError),
                    Err(stream::Upgraded(rx)) => rx,
                    Err(stream::Empty) => unreachable!(),
                },
                Flavor::Shared(ref p) => match p.recv(None) {
                    Ok(t) => return Ok(t),
                    Err(shared::Disconnected) => return Err(RecvError),
                    Err(shared::Empty) => unreachable!(),
                },
                Flavor::Sync(ref p) => return p.recv(None).map_err(|_| RecvError),
            };
            unsafe {
                mem::swap(self.inner_mut(), new_port.inner_mut());
            }
        }
    }

    /// Attempts to wait for a value on this receiver, returning an error if the corresponding channel has hung up, or if it waits more than `timeout`. <br>尝试等待此接收器上的值，如果相应的通道已挂起，或者等待的时间超过 `timeout`，则返回错误。<br>
    ///
    /// This function will always block the current thread if there is no data available and it's possible for more data to be sent (at least one sender still exists). <br>如果没有可用数据并且有可能发送更多数据 (至少一个发送者仍然存在)，此函数将始终阻塞当前线程。<br>
    /// Once a message is sent to the corresponding [`Sender`] (or [`SyncSender`]), this receiver will wake up and return that message. <br>一旦消息发送到相应的 [`Sender`] (或 [`SyncSender`])，该接收者将唤醒并返回该消息。<br>
    ///
    /// If the corresponding [`Sender`] has disconnected, or it disconnects while this call is blocking, this call will wake up and return [`Err`] to indicate that no more messages can ever be received on this channel. <br>如果相应的 [`Sender`] 断开连接，或者在此调用阻塞时断开连接，则此调用将唤醒并返回 [`Err`]，以指示该通道上再也不会收到任何消息。<br>
    ///
    /// However, since channels are buffered, messages sent before the disconnect will still be properly received. <br>但是，由于缓冲了通道，因此在断开连接之前发送的消息仍将被正确接收。<br>
    ///
    /// # Known Issues <br>已知的问题<br>
    ///
    /// There is currently a known issue (see [`#39364`]) that causes `recv_timeout` to panic unexpectedly with the following example: <br>当前存在一个已知问题 (请参见 [`#39364`])，以下示例导致 `recv_timeout` 意外变为 panic：<br>
    ///
    /// ```no_run
    /// use std::sync::mpsc::channel;
    /// use std::thread;
    /// use std::time::Duration;
    ///
    /// let (tx, rx) = channel::<String>();
    ///
    /// thread::spawn(move || {
    ///     let d = Duration::from_millis(10);
    ///     loop {
    ///         println!("recv");
    ///         let _r = rx.recv_timeout(d);
    ///     }
    /// });
    ///
    /// thread::sleep(Duration::from_millis(100));
    /// let _c1 = tx.clone();
    ///
    /// thread::sleep(Duration::from_secs(1));
    /// ```
    ///
    /// [`#39364`]: https://github.com/rust-lang/rust/issues/39364
    ///
    /// # Examples
    ///
    /// Successfully receiving value before encountering timeout: <br>在遇到超时之前成功获得值：<br>
    ///
    /// ```no_run
    /// use std::thread;
    /// use std::time::Duration;
    /// use std::sync::mpsc;
    ///
    /// let (send, recv) = mpsc::channel();
    ///
    /// thread::spawn(move || {
    ///     send.send('a').unwrap();
    /// });
    ///
    /// assert_eq!(
    ///     recv.recv_timeout(Duration::from_millis(400)),
    ///     Ok('a')
    /// );
    /// ```
    ///
    /// Receiving an error upon reaching timeout: <br>到达超时时收到错误：<br>
    ///
    /// ```no_run
    /// use std::thread;
    /// use std::time::Duration;
    /// use std::sync::mpsc;
    ///
    /// let (send, recv) = mpsc::channel();
    ///
    /// thread::spawn(move || {
    ///     thread::sleep(Duration::from_millis(800));
    ///     send.send('a').unwrap();
    /// });
    ///
    /// assert_eq!(
    ///     recv.recv_timeout(Duration::from_millis(400)),
    ///     Err(mpsc::RecvTimeoutError::Timeout)
    /// );
    /// ```
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "mpsc_recv_timeout", since = "1.12.0")]
    pub fn recv_timeout(&self, timeout: Duration) -> Result<T, RecvTimeoutError> {
        // Do an optimistic try_recv to avoid the performance impact of Instant::now() in the full-channel case. <br>进行乐观的 try_recv 以避免全通道情况下 Instant::now () 的性能影响。<br>
        //
        match self.try_recv() {
            Ok(result) => Ok(result),
            Err(TryRecvError::Disconnected) => Err(RecvTimeoutError::Disconnected),
            Err(TryRecvError::Empty) => match Instant::now().checked_add(timeout) {
                Some(deadline) => self.recv_deadline(deadline),
                // So far in the future that it's practically the same as waiting indefinitely. <br>到目前为止，在 future 中，它实际上与无限期等待相同。<br>
                None => self.recv().map_err(RecvTimeoutError::from),
            },
        }
    }

    /// Attempts to wait for a value on this receiver, returning an error if the corresponding channel has hung up, or if `deadline` is reached. <br>尝试等待此接收器上的值，如果相应的通道已挂起，或者到达 `deadline`，则返回错误。<br>
    ///
    /// This function will always block the current thread if there is no data available and it's possible for more data to be sent. <br>如果没有可用数据，并且有可能发送更多数据，则此函数将始终阻止当前线程。<br>
    /// Once a message is sent to the corresponding [`Sender`] (or [`SyncSender`]), then this receiver will wake up and return that message. <br>将消息发送到相应的 [`Sender`] (或 [`SyncSender`]) 后，此接收者将唤醒并返回该消息。<br>
    ///
    /// If the corresponding [`Sender`] has disconnected, or it disconnects while this call is blocking, this call will wake up and return [`Err`] to indicate that no more messages can ever be received on this channel. <br>如果相应的 [`Sender`] 断开连接，或者在此调用阻塞时断开连接，则此调用将唤醒并返回 [`Err`]，以指示该通道上再也不会收到任何消息。<br>
    ///
    /// However, since channels are buffered, messages sent before the disconnect will still be properly received. <br>但是，由于缓冲了通道，因此在断开连接之前发送的消息仍将被正确接收。<br>
    ///
    /// # Examples
    ///
    /// Successfully receiving value before reaching deadline: <br>在截止日期之前成功获得值：<br>
    ///
    /// ```no_run
    /// #![feature(deadline_api)]
    /// use std::thread;
    /// use std::time::{Duration, Instant};
    /// use std::sync::mpsc;
    ///
    /// let (send, recv) = mpsc::channel();
    ///
    /// thread::spawn(move || {
    ///     send.send('a').unwrap();
    /// });
    ///
    /// assert_eq!(
    ///     recv.recv_deadline(Instant::now() + Duration::from_millis(400)),
    ///     Ok('a')
    /// );
    /// ```
    ///
    /// Receiving an error upon reaching deadline: <br>在截止日期前收到错误：<br>
    ///
    /// ```no_run
    /// #![feature(deadline_api)]
    /// use std::thread;
    /// use std::time::{Duration, Instant};
    /// use std::sync::mpsc;
    ///
    /// let (send, recv) = mpsc::channel();
    ///
    /// thread::spawn(move || {
    ///     thread::sleep(Duration::from_millis(800));
    ///     send.send('a').unwrap();
    /// });
    ///
    /// assert_eq!(
    ///     recv.recv_deadline(Instant::now() + Duration::from_millis(400)),
    ///     Err(mpsc::RecvTimeoutError::Timeout)
    /// );
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[unstable(feature = "deadline_api", issue = "46316")]
    pub fn recv_deadline(&self, deadline: Instant) -> Result<T, RecvTimeoutError> {
        use self::RecvTimeoutError::*;

        loop {
            let port_or_empty = match *unsafe { self.inner() } {
                Flavor::Oneshot(ref p) => match p.recv(Some(deadline)) {
                    Ok(t) => return Ok(t),
                    Err(oneshot::Disconnected) => return Err(Disconnected),
                    Err(oneshot::Upgraded(rx)) => Some(rx),
                    Err(oneshot::Empty) => None,
                },
                Flavor::Stream(ref p) => match p.recv(Some(deadline)) {
                    Ok(t) => return Ok(t),
                    Err(stream::Disconnected) => return Err(Disconnected),
                    Err(stream::Upgraded(rx)) => Some(rx),
                    Err(stream::Empty) => None,
                },
                Flavor::Shared(ref p) => match p.recv(Some(deadline)) {
                    Ok(t) => return Ok(t),
                    Err(shared::Disconnected) => return Err(Disconnected),
                    Err(shared::Empty) => None,
                },
                Flavor::Sync(ref p) => match p.recv(Some(deadline)) {
                    Ok(t) => return Ok(t),
                    Err(sync::Disconnected) => return Err(Disconnected),
                    Err(sync::Empty) => None,
                },
            };

            if let Some(new_port) = port_or_empty {
                unsafe {
                    mem::swap(self.inner_mut(), new_port.inner_mut());
                }
            }

            // If we're already passed the deadline, and we're here without data, return a timeout, else try again. <br>如果我们已经超过了最后期限，并且我们没有数据，请返回超时，否则重试。<br>
            //
            if Instant::now() >= deadline {
                return Err(Timeout);
            }
        }
    }

    /// Returns an iterator that will block waiting for messages, but never [`panic!`]. <br>返回一个迭代器，该迭代器将阻止等待消息，但不会阻止 [`panic!`]。<br>
    /// It will return [`None`] when the channel has hung up. <br>通道挂起时，它将返回 [`None`]。<br>
    ///
    /// # Examples
    ///
    /// ```rust
    /// use std::sync::mpsc::channel;
    /// use std::thread;
    ///
    /// let (send, recv) = channel();
    ///
    /// thread::spawn(move || {
    ///     send.send(1).unwrap();
    ///     send.send(2).unwrap();
    ///     send.send(3).unwrap();
    /// });
    ///
    /// let mut iter = recv.iter();
    /// assert_eq!(iter.next(), Some(1));
    /// assert_eq!(iter.next(), Some(2));
    /// assert_eq!(iter.next(), Some(3));
    /// assert_eq!(iter.next(), None);
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn iter(&self) -> Iter<'_, T> {
        Iter { rx: self }
    }

    /// Returns an iterator that will attempt to yield all pending values. <br>返回一个迭代器，它将尝试产生所有挂起的值。<br>
    /// It will return `None` if there are no more pending values or if the channel has hung up. <br>如果没有其他未决值或通道已挂断，它将返回 `None`。<br>
    /// The iterator will never [`panic!`] or block the user by waiting for values. <br>迭代器永远不会 [`panic!`] 或通过等待值来阻止用户。<br>
    ///
    /// # Examples
    ///
    /// ```no_run
    /// use std::sync::mpsc::channel;
    /// use std::thread;
    /// use std::time::Duration;
    ///
    /// let (sender, receiver) = channel();
    ///
    /// // nothing is in the buffer yet <br>缓冲区中还没有任何东西<br>
    /// assert!(receiver.try_iter().next().is_none());
    ///
    /// thread::spawn(move || {
    ///     thread::sleep(Duration::from_secs(1));
    ///     sender.send(1).unwrap();
    ///     sender.send(2).unwrap();
    ///     sender.send(3).unwrap();
    /// });
    ///
    /// // nothing is in the buffer yet <br>缓冲区中还没有任何东西<br>
    /// assert!(receiver.try_iter().next().is_none());
    ///
    /// // block for two seconds <br>阻塞两秒钟<br>
    /// thread::sleep(Duration::from_secs(2));
    ///
    /// let mut iter = receiver.try_iter();
    /// assert_eq!(iter.next(), Some(1));
    /// assert_eq!(iter.next(), Some(2));
    /// assert_eq!(iter.next(), Some(3));
    /// assert_eq!(iter.next(), None);
    /// ```
    ///
    #[stable(feature = "receiver_try_iter", since = "1.15.0")]
    pub fn try_iter(&self) -> TryIter<'_, T> {
        TryIter { rx: self }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<'a, T> Iterator for Iter<'a, T> {
    type Item = T;

    fn next(&mut self) -> Option<T> {
        self.rx.recv().ok()
    }
}

#[stable(feature = "receiver_try_iter", since = "1.15.0")]
impl<'a, T> Iterator for TryIter<'a, T> {
    type Item = T;

    fn next(&mut self) -> Option<T> {
        self.rx.try_recv().ok()
    }
}

#[stable(feature = "receiver_into_iter", since = "1.1.0")]
impl<'a, T> IntoIterator for &'a Receiver<T> {
    type Item = T;
    type IntoIter = Iter<'a, T>;

    fn into_iter(self) -> Iter<'a, T> {
        self.iter()
    }
}

#[stable(feature = "receiver_into_iter", since = "1.1.0")]
impl<T> Iterator for IntoIter<T> {
    type Item = T;
    fn next(&mut self) -> Option<T> {
        self.rx.recv().ok()
    }
}

#[stable(feature = "receiver_into_iter", since = "1.1.0")]
impl<T> IntoIterator for Receiver<T> {
    type Item = T;
    type IntoIter = IntoIter<T>;

    fn into_iter(self) -> IntoIter<T> {
        IntoIter { rx: self }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Drop for Receiver<T> {
    fn drop(&mut self) {
        match *unsafe { self.inner() } {
            Flavor::Oneshot(ref p) => p.drop_port(),
            Flavor::Stream(ref p) => p.drop_port(),
            Flavor::Shared(ref p) => p.drop_port(),
            Flavor::Sync(ref p) => p.drop_port(),
        }
    }
}

#[stable(feature = "mpsc_debug", since = "1.8.0")]
impl<T> fmt::Debug for Receiver<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Receiver").finish_non_exhaustive()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> fmt::Debug for SendError<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("SendError").finish_non_exhaustive()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> fmt::Display for SendError<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        "sending on a closed channel".fmt(f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Send> error::Error for SendError<T> {
    #[allow(deprecated)]
    fn description(&self) -> &str {
        "sending on a closed channel"
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> fmt::Debug for TrySendError<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match *self {
            TrySendError::Full(..) => "Full(..)".fmt(f),
            TrySendError::Disconnected(..) => "Disconnected(..)".fmt(f),
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> fmt::Display for TrySendError<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match *self {
            TrySendError::Full(..) => "sending on a full channel".fmt(f),
            TrySendError::Disconnected(..) => "sending on a closed channel".fmt(f),
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T: Send> error::Error for TrySendError<T> {
    #[allow(deprecated)]
    fn description(&self) -> &str {
        match *self {
            TrySendError::Full(..) => "sending on a full channel",
            TrySendError::Disconnected(..) => "sending on a closed channel",
        }
    }
}

#[stable(feature = "mpsc_error_conversions", since = "1.24.0")]
impl<T> From<SendError<T>> for TrySendError<T> {
    /// Converts a `SendError<T>` into a `TrySendError<T>`. <br>将 `SendError<T>` 转换为 `TrySendError<T>`。<br>
    ///
    /// This conversion always returns a `TrySendError::Disconnected` containing the data in the `SendError<T>`. <br>这种转换总是返回一个包含 `SendError<T>` 中数据的 `TrySendError::Disconnected`。<br>
    ///
    /// No data is allocated on the heap. <br>没有在堆上分配数据。<br>
    fn from(err: SendError<T>) -> TrySendError<T> {
        match err {
            SendError(t) => TrySendError::Disconnected(t),
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Display for RecvError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        "receiving on a closed channel".fmt(f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl error::Error for RecvError {
    #[allow(deprecated)]
    fn description(&self) -> &str {
        "receiving on a closed channel"
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Display for TryRecvError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match *self {
            TryRecvError::Empty => "receiving on an empty channel".fmt(f),
            TryRecvError::Disconnected => "receiving on a closed channel".fmt(f),
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl error::Error for TryRecvError {
    #[allow(deprecated)]
    fn description(&self) -> &str {
        match *self {
            TryRecvError::Empty => "receiving on an empty channel",
            TryRecvError::Disconnected => "receiving on a closed channel",
        }
    }
}

#[stable(feature = "mpsc_error_conversions", since = "1.24.0")]
impl From<RecvError> for TryRecvError {
    /// Converts a `RecvError` into a `TryRecvError`. <br>将 `RecvError` 转换为 `TryRecvError`。<br>
    ///
    /// This conversion always returns `TryRecvError::Disconnected`. <br>此转换始终返回 `TryRecvError::Disconnected`。<br>
    ///
    /// No data is allocated on the heap. <br>没有在堆上分配数据。<br>
    fn from(err: RecvError) -> TryRecvError {
        match err {
            RecvError => TryRecvError::Disconnected,
        }
    }
}

#[stable(feature = "mpsc_recv_timeout_error", since = "1.15.0")]
impl fmt::Display for RecvTimeoutError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match *self {
            RecvTimeoutError::Timeout => "timed out waiting on channel".fmt(f),
            RecvTimeoutError::Disconnected => "channel is empty and sending half is closed".fmt(f),
        }
    }
}

#[stable(feature = "mpsc_recv_timeout_error", since = "1.15.0")]
impl error::Error for RecvTimeoutError {
    #[allow(deprecated)]
    fn description(&self) -> &str {
        match *self {
            RecvTimeoutError::Timeout => "timed out waiting on channel",
            RecvTimeoutError::Disconnected => "channel is empty and sending half is closed",
        }
    }
}

#[stable(feature = "mpsc_error_conversions", since = "1.24.0")]
impl From<RecvError> for RecvTimeoutError {
    /// Converts a `RecvError` into a `RecvTimeoutError`. <br>将 `RecvError` 转换为 `RecvTimeoutError`。<br>
    ///
    /// This conversion always returns `RecvTimeoutError::Disconnected`. <br>此转换始终返回 `RecvTimeoutError::Disconnected`。<br>
    ///
    /// No data is allocated on the heap. <br>没有在堆上分配数据。<br>
    fn from(err: RecvError) -> RecvTimeoutError {
        match err {
            RecvError => RecvTimeoutError::Disconnected,
        }
    }
}
