/// Oneshot channels/ports
///
/// This is the initial flavor of channels/ports used for comm module. <br>这是用于通讯模块的 channels/ports 的初始风格。<br> This is an optimization for the one-use case of a channel. <br>这是针对通道的一次使用情况的优化。<br> The major optimization of this type is to have one and exactly one allocation when the chan/port pair is created. <br>这种类型的主要优化是在创建 chan/port 对时仅分配一个且仅分配一个。<br>
///
/// Another possible optimization would be to not use an Arc box because in theory we know when the shared packet can be deallocated (no real need for the atomic reference counting), but I was having trouble how to destroy the data early in a drop of a Port. <br>另一个可能的优化是不使用 Arc box，因为从理论上讲我们知道何时可以重新分配共享数据包 (不需要真正的原子引用计数)，但我遇到了如何在一个端口丢弃的早期销毁数据的麻烦。<br>
///
///
/// # Implementation
///
/// Oneshots are implemented around one atomic usize variable. <br>Oneshots 是围绕一个原子的 usize 变量实现的。<br> This variable indicates both the state of the port/chan but also contains any threads blocked on the port. <br>该变量不仅指示 port/chan 的状态，还包含端口上阻塞的所有线程。<br> All atomic operations happen on this one word. <br>所有的原子运算都发生在这个词上。<br>
///
/// In order to upgrade a oneshot channel, an upgrade is considered a disconnect on behalf of the channel side of things (it can be mentally thought of as consuming the port). <br>为了升级 oneshot 通道，升级被视为代表事物在通道方面的断开连接 (在心理上可以认为是在占用端口)。<br> This upgrade is then also stored in the shared packet. <br>然后，此升级也存储在共享数据包中。<br>
/// The one caveat to consider is that when a port sees a disconnected channel it must check for data because there is no "data plus upgrade" state. <br>需要考虑的一个警告是，当端口看到断开的通道时，它必须检查数据，因为没有 "数据加升级" 状态。<br>
///
///
///
///
///
///
///
///
///
///
pub use self::Failure::*;
use self::MyUpgrade::*;
pub use self::UpgradeResult::*;

use crate::cell::UnsafeCell;
use crate::ptr;
use crate::sync::atomic::{AtomicPtr, Ordering};
use crate::sync::mpsc::blocking::{self, SignalToken};
use crate::sync::mpsc::Receiver;
use crate::time::Instant;

// Various states you can find a port in. <br>您可以在各种状态下找到端口。<br>
const EMPTY: *mut u8 = ptr::invalid_mut::<u8>(0); // initial state: no data, no blocked receiver <br>初始状态：无数据，无阻塞接收者<br>
const DATA: *mut u8 = ptr::invalid_mut::<u8>(1); // data ready for receiver to take <br>准备好供接收者接收的数据<br>
const DISCONNECTED: *mut u8 = ptr::invalid_mut::<u8>(2); // channel is disconnected OR upgraded <br>通道已断开连接或已升级<br>
// Any other value represents a pointer to a SignalToken value. <br>任何其他值都表示一个指向 SignalToken 值的指针。<br>
// The protocol ensures that when the state moves *to* a pointer, ownership of the token is given to the packet, and when the state moves *from* a pointer, ownership of the token is transferred to whoever changed the state. <br>该协议确保当状态从 *移到指针时，将 token 的所有权提供给数据包，并且在状态从* 指针移向 * 时，token 的所有权将转移给更改状态的任何人。<br>
//
//
//

pub struct Packet<T> {
    // Internal state of the chan/port pair (stores the blocked thread as well) <br>chan/port 对的内部状态 (也存储阻塞的线程)<br>
    state: AtomicPtr<u8>,
    // One-shot data slot location <br>一键式数据插槽位置<br>
    data: UnsafeCell<Option<T>>,
    // when used for the second time, a oneshot channel must be upgraded, and this contains the slot for the upgrade <br>第二次使用时，必须升级 oneshot 通道，其中包含用于升级的插槽<br>
    //
    upgrade: UnsafeCell<MyUpgrade<T>>,
}

pub enum Failure<T> {
    Empty,
    Disconnected,
    Upgraded(Receiver<T>),
}

pub enum UpgradeResult {
    UpSuccess,
    UpDisconnected,
    UpWoke(SignalToken),
}

enum MyUpgrade<T> {
    NothingSent,
    SendUsed,
    GoUp(Receiver<T>),
}

impl<T> Packet<T> {
    pub fn new() -> Packet<T> {
        Packet {
            data: UnsafeCell::new(None),
            upgrade: UnsafeCell::new(NothingSent),
            state: AtomicPtr::new(EMPTY),
        }
    }

    pub fn send(&self, t: T) -> Result<(), T> {
        unsafe {
            // Sanity check <br>完整性检查<br>
            match *self.upgrade.get() {
                NothingSent => {}
                _ => panic!("sending on a oneshot that's already sent on "),
            }
            assert!((*self.data.get()).is_none());
            ptr::write(self.data.get(), Some(t));
            ptr::write(self.upgrade.get(), SendUsed);

            match self.state.swap(DATA, Ordering::SeqCst) {
                // Sent the data, no one was waiting <br>发送数据，没有人在等待<br>
                EMPTY => Ok(()),

                // Couldn't send the data, the port hung up first. <br>无法发送数据，端口首先挂断。<br>
                // Return the data back up the stack. <br>将数据返回栈。<br>
                DISCONNECTED => {
                    self.state.swap(DISCONNECTED, Ordering::SeqCst);
                    ptr::write(self.upgrade.get(), NothingSent);
                    Err((&mut *self.data.get()).take().unwrap())
                }

                // Not possible, these are one-use channels <br>不可能，这些是一次性通道<br>
                DATA => unreachable!(),

                // There is a thread waiting on the other end. <br>另一端有一个线程在等待。<br>
                // We leave the 'DATA' state inside so it'll pick it up on the other end. <br>我们将 'DATA' 状态留在内部，以便在另一端将其拾取。<br>
                ptr => {
                    SignalToken::from_raw(ptr).signal();
                    Ok(())
                }
            }
        }
    }

    // Just tests whether this channel has been sent on or not, this is only safe to use from the sender. <br>只需测试此通道是否已发送，就可以从发送方安全地使用它。<br>
    //
    pub fn sent(&self) -> bool {
        unsafe { !matches!(*self.upgrade.get(), NothingSent) }
    }

    pub fn recv(&self, deadline: Option<Instant>) -> Result<T, Failure<T>> {
        // Attempt to not block the thread (it's a little expensive). <br>尝试不阻塞线程 (这有点贵)。<br>
        // If it looks like we're not empty, then immediately go through to `try_recv`. <br>如果看起来我们不是空的，那么请立即进行 `try_recv` 的操作。<br>
        if self.state.load(Ordering::SeqCst) == EMPTY {
            let (wait_token, signal_token) = blocking::tokens();
            let ptr = unsafe { signal_token.to_raw() };

            // race with senders to enter the blocking state <br>与发送者竞争进入阻止状态<br>
            if self.state.compare_exchange(EMPTY, ptr, Ordering::SeqCst, Ordering::SeqCst).is_ok() {
                if let Some(deadline) = deadline {
                    let timed_out = !wait_token.wait_max_until(deadline);
                    // Try to reset the state <br>尝试重置状态<br>
                    if timed_out {
                        self.abort_selection().map_err(Upgraded)?;
                    }
                } else {
                    wait_token.wait();
                    debug_assert!(self.state.load(Ordering::SeqCst) != EMPTY);
                }
            } else {
                // drop the signal token, since we never blocked <br>丢弃信号 token，因为我们从未阻止<br>
                drop(unsafe { SignalToken::from_raw(ptr) });
            }
        }

        self.try_recv()
    }

    pub fn try_recv(&self) -> Result<T, Failure<T>> {
        unsafe {
            match self.state.load(Ordering::SeqCst) {
                EMPTY => Err(Empty),

                // We saw some data on the channel, but the channel can be used again to send us an upgrade. <br>我们在通道上看到了一些数据，但是可以再次使用通道来向我们发送升级。<br>
                // As a result, we need to re-insert into the channel that there's no data available (otherwise we'll just see DATA next time). <br>结果，我们需要重新插入没有可用数据的通道 (否则，下一次我们将只看到 DATA)。<br>
                //
                // This is done as a cmpxchg because if the state changes under our feet we'd rather just see that state change. <br>这是作为 cmpxchg 完成的，因为如果状态在我们的脚下改变，我们宁愿只看到状态改变。<br>
                //
                //
                DATA => {
                    let _ = self.state.compare_exchange(
                        DATA,
                        EMPTY,
                        Ordering::SeqCst,
                        Ordering::SeqCst,
                    );
                    match (&mut *self.data.get()).take() {
                        Some(data) => Ok(data),
                        None => unreachable!(),
                    }
                }

                // There's no guarantee that we receive before an upgrade happens, and an upgrade flags the channel as disconnected, so when we see this we first need to check if there's data available and *then* we go through and process the upgrade. <br>无法保证在升级发生之前我们会收到，并且升级会将通道标记为断开连接，因此，当我们看到此消息时，我们首先需要检查是否有可用数据，然后 *然后* 进行升级。<br>
                //
                //
                //
                DISCONNECTED => match (&mut *self.data.get()).take() {
                    Some(data) => Ok(data),
                    None => match ptr::replace(self.upgrade.get(), SendUsed) {
                        SendUsed | NothingSent => Err(Disconnected),
                        GoUp(upgrade) => Err(Upgraded(upgrade)),
                    },
                },

                // We are the sole receiver; <br>我们是唯一的接收者；<br> there cannot be a blocking receiver already. <br>已经没有阻塞的接收者了。<br>
                //
                _ => unreachable!(),
            }
        }
    }

    // Returns whether the upgrade was completed. <br>返回升级是否完成。<br>
    // If the upgrade wasn't completed, then the port couldn't get sent to the other half (it will never receive it). <br>如果升级未完成，则无法将端口发送到另一半 (它将永远不会收到)。<br>
    //
    pub fn upgrade(&self, up: Receiver<T>) -> UpgradeResult {
        unsafe {
            let prev = match *self.upgrade.get() {
                NothingSent => NothingSent,
                SendUsed => SendUsed,
                _ => panic!("upgrading again"),
            };
            ptr::write(self.upgrade.get(), GoUp(up));

            match self.state.swap(DISCONNECTED, Ordering::SeqCst) {
                // If the channel is empty or has data on it, then we're good to go. <br>如果通道为空或上面有数据，那么我们很好。<br>
                // Senders will check the data before the upgrade (in case we plastered over the DATA state). <br>发送者将在升级之前检查数据 (以防我们粘贴 DATA 状态)。<br>
                //
                DATA | EMPTY => UpSuccess,

                // If the other end is already disconnected, then we failed the upgrade. <br>如果另一端已经断开连接，则升级失败。<br>
                // Be sure to trash the port we were given. <br>请确保将我们给的端口丢掉。<br>
                DISCONNECTED => {
                    ptr::replace(self.upgrade.get(), prev);
                    UpDisconnected
                }

                // If someone's waiting, we gotta wake them up <br>如果有人在等，我们得叫醒他们<br>
                ptr => UpWoke(SignalToken::from_raw(ptr)),
            }
        }
    }

    pub fn drop_chan(&self) {
        match self.state.swap(DISCONNECTED, Ordering::SeqCst) {
            DATA | DISCONNECTED | EMPTY => {}

            // If someone's waiting, we gotta wake them up <br>如果有人在等，我们得叫醒他们<br>
            ptr => unsafe {
                SignalToken::from_raw(ptr).signal();
            },
        }
    }

    pub fn drop_port(&self) {
        match self.state.swap(DISCONNECTED, Ordering::SeqCst) {
            // An empty channel has nothing to do, and a remotely disconnected channel also has nothing to do b/c we're about to run the drop glue <br>空通道无关紧要，而远程断开连接的通道也无关紧要 b/c 我们将要运行 drop glue<br>
            //
            //
            DISCONNECTED | EMPTY => {}

            // There's data on the channel, so make sure we destroy it promptly. <br>通道上有数据，因此请确保我们立即销毁它。<br>
            // This is why not using an arc is a little difficult (need the box to stay valid while we take the data). <br>这就是为什么不使用 Arc 会有些困难的原因 (需要 box 才能在获取数据时保持有效)。<br>
            //
            DATA => unsafe {
                (&mut *self.data.get()).take().unwrap();
            },

            // We're the only ones that can block on this port <br>我们是唯一可以阻止此端口的人<br>
            _ => unreachable!(),
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    // select implementation <br>选择实现<br>
    ////////////////////////////////////////////////////////////////////////////

    // Remove a previous selecting thread from this port. <br>从该端口中删除先前的选择线程。<br>
    // This ensures that the blocked thread will no longer be visible to any other threads. <br>这确保了被阻塞的线程将不再对任何其他线程可见。<br>
    //
    // The return value indicates whether there's data on this port. <br>返回值指示此端口上是否有数据。<br>
    pub fn abort_selection(&self) -> Result<bool, Receiver<T>> {
        let state = match self.state.load(Ordering::SeqCst) {
            // Each of these states means that no further activity will happen with regard to abortion selection <br>这些状态中的每一个都意味着在堕胎选择方面不会发生进一步的活动<br>
            //
            s @ (EMPTY | DATA | DISCONNECTED) => s,

            // If we've got a blocked thread, then use an atomic to gain ownership of it (may fail) <br>如果我们有一个阻塞的线程，则使用原子来获得它的所有权 (可能会失败)<br>
            //
            ptr => self
                .state
                .compare_exchange(ptr, EMPTY, Ordering::SeqCst, Ordering::SeqCst)
                .unwrap_or_else(|x| x),
        };

        // Now that we've got ownership of our state, figure out what to do about it. <br>既然我们已经拥有了国家的所有权，那么想出办法去做。<br>
        //
        match state {
            EMPTY => unreachable!(),
            // our thread used for select was stolen <br>用于选择的线程被盗<br>
            DATA => Ok(true),

            // If the other end has hung up, then we have complete ownership of the port. <br>如果另一端挂断了，那么我们就拥有该端口的完全所有权。<br> First, check if there was data waiting for us. <br>首先，检查是否有数据在等待我们。<br>
            // This is possible if the other end sent something and then hung up. <br>如果另一端发送了一些内容然后挂断，则可能发生这种情况。<br>
            //
            // We then need to check to see if there was an upgrade requested, and if so, the upgraded port needs to have its selection aborted. <br>然后，我们需要检查是否请求了升级，如果是，则升级后的端口需要中止选择。<br>
            //
            //
            DISCONNECTED => unsafe {
                if (*self.data.get()).is_some() {
                    Ok(true)
                } else {
                    match ptr::replace(self.upgrade.get(), SendUsed) {
                        GoUp(port) => Err(port),
                        _ => Ok(true),
                    }
                }
            },

            // We woke ourselves up from select. <br>我们从精选中唤醒了自己。<br>
            ptr => unsafe {
                drop(SignalToken::from_raw(ptr));
                Ok(false)
            },
        }
    }
}

impl<T> Drop for Packet<T> {
    fn drop(&mut self) {
        assert_eq!(self.state.load(Ordering::SeqCst), DISCONNECTED);
    }
}
