/// Shared channels. <br>共享通道。<br>
///
/// This is the flavor of channels which are not necessarily optimized for any particular use case, but are the most general in how they are used. <br>这是通道的风格，不一定针对任何特定用例进行优化，但在使用方式上是最通用的。<br>
/// Shared channels are cloneable allowing for multiple senders. <br>共享通道是可克隆的，允许多个发送者。<br>
///
/// High level implementation details can be found in the comment of the parent module. <br>可以在父模块的注释中找到高级实现细节。<br>
/// You'll also note that the implementation of the shared and stream channels are quite similar, and this is no coincidence! <br>您还将注意到，共享通道和流通道的实现非常相似，这绝非偶然！<br>
///
///
pub use self::Failure::*;
use self::StartResult::*;

use core::cmp;
use core::intrinsics::abort;

use crate::cell::UnsafeCell;
use crate::ptr;
use crate::sync::atomic::{AtomicBool, AtomicIsize, AtomicPtr, AtomicUsize, Ordering};
use crate::sync::mpsc::blocking::{self, SignalToken};
use crate::sync::mpsc::mpsc_queue as mpsc;
use crate::sync::{Mutex, MutexGuard};
use crate::thread;
use crate::time::Instant;

const DISCONNECTED: isize = isize::MIN;
const FUDGE: isize = 1024;
const MAX_REFCOUNT: usize = (isize::MAX) as usize;
#[cfg(test)]
const MAX_STEALS: isize = 5;
#[cfg(not(test))]
const MAX_STEALS: isize = 1 << 20;
const EMPTY: *mut u8 = ptr::null_mut(); // initial state: no data, no blocked receiver <br>初始状态：无数据，无阻塞接收者<br>

pub struct Packet<T> {
    queue: mpsc::Queue<T>,
    cnt: AtomicIsize,          // How many items are on this channel <br>该通道上有多少项<br>
    steals: UnsafeCell<isize>, // How many times has a port received without blocking? <br>一个端口在没有阻塞的情况下接收了多少次？<br>
    to_wake: AtomicPtr<u8>,    // SignalToken for wake up <br>SignalToken 唤醒<br>

    // The number of channels which are currently using this packet. <br>当前正在使用此数据包的通道数。<br>
    channels: AtomicUsize,

    // See the discussion in Port::drop and the channel send methods for what these are used for <br>有关这些功能的用途，请参见 Port::drop 中的讨论和通道发送方法。<br>
    //
    port_dropped: AtomicBool,
    sender_drain: AtomicIsize,

    // this lock protects various portions of this implementation during <br>此锁在执行过程中保护此实现的各个部分<br>
    // select()
    select_lock: Mutex<()>,
}

pub enum Failure {
    Empty,
    Disconnected,
}

#[derive(PartialEq, Eq)]
enum StartResult {
    Installed,
    Abort,
}

impl<T> Packet<T> {
    // Creation of a packet *must* be followed by a call to postinit_lock and later by inherit_blocker <br>数据包的创建 (必须) 之后必须先调用 postinit_lock，然后再调用 Inherited_blocker<br>
    //
    pub fn new() -> Packet<T> {
        Packet {
            queue: mpsc::Queue::new(),
            cnt: AtomicIsize::new(0),
            steals: UnsafeCell::new(0),
            to_wake: AtomicPtr::new(EMPTY),
            channels: AtomicUsize::new(2),
            port_dropped: AtomicBool::new(false),
            sender_drain: AtomicIsize::new(0),
            select_lock: Mutex::new(()),
        }
    }

    // This function should be used after newly created Packet was wrapped with an Arc In other case mutex data will be duplicated while cloning and that could cause problems on platforms where it is represented by opaque data structure <br>这个函数应该在新创建的 Packet 用 Arc 包装后使用，否则在克隆时互斥锁数据将被复制，这可能会导致在它由不透明数据结构表示的平台上出现问题<br>
    //
    //
    //
    //
    pub fn postinit_lock(&self) -> MutexGuard<'_, ()> {
        self.select_lock.lock().unwrap()
    }

    // This function is used at the creation of a shared packet to inherit a previously blocked thread. <br>在创建共享数据包时，可以使用此函数来继承以前阻塞的线程。<br>
    // This is done to prevent spurious wakeups of threads in select(). <br>这样做是为了防止 select() 中的线程被虚假唤醒。<br>
    //
    // This can only be called at channel-creation time <br>这只能在通道创建时调用<br>
    //
    pub fn inherit_blocker(&self, token: Option<SignalToken>, guard: MutexGuard<'_, ()>) {
        if let Some(token) = token {
            assert_eq!(self.cnt.load(Ordering::SeqCst), 0);
            assert_eq!(self.to_wake.load(Ordering::SeqCst), EMPTY);
            self.to_wake.store(unsafe { token.to_raw() }, Ordering::SeqCst);
            self.cnt.store(-1, Ordering::SeqCst);

            // This store is a little sketchy. <br>这家商店有点简陋。<br> What's happening here is that we're transferring a blocker from a oneshot or stream channel to this shared channel. <br>这里发生的是我们正在将阻止程序从 oneshot 或流通道传输到此共享通道。<br> In doing so, we never spuriously wake them up and rather only wake them up at the appropriate time. <br>这样做时，我们绝不会虚假地唤醒它们，而只是在适当的时间唤醒它们。<br> This implementation of shared channels assumes that any blocking recv() will undo the increment of steals performed in try_recv() once the recv is complete. <br>共享通道的此实现假定，一旦接收完成，任何阻塞的 recv() 都将撤消 try_recv() 中执行的窃取增量。<br>
            //
            // This thread that we're inheriting, however, is not in the middle of recv. Hence, the first time we wake them up, they're going to wake up from their old port, move on to the upgraded port, and then call the block recv() function. <br>但是，我们正在继承的该线程不在 recv 的中间。因此，我们第一次唤醒它们时，它们将从原来的端口中唤醒，移至升级的端口，然后调用块 recv() 函数。<br>
            //
            // When calling this function, they'll find there's data immediately available, counting it as a steal. <br>调用此函数时，他们会发现立即有可用数据，将其算作窃取。<br> This in fact wasn't a steal because we appropriately blocked them waiting for data. <br>实际上，这并不是偷窃，因为我们适当地阻止了他们等待数据。<br>
            //
            // To offset this bad increment, we initially set the steal count to <br>为了弥补这一不良增长，我们最初将窃取计数设置为<br>
            // -1. You'll find some special code in abort_selection() as well to ensure that this -1 steal count doesn't escape too far. <br>-1。您还会在 abort_selection() 中找到一些特殊代码，以确保此 -1 窃取次数不会逃逸太多。<br>
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            unsafe {
                *self.steals.get() = -1;
            }
        }

        // When the shared packet is constructed, we grabbed this lock. <br>构建共享数据包后，我们抓住了这个锁。<br>
        // The purpose of this lock is to ensure that abort_selection() doesn't interfere with this method. <br>此锁定的目的是确保 abort_selection() 不会干扰此方法。<br>
        // After we unlock this lock, we're signifying that we're done modifying self.cnt and self.to_wake and the port is ready for the world to continue using it. <br>解锁此锁后，表示完成 self.cnt 和 self.to_wake 的修改，并且该端口已准备就绪，世界各地均可继续使用它。<br>
        //
        //
        drop(guard);
    }

    pub fn send(&self, t: T) -> Result<(), T> {
        // See Port::drop for what's going on <br>请参见 Port::drop，了解发生了什么情况<br>
        if self.port_dropped.load(Ordering::SeqCst) {
            return Err(t);
        }

        // Note that the multiple sender case is a little trickier semantically than the single sender case. <br>请注意，多发送者案例在语义上要比单发送者案例稍微复杂一些。<br> The logic for incrementing is "add and if disconnected store disconnected". <br>递增的逻辑为 "添加，如果断开连接，则存储断开连接"。<br>
        // This could end up leading some senders to believe that there wasn't a disconnect if in fact there was a disconnect. <br>这可能最终导致某些发送者认为，即使实际上存在断开连接，也不会断开连接。<br>
        // This means that while one thread is attempting to re-store the disconnected states, other threads could walk through merrily incrementing this very-negative disconnected count. <br>这意味着，当一个线程试图恢复断开连接状态时，其他线程可能会通过愉快地增加此非常负的断开连接计数来进行遍历。<br>
        //
        // To prevent senders from spuriously attempting to send when the channels is actually disconnected, the count has a ranged check here. <br>为了防止发送方在实际断开通道时虚假地尝试发送，此处对计数进行了范围检查。<br>
        //
        // This is also done for another reason. <br>这样做也是出于另一个原因。<br> Remember that the return value of this function is: <br>请记住，此函数的返回值为：<br>
        //
        //  `true` == the data *may* be received, this essentially has no meaning `false` == the data will *never* be received, this has a lot of meaning <br>`true` == 数据*可能*被接收，这基本上没有意义 `false` == 数据将永远不会被接收，这有很多意义<br>
        //
        // In the SPSC case, we have a check of 'queue.is_empty()' to see whether the data was actually received, but this same condition means nothing in a multi-producer context. <br>在 SPSC 情况下，我们检查 'queue.is_empty()' 以查看是否实际接收到数据，但是在多生产者环境中，这种相同的条件没有任何意义。<br>
        // As a result, this preflight check serves as the definitive "this will never be received". <br>结果，此预检检查将用作确定的 "永远不会收到"。<br>
        // Once we get beyond this check, we have permanently entered the realm of "this may be received" <br>一旦超出此检查范围，我们将永久进入 "这可能会收到" 的领域<br>
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        if self.cnt.load(Ordering::SeqCst) < DISCONNECTED + FUDGE {
            return Err(t);
        }

        self.queue.push(t);
        match self.cnt.fetch_add(1, Ordering::SeqCst) {
            -1 => {
                self.take_to_wake().signal();
            }

            // In this case, we have possibly failed to send our data, and we need to consider re-popping the data in order to fully destroy it. <br>在这种情况下，我们可能无法发送数据，我们需要考虑重新弹出数据以完全销毁它。<br>
            // We must arbitrate among the multiple senders, however, because the queues that we're using are single-consumer queues. <br>但是，我们必须在多个发送者之间进行仲裁，因为我们使用的队列是单消费者队列。<br>
            // In order to do this, all exiting pushers will use an atomic count in order to count those flowing through. <br>为了做到这一点，所有退出的推动器都将使用原子计数，以便对流经的推动器进行计数。<br>
            // Pushers who see 0 are required to drain as much as possible, and then can only exit when they are the only pusher (otherwise they must try again). <br>看到 0 的推动器必须尽可能多地进入 drain，然后只有在它们是唯一推动器时才能退出 (否则必须再次尝试)。<br>
            //
            //
            //
            //
            //
            n if n < DISCONNECTED + FUDGE => {
                // see the comment in 'try' for a shared channel for why this window of "not disconnected" is ok. <br>请参见 'try' 中有关共享通道的文档，以了解 "未断开连接" 窗口正常的原因。<br>
                //
                self.cnt.store(DISCONNECTED, Ordering::SeqCst);

                if self.sender_drain.fetch_add(1, Ordering::SeqCst) == 0 {
                    loop {
                        // drain the queue, for info on the thread yield see the discussion in try_recv <br>drain 队列，有关线程产量的信息，请参见 try_recv 中的讨论<br>
                        //
                        loop {
                            match self.queue.pop() {
                                mpsc::Data(..) => {}
                                mpsc::Empty => break,
                                mpsc::Inconsistent => thread::yield_now(),
                            }
                        }
                        // maybe we're done, if we're not the last ones here, then we need to go try again. <br>也许我们已经完成了，如果不是最后一个，那么我们需要再试一次。<br>
                        //
                        if self.sender_drain.fetch_sub(1, Ordering::SeqCst) == 1 {
                            break;
                        }
                    }

                    // At this point, there may still be data on the queue, but only if the count hasn't been incremented and some other sender hasn't finished pushing data just yet. <br>此时，队列上可能仍然有数据，但前提是计数没有增加，并且其他一些发送方尚未完成推送数据。<br>
                    // That sender in question will drain its own data. <br>该发送者将 drain 自己的数据。<br>
                    //
                    //
                }
            }

            // Can't make any assumptions about this case like in the SPSC case. <br>不能像 SPSC 案例那样对此案例做出任何假设。<br>
            _ => {}
        }

        Ok(())
    }

    pub fn recv(&self, deadline: Option<Instant>) -> Result<T, Failure> {
        // This code is essentially the exact same as that found in the stream case (see stream.rs) <br>该代码本质上与流情况下的代码完全相同 (请参见 stream.rs)<br>
        //
        match self.try_recv() {
            Err(Empty) => {}
            data => return data,
        }

        let (wait_token, signal_token) = blocking::tokens();
        if self.decrement(signal_token) == Installed {
            if let Some(deadline) = deadline {
                let timed_out = !wait_token.wait_max_until(deadline);
                if timed_out {
                    self.abort_selection(false);
                }
            } else {
                wait_token.wait();
            }
        }

        match self.try_recv() {
            data @ Ok(..) => unsafe {
                *self.steals.get() -= 1;
                data
            },
            data => data,
        }
    }

    // Essentially the exact same thing as the stream decrement function. <br>本质上与流递减函数完全相同。<br>
    // Returns true if blocking should proceed. <br>如果应继续执行阻塞，则返回 true。<br>
    fn decrement(&self, token: SignalToken) -> StartResult {
        unsafe {
            assert_eq!(
                self.to_wake.load(Ordering::SeqCst),
                EMPTY,
                "This is a known bug in the Rust standard library. See https://github.com/rust-lang/rust/issues/39364"
            );
            let ptr = token.to_raw();
            self.to_wake.store(ptr, Ordering::SeqCst);

            let steals = ptr::replace(self.steals.get(), 0);

            match self.cnt.fetch_sub(1 + steals, Ordering::SeqCst) {
                DISCONNECTED => {
                    self.cnt.store(DISCONNECTED, Ordering::SeqCst);
                }
                // If we factor in our steals and notice that the channel has no data, we successfully sleep <br>如果我们将盗窃因素考虑在内，并且注意到该通道没有数据，那么我们就可以成功入睡<br>
                //
                n => {
                    assert!(n >= 0);
                    if n - steals <= 0 {
                        return Installed;
                    }
                }
            }

            self.to_wake.store(EMPTY, Ordering::SeqCst);
            drop(SignalToken::from_raw(ptr));
            Abort
        }
    }

    pub fn try_recv(&self) -> Result<T, Failure> {
        let ret = match self.queue.pop() {
            mpsc::Data(t) => Some(t),
            mpsc::Empty => None,

            // This is a bit of an interesting case. <br>这是一个有趣的案例。<br> The channel is reported as having data available, but our pop() has failed due to the queue being in an inconsistent state. <br>通道被报告为有可用数据，但是由于队列处于不一致状态，我们的 pop() 失败了。<br>
            // This means that there is some pusher somewhere which has yet to complete, but we are guaranteed that a pop will eventually succeed. <br>这意味着某个地方还需要完成一些推动器，但是我们可以保证弹出操作最终会成功。<br>
            // In this case, we spin in a yield loop because the remote sender should finish their enqueue operation "very quickly". <br>在这种情况下，我们在一个 yield 循环中旋转，因为远程发送方应该 "非常快" 地完成他们的入队操作。<br>
            //
            // Avoiding this yield loop would require a different queue abstraction which provides the guarantee that after M pushes have succeeded, at least M pops will succeed. <br>避免这种良率循环将需要不同的队列抽象，这可以保证在成功完成 M 次推送之后，至少 M 次弹出会成功。<br>
            // The current queues guarantee that if there are N active pushes, you can pop N times once all N have finished. <br>当前队列可确保如果有 N 次活动推送，则所有 N 完成后，您可以弹出 N 次。<br>
            //
            //
            //
            //
            //
            //
            //
            mpsc::Inconsistent => {
                let data;
                loop {
                    thread::yield_now();
                    match self.queue.pop() {
                        mpsc::Data(t) => {
                            data = t;
                            break;
                        }
                        mpsc::Empty => panic!("inconsistent => empty"),
                        mpsc::Inconsistent => {}
                    }
                }
                Some(data)
            }
        };
        match ret {
            // See the discussion in the stream implementation for why we might decrement steals. <br>有关为什么我们可能减少盗窃的信息，请参见流实现中的讨论。<br>
            //
            Some(data) => unsafe {
                if *self.steals.get() > MAX_STEALS {
                    match self.cnt.swap(0, Ordering::SeqCst) {
                        DISCONNECTED => {
                            self.cnt.store(DISCONNECTED, Ordering::SeqCst);
                        }
                        n => {
                            let m = cmp::min(n, *self.steals.get());
                            *self.steals.get() -= m;
                            self.bump(n - m);
                        }
                    }
                    assert!(*self.steals.get() >= 0);
                }
                *self.steals.get() += 1;
                Ok(data)
            },

            // See the discussion in the stream implementation for why we try again. <br>有关为什么再次尝试的信息，请参见流实现中的讨论。<br>
            //
            None => {
                match self.cnt.load(Ordering::SeqCst) {
                    n if n != DISCONNECTED => Err(Empty),
                    _ => {
                        match self.queue.pop() {
                            mpsc::Data(t) => Ok(t),
                            mpsc::Empty => Err(Disconnected),
                            // with no senders, an inconsistency is impossible. <br>没有发送者，不一致是不可能的。<br>
                            mpsc::Inconsistent => unreachable!(),
                        }
                    }
                }
            }
        }
    }

    // Prepares this shared packet for a channel clone, essentially just bumping a refcount. <br>为通道克隆准备此共享数据包，实际上只是增加引用计数。<br>
    //
    pub fn clone_chan(&self) {
        let old_count = self.channels.fetch_add(1, Ordering::SeqCst);

        // See comments on Arc::clone() on why we do this (for `mem::forget`). <br>请参见 Arc::clone () 上的注释，了解为什么要这样做 (对于 `mem::forget`)。<br>
        if old_count > MAX_REFCOUNT {
            abort();
        }
    }

    // Decrement the reference count on a channel. <br>减少通道上的引用计数。<br>
    // This is called whenever a Chan is dropped and may end up waking up a receiver. <br>每当 Chan 丢弃并可能最终唤醒接收者时，就会调用此方法。<br>
    // It's the receiver's responsibility on the other end to figure out that we've disconnected. <br>另一端，接收者的责任是确定我们已断开连接。<br>
    pub fn drop_chan(&self) {
        match self.channels.fetch_sub(1, Ordering::SeqCst) {
            1 => {}
            n if n > 1 => return,
            n => panic!("bad number of channels left {n}"),
        }

        match self.cnt.swap(DISCONNECTED, Ordering::SeqCst) {
            -1 => {
                self.take_to_wake().signal();
            }
            DISCONNECTED => {}
            n => {
                assert!(n >= 0);
            }
        }
    }

    // See the long discussion inside of stream.rs for why the queue is drained, and why it is done in this fashion. <br>请参见 stream.rs 内部的冗长讨论，以了解为何耗尽队列以及为何以这种方式完成队列。<br>
    //
    pub fn drop_port(&self) {
        self.port_dropped.store(true, Ordering::SeqCst);
        let mut steals = unsafe { *self.steals.get() };
        while {
            match self.cnt.compare_exchange(
                steals,
                DISCONNECTED,
                Ordering::SeqCst,
                Ordering::SeqCst,
            ) {
                Ok(_) => false,
                Err(old) => old != DISCONNECTED,
            }
        } {
            // See the discussion in 'try_recv' for why we yield control of this thread. <br>请参见 'try_recv' 中的讨论，以了解为什么我们要对此线程进行控制。<br>
            //
            loop {
                match self.queue.pop() {
                    mpsc::Data(..) => {
                        steals += 1;
                    }
                    mpsc::Empty | mpsc::Inconsistent => break,
                }
            }
        }
    }

    // Consumes ownership of the 'to_wake' field. <br>占用 'to_wake' 字段的所有权。<br>
    fn take_to_wake(&self) -> SignalToken {
        let ptr = self.to_wake.load(Ordering::SeqCst);
        self.to_wake.store(EMPTY, Ordering::SeqCst);
        assert!(ptr != EMPTY);
        unsafe { SignalToken::from_raw(ptr) }
    }

    ////////////////////////////////////////////////////////////////////////////
    // select implementation <br>选择实现<br>
    ////////////////////////////////////////////////////////////////////////////

    // increment the count on the channel (used for selection) <br>增加通道上的计数 (用于选择)<br>
    fn bump(&self, amt: isize) -> isize {
        match self.cnt.fetch_add(amt, Ordering::SeqCst) {
            DISCONNECTED => {
                self.cnt.store(DISCONNECTED, Ordering::SeqCst);
                DISCONNECTED
            }
            n => n,
        }
    }

    // Cancels a previous thread waiting on this port, returning whether there's data on the port. <br>取消先前在该端口上等待的线程，并返回该端口上是否有数据。<br>
    //
    // This is similar to the stream implementation (hence fewer comments), but uses a different value for the "steals" variable. <br>这与流实现类似 (因此，注释较少)，但对 "steals" 变量使用不同的值。<br>
    //
    //
    pub fn abort_selection(&self, _was_upgrade: bool) -> bool {
        // Before we do anything else, we bounce on this lock. <br>在我们做其他事情之前，我们先弹开这个锁。<br>
        // The reason for doing this is to ensure that any upgrade-in-progress is gone and done with. <br>这样做的原因是为了确保进行中的所有升级都不会进行。<br>
        // Without this bounce, we can race with inherit_blocker about looking at and dealing with to_wake. <br>没有这种反弹，我们就可以与 Inherited_blocker 竞争有关查看和处理 to_wake 的问题。<br>
        //
        // Once we have acquired the lock, we are guaranteed that inherit_blocker is done. <br>一旦获得了锁，我们就可以保证 Inherited_blocker 已完成。<br>
        {
            let _guard = self.select_lock.lock().unwrap();
        }

        // Like the stream implementation, we want to make sure that the count on the channel goes non-negative. <br>像流实现一样，我们要确保通道上的计数变为非负数。<br>
        // We don't know how negative the stream currently is, so instead of using a steal value of 1, we load the channel count and figure out what we should do to make it positive. <br>我们不知道当前流的负值如何，因此，我们不使用窃取值 1，而是加载通道计数并弄清楚应该如何使它变为正值。<br>
        //
        //
        //
        let steals = {
            let cnt = self.cnt.load(Ordering::SeqCst);
            if cnt < 0 && cnt != DISCONNECTED { -cnt } else { 0 }
        };
        let prev = self.bump(steals + 1);

        if prev == DISCONNECTED {
            assert_eq!(self.to_wake.load(Ordering::SeqCst), EMPTY);
            true
        } else {
            let cur = prev + steals + 1;
            assert!(cur >= 0);
            if prev < 0 {
                drop(self.take_to_wake());
            } else {
                while self.to_wake.load(Ordering::SeqCst) != EMPTY {
                    thread::yield_now();
                }
            }
            unsafe {
                // if the number of steals is -1, it was the pre-emptive -1 steal count from when we inherited a blocker. <br>如果窃取次数为 -1，则它是从我们继承阻止程序起的先发制人的 -1 窃取次数。<br>
                // This is fine because we're just going to overwrite it with a real value. <br>这很好，因为我们将使用实际值覆盖它。<br>
                //
                let old = self.steals.get();
                assert!(*old == 0 || *old == -1);
                *old = steals;
                prev >= 0
            }
        }
    }
}

impl<T> Drop for Packet<T> {
    fn drop(&mut self) {
        // Note that this load is not only an assert for correctness about disconnection, but also a proper fence before the read of `to_wake`, so this assert cannot be removed with also removing the `to_wake` assert. <br>请注意，此负载不仅是断开连接正确性的断言，而且还是读取 `to_wake` 之前的适当栅栏，因此无法在删除 `to_wake` 断言的同时删除此断言。<br>
        //
        //
        //
        assert_eq!(self.cnt.load(Ordering::SeqCst), DISCONNECTED);
        assert_eq!(self.to_wake.load(Ordering::SeqCst), EMPTY);
        assert_eq!(self.channels.load(Ordering::SeqCst), 0);
    }
}
