//! A "once initialization" primitive <br>"一次初始化" 原语<br>
//!
//! This primitive is meant to be used to run one-time initialization. <br>该原语旨在用于运行一次性初始化。<br>
//! An example use case would be for initializing an FFI library. <br>一个示例用例将是初始化 FFI 库。<br>

// A "once" is a relatively simple primitive, and it's also typically provided by the OS as well (see `pthread_once` or `InitOnceExecuteOnce`). <br>"once" 是一个相对简单的原语，通常也由操作系统提供 (请参见 `pthread_once` 或 `InitOnceExecuteOnce`)。<br>
// The OS primitives, however, tend to have surprising restrictions, such as the Unix one doesn't allow an argument to be passed to the function. <br>但是，操作系统原语往往具有令人惊讶的限制，例如 Unix 不允许将参数传递给函数。<br>
//
// As a result, we end up implementing it ourselves in the standard library. <br>结果，我们最终在标准库中自己实现了它。<br>
// This also gives us the opportunity to optimize the implementation a bit which should help the fast path on call sites. <br>这也为我们提供了优化实现的机会，这将有助于在调用站点上实现快速路径。<br> Consequently, let's explain how this primitive works now! <br>因此，让我们解释一下该原语现在是如何工作的！<br>
//
// So to recap, the guarantees of a Once are that it will call the initialization closure at most once, and it will never return until the one that's running has finished running. <br>因此，回顾一下，一次的保证是它将最多调用一次初始化闭包，并且直到运行的那个完成运行后它才会返回。<br>
// This means that we need some form of blocking here while the custom callback is running at the very least. <br>这意味着，至少在自定义回调运行时，我们需要在此处进行某种形式的阻塞。<br>
// Additionally, we add on the restriction of **poisoning**. <br>此外，我们增加了中毒的限制。<br>
// Whenever an initialization closure panics, the Once enters a "poisoned" state which means that all future calls will immediately panic as well. <br>每当初始化闭包 panics 时，`Once` 都会进入 "poisoned" 状态，这意味着所有 future 调用也将立即变为 panic。<br>
//
// So to implement this, one might first reach for a `Mutex`, but those cannot be put into a `static`. <br>因此，要实现此目的，可能首先需要购买 `Mutex`，但不能将其放入 `static`。<br>
// It also gets a lot harder with poisoning to figure out when the mutex needs to be deallocated because it's not after the closure finishes, but after the first successful closure finishes. <br>通过中毒来确定何时需要释放互斥锁也会变得更加困难，因为它不是在闭包结束之后，而是在第一个成功的闭包结束之后。<br>
//
// All in all, this is instead implemented with atomics and lock-free operations! <br>总而言之，这是通过原子和无锁操作实现的！<br> Whee! Each `Once` has one word of atomic state, and this state is CAS'd on to determine what to do. <br>呜！每个 `Once` 都有一个原子状态的字，此状态已通过 CAS 确定要做什么。<br>
// There are four possible state of a `Once`: <br>`Once` 有四种可能的状态：<br>
//
// * Incomplete - no initialization has run yet, and no thread is currently using the Once. <br>未完成 - 尚未运行任何初始化，并且当前没有线程在使用 `Once`。<br>
// * Poisoned - some thread has previously attempted to initialize the Once, but it panicked, so the Once is now poisoned. <br>中毒 - 一些线程之前尝试初始化 `Once`，但是它 panic 了，所以 `Once` 现在是中毒的。<br> There are no other threads currently accessing this Once. <br>当前没有其他线程访问此一次。<br>
// * Running - some thread is currently attempting to run initialization. <br>正在运行 - 某些线程当前正在尝试运行初始化。<br> It may succeed, so all future threads need to wait for it to finish. <br>它可能会成功，因此所有 future 线程都需要等待它完成。<br>
//             Note that this state is accompanied with a payload, described below. <br>请注意，这种状态伴随着一个有效载荷，如下所述。<br>
// * Complete - initialization has completed and all future calls should finish immediately. <br>完成 - 初始化已完成，所有 future 调用应立即完成。<br>
//
// With 4 states we need 2 bits to encode this, and we use the remaining bits in the word we have allocated as a queue of threads waiting for the thread responsible for entering the RUNNING state. <br>对于 4 种状态，我们需要 2 位来对此进行编码，并且我们将已分配的字中的其余位用作线程队列，等待负责进入 RUNNING 状态的线程。<br>
//
// This queue is just a linked list of Waiter nodes which is monotonically increasing in size. <br>该队列只是一个 Waiter 节点的链表，其大小单调增加。<br>
// Each node is allocated on the stack, and whenever the running closure finishes it will consume the entire queue and notify all waiters they should try again. <br>每个节点都分配在栈上，并且每当运行的闭包完成时，它将消耗整个队列，并通知所有等待者他们应再次尝试。<br>
//
// You'll find a few more details in the implementation, but that's the gist of it! <br>您会在实现中找到更多细节，但这就是要点！<br>
//
// Atomic orderings: <br>原子顺序：<br>
// When running `Once` we deal with multiple atomics: <br>当运行 `Once` 时，我们处理多个原子：<br>
// `Once.state_and_queue` and an unknown number of `Waiter.signaled`. <br>`Once.state_and_queue` 和未知数量的 `Waiter.signaled`。<br>
// * `state_and_queue` is used (1) as a state flag, (2) for synchronizing the result of the `Once`, and (3) for synchronizing `Waiter` nodes. <br>`state_and_queue` 使用 (1) 作为状态标志，(2) 用于同步 `Once` 的结果，(3) 用于同步 `Waiter` 节点。<br>
//     - At the end of the `call_inner` function we have to make sure the result of the `Once` is acquired. <br>在 `call_inner` 函数的末尾，我们必须确保已获取 `Once` 的结果。<br>
//     So every load which can be the only one to load COMPLETED must have at least Acquire ordering, which means all three of them. <br>因此，可能是唯一要加载完成的负载，每个负载都必须至少具有获取顺序，这意味着它们全部都是三个。<br>
//     - `WaiterQueue::Drop` is the only place that may store COMPLETED, and must do so with Release ordering to make the result available. <br>`WaiterQueue::Drop` 是唯一可以存储 COMPLETED 的地方，并且必须使用 Release 排序才能使结果可用。<br>
//     - `wait` inserts `Waiter` nodes as a pointer in `state_and_queue`, and needs to make the nodes available with Release ordering. <br>`wait` 将 `Waiter` 节点作为 `state_and_queue` 中的指针插入，并且需要使用 Release 排序使节点可用。<br>
//     The load in its `compare_exchange` can be Relaxed because it only has to compare the atomic, not to read other data. <br>`compare_exchange` 中的负载可以放宽，因为它只需要比较原子，而不必读取其他数据。<br>
//     - `WaiterQueue::Drop` must see the `Waiter` nodes, so it must load `state_and_queue` with Acquire ordering. <br>`WaiterQueue::Drop` 必须看到 `Waiter` 节点，因此它必须使用 Acquire ordering 加载 `state_and_queue`。<br>
//     - There is just one store where `state_and_queue` is used only as a state flag, without having to synchronize data: switching the state from INCOMPLETE to RUNNING in `call_inner`. <br>只有一个存储区，其中 `state_and_queue` 仅用作状态标志，而不必同步数据：在 `call_inner` 中将状态从 INCOMPLETE 切换为 RUNNING。<br>
// This store can be Relaxed, but the read has to be Acquire because of the requirements mentioned above. <br>这个 store 可以是 Relaxed，但是 read 必须是 Acquire，因为上面提到的要求。<br>
// * `Waiter.signaled` is both used as a flag, and to protect a field with interior mutability in `Waiter`. <br>`Waiter.signaled` 既用作标志，又用于保护 `Waiter` 中具有内部可变性的字段。<br>
// `Waiter.thread` is changed in `WaiterQueue::Drop` which then sets `signaled` with Release ordering. <br>`Waiter.thread` 在 `WaiterQueue::Drop` 中更改，然后将 `signaled` 设置为发布顺序。<br>
//   After `wait` loads `signaled` with Acquire and sees it is true, it needs to see the changes to drop the `Waiter` struct correctly. <br>`wait` 用 Acquire 加载 `signaled` 后，如果发现它是正确的，则需要查看更改以正确放置 `Waiter` 结构体。<br>
// * There is one place where the two atomics `Once.state_and_queue` and `Waiter.signaled` come together, and might be reordered by the compiler or processor. <br>在一个地方，两个原子 `Once.state_and_queue` 和 `Waiter.signaled` 在一起，并且可能由编译器或处理器重新排序。<br>
// Because both use Acquire ordering such a reordering is not allowed, so no need for SeqCst. <br>因为两者都使用 Acquire 排序，所以不允许这样的重新排序，因此不需要 SeqCst。<br>
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//

#[cfg(all(test, not(target_os = "emscripten")))]
mod tests;

use crate::cell::Cell;
use crate::fmt;
use crate::marker;
use crate::panic::{RefUnwindSafe, UnwindSafe};
use crate::ptr;
use crate::sync::atomic::{AtomicBool, AtomicPtr, Ordering};
use crate::thread::{self, Thread};

type Masked = ();

/// A synchronization primitive which can be used to run a one-time global initialization. <br>同步原语，可用于运行一次性初始化。<br>
/// Useful for one-time initialization for FFI or related functionality. <br>对于 FFI 或相关功能的一次性初始化很有用。<br>
/// This type can only be constructed with [`Once::new()`]. <br>该类型只能用 [`Once::new()`] 构造。<br>
///
/// # Examples
///
/// ```
/// use std::sync::Once;
///
/// static START: Once = Once::new();
///
/// START.call_once(|| {
///     // run initialization here <br>在这里运行初始化<br>
/// });
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
pub struct Once {
    // `state_and_queue` is actually a pointer to a `Waiter` with extra state bits, so we add the `PhantomData` appropriately. <br>`state_and_queue` 实际上是一个指向带有额外状态位的 `Waiter` 的指针，因此我们适当地添加了 `PhantomData`。<br>
    //
    state_and_queue: AtomicPtr<Masked>,
    _marker: marker::PhantomData<*const Waiter>,
}

// The `PhantomData` of a raw pointer removes these two auto traits, but we enforce both below in the implementation so this should be safe to add. <br>裸指针的 `PhantomData` 删除了这两个自动 traits，但是我们在实现中强制执行以下两项，因此添加起来应该很安全。<br>
//
#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl Sync for Once {}
#[stable(feature = "rust1", since = "1.0.0")]
unsafe impl Send for Once {}

#[stable(feature = "sync_once_unwind_safe", since = "1.59.0")]
impl UnwindSafe for Once {}

#[stable(feature = "sync_once_unwind_safe", since = "1.59.0")]
impl RefUnwindSafe for Once {}

/// State yielded to [`Once::call_once_force()`]’s closure parameter. <br>状态产生于 [`Once::call_once_force () `] 的闭包参数。<br>
/// The state can be used to query the poison status of the [`Once`]. <br>该状态可用于查询 [`Once`] 的中毒状态。<br>
#[stable(feature = "once_poison", since = "1.51.0")]
#[derive(Debug)]
pub struct OnceState {
    poisoned: bool,
    set_state_on_drop_to: Cell<*mut Masked>,
}

/// Initialization value for static [`Once`] values. <br>静态 [`Once`] 值的初始化值。<br>
///
/// # Examples
///
/// ```
/// use std::sync::{Once, ONCE_INIT};
///
/// static START: Once = ONCE_INIT;
/// ```
#[stable(feature = "rust1", since = "1.0.0")]
#[deprecated(
    since = "1.38.0",
    note = "the `new` function is now preferred",
    suggestion = "Once::new()"
)]
pub const ONCE_INIT: Once = Once::new();

// Four states that a Once can be in, encoded into the lower bits of `state_and_queue` in the Once structure. <br>一次可以进入的四个状态，被编码为一次结构体中 `state_and_queue` 的低位。<br>
//
const INCOMPLETE: usize = 0x0;
const POISONED: usize = 0x1;
const RUNNING: usize = 0x2;
const COMPLETE: usize = 0x3;

// Mask to learn about the state. <br>隐含的了解状态。<br>
// All other bits are the queue of waiters if this is in the RUNNING state. <br>如果其他所有位都处于 RUNNING 状态，则它们都是等待者的队列。<br>
const STATE_MASK: usize = 0x3;

// Representation of a node in the linked list of waiters, used while in the RUNNING state. <br>处于 RUNNING 状态时使用的等待者的链表中的节点表示。<br>
// Note: `Waiter` can't hold a mutable pointer to the next thread, because then `wait` would both hand out a mutable reference to its `Waiter` node, and keep a shared reference to check `signaled`. <br>`Waiter` 不能持有指向下一个线程的可变指针，因为 `wait` 会同时向其 `Waiter` 节点发出变量引用，并保留共享的引用以检查 `signaled`。<br>
// Instead we hold shared references and use interior mutability. <br>相反，我们使用共享引用，并使用内部可变性。<br>
//
//
//
#[repr(align(4))] // Ensure the two lower bits are free to use as state bits. <br>确保两个低位可以自由用作状态位。<br>
struct Waiter {
    thread: Cell<Option<Thread>>,
    signaled: AtomicBool,
    next: *const Waiter,
}

// Head of a linked list of waiters. <br>等待者链表的头。<br>
// Every node is a struct on the stack of a waiting thread. <br>每个节点都是等待线程栈上的结构体。<br>
// Will wake up the waiters when it gets dropped, i.e. also on panic. <br>丢弃时将唤醒等待者，例如在 panic 上。<br>
struct WaiterQueue<'a> {
    state_and_queue: &'a AtomicPtr<Masked>,
    set_state_on_drop_to: *mut Masked,
}

impl Once {
    /// Creates a new `Once` value. <br>创建一个新的 `Once` 值。<br>
    #[inline]
    #[stable(feature = "once_new", since = "1.2.0")]
    #[rustc_const_stable(feature = "const_once_new", since = "1.32.0")]
    #[must_use]
    pub const fn new() -> Once {
        Once {
            state_and_queue: AtomicPtr::new(ptr::invalid_mut(INCOMPLETE)),
            _marker: marker::PhantomData,
        }
    }

    /// Performs an initialization routine once and only once. <br>仅执行一次初始化例程。<br> The given closure will be executed if this is the first time `call_once` has been called, and otherwise the routine will *not* be invoked. <br>如果这是第一次调用 `call_once`，则将执行给定的闭包，否则将 *不* 调用例程。<br>
    ///
    /// This method will block the calling thread if another initialization routine is currently running. <br>如果当前正在运行另一个初始化例程，则此方法将阻止调用线程。<br>
    ///
    /// When this function returns, it is guaranteed that some initialization has run and completed (it might not be the closure specified). <br>当这个函数返回时，保证一些初始化已经运行并完成 (它可能不是指定的闭包)。<br>
    /// It is also guaranteed that any memory writes performed by the executed closure can be reliably observed by other threads at this point (there is a happens-before relation between the closure and code executing after the return). <br>还可以确保此时其他线程可以可靠地观察到由执行的闭包执行的任何内存写操作 (闭包与返回之后执行的代码之间存在先于发生的关系)。<br>
    ///
    ///
    /// If the given closure recursively invokes `call_once` on the same [`Once`] instance the exact behavior is not specified, allowed outcomes are a panic or a deadlock. <br>如果给定的闭包在同一 [`Once`] 实例上递归调用 `call_once`，则未指定确切的行为，则允许的结果为 panic 或死锁。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::Once;
    ///
    /// static mut VAL: usize = 0;
    /// static INIT: Once = Once::new();
    ///
    /// // Accessing a `static mut` is unsafe much of the time, but if we do so in a synchronized fashion (e.g., write once or read all) then we're good to go! <br>在很多情况下，访问 `static mut` 是不安全的，但是如果我们以同步方式进行操作 (例如，一次写入或全部读取)，那么我们就可以开始了！<br>
    /////
    /////
    /// // This function will only call `expensive_computation` once, and will otherwise always return the value returned from the first invocation. <br>此函数将只调用一次 `expensive_computation`，否则将始终返回从第一次调用返回的值。<br>
    /////
    /////
    /// fn get_cached_val() -> usize {
    ///     unsafe {
    ///         INIT.call_once(|| {
    ///             VAL = expensive_computation();
    ///         });
    ///         VAL
    ///     }
    /// }
    ///
    /// fn expensive_computation() -> usize {
    ///     // ...
    /// # 2
    /// }
    /// ```
    ///
    /// # Panics
    ///
    /// The closure `f` will only be executed once if this is called concurrently amongst many threads. <br>如果在多个线程中同时调用闭包 `f`，则将仅执行一次。<br>
    /// If that closure panics, however, then it will *poison* this [`Once`] instance, causing all future invocations of `call_once` to also panic. <br>但是，如果该闭包 panics 将会毒化该 [`Once`] 实例，从而导致 `call_once` 的所有 future 调用也变为 panic。<br>
    ///
    /// This is similar to [poisoning with mutexes][poison]. <br>这类似于 [互斥体中毒][poison]。<br>
    ///
    /// [poison]: struct.Mutex.html#poisoning
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[track_caller]
    pub fn call_once<F>(&self, f: F)
    where
        F: FnOnce(),
    {
        // Fast path check <br>快速路径检查<br>
        if self.is_completed() {
            return;
        }

        let mut f = Some(f);
        self.call_inner(false, &mut |_| f.take().unwrap()());
    }

    /// Performs the same function as [`call_once()`] except ignores poisoning. <br>执行与 [`call_once()`] 相同的函数，只是忽略中毒。<br>
    ///
    /// Unlike [`call_once()`], if this [`Once`] has been poisoned (i.e., a previous call to [`call_once()`] or [`call_once_force()`] caused a panic), calling [`call_once_force()`] will still invoke the closure `f` and will _not_ result in an immediate panic. <br>与 [`call_once()`] 不同，如果此 [`Once`] 已中毒 (例如，先前对 [`call_once()`] 或 [`call_once_force()`] 的调用导致 panic)，则调用 [`call_once_force()`] 仍将调用闭包 `f`，并且 _not_ 会立即导致 panic。<br>
    /// If `f` panics, the [`Once`] will remain in a poison state. <br>如果 `f` panics，则 [`Once`] 将保持中毒状态。<br>
    /// If `f` does _not_ panic, the [`Once`] will no longer be in a poison state and all future calls to [`call_once()`] or [`call_once_force()`] will be no-ops. <br>如果 `f` 执行 _not_ panic，则 [`Once`] 将不再处于中毒状态，并且所有对 [`call_once()`] 或 [`call_once_force()`] 的 future 调用都将变为无操作。<br>
    ///
    /// The closure `f` is yielded a [`OnceState`] structure which can be used to query the poison status of the [`Once`]. <br>闭包 `f` 产生 [`OnceState`] 结构体，可用于查询 [`Once`] 的中毒状态。<br>
    ///
    /// [`call_once()`]: Once::call_once
    /// [`call_once_force()`]: Once::call_once_force
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::Once;
    /// use std::thread;
    ///
    /// static INIT: Once = Once::new();
    ///
    /// // poison the once <br>中毒一次<br>
    /// let handle = thread::spawn(|| {
    ///     INIT.call_once(|| panic!());
    /// });
    /// assert!(handle.join().is_err());
    ///
    /// // poisoning propagates <br>中毒传播<br>
    /// let handle = thread::spawn(|| {
    ///     INIT.call_once(|| {});
    /// });
    /// assert!(handle.join().is_err());
    ///
    /// // call_once_force will still run and reset the poisoned state <br>call_once_force 仍将运行并重置中毒状态<br>
    /// INIT.call_once_force(|state| {
    ///     assert!(state.is_poisoned());
    /// });
    ///
    /// // once any success happens, we stop propagating the poison <br>一旦成功，我们就停止传播毒药<br>
    /// INIT.call_once(|| {});
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "once_poison", since = "1.51.0")]
    pub fn call_once_force<F>(&self, f: F)
    where
        F: FnOnce(&OnceState),
    {
        // Fast path check <br>快速路径检查<br>
        if self.is_completed() {
            return;
        }

        let mut f = Some(f);
        self.call_inner(true, &mut |p| f.take().unwrap()(p));
    }

    /// Returns `true` if some [`call_once()`] call has completed successfully. <br>如果某些 [`call_once()`] 调用已成功完成，则返回 `true`。<br> Specifically, `is_completed` will return false in the following situations: <br>具体而言，在以下情况下，`is_completed` 将返回 false：<br>
    ///   * [`call_once()`] was not called at all, <br>[`call_once()`] 根本没有被调用，<br>
    ///   * [`call_once()`] was called, but has not yet completed, <br>[`call_once()`] 已被调用，但尚未完成，<br>
    ///   * the [`Once`] instance is poisoned <br>[`Once`] 实例中毒<br>
    ///
    /// This function returning `false` does not mean that [`Once`] has not been executed. <br>此函数返回 `false` 并不意味着 [`Once`] 尚未执行。<br>
    /// For example, it may have been executed in the time between when `is_completed` starts executing and when it returns, in which case the `false` return value would be stale (but still permissible). <br>例如，它可能是在 `is_completed` 开始执行到返回之间的时间执行的，在这种情况下，`false` 的返回值将是陈旧的 (但仍然是允许的)。<br>
    ///
    ///
    /// [`call_once()`]: Once::call_once
    ///
    /// # Examples
    ///
    /// ```
    /// use std::sync::Once;
    ///
    /// static INIT: Once = Once::new();
    ///
    /// assert_eq!(INIT.is_completed(), false);
    /// INIT.call_once(|| {
    ///     assert_eq!(INIT.is_completed(), false);
    /// });
    /// assert_eq!(INIT.is_completed(), true);
    /// ```
    ///
    /// ```
    /// use std::sync::Once;
    /// use std::thread;
    ///
    /// static INIT: Once = Once::new();
    ///
    /// assert_eq!(INIT.is_completed(), false);
    /// let handle = thread::spawn(|| {
    ///     INIT.call_once(|| panic!());
    /// });
    /// assert!(handle.join().is_err());
    /// assert_eq!(INIT.is_completed(), false);
    /// ```
    ///
    ///
    ///
    #[stable(feature = "once_is_completed", since = "1.43.0")]
    #[inline]
    pub fn is_completed(&self) -> bool {
        // An `Acquire` load is enough because that makes all the initialization operations visible to us, and, this being a fast path, weaker ordering helps with performance. <br>`Acquire` 负载就足够了，因为这使我们可以看到所有初始化操作，并且这是一条捷径，较弱的排序有助于提高性能。<br>
        // This `Acquire` synchronizes with `Release` operations on the slow path. <br>该 `Acquire` 与慢速路径上的 `Release` 操作同步。<br>
        //
        //
        self.state_and_queue.load(Ordering::Acquire).addr() == COMPLETE
    }

    // This is a non-generic function to reduce the monomorphization cost of using `call_once` (this isn't exactly a trivial or small implementation). <br>这是一个非通用类型的函数，可以减少使用 `call_once` 的单态化成本 (这并非微不足道或很小的实现)。<br>
    //
    // Additionally, this is tagged with `#[cold]` as it should indeed be cold and it helps let LLVM know that calls to this function should be off the fast path. <br>另外，它用 `#[cold]` 标记，因为它确实应该是冷的，它有助于让 LLVM 知道对该函数的调用应该不在快速路径之内。<br>
    // Essentially, this should help generate more straight line code in LLVM. <br>从本质上讲，这应该有助于在 LLVM 中生成更多直线代码。<br>
    //
    // Finally, this takes an `FnMut` instead of a `FnOnce` because there's currently no way to take an `FnOnce` and call it via virtual dispatch without some allocation overhead. <br>最后，这需要 `FnMut` 而不是 `FnOnce`，因为当前没有办法通过虚拟调度来获取 `FnOnce` 并调用它，而没有一些分配开销。<br>
    //
    //
    //
    //
    //
    #[cold]
    #[track_caller]
    fn call_inner(&self, ignore_poisoning: bool, init: &mut dyn FnMut(&OnceState)) {
        let mut state_and_queue = self.state_and_queue.load(Ordering::Acquire);
        loop {
            match state_and_queue.addr() {
                COMPLETE => break,
                POISONED if !ignore_poisoning => {
                    // Panic to propagate the poison. <br>panic 会传播 poison。<br>
                    panic!("Once instance has previously been poisoned");
                }
                POISONED | INCOMPLETE => {
                    // Try to register this thread as the one RUNNING. <br>尝试将该线程注册为一个正在运行的线程。<br>
                    let exchange_result = self.state_and_queue.compare_exchange(
                        state_and_queue,
                        ptr::invalid_mut(RUNNING),
                        Ordering::Acquire,
                        Ordering::Acquire,
                    );
                    if let Err(old) = exchange_result {
                        state_and_queue = old;
                        continue;
                    }
                    // `waiter_queue` will manage other waiting threads, and wake them up on drop. <br>`waiter_queue` 将管理其他等待线程，并在丢弃时唤醒它们。<br>
                    //
                    let mut waiter_queue = WaiterQueue {
                        state_and_queue: &self.state_and_queue,
                        set_state_on_drop_to: ptr::invalid_mut(POISONED),
                    };
                    // Run the initialization function, letting it know if we're poisoned or not. <br>运行初始化函数，让它知道我们是否中毒。<br>
                    //
                    let init_state = OnceState {
                        poisoned: state_and_queue.addr() == POISONED,
                        set_state_on_drop_to: Cell::new(ptr::invalid_mut(COMPLETE)),
                    };
                    init(&init_state);
                    waiter_queue.set_state_on_drop_to = init_state.set_state_on_drop_to.get();
                    break;
                }
                _ => {
                    // All other values must be RUNNING with possibly a pointer to the waiter queue in the more significant bits. <br>所有其他值都必须为 RUNNING，并且可能有一个指向更高有效位的侍者队列的指针。<br>
                    //
                    assert!(state_and_queue.addr() & STATE_MASK == RUNNING);
                    wait(&self.state_and_queue, state_and_queue);
                    state_and_queue = self.state_and_queue.load(Ordering::Acquire);
                }
            }
        }
    }
}

fn wait(state_and_queue: &AtomicPtr<Masked>, mut current_state: *mut Masked) {
    // Note: the following code was carefully written to avoid creating a mutable reference to `node` that gets aliased. <br>以下代码经过精心编写，以避免对 `node` 创建别名。<br>
    //
    loop {
        // Don't queue this thread if the status is no longer running, otherwise we will not be woken up. <br>如果状态不再运行，请不要将此线程排队，否则我们不会被唤醒。<br>
        //
        if current_state.addr() & STATE_MASK != RUNNING {
            return;
        }

        // Create the node for our current thread. <br>为我们当前的线程创建节点。<br>
        let node = Waiter {
            thread: Cell::new(Some(thread::current())),
            signaled: AtomicBool::new(false),
            next: current_state.with_addr(current_state.addr() & !STATE_MASK) as *const Waiter,
        };
        let me = &node as *const Waiter as *const Masked as *mut Masked;

        // Try to slide in the node at the head of the linked list, making sure that another thread didn't just replace the head of the linked list. <br>尝试在链表顶部的节点中滑动，确保其他线程不只是替换链表的头部。<br>
        //
        let exchange_result = state_and_queue.compare_exchange(
            current_state,
            me.with_addr(me.addr() | RUNNING),
            Ordering::Release,
            Ordering::Relaxed,
        );
        if let Err(old) = exchange_result {
            current_state = old;
            continue;
        }

        // We have enqueued ourselves, now lets wait. <br>我们已经入队，现在让我们等待。<br>
        // It is important not to return before being signaled, otherwise we would drop our `Waiter` node and leave a hole in the linked list (and a dangling reference). <br>重要的是不要在收到信号之前返回，否则我们将丢弃 `Waiter` 节点，并将 hole 留在链表中 (并悬垂引用)。<br>
        // Guard against spurious wakeups by reparking ourselves until we are signaled. <br>提醒自己，直到收到信号为止，以防虚假唤醒。<br>
        //
        //
        while !node.signaled.load(Ordering::Acquire) {
            // If the managing thread happens to signal and unpark us before we can park ourselves, the result could be this thread never gets unparked. <br>如果管理线程恰好在我们可以停泊自己之前发出信号并释放了我们，则结果可能是该线程永远不会停泊。<br>
            // Luckily `park` comes with the guarantee that if it got an `unpark` just before on an unparked thread it does not park. <br>幸运的是，`park` 保证如果它在未停放的线程上获得 `unpark`，则它不会 park。<br>
            //
            //
            thread::park();
        }
        break;
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl fmt::Debug for Once {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Once").finish_non_exhaustive()
    }
}

impl Drop for WaiterQueue<'_> {
    fn drop(&mut self) {
        // Swap out our state with however we finished. <br>与我们交换状态，但是我们完成了。<br>
        let state_and_queue =
            self.state_and_queue.swap(self.set_state_on_drop_to, Ordering::AcqRel);

        // We should only ever see an old state which was RUNNING. <br>我们应该只看到一个正在运行的旧状态。<br>
        assert_eq!(state_and_queue.addr() & STATE_MASK, RUNNING);

        // Walk the entire linked list of waiters and wake them up (in lifo order, last to register is first to wake up). <br>遍历整个 waiters 的链表并唤醒他们 (按照 lifo 顺序，最后注册要首先唤醒)。<br>
        //
        unsafe {
            // Right after setting `node.signaled = true` the other thread may free `node` if there happens to be has a spurious wakeup. <br>设置 `node.signaled = true` 之后，如果恰好有虚假唤醒，其他线程可能会释放 `node`。<br>
            //
            // So we have to take out the `thread` field and copy the pointer to `next` first. <br>因此，我们必须取出 `thread` 字段，然后首先将指针复制到 `next`。<br>
            //
            let mut queue =
                state_and_queue.with_addr(state_and_queue.addr() & !STATE_MASK) as *const Waiter;
            while !queue.is_null() {
                let next = (*queue).next;
                let thread = (*queue).thread.take().unwrap();
                (*queue).signaled.store(true, Ordering::Release);
                // ^- FIXME (maybe): This is another case of issue #55005 `store()` has a potentially dangling ref to `signaled`. <br>这是 issue #55005 的另一种情况 `store()` 潜在地悬挂了对 `signaled` 的引用。<br>
                //
                queue = next;
                thread.unpark();
            }
        }
    }
}

impl OnceState {
    /// Returns `true` if the associated [`Once`] was poisoned prior to the invocation of the closure passed to [`Once::call_once_force()`]. <br>如果关联的 [`Once`] 在调用传递给 [`Once::call_once_force()`] 的闭包之前中毒，则返回 `true`。<br>
    ///
    ///
    /// # Examples
    ///
    /// A poisoned [`Once`]: <br>中毒的 [`Once`]：<br>
    ///
    /// ```
    /// use std::sync::Once;
    /// use std::thread;
    ///
    /// static INIT: Once = Once::new();
    ///
    /// // poison the once <br>中毒一次<br>
    /// let handle = thread::spawn(|| {
    ///     INIT.call_once(|| panic!());
    /// });
    /// assert!(handle.join().is_err());
    ///
    /// INIT.call_once_force(|state| {
    ///     assert!(state.is_poisoned());
    /// });
    /// ```
    ///
    /// An unpoisoned [`Once`]: <br>无毒的 [`Once`]：<br>
    ///
    /// ```
    /// use std::sync::Once;
    ///
    /// static INIT: Once = Once::new();
    ///
    /// INIT.call_once_force(|state| {
    ///     assert!(!state.is_poisoned());
    /// });
    #[stable(feature = "once_poison", since = "1.51.0")]
    pub fn is_poisoned(&self) -> bool {
        self.poisoned
    }

    /// Poison the associated [`Once`] without explicitly panicking. <br>中毒关联的 [`Once`]，而不显式 panic。<br>
    // NOTE: This is currently only exposed for the `lazy` module <br>当前仅针对 `lazy` 模块公开<br>
    pub(crate) fn poison(&self) {
        self.set_state_on_drop_to.set(ptr::invalid_mut(POISONED));
    }
}
