use crate::cell::UnsafeCell;
use crate::fmt;
use crate::marker::PhantomData;
use crate::mem::MaybeUninit;
use crate::panic::{RefUnwindSafe, UnwindSafe};
use crate::pin::Pin;
use crate::sync::Once;

/// A synchronization primitive which can be written to only once. <br>只能写入一次的同步原语。<br>
///
/// This type is a thread-safe `OnceCell`. <br>此类型是线程安全的 `OnceCell`。<br>
///
/// # Examples
///
/// ```
/// #![feature(once_cell)]
///
/// use std::sync::OnceLock;
///
/// static CELL: OnceLock<String> = OnceLock::new();
/// assert!(CELL.get().is_none());
///
/// std::thread::spawn(|| {
///     let value: &String = CELL.get_or_init(|| {
///         "Hello, World!".to_string()
///     });
///     assert_eq!(value, "Hello, World!");
/// }).join().unwrap();
///
/// let value: Option<&String> = CELL.get();
/// assert!(value.is_some());
/// assert_eq!(value.unwrap().as_str(), "Hello, World!");
/// ```
#[unstable(feature = "once_cell", issue = "74465")]
pub struct OnceLock<T> {
    once: Once,
    // Whether or not the value is initialized is tracked by `state_and_queue`. <br>`state_and_queue` 跟踪该值是否已初始化。<br>
    value: UnsafeCell<MaybeUninit<T>>,
    /// `PhantomData` to make sure dropck understands we're dropping T in our Drop impl. <br>`PhantomData` 以确保 dropck 理解我们在 Drop impl 中丢弃了 T。<br>
    ///
    /// ```compile_fail,E0597
    /// #![feature(once_cell)]
    ///
    /// use std::sync::OnceLock;
    ///
    /// struct A<'a>(&'a str);
    ///
    /// impl<'a> Drop for A<'a> {
    ///     fn drop(&mut self) {}
    /// }
    ///
    /// let cell = OnceLock::new();
    /// {
    ///     let s = String::new();
    ///     let _ = cell.set(A(&s));
    /// }
    /// ```
    _marker: PhantomData<T>,
}

impl<T> OnceLock<T> {
    /// Creates a new empty cell. <br>创建一个新的空 cell。<br>
    #[unstable(feature = "once_cell", issue = "74465")]
    #[must_use]
    pub const fn new() -> OnceLock<T> {
        OnceLock {
            once: Once::new(),
            value: UnsafeCell::new(MaybeUninit::uninit()),
            _marker: PhantomData,
        }
    }

    /// Gets the reference to the underlying value. <br>获取对底层值的引用。<br>
    ///
    /// Returns `None` if the cell is empty, or being initialized. <br>如果 cell 为空或正在初始化，则返回 `None`。<br>
    /// This method never blocks. <br>此方法永远不会阻塞。<br>
    #[unstable(feature = "once_cell", issue = "74465")]
    pub fn get(&self) -> Option<&T> {
        if self.is_initialized() {
            // Safe b/c checked is_initialized <br>安全 b/c 已检查 is_initialized<br>
            Some(unsafe { self.get_unchecked() })
        } else {
            None
        }
    }

    /// Gets the mutable reference to the underlying value. <br>获取对底层值的可变引用。<br>
    ///
    /// Returns `None` if the cell is empty. <br>如果 cell 为空，则返回 `None`。<br> This method never blocks. <br>此方法永远不会阻塞。<br>
    #[unstable(feature = "once_cell", issue = "74465")]
    pub fn get_mut(&mut self) -> Option<&mut T> {
        if self.is_initialized() {
            // Safe b/c checked is_initialized and we have a unique access <br>安全 b/c 已检查 is_initialized，我们拥有唯一的访问权<br>
            Some(unsafe { self.get_unchecked_mut() })
        } else {
            None
        }
    }

    /// Sets the contents of this cell to `value`. <br>将此 cell 的内容设置为 `value`。<br>
    ///
    /// May block if another thread is currently attempting to initialize the cell. <br>如果另一个线程当前正在尝试初始化该 cell，则可能会阻塞。<br>
    /// The cell is guaranteed to contain a value when set returns, though not necessarily the one provided. <br>尽管 set 不一定返回，但保证该 cell 包含一个值。<br>
    ///
    /// Returns `Ok(())` if the cell's value was set by this call. <br>如果此调用设置了 cell 的值，则返回 `Ok(())`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(once_cell)]
    ///
    /// use std::sync::OnceLock;
    ///
    /// static CELL: OnceLock<i32> = OnceLock::new();
    ///
    /// fn main() {
    ///     assert!(CELL.get().is_none());
    ///
    ///     std::thread::spawn(|| {
    ///         assert_eq!(CELL.set(92), Ok(()));
    ///     }).join().unwrap();
    ///
    ///     assert_eq!(CELL.set(62), Err(62));
    ///     assert_eq!(CELL.get(), Some(&92));
    /// }
    /// ```
    #[unstable(feature = "once_cell", issue = "74465")]
    pub fn set(&self, value: T) -> Result<(), T> {
        let mut value = Some(value);
        self.get_or_init(|| value.take().unwrap());
        match value {
            None => Ok(()),
            Some(value) => Err(value),
        }
    }

    /// Gets the contents of the cell, initializing it with `f` if the cell was empty. <br>获取 cell 的内容，如果 cell 为空，则使用 `f` 对其进行初始化。<br>
    ///
    /// Many threads may call `get_or_init` concurrently with different initializing functions, but it is guaranteed that only one function will be executed. <br>许多线程可以使用不同的初始化函数并发调用 `get_or_init`，但是可以保证仅执行一个函数。<br>
    ///
    ///
    /// # Panics
    ///
    /// If `f` panics, the panic is propagated to the caller, and the cell remains uninitialized. <br>如果 `f` panics，则 panic 会传播给调用者，并且 cell 仍保持未初始化状态。<br>
    ///
    /// It is an error to reentrantly initialize the cell from `f`. <br>重新从 `f` 初始化 cell 是错误的。<br> The exact outcome is unspecified. <br>确切的结果是不确定的。<br>
    /// Current implementation deadlocks, but this may be changed to a panic in the future. <br>当前的实现死锁，但是可以在 future 中将其更改为 panic。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(once_cell)]
    ///
    /// use std::sync::OnceLock;
    ///
    /// let cell = OnceLock::new();
    /// let value = cell.get_or_init(|| 92);
    /// assert_eq!(value, &92);
    /// let value = cell.get_or_init(|| unreachable!());
    /// assert_eq!(value, &92);
    /// ```
    ///
    ///
    ///
    ///
    #[unstable(feature = "once_cell", issue = "74465")]
    pub fn get_or_init<F>(&self, f: F) -> &T
    where
        F: FnOnce() -> T,
    {
        match self.get_or_try_init(|| Ok::<T, !>(f())) {
            Ok(val) => val,
        }
    }

    /// Gets the contents of the cell, initializing it with `f` if the cell was empty. <br>获取 cell 的内容，如果 cell 为空，则使用 `f` 对其进行初始化。<br>
    /// If the cell was empty and `f` failed, an error is returned. <br>如果 cell 为空并且 `f` 失败，则返回错误。<br>
    ///
    /// # Panics
    ///
    /// If `f` panics, the panic is propagated to the caller, and the cell remains uninitialized. <br>如果 `f` panics，则 panic 会传播给调用者，并且 cell 仍保持未初始化状态。<br>
    ///
    ///
    /// It is an error to reentrantly initialize the cell from `f`. <br>重新从 `f` 初始化 cell 是错误的。<br>
    /// The exact outcome is unspecified. <br>确切的结果是不确定的。<br>
    /// Current implementation deadlocks, but this may be changed to a panic in the future. <br>当前的实现死锁，但是可以在 future 中将其更改为 panic。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(once_cell)]
    ///
    /// use std::sync::OnceLock;
    ///
    /// let cell = OnceLock::new();
    /// assert_eq!(cell.get_or_try_init(|| Err(())), Err(()));
    /// assert!(cell.get().is_none());
    /// let value = cell.get_or_try_init(|| -> Result<i32, ()> {
    ///     Ok(92)
    /// });
    /// assert_eq!(value, Ok(&92));
    /// assert_eq!(cell.get(), Some(&92))
    /// ```
    ///
    #[unstable(feature = "once_cell", issue = "74465")]
    pub fn get_or_try_init<F, E>(&self, f: F) -> Result<&T, E>
    where
        F: FnOnce() -> Result<T, E>,
    {
        // Fast path check <br>快速路径检查<br>
        // NOTE: We need to perform an acquire on the state in this method in order to correctly synchronize `LazyLock::force`. <br>为了正确同步 `LazyLock::force`，我们需要在这个方法中对状态进行获取。<br>
        // This is currently done by calling `self.get()`, which in turn calls `self.is_initialized()`, which in turn performs the acquire. <br>当前，这是通过调用 `self.get()` (依次调用 `self.is_initialized()`) 来完成的，而 `self.is_initialized()` 依次执行获取。<br>
        //
        //
        if let Some(value) = self.get() {
            return Ok(value);
        }
        self.initialize(f)?;

        debug_assert!(self.is_initialized());

        // SAFETY: The inner value has been initialized <br>内部值已初始化<br>
        Ok(unsafe { self.get_unchecked() })
    }

    /// Internal-only API that gets the contents of the cell, initializing it in two steps with `f` and `g` if the cell was empty. <br>仅限内部的 API，用于获取 cell 的内容，如果 cell 为空，则使用 `f` 和 `g` 分两步对其进行初始化。<br>
    ///
    /// `f` is called to construct the value, which is then moved into the cell and given as a (pinned) mutable reference to `g` to finish initialization. <br>调用 `f` 构造一个值，然后将其移入到 cell 中，并作为对 `g` 的（pinned）可变引用给出以完成初始化。<br>
    ///
    /// This allows `g` to inspect an manipulate the value after it has been moved into its final place in the cell, but before the cell is considered initialized. <br>这使 `g` 在将值移到 cell 中的最终位置之后但在将其视为初始化之前，可以检查该值。<br>
    ///
    ///
    /// # Panics
    ///
    /// If `f` or `g` panics, the panic is propagated to the caller, and the cell remains uninitialized. <br>如果为 `f` 或 `g` panics，则 panic 会传播到调用者，并且 cell 仍保持未初始化状态。<br>
    ///
    /// With the current implementation, if `g` panics, the value from `f` will not be dropped. <br>在当前实现中，如果 `g` panics，则不会丢弃 `f` 中的值。<br>
    /// This should probably be fixed if this is ever used for a type where this matters. <br>如果此类型曾用于此类型，则可能应该修复此问题。<br>
    ///
    /// It is an error to reentrantly initialize the cell from `f`. <br>重新从 `f` 初始化 cell 是错误的。<br> The exact outcome is unspecified. <br>确切的结果是不确定的。<br>
    /// Current implementation deadlocks, but this may be changed to a panic in the future. <br>当前的实现死锁，但是可以在 future 中将其更改为 panic。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    pub(crate) fn get_or_init_pin<F, G>(self: Pin<&Self>, f: F, g: G) -> Pin<&T>
    where
        F: FnOnce() -> T,
        G: FnOnce(Pin<&mut T>),
    {
        if let Some(value) = self.get_ref().get() {
            // SAFETY: The inner value was already initialized, and will not be moved anymore. <br>内部值已被初始化，将不再移动。<br>
            //
            return unsafe { Pin::new_unchecked(value) };
        }

        let slot = &self.value;

        // Ignore poisoning from other threads If another thread panics, then we'll be able to run our closure <br>忽略其他线程的中毒如果另一个线程 panics，那么我们将能够运行我们的闭包<br>
        //
        self.once.call_once_force(|_| {
            let value = f();
            // SAFETY: We use the Once (self.once) to guarantee unique access to the UnsafeCell (slot). <br>我们使用一次 (self.once) 来保证对 UnsafeCell (slot) 的唯一访问。<br>
            //
            let value: &mut T = unsafe { (&mut *slot.get()).write(value) };
            // SAFETY: The value has been written to its final place in self.value. <br>该值已被写入 self.value 的最终位置。<br>
            // We do not to move it anymore, which we promise here with a Pin<&mut T>. <br>我们不再移动它，我们在这里用 Pin<&mut T> 将其 promise 移动。<br>
            //
            g(unsafe { Pin::new_unchecked(value) });
        });

        // SAFETY: The inner value has been initialized, and will not be moved anymore. <br>内部值已初始化，将不再移动。<br>
        //
        unsafe { Pin::new_unchecked(self.get_ref().get_unchecked()) }
    }

    /// Consumes the `OnceLock`, returning the wrapped value. <br>使用 `OnceLock`，返回包装后的值。<br>
    /// Returns `None` if the cell was empty. <br>如果 cell 为空，则返回 `None`。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(once_cell)]
    ///
    /// use std::sync::OnceLock;
    ///
    /// let cell: OnceLock<String> = OnceLock::new();
    /// assert_eq!(cell.into_inner(), None);
    ///
    /// let cell = OnceLock::new();
    /// cell.set("hello".to_string()).unwrap();
    /// assert_eq!(cell.into_inner(), Some("hello".to_string()));
    /// ```
    #[unstable(feature = "once_cell", issue = "74465")]
    pub fn into_inner(mut self) -> Option<T> {
        self.take()
    }

    /// Takes the value out of this `OnceLock`, moving it back to an uninitialized state. <br>从此 `OnceLock` 中取出值，将其移回未初始化状态。<br>
    ///
    /// Has no effect and returns `None` if the `OnceLock` hasn't been initialized. <br>如果 `OnceLock` 尚未初始化，则无效并返回 `None`。<br>
    ///
    /// Safety is guaranteed by requiring a mutable reference. <br>通过要求可变引用来保证安全。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(once_cell)]
    ///
    /// use std::sync::OnceLock;
    ///
    /// let mut cell: OnceLock<String> = OnceLock::new();
    /// assert_eq!(cell.take(), None);
    ///
    /// let mut cell = OnceLock::new();
    /// cell.set("hello".to_string()).unwrap();
    /// assert_eq!(cell.take(), Some("hello".to_string()));
    /// assert_eq!(cell.get(), None);
    /// ```
    #[unstable(feature = "once_cell", issue = "74465")]
    pub fn take(&mut self) -> Option<T> {
        if self.is_initialized() {
            self.once = Once::new();
            // SAFETY: `self.value` is initialized and contains a valid `T`. <br>`self.value` 已初始化，并包含有效的 `T`。<br>
            // `self.once` is reset, so `is_initialized()` will be false again which prevents the value from being read twice. <br>`self.once` 被重置，因此 `is_initialized()` 将再次为 false，从而防止该值被读取两次。<br>
            //
            unsafe { Some((&mut *self.value.get()).assume_init_read()) }
        } else {
            None
        }
    }

    #[inline]
    fn is_initialized(&self) -> bool {
        self.once.is_completed()
    }

    #[cold]
    fn initialize<F, E>(&self, f: F) -> Result<(), E>
    where
        F: FnOnce() -> Result<T, E>,
    {
        let mut res: Result<(), E> = Ok(());
        let slot = &self.value;

        // Ignore poisoning from other threads If another thread panics, then we'll be able to run our closure <br>忽略其他线程的中毒如果另一个线程 panics，那么我们将能够运行我们的闭包<br>
        //
        self.once.call_once_force(|p| {
            match f() {
                Ok(value) => {
                    unsafe { (&mut *slot.get()).write(value) };
                }
                Err(e) => {
                    res = Err(e);

                    // Treat the underlying `Once` as poisoned since we failed to initialize our value. <br>因为我们未能初始化我们的值，所以将底层 `Once` 视为中毒。<br>
                    // Calls
                    p.poison();
                }
            }
        });
        res
    }

    /// # Safety
    ///
    /// The value must be initialized <br>该值必须初始化<br>
    unsafe fn get_unchecked(&self) -> &T {
        debug_assert!(self.is_initialized());
        (&*self.value.get()).assume_init_ref()
    }

    /// # Safety
    ///
    /// The value must be initialized <br>该值必须初始化<br>
    unsafe fn get_unchecked_mut(&mut self) -> &mut T {
        debug_assert!(self.is_initialized());
        (&mut *self.value.get()).assume_init_mut()
    }
}

// Why do we need `T: Send`? <br>为什么我们需要 `T: Send`？<br>
// Thread A creates a `OnceLock` and shares it with scoped thread B, which fills the cell, which is then destroyed by A. <br>线程 A 创建一个 X0 并与作用域线程 B 共享它，该线程 B 填充 cell，然后是 A 的 X00。<br>
// That is, destructor observes a sent value. <br>也就是说，析构函数观察发送的值。<br>
//
//
#[unstable(feature = "once_cell", issue = "74465")]
unsafe impl<T: Sync + Send> Sync for OnceLock<T> {}
#[unstable(feature = "once_cell", issue = "74465")]
unsafe impl<T: Send> Send for OnceLock<T> {}

#[unstable(feature = "once_cell", issue = "74465")]
impl<T: RefUnwindSafe + UnwindSafe> RefUnwindSafe for OnceLock<T> {}
#[unstable(feature = "once_cell", issue = "74465")]
impl<T: UnwindSafe> UnwindSafe for OnceLock<T> {}

#[unstable(feature = "once_cell", issue = "74465")]
#[rustc_const_unstable(feature = "const_default_impls", issue = "87864")]
impl<T> const Default for OnceLock<T> {
    /// Creates a new empty cell. <br>创建一个新的空 cell。<br>
    ///
    /// # Example
    ///
    /// ```
    /// #![feature(once_cell)]
    ///
    /// use std::sync::OnceLock;
    ///
    /// fn main() {
    ///     assert_eq!(OnceLock::<()>::new(), OnceLock::default());
    /// }
    /// ```
    fn default() -> OnceLock<T> {
        OnceLock::new()
    }
}

#[unstable(feature = "once_cell", issue = "74465")]
impl<T: fmt::Debug> fmt::Debug for OnceLock<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self.get() {
            Some(v) => f.debug_tuple("Once").field(v).finish(),
            None => f.write_str("Once(Uninit)"),
        }
    }
}

#[unstable(feature = "once_cell", issue = "74465")]
impl<T: Clone> Clone for OnceLock<T> {
    fn clone(&self) -> OnceLock<T> {
        let cell = Self::new();
        if let Some(value) = self.get() {
            match cell.set(value.clone()) {
                Ok(()) => (),
                Err(_) => unreachable!(),
            }
        }
        cell
    }
}

#[unstable(feature = "once_cell", issue = "74465")]
impl<T> From<T> for OnceLock<T> {
    /// Create a new cell with its contents set to `value`. <br>创建一个新的 cell，其内容设置为 `value`。<br>
    ///
    /// # Example
    ///
    /// ```
    /// #![feature(once_cell)]
    ///
    /// use std::sync::OnceLock;
    ///
    /// # fn main() -> Result<(), i32> {
    /// let a = OnceLock::from(3);
    /// let b = OnceLock::new();
    /// b.set(3)?;
    /// assert_eq!(a, b);
    /// Ok(())
    /// # }
    /// ```
    fn from(value: T) -> Self {
        let cell = Self::new();
        match cell.set(value) {
            Ok(()) => cell,
            Err(_) => unreachable!(),
        }
    }
}

#[unstable(feature = "once_cell", issue = "74465")]
impl<T: PartialEq> PartialEq for OnceLock<T> {
    fn eq(&self, other: &OnceLock<T>) -> bool {
        self.get() == other.get()
    }
}

#[unstable(feature = "once_cell", issue = "74465")]
impl<T: Eq> Eq for OnceLock<T> {}

#[unstable(feature = "once_cell", issue = "74465")]
unsafe impl<#[may_dangle] T> Drop for OnceLock<T> {
    fn drop(&mut self) {
        if self.is_initialized() {
            // SAFETY: The cell is initialized and being dropped, so it can't be accessed again. <br>cell 已初始化并已丢弃，因此无法再次访问。<br>
            // We also don't touch the `T` other than dropping it, which validates our usage of #[may_dangle]. <br>除了 drop 它之外，我们也不会触及 `T`，它可以验证我们对 #[may_dangle] 的用法。<br>
            //
            unsafe { (&mut *self.value.get()).assume_init_drop() };
        }
    }
}

#[cfg(test)]
mod tests;
