use crate::error::Error;
use crate::fmt;
use crate::sync::atomic::{AtomicBool, Ordering};
use crate::thread;

pub struct Flag {
    failed: AtomicBool,
}

// Note that the Ordering uses to access the `failed` field of `Flag` below is always `Relaxed`, and that's because this isn't actually protecting any data, it's just a flag whether we've panicked or not. <br>请注意，订购用于访问下面的 `Flag` 的 `failed` 字段始终是 `Relaxed`，这是因为这实际上并没有保护任何数据，它只是我们是否 panic 的标志。<br>
//
//
// The actual location that this matters is when a mutex is **locked** which is where we have external synchronization ensuring that we see memory reads/writes to this flag. <br>重要的实际位置是当互斥锁被锁定时，这是我们进行外部同步的地方，确保我们看到内存读/写到这个标志。<br>
//
// As a result, if it matters, we should see the correct value for `failed` in all cases. <br>因此，如果有关系，我们应该在所有情况下都可以看到 `failed` 的正确值。<br>
//
//
//
//

impl Flag {
    #[inline]
    pub const fn new() -> Flag {
        Flag { failed: AtomicBool::new(false) }
    }

    /// Check the flag for an unguarded borrow, where we only care about existing poison. <br>检查标志是否有无人看管的借用，我们只关心现有的毒药。<br>
    #[inline]
    pub fn borrow(&self) -> LockResult<()> {
        if self.get() { Err(PoisonError::new(())) } else { Ok(()) }
    }

    /// Check the flag for a guarded borrow, where we may also set poison when `done`. <br>检查标志是否有守卫的借用，我们也可能在 `done` 时设置毒药。<br>
    #[inline]
    pub fn guard(&self) -> LockResult<Guard> {
        let ret = Guard { panicking: thread::panicking() };
        if self.get() { Err(PoisonError::new(ret)) } else { Ok(ret) }
    }

    #[inline]
    pub fn done(&self, guard: &Guard) {
        if !guard.panicking && thread::panicking() {
            self.failed.store(true, Ordering::Relaxed);
        }
    }

    #[inline]
    pub fn get(&self) -> bool {
        self.failed.load(Ordering::Relaxed)
    }

    #[inline]
    pub fn clear(&self) {
        self.failed.store(false, Ordering::Relaxed)
    }
}

pub struct Guard {
    panicking: bool,
}

/// A type of error which can be returned whenever a lock is acquired. <br>一种错误类型，每当获取锁时都可以返回该错误。<br>
///
/// Both [`Mutex`]es and [`RwLock`]s are poisoned whenever a thread fails while the lock is held. <br>每当持有锁的线程发生故障时，[`Mutex`] 和 [`RwLock`] 都会中毒。<br>
/// The precise semantics for when a lock is poisoned is documented on each lock, but once a lock is poisoned then all future acquisitions will return this error. <br>锁中毒的确切语义记录在每个锁上，但是一旦锁中毒，则所有 future 获取都将返回此错误。<br>
///
///
/// # Examples
///
/// ```
/// use std::sync::{Arc, Mutex};
/// use std::thread;
///
/// let mutex = Arc::new(Mutex::new(1));
///
/// // poison the mutex <br>互斥锁中毒<br>
/// let c_mutex = Arc::clone(&mutex);
/// let _ = thread::spawn(move || {
///     let mut data = c_mutex.lock().unwrap();
///     *data = 2;
///     panic!();
/// }).join();
///
/// match mutex.lock() {
///     Ok(_) => unreachable!(),
///     Err(p_err) => {
///         let data = p_err.get_ref();
///         println!("recovered: {data}");
///     }
/// };
/// ```
/// [`Mutex`]: crate::sync::Mutex
/// [`RwLock`]: crate::sync::RwLock
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct PoisonError<T> {
    guard: T,
}

/// An enumeration of possible errors associated with a [`TryLockResult`] which can occur while trying to acquire a lock, from the [`try_lock`] method on a [`Mutex`] or the [`try_read`] and [`try_write`] methods on an [`RwLock`]. <br>枚举可能与 [`TryLockResult`] 相关的错误，这些错误可能是从 [`Mutex`] 上的 [`try_lock`] 方法或 [`RwLock`] 上的 [`try_read`] 和 [`try_write`] 方法获取锁时发生的。<br>
///
///
/// [`try_lock`]: crate::sync::Mutex::try_lock
/// [`try_read`]: crate::sync::RwLock::try_read
/// [`try_write`]: crate::sync::RwLock::try_write
/// [`Mutex`]: crate::sync::Mutex
/// [`RwLock`]: crate::sync::RwLock
///
#[stable(feature = "rust1", since = "1.0.0")]
pub enum TryLockError<T> {
    /// The lock could not be acquired because another thread failed while holding the lock. <br>由于另一个线程在持有锁时失败，因此无法获取该锁。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    Poisoned(#[stable(feature = "rust1", since = "1.0.0")] PoisonError<T>),
    /// The lock could not be acquired at this time because the operation would otherwise block. <br>此时无法获取该锁，因为否则该操作将阻塞。<br>
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    WouldBlock,
}

/// A type alias for the result of a lock method which can be poisoned. <br>一种类型别名，用于可能导致中毒的锁定方法。<br>
///
/// The [`Ok`] variant of this result indicates that the primitive was not poisoned, and the `Guard` is contained within. <br>此结果的 [`Ok`] 变体表示该原语没有中毒，并且其中包含 `Guard`。<br>
/// The [`Err`] variant indicates that the primitive was poisoned. <br>[`Err`] 变体表示该原语已中毒。<br>
/// Note that the [`Err`] variant *also* carries the associated guard, and it can be acquired through the [`into_inner`] method. <br>请注意，[`Err`] 变体还带有关联的守卫，可以通过 [`into_inner`] 方法获取它。<br>
///
///
/// [`into_inner`]: PoisonError::into_inner
///
#[stable(feature = "rust1", since = "1.0.0")]
pub type LockResult<Guard> = Result<Guard, PoisonError<Guard>>;

/// A type alias for the result of a nonblocking locking method. <br>一种非别名锁定方法结果的类型别名。<br>
///
/// For more information, see [`LockResult`]. <br>有关更多信息，请参见 [`LockResult`]。<br>
/// A `TryLockResult` doesn't necessarily hold the associated guard in the [`Err`] type as the lock might not have been acquired for other reasons. <br>`TryLockResult` 不一定持有 [`Err`] 类型的相关保护，因为可能由于其他原因没有获得锁。<br>
///
#[stable(feature = "rust1", since = "1.0.0")]
pub type TryLockResult<Guard> = Result<Guard, TryLockError<Guard>>;

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> fmt::Debug for PoisonError<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("PoisonError").finish_non_exhaustive()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> fmt::Display for PoisonError<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        "poisoned lock: another task failed inside".fmt(f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Error for PoisonError<T> {
    #[allow(deprecated)]
    fn description(&self) -> &str {
        "poisoned lock: another task failed inside"
    }
}

impl<T> PoisonError<T> {
    /// Creates a `PoisonError`. <br>创建一个 `PoisonError`。<br>
    ///
    /// This is generally created by methods like [`Mutex::lock`](crate::sync::Mutex::lock) or [`RwLock::read`](crate::sync::RwLock::read). <br>这通常是由 [`Mutex::lock`](crate::sync::Mutex::lock) 或 [`RwLock::read`](crate::sync::RwLock::read) 之类的方法创建的。<br>
    ///
    #[stable(feature = "sync_poison", since = "1.2.0")]
    pub fn new(guard: T) -> PoisonError<T> {
        PoisonError { guard }
    }

    /// Consumes this error indicating that a lock is poisoned, returning the underlying guard to allow access regardless. <br>消耗这个错误指示锁已中毒，无论如何都将返回底层防护以允许访问。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::collections::HashSet;
    /// use std::sync::{Arc, Mutex};
    /// use std::thread;
    ///
    /// let mutex = Arc::new(Mutex::new(HashSet::new()));
    ///
    /// // poison the mutex <br>互斥锁中毒<br>
    /// let c_mutex = Arc::clone(&mutex);
    /// let _ = thread::spawn(move || {
    ///     let mut data = c_mutex.lock().unwrap();
    ///     data.insert(10);
    ///     panic!();
    /// }).join();
    ///
    /// let p_err = mutex.lock().unwrap_err();
    /// let data = p_err.into_inner();
    /// println!("recovered {} items", data.len());
    /// ```
    #[stable(feature = "sync_poison", since = "1.2.0")]
    pub fn into_inner(self) -> T {
        self.guard
    }

    /// Reaches into this error indicating that a lock is poisoned, returning a reference to the underlying guard to allow access regardless. <br>到达此错误时就表示锁已中毒，返回对底层保护的引用以允许访问。<br>
    ///
    #[stable(feature = "sync_poison", since = "1.2.0")]
    pub fn get_ref(&self) -> &T {
        &self.guard
    }

    /// Reaches into this error indicating that a lock is poisoned, returning a mutable reference to the underlying guard to allow access regardless. <br>到达此错误时就表示锁已中毒，返回对底层保护的可变引用以允许访问。<br>
    ///
    #[stable(feature = "sync_poison", since = "1.2.0")]
    pub fn get_mut(&mut self) -> &mut T {
        &mut self.guard
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> From<PoisonError<T>> for TryLockError<T> {
    fn from(err: PoisonError<T>) -> TryLockError<T> {
        TryLockError::Poisoned(err)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> fmt::Debug for TryLockError<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match *self {
            TryLockError::Poisoned(..) => "Poisoned(..)".fmt(f),
            TryLockError::WouldBlock => "WouldBlock".fmt(f),
        }
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> fmt::Display for TryLockError<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match *self {
            TryLockError::Poisoned(..) => "poisoned lock: another task failed inside",
            TryLockError::WouldBlock => "try_lock failed because the operation would block",
        }
        .fmt(f)
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl<T> Error for TryLockError<T> {
    #[allow(deprecated, deprecated_in_future)]
    fn description(&self) -> &str {
        match *self {
            TryLockError::Poisoned(ref p) => p.description(),
            TryLockError::WouldBlock => "try_lock failed because the operation would block",
        }
    }

    #[allow(deprecated)]
    fn cause(&self) -> Option<&dyn Error> {
        match *self {
            TryLockError::Poisoned(ref p) => Some(p),
            _ => None,
        }
    }
}

pub fn map_result<T, U, F>(result: LockResult<T>, f: F) -> LockResult<U>
where
    F: FnOnce(T) -> U,
{
    match result {
        Ok(t) => Ok(f(t)),
        Err(PoisonError { guard }) => Err(PoisonError::new(f(guard))),
    }
}
