//! System bindings for HermitCore <br>HermitCore 的系统绑定<br>
//!
//! This module contains the facade (aka platform-specific) implementations of OS level functionality for HermitCore. <br>该模块包含 HermitCore 的 OS 级功能的外观 (又名平台特定) 实现。<br>
//!
//! This is all super highly experimental and not actually intended for wide/production use yet, it's still all in the experimental category. <br>这都是超级实验性的，实际上还不适合 wide/production 使用，它仍然属于实验类别。<br>
//! This will likely change over time. <br>随着时间的流逝，这可能会改变。<br>
//!
//! Currently all functions here are basically stubs that immediately return errors. <br>当前，这里的所有函数基本上都是立即返回错误的存根。<br>
//! The hope is that with a portability lint we can turn actually just remove all this and just omit parts of the standard library if we're compiling for wasm. <br>希望是，有了可移植性 lint，如果我们要为 wasm 进行编译，我们实际上可以删除所有这些内容并忽略标准库的某些部分。<br>
//!
//! That way it's a compile time error for something that's guaranteed to be a runtime error! <br>这样一来，就可以保证它是运行时错误，这是一个编译时错误！<br>
//!
//!
//!

#![allow(unsafe_op_in_unsafe_fn)]

use crate::intrinsics;
use crate::os::raw::c_char;

pub mod alloc;
pub mod args;
#[path = "../unix/cmath.rs"]
pub mod cmath;
pub mod env;
pub mod fd;
pub mod fs;
#[path = "../unsupported/io.rs"]
pub mod io;
pub mod memchr;
pub mod net;
pub mod os;
#[path = "../unix/os_str.rs"]
pub mod os_str;
#[path = "../unix/path.rs"]
pub mod path;
#[path = "../unsupported/pipe.rs"]
pub mod pipe;
#[path = "../unsupported/process.rs"]
pub mod process;
pub mod stdio;
pub mod thread;
pub mod thread_local_dtor;
#[path = "../unsupported/thread_local_key.rs"]
pub mod thread_local_key;
pub mod time;

mod condvar;
mod mutex;
mod rwlock;

pub mod locks {
    pub use super::condvar::*;
    pub use super::mutex::*;
    pub use super::rwlock::*;
}

use crate::io::ErrorKind;

#[allow(unused_extern_crates)]
pub extern crate hermit_abi as abi;

pub fn unsupported<T>() -> crate::io::Result<T> {
    Err(unsupported_err())
}

pub fn unsupported_err() -> crate::io::Error {
    crate::io::const_io_error!(
        crate::io::ErrorKind::Unsupported,
        "operation not supported on HermitCore yet",
    )
}

#[no_mangle]
pub extern "C" fn floor(x: f64) -> f64 {
    unsafe { intrinsics::floorf64(x) }
}

pub fn abort_internal() -> ! {
    unsafe {
        abi::abort();
    }
}

// FIXME: just a workaround to test the system <br>只是测试系统的一种解决方法<br>
pub fn hashmap_random_keys() -> (u64, u64) {
    (1, 2)
}

// This function is needed by the panic runtime. <br>panic 运行时需要这个函数。<br>
// The symbol is named in pre-link args for the target specification, so keep that in sync. <br>该符号在目标规范的预链接 args 中命名，因此请使其保持同步。<br>
#[cfg(not(test))]
#[no_mangle]
// NB. used by both libunwind and libpanic_abort <br>libunwind 和 libpanic_abort 都会使用<br>
pub extern "C" fn __rust_abort() {
    abort_internal();
}

// SAFETY: must be called only once during runtime initialization. <br>在运行时初始化期间只能调用一次。<br>
// NOTE: this is not guaranteed to run, for example when Rust code is called externally. <br>这不能保证运行，例如在外部调用 Rust 代码时。<br>
pub unsafe fn init(argc: isize, argv: *const *const u8) {
    let _ = net::init();
    args::init(argc, argv);
}

// SAFETY: must be called only once during runtime cleanup. <br>在运行时清理期间只能调用一次。<br>
// NOTE: this is not guaranteed to run, for example when the program aborts. <br>这不能保证运行，例如当程序中止时。<br>
pub unsafe fn cleanup() {
    args::cleanup();
}

#[cfg(not(test))]
#[no_mangle]
pub unsafe extern "C" fn runtime_entry(
    argc: i32,
    argv: *const *const c_char,
    env: *const *const c_char,
) -> ! {
    use crate::sys::hermit::thread_local_dtor::run_dtors;
    extern "C" {
        fn main(argc: isize, argv: *const *const c_char) -> i32;
    }

    // initialize environment <br>初始化环境<br>
    os::init_environment(env as *const *const i8);

    let result = main(argc as isize, argv);

    run_dtors();
    abi::exit(result);
}

pub fn decode_error_kind(errno: i32) -> ErrorKind {
    match errno {
        x if x == 13 as i32 => ErrorKind::PermissionDenied,
        x if x == 98 as i32 => ErrorKind::AddrInUse,
        x if x == 99 as i32 => ErrorKind::AddrNotAvailable,
        x if x == 11 as i32 => ErrorKind::WouldBlock,
        x if x == 103 as i32 => ErrorKind::ConnectionAborted,
        x if x == 111 as i32 => ErrorKind::ConnectionRefused,
        x if x == 104 as i32 => ErrorKind::ConnectionReset,
        x if x == 17 as i32 => ErrorKind::AlreadyExists,
        x if x == 4 as i32 => ErrorKind::Interrupted,
        x if x == 22 as i32 => ErrorKind::InvalidInput,
        x if x == 2 as i32 => ErrorKind::NotFound,
        x if x == 107 as i32 => ErrorKind::NotConnected,
        x if x == 1 as i32 => ErrorKind::PermissionDenied,
        x if x == 32 as i32 => ErrorKind::BrokenPipe,
        x if x == 110 as i32 => ErrorKind::TimedOut,
        _ => ErrorKind::Uncategorized,
    }
}

pub fn cvt(result: i32) -> crate::io::Result<usize> {
    if result < 0 { Err(crate::io::Error::from_raw_os_error(-result)) } else { Ok(result as usize) }
}
