//! POSIX conditional variable implementation based on user-space wait queues. <br>基于用户空间等待队列的 POSIX 条件变量实现。<br>
use super::{abi, error::expect_success_aborting, spin::SpinMutex, task, time::with_tmos_strong};
use crate::{mem::replace, ptr::NonNull, sys::locks::Mutex, time::Duration};

// The implementation is inspired by the queue-based implementation shown in Andrew D. <br>该实现受到了 Andrew D. 中所示的基于队列的实现的启发<br>
// Birrell's paper "Implementing Condition Variables with Semaphores" <br>Birrell 的论文 "用信号量实现条件变量"<br>

pub struct Condvar {
    waiters: SpinMutex<waiter_queue::WaiterQueue>,
}

unsafe impl Send for Condvar {}
unsafe impl Sync for Condvar {}

pub type MovableCondvar = Condvar;

impl Condvar {
    #[inline]
    pub const fn new() -> Condvar {
        Condvar { waiters: SpinMutex::new(waiter_queue::WaiterQueue::new()) }
    }

    #[inline]
    pub unsafe fn init(&mut self) {}

    pub unsafe fn notify_one(&self) {
        self.waiters.with_locked(|waiters| {
            if let Some(task) = waiters.pop_front() {
                // Unpark the task <br>Unpark 任务<br>
                match unsafe { abi::wup_tsk(task) } {
                    // The task already has a token. <br>任务已经有一个 token。<br>
                    abi::E_QOVR => {}
                    // Can't undo the effect; <br>无法撤消效果；<br> abort the program on failure <br>失败时中止程序<br>
                    er => {
                        expect_success_aborting(er, &"wup_tsk");
                    }
                }
            }
        });
    }

    pub unsafe fn notify_all(&self) {
        self.waiters.with_locked(|waiters| {
            while let Some(task) = waiters.pop_front() {
                // Unpark the task <br>Unpark 任务<br>
                match unsafe { abi::wup_tsk(task) } {
                    // The task already has a token. <br>任务已经有一个 token。<br>
                    abi::E_QOVR => {}
                    // Can't undo the effect; <br>无法撤消效果；<br> abort the program on failure <br>失败时中止程序<br>
                    er => {
                        expect_success_aborting(er, &"wup_tsk");
                    }
                }
            }
        });
    }

    pub unsafe fn wait(&self, mutex: &Mutex) {
        // Construct `Waiter`. <br>构建 `Waiter`。<br>
        let mut waiter = waiter_queue::Waiter::new();
        let waiter = NonNull::from(&mut waiter);

        self.waiters.with_locked(|waiters| unsafe {
            waiters.insert(waiter);
        });

        unsafe { mutex.unlock() };

        // Wait until `waiter` is removed from the queue <br>等待 `waiter` 从队列中移除<br>
        loop {
            // Park the current task <br>Park 当前任务<br>
            expect_success_aborting(unsafe { abi::slp_tsk() }, &"slp_tsk");

            if !self.waiters.with_locked(|waiters| unsafe { waiters.is_queued(waiter) }) {
                break;
            }
        }

        unsafe { mutex.lock() };
    }

    pub unsafe fn wait_timeout(&self, mutex: &Mutex, dur: Duration) -> bool {
        // Construct and pin `Waiter` <br>构建和固定 `Waiter`<br>
        let mut waiter = waiter_queue::Waiter::new();
        let waiter = NonNull::from(&mut waiter);

        self.waiters.with_locked(|waiters| unsafe {
            waiters.insert(waiter);
        });

        unsafe { mutex.unlock() };

        // Park the current task and do not wake up until the timeout elapses or the task gets woken up by `notify_*` <br>Park 当前任务，直到超时或任务被 `notify_*` 唤醒后才唤醒<br>
        //
        match with_tmos_strong(dur, |tmo| {
            let er = unsafe { abi::tslp_tsk(tmo) };
            if er == 0 {
                // We were unparked. <br>我们没有 unpark。<br> Are we really dequeued? <br>我们真的出队了吗？<br>
                if self.waiters.with_locked(|waiters| unsafe { waiters.is_queued(waiter) }) {
                    // No we are not. <br>不，我们没有。<br> Continue waiting. <br>继续等待。<br>
                    return abi::E_TMOUT;
                }
            }
            er
        }) {
            abi::E_TMOUT => {}
            er => {
                expect_success_aborting(er, &"tslp_tsk");
            }
        }

        // Remove `waiter` from `self.waiters`. <br>从 `self.waiters` 上移除 `waiter`。<br> If `waiter` is still in `waiters`, it means we woke up because of a timeout. <br>如果 `waiter` 还在 `waiters` 中，则表示我们因为超时而醒来。<br>
        //
        // Otherwise, we woke up because of `notify_*`. <br>否则，我们会因为 `notify_*` 而醒来。<br>
        let success = self.waiters.with_locked(|waiters| unsafe { !waiters.remove(waiter) });

        unsafe { mutex.lock() };
        success
    }
}

mod waiter_queue {
    use super::*;

    pub struct WaiterQueue {
        head: Option<ListHead>,
    }

    #[derive(Copy, Clone)]
    struct ListHead {
        first: NonNull<Waiter>,
        last: NonNull<Waiter>,
    }

    unsafe impl Send for ListHead {}
    unsafe impl Sync for ListHead {}

    pub struct Waiter {
        // These fields are only accessed through `&[mut] WaiterQueue`. <br>这些字段只能通过 `&[mut] WaiterQueue` 访问。<br>
        /// The waiting task's ID. <br>等待任务的 ID。<br>
        /// Will be zeroed when the task is woken up and removed from a queue. <br>当任务被唤醒并从队列中移除时将被清零。<br>
        task: abi::ID,
        priority: abi::PRI,
        prev: Option<NonNull<Waiter>>,
        next: Option<NonNull<Waiter>>,
    }

    unsafe impl Send for Waiter {}
    unsafe impl Sync for Waiter {}

    impl Waiter {
        #[inline]
        pub fn new() -> Self {
            let task = task::current_task_id();
            let priority = task::task_priority(abi::TSK_SELF);

            // Zeroness of `Waiter::task` indicates whether the `Waiter` is linked to a queue or not. <br>`Waiter::task` 为零表示 `Waiter` 是否链接到队列。<br>
            // This invariant is important for the correctness. <br>这个不变量对于正确性很重要。<br>
            //
            debug_assert_ne!(task, 0);

            Self { task, priority, prev: None, next: None }
        }
    }

    impl WaiterQueue {
        #[inline]
        pub const fn new() -> Self {
            Self { head: None }
        }

        /// # Safety
        ///
        ///  - The caller must own `*waiter_ptr`. <br>调用者必须拥有 `*waiter_ptr`。<br>
        ///  The caller will lose the ownership until `*waiter_ptr` is removed from `self`. <br>调用者将失去所有权，直到 `*waiter_ptr` 从 `self` 中删除。<br>
        ///
        ///
        ///  - `*waiter_ptr` must be valid until it's removed from the queue. <br>`*waiter_ptr` 在从队列中移除之前必须有效。<br>
        ///
        ///  - `*waiter_ptr` must not have been previously inserted to a `WaiterQueue`. <br>`*waiter_ptr` 之前不能插入到 `WaiterQueue` 中。<br>
        pub unsafe fn insert(&mut self, mut waiter_ptr: NonNull<Waiter>) {
            unsafe {
                let waiter = waiter_ptr.as_mut();

                debug_assert!(waiter.prev.is_none());
                debug_assert!(waiter.next.is_none());

                if let Some(head) = &mut self.head {
                    // Find the insertion position and insert `waiter` <br>找到插入位置并插入 `waiter`<br>
                    let insert_after = {
                        let mut cursor = head.last;
                        loop {
                            if waiter.priority >= cursor.as_ref().priority {
                                // `cursor` and all previous waiters have the same or higher priority than `current_task_priority`. <br>`cursor` 和所有以前的等待者的优先级与 `current_task_priority` 相同或更高。<br>
                                // Insert the new waiter right after `cursor`. <br>在 `cursor` 之后立即插入新的 waiter。<br>
                                //
                                break Some(cursor);
                            }
                            cursor = if let Some(prev) = cursor.as_ref().prev {
                                prev
                            } else {
                                break None;
                            };
                        }
                    };

                    if let Some(mut insert_after) = insert_after {
                        // Insert `waiter` after `insert_after` <br>在 `insert_after` 之后插入 `waiter`<br>
                        let insert_before = insert_after.as_ref().next;

                        waiter.prev = Some(insert_after);
                        insert_after.as_mut().next = Some(waiter_ptr);

                        waiter.next = insert_before;
                        if let Some(mut insert_before) = insert_before {
                            insert_before.as_mut().prev = Some(waiter_ptr);
                        } else {
                            head.last = waiter_ptr;
                        }
                    } else {
                        // Insert `waiter` to the front <br>在前面插入 `waiter`<br>
                        waiter.next = Some(head.first);
                        head.first.as_mut().prev = Some(waiter_ptr);
                        head.first = waiter_ptr;
                    }
                } else {
                    // `waiter` is the only element <br>`waiter` 是唯一的元素<br>
                    self.head = Some(ListHead { first: waiter_ptr, last: waiter_ptr });
                }
            }
        }

        /// Given a `Waiter` that was previously inserted to `self`, remove it from `self` if it's still there. <br>给定一个之前插入到 `self` 的 `Waiter`，如果它仍然存在，请将其从 `self` 中删除。<br>
        ///
        #[inline]
        pub unsafe fn remove(&mut self, mut waiter_ptr: NonNull<Waiter>) -> bool {
            unsafe {
                let waiter = waiter_ptr.as_mut();
                if waiter.task != 0 {
                    let head = self.head.as_mut().unwrap();

                    match (waiter.prev, waiter.next) {
                        (Some(mut prev), Some(mut next)) => {
                            prev.as_mut().next = Some(next);
                            next.as_mut().prev = Some(prev);
                        }
                        (None, Some(mut next)) => {
                            head.first = next;
                            next.as_mut().prev = None;
                        }
                        (Some(mut prev), None) => {
                            prev.as_mut().next = None;
                            head.last = prev;
                        }
                        (None, None) => {
                            self.head = None;
                        }
                    }

                    waiter.task = 0;

                    true
                } else {
                    false
                }
            }
        }

        /// Given a `Waiter` that was previously inserted to `self`, return a flag indicating whether it's still in `self`. <br>给定一个先前插入到 `self` 的 `Waiter`，返回一个标志，指示它是否仍在 `self` 中。<br>
        ///
        #[inline]
        pub unsafe fn is_queued(&self, waiter: NonNull<Waiter>) -> bool {
            unsafe { waiter.as_ref().task != 0 }
        }

        #[inline]
        pub fn pop_front(&mut self) -> Option<abi::ID> {
            unsafe {
                let head = self.head.as_mut()?;
                let waiter = head.first.as_mut();

                // Get the ID <br>获取 ID<br>
                let id = replace(&mut waiter.task, 0);

                // Unlink the waiter <br>断开 waiter 的连接<br>
                if let Some(mut next) = waiter.next {
                    head.first = next;
                    next.as_mut().prev = None;
                } else {
                    self.head = None;
                }

                Some(id)
            }
        }
    }
}
