//! 由 μITRON 任务支持的线程实现。
//! 假设 `acre_tsk` 和 `exd_tsk` 可用。
use super::{
    abi,
    error::{expect_success, expect_success_aborting, ItronError},
    task,
    time::dur2reltims,
};
use crate::{
    cell::UnsafeCell,
    ffi::CStr,
    hint, io,
    mem::ManuallyDrop,
    sync::atomic::{AtomicUsize, Ordering},
    sys::thread_local_dtor::run_dtors,
    time::Duration,
};

pub struct Thread {
    inner: ManuallyDrop<Box<ThreadInner>>,

    /// 底层任务的 ID。
    task: abi::ID,
}

/// 父线程和子线程之间共享的状态数据。
/// 它是在过渡到最终状态之一时被丢弃的。
struct ThreadInner {
    /// 该字段用于线程创建以将来自 `Thread::new` 的闭包传递给创建的任务。
    ///
    start: UnsafeCell<ManuallyDrop<Box<dyn FnOnce()>>>,

    /// 状态机。每个转换都在源代码中用 `[...]` 注解。
    ///
    /// ```text
    ///
    ///    <P>: parent, <C>: child, (?): don't-care
    ///
    ///       DETACHED (-1)  -------------------->  EXITED (?)
    ///                        <C>finish/exd_tsk
    ///          ^
    ///          |
    ///          | <P>detach
    ///          |
    ///
    ///       INIT (0)  ----------------------->  FINISHED (-1)
    ///                        <C>finish
    ///          |                                    |
    ///          | <P>join/slp_tsk                    | <P>join/del_tsk
    ///          |                                    | <P>detach/del_tsk v                                    v
    ///
    ///       JOINING                              JOINED (?)
    ///     (parent_tid)
    ///                                            ^
    ///             \                             /
    ///              \  <C>finish/wup_tsk        / <P>slp_tsk-complete/ter_tsk \                         /                      & del_tsk
    ///
    ///                \                       /
    ///                 '--> JOIN_FINALIZE ---'
    ///                          (-1)
    ///
    ///
    ///
    lifecycle: AtomicUsize,
}

// 安全性：唯一的 `!Sync` 字段，`ThreadInner::start`，仅被 `ThreadInner` 所代表的任务触及。
//
unsafe impl Sync for ThreadInner {}

const LIFECYCLE_INIT: usize = 0;
const LIFECYCLE_FINISHED: usize = usize::MAX;
const LIFECYCLE_DETACHED: usize = usize::MAX;
const LIFECYCLE_JOIN_FINALIZE: usize = usize::MAX;
const LIFECYCLE_DETACHED_OR_JOINED: usize = usize::MAX;
const LIFECYCLE_EXITED_OR_FINISHED_OR_JOIN_FINALIZE: usize = usize::MAX;
// `JOINING` 没有单一的值

// 64KiB 用于 32 位 ISA，128KiB 用于 64 位 ISA。
pub const DEFAULT_MIN_STACK_SIZE: usize = 0x4000 * crate::mem::size_of::<usize>();

impl Thread {
    /// # Safety
    ///
    /// 有关安全要求，请参见 `thread::Builder::spawn_unchecked`。
    pub unsafe fn new(stack: usize, p: Box<dyn FnOnce()>) -> io::Result<Thread> {
        let inner = Box::new(ThreadInner {
            start: UnsafeCell::new(ManuallyDrop::new(p)),
            lifecycle: AtomicUsize::new(LIFECYCLE_INIT),
        });

        unsafe extern "C" fn trampoline(exinf: isize) {
            // 安全性：此时 `ThreadInner` 此时处于活动状态
            let inner = unsafe { &*(exinf as *const ThreadInner) };

            // 安全性：由于每个 `ThreadInner` 只调用一次 `trampoline`，并且只有 `trampoline` 接触 `start`，因此 `start` 包含内容，可以安全地可变借用。
            //
            //
            let p = unsafe { ManuallyDrop::take(&mut *inner.start.get()) };
            p();

            // 修复当前线程的状态以防万一，以便析构函数不会终止安全：并不是真的不安全
            //
            //
            let _ = unsafe { abi::unl_cpu() };
            let _ = unsafe { abi::ena_dsp() };

            // 现在运行 TLS 析构函数，因为终止的任务不会自动调用它们。
            //
            unsafe { run_dtors() };

            let old_lifecycle = inner
                .lifecycle
                .swap(LIFECYCLE_EXITED_OR_FINISHED_OR_JOIN_FINALIZE, Ordering::Release);

            match old_lifecycle {
                LIFECYCLE_DETACHED => {
                    // [DETACHED → EXITED]
                    // 没有人会加入，因此我们将要求收集器任务删除该任务。
                    //

                    // 在这种情况下，`inner` 的所有权已经转移给我们，我们负责丢弃它。
                    // 获取顺序不是必需的，因为从调用 `acre_tsk` 以来，父线程没有进行需要同步的内存访问。
                    //
                    // 安全：见上文。
                    //
                    //
                    let _ = unsafe { Box::from_raw(inner as *const _ as *mut ThreadInner) };

                    // 安全性：没有对栈的固定引用
                    unsafe { terminate_and_delete_current_task() };
                }
                LIFECYCLE_INIT => {
                    // [INIT → FINISHED]
                    // 父任务还没有决定是加入还是分离这个线程。
                    // 无论父任务选择哪个选项，它都必须删除此任务。
                    // 由于父任务可能会在看到 `FINISHED` 时立即丢弃 `*inner`，因此必须在上述 `swap` 调用中使用发布顺序。
                    //
                    //
                    //
                }
                parent_tid => {
                    // 由于父任务可能会丢弃 `*inner` 并在它看到 `JOIN_FINALIZE` 后立即终止我们，因此必须在上述 `swap` 调用中使用发布顺序。
                    //
                    //

                    // [JOINING → JOIN_FINALIZE]
                    // 唤醒父任务。
                    expect_success(
                        unsafe {
                            let mut er = abi::wup_tsk(parent_tid as _);
                            if er == abi::E_QOVR {
                                // `E_QOVR` 表示已经有一个 parking token
                                //
                                er = abi::E_OK;
                            }
                            er
                        },
                        &"wup_tsk",
                    );
                }
            }
        }

        let inner_ptr = (&*inner) as *const ThreadInner;

        let new_task = ItronError::err_if_negative(unsafe {
            abi::acre_tsk(&abi::T_CTSK {
                // 立即激活此任务
                tskatr: abi::TA_ACT,
                exinf: inner_ptr as abi::EXINF,
                // 入口点
                task: Some(trampoline),
                // 继承调用任务的基本优先级
                itskpri: abi::TPRI_SELF,
                stksz: stack,
                // 让内核分配栈，
                stk: crate::ptr::null_mut(),
            })
        })
        .map_err(|e| e.as_io_error())?;

        Ok(Self { inner: ManuallyDrop::new(inner), task: new_task })
    }

    pub fn yield_now() {
        expect_success(unsafe { abi::rot_rdq(abi::TPRI_SELF) }, &"rot_rdq");
    }

    pub fn set_name(_name: &CStr) {
        // nope
    }

    pub fn sleep(dur: Duration) {
        for timeout in dur2reltims(dur) {
            expect_success(unsafe { abi::dly_tsk(timeout) }, &"dly_tsk");
        }
    }

    pub fn join(mut self) {
        let inner = &*self.inner;
        // 获取当前任务的 ID。
        // 在这里 panic 会导致资源泄漏，所以只要在失败时中止即可。
        let current_task = task::current_task_id_aborting();
        debug_assert!(usize::try_from(current_task).is_ok());
        debug_assert_ne!(current_task as usize, LIFECYCLE_INIT);
        debug_assert_ne!(current_task as usize, LIFECYCLE_DETACHED);

        let current_task = current_task as usize;

        match inner.lifecycle.swap(current_task, Ordering::Acquire) {
            LIFECYCLE_INIT => {
                // [INIT → JOINING]
                // 子任务将状态转换为 `JOIN_FINALIZE` 并唤醒我们。
                //
                loop {
                    expect_success_aborting(unsafe { abi::slp_tsk() }, &"slp_tsk");
                    // 为了与子任务对 `inner` 的内存访问同步到 `JOIN_FINALIZE` 的分配点，`Ordering::Acquire` 必须用于 `load`。
                    //
                    //
                    //
                    if inner.lifecycle.load(Ordering::Acquire) == LIFECYCLE_JOIN_FINALIZE {
                        break;
                    }
                }

                // [JOIN_FINALIZE → JOINED]
            }
            LIFECYCLE_FINISHED => {
                // [FINISHED → JOINED]
                // 为了与子任务对 `inner` 的内存访问同步到 `FINISHED` 的分配点，`Ordering::Acquire` 必须用于上述 `swap` 调用 `。
                //
                //
            }
            _ => unsafe { hint::unreachable_unchecked() },
        }

        // 终止并删除任务 Safety: `self.task` 仍然代表我们拥有所有权的任务 (因为这个方法或 `detach_inner` 对于每个 `Thread` 只调用一次)。
        // 该任务表明可以通过进入 `FINISHED` 或 `JOIN_FINALIZE` 状态进行安全删除。
        //
        //
        //
        unsafe { terminate_and_delete_task(self.task) };

        // 在任何一种情况下，我们都应对丢弃 `inner` 负责。
        // 安全: `self.inner` 的内容以后将不会被访问
        let _inner = unsafe { ManuallyDrop::take(&mut self.inner) };

        // 跳过析构函数 (因为它会尝试分离线程)
        crate::mem::forget(self);
    }
}

impl Drop for Thread {
    fn drop(&mut self) {
        // 分离线程。
        match self.inner.lifecycle.swap(LIFECYCLE_DETACHED_OR_JOINED, Ordering::Acquire) {
            LIFECYCLE_INIT => {
                // [INIT → DETACHED]
                // 当时机成熟时，子节点会发现没有人会加入。
                // `self.inner` 的所有权转移到子线程。
                // 但是，发布顺序不是必需的，因为自从调用 `acre_tsk` 后，我们没有进行需要同步的内存访问。
                //
                //
                //
            }
            LIFECYCLE_FINISHED => {
                // [FINISHED → JOINED]
                // 任务已经决定我们应该删除任务。
                // 为了与子任务对 `inner` 的内存访问同步到 `FINISHED` 的分配点，上述 `swap` 调用需要获取排序。
                //
                //

                // 终止并删除任务安全: `self.task` 仍然代表我们拥有的任务 (因为这个方法或 `join_inner` 对于每个 `Thread` 只调用一次)。
                // 任务表明可以通过进入 `FINISHED` 状态进行安全删除。
                //
                //
                //
                unsafe { terminate_and_delete_task(self.task) };

                // Wwe 负责丢弃 `inner`。
                // 安全: `self.inner` 的内容以后将不会被访问
                //
                unsafe { ManuallyDrop::drop(&mut self.inner) };
            }
            _ => unsafe { hint::unreachable_unchecked() },
        }
    }
}

pub mod guard {
    pub type Guard = !;
    pub unsafe fn current() -> Option<Guard> {
        None
    }
    pub unsafe fn init() -> Option<Guard> {
        None
    }
}

/// 终止并删除指定的任务。
///
/// 如果 `deleted_task` 指的是调用任务，则这个函数将终止。
///
/// 假设指定的任务是由调用者单独管理的 -- 即，在此函数仍在进行中时，其他线程不得对指定的任务进行 "resuscitate" 或过早删除。
/// 允许指定的任务自行退出。
///
/// # Safety
///
/// 任务必须可以安全终止。这通常是不正确的，因为可能存在对任务栈的固定引用。
///
///
///
unsafe fn terminate_and_delete_task(deleted_task: abi::ID) {
    // 终止任务安全：由调用方支持
    //
    match unsafe { abi::ter_tsk(deleted_task) } {
        // 表示任务已经休眠，请忽略它
        abi::E_OBJ => {}
        er => {
            expect_success_aborting(er, &"ter_tsk");
        }
    }

    // 删除任务安全：调用方支持
    //
    expect_success_aborting(unsafe { abi::del_tsk(deleted_task) }, &"del_tsk");
}

/// 终止并删除调用任务。
///
/// 原子性不是必需的 - 即，可以假设其他线程不会在此函数仍在进行中时 `ter_tsk` 调用任务。
/// (这个属性使得在不支持 `exd_tsk` 的 μITRON 派生内核上实现这个操作变得很容易。)
///
/// # Safety
///
/// 任务必须可以安全终止。这通常是不正确的，因为可能存在对任务栈的固定引用。
///
///
///
unsafe fn terminate_and_delete_current_task() -> ! {
    expect_success_aborting(unsafe { abi::exd_tsk() }, &"exd_tsk");
    // 安全性: `exd_tsk` 永远不会成功返回
    unsafe { crate::hint::unreachable_unchecked() };
}

pub fn available_parallelism() -> io::Result<crate::num::NonZeroUsize> {
    super::unsupported()
}
