//! Thread implementation backed by μITRON tasks. <br>由 μITRON 任务支持的线程实现。<br>
//! Assumes `acre_tsk` and `exd_tsk` are available. <br>假设 `acre_tsk` 和 `exd_tsk` 可用。<br>
use super::{
    abi,
    error::{expect_success, expect_success_aborting, ItronError},
    task,
    time::dur2reltims,
};
use crate::{
    cell::UnsafeCell,
    ffi::CStr,
    hint, io,
    mem::ManuallyDrop,
    sync::atomic::{AtomicUsize, Ordering},
    sys::thread_local_dtor::run_dtors,
    time::Duration,
};

pub struct Thread {
    inner: ManuallyDrop<Box<ThreadInner>>,

    /// The ID of the underlying task. <br>底层任务的 ID。<br>
    task: abi::ID,
}

/// State data shared between a parent thread and child thread. <br>父线程和子线程之间共享的状态数据。<br>
/// It's dropped on a transition to one of the final states. <br>它是在过渡到最终状态之一时被丢弃的。<br>
struct ThreadInner {
    /// This field is used on thread creation to pass a closure from `Thread::new` to the created task. <br>该字段用于线程创建以将来自 `Thread::new` 的闭包传递给创建的任务。<br>
    ///
    start: UnsafeCell<ManuallyDrop<Box<dyn FnOnce()>>>,

    /// A state machine. <br>状态机。<br> Each transition is annotated with `[...]` in the source code. <br>每个转换都在源代码中用 `[...]` 注解。<br>
    ///
    /// ```text
    ///
    ///    <P>: parent, <C>: child, (?): don't-care
    ///
    ///       DETACHED (-1)  -------------------->  EXITED (?)
    ///                        <C>finish/exd_tsk
    ///          ^
    ///          |
    ///          | <P>detach
    ///          |
    ///
    ///       INIT (0)  ----------------------->  FINISHED (-1)
    ///                        <C>finish
    ///          |                                    |
    ///          | <P>join/slp_tsk                    | <P>join/del_tsk
    ///          |                                    | <P>detach/del_tsk v                                    v
    ///
    ///       JOINING                              JOINED (?)
    ///     (parent_tid)
    ///                                            ^
    ///             \                             /
    ///              \  <C>finish/wup_tsk        / <P>slp_tsk-complete/ter_tsk \                         /                      & del_tsk
    ///
    ///                \                       /
    ///                 '--> JOIN_FINALIZE ---'
    ///                          (-1)
    ///
    ///
    ///
    lifecycle: AtomicUsize,
}

// Safety: The only `!Sync` field, `ThreadInner::start`, is only touched by the task represented by `ThreadInner`. <br>安全性：唯一的 `!Sync` 字段，`ThreadInner::start`，仅被 `ThreadInner` 所代表的任务触及。<br>
//
unsafe impl Sync for ThreadInner {}

const LIFECYCLE_INIT: usize = 0;
const LIFECYCLE_FINISHED: usize = usize::MAX;
const LIFECYCLE_DETACHED: usize = usize::MAX;
const LIFECYCLE_JOIN_FINALIZE: usize = usize::MAX;
const LIFECYCLE_DETACHED_OR_JOINED: usize = usize::MAX;
const LIFECYCLE_EXITED_OR_FINISHED_OR_JOIN_FINALIZE: usize = usize::MAX;
// there's no single value for `JOINING` <br>`JOINING` 没有单一的值<br>

// 64KiB for 32-bit ISAs, 128KiB for 64-bit ISAs. <br>64KiB 用于 32 位 ISA，128KiB 用于 64 位 ISA。<br>
pub const DEFAULT_MIN_STACK_SIZE: usize = 0x4000 * crate::mem::size_of::<usize>();

impl Thread {
    /// # Safety
    ///
    /// See `thread::Builder::spawn_unchecked` for safety requirements. <br>有关安全要求，请参见 `thread::Builder::spawn_unchecked`。<br>
    pub unsafe fn new(stack: usize, p: Box<dyn FnOnce()>) -> io::Result<Thread> {
        let inner = Box::new(ThreadInner {
            start: UnsafeCell::new(ManuallyDrop::new(p)),
            lifecycle: AtomicUsize::new(LIFECYCLE_INIT),
        });

        unsafe extern "C" fn trampoline(exinf: isize) {
            // Safety: `ThreadInner` is alive at this point <br>安全性：此时 `ThreadInner` 此时处于活动状态<br>
            let inner = unsafe { &*(exinf as *const ThreadInner) };

            // Safety: Since `trampoline` is called only once for each `ThreadInner` and only `trampoline` touches `start`, `start` contains contents and is safe to mutably borrow. <br>安全性：由于每个 `ThreadInner` 只调用一次 `trampoline`，并且只有 `trampoline` 接触 `start`，因此 `start` 包含内容，可以安全地可变借用。<br>
            //
            //
            let p = unsafe { ManuallyDrop::take(&mut *inner.start.get()) };
            p();

            // Fix the current thread's state just in case, so that the destructors won't abort Safety: Not really unsafe <br>修复当前线程的状态以防万一，以便析构函数不会终止安全：并不是真的不安全<br>
            //
            //
            let _ = unsafe { abi::unl_cpu() };
            let _ = unsafe { abi::ena_dsp() };

            // Run TLS destructors now because they are not called automatically for terminated tasks. <br>现在运行 TLS 析构函数，因为终止的任务不会自动调用它们。<br>
            //
            unsafe { run_dtors() };

            let old_lifecycle = inner
                .lifecycle
                .swap(LIFECYCLE_EXITED_OR_FINISHED_OR_JOIN_FINALIZE, Ordering::Release);

            match old_lifecycle {
                LIFECYCLE_DETACHED => {
                    // [DETACHED → EXITED]
                    // No one will ever join, so we'll ask the collector task to delete the task. <br>没有人会加入，因此我们将要求收集器任务删除该任务。<br>
                    //

                    // In this case, `inner`'s ownership has been moved to us, And we are responsible for dropping it. <br>在这种情况下，`inner` 的所有权已经转移给我们，我们负责丢弃它。<br>
                    // The acquire ordering is not necessary because the parent thread made no memory access needing synchronization since the call to `acre_tsk`. <br>获取顺序不是必需的，因为从调用 `acre_tsk` 以来，父线程没有进行需要同步的内存访问。<br>
                    //
                    // Safety: See above. <br>安全：见上文。<br>
                    //
                    //
                    let _ = unsafe { Box::from_raw(inner as *const _ as *mut ThreadInner) };

                    // Safety: There are no pinned references to the stack <br>安全性：没有对栈的固定引用<br>
                    unsafe { terminate_and_delete_current_task() };
                }
                LIFECYCLE_INIT => {
                    // [INIT → FINISHED]
                    // The parent hasn't decided whether to join or detach this thread yet. <br>父任务还没有决定是加入还是分离这个线程。<br>
                    // Whichever option the parent chooses, it'll have to delete this task. <br>无论父任务选择哪个选项，它都必须删除此任务。<br>
                    // Since the parent might drop `*inner` as soon as it sees `FINISHED`, the release ordering must be used in the above `swap` call. <br>由于父任务可能会在看到 `FINISHED` 时立即丢弃 `*inner`，因此必须在上述 `swap` 调用中使用发布顺序。<br>
                    //
                    //
                    //
                }
                parent_tid => {
                    // Since the parent might drop `*inner` and terminate us as soon as it sees `JOIN_FINALIZE`, the release ordering must be used in the above `swap` call. <br>由于父任务可能会丢弃 `*inner` 并在它看到 `JOIN_FINALIZE` 后立即终止我们，因此必须在上述 `swap` 调用中使用发布顺序。<br>
                    //
                    //

                    // [JOINING → JOIN_FINALIZE]
                    // Wake up the parent task. <br>唤醒父任务。<br>
                    expect_success(
                        unsafe {
                            let mut er = abi::wup_tsk(parent_tid as _);
                            if er == abi::E_QOVR {
                                // `E_QOVR` indicates there's already a parking token <br>`E_QOVR` 表示已经有一个 parking token<br>
                                //
                                er = abi::E_OK;
                            }
                            er
                        },
                        &"wup_tsk",
                    );
                }
            }
        }

        let inner_ptr = (&*inner) as *const ThreadInner;

        let new_task = ItronError::err_if_negative(unsafe {
            abi::acre_tsk(&abi::T_CTSK {
                // Activate this task immediately <br>立即激活此任务<br>
                tskatr: abi::TA_ACT,
                exinf: inner_ptr as abi::EXINF,
                // The entry point <br>入口点<br>
                task: Some(trampoline),
                // Inherit the calling task's base priority <br>继承调用任务的基本优先级<br>
                itskpri: abi::TPRI_SELF,
                stksz: stack,
                // Let the kernel allocate the stack, <br>让内核分配栈，<br>
                stk: crate::ptr::null_mut(),
            })
        })
        .map_err(|e| e.as_io_error())?;

        Ok(Self { inner: ManuallyDrop::new(inner), task: new_task })
    }

    pub fn yield_now() {
        expect_success(unsafe { abi::rot_rdq(abi::TPRI_SELF) }, &"rot_rdq");
    }

    pub fn set_name(_name: &CStr) {
        // nope
    }

    pub fn sleep(dur: Duration) {
        for timeout in dur2reltims(dur) {
            expect_success(unsafe { abi::dly_tsk(timeout) }, &"dly_tsk");
        }
    }

    pub fn join(mut self) {
        let inner = &*self.inner;
        // Get the current task ID. <br>获取当前任务的 ID。<br>
        // Panicking here would cause a resource leak, so just abort on failure. <br>在这里 panic 会导致资源泄漏，所以只要在失败时中止即可。<br>
        let current_task = task::current_task_id_aborting();
        debug_assert!(usize::try_from(current_task).is_ok());
        debug_assert_ne!(current_task as usize, LIFECYCLE_INIT);
        debug_assert_ne!(current_task as usize, LIFECYCLE_DETACHED);

        let current_task = current_task as usize;

        match inner.lifecycle.swap(current_task, Ordering::Acquire) {
            LIFECYCLE_INIT => {
                // [INIT → JOINING]
                // The child task will transition the state to `JOIN_FINALIZE` and wake us up. <br>子任务将状态转换为 `JOIN_FINALIZE` 并唤醒我们。<br>
                //
                loop {
                    expect_success_aborting(unsafe { abi::slp_tsk() }, &"slp_tsk");
                    // To synchronize with the child task's memory accesses to `inner` up to the point of the assignment of `JOIN_FINALIZE`, `Ordering::Acquire` must be used for the `load`. <br>为了与子任务对 `inner` 的内存访问同步到 `JOIN_FINALIZE` 的分配点，`Ordering::Acquire` 必须用于 `load`。<br>
                    //
                    //
                    //
                    if inner.lifecycle.load(Ordering::Acquire) == LIFECYCLE_JOIN_FINALIZE {
                        break;
                    }
                }

                // [JOIN_FINALIZE → JOINED]
            }
            LIFECYCLE_FINISHED => {
                // [FINISHED → JOINED]
                // To synchronize with the child task's memory accesses to `inner` up to the point of the assignment of `FINISHED`, `Ordering::Acquire` must be used for the above `swap` call`. <br>为了与子任务对 `inner` 的内存访问同步到 `FINISHED` 的分配点，`Ordering::Acquire` 必须用于上述 `swap` 调用 `。<br>
                //
                //
            }
            _ => unsafe { hint::unreachable_unchecked() },
        }

        // Terminate and delete the task Safety: `self.task` still represents a task we own (because this method or `detach_inner` is called only once for each `Thread`). <br>终止并删除任务 Safety: `self.task` 仍然代表我们拥有所有权的任务 (因为这个方法或 `detach_inner` 对于每个 `Thread` 只调用一次)。<br>
        // The task indicated that it's safe to delete by entering the `FINISHED` or `JOIN_FINALIZE` state. <br>该任务表明可以通过进入 `FINISHED` 或 `JOIN_FINALIZE` 状态进行安全删除。<br>
        //
        //
        //
        unsafe { terminate_and_delete_task(self.task) };

        // In either case, we are responsible for dropping `inner`. <br>在任何一种情况下，我们都应对丢弃 `inner` 负责。<br>
        // Safety: The contents of `self.inner` will not be accessed hereafter <br>安全: `self.inner` 的内容以后将不会被访问<br>
        let _inner = unsafe { ManuallyDrop::take(&mut self.inner) };

        // Skip the destructor (because it would attempt to detach the thread) <br>跳过析构函数 (因为它会尝试分离线程)<br>
        crate::mem::forget(self);
    }
}

impl Drop for Thread {
    fn drop(&mut self) {
        // Detach the thread. <br>分离线程。<br>
        match self.inner.lifecycle.swap(LIFECYCLE_DETACHED_OR_JOINED, Ordering::Acquire) {
            LIFECYCLE_INIT => {
                // [INIT → DETACHED]
                // When the time comes, the child will figure out that no one will ever join it. <br>当时机成熟时，子节点会发现没有人会加入。<br>
                // The ownership of `self.inner` is moved to the child thread. <br>`self.inner` 的所有权转移到子线程。<br>
                // However, the release ordering is not necessary because we made no memory access needing synchronization since the call to `acre_tsk`. <br>但是，发布顺序不是必需的，因为自从调用 `acre_tsk` 后，我们没有进行需要同步的内存访问。<br>
                //
                //
                //
            }
            LIFECYCLE_FINISHED => {
                // [FINISHED → JOINED]
                // The task has already decided that we should delete the task. <br>任务已经决定我们应该删除任务。<br>
                // To synchronize with the child task's memory accesses to `inner` up to the point of the assignment of `FINISHED`, the acquire ordering is required for the above `swap` call. <br>为了与子任务对 `inner` 的内存访问同步到 `FINISHED` 的分配点，上述 `swap` 调用需要获取排序。<br>
                //
                //

                // Terminate and delete the task Safety: `self.task` still represents a task we own (because this method or `join_inner` is called only once for each `Thread`). <br>终止并删除任务安全: `self.task` 仍然代表我们拥有的任务 (因为这个方法或 `join_inner` 对于每个 `Thread` 只调用一次)。<br>
                // The task  indicated that it's safe to delete by entering the `FINISHED` state. <br>任务表明可以通过进入 `FINISHED` 状态进行安全删除。<br>
                //
                //
                //
                unsafe { terminate_and_delete_task(self.task) };

                // Wwe are responsible for dropping `inner`. <br>Wwe 负责丢弃 `inner`。<br>
                // Safety: The contents of `self.inner` will not be accessed hereafter <br>安全: `self.inner` 的内容以后将不会被访问<br>
                //
                unsafe { ManuallyDrop::drop(&mut self.inner) };
            }
            _ => unsafe { hint::unreachable_unchecked() },
        }
    }
}

pub mod guard {
    pub type Guard = !;
    pub unsafe fn current() -> Option<Guard> {
        None
    }
    pub unsafe fn init() -> Option<Guard> {
        None
    }
}

/// Terminate and delete the specified task. <br>终止并删除指定的任务。<br>
///
/// This function will abort if `deleted_task` refers to the calling task. <br>如果 `deleted_task` 指的是调用任务，则这个函数将终止。<br>
///
/// It is assumed that the specified task is solely managed by the caller - i.e., other threads must not "resuscitate" the specified task or delete it prematurely while this function is still in progress. <br>假设指定的任务是由调用者单独管理的 -- 即，在此函数仍在进行中时，其他线程不得对指定的任务进行 "resuscitate" 或过早删除。<br>
/// It is allowed for the specified task to exit by its own. <br>允许指定的任务自行退出。<br>
///
/// # Safety
///
/// The task must be safe to terminate. <br>任务必须可以安全终止。<br> This is in general not true because there might be pinned references to the task's stack. <br>这通常是不正确的，因为可能存在对任务栈的固定引用。<br>
///
///
///
unsafe fn terminate_and_delete_task(deleted_task: abi::ID) {
    // Terminate the task Safety: Upheld by the caller <br>终止任务安全：由调用方支持<br>
    //
    match unsafe { abi::ter_tsk(deleted_task) } {
        // Indicates the task is already dormant, ignore it <br>表示任务已经休眠，请忽略它<br>
        abi::E_OBJ => {}
        er => {
            expect_success_aborting(er, &"ter_tsk");
        }
    }

    // Delete the task Safety: Upheld by the caller <br>删除任务安全：调用方支持<br>
    //
    expect_success_aborting(unsafe { abi::del_tsk(deleted_task) }, &"del_tsk");
}

/// Terminate and delete the calling task. <br>终止并删除调用任务。<br>
///
/// Atomicity is not required - i.e., it can be assumed that other threads won't `ter_tsk` the calling task while this function is still in progress. <br>原子性不是必需的 - 即，可以假设其他线程不会在此函数仍在进行中时 `ter_tsk` 调用任务。<br>
/// (This property makes it easy to implement this operation on μITRON-derived kernels that don't support `exd_tsk`.) <br>(这个属性使得在不支持 `exd_tsk` 的 μITRON 派生内核上实现这个操作变得很容易。)<br>
///
/// # Safety
///
/// The task must be safe to terminate. <br>任务必须可以安全终止。<br> This is in general not true because there might be pinned references to the task's stack. <br>这通常是不正确的，因为可能存在对任务栈的固定引用。<br>
///
///
///
unsafe fn terminate_and_delete_current_task() -> ! {
    expect_success_aborting(unsafe { abi::exd_tsk() }, &"exd_tsk");
    // Safety: `exd_tsk` never returns on success <br>安全性: `exd_tsk` 永远不会成功返回<br>
    unsafe { crate::hint::unreachable_unchecked() };
}

pub fn available_parallelism() -> io::Result<crate::num::NonZeroUsize> {
    super::unsupported()
}
