use core::arch::asm;

// Do not remove inline: will result in relocation failure <br>不要删除内联：将导致重定位失败<br>
#[inline(always)]
pub(crate) unsafe fn rel_ptr<T>(offset: u64) -> *const T {
    (image_base() + offset) as *const T
}

// Do not remove inline: will result in relocation failure <br>不要删除内联：将导致重定位失败<br>
#[inline(always)]
pub(crate) unsafe fn rel_ptr_mut<T>(offset: u64) -> *mut T {
    (image_base() + offset) as *mut T
}

extern "C" {
    static ENCLAVE_SIZE: usize;
    static HEAP_BASE: u64;
    static HEAP_SIZE: usize;
}

/// Returns the base memory address of the heap <br>返回堆的基本内存地址<br>
pub(crate) fn heap_base() -> *const u8 {
    unsafe { rel_ptr_mut(HEAP_BASE) }
}

/// Returns the size of the heap <br>返回堆的大小<br>
pub(crate) fn heap_size() -> usize {
    unsafe { HEAP_SIZE }
}

// Do not remove inline: will result in relocation failure For the same reason we use inline ASM here instead of an extern static to locate the base <br>不要删除内联：将导致重定位失败出于同样的原因，我们在这里使用内联 ASM 而不是外部静态变量来定位基址。<br>
//
//
/// Returns address at which current enclave is loaded. <br>返回加载当前飞地的地址。<br>
#[inline(always)]
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn image_base() -> u64 {
    let base: u64;
    unsafe {
        asm!(
            "lea IMAGE_BASE(%rip), {}",
            lateout(reg) base,
            options(att_syntax, nostack, preserves_flags, nomem, pure),
        )
    };
    base
}

/// Returns `true` if the specified memory range is in the enclave. <br>如果指定的内存范围在安全区内，则返回 `true`。<br>
///
/// For safety, this function also checks whether the range given overflows, returning `false` if so. <br>为了安全起见，此函数还检查给定的范围是否溢出，如果是，则返回 `false`。<br>
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn is_enclave_range(p: *const u8, len: usize) -> bool {
    let start = p as usize;

    // Subtract one from `len` when calculating `end` in case `p + len` is exactly at the end of addressable memory (`p + len` would overflow, but the range is still valid). <br>如果 `p + len` 恰好位于可寻址存储器的末尾 (`p + len` 会溢出，但范围仍然有效)，则在计算 `end` 时应从 `len` 中减去一个。<br>
    //
    //
    let end = if len == 0 {
        start
    } else if let Some(end) = start.checked_add(len - 1) {
        end
    } else {
        return false;
    };

    let base = image_base() as usize;
    start >= base && end <= base + (unsafe { ENCLAVE_SIZE } - 1) // unsafe ok: link-time constant <br>不安全 ok: 链接时间常量<br>
}

/// Returns `true` if the specified memory range is in userspace. <br>如果指定的内存范围在用户空间中，则返回 `true`。<br>
///
/// For safety, this function also checks whether the range given overflows, returning `false` if so. <br>为了安全起见，此函数还检查给定的范围是否溢出，如果是，则返回 `false`。<br>
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn is_user_range(p: *const u8, len: usize) -> bool {
    let start = p as usize;

    // Subtract one from `len` when calculating `end` in case `p + len` is exactly at the end of addressable memory (`p + len` would overflow, but the range is still valid). <br>如果 `p + len` 恰好位于可寻址存储器的末尾 (`p + len` 会溢出，但范围仍然有效)，则在计算 `end` 时应从 `len` 中减去一个。<br>
    //
    //
    let end = if len == 0 {
        start
    } else if let Some(end) = start.checked_add(len - 1) {
        end
    } else {
        return false;
    };

    let base = image_base() as usize;
    end < base || start > base + (unsafe { ENCLAVE_SIZE } - 1) // unsafe ok: link-time constant <br>不安全 ok: 链接时间常量<br>
}
