use core::arch::asm;

// 不要删除内联：将导致重定位失败
#[inline(always)]
pub(crate) unsafe fn rel_ptr<T>(offset: u64) -> *const T {
    (image_base() + offset) as *const T
}

// 不要删除内联：将导致重定位失败
#[inline(always)]
pub(crate) unsafe fn rel_ptr_mut<T>(offset: u64) -> *mut T {
    (image_base() + offset) as *mut T
}

extern "C" {
    static ENCLAVE_SIZE: usize;
    static HEAP_BASE: u64;
    static HEAP_SIZE: usize;
}

/// 返回堆的基本内存地址
pub(crate) fn heap_base() -> *const u8 {
    unsafe { rel_ptr_mut(HEAP_BASE) }
}

/// 返回堆的大小
pub(crate) fn heap_size() -> usize {
    unsafe { HEAP_SIZE }
}

// 不要删除内联：将导致重定位失败出于同样的原因，我们在这里使用内联 ASM 而不是外部静态变量来定位基址。
//
//
/// 返回加载当前飞地的地址。
#[inline(always)]
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn image_base() -> u64 {
    let base: u64;
    unsafe {
        asm!(
            "lea IMAGE_BASE(%rip), {}",
            lateout(reg) base,
            options(att_syntax, nostack, preserves_flags, nomem, pure),
        )
    };
    base
}

/// 如果指定的内存范围在安全区内，则返回 `true`。
///
/// 为了安全起见，此函数还检查给定的范围是否溢出，如果是，则返回 `false`。
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn is_enclave_range(p: *const u8, len: usize) -> bool {
    let start = p as usize;

    // 如果 `p + len` 恰好位于可寻址存储器的末尾 (`p + len` 会溢出，但范围仍然有效)，则在计算 `end` 时应从 `len` 中减去一个。
    //
    //
    let end = if len == 0 {
        start
    } else if let Some(end) = start.checked_add(len - 1) {
        end
    } else {
        return false;
    };

    let base = image_base() as usize;
    start >= base && end <= base + (unsafe { ENCLAVE_SIZE } - 1) // 不安全 ok: 链接时间常量
}

/// 如果指定的内存范围在用户空间中，则返回 `true`。
///
/// 为了安全起见，此函数还检查给定的范围是否溢出，如果是，则返回 `false`。
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn is_user_range(p: *const u8, len: usize) -> bool {
    let start = p as usize;

    // 如果 `p + len` 恰好位于可寻址存储器的末尾 (`p + len` 会溢出，但范围仍然有效)，则在计算 `end` 时应从 `len` 中减去一个。
    //
    //
    let end = if len == 0 {
        start
    } else if let Some(end) = start.checked_add(len - 1) {
        end
    } else {
        return false;
    };

    let base = image_base() as usize;
    end < base || start > base + (unsafe { ENCLAVE_SIZE } - 1) // 不安全 ok: 链接时间常量
}
