#![cfg_attr(test, allow(unused))] // RT initialization logic is not compiled for test <br>RT 初始化逻辑未编译以进行测试<br>

use crate::io::Write;
use core::arch::global_asm;
use core::sync::atomic::{AtomicUsize, Ordering};

// runtime features <br>运行时特性<br>
pub(super) mod panic;
mod reloc;

// library features <br>library 特性<br>
pub mod mem;
pub mod thread;
pub mod tls;
#[macro_use]
pub mod usercalls;

#[cfg(not(test))]
global_asm!(include_str!("entry.S"), options(att_syntax));

#[repr(C)]
struct EntryReturn(u64, u64);

#[cfg(not(test))]
#[no_mangle]
unsafe extern "C" fn tcs_init(secondary: bool) {
    // Be very careful when changing this code: it runs before the binary has been relocated. <br>更改此代码时要非常小心：它在二进制文件重定位之前运行。<br>
    // Any indirect accesses to symbols will likely fail. <br>对符号的任何间接访问都可能会失败。<br>
    const UNINIT: usize = 0;
    const BUSY: usize = 1;
    const DONE: usize = 2;
    // Three-state spin-lock <br>三态自旋锁<br>
    static RELOC_STATE: AtomicUsize = AtomicUsize::new(UNINIT);

    if secondary && RELOC_STATE.load(Ordering::Relaxed) != DONE {
        rtabort!("Entered secondary TCS before main TCS!")
    }

    // Try to atomically swap UNINIT with BUSY. <br>尝试自动将 UNINIT 与 BUSY 交换。<br> The returned state can be: <br>返回的状态可以是：<br>
    match RELOC_STATE.compare_exchange(UNINIT, BUSY, Ordering::Acquire, Ordering::Acquire) {
        // This thread just obtained the lock and other threads will observe BUSY <br>这个线程刚刚获得了锁，其他线程将观察到忙<br>
        Ok(_) => {
            reloc::relocate_elf_rela();
            RELOC_STATE.store(DONE, Ordering::Release);
        }
        // We need to wait until the initialization is done. <br>我们需要等到初始化完成。<br>
        Err(BUSY) => {
            while RELOC_STATE.load(Ordering::Acquire) == BUSY {
                core::hint::spin_loop();
            }
        }
        // Initialization is done. <br>初始化完成。<br>
        Err(DONE) => {}
        _ => unreachable!(),
    }
}

// FIXME: this item should only exist if this is linked into an executable (main function exists). <br>仅当此项目链接到可执行文件时 (main 函数存在)，该项目才应该存在。<br>
// If this is a library, the crate author should be able to specify this <br>如果这是一个库，则 crate 作者应该可以指定此库<br>
//
#[cfg(not(test))]
#[no_mangle]
extern "C" fn entry(p1: u64, p2: u64, p3: u64, secondary: bool, p4: u64, p5: u64) -> EntryReturn {
    // FIXME: how to support TLS in library mode? <br>如何在库模式下支持 TLS?<br>
    let tls = Box::new(tls::Tls::new());
    let tls_guard = unsafe { tls.activate() };

    if secondary {
        let join_notifier = super::thread::Thread::entry();
        drop(tls_guard);
        drop(join_notifier);

        EntryReturn(0, 0)
    } else {
        extern "C" {
            fn main(argc: isize, argv: *const *const u8) -> isize;
        }

        // check entry is being called according to ABI <br>根据 ABI 调用检查条目<br>
        rtassert!(p3 == 0);
        rtassert!(p4 == 0);
        rtassert!(p5 == 0);

        unsafe {
            // The actual types of these arguments are `p1: *const Arg, p2: <br>这些参数的实际类型是 `p1: *const Arg, p2：<br>
            // usize`.
            // We can't currently customize the argument list of Rust's main function, so we pass these in as the standard pointer-sized values in `argc` and `argv`. <br>我们目前无法自定义 Rust 的 main 函数的参数列表，因此我们将它们作为 `argc` 和 `argv` 中标准指针大小的值传入。<br>
            //
            let ret = main(p2 as _, p1 as _);
            exit_with_code(ret)
        }
    }
}

pub(super) fn exit_with_code(code: isize) -> ! {
    if code != 0 {
        if let Some(mut out) = panic::SgxPanicOutput::new() {
            let _ = write!(out, "Exited with status code {code}");
        }
    }
    usercalls::exit(code != 0);
}

#[cfg(not(test))]
#[no_mangle]
extern "C" fn abort_reentry() -> ! {
    usercalls::exit(false)
}
