#![allow(unused)]

use crate::arch::asm;
use crate::cell::UnsafeCell;
use crate::cmp;
use crate::convert::TryInto;
use crate::mem;
use crate::ops::{CoerceUnsized, Deref, DerefMut, Index, IndexMut};
use crate::ptr::{self, NonNull};
use crate::slice;
use crate::slice::SliceIndex;

use super::super::mem::{is_enclave_range, is_user_range};
use fortanix_sgx_abi::*;

/// 可以安全地从用户空间读取或写入用户空间的类型。
///
/// reading 和 writing 的具体要求的非详尽清单：
/// * **类型为 `Copy`**(因此也不是 `Drop`)。复制 from/to 用户空间时将创建副本。析构函数将不会被调用。
/// * **没有引用或 Rust 风格的指针**(`Vec`，`Arc` 等)。从用户空间读取时，不得创建对安全区内存的引用。
/// 同样，Rust 编译器的静态分析仅考虑飞地内存。
/// 从用户空间读取时，不能保证该值正确符合该类型的期望。
/// 写入用户空间时，出于机密原因，绝不能泄漏安全区内存储器中数据的存储器地址。
/// 出于同样的原因，也不允许使用 `User` 和 `UserRef`。
/// * **没有胖指针。** 从用户空间读取时，代码可以自动解释和使用 size 或 vtable 指针。
/// 写入用户空间时，出于机密性原因，一定不要泄漏安全区内存中数据的内存地址 (例如 vtable 指针)。
///
/// 从用户空间读取的特定要求的非详尽清单：
/// * **任何位模式对于此类型均有效**(没有枚举)。
/// 不能保证该值正确符合该类型的期望，因此任何值都必须对该类型有效。
///
/// 写入用户空间的特定要求的非详尽清单：
/// * **没有指向飞地存储器的指针。** 出于机密原因，一定不能泄漏飞地存储器中数据的存储器地址。
/// * **没有内部填充。** 填充可能包含存储在该内存位置的先前初始化的机密数据，并且出于机密原因，不得泄漏。
///
///
///
///
///
///
///
///
///
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub unsafe trait UserSafeSized: Copy + Sized {}

#[unstable(feature = "sgx_platform", issue = "56975")]
unsafe impl UserSafeSized for u8 {}
#[unstable(feature = "sgx_platform", issue = "56975")]
unsafe impl<T> UserSafeSized for FifoDescriptor<T> {}
#[unstable(feature = "sgx_platform", issue = "56975")]
unsafe impl UserSafeSized for ByteBuffer {}
#[unstable(feature = "sgx_platform", issue = "56975")]
unsafe impl UserSafeSized for Usercall {}
#[unstable(feature = "sgx_platform", issue = "56975")]
unsafe impl UserSafeSized for Return {}
#[unstable(feature = "sgx_platform", issue = "56975")]
unsafe impl<T: UserSafeSized> UserSafeSized for [T; 2] {}

/// 一种可以在内存中表示为一个或多个 UserSafeSized 的类型。
#[unstable(feature = "sgx_platform", issue = "56975")]
pub unsafe trait UserSafe {
    /// 等效于 `mem::align_of::<Self>`。
    fn align_of() -> usize;

    /// 给定用户空间中的内存范围，构造指向 `Self` 的指针。
    ///
    /// 注意，这需要一个大小，而不是一个长度！
    ///
    /// # Safety
    ///
    /// 调用者必须确保存储范围在用户存储区中，并且大小正确，对齐正确并指向正确的类型。
    ///
    unsafe fn from_raw_sized_unchecked(ptr: *mut u8, size: usize) -> *mut Self;

    /// 给定一个内存范围，创建一个指向 `Self` 的指针。
    ///
    /// 注意，这需要一个大小，而不是一个长度！
    ///
    /// # Safety
    ///
    /// 调用者必须确保内存范围指向正确的类型。
    ///
    /// # Panics
    ///
    /// 如果满足以下条件，则此函数 panics：
    ///
    /// * 指针未对齐。
    /// * 指针为空。
    /// * 指向范围不适合地址空间。
    /// * 指向范围不在用户内存中。
    unsafe fn from_raw_sized(ptr: *mut u8, size: usize) -> NonNull<Self> {
        assert!(ptr.wrapping_add(size) >= ptr);
        // SAFETY: 调用者已保证指针有效
        let ret = unsafe { Self::from_raw_sized_unchecked(ptr, size) };
        unsafe {
            Self::check_ptr(ret);
            NonNull::new_unchecked(ret as _)
        }
    }

    /// 检查指针是否可以指向用户存储器中的 `Self`。
    ///
    /// # Safety
    ///
    /// 调用者必须确保内存范围指向正确的类型和长度 (如果这是切片)。
    ///
    ///
    /// # Panics
    ///
    /// 如果满足以下条件，则此函数 panics：
    ///
    /// * 指针未对齐。
    /// * 指针为空。
    /// * 指向范围不在用户内存中。
    unsafe fn check_ptr(ptr: *const Self) {
        let is_aligned = |p| -> bool { 0 == (p as usize) & (Self::align_of() - 1) };

        assert!(is_aligned(ptr as *const u8));
        assert!(is_user_range(ptr as _, mem::size_of_val(unsafe { &*ptr })));
        assert!(!ptr.is_null());
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
unsafe impl<T: UserSafeSized> UserSafe for T {
    fn align_of() -> usize {
        mem::align_of::<T>()
    }

    unsafe fn from_raw_sized_unchecked(ptr: *mut u8, size: usize) -> *mut Self {
        assert_eq!(size, mem::size_of::<T>());
        ptr as _
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
unsafe impl<T: UserSafeSized> UserSafe for [T] {
    fn align_of() -> usize {
        mem::align_of::<T>()
    }

    /// # Safety
    ///
    /// 如果违反以下任何条件，则行为是不确定的：
    /// * 对于 `size` 多个字节的写入，`ptr` 必须是 [有效的][valid]，并且必须正确对齐。
    ///
    /// [valid]: core::ptr#safety
    /// # Panics
    ///
    /// 如果满足以下条件，则此函数 panics：
    ///
    /// * 元素大小不是大小的因素
    unsafe fn from_raw_sized_unchecked(ptr: *mut u8, size: usize) -> *mut Self {
        let elem_size = mem::size_of::<T>();
        assert_eq!(size % elem_size, 0);
        let len = size / elem_size;
        // SAFETY: 调用者必须遵守 `from_raw_sized_unchecked` 的安全保证
        unsafe { slice::from_raw_parts_mut(ptr as _, len) }
    }
}

/// 对用户空间内存中某种类型的引用。`&UserRef<T>` 等同于安全区内存中的 `&T`。
/// 只有通过复制才能访问内存，以避免 TOCTTOU 问题。
/// 复制后，代码应确保在使用前完全检查该值。
///
/// 也有可能获得一个可变引用 `&mut UserRef<T>`。与常规可变引用不同，它们不是排他性的。
/// 用户空间可以随时随时写入后备存储器，因此不能假定指向的存储器是唯一借用的。
/// 两种不同的引用类型仅用于表示意图：可变引用用于写入用户存储器，不可变引用用于从用户存储器读取。
///
///
///
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub struct UserRef<T: ?Sized>(UnsafeCell<T>);
/// 用户空间内存中的所有类型。
/// `User<T>` 相当于 enclave 内存中的 `Box<T>`。
/// 只有通过复制才能访问内存，以避免 TOCTTOU 问题。
/// 丢弃该值后，将释放用户内存。
/// 复制后，代码应确保在使用前完全检查该值。
#[unstable(feature = "sgx_platform", issue = "56975")]
pub struct User<T: UserSafe + ?Sized>(NonNull<UserRef<T>>);

trait NewUserRef<T: ?Sized> {
    unsafe fn new_userref(v: T) -> Self;
}

impl<T: ?Sized> NewUserRef<*mut T> for NonNull<UserRef<T>> {
    unsafe fn new_userref(v: *mut T) -> Self {
        // SAFETY: 调用者已保证指针有效
        unsafe { NonNull::new_unchecked(v as _) }
    }
}

impl<T: ?Sized> NewUserRef<NonNull<T>> for NonNull<UserRef<T>> {
    unsafe fn new_userref(v: NonNull<T>) -> Self {
        // SAFETY: 调用者已保证指针有效
        unsafe { NonNull::new_userref(v.as_ptr()) }
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T: ?Sized> User<T>
where
    T: UserSafe,
{
    // 该函数返回实际上未初始化的内存，但出于优化编译器的目的，不将其视为 "unspecified" 或 "undefined"。
    // 这是通过从外部返回由 `super::alloc` 获得的指针来实现的。
    //
    //
    fn new_uninit_bytes(size: usize) -> Self {
        unsafe {
            // 不得调用大小为 0 的 alloc。
            let ptr = if size > 0 {
                // 当数据为 8 字节对齐时，`copy_to_userspace` 效率更高
                let alignment = cmp::max(T::align_of(), 8);
                rtunwrap!(Ok, super::alloc(size, alignment)) as _
            } else {
                T::align_of() as _ // 大小为 0 的悬垂指针正常
            };
            if let Ok(v) = crate::panic::catch_unwind(|| T::from_raw_sized(ptr, size)) {
                User(NonNull::new_userref(v))
            } else {
                rtabort!("Got invalid pointer from alloc() usercall")
            }
        }
    }

    /// 将 `val` 复制到用户内存中新分配的空间中。
    pub fn new_from_enclave(val: &T) -> Self {
        unsafe {
            let mut user = Self::new_uninit_bytes(mem::size_of_val(val));
            user.copy_from_enclave(val);
            user
        }
    }

    /// 从裸指针创建一个拥有的 `User<T>`。
    ///
    /// # Safety
    /// 调用者必须确保 `ptr` 指向 `T`，可以通过 `free` 用户调用和 `T` 的对齐释放，并且是唯一的。
    ///
    ///
    /// # Panics
    /// 如果满足以下条件，则此函数 panics：
    ///
    /// * 指针未对齐
    /// * 指针为空
    /// * 指向范围不在用户内存中
    pub unsafe fn from_raw(ptr: *mut T) -> Self {
        // SAFETY: 调用者必须遵守 `from_raw` 的安全保证。
        unsafe { T::check_ptr(ptr) };
        User(unsafe { NonNull::new_userref(ptr) })
    }

    /// 将此值转换为裸指针。
    /// 该值将不再自动释放。
    pub fn into_raw(self) -> *mut T {
        let ret = self.0;
        mem::forget(self);
        ret.as_ptr() as _
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T> User<T>
where
    T: UserSafe,
{
    /// 在用户内存中为 `T` 分配空间。
    pub fn uninitialized() -> Self {
        Self::new_uninit_bytes(mem::size_of::<T>())
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T> User<[T]>
where
    [T]: UserSafe,
{
    /// 在用户存储器中为 `n` 元素的 `[T]` 分配空间。
    pub fn uninitialized(n: usize) -> Self {
        Self::new_uninit_bytes(n * mem::size_of::<T>())
    }

    /// 从原始的薄指针和切片长度创建一个拥有的 `User<[T]>`。
    ///
    /// # Safety
    /// 调用者必须确保 `ptr` 指向 `T` 的 `len` 元素，并且可以通过 `free` 用户调用和 `T` 的对齐释放，并且是唯一的。
    ///
    ///
    /// # Panics
    /// 如果满足以下条件，则此函数 panics：
    ///
    /// * 指针未对齐
    /// * 指针为空
    /// * 指向范围不适合地址空间
    /// * 指向范围不在用户内存中
    ///
    pub unsafe fn from_raw_parts(ptr: *mut T, len: usize) -> Self {
        User(unsafe {
            NonNull::new_userref(<[T]>::from_raw_sized(ptr as _, len * mem::size_of::<T>()))
        })
    }
}

/// 将 `len` 字节的数据从飞地指针 `src` 复制到用户空间 `dst`
///
/// 此函数通过确保对不受信任内存的所有写入是以下两种情况之一来缓解陈旧数据漏洞:
///  - 前面是 VERW 指令，后面是 MFENCE; LFENCE 指令序列
///  - 或者是 8 字节的倍数，与 8 字节边界对齐
///
/// # Panics
/// 如果满足以下条件，则此函数 panics：
///
/// * `src` 指针为空
/// * `dst` 指针为空
/// * `src` 内存范围不在 enclave 内存中
/// * `dst` 内存范围不在用户内存中
///
/// # References
///  - https://www.intel.com/content/www/us/en/security-center/advisory/intel-sa-00615.html
///  - https://www.intel.com/content/www/us/en/developer/articles/technical/software-security-guidance/technical-documentation/processor-mmio-stale-data-vulnerabilities.html#inpage-nav-3-2-2
pub(crate) unsafe fn copy_to_userspace(src: *const u8, dst: *mut u8, len: usize) {
    unsafe fn copy_bytewise_to_userspace(src: *const u8, dst: *mut u8, len: usize) {
        unsafe {
            let mut seg_sel: u16 = 0;
            for off in 0..len {
                asm!("
                    mov %ds, ({seg_sel})
                    verw ({seg_sel})
                    movb {val}, ({dst})
                    mfence
                    lfence
                    ",
                    val = in(reg_byte) *src.offset(off as isize),
                    dst = in(reg) dst.offset(off as isize),
                    seg_sel = in(reg) &mut seg_sel,
                    options(nostack, att_syntax)
                );
            }
        }
    }

    unsafe fn copy_aligned_quadwords_to_userspace(src: *const u8, dst: *mut u8, len: usize) {
        unsafe {
            asm!(
                "rep movsq (%rsi), (%rdi)",
                inout("rcx") len / 8 => _,
                inout("rdi") dst => _,
                inout("rsi") src => _,
                options(att_syntax, nostack, preserves_flags)
            );
        }
    }
    assert!(!src.is_null());
    assert!(!dst.is_null());
    assert!(is_enclave_range(src, len));
    assert!(is_user_range(dst, len));
    assert!(len < isize::MAX as usize);
    assert!(!(src as usize).overflowing_add(len).1);
    assert!(!(dst as usize).overflowing_add(len).1);

    if len < 8 {
        // 无法在 8 字节边界上对齐: 逐字节安全复制
        unsafe {
            copy_bytewise_to_userspace(src, dst, len);
        }
    } else if len % 8 == 0 && dst as usize % 8 == 0 {
        // 复制 8 字节对齐的四字: 每个四字复制一个四字
        unsafe {
            copy_aligned_quadwords_to_userspace(src, dst, len);
        }
    } else {
        // 将副本分成三部分:
        //   +--------+
        //   | small0 | Chunk smaller than 8 bytes
        //   +--------+
        //   |   big  | Chunk 8-byte aligned, and size a multiple of 8 bytes
        //   +--------+
        //   | small1 | Chunk smaller than 8 bytes
        //   +--------+

        unsafe {
            // 复制 small0
            let small0_size = (8 - dst as usize % 8) as u8;
            let small0_src = src;
            let small0_dst = dst;
            copy_bytewise_to_userspace(small0_src as _, small0_dst, small0_size as _);

            // 复制 big
            let small1_size = ((len - small0_size as usize) % 8) as u8;
            let big_size = len - small0_size as usize - small1_size as usize;
            let big_src = src.offset(small0_size as _);
            let big_dst = dst.offset(small0_size as _);
            copy_aligned_quadwords_to_userspace(big_src as _, big_dst, big_size);

            // 复制 small1
            let small1_src = src.offset(big_size as isize + small0_size as isize);
            let small1_dst = dst.offset(big_size as isize + small0_size as isize);
            copy_bytewise_to_userspace(small1_src, small1_dst, small1_size as _);
        }
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T: ?Sized> UserRef<T>
where
    T: UserSafe,
{
    /// 从裸指针创建 `&UserRef<[T]>`。
    ///
    /// # Safety
    /// 调用者必须确保 `ptr` 指向 `T`。
    ///
    /// # Panics
    /// 如果满足以下条件，则此函数 panics：
    ///
    /// * 指针未对齐
    /// * 指针为空
    /// * 指向范围不在用户内存中
    pub unsafe fn from_ptr<'a>(ptr: *const T) -> &'a Self {
        // SAFETY: 调用者必须遵守 `from_ptr` 的安全保证。
        unsafe { T::check_ptr(ptr) };
        unsafe { &*(ptr as *const Self) }
    }

    /// 从裸指针创建 `&mut UserRef<[T]>`。
    /// 有关 `&mut UserRef<T>` 的细微差别，请参见结构体文档。
    ///
    /// # Safety
    /// 调用者必须确保 `ptr` 指向 `T`。
    ///
    /// # Panics
    /// 如果满足以下条件，则此函数 panics：
    ///
    /// * 指针未对齐
    /// * 指针为空
    /// * 指向范围不在用户内存中
    pub unsafe fn from_mut_ptr<'a>(ptr: *mut T) -> &'a mut Self {
        // SAFETY: 调用者必须遵守 `from_mut_ptr` 的安全保证。
        unsafe { T::check_ptr(ptr) };
        unsafe { &mut *(ptr as *mut Self) }
    }

    /// 将 `val` 复制到用户内存中。
    ///
    /// # Panics
    /// 如果目标的大小与源的大小不同，则此函数 panics。
    /// 对于切片等动态大小的类型，可能会发生这种情况。
    pub fn copy_from_enclave(&mut self, val: &T) {
        unsafe {
            assert_eq!(mem::size_of_val(val), mem::size_of_val(&*self.0.get()));
            copy_to_userspace(
                val as *const T as *const u8,
                self.0.get() as *mut T as *mut u8,
                mem::size_of_val(val),
            );
        }
    }

    /// 从用户存储器中复制值并将其放入 `dest`。
    ///
    /// # Panics
    /// 如果目标的大小与源的大小不同，则此函数 panics。
    /// 对于切片等动态大小的类型，可能会发生这种情况。
    pub fn copy_to_enclave(&self, dest: &mut T) {
        unsafe {
            assert_eq!(mem::size_of_val(dest), mem::size_of_val(&*self.0.get()));
            ptr::copy(
                self.0.get() as *const T as *const u8,
                dest as *mut T as *mut u8,
                mem::size_of_val(dest),
            );
        }
    }

    /// 从此引用中获取一个裸指针。
    pub fn as_raw_ptr(&self) -> *const T {
        self as *const _ as _
    }

    /// 从此引用中获取一个裸指针。
    pub fn as_raw_mut_ptr(&mut self) -> *mut T {
        self as *mut _ as _
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T> UserRef<T>
where
    T: UserSafe,
{
    /// 将值从用户存储器复制到安全区存储器。
    pub fn to_enclave(&self) -> T {
        unsafe { ptr::read(self.0.get()) }
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T> UserRef<[T]>
where
    [T]: UserSafe,
{
    /// 从原始的薄指针和切片长度创建 `&UserRef<[T]>`。
    ///
    /// # Safety
    /// 调用者必须确保 `ptr` 指向 `T` 的 `n` 元素。
    ///
    /// # Panics
    /// 如果满足以下条件，则此函数 panics：
    ///
    /// * 指针未对齐
    /// * 指针为空
    /// * 指向范围不适合地址空间
    /// * 指向范围不在用户内存中
    pub unsafe fn from_raw_parts<'a>(ptr: *const T, len: usize) -> &'a Self {
        // SAFETY: 调用者必须遵守 `from_raw_parts` 的安全保证。
        unsafe {
            &*(<[T]>::from_raw_sized(ptr as _, len * mem::size_of::<T>()).as_ptr() as *const Self)
        }
    }

    /// 从原始的薄指针和切片长度创建 `&mut UserRef<[T]>`。
    /// 有关 `&mut UserRef<T>` 的细微差别，请参见结构体文档。
    ///
    ///
    /// # Safety
    /// 调用者必须确保 `ptr` 指向 `T` 的 `n` 元素。
    ///
    /// # Panics
    /// 如果满足以下条件，则此函数 panics：
    ///
    /// * 指针未对齐
    /// * 指针为空
    /// * 指向范围不适合地址空间
    /// * 指向范围不在用户内存中
    pub unsafe fn from_raw_parts_mut<'a>(ptr: *mut T, len: usize) -> &'a mut Self {
        // SAFETY: 调用者必须遵守 `from_raw_parts_mut` 的安全保证。
        unsafe {
            &mut *(<[T]>::from_raw_sized(ptr as _, len * mem::size_of::<T>()).as_ptr() as *mut Self)
        }
    }

    /// 获取对此用户切片的第一个元素的裸指针。
    pub fn as_ptr(&self) -> *const T {
        self.0.get() as _
    }

    /// 获取对此用户切片的第一个元素的裸指针。
    pub fn as_mut_ptr(&mut self) -> *mut T {
        self.0.get() as _
    }

    /// 获取此用户切片中的元素数量。
    pub fn len(&self) -> usize {
        unsafe { (*self.0.get()).len() }
    }

    /// 从用户存储器中复制值并将其放入 `dest`。
    /// 之后，`dest` 将完全包含 `self.len()` 元素。
    ///
    /// # Panics
    /// 如果目标的大小与源的大小不同，则此函数 panics。
    /// 对于切片等动态大小的类型，可能会发生这种情况。
    pub fn copy_to_enclave_vec(&self, dest: &mut Vec<T>) {
        if let Some(missing) = self.len().checked_sub(dest.capacity()) {
            dest.reserve(missing)
        }
        // SAFETY: 我们在上面预留了足够的空间。
        unsafe { dest.set_len(self.len()) };
        self.copy_to_enclave(&mut dest[..]);
    }

    /// 将用户存储器中的值复制到安全区存储器中的 vector 中。
    pub fn to_enclave(&self) -> Vec<T> {
        let mut ret = Vec::with_capacity(self.len());
        self.copy_to_enclave_vec(&mut ret);
        ret
    }

    /// 返回切片上的迭代器。
    pub fn iter(&self) -> Iter<'_, T>
    where
        T: UserSafe, // FIXME: [T]: UserSafe 应该暗示什么？
    {
        unsafe { Iter((&*self.as_raw_ptr()).iter()) }
    }

    /// 返回允许修改每个值的迭代器。
    pub fn iter_mut(&mut self) -> IterMut<'_, T>
    where
        T: UserSafe, // FIXME: [T]: UserSafe 应该暗示什么？
    {
        unsafe { IterMut((&mut *self.as_raw_mut_ptr()).iter_mut()) }
    }
}

/// 不可变的用户切片迭代器
///
/// 该结构体是通过 `UserRef<[T]>` 上的 `iter` 方法创建的。
#[unstable(feature = "sgx_platform", issue = "56975")]
pub struct Iter<'a, T: 'a + UserSafe>(slice::Iter<'a, T>);

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<'a, T: UserSafe> Iterator for Iter<'a, T> {
    type Item = &'a UserRef<T>;

    #[inline]
    fn next(&mut self) -> Option<Self::Item> {
        unsafe { self.0.next().map(|e| UserRef::from_ptr(e)) }
    }
}

/// 可变用户切片迭代器
///
/// 该结构体是通过 `UserRef<[T]>` 上的 `iter_mut` 方法创建的。
#[unstable(feature = "sgx_platform", issue = "56975")]
pub struct IterMut<'a, T: 'a + UserSafe>(slice::IterMut<'a, T>);

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<'a, T: UserSafe> Iterator for IterMut<'a, T> {
    type Item = &'a mut UserRef<T>;

    #[inline]
    fn next(&mut self) -> Option<Self::Item> {
        unsafe { self.0.next().map(|e| UserRef::from_mut_ptr(e)) }
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T: ?Sized> Deref for User<T>
where
    T: UserSafe,
{
    type Target = UserRef<T>;

    fn deref(&self) -> &Self::Target {
        unsafe { &*self.0.as_ptr() }
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T: ?Sized> DerefMut for User<T>
where
    T: UserSafe,
{
    fn deref_mut(&mut self) -> &mut Self::Target {
        unsafe { &mut *self.0.as_ptr() }
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T: ?Sized> Drop for User<T>
where
    T: UserSafe,
{
    fn drop(&mut self) {
        unsafe {
            let ptr = (*self.0.as_ptr()).0.get();
            super::free(ptr as _, mem::size_of_val(&mut *ptr), T::align_of());
        }
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T: CoerceUnsized<U>, U> CoerceUnsized<UserRef<U>> for UserRef<T> {}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T, I> Index<I> for UserRef<[T]>
where
    [T]: UserSafe,
    I: SliceIndex<[T]>,
    I::Output: UserSafe,
{
    type Output = UserRef<I::Output>;

    #[inline]
    fn index(&self, index: I) -> &UserRef<I::Output> {
        unsafe {
            if let Some(slice) = index.get(&*self.as_raw_ptr()) {
                UserRef::from_ptr(slice)
            } else {
                rtabort!("index out of range for user slice");
            }
        }
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl<T, I> IndexMut<I> for UserRef<[T]>
where
    [T]: UserSafe,
    I: SliceIndex<[T]>,
    I::Output: UserSafe,
{
    #[inline]
    fn index_mut(&mut self, index: I) -> &mut UserRef<I::Output> {
        unsafe {
            if let Some(slice) = index.get_mut(&mut *self.as_raw_mut_ptr()) {
                UserRef::from_mut_ptr(slice)
            } else {
                rtabort!("index out of range for user slice");
            }
        }
    }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
impl UserRef<super::raw::ByteBuffer> {
    /// 将用户 `ByteBuffer` 指向的用户内存范围复制到安全区内存中。
    ///
    ///
    /// # Panics
    /// 如果在用户 `ByteBuffer` 中，此函数 panics 如果：
    ///
    /// * 指针为空
    /// * 指向范围不适合地址空间
    /// * 指向范围不在用户内存中
    pub fn copy_user_buffer(&self) -> Vec<u8> {
        unsafe {
            let buf = self.to_enclave();
            if buf.len > 0 {
                User::from_raw_parts(buf.data as _, buf.len).to_enclave()
            } else {
                // 如果 `len` 是 `0`，则不得查看 `data` 或调用 `free`。
                Vec::with_capacity(0)
            }
        }
    }
}
