use crate::cmp;
use crate::io::{Error as IoError, ErrorKind, IoSlice, IoSliceMut, Result as IoResult};
use crate::sys::rand::rdrand64;
use crate::time::{Duration, Instant};

pub(crate) mod alloc;
#[macro_use]
pub(crate) mod raw;
#[cfg(test)]
mod tests;

use self::raw::*;

/// Usercall `read`. <br>用户调用 `read`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
///
/// This will do a single `read` usercall and scatter the read data among `bufs`. <br>这将执行一个 `read` 用户调用，并将读取的数据分散到 `bufs` 之间。<br>
/// To read to a single buffer, just pass a slice of length one. <br>要读取单个缓冲区，只需传递长度为 1 的切片即可。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn read(fd: Fd, bufs: &mut [IoSliceMut<'_>]) -> IoResult<usize> {
    unsafe {
        let total_len = bufs.iter().fold(0usize, |sum, buf| sum.saturating_add(buf.len()));
        let mut userbuf = alloc::User::<[u8]>::uninitialized(total_len);
        let ret_len = raw::read(fd, userbuf.as_mut_ptr(), userbuf.len()).from_sgx_result()?;
        let userbuf = &userbuf[..ret_len];
        let mut index = 0;
        for buf in bufs {
            let end = cmp::min(index + buf.len(), userbuf.len());
            if let Some(buflen) = end.checked_sub(index) {
                userbuf[index..end].copy_to_enclave(&mut buf[..buflen]);
                index += buf.len();
            } else {
                break;
            }
        }
        Ok(userbuf.len())
    }
}

/// Usercall `read_alloc`. <br>用户调用 `read_alloc`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn read_alloc(fd: Fd) -> IoResult<Vec<u8>> {
    unsafe {
        let userbuf = ByteBuffer { data: crate::ptr::null_mut(), len: 0 };
        let mut userbuf = alloc::User::new_from_enclave(&userbuf);
        raw::read_alloc(fd, userbuf.as_raw_mut_ptr()).from_sgx_result()?;
        Ok(userbuf.copy_user_buffer())
    }
}

/// Usercall `write`. <br>用户调用 `write`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
///
/// This will do a single `write` usercall and gather the written data from `bufs`. <br>这将执行一个 `write` 用户调用，并从 `bufs` 收集写入的数据。<br>
/// To write from a single buffer, just pass a slice of length one. <br>要从单个缓冲区写入数据，只需传递一个长度为 1 的切片即可。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn write(fd: Fd, bufs: &[IoSlice<'_>]) -> IoResult<usize> {
    unsafe {
        let total_len = bufs.iter().fold(0usize, |sum, buf| sum.saturating_add(buf.len()));
        let mut userbuf = alloc::User::<[u8]>::uninitialized(total_len);
        let mut index = 0;
        for buf in bufs {
            let end = cmp::min(index + buf.len(), userbuf.len());
            if let Some(buflen) = end.checked_sub(index) {
                userbuf[index..end].copy_from_enclave(&buf[..buflen]);
                index += buf.len();
            } else {
                break;
            }
        }
        raw::write(fd, userbuf.as_ptr(), userbuf.len()).from_sgx_result()
    }
}

/// Usercall `flush`. <br>用户调用 `flush`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn flush(fd: Fd) -> IoResult<()> {
    unsafe { raw::flush(fd).from_sgx_result() }
}

/// Usercall `close`. <br>用户调用 `close`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn close(fd: Fd) {
    unsafe { raw::close(fd) }
}

fn string_from_bytebuffer(buf: &alloc::UserRef<ByteBuffer>, usercall: &str, arg: &str) -> String {
    String::from_utf8(buf.copy_user_buffer())
        .unwrap_or_else(|_| rtabort!("Usercall {usercall}: expected {arg} to be valid UTF-8"))
}

/// Usercall `bind_stream`. <br>用户调用 `bind_stream`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn bind_stream(addr: &str) -> IoResult<(Fd, String)> {
    unsafe {
        let addr_user = alloc::User::new_from_enclave(addr.as_bytes());
        let mut local = alloc::User::<ByteBuffer>::uninitialized();
        let fd = raw::bind_stream(addr_user.as_ptr(), addr_user.len(), local.as_raw_mut_ptr())
            .from_sgx_result()?;
        let local = string_from_bytebuffer(&local, "bind_stream", "local_addr");
        Ok((fd, local))
    }
}

/// Usercall `accept_stream`. <br>用户调用 `accept_stream`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn accept_stream(fd: Fd) -> IoResult<(Fd, String, String)> {
    unsafe {
        let mut bufs = alloc::User::<[ByteBuffer; 2]>::uninitialized();
        let mut buf_it = alloc::UserRef::iter_mut(&mut *bufs); // FIXME: can this be done <br>能做到吗<br>
        // without forcing coercion? <br>没有强制转换？<br>
        let (local, peer) = (buf_it.next().unwrap(), buf_it.next().unwrap());
        let fd = raw::accept_stream(fd, local.as_raw_mut_ptr(), peer.as_raw_mut_ptr())
            .from_sgx_result()?;
        let local = string_from_bytebuffer(&local, "accept_stream", "local_addr");
        let peer = string_from_bytebuffer(&peer, "accept_stream", "peer_addr");
        Ok((fd, local, peer))
    }
}

/// Usercall `connect_stream`. <br>用户调用 `connect_stream`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn connect_stream(addr: &str) -> IoResult<(Fd, String, String)> {
    unsafe {
        let addr_user = alloc::User::new_from_enclave(addr.as_bytes());
        let mut bufs = alloc::User::<[ByteBuffer; 2]>::uninitialized();
        let mut buf_it = alloc::UserRef::iter_mut(&mut *bufs); // FIXME: can this be done <br>能做到吗<br>
        // without forcing coercion? <br>没有强制转换？<br>
        let (local, peer) = (buf_it.next().unwrap(), buf_it.next().unwrap());
        let fd = raw::connect_stream(
            addr_user.as_ptr(),
            addr_user.len(),
            local.as_raw_mut_ptr(),
            peer.as_raw_mut_ptr(),
        )
        .from_sgx_result()?;
        let local = string_from_bytebuffer(&local, "connect_stream", "local_addr");
        let peer = string_from_bytebuffer(&peer, "connect_stream", "peer_addr");
        Ok((fd, local, peer))
    }
}

/// Usercall `launch_thread`. <br>用户调用 `launch_thread`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub unsafe fn launch_thread() -> IoResult<()> {
    // SAFETY: The caller must uphold the safety contract for `launch_thread`. <br>调用者必须遵守 `launch_thread` 的安全保证。<br>
    unsafe { raw::launch_thread().from_sgx_result() }
}

/// Usercall `exit`. <br>用户调用 `exit`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn exit(panic: bool) -> ! {
    unsafe { raw::exit(panic) }
}

/// Usercall `wait`. <br>用户调用 `wait`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn wait(event_mask: u64, mut timeout: u64) -> IoResult<u64> {
    if timeout != WAIT_NO && timeout != WAIT_INDEFINITE {
        // We don't want people to rely on accuracy of timeouts to make security decisions in an SGX enclave. <br>我们不希望人们依靠超时的准确性来在 SGX 飞地中做出安全决策。<br>
        // That's why we add a random amount not exceeding +/- 10% to the timeout value to discourage people from relying on accuracy of timeouts while providing a way to make things work in other cases. <br>这就是为什么我们在超时值上添加不超过 +/-10% 的随机数，以阻止人们依赖超时的准确性，同时提供一种使事情在其他情况下可以正常工作的方式。<br>
        //
        // Note that in the SGX threat model the enclave runner which is serving the wait usercall is not trusted to ensure accurate timeouts. <br>请注意，在 SGX 威胁模型中，为等待用户调用服务的飞地赛跑者不受信任以确保准确的超时。<br>
        //
        //
        //
        if let Ok(timeout_signed) = i64::try_from(timeout) {
            let tenth = timeout_signed / 10;
            let deviation = (rdrand64() as i64).checked_rem(tenth).unwrap_or(0);
            timeout = timeout_signed.saturating_add(deviation) as _;
        }
    }
    unsafe { raw::wait(event_mask, timeout).from_sgx_result() }
}

/// This function makes an effort to wait for a non-spurious event at least as long as `duration`. <br>该函数会努力等待至少至 `duration` 的非虚假事件。<br>
/// Note that in general there is no guarantee about accuracy of time and timeouts in SGX model. <br>请注意，一般而言，无法保证 SGX 模型中时间和超时的准确性。<br>
/// The enclave runner serving usercalls may lie about current time and/or ignore timeout values. <br>服务用户调用的飞地赛跑者可能在当前时间撒谎或者忽略超时值。<br>
///
/// Once the event is observed, `should_wake_up` will be used to determine whether or not the event was spurious. <br>观察到事件后，将使用 `should_wake_up` 来确定事件是否是虚假的。<br>
///
///
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn wait_timeout<F>(event_mask: u64, duration: Duration, should_wake_up: F)
where
    F: Fn() -> bool,
{
    // Calls the wait usercall and checks the result. <br>调用等待用户调用并检查结果。<br>
    // Returns true if event was returned, and false if WouldBlock/TimedOut was returned. <br>如果返回事件，则返回 true; 如果返回 WouldBlock/TimedOut，则返回 false。<br>
    // If duration is None, it will use WAIT_NO. <br>如果 duration 为 None，它将使用 WAIT_NO。<br>
    fn wait_checked(event_mask: u64, duration: Option<Duration>) -> bool {
        let timeout = duration.map_or(raw::WAIT_NO, |duration| {
            cmp::min((u64::MAX - 1) as u128, duration.as_nanos()) as u64
        });
        match wait(event_mask, timeout) {
            Ok(eventset) => {
                if event_mask == 0 {
                    rtabort!("expected wait() to return Err, found Ok.");
                }
                rtassert!(eventset != 0 && eventset & !event_mask == 0);
                true
            }
            Err(e) => {
                rtassert!(e.kind() == ErrorKind::TimedOut || e.kind() == ErrorKind::WouldBlock);
                false
            }
        }
    }

    match wait_checked(event_mask, Some(duration)) {
        false => return,                    // timed out <br>时间到<br>
        true if should_wake_up() => return, // woken up <br>醒了<br>
        true => {}                          // spurious event <br>虚假事件<br>
    }

    // Drain all cached events. <br>Drain 所有缓存的事件。<br>
    // Note that `event_mask != 0` is implied if we get here. <br>请注意，如果我们到达此处，则暗示 `event_mask != 0`。<br>
    loop {
        match wait_checked(event_mask, None) {
            false => break,                     // no more cached events <br>不再缓存事件<br>
            true if should_wake_up() => return, // woken up <br>醒了<br>
            true => {}                          // spurious event <br>虚假事件<br>
        }
    }

    // Continue waiting, but take note of time spent waiting so we don't wait forever. <br>继续等待，但要注意等待所花费的时间，以免我们永远等待。<br>
    // We intentionally don't call `Instant::now()` before this point to avoid the cost of the `insecure_time` usercall in case there are no spurious wakeups. <br>在此之前，我们故意不调用 `Instant::now()`，以避免在没有虚假唤醒的情况下 `insecure_time` 用户调用的费用。<br>
    //
    //

    let start = Instant::now();
    let mut remaining = duration;
    loop {
        match wait_checked(event_mask, Some(remaining)) {
            false => return,                    // timed out <br>时间到<br>
            true if should_wake_up() => return, // woken up <br>醒了<br>
            true => {}                          // spurious event <br>虚假事件<br>
        }
        remaining = match duration.checked_sub(start.elapsed()) {
            Some(remaining) => remaining,
            None => break,
        }
    }
}

/// Usercall `send`. <br>用户调用 `send`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn send(event_set: u64, tcs: Option<Tcs>) -> IoResult<()> {
    unsafe { raw::send(event_set, tcs).from_sgx_result() }
}

/// Usercall `insecure_time`. <br>用户调用 `insecure_time`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn insecure_time() -> Duration {
    let t = unsafe { raw::insecure_time() };
    Duration::new(t / 1_000_000_000, (t % 1_000_000_000) as _)
}

/// Usercall `alloc`. <br>用户调用 `alloc`。<br> See the ABI documentation for more information. <br>有关更多信息，请参见 ABI 文档。<br>
#[unstable(feature = "sgx_platform", issue = "56975")]
pub fn alloc(size: usize, alignment: usize) -> IoResult<*mut u8> {
    unsafe { raw::alloc(size, alignment).from_sgx_result() }
}

#[unstable(feature = "sgx_platform", issue = "56975")]
#[doc(inline)]
pub use self::raw::free;

fn check_os_error(err: Result) -> i32 {
    // FIXME: not sure how to make sure all variants of Error are covered <br>不确定如何确保涵盖所有错误变体<br>
    if err == Error::NotFound as _
        || err == Error::PermissionDenied as _
        || err == Error::ConnectionRefused as _
        || err == Error::ConnectionReset as _
        || err == Error::ConnectionAborted as _
        || err == Error::NotConnected as _
        || err == Error::AddrInUse as _
        || err == Error::AddrNotAvailable as _
        || err == Error::BrokenPipe as _
        || err == Error::AlreadyExists as _
        || err == Error::WouldBlock as _
        || err == Error::InvalidInput as _
        || err == Error::InvalidData as _
        || err == Error::TimedOut as _
        || err == Error::WriteZero as _
        || err == Error::Interrupted as _
        || err == Error::Other as _
        || err == Error::UnexpectedEof as _
        || ((Error::UserRangeStart as _)..=(Error::UserRangeEnd as _)).contains(&err)
    {
        err
    } else {
        rtabort!("Usercall: returned invalid error value {err}")
    }
}

trait FromSgxResult {
    type Return;

    fn from_sgx_result(self) -> IoResult<Self::Return>;
}

impl<T> FromSgxResult for (Result, T) {
    type Return = T;

    fn from_sgx_result(self) -> IoResult<Self::Return> {
        if self.0 == RESULT_SUCCESS {
            Ok(self.1)
        } else {
            Err(IoError::from_raw_os_error(check_os_error(self.0)))
        }
    }
}

impl FromSgxResult for Result {
    type Return = ();

    fn from_sgx_result(self) -> IoResult<Self::Return> {
        if self == RESULT_SUCCESS {
            Ok(())
        } else {
            Err(IoError::from_raw_os_error(check_os_error(self)))
        }
    }
}
