use super::waitqueue::{try_lock_or_false, SpinMutex, WaitQueue, WaitVariable};
use crate::sys_common::lazy_box::{LazyBox, LazyInit};

pub struct Mutex {
    inner: SpinMutex<WaitVariable<bool>>,
}

// not movable: see UnsafeList implementation <br>不可移动：请参见 UnsafeList 实现<br>
pub(crate) type MovableMutex = LazyBox<Mutex>;

impl LazyInit for Mutex {
    fn init() -> Box<Self> {
        Box::new(Self::new())
    }
}

// Implementation according to “Operating Systems: Three Easy Pieces”, chapter 28 <br>根据 `操作系统：三大要素` 的实现，第 28 章<br>
impl Mutex {
    pub const fn new() -> Mutex {
        Mutex { inner: SpinMutex::new(WaitVariable::new(false)) }
    }

    #[inline]
    pub unsafe fn init(&mut self) {}

    #[inline]
    pub unsafe fn lock(&self) {
        let mut guard = self.inner.lock();
        if *guard.lock_var() {
            // Another thread has the lock, wait <br>另一个线程有锁，等待<br>
            WaitQueue::wait(guard, || {})
        // Another thread has passed the lock to us <br>另一个线程将锁传递给了我们<br>
        } else {
            // We are just now obtaining the lock <br>我们刚刚获得了锁<br>
            *guard.lock_var_mut() = true;
        }
    }

    #[inline]
    pub unsafe fn unlock(&self) {
        let guard = self.inner.lock();
        if let Err(mut guard) = WaitQueue::notify_one(guard) {
            // No other waiters, unlock <br>没有其他等待者，请解锁<br>
            *guard.lock_var_mut() = false;
        } else {
            // There was a thread waiting, just pass the lock <br>有一个线程在等待，只是通过锁<br>
        }
    }

    #[inline]
    pub unsafe fn try_lock(&self) -> bool {
        let mut guard = try_lock_or_false!(self.inner);
        if *guard.lock_var() {
            // Another thread has the lock <br>另一个线程有锁<br>
            false
        } else {
            // We are just now obtaining the lock <br>我们刚刚获得了锁<br>
            *guard.lock_var_mut() = true;
            true
        }
    }
}
