use super::waitqueue::{try_lock_or_false, SpinMutex, WaitQueue, WaitVariable};
use crate::sys_common::lazy_box::{LazyBox, LazyInit};

pub struct Mutex {
    inner: SpinMutex<WaitVariable<bool>>,
}

// 不可移动：请参见 UnsafeList 实现
pub(crate) type MovableMutex = LazyBox<Mutex>;

impl LazyInit for Mutex {
    fn init() -> Box<Self> {
        Box::new(Self::new())
    }
}

// 根据 `操作系统：三大要素` 的实现，第 28 章
impl Mutex {
    pub const fn new() -> Mutex {
        Mutex { inner: SpinMutex::new(WaitVariable::new(false)) }
    }

    #[inline]
    pub unsafe fn init(&mut self) {}

    #[inline]
    pub unsafe fn lock(&self) {
        let mut guard = self.inner.lock();
        if *guard.lock_var() {
            // 另一个线程有锁，等待
            WaitQueue::wait(guard, || {})
        // 另一个线程将锁传递给了我们
        } else {
            // 我们刚刚获得了锁
            *guard.lock_var_mut() = true;
        }
    }

    #[inline]
    pub unsafe fn unlock(&self) {
        let guard = self.inner.lock();
        if let Err(mut guard) = WaitQueue::notify_one(guard) {
            // 没有其他等待者，请解锁
            *guard.lock_var_mut() = false;
        } else {
            // 有一个线程在等待，只是通过锁
        }
    }

    #[inline]
    pub unsafe fn try_lock(&self) -> bool {
        let mut guard = try_lock_or_false!(self.inner);
        if *guard.lock_var() {
            // 另一个线程有锁
            false
        } else {
            // 我们刚刚获得了锁
            *guard.lock_var_mut() = true;
            true
        }
    }
}
