#[cfg(test)]
mod tests;

use crate::num::NonZeroUsize;
use crate::sys_common::lazy_box::{LazyBox, LazyInit};

use super::waitqueue::{
    try_lock_or_false, NotifiedTcs, SpinMutex, SpinMutexGuard, WaitQueue, WaitVariable,
};
use crate::mem;

pub struct RwLock {
    readers: SpinMutex<WaitVariable<Option<NonZeroUsize>>>,
    writer: SpinMutex<WaitVariable<bool>>,
}

pub(crate) type MovableRwLock = LazyBox<RwLock>;

impl LazyInit for RwLock {
    fn init() -> Box<Self> {
        Box::new(Self::new())
    }
}

// Check at compile time that RwLock size matches C definition (see test_c_rwlock_initializer below) <br>在编译时检查 RwLock 大小是否与 C 定义匹配 (参见下面的 test_c_rwlock_initializer)<br>
//
// # Safety
// Never called, as it is a compile time check. <br>从未调用，因为它是编译时检查。<br>
#[allow(dead_code)]
unsafe fn rw_lock_size_assert(r: RwLock) {
    unsafe { mem::transmute::<RwLock, [u8; 144]>(r) };
}

impl RwLock {
    pub const fn new() -> RwLock {
        RwLock {
            readers: SpinMutex::new(WaitVariable::new(None)),
            writer: SpinMutex::new(WaitVariable::new(false)),
        }
    }

    #[inline]
    pub unsafe fn read(&self) {
        let mut rguard = self.readers.lock();
        let wguard = self.writer.lock();
        if *wguard.lock_var() || !wguard.queue_empty() {
            // Another thread has or is waiting for the write lock, wait <br>另一个线程具有或正在等待写锁，等待<br>
            drop(wguard);
            WaitQueue::wait(rguard, || {});
        // Another thread has passed the lock to us <br>另一个线程将锁传递给了我们<br>
        } else {
            // No waiting writers, acquire the read lock <br>没有等待的 writers，获取读锁<br>
            *rguard.lock_var_mut() =
                NonZeroUsize::new(rguard.lock_var().map_or(0, |n| n.get()) + 1);
        }
    }

    #[inline]
    pub unsafe fn try_read(&self) -> bool {
        let mut rguard = try_lock_or_false!(self.readers);
        let wguard = try_lock_or_false!(self.writer);
        if *wguard.lock_var() || !wguard.queue_empty() {
            // Another thread has or is waiting for the write lock <br>另一个线程具有或正在等待写锁定<br>
            false
        } else {
            // No waiting writers, acquire the read lock <br>没有等待的 writers，获取读锁<br>
            *rguard.lock_var_mut() =
                NonZeroUsize::new(rguard.lock_var().map_or(0, |n| n.get()) + 1);
            true
        }
    }

    #[inline]
    pub unsafe fn write(&self) {
        let rguard = self.readers.lock();
        let mut wguard = self.writer.lock();
        if *wguard.lock_var() || rguard.lock_var().is_some() {
            // Another thread has the lock, wait <br>另一个线程有锁，等待<br>
            drop(rguard);
            WaitQueue::wait(wguard, || {});
        // Another thread has passed the lock to us <br>另一个线程将锁传递给了我们<br>
        } else {
            // We are just now obtaining the lock <br>我们刚刚获得了锁<br>
            *wguard.lock_var_mut() = true;
        }
    }

    #[inline]
    pub unsafe fn try_write(&self) -> bool {
        let rguard = try_lock_or_false!(self.readers);
        let mut wguard = try_lock_or_false!(self.writer);
        if *wguard.lock_var() || rguard.lock_var().is_some() {
            // Another thread has the lock <br>另一个线程有锁<br>
            false
        } else {
            // We are just now obtaining the lock <br>我们刚刚获得了锁<br>
            *wguard.lock_var_mut() = true;
            true
        }
    }

    #[inline]
    unsafe fn __read_unlock(
        &self,
        mut rguard: SpinMutexGuard<'_, WaitVariable<Option<NonZeroUsize>>>,
        wguard: SpinMutexGuard<'_, WaitVariable<bool>>,
    ) {
        *rguard.lock_var_mut() = NonZeroUsize::new(rguard.lock_var().unwrap().get() - 1);
        if rguard.lock_var().is_some() {
            // There are other active readers <br>还有其他活跃的 readers<br>
        } else {
            if let Ok(mut wguard) = WaitQueue::notify_one(wguard) {
                // A writer was waiting, pass the lock <br>writer 正在等待，请通过锁<br>
                *wguard.lock_var_mut() = true;
                wguard.drop_after(rguard);
            } else {
                // No writers were waiting, the lock is released <br>没有 writers 在等待，锁被释放<br>
                rtassert!(rguard.queue_empty());
            }
        }
    }

    #[inline]
    pub unsafe fn read_unlock(&self) {
        let rguard = self.readers.lock();
        let wguard = self.writer.lock();
        unsafe { self.__read_unlock(rguard, wguard) };
    }

    #[inline]
    unsafe fn __write_unlock(
        &self,
        rguard: SpinMutexGuard<'_, WaitVariable<Option<NonZeroUsize>>>,
        wguard: SpinMutexGuard<'_, WaitVariable<bool>>,
    ) {
        match WaitQueue::notify_one(wguard) {
            Err(mut wguard) => {
                // No writers waiting, release the write lock <br>没有 writers 等待，释放写锁定<br>
                *wguard.lock_var_mut() = false;
                if let Ok(mut rguard) = WaitQueue::notify_all(rguard) {
                    // One or more readers were waiting, pass the lock to them <br>一个或多个 readers 正在等待，将锁传递给他们<br>
                    if let NotifiedTcs::All { count } = rguard.notified_tcs() {
                        *rguard.lock_var_mut() = Some(count)
                    } else {
                        unreachable!() // called notify_all <br>称为 notify_all<br>
                    }
                    rguard.drop_after(wguard);
                } else {
                    // No readers waiting, the lock is released <br>没有 readers 等待，锁定被释放<br>
                }
            }
            Ok(wguard) => {
                // There was a thread waiting for write, just pass the lock <br>有一个线程在等待写，只是传递了锁<br>
                wguard.drop_after(rguard);
            }
        }
    }

    #[inline]
    pub unsafe fn write_unlock(&self) {
        let rguard = self.readers.lock();
        let wguard = self.writer.lock();
        unsafe { self.__write_unlock(rguard, wguard) };
    }

    // only used by __rust_rwlock_unlock below <br>仅由下面的 __rust_rwlock_unlock 使用<br>
    #[inline]
    #[cfg_attr(test, allow(dead_code))]
    unsafe fn unlock(&self) {
        let rguard = self.readers.lock();
        let wguard = self.writer.lock();
        if *wguard.lock_var() == true {
            unsafe { self.__write_unlock(rguard, wguard) };
        } else {
            unsafe { self.__read_unlock(rguard, wguard) };
        }
    }
}

// The following functions are needed by libunwind. <br>libunwind 需要以下函数。<br>
// These symbols are named in pre-link args for the target specification, so keep that in sync. <br>这些符号在目标规范的预链接参数中命名，因此请使其保持同步。<br>
#[cfg(not(test))]
const EINVAL: i32 = 22;

#[cfg(not(test))]
#[no_mangle]
pub unsafe extern "C" fn __rust_rwlock_rdlock(p: *mut RwLock) -> i32 {
    if p.is_null() {
        return EINVAL;
    }
    unsafe { (*p).read() };
    return 0;
}

#[cfg(not(test))]
#[no_mangle]
pub unsafe extern "C" fn __rust_rwlock_wrlock(p: *mut RwLock) -> i32 {
    if p.is_null() {
        return EINVAL;
    }
    unsafe { (*p).write() };
    return 0;
}
#[cfg(not(test))]
#[no_mangle]
pub unsafe extern "C" fn __rust_rwlock_unlock(p: *mut RwLock) -> i32 {
    if p.is_null() {
        return EINVAL;
    }
    unsafe { (*p).unlock() };
    return 0;
}
