//! 同步原语的简单队列实现。
//!
//! 该队列用于实现条件变量和互斥锁。
//!
//! 此 API 的用户应使用 `WaitVariable<T>` 类型。
//! 由于该类型不是 `Sync`，因此需要使用 `SpinMutex` 进行保护以允许共享访问。
//!
//!
//! 由于用户空间可能发送虚假的唤醒，因此唤醒事件状态记录在安全区域中。
//! 唤醒事件状态受自旋锁保护。
//! 队列和相关的等待状态存储在 `WaitVariable` 中。

#[cfg(test)]
mod tests;

mod spin_mutex;
mod unsafe_list;

use crate::num::NonZeroUsize;
use crate::ops::{Deref, DerefMut};
use crate::time::Duration;

use super::abi::thread;
use super::abi::usercalls;
use fortanix_sgx_abi::{Tcs, EV_UNPARK, WAIT_INDEFINITE};

pub use self::spin_mutex::{try_lock_or_false, SpinMutex, SpinMutexGuard};
use self::unsafe_list::{UnsafeList, UnsafeListEntry};

/// `WaitQueue` 中的队列条目。
struct WaitEntry {
    /// 正在等待的线程的 TCS 地址
    tcs: Tcs,
    /// 此线程是否已被通知唤醒
    wake: bool,
}

/// 与 `WaitQueue` 一起存储的数据。
/// 由于类型本身不是 `Sync`，因此可以确保对队列的访问和数据是同步的。
///
///
/// 此 API 的使用者应使用同步原语进行共享访问，例如 `SpinMutex`。
#[derive(Default)]
pub struct WaitVariable<T> {
    queue: WaitQueue,
    lock: T,
}

impl<T> WaitVariable<T> {
    pub const fn new(var: T) -> Self {
        WaitVariable { queue: WaitQueue::new(), lock: var }
    }

    pub fn queue_empty(&self) -> bool {
        self.queue.is_empty()
    }

    pub fn lock_var(&self) -> &T {
        &self.lock
    }

    pub fn lock_var_mut(&mut self) -> &mut T {
        &mut self.lock
    }
}

#[derive(Copy, Clone)]
pub enum NotifiedTcs {
    Single(Tcs),
    All { count: NonZeroUsize },
}

/// RAII 防护程序将通知一组目标线程，并在挂起时解锁互斥锁。
///
pub struct WaitGuard<'a, T: 'a> {
    mutex_guard: Option<SpinMutexGuard<'a, WaitVariable<T>>>,
    notified_tcs: NotifiedTcs,
}

/// 正在等待某些同步原语的线程队列。
///
/// `UnsafeList` 被分配在等待线程的栈上。
/// 这样可以避免堆分配器中可能发生的任何锁定。
/// 这是安全的，因为等待的线程直到被通知后才从该栈帧返回。
/// 通知线程可确保在发送唤醒事件之前清除对列表条目的所有引用。
///
pub struct WaitQueue {
    // 我们在此使用内部互斥锁来保护面对虚假唤醒的数据。
    //
    inner: UnsafeList<SpinMutex<WaitEntry>>,
}
unsafe impl Send for WaitQueue {}

impl Default for WaitQueue {
    fn default() -> Self {
        Self::new()
    }
}

impl<'a, T> WaitGuard<'a, T> {
    /// 返回此守卫丢弃时将通知哪些 TCS。
    pub fn notified_tcs(&self) -> NotifiedTcs {
        self.notified_tcs
    }

    /// 丢弃另一个 `guard` 之后，丢弃 `WaitGuard`。
    pub fn drop_after<U>(self, guard: U) {
        drop(guard);
        drop(self);
    }
}

impl<'a, T> Deref for WaitGuard<'a, T> {
    type Target = SpinMutexGuard<'a, WaitVariable<T>>;

    fn deref(&self) -> &Self::Target {
        self.mutex_guard.as_ref().unwrap()
    }
}

impl<'a, T> DerefMut for WaitGuard<'a, T> {
    fn deref_mut(&mut self) -> &mut Self::Target {
        self.mutex_guard.as_mut().unwrap()
    }
}

impl<'a, T> Drop for WaitGuard<'a, T> {
    fn drop(&mut self) {
        drop(self.mutex_guard.take());
        let target_tcs = match self.notified_tcs {
            NotifiedTcs::Single(tcs) => Some(tcs),
            NotifiedTcs::All { .. } => None,
        };
        rtunwrap!(Ok, usercalls::send(EV_UNPARK, target_tcs));
    }
}

impl WaitQueue {
    pub const fn new() -> Self {
        WaitQueue { inner: UnsafeList::new() }
    }

    pub fn is_empty(&self) -> bool {
        self.inner.is_empty()
    }

    /// 将调用线程添加到 `WaitVariable` 的等待队列中，然后等待直到唤醒事件。
    ///
    ///
    /// 直到该线程被唤醒，该函数才返回。
    pub fn wait<T, F: FnOnce()>(mut guard: SpinMutexGuard<'_, WaitVariable<T>>, before_wait: F) {
        // 非常不安全：检查 UnsafeList::push 的要求
        unsafe {
            let mut entry = UnsafeListEntry::new(SpinMutex::new(WaitEntry {
                tcs: thread::current(),
                wake: false,
            }));
            let entry = guard.queue.inner.push(&mut entry);
            drop(guard);
            before_wait();
            while !entry.lock().wake {
                // 请勿 panic，否则在展开期间会使 `entry` 失效
                let eventset = rtunwrap!(Ok, usercalls::wait(EV_UNPARK, WAIT_INDEFINITE));
                rtassert!(eventset & EV_UNPARK == EV_UNPARK);
            }
        }
    }

    /// 将调用线程添加到 `WaitVariable` 的等待队列中，然后等待直到唤醒事件或超时。
    /// 如果观察到事件，则返回 true。
    /// 如果没有，它将从等待队列中删除调用线程。
    pub fn wait_timeout<T, F: FnOnce()>(
        lock: &SpinMutex<WaitVariable<T>>,
        timeout: Duration,
        before_wait: F,
    ) -> bool {
        // 非常不安全：检查 UnsafeList::push 的要求
        unsafe {
            let mut entry = UnsafeListEntry::new(SpinMutex::new(WaitEntry {
                tcs: thread::current(),
                wake: false,
            }));
            let entry_lock = lock.lock().queue.inner.push(&mut entry);
            before_wait();
            usercalls::wait_timeout(EV_UNPARK, timeout, || entry_lock.lock().wake);
            // 首先获取等待队列的锁，以避免死锁。
            let mut guard = lock.lock();
            let success = entry_lock.lock().wake;
            if !success {
                // 没有人唤醒我们，因此从等待队列中删除我们的条目。
                guard.queue.inner.remove(&mut entry);
            }
            success
        }
    }

    /// 在等待队列中找到下一个等待者，或者不更改互斥锁守卫。
    ///
    /// 如果找到等待者，则返回 `WaitGuard`，当 `WaitGuard` 丢弃时，它将通知等待者。
    ///
    ///
    pub fn notify_one<T>(
        mut guard: SpinMutexGuard<'_, WaitVariable<T>>,
    ) -> Result<WaitGuard<'_, T>, SpinMutexGuard<'_, WaitVariable<T>>> {
        unsafe {
            if let Some(entry) = guard.queue.inner.pop() {
                let mut entry_guard = entry.lock();
                let tcs = entry_guard.tcs;
                entry_guard.wake = true;
                drop(entry);
                Ok(WaitGuard { mutex_guard: Some(guard), notified_tcs: NotifiedTcs::Single(tcs) })
            } else {
                Err(guard)
            }
        }
    }

    /// 在等待队列中找到所有等待者，或者不更改互斥锁守卫。
    ///
    /// 如果找到至少一个等待者，则返回 `WaitGuard`，当 `WaitGuard` 被丢弃时，它将通知所有等待者。
    ///
    ///
    pub fn notify_all<T>(
        mut guard: SpinMutexGuard<'_, WaitVariable<T>>,
    ) -> Result<WaitGuard<'_, T>, SpinMutexGuard<'_, WaitVariable<T>>> {
        unsafe {
            let mut count = 0;
            while let Some(entry) = guard.queue.inner.pop() {
                count += 1;
                let mut entry_guard = entry.lock();
                entry_guard.wake = true;
            }
            if let Some(count) = NonZeroUsize::new(count) {
                Ok(WaitGuard { mutex_guard: Some(guard), notified_tcs: NotifiedTcs::All { count } })
            } else {
                Err(guard)
            }
        }
    }
}
