//! Android ABI-compatibility module <br>Android ABI 兼容模块<br>
//!
//! The ABI of Android has changed quite a bit over time, and libstd attempts to be both forwards and backwards compatible as much as possible. <br>Android 的 ABI 随时间发生了很大变化，libstd 尝试尽可能地向前和向后兼容。<br>
//! We want to always work with the most recent version of Android, but we also want to work with older versions of Android for whenever projects need to. <br>我们希望始终使用最新版本的 Android，但也希望在需要项目时使用 Android 的较早版本。<br>
//!
//! Our current minimum supported Android version is `android-9`, e.g., Android with API level 9. <br>我们当前支持的最低 Android 版本是 `android-9`，例如 API 级别为 9 的 Android。<br>
//! We then in theory want to work on that and all future versions of Android! <br>从理论上讲，我们接下来要研究所有 Android 的 future 版本！<br>
//!
//! Some of the detection here is done at runtime via `dlopen` and introspection. <br>这里的某些检测是在运行时通过 `dlopen` 和自省进行的。<br>
//! Other times no detection is performed at all and we just provide a fallback implementation as some versions of Android we support don't have the function. <br>其他时候根本没有执行检测，我们只是提供了一个后备实现，因为我们支持的某些 Android 版本没有函数。<br>
//!
//!
//! You'll find more details below about why each compatibility shim is needed. <br>您将在下面找到有关为什么需要每个兼容性垫片的更多详细信息。<br>
//!
//!
//!
//!

#![cfg(target_os = "android")]

use libc::{c_int, sighandler_t};

use super::weak::weak;

// The `log2` and `log2f` functions apparently appeared in android-18, or at least you can see they're not present in the android-17 header [1] and they are present in android-18 [2]. <br>`log2` 和 `log2f` 函数显然出现在 android-18 中，或者至少您可以看到它们不在 android-17 标头 [1] 中，而在 android-18 [2] 中。<br>
//
//
// [1]: https://chromium.googlesource.com/android_tools/+/20ee6d20/ndk/platforms
//                                       /android-17/arch-arm/usr/include/math.h
// [2]: https://chromium.googlesource.com/android_tools/+/20ee6d20/ndk/platforms
//                                       /android-18/arch-arm/usr/include/math.h
//
// Note that these shims are likely less precise than directly calling `log2`, but hopefully that should be enough for now... <br>请注意，这些垫片的精确度可能不如直接调用 `log2` 精确，但希望现在就足够了……<br>
//
// Note that mathematically, for any arbitrary `y`: <br>请注意，在数学上，对于任意 `y`：<br>
//
//      log_2(x) = log_y(x) / log_y(2) = log_y(x) / (1 / log_2(y)) = log_y(x) * log_2(y)
//
// Hence because `ln` (log_e) is available on all Android we just choose `y = e` and get: <br>因此，因为 `ln` (log_e) 在所有 Android 上都可用，所以我们只需选择 `y = e` 并获得：<br>
//
//      log_2(x) = ln(x) * log_2(e)
//
//
//
//
//

#[cfg(not(test))]
pub fn log2f32(f: f32) -> f32 {
    f.ln() * crate::f32::consts::LOG2_E
}

#[cfg(not(test))]
pub fn log2f64(f: f64) -> f64 {
    f.ln() * crate::f64::consts::LOG2_E
}

// Back in the day [1] the `signal` function was just an inline wrapper around `bsd_signal`, but starting in API level android-20 the `signal` symbols was introduced [2]. <br>早在 [1] 之前，`signal` 函数只是 `bsd_signal` 的内联包装，但是从 API 级别 android-20 开始，`signal` 符号被引入 [2]。<br>
// Finally, in android-21 the API `bsd_signal` was removed [3]. <br>最后，在 android-21 中，API `bsd_signal` 被删除。<br>
//
// Basically this means that if we want to be binary compatible with multiple Android releases (oldest being 9 and newest being 21) then we need to check for both symbols and not actually link against either. <br>基本上，这意味着如果我们要与多个 Android 版本二进制兼容 (最旧的版本为 9，最新的版本为 21)，则需要检查两个符号，而实际上并未针对任何一个进行链接。<br>
//
//
// [1]: https://chromium.googlesource.com/android_tools/+/20ee6d20/ndk/platforms
//                                       /android-18/arch-arm/usr/include/signal.h
// [2]: https://chromium.googlesource.com/android_tools/+/fbd420/ndk_experimental
//                                       /platforms/android-20/arch-arm
//                                       /usr/include/signal.h
// [3]: https://chromium.googlesource.com/android_tools/+/20ee6d/ndk/platforms
//                                       /android-21/arch-arm/usr/include/signal.h
//
//
//
pub unsafe fn signal(signum: c_int, handler: sighandler_t) -> sighandler_t {
    weak!(fn signal(c_int, sighandler_t) -> sighandler_t);
    weak!(fn bsd_signal(c_int, sighandler_t) -> sighandler_t);

    let f = signal.get().or_else(|| bsd_signal.get());
    let f = f.expect("neither `signal` nor `bsd_signal` symbols found");
    f(signum, handler)
}
