//! Android ABI 兼容模块
//!
//! Android 的 ABI 随时间发生了很大变化，libstd 尝试尽可能地向前和向后兼容。
//! 我们希望始终使用最新版本的 Android，但也希望在需要项目时使用 Android 的较早版本。
//!
//! 我们当前支持的最低 Android 版本是 `android-9`，例如 API 级别为 9 的 Android。
//! 从理论上讲，我们接下来要研究所有 Android 的 future 版本！
//!
//! 这里的某些检测是在运行时通过 `dlopen` 和自省进行的。
//! 其他时候根本没有执行检测，我们只是提供了一个后备实现，因为我们支持的某些 Android 版本没有函数。
//!
//!
//! 您将在下面找到有关为什么需要每个兼容性垫片的更多详细信息。
//!
//!
//!
//!

#![cfg(target_os = "android")]

use libc::{c_int, sighandler_t};

use super::weak::weak;

// `log2` 和 `log2f` 函数显然出现在 android-18 中，或者至少您可以看到它们不在 android-17 标头 [1] 中，而在 android-18 [2] 中。
//
//
// [1]: https://chromium.googlesource.com/android_tools/+/20ee6d20/ndk/platforms
//                                       /android-17/arch-arm/usr/include/math.h
// [2]: https://chromium.googlesource.com/android_tools/+/20ee6d20/ndk/platforms
//                                       /android-18/arch-arm/usr/include/math.h
//
// 请注意，这些垫片的精确度可能不如直接调用 `log2` 精确，但希望现在就足够了……
//
// 请注意，在数学上，对于任意 `y`：
//
//      log_2(x) = log_y(x) / log_y(2) = log_y(x) / (1 / log_2(y)) = log_y(x) * log_2(y)
//
// 因此，因为 `ln` (log_e) 在所有 Android 上都可用，所以我们只需选择 `y = e` 并获得：
//
//      log_2(x) = ln(x) * log_2(e)
//
//
//
//
//

#[cfg(not(test))]
pub fn log2f32(f: f32) -> f32 {
    f.ln() * crate::f32::consts::LOG2_E
}

#[cfg(not(test))]
pub fn log2f64(f: f64) -> f64 {
    f.ln() * crate::f64::consts::LOG2_E
}

// 早在 [1] 之前，`signal` 函数只是 `bsd_signal` 的内联包装，但是从 API 级别 android-20 开始，`signal` 符号被引入 [2]。
// 最后，在 android-21 中，API `bsd_signal` 被删除。
//
// 基本上，这意味着如果我们要与多个 Android 版本二进制兼容 (最旧的版本为 9，最新的版本为 21)，则需要检查两个符号，而实际上并未针对任何一个进行链接。
//
//
// [1]: https://chromium.googlesource.com/android_tools/+/20ee6d20/ndk/platforms
//                                       /android-18/arch-arm/usr/include/signal.h
// [2]: https://chromium.googlesource.com/android_tools/+/fbd420/ndk_experimental
//                                       /platforms/android-20/arch-arm
//                                       /usr/include/signal.h
// [3]: https://chromium.googlesource.com/android_tools/+/20ee6d/ndk/platforms
//                                       /android-21/arch-arm/usr/include/signal.h
//
//
//
pub unsafe fn signal(signum: c_int, handler: sighandler_t) -> sighandler_t {
    weak!(fn signal(c_int, sighandler_t) -> sighandler_t);
    weak!(fn bsd_signal(c_int, sighandler_t) -> sighandler_t);

    let f = signal.get().or_else(|| bsd_signal.get());
    let f = f.expect("neither `signal` nor `bsd_signal` symbols found");
    f(signum, handler)
}
