//! This module contains specializations that can offload `io::copy()` operations on file descriptor containing types (`File`, `TcpStream`, etc.) to more efficient syscalls than `read(2)` and `write(2)`. <br>该模块包含一些专门知识，可以将包含类型 (`File`，`TcpStream` 等) 的文件描述符上的 `io::copy()` 操作卸载到比 `read(2)` 和 `write(2)` 更有效的系统调用中。<br>
//!
//! Specialization is only applied to wholly std-owned types so that user code can't observe that the `Read` and `Write` traits are not used. <br>专业化仅适用于 std 完全拥有的类型，因此用户代码无法观察到未使用 `Read` 和 `Write` traits。<br>
//!
//! Since a copy operation involves a reader and writer side where each can consist of different types and also involve generic wrappers (e.g. `Take`, `BufReader`) it is not practical to specialize a single method on all possible combinations. <br>由于复制操作涉及 reader 和 writer 端，其中每个端可以由不同类型组成，并且还涉及泛型包装器 (例如 `Take`，`BufReader`)，因此在所有可能的组合上专门使用单个方法是不切实际的。<br>
//!
//! Instead readers and writers are handled separately by the `CopyRead` and `CopyWrite` specialization traits and then specialized on by the `Copier::copy` method. <br>取而代之的是，由 `CopyRead` 和 `CopyWrite` 专业化 traits 分别处理 readers 和 writers，然后由 `Copier::copy` 方法进行专门化。<br>
//!
//! `Copier` uses the specialization traits to unpack the underlying file descriptors and additional prerequisites and constraints imposed by the wrapper types. <br>`Copier` 使用专门化 traits 来解开包装的底层文件描述符，以及包装器类型强加的附加先决条件和约束。<br>
//!
//! Once it has obtained all necessary pieces and brought any wrapper types into a state where they can be safely bypassed it will attempt to use the `copy_file_range(2)`, `sendfile(2)` or `splice(2)` syscalls to move data directly between file descriptors. <br>一旦获得了所有必需的片段并将所有包装器类型置于可以安全绕过它们的状态，它将尝试使用 `copy_file_range(2)`，`sendfile(2)` 或 `splice(2)` 系统调用在文件描述符之间直接移动数据。<br>
//! Since those syscalls have requirements that cannot be fully checked in advance and gathering additional information about file descriptors would require additional syscalls anyway it simply attempts to use them one after another (guided by inaccurate hints) to figure out which one works and and falls back to the generic read-write copy loop if none of them does. <br>由于这些系统调用具有无法事先进行全面检查的要求，并且收集有关文件描述符的其他信息将需要额外的系统调用，因此它只是简单地尝试一个接一个地使用它们 (以不正确的提示为指导)，以找出哪个起作用并且回退到如果它们都不做，则泛型读写复制循环。<br>
//!
//! Once a working syscall is found for a pair of file descriptors it will be called in a loop until the copy operation is completed. <br>一旦找到了一对文件描述符的有效系统调用，它将在循环中被调用，直到复制操作完成为止。<br>
//!
//! Advantages of using these syscalls: <br>使用这些系统调用的优点：<br>
//!
//! * fewer context switches since reads and writes are coalesced into a single syscall and more bytes are transferred per syscall. <br>更少的上下文切换，因为将读和写合并为一个系统调用，并且每个系统调用传输更多的字节。<br> This translates to higher throughput and fewer CPU cycles, at least for sufficiently large transfers to amortize the initial probing. <br>至少对于足够大的传输以摊销初始探测，这意味着更高的吞吐量和更少的 CPU 周期。<br>
//! * `copy_file_range` creates reflink copies on CoW filesystems, thus moving less data and consuming less disk space <br>`copy_file_range` 在 CoW 文件系统上创建 reflink 副本，从而移动更少的数据并消耗更少的磁盘空间<br>
//! * `sendfile` and `splice` can perform zero-copy IO under some circumstances while a naive copy loop would move every byte through the CPU. <br>`sendfile` 和 `splice` 在某些情况下可以执行零拷贝输入输出，而简单的拷贝循环会通过 CPU 移动每个字节。<br>
//!
//! Drawbacks:
//!
//! * copy operations smaller than the default buffer size can under some circumstances, especially on older kernels, incur more syscalls than the naive approach would. <br>在某些情况下 (特别是在较早的内核上)，小于默认缓冲区大小的复制操作可能会比幼稚的方法引起更多的系统调用。<br> As mentioned above the syscall selection is guided by hints to minimize this possibility but they are not perfect. <br>如上所述，系统调用选择受提示的指导，以最大程度地减少这种可能性，但它们并不完美。<br>
//! * optimizations only apply to std types. <br>优化仅适用于 std 类型。<br> If a user adds a custom wrapper type, e.g. to report progress, they can hit a performance cliff. <br>如果用户添加自定义包装器类型 (例如，以报告进度)，则他们可能会遇到性能问题。<br>
//! * complexity
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

use crate::cmp::min;
use crate::fs::{File, Metadata};
use crate::io::copy::generic_copy;
use crate::io::{
    BufRead, BufReader, BufWriter, Error, Read, Result, StderrLock, StdinLock, StdoutLock, Take,
    Write,
};
use crate::mem::ManuallyDrop;
use crate::net::TcpStream;
use crate::os::unix::fs::FileTypeExt;
use crate::os::unix::io::{AsRawFd, FromRawFd, RawFd};
use crate::os::unix::net::UnixStream;
use crate::process::{ChildStderr, ChildStdin, ChildStdout};
use crate::ptr;
use crate::sync::atomic::{AtomicBool, AtomicU8, Ordering};
use crate::sys::cvt;
use crate::sys::weak::syscall;
use libc::{EBADF, EINVAL, ENOSYS, EOPNOTSUPP, EOVERFLOW, EPERM, EXDEV};

#[cfg(test)]
mod tests;

pub(crate) fn copy_spec<R: Read + ?Sized, W: Write + ?Sized>(
    read: &mut R,
    write: &mut W,
) -> Result<u64> {
    let copier = Copier { read, write };
    SpecCopy::copy(copier)
}

/// This type represents either the inferred `FileType` of a `RawFd` based on the source type from which it was extracted or the actual metadata <br>此类型表示根据提取来源的类型推断的 `RawFd` 的 `FileType` 或实际的元数据<br>
///
///
/// The methods on this type only provide hints, due to `AsRawFd` and `FromRawFd` the inferred type may be wrong. <br>此类型的方法仅提供提示，由于 `AsRawFd` 和 `FromRawFd` 推断的类型可能是错误的。<br>
///
enum FdMeta {
    /// We obtained the FD from a type that can contain any type of `FileType` and queried the metadata because it is cheaper than probing all possible syscalls (reader side) <br>我们从可以包含任何 `FileType` 类型的类型获得 FD，并查询元数据，因为它比探测所有可能的系统调用 (reader 端) 方便<br>
    ///
    Metadata(Metadata),
    Socket,
    Pipe,
    /// We don't have any metadata, e.g. <br>我们没有任何元数据，例如<br>
    /// because the original type was `File` which can represent any `FileType` and we did not query the metadata either since it did not seem beneficial (writer side) <br>因为原始类型是 `File`，它可以表示任何 `FileType`，并且我们也没有查询元数据，因为它似乎没有用 (writer 端)<br>
    ///
    NoneObtained,
}

impl FdMeta {
    fn maybe_fifo(&self) -> bool {
        match self {
            FdMeta::Metadata(meta) => meta.file_type().is_fifo(),
            FdMeta::Socket => false,
            FdMeta::Pipe => true,
            FdMeta::NoneObtained => true,
        }
    }

    fn potential_sendfile_source(&self) -> bool {
        match self {
            // procfs erroneously shows 0 length on non-empty readable files. <br>procfs 错误地在非空可读文件上显示 0 长度。<br>
            // and if a file is truly empty then a `read` syscall will determine that and skip the write syscall thus there would be benefit from attempting sendfile <br>如果文件确实为空，则 `read` syscall 将确定该文件并跳过写 syscall，因此尝试发送 sendfile 将有好处<br>
            //
            FdMeta::Metadata(meta)
                if meta.file_type().is_file() && meta.len() > 0
                    || meta.file_type().is_block_device() =>
            {
                true
            }
            _ => false,
        }
    }

    fn copy_file_range_candidate(&self) -> bool {
        match self {
            // copy_file_range will fail on empty procfs files. <br>对于空的 procfs 文件，copy_file_range 将失败。<br>
            // `read` can determine whether EOF has been reached without extra cost and skip the write, thus there is no benefit in attempting copy_file_range <br>`read` 可以确定是否已经达到 EOF 而不需要额外的成本，并跳过写入，因此尝试 copy_file_range 没有任何好处<br>
            FdMeta::Metadata(meta) if meta.is_file() && meta.len() > 0 => true,
            FdMeta::NoneObtained => true,
            _ => false,
        }
    }
}

struct CopyParams(FdMeta, Option<RawFd>);

struct Copier<'a, 'b, R: Read + ?Sized, W: Write + ?Sized> {
    read: &'a mut R,
    write: &'b mut W,
}

trait SpecCopy {
    fn copy(self) -> Result<u64>;
}

impl<R: Read + ?Sized, W: Write + ?Sized> SpecCopy for Copier<'_, '_, R, W> {
    default fn copy(self) -> Result<u64> {
        generic_copy(self.read, self.write)
    }
}

impl<R: CopyRead, W: CopyWrite> SpecCopy for Copier<'_, '_, R, W> {
    fn copy(self) -> Result<u64> {
        let (reader, writer) = (self.read, self.write);
        let r_cfg = reader.properties();
        let w_cfg = writer.properties();

        // before direct operations on file descriptors ensure that all source and sink buffers are empty <br>在对文件描述符进行直接操作之前，请确保所有源缓冲区和 sink 缓冲区为空<br>
        let mut flush = || -> crate::io::Result<u64> {
            let bytes = reader.drain_to(writer, u64::MAX)?;
            // BufWriter buffered bytes have already been accounted for in earlier write() calls <br>在较早的 write() 调用中已经考虑了 BufWriter 缓冲的字节<br>
            writer.flush()?;
            Ok(bytes)
        };

        let mut written = 0u64;

        if let (CopyParams(input_meta, Some(readfd)), CopyParams(output_meta, Some(writefd))) =
            (r_cfg, w_cfg)
        {
            written += flush()?;
            let max_write = reader.min_limit();

            if input_meta.copy_file_range_candidate() && output_meta.copy_file_range_candidate() {
                let result = copy_regular_files(readfd, writefd, max_write);
                result.update_take(reader);

                match result {
                    CopyResult::Ended(bytes_copied) => return Ok(bytes_copied + written),
                    CopyResult::Error(e, _) => return Err(e),
                    CopyResult::Fallback(bytes) => written += bytes,
                }
            }

            // on modern kernels sendfile can copy from any mmapable type (some but not all regular files and block devices) to any writable file descriptor. <br>在现代内核上，sendfile 可以从任何可映射类型 (某些但不是全部常规文件和块设备) 复制到任何可写文件描述符。<br>
            // On older kernels the writer side can only be a socket. <br>在较早的内核上，writer 端只能是一个套接字。<br>
            // So we just try and fallback if needed. <br>因此，我们只是在需要时尝试回退。<br>
            // If current file offsets + write sizes overflow it may also fail, we do not try to fix that and instead fall back to the generic copy loop. <br>如果当前文件偏移量 + 写入大小溢出，它也可能会失败，我们不会尝试解决此问题，而是会退回到泛型复制循环。<br>
            //
            if input_meta.potential_sendfile_source() {
                let result = sendfile_splice(SpliceMode::Sendfile, readfd, writefd, max_write);
                result.update_take(reader);

                match result {
                    CopyResult::Ended(bytes_copied) => return Ok(bytes_copied + written),
                    CopyResult::Error(e, _) => return Err(e),
                    CopyResult::Fallback(bytes) => written += bytes,
                }
            }

            if input_meta.maybe_fifo() || output_meta.maybe_fifo() {
                let result = sendfile_splice(SpliceMode::Splice, readfd, writefd, max_write);
                result.update_take(reader);

                match result {
                    CopyResult::Ended(bytes_copied) => return Ok(bytes_copied + written),
                    CopyResult::Error(e, _) => return Err(e),
                    CopyResult::Fallback(0) => { /* use the fallback below */ }
                    CopyResult::Fallback(_) => {
                        unreachable!("splice should not return > 0 bytes on the fallback path")
                    }
                }
            }
        }

        // fallback if none of the more specialized syscalls wants to work with these file descriptors <br>如果没有更专业的 syscall 想要使用这些文件描述符，则回退<br>
        match generic_copy(reader, writer) {
            Ok(bytes) => Ok(bytes + written),
            err => err,
        }
    }
}

#[rustc_specialization_trait]
trait CopyRead: Read {
    /// Implementations that contain buffers (i.e. <br>包含缓冲区的实现 (即<br>
    /// `BufReader`) must transfer data from their internal buffers into `writer` until either the buffers are emptied or `limit` bytes have been transferred, whichever occurs sooner. <br>`BufReader`) 必须将数据从其内部缓冲区传输到 `writer` 中，直到清空缓冲区或传输 `limit` 字节为止 (以较早发生者为准)。<br>
    ///
    /// If nested buffers are present the outer buffers must be drained first. <br>如果存在嵌套缓冲区，则必须先清空外部缓冲区。<br>
    ///
    /// This is necessary to directly bypass the wrapper types while preserving the data order when operating directly on the underlying file descriptors. <br>当直接对底层文件描述符进行操作时，在保留数据顺序的同时直接绕过包装器类型是必要的。<br>
    ///
    fn drain_to<W: Write>(&mut self, _writer: &mut W, _limit: u64) -> Result<u64> {
        Ok(0)
    }

    /// Updates `Take` wrappers to remove the number of bytes copied. <br>更新 `Take` 包装器以删除复制的字节数。<br>
    fn taken(&mut self, _bytes: u64) {}

    /// The minimum of the limit of all `Take<_>` wrappers, `u64::MAX` otherwise. <br>所有 `Take<_>` 包装器的最小限制，否则为 `u64::MAX`。<br>
    /// This method does not account for data `BufReader` buffers and would underreport the limit of a `Take<BufReader<Take<_>>>` type. <br>此方法不考虑数据 `BufReader` 缓冲区，并且会低估 `Take<BufReader<Take<_>>>` 类型的限制。<br>
    /// Thus its result is only valid after draining the buffers via `drain_to`. <br>因此，其结果仅在通过 `drain_to` 对缓冲区进行 draining 之后才有效。<br>
    ///
    fn min_limit(&self) -> u64 {
        u64::MAX
    }

    /// Extracts the file descriptor and hints/metadata, delegating through wrappers if necessary. <br>提取文件描述符和 hints/metadata，必要时通过包装委派。<br>
    fn properties(&self) -> CopyParams;
}

#[rustc_specialization_trait]
trait CopyWrite: Write {
    /// Extracts the file descriptor and hints/metadata, delegating through wrappers if necessary. <br>提取文件描述符和 hints/metadata，必要时通过包装委派。<br>
    fn properties(&self) -> CopyParams;
}

impl<T> CopyRead for &mut T
where
    T: CopyRead,
{
    fn drain_to<W: Write>(&mut self, writer: &mut W, limit: u64) -> Result<u64> {
        (**self).drain_to(writer, limit)
    }

    fn taken(&mut self, bytes: u64) {
        (**self).taken(bytes);
    }

    fn min_limit(&self) -> u64 {
        (**self).min_limit()
    }

    fn properties(&self) -> CopyParams {
        (**self).properties()
    }
}

impl<T> CopyWrite for &mut T
where
    T: CopyWrite,
{
    fn properties(&self) -> CopyParams {
        (**self).properties()
    }
}

impl CopyRead for File {
    fn properties(&self) -> CopyParams {
        CopyParams(fd_to_meta(self), Some(self.as_raw_fd()))
    }
}

impl CopyRead for &File {
    fn properties(&self) -> CopyParams {
        CopyParams(fd_to_meta(*self), Some(self.as_raw_fd()))
    }
}

impl CopyWrite for File {
    fn properties(&self) -> CopyParams {
        CopyParams(FdMeta::NoneObtained, Some(self.as_raw_fd()))
    }
}

impl CopyWrite for &File {
    fn properties(&self) -> CopyParams {
        CopyParams(FdMeta::NoneObtained, Some(self.as_raw_fd()))
    }
}

impl CopyRead for TcpStream {
    fn properties(&self) -> CopyParams {
        // avoid the stat syscall since we can be fairly sure it's a socket <br>避免 stat syscall，因为我们可以确定它是一个套接字<br>
        CopyParams(FdMeta::Socket, Some(self.as_raw_fd()))
    }
}

impl CopyRead for &TcpStream {
    fn properties(&self) -> CopyParams {
        // avoid the stat syscall since we can be fairly sure it's a socket <br>避免 stat syscall，因为我们可以确定它是一个套接字<br>
        CopyParams(FdMeta::Socket, Some(self.as_raw_fd()))
    }
}

impl CopyWrite for TcpStream {
    fn properties(&self) -> CopyParams {
        // avoid the stat syscall since we can be fairly sure it's a socket <br>避免 stat syscall，因为我们可以确定它是一个套接字<br>
        CopyParams(FdMeta::Socket, Some(self.as_raw_fd()))
    }
}

impl CopyWrite for &TcpStream {
    fn properties(&self) -> CopyParams {
        // avoid the stat syscall since we can be fairly sure it's a socket <br>避免 stat syscall，因为我们可以确定它是一个套接字<br>
        CopyParams(FdMeta::Socket, Some(self.as_raw_fd()))
    }
}

impl CopyRead for UnixStream {
    fn properties(&self) -> CopyParams {
        // avoid the stat syscall since we can be fairly sure it's a socket <br>避免 stat syscall，因为我们可以确定它是一个套接字<br>
        CopyParams(FdMeta::Socket, Some(self.as_raw_fd()))
    }
}

impl CopyRead for &UnixStream {
    fn properties(&self) -> CopyParams {
        // avoid the stat syscall since we can be fairly sure it's a socket <br>避免 stat syscall，因为我们可以确定它是一个套接字<br>
        CopyParams(FdMeta::Socket, Some(self.as_raw_fd()))
    }
}

impl CopyWrite for UnixStream {
    fn properties(&self) -> CopyParams {
        // avoid the stat syscall since we can be fairly sure it's a socket <br>避免 stat syscall，因为我们可以确定它是一个套接字<br>
        CopyParams(FdMeta::Socket, Some(self.as_raw_fd()))
    }
}

impl CopyWrite for &UnixStream {
    fn properties(&self) -> CopyParams {
        // avoid the stat syscall since we can be fairly sure it's a socket <br>避免 stat syscall，因为我们可以确定它是一个套接字<br>
        CopyParams(FdMeta::Socket, Some(self.as_raw_fd()))
    }
}

impl CopyWrite for ChildStdin {
    fn properties(&self) -> CopyParams {
        CopyParams(FdMeta::Pipe, Some(self.as_raw_fd()))
    }
}

impl CopyRead for ChildStdout {
    fn properties(&self) -> CopyParams {
        CopyParams(FdMeta::Pipe, Some(self.as_raw_fd()))
    }
}

impl CopyRead for ChildStderr {
    fn properties(&self) -> CopyParams {
        CopyParams(FdMeta::Pipe, Some(self.as_raw_fd()))
    }
}

impl CopyRead for StdinLock<'_> {
    fn drain_to<W: Write>(&mut self, writer: &mut W, outer_limit: u64) -> Result<u64> {
        let buf_reader = self.as_mut_buf();
        let buf = buf_reader.buffer();
        let buf = &buf[0..min(buf.len(), outer_limit.try_into().unwrap_or(usize::MAX))];
        let bytes_drained = buf.len();
        writer.write_all(buf)?;
        buf_reader.consume(bytes_drained);

        Ok(bytes_drained as u64)
    }

    fn properties(&self) -> CopyParams {
        CopyParams(fd_to_meta(self), Some(self.as_raw_fd()))
    }
}

impl CopyWrite for StdoutLock<'_> {
    fn properties(&self) -> CopyParams {
        CopyParams(FdMeta::NoneObtained, Some(self.as_raw_fd()))
    }
}

impl CopyWrite for StderrLock<'_> {
    fn properties(&self) -> CopyParams {
        CopyParams(FdMeta::NoneObtained, Some(self.as_raw_fd()))
    }
}

impl<T: CopyRead> CopyRead for Take<T> {
    fn drain_to<W: Write>(&mut self, writer: &mut W, outer_limit: u64) -> Result<u64> {
        let local_limit = self.limit();
        let combined_limit = min(outer_limit, local_limit);
        let bytes_drained = self.get_mut().drain_to(writer, combined_limit)?;
        // update limit since read() was bypassed <br>由于 read() 被绕过而导致的更新限制<br>
        self.set_limit(local_limit - bytes_drained);

        Ok(bytes_drained)
    }

    fn taken(&mut self, bytes: u64) {
        self.set_limit(self.limit() - bytes);
        self.get_mut().taken(bytes);
    }

    fn min_limit(&self) -> u64 {
        min(Take::limit(self), self.get_ref().min_limit())
    }

    fn properties(&self) -> CopyParams {
        self.get_ref().properties()
    }
}

impl<T: CopyRead> CopyRead for BufReader<T> {
    fn drain_to<W: Write>(&mut self, writer: &mut W, outer_limit: u64) -> Result<u64> {
        let buf = self.buffer();
        let buf = &buf[0..min(buf.len(), outer_limit.try_into().unwrap_or(usize::MAX))];
        let bytes = buf.len();
        writer.write_all(buf)?;
        self.consume(bytes);

        let remaining = outer_limit - bytes as u64;

        // in case of nested bufreaders we also need to drain the ones closer to the source <br>如果是嵌套的 bufreader，我们还需要 drain 靠近源<br>
        let inner_bytes = self.get_mut().drain_to(writer, remaining)?;

        Ok(bytes as u64 + inner_bytes)
    }

    fn taken(&mut self, bytes: u64) {
        self.get_mut().taken(bytes);
    }

    fn min_limit(&self) -> u64 {
        self.get_ref().min_limit()
    }

    fn properties(&self) -> CopyParams {
        self.get_ref().properties()
    }
}

impl<T: CopyWrite> CopyWrite for BufWriter<T> {
    fn properties(&self) -> CopyParams {
        self.get_ref().properties()
    }
}

fn fd_to_meta<T: AsRawFd>(fd: &T) -> FdMeta {
    let fd = fd.as_raw_fd();
    let file: ManuallyDrop<File> = ManuallyDrop::new(unsafe { File::from_raw_fd(fd) });
    match file.metadata() {
        Ok(meta) => FdMeta::Metadata(meta),
        Err(_) => FdMeta::NoneObtained,
    }
}

pub(super) enum CopyResult {
    Ended(u64),
    Error(Error, u64),
    Fallback(u64),
}

impl CopyResult {
    fn update_take(&self, reader: &mut impl CopyRead) {
        match *self {
            CopyResult::Fallback(bytes)
            | CopyResult::Ended(bytes)
            | CopyResult::Error(_, bytes) => reader.taken(bytes),
        }
    }
}

/// Invalid file descriptor. <br>无效的文件描述符。<br>
///
/// Valid file descriptors are guaranteed to be positive numbers (see `open()` manpage) while negative values are used to indicate errors. <br>有效的文件描述符保证为正数 (请参见 `open()` 联机帮助页)，而负值用于指示错误。<br>
///
/// Thus -1 will never be overlap with a valid open file. <br>因此，-1 永远不会与有效的打开文件重叠。<br>
const INVALID_FD: RawFd = -1;

/// Linux-specific implementation that will attempt to use copy_file_range for copy offloading. <br>特定于 Linux 的实现，将尝试使用 copy_file_range 进行副本分载。<br>
/// As the name says, it only works on regular files. <br>顾名思义，它仅适用于常规文件。<br>
///
/// Callers must handle fallback to a generic copy loop. <br>调用者必须处理回退到泛型复制循环的操作。<br>
/// `Fallback` may indicate non-zero number of bytes already written if one of the files' cursor +`max_len` would exceed u64::MAX (`EOVERFLOW`). <br>如果文件的游标 +`max_len` 之一将超过 u64::MAX (`EOVERFLOW`)，则 `Fallback` 可能表示已写入的字节数为非零。<br>
///
pub(super) fn copy_regular_files(reader: RawFd, writer: RawFd, max_len: u64) -> CopyResult {
    use crate::cmp;

    const NOT_PROBED: u8 = 0;
    const UNAVAILABLE: u8 = 1;
    const AVAILABLE: u8 = 2;

    // Kernel prior to 4.5 don't have copy_file_range We store the availability in a global to avoid unnecessary syscalls <br>4.5 之前的内核没有 copy_file_range 我们将可用性存储在一个以避免不必要的 syscalls<br>
    //
    static HAS_COPY_FILE_RANGE: AtomicU8 = AtomicU8::new(NOT_PROBED);

    syscall! {
        fn copy_file_range(
            fd_in: libc::c_int,
            off_in: *mut libc::loff_t,
            fd_out: libc::c_int,
            off_out: *mut libc::loff_t,
            len: libc::size_t,
            flags: libc::c_uint
        ) -> libc::ssize_t
    }

    match HAS_COPY_FILE_RANGE.load(Ordering::Relaxed) {
        NOT_PROBED => {
            // EPERM can indicate seccomp filters or an immutable file. <br>EPERM 可以指示 seccomp 过滤器或不可变文件。<br>
            // To distinguish these cases we probe with invalid file descriptors which should result in EBADF if the syscall is supported and some other error (ENOSYS or EPERM) if it's not available <br>为了区分这些情况，我们使用无效的文件描述符进行探测，如果系统调用受支持，则文件描述符将导致 EBADF; 如果不可用，则将导致其他错误 (ENOSYS 或 EPERM)<br>
            //
            let result = unsafe {
                cvt(copy_file_range(INVALID_FD, ptr::null_mut(), INVALID_FD, ptr::null_mut(), 1, 0))
            };

            if matches!(result.map_err(|e| e.raw_os_error()), Err(Some(EBADF))) {
                HAS_COPY_FILE_RANGE.store(AVAILABLE, Ordering::Relaxed);
            } else {
                HAS_COPY_FILE_RANGE.store(UNAVAILABLE, Ordering::Relaxed);
                return CopyResult::Fallback(0);
            }
        }
        UNAVAILABLE => return CopyResult::Fallback(0),
        _ => {}
    };

    let mut written = 0u64;
    while written < max_len {
        let bytes_to_copy = cmp::min(max_len - written, usize::MAX as u64);
        // cap to 1GB chunks in case u64::MAX is passed as max_len and the file has a non-zero seek position this allows us to copy large chunks without hitting EOVERFLOW, unless someone sets a file offset close to u64::MAX - 1GB, in which case a fallback would be required <br>如果将 u64::MAX 作为 max_len 传递，并且文件具有非零查找位置，则上限为 1GB 块，这使我们能够复制大块而不会达到 EOVERFLOW，除非有人将文件偏移量设置为接近 u64::MAX-1GB，在这种情况下回退需要<br>
        //
        //
        let bytes_to_copy = cmp::min(bytes_to_copy as usize, 0x4000_0000usize);
        let copy_result = unsafe {
            // We actually don't have to adjust the offsets, because copy_file_range adjusts the file offset automatically <br>实际上，我们不必调整偏移量，因为 copy_file_range 会自动调整文件偏移量<br>
            //
            cvt(copy_file_range(reader, ptr::null_mut(), writer, ptr::null_mut(), bytes_to_copy, 0))
        };

        match copy_result {
            Ok(0) if written == 0 => {
                // fallback to work around several kernel bugs where copy_file_range will fail to copy any bytes and return 0 instead of an error if <br>回退以解决几个内核错误，其中 copy_file_range 将无法复制任何字节并返回 0 而不是错误，如果<br>
                //
                // - reading virtual files from the proc filesystem which appear to have 0 size but are not empty. <br>从 proc 文件系统中读取虚拟文件，这些虚拟文件的大小似乎为 0，但不为空。<br>
                // noted in coreutils to affect kernels at least up to 5.6.19. <br>在 coreutils 中指出，至少会影响 5.6.19 的内核。<br>
                // - copying from an overlay filesystem in docker. <br>从 docker 中的覆盖文件系统复制。<br> reported to occur on fedora 32. <br>据报道发生在 fedora 32 上。<br>
                return CopyResult::Fallback(0);
            }
            Ok(0) => return CopyResult::Ended(written), // reached EOF <br>达到了 EOF<br>
            Ok(ret) => written += ret as u64,
            Err(err) => {
                return match err.raw_os_error() {
                    // when file offset + max_length > u64::MAX <br>当文件 `offset + max_length > u64::MAX` 时<br>
                    Some(EOVERFLOW) => CopyResult::Fallback(written),
                    Some(ENOSYS | EXDEV | EINVAL | EPERM | EOPNOTSUPP | EBADF) if written == 0 => {
                        // Try fallback io::copy if either: <br>如果出现以下情况，请尝试使用备用 io::copy：<br>
                        // - Kernel version is < 4.5 (ENOSYS¹) <br>内核版本 < 4.5 (ENOSYS¹)<br>
                        // - Files are mounted on different fs (EXDEV) <br>文件安装在不同的 fs (EXDEV) 上<br>
                        // - copy_file_range is broken in various ways on RHEL/CentOS 7 (EOPNOTSUPP) <br>RHEL/CentOS 7 (EOPNOTSUPP) 上的 copy_file_range 被各种方式破坏<br>
                        // - copy_file_range file is immutable or syscall is blocked by seccomp¹ (EPERM) <br>seccomp¹ (EPERM) 的 copy_file_range 文件不可变或 syscall 被阻止<br>
                        // - copy_file_range cannot be used with pipes or device nodes (EINVAL) <br>copy_file_range 不能与管道或设备节点 (EINVAL) 一起使用<br>
                        // - the writer fd was opened with O_APPEND (EBADF²) and no bytes were written successfully yet. <br>writer fd 是用 O_APPEND (EBADF²) 打开的，但尚未成功写入任何字节。<br> (All these errnos should not be returned if something was already written, but they happen in the wild, see #91152.) <br>(如果已经写入了某些内容，则不应返回所有这些 errnos，但它们会在野外发生，请参见 #91152。)<br>
                        //
                        // ¹ these cases should be detected by the initial probe but we handle them here anyway in case syscall interception changes during runtime ² actually invalid file descriptors would cause this too, but in that case the fallback code path is expected to encounter the same error again <br>¹ 这些情况应该由初始探针检测到，但是无论如何我们还是在这里处理它们，以防在运行时发生系统调用拦截变化 ² 实际上无效的文件描述符也会导致这种情况，但是在这种情况下，后备代码路径有望再次遇到相同的错误<br>
                        //
                        //
                        //
                        //
                        //
                        //
                        CopyResult::Fallback(0)
                    }
                    _ => CopyResult::Error(err, written),
                };
            }
        }
    }
    CopyResult::Ended(written)
}

#[derive(PartialEq)]
enum SpliceMode {
    Sendfile,
    Splice,
}

/// performs splice or sendfile between file descriptors Does _not_ fall back to a generic copy loop. <br>在文件描述符之间执行拼接或 sendfile _not_ 是否回退到泛型复制循环。<br>
///
fn sendfile_splice(mode: SpliceMode, reader: RawFd, writer: RawFd, len: u64) -> CopyResult {
    static HAS_SENDFILE: AtomicBool = AtomicBool::new(true);
    static HAS_SPLICE: AtomicBool = AtomicBool::new(true);

    // Android builds use feature level 14, but the libc wrapper for splice is gated on feature level 21+, so we have to invoke the syscall directly. <br>Android 构建使用特性级别 14，但用于拼接的 libc 包装器在特性级别 21+ 上进行设限，因此我们必须直接调用 syscall。<br>
    //
    #[cfg(target_os = "android")]
    syscall! {
        fn splice(
            srcfd: libc::c_int,
            src_offset: *const i64,
            dstfd: libc::c_int,
            dst_offset: *const i64,
            len: libc::size_t,
            flags: libc::c_int
        ) -> libc::ssize_t
    }

    #[cfg(target_os = "linux")]
    use libc::splice;

    match mode {
        SpliceMode::Sendfile if !HAS_SENDFILE.load(Ordering::Relaxed) => {
            return CopyResult::Fallback(0);
        }
        SpliceMode::Splice if !HAS_SPLICE.load(Ordering::Relaxed) => {
            return CopyResult::Fallback(0);
        }
        _ => (),
    }

    let mut written = 0u64;
    while written < len {
        // according to its manpage that's the maximum size sendfile() will copy per invocation <br>根据其手册，这是每次调用 sendfile() 将复制的最大大小<br>
        let chunk_size = crate::cmp::min(len - written, 0x7ffff000_u64) as usize;

        let result = match mode {
            SpliceMode::Sendfile => {
                cvt(unsafe { libc::sendfile(writer, reader, ptr::null_mut(), chunk_size) })
            }
            SpliceMode::Splice => cvt(unsafe {
                splice(reader, ptr::null_mut(), writer, ptr::null_mut(), chunk_size, 0)
            }),
        };

        match result {
            Ok(0) => break, // EOF
            Ok(ret) => written += ret as u64,
            Err(err) => {
                return match err.raw_os_error() {
                    Some(ENOSYS | EPERM) => {
                        // syscall not supported (ENOSYS) syscall is disallowed, e.g. <br>不支持 syscall (ENOSYS) 不允许进行 syscall，例如<br>
                        // by seccomp (EPERM) <br>通过 seccomp (EPERM)<br>
                        match mode {
                            SpliceMode::Sendfile => HAS_SENDFILE.store(false, Ordering::Relaxed),
                            SpliceMode::Splice => HAS_SPLICE.store(false, Ordering::Relaxed),
                        }
                        assert_eq!(written, 0);
                        CopyResult::Fallback(0)
                    }
                    Some(EINVAL) => {
                        // splice/sendfile do not support this particular file descriptor (EINVAL) <br>splice/sendfile 不支持这个特定的文件描述符 (EINVAL)<br>
                        assert_eq!(written, 0);
                        CopyResult::Fallback(0)
                    }
                    Some(os_err) if mode == SpliceMode::Sendfile && os_err == EOVERFLOW => {
                        CopyResult::Fallback(written)
                    }
                    _ => CopyResult::Error(err, written),
                };
            }
        }
    }
    CopyResult::Ended(written)
}
