use crate::sync::atomic::{
    AtomicU32,
    Ordering::{Acquire, Relaxed, Release},
};
use crate::sys::futex::{futex_wait, futex_wake, futex_wake_all};
use crate::time::Duration;

pub type MovableMutex = Mutex;
pub type MovableCondvar = Condvar;

pub struct Mutex {
    /// 0: unlocked
    /// 1: locked, no other threads waiting <br>锁定，没有其他线程在等待<br>
    /// 2: locked, and other threads waiting (contended) <br>锁定，其他线程等待 (contended)<br>
    futex: AtomicU32,
}

impl Mutex {
    #[inline]
    pub const fn new() -> Self {
        Self { futex: AtomicU32::new(0) }
    }

    #[inline]
    pub unsafe fn init(&mut self) {}

    #[inline]
    pub unsafe fn try_lock(&self) -> bool {
        self.futex.compare_exchange(0, 1, Acquire, Relaxed).is_ok()
    }

    #[inline]
    pub unsafe fn lock(&self) {
        if self.futex.compare_exchange(0, 1, Acquire, Relaxed).is_err() {
            self.lock_contended();
        }
    }

    #[cold]
    fn lock_contended(&self) {
        // Spin first to speed things up if the lock is released quickly. <br>如果锁被快速释放，请先自旋以加快速度。<br>
        let mut state = self.spin();

        // If it's unlocked now, attempt to take the lock without marking it as contended. <br>如果它现在已解锁，请尝试获取锁而不将其标记为已竞争。<br>
        //
        if state == 0 {
            match self.futex.compare_exchange(0, 1, Acquire, Relaxed) {
                Ok(_) => return, // Locked!
                Err(s) => state = s,
            }
        }

        loop {
            // Put the lock in contended state. <br>将锁置于竞争状态。<br>
            // We avoid an unnecessary write if it as already set to 2, to be friendlier for the caches. <br>如果它已经设置为 2，我们会避免不必要的写入，以便对缓存更友好。<br>
            //
            if state != 2 && self.futex.swap(2, Acquire) == 0 {
                // We changed it from 0 to 2, so we just succesfully locked it. <br>我们将其从 0 更改为 2，因此我们只是成功锁定了它。<br>
                return;
            }

            // Wait for the futex to change state, assuming it is still 2. <br>等待 futex 改变状态，假设它仍然是 2。<br>
            futex_wait(&self.futex, 2, None);

            // Spin again after waking up. <br>醒来后再次自旋。<br>
            state = self.spin();
        }
    }

    fn spin(&self) -> u32 {
        let mut spin = 100;
        loop {
            // We only use `load` (and not `swap` or `compare_exchange`) while spinning, to be easier on the caches. <br>我们只在自旋时使用 `load` (而不是 `swap` 或 `compare_exchange`)，以便更轻松地处理缓存。<br>
            //
            let state = self.futex.load(Relaxed);

            // We stop spinning when the mutex is unlocked (0), but also when it's contended (2). <br>当互斥锁解锁 (0) 时，我们停止自旋，当它与 (2) 竞争时也是如此。<br>
            //
            if state != 1 || spin == 0 {
                return state;
            }

            crate::hint::spin_loop();
            spin -= 1;
        }
    }

    #[inline]
    pub unsafe fn unlock(&self) {
        if self.futex.swap(0, Release) == 2 {
            // We only wake up one thread. <br>我们只唤醒一个线程。<br>
            // When that thread locks the mutex, it will mark the mutex as contended (2) (see lock_contended above), which makes sure that any other waiting threads will also be woken up eventually. <br>当该线程锁定上面的互斥锁时，它会将互斥锁标记为竞争 (2) (参见 lock_contended)，这确保任何其他等待的线程最终也会被唤醒。<br>
            //
            //
            self.wake();
        }
    }

    #[cold]
    fn wake(&self) {
        futex_wake(&self.futex);
    }
}

pub struct Condvar {
    // The value of this atomic is simply incremented on every notification. <br>这个原子的值只是在每次通知时递增。<br>
    // This is used by `.wait()` to not miss any notifications after unlocking the mutex and before waiting for notifications. <br>`.wait()` 使用它在解锁互斥锁之后和等待通知之前不会错过任何通知。<br>
    //
    futex: AtomicU32,
}

impl Condvar {
    #[inline]
    pub const fn new() -> Self {
        Self { futex: AtomicU32::new(0) }
    }

    // All the memory orderings here are `Relaxed`, because synchronization is done by unlocking and locking the mutex. <br>这里所有的内存排序都是 `Relaxed`，因为同步是通过互斥锁的解锁和锁定来完成的。<br>
    //

    pub unsafe fn notify_one(&self) {
        self.futex.fetch_add(1, Relaxed);
        futex_wake(&self.futex);
    }

    pub unsafe fn notify_all(&self) {
        self.futex.fetch_add(1, Relaxed);
        futex_wake_all(&self.futex);
    }

    pub unsafe fn wait(&self, mutex: &Mutex) {
        self.wait_optional_timeout(mutex, None);
    }

    pub unsafe fn wait_timeout(&self, mutex: &Mutex, timeout: Duration) -> bool {
        self.wait_optional_timeout(mutex, Some(timeout))
    }

    unsafe fn wait_optional_timeout(&self, mutex: &Mutex, timeout: Option<Duration>) -> bool {
        // Examine the notification counter _before_ we unlock the mutex. <br>在我们解锁互斥锁之前检查通知计数器。<br>
        let futex_value = self.futex.load(Relaxed);

        // Unlock the mutex before going to sleep. <br>睡觉前解锁互斥锁。<br>
        mutex.unlock();

        // Wait, but only if there hasn't been any notification since we unlocked the mutex. <br>等等，但前提是我们解锁互斥锁后没有任何通知。<br>
        //
        let r = futex_wait(&self.futex, futex_value, timeout);

        // Lock the mutex again. <br>再次锁定互斥锁。<br>
        mutex.lock();

        r
    }
}
