use crate::cell::UnsafeCell;
use crate::mem::MaybeUninit;
use crate::sys::cvt_nz;
use crate::sys_common::lazy_box::{LazyBox, LazyInit};

pub struct Mutex {
    inner: UnsafeCell<libc::pthread_mutex_t>,
}

pub(crate) type MovableMutex = LazyBox<Mutex>;

#[inline]
pub unsafe fn raw(m: &Mutex) -> *mut libc::pthread_mutex_t {
    m.inner.get()
}

unsafe impl Send for Mutex {}
unsafe impl Sync for Mutex {}

impl LazyInit for Mutex {
    fn init() -> Box<Self> {
        let mut mutex = Box::new(Self::new());
        unsafe { mutex.init() };
        mutex
    }
}

impl Mutex {
    pub const fn new() -> Mutex {
        // Might be moved to a different address, so it is better to avoid initialization of potentially opaque OS data before it landed. <br>可能将其移动到其他地址，因此最好避免在登陆之前可能不透明的 OS 数据进行初始化。<br>
        //
        // Be very careful using this newly constructed `Mutex`, reentrant locking is undefined behavior until `init` is called! <br>使用此新构造的 `Mutex` 时要格外小心，在调用 `init` 之前，可重入锁定是未定义的行为！<br>
        //
        Mutex { inner: UnsafeCell::new(libc::PTHREAD_MUTEX_INITIALIZER) }
    }
    #[inline]
    pub unsafe fn init(&mut self) {
        // Issue #33770
        //
        // A pthread mutex initialized with PTHREAD_MUTEX_INITIALIZER will have a type of PTHREAD_MUTEX_DEFAULT, which has undefined behavior if you try to re-lock it from the same thread when you already hold a lock (https://pubs.opengroup.org/onlinepubs/9699919799/functions/pthread_mutex_init.html). <br>使用 PTHREAD_MUTEX_INITIALIZER 初始化的 pthread 互斥锁将具有 PTHREAD_MUTEX_DEFAULT 类型，如果您在已经持有锁 (https://pubs.opengroup.org/onlinepubs/9699919799/functions/pthread_mutex_init.html) 的情况下尝试从同一线程重新锁定它，则该类型的行为不确定。<br>
        // This is the case even if PTHREAD_MUTEX_DEFAULT == PTHREAD_MUTEX_NORMAL (https://github.com/rust-lang/rust/issues/33770#issuecomment-220847521) -- in that case, `pthread_mutexattr_settype(PTHREAD_MUTEX_DEFAULT)` will of course be the same as setting it to `PTHREAD_MUTEX_NORMAL`, but not setting any mode will result in a Mutex where re-locking is UB. <br>即使 PTHREAD_MUTEX_DEFAULT == PTHREAD_MUTEX_NORMAL (https://github.com/rust-lang/rust/issues/33770#issuecomment-220847521) 也是这种情况 - 在这种情况下，`pthread_mutexattr_settype(PTHREAD_MUTEX_DEFAULT)` 当然与将其设置为 `PTHREAD_MUTEX_NORMAL` 相同，但是不设置任何模式都会导致互锁锁，其中重新锁定为 UB。<br>
        //
        //
        // In practice, glibc takes advantage of this undefined behavior to implement hardware lock elision, which uses hardware transactional memory to avoid acquiring the lock. <br>实际上，glibc 利用这种未定义的行为来实现硬件锁清除，该操作使用硬件事务性内存来避免获取锁。<br> While a transaction is in progress, the lock appears to be unlocked. <br>在进行事务时，锁似乎已解锁。<br>
        // This isn't a problem for other threads since the transactional memory will abort if a conflict is detected, however no abort is generated when re-locking from the same thread. <br>对于其他线程而言，这不是问题，因为如果检测到冲突，事务内存将终止，但是从同一线程重新锁定时，不会生成终止。<br>
        //
        // Since locking the same mutex twice will result in two aliasing &mut references, we instead create the mutex with type PTHREAD_MUTEX_NORMAL which is guaranteed to deadlock if we try to re-lock it from the same thread, thus avoiding undefined behavior. <br>由于两次锁定相同的互斥锁将导致两个别名 &mut references，因此，我们创建类型为 PTHREAD_MUTEX_NORMAL 的互斥锁，如果尝试从同一线程重新锁定它，则可以确保死锁，从而避免了未定义的行为。<br>
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        let mut attr = MaybeUninit::<libc::pthread_mutexattr_t>::uninit();
        cvt_nz(libc::pthread_mutexattr_init(attr.as_mut_ptr())).unwrap();
        let attr = PthreadMutexAttr(&mut attr);
        cvt_nz(libc::pthread_mutexattr_settype(attr.0.as_mut_ptr(), libc::PTHREAD_MUTEX_NORMAL))
            .unwrap();
        cvt_nz(libc::pthread_mutex_init(self.inner.get(), attr.0.as_ptr())).unwrap();
    }
    #[inline]
    pub unsafe fn lock(&self) {
        let r = libc::pthread_mutex_lock(self.inner.get());
        debug_assert_eq!(r, 0);
    }
    #[inline]
    pub unsafe fn unlock(&self) {
        let r = libc::pthread_mutex_unlock(self.inner.get());
        debug_assert_eq!(r, 0);
    }
    #[inline]
    pub unsafe fn try_lock(&self) -> bool {
        libc::pthread_mutex_trylock(self.inner.get()) == 0
    }
    #[inline]
    #[cfg(not(target_os = "dragonfly"))]
    unsafe fn destroy(&mut self) {
        let r = libc::pthread_mutex_destroy(self.inner.get());
        debug_assert_eq!(r, 0);
    }
    #[inline]
    #[cfg(target_os = "dragonfly")]
    unsafe fn destroy(&mut self) {
        let r = libc::pthread_mutex_destroy(self.inner.get());
        // On DragonFly pthread_mutex_destroy() returns EINVAL if called on a mutex that was just initialized with libc::PTHREAD_MUTEX_INITIALIZER. <br>在 DragonFly 上，如果在刚刚用 libc::PTHREAD_MUTEX_INITIALIZER 初始化的互斥锁上调用 pthread_mutex_destroy()，则返回 EINVAL。<br>
        //
        // Once it is used (locked/unlocked) or pthread_mutex_init() is called, this behaviour no longer occurs. <br>一旦使用了 (locked/unlocked) 或 pthread_mutex_init()，将不再发生此行为。<br>
        //
        debug_assert!(r == 0 || r == libc::EINVAL);
    }
}

impl Drop for Mutex {
    #[inline]
    fn drop(&mut self) {
        unsafe { self.destroy() };
    }
}

pub(super) struct PthreadMutexAttr<'a>(pub &'a mut MaybeUninit<libc::pthread_mutexattr_t>);

impl Drop for PthreadMutexAttr<'_> {
    fn drop(&mut self) {
        unsafe {
            let result = libc::pthread_mutexattr_destroy(self.0.as_mut_ptr());
            debug_assert_eq!(result, 0);
        }
    }
}
