use crate::cell::UnsafeCell;
use crate::sync::atomic::{AtomicUsize, Ordering};
use crate::sys_common::lazy_box::{LazyBox, LazyInit};

pub struct RwLock {
    inner: UnsafeCell<libc::pthread_rwlock_t>,
    write_locked: UnsafeCell<bool>, // 由 `inner` RwLock 保护
    num_readers: AtomicUsize,
}

pub(crate) type MovableRwLock = LazyBox<RwLock>;

unsafe impl Send for RwLock {}
unsafe impl Sync for RwLock {}

impl LazyInit for RwLock {
    fn init() -> Box<Self> {
        Box::new(Self::new())
    }
}

impl RwLock {
    pub const fn new() -> RwLock {
        RwLock {
            inner: UnsafeCell::new(libc::PTHREAD_RWLOCK_INITIALIZER),
            write_locked: UnsafeCell::new(false),
            num_readers: AtomicUsize::new(0),
        }
    }
    #[inline]
    pub unsafe fn read(&self) {
        let r = libc::pthread_rwlock_rdlock(self.inner.get());

        // 根据 POSIX，当一个线程在它已经持有写锁的情况下试图获取这个读锁时 (反之亦然，或者试图获取写锁两次)，这个调用要么死锁，要么返回 [EDEADLK]
        //
        // (https://pubs.opengroup.org/onlinepubs/9699919799/functions/pthread_rwlock_wrlock.html,
        // https://pubs.opengroup.org/onlinepubs/9699919799/functions/pthread_rwlock_rdlock.html).
        // 因此，原则上，我们要做的就是检查 `r == 0`，以确保我们正确地获得了锁。
        //
        // 但是，(至少) 版本 2.25 之前的 glibc 不符合该规范，并且即使该线程已持有写锁，它也可以返回 `r == 0`。
        // 因此，当我们检测到线程多次获得写锁定，或者获得读和写锁定时，我们自己和 panic 都检查了这种情况。
        //
        //
        //
        //
        //
        if r == libc::EAGAIN {
            panic!("rwlock maximum reader count exceeded");
        } else if r == libc::EDEADLK || (r == 0 && *self.write_locked.get()) {
            // 上面，我们确保仅在 `r == 0` 时访问 `write_locked`，以避免数据竞争。
            //
            if r == 0 {
                // `pthread_rwlock_rdlock` 在它不应该成功的时候成功了。
                self.raw_unlock();
            }
            panic!("rwlock read lock would result in deadlock");
        } else {
            // POSIX 不保证所有可能返回的错误。
            // 有关详细信息，请参见 issue #94705。
            assert_eq!(r, 0, "unexpected error during rwlock read lock: {:?}", r);
            self.num_readers.fetch_add(1, Ordering::Relaxed);
        }
    }
    #[inline]
    pub unsafe fn try_read(&self) -> bool {
        let r = libc::pthread_rwlock_tryrdlock(self.inner.get());
        if r == 0 {
            if *self.write_locked.get() {
                // `pthread_rwlock_tryrdlock` 在它不应该成功的时候成功了。
                self.raw_unlock();
                false
            } else {
                self.num_readers.fetch_add(1, Ordering::Relaxed);
                true
            }
        } else {
            false
        }
    }
    #[inline]
    pub unsafe fn write(&self) {
        let r = libc::pthread_rwlock_wrlock(self.inner.get());
        // 有关我们为什么检查 EDEADLK 和 write_locked 的信息，请参见上面的评论。
        // 出于同样的原因，我们还需要检查是否没有 readers (在 `num_readers` 中跟踪)。
        if r == libc::EDEADLK
            || (r == 0 && *self.write_locked.get())
            || self.num_readers.load(Ordering::Relaxed) != 0
        {
            // 上面，我们确保仅在 `r == 0` 时访问 `write_locked`，以避免数据竞争。
            //
            if r == 0 {
                // `pthread_rwlock_wrlock` 在它不应该成功的时候成功了。
                self.raw_unlock();
            }
            panic!("rwlock write lock would result in deadlock");
        } else {
            // 根据 POSIX，对于正确初始化的 rwlock，此操作只能返回 EDEADLK 或 0。
            // 我们依靠这一点。
            debug_assert_eq!(r, 0);
        }
        *self.write_locked.get() = true;
    }
    #[inline]
    pub unsafe fn try_write(&self) -> bool {
        let r = libc::pthread_rwlock_trywrlock(self.inner.get());
        if r == 0 {
            if *self.write_locked.get() || self.num_readers.load(Ordering::Relaxed) != 0 {
                // `pthread_rwlock_trywrlock` 在它不应该成功的时候成功了。
                self.raw_unlock();
                false
            } else {
                *self.write_locked.get() = true;
                true
            }
        } else {
            false
        }
    }
    #[inline]
    unsafe fn raw_unlock(&self) {
        let r = libc::pthread_rwlock_unlock(self.inner.get());
        debug_assert_eq!(r, 0);
    }
    #[inline]
    pub unsafe fn read_unlock(&self) {
        debug_assert!(!*self.write_locked.get());
        self.num_readers.fetch_sub(1, Ordering::Relaxed);
        self.raw_unlock();
    }
    #[inline]
    pub unsafe fn write_unlock(&self) {
        debug_assert_eq!(self.num_readers.load(Ordering::Relaxed), 0);
        debug_assert!(*self.write_locked.get());
        *self.write_locked.get() = false;
        self.raw_unlock();
    }
    #[inline]
    unsafe fn destroy(&mut self) {
        let r = libc::pthread_rwlock_destroy(self.inner.get());
        // 在 DragonFly 上，如果在刚刚用 libc::PTHREAD_RWLOCK_INITIALIZER 初始化的 rwlock 上调用，则 pthread_rwlock_destroy() 返回 EINVAL。
        // 一旦使用了 (locked/unlocked) 或 pthread_rwlock_init()，将不再发生此行为。
        //
        //
        if cfg!(target_os = "dragonfly") {
            debug_assert!(r == 0 || r == libc::EINVAL);
        } else {
            debug_assert_eq!(r, 0);
        }
    }
}

impl Drop for RwLock {
    #[inline]
    fn drop(&mut self) {
        unsafe { self.destroy() };
    }
}
