use crate::cell::UnsafeCell;
use crate::sync::atomic::{AtomicUsize, Ordering};
use crate::sys_common::lazy_box::{LazyBox, LazyInit};

pub struct RwLock {
    inner: UnsafeCell<libc::pthread_rwlock_t>,
    write_locked: UnsafeCell<bool>, // guarded by the `inner` RwLock <br>由 `inner` RwLock 保护<br>
    num_readers: AtomicUsize,
}

pub(crate) type MovableRwLock = LazyBox<RwLock>;

unsafe impl Send for RwLock {}
unsafe impl Sync for RwLock {}

impl LazyInit for RwLock {
    fn init() -> Box<Self> {
        Box::new(Self::new())
    }
}

impl RwLock {
    pub const fn new() -> RwLock {
        RwLock {
            inner: UnsafeCell::new(libc::PTHREAD_RWLOCK_INITIALIZER),
            write_locked: UnsafeCell::new(false),
            num_readers: AtomicUsize::new(0),
        }
    }
    #[inline]
    pub unsafe fn read(&self) {
        let r = libc::pthread_rwlock_rdlock(self.inner.get());

        // According to POSIX, when a thread tries to acquire this read lock while it already holds the write lock (or vice versa, or tries to acquire the write lock twice), "the call shall either deadlock or return [EDEADLK]" <br>根据 POSIX，当一个线程在它已经持有写锁的情况下试图获取这个读锁时 (反之亦然，或者试图获取写锁两次)，这个调用要么死锁，要么返回 [EDEADLK]<br>
        //
        // (https://pubs.opengroup.org/onlinepubs/9699919799/functions/pthread_rwlock_wrlock.html,
        // https://pubs.opengroup.org/onlinepubs/9699919799/functions/pthread_rwlock_rdlock.html).
        // So, in principle, all we have to do here is check `r == 0` to be sure we properly got the lock. <br>因此，原则上，我们要做的就是检查 `r == 0`，以确保我们正确地获得了锁。<br>
        //
        // However, (at least) glibc before version 2.25 does not conform to this spec, and can return `r == 0` even when this thread already holds the write lock. <br>但是，(至少) 版本 2.25 之前的 glibc 不符合该规范，并且即使该线程已持有写锁，它也可以返回 `r == 0`。<br>
        // We thus check for this situation ourselves and panic when detecting that a thread got the write lock more than once, or got a read and a write lock. <br>因此，当我们检测到线程多次获得写锁定，或者获得读和写锁定时，我们自己和 panic 都检查了这种情况。<br>
        //
        //
        //
        //
        //
        if r == libc::EAGAIN {
            panic!("rwlock maximum reader count exceeded");
        } else if r == libc::EDEADLK || (r == 0 && *self.write_locked.get()) {
            // Above, we make sure to only access `write_locked` when `r == 0` to avoid data races. <br>上面，我们确保仅在 `r == 0` 时访问 `write_locked`，以避免数据竞争。<br>
            //
            if r == 0 {
                // `pthread_rwlock_rdlock` succeeded when it should not have. <br>`pthread_rwlock_rdlock` 在它不应该成功的时候成功了。<br>
                self.raw_unlock();
            }
            panic!("rwlock read lock would result in deadlock");
        } else {
            // POSIX does not make guarantees about all the errors that may be returned. <br>POSIX 不保证所有可能返回的错误。<br>
            // See issue #94705 for more details. <br>有关详细信息，请参见 issue #94705。<br>
            assert_eq!(r, 0, "unexpected error during rwlock read lock: {:?}", r);
            self.num_readers.fetch_add(1, Ordering::Relaxed);
        }
    }
    #[inline]
    pub unsafe fn try_read(&self) -> bool {
        let r = libc::pthread_rwlock_tryrdlock(self.inner.get());
        if r == 0 {
            if *self.write_locked.get() {
                // `pthread_rwlock_tryrdlock` succeeded when it should not have. <br>`pthread_rwlock_tryrdlock` 在它不应该成功的时候成功了。<br>
                self.raw_unlock();
                false
            } else {
                self.num_readers.fetch_add(1, Ordering::Relaxed);
                true
            }
        } else {
            false
        }
    }
    #[inline]
    pub unsafe fn write(&self) {
        let r = libc::pthread_rwlock_wrlock(self.inner.get());
        // See comments above for why we check for EDEADLK and write_locked. <br>有关我们为什么检查 EDEADLK 和 write_locked 的信息，请参见上面的评论。<br>
        // For the same reason, we also need to check that there are no readers (tracked in `num_readers`). <br>出于同样的原因，我们还需要检查是否没有 readers (在 `num_readers` 中跟踪)。<br>
        if r == libc::EDEADLK
            || (r == 0 && *self.write_locked.get())
            || self.num_readers.load(Ordering::Relaxed) != 0
        {
            // Above, we make sure to only access `write_locked` when `r == 0` to avoid data races. <br>上面，我们确保仅在 `r == 0` 时访问 `write_locked`，以避免数据竞争。<br>
            //
            if r == 0 {
                // `pthread_rwlock_wrlock` succeeded when it should not have. <br>`pthread_rwlock_wrlock` 在它不应该成功的时候成功了。<br>
                self.raw_unlock();
            }
            panic!("rwlock write lock would result in deadlock");
        } else {
            // According to POSIX, for a properly initialized rwlock this can only return EDEADLK or 0. <br>根据 POSIX，对于正确初始化的 rwlock，此操作只能返回 EDEADLK 或 0。<br>
            // We rely on that. <br>我们依靠这一点。<br>
            debug_assert_eq!(r, 0);
        }
        *self.write_locked.get() = true;
    }
    #[inline]
    pub unsafe fn try_write(&self) -> bool {
        let r = libc::pthread_rwlock_trywrlock(self.inner.get());
        if r == 0 {
            if *self.write_locked.get() || self.num_readers.load(Ordering::Relaxed) != 0 {
                // `pthread_rwlock_trywrlock` succeeded when it should not have. <br>`pthread_rwlock_trywrlock` 在它不应该成功的时候成功了。<br>
                self.raw_unlock();
                false
            } else {
                *self.write_locked.get() = true;
                true
            }
        } else {
            false
        }
    }
    #[inline]
    unsafe fn raw_unlock(&self) {
        let r = libc::pthread_rwlock_unlock(self.inner.get());
        debug_assert_eq!(r, 0);
    }
    #[inline]
    pub unsafe fn read_unlock(&self) {
        debug_assert!(!*self.write_locked.get());
        self.num_readers.fetch_sub(1, Ordering::Relaxed);
        self.raw_unlock();
    }
    #[inline]
    pub unsafe fn write_unlock(&self) {
        debug_assert_eq!(self.num_readers.load(Ordering::Relaxed), 0);
        debug_assert!(*self.write_locked.get());
        *self.write_locked.get() = false;
        self.raw_unlock();
    }
    #[inline]
    unsafe fn destroy(&mut self) {
        let r = libc::pthread_rwlock_destroy(self.inner.get());
        // On DragonFly pthread_rwlock_destroy() returns EINVAL if called on a rwlock that was just initialized with libc::PTHREAD_RWLOCK_INITIALIZER. <br>在 DragonFly 上，如果在刚刚用 libc::PTHREAD_RWLOCK_INITIALIZER 初始化的 rwlock 上调用，则 pthread_rwlock_destroy() 返回 EINVAL。<br>
        // Once it is used (locked/unlocked) or pthread_rwlock_init() is called, this behaviour no longer occurs. <br>一旦使用了 (locked/unlocked) 或 pthread_rwlock_init()，将不再发生此行为。<br>
        //
        //
        if cfg!(target_os = "dragonfly") {
            debug_assert!(r == 0 || r == libc::EINVAL);
        } else {
            debug_assert_eq!(r, 0);
        }
    }
}

impl Drop for RwLock {
    #[inline]
    fn drop(&mut self) {
        unsafe { self.destroy() };
    }
}
