#![allow(missing_docs, nonstandard_style)]

use crate::ffi::CStr;
use crate::io::ErrorKind;

pub use self::rand::hashmap_random_keys;

#[cfg(not(target_os = "espidf"))]
#[macro_use]
pub mod weak;

pub mod alloc;
pub mod android;
pub mod args;
#[path = "../unix/cmath.rs"]
pub mod cmath;
pub mod env;
pub mod fd;
pub mod fs;
pub mod futex;
pub mod io;
#[cfg(any(target_os = "linux", target_os = "android"))]
pub mod kernel_copy;
#[cfg(target_os = "l4re")]
mod l4re;
pub mod locks;
pub mod memchr;
#[cfg(not(target_os = "l4re"))]
pub mod net;
#[cfg(target_os = "l4re")]
pub use self::l4re::net;
pub mod os;
pub mod os_str;
pub mod path;
pub mod pipe;
pub mod process;
pub mod rand;
pub mod stack_overflow;
pub mod stdio;
pub mod thread;
pub mod thread_local_dtor;
pub mod thread_local_key;
pub mod thread_parker;
pub mod time;

#[cfg(target_os = "espidf")]
pub fn init(argc: isize, argv: *const *const u8) {}

#[cfg(not(target_os = "espidf"))]
// SAFETY: must be called only once during runtime initialization. <br>在运行时初始化期间只能调用一次。<br>
// NOTE: this is not guaranteed to run, for example when Rust code is called externally. <br>这不能保证运行，例如在外部调用 Rust 代码时。<br>
pub unsafe fn init(argc: isize, argv: *const *const u8) {
    // The standard streams might be closed on application startup. <br>标准流可能会在应用程序启动时关闭。<br>
    // To prevent std::io::{stdin, stdout,stderr} objects from using other unrelated file resources opened later, we reopen standards streams when they are closed. <br>为防止 `std::io::{stdin，stdout，stderr}` 对象使用稍后打开的其他不相关文件资源，我们在关闭标准流后会重新打开它们。<br>
    //
    sanitize_standard_fds();

    // By default, some platforms will send a *signal* when an EPIPE error would otherwise be delivered. <br>默认情况下，某些平台会在否则会传递 EPIPE 错误的情况下发送 *信号*。<br>
    // This runtime doesn't install a SIGPIPE handler, causing it to kill the program, which isn't exactly what we want! <br>此运行时未安装 SIGPIPE 处理程序，导致它终止了程序，这不正是我们想要的！<br>
    //
    //
    // Hence, we set SIGPIPE to ignore when the program starts up in order to prevent this problem. <br>因此，为了防止出现此问题，我们将 SIGPIPE 设置为在程序启动时忽略。<br>
    //
    //
    reset_sigpipe();

    stack_overflow::init();
    args::init(argc, argv);

    // Normally, `thread::spawn` will call `Thread::set_name` but since this thread already exists, we have to call it ourselves. <br>通常，`thread::spawn` 会调用 `Thread::set_name`，但由于该线程已经存在，我们必须自己调用它。<br>
    // We only do this on macos because some unix-like operating systems such as Linux share process-id and thread-id for the main thread and so renaming the main thread will rename the process and we only want to enable this on platforms we've tested. <br>我们只在 macos 上这样做，因为一些类似 unix 的操作系统，例如 Linux 共享主线程的 process-id 和 thread-id，因此重命名主线程将重命名进程，我们只想在我们已经启用的平台上启用它测试。<br>
    //
    //
    //
    if cfg!(target_os = "macos") {
        thread::Thread::set_name(&CStr::from_bytes_with_nul_unchecked(b"main\0"));
    }

    unsafe fn sanitize_standard_fds() {
        // fast path with a single syscall for systems with poll() <br>带有 poll() 的系统的单个 syscall 的快速路径<br>
        #[cfg(not(any(
            miri,
            target_os = "emscripten",
            target_os = "fuchsia",
            target_os = "vxworks",
            // The poll on Darwin doesn't set POLLNVAL for closed fds. <br>达尔文 (Darwin) 的民意调查并未为封闭的 fds 设置 POLLNVAL。<br>
            target_os = "macos",
            target_os = "ios",
            target_os = "redox",
            target_os = "l4re",
            target_os = "horizon",
        )))]
        'poll: {
            use crate::sys::os::errno;
            let pfds: &mut [_] = &mut [
                libc::pollfd { fd: 0, events: 0, revents: 0 },
                libc::pollfd { fd: 1, events: 0, revents: 0 },
                libc::pollfd { fd: 2, events: 0, revents: 0 },
            ];

            while libc::poll(pfds.as_mut_ptr(), 3, 0) == -1 {
                match errno() {
                    libc::EINTR => continue,
                    libc::EINVAL | libc::EAGAIN | libc::ENOMEM => {
                        // RLIMIT_NOFILE or temporary allocation failures may be preventing use of poll(), fall back to fcntl <br>RLIMIT_NOFILE 或临时分配失败可能会阻止使用 poll()，回退到 fcntl<br>
                        //
                        break 'poll;
                    }
                    _ => libc::abort(),
                }
            }
            for pfd in pfds {
                if pfd.revents & libc::POLLNVAL == 0 {
                    continue;
                }
                if libc::open("/dev/null\0".as_ptr().cast(), libc::O_RDWR, 0) == -1 {
                    // If the stream is closed but we failed to reopen it, abort the process. <br>如果流已关闭，但未能重新打开它，则中止该进程。<br>
                    // Otherwise we wouldn't preserve the safety of operations on the corresponding Rust object Stdin, Stdout, or Stderr. <br>否则，我们不会在相应的 Rust 对象 Stdin、Stdout 或 Stderr 上保持操作的安全性。<br>
                    //
                    //
                    libc::abort();
                }
            }
            return;
        }

        // fallback in case poll isn't available or limited by RLIMIT_NOFILE <br>如果轮询不可用或受 RLIMIT_NOFILE 限制，则回退<br>
        #[cfg(not(any(
            // The standard fds are always available in Miri. <br>标准 fds 始终在 Miri 中可用。<br>
            miri,
            target_os = "emscripten",
            target_os = "fuchsia",
            target_os = "vxworks",
            target_os = "l4re",
            target_os = "horizon",
        )))]
        {
            use crate::sys::os::errno;
            for fd in 0..3 {
                if libc::fcntl(fd, libc::F_GETFD) == -1 && errno() == libc::EBADF {
                    if libc::open("/dev/null\0".as_ptr().cast(), libc::O_RDWR, 0) == -1 {
                        // If the stream is closed but we failed to reopen it, abort the process. <br>如果流已关闭，但未能重新打开它，则中止该进程。<br>
                        // Otherwise we wouldn't preserve the safety of operations on the corresponding Rust object Stdin, Stdout, or Stderr. <br>否则，我们不会在相应的 Rust 对象 Stdin、Stdout 或 Stderr 上保持操作的安全性。<br>
                        //
                        //
                        libc::abort();
                    }
                }
            }
        }
    }

    unsafe fn reset_sigpipe() {
        #[cfg(not(any(target_os = "emscripten", target_os = "fuchsia", target_os = "horizon")))]
        rtassert!(signal(libc::SIGPIPE, libc::SIG_IGN) != libc::SIG_ERR);
    }
}

// SAFETY: must be called only once during runtime cleanup. <br>在运行时清理期间只能调用一次。<br>
// NOTE: this is not guaranteed to run, for example when the program aborts. <br>这不能保证运行，例如当程序中止时。<br>
pub unsafe fn cleanup() {
    stack_overflow::cleanup();
}

#[cfg(target_os = "android")]
pub use crate::sys::android::signal;
#[cfg(not(target_os = "android"))]
pub use libc::signal;

pub fn decode_error_kind(errno: i32) -> ErrorKind {
    use ErrorKind::*;
    match errno as libc::c_int {
        libc::E2BIG => ArgumentListTooLong,
        libc::EADDRINUSE => AddrInUse,
        libc::EADDRNOTAVAIL => AddrNotAvailable,
        libc::EBUSY => ResourceBusy,
        libc::ECONNABORTED => ConnectionAborted,
        libc::ECONNREFUSED => ConnectionRefused,
        libc::ECONNRESET => ConnectionReset,
        libc::EDEADLK => Deadlock,
        libc::EDQUOT => FilesystemQuotaExceeded,
        libc::EEXIST => AlreadyExists,
        libc::EFBIG => FileTooLarge,
        libc::EHOSTUNREACH => HostUnreachable,
        libc::EINTR => Interrupted,
        libc::EINVAL => InvalidInput,
        libc::EISDIR => IsADirectory,
        libc::ELOOP => FilesystemLoop,
        libc::ENOENT => NotFound,
        libc::ENOMEM => OutOfMemory,
        libc::ENOSPC => StorageFull,
        libc::ENOSYS => Unsupported,
        libc::EMLINK => TooManyLinks,
        libc::ENAMETOOLONG => InvalidFilename,
        libc::ENETDOWN => NetworkDown,
        libc::ENETUNREACH => NetworkUnreachable,
        libc::ENOTCONN => NotConnected,
        libc::ENOTDIR => NotADirectory,
        libc::ENOTEMPTY => DirectoryNotEmpty,
        libc::EPIPE => BrokenPipe,
        libc::EROFS => ReadOnlyFilesystem,
        libc::ESPIPE => NotSeekable,
        libc::ESTALE => StaleNetworkFileHandle,
        libc::ETIMEDOUT => TimedOut,
        libc::ETXTBSY => ExecutableFileBusy,
        libc::EXDEV => CrossesDevices,

        libc::EACCES | libc::EPERM => PermissionDenied,

        // These two constants can have the same value on some systems, but different values on others, so we can't use a match clause <br>这两个常量在某些系统上可以具有相同的值，但在其他系统上可以具有不同的值，因此我们不能使用 match 子句<br>
        //
        //
        x if x == libc::EAGAIN || x == libc::EWOULDBLOCK => WouldBlock,

        _ => Uncategorized,
    }
}

#[doc(hidden)]
pub trait IsMinusOne {
    fn is_minus_one(&self) -> bool;
}

macro_rules! impl_is_minus_one {
    ($($t:ident)*) => ($(impl IsMinusOne for $t {
        fn is_minus_one(&self) -> bool {
            *self == -1
        }
    })*)
}

impl_is_minus_one! { i8 i16 i32 i64 isize }

pub fn cvt<T: IsMinusOne>(t: T) -> crate::io::Result<T> {
    if t.is_minus_one() { Err(crate::io::Error::last_os_error()) } else { Ok(t) }
}

pub fn cvt_r<T, F>(mut f: F) -> crate::io::Result<T>
where
    T: IsMinusOne,
    F: FnMut() -> T,
{
    loop {
        match cvt(f()) {
            Err(ref e) if e.kind() == ErrorKind::Interrupted => {}
            other => return other,
        }
    }
}

#[allow(dead_code)] // Not used on all platforms. <br>并非在所有平台上都使用。<br>
pub fn cvt_nz(error: libc::c_int) -> crate::io::Result<()> {
    if error == 0 { Ok(()) } else { Err(crate::io::Error::from_raw_os_error(error)) }
}

// libc::abort() will run the SIGABRT handler. <br>libc::abort () 将运行 SIGABRT 处理程序。<br> That's fine because anyone who installs a SIGABRT handler already has to expect it to run in Very Bad situations (eg, malloc crashing). <br>这很好，因为任何安装 SIGABRT 处理程序的人都必须期望它在非常糟糕的情况下运行 (例如，malloc 崩溃)。<br>
//
// Current glibc's abort() function unblocks SIGABRT, raises SIGABRT, clears the SIGABRT handler and raises it again, and then starts to get creative. <br>当前 glibc 的 abort() 函数解除对 SIGABRT 的阻塞，引发 SIGABRT，清除 SIGABRT 处理程序并再次引发它，然后开始发挥创意。<br>
//
// See the public documentation for `intrinsics::abort()` and `process::abort()` for further discussion. <br>有关进一步讨论，请参见 `intrinsics::abort()` 和 `process::abort()` 的公共文档。<br>
//
// There is confusion about whether libc::abort() flushes stdio streams. <br>关于 libc::abort () 是否刷新标准输入输出流存在混淆。<br>
// libc::abort() is required by ISO C 99 (7.14.1.1p5) to be async-signal-safe, so flushing streams is at least extremely hard, if not entirely impossible. <br>ISO C 99 (7.14.1.1p5) 要求 libc::abort () 是异步信号安全的，因此刷新流至少非常困难，如果不是完全不可能的话。<br>
//
// However, some versions of POSIX (eg IEEE Std 1003.1-2001) required abort to do so. <br>但是，某些版本的 POSIX (例如 IEEE Std 1003.1-2001) 需要终止来执行此操作。<br> In 1003.1-2004 this was fixed. <br>在 1003.1-2004 中，这是确定的。<br>
//
// glibc's implementation did the flush, unsafely, before glibc commit <br>glibc 的实现在 glibc 提交之前不安全地进行了刷新<br>
// 91e7cf982d01 `abort: Do not flush stdio streams [BZ #15436]' by Florian Weimer. <br>91e7cf982d01 `关于：不要刷新标准输入输出流 [BZ #15436] by Florian Weimer。<br> According to glibc's NEWS: <br>根据 glibc 的新闻：<br>
//
//    The abort function terminates the process immediately, without flushing stdio streams. <br>终止函数立即终止该过程，而不刷新标准输入输出流。<br> Previous glibc versions used to flush streams, resulting in deadlocks and further data corruption. <br>以前的 glibc 版本用于刷新流，导致死锁和进一步的数据损坏。<br>
//    This change also affects process aborts as the result of assertion failures. <br>由于断言失败，此更改还会影响进程中止。<br>
//
// This is an accurate description of the problem. <br>这是对问题的准确描述。<br>
// The only solution for program with nontrivial use of C stdio is a fixed libc - one which does not try to flush in abort - since even libc-internal errors, and assertion failures generated from C, will go via abort(). <br>对于非平凡使用 C 标准输入输出的程序，唯一的解决方案是固定的 libc - 一个不会尝试在终止中刷新 - 因为即使是 libc 内部错误，以及由 C 产生的断言失败，也会通过 abort()。<br>
//
//
// On systems with old, buggy, libcs, the impact can be severe for a multithreaded C program. <br>在具有旧的 buggy、libcs 的系统上，对多线程 C 程序的影响可能很严重。<br> It is much less severe for Rust, because Rust stdlib doesn't use libc stdio buffering. <br>Rust 的严重程度要低得多，因为 Rust stdlib 不使用 libc 标准输入输出缓冲。<br>
// In a typical Rust program, which does not use C stdio, even a buggy libc::abort() is, in fact, safe. <br>在典型的 Rust 程序中，不使用 C 标准输入输出，即使是 buggy libc::abort () 实际上也是安全的。<br>
//
//
//
//
//
//
//
//
//
//
//
//
pub fn abort_internal() -> ! {
    unsafe { libc::abort() }
}

cfg_if::cfg_if! {
    if #[cfg(target_os = "android")] {
        #[link(name = "dl")]
        #[link(name = "log")]
        extern "C" {}
    } else if #[cfg(target_os = "freebsd")] {
        #[link(name = "execinfo")]
        #[link(name = "pthread")]
        extern "C" {}
    } else if #[cfg(target_os = "netbsd")] {
        #[link(name = "pthread")]
        #[link(name = "rt")]
        extern "C" {}
    } else if #[cfg(any(target_os = "dragonfly", target_os = "openbsd"))] {
        #[link(name = "pthread")]
        extern "C" {}
    } else if #[cfg(target_os = "solaris")] {
        #[link(name = "socket")]
        #[link(name = "posix4")]
        #[link(name = "pthread")]
        #[link(name = "resolv")]
        extern "C" {}
    } else if #[cfg(target_os = "illumos")] {
        #[link(name = "socket")]
        #[link(name = "posix4")]
        #[link(name = "pthread")]
        #[link(name = "resolv")]
        #[link(name = "nsl")]
        // Use libumem for the (malloc-compatible) allocator <br>对 (malloc-compatible) 分配器使用 libumem<br>
        #[link(name = "umem")]
        extern "C" {}
    } else if #[cfg(target_os = "macos")] {
        #[link(name = "System")]
        // res_init and friends require -lresolv on macOS/iOS. <br>res_init 和朋友需要在 macOS/iOS 上使用 -lresolv。<br>
        // See #41582 and https://blog.achernya.com/2013/03/os-x-has-silly-libsystem.html <br>请参见 #41582 和 https://blog.achernya.com/2013/03/os-x-has-silly-libsystem.html<br>
        #[link(name = "resolv")]
        extern "C" {}
    } else if #[cfg(target_os = "ios")] {
        #[link(name = "System")]
        #[link(name = "objc")]
        #[link(name = "Security", kind = "framework")]
        #[link(name = "Foundation", kind = "framework")]
        #[link(name = "resolv")]
        extern "C" {}
    } else if #[cfg(target_os = "fuchsia")] {
        #[link(name = "zircon")]
        #[link(name = "fdio")]
        extern "C" {}
    } else if #[cfg(all(target_os = "linux", target_env = "uclibc"))] {
        #[link(name = "dl")]
        extern "C" {}
    }
}

#[cfg(any(target_os = "espidf", target_os = "horizon"))]
mod unsupported {
    use crate::io;

    pub fn unsupported<T>() -> io::Result<T> {
        Err(unsupported_err())
    }

    pub fn unsupported_err() -> io::Error {
        io::const_io_error!(io::ErrorKind::Unsupported, "operation not supported on this platform",)
    }
}
