use super::*;

use crate::ffi::OsStr;
use crate::mem;
use crate::ptr;
use crate::sys::{cvt, cvt_nz};

macro_rules! t {
    ($e:expr) => {
        match $e {
            Ok(t) => t,
            Err(e) => panic!("received error for `{}`: {}", stringify!($e), e),
        }
    };
}

#[test]
#[cfg_attr(
    any(
        // See #14232 for more information, but it appears that signal delivery to a newly spawned process may just be raced in the macOS, so to prevent this test from being flaky we ignore it on macOS. <br>有关更多信息，请参见 #14232。但是似乎可以在 macOS 中竞争向新生成的进程的信号传递，因此，为防止此测试不稳定，我们在 macOS 上将其忽略。<br>
        //
        //
        target_os = "macos",
        // When run under our current QEMU emulation test suite this test fails, although the reason isn't very clear as to why. <br>当在我们当前的 QEMU 仿真测试套件下运行时，该测试失败，尽管原因尚不清楚。<br>
        // For now this test is ignored there. <br>目前，此测试在那里被忽略。<br>
        //
        target_arch = "arm",
        target_arch = "aarch64",
        target_arch = "riscv64",
    ),
    ignore
)]
fn test_process_mask() {
    unsafe {
        // Test to make sure that a signal mask does not get inherited. <br>测试以确保信号掩码不会被继承。<br>
        let mut cmd = Command::new(OsStr::new("cat"));

        let mut set = mem::MaybeUninit::<libc::sigset_t>::uninit();
        let mut old_set = mem::MaybeUninit::<libc::sigset_t>::uninit();
        t!(cvt(sigemptyset(set.as_mut_ptr())));
        t!(cvt(sigaddset(set.as_mut_ptr(), libc::SIGINT)));
        t!(cvt_nz(libc::pthread_sigmask(libc::SIG_SETMASK, set.as_ptr(), old_set.as_mut_ptr())));

        cmd.stdin(Stdio::MakePipe);
        cmd.stdout(Stdio::MakePipe);

        let (mut cat, mut pipes) = t!(cmd.spawn(Stdio::Null, true));
        let stdin_write = pipes.stdin.take().unwrap();
        let stdout_read = pipes.stdout.take().unwrap();

        t!(cvt_nz(libc::pthread_sigmask(libc::SIG_SETMASK, old_set.as_ptr(), ptr::null_mut())));

        t!(cvt(libc::kill(cat.id() as libc::pid_t, libc::SIGINT)));
        // We need to wait until SIGINT is definitely delivered. <br>我们需要等到 SIGINT 正式交付为止。<br>
        // The easiest way is to write something to cat, and try to read it back: if SIGINT is unmasked, it'll get delivered when cat is next scheduled. <br>最简单的方法是写一些东西给 cat，然后试着读回去：如果 SIGINT 被取消屏蔽，它将在下次安排 cat 时交付。<br>
        //
        //
        let _ = stdin_write.write(b"Hello");
        drop(stdin_write);

        // Either EOF or failure (EPIPE) is okay. <br>EOF 或 (EPIPE) 故障都可以。<br>
        let mut buf = [0; 5];
        if let Ok(ret) = stdout_read.read(&mut buf) {
            assert_eq!(ret, 0);
        }

        t!(cat.wait());
    }
}

#[test]
#[cfg_attr(
    any(
        // See test_process_mask <br>请参见 test_process_mask<br>
        target_os = "macos",
        target_arch = "arm",
        target_arch = "aarch64",
        target_arch = "riscv64",
    ),
    ignore
)]
fn test_process_group_posix_spawn() {
    unsafe {
        // Spawn a cat subprocess that's just going to hang since there is no I/O. <br>Spawn 是一个 cat 子进程，由于没有 I/O，它将挂起。<br>
        let mut cmd = Command::new(OsStr::new("cat"));
        cmd.pgroup(0);
        cmd.stdin(Stdio::MakePipe);
        cmd.stdout(Stdio::MakePipe);
        let (mut cat, _pipes) = t!(cmd.spawn(Stdio::Null, true));

        // Check that we can kill its process group, which means there *is* one. <br>检查我们是否可以杀死它的进程组，这意味着*存在*一个。<br>
        t!(cvt(libc::kill(-(cat.id() as libc::pid_t), libc::SIGINT)));

        t!(cat.wait());
    }
}

#[test]
#[cfg_attr(
    any(
        // See test_process_mask <br>请参见 test_process_mask<br>
        target_os = "macos",
        target_arch = "arm",
        target_arch = "aarch64",
        target_arch = "riscv64",
    ),
    ignore
)]
fn test_process_group_no_posix_spawn() {
    unsafe {
        // Same as above, create hang-y cat. <br>同上，创建 hang-y cat。<br> This time, force using the non-posix_spawnp path. <br>这一次，强制使用 non-posix_spawnp 路径。<br>
        let mut cmd = Command::new(OsStr::new("cat"));
        cmd.pgroup(0);
        cmd.pre_exec(Box::new(|| Ok(()))); // pre_exec forces fork + exec <br>pre_exec 强制 fork + exec<br>
        cmd.stdin(Stdio::MakePipe);
        cmd.stdout(Stdio::MakePipe);
        let (mut cat, _pipes) = t!(cmd.spawn(Stdio::Null, true));

        // Check that we can kill its process group, which means there *is* one. <br>检查我们是否可以杀死它的进程组，这意味着*存在*一个。<br>
        t!(cvt(libc::kill(-(cat.id() as libc::pid_t), libc::SIGINT)));

        t!(cat.wait());
    }
}
