use crate::fmt;
use crate::io;
use crate::mem;
use crate::num::{NonZeroI32, NonZeroI64};
use crate::ptr;

use crate::sys::process::process_common::*;
use crate::sys::process::zircon::{zx_handle_t, Handle};

use libc::{c_int, size_t};

////////////////////////////////////////////////////////////////////////////////
// Command
////////////////////////////////////////////////////////////////////////////////

impl Command {
    pub fn spawn(
        &mut self,
        default: Stdio,
        needs_stdin: bool,
    ) -> io::Result<(Process, StdioPipes)> {
        let envp = self.capture_env();

        if self.saw_nul() {
            return Err(io::const_io_error!(
                io::ErrorKind::InvalidInput,
                "nul byte found in provided data",
            ));
        }

        let (ours, theirs) = self.setup_io(default, needs_stdin)?;

        let process_handle = unsafe { self.do_exec(theirs, envp.as_ref())? };

        Ok((Process { handle: Handle::new(process_handle) }, ours))
    }

    pub fn exec(&mut self, default: Stdio) -> io::Error {
        if self.saw_nul() {
            return io::const_io_error!(
                io::ErrorKind::InvalidInput,
                "nul byte found in provided data",
            );
        }

        match self.setup_io(default, true) {
            Ok((_, _)) => {
                // FIXME: This is tough because we don't support the exec syscalls <br>这很困难，因为我们不支持 exec syscalls<br>
                unimplemented!();
            }
            Err(e) => e,
        }
    }

    unsafe fn do_exec(
        &mut self,
        stdio: ChildPipes,
        maybe_envp: Option<&CStringArray>,
    ) -> io::Result<zx_handle_t> {
        use crate::sys::process::zircon::*;

        let envp = match maybe_envp {
            // None means to clone the current environment, which is done in the flags below. <br>None 表示克隆当前环境，这在下面的标志中完成。<br>
            //
            None => ptr::null(),
            Some(envp) => envp.as_ptr(),
        };

        let make_action = |local_io: &ChildStdio, target_fd| -> io::Result<fdio_spawn_action_t> {
            if let Some(local_fd) = local_io.fd() {
                Ok(fdio_spawn_action_t {
                    action: FDIO_SPAWN_ACTION_TRANSFER_FD,
                    local_fd,
                    target_fd,
                    ..Default::default()
                })
            } else {
                if let ChildStdio::Null = local_io {
                    // acts as no-op <br>充当无操作者<br>
                    return Ok(Default::default());
                }

                let mut handle = ZX_HANDLE_INVALID;
                let status = fdio_fd_clone(target_fd, &mut handle);
                if status == ERR_INVALID_ARGS || status == ERR_NOT_SUPPORTED {
                    // This descriptor is closed; <br>该描述符已关闭；<br> skip it rather than generating an error. <br>跳过它而不是生成错误。<br>
                    //
                    return Ok(Default::default());
                }
                zx_cvt(status)?;

                let mut cloned_fd = 0;
                zx_cvt(fdio_fd_create(handle, &mut cloned_fd))?;

                Ok(fdio_spawn_action_t {
                    action: FDIO_SPAWN_ACTION_TRANSFER_FD,
                    local_fd: cloned_fd as i32,
                    target_fd,
                    ..Default::default()
                })
            }
        };

        // Clone stdin, stdout, and stderr <br>克隆 stdin，stdout 和 stderr<br>
        let action1 = make_action(&stdio.stdin, 0)?;
        let action2 = make_action(&stdio.stdout, 1)?;
        let action3 = make_action(&stdio.stderr, 2)?;
        let actions = [action1, action2, action3];

        // We don't want FileDesc::drop to be called on any stdio. <br>我们不希望在任何标准输入输出上调用 FileDesc::drop。<br>
        // fdio_spawn_etc always consumes transferred file descriptors. <br>fdio_spawn_etc 始终消费传输的文件描述符。<br>
        mem::forget(stdio);

        for callback in self.get_closures().iter_mut() {
            callback()?;
        }

        let mut process_handle: zx_handle_t = 0;
        zx_cvt(fdio_spawn_etc(
            ZX_HANDLE_INVALID,
            FDIO_SPAWN_CLONE_JOB
                | FDIO_SPAWN_CLONE_LDSVC
                | FDIO_SPAWN_CLONE_NAMESPACE
                | FDIO_SPAWN_CLONE_ENVIRON // this is ignored when envp is non-null <br>当 envp 为非 null 时，将忽略此设置<br>
                | FDIO_SPAWN_CLONE_UTC_CLOCK,
            self.get_program_cstr().as_ptr(),
            self.get_argv().as_ptr(),
            envp,
            actions.len() as size_t,
            actions.as_ptr(),
            &mut process_handle,
            ptr::null_mut(),
        ))?;
        // FIXME: See if we want to do something with that err_msg <br>看看我们是否想对那个 err_msg 做些什么<br>

        Ok(process_handle)
    }
}

////////////////////////////////////////////////////////////////////////////////
// Processes
////////////////////////////////////////////////////////////////////////////////

pub struct Process {
    handle: Handle,
}

impl Process {
    pub fn id(&self) -> u32 {
        self.handle.raw() as u32
    }

    pub fn kill(&mut self) -> io::Result<()> {
        use crate::sys::process::zircon::*;

        unsafe {
            zx_cvt(zx_task_kill(self.handle.raw()))?;
        }

        Ok(())
    }

    pub fn wait(&mut self) -> io::Result<ExitStatus> {
        use crate::default::Default;
        use crate::sys::process::zircon::*;

        let mut proc_info: zx_info_process_t = Default::default();
        let mut actual: size_t = 0;
        let mut avail: size_t = 0;

        unsafe {
            zx_cvt(zx_object_wait_one(
                self.handle.raw(),
                ZX_TASK_TERMINATED,
                ZX_TIME_INFINITE,
                ptr::null_mut(),
            ))?;
            zx_cvt(zx_object_get_info(
                self.handle.raw(),
                ZX_INFO_PROCESS,
                &mut proc_info as *mut _ as *mut libc::c_void,
                mem::size_of::<zx_info_process_t>(),
                &mut actual,
                &mut avail,
            ))?;
        }
        if actual != 1 {
            return Err(io::const_io_error!(
                io::ErrorKind::InvalidData,
                "Failed to get exit status of process",
            ));
        }
        Ok(ExitStatus(proc_info.return_code))
    }

    pub fn try_wait(&mut self) -> io::Result<Option<ExitStatus>> {
        use crate::default::Default;
        use crate::sys::process::zircon::*;

        let mut proc_info: zx_info_process_t = Default::default();
        let mut actual: size_t = 0;
        let mut avail: size_t = 0;

        unsafe {
            let status =
                zx_object_wait_one(self.handle.raw(), ZX_TASK_TERMINATED, 0, ptr::null_mut());
            match status {
                0 => {} // Success
                x if x == ERR_TIMED_OUT => {
                    return Ok(None);
                }
                _ => {
                    panic!("Failed to wait on process handle: {status}");
                }
            }
            zx_cvt(zx_object_get_info(
                self.handle.raw(),
                ZX_INFO_PROCESS,
                &mut proc_info as *mut _ as *mut libc::c_void,
                mem::size_of::<zx_info_process_t>(),
                &mut actual,
                &mut avail,
            ))?;
        }
        if actual != 1 {
            return Err(io::const_io_error!(
                io::ErrorKind::InvalidData,
                "Failed to get exit status of process",
            ));
        }
        Ok(Some(ExitStatus(proc_info.return_code)))
    }
}

#[derive(PartialEq, Eq, Clone, Copy, Debug)]
pub struct ExitStatus(i64);

impl ExitStatus {
    pub fn exit_ok(&self) -> Result<(), ExitStatusError> {
        match NonZeroI64::try_from(self.0) {
            /* was nonzero */ Ok(failure) => Err(ExitStatusError(failure)),
            /* was zero, couldn't convert */ Err(_) => Ok(()),
        }
    }

    pub fn code(&self) -> Option<i32> {
        // FIXME: support extracting return code as an i64 <br>支持提取返回码作为 i64<br>
        self.0.try_into().ok()
    }

    pub fn signal(&self) -> Option<i32> {
        None
    }

    // FIXME: The actually-Unix implementation in process_unix.rs uses WSTOPSIG, WCOREDUMP et al. <br>process_unix.rs 中的实际 Unix 实现使用 WSTOPSIG，WCOREDUMP 等。<br>
    // I infer from the implementation of `success`, `code` and `signal` above that these are not available on Fuchsia. <br>从上面的 `success`，`code` 和 `signal` 的实现中可以推断出，Fuchsia 没有这些功能。<br>
    //
    // It does not appear that Fuchsia is Unix-like enough to implement ExitStatus (or indeed many other things from std::os::unix) properly. <br>看起来 Fuchsia 似乎不像 Unix 那样足以正确实现 ExitStatus (或者实际上来自 std::os::unix 的许多其他事情)。<br>
    // This veneer is always going to be a bodge. <br>这种贴面总是会成为一个庞然大物。<br>
    // So while I don't know if these implementations are actually correct, I think they will do for now at least. <br>因此，虽然我不知道这些实现是否真的正确，但我认为至少现在可以做到。<br>
    //
    //
    pub fn core_dumped(&self) -> bool {
        false
    }
    pub fn stopped_signal(&self) -> Option<i32> {
        None
    }
    pub fn continued(&self) -> bool {
        false
    }

    pub fn into_raw(&self) -> c_int {
        // We don't know what someone who calls into_raw() will do with this value, but it should have the conventional Unix representation. <br>我们不知道调用 into_raw() 的人会用这个值做什么，但是它应该具有常规的 Unix 表示形式。<br>
        // Despite the fact that this is not standardised in SuS or POSIX, all Unix systems encode the signal and exit status the same way. <br>尽管在 SuS 或 POSIX 中未对此进行标准化，但所有 Unix 系统均以相同的方式对信号和退出状态进行编码。<br>
        // (Ie the WIFEXITED, WEXITSTATUS etc. <br>(即 WIFEXITED，WEXITSTATUS 等。<br>
        // macros have identical behaviour on every Unix.) <br>宏在每个 Unix 上的行为都相同。)<br>
        //
        // The caller of `std::os::unix::into_raw` is probably wanting a Unix exit status, and may do their own shifting and masking, or even pass the status to another computer running a different Unix variant. <br>`std::os::unix::into_raw` 的调用者可能想要 Unix 退出状态，并且可能会进行自己的移动和屏蔽，甚至将状态传递给运行其他 Unix 变体的另一台计算机。<br>
        //
        //
        // The other view would be to say that the caller on Fuchsia ought to know that `into_raw` will give a raw Fuchsia status (whatever that is - I don't know, personally). <br>另一种观点是说， Fuchsia 的调用者应该知道 `into_raw` 会赋予原始的 Fuchsia 状态 (无论如何 - 我个人都不知道)。<br>
        // That is not possible here because we must return a c_int because that's what Unix (including SuS and POSIX) say a wait status is, but Fuchsia apparently uses a u64, so it won't necessarily fit. <br>这在这里是不可能的，因为我们必须返回一个 c_int，因为这就是 Unix (包括 SuS 和 POSIX) 所说的等待状态，但 Fuchsia 显然使用 u64，所以它不一定适合。<br>
        //
        // It seems to me that that the right answer would be to provide std::os::fuchsia with its own ExitStatusExt, rather that trying to provide a not very convincing imitation of Unix. <br>在我看来，正确的答案是为 std::os::fuchsia 提供其自己的 ExitStatusExt，而不是试图提供对 Unix 不太令人信服的模仿。<br>
        // Ie, std::os::unix::process:ExitStatusExt ought not to exist on Fuchsia. <br>即，std::process: ExitStatusExt 不应该在 Fuchsia 上存在。<br>
        // But fixing this up that is beyond the scope of my efforts now. <br>但是解决此问题超出了我现在的努力范围。<br>
        //
        //
        //
        //
        //
        //
        let exit_status_as_if_unix: u8 = self.0.try_into().expect("Fuchsia process return code bigger than 8 bits, but std::os::unix::ExitStatusExt::into_raw() was called to try to convert the value into a traditional Unix-style wait status, which cannot represent values greater than 255.");
        let wait_status_as_if_unix = (exit_status_as_if_unix as c_int) << 8;
        wait_status_as_if_unix
    }
}

/// Converts a raw `c_int` to a type-safe `ExitStatus` by wrapping it without copying. <br>通过包装原始 `c_int` 而无需复制将其转换为类型安全的 `ExitStatus`。<br>
impl From<c_int> for ExitStatus {
    fn from(a: c_int) -> ExitStatus {
        ExitStatus(a as i64)
    }
}

impl fmt::Display for ExitStatus {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "exit code: {}", self.0)
    }
}

#[derive(PartialEq, Eq, Clone, Copy, Debug)]
pub struct ExitStatusError(NonZeroI64);

impl Into<ExitStatus> for ExitStatusError {
    fn into(self) -> ExitStatus {
        ExitStatus(self.0.into())
    }
}

impl ExitStatusError {
    pub fn code(self) -> Option<NonZeroI32> {
        // fixme: affected by the same bug as ExitStatus::code() <br>受与 ExitStatus::code () 相同的 bug 影响<br>
        ExitStatus(self.0.into()).code().map(|st| st.try_into().unwrap())
    }
}
