#![allow(non_camel_case_types, unused)]

use crate::io;
use crate::mem::MaybeUninit;
use crate::os::raw::c_char;

use libc::{c_int, c_void, size_t};

pub type zx_handle_t = u32;
pub type zx_vaddr_t = usize;
pub type zx_rights_t = u32;
pub type zx_status_t = i32;

pub const ZX_HANDLE_INVALID: zx_handle_t = 0;

pub type zx_time_t = i64;
pub const ZX_TIME_INFINITE: zx_time_t = i64::MAX;

pub type zx_signals_t = u32;

pub const ZX_OBJECT_SIGNAL_3: zx_signals_t = 1 << 3;

pub const ZX_TASK_TERMINATED: zx_signals_t = ZX_OBJECT_SIGNAL_3;

pub const ZX_RIGHT_SAME_RIGHTS: zx_rights_t = 1 << 31;

// The upper four bits gives the minor version. <br>高四位给出了次要版本。<br>
pub type zx_object_info_topic_t = u32;

pub const ZX_INFO_PROCESS: zx_object_info_topic_t = 3 | (1 << 28);

pub type zx_info_process_flags_t = u32;

pub fn zx_cvt<T>(t: T) -> io::Result<T>
where
    T: TryInto<zx_status_t> + Copy,
{
    if let Ok(status) = TryInto::try_into(t) {
        if status < 0 { Err(io::Error::from_raw_os_error(status)) } else { Ok(t) }
    } else {
        Err(io::Error::last_os_error())
    }
}

// Safe wrapper around zx_handle_t <br>zx_handle_t 的安全包装<br>
pub struct Handle {
    raw: zx_handle_t,
}

impl Handle {
    pub fn new(raw: zx_handle_t) -> Handle {
        Handle { raw }
    }

    pub fn raw(&self) -> zx_handle_t {
        self.raw
    }
}

impl Drop for Handle {
    fn drop(&mut self) {
        unsafe {
            zx_cvt(zx_handle_close(self.raw)).expect("Failed to close zx_handle_t");
        }
    }
}

// Returned for topic ZX_INFO_PROCESS <br>返回主题 ZX_INFO_PROCESS<br>
#[derive(Default)]
#[repr(C)]
pub struct zx_info_process_t {
    pub return_code: i64,
    pub start_time: zx_time_t,
    pub flags: zx_info_process_flags_t,
    pub reserved1: u32,
}

extern "C" {
    pub fn zx_job_default() -> zx_handle_t;

    pub fn zx_task_kill(handle: zx_handle_t) -> zx_status_t;

    pub fn zx_handle_close(handle: zx_handle_t) -> zx_status_t;

    pub fn zx_handle_duplicate(
        handle: zx_handle_t,
        rights: zx_rights_t,
        out: *const zx_handle_t,
    ) -> zx_handle_t;

    pub fn zx_object_wait_one(
        handle: zx_handle_t,
        signals: zx_signals_t,
        timeout: zx_time_t,
        pending: *mut zx_signals_t,
    ) -> zx_status_t;

    pub fn zx_object_get_info(
        handle: zx_handle_t,
        topic: u32,
        buffer: *mut c_void,
        buffer_size: size_t,
        actual_size: *mut size_t,
        avail: *mut size_t,
    ) -> zx_status_t;
}

#[derive(Default)]
#[repr(C)]
pub struct fdio_spawn_action_t {
    pub action: u32,
    pub reserved0: u32,
    pub local_fd: i32,
    pub target_fd: i32,
    pub reserved1: u64,
}

extern "C" {
    pub fn fdio_spawn_etc(
        job: zx_handle_t,
        flags: u32,
        path: *const c_char,
        argv: *const *const c_char,
        envp: *const *const c_char,
        action_count: size_t,
        actions: *const fdio_spawn_action_t,
        process: *mut zx_handle_t,
        err_msg: *mut c_char,
    ) -> zx_status_t;

    pub fn fdio_fd_clone(fd: c_int, out_handle: *mut zx_handle_t) -> zx_status_t;
    pub fn fdio_fd_create(handle: zx_handle_t, fd: *mut c_int) -> zx_status_t;
}

// fdio_spawn_etc flags <br>fdio_spawn_etc 标志<br>

pub const FDIO_SPAWN_CLONE_JOB: u32 = 0x0001;
pub const FDIO_SPAWN_CLONE_LDSVC: u32 = 0x0002;
pub const FDIO_SPAWN_CLONE_NAMESPACE: u32 = 0x0004;
pub const FDIO_SPAWN_CLONE_STDIO: u32 = 0x0008;
pub const FDIO_SPAWN_CLONE_ENVIRON: u32 = 0x0010;
pub const FDIO_SPAWN_CLONE_UTC_CLOCK: u32 = 0x0020;
pub const FDIO_SPAWN_CLONE_ALL: u32 = 0xFFFF;

// fdio_spawn_etc actions <br>fdio_spawn_etc 操作<br>

pub const FDIO_SPAWN_ACTION_CLONE_FD: u32 = 0x0001;
pub const FDIO_SPAWN_ACTION_TRANSFER_FD: u32 = 0x0002;

// Errors

#[allow(unused)]
pub const ERR_INTERNAL: zx_status_t = -1;

// ERR_NOT_SUPPORTED: The operation is not implemented, supported, or enabled. <br>ERR_NOT_SUPPORTED: 未实现，不支持或未启用该操作。<br>
//
#[allow(unused)]
pub const ERR_NOT_SUPPORTED: zx_status_t = -2;

// ERR_NO_RESOURCES: The system was not able to allocate some resource needed for the operation. <br>ERR_NO_RESOURCES: 系统无法分配该操作所需的某些资源。<br>
//
#[allow(unused)]
pub const ERR_NO_RESOURCES: zx_status_t = -3;

// ERR_NO_MEMORY: The system was not able to allocate memory needed for the operation. <br>ERR_NO_MEMORY: 系统无法分配该操作所需的内存。<br>
//
#[allow(unused)]
pub const ERR_NO_MEMORY: zx_status_t = -4;

// ERR_CALL_FAILED: The second phase of zx_channel_call(; did not complete successfully. <br>ERR_CALL_FAILED: zx_channel_call (; 的第二阶段未成功完成。<br>
//
#[allow(unused)]
pub const ERR_CALL_FAILED: zx_status_t = -5;

// ERR_INTERRUPTED_RETRY: The system call was interrupted, but should be retried. <br>ERR_INTERRUPTED_RETRY: 系统调用被中断，但是应该重试。<br>
// This should not be seen outside of the VDSO. <br>这不应在 VDSO 之外看到。<br>
#[allow(unused)]
pub const ERR_INTERRUPTED_RETRY: zx_status_t = -6;

// ======= Parameter errors <br>参数错误<br> =======
// ERR_INVALID_ARGS: an argument is invalid, ex. <br>ERR_INVALID_ARGS: 参数无效，例如。<br> null pointer <br>空指针<br>
#[allow(unused)]
pub const ERR_INVALID_ARGS: zx_status_t = -10;

// ERR_BAD_HANDLE: A specified handle value does not refer to a handle. <br>ERR_BAD_HANDLE: 指定的句柄值未引用句柄。<br>
#[allow(unused)]
pub const ERR_BAD_HANDLE: zx_status_t = -11;

// ERR_WRONG_TYPE: The subject of the operation is the wrong type to perform the operation. <br>ERR_WRONG_TYPE: 操作的主题是执行该操作的错误类型。<br>
//
// Example: Attempting a message_read on a thread handle. <br>示例：尝试在线程句柄上进行 message_read。<br>
#[allow(unused)]
pub const ERR_WRONG_TYPE: zx_status_t = -12;

// ERR_BAD_SYSCALL: The specified syscall number is invalid. <br>ERR_BAD_SYSCALL: 指定的系统调用号无效。<br>
#[allow(unused)]
pub const ERR_BAD_SYSCALL: zx_status_t = -13;

// ERR_OUT_OF_RANGE: An argument is outside the valid range for this operation. <br>ERR_OUT_OF_RANGE: 参数超出此操作的有效范围。<br>
//
#[allow(unused)]
pub const ERR_OUT_OF_RANGE: zx_status_t = -14;

// ERR_BUFFER_TOO_SMALL: A caller provided buffer is too small for this operation. <br>ERR_BUFFER_TOO_SMALL: 调用者提供的缓冲区对于此操作而言太小。<br>
//
#[allow(unused)]
pub const ERR_BUFFER_TOO_SMALL: zx_status_t = -15;

// ======= Precondition or state errors <br>前提条件或状态错误<br> =======
// ERR_BAD_STATE: operation failed because the current state of the object does not allow it, or a precondition of the operation is not satisfied <br>ERR_BAD_STATE: 操作失败，因为对象的当前状态不允许它，或者不满足该操作的前提条件<br>
//
//
#[allow(unused)]
pub const ERR_BAD_STATE: zx_status_t = -20;

// ERR_TIMED_OUT: The time limit for the operation elapsed before the operation completed. <br>ERR_TIMED_OUT: 操作完成之前经过的时间限制。<br>
//
#[allow(unused)]
pub const ERR_TIMED_OUT: zx_status_t = -21;

// ERR_SHOULD_WAIT: The operation cannot be performed currently but potentially could succeed if the caller waits for a prerequisite to be satisfied, for example waiting for a handle to be readable or writable. <br>ERR_SHOULD_WAIT: 该操作当前无法执行，但是如果调用者等待先决条件得到满足，例如等待句柄可读写，则可能会成功执行该操作。<br>
//
// Example: Attempting to read from a message pipe that has no messages waiting but has an open remote will return ERR_SHOULD_WAIT. <br>示例：尝试从没有等待消息但具有打开的远程消息的消息管道中读取，将返回 ERR_SHOULD_WAIT。<br>
// Attempting to read from a message pipe that has no messages waiting and has a closed remote end will return ERR_REMOTE_CLOSED. <br>尝试从没有消息等待且具有封闭的远程端的消息管道中读取将返回 ERR_REMOTE_CLOSED。<br>
//
//
//
//
#[allow(unused)]
pub const ERR_SHOULD_WAIT: zx_status_t = -22;

// ERR_CANCELED: The in-progress operation (e.g., a wait) has been <br>ERR_CANCELED: 正在进行的操作 (例如，等待)<br>
// // canceled.
#[allow(unused)]
pub const ERR_CANCELED: zx_status_t = -23;

// ERR_PEER_CLOSED: The operation failed because the remote end of the subject of the operation was closed. <br>ERR_PEER_CLOSED: 操作失败，因为操作对象的远端已关闭。<br>
//
#[allow(unused)]
pub const ERR_PEER_CLOSED: zx_status_t = -24;

// ERR_NOT_FOUND: The requested entity is not found. <br>ERR_NOT_FOUND: 找不到请求的实体。<br>
#[allow(unused)]
pub const ERR_NOT_FOUND: zx_status_t = -25;

// ERR_ALREADY_EXISTS: An object with the specified identifier already exists. <br>ERR_ALREADY_EXISTS: 具有指定标识符的对象已经存在。<br>
// Example: Attempting to create a file when a file already exists with that name. <br>示例：当已经存在具有该名称的文件时，尝试创建一个文件。<br>
//
//
#[allow(unused)]
pub const ERR_ALREADY_EXISTS: zx_status_t = -26;

// ERR_ALREADY_BOUND: The operation failed because the named entity is already owned or controlled by another entity. <br>ERR_ALREADY_BOUND: 操作失败，因为命名实体已经由另一个实体拥有或控制。<br>
// The operation could succeed later if the current owner releases the entity. <br>如果当前所有者释放该实体，则该操作可能会在以后成功。<br>
//
#[allow(unused)]
pub const ERR_ALREADY_BOUND: zx_status_t = -27;

// ERR_UNAVAILABLE: The subject of the operation is currently unable to perform the operation. <br>ERR_UNAVAILABLE: 该操作的主题当前无法执行该操作。<br>
// Note: This is used when there's no direct way for the caller to observe when the subject will be able to perform the operation and should thus retry. <br>当调用者无法直接观察对象何时能够执行操作并应重试时，可以使用此方法。<br>
//
//
//
#[allow(unused)]
pub const ERR_UNAVAILABLE: zx_status_t = -28;

// ======= Permission check errors <br>权限检查错误<br> =======
// ERR_ACCESS_DENIED: The caller did not have permission to perform the specified operation. <br>ERR_ACCESS_DENIED: 调用者没有执行指定操作的权限。<br>
//
#[allow(unused)]
pub const ERR_ACCESS_DENIED: zx_status_t = -30;

// ======= Input-output errors <br>输入输出错误<br> =======
// ERR_IO: Otherwise unspecified error occurred during I/O. <br>ERR_IO: 否则在 I/O 期间发生未指定的错误。<br>
#[allow(unused)]
pub const ERR_IO: zx_status_t = -40;

// ERR_REFUSED: The entity the I/O operation is being performed on rejected the operation. <br>ERR_REFUSED: I/O 操作正在被拒绝的实体。<br>
// Example: an I2C device NAK'ing a transaction or a disk controller rejecting an invalid command. <br>例如: I2C 设备 NAK' 事务或磁盘控制器拒绝无效命令。<br>
//
//
#[allow(unused)]
pub const ERR_IO_REFUSED: zx_status_t = -41;

// ERR_IO_DATA_INTEGRITY: The data in the operation failed an integrity check and is possibly corrupted. <br>ERR_IO_DATA_INTEGRITY: 操作中的数据完整性检查失败，并且可能已损坏。<br>
//
// Example: CRC or Parity error. <br>示例: CRC 或奇偶校验错误。<br>
#[allow(unused)]
pub const ERR_IO_DATA_INTEGRITY: zx_status_t = -42;

// ERR_IO_DATA_LOSS: The data in the operation is currently unavailable and may be permanently lost. <br>ERR_IO_DATA_LOSS: 操作中的数据当前不可用，并且可能会永久丢失。<br>
//
// Example: A disk block is irrecoverably damaged. <br>示例：磁盘块不可恢复地损坏。<br>
#[allow(unused)]
pub const ERR_IO_DATA_LOSS: zx_status_t = -43;

// Filesystem specific errors <br>文件系统特定错误<br>
#[allow(unused)]
pub const ERR_BAD_PATH: zx_status_t = -50;
#[allow(unused)]
pub const ERR_NOT_DIR: zx_status_t = -51;
#[allow(unused)]
pub const ERR_NOT_FILE: zx_status_t = -52;
// ERR_FILE_BIG: A file exceeds a filesystem-specific size limit. <br>ERR_FILE_BIG: 文件超出了文件系统特定的大小限制。<br>
#[allow(unused)]
pub const ERR_FILE_BIG: zx_status_t = -53;
// ERR_NO_SPACE: Filesystem or device space is exhausted. <br>ERR_NO_SPACE: 文件系统或设备空间已用尽。<br>
#[allow(unused)]
pub const ERR_NO_SPACE: zx_status_t = -54;
