#![cfg_attr(test, allow(dead_code))]

use self::imp::{drop_handler, make_handler};

pub use self::imp::cleanup;
pub use self::imp::init;

pub struct Handler {
    data: *mut libc::c_void,
}

impl Handler {
    pub unsafe fn new() -> Handler {
        make_handler()
    }

    fn null() -> Handler {
        Handler { data: crate::ptr::null_mut() }
    }
}

impl Drop for Handler {
    fn drop(&mut self) {
        unsafe {
            drop_handler(self.data);
        }
    }
}

#[cfg(any(
    target_os = "linux",
    target_os = "macos",
    target_os = "dragonfly",
    target_os = "freebsd",
    target_os = "solaris",
    target_os = "illumos",
    target_os = "netbsd",
    target_os = "openbsd"
))]
mod imp {
    use super::Handler;
    use crate::io;
    use crate::mem;
    use crate::ptr;
    use crate::thread;

    use libc::MAP_FAILED;
    use libc::{mmap, munmap};
    use libc::{sigaction, sighandler_t, SA_ONSTACK, SA_SIGINFO, SIGBUS, SIG_DFL};
    use libc::{sigaltstack, SIGSTKSZ, SS_DISABLE};
    use libc::{MAP_ANON, MAP_PRIVATE, PROT_NONE, PROT_READ, PROT_WRITE, SIGSEGV};

    use crate::sync::atomic::{AtomicBool, AtomicPtr, Ordering};
    use crate::sys::unix::os::page_size;
    use crate::sys_common::thread_info;

    // Signal handler for the SIGSEGV and SIGBUS handlers. <br>SIGSEGV 和 SIGBUS 处理程序的信号处理程序。<br> We've got guard pages (unmapped pages) at the end of every thread's stack, so if a thread ends up running into the guard page it'll trigger this handler. <br>每个线程栈的末尾都有保护页面 (未映射的页面)，因此，如果线程最终运行到保护页面中，它将触发此处理程序。<br>
    // We want to detect these cases and print out a helpful error saying that the stack has overflowed. <br>我们要检测这些情况并打印出一个有用的错误，指出栈已溢出。<br>
    // All other signals, however, should go back to what they were originally supposed to do. <br>但是，所有其他信号应返回到它们最初应该执行的操作。<br>
    //
    // This handler currently exists purely to print an informative message whenever a thread overflows its stack. <br>当前，此处理程序纯粹是为了在线程溢出其栈时打印提示性消息而存在。<br>
    // We then abort to exit and indicate a crash, but to avoid a misleading SIGSEGV that might lead users to believe that unsafe code has accessed an invalid pointer; <br>然后，我们终止以退出并指示崩溃，但要避免产生误导性的 SIGSEGV，这可能会导致用户认为不安全的代码已访问了无效的指针；<br> the SIGSEGV encountered when overflowing the stack is expected and well-defined. <br>栈溢出时遇到的 SIGSEGV 是预期的，并且定义明确。<br>
    //
    //
    // If this is not a stack overflow, the handler un-registers itself and then returns (to allow the original signal to be delivered again). <br>如果这不是栈溢出，则处理程序将对其自身进行注销，然后返回 (以允许再次发送原始信号)。<br>
    // Returning from this kind of signal handler is technically not defined to work when reading the POSIX spec strictly, but in practice it turns out many large systems and all implementations allow returning from a signal handler to work. <br>从技术上说，严格阅读 POSIX 规范时，从这种信号处理程序返回的函数没有定义为可以正常工作，但实际上，事实证明，这是许多大型系统，并且所有实现都允许从信号处理程序返回工作。<br>
    // For a more detailed explanation see the comments on #26458. <br>有关更详细的说明，请参见 #26458 上的注释。<br>
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    unsafe extern "C" fn signal_handler(
        signum: libc::c_int,
        info: *mut libc::siginfo_t,
        _data: *mut libc::c_void,
    ) {
        let guard = thread_info::stack_guard().unwrap_or(0..0);
        let addr = (*info).si_addr() as usize;

        // If the faulting address is within the guard page, then we print a message saying so and abort. <br>如果故障地址在保护页面内，则我们将显示一条消息，然后终止。<br>
        //
        if guard.start <= addr && addr < guard.end {
            rtprintpanic!(
                "\nthread '{}' has overflowed its stack\n",
                thread::current().name().unwrap_or("<unknown>")
            );
            rtabort!("stack overflow");
        } else {
            // Unregister ourselves by reverting back to the default behavior. <br>通过恢复为默认行为来注销自己。<br>
            let mut action: sigaction = mem::zeroed();
            action.sa_sigaction = SIG_DFL;
            sigaction(signum, &action, ptr::null_mut());

            // See comment above for why this function returns. <br>有关为什么返回此函数的信息，请参见上面的注释。<br>
        }
    }

    static MAIN_ALTSTACK: AtomicPtr<libc::c_void> = AtomicPtr::new(ptr::null_mut());
    static NEED_ALTSTACK: AtomicBool = AtomicBool::new(false);

    pub unsafe fn init() {
        let mut action: sigaction = mem::zeroed();
        for &signal in &[SIGSEGV, SIGBUS] {
            sigaction(signal, ptr::null_mut(), &mut action);
            // Configure our signal handler if one is not already set. <br>如果尚未设置信号处理程序，请进行配置。<br>
            if action.sa_sigaction == SIG_DFL {
                action.sa_flags = SA_SIGINFO | SA_ONSTACK;
                action.sa_sigaction = signal_handler as sighandler_t;
                sigaction(signal, &action, ptr::null_mut());
                NEED_ALTSTACK.store(true, Ordering::Relaxed);
            }
        }

        let handler = make_handler();
        MAIN_ALTSTACK.store(handler.data, Ordering::Relaxed);
        mem::forget(handler);
    }

    pub unsafe fn cleanup() {
        drop_handler(MAIN_ALTSTACK.load(Ordering::Relaxed));
    }

    unsafe fn get_stackp() -> *mut libc::c_void {
        // OpenBSD requires this flag for stack mapping otherwise the said mapping will fail as a no-op on most systems and has a different meaning on FreeBSD <br>OpenBSD 需要此标志用于栈映射，否则所述映射将在大多数系统上作为无操作失败，并且在 FreeBSD 上具有不同的含义<br>
        //
        //
        #[cfg(any(target_os = "openbsd", target_os = "netbsd", target_os = "linux",))]
        let flags = MAP_PRIVATE | MAP_ANON | libc::MAP_STACK;
        #[cfg(not(any(target_os = "openbsd", target_os = "netbsd", target_os = "linux",)))]
        let flags = MAP_PRIVATE | MAP_ANON;
        let stackp =
            mmap(ptr::null_mut(), SIGSTKSZ + page_size(), PROT_READ | PROT_WRITE, flags, -1, 0);
        if stackp == MAP_FAILED {
            panic!("failed to allocate an alternative stack: {}", io::Error::last_os_error());
        }
        let guard_result = libc::mprotect(stackp, page_size(), PROT_NONE);
        if guard_result != 0 {
            panic!("failed to set up alternative stack guard page: {}", io::Error::last_os_error());
        }
        stackp.add(page_size())
    }

    unsafe fn get_stack() -> libc::stack_t {
        libc::stack_t { ss_sp: get_stackp(), ss_flags: 0, ss_size: SIGSTKSZ }
    }

    pub unsafe fn make_handler() -> Handler {
        if !NEED_ALTSTACK.load(Ordering::Relaxed) {
            return Handler::null();
        }
        let mut stack = mem::zeroed();
        sigaltstack(ptr::null(), &mut stack);
        // Configure alternate signal stack, if one is not already set. <br>如果尚未设置备用信号栈，则配置备用信号栈。<br>
        if stack.ss_flags & SS_DISABLE != 0 {
            stack = get_stack();
            sigaltstack(&stack, ptr::null_mut());
            Handler { data: stack.ss_sp as *mut libc::c_void }
        } else {
            Handler::null()
        }
    }

    pub unsafe fn drop_handler(data: *mut libc::c_void) {
        if !data.is_null() {
            let stack = libc::stack_t {
                ss_sp: ptr::null_mut(),
                ss_flags: SS_DISABLE,
                // Workaround for bug in macOS implementation of sigaltstack UNIX2003 which returns ENOMEM when disabling a stack while passing ss_size smaller than MINSIGSTKSZ. <br>sigaltstack UNIX2003 的 macOS 实现中的错误的变通办法，当禁用 ss_size 小于 MINSIGSTKSZ 时，该栈在禁用栈时返回 ENOMEM。<br>
                // According to POSIX both ss_sp and ss_size should be ignored in this case. <br>根据 POSIX，在这种情况下，应同时忽略 ss_sp 和 ss_size。<br>
                //
                //
                ss_size: SIGSTKSZ,
            };
            sigaltstack(&stack, ptr::null_mut());
            // We know from `get_stackp` that the alternate stack we installed is part of a mapping that started one page earlier, so walk back a page and unmap from there. <br>从 `get_stackp` 中我们知道，我们安装的备用栈是早于一页开始的映射的一部分，因此请返回页面并从此处取消映射。<br>
            //
            munmap(data.sub(page_size()), SIGSTKSZ + page_size());
        }
    }
}

#[cfg(not(any(
    target_os = "linux",
    target_os = "macos",
    target_os = "dragonfly",
    target_os = "freebsd",
    target_os = "solaris",
    target_os = "illumos",
    target_os = "netbsd",
    target_os = "openbsd",
)))]
mod imp {
    pub unsafe fn init() {}

    pub unsafe fn cleanup() {}

    pub unsafe fn make_handler() -> super::Handler {
        super::Handler::null()
    }

    pub unsafe fn drop_handler(_data: *mut libc::c_void) {}
}
