use crate::cmp;
use crate::ffi::CStr;
use crate::io;
use crate::mem;
use crate::num::NonZeroUsize;
use crate::ptr;
use crate::sys::{os, stack_overflow};
use crate::time::Duration;

#[cfg(all(target_os = "linux", target_env = "gnu"))]
use crate::sys::weak::dlsym;
#[cfg(any(target_os = "solaris", target_os = "illumos"))]
use crate::sys::weak::weak;
#[cfg(not(any(target_os = "l4re", target_os = "vxworks", target_os = "espidf")))]
pub const DEFAULT_MIN_STACK_SIZE: usize = 2 * 1024 * 1024;
#[cfg(target_os = "l4re")]
pub const DEFAULT_MIN_STACK_SIZE: usize = 1024 * 1024;
#[cfg(target_os = "vxworks")]
pub const DEFAULT_MIN_STACK_SIZE: usize = 256 * 1024;
#[cfg(target_os = "espidf")]
pub const DEFAULT_MIN_STACK_SIZE: usize = 0; // 0 indicates that the stack size configured in the ESP-IDF menuconfig system should be used <br>0 表示应使用 ESP-IDF menuconfig 系统中配置的栈大小<br>

#[cfg(target_os = "fuchsia")]
mod zircon {
    type zx_handle_t = u32;
    type zx_status_t = i32;
    pub const ZX_PROP_NAME: u32 = 3;

    extern "C" {
        pub fn zx_object_set_property(
            handle: zx_handle_t,
            property: u32,
            value: *const libc::c_void,
            value_size: libc::size_t,
        ) -> zx_status_t;
        pub fn zx_thread_self() -> zx_handle_t;
    }
}

pub struct Thread {
    id: libc::pthread_t,
}

// Some platforms may have pthread_t as a pointer in which case we still want a thread to be Send/Sync <br>某些平台可能将 pthread_t 用作指针，在这种情况下，我们仍然希望线程为 Send/Sync<br>
//
unsafe impl Send for Thread {}
unsafe impl Sync for Thread {}

impl Thread {
    // unsafe: see thread::Builder::spawn_unchecked for safety requirements <br>不安全：有关安全要求，请参见 thread::Builder::spawn_unchecked<br>
    pub unsafe fn new(stack: usize, p: Box<dyn FnOnce()>) -> io::Result<Thread> {
        let p = Box::into_raw(box p);
        let mut native: libc::pthread_t = mem::zeroed();
        let mut attr: libc::pthread_attr_t = mem::zeroed();
        assert_eq!(libc::pthread_attr_init(&mut attr), 0);

        #[cfg(target_os = "espidf")]
        if stack > 0 {
            // Only set the stack if a non-zero value is passed <br>仅在传递非零值时才设置栈<br>
            // 0 is used as an indication that the default stack size configured in the ESP-IDF menuconfig system should be used <br>0 用作指示应使用 ESP-IDF menuconfig 系统中配置的默认栈大小<br>
            assert_eq!(
                libc::pthread_attr_setstacksize(&mut attr, cmp::max(stack, min_stack_size(&attr))),
                0
            );
        }

        #[cfg(not(target_os = "espidf"))]
        {
            let stack_size = cmp::max(stack, min_stack_size(&attr));

            match libc::pthread_attr_setstacksize(&mut attr, stack_size) {
                0 => {}
                n => {
                    assert_eq!(n, libc::EINVAL);
                    // EINVAL means |stack_size| is either too small or not a multiple of the system page size. <br>EINVAL 表示 | stack_size | 太小或不是系统页面大小的倍数。<br>
                    // Because it's definitely >= PTHREAD_STACK_MIN, it must be an alignment issue. <br>因为肯定是 >= PTHREAD_STACK_MIN，所以它一定是对齐问题。<br>
                    // Round up to the nearest page and try again. <br>向上舍入到最近的页面，然后重试。<br>
                    //
                    let page_size = os::page_size();
                    let stack_size =
                        (stack_size + page_size - 1) & (-(page_size as isize - 1) as usize - 1);
                    assert_eq!(libc::pthread_attr_setstacksize(&mut attr, stack_size), 0);
                }
            };
        }

        let ret = libc::pthread_create(&mut native, &attr, thread_start, p as *mut _);
        // Note: if the thread creation fails and this assert fails, then p will be leaked. <br>如果线程创建失败并且该断言失败，则 p 将被泄漏。<br>
        // However, an alternative design could cause double-free which is clearly worse. <br>但是，替代设计可能会导致双重释放，这显然更糟。<br>
        //
        assert_eq!(libc::pthread_attr_destroy(&mut attr), 0);

        return if ret != 0 {
            // The thread failed to start and as a result p was not consumed. <br>线程无法启动，结果 p 没有被消费。<br>
            // Therefore, it is safe to reconstruct the box so that it gets deallocated. <br>因此，重建 box 使其被释放是安全的。<br>
            drop(Box::from_raw(p));
            Err(io::Error::from_raw_os_error(ret))
        } else {
            Ok(Thread { id: native })
        };

        extern "C" fn thread_start(main: *mut libc::c_void) -> *mut libc::c_void {
            unsafe {
                // Next, set up our stack overflow handler which may get triggered if we run out of stack. <br>接下来，设置我们的栈溢出处理程序，如果我们用完栈，可能会触发该处理程序。<br>
                //
                let _handler = stack_overflow::Handler::new();
                // Finally, let's run some code. <br>最后，让我们运行一些代码。<br>
                Box::from_raw(main as *mut Box<dyn FnOnce()>)();
            }
            ptr::null_mut()
        }
    }

    pub fn yield_now() {
        let ret = unsafe { libc::sched_yield() };
        debug_assert_eq!(ret, 0);
    }

    #[cfg(any(target_os = "linux", target_os = "android"))]
    pub fn set_name(name: &CStr) {
        const PR_SET_NAME: libc::c_int = 15;
        // pthread wrapper only appeared in glibc 2.12, so we use syscall directly. <br>pthread 包装器仅出现在 glibc 2.12 中，因此我们直接使用 syscall。<br>
        //
        unsafe {
            libc::prctl(
                PR_SET_NAME,
                name.as_ptr(),
                0 as libc::c_ulong,
                0 as libc::c_ulong,
                0 as libc::c_ulong,
            );
        }
    }

    #[cfg(any(target_os = "freebsd", target_os = "dragonfly", target_os = "openbsd"))]
    pub fn set_name(name: &CStr) {
        unsafe {
            libc::pthread_set_name_np(libc::pthread_self(), name.as_ptr());
        }
    }

    #[cfg(any(target_os = "macos", target_os = "ios"))]
    pub fn set_name(name: &CStr) {
        unsafe {
            libc::pthread_setname_np(name.as_ptr());
        }
    }

    #[cfg(target_os = "netbsd")]
    pub fn set_name(name: &CStr) {
        use crate::ffi::CString;
        let cname = CString::new(&b"%s"[..]).unwrap();
        unsafe {
            libc::pthread_setname_np(
                libc::pthread_self(),
                cname.as_ptr(),
                name.as_ptr() as *mut libc::c_void,
            );
        }
    }

    #[cfg(any(target_os = "solaris", target_os = "illumos"))]
    pub fn set_name(name: &CStr) {
        weak! {
            fn pthread_setname_np(
                libc::pthread_t, *const libc::c_char
            ) -> libc::c_int
        }

        if let Some(f) = pthread_setname_np.get() {
            unsafe {
                f(libc::pthread_self(), name.as_ptr());
            }
        }
    }

    #[cfg(target_os = "fuchsia")]
    pub fn set_name(name: &CStr) {
        use self::zircon::*;
        unsafe {
            zx_object_set_property(
                zx_thread_self(),
                ZX_PROP_NAME,
                name.as_ptr() as *const libc::c_void,
                name.to_bytes().len(),
            );
        }
    }

    #[cfg(target_os = "haiku")]
    pub fn set_name(name: &CStr) {
        unsafe {
            let thread_self = libc::find_thread(ptr::null_mut());
            libc::rename_thread(thread_self, name.as_ptr());
        }
    }

    #[cfg(any(
        target_env = "newlib",
        target_os = "l4re",
        target_os = "emscripten",
        target_os = "redox",
        target_os = "vxworks"
    ))]
    pub fn set_name(_name: &CStr) {
        // Newlib, Emscripten, and VxWorks have no way to set a thread name. <br>Newlib、Emscripten 和 VxWorks 无法设置线程名称。<br>
    }

    #[cfg(not(target_os = "espidf"))]
    pub fn sleep(dur: Duration) {
        let mut secs = dur.as_secs();
        let mut nsecs = dur.subsec_nanos() as _;

        // If we're awoken with a signal then the return value will be -1 and nanosleep will fill in `ts` with the remaining time. <br>如果我们被信号唤醒，则返回值为 -1，nanosleep 将使用剩余时间填充 `ts`。<br>
        //
        unsafe {
            while secs > 0 || nsecs > 0 {
                let mut ts = libc::timespec {
                    tv_sec: cmp::min(libc::time_t::MAX as u64, secs) as libc::time_t,
                    tv_nsec: nsecs,
                };
                secs -= ts.tv_sec as u64;
                let ts_ptr = &mut ts as *mut _;
                if libc::nanosleep(ts_ptr, ts_ptr) == -1 {
                    assert_eq!(os::errno(), libc::EINTR);
                    secs += ts.tv_sec as u64;
                    nsecs = ts.tv_nsec;
                } else {
                    nsecs = 0;
                }
            }
        }
    }

    #[cfg(target_os = "espidf")]
    pub fn sleep(dur: Duration) {
        let mut micros = dur.as_micros();
        unsafe {
            while micros > 0 {
                let st = if micros > u32::MAX as u128 { u32::MAX } else { micros as u32 };
                libc::usleep(st);

                micros -= st as u128;
            }
        }
    }

    pub fn join(self) {
        unsafe {
            let ret = libc::pthread_join(self.id, ptr::null_mut());
            mem::forget(self);
            assert!(ret == 0, "failed to join thread: {}", io::Error::from_raw_os_error(ret));
        }
    }

    pub fn id(&self) -> libc::pthread_t {
        self.id
    }

    pub fn into_id(self) -> libc::pthread_t {
        let id = self.id;
        mem::forget(self);
        id
    }
}

impl Drop for Thread {
    fn drop(&mut self) {
        let ret = unsafe { libc::pthread_detach(self.id) };
        debug_assert_eq!(ret, 0);
    }
}

pub fn available_parallelism() -> io::Result<NonZeroUsize> {
    cfg_if::cfg_if! {
        if #[cfg(any(
            target_os = "android",
            target_os = "emscripten",
            target_os = "fuchsia",
            target_os = "ios",
            target_os = "linux",
            target_os = "macos",
            target_os = "solaris",
            target_os = "illumos",
        ))] {
            #[cfg(any(target_os = "android", target_os = "linux"))]
            {
                let quota = cgroup2_quota().max(1);
                let mut set: libc::cpu_set_t = unsafe { mem::zeroed() };
                unsafe {
                    if libc::sched_getaffinity(0, mem::size_of::<libc::cpu_set_t>(), &mut set) == 0 {
                        let count = libc::CPU_COUNT(&set) as usize;
                        let count = count.min(quota);
                        // SAFETY: affinity mask can't be empty and the quota gets clamped to a minimum of 1 <br>关联掩码不能为空，并且配额被限制为至少 1<br>
                        return Ok(NonZeroUsize::new_unchecked(count));
                    }
                }
            }
            match unsafe { libc::sysconf(libc::_SC_NPROCESSORS_ONLN) } {
                -1 => Err(io::Error::last_os_error()),
                0 => Err(io::const_io_error!(io::ErrorKind::NotFound, "The number of hardware threads is not known for the target platform")),
                cpus => Ok(unsafe { NonZeroUsize::new_unchecked(cpus as usize) }),
            }
        } else if #[cfg(any(target_os = "freebsd", target_os = "dragonfly", target_os = "netbsd"))] {
            use crate::ptr;

            let mut cpus: libc::c_uint = 0;
            let mut cpus_size = crate::mem::size_of_val(&cpus);

            unsafe {
                cpus = libc::sysconf(libc::_SC_NPROCESSORS_ONLN) as libc::c_uint;
            }

            // Fallback approach in case of errors or no hardware threads. <br>在出现错误或没有硬件线程的情况下的后备方法。<br>
            if cpus < 1 {
                let mut mib = [libc::CTL_HW, libc::HW_NCPU, 0, 0];
                let res = unsafe {
                    libc::sysctl(
                        mib.as_mut_ptr(),
                        2,
                        &mut cpus as *mut _ as *mut _,
                        &mut cpus_size as *mut _ as *mut _,
                        ptr::null_mut(),
                        0,
                    )
                };

                // Handle errors if any. <br>处理错误 (如果有)。<br>
                if res == -1 {
                    return Err(io::Error::last_os_error());
                } else if cpus == 0 {
                    return Err(io::const_io_error!(io::ErrorKind::NotFound, "The number of hardware threads is not known for the target platform"));
                }
            }
            Ok(unsafe { NonZeroUsize::new_unchecked(cpus as usize) })
        } else if #[cfg(target_os = "openbsd")] {
            use crate::ptr;

            let mut cpus: libc::c_uint = 0;
            let mut cpus_size = crate::mem::size_of_val(&cpus);
            let mut mib = [libc::CTL_HW, libc::HW_NCPU, 0, 0];

            let res = unsafe {
                libc::sysctl(
                    mib.as_mut_ptr(),
                    2,
                    &mut cpus as *mut _ as *mut _,
                    &mut cpus_size as *mut _ as *mut _,
                    ptr::null_mut(),
                    0,
                )
            };

            // Handle errors if any. <br>处理错误 (如果有)。<br>
            if res == -1 {
                return Err(io::Error::last_os_error());
            } else if cpus == 0 {
                return Err(io::const_io_error!(io::ErrorKind::NotFound, "The number of hardware threads is not known for the target platform"));
            }

            Ok(unsafe { NonZeroUsize::new_unchecked(cpus as usize) })
        } else if #[cfg(target_os = "haiku")] {
            // system_info cpu_count field gets the static data set at boot time with `smp_set_num_cpus` `get_system_info` calls then `smp_get_num_cpus` <br>system_info cpu_count 字段在启动时使用 `smp_set_num_cpus` `get_system_info` 调用然后 `smp_get_num_cpus` 获取静态数据集<br>
            //
            unsafe {
                let mut sinfo: libc::system_info = crate::mem::zeroed();
                let res = libc::get_system_info(&mut sinfo);

                if res != libc::B_OK {
                    return Err(io::const_io_error!(io::ErrorKind::NotFound, "The number of hardware threads is not known for the target platform"));
                }

                Ok(NonZeroUsize::new_unchecked(sinfo.cpu_count as usize))
            }
        } else {
            // FIXME: implement on vxWorks, Redox, l4re <br>在 vxWorks、Redox、l4re 上实现<br>
            Err(io::const_io_error!(io::ErrorKind::Unsupported, "Getting the number of hardware threads is not supported on the target platform"))
        }
    }
}

/// Returns cgroup CPU quota in core-equivalents, rounded down, or usize::MAX if the quota cannot be determined or is not set. <br>如果配额无法确定或未设置，则以核心等效值、向下舍入或 usize::MAX 返回 cgroup CPU 配额。<br>
///
#[cfg(any(target_os = "android", target_os = "linux"))]
fn cgroup2_quota() -> usize {
    use crate::ffi::OsString;
    use crate::fs::{try_exists, File};
    use crate::io::Read;
    use crate::os::unix::ffi::OsStringExt;
    use crate::path::PathBuf;

    let mut quota = usize::MAX;
    if cfg!(miri) {
        // Attempting to open a file fails under default flags due to isolation. <br>由于隔离，在默认标志下尝试打开文件会失败。<br>
        // And Miri does not have parallelism anyway. <br>而且 Miri 无论如何都没有并行性。<br>
        return quota;
    }

    let _: Option<()> = try {
        let mut buf = Vec::with_capacity(128);
        // find our place in the cgroup hierarchy <br>在 cgroup 层次结构中找到我们的位置<br>
        File::open("/proc/self/cgroup").ok()?.read_to_end(&mut buf).ok()?;
        let cgroup_path = buf
            .split(|&c| c == b'\n')
            .filter_map(|line| {
                let mut fields = line.splitn(3, |&c| c == b':');
                // expect cgroupv2 which has an empty 2nd field <br>期望 cgroupv2 的第二个字段为空<br>
                if fields.nth(1) != Some(b"") {
                    return None;
                }
                let path = fields.last()?;
                // skip leading slash <br>跳过前导斜杠<br>
                Some(path[1..].to_owned())
            })
            .next()?;
        let cgroup_path = PathBuf::from(OsString::from_vec(cgroup_path));

        let mut path = PathBuf::with_capacity(128);
        let mut read_buf = String::with_capacity(20);

        let cgroup_mount = "/sys/fs/cgroup";

        path.push(cgroup_mount);
        path.push(&cgroup_path);

        path.push("cgroup.controllers");

        // skip if we're not looking at cgroup2 <br>如果我们没有看到 cgroup2，请跳过<br>
        if matches!(try_exists(&path), Err(_) | Ok(false)) {
            return usize::MAX;
        };

        path.pop();

        while path.starts_with(cgroup_mount) {
            path.push("cpu.max");

            read_buf.clear();

            if File::open(&path).and_then(|mut f| f.read_to_string(&mut read_buf)).is_ok() {
                let raw_quota = read_buf.lines().next()?;
                let mut raw_quota = raw_quota.split(' ');
                let limit = raw_quota.next()?;
                let period = raw_quota.next()?;
                match (limit.parse::<usize>(), period.parse::<usize>()) {
                    (Ok(limit), Ok(period)) => {
                        quota = quota.min(limit / period);
                    }
                    _ => {}
                }
            }

            path.pop(); // pop filename <br>弹出文件名<br>
            path.pop(); // pop dir <br>弹出目录<br>
        }
    };

    quota
}

#[cfg(all(
    not(target_os = "linux"),
    not(target_os = "freebsd"),
    not(target_os = "macos"),
    not(target_os = "netbsd"),
    not(target_os = "openbsd"),
    not(target_os = "solaris")
))]
#[cfg_attr(test, allow(dead_code))]
pub mod guard {
    use crate::ops::Range;
    pub type Guard = Range<usize>;
    pub unsafe fn current() -> Option<Guard> {
        None
    }
    pub unsafe fn init() -> Option<Guard> {
        None
    }
}

#[cfg(any(
    target_os = "linux",
    target_os = "freebsd",
    target_os = "macos",
    target_os = "netbsd",
    target_os = "openbsd",
    target_os = "solaris"
))]
#[cfg_attr(test, allow(dead_code))]
pub mod guard {
    use libc::{mmap, mprotect};
    use libc::{MAP_ANON, MAP_FAILED, MAP_FIXED, MAP_PRIVATE, PROT_NONE, PROT_READ, PROT_WRITE};

    use crate::io;
    use crate::ops::Range;
    use crate::sync::atomic::{AtomicUsize, Ordering};
    use crate::sys::os;

    // This is initialized in init() and only read from after <br>这是在 init() 中初始化的，只能在之后读取<br>
    static PAGE_SIZE: AtomicUsize = AtomicUsize::new(0);

    pub type Guard = Range<usize>;

    #[cfg(target_os = "solaris")]
    unsafe fn get_stack_start() -> Option<*mut libc::c_void> {
        let mut current_stack: libc::stack_t = crate::mem::zeroed();
        assert_eq!(libc::stack_getbounds(&mut current_stack), 0);
        Some(current_stack.ss_sp)
    }

    #[cfg(target_os = "macos")]
    unsafe fn get_stack_start() -> Option<*mut libc::c_void> {
        let th = libc::pthread_self();
        let stackptr = libc::pthread_get_stackaddr_np(th);
        Some(stackptr.map_addr(|addr| addr - libc::pthread_get_stacksize_np(th)))
    }

    #[cfg(target_os = "openbsd")]
    unsafe fn get_stack_start() -> Option<*mut libc::c_void> {
        let mut current_stack: libc::stack_t = crate::mem::zeroed();
        assert_eq!(libc::pthread_stackseg_np(libc::pthread_self(), &mut current_stack), 0);

        let stack_ptr = current_stack.ss_sp;
        let stackaddr = if libc::pthread_main_np() == 1 {
            // main thread <br>主线程<br>
            stack_ptr.addr() - current_stack.ss_size + PAGE_SIZE.load(Ordering::Relaxed)
        } else {
            // new thread <br>新线程<br>
            stack_ptr.addr() - current_stack.ss_size
        };
        Some(stack_ptr.with_addr(stackaddr))
    }

    #[cfg(any(
        target_os = "android",
        target_os = "freebsd",
        target_os = "linux",
        target_os = "netbsd",
        target_os = "l4re"
    ))]
    unsafe fn get_stack_start() -> Option<*mut libc::c_void> {
        let mut ret = None;
        let mut attr: libc::pthread_attr_t = crate::mem::zeroed();
        #[cfg(target_os = "freebsd")]
        assert_eq!(libc::pthread_attr_init(&mut attr), 0);
        #[cfg(target_os = "freebsd")]
        let e = libc::pthread_attr_get_np(libc::pthread_self(), &mut attr);
        #[cfg(not(target_os = "freebsd"))]
        let e = libc::pthread_getattr_np(libc::pthread_self(), &mut attr);
        if e == 0 {
            let mut stackaddr = crate::ptr::null_mut();
            let mut stacksize = 0;
            assert_eq!(libc::pthread_attr_getstack(&attr, &mut stackaddr, &mut stacksize), 0);
            ret = Some(stackaddr);
        }
        if e == 0 || cfg!(target_os = "freebsd") {
            assert_eq!(libc::pthread_attr_destroy(&mut attr), 0);
        }
        ret
    }

    // Precondition: PAGE_SIZE is initialized. <br>前提条件: PAGE_SIZE 已初始化。<br>
    unsafe fn get_stack_start_aligned() -> Option<*mut libc::c_void> {
        let page_size = PAGE_SIZE.load(Ordering::Relaxed);
        assert!(page_size != 0);
        let stackptr = get_stack_start()?;
        let stackaddr = stackptr.addr();

        // Ensure stackaddr is page aligned! <br>确保 stackaddr 是页对齐的！<br> A parent process might have reset RLIMIT_STACK to be non-page aligned. <br>父进程可能已将 RLIMIT_STACK 重置为非页面对齐。<br>
        // The pthread_attr_getstack() reports the usable stack area stackaddr < stackaddr + stacksize, so if stackaddr is not page-aligned, calculate the fix such that stackaddr < new_page_aligned_stackaddr < stackaddr + stacksize <br>pthread_attr_getstack() 报告可用的栈区域 stackaddr <stackaddr + stacksize，因此，如果 stackaddr 不是页面对齐的，则计算修正值，使 stackaddr <new_page_aligned_stackaddr <stackaddr + stacksize<br>
        //
        //
        //
        //
        let remainder = stackaddr % page_size;
        Some(if remainder == 0 {
            stackptr
        } else {
            stackptr.with_addr(stackaddr + page_size - remainder)
        })
    }

    pub unsafe fn init() -> Option<Guard> {
        let page_size = os::page_size();
        PAGE_SIZE.store(page_size, Ordering::Relaxed);

        if cfg!(all(target_os = "linux", not(target_env = "musl"))) {
            // Linux doesn't allocate the whole stack right away, and the kernel has its own stack-guard mechanism to fault when growing too close to an existing mapping. <br>Linux 不会立即分配整个栈，并且内核具有自己的栈保护机制，以便在与现有映射太接近时发生故障。<br>
            // If we map our own guard, then the kernel starts enforcing a rather large gap above that, rendering much of the possible stack space useless. <br>如果我们使用自己的保护程序 map，则内核将开始在此之上强制执行一个较大的间隙，从而使很多可能的栈空间变得无用。<br>
            // See #43052. <br>请参见 #43052。<br>
            //
            // Instead, we'll just note where we expect rlimit to start faulting, so our handler can report "stack overflow", and trust that the kernel's own stack guard will work. <br>取而代之的是，我们只需要注意 rlimit 会在哪里开始出错，以便我们的处理程序可以报告 "栈溢出"，并相信内核自己的栈保护会起作用。<br>
            //
            //
            //
            //
            //
            let stackptr = get_stack_start_aligned()?;
            let stackaddr = stackptr.addr();
            Some(stackaddr - page_size..stackaddr)
        } else if cfg!(all(target_os = "linux", target_env = "musl")) {
            // For the main thread, the musl's pthread_attr_getstack returns the current stack size, rather than maximum size it can eventually grow to. <br>对于主线程，musl 的 pthread_attr_getstack 返回当前的栈大小，而不是其最终可以增长到的最大大小。<br>
            // It cannot be used to determine the position of kernel's stack guard. <br>它不能用于确定内核的栈保护的位置。<br>
            //
            //
            None
        } else if cfg!(target_os = "freebsd") {
            // FreeBSD's stack autogrows, and optionally includes a guard page at the bottom. <br>FreeBSD 的栈会自动增长，并且可以选择在底部包含一个保护页面。<br>
            // If we try to remap the bottom of the stack ourselves, FreeBSD's guard page moves upwards. <br>如果我们自己尝试重新映射栈底，FreeBSD 的保护页面就会向上移动。<br>
            //
            // So we'll just use the builtin guard page. <br>所以我们将只使用内置的保护页面。<br>
            let stackptr = get_stack_start_aligned()?;
            let guardaddr = stackptr.addr();
            // Technically the number of guard pages is tunable and controlled by the security.bsd.stack_guard_page sysctl, but there are few reasons to change it from the default. <br>从技术上讲，保护页面的数量是由 security.bsd.stack_guard_page sysctl 可调和控制的，但几乎没有理由将其从默认值更改。<br>
            // The default value has been 1 ever since FreeBSD 11.1 and 10.4. <br>自 FreeBSD 11.1 和 10.4 以来，默认值一直为 1。<br>
            //
            //
            const GUARD_PAGES: usize = 1;
            let guard = guardaddr..guardaddr + GUARD_PAGES * page_size;
            Some(guard)
        } else {
            // Reallocate the last page of the stack. <br>重新分配栈的最后一页。<br>
            // This ensures SIGBUS will be raised on stack overflow. <br>这样可以确保在栈溢出时引发 SIGBUS。<br>
            // Systems which enforce strict PAX MPROTECT do not allow to mprotect() a mapping with less restrictive permissions than the initial mmap() used, so we mmap() here with read/write permissions and only then mprotect() it to no permissions at all. <br>强制执行严格 PAX MPROTECT 的系统不允许 mprotect() 具有比使用的初始 mmap() 少的限制性权限的映射，因此我们这里的 mmap() 具有 read/write 权限，然后只有 mprotect() 完全没有权限。<br>
            // See issue #50313. <br>请参见 issue #50313。<br>
            //
            //
            //
            //
            let stackptr = get_stack_start_aligned()?;
            let result = mmap(
                stackptr,
                page_size,
                PROT_READ | PROT_WRITE,
                MAP_PRIVATE | MAP_ANON | MAP_FIXED,
                -1,
                0,
            );
            if result != stackptr || result == MAP_FAILED {
                panic!("failed to allocate a guard page: {}", io::Error::last_os_error());
            }

            let result = mprotect(stackptr, page_size, PROT_NONE);
            if result != 0 {
                panic!("failed to protect the guard page: {}", io::Error::last_os_error());
            }

            let guardaddr = stackptr.addr();

            Some(guardaddr..guardaddr + page_size)
        }
    }

    #[cfg(any(target_os = "macos", target_os = "openbsd", target_os = "solaris"))]
    pub unsafe fn current() -> Option<Guard> {
        let stackptr = get_stack_start()?;
        let stackaddr = stackptr.addr();
        Some(stackaddr - PAGE_SIZE.load(Ordering::Relaxed)..stackaddr)
    }

    #[cfg(any(
        target_os = "android",
        target_os = "freebsd",
        target_os = "linux",
        target_os = "netbsd",
        target_os = "l4re"
    ))]
    pub unsafe fn current() -> Option<Guard> {
        let mut ret = None;
        let mut attr: libc::pthread_attr_t = crate::mem::zeroed();
        #[cfg(target_os = "freebsd")]
        assert_eq!(libc::pthread_attr_init(&mut attr), 0);
        #[cfg(target_os = "freebsd")]
        let e = libc::pthread_attr_get_np(libc::pthread_self(), &mut attr);
        #[cfg(not(target_os = "freebsd"))]
        let e = libc::pthread_getattr_np(libc::pthread_self(), &mut attr);
        if e == 0 {
            let mut guardsize = 0;
            assert_eq!(libc::pthread_attr_getguardsize(&attr, &mut guardsize), 0);
            if guardsize == 0 {
                if cfg!(all(target_os = "linux", target_env = "musl")) {
                    // musl versions before 1.1.19 always reported guard size obtained from pthread_attr_get_np as zero. <br>1.1.19 之前的 musl 版本始终将从 pthread_attr_get_np 获得的防护大小报告为零。<br>
                    //
                    // Use page size as a fallback. <br>使用页面大小作为后备。<br>
                    guardsize = PAGE_SIZE.load(Ordering::Relaxed);
                } else {
                    panic!("there is no guard page");
                }
            }
            let mut stackptr = crate::ptr::null_mut::<libc::c_void>();
            let mut size = 0;
            assert_eq!(libc::pthread_attr_getstack(&attr, &mut stackptr, &mut size), 0);

            let stackaddr = stackptr.addr();
            ret = if cfg!(any(target_os = "freebsd", target_os = "netbsd")) {
                Some(stackaddr - guardsize..stackaddr)
            } else if cfg!(all(target_os = "linux", target_env = "musl")) {
                Some(stackaddr - guardsize..stackaddr)
            } else if cfg!(all(target_os = "linux", any(target_env = "gnu", target_env = "uclibc")))
            {
                // glibc used to include the guard area within the stack, as noted in the BUGS section of `man pthread_attr_getguardsize`. <br>如 `man pthread_attr_getguardsize` 的 `BUGS` 部分所述，glibc 曾经将保护区包括在栈内。<br>
                // This has been corrected starting with glibc 2.27, and in some distro backports, so the guard is now placed at the end (below) the stack. <br>从 glibc 2.27 开始，并且在某些发行版本的反向移植中，此问题已得到纠正，因此现在将保护放置在栈的 (below) 的末尾。<br>
                // There's no easy way for us to know which we have at runtime, so we'll just match any fault in the range right above or below the stack base to call that fault a stack overflow. <br>没有简单的方法让我们知道在运行时有哪些故障，因此我们只匹配栈基础上方或下方范围内的任何故障，以调用导致栈溢出的故障。<br>
                //
                //
                //
                Some(stackaddr - guardsize..stackaddr + guardsize)
            } else {
                Some(stackaddr..stackaddr + guardsize)
            };
        }
        if e == 0 || cfg!(target_os = "freebsd") {
            assert_eq!(libc::pthread_attr_destroy(&mut attr), 0);
        }
        ret
    }
}

// glibc >= 2.15 has a __pthread_get_minstack() function that returns PTHREAD_STACK_MIN plus bytes needed for thread-local storage. <br>glibc>= 2.15 具有一个 __pthread_get_minstack() 函数，该函数返回 PTHREAD_STACK_MIN 以及线程本地存储所需的字节。<br>
// We need that information to avoid blowing up when a small stack is created in an application with big thread-local storage requirements. <br>我们需要这些信息，以避免在具有大量线程本地存储需求的应用程序中创建一个小的栈时发生崩溃。<br>
//
// See #6233 for rationale and details. <br>有关原理和详细信息，请参见 #6233。<br>
//
#[cfg(all(target_os = "linux", target_env = "gnu"))]
fn min_stack_size(attr: *const libc::pthread_attr_t) -> usize {
    // We use dlsym to avoid an ELF version dependency on GLIBC_PRIVATE. <br>我们使用 dlsym 来避免 ELF 版本对 GLIBC_PRIVATE 的依赖。<br>
    // (#23628) We shouldn't really be using such an internal symbol, but there's currently no other way to account for the TLS size. <br>(#23628) 我们真的不应该使用这样的内部符号，但目前没有其他方法可以解释 TLS 大小。<br>
    //
    dlsym!(fn __pthread_get_minstack(*const libc::pthread_attr_t) -> libc::size_t);

    match __pthread_get_minstack.get() {
        None => libc::PTHREAD_STACK_MIN,
        Some(f) => unsafe { f(attr) },
    }
}

// No point in looking up __pthread_get_minstack() on non-glibc platforms. <br>在非 glibc 平台上查找 __pthread_get_minstack() 毫无意义。<br>
#[cfg(all(not(all(target_os = "linux", target_env = "gnu")), not(target_os = "netbsd")))]
fn min_stack_size(_: *const libc::pthread_attr_t) -> usize {
    libc::PTHREAD_STACK_MIN
}

#[cfg(target_os = "netbsd")]
fn min_stack_size(_: *const libc::pthread_attr_t) -> usize {
    2048 // just a guess <br>只是一个猜测<br>
}
