#![cfg(target_thread_local)]
#![unstable(feature = "thread_local_internals", issue = "none")]

//! 提供不具有关联的 "key" 的线程局部析构函数，这可以提高效率。
//!

// 由于看起来好像是 glibc 2.18，因此已经发布了该符号，GCC 和 clang 都使用它们在 thread_local 全局变量中调用析构函数，所以我们做同样的事情！
//
// 但是请注意，我们在许多较旧的 Linux 上运行，以及从较新的 linux 到较旧的 linux 进行交叉编译，因此我们也有一个可替代的实现。
//
//
//
//
#[cfg(any(
    target_os = "linux",
    target_os = "fuchsia",
    target_os = "redox",
    target_os = "emscripten"
))]
pub unsafe fn register_dtor(t: *mut u8, dtor: unsafe extern "C" fn(*mut u8)) {
    use crate::mem;
    use crate::sys_common::thread_local_dtor::register_dtor_fallback;

    extern "C" {
        #[linkage = "extern_weak"]
        static __dso_handle: *mut u8;
        #[linkage = "extern_weak"]
        static __cxa_thread_atexit_impl: *const libc::c_void;
    }
    if !__cxa_thread_atexit_impl.is_null() {
        type F = unsafe extern "C" fn(
            dtor: unsafe extern "C" fn(*mut u8),
            arg: *mut u8,
            dso_handle: *mut u8,
        ) -> libc::c_int;
        mem::transmute::<*const libc::c_void, F>(__cxa_thread_atexit_impl)(
            dtor,
            t,
            &__dso_handle as *const _ as *mut _,
        );
        return;
    }
    register_dtor_fallback(t, dtor);
}

// 此实现与 sys_common/thread_local.rs 中的 register_dtor_fallback 非常相似。
// 主要区别在于我们希望将钩子转换为上述 linux 函数 _tlv_atexit 的 macOS 模拟。
//
// OSX 将在任何 TLS 插槽被释放之前，以及主线程退出时运行注册的 dtor。
//
// 不幸的是，在 tls dtor 运行时调用 _tlv_atexit 是 UB。
// 以下解决方法是每个线程通过 _tlv_atexit 注册一个自定义 DTOR 列表。
// 将 thread_local dtor 推送到 DTOR 列表，而无需调用 _tlv_atexit。
//
//
#[cfg(target_os = "macos")]
pub unsafe fn register_dtor(t: *mut u8, dtor: unsafe extern "C" fn(*mut u8)) {
    use crate::cell::Cell;
    use crate::ptr;

    #[thread_local]
    static REGISTERED: Cell<bool> = Cell::new(false);
    if !REGISTERED.get() {
        _tlv_atexit(run_dtors, ptr::null_mut());
        REGISTERED.set(true);
    }

    type List = Vec<(*mut u8, unsafe extern "C" fn(*mut u8))>;

    #[thread_local]
    static DTORS: Cell<*mut List> = Cell::new(ptr::null_mut());
    if DTORS.get().is_null() {
        let v: Box<List> = box Vec::new();
        DTORS.set(Box::into_raw(v));
    }

    extern "C" {
        fn _tlv_atexit(dtor: unsafe extern "C" fn(*mut u8), arg: *mut u8);
    }

    let list: &mut List = &mut *DTORS.get();
    list.push((t, dtor));

    unsafe extern "C" fn run_dtors(_: *mut u8) {
        let mut ptr = DTORS.replace(ptr::null_mut());
        while !ptr.is_null() {
            let list = Box::from_raw(ptr);
            for (ptr, dtor) in list.into_iter() {
                dtor(ptr);
            }
            ptr = DTORS.replace(ptr::null_mut());
        }
    }
}

#[cfg(any(target_os = "vxworks", target_os = "horizon"))]
pub unsafe fn register_dtor(t: *mut u8, dtor: unsafe extern "C" fn(*mut u8)) {
    use crate::sys_common::thread_local_dtor::register_dtor_fallback;
    register_dtor_fallback(t, dtor);
}
