//! 使用 `pthread` 同步原语的没有 `futex` 的线程停放。

#![cfg(not(any(
    target_os = "linux",
    target_os = "android",
    all(target_os = "emscripten", target_feature = "atomics"),
    target_os = "freebsd",
    target_os = "openbsd",
    target_os = "dragonfly",
    target_os = "fuchsia",
)))]

use crate::cell::UnsafeCell;
use crate::marker::PhantomPinned;
use crate::pin::Pin;
use crate::ptr::addr_of_mut;
use crate::sync::atomic::AtomicUsize;
use crate::sync::atomic::Ordering::SeqCst;
use crate::time::Duration;

const EMPTY: usize = 0;
const PARKED: usize = 1;
const NOTIFIED: usize = 2;

unsafe fn lock(lock: *mut libc::pthread_mutex_t) {
    let r = libc::pthread_mutex_lock(lock);
    debug_assert_eq!(r, 0);
}

unsafe fn unlock(lock: *mut libc::pthread_mutex_t) {
    let r = libc::pthread_mutex_unlock(lock);
    debug_assert_eq!(r, 0);
}

unsafe fn notify_one(cond: *mut libc::pthread_cond_t) {
    let r = libc::pthread_cond_signal(cond);
    debug_assert_eq!(r, 0);
}

unsafe fn wait(cond: *mut libc::pthread_cond_t, lock: *mut libc::pthread_mutex_t) {
    let r = libc::pthread_cond_wait(cond, lock);
    debug_assert_eq!(r, 0);
}

const TIMESPEC_MAX: libc::timespec =
    libc::timespec { tv_sec: <libc::time_t>::MAX, tv_nsec: 1_000_000_000 - 1 };

unsafe fn wait_timeout(
    cond: *mut libc::pthread_cond_t,
    lock: *mut libc::pthread_mutex_t,
    dur: Duration,
) {
    // 在不支持 pthread_condattr_setclock 的系统上使用系统时钟。
    // 不幸的是，当系统时间更改时，这会导致问题。
    #[cfg(any(target_os = "macos", target_os = "ios", target_os = "espidf"))]
    let (now, dur) = {
        use super::time::SystemTime;
        use crate::cmp::min;

        // `pthread_cond_timedwait` 的 OSX 实现存在超长持续时间的错误。当持续时间大于
        // 0x100_0000_0000_0000 秒，macOS Sierra 中的 `pthread_cond_timedwait` 返回错误 316。
        //
        // 该程序演示了该问题：
        // https://gist.github.com/stepancheg/198db4623a20aad2ad7cddb8fda4a63c
        //
        // 为了解决这个问题以及其他操作系统可能存在的错误，超时被限制为 1000 年，由于虚假唤醒，这是 `park_timeout` 的 API 允许的。
        //
        //
        //
        //
        let dur = min(dur, Duration::from_secs(1000 * 365 * 86400));
        let now = SystemTime::now().t;
        (now, dur)
    };
    // 在其他系统上使用单调时钟。
    #[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "espidf")))]
    let (now, dur) = {
        use super::time::Timespec;

        (Timespec::now(libc::CLOCK_MONOTONIC), dur)
    };

    let timeout =
        now.checked_add_duration(&dur).and_then(|t| t.to_timespec()).unwrap_or(TIMESPEC_MAX);
    let r = libc::pthread_cond_timedwait(cond, lock, &timeout);
    debug_assert!(r == libc::ETIMEDOUT || r == 0);
}

pub struct Parker {
    state: AtomicUsize,
    lock: UnsafeCell<libc::pthread_mutex_t>,
    cvar: UnsafeCell<libc::pthread_cond_t>,
    // `pthread` 原语需要一个稳定的地址，所以使这个结构体 `!Unpin`。
    _pinned: PhantomPinned,
}

impl Parker {
    /// 就地构建 UNIX parker。
    ///
    /// # Safety
    /// 绝不能移动构造好的 parker。
    pub unsafe fn new(parker: *mut Parker) {
        // 使用默认的互斥锁实现以允许更简单的初始化。
        // 这可能会导致死锁时出现未定义的行为。这可以通过不死锁来避免。
        // 特别注意任何 panic 之前的解锁操作，因为 panic 之后的代码可能会再次尝试 park。
        //
        addr_of_mut!((*parker).state).write(AtomicUsize::new(EMPTY));
        addr_of_mut!((*parker).lock).write(UnsafeCell::new(libc::PTHREAD_MUTEX_INITIALIZER));

        cfg_if::cfg_if! {
            if #[cfg(any(
                target_os = "macos",
                target_os = "ios",
                target_os = "l4re",
                target_os = "android",
                target_os = "redox"
            ))] {
                addr_of_mut!((*parker).cvar).write(UnsafeCell::new(libc::PTHREAD_COND_INITIALIZER));
            } else if #[cfg(any(target_os = "espidf", target_os = "horizon"))] {
                let r = libc::pthread_cond_init(addr_of_mut!((*parker).cvar).cast(), crate::ptr::null());
                assert_eq!(r, 0);
            } else {
                use crate::mem::MaybeUninit;
                let mut attr = MaybeUninit::<libc::pthread_condattr_t>::uninit();
                let r = libc::pthread_condattr_init(attr.as_mut_ptr());
                assert_eq!(r, 0);
                let r = libc::pthread_condattr_setclock(attr.as_mut_ptr(), libc::CLOCK_MONOTONIC);
                assert_eq!(r, 0);
                let r = libc::pthread_cond_init(addr_of_mut!((*parker).cvar).cast(), attr.as_ptr());
                assert_eq!(r, 0);
                let r = libc::pthread_condattr_destroy(attr.as_mut_ptr());
                assert_eq!(r, 0);
            }
        }
    }

    // 此实现不需要 `unsafe`，但其他实现可能假定这仅由拥有 Parker 的线程调用。
    //
    pub unsafe fn park(self: Pin<&Self>) {
        // 如果我们之前收到通知，那么我们将消耗此通知并迅速返回。
        //
        if self.state.compare_exchange(NOTIFIED, EMPTY, SeqCst, SeqCst).is_ok() {
            return;
        }

        // 否则我们需要协调睡眠
        lock(self.lock.get());
        match self.state.compare_exchange(EMPTY, PARKED, SeqCst, SeqCst) {
            Ok(_) => {}
            Err(NOTIFIED) => {
                // 即使我们知道它将是 `NOTIFIED`，也必须在这里读取。
                // 这是因为自从我们在上面的 `compare_exchange` 中读取 `NOTIFIED` 以来，可能已经再次调用 `unpark`。
                // 我们必须执行与该 `unpark` 同步的获取操作，以观察它在取消驻留之前进行的所有写操作。
                //
                // 为此，我们必须从对 `state` 的写入中读取。
                //
                let old = self.state.swap(EMPTY, SeqCst);

                unlock(self.lock.get());

                assert_eq!(old, NOTIFIED, "park state changed unexpectedly");
                return;
            } // 应该消费这个通知，因此禁止在下一个 park 中进行虚假唤醒。
            Err(_) => {
                unlock(self.lock.get());

                panic!("inconsistent park state")
            }
        }

        loop {
            wait(self.cvar.get(), self.lock.get());

            match self.state.compare_exchange(NOTIFIED, EMPTY, SeqCst, SeqCst) {
                Ok(_) => break, // 收到通知
                Err(_) => {}    // 虚假唤醒，回去睡觉
            }
        }

        unlock(self.lock.get());
    }

    // 此实现不需要 `unsafe`，但其他实现可能假定这仅由拥有 Parker 的线程调用。
    // 使用 `Pin` 保证互斥锁和条件变量的地址稳定。
    //
    pub unsafe fn park_timeout(self: Pin<&Self>, dur: Duration) {
        // 像上面的 `park` 一样，我们为已通知的线程提供了一条快速路径，然后我们开始协调睡眠。
        //
        // 快速返回。
        if self.state.compare_exchange(NOTIFIED, EMPTY, SeqCst, SeqCst).is_ok() {
            return;
        }

        lock(self.lock.get());
        match self.state.compare_exchange(EMPTY, PARKED, SeqCst, SeqCst) {
            Ok(_) => {}
            Err(NOTIFIED) => {
                // 我们必须在这里再次读取，请参见 `park`。
                let old = self.state.swap(EMPTY, SeqCst);
                unlock(self.lock.get());

                assert_eq!(old, NOTIFIED, "park state changed unexpectedly");
                return;
            } // 应该消费这个通知，因此禁止在下一个 park 中进行虚假唤醒。
            Err(_) => {
                unlock(self.lock.get());
                panic!("inconsistent park_timeout state")
            }
        }

        // 等待超时，如果我们虚假地唤醒或以其他方式从通知中醒来，我们只是想无条件地将状态设置为空，既可以使用通知，也可以不标记自己为停止状态。
        //
        //
        //
        wait_timeout(self.cvar.get(), self.lock.get(), dur);

        match self.state.swap(EMPTY, SeqCst) {
            NOTIFIED => unlock(self.lock.get()), // 收到通知，欢呼！
            PARKED => unlock(self.lock.get()),   // 没有通知，alas
            n => {
                unlock(self.lock.get());
                panic!("inconsistent park_timeout state: {n}")
            }
        }
    }

    pub fn unpark(self: Pin<&Self>) {
        // 为了确保未驻留线程可以观察到我们在此调用之前所做的任何写操作，我们必须执行 `park` 可以与之同步的释放操作。
        // 为此，即使 `state` 已经是 `NOTIFIED`，我们也必须编写 `NOTIFIED`。
        // 这就是为什么这必须是交换操作，而不是如果失败读取 `NOTIFIED` 时返回的比较交换。
        //
        //
        //
        match self.state.swap(NOTIFIED, SeqCst) {
            EMPTY => return,    // 没有人在等
            NOTIFIED => return, // 已经停止
            PARKED => {}        // 要去叫醒某人
            _ => panic!("inconsistent state in unpark"),
        }

        // 从暂停的线程将 `state` 设置为 `PARKED` (或在虚假唤醒的情况下最后检查的 `state`) 到它实际等待 `cvar` 之间存在一段时间。
        // 如果我们在此期间进行通知，它将被忽略，然后当驻留线程进入睡眠状态时，它将永远不会醒来。
        // 幸运的是，它在此阶段已锁定 `lock`，因此我们可以获取 `lock` 以等待它准备好接收通知。
        //
        // 在调用 `notify_one` 之前释放 `lock` 意味着，当驻留线程唤醒时，不必等待我们释放 `lock` 即可唤醒它。
        //
        //
        //
        //
        //
        //
        unsafe {
            lock(self.lock.get());
            unlock(self.lock.get());
            notify_one(self.cvar.get());
        }
    }
}

impl Drop for Parker {
    fn drop(&mut self) {
        unsafe {
            libc::pthread_cond_destroy(self.cvar.get_mut());
            libc::pthread_mutex_destroy(self.lock.get_mut());
        }
    }
}

unsafe impl Sync for Parker {}
unsafe impl Send for Parker {}
