//! Support for "weak linkage" to symbols on Unix <br>支持 "weak linkage" 到 Unix 上的符号<br>
//!
//! Some I/O operations we do in libstd require newer versions of OSes but we need to maintain binary compatibility with older releases for now. <br>我们在 libstd 中执行的某些 I/O 操作需要较新版本的 OS，但目前我们需要与较早版本保持二进制兼容性。<br>
//! In order to use the new functionality when available we use this module for detection. <br>为了在可用时使用新功能，我们使用此模块进行检测。<br>
//!
//! One option to use here is weak linkage, but that is unfortunately only really workable with ELF. <br>此处使用的一种选择是弱链接，但不幸的是，这仅适用于 ELF。<br> Otherwise, use dlsym to get the symbol value at runtime. <br>否则，使用 dlsym 在运行时获取符号值。<br>
//! This is also done for compatibility with older versions of glibc, and to avoid creating dependencies on GLIBC_PRIVATE symbols. <br>这样做也是为了与 glibc 的较早版本兼容，并避免在 GLIBC_PRIVATE 符号上创建依赖项。<br>
//! It assumes that we've been dynamically linked to the library the symbol comes from, but that is currently always the case for things like libpthread/libc. <br>假定我们已经动态链接到符号所来自的库，但是对于诸如 libpthread/libc 之类的东西，目前始终如此。<br>
//!
//! A long time ago this used weak linkage for the __pthread_get_minstack symbol, but that caused Debian to detect an unnecessarily strict versioned dependency on libc6 (#23628) because it is GLIBC_PRIVATE. <br>很久以前，这对 __pthread_get_minstack 符号使用了弱链接，但这导致 Debian 检测到对 libc6 (#23628) 的不必要的严格版本依赖，因为它是 GLIBC_PRIVATE。<br>
//!
//! We now use `dlsym` for a runtime lookup of that symbol to avoid the ELF versioned dependency. <br>我们现在使用 `dlsym` 进行该符号的运行时查找，以避免 ELF 版本依赖。<br>
//!
//!
//!
//!
//!
//!

// There are a variety of `#[cfg]`s controlling which targets are involved in each instance of `weak!` and `syscall!`. <br>有各种 `#[cfg]` 控制 `weak!` 和 `syscall!` 的每个实例中涉及哪些目标。<br>
// Rather than trying to unify all of that, we'll just allow that some unix targets don't use this module at all. <br>而不是试图统一所有这些，我们只允许某些 unix 目标根本不使用此模块。<br>
//
#![allow(dead_code, unused_macros)]

use crate::ffi::CStr;
use crate::marker::PhantomData;
use crate::mem;
use crate::ptr;
use crate::sync::atomic::{self, AtomicPtr, Ordering};

// We can use true weak linkage on ELF targets. <br>我们可以在 ELF 目标上使用真正的弱链接。<br>
#[cfg(not(any(target_os = "macos", target_os = "ios")))]
pub(crate) macro weak {
    (fn $name:ident($($t:ty),*) -> $ret:ty) => (
        let ref $name: ExternWeak<unsafe extern "C" fn($($t),*) -> $ret> = {
            extern "C" {
                #[linkage = "extern_weak"]
                static $name: *const libc::c_void;
            }
            #[allow(unused_unsafe)]
            ExternWeak::new(unsafe { $name })
        };
    )
}

// On non-ELF targets, use the dlsym approximation of weak linkage. <br>在非 ELF 目标上，使用弱链接的 dlsym 近似值。<br>
#[cfg(any(target_os = "macos", target_os = "ios"))]
pub(crate) use self::dlsym as weak;

pub(crate) struct ExternWeak<F> {
    weak_ptr: *const libc::c_void,
    _marker: PhantomData<F>,
}

impl<F> ExternWeak<F> {
    #[inline]
    pub(crate) fn new(weak_ptr: *const libc::c_void) -> Self {
        ExternWeak { weak_ptr, _marker: PhantomData }
    }
}

impl<F> ExternWeak<F> {
    #[inline]
    pub(crate) fn get(&self) -> Option<F> {
        unsafe {
            if self.weak_ptr.is_null() {
                None
            } else {
                Some(mem::transmute_copy::<*const libc::c_void, F>(&self.weak_ptr))
            }
        }
    }
}

pub(crate) macro dlsym {
    (fn $name:ident($($t:ty),*) -> $ret:ty) => (
         dlsym!(fn $name($($t),*) -> $ret, stringify!($name));
    ),
    (fn $name:ident($($t:ty),*) -> $ret:ty, $sym:expr) => (
        static DLSYM: DlsymWeak<unsafe extern "C" fn($($t),*) -> $ret> =
            DlsymWeak::new(concat!($sym, '\0'));
        let $name = &DLSYM;
    )
}
pub(crate) struct DlsymWeak<F> {
    name: &'static str,
    func: AtomicPtr<libc::c_void>,
    _marker: PhantomData<F>,
}

impl<F> DlsymWeak<F> {
    pub(crate) const fn new(name: &'static str) -> Self {
        DlsymWeak { name, func: AtomicPtr::new(ptr::invalid_mut(1)), _marker: PhantomData }
    }

    #[inline]
    pub(crate) fn get(&self) -> Option<F> {
        unsafe {
            // Relaxed is fine here because we fence before reading through the pointer (see the comment below). <br>在这里放松是很好的，因为在读取指针之前设置了栅栏 (请参见下面的注释)。<br>
            //
            match self.func.load(Ordering::Relaxed) {
                func if func.addr() == 1 => self.initialize(),
                func if func.is_null() => None,
                func => {
                    let func = mem::transmute_copy::<*mut libc::c_void, F>(&func);
                    // The caller is presumably going to read through this value (by calling the function we've dlsymed). <br>可能是调用者将读取该值 (通过调用 dlsymed 的函数)。<br> This means we'd need to have loaded it with at least C11's consume ordering in order to be guaranteed that the data we read from the pointer isn't from before the pointer was stored. <br>这意味着我们需要至少用 C11 的消耗顺序加载它，以确保从指针读取的数据不是存储指针之前的数据。<br>
                    //
                    // Rust has no equivalent to memory_order_consume, so we use an acquire fence (sorry, ARM). <br>Rust 不具有 memory_order_consume 的等效项，所以我们使用了 acquire 栅栏 (对不起，ARM)。<br>
                    //
                    // Now, in practice this likely isn't needed even on CPUs where relaxed and consume mean different things. <br>现在，实际上，即使在放松和消耗意味着不同的 CPU 上也不需要这样做。<br>
                    // The symbols we're loading are probably present (or not) at init, and even if they aren't the runtime dynamic loader is extremely likely have sufficient barriers internally (possibly implicitly, for example the ones provided by invoking `mprotect`). <br>我们正在加载的符号可能在 init 上存在 (或不存在)，即使它们不是运行时动态加载程序，也极有可能在内部具有足够的屏障 (可能是隐式的，例如，调用 `mprotect` 所提供的屏障)。<br>
                    //
                    // That said, none of that's *guaranteed*, and so we fence. <br>也就是说，所有这些都不能保证，所以我们使用了栅栏。<br>
                    //
                    //
                    //
                    //
                    //
                    //
                    //
                    //
                    //
                    atomic::fence(Ordering::Acquire);
                    Some(func)
                }
            }
        }
    }

    // Cold because it should only happen during first-time initialization. <br>Cold，因为它应该只在第一次初始化时发生。<br>
    #[cold]
    unsafe fn initialize(&self) -> Option<F> {
        assert_eq!(mem::size_of::<F>(), mem::size_of::<*mut libc::c_void>());

        let val = fetch(self.name);
        // This synchronizes with the acquire fence in `get`. <br>这与 `get` 中的 acquire 栅栏同步。<br>
        self.func.store(val, Ordering::Release);

        if val.is_null() { None } else { Some(mem::transmute_copy::<*mut libc::c_void, F>(&val)) }
    }
}

unsafe fn fetch(name: &str) -> *mut libc::c_void {
    let name = match CStr::from_bytes_with_nul(name.as_bytes()) {
        Ok(cstr) => cstr,
        Err(..) => return ptr::null_mut(),
    };
    libc::dlsym(libc::RTLD_DEFAULT, name.as_ptr())
}

#[cfg(not(any(target_os = "linux", target_os = "android")))]
pub(crate) macro syscall {
    (fn $name:ident($($arg_name:ident: $t:ty),*) -> $ret:ty) => (
        unsafe fn $name($($arg_name: $t),*) -> $ret {
            weak! { fn $name($($t),*) -> $ret }

            if let Some(fun) = $name.get() {
                fun($($arg_name),*)
            } else {
                super::os::set_errno(libc::ENOSYS);
                -1
            }
        }
    )
}

#[cfg(any(target_os = "linux", target_os = "android"))]
pub(crate) macro syscall {
    (fn $name:ident($($arg_name:ident: $t:ty),*) -> $ret:ty) => (
        unsafe fn $name($($arg_name:$t),*) -> $ret {
            weak! { fn $name($($t),*) -> $ret }

            // Use a weak symbol from libc when possible, allowing `LD_PRELOAD` interposition, but if it's not found just use a raw syscall. <br>尽可能使用 libc 的弱符号，允许 `LD_PRELOAD` 插入，但如果找不到，则使用原始 syscall。<br>
            //
            if let Some(fun) = $name.get() {
                fun($($arg_name),*)
            } else {
                // This looks like a hack, but concat_idents only accepts idents (not paths). <br>这看起来像黑客，但是 concat_idents 仅接受身份 (不包含路径)。<br>
                //
                use libc::*;

                syscall(
                    concat_idents!(SYS_, $name),
                    $($arg_name),*
                ) as $ret
            }
        }
    )
}

#[cfg(any(target_os = "linux", target_os = "android"))]
pub(crate) macro raw_syscall {
    (fn $name:ident($($arg_name:ident: $t:ty),*) -> $ret:ty) => (
        unsafe fn $name($($arg_name:$t),*) -> $ret {
            // This looks like a hack, but concat_idents only accepts idents (not paths). <br>这看起来像黑客，但是 concat_idents 仅接受身份 (不包含路径)。<br>
            //
            use libc::*;

            syscall(
                concat_idents!(SYS_, $name),
                $($arg_name),*
            ) as $ret
        }
    )
}
