//! 这是不使用 emscripten 时 wasm 目标上的全局分配器的实现。
//! 在这种情况下，我们没有实际的运行时可以依靠分配，因此我们提供了自己的运行时！
//!
//! wasm 指令集有两条指令用于获取当前内存量和增长内存量。
//! 这些说明是我们能够构建分配器的基础，所以我们做到了！ 请注意，指令也很漂亮 "global"，毕竟这就是 "global" 分配器!
//!
//!
//! 这里的当前分配器是 `dlmalloc` crate，我们将其作为子模块包含在 rust-lang/rust 仓库中。
//! crate 是 dlmalloc.c 从 C 到 Rust 的端口，基本上就是这样，我们现在可以拥有 "纯 Rust"，这是当前在技术上需要的 (尚不能与 C 链接)。
//!
//! crate 本身提供了一个分配器，该分配器在 wasm 上没有同步，因为没有线程！
//!
//!
//!
//!
//!
//!

use crate::alloc::{GlobalAlloc, Layout, System};

static mut DLMALLOC: dlmalloc::Dlmalloc = dlmalloc::Dlmalloc::new();

#[stable(feature = "alloc_system_type", since = "1.28.0")]
unsafe impl GlobalAlloc for System {
    #[inline]
    unsafe fn alloc(&self, layout: Layout) -> *mut u8 {
        // SAFETY: DLMALLOC 访问保证是安全的，因为锁为我们提供了唯一且不可重入的访问。
        // 调用 malloc() 是安全的，因为此函数上的前提条件与 trait 方法的前提条件匹配。
        let _lock = lock::lock();
        unsafe { DLMALLOC.malloc(layout.size(), layout.align()) }
    }

    #[inline]
    unsafe fn alloc_zeroed(&self, layout: Layout) -> *mut u8 {
        // SAFETY: DLMALLOC 访问保证是安全的，因为锁为我们提供了唯一且不可重入的访问。
        // 调用 calloc() 是安全的，因为此函数上的前提条件与 trait 方法的前提条件匹配。
        let _lock = lock::lock();
        unsafe { DLMALLOC.calloc(layout.size(), layout.align()) }
    }

    #[inline]
    unsafe fn dealloc(&self, ptr: *mut u8, layout: Layout) {
        // SAFETY: DLMALLOC 访问保证是安全的，因为锁为我们提供了唯一且不可重入的访问。
        // 调用 free() 是安全的，因为此函数上的前提条件与 trait 方法的前提条件匹配。
        let _lock = lock::lock();
        unsafe { DLMALLOC.free(ptr, layout.size(), layout.align()) }
    }

    #[inline]
    unsafe fn realloc(&self, ptr: *mut u8, layout: Layout, new_size: usize) -> *mut u8 {
        // SAFETY: DLMALLOC 访问保证是安全的，因为锁为我们提供了唯一且不可重入的访问。
        // 调用 realloc() 是安全的，因为此函数上的前提条件与 trait 方法的前提条件匹配。
        let _lock = lock::lock();
        unsafe { DLMALLOC.realloc(ptr, layout.size(), layout.align(), new_size) }
    }
}

#[cfg(target_feature = "atomics")]
mod lock {
    use crate::sync::atomic::{AtomicI32, Ordering::SeqCst};

    static LOCKED: AtomicI32 = AtomicI32::new(0);

    pub struct DropLock;

    pub fn lock() -> DropLock {
        loop {
            if LOCKED.swap(1, SeqCst) == 0 {
                return DropLock;
            }
            // 好的，这就是令人沮丧的地方。在这一点上，我们需要同步获取一个锁，但是我们正在与其他线程竞争。
            // 通常，我们将执行某种形式的 `i32.atomic.wait`，如下所示：
            //
            //     unsafe {
            //         let r = core::arch::wasm32::i32_atomic_wait( LOCKED.as_mut_ptr(),
            //             1，  // 期望值
            //             -1, //     timeout );
            //         debug_assert!(r == 0 || r == 1);
            //     }
            //
            // 不幸的是，尽管这样做会导致主线程出现问题。Web 浏览器中的主线程永远不会阻塞，没有任何的例外。
            // 这意味着主线程实际上不能执行 `i32.atomic.wait` 指令。
            //
            // 因此，如果我们想在浏览器的上下文中工作，我们需要为主线程找出某种分配方案，当在 malloc 锁上发生争用时，我们会执行... something.
            //
            // 可能的想法包括：
            //
            // 1. 尝试获取 δ 锁。如果失败，则通过 `memory.grow` 回退到内存分配。后来只是... 不知何故... 随着时间的推移，将原始页面重新注入到主分配器中。
            // 这种策略的缺点是，每当主线程与其他线程一起出现时，就强制进行页面分配，这是不幸的。
            //
            // 2. 维护一种 "两级" 分配器方案的形式，其中主线程具有其自己的分配器。
            // 某种程度上，该分配器也将与分配器保持平衡，这不仅可以使线程之间的分配交叉，而且可以确保两个分配器在释放内存等方面保持 "balanced"。
            //
            // 但是，这似乎非常复杂。
            //
            // 由于缺乏其他想法，此处实现的当前策略只是简单地旋转。
            // 典型的自旋循环算法在这里对 CPU 使用某种形式的 "hint"，这是我们正在做的工作，以确保 CPU 不会过热，但是 wasm 没有这样的指令。
            //
            // 需要明确的是，在此处旋转不是一个很好的解决方案。
            // 另一个带锁的线程可能需要很长时间才能唤醒。例如，它可能位于 `memory.grow` 中，或者可能从 CPU 驱逐了 10ms 之类的时间片。
            // 在这些时间段内，我们的线程将 "helpfully" 坐在这里并占用 CPU 时间，直到它本身被驱逐或锁持有者完成为止。
            // 这意味着我们只是在浪费 CPU 时间，没有人受益。
            //
            // 但是，旋转确实具有不错的属性，即语义上正确，对所有线程公平分配内存，并且实现起来非常简单。
            //
            // 在 future 中，肯定会将 (hopefully) 替换为可以处理主线程限制的实际内存分配器。
            //
            // FIXME: 我们还可以在此处添加优化，以检测线程何时为主线程，并在所有非主线程上阻塞。
            // 但是，当前，我们无法知道浏览器主线程上有哪个 wasm 线程，但是如果可以确定，我们至少可以在某种程度上减轻这种旋转的成本。
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
        }
    }

    impl Drop for DropLock {
        fn drop(&mut self) {
            let r = LOCKED.swap(0, SeqCst);
            debug_assert_eq!(r, 1);

            // 请注意，由于上述逻辑，我们实际上不需要唤醒任何人，但是如果我们这样做，则可能看起来像这样：
            //
            //
            //     unsafe {        core::arch::wasm32::atomic_notify( LOCKED.as_mut_ptr(),
            //             1，// 只有一个线程) ； }
            //
            //
            //
            //
        }
    }
}

#[cfg(not(target_feature = "atomics"))]
mod lock {
    #[inline]
    pub fn lock() {} // 没有原子，没有线程，这很容易！
}
