//! This is an implementation of a global allocator on wasm targets when emscripten is not in use. <br>这是不使用 emscripten 时 wasm 目标上的全局分配器的实现。<br>
//! In that situation there's no actual runtime for us to lean on for allocation, so instead we provide our own! <br>在这种情况下，我们没有实际的运行时可以依靠分配，因此我们提供了自己的运行时！<br>
//!
//! The wasm instruction set has two instructions for getting the current amount of memory and growing the amount of memory. <br>wasm 指令集有两条指令用于获取当前内存量和增长内存量。<br>
//! These instructions are the foundation on which we're able to build an allocator, so we do so! <br>这些说明是我们能够构建分配器的基础，所以我们做到了！<br> Note that the instructions are also pretty "global" and this is the "global" allocator after all! <br>请注意，指令也很漂亮 "global"，毕竟这就是 "global" 分配器!<br>
//!
//!
//! The current allocator here is the `dlmalloc` crate which we've got included in the rust-lang/rust repository as a submodule. <br>这里的当前分配器是 `dlmalloc` crate，我们将其作为子模块包含在 rust-lang/rust 仓库中。<br>
//! The crate is a port of dlmalloc.c from C to Rust and is basically just so we can have "pure Rust" for now which is currently technically required (can't link with C yet). <br>crate 是 dlmalloc.c 从 C 到 Rust 的端口，基本上就是这样，我们现在可以拥有 "纯 Rust"，这是当前在技术上需要的 (尚不能与 C 链接)。<br>
//!
//! The crate itself provides a global allocator which on wasm has no synchronization as there are no threads! <br>crate 本身提供了一个分配器，该分配器在 wasm 上没有同步，因为没有线程！<br>
//!
//!
//!
//!
//!
//!

use crate::alloc::{GlobalAlloc, Layout, System};

static mut DLMALLOC: dlmalloc::Dlmalloc = dlmalloc::Dlmalloc::new();

#[stable(feature = "alloc_system_type", since = "1.28.0")]
unsafe impl GlobalAlloc for System {
    #[inline]
    unsafe fn alloc(&self, layout: Layout) -> *mut u8 {
        // SAFETY: DLMALLOC access is guaranteed to be safe because the lock gives us unique and non-reentrant access. <br>DLMALLOC 访问保证是安全的，因为锁为我们提供了唯一且不可重入的访问。<br>
        // Calling malloc() is safe because preconditions on this function match the trait method preconditions. <br>调用 malloc() 是安全的，因为此函数上的前提条件与 trait 方法的前提条件匹配。<br>
        let _lock = lock::lock();
        unsafe { DLMALLOC.malloc(layout.size(), layout.align()) }
    }

    #[inline]
    unsafe fn alloc_zeroed(&self, layout: Layout) -> *mut u8 {
        // SAFETY: DLMALLOC access is guaranteed to be safe because the lock gives us unique and non-reentrant access. <br>DLMALLOC 访问保证是安全的，因为锁为我们提供了唯一且不可重入的访问。<br>
        // Calling calloc() is safe because preconditions on this function match the trait method preconditions. <br>调用 calloc() 是安全的，因为此函数上的前提条件与 trait 方法的前提条件匹配。<br>
        let _lock = lock::lock();
        unsafe { DLMALLOC.calloc(layout.size(), layout.align()) }
    }

    #[inline]
    unsafe fn dealloc(&self, ptr: *mut u8, layout: Layout) {
        // SAFETY: DLMALLOC access is guaranteed to be safe because the lock gives us unique and non-reentrant access. <br>DLMALLOC 访问保证是安全的，因为锁为我们提供了唯一且不可重入的访问。<br>
        // Calling free() is safe because preconditions on this function match the trait method preconditions. <br>调用 free() 是安全的，因为此函数上的前提条件与 trait 方法的前提条件匹配。<br>
        let _lock = lock::lock();
        unsafe { DLMALLOC.free(ptr, layout.size(), layout.align()) }
    }

    #[inline]
    unsafe fn realloc(&self, ptr: *mut u8, layout: Layout, new_size: usize) -> *mut u8 {
        // SAFETY: DLMALLOC access is guaranteed to be safe because the lock gives us unique and non-reentrant access. <br>DLMALLOC 访问保证是安全的，因为锁为我们提供了唯一且不可重入的访问。<br>
        // Calling realloc() is safe because preconditions on this function match the trait method preconditions. <br>调用 realloc() 是安全的，因为此函数上的前提条件与 trait 方法的前提条件匹配。<br>
        let _lock = lock::lock();
        unsafe { DLMALLOC.realloc(ptr, layout.size(), layout.align(), new_size) }
    }
}

#[cfg(target_feature = "atomics")]
mod lock {
    use crate::sync::atomic::{AtomicI32, Ordering::SeqCst};

    static LOCKED: AtomicI32 = AtomicI32::new(0);

    pub struct DropLock;

    pub fn lock() -> DropLock {
        loop {
            if LOCKED.swap(1, SeqCst) == 0 {
                return DropLock;
            }
            // Ok so here's where things get a little depressing. <br>好的，这就是令人沮丧的地方。<br> At this point in time we need to synchronously acquire a lock, but we're contending with some other thread. <br>在这一点上，我们需要同步获取一个锁，但是我们正在与其他线程竞争。<br>
            // Typically we'd execute some form of `i32.atomic.wait` like so: <br>通常，我们将执行某种形式的 `i32.atomic.wait`，如下所示：<br>
            //
            //     unsafe {
            //         let r = core::arch::wasm32::i32_atomic_wait( LOCKED.as_mut_ptr(),
            //             1,  //     expected value <br>1，  // 期望值<br>
            //             -1, //     timeout );
            //         debug_assert!(r == 0 || r == 1);
            //     }
            //
            // Unfortunately though in doing so we would cause issues for the main thread. <br>不幸的是，尽管这样做会导致主线程出现问题。<br> The main thread in a web browser *cannot ever block*, no exceptions. <br>Web 浏览器中的主线程永远不会阻塞，没有任何的例外。<br>
            // This means that the main thread can't actually execute the `i32.atomic.wait` instruction. <br>这意味着主线程实际上不能执行 `i32.atomic.wait` 指令。<br>
            //
            // As a result if we want to work within the context of browsers we need to figure out some sort of allocation scheme for the main thread where when there's contention on the global malloc lock we do... <br>因此，如果我们想在浏览器的上下文中工作，我们需要为主线程找出某种分配方案，当在 malloc 锁上发生争用时，我们会执行...<br> something.
            //
            // Possible ideas include: <br>可能的想法包括：<br>
            //
            // 1. Attempt to acquire the global lock. <br>尝试获取 δ 锁。<br> If it fails, fall back to memory allocation via `memory.grow`. <br>如果失败，则通过 `memory.grow` 回退到内存分配。<br> Later just ... <br>后来只是...<br> somehow ... <br>不知何故...<br> inject this raw page back into the main allocator as it gets sliced up over time. <br>随着时间的推移，将原始页面重新注入到主分配器中。<br>
            // This strategy has the downside of forcing allocation of a page to happen whenever the main thread contents with other threads, which is unfortunate. <br>这种策略的缺点是，每当主线程与其他线程一起出现时，就强制进行页面分配，这是不幸的。<br>
            //
            // 2. Maintain a form of "two level" allocator scheme where the main thread has its own allocator. <br>维护一种 "两级" 分配器方案的形式，其中主线程具有其自己的分配器。<br>
            // Somehow this allocator would also be balanced with a global allocator, not only to have allocations cross between threads but also to ensure that the two allocators stay "balanced" in terms of free'd memory and such. <br>某种程度上，该分配器也将与分配器保持平衡，这不仅可以使线程之间的分配交叉，而且可以确保两个分配器在释放内存等方面保持 "balanced"。<br>
            //
            // This, however, seems significantly complicated. <br>但是，这似乎非常复杂。<br>
            //
            // Out of a lack of other ideas, the current strategy implemented here is to simply spin. <br>由于缺乏其他想法，此处实现的当前策略只是简单地旋转。<br>
            // Typical spin loop algorithms have some form of "hint" here to the CPU that it's what we're doing to ensure that the CPU doesn't get too hot, but wasm doesn't have such an instruction. <br>典型的自旋循环算法在这里对 CPU 使用某种形式的 "hint"，这是我们正在做的工作，以确保 CPU 不会过热，但是 wasm 没有这样的指令。<br>
            //
            // To be clear, spinning here is not a great solution. <br>需要明确的是，在此处旋转不是一个很好的解决方案。<br>
            // Another thread with the lock may take quite a long time to wake up. <br>另一个带锁的线程可能需要很长时间才能唤醒。<br> For example it could be in `memory.grow` or it could be evicted from the CPU for a timeslice like 10ms. <br>例如，它可能位于 `memory.grow` 中，或者可能从 CPU 驱逐了 10ms 之类的时间片。<br>
            // For these periods of time our thread will "helpfully" sit here and eat CPU time until it itself is evicted or the lock holder finishes. <br>在这些时间段内，我们的线程将 "helpfully" 坐在这里并占用 CPU 时间，直到它本身被驱逐或锁持有者完成为止。<br>
            // This means we're just burning and wasting CPU time to no one's benefit. <br>这意味着我们只是在浪费 CPU 时间，没有人受益。<br>
            //
            // Spinning does have the nice properties, though, of being semantically correct, being fair to all threads for memory allocation, and being simple enough to implement. <br>但是，旋转确实具有不错的属性，即语义上正确，对所有线程公平分配内存，并且实现起来非常简单。<br>
            //
            // This will surely (hopefully) be replaced in the future with a real memory allocator that can handle the restriction of the main thread. <br>在 future 中，肯定会将 (hopefully) 替换为可以处理主线程限制的实际内存分配器。<br>
            //
            // FIXME: We can also possibly add an optimization here to detect when a thread is the main thread or not and block on all non-main-thread threads. <br>我们还可以在此处添加优化，以检测线程何时为主线程，并在所有非主线程上阻塞。<br>
            // Currently, however, we have no way of knowing which wasm thread is on the browser main thread, but if we could figure out we could at least somewhat mitigate the cost of this spinning. <br>但是，当前，我们无法知道浏览器主线程上有哪个 wasm 线程，但是如果可以确定，我们至少可以在某种程度上减轻这种旋转的成本。<br>
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
            //
        }
    }

    impl Drop for DropLock {
        fn drop(&mut self) {
            let r = LOCKED.swap(0, SeqCst);
            debug_assert_eq!(r, 1);

            // Note that due to the above logic we don't actually need to wake anyone up, but if we did it'd likely look something like this: <br>请注意，由于上述逻辑，我们实际上不需要唤醒任何人，但是如果我们这样做，则可能看起来像这样：<br>
            //
            //
            //     unsafe {        core::arch::wasm32::atomic_notify( LOCKED.as_mut_ptr(),
            //             1, //     only one thread ); <br>1，// 只有一个线程) ；<br> }
            //
            //
            //
            //
        }
    }
}

#[cfg(not(target_feature = "atomics"))]
mod lock {
    #[inline]
    pub fn lock() {} // no atomics, no threads, that's easy! <br>没有原子，没有线程，这很容易！<br>
}
