use crate::arch::wasm32;
use crate::sync::atomic::AtomicU32;
use crate::time::Duration;

/// Wait for a futex_wake operation to wake us. <br>等待 futex_wake 操作唤醒我们。<br>
///
/// Returns directly if the futex doesn't hold the expected value. <br>如果 futex 不包含预期值，则直接返回。<br>
///
/// Returns false on timeout, and true in all other cases. <br>在超时时返回 false，在所有其他情况下返回 true。<br>
pub fn futex_wait(futex: &AtomicU32, expected: u32, timeout: Option<Duration>) -> bool {
    let timeout = timeout.and_then(|t| t.as_nanos().try_into().ok()).unwrap_or(-1);
    unsafe {
        wasm32::memory_atomic_wait32(
            futex as *const AtomicU32 as *mut i32,
            expected as i32,
            timeout,
        ) < 2
    }
}

/// Wake up one thread that's blocked on futex_wait on this futex. <br>在这个 futex 上唤醒一个在 futex_wait 上被阻塞的线程。<br>
///
/// Returns true if this actually woke up such a thread, or false if no thread was waiting on this futex. <br>如果这实际上唤醒了这样一个线程，则返回 true，如果没有线程在此 futex 上等待，则返回 false。<br>
///
pub fn futex_wake(futex: &AtomicU32) -> bool {
    unsafe { wasm32::memory_atomic_notify(futex as *const AtomicU32 as *mut i32, 1) > 0 }
}

/// Wake up all threads that are waiting on futex_wait on this futex. <br>唤醒在这个 futex 上等待 futex_wait 的所有线程。<br>
pub fn futex_wake_all(futex: &AtomicU32) {
    unsafe {
        wasm32::memory_atomic_notify(futex as *const AtomicU32 as *mut i32, i32::MAX as u32);
    }
}
