#![deny(unsafe_op_in_unsafe_fn)]

use crate::alloc::{GlobalAlloc, Layout, System};
use crate::ffi::c_void;
use crate::ptr;
use crate::sync::atomic::{AtomicPtr, Ordering};
use crate::sys::c;
use crate::sys::common::alloc::{realloc_fallback, MIN_ALIGN};

#[cfg(test)]
mod tests;

// Heap memory management on Windows is done by using the system Heap API (heapapi.h) <br>Windows 上的堆内存管理是通过使用系统 Heap API (heapapi.h) 完成的<br>
// See https://docs.microsoft.com/windows/win32/api/heapapi/

// Flag to indicate that the memory returned by `HeapAlloc` should be zeroed. <br>指示 `HeapAlloc` 返回的内存应清零的标志。<br>
const HEAP_ZERO_MEMORY: c::DWORD = 0x00000008;

extern "system" {
    // Get a handle to the default heap of the current process, or null if the operation fails. <br>获取当前进程的默认堆的句柄，如果操作失败，则为 null。<br>
    //
    // SAFETY: Successful calls to this function within the same process are assumed to always return the same handle, which remains valid for the entire lifetime of the process. <br>假设在同一进程中成功调用此函数会返回相同的句柄，该句柄在整个进程生命周期内始终有效。<br>
    //
    //
    // See https://docs.microsoft.com/windows/win32/api/heapapi/nf-heapapi-getprocessheap
    fn GetProcessHeap() -> c::HANDLE;

    // Allocate a block of `dwBytes` bytes of memory from a given heap `hHeap`. <br>从给定的堆 `hHeap` 分配一块 `dwBytes` 字节的内存。<br>
    // The allocated memory may be uninitialized, or zeroed if `dwFlags` is set to `HEAP_ZERO_MEMORY`. <br>如果 `dwFlags` 设置为 `HEAP_ZERO_MEMORY`，分配的内存可能未初始化或归零。<br>
    //
    //
    // Returns a pointer to the newly-allocated memory or null if the operation fails. <br>返回指向新分配内存的指针，如果操作失败，则返回 null。<br>
    // The returned pointer will be aligned to at least `MIN_ALIGN`. <br>返回的指针将至少与 `MIN_ALIGN` 对齐。<br>
    //
    // SAFETY:
    //  - `hHeap` must be a non-null handle returned by `GetProcessHeap`. <br>`hHeap` 必须是 `GetProcessHeap` 返回的非空句柄。<br>
    //  - `dwFlags` must be set to either zero or `HEAP_ZERO_MEMORY`. <br>`dwFlags` 必须设置为零或 `HEAP_ZERO_MEMORY`。<br>
    //
    // Note that `dwBytes` is allowed to be zero, contrary to some other allocators. <br>请注意，与其他一些分配器相反，`dwBytes` 可以为零。<br>
    //
    // See https://docs.microsoft.com/windows/win32/api/heapapi/nf-heapapi-heapalloc
    fn HeapAlloc(hHeap: c::HANDLE, dwFlags: c::DWORD, dwBytes: c::SIZE_T) -> c::LPVOID;

    // Reallocate a block of memory behind a given pointer `lpMem` from a given heap `hHeap`, to a block of at least `dwBytes` bytes, either shrinking the block in place, or allocating at a new location, copying memory, and freeing the original location. <br>将给定指针 `lpMem` 后面的一块内存从给定的堆 `hHeap` 重新分配到至少 `dwBytes` 字节的块，要么将块缩小到位，要么在新位置分配，复制内存并释放原始位置。<br>
    //
    //
    // Returns a pointer to the reallocated memory or null if the operation fails. <br>返回指向重新分配的内存的指针，如果操作失败，则返回 null。<br>
    // The returned pointer will be aligned to at least `MIN_ALIGN`. <br>返回的指针将至少与 `MIN_ALIGN` 对齐。<br>
    // If the operation fails the given block will never have been freed. <br>如果操作失败，给定的块将永远不会被释放。<br>
    //
    // SAFETY:
    //  - `hHeap` must be a non-null handle returned by `GetProcessHeap`. <br>`hHeap` 必须是 `GetProcessHeap` 返回的非空句柄。<br>
    //  - `dwFlags` must be set to zero. <br>`dwFlags` 必须设置为零。<br>
    //  - `lpMem` must be a non-null pointer to an allocated block returned by `HeapAlloc` or `HeapReAlloc`, that has not already been freed. <br>`lpMem` 必须是一个非空指针，指向由 `HeapAlloc` 或 `HeapReAlloc` 返回的尚未释放的已分配块。<br>
    // If the block was successfully reallocated at a new location, pointers pointing to the freed memory, such as `lpMem`, must not be dereferenced ever again. <br>如果块在新位置成功重新分配，则不能再次解引用指向已释放内存的指针，例如 `lpMem`。<br>
    //
    // Note that `dwBytes` is allowed to be zero, contrary to some other allocators. <br>请注意，与其他一些分配器相反，`dwBytes` 可以为零。<br>
    //
    // See https://docs.microsoft.com/windows/win32/api/heapapi/nf-heapapi-heaprealloc
    //
    //
    //
    fn HeapReAlloc(
        hHeap: c::HANDLE,
        dwFlags: c::DWORD,
        lpMem: c::LPVOID,
        dwBytes: c::SIZE_T,
    ) -> c::LPVOID;

    // Free a block of memory behind a given pointer `lpMem` from a given heap `hHeap`. <br>从给定的堆 `hHeap` 中释放给定指针 `lpMem` 后面的一块内存。<br>
    // Returns a nonzero value if the operation is successful, and zero if the operation fails. <br>如果操作成功则返回一个非零值，如果操作失败则返回零。<br>
    //
    // SAFETY:
    //  - `hHeap` must be a non-null handle returned by `GetProcessHeap`. <br>`hHeap` 必须是 `GetProcessHeap` 返回的非空句柄。<br>
    //  - `dwFlags` must be set to zero. <br>`dwFlags` 必须设置为零。<br>
    //  - `lpMem` must be a pointer to an allocated block returned by `HeapAlloc` or `HeapReAlloc`, that has not already been freed. <br>`lpMem` 必须是指向由 `HeapAlloc` 或 `HeapReAlloc` 返回的尚未释放的已分配块的指针。<br>
    // If the block was successfully freed, pointers pointing to the freed memory, such as `lpMem`, must not be dereferenced ever again. <br>如果块被成功释放，指向被释放内存的指针，如 `lpMem`，不得再次解引用。<br>
    //
    //
    // Note that `lpMem` is allowed to be null, which will not cause the operation to fail. <br>注意 `lpMem` 允许为空，这不会导致操作失败。<br>
    //
    // See https://docs.microsoft.com/windows/win32/api/heapapi/nf-heapapi-heapfree
    //
    fn HeapFree(hHeap: c::HANDLE, dwFlags: c::DWORD, lpMem: c::LPVOID) -> c::BOOL;
}

// Cached handle to the default heap of the current process. <br>缓存到当前进程的默认堆的句柄。<br>
// Either a non-null handle returned by `GetProcessHeap`, or null when not yet initialized or `GetProcessHeap` failed. <br>`GetProcessHeap` 返回的非空句柄，或在尚未初始化或 `GetProcessHeap` 失败时为空。<br>
static HEAP: AtomicPtr<c_void> = AtomicPtr::new(ptr::null_mut());

// Get a handle to the default heap of the current process, or null if the operation fails. <br>获取当前进程的默认堆的句柄，如果操作失败，则为 null。<br>
// If this operation is successful, `HEAP` will be successfully initialized and contain a non-null handle returned by `GetProcessHeap`. <br>如果此操作成功，则 `HEAP` 将成功初始化并包含 `GetProcessHeap` 返回的非空句柄。<br>
//
#[inline]
fn init_or_get_process_heap() -> c::HANDLE {
    let heap = HEAP.load(Ordering::Relaxed);
    if heap.is_null() {
        // `HEAP` has not yet been successfully initialized <br>`HEAP` 尚未成功初始化<br>
        let heap = unsafe { GetProcessHeap() };
        if !heap.is_null() {
            // SAFETY: No locking is needed because within the same process, successful calls to `GetProcessHeap` will always return the same value, even on different threads. <br>不需要锁定，因为在同一个进程中，成功调用 `GetProcessHeap` 将始终返回相同的值，即使在不同的线程上也是如此。<br>
            //
            HEAP.store(heap, Ordering::Release);

            // SAFETY: `HEAP` contains a non-null handle returned by `GetProcessHeap` <br>`HEAP` 包含一个由 `GetProcessHeap` 返回的非空句柄<br>
            heap
        } else {
            // Could not get the current process heap. <br>无法获取当前进程堆。<br>
            ptr::null_mut()
        }
    } else {
        // SAFETY: `HEAP` contains a non-null handle returned by `GetProcessHeap` <br>`HEAP` 包含一个由 `GetProcessHeap` 返回的非空句柄<br>
        heap
    }
}

// Get a non-null handle to the default heap of the current process. <br>获取当前进程默认堆的非空句柄。<br>
// SAFETY: `HEAP` must have been successfully initialized. <br>`HEAP` 必须已成功初始化。<br>
#[inline]
unsafe fn get_process_heap() -> c::HANDLE {
    HEAP.load(Ordering::Acquire)
}

// Header containing a pointer to the start of an allocated block. <br>包含指向已分配块开头的指针的标头。<br>
// SAFETY: Size and alignment must be <= `MIN_ALIGN`. <br>大小和对齐方式必须 <= `MIN_ALIGN`。<br>
#[repr(C)]
struct Header(*mut u8);

// Allocate a block of optionally zeroed memory for a given `layout`. <br>为给定的 `layout` 分配一块可选为零的内存。<br>
// SAFETY: Returns a pointer satisfying the guarantees of `System` about allocated pointers, or null if the operation fails. <br>返回一个满足 `System` 关于已分配指针的保证的指针，如果操作失败，则返回 null。<br>
// If this returns non-null `HEAP` will have been successfully initialized. <br>如果返回非空，那么 `HEAP` 将被成功初始化。<br>
//
#[inline]
unsafe fn allocate(layout: Layout, zeroed: bool) -> *mut u8 {
    let heap = init_or_get_process_heap();
    if heap.is_null() {
        // Allocation has failed, could not get the current process heap. <br>分配失败，无法获取当前进程堆。<br>
        return ptr::null_mut();
    }

    // Allocated memory will be either zeroed or uninitialized. <br>分配的内存将被清零或未初始化。<br>
    let flags = if zeroed { HEAP_ZERO_MEMORY } else { 0 };

    if layout.align() <= MIN_ALIGN {
        // SAFETY: `heap` is a non-null handle returned by `GetProcessHeap`. <br>`heap` 是 `GetProcessHeap` 返回的非空句柄。<br>
        // The returned pointer points to the start of an allocated block. <br>返回的指针指向已分配块的开始。<br>
        unsafe { HeapAlloc(heap, flags, layout.size()) as *mut u8 }
    } else {
        // Allocate extra padding in order to be able to satisfy the alignment. <br>分配额外的填充以便能够满足对齐。<br>
        let total = layout.align() + layout.size();

        // SAFETY: `heap` is a non-null handle returned by `GetProcessHeap`. <br>`heap` 是 `GetProcessHeap` 返回的非空句柄。<br>
        let ptr = unsafe { HeapAlloc(heap, flags, total) as *mut u8 };
        if ptr.is_null() {
            // Allocation has failed. <br>分配失败。<br>
            return ptr::null_mut();
        }

        // Create a correctly aligned pointer offset from the start of the allocated block, and write a header before it. <br>从分配的块的开头创建一个正确对齐的指针偏移量，并在它之前写入一个头。<br>
        //

        let offset = layout.align() - (ptr.addr() & (layout.align() - 1));
        // SAFETY: `MIN_ALIGN` <= `offset` <= `layout.align()` and the size of the allocated block is `layout.align() + layout.size()`. <br>`MIN_ALIGN` <= `offset` <= `layout.align()`，分配块的大小为 `layout.align() + layout.size()`。<br>
        // `aligned` will thus be a correctly aligned pointer inside the allocated block with at least `layout.size()` bytes after it and at least `MIN_ALIGN` bytes of padding before it. <br>因此，`aligned` 将是分配块内正确对齐的指针，在它之后至少有 `layout.size()` 个字节，在它之前至少有 `MIN_ALIGN` 个字节的填充。<br>
        //
        //
        let aligned = unsafe { ptr.add(offset) };
        // SAFETY: Because the size and alignment of a header is <= `MIN_ALIGN` and `aligned` is aligned to at least `MIN_ALIGN` and has at least `MIN_ALIGN` bytes of padding before it, it is safe to write a header directly before it. <br>因为标头的大小和对齐方式 <= `MIN_ALIGN` 并且 `aligned` 至少与 `MIN_ALIGN` 对齐并且在它之前至少有 `MIN_ALIGN` 个字节的填充，所以直接在它之前写入标头是安全的。<br>
        //
        //
        unsafe { ptr::write((aligned as *mut Header).offset(-1), Header(ptr)) };

        // SAFETY: The returned pointer does not point to the to the start of an allocated block, but there is a header readable directly before it containing the location of the start of the block. <br>返回的指针不指向已分配块的开始，但在它包含块开始位置之前有一个直接可读的标头。<br>
        //
        //
        aligned
    }
}

// All pointers returned by this allocator have, in addition to the guarantees of `GlobalAlloc`, the following properties: <br>此分配器返回的所有指针除了 `GlobalAlloc` 的保证外，还具有以下属性：<br>
//
// If the pointer was allocated or reallocated with a `layout` specifying an alignment <= `MIN_ALIGN` the pointer will be aligned to at least `MIN_ALIGN` and point to the start of the allocated block. <br>如果使用 `layout` 指定对齐 <= `MIN_ALIGN` 分配或重新分配指针，则指针将至少与 `MIN_ALIGN` 对齐并指向已分配块的开头。<br>
//
// If the pointer was allocated or reallocated with a `layout` specifying an alignment > `MIN_ALIGN` the pointer will be aligned to the specified alignment and not point to the start of the allocated block. <br>如果使用 `layout` 指定对齐 > `MIN_ALIGN` 分配或重新分配指针，则指针将与指定对齐对齐，而不指向已分配块的开头。<br>
//
// Instead there will be a header readable directly before the returned pointer, containing the actual location of the start of the block. <br>相反，在返回的指针之前会有一个直接可读的标头，包含块开始的实际位置。<br>
//
//
//
#[stable(feature = "alloc_system_type", since = "1.28.0")]
unsafe impl GlobalAlloc for System {
    #[inline]
    unsafe fn alloc(&self, layout: Layout) -> *mut u8 {
        // SAFETY: Pointers returned by `allocate` satisfy the guarantees of `System` <br>`allocate` 返回的指针满足 `System` 的保证<br>
        let zeroed = false;
        unsafe { allocate(layout, zeroed) }
    }

    #[inline]
    unsafe fn alloc_zeroed(&self, layout: Layout) -> *mut u8 {
        // SAFETY: Pointers returned by `allocate` satisfy the guarantees of `System` <br>`allocate` 返回的指针满足 `System` 的保证<br>
        let zeroed = true;
        unsafe { allocate(layout, zeroed) }
    }

    #[inline]
    unsafe fn dealloc(&self, ptr: *mut u8, layout: Layout) {
        let block = {
            if layout.align() <= MIN_ALIGN {
                ptr
            } else {
                // The location of the start of the block is stored in the padding before `ptr`. <br>块开始的位置存储在 `ptr` 之前的填充中。<br>

                // SAFETY: Because of the contract of `System`, `ptr` is guaranteed to be non-null and have a header readable directly before it. <br>由于 `System` 的约定，`ptr` 被保证为非空并且在它之前有一个直接可读的标头。<br>
                //
                unsafe { ptr::read((ptr as *mut Header).offset(-1)).0 }
            }
        };

        // SAFETY: because `ptr` has been successfully allocated with this allocator, `HEAP` must have been successfully initialized. <br>因为 `ptr` 已经用这个分配器成功分配了，所以 `HEAP` 一定已经成功初始化了。<br>
        //
        let heap = unsafe { get_process_heap() };

        // SAFETY: `heap` is a non-null handle returned by `GetProcessHeap`, `block` is a pointer to the start of an allocated block. <br>`heap` 是 `GetProcessHeap` 返回的非空句柄，`block` 是指向已分配块开头的指针。<br>
        //
        unsafe { HeapFree(heap, 0, block as c::LPVOID) };
    }

    #[inline]
    unsafe fn realloc(&self, ptr: *mut u8, layout: Layout, new_size: usize) -> *mut u8 {
        if layout.align() <= MIN_ALIGN {
            // SAFETY: because `ptr` has been successfully allocated with this allocator, `HEAP` must have been successfully initialized. <br>因为 `ptr` 已经用这个分配器成功分配了，所以 `HEAP` 一定已经成功初始化了。<br>
            //
            let heap = unsafe { get_process_heap() };

            // SAFETY: `heap` is a non-null handle returned by `GetProcessHeap`, `ptr` is a pointer to the start of an allocated block. <br>`heap` 是 `GetProcessHeap` 返回的非空句柄，`ptr` 是指向已分配块开头的指针。<br>
            //
            // The returned pointer points to the start of an allocated block. <br>返回的指针指向已分配块的开始。<br>
            unsafe { HeapReAlloc(heap, 0, ptr as c::LPVOID, new_size) as *mut u8 }
        } else {
            // SAFETY: `realloc_fallback` is implemented using `dealloc` and `alloc`, which will correctly handle `ptr` and return a pointer satisfying the guarantees of `System` <br>`realloc_fallback` 使用 `dealloc` 和 `alloc` 实现，它们将正确处理 `ptr` 并返回满足 `System` 保证的指针<br>
            //
            unsafe { realloc_fallback(self, ptr, layout, new_size) }
        }
    }
}
