//! Windows 命令行只是一个字符串
//! <https://docs.microsoft.com/en-us/archive/blogs/larryosterman/the-windows-command-line-is-just-a-string>
//!
//! 该模块实现了将该字符串转换为参数列表所需的解析。

#[cfg(test)]
mod tests;

use crate::ffi::OsString;
use crate::fmt;
use crate::io;
use crate::marker::PhantomData;
use crate::num::NonZeroU16;
use crate::os::windows::prelude::*;
use crate::path::PathBuf;
use crate::ptr::NonNull;
use crate::sys::c;
use crate::sys::process::ensure_no_nuls;
use crate::sys::windows::os::current_exe;
use crate::vec;

use core::iter;

pub fn args() -> Args {
    // SAFETY: `GetCommandLineW` 返回一个指向以空字符结尾的 UTF-16 字符串的指针，因此使用 `WStrUnits` 是安全的。
    //
    unsafe {
        let lp_cmd_line = c::GetCommandLineW();
        let parsed_args_list = parse_lp_cmd_line(WStrUnits::new(lp_cmd_line), || {
            current_exe().map(PathBuf::into_os_string).unwrap_or_else(|_| OsString::new())
        });

        Args { parsed_args_list: parsed_args_list.into_iter() }
    }
}

/// 实现 Windows 命令行参数解析算法。
///
/// 可以在以下位置找到 Microsoft 的 Windows CLI 参数格式文档
/// <https://docs.microsoft.com/en-us/cpp/cpp/main-function-command-line-args?view=msvc-160#parsing-c-command-line-arguments>
///
/// 更深入的解释在这里：
/// <https://daviddeley.com/autohotkey/parameters/parameters.htm#WIN>
///
/// Windows 包含一个在 shell32.dll 中进行命令行解析的函数。
/// 但是，不使用它有两个原因：
///
/// 1. 与该 DLL 链接会使进程注册为 GUI 应用程序。
/// 即使没有绘制 windows，GUI 应用程序也会增加大量开销。
/// 请参见 <https://randomascii.wordpress.com/2018/12/03/a-not-called-function-can-cause-a-5x-slowdown/>。
///
/// 2. 它不遵循上面前两个链接中概述的现代 C/C++ argv 规则。
///
/// 使用可从以下站点获得的扩展测试套件测试了该函数与 C/C++ 解析规则的等效性：<https://github.com/ChrisDenton/winarg/tree/std>。
///
///
fn parse_lp_cmd_line<'a, F: Fn() -> OsString>(
    lp_cmd_line: Option<WStrUnits<'a>>,
    exe_name: F,
) -> Vec<OsString> {
    const BACKSLASH: NonZeroU16 = NonZeroU16::new(b'\\' as u16).unwrap();
    const QUOTE: NonZeroU16 = NonZeroU16::new(b'"' as u16).unwrap();
    const TAB: NonZeroU16 = NonZeroU16::new(b'\t' as u16).unwrap();
    const SPACE: NonZeroU16 = NonZeroU16::new(b' ' as u16).unwrap();

    let mut ret_val = Vec::new();
    // 如果 cmd 行指针为空或指向空字符串，则将可执行文件的名称返回为 argv[0]。
    //
    if lp_cmd_line.as_ref().and_then(|cmd| cmd.peek()).is_none() {
        ret_val.push(exe_name());
        return ret_val;
    }
    let mut code_units = lp_cmd_line.unwrap();

    // 开头的可执行文件名称很特殊。
    let mut in_quotes = false;
    let mut cur = Vec::new();
    for w in &mut code_units {
        match w {
            // 无论如何，引号总是切换 `in_quotes`，因为在解析可执行文件名称时没有转义字符。
            //
            QUOTE => in_quotes = !in_quotes,
            // 如果不是 `in_quotes`，则空格以 argv[0] 结尾。
            SPACE | TAB if !in_quotes => break,
            // 在所有其他情况下，代码单元是按字面意思理解的。
            _ => cur.push(w.get()),
        }
    }
    // 跳过空格。
    code_units.advance_while(|w| w == SPACE || w == TAB);
    ret_val.push(OsString::from_wide(&cur));

    // 根据以下规则解析参数：
    // * 除空格、制表符、引号和反斜杠外，所有代码单元均按字面意思表示。
    // * 当不是 `in_quotes` 时，空格和制表符将参数分开。连续的空格和制表符被视为单个分隔符。
    // * 空格或制表符 `in_quotes` 按字面意思表示。
    // * 除非转义，否则引号会切换 `in_quotes` 模式。转义引用是按字面意思理解的。
    // * 如果前面有奇数个反斜杠，则可以对引号进行转义。
    // * 如果任何数量的反斜杠后面紧跟一个引号，则反斜杠的数量减半 (四舍五入)。
    //
    // * 后面没有引号的反斜杠都是字面意思。
    // * 如果 `in_quotes` 那么一个引号也可以使用另一个引号进行转义 (比如
    // 两个连续的引号变成一个字面量引号)。
    //
    let mut cur = Vec::new();
    let mut in_quotes = false;
    while let Some(w) = code_units.next() {
        match w {
            // 如果不是 `in_quotes`，则以空格或制表符结束参数。
            SPACE | TAB if !in_quotes => {
                ret_val.push(OsString::from_wide(&cur[..]));
                cur.truncate(0);

                // 跳过空格。
                code_units.advance_while(|w| w == SPACE || w == TAB);
            }
            // 反斜杠可以转义引号或反斜杠，但前提是连续的反斜杠后跟一个引号。
            BACKSLASH => {
                let backslash_count = code_units.advance_while(|w| w == BACKSLASH) + 1;
                if code_units.peek() == Some(QUOTE) {
                    cur.extend(iter::repeat(BACKSLASH.get()).take(backslash_count / 2));
                    // 如果有奇数个反斜杠，则转义引号。
                    if backslash_count % 2 == 1 {
                        code_units.next();
                        cur.push(QUOTE.get());
                    }
                } else {
                    // 如果最后没有引用，则没有转义。
                    cur.extend(iter::repeat(BACKSLASH.get()).take(backslash_count));
                }
            }
            // 如果 `in_quotes` 而不是反斜杠转义 (见上文)，则引用要么取消设置 `in_quote`，要么被另一个引用转义。
            //
            QUOTE if in_quotes => match code_units.peek() {
                // 当 `in_quotes` 产生一个文字引号时，两个连续的引号。
                Some(QUOTE) => {
                    cur.push(QUOTE.get());
                    code_units.next();
                }
                // 否则设置 `in_quotes`。
                Some(_) => in_quotes = false,
                // 命令行结束。
                // 即使是空的，也要推送 `cur`，当 `in_quotes` 仍然设置时，我们通过中断来实现。
                None => break,
            },
            // 如果不是 `in_quotes` 并且没有转义反斜杠 (见上文)，则引用设置 `in_quote`。
            QUOTE => in_quotes = true,
            // 其他的一切总是从字面上理解。
            _ => cur.push(w.get()),
        }
    }
    // 推送最后一个参数，如果有的话。
    if !cur.is_empty() || in_quotes {
        ret_val.push(OsString::from_wide(&cur[..]));
    }
    ret_val
}

pub struct Args {
    parsed_args_list: vec::IntoIter<OsString>,
}

impl fmt::Debug for Args {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.parsed_args_list.as_slice().fmt(f)
    }
}

impl Iterator for Args {
    type Item = OsString;
    fn next(&mut self) -> Option<OsString> {
        self.parsed_args_list.next()
    }
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.parsed_args_list.size_hint()
    }
}

impl DoubleEndedIterator for Args {
    fn next_back(&mut self) -> Option<OsString> {
        self.parsed_args_list.next_back()
    }
}

impl ExactSizeIterator for Args {
    fn len(&self) -> usize {
        self.parsed_args_list.len()
    }
}

/// LPWSTR 上的安全迭代器 (又名指向以 NULL 结尾的一系列 UTF-16 代码单元的指针)。
///
struct WStrUnits<'a> {
    // 指针永远不能为空...
    lpwstr: NonNull<u16>,
    // ... 并且它指向的内存必须在此生命周期内有效。
    lifetime: PhantomData<&'a [u16]>,
}
impl WStrUnits<'_> {
    /// 创建迭代器。如果 `lpwstr` 为空，则返回 `None`。
    ///
    /// SAFETY: `lpwstr` 必须指向一个以空字符结尾的宽字符串，该字符串至少与此结构体的生命周期一样长。
    ///
    unsafe fn new(lpwstr: *const u16) -> Option<Self> {
        Some(Self { lpwstr: NonNull::new(lpwstr as _)?, lifetime: PhantomData })
    }
    fn peek(&self) -> Option<NonZeroU16> {
        // SAFETY: 读取当前的项总是安全的，因为我们永远不会移出数组的边界。
        //
        unsafe { NonZeroU16::new(*self.lpwstr.as_ptr()) }
    }
    /// 在 `predicate` 返回 true 时推进迭代器。
    /// 返回其前进的项数。
    fn advance_while<P: FnMut(NonZeroU16) -> bool>(&mut self, mut predicate: P) -> usize {
        let mut counter = 0;
        while let Some(w) = self.peek() {
            if !predicate(w) {
                break;
            }
            counter += 1;
            self.next();
        }
        counter
    }
}
impl Iterator for WStrUnits<'_> {
    // 这永远不会返回零，因为这标志着字符串的结尾。
    type Item = NonZeroU16;
    fn next(&mut self) -> Option<NonZeroU16> {
        // SAFETY: 如果达到 NULL，我们立即返回。
        // 因此，在那之后推进指针是安全的。
        unsafe {
            let next = self.peek()?;
            self.lpwstr = NonNull::new_unchecked(self.lpwstr.as_ptr().add(1));
            Some(next)
        }
    }
}

#[derive(Debug)]
pub(crate) enum Arg {
    /// 添加引号 (如果需要)
    Regular(OsString),
    /// 追加没有引号的原始字符串
    Raw(OsString),
}

enum Quote {
    // 每个 arg 都被引用
    Always,
    // 引用空格和空参数
    Auto,
    // 附加参数没有任何变化 (#29494)
    Never,
}

pub(crate) fn append_arg(cmd: &mut Vec<u16>, arg: &Arg, force_quotes: bool) -> io::Result<()> {
    let (arg, quote) = match arg {
        Arg::Regular(arg) => (arg, if force_quotes { Quote::Always } else { Quote::Auto }),
        Arg::Raw(arg) => (arg, Quote::Never),
    };

    // 如果一个参数有 0 个字符，那么我们需要用引号将其括起来，以确保它实际上是在命令行中通过的，否则在另一端进行解析时它将被完全丢弃。
    //
    //
    ensure_no_nuls(arg)?;
    let arg_bytes = arg.bytes();
    let (quote, escape) = match quote {
        Quote::Always => (true, true),
        Quote::Auto => {
            (arg_bytes.iter().any(|c| *c == b' ' || *c == b'\t') || arg_bytes.is_empty(), true)
        }
        Quote::Never => (false, false),
    };
    if quote {
        cmd.push('"' as u16);
    }

    let mut backslashes: usize = 0;
    for x in arg.encode_wide() {
        if escape {
            if x == '\\' as u16 {
                backslashes += 1;
            } else {
                if x == '"' as u16 {
                    // 在内部 '"' 之前添加 n+1 个反斜杠，使总数达到 2n+1
                    cmd.extend((0..=backslashes).map(|_| '\\' as u16));
                }
                backslashes = 0;
            }
        }
        cmd.push(x);
    }

    if quote {
        // 在 '"' 结尾之前，将 n 个反斜杠加到总数 2n。
        cmd.extend((0..backslashes).map(|_| '\\' as u16));
        cmd.push('"' as u16);
    }
    Ok(())
}

pub(crate) fn make_bat_command_line(
    script: &[u16],
    args: &[Arg],
    force_quotes: bool,
) -> io::Result<Vec<u16>> {
    // 将命令行的开头设置为 `cmd.exe /c "` 需要将命令括在一对额外的引号中，因此此处使用尾引号。
    // 添加所有参数后，它将关闭。
    //
    //
    let mut cmd: Vec<u16> = "cmd.exe /c \"".encode_utf16().collect();

    // 推送由其引号对包围的脚本名称。
    cmd.push(b'"' as u16);
    // Windows 文件名不能包含 `"` 字符或以 `\\` 结尾。
    // 如果脚本名称存在，则返回错误。
    if script.contains(&(b'"' as u16)) || script.last() == Some(&(b'\\' as u16)) {
        return Err(io::const_io_error!(
            io::ErrorKind::InvalidInput,
            "Windows file names may not contain `\"` or end with `\\`"
        ));
    }
    cmd.extend_from_slice(script.strip_suffix(&[0]).unwrap_or(script));
    cmd.push(b'"' as u16);

    // 追加参数。
    // FIXME: 这需要测试以确保默认情况下批处理脚本正确重建参数。
    //
    for arg in args {
        cmd.push(' ' as u16);
        append_arg(&mut cmd, arg, force_quotes)?;
    }

    // 关闭我们之前打开的引用。
    cmd.push(b'"' as u16);

    Ok(cmd)
}

/// 采用路径并尝试返回非逐字路径。
///
/// 这是必要的，因为 cmd.exe 不支持逐字路径。
pub(crate) fn to_user_path(mut path: Vec<u16>) -> io::Result<Vec<u16>> {
    use crate::ptr;
    use crate::sys::windows::fill_utf16_buf;

    // UTF-16 编码的代码点，用于解析和构建 UTF-16 路径。
    // 所有这些都在 ASCII 范围内，因此它们可以直接转换为 `u16`。
    const SEP: u16 = b'\\' as _;
    const QUERY: u16 = b'?' as _;
    const COLON: u16 = b':' as _;
    const U: u16 = b'U' as _;
    const N: u16 = b'N' as _;
    const C: u16 = b'C' as _;

    // 如果路径太长而无法删除逐字前缀，则提前返回。
    const LEGACY_MAX_PATH: usize = 260;
    if path.len() > LEGACY_MAX_PATH {
        return Ok(path);
    }

    match &path[..] {
        // `\\?\C:\...` => `C:\...`
        [SEP, SEP, QUERY, SEP, _, COLON, SEP, ..] => unsafe {
            let lpfilename = path[4..].as_ptr();
            fill_utf16_buf(
                |buffer, size| c::GetFullPathNameW(lpfilename, size, buffer, ptr::null_mut()),
                |full_path: &[u16]| {
                    if full_path == &path[4..path.len() - 1] { full_path.into() } else { path }
                },
            )
        },
        // `\\?\UNC\...` => `\\...`
        [SEP, SEP, QUERY, SEP, U, N, C, SEP, ..] => unsafe {
            // 将 `UNC\` 中的 `C` 更改为 `\`，这样我们就可以得到一个以 `\\` 开头的切片。
            path[6] = b'\\' as u16;
            let lpfilename = path[6..].as_ptr();
            fill_utf16_buf(
                |buffer, size| c::GetFullPathNameW(lpfilename, size, buffer, ptr::null_mut()),
                |full_path: &[u16]| {
                    if full_path == &path[6..path.len() - 1] {
                        full_path.into()
                    } else {
                        // 在 "UNC" 中恢复 'C'。
                        path[6] = b'C' as u16;
                        path
                    }
                },
            )
        },
        // 对于其他一切，保持路径不变。
        _ => Ok(path),
    }
}
