//! The Windows command line is just a string <br>Windows 命令行只是一个字符串<br>
//! <https://docs.microsoft.com/en-us/archive/blogs/larryosterman/the-windows-command-line-is-just-a-string>
//!
//! This module implements the parsing necessary to turn that string into a list of arguments. <br>该模块实现了将该字符串转换为参数列表所需的解析。<br>

#[cfg(test)]
mod tests;

use crate::ffi::OsString;
use crate::fmt;
use crate::io;
use crate::marker::PhantomData;
use crate::num::NonZeroU16;
use crate::os::windows::prelude::*;
use crate::path::PathBuf;
use crate::ptr::NonNull;
use crate::sys::c;
use crate::sys::process::ensure_no_nuls;
use crate::sys::windows::os::current_exe;
use crate::vec;

use core::iter;

pub fn args() -> Args {
    // SAFETY: `GetCommandLineW` returns a pointer to a null terminated UTF-16 string so it's safe for `WStrUnits` to use. <br>`GetCommandLineW` 返回一个指向以空字符结尾的 UTF-16 字符串的指针，因此使用 `WStrUnits` 是安全的。<br>
    //
    unsafe {
        let lp_cmd_line = c::GetCommandLineW();
        let parsed_args_list = parse_lp_cmd_line(WStrUnits::new(lp_cmd_line), || {
            current_exe().map(PathBuf::into_os_string).unwrap_or_else(|_| OsString::new())
        });

        Args { parsed_args_list: parsed_args_list.into_iter() }
    }
}

/// Implements the Windows command-line argument parsing algorithm. <br>实现 Windows 命令行参数解析算法。<br>
///
/// Microsoft's documentation for the Windows CLI argument format can be found at <br>可以在以下位置找到 Microsoft 的 Windows CLI 参数格式文档<br>
/// <https://docs.microsoft.com/en-us/cpp/cpp/main-function-command-line-args?view=msvc-160#parsing-c-command-line-arguments>
///
/// A more in-depth explanation is here: <br>更深入的解释在这里：<br>
/// <https://daviddeley.com/autohotkey/parameters/parameters.htm#WIN>
///
/// Windows includes a function to do command line parsing in shell32.dll. <br>Windows 包含一个在 shell32.dll 中进行命令行解析的函数。<br>
/// However, this is not used for two reasons: <br>但是，不使用它有两个原因：<br>
///
/// 1. Linking with that DLL causes the process to be registered as a GUI application. <br>与该 DLL 链接会使进程注册为 GUI 应用程序。<br>
/// GUI applications add a bunch of overhead, even if no windows are drawn. <br>即使没有绘制 windows，GUI 应用程序也会增加大量开销。<br>
/// See <https://randomascii.wordpress.com/2018/12/03/a-not-called-function-can-cause-a-5x-slowdown/>. <br>请参见 <https://randomascii.wordpress.com/2018/12/03/a-not-called-function-can-cause-a-5x-slowdown/>。<br>
///
/// 2. It does not follow the modern C/C++ argv rules outlined in the first two links above. <br>它不遵循上面前两个链接中概述的现代 C/C++ argv 规则。<br>
///
/// This function was tested for equivalence to the C/C++ parsing rules using an extensive test suite available at <https://github.com/ChrisDenton/winarg/tree/std>. <br>使用可从以下站点获得的扩展测试套件测试了该函数与 C/C++ 解析规则的等效性：<https://github.com/ChrisDenton/winarg/tree/std>。<br>
///
///
fn parse_lp_cmd_line<'a, F: Fn() -> OsString>(
    lp_cmd_line: Option<WStrUnits<'a>>,
    exe_name: F,
) -> Vec<OsString> {
    const BACKSLASH: NonZeroU16 = NonZeroU16::new(b'\\' as u16).unwrap();
    const QUOTE: NonZeroU16 = NonZeroU16::new(b'"' as u16).unwrap();
    const TAB: NonZeroU16 = NonZeroU16::new(b'\t' as u16).unwrap();
    const SPACE: NonZeroU16 = NonZeroU16::new(b' ' as u16).unwrap();

    let mut ret_val = Vec::new();
    // If the cmd line pointer is null or it points to an empty string then return the name of the executable as argv[0]. <br>如果 cmd 行指针为空或指向空字符串，则将可执行文件的名称返回为 argv[0]。<br>
    //
    if lp_cmd_line.as_ref().and_then(|cmd| cmd.peek()).is_none() {
        ret_val.push(exe_name());
        return ret_val;
    }
    let mut code_units = lp_cmd_line.unwrap();

    // The executable name at the beginning is special. <br>开头的可执行文件名称很特殊。<br>
    let mut in_quotes = false;
    let mut cur = Vec::new();
    for w in &mut code_units {
        match w {
            // A quote mark always toggles `in_quotes` no matter what because there are no escape characters when parsing the executable name. <br>无论如何，引号总是切换 `in_quotes`，因为在解析可执行文件名称时没有转义字符。<br>
            //
            QUOTE => in_quotes = !in_quotes,
            // If not `in_quotes` then whitespace ends argv[0]. <br>如果不是 `in_quotes`，则空格以 argv[0] 结尾。<br>
            SPACE | TAB if !in_quotes => break,
            // In all other cases the code unit is taken literally. <br>在所有其他情况下，代码单元是按字面意思理解的。<br>
            _ => cur.push(w.get()),
        }
    }
    // Skip whitespace. <br>跳过空格。<br>
    code_units.advance_while(|w| w == SPACE || w == TAB);
    ret_val.push(OsString::from_wide(&cur));

    // Parse the arguments according to these rules: <br>根据以下规则解析参数：<br>
    // * All code units are taken literally except space, tab, quote and backslash. <br>除空格、制表符、引号和反斜杠外，所有代码单元均按字面意思表示。<br>
    // * When not `in_quotes`, space and tab separate arguments. <br>当不是 `in_quotes` 时，空格和制表符将参数分开。<br> Consecutive spaces and tabs are treated as a single separator. <br>连续的空格和制表符被视为单个分隔符。<br>
    // * A space or tab `in_quotes` is taken literally. <br>空格或制表符 `in_quotes` 按字面意思表示。<br>
    // * A quote toggles `in_quotes` mode unless it's escaped. <br>除非转义，否则引号会切换 `in_quotes` 模式。<br> An escaped quote is taken literally. <br>转义引用是按字面意思理解的。<br>
    // * A quote can be escaped if preceded by an odd number of backslashes. <br>如果前面有奇数个反斜杠，则可以对引号进行转义。<br>
    // * If any number of backslashes is immediately followed by a quote then the number of backslashes is halved (rounding down). <br>如果任何数量的反斜杠后面紧跟一个引号，则反斜杠的数量减半 (四舍五入)。<br>
    //
    // * Backslashes not followed by a quote are all taken literally. <br>后面没有引号的反斜杠都是字面意思。<br>
    // * If `in_quotes` then a quote can also be escaped using another quote (i.e. <br>如果 `in_quotes` 那么一个引号也可以使用另一个引号进行转义 (比如<br>
    // two consecutive quotes become one literal quote). <br>两个连续的引号变成一个字面量引号)。<br>
    //
    let mut cur = Vec::new();
    let mut in_quotes = false;
    while let Some(w) = code_units.next() {
        match w {
            // If not `in_quotes`, a space or tab ends the argument. <br>如果不是 `in_quotes`，则以空格或制表符结束参数。<br>
            SPACE | TAB if !in_quotes => {
                ret_val.push(OsString::from_wide(&cur[..]));
                cur.truncate(0);

                // Skip whitespace. <br>跳过空格。<br>
                code_units.advance_while(|w| w == SPACE || w == TAB);
            }
            // Backslashes can escape quotes or backslashes but only if consecutive backslashes are followed by a quote. <br>反斜杠可以转义引号或反斜杠，但前提是连续的反斜杠后跟一个引号。<br>
            BACKSLASH => {
                let backslash_count = code_units.advance_while(|w| w == BACKSLASH) + 1;
                if code_units.peek() == Some(QUOTE) {
                    cur.extend(iter::repeat(BACKSLASH.get()).take(backslash_count / 2));
                    // The quote is escaped if there are an odd number of backslashes. <br>如果有奇数个反斜杠，则转义引号。<br>
                    if backslash_count % 2 == 1 {
                        code_units.next();
                        cur.push(QUOTE.get());
                    }
                } else {
                    // If there is no quote on the end then there is no escaping. <br>如果最后没有引用，则没有转义。<br>
                    cur.extend(iter::repeat(BACKSLASH.get()).take(backslash_count));
                }
            }
            // If `in_quotes` and not backslash escaped (see above) then a quote either unsets `in_quote` or is escaped by another quote. <br>如果 `in_quotes` 而不是反斜杠转义 (见上文)，则引用要么取消设置 `in_quote`，要么被另一个引用转义。<br>
            //
            QUOTE if in_quotes => match code_units.peek() {
                // Two consecutive quotes when `in_quotes` produces one literal quote. <br>当 `in_quotes` 产生一个文字引号时，两个连续的引号。<br>
                Some(QUOTE) => {
                    cur.push(QUOTE.get());
                    code_units.next();
                }
                // Otherwise set `in_quotes`. <br>否则设置 `in_quotes`。<br>
                Some(_) => in_quotes = false,
                // The end of the command line. <br>命令行结束。<br>
                // Push `cur` even if empty, which we do by breaking while `in_quotes` is still set. <br>即使是空的，也要推送 `cur`，当 `in_quotes` 仍然设置时，我们通过中断来实现。<br>
                None => break,
            },
            // If not `in_quotes` and not BACKSLASH escaped (see above) then a quote sets `in_quote`. <br>如果不是 `in_quotes` 并且没有转义反斜杠 (见上文)，则引用设置 `in_quote`。<br>
            QUOTE => in_quotes = true,
            // Everything else is always taken literally. <br>其他的一切总是从字面上理解。<br>
            _ => cur.push(w.get()),
        }
    }
    // Push the final argument, if any. <br>推送最后一个参数，如果有的话。<br>
    if !cur.is_empty() || in_quotes {
        ret_val.push(OsString::from_wide(&cur[..]));
    }
    ret_val
}

pub struct Args {
    parsed_args_list: vec::IntoIter<OsString>,
}

impl fmt::Debug for Args {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.parsed_args_list.as_slice().fmt(f)
    }
}

impl Iterator for Args {
    type Item = OsString;
    fn next(&mut self) -> Option<OsString> {
        self.parsed_args_list.next()
    }
    fn size_hint(&self) -> (usize, Option<usize>) {
        self.parsed_args_list.size_hint()
    }
}

impl DoubleEndedIterator for Args {
    fn next_back(&mut self) -> Option<OsString> {
        self.parsed_args_list.next_back()
    }
}

impl ExactSizeIterator for Args {
    fn len(&self) -> usize {
        self.parsed_args_list.len()
    }
}

/// A safe iterator over a LPWSTR (aka a pointer to a series of UTF-16 code units terminated by a NULL). <br>LPWSTR 上的安全迭代器 (又名指向以 NULL 结尾的一系列 UTF-16 代码单元的指针)。<br>
///
struct WStrUnits<'a> {
    // The pointer must never be null... <br>指针永远不能为空...<br>
    lpwstr: NonNull<u16>,
    // ...and the memory it points to must be valid for this lifetime. <br>... 并且它指向的内存必须在此生命周期内有效。<br>
    lifetime: PhantomData<&'a [u16]>,
}
impl WStrUnits<'_> {
    /// Create the iterator. <br>创建迭代器。<br> Returns `None` if `lpwstr` is null. <br>如果 `lpwstr` 为空，则返回 `None`。<br>
    ///
    /// SAFETY: `lpwstr` must point to a null-terminated wide string that lives at least as long as the lifetime of this struct. <br>`lpwstr` 必须指向一个以空字符结尾的宽字符串，该字符串至少与此结构体的生命周期一样长。<br>
    ///
    unsafe fn new(lpwstr: *const u16) -> Option<Self> {
        Some(Self { lpwstr: NonNull::new(lpwstr as _)?, lifetime: PhantomData })
    }
    fn peek(&self) -> Option<NonZeroU16> {
        // SAFETY: It's always safe to read the current item because we don't ever move out of the array's bounds. <br>读取当前的项总是安全的，因为我们永远不会移出数组的边界。<br>
        //
        unsafe { NonZeroU16::new(*self.lpwstr.as_ptr()) }
    }
    /// Advance the iterator while `predicate` returns true. <br>在 `predicate` 返回 true 时推进迭代器。<br>
    /// Returns the number of items it advanced by. <br>返回其前进的项数。<br>
    fn advance_while<P: FnMut(NonZeroU16) -> bool>(&mut self, mut predicate: P) -> usize {
        let mut counter = 0;
        while let Some(w) = self.peek() {
            if !predicate(w) {
                break;
            }
            counter += 1;
            self.next();
        }
        counter
    }
}
impl Iterator for WStrUnits<'_> {
    // This can never return zero as that marks the end of the string. <br>这永远不会返回零，因为这标志着字符串的结尾。<br>
    type Item = NonZeroU16;
    fn next(&mut self) -> Option<NonZeroU16> {
        // SAFETY: If NULL is reached we immediately return. <br>如果达到 NULL，我们立即返回。<br>
        // Therefore it's safe to advance the pointer after that. <br>因此，在那之后推进指针是安全的。<br>
        unsafe {
            let next = self.peek()?;
            self.lpwstr = NonNull::new_unchecked(self.lpwstr.as_ptr().add(1));
            Some(next)
        }
    }
}

#[derive(Debug)]
pub(crate) enum Arg {
    /// Add quotes (if needed) <br>添加引号 (如果需要)<br>
    Regular(OsString),
    /// Append raw string without quoting <br>追加没有引号的原始字符串<br>
    Raw(OsString),
}

enum Quote {
    // Every arg is quoted <br>每个 arg 都被引用<br>
    Always,
    // Whitespace and empty args are quoted <br>引用空格和空参数<br>
    Auto,
    // Arg appended without any changes (#29494) <br>附加参数没有任何变化 (#29494)<br>
    Never,
}

pub(crate) fn append_arg(cmd: &mut Vec<u16>, arg: &Arg, force_quotes: bool) -> io::Result<()> {
    let (arg, quote) = match arg {
        Arg::Regular(arg) => (arg, if force_quotes { Quote::Always } else { Quote::Auto }),
        Arg::Raw(arg) => (arg, Quote::Never),
    };

    // If an argument has 0 characters then we need to quote it to ensure that it actually gets passed through on the command line or otherwise it will be dropped entirely when parsed on the other end. <br>如果一个参数有 0 个字符，那么我们需要用引号将其括起来，以确保它实际上是在命令行中通过的，否则在另一端进行解析时它将被完全丢弃。<br>
    //
    //
    ensure_no_nuls(arg)?;
    let arg_bytes = arg.bytes();
    let (quote, escape) = match quote {
        Quote::Always => (true, true),
        Quote::Auto => {
            (arg_bytes.iter().any(|c| *c == b' ' || *c == b'\t') || arg_bytes.is_empty(), true)
        }
        Quote::Never => (false, false),
    };
    if quote {
        cmd.push('"' as u16);
    }

    let mut backslashes: usize = 0;
    for x in arg.encode_wide() {
        if escape {
            if x == '\\' as u16 {
                backslashes += 1;
            } else {
                if x == '"' as u16 {
                    // Add n+1 backslashes to total 2n+1 before internal '"'. <br>在内部 '"' 之前添加 n+1 个反斜杠，使总数达到 2n+1<br>
                    cmd.extend((0..=backslashes).map(|_| '\\' as u16));
                }
                backslashes = 0;
            }
        }
        cmd.push(x);
    }

    if quote {
        // Add n backslashes to total 2n before ending '"'. <br>在 '"' 结尾之前，将 n 个反斜杠加到总数 2n。<br>
        cmd.extend((0..backslashes).map(|_| '\\' as u16));
        cmd.push('"' as u16);
    }
    Ok(())
}

pub(crate) fn make_bat_command_line(
    script: &[u16],
    args: &[Arg],
    force_quotes: bool,
) -> io::Result<Vec<u16>> {
    // Set the start of the command line to `cmd.exe /c "` It is necessary to surround the command in an extra pair of quotes, hence the trailing quote here. <br>将命令行的开头设置为 `cmd.exe /c "` 需要将命令括在一对额外的引号中，因此此处使用尾引号。<br>
    // It will be closed after all arguments have been added. <br>添加所有参数后，它将关闭。<br>
    //
    //
    let mut cmd: Vec<u16> = "cmd.exe /c \"".encode_utf16().collect();

    // Push the script name surrounded by its quote pair. <br>推送由其引号对包围的脚本名称。<br>
    cmd.push(b'"' as u16);
    // Windows file names cannot contain a `"` character or end with `\\`. <br>Windows 文件名不能包含 `"` 字符或以 `\\` 结尾。<br>
    // If the script name does then return an error. <br>如果脚本名称存在，则返回错误。<br>
    if script.contains(&(b'"' as u16)) || script.last() == Some(&(b'\\' as u16)) {
        return Err(io::const_io_error!(
            io::ErrorKind::InvalidInput,
            "Windows file names may not contain `\"` or end with `\\`"
        ));
    }
    cmd.extend_from_slice(script.strip_suffix(&[0]).unwrap_or(script));
    cmd.push(b'"' as u16);

    // Append the arguments. <br>追加参数。<br>
    // FIXME: This needs tests to ensure that the arguments are properly reconstructed by the batch script by default. <br>这需要测试以确保默认情况下批处理脚本正确重建参数。<br>
    //
    for arg in args {
        cmd.push(' ' as u16);
        append_arg(&mut cmd, arg, force_quotes)?;
    }

    // Close the quote we left opened earlier. <br>关闭我们之前打开的引用。<br>
    cmd.push(b'"' as u16);

    Ok(cmd)
}

/// Takes a path and tries to return a non-verbatim path. <br>采用路径并尝试返回非逐字路径。<br>
///
/// This is necessary because cmd.exe does not support verbatim paths. <br>这是必要的，因为 cmd.exe 不支持逐字路径。<br>
pub(crate) fn to_user_path(mut path: Vec<u16>) -> io::Result<Vec<u16>> {
    use crate::ptr;
    use crate::sys::windows::fill_utf16_buf;

    // UTF-16 encoded code points, used in parsing and building UTF-16 paths. <br>UTF-16 编码的代码点，用于解析和构建 UTF-16 路径。<br>
    // All of these are in the ASCII range so they can be cast directly to `u16`. <br>所有这些都在 ASCII 范围内，因此它们可以直接转换为 `u16`。<br>
    const SEP: u16 = b'\\' as _;
    const QUERY: u16 = b'?' as _;
    const COLON: u16 = b':' as _;
    const U: u16 = b'U' as _;
    const N: u16 = b'N' as _;
    const C: u16 = b'C' as _;

    // Early return if the path is too long to remove the verbatim prefix. <br>如果路径太长而无法删除逐字前缀，则提前返回。<br>
    const LEGACY_MAX_PATH: usize = 260;
    if path.len() > LEGACY_MAX_PATH {
        return Ok(path);
    }

    match &path[..] {
        // `\\?\C:\...` => `C:\...`
        [SEP, SEP, QUERY, SEP, _, COLON, SEP, ..] => unsafe {
            let lpfilename = path[4..].as_ptr();
            fill_utf16_buf(
                |buffer, size| c::GetFullPathNameW(lpfilename, size, buffer, ptr::null_mut()),
                |full_path: &[u16]| {
                    if full_path == &path[4..path.len() - 1] { full_path.into() } else { path }
                },
            )
        },
        // `\\?\UNC\...` => `\\...`
        [SEP, SEP, QUERY, SEP, U, N, C, SEP, ..] => unsafe {
            // Change the `C` in `UNC\` to `\` so we can get a slice that starts with `\\`. <br>将 `UNC\` 中的 `C` 更改为 `\`，这样我们就可以得到一个以 `\\` 开头的切片。<br>
            path[6] = b'\\' as u16;
            let lpfilename = path[6..].as_ptr();
            fill_utf16_buf(
                |buffer, size| c::GetFullPathNameW(lpfilename, size, buffer, ptr::null_mut()),
                |full_path: &[u16]| {
                    if full_path == &path[6..path.len() - 1] {
                        full_path.into()
                    } else {
                        // Restore the 'C' in "UNC". <br>在 "UNC" 中恢复 'C'。<br>
                        path[6] = b'C' as u16;
                        path
                    }
                },
            )
        },
        // For everything else, leave the path unchanged. <br>对于其他一切，保持路径不变。<br>
        _ => Ok(path),
    }
}
