//! 系统互斥锁
//!
//! 互斥锁的 Windows 实现有点奇怪，可能不会立即明白发生了什么。
//! 最奇怪的是，使用 SRWLock 而不是 CriticalSection，这是因为：
//!
//! 1.
//! 根据在 Windows 8 和 Windows 7 上执行的基准测试，SRWLock 比 CriticalSection 快几倍。
//!
//!
//! 2. CriticalSection 允许在 SRWLock 死锁时进行递归锁定。
//! Unix 实现死锁，因此首选一致性。
//! 有关更多详细信息，请参见 #19962。
//!
//! 3.
//! 尽管 CriticalSection 是公平的，而 SRWLock 不公平，但当前的 Rust 策略是不保证公平。

use crate::cell::UnsafeCell;
use crate::sys::c;

pub struct Mutex {
    srwlock: UnsafeCell<c::SRWLOCK>,
}

// Windows SRW 锁是可移动的 (而不是借用的)。
pub type MovableMutex = Mutex;

unsafe impl Send for Mutex {}
unsafe impl Sync for Mutex {}

#[inline]
pub unsafe fn raw(m: &Mutex) -> c::PSRWLOCK {
    m.srwlock.get()
}

impl Mutex {
    #[inline]
    pub const fn new() -> Mutex {
        Mutex { srwlock: UnsafeCell::new(c::SRWLOCK_INIT) }
    }
    #[inline]
    pub unsafe fn init(&mut self) {}

    #[inline]
    pub unsafe fn lock(&self) {
        c::AcquireSRWLockExclusive(raw(self));
    }

    #[inline]
    pub unsafe fn try_lock(&self) -> bool {
        c::TryAcquireSRWLockExclusive(raw(self)) != 0
    }

    #[inline]
    pub unsafe fn unlock(&self) {
        c::ReleaseSRWLockExclusive(raw(self));
    }
}
