#![allow(missing_docs, nonstandard_style)]

use crate::ffi::{CStr, OsStr, OsString};
use crate::io::ErrorKind;
use crate::os::windows::ffi::{OsStrExt, OsStringExt};
use crate::path::PathBuf;
use crate::time::Duration;

pub use self::rand::hashmap_random_keys;

#[macro_use]
pub mod compat;

pub mod alloc;
pub mod args;
pub mod c;
pub mod cmath;
pub mod env;
pub mod fs;
pub mod handle;
pub mod io;
pub mod locks;
pub mod memchr;
pub mod net;
pub mod os;
pub mod os_str;
pub mod path;
pub mod pipe;
pub mod process;
pub mod rand;
pub mod thread;
pub mod thread_local_dtor;
pub mod thread_local_key;
pub mod thread_parker;
pub mod time;
cfg_if::cfg_if! {
    if #[cfg(not(target_vendor = "uwp"))] {
        pub mod stdio;
        pub mod stack_overflow;
    } else {
        pub mod stdio_uwp;
        pub mod stack_overflow_uwp;
        pub use self::stdio_uwp as stdio;
        pub use self::stack_overflow_uwp as stack_overflow;
    }
}

// SAFETY: must be called only once during runtime initialization. <br>在运行时初始化期间只能调用一次。<br>
// NOTE: this is not guaranteed to run, for example when Rust code is called externally. <br>这不能保证运行，例如在外部调用 Rust 代码时。<br>
pub unsafe fn init(_argc: isize, _argv: *const *const u8) {
    stack_overflow::init();

    // Normally, `thread::spawn` will call `Thread::set_name` but since this thread already exists, we have to call it ourselves. <br>通常，`thread::spawn` 会调用 `Thread::set_name`，但由于该线程已经存在，我们必须自己调用它。<br>
    //
    thread::Thread::set_name(&CStr::from_bytes_with_nul_unchecked(b"main\0"));
}

// SAFETY: must be called only once during runtime cleanup. <br>在运行时清理期间只能调用一次。<br>
// NOTE: this is not guaranteed to run, for example when the program aborts. <br>这不能保证运行，例如当程序中止时。<br>
pub unsafe fn cleanup() {
    net::cleanup();
}

pub fn decode_error_kind(errno: i32) -> ErrorKind {
    use ErrorKind::*;

    match errno as c::DWORD {
        c::ERROR_ACCESS_DENIED => return PermissionDenied,
        c::ERROR_ALREADY_EXISTS => return AlreadyExists,
        c::ERROR_FILE_EXISTS => return AlreadyExists,
        c::ERROR_BROKEN_PIPE => return BrokenPipe,
        c::ERROR_FILE_NOT_FOUND => return NotFound,
        c::ERROR_PATH_NOT_FOUND => return NotFound,
        c::ERROR_NO_DATA => return BrokenPipe,
        c::ERROR_INVALID_NAME => return InvalidFilename,
        c::ERROR_INVALID_PARAMETER => return InvalidInput,
        c::ERROR_NOT_ENOUGH_MEMORY | c::ERROR_OUTOFMEMORY => return OutOfMemory,
        c::ERROR_SEM_TIMEOUT
        | c::WAIT_TIMEOUT
        | c::ERROR_DRIVER_CANCEL_TIMEOUT
        | c::ERROR_OPERATION_ABORTED
        | c::ERROR_SERVICE_REQUEST_TIMEOUT
        | c::ERROR_COUNTER_TIMEOUT
        | c::ERROR_TIMEOUT
        | c::ERROR_RESOURCE_CALL_TIMED_OUT
        | c::ERROR_CTX_MODEM_RESPONSE_TIMEOUT
        | c::ERROR_CTX_CLIENT_QUERY_TIMEOUT
        | c::FRS_ERR_SYSVOL_POPULATE_TIMEOUT
        | c::ERROR_DS_TIMELIMIT_EXCEEDED
        | c::DNS_ERROR_RECORD_TIMED_OUT
        | c::ERROR_IPSEC_IKE_TIMED_OUT
        | c::ERROR_RUNLEVEL_SWITCH_TIMEOUT
        | c::ERROR_RUNLEVEL_SWITCH_AGENT_TIMEOUT => return TimedOut,
        c::ERROR_CALL_NOT_IMPLEMENTED => return Unsupported,
        c::ERROR_HOST_UNREACHABLE => return HostUnreachable,
        c::ERROR_NETWORK_UNREACHABLE => return NetworkUnreachable,
        c::ERROR_DIRECTORY => return NotADirectory,
        c::ERROR_DIRECTORY_NOT_SUPPORTED => return IsADirectory,
        c::ERROR_DIR_NOT_EMPTY => return DirectoryNotEmpty,
        c::ERROR_WRITE_PROTECT => return ReadOnlyFilesystem,
        c::ERROR_DISK_FULL | c::ERROR_HANDLE_DISK_FULL => return StorageFull,
        c::ERROR_SEEK_ON_DEVICE => return NotSeekable,
        c::ERROR_DISK_QUOTA_EXCEEDED => return FilesystemQuotaExceeded,
        c::ERROR_FILE_TOO_LARGE => return FileTooLarge,
        c::ERROR_BUSY => return ResourceBusy,
        c::ERROR_POSSIBLE_DEADLOCK => return Deadlock,
        c::ERROR_NOT_SAME_DEVICE => return CrossesDevices,
        c::ERROR_TOO_MANY_LINKS => return TooManyLinks,
        c::ERROR_FILENAME_EXCED_RANGE => return InvalidFilename,
        _ => {}
    }

    match errno {
        c::WSAEACCES => PermissionDenied,
        c::WSAEADDRINUSE => AddrInUse,
        c::WSAEADDRNOTAVAIL => AddrNotAvailable,
        c::WSAECONNABORTED => ConnectionAborted,
        c::WSAECONNREFUSED => ConnectionRefused,
        c::WSAECONNRESET => ConnectionReset,
        c::WSAEINVAL => InvalidInput,
        c::WSAENOTCONN => NotConnected,
        c::WSAEWOULDBLOCK => WouldBlock,
        c::WSAETIMEDOUT => TimedOut,
        c::WSAEHOSTUNREACH => HostUnreachable,
        c::WSAENETDOWN => NetworkDown,
        c::WSAENETUNREACH => NetworkUnreachable,

        _ => Uncategorized,
    }
}

pub fn unrolled_find_u16s(needle: u16, haystack: &[u16]) -> Option<usize> {
    let ptr = haystack.as_ptr();
    let mut start = &haystack[..];

    // For performance reasons unfold the loop eight times. <br>出于性能原因，将循环展开八次。<br>
    while start.len() >= 8 {
        macro_rules! if_return {
            ($($n:literal,)+) => {
                $(
                    if start[$n] == needle {
                        return Some(((&start[$n] as *const u16).addr() - ptr.addr()) / 2);
                    }
                )+
            }
        }

        if_return!(0, 1, 2, 3, 4, 5, 6, 7,);

        start = &start[8..];
    }

    for c in start {
        if *c == needle {
            return Some(((c as *const u16).addr() - ptr.addr()) / 2);
        }
    }
    None
}

pub fn to_u16s<S: AsRef<OsStr>>(s: S) -> crate::io::Result<Vec<u16>> {
    fn inner(s: &OsStr) -> crate::io::Result<Vec<u16>> {
        // Most paths are ASCII, so reserve capacity for as much as there are bytes in the OsStr plus one for the null-terminating character. <br>大多数路径都是 ASCII 格式，因此保留容量为 OsStr 中的字节数加上一个空终止字符。<br>
        // We are not wasting bytes here as paths created by this function are primarily used in an ephemeral fashion. <br>我们并没有在这里浪费字节，因为这个函数创建的路径主要以短暂的方式使用。<br>
        //
        //
        let mut maybe_result = Vec::with_capacity(s.len() + 1);
        maybe_result.extend(s.encode_wide());

        if unrolled_find_u16s(0, &maybe_result).is_some() {
            return Err(crate::io::const_io_error!(
                ErrorKind::InvalidInput,
                "strings passed to WinAPI cannot contain NULs",
            ));
        }
        maybe_result.push(0);
        Ok(maybe_result)
    }
    inner(s.as_ref())
}

// Many Windows APIs follow a pattern of where we hand a buffer and then they will report back to us how large the buffer should be or how many bytes currently reside in the buffer. <br>许多 Windows API 遵循一种模式，将缓冲区交给我们，然后它们会向我们报告缓冲区应该有多大或缓冲区中当前有多少字节。<br>
// This function is an abstraction over these functions by making them easier to call. <br>通过使这些函数更易于调用，该函数是这些函数的抽象。<br>
//
// The first callback, `f1`, is yielded a (pointer, len) pair which can be passed to a syscall. <br>第一个回调 `f1` 产生了 (指针，len) 对，该对可以传递给系统调用。<br> The `ptr` is valid for `len` items (u16 in this case). <br>`ptr` 对 `len` 项 (在这种情况下为 u16) 有效。<br>
// The closure is expected to return what the syscall returns which will be interpreted by this function to determine if the syscall needs to be invoked again (with more buffer space). <br>预计闭包将返回系统调用返回的内容，此函数将对此进行解释，以确定是否需要再次调用系统调用 (具有更多的缓冲区空间)。<br>
//
//
// Once the syscall has completed (errors bail out early) the second closure is yielded the data which has been read from the syscall. <br>一旦系统调用完成 (错误可以尽早解决)，第二个闭包就会产生从系统调用中读取的数据。<br>
// The return value from this closure is then the return value of the function. <br>该闭包的返回值就是函数的返回值。<br>
//
//
//
//
//
fn fill_utf16_buf<F1, F2, T>(mut f1: F1, f2: F2) -> crate::io::Result<T>
where
    F1: FnMut(*mut u16, c::DWORD) -> c::DWORD,
    F2: FnOnce(&[u16]) -> T,
{
    // Start off with a stack buf but then spill over to the heap if we end up needing more space. <br>从栈缓冲区开始，但如果最终需要更多空间，则溢出到堆中。<br>
    //
    // This initial size also works around `GetFullPathNameW` returning incorrect size hints for some short paths: <br>这个初始大小也适用于 `GetFullPathNameW` 为一些短路径返回不正确的大小提示:<br>
    //
    // https://github.com/dylni/normpath/issues/5
    //
    let mut stack_buf = [0u16; 512];
    let mut heap_buf = Vec::new();
    unsafe {
        let mut n = stack_buf.len();
        loop {
            let buf = if n <= stack_buf.len() {
                &mut stack_buf[..]
            } else {
                let extra = n - heap_buf.len();
                heap_buf.reserve(extra);
                heap_buf.set_len(n);
                &mut heap_buf[..]
            };

            // This function is typically called on windows API functions which will return the correct length of the string, but these functions also return the `0` on error. <br>通常在 windows API 函数上调用此函数，该函数将返回正确的字符串长度，但是这些函数也会在出错时返回 `0`。<br>
            //
            // In some cases, however, the returned "correct length" may actually be 0! <br>但是，在某些情况下，返回的 "正确长度" 实际上可能为 0!<br>
            //
            // To handle this case we call `SetLastError` to reset it to 0 and then check it again if we get the "0 error value". <br>为了处理这种情况，我们调用 `SetLastError` 将其重置为 0，然后再次检查是否获得 "0 错误值"。<br>
            // If the "last error" is still 0 then we interpret it as a 0 length buffer and not an actual error. <br>如果 "最后一个错误" 仍为 0，则我们将其解释为长度为 0 的缓冲区，而不是实际错误。<br>
            //
            //
            //
            c::SetLastError(0);
            let k = match f1(buf.as_mut_ptr(), n as c::DWORD) {
                0 if c::GetLastError() == 0 => 0,
                0 => return Err(crate::io::Error::last_os_error()),
                n => n,
            } as usize;
            if k == n && c::GetLastError() == c::ERROR_INSUFFICIENT_BUFFER {
                n *= 2;
            } else if k > n {
                n = k;
            } else if k == n {
                // It is impossible to reach this point. <br>达到这一点是不可能的。<br>
                // On success, k is the returned string length excluding the null. <br>成功时，k 是返回的字符串长度，不包括 null。<br>
                // On failure, k is the required buffer length including the null. <br>失败时，k 是所需的缓冲区长度，包括空值。<br>
                // Therefore k never equals n. <br>因此 k 永远不会等于 n。<br>
                unreachable!();
            } else {
                return Ok(f2(&buf[..k]));
            }
        }
    }
}

fn os2path(s: &[u16]) -> PathBuf {
    PathBuf::from(OsString::from_wide(s))
}

pub fn truncate_utf16_at_nul(v: &[u16]) -> &[u16] {
    match unrolled_find_u16s(0, v) {
        // don't include the 0 <br>不要包含 0<br>
        Some(i) => &v[..i],
        None => v,
    }
}

pub trait IsZero {
    fn is_zero(&self) -> bool;
}

macro_rules! impl_is_zero {
    ($($t:ident)*) => ($(impl IsZero for $t {
        fn is_zero(&self) -> bool {
            *self == 0
        }
    })*)
}

impl_is_zero! { i8 i16 i32 i64 isize u8 u16 u32 u64 usize }

pub fn cvt<I: IsZero>(i: I) -> crate::io::Result<I> {
    if i.is_zero() { Err(crate::io::Error::last_os_error()) } else { Ok(i) }
}

pub fn dur2timeout(dur: Duration) -> c::DWORD {
    // Note that a duration is a (u64, u32) (seconds, nanoseconds) pair, and the timeouts in windows APIs are typically u32 milliseconds. <br>请注意，持续时间是 (u64，u32) (秒，纳秒) 对，并且 windows API 中的超时通常为 u32 毫秒。<br>
    // To translate, we have two pieces to take care of: <br>要翻译，我们需要注意以下两点：<br>
    //
    // * Nanosecond precision is rounded up <br>纳秒精度舍入<br>
    // * Greater than u32::MAX milliseconds (50 days) is rounded up to INFINITE (never time out). <br>大于 u32::MAX 毫秒 (50 天) 的值会四舍五入为 INFINITE (永远不会超时)。<br>
    //
    //
    dur.as_secs()
        .checked_mul(1000)
        .and_then(|ms| ms.checked_add((dur.subsec_nanos() as u64) / 1_000_000))
        .and_then(|ms| ms.checked_add(if dur.subsec_nanos() % 1_000_000 > 0 { 1 } else { 0 }))
        .map(|ms| if ms > <c::DWORD>::MAX as u64 { c::INFINITE } else { ms as c::DWORD })
        .unwrap_or(c::INFINITE)
}

/// Use `__fastfail` to abort the process <br>使用 `__fastfail` 中止该进程<br>
///
/// This is the same implementation as in libpanic_abort's `__rust_start_panic`. <br>这与 libpanic_abort 的 `__rust_start_panic` 中的实现相同。<br>
/// See that function for more information on `__fastfail` <br>有关 `__fastfail` 的更多信息，请参见该函数。<br>
#[allow(unreachable_code)]
pub fn abort_internal() -> ! {
    #[allow(unused)]
    const FAST_FAIL_FATAL_APP_EXIT: usize = 7;
    #[cfg(not(miri))] // inline assembly does not work in Miri <br>内联汇编在 Miri 中不起作用<br>
    unsafe {
        cfg_if::cfg_if! {
            if #[cfg(any(target_arch = "x86", target_arch = "x86_64"))] {
                core::arch::asm!("int $$0x29", in("ecx") FAST_FAIL_FATAL_APP_EXIT);
                crate::intrinsics::unreachable();
            } else if #[cfg(all(target_arch = "arm", target_feature = "thumb-mode"))] {
                core::arch::asm!(".inst 0xDEFB", in("r0") FAST_FAIL_FATAL_APP_EXIT);
                crate::intrinsics::unreachable();
            } else if #[cfg(target_arch = "aarch64")] {
                core::arch::asm!("brk 0xF003", in("x0") FAST_FAIL_FATAL_APP_EXIT);
                crate::intrinsics::unreachable();
            }
        }
    }
    crate::intrinsics::abort();
}
