use super::{c, fill_utf16_buf, to_u16s};
use crate::ffi::{OsStr, OsString};
use crate::io;
use crate::mem;
use crate::path::{Path, PathBuf, Prefix};
use crate::ptr;

#[cfg(test)]
mod tests;

pub const MAIN_SEP_STR: &str = "\\";
pub const MAIN_SEP: char = '\\';

/// # Safety
///
/// `bytes` must be a valid wtf8 encoded slice <br>`bytes` 必须是有效的 wtf8 编码切片<br>
#[inline]
unsafe fn bytes_as_os_str(bytes: &[u8]) -> &OsStr {
    // &OsStr is layout compatible with &Slice, which is compatible with &Wtf8, which is compatible with &[u8]. <br>&OsStr 布局兼容 &Slice，它与 &Wtf8 兼容，与 &[u8] 兼容。<br>
    //
    mem::transmute(bytes)
}

#[inline]
pub fn is_sep_byte(b: u8) -> bool {
    b == b'/' || b == b'\\'
}

#[inline]
pub fn is_verbatim_sep(b: u8) -> bool {
    b == b'\\'
}

/// Returns true if `path` looks like a lone filename. <br>如果 `path` 看起来像一个单独的文件名，则返回 true。<br>
pub(crate) fn is_file_name(path: &OsStr) -> bool {
    !path.bytes().iter().copied().any(is_sep_byte)
}
pub(crate) fn has_trailing_slash(path: &OsStr) -> bool {
    let is_verbatim = path.bytes().starts_with(br"\\?\");
    let is_separator = if is_verbatim { is_verbatim_sep } else { is_sep_byte };
    if let Some(&c) = path.bytes().last() { is_separator(c) } else { false }
}

/// Appends a suffix to a path. <br>将后缀追加到路径。<br>
///
/// Can be used to append an extension without removing an existing extension. <br>可用于在不删除现有扩展名的情况下追加扩展名。<br>
pub(crate) fn append_suffix(path: PathBuf, suffix: &OsStr) -> PathBuf {
    let mut path = OsString::from(path);
    path.push(suffix);
    path.into()
}

struct PrefixParser<'a, const LEN: usize> {
    path: &'a OsStr,
    prefix: [u8; LEN],
}

impl<'a, const LEN: usize> PrefixParser<'a, LEN> {
    #[inline]
    fn get_prefix(path: &OsStr) -> [u8; LEN] {
        let mut prefix = [0; LEN];
        // SAFETY: Only ASCII characters are modified. <br>仅修改 ASCII 字符。<br>
        for (i, &ch) in path.bytes().iter().take(LEN).enumerate() {
            prefix[i] = if ch == b'/' { b'\\' } else { ch };
        }
        prefix
    }

    fn new(path: &'a OsStr) -> Self {
        Self { path, prefix: Self::get_prefix(path) }
    }

    fn as_slice(&self) -> PrefixParserSlice<'a, '_> {
        PrefixParserSlice {
            path: self.path,
            prefix: &self.prefix[..LEN.min(self.path.len())],
            index: 0,
        }
    }
}

struct PrefixParserSlice<'a, 'b> {
    path: &'a OsStr,
    prefix: &'b [u8],
    index: usize,
}

impl<'a> PrefixParserSlice<'a, '_> {
    fn strip_prefix(&self, prefix: &str) -> Option<Self> {
        self.prefix[self.index..]
            .starts_with(prefix.as_bytes())
            .then(|| Self { index: self.index + prefix.len(), ..*self })
    }

    fn prefix_bytes(&self) -> &'a [u8] {
        &self.path.bytes()[..self.index]
    }

    fn finish(self) -> &'a OsStr {
        // SAFETY: The unsafety here stems from converting between &OsStr and &[u8] and back. <br>这里的不安全性源于 &OsStr 和 &[u8] and 之间的转换。<br>
        // This is safe to do because (1) we only look at ASCII contents of the encoding and (2) new &OsStr values are produced only from ASCII-bounded slices of existing &OsStr values. <br>这样做是安全的，因为 (1) 我们仅查看编码的 ASCII 内容，而 (2) 新的 &OsStr 值仅从现有 &OsStr values 的 ASCII 限制切片中产生。<br>
        //
        //
        unsafe { bytes_as_os_str(&self.path.bytes()[self.index..]) }
    }
}

pub fn parse_prefix(path: &OsStr) -> Option<Prefix<'_>> {
    use Prefix::{DeviceNS, Disk, Verbatim, VerbatimDisk, VerbatimUNC, UNC};

    let parser = PrefixParser::<8>::new(path);
    let parser = parser.as_slice();
    if let Some(parser) = parser.strip_prefix(r"\\") {
        // \\

        // The meaning of verbatim paths can change when they use a different separator. <br>当逐字路径使用不同的分隔符时，它们的含义可能会发生变化。<br>
        //
        if let Some(parser) = parser.strip_prefix(r"?\") && !parser.prefix_bytes().iter().any(|&x| x == b'/') {
            // \\?\
            if let Some(parser) = parser.strip_prefix(r"UNC\") {
                // \\?\UNC\server\share

                let path = parser.finish();
                let (server, path) = parse_next_component(path, true);
                let (share, _) = parse_next_component(path, true);

                Some(VerbatimUNC(server, share))
            } else {
                let path = parser.finish();

                // in verbatim paths only recognize an exact drive prefix <br>逐字路径中的字符只能识别确切的驱动器前缀<br>
                if let Some(drive) = parse_drive_exact(path) {
                    // \\?\C:
                    Some(VerbatimDisk(drive))
                } else {
                    // \\?\prefix
                    let (prefix, _) = parse_next_component(path, true);
                    Some(Verbatim(prefix))
                }
            }
        } else if let Some(parser) = parser.strip_prefix(r".\") {
            // \\.\COM42
            let path = parser.finish();
            let (prefix, _) = parse_next_component(path, false);
            Some(DeviceNS(prefix))
        } else {
            let path = parser.finish();
            let (server, path) = parse_next_component(path, false);
            let (share, _) = parse_next_component(path, false);

            if !server.is_empty() && !share.is_empty() {
                // \\server\share
                Some(UNC(server, share))
            } else {
                // no valid prefix beginning with "\\" recognized <br>无法识别以 "\\" 开头的有效前缀<br>
                None
            }
        }
    } else if let Some(drive) = parse_drive(path) {
        // C:
        Some(Disk(drive))
    } else {
        // no prefix <br>无前缀<br>
        None
    }
}

// Parses a drive prefix, e.g. "C:" and "C:\whatever" <br>解析驱动器前缀，例如 "C:" 和 "C:\whatever"<br>
fn parse_drive(path: &OsStr) -> Option<u8> {
    // In most DOS systems, it is not possible to have more than 26 drive letters. <br>在大多数 DOS 系统中，驱动器号不能超过 26 个。<br>
    // See <https://en.wikipedia.org/wiki/Drive_letter_assignment#Common_assignments>. <br>请参见 <https://en.wikipedia.org/wiki/Drive_letter_assignment#Common_assignments>。<br>
    fn is_valid_drive_letter(drive: &u8) -> bool {
        drive.is_ascii_alphabetic()
    }

    match path.bytes() {
        [drive, b':', ..] if is_valid_drive_letter(drive) => Some(drive.to_ascii_uppercase()),
        _ => None,
    }
}

// Parses a drive prefix exactly, e.g. "C:" <br>准确解析驱动器前缀，例如 "C:"<br>
fn parse_drive_exact(path: &OsStr) -> Option<u8> {
    // only parse two bytes: the drive letter and the drive separator <br>仅解析两个字节：驱动器号和驱动器分隔符<br>
    if path.bytes().get(2).map(|&x| is_sep_byte(x)).unwrap_or(true) {
        parse_drive(path)
    } else {
        None
    }
}

// Parse the next path component. <br>解析下一个路径组件。<br>
//
// Returns the next component and the rest of the path excluding the component and separator. <br>返回路径的下一个组成部分和剩余部分，不包括组成部分和分隔符。<br>
// Does not recognize `/` as a separator character if `verbatim` is true. <br>如果 `verbatim` 为 true，则不能将 `/` 识别为分隔符。<br>
fn parse_next_component(path: &OsStr, verbatim: bool) -> (&OsStr, &OsStr) {
    let separator = if verbatim { is_verbatim_sep } else { is_sep_byte };

    match path.bytes().iter().position(|&x| separator(x)) {
        Some(separator_start) => {
            let mut separator_end = separator_start + 1;

            // a series of multiple separator characters is treated as a single separator, except in verbatim paths <br>一系列的多个分隔符被视为单个分隔符 (逐字路径中除外)<br>
            //
            while !verbatim && separator_end < path.len() && separator(path.bytes()[separator_end])
            {
                separator_end += 1;
            }

            let component = &path.bytes()[..separator_start];

            // Panic safe The max `separator_end` is `bytes.len()` and `bytes[bytes.len()..]` is a valid index. <br>Panic 安全的最大值 `separator_end` 为 `bytes.len()`，`bytes[bytes.len()..]` 是一个有效的索引。<br>
            //
            let path = &path.bytes()[separator_end..];

            // SAFETY: `path` is a valid wtf8 encoded slice and each of the separators ('/', '\') is encoded in a single byte, therefore `bytes[separator_start]` and `bytes[separator_end]` must be code point boundaries and thus `bytes[..separator_start]` and `bytes[separator_end..]` are valid wtf8 slices. <br>`path` 是有效的 wtf8 编码切片，并且每个分隔符 (`/`，'\') 都编码在一个字节中，因此 `bytes[separator_start]` 和 `bytes[separator_end]` 必须是代码点边界，因此 `bytes[..separator_start]` 和 `bytes[separator_end..]` 是有效的 wtf8 切片。<br>
            //
            //
            //
            unsafe { (bytes_as_os_str(component), bytes_as_os_str(path)) }
        }
        None => (path, OsStr::new("")),
    }
}

/// Returns a UTF-16 encoded path capable of bypassing the legacy `MAX_PATH` limits. <br>返回能够绕过传统 `MAX_PATH` 限制的 UTF-16 编码路径。<br>
///
/// This path may or may not have a verbatim prefix. <br>此路径可能有也可能没有 verbatim 前缀。<br>
pub(crate) fn maybe_verbatim(path: &Path) -> io::Result<Vec<u16>> {
    // Normally the MAX_PATH is 260 UTF-16 code units (including the NULL). <br>通常 MAX_PATH 是 260 个 UTF-16 代码单元 (包括 NULL)。<br>
    // However, for APIs such as CreateDirectory[1], the limit is 248. <br>但是，对于 CreateDirectory[1] 等 API，限制为 248 个。<br>
    //
    // [1]: https://docs.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-createdirectorya#parameters
    const LEGACY_MAX_PATH: usize = 248;
    // UTF-16 encoded code points, used in parsing and building UTF-16 paths. <br>UTF-16 编码的代码点，用于解析和构建 UTF-16 路径。<br>
    // All of these are in the ASCII range so they can be cast directly to `u16`. <br>所有这些都在 ASCII 范围内，因此它们可以直接转换为 `u16`。<br>
    const SEP: u16 = b'\\' as _;
    const ALT_SEP: u16 = b'/' as _;
    const QUERY: u16 = b'?' as _;
    const COLON: u16 = b':' as _;
    const DOT: u16 = b'.' as _;
    const U: u16 = b'U' as _;
    const N: u16 = b'N' as _;
    const C: u16 = b'C' as _;

    // \\?\
    const VERBATIM_PREFIX: &[u16] = &[SEP, SEP, QUERY, SEP];
    // \??\
    const NT_PREFIX: &[u16] = &[SEP, QUERY, QUERY, SEP];
    // \\?\UNC\
    const UNC_PREFIX: &[u16] = &[SEP, SEP, QUERY, SEP, U, N, C, SEP];

    let mut path = to_u16s(path)?;
    if path.starts_with(VERBATIM_PREFIX) || path.starts_with(NT_PREFIX) || path == &[0] {
        // Early return for paths that are already verbatim or empty. <br>提前返回已经逐字或空的路径。<br>
        return Ok(path);
    } else if path.len() < LEGACY_MAX_PATH {
        // Early return if an absolute path is less < 260 UTF-16 code units. <br>如果绝对路径小于 260 个 UTF-16 代码单元，则提前返回。<br>
        // This is an optimization to avoid calling `GetFullPathNameW` unnecessarily. <br>这是为了避免不必要地调用 `GetFullPathNameW` 的优化。<br>
        match path.as_slice() {
            // Starts with `D:`, `D:\`, `D:/`, etc. <br>以 `D:`、`D:\`、`D:/` 等开头。<br>
            // Does not match if the path starts with a `\` or `/`. <br>如果路径以 `\` 或 `/` 开头，则不匹配。<br>
            [drive, COLON, 0] | [drive, COLON, SEP | ALT_SEP, ..]
                if *drive != SEP && *drive != ALT_SEP =>
            {
                return Ok(path);
            }
            // Starts with `\\`, `//`, etc <br>以 `\\`、`//` 等开头<br>
            [SEP | ALT_SEP, SEP | ALT_SEP, ..] => return Ok(path),
            _ => {}
        }
    }

    // Firstly, get the absolute path using `GetFullPathNameW`. <br>首先，使用 `GetFullPathNameW` 获取绝对路径。<br>
    // https://docs.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-getfullpathnamew
    let lpfilename = path.as_ptr();
    fill_utf16_buf(
        // SAFETY: `fill_utf16_buf` ensures the `buffer` and `size` are valid. <br>`fill_utf16_buf` 确保 `buffer` 和 `size` 有效。<br>
        // `lpfilename` is a pointer to a null terminated string that is not invalidated until after `GetFullPathNameW` returns successfully. <br>`lpfilename` 是一个指向空终止字符串的指针，该字符串直到 `GetFullPathNameW` 成功返回后才会失效。<br>
        //
        |buffer, size| unsafe { c::GetFullPathNameW(lpfilename, size, buffer, ptr::null_mut()) },
        |mut absolute| {
            path.clear();

            // Secondly, add the verbatim prefix. <br>其次，添加 verbatim 前缀。<br> This is easier here because we know the path is now absolute and fully normalized (e.g. <br>这在这里更容易，因为我们知道路径现在是绝对的并且完全标准化 (例如<br>
            // `/` has been changed to `\`). <br>`/` 已更改为 `\`)。<br>
            let prefix = match absolute {
                // C:\ => \\?\C:\
                [_, COLON, SEP, ..] => VERBATIM_PREFIX,
                // \\.\ => \\?\
                [SEP, SEP, DOT, SEP, ..] => {
                    absolute = &absolute[4..];
                    VERBATIM_PREFIX
                }
                // Leave \\?\ and \??\ as-is. <br>保留 \\?\ 和 \??\ 原样。<br>
                [SEP, SEP, QUERY, SEP, ..] | [SEP, QUERY, QUERY, SEP, ..] => &[],
                // \\ => \\?\UNC\
                [SEP, SEP, ..] => {
                    absolute = &absolute[2..];
                    UNC_PREFIX
                }
                // Anything else we leave alone. <br>任何其他的事我们都不去管了。<br>
                _ => &[],
            };

            path.reserve_exact(prefix.len() + absolute.len() + 1);
            path.extend_from_slice(prefix);
            path.extend_from_slice(absolute);
            path.push(0);
        },
    )?;
    Ok(path)
}

/// Make a Windows path absolute. <br>将 Windows 路径设为绝对路径。<br>
pub(crate) fn absolute(path: &Path) -> io::Result<PathBuf> {
    let path = path.as_os_str();
    let prefix = parse_prefix(path);
    // Verbatim paths should not be modified. <br>不应修改逐字记录路径。<br>
    if prefix.map(|x| x.is_verbatim()).unwrap_or(false) {
        // NULs in verbatim paths are rejected for consistency. <br>为了一致性，逐字路径中的 NUL 将被拒绝。<br>
        if path.bytes().contains(&0) {
            return Err(io::const_io_error!(
                io::ErrorKind::InvalidInput,
                "strings passed to WinAPI cannot contain NULs",
            ));
        }
        return Ok(path.to_owned().into());
    }

    let path = to_u16s(path)?;
    let lpfilename = path.as_ptr();
    fill_utf16_buf(
        // SAFETY: `fill_utf16_buf` ensures the `buffer` and `size` are valid. <br>`fill_utf16_buf` 确保 `buffer` 和 `size` 有效。<br>
        // `lpfilename` is a pointer to a null terminated string that is not invalidated until after `GetFullPathNameW` returns successfully. <br>`lpfilename` 是一个指向空终止字符串的指针，该字符串直到 `GetFullPathNameW` 成功返回后才会失效。<br>
        //
        |buffer, size| unsafe { c::GetFullPathNameW(lpfilename, size, buffer, ptr::null_mut()) },
        super::os2path,
    )
}
