use crate::io;
use crate::mem;
use crate::sync;
use crate::sys::c;

/// 可能可用的 HashMap RNG 的种类
#[derive(Clone, Copy, Debug, PartialEq)]
enum HashMapRng {
    Preferred,
    Fallback,
}

pub fn hashmap_random_keys() -> (u64, u64) {
    match get_hashmap_rng() {
        HashMapRng::Preferred => {
            preferred_rng().expect("couldn't generate random bytes with preferred RNG")
        }
        HashMapRng::Fallback => {
            fallback_rng().expect("couldn't generate random bytes with fallback RNG")
        }
    }
}

/// 返回应该使用的 HashMap RNG
///
/// 如果他们两个都坏了，就会出现 panic
fn get_hashmap_rng() -> HashMapRng {
    // 假设如果我们第一次使用首选的 RNG 就坏了，那么很可能意味着: DLL 加载失败，没有必要一遍又一遍地调用它，我们应该缓存结果
    //
    //
    static VALUE: sync::OnceLock<HashMapRng> = sync::OnceLock::new();
    *VALUE.get_or_init(choose_hashmap_rng)
}

/// 测试我们是否应该使用首选或后备 RNG
///
/// 如果首选 RNG 成功，我们选择它。
/// 否则，如果回退 RNG 成功，我们选择
///
/// 如果首选 RNG 和后备 RNG 都不起作用，则会发生 panic
fn choose_hashmap_rng() -> HashMapRng {
    let preferred_error = match preferred_rng() {
        Ok(_) => return HashMapRng::Preferred,
        Err(e) => e,
    };

    match fallback_rng() {
        Ok(_) => return HashMapRng::Fallback,
        Err(fallback_error) => panic!(
            "preferred RNG broken: `{}`, fallback RNG broken: `{}`",
            preferred_error, fallback_error
        ),
    }
}

/// 使用首选的 RNG 函数 (BCryptGenRandom) 生成随机数
fn preferred_rng() -> Result<(u64, u64), io::Error> {
    use crate::ptr;

    let mut v = (0, 0);
    let ret = unsafe {
        c::BCryptGenRandom(
            ptr::null_mut(),
            &mut v as *mut _ as *mut u8,
            mem::size_of_val(&v) as c::ULONG,
            c::BCRYPT_USE_SYSTEM_PREFERRED_RNG,
        )
    };

    if ret == 0 { Ok(v) } else { Err(io::Error::last_os_error()) }
}

/// 使用回退 RNG 函数 (RtlGenRandom) 生成随机数
#[cfg(not(target_vendor = "uwp"))]
fn fallback_rng() -> Result<(u64, u64), io::Error> {
    let mut v = (0, 0);
    let ret =
        unsafe { c::RtlGenRandom(&mut v as *mut _ as *mut u8, mem::size_of_val(&v) as c::ULONG) };

    if ret != 0 { Ok(v) } else { Err(io::Error::last_os_error()) }
}

/// 我们不能在 UWP 中使用 RtlGenRandom，所以没有回退
#[cfg(target_vendor = "uwp")]
fn fallback_rng() -> Result<(u64, u64), io::Error> {
    Err(io::const_io_error!(io::ErrorKind::Unsupported, "RtlGenRandom() not supported on UWP"))
}
