use crate::ffi::CStr;
use crate::io;
use crate::num::NonZeroUsize;
use crate::os::windows::io::AsRawHandle;
use crate::ptr;
use crate::sys::c;
use crate::sys::handle::Handle;
use crate::sys::stack_overflow;
use crate::sys_common::FromInner;
use crate::time::Duration;

use libc::c_void;

use super::to_u16s;

pub const DEFAULT_MIN_STACK_SIZE: usize = 2 * 1024 * 1024;

pub struct Thread {
    handle: Handle,
}

impl Thread {
    // unsafe: see thread::Builder::spawn_unchecked for safety requirements <br>不安全：有关安全要求，请参见 thread::Builder::spawn_unchecked<br>
    pub unsafe fn new(stack: usize, p: Box<dyn FnOnce()>) -> io::Result<Thread> {
        let p = Box::into_raw(box p);

        // FIXME On UNIX, we guard against stack sizes that are too small but that's because pthreads enforces that stacks are at least PTHREAD_STACK_MIN bytes big. <br>在 UNIX 上，我们会防止栈大小过小，但这是因为 pthread 强制栈的大小至少为 PTHREAD_STACK_MIN 个字节。<br>
        // Windows has no such lower limit, it's just that below a certain threshold you can't do anything useful. <br>Windows 没有这样的下限，只是低于一定的阈值就不能做任何有用的事情。<br>
        // That threshold is application and architecture-specific, however. <br>但是，该阈值是特定于应用程序和体系结构的。<br>
        //
        //
        let ret = c::CreateThread(
            ptr::null_mut(),
            stack,
            thread_start,
            p as *mut _,
            c::STACK_SIZE_PARAM_IS_A_RESERVATION,
            ptr::null_mut(),
        );

        return if let Ok(handle) = ret.try_into() {
            Ok(Thread { handle: Handle::from_inner(handle) })
        } else {
            // The thread failed to start and as a result p was not consumed. <br>线程无法启动，结果 p 没有被消费。<br>
            // Therefore, it is safe to reconstruct the box so that it gets deallocated. <br>因此，重建 box 使其被释放是安全的。<br>
            drop(Box::from_raw(p));
            Err(io::Error::last_os_error())
        };

        extern "system" fn thread_start(main: *mut c_void) -> c::DWORD {
            unsafe {
                // Next, set up our stack overflow handler which may get triggered if we run out of stack. <br>接下来，设置我们的栈溢出处理程序，如果我们用完栈，可能会触发该处理程序。<br>
                //
                let _handler = stack_overflow::Handler::new();
                // Finally, let's run some code. <br>最后，让我们运行一些代码。<br>
                Box::from_raw(main as *mut Box<dyn FnOnce()>)();
            }
            0
        }
    }

    pub fn set_name(name: &CStr) {
        if let Ok(utf8) = name.to_str() {
            if let Ok(utf16) = to_u16s(utf8) {
                unsafe {
                    c::SetThreadDescription(c::GetCurrentThread(), utf16.as_ptr());
                };
            };
        };
    }

    pub fn join(self) {
        let rc = unsafe { c::WaitForSingleObject(self.handle.as_raw_handle(), c::INFINITE) };
        if rc == c::WAIT_FAILED {
            panic!("failed to join on thread: {}", io::Error::last_os_error());
        }
    }

    pub fn yield_now() {
        // This function will return 0 if there are no other threads to execute, but this also means that the yield was useless so this isn't really a case that needs to be worried about. <br>如果没有其他线程要执行，则此函数将返回 0，但这也意味着 yield 没用，因此实际上不必担心这种情况。<br>
        //
        //
        unsafe {
            c::SwitchToThread();
        }
    }

    pub fn sleep(dur: Duration) {
        unsafe { c::Sleep(super::dur2timeout(dur)) }
    }

    pub fn handle(&self) -> &Handle {
        &self.handle
    }

    pub fn into_handle(self) -> Handle {
        self.handle
    }
}

pub fn available_parallelism() -> io::Result<NonZeroUsize> {
    let res = unsafe {
        let mut sysinfo: c::SYSTEM_INFO = crate::mem::zeroed();
        c::GetSystemInfo(&mut sysinfo);
        sysinfo.dwNumberOfProcessors as usize
    };
    match res {
        0 => Err(io::const_io_error!(
            io::ErrorKind::NotFound,
            "The number of hardware threads is not known for the target platform",
        )),
        cpus => Ok(unsafe { NonZeroUsize::new_unchecked(cpus) }),
    }
}

#[cfg_attr(test, allow(dead_code))]
pub mod guard {
    pub type Guard = !;
    pub unsafe fn current() -> Option<Guard> {
        None
    }
    pub unsafe fn init() -> Option<Guard> {
        None
    }
}
