use crate::sys::locks as imp;
use crate::sys_common::mutex::MovableMutex;
use crate::time::Duration;

mod check;

type CondvarCheck = <imp::MovableMutex as check::CondvarCheck>::Check;

/// 基于 OS 的条件变量。
pub struct Condvar {
    inner: imp::MovableCondvar,
    check: CondvarCheck,
}

impl Condvar {
    /// 创建一个新的条件变量以供使用。
    #[inline]
    pub const fn new() -> Self {
        Self { inner: imp::MovableCondvar::new(), check: CondvarCheck::new() }
    }

    /// 在此条件变量上发信号通知一位等待者醒来。
    #[inline]
    pub fn notify_one(&self) {
        unsafe { self.inner.notify_one() };
    }

    /// 在此条件变量上唤醒所有当前的等待者。
    #[inline]
    pub fn notify_all(&self) {
        unsafe { self.inner.notify_all() };
    }

    /// 在指定的互斥锁上等待信号。
    ///
    /// 如果互斥锁没有被当前线程锁定，则行为是不确定的。
    ///
    /// 如果与多个互斥锁一起使用，则可能为 panic。
    #[inline]
    pub unsafe fn wait(&self, mutex: &MovableMutex) {
        self.check.verify(mutex);
        self.inner.wait(mutex.raw())
    }

    /// 等待指定的互斥锁上的信号，其超时持续时间由 `dur` 指定 (进入 future 的相对时间)。
    ///
    ///
    /// 如果互斥锁没有被当前线程锁定，则行为是不确定的。
    ///
    /// 如果与多个互斥锁一起使用，则可能为 panic。
    #[inline]
    pub unsafe fn wait_timeout(&self, mutex: &MovableMutex, dur: Duration) -> bool {
        self.check.verify(mutex);
        self.inner.wait_timeout(mutex.raw(), dur)
    }
}
