use crate::sys::locks as imp;
use crate::sys_common::mutex::MovableMutex;
use crate::time::Duration;

mod check;

type CondvarCheck = <imp::MovableMutex as check::CondvarCheck>::Check;

/// An OS-based condition variable. <br>基于 OS 的条件变量。<br>
pub struct Condvar {
    inner: imp::MovableCondvar,
    check: CondvarCheck,
}

impl Condvar {
    /// Creates a new condition variable for use. <br>创建一个新的条件变量以供使用。<br>
    #[inline]
    pub const fn new() -> Self {
        Self { inner: imp::MovableCondvar::new(), check: CondvarCheck::new() }
    }

    /// Signals one waiter on this condition variable to wake up. <br>在此条件变量上发信号通知一位等待者醒来。<br>
    #[inline]
    pub fn notify_one(&self) {
        unsafe { self.inner.notify_one() };
    }

    /// Awakens all current waiters on this condition variable. <br>在此条件变量上唤醒所有当前的等待者。<br>
    #[inline]
    pub fn notify_all(&self) {
        unsafe { self.inner.notify_all() };
    }

    /// Waits for a signal on the specified mutex. <br>在指定的互斥锁上等待信号。<br>
    ///
    /// Behavior is undefined if the mutex is not locked by the current thread. <br>如果互斥锁没有被当前线程锁定，则行为是不确定的。<br>
    ///
    /// May panic if used with more than one mutex. <br>如果与多个互斥锁一起使用，则可能为 panic。<br>
    #[inline]
    pub unsafe fn wait(&self, mutex: &MovableMutex) {
        self.check.verify(mutex);
        self.inner.wait(mutex.raw())
    }

    /// Waits for a signal on the specified mutex with a timeout duration specified by `dur` (a relative time into the future). <br>等待指定的互斥锁上的信号，其超时持续时间由 `dur` 指定 (进入 future 的相对时间)。<br>
    ///
    ///
    /// Behavior is undefined if the mutex is not locked by the current thread. <br>如果互斥锁没有被当前线程锁定，则行为是不确定的。<br>
    ///
    /// May panic if used with more than one mutex. <br>如果与多个互斥锁一起使用，则可能为 panic。<br>
    #[inline]
    pub unsafe fn wait_timeout(&self, mutex: &MovableMutex, dur: Duration) -> bool {
        self.check.verify(mutex);
        self.inner.wait_timeout(mutex.raw(), dur)
    }
}
