#![allow(dead_code)] // Only used on some platforms. <br>仅在某些平台上使用。<br>

// This is used to wrap pthread {Mutex, Condvar, RwLock} in. <br>这用于包装 pthread {Mutex, Condvar, RwLock}。<br>

use crate::marker::PhantomData;
use crate::ops::{Deref, DerefMut};
use crate::ptr::null_mut;
use crate::sync::atomic::{
    AtomicPtr,
    Ordering::{AcqRel, Acquire},
};

pub(crate) struct LazyBox<T: LazyInit> {
    ptr: AtomicPtr<T>,
    _phantom: PhantomData<T>,
}

pub(crate) trait LazyInit {
    /// This is called before the box is allocated, to provide the value to move into the new box. <br>这是在分配 box 之前调用的，以提供移动到新 box 的值。<br>
    ///
    /// It might be called more than once per LazyBox, as multiple threads might race to initialize it concurrently, each constructing and initializing their own box. <br>每个 LazyBox 可能会多次调用它，因为多个线程可能会竞相同时初始化它，每个线程都构造和初始化自己的 box。<br>
    /// (All but one of them will be destroyed right after.) <br>(除了其中一个之外，它们都将是正确的。)<br>
    ///
    ///
    fn init() -> Box<Self>;
}

impl<T: LazyInit> LazyBox<T> {
    #[inline]
    pub const fn new() -> Self {
        Self { ptr: AtomicPtr::new(null_mut()), _phantom: PhantomData }
    }

    #[inline]
    fn get_pointer(&self) -> *mut T {
        let ptr = self.ptr.load(Acquire);
        if ptr.is_null() { self.initialize() } else { ptr }
    }

    #[cold]
    fn initialize(&self) -> *mut T {
        let new_ptr = Box::into_raw(T::init());
        match self.ptr.compare_exchange(null_mut(), new_ptr, AcqRel, Acquire) {
            Ok(_) => new_ptr,
            Err(ptr) => {
                // Lost the race to another thread. <br>输给了另一个线程。<br>
                // Drop the box we created, and use the one from the other thread instead. <br>丢弃我们创建的 box，并使用另一个线程中的那个。<br>
                drop(unsafe { Box::from_raw(new_ptr) });
                ptr
            }
        }
    }
}

impl<T: LazyInit> Deref for LazyBox<T> {
    type Target = T;
    #[inline]
    fn deref(&self) -> &T {
        unsafe { &*self.get_pointer() }
    }
}

impl<T: LazyInit> DerefMut for LazyBox<T> {
    #[inline]
    fn deref_mut(&mut self) -> &mut T {
        unsafe { &mut *self.get_pointer() }
    }
}

impl<T: LazyInit> Drop for LazyBox<T> {
    fn drop(&mut self) {
        let ptr = *self.ptr.get_mut();
        if !ptr.is_null() {
            drop(unsafe { Box::from_raw(ptr) });
        }
    }
}
