use crate::sys::locks as imp;

/// An OS-based mutual exclusion lock, meant for use in static variables. <br>基于操作系统的互斥锁，用于静态变量。<br>
///
/// This mutex has a const constructor ([`StaticMutex::new`]), does not implement `Drop` to cleanup resources, and causes UB when used reentrantly. <br>此互斥锁具有 const 构造函数 ([`StaticMutex::new`])，未实现 `Drop` 来清理资源，并且在重新使用时会导致 UB。<br>
///
///
/// This mutex does not implement poisoning. <br>此互斥锁不会实现中毒。<br>
///
/// This is a wrapper around `imp::Mutex` that does *not* call `init()` and `destroy()`. <br>这是 `imp::Mutex` 的包装，它不 * 调用 `init()` 和 `destroy()`。<br>
///
pub struct StaticMutex(imp::Mutex);

unsafe impl Sync for StaticMutex {}

impl StaticMutex {
    /// Creates a new mutex for use. <br>创建一个新的互斥锁以供使用。<br>
    #[inline]
    pub const fn new() -> Self {
        Self(imp::Mutex::new())
    }

    /// Calls raw_lock() and then returns an RAII guard to guarantee the mutex will be unlocked. <br>调用 raw_lock()，然后返回 RAII 防护以确保互斥锁将被解锁。<br>
    ///
    ///
    /// It is undefined behaviour to call this function while locked by the same thread. <br>在同一线程锁定时调用此函数是未定义的行为。<br>
    ///
    #[inline]
    pub unsafe fn lock(&'static self) -> StaticMutexGuard {
        self.0.lock();
        StaticMutexGuard(&self.0)
    }
}

#[must_use]
pub struct StaticMutexGuard(&'static imp::Mutex);

impl Drop for StaticMutexGuard {
    #[inline]
    fn drop(&mut self) {
        unsafe {
            self.0.unlock();
        }
    }
}

/// An OS-based mutual exclusion lock. <br>基于操作系统的互斥锁。<br>
///
/// This mutex cleans up its resources in its `Drop` implementation, may safely be moved (when not borrowed), and does not cause UB when used reentrantly. <br>这种互斥锁在其 `Drop` 实现中清理了其资源，可以安全地移动 (不借用时)，并且在重入使用时不会导致 UB。<br>
///
///
/// This mutex does not implement poisoning. <br>此互斥锁不会实现中毒。<br>
///
/// This is either a wrapper around `LazyBox<imp::Mutex>` or `imp::Mutex`, depending on the platform. <br>这是围绕 `LazyBox<imp::Mutex>` 或 `imp::Mutex` 的包装器，具体取决于平台。<br>
/// It is boxed on platforms where `imp::Mutex` may not be moved. <br>在 `imp::Mutex` 可能不会移动的平台上，它是 boxed。<br>
///
pub struct MovableMutex(imp::MovableMutex);

unsafe impl Sync for MovableMutex {}

impl MovableMutex {
    /// Creates a new mutex. <br>创建一个新的互斥锁。<br>
    #[inline]
    pub const fn new() -> Self {
        Self(imp::MovableMutex::new())
    }

    pub(super) fn raw(&self) -> &imp::Mutex {
        &self.0
    }

    /// Locks the mutex blocking the current thread until it is available. <br>锁定互斥锁以阻止当前线程，直到可用为止。<br>
    #[inline]
    pub fn raw_lock(&self) {
        unsafe { self.0.lock() }
    }

    /// Attempts to lock the mutex without blocking, returning whether it was successfully acquired or not. <br>尝试不间断地锁定互斥锁，并返回是否已成功获取它。<br>
    ///
    #[inline]
    pub fn try_lock(&self) -> bool {
        unsafe { self.0.try_lock() }
    }

    /// Unlocks the mutex. <br>解锁互斥锁。<br>
    ///
    /// Behavior is undefined if the current thread does not actually hold the mutex. <br>如果当前线程实际上没有持有互斥锁，则行为是不确定的。<br>
    ///
    #[inline]
    pub unsafe fn raw_unlock(&self) {
        self.0.unlock()
    }
}
