#[cfg(all(test, not(target_os = "emscripten")))]
mod tests;

use crate::cell::UnsafeCell;
use crate::marker::PhantomPinned;
use crate::ops::Deref;
use crate::panic::{RefUnwindSafe, UnwindSafe};
use crate::pin::Pin;
use crate::sync::atomic::{AtomicUsize, Ordering::Relaxed};
use crate::sys::locks as sys;

/// A re-entrant mutual exclusion <br>可重入的相互排斥<br>
///
/// This mutex will block *other* threads waiting for the lock to become available. <br>此互斥锁将阻止 *其他* 线程，等待该锁可用。<br> The thread which has already locked the mutex can lock it multiple times without blocking, preventing a common source of deadlocks. <br>已经锁定互斥锁的线程可以多次将其锁定而不会阻塞，从而避免了常见的死锁源。<br>
///
/// This is used by stdout().lock() and friends. <br>这是由 stdout().lock() 和朋友使用的。<br>
///
/// ## Implementation details <br>实现细节<br>
///
/// The 'owner' field tracks which thread has locked the mutex. <br>'owner' 字段跟踪哪个线程锁定了互斥锁。<br>
///
/// We use current_thread_unique_ptr() as the thread identifier, which is just the address of a thread local variable. <br>我们使用 current_thread_unique_ptr() 作为线程标识符，它只是一个线程局部变量的地址。<br>
///
/// If `owner` is set to the identifier of the current thread, we assume the mutex is already locked and instead of locking it again, we increment `lock_count`. <br>如果 `owner` 设置为当前线程的标识符，我们假设互斥锁已经被锁定，而不是再次锁定它，我们增加 `lock_count`。<br>
///
/// When unlocking, we decrement `lock_count`, and only unlock the mutex when it reaches zero. <br>解锁时，我们将 `lock_count` 递减，互斥锁达到 0 时才解锁。<br>
///
/// `lock_count` is protected by the mutex and only accessed by the thread that has locked the mutex, so needs no synchronization. <br>`lock_count` 受互斥锁保护，只有锁定了互斥锁的线程才能访问，所以不需要同步。<br>
///
/// `owner` can be checked by other threads that want to see if they already hold the lock, so needs to be atomic. <br>`owner` 可以被其他想要查看它们是否已经持有锁的线程检查，因此需要是原子的。<br> If it compares equal, we're on the same thread that holds the mutex and memory access can use relaxed ordering since we're not dealing with multiple threads. <br>如果比较相等，我们就在持有互斥锁的同一个线程上，并且内存访问可以使用宽松的排序，因为我们不处理多个线程。<br>
///
/// If it compares unequal, synchronization is left to the mutex, making relaxed memory ordering for the `owner` field fine in all cases. <br>如果比较不相等，则将同步留给互斥锁，使 `owner` 字段的宽松内存排序在所有情况下都很好。<br>
///
///
///
///
///
///
///
///
///
///
pub struct ReentrantMutex<T> {
    mutex: sys::Mutex,
    owner: AtomicUsize,
    lock_count: UnsafeCell<u32>,
    data: T,
    _pinned: PhantomPinned,
}

unsafe impl<T: Send> Send for ReentrantMutex<T> {}
unsafe impl<T: Send> Sync for ReentrantMutex<T> {}

impl<T> UnwindSafe for ReentrantMutex<T> {}
impl<T> RefUnwindSafe for ReentrantMutex<T> {}

/// An RAII implementation of a "scoped lock" of a mutex. <br>互斥锁的 "scoped lock" 的 RAII 实现。<br> When this structure is dropped (falls out of scope), the lock will be unlocked. <br>当此结构体被丢弃 (离开作用域) 时，这个锁将被解锁。<br>
///
/// The data protected by the mutex can be accessed through this guard via its Deref implementation. <br>可以通过此保护程序的 Deref 实现访问由互斥锁保护的数据。<br>
///
/// # Mutability
///
/// Unlike `MutexGuard`, `ReentrantMutexGuard` does not implement `DerefMut`, because implementation of the trait would violate Rust’s reference aliasing rules. <br>与 `MutexGuard` 不同，`ReentrantMutexGuard` 不实现 `DerefMut`，因为实现 trait 会违反 Rust 的引用别名规则。<br>
/// Use interior mutability (usually `RefCell`) in order to mutate the guarded data. <br>使用内部可变性 (通常为 `RefCell`) 以更改受保护的数据。<br>
///
///
///
///
#[must_use = "if unused the ReentrantMutex will immediately unlock"]
pub struct ReentrantMutexGuard<'a, T: 'a> {
    lock: Pin<&'a ReentrantMutex<T>>,
}

impl<T> !Send for ReentrantMutexGuard<'_, T> {}

impl<T> ReentrantMutex<T> {
    /// Creates a new reentrant mutex in an unlocked state. <br>在解锁状态下创建一个新的可重入互斥锁。<br>
    ///
    /// # Unsafety
    ///
    /// This function is unsafe because it is required that `init` is called once this mutex is in its final resting place, and only then are the lock/unlock methods safe. <br>该函数是不安全的，因为一旦此互斥锁处于其最终的静止位置，就要求调用 `init`，只有这样 lock/unlock 的方法才是安全的。<br>
    ///
    ///
    pub const unsafe fn new(t: T) -> ReentrantMutex<T> {
        ReentrantMutex {
            mutex: sys::Mutex::new(),
            owner: AtomicUsize::new(0),
            lock_count: UnsafeCell::new(0),
            data: t,
            _pinned: PhantomPinned,
        }
    }

    /// Initializes this mutex so it's ready for use. <br>初始化此互斥锁，以便可以使用。<br>
    ///
    /// # Unsafety
    ///
    /// Unsafe to call more than once, and must be called after this will no longer move in memory. <br>多次调用是不安全的，必须在此之后调用，它将不再在内存中移动。<br>
    ///
    pub unsafe fn init(self: Pin<&mut Self>) {
        self.get_unchecked_mut().mutex.init()
    }

    /// Acquires a mutex, blocking the current thread until it is able to do so. <br>获取一个互斥锁，阻塞当前线程，直到能够这样做为止。<br>
    ///
    /// This function will block the caller until it is available to acquire the mutex. <br>此函数将阻止调用者，直到可以获取互斥锁为止。<br>
    /// Upon returning, the thread is the only thread with the mutex held. <br>返回时，该线程是唯一具有互斥锁的线程。<br>
    /// When the thread calling this method already holds the lock, the call shall succeed without blocking. <br>当调用此方法的线程已持有锁时，调用将成功而不会阻塞。<br>
    ///
    /// # Errors
    ///
    /// If another user of this mutex panicked while holding the mutex, then this call will return failure if the mutex would otherwise be acquired. <br>如果此互斥锁的另一个用户在持有互斥锁时 panic，那么如果以其他方式获取互斥锁，则此调用将返回失败。<br>
    ///
    ///
    ///
    pub fn lock(self: Pin<&Self>) -> ReentrantMutexGuard<'_, T> {
        let this_thread = current_thread_unique_ptr();
        // Safety: We only touch lock_count when we own the lock, and since self is pinned we can safely call the lock() on the mutex. <br>安全性: 我们只有在拥有锁时才触摸 lock_count，并且由于 self 被固定，我们可以安全地调用互斥锁上的 lock()。<br>
        //
        unsafe {
            if self.owner.load(Relaxed) == this_thread {
                self.increment_lock_count();
            } else {
                self.mutex.lock();
                self.owner.store(this_thread, Relaxed);
                debug_assert_eq!(*self.lock_count.get(), 0);
                *self.lock_count.get() = 1;
            }
        }
        ReentrantMutexGuard { lock: self }
    }

    /// Attempts to acquire this lock. <br>尝试获取此锁。<br>
    ///
    /// If the lock could not be acquired at this time, then `Err` is returned. <br>如果此时无法获取锁，则返回 `Err`。<br>
    /// Otherwise, an RAII guard is returned. <br>否则，将返回 RAII 守卫。<br>
    ///
    /// This function does not block. <br>该函数不会阻止。<br>
    ///
    /// # Errors
    ///
    /// If another user of this mutex panicked while holding the mutex, then this call will return failure if the mutex would otherwise be acquired. <br>如果此互斥锁的另一个用户在持有互斥锁时 panic，那么如果以其他方式获取互斥锁，则此调用将返回失败。<br>
    ///
    ///
    pub fn try_lock(self: Pin<&Self>) -> Option<ReentrantMutexGuard<'_, T>> {
        let this_thread = current_thread_unique_ptr();
        // Safety: We only touch lock_count when we own the lock, and since self is pinned we can safely call the try_lock on the mutex. <br>安全性: 我们只有在拥有锁时才接触 lock_count，并且由于 self 被固定，我们可以安全地调用互斥锁上的 try_lock。<br>
        //
        unsafe {
            if self.owner.load(Relaxed) == this_thread {
                self.increment_lock_count();
                Some(ReentrantMutexGuard { lock: self })
            } else if self.mutex.try_lock() {
                self.owner.store(this_thread, Relaxed);
                debug_assert_eq!(*self.lock_count.get(), 0);
                *self.lock_count.get() = 1;
                Some(ReentrantMutexGuard { lock: self })
            } else {
                None
            }
        }
    }

    unsafe fn increment_lock_count(&self) {
        *self.lock_count.get() = (*self.lock_count.get())
            .checked_add(1)
            .expect("lock count overflow in reentrant mutex");
    }
}

impl<T> Deref for ReentrantMutexGuard<'_, T> {
    type Target = T;

    fn deref(&self) -> &T {
        &self.lock.data
    }
}

impl<T> Drop for ReentrantMutexGuard<'_, T> {
    #[inline]
    fn drop(&mut self) {
        // Safety: We own the lock, and the lock is pinned. <br>安全: 我们拥有锁，而且锁是被固定的。<br>
        unsafe {
            *self.lock.lock_count.get() -= 1;
            if *self.lock.lock_count.get() == 0 {
                self.lock.owner.store(0, Relaxed);
                self.lock.mutex.unlock();
            }
        }
    }
}

/// Get an address that is unique per running thread. <br>获取每个正在运行的线程的唯一地址。<br>
///
/// This can be used as a non-null usize-sized ID. <br>这可以用作非空的 usize 大小的 ID。<br>
pub fn current_thread_unique_ptr() -> usize {
    // Use a non-drop type to make sure it's still available during thread destruction. <br>使用非丢弃类型来确保它在线程销毁期间仍然可用。<br>
    thread_local! { static X: u8 = const { 0 } }
    X.with(|x| <*const _>::addr(x))
}
