use crate::sys::locks as imp;

/// An OS-based reader-writer lock, meant for use in static variables. <br>基于操作系统的读写锁，用于静态变量。<br>
///
/// This rwlock does not implement poisoning. <br>这个 rwlock 没有实现中毒。<br>
///
/// This rwlock has a const constructor ([`StaticRwLock::new`]), does not implement `Drop` to cleanup resources. <br>这个 rwlock 有一个 const 构造函数 ([`StaticRwLock::new`])，没有实现 `Drop` 来清理资源。<br>
///
pub struct StaticRwLock(imp::RwLock);

impl StaticRwLock {
    /// Creates a new rwlock for use. <br>创建一个新的 rwlock 以供使用。<br>
    #[inline]
    pub const fn new() -> Self {
        Self(imp::RwLock::new())
    }

    /// Acquires shared access to the underlying lock, blocking the current thread to do so. <br>获取对底层锁的共享访问，从而阻止当前线程这样做。<br>
    ///
    ///
    /// The lock is automatically unlocked when the returned guard is dropped. <br>当返回的守卫丢弃时，锁会自动解锁。<br>
    #[inline]
    pub fn read(&'static self) -> StaticRwLockReadGuard {
        unsafe { self.0.read() };
        StaticRwLockReadGuard(&self.0)
    }

    /// Acquires write access to the underlying lock, blocking the current thread to do so. <br>获取对底层锁的写访问权，从而阻止当前线程这样做。<br>
    ///
    ///
    /// The lock is automatically unlocked when the returned guard is dropped. <br>当返回的守卫丢弃时，锁会自动解锁。<br>
    #[inline]
    pub fn write(&'static self) -> StaticRwLockWriteGuard {
        unsafe { self.0.write() };
        StaticRwLockWriteGuard(&self.0)
    }
}

#[must_use]
pub struct StaticRwLockReadGuard(&'static imp::RwLock);

impl Drop for StaticRwLockReadGuard {
    #[inline]
    fn drop(&mut self) {
        unsafe {
            self.0.read_unlock();
        }
    }
}

#[must_use]
pub struct StaticRwLockWriteGuard(&'static imp::RwLock);

impl Drop for StaticRwLockWriteGuard {
    #[inline]
    fn drop(&mut self) {
        unsafe {
            self.0.write_unlock();
        }
    }
}

/// An OS-based reader-writer lock. <br>基于 OS 的 reader-writer 锁。<br>
///
/// This rwlock cleans up its resources in its `Drop` implementation and may safely be moved (when not borrowed). <br>此 rwlock 在其 `Drop` 实现中清理其资源，并且可以安全地移动 (当不借用时)。<br>
///
///
/// This rwlock does not implement poisoning. <br>这个 rwlock 没有实现中毒。<br>
///
/// This is either a wrapper around `LazyBox<imp::RwLock>` or `imp::RwLock`, depending on the platform. <br>这是围绕 `LazyBox<imp::RwLock>` 或 `imp::RwLock` 的包装器，具体取决于平台。<br>
/// It is boxed on platforms where `imp::RwLock` may not be moved. <br>在不能移动 `imp::RwLock` 的平台上是 boxed。<br>
///
pub struct MovableRwLock(imp::MovableRwLock);

impl MovableRwLock {
    /// Creates a new reader-writer lock for use. <br>创建一个新的 reader-writer 锁以供使用。<br>
    #[inline]
    pub const fn new() -> Self {
        Self(imp::MovableRwLock::new())
    }

    /// Acquires shared access to the underlying lock, blocking the current thread to do so. <br>获取对底层锁的共享访问，从而阻止当前线程这样做。<br>
    ///
    #[inline]
    pub fn read(&self) {
        unsafe { self.0.read() }
    }

    /// Attempts to acquire shared access to this lock, returning whether it succeeded or not. <br>尝试获取对此锁的共享访问权，并返回是否成功。<br>
    ///
    ///
    /// This function does not block the current thread. <br>此函数不会阻止当前线程。<br>
    #[inline]
    pub fn try_read(&self) -> bool {
        unsafe { self.0.try_read() }
    }

    /// Acquires write access to the underlying lock, blocking the current thread to do so. <br>获取对底层锁的写访问权，从而阻止当前线程这样做。<br>
    ///
    #[inline]
    pub fn write(&self) {
        unsafe { self.0.write() }
    }

    /// Attempts to acquire exclusive access to this lock, returning whether it succeeded or not. <br>尝试获取对此锁的独占访问权，并返回是否成功。<br>
    ///
    ///
    /// This function does not block the current thread. <br>此函数不会阻止当前线程。<br>
    #[inline]
    pub fn try_write(&self) -> bool {
        unsafe { self.0.try_write() }
    }

    /// Unlocks previously acquired shared access to this lock. <br>解锁先前获取的对此锁的共享访问权限。<br>
    ///
    /// Behavior is undefined if the current thread does not have shared access. <br>如果当前线程没有共享访问权限，则行为是不确定的。<br>
    #[inline]
    pub unsafe fn read_unlock(&self) {
        self.0.read_unlock()
    }

    /// Unlocks previously acquired exclusive access to this lock. <br>解锁先前获得的对此锁的独占访问权。<br>
    ///
    /// Behavior is undefined if the current thread does not currently have exclusive access. <br>如果当前线程当前没有独占访问权，则行为是不确定的。<br>
    ///
    #[inline]
    pub unsafe fn write_unlock(&self) {
        self.0.write_unlock()
    }
}
