use crate::sys::locks as imp;

/// 基于操作系统的读写锁，用于静态变量。
///
/// 这个 rwlock 没有实现中毒。
///
/// 这个 rwlock 有一个 const 构造函数 ([`StaticRwLock::new`])，没有实现 `Drop` 来清理资源。
///
pub struct StaticRwLock(imp::RwLock);

impl StaticRwLock {
    /// 创建一个新的 rwlock 以供使用。
    #[inline]
    pub const fn new() -> Self {
        Self(imp::RwLock::new())
    }

    /// 获取对底层锁的共享访问，从而阻止当前线程这样做。
    ///
    ///
    /// 当返回的守卫丢弃时，锁会自动解锁。
    #[inline]
    pub fn read(&'static self) -> StaticRwLockReadGuard {
        unsafe { self.0.read() };
        StaticRwLockReadGuard(&self.0)
    }

    /// 获取对底层锁的写访问权，从而阻止当前线程这样做。
    ///
    ///
    /// 当返回的守卫丢弃时，锁会自动解锁。
    #[inline]
    pub fn write(&'static self) -> StaticRwLockWriteGuard {
        unsafe { self.0.write() };
        StaticRwLockWriteGuard(&self.0)
    }
}

#[must_use]
pub struct StaticRwLockReadGuard(&'static imp::RwLock);

impl Drop for StaticRwLockReadGuard {
    #[inline]
    fn drop(&mut self) {
        unsafe {
            self.0.read_unlock();
        }
    }
}

#[must_use]
pub struct StaticRwLockWriteGuard(&'static imp::RwLock);

impl Drop for StaticRwLockWriteGuard {
    #[inline]
    fn drop(&mut self) {
        unsafe {
            self.0.write_unlock();
        }
    }
}

/// 基于 OS 的 reader-writer 锁。
///
/// 此 rwlock 在其 `Drop` 实现中清理其资源，并且可以安全地移动 (当不借用时)。
///
///
/// 这个 rwlock 没有实现中毒。
///
/// 这是围绕 `LazyBox<imp::RwLock>` 或 `imp::RwLock` 的包装器，具体取决于平台。
/// 在不能移动 `imp::RwLock` 的平台上是 boxed。
///
pub struct MovableRwLock(imp::MovableRwLock);

impl MovableRwLock {
    /// 创建一个新的 reader-writer 锁以供使用。
    #[inline]
    pub const fn new() -> Self {
        Self(imp::MovableRwLock::new())
    }

    /// 获取对底层锁的共享访问，从而阻止当前线程这样做。
    ///
    #[inline]
    pub fn read(&self) {
        unsafe { self.0.read() }
    }

    /// 尝试获取对此锁的共享访问权，并返回是否成功。
    ///
    ///
    /// 此函数不会阻止当前线程。
    #[inline]
    pub fn try_read(&self) -> bool {
        unsafe { self.0.try_read() }
    }

    /// 获取对底层锁的写访问权，从而阻止当前线程这样做。
    ///
    #[inline]
    pub fn write(&self) {
        unsafe { self.0.write() }
    }

    /// 尝试获取对此锁的独占访问权，并返回是否成功。
    ///
    ///
    /// 此函数不会阻止当前线程。
    #[inline]
    pub fn try_write(&self) -> bool {
        unsafe { self.0.try_write() }
    }

    /// 解锁先前获取的对此锁的共享访问权限。
    ///
    /// 如果当前线程没有共享访问权限，则行为是不确定的。
    #[inline]
    pub unsafe fn read_unlock(&self) {
        self.0.read_unlock()
    }

    /// 解锁先前获得的对此锁的独占访问权。
    ///
    /// 如果当前线程当前没有独占访问权，则行为是不确定的。
    ///
    #[inline]
    pub unsafe fn write_unlock(&self) {
        self.0.write_unlock()
    }
}
