//! 线程局部析构函数
//!
//! 除了线程本地 "keys" (具有关联的析构函数的指针大小的不可寻址线程本地存储) 之外，许多平台还提供与任何特定数据都不关联的线程本地析构函数。
//! 这些往往效率更高。
//!
//! 该模块基于效率较低的线程本地 "keys"，为该接口提供了一个回退实现。
//! 每个平台都提供一个 `thread_local_dtor` 模块，该模块将重新导出回退，或者实现一些更有效的东西。
//!
//!
//!
//!

#![unstable(feature = "thread_local_internals", issue = "none")]
#![allow(dead_code)]

use crate::ptr;
use crate::sys_common::thread_local_key::StaticKey;

pub unsafe fn register_dtor_fallback(t: *mut u8, dtor: unsafe extern "C" fn(*mut u8)) {
    // 回退实现使用一个普通的基于操作系统的 TLS 密钥来跟踪这个线程需要运行的析构函数列表。
    // 然后，键有自己的析构函数，它运行所有其他的析构函数。
    //
    // DTORS 的析构函数有点特殊，因为它有一个 `while` 循环来不断地清除已注册的析构函数列表。
    // 在这种情况下，该循环总是会终止，因为我们提供了保证，在将 TLS 密钥标记为要销毁之后，无法设置 TLS 密钥。
    //
    //
    //
    //

    static DTORS: StaticKey = StaticKey::new(Some(run_dtors));
    type List = Vec<(*mut u8, unsafe extern "C" fn(*mut u8))>;
    if DTORS.get().is_null() {
        let v: Box<List> = box Vec::new();
        DTORS.set(Box::into_raw(v) as *mut u8);
    }
    let list: &mut List = &mut *(DTORS.get() as *mut List);
    list.push((t, dtor));

    unsafe extern "C" fn run_dtors(mut ptr: *mut u8) {
        while !ptr.is_null() {
            let list: Box<List> = Box::from_raw(ptr as *mut List);
            for (ptr, dtor) in list.into_iter() {
                dtor(ptr);
            }
            ptr = DTORS.get();
            DTORS.set(ptr::null_mut());
        }
    }
}
