//! OS-based thread local storage <br>基于操作系统的线程本地存储<br>
//!
//! This module provides an implementation of OS-based thread local storage, using the native OS-provided facilities (think `TlsAlloc` or `pthread_setspecific`). <br>该模块提供了一个基于操作系统的线程本地存储的实现，使用了本地操作系统提供的工具 (想想 `TlsAlloc` 或 `pthread_setspecific`)。<br>
//! The interface of this differs from the other types of thread-local-storage provided in this crate in that OS-based TLS can only get/set pointer-sized data, possibly with an associated destructor. <br>它的接口与此 crate 中提供的其他类型的线程本地存储不同，因为基于操作系统的 TLS 只能获取/设置指针大小的数据，可能带有关联的析构函数。<br>
//!
//!
//! This module also provides two flavors of TLS. <br>该模块还提供了两种 TLS。<br> One is intended for static initialization, and does not contain a `Drop` implementation to deallocate the OS-TLS key. <br>其中一个用于静态初始化，不包含用于释放 OS-TLS 密钥的 `Drop` 实现。<br>
//! The other is a type which does implement `Drop` and hence has a safe interface. <br>另一个是实现了 `Drop` 的类型，因此有一个安全的接口。<br>
//!
//! # Usage
//!
//! This module should likely not be used directly unless other primitives are being built on. <br>除非在其他原语的基础上构建，否则不应该直接使用此模块。<br>
//! Types such as `thread_local::spawn::Key` are likely much more useful in practice than this OS-based version which likely requires unsafe code to interoperate with. <br>像 `thread_local::spawn::Key` 这样的类型在实践中可能比此基于 OS 的版本有用得多，后者可能需要不安全的代码才能进行互操作。<br>
//!
//! # Examples
//!
//! Using a dynamically allocated TLS key. <br>使用动态分配的 TLS 密钥。<br> Note that this key can be shared among many threads via an `Arc`. <br>请注意，这个键可以通过 `Arc` 在多个线程中共享。<br>
//!
//! ```ignore (cannot-doctest-private-modules)
//! let key = Key::new(None);
//! assert!(key.get().is_null());
//! key.set(1 as *mut u8);
//! assert!(!key.get().is_null());
//!
//! drop(key); // deallocate this TLS slot. <br>释放此 TLS 插槽。<br>
//! ```
//!
//! Sometimes a statically allocated key is either required or easier to work with, however. <br>有时，静态分配的密钥是必需的，或者更易于使用。<br>
//!
//! ```ignore (cannot-doctest-private-modules)
//! static KEY: StaticKey = INIT;
//!
//! unsafe {
//!     assert!(KEY.get().is_null());
//!     KEY.set(1 as *mut u8);
//! }
//! ```
//!
//!
//!
//!
//!
//!
//!
//!

#![allow(non_camel_case_types)]
#![unstable(feature = "thread_local_internals", issue = "none")]
#![allow(dead_code)]

#[cfg(test)]
mod tests;

use crate::sync::atomic::{self, AtomicUsize, Ordering};
use crate::sys::thread_local_key as imp;
use crate::sys_common::mutex::StaticMutex;

/// A type for TLS keys that are statically allocated. <br>静态分配的 TLS 密钥的类型。<br>
///
/// This type is entirely `unsafe` to use as it does not protect against use-after-deallocation or use-during-deallocation. <br>使用此类型是完全 `unsafe` 的，因为它不能防止在释放后使用或释放​​期间使用。<br>
///
///
/// The actual OS-TLS key is lazily allocated when this is used for the first time. <br>第一次使用时会延迟分配实际的 OS-TLS 密钥。<br>
/// The key is also deallocated when the Rust runtime exits or `destroy` is called, whichever comes first. <br>当 Rust 运行时退出或调用 `destroy` (以先到者为准) 时，也将释放该键。<br>
///
/// # Examples
///
/// ```ignore (cannot-doctest-private-modules)
/// use tls::os::{StaticKey, INIT};
///
/// static KEY: StaticKey = INIT;
///
/// unsafe {
///     assert!(KEY.get().is_null());
///     KEY.set(1 as *mut u8);
/// }
/// ```
///
pub struct StaticKey {
    /// Inner static TLS key (internals). <br>内部静态 TLS 密钥 (internals)。<br>
    key: AtomicUsize,
    /// Destructor for the TLS value. <br>TLS 值的析构函数。<br>
    ///
    /// See `Key::new` for information about when the destructor runs and how it runs. <br>有关析构函数何时运行以及运行方式的信息，请参见 `Key::new`。<br>
    ///
    dtor: Option<unsafe extern "C" fn(*mut u8)>,
}

/// A type for a safely managed OS-based TLS slot. <br>一种安全管理的基于 OS 的 TLS 插槽的类型。<br>
///
/// This type allocates an OS TLS key when it is initialized and will deallocate the key when it falls out of scope. <br>初始化时，此类型分配 OS TLS 密钥; 当它离开离开作用域时，将释放该密钥。<br>
/// When compared with `StaticKey`, this type is entirely safe to use. <br>与 `StaticKey` 相比，此类型完全可以安全使用。<br>
///
/// Implementations will likely, however, contain unsafe code as this type only operates on `*mut u8`, a raw pointer. <br>但是，实现可能会包含不安全的代码，因为此类型仅在 `*mut u8` (裸指针) 上运行。<br>
///
///
/// # Examples
///
/// ```ignore (cannot-doctest-private-modules)
/// use tls::os::Key;
///
/// let key = Key::new(None);
/// assert!(key.get().is_null());
/// key.set(1 as *mut u8);
/// assert!(!key.get().is_null());
///
/// drop(key); // deallocate this TLS slot. <br>释放此 TLS 插槽。<br>
/// ```
///
pub struct Key {
    key: imp::Key,
}

/// Constant initialization value for static TLS keys. <br>静态 TLS 密钥的常量初始化值。<br>
///
/// This value specifies no destructor by default. <br>默认情况下，此值不指定析构函数。<br>
pub const INIT: StaticKey = StaticKey::new(None);

impl StaticKey {
    #[rustc_const_unstable(feature = "thread_local_internals", issue = "none")]
    pub const fn new(dtor: Option<unsafe extern "C" fn(*mut u8)>) -> StaticKey {
        StaticKey { key: atomic::AtomicUsize::new(0), dtor }
    }

    /// Gets the value associated with this TLS key <br>获取与此 TLS 密钥关联的值<br>
    ///
    /// This will lazily allocate a TLS key from the OS if one has not already been allocated. <br>如果尚未分配 TLS 密钥，则会从操作系统延迟分配 TLS 密钥。<br>
    ///
    #[inline]
    pub unsafe fn get(&self) -> *mut u8 {
        imp::get(self.key())
    }

    /// Sets this TLS key to a new value. <br>将此 TLS 密钥设置为新值。<br>
    ///
    /// This will lazily allocate a TLS key from the OS if one has not already been allocated. <br>如果尚未分配 TLS 密钥，则会从操作系统延迟分配 TLS 密钥。<br>
    ///
    #[inline]
    pub unsafe fn set(&self, val: *mut u8) {
        imp::set(self.key(), val)
    }

    #[inline]
    unsafe fn key(&self) -> imp::Key {
        match self.key.load(Ordering::Relaxed) {
            0 => self.lazy_init() as imp::Key,
            n => n as imp::Key,
        }
    }

    unsafe fn lazy_init(&self) -> usize {
        // Currently the Windows implementation of TLS is pretty hairy, and it greatly simplifies creation if we just synchronize everything. <br>当前，TLS 的 Windows 实现非常繁琐，如果我们仅同步所有内容，它将大大简化创建过程。<br>
        //
        //
        // Additionally a 0-index of a tls key hasn't been seen on windows, so we just simplify the whole branch. <br>此外，在 windows 上还没有看到 tls 键的 0 索引，因此我们只是简化了整个分支。<br>
        //
        if imp::requires_synchronized_create() {
            // We never call `INIT_LOCK.init()`, so it is UB to attempt to acquire this mutex reentrantly! <br>我们从不调用 `INIT_LOCK.init()`，因此尝试重新获得此互斥锁是 UB!<br>
            //
            static INIT_LOCK: StaticMutex = StaticMutex::new();
            let _guard = INIT_LOCK.lock();
            let mut key = self.key.load(Ordering::SeqCst);
            if key == 0 {
                key = imp::create(self.dtor) as usize;
                self.key.store(key, Ordering::SeqCst);
            }
            rtassert!(key != 0);
            return key;
        }

        // POSIX allows the key created here to be 0, but the compare_exchange below relies on using 0 as a sentinel value to check who won the race to set the shared TLS key. <br>POSIX 允许在此处创建的密钥为 0，但是下面的 compare_exchange 依靠使用 0 作为标记值来检查谁赢得了设置共享 TLS 密钥的竞争。<br>
        // As far as I know, there is no guaranteed value that cannot be returned as a posix_key_create key, so there is no value we can initialize the inner key with to prove that it has not yet been set. <br>据我所知，没有保证值不能作为 posix_key_create 键返回，因此没有值可以初始化内部键以证明尚未设置。<br>
        //
        // As such, we'll continue using a value of 0, but with some gyrations to make sure we have a non-0 value returned from the creation routine. <br>因此，我们将继续使用 0 值，但要进行一些旋转以确保我们从创建例程返回了 non-0 值。<br>
        // FIXME: this is clearly a hack, and should be cleaned up. <br>这显然是一个 hack，应该对其进行清理。<br>
        //
        //
        //
        //
        let key1 = imp::create(self.dtor);
        let key = if key1 != 0 {
            key1
        } else {
            let key2 = imp::create(self.dtor);
            imp::destroy(key1);
            key2
        };
        rtassert!(key != 0);
        match self.key.compare_exchange(0, key as usize, Ordering::SeqCst, Ordering::SeqCst) {
            // The CAS succeeded, so we've created the actual key <br>CAS 成功了，所以我们创建了实际的密钥<br>
            Ok(_) => key as usize,
            // If someone beat us to the punch, use their key instead <br>如果有人超过了我们，请改用他们的秘钥<br>
            Err(n) => {
                imp::destroy(key);
                n
            }
        }
    }
}

impl Key {
    /// Creates a new managed OS TLS key. <br>创建一个新的托管 OS TLS 密钥。<br>
    ///
    /// This key will be deallocated when the key falls out of scope. <br>当秘钥离开作用域时，该键将被释放。<br>
    ///
    /// The argument provided is an optionally-specified destructor for the value of this TLS key. <br>提供的参数是此 TLS 密钥的值的可选指定的析构函数。<br>
    /// When a thread exits and the value for this key is non-null the destructor will be invoked. <br>当线程退出并且此键的值不为 null 时，将调用析构函数。<br>
    /// The TLS value will be reset to null before the destructor is invoked. <br>在调用析构函数之前，TLS 值将重置为 null。<br>
    ///
    /// Note that the destructor will not be run when the `Key` goes out of scope. <br>请注意，当 `Key` 离开作用域时，析构函数将不会运行。<br>
    ///
    ///
    #[inline]
    pub fn new(dtor: Option<unsafe extern "C" fn(*mut u8)>) -> Key {
        Key { key: unsafe { imp::create(dtor) } }
    }

    /// See StaticKey::get <br>请参见 StaticKey::get<br>
    #[inline]
    pub fn get(&self) -> *mut u8 {
        unsafe { imp::get(self.key) }
    }

    /// See StaticKey::set <br>请参见 StaticKey::set<br>
    #[inline]
    pub fn set(&self, val: *mut u8) {
        unsafe { imp::set(self.key, val) }
    }
}

impl Drop for Key {
    fn drop(&mut self) {
        // Right now Windows doesn't support TLS key destruction, but this also isn't used anywhere other than tests, so just leak the TLS key. <br>目前，Windows 不支持 TLS 密钥销毁，但是除了测试以外，其他任何地方都没有使用它，因此只需泄漏 TLS 密钥即可。<br>
        //
        // unsafe { imp::destroy(self.key) }
    }
}
