use crate::pin::Pin;
use crate::sync::atomic::AtomicU32;
use crate::sync::atomic::Ordering::{Acquire, Release};
use crate::sys::futex::{futex_wait, futex_wake};
use crate::time::Duration;

const PARKED: u32 = u32::MAX;
const EMPTY: u32 = 0;
const NOTIFIED: u32 = 1;

pub struct Parker {
    state: AtomicU32,
}

// Notes about memory ordering: <br>有关内存顺序的注意事项：<br>
//
// Memory ordering is only relevant for the relative ordering of operations between different variables. <br>内存排序仅与不同变量之间操作的相对排序有关。<br>
// Even Ordering::Relaxed guarantees a monotonic/consistent order when looking at just a single atomic variable. <br>仅查看单个原子变量时，甚至 Ordering::Relaxed 都可以保证单调/一致的顺序。<br>
//
// So, since this parker is just a single atomic variable, we only need to look at the ordering guarantees we need to provide to the 'outside world'. <br>因此，由于该 Parker 只是单一的原子变量，因此我们只需要查看我们需要提供向外部世界的排序保证。<br>
//
// The only memory ordering guarantee that parking and unparking provide, is that things which happened before unpark() are visible on the thread returning from park() afterwards. <br>保留和取消保留的唯一内存排序保证是，在 unpark() 之后发生的线程上可以看到 unpark() 之前发生的事情。<br>
// Otherwise, it was effectively unparked before unpark() was called while still consuming the 'token'. <br>否则，在仍然消耗 'token' 的同时，在调用 unpark() 之前将其有效地停了下来。<br>
//
// In other words, unpark() needs to synchronize with the part of park() that consumes the token and returns. <br>换句话说，unpark() 需要与 park() 消耗 token 并返回的部分进行同步。<br>
//
// This is done with a release-acquire synchronization, by using Ordering::Release when writing NOTIFIED (the 'token') in unpark(), and using Ordering::Acquire when checking for this state in park(). <br>通过在 unpark() 中写入 NOTIFIED ('token') 时使用 Ordering::Release，并在 park() 中检查此状态时使用 Ordering::Acquire，可以实现发布 - 获取同步。<br>
//
//
//
//
//
//
//
impl Parker {
    /// Construct the futex parker. <br>构建 futex parker。<br>
    /// The UNIX parker implementation requires this to happen in-place. <br>UNIX parker 实现要求这发生在原地。<br>
    pub unsafe fn new(parker: *mut Parker) {
        parker.write(Self { state: AtomicU32::new(EMPTY) });
    }

    // Assumes this is only called by the thread that owns the Parker, which means that `self.state != PARKED`. <br>假定仅由拥有 Parker 的线程 (称为 `self.state != PARKED`) 调用此方法。<br>
    //
    pub unsafe fn park(self: Pin<&Self>) {
        // Change NOTIFIED=>EMPTY or EMPTY=>PARKED, and directly return in the first case. <br>更改 NOTIFIED => EMPTY 或 EMPTY => PARKED，并在第一种情况下直接返回。<br>
        //
        if self.state.fetch_sub(1, Acquire) == NOTIFIED {
            return;
        }
        loop {
            // Wait for something to happen, assuming it's still set to PARKED. <br>假设它仍然设置为 PARKED，请等待发生的事情。<br>
            futex_wait(&self.state, PARKED, None);
            // Change NOTIFIED=>EMPTY and return in that case. <br>更改 NOTIFIED => EMPTY 并在这种情况下返回。<br>
            if self.state.compare_exchange(NOTIFIED, EMPTY, Acquire, Acquire).is_ok() {
                return;
            } else {
                // Spurious wake up. <br>虚假的醒来。<br> We loop to try again. <br>我们循环播放以重试。<br>
            }
        }
    }

    // Assumes this is only called by the thread that owns the Parker, which means that `self.state != PARKED`. <br>假定仅由拥有 Parker 的线程 (称为 `self.state != PARKED`) 调用此方法。<br>
    // This implementation doesn't require `Pin`, but other implementations do. <br>此实现不需要 `Pin`，但其他实现需要。<br>
    //
    pub unsafe fn park_timeout(self: Pin<&Self>, timeout: Duration) {
        // Change NOTIFIED=>EMPTY or EMPTY=>PARKED, and directly return in the first case. <br>更改 NOTIFIED => EMPTY 或 EMPTY => PARKED，并在第一种情况下直接返回。<br>
        //
        if self.state.fetch_sub(1, Acquire) == NOTIFIED {
            return;
        }
        // Wait for something to happen, assuming it's still set to PARKED. <br>假设它仍然设置为 PARKED，请等待发生的事情。<br>
        futex_wait(&self.state, PARKED, Some(timeout));
        // This is not just a store, because we need to establish a release-acquire ordering with unpark(). <br>这不仅仅是一个存储，因为我们需要用 `unpark()` 建立一个 `释放 - 获取` 的顺序。<br>
        //
        if self.state.swap(EMPTY, Acquire) == NOTIFIED {
            // Woke up because of unpark(). <br>因为 `unpark()` 而醒来。<br>
        } else {
            // Timeout or spurious wake up. <br>超时或虚假唤醒。<br>
            // We return either way, because we can't easily tell if it was the timeout or not. <br>我们以任何一种方式返回，因为我们无法轻易分辨出是否超时。<br>
            //
        }
    }

    // This implementation doesn't require `Pin`, but other implementations do. <br>此实现不需要 `Pin`，但其他实现需要。<br>
    #[inline]
    pub fn unpark(self: Pin<&Self>) {
        // Change PARKED=>NOTIFIED, EMPTY=>NOTIFIED, or NOTIFIED=>NOTIFIED, and wake the thread in the first case. <br>更改 PARKED => NOTIFIED，EMPTY => NOTIFIED 或 NOTIFIED => NOTIFIED，并在第一种情况下唤醒线程。<br>
        //
        //
        // Note that even NOTIFIED=>NOTIFIED results in a write. <br>请注意，即使 NOTIFIED => NOTIFIED 也会导致写入。<br>
        // This is on purpose, to make sure every unpark() has a release-acquire ordering with park(). <br>这是有目的的，以确保每个 unpark() 都具有对 park() 的发布 - 获取命令。<br>
        //
        if self.state.swap(NOTIFIED, Release) == PARKED {
            futex_wake(&self.state);
        }
    }
}
