//! Parker implementation based on a Mutex and Condvar. <br>基于 Mutex 和 Condvar 的 Parker 实现。<br>

use crate::pin::Pin;
use crate::sync::atomic::AtomicUsize;
use crate::sync::atomic::Ordering::SeqCst;
use crate::sync::{Condvar, Mutex};
use crate::time::Duration;

const EMPTY: usize = 0;
const PARKED: usize = 1;
const NOTIFIED: usize = 2;

pub struct Parker {
    state: AtomicUsize,
    lock: Mutex<()>,
    cvar: Condvar,
}

impl Parker {
    /// Construct the generic parker. <br>构建泛型 parker。<br>
    /// The UNIX parker implementation requires this to happen in-place. <br>UNIX parker 实现要求这发生在原地。<br>
    pub unsafe fn new(parker: *mut Parker) {
        parker.write(Parker {
            state: AtomicUsize::new(EMPTY),
            lock: Mutex::new(()),
            cvar: Condvar::new(),
        });
    }

    // This implementation doesn't require `unsafe` and `Pin`, but other implementations do. <br>此实现不需要 `unsafe` 和 `Pin`，但其他实现需要。<br>
    pub unsafe fn park(self: Pin<&Self>) {
        // If we were previously notified then we consume this notification and return quickly. <br>如果我们之前收到通知，那么我们将消耗此通知并迅速返回。<br>
        //
        if self.state.compare_exchange(NOTIFIED, EMPTY, SeqCst, SeqCst).is_ok() {
            return;
        }

        // Otherwise we need to coordinate going to sleep <br>否则我们需要协调睡眠<br>
        let mut m = self.lock.lock().unwrap();
        match self.state.compare_exchange(EMPTY, PARKED, SeqCst, SeqCst) {
            Ok(_) => {}
            Err(NOTIFIED) => {
                // We must read here, even though we know it will be `NOTIFIED`. <br>即使我们知道它将是 `NOTIFIED`，也必须在这里读取。<br>
                // This is because `unpark` may have been called again since we read `NOTIFIED` in the `compare_exchange` above. <br>这是因为自从我们在上面的 `compare_exchange` 中读取 `NOTIFIED` 以来，可能已经再次调用 `unpark`。<br>
                // We must perform an acquire operation that synchronizes with that `unpark` to observe any writes it made before the call to unpark. <br>我们必须执行与该 `unpark` 同步的获取操作，以观察它在取消驻留之前进行的所有写操作。<br>
                //
                // To do that we must read from the write it made to `state`. <br>为此，我们必须从对 `state` 的写入中读取。<br>
                //
                let old = self.state.swap(EMPTY, SeqCst);
                assert_eq!(old, NOTIFIED, "park state changed unexpectedly");
                return;
            } // should consume this notification, so prohibit spurious wakeups in next park. <br>应该消费这个通知，因此禁止在下一个 park 中进行虚假唤醒。<br>
            Err(_) => panic!("inconsistent park state"),
        }
        loop {
            m = self.cvar.wait(m).unwrap();
            match self.state.compare_exchange(NOTIFIED, EMPTY, SeqCst, SeqCst) {
                Ok(_) => return, // got a notification <br>收到通知<br>
                Err(_) => {}     // spurious wakeup, go back to sleep <br>虚假唤醒，回去睡觉<br>
            }
        }
    }

    // This implementation doesn't require `unsafe` and `Pin`, but other implementations do. <br>此实现不需要 `unsafe` 和 `Pin`，但其他实现需要。<br>
    pub unsafe fn park_timeout(self: Pin<&Self>, dur: Duration) {
        // Like `park` above we have a fast path for an already-notified thread, and afterwards we start coordinating for a sleep. <br>像上面的 `park` 一样，我们为已通知的线程提供了一条快速路径，然后我们开始协调睡眠。<br>
        //
        // return quickly. <br>快速返回。<br>
        if self.state.compare_exchange(NOTIFIED, EMPTY, SeqCst, SeqCst).is_ok() {
            return;
        }
        let m = self.lock.lock().unwrap();
        match self.state.compare_exchange(EMPTY, PARKED, SeqCst, SeqCst) {
            Ok(_) => {}
            Err(NOTIFIED) => {
                // We must read again here, see `park`. <br>我们必须在这里再次读取，请参见 `park`。<br>
                let old = self.state.swap(EMPTY, SeqCst);
                assert_eq!(old, NOTIFIED, "park state changed unexpectedly");
                return;
            } // should consume this notification, so prohibit spurious wakeups in next park. <br>应该消费这个通知，因此禁止在下一个 park 中进行虚假唤醒。<br>
            Err(_) => panic!("inconsistent park_timeout state"),
        }

        // Wait with a timeout, and if we spuriously wake up or otherwise wake up from a notification we just want to unconditionally set the state back to empty, either consuming a notification or un-flagging ourselves as parked. <br>等待超时，如果我们虚假地唤醒或以其他方式从通知中醒来，我们只是想无条件地将状态设置为空，既可以使用通知，也可以不标记自己为停止状态。<br>
        //
        //
        //
        let (_m, _result) = self.cvar.wait_timeout(m, dur).unwrap();
        match self.state.swap(EMPTY, SeqCst) {
            NOTIFIED => {} // got a notification, hurray! <br>收到通知，欢呼！<br>
            PARKED => {}   // no notification, alas <br>没有通知，alas<br>
            n => panic!("inconsistent park_timeout state: {n}"),
        }
    }

    // This implementation doesn't require `Pin`, but other implementations do. <br>此实现不需要 `Pin`，但其他实现需要。<br>
    pub fn unpark(self: Pin<&Self>) {
        // To ensure the unparked thread will observe any writes we made before this call, we must perform a release operation that `park` can synchronize with. <br>为了确保未驻留线程可以观察到我们在此调用之前所做的任何写操作，我们必须执行 `park` 可以与之同步的释放操作。<br>
        // To do that we must write `NOTIFIED` even if `state` is already `NOTIFIED`. <br>为此，即使 `state` 已经是 `NOTIFIED`，我们也必须编写 `NOTIFIED`。<br>
        // That is why this must be a swap rather than a compare-and-swap that returns if it reads `NOTIFIED` on failure. <br>这就是为什么这必须是交换操作，而不是如果失败读取 `NOTIFIED` 时返回的比较交换。<br>
        //
        //
        //
        match self.state.swap(NOTIFIED, SeqCst) {
            EMPTY => return,    // no one was waiting <br>没有人在等<br>
            NOTIFIED => return, // already unparked <br>已经停止<br>
            PARKED => {}        // gotta go wake someone up <br>要去叫醒某人<br>
            _ => panic!("inconsistent state in unpark"),
        }

        // There is a period between when the parked thread sets `state` to `PARKED` (or last checked `state` in the case of a spurious wake up) and when it actually waits on `cvar`. <br>从暂停的线程将 `state` 设置为 `PARKED` (或在虚假唤醒的情况下最后检查的 `state`) 到它实际等待 `cvar` 之间存在一段时间。<br>
        // If we were to notify during this period it would be ignored and then when the parked thread went to sleep it would never wake up. <br>如果我们在此期间进行通知，它将被忽略，然后当驻留线程进入睡眠状态时，它将永远不会醒来。<br>
        // Fortunately, it has `lock` locked at this stage so we can acquire `lock` to wait until it is ready to receive the notification. <br>幸运的是，它在此阶段已锁定 `lock`，因此我们可以获取 `lock` 以等待它准备好接收通知。<br>
        //
        // Releasing `lock` before the call to `notify_one` means that when the parked thread wakes it doesn't get woken only to have to wait for us to release `lock`. <br>在调用 `notify_one` 之前释放 `lock` 意味着，当驻留线程唤醒时，不必等待我们释放 `lock` 即可唤醒它。<br>
        //
        //
        //
        //
        //
        //
        drop(self.lock.lock().unwrap());
        self.cvar.notify_one()
    }
}
