//! Thread local storage <br>线程本地存储<br>

#![unstable(feature = "thread_local_internals", issue = "none")]

#[cfg(all(test, not(target_os = "emscripten")))]
mod tests;

#[cfg(test)]
mod dynamic_tests;

use crate::cell::{Cell, RefCell};
use crate::error::Error;
use crate::fmt;

/// A thread local storage key which owns its contents. <br>拥有其内容的线程本地存储密钥。<br>
///
/// This key uses the fastest possible implementation available to it for the target platform. <br>该密钥使用可用于目标平台的最快速度的实现。<br> It is instantiated with the [`thread_local!`] macro and the primary method is the [`with`] method. <br>它用 [`thread_local!`] 宏实例化，主要方法是 [`with`] 方法。<br>
///
/// The [`with`] method yields a reference to the contained value which cannot be sent across threads or escape the given closure. <br>[`with`] 方法产生对所包含值的引用，该引用不能跨线程发送或转义给定的闭包。<br>
///
/// [`thread_local!`]: crate::thread_local
///
/// # Initialization and Destruction <br>初始化与销毁<br>
///
/// Initialization is dynamically performed on the first call to [`with`] within a thread, and values that implement [`Drop`] get destructed when a thread exits. <br>初始化是在线程中对 [`with`] 的第一次调用中动态执行的，并且当线程退出时，实现 [`Drop`] 的值将被销毁。<br> Some caveats apply, which are explained below. <br>一些注意事项适用，下面将进行说明。<br>
///
/// A `LocalKey`'s initializer cannot recursively depend on itself, and using a `LocalKey` in this way will cause the initializer to infinitely recurse on the first call to `with`. <br>LocalKey 的初始化不能递归地依赖于它自己，以这种方式使用 `LocalKey` 会使初始化在第一次调用 `with` 时无限递归。<br>
///
/// # Examples
///
/// ```
/// use std::cell::RefCell;
/// use std::thread;
///
/// thread_local!(static FOO: RefCell<u32> = RefCell::new(1));
///
/// FOO.with(|f| {
///     assert_eq!(*f.borrow(), 1);
///     *f.borrow_mut() = 2;
/// });
///
/// // each thread starts out with the initial value of 1 <br>每个线程以 1 的初始值开始<br>
/// let t = thread::spawn(move|| {
///     FOO.with(|f| {
///         assert_eq!(*f.borrow(), 1);
///         *f.borrow_mut() = 3;
///     });
/// });
///
/// // wait for the thread to complete and bail out on panic <br>等待线程完成并在 panic 上退出<br>
/// t.join().unwrap();
///
/// // we retain our original value of 2 despite the child thread <br>尽管有子线程，我们仍保留原始值 2<br>
/// FOO.with(|f| {
///     assert_eq!(*f.borrow(), 2);
/// });
/// ```
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// Note that a "best effort" is made to ensure that destructors for types stored in thread local storage are run, but not all platforms can guarantee that destructors will be run for all types in thread local storage. <br>请注意，我们尽了 "最大努力" 来确保运行线程本地存储中存储的类型的析构函数，但并非所有平台都能保证运行线程本地存储中所有类型的析构函数。<br>
///
/// For example, there are a number of known caveats where destructors are not run: <br>例如，有许多已知的警告未运行析构函数：<br>
///
/// 1. On Unix systems when pthread-based TLS is being used, destructors will not be run for TLS values on the main thread when it exits. <br>在 Unix 系统上，当使用基于 pthread 的 TLS 时，主线程退出时，不会为主线程上的 TLS 值运行析构函数。<br>
/// Note that the application will exit immediately after the main thread exits as well. <br>请注意，应用程序也将在主线程退出后立即退出。<br>
/// 2. On all platforms it's possible for TLS to re-initialize other TLS slots during destruction. <br>在所有平台上，TLS 都有可能在销毁期间重新初始化其他 TLS 插槽。<br>
/// Some platforms ensure that this cannot happen infinitely by preventing re-initialization of any slot that has been destroyed, but not all platforms have this guard. <br>某些平台通过防止重新初始化已销毁的任何插槽来确保不会无限发生这种情况，但并非所有平台都具有此保护措施。<br>
/// Those platforms that do not guard typically have a synthetic limit after which point no more destructors are run. <br>那些不受约束的平台通常具有综合限制，在此之后，将不再运行析构函数。<br>
/// 3. When the process exits on Windows systems, TLS destructors may only be run on the thread that causes the process to exit. <br>当进程在 Windows 系统上退出时，TLS 析构函数可能只在导致进程退出的线程上运行。<br> This is because the other threads may be forcibly terminated. <br>这是因为其他线程可能会被强制终止。<br>
///
/// ## Synchronization in thread-local destructors <br>线程本地析构函数中的同步<br>
///
/// On Windows, synchronization operations (such as [`JoinHandle::join`]) in thread local destructors are prone to deadlocks and so should be avoided. <br>在 Windows 上，线程本地析构函数中的同步操作 (例如 [`JoinHandle::join`]) 容易出现死锁，因此应该避免。<br>
/// This is because the [loader lock] is held while a destructor is run. <br>这是因为 [loader lock] 在运行析构函数时，仍被持有。<br> The lock is acquired whenever a thread starts or exits or when a DLL is loaded or unloaded. <br>每当线程启动、退出、加载或卸载 DLL 时，都会获取锁。<br>
/// Therefore these events are blocked for as long as a thread local destructor is running. <br>所以，只要线程本地析构函数正在运行，这些事件就会被阻止。<br>
///
/// [loader lock]: https://docs.microsoft.com/en-us/windows/win32/dlls/dynamic-link-library-best-practices
/// [`JoinHandle::join`]: crate::thread::JoinHandle::join
/// [`with`]: LocalKey::with
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct LocalKey<T: 'static> {
    // This outer `LocalKey<T>` type is what's going to be stored in statics, but actual data inside will sometimes be tagged with #[thread_local]. <br>这种外部 `LocalKey<T>` 类型将存储在静态变量中，但是内部的实际数据有时会用 #[thread_local] 标记。<br>
    // It's not valid for a true static to reference a #[thread_local] static, so we get around that by exposing an accessor through a layer of function indirection (this thunk). <br>引用 #[thread_local] 静态变量对真实的静态变量是无效的，因此我们通过在函数间接层 (此重击) 中公开一个访问器来解决此问题。<br>
    //
    // Note that the thunk is itself unsafe because the returned lifetime of the slot where data lives, `'static`, is not actually valid. <br>请注意，该重击本身是不安全的，因为返回的数据所在的插槽 `'static` 的生命周期实际上是无效的。<br>
    // The lifetime here is actually slightly shorter than the currently running thread! <br>此处的生命周期实际上比当前正在运行的线程短一些！<br>
    //
    // Although this is an extra layer of indirection, it should in theory be trivially devirtualizable by LLVM because the value of `inner` never changes and the constant should be readonly within a crate. <br>尽管这是一个间接的额外层，但是从理论上讲，LLVM 应该可以对其进行虚拟化，因为 `inner` 的值永远不会更改，并且常量在 crate 内应为只读。<br>
    //
    // This mainly only runs into problems when TLS statics are exported across crates. <br>仅当在 crates 上导出 TLS 静态信息时，这才主要遇到问题。<br>
    //
    //
    //
    //
    //
    inner: unsafe fn(Option<&mut Option<T>>) -> Option<&'static T>,
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl<T: 'static> fmt::Debug for LocalKey<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("LocalKey").finish_non_exhaustive()
    }
}

/// Declare a new thread local storage key of type [`std::thread::LocalKey`]. <br>声明一个新的 [`std::thread::LocalKey`] 类型的线程本地存储密钥。<br>
///
/// # Syntax
///
/// The macro wraps any number of static declarations and makes them thread local. <br>宏可以包装任意数量的静态声明，并使它们成为局部线程。<br>
///
/// Publicity and attributes for each static are allowed. <br>允许每个静态的公开和属性。<br> Example:
///
/// ```
/// use std::cell::RefCell;
/// thread_local! {
///     pub static FOO: RefCell<u32> = RefCell::new(1);
///
///     #[allow(unused)]
///     static BAR: RefCell<f32> = RefCell::new(1.0);
/// }
/// # fn main() {}
/// ```
///
/// See [`LocalKey` documentation][`std::thread::LocalKey`] for more information. <br>有关更多信息，请参见 [`LocalKey` 文档][`std::thread::LocalKey`]。<br>
///
/// [`std::thread::LocalKey`]: crate::thread::LocalKey
#[macro_export]
#[stable(feature = "rust1", since = "1.0.0")]
#[cfg_attr(not(test), rustc_diagnostic_item = "thread_local_macro")]
#[allow_internal_unstable(thread_local_internals)]
macro_rules! thread_local {
    // empty (base case for the recursion) <br>空 (递归的基本情况)<br>
    () => {};

    ($(#[$attr:meta])* $vis:vis static $name:ident: $t:ty = const { $init:expr }; $($rest:tt)*) => (
        $crate::__thread_local_inner!($(#[$attr])* $vis $name, $t, const $init);
        $crate::thread_local!($($rest)*);
    );

    ($(#[$attr:meta])* $vis:vis static $name:ident: $t:ty = const { $init:expr }) => (
        $crate::__thread_local_inner!($(#[$attr])* $vis $name, $t, const $init);
    );

    // process multiple declarations <br>处理多个声明<br>
    ($(#[$attr:meta])* $vis:vis static $name:ident: $t:ty = $init:expr; $($rest:tt)*) => (
        $crate::__thread_local_inner!($(#[$attr])* $vis $name, $t, $init);
        $crate::thread_local!($($rest)*);
    );

    // handle a single declaration <br>处理一个声明<br>
    ($(#[$attr:meta])* $vis:vis static $name:ident: $t:ty = $init:expr) => (
        $crate::__thread_local_inner!($(#[$attr])* $vis $name, $t, $init);
    );
}

#[doc(hidden)]
#[unstable(feature = "thread_local_internals", reason = "should not be necessary", issue = "none")]
#[macro_export]
#[allow_internal_unstable(thread_local_internals, cfg_target_thread_local, thread_local)]
#[allow_internal_unsafe]
macro_rules! __thread_local_inner {
    // used to generate the `LocalKey` value for const-initialized thread locals <br>用于为常量初始化的线程局部变量生成 `LocalKey` 值<br>
    (@key $t:ty, const $init:expr) => {{
        #[cfg_attr(not(windows), inline)] // see comments below <br>请参见下面的注释<br>
        #[deny(unsafe_op_in_unsafe_fn)]
        unsafe fn __getit(
            _init: $crate::option::Option<&mut $crate::option::Option<$t>>,
        ) -> $crate::option::Option<&'static $t> {
            const INIT_EXPR: $t = $init;

            // wasm without atomics maps directly to `static mut`, and dtors aren't implemented because thread dtors aren't really a thing on wasm right now <br>没有原子的 wasm 直接映射到 `static mut`，并且没有实现 dtor，因为线程 dtor 现在在 wasm 上还不是一个真正的东西<br>
            //
            //
            // FIXME(#84224) this should come after the `target_thread_local` block. <br>这应该在 `target_thread_local` 块之后。<br>
            //
            //
            #[cfg(all(target_family = "wasm", not(target_feature = "atomics")))]
            {
                static mut VAL: $t = INIT_EXPR;
                unsafe { $crate::option::Option::Some(&VAL) }
            }

            // If the platform has support for `#[thread_local]`, use it. <br>如果平台支持 `#[thread_local]`，请使用它。<br>
            #[cfg(all(
                target_thread_local,
                not(all(target_family = "wasm", not(target_feature = "atomics"))),
            ))]
            {
                #[thread_local]
                static mut VAL: $t = INIT_EXPR;

                // If a dtor isn't needed we can do something "very raw" and just get going. <br>如果不需要 dtor，我们可以做一些 "非常原始" 的事情，然后开始。<br>
                //
                if !$crate::mem::needs_drop::<$t>() {
                    unsafe {
                        return $crate::option::Option::Some(&VAL)
                    }
                }

                // 0 == dtor not registered <br>0 == dtor 未注册<br>
                // 1 == dtor registered, dtor not run <br>1 == dtor 已注册，dtor 未运行<br>
                // 2 == dtor registered and is running or has run <br>2 == dtor 已注册并正在运行或已运行<br>
                #[thread_local]
                static mut STATE: $crate::primitive::u8 = 0;

                unsafe extern "C" fn destroy(ptr: *mut $crate::primitive::u8) {
                    let ptr = ptr as *mut $t;

                    unsafe {
                        $crate::debug_assert_eq!(STATE, 1);
                        STATE = 2;
                        $crate::ptr::drop_in_place(ptr);
                    }
                }

                unsafe {
                    match STATE {
                        // 0 == we haven't registered a destructor, so do so now. <br>0 == 我们还没有注册析构函数，所以现在注册。<br>
                        //
                        0 => {
                            $crate::thread::__FastLocalKeyInner::<$t>::register_dtor(
                                $crate::ptr::addr_of_mut!(VAL) as *mut $crate::primitive::u8,
                                destroy,
                            );
                            STATE = 1;
                            $crate::option::Option::Some(&VAL)
                        }
                        // 1 == the destructor is registered and the value is valid, so return the pointer. <br>1 == 析构函数已注册且值有效，因此返回指针。<br>
                        //
                        1 => $crate::option::Option::Some(&VAL),
                        // otherwise the destructor has already run, so we can't give access. <br>否则析构函数已经运行，所以我们不能授予访问权限。<br>
                        //
                        _ => $crate::option::Option::None,
                    }
                }
            }

            // On platforms without `#[thread_local]` we fall back to the same implementation as below for os thread locals. <br>在没有 `#[thread_local]` 的平台上，我们回退到与下面的 os 线程局部变量相同的实现。<br>
            //
            #[cfg(all(
                not(target_thread_local),
                not(all(target_family = "wasm", not(target_feature = "atomics"))),
            ))]
            {
                #[inline]
                const fn __init() -> $t { INIT_EXPR }
                static __KEY: $crate::thread::__OsLocalKeyInner<$t> =
                    $crate::thread::__OsLocalKeyInner::new();
                #[allow(unused_unsafe)]
                unsafe {
                    __KEY.get(move || {
                        if let $crate::option::Option::Some(init) = _init {
                            if let $crate::option::Option::Some(value) = init.take() {
                                return value;
                            } else if $crate::cfg!(debug_assertions) {
                                $crate::unreachable!("missing initial value");
                            }
                        }
                        __init()
                    })
                }
            }
        }

        unsafe {
            $crate::thread::LocalKey::new(__getit)
        }
    }};

    // used to generate the `LocalKey` value for `thread_local!` <br>用于为 `thread_local!` 生成 `LocalKey` 值<br>
    (@key $t:ty, $init:expr) => {
        {
            #[inline]
            fn __init() -> $t { $init }

            // When reading this function you might ask "why is this inlined everywhere other than Windows?", and that's a very reasonable question to ask. <br>在阅读这个函数时，您可能会问 "为什么除了 Windows 之外，其他地方都内联这个函数?"，这是一个非常合理的问题。<br>
            // The short story is that it segfaults rustc if this function is inlined. <br>简而言之，如果这个函数是内联的，它会导致 rustc 段错误。<br>
            // The longer story is that Windows looks to not support `extern` references to thread locals across DLL boundaries. <br>更长的故事是 Windows 看起来不支持跨 DLL 边界的线程局部变量的 `extern` 引用。<br>
            // This appears to at least not be supported in the ABI that LLVM implements. <br>这似乎至少在 LLVM 实现的 ABI 中不受支持。<br>
            //
            // Because of this we never inline on Windows, but we do inline on other platforms (where external references to thread locals across DLLs are supported). <br>正因为如此，我们从不在 Windows 上内联，但我们在其他平台上内联 (在这些平台上，跨 dll 的线程局部变量的外部引用是受支持的)。<br>
            // A better fix for this would be to inline this function on Windows, but only for "statically linked" components. <br>一个更好的解决方法是在 Windows 上内联这个函数，但仅限于 "静态链接" 组件。<br>
            // For example if two separately compiled rlibs end up getting linked into a DLL then it's fine to inline this function across that boundary. <br>例如，如果两个单独编译的 rlib 最终链接到一个 DLL 中，那么可以跨边界内联这个函数。<br>
            // It's only not fine to inline this function across a DLL boundary. <br>跨 DLL 边界内联这个函数是不好的。<br>
            // Unfortunately rustc doesn't currently have this sort of logic available in an attribute, and it's not clear that rustc is even equipped to answer this (it's more of a Cargo question kinda). <br>不幸的是，rustc 目前在属性中没有这种可用的逻辑，而且还不清楚 rustc 是否有能力回答这个问题 (这更像是 Cargo 问题)。<br>
            //
            // This means that, unfortunately, Windows gets the pessimistic path for now where it's never inlined. <br>这意味着，不幸的是，Windows 现在的路径很悲观，它从来没有内联。<br>
            //
            // The issue of "should enable on Windows sometimes" is #84933 <br>"有时应该在 Windows 上启用" 的问题是 #84933<br>
            //
            //
            //
            //
            //
            //
            //
            //
            #[cfg_attr(not(windows), inline)]
            unsafe fn __getit(
                init: $crate::option::Option<&mut $crate::option::Option<$t>>,
            ) -> $crate::option::Option<&'static $t> {
                #[cfg(all(target_family = "wasm", not(target_feature = "atomics")))]
                static __KEY: $crate::thread::__StaticLocalKeyInner<$t> =
                    $crate::thread::__StaticLocalKeyInner::new();

                #[thread_local]
                #[cfg(all(
                    target_thread_local,
                    not(all(target_family = "wasm", not(target_feature = "atomics"))),
                ))]
                static __KEY: $crate::thread::__FastLocalKeyInner<$t> =
                    $crate::thread::__FastLocalKeyInner::new();

                #[cfg(all(
                    not(target_thread_local),
                    not(all(target_family = "wasm", not(target_feature = "atomics"))),
                ))]
                static __KEY: $crate::thread::__OsLocalKeyInner<$t> =
                    $crate::thread::__OsLocalKeyInner::new();

                // FIXME: remove the #[allow(...)] marker when macros don't raise warning for missing/extraneous unsafe blocks anymore. <br>当宏不再对丢失/额外不安全块发出警告时，请删除 #[allow(...)] 标记。<br>
                // See https://github.com/rust-lang/rust/issues/74838.
                //
                #[allow(unused_unsafe)]
                unsafe {
                    __KEY.get(move || {
                        if let $crate::option::Option::Some(init) = init {
                            if let $crate::option::Option::Some(value) = init.take() {
                                return value;
                            } else if $crate::cfg!(debug_assertions) {
                                $crate::unreachable!("missing default value");
                            }
                        }
                        __init()
                    })
                }
            }

            unsafe {
                $crate::thread::LocalKey::new(__getit)
            }
        }
    };
    ($(#[$attr:meta])* $vis:vis $name:ident, $t:ty, $($init:tt)*) => {
        $(#[$attr])* $vis const $name: $crate::thread::LocalKey<$t> =
            $crate::__thread_local_inner!(@key $t, $($init)*);
    }
}

/// An error returned by [`LocalKey::try_with`](struct.LocalKey.html#method.try_with). <br>[`LocalKey::try_with`](struct.LocalKey.html#method.try_with) 返回的错误。<br>
#[stable(feature = "thread_local_try_with", since = "1.26.0")]
#[non_exhaustive]
#[derive(Clone, Copy, Eq, PartialEq)]
pub struct AccessError;

#[stable(feature = "thread_local_try_with", since = "1.26.0")]
impl fmt::Debug for AccessError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("AccessError").finish()
    }
}

#[stable(feature = "thread_local_try_with", since = "1.26.0")]
impl fmt::Display for AccessError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Display::fmt("already destroyed", f)
    }
}

#[stable(feature = "thread_local_try_with", since = "1.26.0")]
impl Error for AccessError {}

impl<T: 'static> LocalKey<T> {
    #[doc(hidden)]
    #[unstable(
        feature = "thread_local_internals",
        reason = "recently added to create a key",
        issue = "none"
    )]
    #[rustc_const_unstable(feature = "thread_local_internals", issue = "none")]
    pub const unsafe fn new(
        inner: unsafe fn(Option<&mut Option<T>>) -> Option<&'static T>,
    ) -> LocalKey<T> {
        LocalKey { inner }
    }

    /// Acquires a reference to the value in this TLS key. <br>获取对此 TLS 密钥中的值的引用。<br>
    ///
    /// This will lazily initialize the value if this thread has not referenced this key yet. <br>如果此线程尚未引用此键，则将延迟地初始化该值。<br>
    ///
    /// # Panics
    ///
    /// This function will `panic!()` if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果该键当前正在运行其析构函数，则此函数将为 `panic!()`; 如果先前已为此线程运行了析构函数，则它可能 panic。<br>
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn with<F, R>(&'static self, f: F) -> R
    where
        F: FnOnce(&T) -> R,
    {
        self.try_with(f).expect(
            "cannot access a Thread Local Storage value \
             during or after destruction",
        )
    }

    /// Acquires a reference to the value in this TLS key. <br>获取对此 TLS 密钥中的值的引用。<br>
    ///
    /// This will lazily initialize the value if this thread has not referenced this key yet. <br>如果此线程尚未引用此键，则将延迟地初始化该值。<br>
    /// If the key has been destroyed (which may happen if this is called in a destructor), this function will return an [`AccessError`]. <br>如果密钥已被销毁 (如果在析构函数中调用它可能会发生这种情况)，此函数将返回 [`AccessError`]。<br>
    ///
    ///
    /// # Panics
    ///
    /// This function will still `panic!()` if the key is uninitialized and the key's initializer panics. <br>如果未初始化密钥并且密钥的初始化 panics，则此函数仍将 `panic!()`。<br>
    ///
    #[stable(feature = "thread_local_try_with", since = "1.26.0")]
    #[inline]
    pub fn try_with<F, R>(&'static self, f: F) -> Result<R, AccessError>
    where
        F: FnOnce(&T) -> R,
    {
        unsafe {
            let thread_local = (self.inner)(None).ok_or(AccessError)?;
            Ok(f(thread_local))
        }
    }

    /// Acquires a reference to the value in this TLS key, initializing it with `init` if it wasn't already initialized on this thread. <br>获取对此 TLS 密钥中的值的引用，如果尚未在此线程上初始化，则使用 `init` 对其进行初始化。<br>
    ///
    /// If `init` was used to initialize the thread local variable, `None` is passed as the first argument to `f`. <br>如果 `init` 用于初始化线程局部变量，则 `None` 作为第一个参数传递给 `f`。<br>
    /// If it was already initialized, `Some(init)` is passed to `f`. <br>如果它已经被初始化，`Some(init)` 被传递给 `f`。<br>
    ///
    /// # Panics
    ///
    /// This function will panic if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果键当前正在运行其析构函数，则此函数将 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    ///
    ///
    fn initialize_with<F, R>(&'static self, init: T, f: F) -> R
    where
        F: FnOnce(Option<T>, &T) -> R,
    {
        unsafe {
            let mut init = Some(init);
            let reference = (self.inner)(Some(&mut init)).expect(
                "cannot access a Thread Local Storage value \
                 during or after destruction",
            );
            f(init, reference)
        }
    }
}

impl<T: 'static> LocalKey<Cell<T>> {
    /// Sets or initializes the contained value. <br>设置或初始化包含的值。<br>
    ///
    /// Unlike the other methods, this will *not* run the lazy initializer of the thread local. <br>与其他方法不同，这将*不会*运行本地线程的惰性初始化。<br>
    /// Instead, it will be directly initialized with the given value if it wasn't initialized yet. <br>相反，如果它还没有初始化，它将直接用给定的值初始化。<br>
    ///
    /// # Panics
    ///
    /// Panics if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果密钥当前正在运行其析构函数，则发生 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(local_key_cell_methods)]
    /// use std::cell::Cell;
    ///
    /// thread_local! {
    ///     static X: Cell<i32> = panic!("!");
    /// }
    ///
    /// // Calling X.get() here would result in a panic. <br>在这里调用 X.get() 会导致 panic。<br>
    ///
    /// X.set(123); // But X.set() is fine, as it skips the initializer above. <br>但是 X.set() 很好，因为它跳过了上面的初始化。<br>
    ///
    /// assert_eq!(X.get(), 123);
    /// ```
    ///
    #[unstable(feature = "local_key_cell_methods", issue = "92122")]
    pub fn set(&'static self, value: T) {
        self.initialize_with(Cell::new(value), |value, cell| {
            if let Some(value) = value {
                // The cell was already initialized, so `value` wasn't used to initialize it. <br>cell 已经初始化，所以没有使用 `value` 来初始化它。<br>
                // So we overwrite the current value with the new one instead. <br>所以我们用新的值覆盖当前值。<br>
                //
                cell.set(value.into_inner());
            }
        });
    }

    /// Returns a copy of the contained value. <br>返回所包含值的副本。<br>
    ///
    /// This will lazily initialize the value if this thread has not referenced this key yet. <br>如果此线程尚未引用此键，则将延迟地初始化该值。<br>
    ///
    /// # Panics
    ///
    /// Panics if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果密钥当前正在运行其析构函数，则发生 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(local_key_cell_methods)]
    /// use std::cell::Cell;
    ///
    /// thread_local! {
    ///     static X: Cell<i32> = Cell::new(1);
    /// }
    ///
    /// assert_eq!(X.get(), 1);
    /// ```
    ///
    #[unstable(feature = "local_key_cell_methods", issue = "92122")]
    pub fn get(&'static self) -> T
    where
        T: Copy,
    {
        self.with(|cell| cell.get())
    }

    /// Takes the contained value, leaving `Default::default()` in its place. <br>获取包含的值，在其位置保留 `Default::default()`。<br>
    ///
    /// This will lazily initialize the value if this thread has not referenced this key yet. <br>如果此线程尚未引用此键，则将延迟地初始化该值。<br>
    ///
    /// # Panics
    ///
    /// Panics if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果密钥当前正在运行其析构函数，则发生 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(local_key_cell_methods)]
    /// use std::cell::Cell;
    ///
    /// thread_local! {
    ///     static X: Cell<Option<i32>> = Cell::new(Some(1));
    /// }
    ///
    /// assert_eq!(X.take(), Some(1));
    /// assert_eq!(X.take(), None);
    /// ```
    ///
    #[unstable(feature = "local_key_cell_methods", issue = "92122")]
    pub fn take(&'static self) -> T
    where
        T: Default,
    {
        self.with(|cell| cell.take())
    }

    /// Replaces the contained value, returning the old value. <br>替换包含的值，返回旧值。<br>
    ///
    /// This will lazily initialize the value if this thread has not referenced this key yet. <br>如果此线程尚未引用此键，则将延迟地初始化该值。<br>
    ///
    /// # Panics
    ///
    /// Panics if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果密钥当前正在运行其析构函数，则发生 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(local_key_cell_methods)]
    /// use std::cell::Cell;
    ///
    /// thread_local! {
    ///     static X: Cell<i32> = Cell::new(1);
    /// }
    ///
    /// assert_eq!(X.replace(2), 1);
    /// assert_eq!(X.replace(3), 2);
    /// ```
    ///
    #[unstable(feature = "local_key_cell_methods", issue = "92122")]
    pub fn replace(&'static self, value: T) -> T {
        self.with(|cell| cell.replace(value))
    }
}

impl<T: 'static> LocalKey<RefCell<T>> {
    /// Acquires a reference to the contained value. <br>获取对所包含值的引用。<br>
    ///
    /// This will lazily initialize the value if this thread has not referenced this key yet. <br>如果此线程尚未引用此键，则将延迟地初始化该值。<br>
    ///
    /// # Panics
    ///
    /// Panics if the value is currently mutably borrowed. <br>如果当前价值是可变借来的，就会出现 panic。<br>
    ///
    /// Panics if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果密钥当前正在运行其析构函数，则发生 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    /// # Example
    ///
    /// ```
    /// #![feature(local_key_cell_methods)]
    /// use std::cell::RefCell;
    ///
    /// thread_local! {
    ///     static X: RefCell<Vec<i32>> = RefCell::new(Vec::new());
    /// }
    ///
    /// X.with_borrow(|v| assert!(v.is_empty()));
    /// ```
    ///
    #[unstable(feature = "local_key_cell_methods", issue = "92122")]
    pub fn with_borrow<F, R>(&'static self, f: F) -> R
    where
        F: FnOnce(&T) -> R,
    {
        self.with(|cell| f(&cell.borrow()))
    }

    /// Acquires a mutable reference to the contained value. <br>获取对所包含值的可变引用。<br>
    ///
    /// This will lazily initialize the value if this thread has not referenced this key yet. <br>如果此线程尚未引用此键，则将延迟地初始化该值。<br>
    ///
    /// # Panics
    ///
    /// Panics if the value is currently borrowed. <br>如果当前的值是借来的，就会出现 panic。<br>
    ///
    /// Panics if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果密钥当前正在运行其析构函数，则发生 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    /// # Example
    ///
    /// ```
    /// #![feature(local_key_cell_methods)]
    /// use std::cell::RefCell;
    ///
    /// thread_local! {
    ///     static X: RefCell<Vec<i32>> = RefCell::new(Vec::new());
    /// }
    ///
    /// X.with_borrow_mut(|v| v.push(1));
    ///
    /// X.with_borrow(|v| assert_eq!(*v, vec![1]));
    /// ```
    ///
    #[unstable(feature = "local_key_cell_methods", issue = "92122")]
    pub fn with_borrow_mut<F, R>(&'static self, f: F) -> R
    where
        F: FnOnce(&mut T) -> R,
    {
        self.with(|cell| f(&mut cell.borrow_mut()))
    }

    /// Sets or initializes the contained value. <br>设置或初始化包含的值。<br>
    ///
    /// Unlike the other methods, this will *not* run the lazy initializer of the thread local. <br>与其他方法不同，这将*不会*运行本地线程的惰性初始化。<br>
    /// Instead, it will be directly initialized with the given value if it wasn't initialized yet. <br>相反，如果它还没有初始化，它将直接用给定的值初始化。<br>
    ///
    /// # Panics
    ///
    /// Panics if the value is currently borrowed. <br>如果当前的值是借来的，就会出现 panic。<br>
    ///
    /// Panics if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果密钥当前正在运行其析构函数，则发生 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(local_key_cell_methods)]
    /// use std::cell::RefCell;
    ///
    /// thread_local! {
    ///     static X: RefCell<Vec<i32>> = panic!("!");
    /// }
    ///
    /// // Calling X.with() here would result in a panic. <br>在这里调用 X.with() 会导致 panic。<br>
    ///
    /// X.set(vec![1, 2, 3]); // But X.set() is fine, as it skips the initializer above. <br>但是 X.set() 很好，因为它跳过了上面的初始化。<br>
    ///
    /// X.with_borrow(|v| assert_eq!(*v, vec![1, 2, 3]));
    /// ```
    ///
    #[unstable(feature = "local_key_cell_methods", issue = "92122")]
    pub fn set(&'static self, value: T) {
        self.initialize_with(RefCell::new(value), |value, cell| {
            if let Some(value) = value {
                // The cell was already initialized, so `value` wasn't used to initialize it. <br>cell 已经初始化，所以没有使用 `value` 来初始化它。<br>
                // So we overwrite the current value with the new one instead. <br>所以我们用新的值覆盖当前值。<br>
                //
                *cell.borrow_mut() = value.into_inner();
            }
        });
    }

    /// Takes the contained value, leaving `Default::default()` in its place. <br>获取包含的值，在其位置保留 `Default::default()`。<br>
    ///
    /// This will lazily initialize the value if this thread has not referenced this key yet. <br>如果此线程尚未引用此键，则将延迟地初始化该值。<br>
    ///
    /// # Panics
    ///
    /// Panics if the value is currently borrowed. <br>如果当前的值是借来的，就会出现 panic。<br>
    ///
    /// Panics if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果密钥当前正在运行其析构函数，则发生 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(local_key_cell_methods)]
    /// use std::cell::RefCell;
    ///
    /// thread_local! {
    ///     static X: RefCell<Vec<i32>> = RefCell::new(Vec::new());
    /// }
    ///
    /// X.with_borrow_mut(|v| v.push(1));
    ///
    /// let a = X.take();
    ///
    /// assert_eq!(a, vec![1]);
    ///
    /// X.with_borrow(|v| assert!(v.is_empty()));
    /// ```
    ///
    #[unstable(feature = "local_key_cell_methods", issue = "92122")]
    pub fn take(&'static self) -> T
    where
        T: Default,
    {
        self.with(|cell| cell.take())
    }

    /// Replaces the contained value, returning the old value. <br>替换包含的值，返回旧值。<br>
    ///
    /// # Panics
    ///
    /// Panics if the value is currently borrowed. <br>如果当前的值是借来的，就会出现 panic。<br>
    ///
    /// Panics if the key currently has its destructor running, and it **may** panic if the destructor has previously been run for this thread. <br>如果密钥当前正在运行其析构函数，则发生 panic，如果析构函数先前已为此线程运行，则它**可能**panic。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(local_key_cell_methods)]
    /// use std::cell::RefCell;
    ///
    /// thread_local! {
    ///     static X: RefCell<Vec<i32>> = RefCell::new(Vec::new());
    /// }
    ///
    /// let prev = X.replace(vec![1, 2, 3]);
    /// assert!(prev.is_empty());
    ///
    /// X.with_borrow(|v| assert_eq!(*v, vec![1, 2, 3]));
    /// ```
    #[unstable(feature = "local_key_cell_methods", issue = "92122")]
    pub fn replace(&'static self, value: T) -> T {
        self.with(|cell| cell.replace(value))
    }
}

mod lazy {
    use crate::cell::UnsafeCell;
    use crate::hint;
    use crate::mem;

    pub struct LazyKeyInner<T> {
        inner: UnsafeCell<Option<T>>,
    }

    impl<T> LazyKeyInner<T> {
        pub const fn new() -> LazyKeyInner<T> {
            LazyKeyInner { inner: UnsafeCell::new(None) }
        }

        pub unsafe fn get(&self) -> Option<&'static T> {
            // SAFETY: The caller must ensure no reference is ever handed out to the inner cell nor mutable reference to the Option<T> inside said cell. <br>调用者必须确保绝不向内部 cell 传递任何引用，也不要向所述 cell 内部的 Option<T> 进行可变引用。<br>
            // This make it safe to hand a reference, though the lifetime of 'static is itself unsafe, making the get method unsafe. <br>尽管'static 的生命周期本身是不安全的，这使 get 方法变得不安全，但这使引用变得安全。<br>
            //
            //
            unsafe { (*self.inner.get()).as_ref() }
        }

        /// The caller must ensure that no reference is active: this method needs unique access. <br>调用者必须确保没有激活引用：此方法需要唯一的访问权限。<br>
        ///
        pub unsafe fn initialize<F: FnOnce() -> T>(&self, init: F) -> &'static T {
            // Execute the initialization up front, *then* move it into our slot, just in case initialization fails. <br>预先执行初始化，然后将其移动到我们的插槽中，以防万一初始化失败。<br>
            //
            let value = init();
            let ptr = self.inner.get();

            // SAFETY:
            //
            // note that this can in theory just be `*ptr = Some(value)`, but due to the compiler will currently codegen that pattern with something like: <br>请注意，从理论上讲，它只能是 `*ptr = Some(value)`，但是由于编译器当前将使用类似以下内容的代码生成该模式：<br>
            //
            //      ptr::drop_in_place(ptr)
            //      ptr::write(ptr, Some(value))
            //
            // Due to this pattern it's possible for the destructor of the value in `ptr` (e.g., if this is being recursively initialized) to re-access TLS, in which case there will be a `&` and `&mut` pointer to the same value (an aliasing violation). <br>由于这种模式，`ptr` 中的值的析构函数有可能重新访问 TLS (例如，如果正在递归初始化)，在这种情况下，将有一个指向相同值的 `&` 和 `&mut` 指针 (违反别名)。<br>
            // To avoid setting the "I'm running a destructor" flag we just use `mem::replace` which should sequence the operations a little differently and make this safe to call. <br>为了避免设置 "我正在运行析构函数" 标志，我们只使用 `mem::replace`，它应该以稍微不同的方式对操作进行排序，并确保调用安全。<br>
            //
            // The precondition also ensures that we are the only one accessing `self` at the moment so replacing is fine. <br>前提条件还确保了我们是目前唯一访问 `self` 的计算机，因此可以很好地进行替换。<br>
            //
            //
            //
            //
            //
            //
            unsafe {
                let _ = mem::replace(&mut *ptr, Some(value));
            }

            // SAFETY: With the call to `mem::replace` it is guaranteed there is a `Some` behind `ptr`, not a `None` so `unreachable_unchecked` will never be reached. <br>通过调用 `mem::replace`，可以确保 `ptr` 后面有一个 `Some`，而不是 `None`，因此将永远无法到达 `unreachable_unchecked`。<br>
            //
            //
            unsafe {
                // After storing `Some` we want to get a reference to the contents of what we just stored. <br>存储 `Some` 之后，我们想对刚刚存储的内容进行引用。<br>
                // While we could use `unwrap` here and it should always work it empirically doesn't seem to always get optimized away, which means that using something like `try_with` can pull in panicking code and cause a large size bloat. <br>尽管我们可以在这里使用 `unwrap`，并且它应该始终有效，但是根据经验，似乎并没有总是对其进行优化，这意味着使用 `try_with` 之类的东西可能会引入 panic 代码并导致大规模的膨胀。<br>
                //
                //
                //
                match *ptr {
                    Some(ref x) => x,
                    None => hint::unreachable_unchecked(),
                }
            }
        }

        /// The other methods hand out references while taking &self. <br>其他方法在取 &self 时分发引用。<br>
        /// As such, callers of this method must ensure no `&` and `&mut` are available and used at the same time. <br>因此，此方法的调用者必须确保没有 `&` 和 `&mut` 同时可用和使用。<br>
        ///
        #[allow(unused)]
        pub unsafe fn take(&mut self) -> Option<T> {
            // SAFETY: See doc comment for this method. <br>有关此方法，请参见文档注释。<br>
            unsafe { (*self.inner.get()).take() }
        }
    }
}

/// On some targets like wasm there's no threads, so no need to generate thread locals and we can instead just use plain statics! <br>在像 wasm 这样的一些目标上没有线程，所以不需要生成线程局部变量，我们可以只使用普通的静态变量！<br>
///
#[doc(hidden)]
#[cfg(all(target_family = "wasm", not(target_feature = "atomics")))]
pub mod statik {
    use super::lazy::LazyKeyInner;
    use crate::fmt;

    pub struct Key<T> {
        inner: LazyKeyInner<T>,
    }

    unsafe impl<T> Sync for Key<T> {}

    impl<T> fmt::Debug for Key<T> {
        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
            f.debug_struct("Key").finish_non_exhaustive()
        }
    }

    impl<T> Key<T> {
        pub const fn new() -> Key<T> {
            Key { inner: LazyKeyInner::new() }
        }

        pub unsafe fn get(&self, init: impl FnOnce() -> T) -> Option<&'static T> {
            // SAFETY: The caller must ensure no reference is ever handed out to the inner cell nor mutable reference to the Option<T> inside said cell. <br>调用者必须确保绝不向内部 cell 传递任何引用，也不要向所述 cell 内部的 Option<T> 进行可变引用。<br>
            // This make it safe to hand a reference, though the lifetime of 'static is itself unsafe, making the get method unsafe. <br>尽管'static 的生命周期本身是不安全的，这使 get 方法变得不安全，但这使引用变得安全。<br>
            //
            //
            let value = unsafe {
                match self.inner.get() {
                    Some(ref value) => value,
                    None => self.inner.initialize(init),
                }
            };

            Some(value)
        }
    }
}

#[doc(hidden)]
#[cfg(target_thread_local)]
pub mod fast {
    use super::lazy::LazyKeyInner;
    use crate::cell::Cell;
    use crate::fmt;
    use crate::mem;
    use crate::sys::thread_local_dtor::register_dtor;

    #[derive(Copy, Clone)]
    enum DtorState {
        Unregistered,
        Registered,
        RunningOrHasRun,
    }

    // This data structure has been carefully constructed so that the fast path only contains one branch on x86. <br>此数据结构体经过精心构造，因此快速路径仅包含 x86 上的一个分支。<br>
    // That optimization is necessary to avoid duplicated tls lookups on OSX. <br>为了避免在 OSX 上重复查找 tls，必须进行优化。<br>
    //
    // LLVM issue: https://bugs.llvm.org/show_bug.cgi?id=41722
    //
    pub struct Key<T> {
        // If `LazyKeyInner::get` returns `None`, that indicates either: <br>如果 `LazyKeyInner::get` 返回 `None`，则表明：<br>
        //   * The value has never been initialized <br>该值从未初始化<br>
        //   * The value is being recursively initialized <br>该值正在递归初始化<br>
        //   * The value has already been destroyed or is being destroyed To determine which kind of `None`, check `dtor_state`. <br>该值已被销毁或正在被销毁。要确定哪种 `None`，请检查 `dtor_state`。<br>
        //
        //
        // This is very optimizer friendly for the fast path - initialized but not yet dropped. <br>这对快速路径非常优化，它非常友好 - 已初始化但尚未丢弃。<br>
        //
        inner: LazyKeyInner<T>,

        // Metadata to keep track of the state of the destructor. <br>跟踪析构函数状态的元数据。<br>
        // Remember that this variable is thread-local, not global. <br>请记住，此变量是线程局部的，而不是局部的。<br>
        dtor_state: Cell<DtorState>,
    }

    impl<T> fmt::Debug for Key<T> {
        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
            f.debug_struct("Key").finish_non_exhaustive()
        }
    }

    impl<T> Key<T> {
        pub const fn new() -> Key<T> {
            Key { inner: LazyKeyInner::new(), dtor_state: Cell::new(DtorState::Unregistered) }
        }

        // note that this is just a publicly-callable function only for the const-initialized form of thread locals, basically a way to call the free `register_dtor` function defined elsewhere in libstd. <br>请注意，这只是一个可公开调用的函数，仅适用于线程局部变量的常量初始化形式，基本上是调用 libstd 中其他地方定义的 free `register_dtor` 函数的一种方式。<br>
        //
        //
        pub unsafe fn register_dtor(a: *mut u8, dtor: unsafe extern "C" fn(*mut u8)) {
            unsafe {
                register_dtor(a, dtor);
            }
        }

        pub unsafe fn get<F: FnOnce() -> T>(&self, init: F) -> Option<&'static T> {
            // SAFETY: See the definitions of `LazyKeyInner::get` and `try_initialize` for more information. <br>有关详细信息，请参见 `LazyKeyInner::get` 和 `try_initialize` 的定义。<br>
            //
            // The caller must ensure no mutable references are ever active to the inner cell or the inner T when this is called. <br>调用者必须确保在调用此内部 cell 或内部 T 时永远不会对它起作用。<br>
            //
            // The `try_initialize` is dependant on the passed `init` function for this. <br>`try_initialize` 为此而依赖于传递的 `init` 函数。<br>
            //
            //
            unsafe {
                match self.inner.get() {
                    Some(val) => Some(val),
                    None => self.try_initialize(init),
                }
            }
        }

        // `try_initialize` is only called once per fast thread local variable, except in corner cases where thread_local dtors reference other thread_local's, or it is being recursively initialized. <br>每个快速线程局部变量只调用一次 `try_initialize`，除非在 thread_local dtor 引用其他 thread_local 的极端情况下，或者它正在被递归初始化。<br>
        //
        //
        // Macos: Inlining this function can cause two `tlv_get_addr` calls to be performed for every call to `Key::get`. <br>Macos: 内联这个函数会导致每个调用到 `Key::get` 时执行两次 `tlv_get_addr` 调用。<br>
        // LLVM issue: https://bugs.llvm.org/show_bug.cgi?id=41722
        //
        //
        #[inline(never)]
        unsafe fn try_initialize<F: FnOnce() -> T>(&self, init: F) -> Option<&'static T> {
            // SAFETY: See comment above (this function doc). <br>请参见上面的注释 (此函数文档)。<br>
            if !mem::needs_drop::<T>() || unsafe { self.try_register_dtor() } {
                // SAFETY: See comment above (this function doc). <br>请参见上面的注释 (此函数文档)。<br>
                Some(unsafe { self.inner.initialize(init) })
            } else {
                None
            }
        }

        // `try_register_dtor` is only called once per fast thread local variable, except in corner cases where thread_local dtors reference other thread_local's, or it is being recursively initialized. <br>每个快速线程局部变量只调用一次 `try_register_dtor`，除非在 thread_local dtor 引用其他 thread_local 的极端情况下，或者它正在被递归初始化。<br>
        //
        //
        unsafe fn try_register_dtor(&self) -> bool {
            match self.dtor_state.get() {
                DtorState::Unregistered => {
                    // SAFETY: dtor registration happens before initialization. <br>dtor 注册发生在初始化之前。<br>
                    // Passing `self` as a pointer while using `destroy_value<T>` is safe because the function will build a pointer to a Key<T>, which is the type of self and so find the correct size. <br>使用 `destroy_value<T>` 时将 `self` 作为指针传递是安全的，因为函数将建立指向 Key<T> 的指针，Key<T> 是 self 的类型，因此可以找到正确的大小。<br>
                    //
                    //
                    //
                    unsafe { register_dtor(self as *const _ as *mut u8, destroy_value::<T>) };
                    self.dtor_state.set(DtorState::Registered);
                    true
                }
                DtorState::Registered => {
                    // recursively initialized <br>递归初始化<br>
                    true
                }
                DtorState::RunningOrHasRun => false,
            }
        }
    }

    unsafe extern "C" fn destroy_value<T>(ptr: *mut u8) {
        let ptr = ptr as *mut Key<T>;

        // SAFETY:
        //
        // The pointer `ptr` has been built just above and comes from `try_register_dtor` where it is originally a Key<T> coming from `self`, making it non-NUL and of the correct type. <br>指针 `ptr` 刚好在上面构建，并且来自 `try_register_dtor`，它最初是 `self` 的 Key<T>，使其成为非 NUL 且类型正确。<br>
        //
        //
        // Right before we run the user destructor be sure to set the `Option<T>` to `None`, and `dtor_state` to `RunningOrHasRun`. <br>在运行用户析构函数之前，请确保将 `Option<T>` 设置为 `None`，将 `dtor_state` 设置为 `RunningOrHasRun`。<br>
        // This causes future calls to `get` to run `try_initialize_drop` again, which will now fail, and return `None`. <br>这将导致对 `get` 的 future 调用再次运行 `try_initialize_drop`，该操作现在将失败，并返回 `None`。<br>
        //
        //
        //
        unsafe {
            let value = (*ptr).inner.take();
            (*ptr).dtor_state.set(DtorState::RunningOrHasRun);
            drop(value);
        }
    }
}

#[doc(hidden)]
pub mod os {
    use super::lazy::LazyKeyInner;
    use crate::cell::Cell;
    use crate::fmt;
    use crate::marker;
    use crate::ptr;
    use crate::sys_common::thread_local_key::StaticKey as OsStaticKey;

    pub struct Key<T> {
        // OS-TLS key that we'll use to key off. <br>我们将用于注销的 OS-TLS 密钥。<br>
        os: OsStaticKey,
        marker: marker::PhantomData<Cell<T>>,
    }

    impl<T> fmt::Debug for Key<T> {
        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
            f.debug_struct("Key").finish_non_exhaustive()
        }
    }

    unsafe impl<T> Sync for Key<T> {}

    struct Value<T: 'static> {
        inner: LazyKeyInner<T>,
        key: &'static Key<T>,
    }

    impl<T: 'static> Key<T> {
        #[rustc_const_unstable(feature = "thread_local_internals", issue = "none")]
        pub const fn new() -> Key<T> {
            Key { os: OsStaticKey::new(Some(destroy_value::<T>)), marker: marker::PhantomData }
        }

        /// It is a requirement for the caller to ensure that no mutable reference is active when this method is called. <br>要求调用者确保在调用此方法时没有激活任何可变引用。<br>
        ///
        pub unsafe fn get(&'static self, init: impl FnOnce() -> T) -> Option<&'static T> {
            // SAFETY: See the documentation for this method. <br>请参见此方法的文档。<br>
            let ptr = unsafe { self.os.get() as *mut Value<T> };
            if ptr.addr() > 1 {
                // SAFETY: the check ensured the pointer is safe (its destructor is not running) + it is coming from a trusted source (self). <br>检查确保指针是安全的 (其析构函数未运行) + 它来自受信任的源 (self)。<br>
                //
                if let Some(ref value) = unsafe { (*ptr).inner.get() } {
                    return Some(value);
                }
            }
            // SAFETY: At this point we are sure we have no value and so initializing (or trying to) is safe. <br>在这一点上，我们确定我们没有任何值，因此初始化 (或尝试进行) 是安全的。<br>
            //
            unsafe { self.try_initialize(init) }
        }

        // `try_initialize` is only called once per os thread local variable, except in corner cases where thread_local dtors reference other thread_local's, or it is being recursively initialized. <br>`try_initialize` 每个 os 线程局部变量只调用一次，除非在 thread_local dtor 引用其他 thread_local 的极端情况下，或者它正在被递归初始化。<br>
        //
        //
        unsafe fn try_initialize(&'static self, init: impl FnOnce() -> T) -> Option<&'static T> {
            // SAFETY: No mutable references are ever handed out meaning getting the value is ok. <br>没有派发任何可变引用，这意味着获得值是可以的。<br>
            //
            let ptr = unsafe { self.os.get() as *mut Value<T> };
            if ptr.addr() == 1 {
                // destructor is running <br>析构函数正在运行<br>
                return None;
            }

            let ptr = if ptr.is_null() {
                // If the lookup returned null, we haven't initialized our own local copy, so do that now. <br>如果查找返回 null，则表明我们尚未初始化自己的本地副本，因此请立即进行初始化。<br>
                //
                let ptr: Box<Value<T>> = box Value { inner: LazyKeyInner::new(), key: self };
                let ptr = Box::into_raw(ptr);
                // SAFETY: At this point we are sure there is no value inside ptr so setting it will not affect anyone else. <br>在这一点上，我们确定 ptr 内没有任何值，因此设置它不会影响任何其他人。<br>
                //
                unsafe {
                    self.os.set(ptr as *mut u8);
                }
                ptr
            } else {
                // recursive initialization <br>递归初始化<br>
                ptr
            };

            // SAFETY: ptr has been ensured as non-NUL just above an so can be dereferenced safely. <br>已确保 ptr 为非 NUL 且正好位于 a 以上，因此可以安全地解引用。<br>
            //
            unsafe { Some((*ptr).inner.initialize(init)) }
        }
    }

    unsafe extern "C" fn destroy_value<T: 'static>(ptr: *mut u8) {
        // SAFETY:
        //
        // The OS TLS ensures that this key contains a null value when this destructor starts to run. <br>当这个析构函数开始运行时，OS TLS 确保这个键包含一个空值。<br>
        // We set it back to a sentinel value of 1 to ensure that any future calls to `get` for this thread will return `None`. <br>我们将其设置回哨兵值 1，以确保对该线程的 `get` 的任何 future 调用都将返回 `None`。<br>
        //
        //
        // Note that to prevent an infinite loop we reset it back to null right before we return from the destructor ourselves. <br>请注意，为防止无限循环，我们在自己从析构函数返回之前就将其重置为 null。<br>
        //
        //
        unsafe {
            let ptr = Box::from_raw(ptr as *mut Value<T>);
            let key = ptr.key;
            key.os.set(ptr::invalid_mut(1));
            drop(ptr);
            key.os.set(ptr::null_mut());
        }
    }
}
