//! Native threads. <br>原生线程。<br>
//!
//! ## The threading model <br>线程模型<br>
//!
//! An executing Rust program consists of a collection of native OS threads, each with their own stack and local state. <br>一个正在执行的 Rust 程序由一组原生操作系统线程组成​​，每个线程都有自己的栈和本地状态。<br> Threads can be named, and provide some built-in support for low-level synchronization. <br>线程可以被命名，并为底层同步提供一些内置支持。<br>
//!
//! Communication between threads can be done through [channels], Rust's message-passing types, along with [other forms of thread synchronization](../../std/sync/index.html) and shared-memory data structures. <br>线程之间的通信可以通过 [通道][channels]、Rust 的消息传递类型、以及 [其他形式的线程同步](../../std/sync/index.html) 和共享内存数据结构来完成。<br>
//! In particular, types that are guaranteed to be threadsafe are easily shared between threads using the atomically-reference-counted container, [`Arc`]. <br>特别是，可以使用原子引用计数容器 [`Arc`] 在线程之间轻松共享保证线程安全的类型。<br>
//!
//! Fatal logic errors in Rust cause *thread panic*, during which a thread will unwind the stack, running destructors and freeing owned resources. <br>Rust 中的致命逻辑错误导致 *线程 panic*，在此期间，线程将展开栈，运行析构函数并释放所拥有的资源。<br>
//! While not meant as a 'try/catch' mechanism, panics in Rust can nonetheless be caught (unless compiling with `panic=abort`) with [`catch_unwind`](../../std/panic/fn.catch_unwind.html) and recovered from, or alternatively be resumed with [`resume_unwind`](../../std/panic/fn.resume_unwind.html). <br>尽管不是 'try/catch' 机制，但仍可以使用 [`catch_unwind`](../../std/panic/fn.catch_unwind.html) 捕获 Rust 中的 panics (除非使用 `panic=abort` 进行编译) 并从中恢复，或者使用 [`resume_unwind`](../../std/panic/fn.resume_unwind.html) 恢复。<br>
//!
//! If the panic is not caught the thread will exit, but the panic may optionally be detected from a different thread with [`join`]. <br>如果未捕获到 panic，则线程将退出，但是可以选择使用 [`join`] 从其他线程中检测到 panic。<br>
//! If the main thread panics without the panic being caught, the application will exit with a non-zero exit code. <br>如果主线程 panic 而没有捕获 panic，应用程序将以非零退出码退出。<br>
//!
//! When the main thread of a Rust program terminates, the entire program shuts down, even if other threads are still running. <br>当 Rust 程序的主线程终止时，整个程序将关闭，即使其他线程仍在运行也不例外。<br> However, this module provides convenient facilities for automatically waiting for the termination of a thread (i.e., join). <br>然而，这个模块为自动等待线程终止 (即 join) 提供了便利。<br>
//!
//! ## Spawning a thread <br>生成一个线程<br>
//!
//! A new thread can be spawned using the [`thread::spawn`][`spawn`] function: <br>可以使用 [`thread::spawn`][`spawn`] 函数来生成一个新线程：<br>
//!
//! ```rust
//! use std::thread;
//!
//! thread::spawn(move || {
//!     // some work here <br>这里一些工作<br>
//! });
//! ```
//!
//! In this example, the spawned thread is "detached," which means that there is no way for the program to learn when the spawned thread completes or otherwise terminates. <br>在此示例中，新建线程是 "detached,"，这意味着程序无法了解新建线程何时完成或以其他方式终止。<br>
//!
//! To learn when a thread completes, it is necessary to capture the [`JoinHandle`] object that is returned by the call to [`spawn`], which provides a `join` method that allows the caller to wait for the completion of the spawned thread: <br>要了解线程何时完成，需要捕获调用返回给 [`spawn`] 的 [`JoinHandle`] 对象，它提供了一个 `join` 方法，允许调用者等待新建线程的完成：<br>
//!
//! ```rust
//! use std::thread;
//!
//! let thread_join_handle = thread::spawn(move || {
//!     // some work here <br>这里一些工作<br>
//! });
//! // some work here <br>这里一些工作<br>
//! let res = thread_join_handle.join();
//! ```
//!
//! The [`join`] method returns a [`thread::Result`] containing [`Ok`] of the final value produced by the spawned thread, or [`Err`] of the value given to a call to [`panic!`] if the thread panicked. <br>[`join`] 方法返回一个 [`thread::Result`]，其中包含由新建线程生成的最终值的 [`Ok`]，或者如果线程 panicked，则返回给 [`panic!`] 的调用值的 [`Err`]。<br>
//!
//! Note that there is no parent/child relationship between a thread that spawns a new thread and the thread being spawned. <br>请注意，生成新线程的线程与生成的线程之间没有 parent/child 关系。<br> In particular, the spawned thread may or may not outlive the spawning thread, unless the spawning thread is the main thread. <br>特别是，除非生成线程是主线程，否则新建线程可能会也可能不会比生成线程的生命周期长。<br>
//!
//! ## Configuring threads <br>配置线程<br>
//!
//! A new thread can be configured before it is spawned via the [`Builder`] type, which currently allows you to set the name and stack size for the thread: <br>一个新线程可以在通过 [`Builder`] 类型生成之前进行配置，目前允许您设置线程的名称和栈大小：<br>
//!
//! ```rust
//! # #![allow(unused_must_use)]
//! use std::thread;
//!
//! thread::Builder::new().name("thread1".to_string()).spawn(move || {
//!     println!("Hello, world!");
//! });
//! ```
//!
//! ## The `Thread` type <br>`Thread` 的类型<br>
//!
//! Threads are represented via the [`Thread`] type, which you can get in one of two ways: <br>线程是通过 [`Thread`] 类型来表示的，您可以通过以下两种方式之一获得该类型：<br>
//!
//! * By spawning a new thread, e.g., using the [`thread::spawn`][`spawn`] function, and calling [`thread`][`JoinHandle::thread`] on the [`JoinHandle`]. <br>通过生成一个新线程，例如使用 [`thread::spawn`][`spawn`] 函数，并在 [`JoinHandle`] 上调用 [`thread`][`JoinHandle::thread`]。<br>
//! * By requesting the current thread, using the [`thread::current`] function. <br>通过使用 [`thread::current`] 函数来请求当前线程。<br>
//!
//! The [`thread::current`] function is available even for threads not spawned by the APIs of this module. <br>[`thread::current`] 函数甚至可用于不是由该模块的 API 生成的线程。<br>
//!
//! ## Thread-local storage <br>线程本地存储<br>
//!
//! This module also provides an implementation of thread-local storage for Rust programs. <br>该模块还为 Rust 程序提供了线程本地存储的实现。<br> Thread-local storage is a method of storing data into a global variable that each thread in the program will have its own copy of. <br>线程本地存储是一种将数据存储到全局变量的方法，程序中的每个线程都有自己的副本。<br>
//! Threads do not share this data, so accesses do not need to be synchronized. <br>线程不共享此数据，因此不需要同步访问。<br>
//!
//! A thread-local key owns the value it contains and will destroy the value when the thread exits. <br>线程本地键拥有它所包含的值，并在线程退出时销毁该值。<br> It is created with the [`thread_local!`] macro and can contain any value that is `'static` (no borrowed pointers). <br>它是使用 [`thread_local!`] 宏创建的，可以包含 `'static` 的任何值 (没有借用指针)。<br>
//! It provides an accessor function, [`with`], that yields a shared reference to the value to the specified closure. <br>它提供了一个访问器函数 [`with`]，该访问器函数产生对指定闭包的值的共享引用。<br> Thread-local keys allow only shared access to values, as there would be no way to guarantee uniqueness if mutable borrows were allowed. <br>线程本地键只允许共享访问值，因为如果允许可变借用，就无法保证惟一性。<br>
//! Most values will want to make use of some form of **interior mutability** through the [`Cell`] or [`RefCell`] types. <br>大多数值都希望通过 [`Cell`] 或 [`RefCell`] 类型使用某种形式的 **内部可变性**。<br>
//!
//! ## Naming threads <br>命名线程<br>
//!
//! Threads are able to have associated names for identification purposes. <br>出于识别目的，线程可以有关联的名称。<br> By default, spawned threads are unnamed. <br>默认情况下，生成的线程是未命名的。<br> To specify a name for a thread, build the thread with [`Builder`] and pass the desired thread name to [`Builder::name`]. <br>要为线程指定名称，请使用 [`Builder`] 构建该线程，然后将所需的线程名称传递给 [`Builder::name`]。<br>
//! To retrieve the thread name from within the thread, use [`Thread::name`]. <br>要从线程内检索线程名，请使用 [`Thread::name`]。<br>
//! A couple examples of where the name of a thread gets used: <br>下面是使用线程名的几个例子：<br>
//!
//! * If a panic occurs in a named thread, the thread name will be printed in the panic message. <br>如果在命名线程中出现 panic，则线程名将显示在 panic 消息中。<br>
//! * The thread name is provided to the OS where applicable (e.g., `pthread_setname_np` in unix-like platforms). <br>线程名在适用的情况下提供给操作系统 (例如，在类 Unix 平台中为 `pthread_setname_np`)。<br>
//!
//! ## Stack size <br>栈大小<br>
//!
//! The default stack size for spawned threads is 2 MiB, though this particular stack size is subject to change in the future. <br>衍生线程的默认栈大小为 2 MiB，尽管这个特定的堆栈大小会在将来发生改变。<br> There are two ways to manually specify the stack size for spawned threads: <br>有两种方法可以手动指定衍生线程的栈大小：<br>
//!
//! * Build the thread with [`Builder`] and pass the desired stack size to [`Builder::stack_size`]. <br>使用 [`Builder`] 构建线程，并将所需的栈大小传递给 [`Builder::stack_size`]。<br>
//! * Set the `RUST_MIN_STACK` environment variable to an integer representing the desired stack size (in bytes). <br>将 `RUST_MIN_STACK` 环境变量设置为代表所需栈大小 (以字节为单位) 的整数。<br> Note that setting [`Builder::stack_size`] will override this. <br>请注意，设置 [`Builder::stack_size`] 将覆盖此设置。<br>
//!
//! Note that the stack size of the main thread is *not* determined by Rust. <br>注意，主线程的栈大小不是由 Rust 决定的。<br>
//!
//! [channels]: crate::sync::mpsc
//! [`join`]: JoinHandle::join
//! [`Result`]: crate::result::Result
//! [`Ok`]: crate::result::Result::Ok
//! [`Err`]: crate::result::Result::Err
//! [`thread::current`]: current
//! [`thread::Result`]: Result
//! [`unpark`]: Thread::unpark
//! [`thread::park_timeout`]: park_timeout
//! [`Cell`]: crate::cell::Cell
//! [`RefCell`]: crate::cell::RefCell
//! [`with`]: LocalKey::with
//! [`thread_local!`]: crate::thread_local
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]
#![deny(unsafe_op_in_unsafe_fn)]

#[cfg(all(test, not(target_os = "emscripten")))]
mod tests;

use crate::any::Any;
use crate::cell::UnsafeCell;
use crate::ffi::{CStr, CString};
use crate::fmt;
use crate::io;
use crate::marker::PhantomData;
use crate::mem;
use crate::num::NonZeroU64;
use crate::num::NonZeroUsize;
use crate::panic;
use crate::panicking;
use crate::pin::Pin;
use crate::ptr::addr_of_mut;
use crate::str;
use crate::sync::Arc;
use crate::sys::thread as imp;
use crate::sys_common::mutex;
use crate::sys_common::thread;
use crate::sys_common::thread_info;
use crate::sys_common::thread_parker::Parker;
use crate::sys_common::{AsInner, IntoInner};
use crate::time::Duration;

////////////////////////////////////////////////////////////////////////////////
// Thread-local storage <br>线程本地存储<br>
////////////////////////////////////////////////////////////////////////////////

#[macro_use]
mod local;

#[stable(feature = "scoped_threads", since = "1.63.0")]
mod scoped;

#[stable(feature = "scoped_threads", since = "1.63.0")]
pub use scoped::{scope, Scope, ScopedJoinHandle};

#[stable(feature = "rust1", since = "1.0.0")]
pub use self::local::{AccessError, LocalKey};

// The types used by the thread_local! <br>thread_local 使用的类型！<br> macro to access TLS keys. <br>访问 TLS 密钥的宏。<br> Note that there are two types, the "OS" type and the "fast" type. <br>注意，有两种类型，"OS" 类型和 "fast" 的类型。<br>
// The OS thread local key type is accessed via platform-specific API calls and is slow, while the fast key type is accessed via code generated via LLVM, where TLS keys are set up by the elf linker. <br>OS 线程本地密钥类型是通过平台特定的 API 调用访问的，速度很慢，而快速密钥类型是通过 LLVM 生成的代码访问的，其中 TLS 密钥是由 elf 链接器设置的。<br>
// Note that the OS TLS type is always available: on macOS the standard library is compiled with support for older platform versions where fast TLS was not available; <br>请注意，OS TLS 类型始终可用：在 macOS 上，标准库的编译是对较早的平台版本 (不提供快速 TLS 的支持) 的支持；<br> end-user code is compiled with fast TLS where available, but both are needed. <br>最终用户代码在可用的情况下使用快速 TLS 进行编译，但两者都是必需的。<br>
//
//
//
//
//

#[unstable(feature = "libstd_thread_internals", issue = "none")]
#[cfg(target_thread_local)]
#[doc(hidden)]
pub use self::local::fast::Key as __FastLocalKeyInner;
#[unstable(feature = "libstd_thread_internals", issue = "none")]
#[doc(hidden)]
pub use self::local::os::Key as __OsLocalKeyInner;
#[unstable(feature = "libstd_thread_internals", issue = "none")]
#[cfg(all(target_family = "wasm", not(target_feature = "atomics")))]
#[doc(hidden)]
pub use self::local::statik::Key as __StaticLocalKeyInner;

////////////////////////////////////////////////////////////////////////////////
// Builder
////////////////////////////////////////////////////////////////////////////////

/// Thread factory, which can be used in order to configure the properties of a new thread. <br>线程工厂，可用于配置新线程的属性。<br>
///
/// Methods can be chained on it in order to configure it. <br>可以在其上链接方法以对其进行配置。<br>
///
/// The two configurations available are: <br>有两种配置可供选择：<br>
///
/// - [`name`]: specifies an [associated name for the thread][naming-threads]
/// - [`stack_size`]: specifies the [desired stack size for the thread][stack-size]
///
/// The [`spawn`] method will take ownership of the builder and create an [`io::Result`] to the thread handle with the given configuration. <br>[`spawn`] 方法将获取构建器的所有权，并使用给定的配置为线程句柄创建 [`io::Result`]。<br>
///
/// The [`thread::spawn`] free function uses a `Builder` with default configuration and [`unwrap`]s its return value. <br>[`thread::spawn`] free 函数使用带有默认配置的 `Builder`，并且 [`unwrap`] 是其返回值。<br>
///
/// You may want to use [`spawn`] instead of [`thread::spawn`], when you want to recover from a failure to launch a thread, indeed the free function will panic where the `Builder` method will return a [`io::Result`]. <br>当您想从启动线程失败中恢复时，您可能想使用 [`spawn`] 而不是 [`thread::spawn`]，实际上，free 函数会在 `Builder` 方法返回 [`io::Result`] 时 panic。<br>
///
///
/// # Examples
///
/// ```
/// use std::thread;
///
/// let builder = thread::Builder::new();
///
/// let handler = builder.spawn(|| {
///     // thread code <br>线程代码<br>
/// }).unwrap();
///
/// handler.join().unwrap();
/// ```
///
/// [`stack_size`]: Builder::stack_size
/// [`name`]: Builder::name
/// [`spawn`]: Builder::spawn
/// [`thread::spawn`]: spawn
/// [`io::Result`]: crate::io::Result
/// [`unwrap`]: crate::result::Result::unwrap
/// [naming-threads]: ./index.html#naming-threads
/// [stack-size]: ./index.html#stack-size
///
///
///
///
#[must_use = "must eventually spawn the thread"]
#[stable(feature = "rust1", since = "1.0.0")]
#[derive(Debug)]
pub struct Builder {
    // A name for the thread-to-be, for identification in panic messages <br>未来线程的名称，以在 panic 消息中进行标识<br>
    name: Option<String>,
    // The size of the stack for the spawned thread in bytes <br>衍生线程的栈大小 (以字节为单位)<br>
    stack_size: Option<usize>,
}

impl Builder {
    /// Generates the base configuration for spawning a thread, from which configuration methods can be chained. <br>生成用于生成线程的基本配置，从中可以链接配置方法。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::thread;
    ///
    /// let builder = thread::Builder::new()
    ///                               .name("foo".into())
    ///                               .stack_size(32 * 1024);
    ///
    /// let handler = builder.spawn(|| {
    ///     // thread code <br>线程代码<br>
    /// }).unwrap();
    ///
    /// handler.join().unwrap();
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn new() -> Builder {
        Builder { name: None, stack_size: None }
    }

    /// Names the thread-to-be. <br>命名未来线程。<br> Currently the name is used for identification only in panic messages. <br>当前，该名称仅用于 panic 消息中的标识。<br>
    ///
    /// The name must not contain null bytes (`\0`). <br>该名称不能包含空字节 (`\0`)。<br>
    ///
    /// For more information about named threads, see [this module-level documentation][naming-threads]. <br>有关命名线程的更多信息，请参见 [模块级文档][naming-threads]。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::thread;
    ///
    /// let builder = thread::Builder::new()
    ///     .name("foo".into());
    ///
    /// let handler = builder.spawn(|| {
    ///     assert_eq!(thread::current().name(), Some("foo"))
    /// }).unwrap();
    ///
    /// handler.join().unwrap();
    /// ```
    ///
    /// [naming-threads]: ./index.html#naming-threads
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn name(mut self, name: String) -> Builder {
        self.name = Some(name);
        self
    }

    /// Sets the size of the stack (in bytes) for the new thread. <br>设置新线程的栈大小 (以字节为单位)。<br>
    ///
    /// The actual stack size may be greater than this value if the platform specifies a minimal stack size. <br>如果平台指定最小栈大小，则实际栈大小可能大于这个值。<br>
    ///
    /// For more information about the stack size for threads, see [this module-level documentation][stack-size]. <br>有关线程的栈大小的更多信息，请参见 [模块级文档][stack-size]。<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// use std::thread;
    ///
    /// let builder = thread::Builder::new().stack_size(32 * 1024);
    /// ```
    ///
    /// [stack-size]: ./index.html#stack-size
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn stack_size(mut self, size: usize) -> Builder {
        self.stack_size = Some(size);
        self
    }

    /// Spawns a new thread by taking ownership of the `Builder`, and returns an [`io::Result`] to its [`JoinHandle`]. <br>通过获取 `Builder` 的所有权产生一个新线程，并向其 [`JoinHandle`] 返回一个 [`io::Result`]。<br>
    ///
    /// The spawned thread may outlive the caller (unless the caller thread is the main thread; the whole process is terminated when the main thread finishes). <br>衍生的线程可能比调用者活得长 (除非调用者线程是主线程; 当主线程结束时，整个进程将终止)。<br>
    /// The join handle can be used to block on termination of the spawned thread, including recovering its panics. <br>连接句柄可用于在新线程终止时阻塞，包括恢复其 panics。<br>
    ///
    /// For a more complete documentation see [`thread::spawn`][`spawn`]. <br>有关更完整的文档，请参见 [`thread::spawn`][`spawn`]。<br>
    ///
    /// # Errors
    ///
    /// Unlike the [`spawn`] free function, this method yields an [`io::Result`] to capture any failure to create the thread at the OS level. <br>与 [`spawn`] 的 free 函数不同，此方法产生 [`io::Result`] 来捕获在操作系统级别创建线程的任何失败。<br>
    ///
    /// [`io::Result`]: crate::io::Result
    ///
    /// # Panics
    ///
    /// Panics if a thread name was set and it contained null bytes. <br>如果设置了线程名称并且它包含空字节，就会出现 panic。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::thread;
    ///
    /// let builder = thread::Builder::new();
    ///
    /// let handler = builder.spawn(|| {
    ///     // thread code <br>线程代码<br>
    /// }).unwrap();
    ///
    /// handler.join().unwrap();
    /// ```
    ///
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn spawn<F, T>(self, f: F) -> io::Result<JoinHandle<T>>
    where
        F: FnOnce() -> T,
        F: Send + 'static,
        T: Send + 'static,
    {
        unsafe { self.spawn_unchecked(f) }
    }

    /// Spawns a new thread without any lifetime restrictions by taking ownership of the `Builder`, and returns an [`io::Result`] to its [`JoinHandle`]. <br>通过获取 `Builder` 的所有权来产生不受任何生命周期限制的新线程，并将 [`io::Result`] 返回其 [`JoinHandle`]。<br>
    ///
    /// The spawned thread may outlive the caller (unless the caller thread is the main thread; the whole process is terminated when the main thread finishes). <br>衍生的线程可能比调用者活得长 (除非调用者线程是主线程; 当主线程结束时，整个进程将终止)。<br>
    /// The join handle can be used to block on termination of the spawned thread, including recovering its panics. <br>连接句柄可用于在新线程终止时阻塞，包括恢复其 panics。<br>
    ///
    /// This method is identical to [`thread::Builder::spawn`][`Builder::spawn`], except for the relaxed lifetime bounds, which render it unsafe. <br>此方法与 [`thread::Builder::spawn`][`Builder::spawn`] 相同，不同之处在于宽松的生命周期界限使其不安全。<br>
    /// For a more complete documentation see [`thread::spawn`][`spawn`]. <br>有关更完整的文档，请参见 [`thread::spawn`][`spawn`]。<br>
    ///
    /// # Errors
    ///
    /// Unlike the [`spawn`] free function, this method yields an [`io::Result`] to capture any failure to create the thread at the OS level. <br>与 [`spawn`] 的 free 函数不同，此方法产生 [`io::Result`] 来捕获在操作系统级别创建线程的任何失败。<br>
    ///
    /// # Panics
    ///
    /// Panics if a thread name was set and it contained null bytes. <br>如果设置了线程名称并且它包含空字节，就会出现 panic。<br>
    ///
    /// # Safety
    ///
    /// The caller has to ensure that the spawned thread does not outlive any references in the supplied thread closure and its return type. <br>调用者必须确保新建线程的生命周期不会超过所提供线程闭包及其返回类型中的任何引用。<br>
    /// This can be guaranteed in two ways: <br>可以通过两种方式保证这一点：<br>
    ///
    /// - ensure that [`join`][`JoinHandle::join`] is called before any referenced data is dropped <br>确保在丢弃任何引用数据之前已调用 [`join`][`JoinHandle::join`]<br>
    /// - use only types with `'static` lifetime bounds, i.e., those with no or only `'static` references (both [`thread::Builder::spawn`][`Builder::spawn`] and [`thread::spawn`][`spawn`] enforce this property statically) <br>仅使用具有 `'static` 生命周期界限的类型，即没有 `'static` 引用或仅具有 `'static` 引用的类型 ([`thread::Builder::spawn`][`Builder::spawn`] 和 [`thread::spawn`][`spawn`] 都静态地强制执行此属性)<br>
    ///
    ///
    /// # Examples
    ///
    /// ```
    /// #![feature(thread_spawn_unchecked)]
    /// use std::thread;
    ///
    /// let builder = thread::Builder::new();
    ///
    /// let x = 1;
    /// let thread_x = &x;
    ///
    /// let handler = unsafe {
    ///     builder.spawn_unchecked(move || {
    ///         println!("x = {}", *thread_x);
    ///     }).unwrap()
    /// };
    ///
    /// // caller has to ensure `join()` is called, otherwise it is possible to access freed memory if `x` gets dropped before the thread closure is executed! <br>调用者必须确保已调用 `join()`，否则，如果在执行线程闭包之前 `x` 被丢弃，则可以访问释放的内存！<br>
    /////
    /////
    /// handler.join().unwrap();
    /// ```
    ///
    /// [`io::Result`]: crate::io::Result
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[unstable(feature = "thread_spawn_unchecked", issue = "55132")]
    pub unsafe fn spawn_unchecked<'a, F, T>(self, f: F) -> io::Result<JoinHandle<T>>
    where
        F: FnOnce() -> T,
        F: Send + 'a,
        T: Send + 'a,
    {
        Ok(JoinHandle(unsafe { self.spawn_unchecked_(f, None) }?))
    }

    unsafe fn spawn_unchecked_<'a, 'scope, F, T>(
        self,
        f: F,
        scope_data: Option<Arc<scoped::ScopeData>>,
    ) -> io::Result<JoinInner<'scope, T>>
    where
        F: FnOnce() -> T,
        F: Send + 'a,
        T: Send + 'a,
        'scope: 'a,
    {
        let Builder { name, stack_size } = self;

        let stack_size = stack_size.unwrap_or_else(thread::min_stack);

        let my_thread = Thread::new(name.map(|name| {
            CString::new(name).expect("thread name may not contain interior null bytes")
        }));
        let their_thread = my_thread.clone();

        let my_packet: Arc<Packet<'scope, T>> = Arc::new(Packet {
            scope: scope_data,
            result: UnsafeCell::new(None),
            _marker: PhantomData,
        });
        let their_packet = my_packet.clone();

        let output_capture = crate::io::set_output_capture(None);
        crate::io::set_output_capture(output_capture.clone());

        let main = move || {
            if let Some(name) = their_thread.cname() {
                imp::Thread::set_name(name);
            }

            crate::io::set_output_capture(output_capture);

            // SAFETY: the stack guard passed is the one for the current thread. <br>传递的堆栈守卫是当前线程的守卫。<br>
            // This means the current thread's stack and the new thread's stack are properly set and protected from each other. <br>这意味着当前线程的栈和新线程的栈已被正确设置并相互保护。<br>
            //
            thread_info::set(unsafe { imp::guard::current() }, their_thread);
            let try_result = panic::catch_unwind(panic::AssertUnwindSafe(|| {
                crate::sys_common::backtrace::__rust_begin_short_backtrace(f)
            }));
            // SAFETY: `their_packet` as been built just above and moved by the closure (it is an Arc<...>) and `my_packet` will be stored in the same `JoinInner` as this closure meaning the mutation will be safe (not modify it and affect a value far away). <br>`their_packet` 正好在其上方构建并由闭包 (它是 Arc<...>) 移动，并且 `my_packet` 将与该闭包存储在同一 `JoinInner` 中，这意味着该可变的是安全的 (不对其进行修改并影响一个远的值)。<br>
            //
            //
            //
            unsafe { *their_packet.result.get() = Some(try_result) };
        };

        if let Some(scope_data) = &my_packet.scope {
            scope_data.increment_num_running_threads();
        }

        Ok(JoinInner {
            // SAFETY:
            //
            // `imp::Thread::new` takes a closure with a `'static` lifetime, since it's passed through FFI or otherwise used with low-level threading primitives that have no notion of or way to enforce lifetimes. <br>`imp::Thread::new` 接受一个带有 `'static` 生命周期的闭包，因为它通过 FFI 传递或以其他方式与没有概念或方式执行生命周期的线程原语一起使用。<br>
            //
            // As mentioned in the `Safety` section of this function's documentation, the caller of this function needs to guarantee that the passed-in lifetime is sufficiently long for the lifetime of the thread. <br>如本函数文档的 `Safety` 部分所述，此函数的调用者需要确保传入的生命周期对于线程的生命周期足够长。<br>
            //
            //
            // Similarly, the `sys` implementation must guarantee that no references to the closure exist after the thread has terminated, which is signaled by `Thread::join` returning. <br>同样，`sys` 实现必须保证在线程终止后，对闭包的引用不存在，这由 `Thread::join` 返回表示。<br>
            //
            //
            //
            //
            //
            native: unsafe {
                imp::Thread::new(
                    stack_size,
                    mem::transmute::<Box<dyn FnOnce() + 'a>, Box<dyn FnOnce() + 'static>>(
                        Box::new(main),
                    ),
                )?
            },
            thread: my_thread,
            packet: my_packet,
        })
    }
}

////////////////////////////////////////////////////////////////////////////////
// Free functions <br>free 函数<br>
////////////////////////////////////////////////////////////////////////////////

/// Spawns a new thread, returning a [`JoinHandle`] for it. <br>产生一个新线程，并为其返回一个 [`JoinHandle`]。<br>
///
/// The join handle provides a [`join`] method that can be used to join the spawned thread. <br>连接句柄提供了一个 [`join`] 方法，可以用来连接新生成的线程。<br> If the spawned thread panics, [`join`] will return an [`Err`] containing the argument given to [`panic!`]. <br>如果生成的线程发生 panics，那么 [`join`] 将返回一个 [`Err`]，其中包含了提供给 [`panic!`] 的参数。<br>
///
/// If the join handle is dropped, the spawned thread will implicitly be *detached*. <br>如果连接句柄被丢弃了，则新生成的线程将被隐式地 *分离*。<br>
/// In this case, the spawned thread may no longer be joined. <br>在这种情况下，新生成的线程可能不再被连接。<br>
/// (It is the responsibility of the program to either eventually join threads it creates or detach them; otherwise, a resource leak will result.) <br>(程序有责任最终连接它创建的线程，或者将它们分离; 否则，将导致资源泄漏。)<br>
///
/// This call will create a thread using default parameters of [`Builder`], if you want to specify the stack size or the name of the thread, use this API instead. <br>此调用将使用 [`Builder`] 的默认参数创建一个线程，如果要指定栈大小或线程名称，请使用此 API。<br>
///
/// As you can see in the signature of `spawn` there are two constraints on both the closure given to `spawn` and its return value, let's explain them: <br>正如您在 `spawn` 的签名中看到的那样，对于赋予 `spawn` 的闭包及其返回值都有两个约束，让我们对其进行解释：<br>
///
/// - The `'static` constraint means that the closure and its return value must have a lifetime of the whole program execution. <br>`'static` 约束意味着闭包及其返回值必须具有整个程序执行的生命周期。<br> The reason for this is that threads can outlive the lifetime they have been created in. <br>这是因为线程可以比它们被创建时的生命周期更长。<br>
///
///   Indeed if the thread, and by extension its return value, can outlive their caller, we need to make sure that they will be valid afterwards, and since we *can't* know when it will return we need to have them valid as long as possible, that is until the end of the program, hence the `'static` lifetime. <br>确实，如果线程以及它的返回值可以比它们的调用者活得更久，我们需要确保它们以后仍然有效，并且因为我们不能知道它什么时候返回，因此需要使它们直到程序结束时尽可能有效，因此是 `'static` 生命周期。<br>
///
/// - The [`Send`] constraint is because the closure will need to be passed *by value* from the thread where it is spawned to the new thread. <br>[`Send`] 约束是因为闭包需要通过值从产生它的线程传递到新线程。<br> Its return value will need to be passed from the new thread to the thread where it is `join`ed. <br>它的返回值将需要从新线程传递到它被 `join` 的线程。<br>
///   As a reminder, the [`Send`] marker trait expresses that it is safe to be passed from thread to thread. <br>提醒一下，[`Send`] 标记 trait 表示从一个线程传递到另一个线程是安全的。<br> [`Sync`] expresses that it is safe to have a reference be passed from thread to thread. <br>[`Sync`] 表示将引用从一个线程传递到另一个线程是安全的。<br>
///
/// # Panics
///
/// Panics if the OS fails to create a thread; <br>如果操作系统无法创建线程，就会出现 panics。<br> use [`Builder::spawn`] to recover from such errors. <br>使用 [`Builder::spawn`] 从此类错误中恢复。<br>
///
/// # Examples
///
/// Creating a thread. <br>创建一个线程。<br>
///
/// ```
/// use std::thread;
///
/// let handler = thread::spawn(|| {
///     // thread code <br>线程代码<br>
/// });
///
/// handler.join().unwrap();
/// ```
///
/// As mentioned in the module documentation, threads are usually made to communicate using [`channels`], here is how it usually looks. <br>如模块文档中所述，线程通常是使用 [`channels`] 进行通信的，通常情况下是这样的。<br>
///
/// This example also shows how to use `move`, in order to give ownership of values to a thread. <br>此示例还展示了如何使用 `move`，以便将值的所有权授予线程。<br>
///
/// ```
/// use std::thread;
/// use std::sync::mpsc::channel;
///
/// let (tx, rx) = channel();
///
/// let sender = thread::spawn(move || {
///     tx.send("Hello, thread".to_owned())
///         .expect("Unable to send on channel");
/// });
///
/// let receiver = thread::spawn(move || {
///     let value = rx.recv().expect("Unable to receive from channel");
///     println!("{value}");
/// });
///
/// sender.join().expect("The sender thread has panicked");
/// receiver.join().expect("The receiver thread has panicked");
/// ```
///
/// A thread can also return a value through its [`JoinHandle`], you can use this to make asynchronous computations (futures might be more appropriate though). <br>一个线程也可以通过 [`JoinHandle`] 来返回一个值，您可以使用它进行异步计算 (不过 futures 可能更合适)。<br>
///
/// ```
/// use std::thread;
///
/// let computation = thread::spawn(|| {
///     // Some expensive computation. <br>一些昂贵的计算。<br>
///     42
/// });
///
/// let result = computation.join().unwrap();
/// println!("{result}");
/// ```
///
/// [`channels`]: crate::sync::mpsc
/// [`join`]: JoinHandle::join
/// [`Err`]: crate::result::Result::Err
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn spawn<F, T>(f: F) -> JoinHandle<T>
where
    F: FnOnce() -> T,
    F: Send + 'static,
    T: Send + 'static,
{
    Builder::new().spawn(f).expect("failed to spawn thread")
}

/// Gets a handle to the thread that invokes it. <br>获取调用它的线程的句柄。<br>
///
/// # Examples
///
/// Getting a handle to the current thread with `thread::current()`: <br>使用 `thread::current()` 获取当前线程的句柄：<br>
///
/// ```
/// use std::thread;
///
/// let handler = thread::Builder::new()
///     .name("named thread".into())
///     .spawn(|| {
///         let handle = thread::current();
///         assert_eq!(handle.name(), Some("named thread"));
///     })
///     .unwrap();
///
/// handler.join().unwrap();
/// ```
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn current() -> Thread {
    thread_info::current_thread().expect(
        "use of std::thread::current() is not possible \
         after the thread's local data has been destroyed",
    )
}

/// Cooperatively gives up a timeslice to the OS scheduler. <br>协同地将时间片交给操作系统调度程序。<br>
///
/// This calls the underlying OS scheduler's yield primitive, signaling that the calling thread is willing to give up its remaining timeslice so that the OS may schedule other threads on the CPU. <br>这会调用底层操作系统调度程序的 yield 原语，表明调用线程愿意放弃其剩余的时间片，以便操作系统可以在 CPU 上调度其他线程。<br>
///
/// A drawback of yielding in a loop is that if the OS does not have any other ready threads to run on the current CPU, the thread will effectively busy-wait, which wastes CPU time and energy. <br>在循环中让步的一个缺点是，如果操作系统没有任何其他就绪线程在当前 CPU 上运行，该线程将有效地进行忙等待，这会浪费 CPU 时间和精力。<br>
///
/// Therefore, when waiting for events of interest, a programmer's first choice should be to use synchronization devices such as [`channel`]s, [`Condvar`]s, [`Mutex`]es or [`join`] since these primitives are implemented in a blocking manner, giving up the CPU until the event of interest has occurred which avoids repeated yielding. <br>因此，在等待感兴趣的事件时，程序员的首选应该是使用同步设备，例如 [`channel`]、[`Condvar`]、[`Mutex`] 或 [`join`]，因为这些原语是以阻塞方式实现，放弃 CPU 直到感兴趣的事件发生，避免重复让步。<br>
///
///
/// `yield_now` should thus be used only rarely, mostly in situations where repeated polling is required because there is no other suitable way to learn when an event of interest has occurred. <br>因此，`yield_now` 应该很少使用，主要是在需要重复轮询的情况下，因为没有其他合适的方法可以了解何时发生了感兴趣的事件。<br>
///
/// # Examples
///
/// ```
/// use std::thread;
///
/// thread::yield_now();
/// ```
///
/// [`channel`]: crate::sync::mpsc
/// [`join`]: JoinHandle::join
/// [`Condvar`]: crate::sync::Condvar
/// [`Mutex`]: crate::sync::Mutex
///
///
///
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn yield_now() {
    imp::Thread::yield_now()
}

/// Determines whether the current thread is unwinding because of panic. <br>确定当前线程是否由于 panic 而处于展开状态。<br>
///
/// A common use of this feature is to poison shared resources when writing unsafe code, by checking `panicking` when the `drop` is called. <br>此特性的一个常见用途是在编写不安全代码时毒害共享资源，方法是在调用 `drop` 时检查 `panic`。<br>
///
/// This is usually not needed when writing safe code, as [`Mutex`es][Mutex] already poison themselves when a thread panics while holding the lock. <br>编写安全代码时通常不需要这样做，因为当线程在持有锁时 panic，[`Mutex`][Mutex] 已经中毒了。<br>
///
/// This can also be used in multithreaded applications, in order to send a message to other threads warning that a thread has panicked (e.g., for monitoring purposes). <br>这也可以在多线程应用程序中使用，以便向其他线程发送消息，警告某个线程已发生 panic (例如，出于监视目的)。<br>
///
///
/// # Examples
///
/// ```should_panic
/// use std::thread;
///
/// struct SomeStruct;
///
/// impl Drop for SomeStruct {
///     fn drop(&mut self) {
///         if thread::panicking() {
///             println!("dropped while unwinding");
///         } else {
///             println!("dropped while not unwinding");
///         }
///     }
/// }
///
/// {
///     print!("a: ");
///     let a = SomeStruct;
/// }
///
/// {
///     print!("b: ");
///     let b = SomeStruct;
///     panic!()
/// }
/// ```
///
/// [Mutex]: crate::sync::Mutex
///
///
///
#[inline]
#[must_use]
#[stable(feature = "rust1", since = "1.0.0")]
pub fn panicking() -> bool {
    panicking::panicking()
}

/// Puts the current thread to sleep for at least the specified amount of time. <br>使当前线程休眠至少指定的时间。<br>
///
/// The thread may sleep longer than the duration specified due to scheduling specifics or platform-dependent functionality. <br>由于调度细节或平台相关的功能，线程的睡眠时间可能比指定的持续时间更长。<br>
/// It will never sleep less. <br>它永远不会少睡。<br>
///
/// This function is blocking, and should not be used in `async` functions. <br>该函数正在阻塞，因此不应在 `async` 函数中使用。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// On Unix platforms, the underlying syscall may be interrupted by a spurious wakeup or signal handler. <br>在 Unix 平台上，底层的系统调用可能会由于虚假唤醒或信号处理程序而中断。<br>
/// To ensure the sleep occurs for at least the specified duration, this function may invoke that system call multiple times. <br>为了确保至少在指定的持续时间内发生睡眠，此函数可以多次调用该系统。<br>
///
///
/// # Examples
///
/// ```no_run
/// use std::thread;
///
/// // Let's sleep for 2 seconds: <br>让我们睡 2 秒钟：<br>
/// thread::sleep_ms(2000);
/// ```
///
#[stable(feature = "rust1", since = "1.0.0")]
#[deprecated(since = "1.6.0", note = "replaced by `std::thread::sleep`")]
pub fn sleep_ms(ms: u32) {
    sleep(Duration::from_millis(ms as u64))
}

/// Puts the current thread to sleep for at least the specified amount of time. <br>使当前线程休眠至少指定的时间。<br>
///
/// The thread may sleep longer than the duration specified due to scheduling specifics or platform-dependent functionality. <br>由于调度细节或平台相关的功能，线程的睡眠时间可能比指定的持续时间更长。<br> It will never sleep less. <br>它永远不会少睡。<br>
///
/// This function is blocking, and should not be used in `async` functions. <br>该函数正在阻塞，因此不应在 `async` 函数中使用。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// On Unix platforms, the underlying syscall may be interrupted by a spurious wakeup or signal handler. <br>在 Unix 平台上，底层的系统调用可能会由于虚假唤醒或信号处理程序而中断。<br>
/// To ensure the sleep occurs for at least the specified duration, this function may invoke that system call multiple times. <br>为了确保至少在指定的持续时间内发生睡眠，此函数可以多次调用该系统。<br>
/// Platforms which do not support nanosecond precision for sleeping will have `dur` rounded up to the nearest granularity of time they can sleep for. <br>不支持纳秒级睡眠精度的平台会将 `dur` 舍入为最接近的睡眠时间粒度。<br>
///
/// Currently, specifying a zero duration on Unix platforms returns immediately without invoking the underlying [`nanosleep`] syscall, whereas on Windows platforms the underlying [`Sleep`] syscall is always invoked. <br>目前，在 Unix 平台上指定零持续时间会立即返回，而不会调用底层 [`nanosleep`] syscall，而在 Windows 平台上，始终调用底层 [`Sleep`] syscall。<br>
///
/// If the intention is to yield the current time-slice you may want to use [`yield_now`] instead. <br>如果要产生当前时间片，则可能需要使用 [`yield_now`]。<br>
///
/// [`nanosleep`]: https://linux.die.net/man/2/nanosleep
/// [`Sleep`]: https://docs.microsoft.com/en-us/windows/win32/api/synchapi/nf-synchapi-sleep
///
/// # Examples
///
/// ```no_run
/// use std::{thread, time};
///
/// let ten_millis = time::Duration::from_millis(10);
/// let now = time::Instant::now();
///
/// thread::sleep(ten_millis);
///
/// assert!(now.elapsed() >= ten_millis);
/// ```
///
///
///
///
///
///
#[stable(feature = "thread_sleep", since = "1.4.0")]
pub fn sleep(dur: Duration) {
    imp::Thread::sleep(dur)
}

/// Blocks unless or until the current thread's token is made available. <br>阻塞，除非或直到当前线程的 token 可用为止。<br>
///
/// A call to `park` does not guarantee that the thread will remain parked forever, and callers should be prepared for this possibility. <br>对 `park` 的调用不能保证线程将永远保持驻留状态，因此调用者应为此做好准备。<br>
///
/// # park and unpark <br>park 和 unpark<br>
///
/// Every thread is equipped with some basic low-level blocking support, via the [`thread::park`][`park`] function and [`thread::Thread::unpark`][`unpark`] method. <br>每个线程都通过 [`thread::park`][`park`] 函数和 [`thread::Thread::unpark`][`unpark`] 方法提供了一些基本的阻塞支持。<br>
/// [`park`] blocks the current thread, which can then be resumed from another thread by calling the [`unpark`] method on the blocked thread's handle. <br>[`park`] 阻塞当前线程，然后可以通过在阻塞线程的句柄上调用 [`unpark`] 方法从另一个线程恢复当前线程。<br>
///
/// Conceptually, each [`Thread`] handle has an associated token, which is initially not present: <br>从概念上讲，每个 [`Thread`] 句柄都有一个关联的 token，该 token 最初不存在：<br>
///
/// * The [`thread::park`][`park`] function blocks the current thread unless or until the token is available for its thread handle, at which point it atomically consumes the token. <br>[`thread::park`][`park`] 函数会阻塞当前线程，除非或直到 token 可用于其线程句柄为止，否则该原子将自动消耗 token。<br>
/// It may also return *spuriously*, without consuming the token. <br>它也可能虚假地返回，而不消耗 token。<br>
/// [`thread::park_timeout`] does the same, but allows specifying a maximum time to block the thread for. <br>[`thread::park_timeout`] 执行相同的操作，但允许指定阻塞线程的最长时间。<br>
///
/// * The [`unpark`] method on a [`Thread`] atomically makes the token available if it wasn't already. <br>[`Thread`] 上的 [`unpark`] 方法原子地使 token (如果尚未提供) 可用。<br>
/// Because the token is initially absent, [`unpark`] followed by [`park`] will result in the second call returning immediately. <br>由于最初不存在 token，因此 [`unpark`] 后跟 [`park`] 将导致第二个调用立即返回。<br>
///
/// In other words, each [`Thread`] acts a bit like a spinlock that can be locked and unlocked using `park` and `unpark`. <br>换句话说，每个 [`Thread`] 的行为都类似于自旋锁，可以使用 `park` 和 `unpark` 进行锁定和解锁。<br>
///
/// Notice that being unblocked does not imply any synchronization with someone that unparked this thread, it could also be spurious. <br>请注意，被取消阻止并不意味着与取消该线程的某个人进行任何同步，这也可能是虚假的。<br>
/// For example, it would be a valid, but inefficient, implementation to make both [`park`] and [`unpark`] return immediately without doing anything. <br>例如，将 [`park`] 和 [`unpark`] 都立即返回而无需执行任何操作将是有效但效率低下的实现。<br>
///
/// The API is typically used by acquiring a handle to the current thread, placing that handle in a shared data structure so that other threads can find it, and then `park`ing in a loop. <br>通常通过获取当前线程的句柄，将该句柄放置在共享数据结构体中，以便其他线程可以找到它，然后 `park` 在循环中来使用该 API。<br>
///
/// When some desired condition is met, another thread calls [`unpark`] on the handle. <br>当满足某些所需条件时，另一个线程将在句柄上调用 [`unpark`]。<br>
///
/// The motivation for this design is twofold: <br>这种设计的动机是双重的：<br>
///
/// * It avoids the need to allocate mutexes and condvars when building new synchronization primitives; <br>在构建新的同步原语时，它无需分配互斥锁和 condvar。<br> the threads already provide basic blocking/signaling. <br>线程已经提供了基本的 blocking/signaling。<br>
///
/// * It can be implemented very efficiently on many platforms. <br>它可以在许多平台上非常有效地实现。<br>
///
/// # Examples
///
/// ```
/// use std::thread;
/// use std::sync::{Arc, atomic::{Ordering, AtomicBool}};
/// use std::time::Duration;
///
/// let flag = Arc::new(AtomicBool::new(false));
/// let flag2 = Arc::clone(&flag);
///
/// let parked_thread = thread::spawn(move || {
///     // We want to wait until the flag is set. <br>我们要等到标志被设置。<br>
///     // We *could* just spin, but using park/unpark is more efficient. <br>我们可以旋转，但是使用 park/unpark 效率更高。<br>
///     while !flag2.load(Ordering::Acquire) {
///         println!("Parking thread");
///         thread::park();
///         // We *could* get here spuriously, i.e., way before the 10ms below are over! <br>我们可以伪装地到达这里，即在下面的 10ms 结束之前！<br>
///         // But that is no problem, we are in a loop until the flag is set anyway. <br>但这没问题，我们一直处于循环状态，直到仍然设置了标志。<br>
///         println!("Thread unparked");
///     }
///     println!("Flag received");
/// });
///
/// // Let some time pass for the thread to be spawned. <br>花费一些时间来生成线程。<br>
/// thread::sleep(Duration::from_millis(10));
///
/// // Set the flag, and let the thread wake up. <br>设置标志，并让线程唤醒。<br>
/// // There is no race condition here, if `unpark` happens first, `park` will return immediately. <br>这里没有竞态条件，如果 `unpark` 首先出现，则 `park` 将立即返回。<br>
/////
/// // Hence there is no risk of a deadlock. <br>因此，没有死锁的风险。<br>
/// flag.store(true, Ordering::Release);
/// println!("Unpark the thread");
/// parked_thread.thread().unpark();
///
/// parked_thread.join().unwrap();
/// ```
///
/// [`unpark`]: Thread::unpark
/// [`thread::park_timeout`]: park_timeout
///
///
///
///
///
///
///
///
///
///
///
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub fn park() {
    // SAFETY: park_timeout is called on the parker owned by this thread. <br>在该线程所拥有的 parker 上调用 park_timeout。<br>
    unsafe {
        current().inner.as_ref().parker().park();
    }
}

/// Use [`park_timeout`]. <br>使用 [`park_timeout`]。<br>
///
/// Blocks unless or until the current thread's token is made available or the specified duration has been reached (may wake spuriously). <br>除非直到当前线程的 token 可用或达到指定的持续时间 (否则可能会虚假唤醒)，否则将阻塞。<br>
///
/// The semantics of this function are equivalent to [`park`] except that the thread will be blocked for roughly no longer than `dur`. <br>该函数的语义与 [`park`] 等效，除了线程被阻塞的时间不超过 `dur`。<br>
/// This method should not be used for precise timing due to anomalies such as preemption or platform differences that might not cause the maximum amount of time waited to be precisely `ms` long. <br>由于抢占或平台差异等异常情况可能不会导致最长等待时间精确到 `ms` 长，因此不应将此方法用于精确计时。<br>
///
///
/// See the [park documentation][`park`] for more detail. <br>有关更多详细信息，请参见 [park 文档][`park`]。<br>
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
#[deprecated(since = "1.6.0", note = "replaced by `std::thread::park_timeout`")]
pub fn park_timeout_ms(ms: u32) {
    park_timeout(Duration::from_millis(ms as u64))
}

/// Blocks unless or until the current thread's token is made available or the specified duration has been reached (may wake spuriously). <br>除非直到当前线程的 token 可用或达到指定的持续时间 (否则可能会虚假唤醒)，否则将阻塞。<br>
///
/// The semantics of this function are equivalent to [`park`][park] except that the thread will be blocked for roughly no longer than `dur`. <br>该函数的语义与 [`park`][park] 等效，除了线程被阻塞的时间不超过 `dur`。<br>
/// This method should not be used for precise timing due to anomalies such as preemption or platform differences that might not cause the maximum amount of time waited to be precisely `dur` long. <br>由于抢占或平台差异等异常情况可能不会导致最长等待时间精确到 `dur` 长，因此不应将此方法用于精确计时。<br>
///
///
/// See the [park documentation][park] for more details. <br>有关更多详细信息，请参见 [park 文档][park]。<br>
///
/// # Platform-specific behavior <br>特定于平台的行为<br>
///
/// Platforms which do not support nanosecond precision for sleeping will have `dur` rounded up to the nearest granularity of time they can sleep for. <br>不支持纳秒级睡眠精度的平台会将 `dur` 舍入为最接近的睡眠时间粒度。<br>
///
/// # Examples
///
/// Waiting for the complete expiration of the timeout: <br>等待超时完全到期：<br>
///
/// ```rust,no_run
/// use std::thread::park_timeout;
/// use std::time::{Instant, Duration};
///
/// let timeout = Duration::from_secs(2);
/// let beginning_park = Instant::now();
///
/// let mut timeout_remaining = timeout;
/// loop {
///     park_timeout(timeout_remaining);
///     let elapsed = beginning_park.elapsed();
///     if elapsed >= timeout {
///         break;
///     }
///     println!("restarting park_timeout after {elapsed:?}");
///     timeout_remaining = timeout - elapsed;
/// }
/// ```
///
///
///
///
#[stable(feature = "park_timeout", since = "1.4.0")]
pub fn park_timeout(dur: Duration) {
    // SAFETY: park_timeout is called on the parker owned by this thread. <br>在该线程所拥有的 parker 上调用 park_timeout。<br>
    unsafe {
        current().inner.as_ref().parker().park_timeout(dur);
    }
}

////////////////////////////////////////////////////////////////////////////////
// ThreadId
////////////////////////////////////////////////////////////////////////////////

/// A unique identifier for a running thread. <br>正在运行的线程的唯一标识符。<br>
///
/// A `ThreadId` is an opaque object that uniquely identifies each thread created during the lifetime of a process. <br>`ThreadId` 是一个不透明的对象，它唯一的标识了在进程生命周期期中创建的每个线程。<br> `ThreadId`s are guaranteed not to be reused, even when a thread terminates. <br>`ThreadId`s 保证不会被重用，即使线程终止时也是如此。<br>
/// `ThreadId`s are under the control of Rust's standard library and there may not be any relationship between `ThreadId` and the underlying platform's notion of a thread identifier -- the two concepts cannot, therefore, be used interchangeably. <br>`ThreadId` 受 Rust 标准库的控制，`ThreadId` 和底层平台的线程标识符概念之间可能没有任何关系 -- 因此，这两个概念不能互换使用。<br>
///
/// A `ThreadId` can be retrieved from the [`id`] method on a [`Thread`]. <br>可以从 [`Thread`] 上的 [`id`] 方法中检索 `ThreadId`。<br>
///
/// # Examples
///
/// ```
/// use std::thread;
///
/// let other_thread = thread::spawn(|| {
///     thread::current().id()
/// });
///
/// let other_thread_id = other_thread.join().unwrap();
/// assert!(thread::current().id() != other_thread_id);
/// ```
///
/// [`id`]: Thread::id
///
///
///
#[stable(feature = "thread_id", since = "1.19.0")]
#[derive(Eq, PartialEq, Clone, Copy, Hash, Debug)]
pub struct ThreadId(NonZeroU64);

impl ThreadId {
    // Generate a new unique thread ID. <br>生成一个新的唯一线程 ID。<br>
    fn new() -> ThreadId {
        // It is UB to attempt to acquire this mutex reentrantly! <br>尝试重新获得此互斥锁是 UB!<br>
        static GUARD: mutex::StaticMutex = mutex::StaticMutex::new();
        static mut COUNTER: u64 = 1;

        unsafe {
            let guard = GUARD.lock();

            // If we somehow use up all our bits, panic so that we're not covering up subtle bugs of IDs being reused. <br>如果我们以某种方式用尽了所有位 panic，那么我们就不会掩盖正在重用的 ID 的细微错误。<br>
            //
            if COUNTER == u64::MAX {
                drop(guard); // in case the panic handler ends up calling `ThreadId::new()`, avoid reentrant lock acquire. <br>如果 panic 处理程序最终调用 `ThreadId::new()`，请避免获取可重入锁。<br>
                panic!("failed to generate unique thread ID: bitspace exhausted");
            }

            let id = COUNTER;
            COUNTER += 1;

            ThreadId(NonZeroU64::new(id).unwrap())
        }
    }

    /// This returns a numeric identifier for the thread identified by this `ThreadId`. <br>这将返回此 `ThreadId` 标识的线程的数字标识符。<br>
    ///
    /// As noted in the documentation for the type itself, it is essentially an opaque ID, but is guaranteed to be unique for each thread. <br>如类型本身的文档中所述，它本质上是一个不透明的 ID，但可以保证每个线程都是唯一的。<br>
    /// The returned value is entirely opaque -- only equality testing is stable. <br>返回的值是完全不透明的 - 仅相等测试是稳定的。<br>
    /// Note that it is not guaranteed which values new threads will return, and this may change across Rust versions. <br>请注意，不能保证新线程将返回哪些值，并且在 Rust 版本之间可能会改变。<br>
    ///
    ///
    ///
    #[must_use]
    #[unstable(feature = "thread_id_value", issue = "67939")]
    pub fn as_u64(&self) -> NonZeroU64 {
        self.0
    }
}

////////////////////////////////////////////////////////////////////////////////
// Thread
////////////////////////////////////////////////////////////////////////////////

/// The internal representation of a `Thread` handle <br>`Thread` 句柄的内部表示<br>
struct Inner {
    name: Option<CString>, // Guaranteed to be UTF-8 <br>保证为 UTF-8<br>
    id: ThreadId,
    parker: Parker,
}

impl Inner {
    fn parker(self: Pin<&Self>) -> Pin<&Parker> {
        unsafe { Pin::map_unchecked(self, |inner| &inner.parker) }
    }
}

#[derive(Clone)]
#[stable(feature = "rust1", since = "1.0.0")]
/// A handle to a thread. <br>线程的句柄。<br>
///
/// Threads are represented via the `Thread` type, which you can get in one of two ways: <br>线程通过 `Thread` 类型表示，您可以通过以下两种方式之一来获取：<br>
///
/// * By spawning a new thread, e.g., using the [`thread::spawn`][`spawn`] function, and calling [`thread`][`JoinHandle::thread`] on the [`JoinHandle`]. <br>通过生成一个新线程，例如使用 [`thread::spawn`][`spawn`] 函数，并在 [`JoinHandle`] 上调用 [`thread`][`JoinHandle::thread`]。<br>
/// * By requesting the current thread, using the [`thread::current`] function. <br>通过使用 [`thread::current`] 函数来请求当前线程。<br>
///
/// The [`thread::current`] function is available even for threads not spawned by the APIs of this module. <br>[`thread::current`] 函数甚至可用于不是由该模块的 API 生成的线程。<br>
///
/// There is usually no need to create a `Thread` struct yourself, one should instead use a function like `spawn` to create new threads, see the docs of [`Builder`] and [`spawn`] for more details. <br>通常不需要自己创建 `Thread` 结构体，而应使用 `spawn` 之类的函数来创建新线程，有关更多详细信息，请参见 [`Builder`] 和 [`spawn`] 的文档。<br>
///
///
/// [`thread::current`]: current
///
///
///
///
///
pub struct Thread {
    inner: Pin<Arc<Inner>>,
}

impl Thread {
    // Used only internally to construct a thread object without spawning Panics if the name contains nuls. <br>如果名称包含 nul，则仅在内部用于构造线程对象，而不会发生 panics。<br>
    //
    pub(crate) fn new(name: Option<CString>) -> Thread {
        // We have to use `unsafe` here to constuct the `Parker` in-place, which is required for the UNIX implementation. <br>我们必须在这里使用 `unsafe` 来就地构造 `Parker`，这是 UNIX 实现所必需的。<br>
        //
        //
        // SAFETY: We pin the Arc immediately after creation, so its address never changes. <br>我们在创建后立即固定 Arc，因此它的地址永远不会改变。<br>
        //
        let inner = unsafe {
            let mut arc = Arc::<Inner>::new_uninit();
            let ptr = Arc::get_mut_unchecked(&mut arc).as_mut_ptr();
            addr_of_mut!((*ptr).name).write(name);
            addr_of_mut!((*ptr).id).write(ThreadId::new());
            Parker::new(addr_of_mut!((*ptr).parker));
            Pin::new_unchecked(arc.assume_init())
        };

        Thread { inner }
    }

    /// Atomically makes the handle's token available if it is not already. <br>通过原子方式使句柄的 token 可用 (如果尚不可用)。<br>
    ///
    /// Every thread is equipped with some basic low-level blocking support, via the [`park`][park] function and the `unpark()` method. <br>每个线程都通过 [`park`][park] 函数和 `unpark()` 方法提供了一些基本的阻塞支持。<br>
    /// These can be used as a more CPU-efficient implementation of a spinlock. <br>这些可用作自旋锁的 CPU 效率更高的实现。<br>
    ///
    /// See the [park documentation][park] for more details. <br>有关更多详细信息，请参见 [park 文档][park]。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::thread;
    /// use std::time::Duration;
    ///
    /// let parked_thread = thread::Builder::new()
    ///     .spawn(|| {
    ///         println!("Parking thread");
    ///         thread::park();
    ///         println!("Thread unparked");
    ///     })
    ///     .unwrap();
    ///
    /// // Let some time pass for the thread to be spawned. <br>花费一些时间来生成线程。<br>
    /// thread::sleep(Duration::from_millis(10));
    ///
    /// println!("Unpark the thread");
    /// parked_thread.thread().unpark();
    ///
    /// parked_thread.join().unwrap();
    /// ```
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    #[inline]
    pub fn unpark(&self) {
        self.inner.as_ref().parker().unpark();
    }

    /// Gets the thread's unique identifier. <br>获取线程的唯一标识符。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::thread;
    ///
    /// let other_thread = thread::spawn(|| {
    ///     thread::current().id()
    /// });
    ///
    /// let other_thread_id = other_thread.join().unwrap();
    /// assert!(thread::current().id() != other_thread_id);
    /// ```
    #[stable(feature = "thread_id", since = "1.19.0")]
    #[must_use]
    pub fn id(&self) -> ThreadId {
        self.inner.id
    }

    /// Gets the thread's name. <br>获取线程的名称。<br>
    ///
    /// For more information about named threads, see [this module-level documentation][naming-threads]. <br>有关命名线程的更多信息，请参见 [模块级文档][naming-threads]。<br>
    ///
    ///
    /// # Examples
    ///
    /// Threads by default have no name specified: <br>默认情况下，线程未指定名称：<br>
    ///
    /// ```
    /// use std::thread;
    ///
    /// let builder = thread::Builder::new();
    ///
    /// let handler = builder.spawn(|| {
    ///     assert!(thread::current().name().is_none());
    /// }).unwrap();
    ///
    /// handler.join().unwrap();
    /// ```
    ///
    /// Thread with a specified name: <br>具有指定名称的线程：<br>
    ///
    /// ```
    /// use std::thread;
    ///
    /// let builder = thread::Builder::new()
    ///     .name("foo".into());
    ///
    /// let handler = builder.spawn(|| {
    ///     assert_eq!(thread::current().name(), Some("foo"))
    /// }).unwrap();
    ///
    /// handler.join().unwrap();
    /// ```
    ///
    /// [naming-threads]: ./index.html#naming-threads
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn name(&self) -> Option<&str> {
        self.cname().map(|s| unsafe { str::from_utf8_unchecked(s.to_bytes()) })
    }

    fn cname(&self) -> Option<&CStr> {
        self.inner.name.as_deref()
    }
}

#[stable(feature = "rust1", since = "1.0.0")]
impl fmt::Debug for Thread {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("Thread")
            .field("id", &self.id())
            .field("name", &self.name())
            .finish_non_exhaustive()
    }
}

////////////////////////////////////////////////////////////////////////////////
// JoinHandle
////////////////////////////////////////////////////////////////////////////////

/// A specialized [`Result`] type for threads. <br>线程专用的 [`Result`] 的类型。<br>
///
/// Indicates the manner in which a thread exited. <br>指示线程退出的方式。<br>
///
/// The value contained in the `Result::Err` variant is the value the thread panicked with; <br>`Result::Err` 变体中包含的值是线程 panic 时使用的值；<br>
/// that is, the argument the `panic!` macro was called with. <br>也就是说，调用了 `panic!` 宏的参数。<br>
/// Unlike with normal errors, this value doesn't implement the [`Error`](crate::error::Error) trait. <br>与正常错误不同，此值不实现 [`Error`](crate::error::Error) trait。<br>
///
/// Thus, a sensible way to handle a thread panic is to either: <br>因此，处理线程 panic 的明智方法是：<br>
///
/// 1. propagate the panic with [`std::panic::resume_unwind`] <br>用 [`std::panic::resume_unwind`] 传播 panic<br>
/// 2.
/// or in case the thread is intended to be a subsystem boundary that is supposed to isolate system-level failures, match on the `Err` variant and handle the panic in an appropriate way <br>或者，如果线程是用来隔离系统级故障的子系统边界，则匹配 `Err` 变体，并以适当的方式处理 panic<br>
///
///
/// A thread that completes without panicking is considered to exit successfully. <br>一个线程在没有 panic 的情况下完成被认为是成功退出。<br>
///
/// # Examples
///
/// Matching on the result of a joined thread: <br>匹配已连接线程的结果：<br>
///
/// ```no_run
/// use std::{fs, thread, panic};
///
/// fn copy_in_thread() -> thread::Result<()> {
///     thread::spawn(|| {
///         fs::copy("foo.txt", "bar.txt").unwrap();
///     }).join()
/// }
///
/// fn main() {
///     match copy_in_thread() {
///         Ok(_) => println!("copy succeeded"),
///         Err(e) => panic::resume_unwind(e),
///     }
/// }
/// ```
///
/// [`Result`]: crate::result::Result
/// [`std::panic::resume_unwind`]: crate::panic::resume_unwind
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub type Result<T> = crate::result::Result<T, Box<dyn Any + Send + 'static>>;

// This packet is used to communicate the return value between the spawned thread and the rest of the program. <br>该数据包用于在程序的新建线程和对象部分之间传递返回值。<br>
// It is shared through an `Arc` and there's no need for a mutex here because synchronization happens with `join()` (the caller will never read this packet until the thread has exited). <br>它是通过 `Arc` 共享的，这里不需要互斥锁，因为同步发生在 `join()` 上 (调用者在线程退出之前，永远不会读取这个数据包)。<br>
//
//
// An Arc to the packet is stored into a `JoinInner` which in turns is placed in `JoinHandle`. <br>数据包的 Arc 存储到 `JoinInner` 中，而 `JoinInner` 又放置在 `JoinHandle` 中。<br>
//
//
struct Packet<'scope, T> {
    scope: Option<Arc<scoped::ScopeData>>,
    result: UnsafeCell<Option<Result<T>>>,
    _marker: PhantomData<Option<&'scope scoped::ScopeData>>,
}

// Due to the usage of `UnsafeCell` we need to manually implement Sync. <br>由于使用了 `UnsafeCell`，我们需要手动实现 Sync。<br>
// The type `T` should already always be Send (otherwise the thread could not have been created) and the Packet is Sync because all access to the `UnsafeCell` synchronized (by the `join()` boundary), and `ScopeData` is Sync. <br>`T` 类型应该一直是 Send (否则无法创建线程) 并且 Packet 是 Sync，因为所有对 `UnsafeCell` 的访问都是同步的 (通过 `join()` 边界)，并且 `ScopeData` 是 Sync。<br>
//
//
unsafe impl<'scope, T: Sync> Sync for Packet<'scope, T> {}

impl<'scope, T> Drop for Packet<'scope, T> {
    fn drop(&mut self) {
        // If this packet was for a thread that ran in a scope, the thread panicked, and nobody consumed the panic payload, we make sure the scope function will panic. <br>如果此数据包是针对在作用域中运行的线程，该线程发生了 panic，并且没有人使用 panic 的有效载荷，我们确保作用域函数也发生 panic。<br>
        //
        //
        let unhandled_panic = matches!(self.result.get_mut(), Some(Err(_)));
        // Drop the result without causing unwinding. <br>在不导致展开的情况下丢弃结果。<br>
        // This is only relevant for threads that aren't join()ed, as join() will take the `result` and set it to None, such that there is nothing left to drop here. <br>这仅与未加入 () 的线程有关，因为 join() 将采用 `result` 并将其设置为 None，这样这里就没有任何东西可以丢弃了。<br>
        //
        // If this panics, we should handle that, because we're outside the outermost `catch_unwind` of our thread. <br>如果出现这种情况，我们应该处理它，因为我们在线程的最外层 `catch_unwind` 之外。<br>
        // We just abort in that case, since there's nothing else we can do. <br>在那种情况下，我们只是终止，因为我们无能为力。<br>
        // (And even if we tried to handle it somehow, we'd also need to handle the case where the panic payload we get out of it also panics on drop, and so on. <br>(即使我们试图以某种方式处理它，我们也需要处理我们从中得到的 panic 有效，载荷，在丢弃时也会 panic 的情况，依此类推。<br>
        // See issue #86027.) <br>请参见问题 #86027。)<br>
        //
        //
        //
        if let Err(_) = panic::catch_unwind(panic::AssertUnwindSafe(|| {
            *self.result.get_mut() = None;
        })) {
            rtabort!("thread result panicked on drop");
        }
        // Book-keeping so the scope knows when it's done. <br>簿记，以便作用域知道何时完成。<br>
        if let Some(scope) = &self.scope {
            // Now that there will be no more user code running on this thread that can use 'scope, mark the thread as 'finished'. <br>现在将不再有用户代码在此线程上运行可以使用 'scope，将线程标记为 'finished'。<br>
            // It's important we only do this after the `result` has been dropped, since dropping it might still use things it borrowed from 'scope. <br>重要的是我们只在 `result` 被丢弃之后才这样做，因为丢弃它可能仍然使用从 'scope 借来的东西<br>
            //
            //
            scope.decrement_num_running_threads(unhandled_panic);
        }
    }
}

/// Inner representation for JoinHandle <br>JoinHandle 的内部表示<br>
struct JoinInner<'scope, T> {
    native: imp::Thread,
    thread: Thread,
    packet: Arc<Packet<'scope, T>>,
}

impl<'scope, T> JoinInner<'scope, T> {
    fn join(mut self) -> Result<T> {
        self.native.join();
        Arc::get_mut(&mut self.packet).unwrap().result.get_mut().take().unwrap()
    }
}

/// An owned permission to join on a thread (block on its termination). <br>拥有加入线程的权限 (在线程终止时阻止)。<br>
///
/// A `JoinHandle` *detaches* the associated thread when it is dropped, which means that there is no longer any handle to the thread and no way to `join` on it. <br>当 `JoinHandle` 被丢弃时，它会*分离*关联的线程，这意味着不再有线程的任何句柄，也无法在其上使用 `join`。<br>
///
///
/// Due to platform restrictions, it is not possible to [`Clone`] this handle: the ability to join a thread is a uniquely-owned permission. <br>由于平台的限制，无法使用 [`Clone`] 此句柄：加入线程的能力是唯一拥有的权限。<br>
///
/// This `struct` is created by the [`thread::spawn`] function and the [`thread::Builder::spawn`] method. <br>该 `struct` 由 [`thread::spawn`] 函数和 [`thread::Builder::spawn`] 方法创建。<br>
///
/// # Examples
///
/// Creation from [`thread::spawn`]: <br>从 [`thread::spawn`] 创建：<br>
///
/// ```
/// use std::thread;
///
/// let join_handle: thread::JoinHandle<_> = thread::spawn(|| {
///     // some work here <br>这里一些工作<br>
/// });
/// ```
///
/// Creation from [`thread::Builder::spawn`]: <br>从 [`thread::Builder::spawn`] 创建：<br>
///
/// ```
/// use std::thread;
///
/// let builder = thread::Builder::new();
///
/// let join_handle: thread::JoinHandle<_> = builder.spawn(|| {
///     // some work here <br>这里一些工作<br>
/// }).unwrap();
/// ```
///
/// A thread being detached and outliving the thread that spawned it: <br>一个线程被分离并且比产生它的线程生命周期更长：<br>
///
/// ```no_run
/// use std::thread;
/// use std::time::Duration;
///
/// let original_thread = thread::spawn(|| {
///     let _detached_thread = thread::spawn(|| {
///         // Here we sleep to make sure that the first thread returns before. <br>在这里我们睡觉以确保第一个线程在此之前返回。<br>
///         thread::sleep(Duration::from_millis(10));
///         // This will be called, even though the JoinHandle is dropped. <br>即使 JoinHandle 被丢弃，也将调用它。<br>
///         println!("♫ Still alive ♫");
///     });
/// });
///
/// original_thread.join().expect("The thread being joined has panicked");
/// println!("Original thread is joined.");
///
/// // We make sure that the new thread has time to run, before the main thread returns. <br>我们确保在主线程返回之前，新线程有时间运行。<br>
/////
///
/// thread::sleep(Duration::from_millis(1000));
/// ```
///
/// [`thread::Builder::spawn`]: Builder::spawn
/// [`thread::spawn`]: spawn
///
///
///
#[stable(feature = "rust1", since = "1.0.0")]
pub struct JoinHandle<T>(JoinInner<'static, T>);

#[stable(feature = "joinhandle_impl_send_sync", since = "1.29.0")]
unsafe impl<T> Send for JoinHandle<T> {}
#[stable(feature = "joinhandle_impl_send_sync", since = "1.29.0")]
unsafe impl<T> Sync for JoinHandle<T> {}

impl<T> JoinHandle<T> {
    /// Extracts a handle to the underlying thread. <br>提取底层线程的句柄。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::thread;
    ///
    /// let builder = thread::Builder::new();
    ///
    /// let join_handle: thread::JoinHandle<_> = builder.spawn(|| {
    ///     // some work here <br>这里一些工作<br>
    /// }).unwrap();
    ///
    /// let thread = join_handle.thread();
    /// println!("thread id: {:?}", thread.id());
    /// ```
    #[stable(feature = "rust1", since = "1.0.0")]
    #[must_use]
    pub fn thread(&self) -> &Thread {
        &self.0.thread
    }

    /// Waits for the associated thread to finish. <br>等待关联的线程完成。<br>
    ///
    /// This function will return immediately if the associated thread has already finished. <br>如果关联的线程已经完成，这个函数将立即返回。<br>
    ///
    /// In terms of [atomic memory orderings],  the completion of the associated thread synchronizes with this function returning. <br>就 [原子内存排序][atomic memory orderings] 而言，关联线程的完成与此函数返回同步。<br>
    /// In other words, all operations performed by that thread [happen before](https://doc.rust-lang.org/nomicon/atomics.html#data-accesses) all operations that happen after `join` returns. <br>换句话说，该线程 [之前发生过](https://doc.rust-lang.org/nomicon/atomics.html#data-accesses) 执行的所有操作都是在 `join` 返回之后发生的所有操作。<br>
    ///
    ///
    /// If the associated thread panics, [`Err`] is returned with the parameter given to [`panic!`]. <br>如果关联的线程 panics，则返回 [`Err`]，并返回给 [`panic!`] 的参数。<br>
    ///
    /// [`Err`]: crate::result::Result::Err
    /// [atomic memory orderings]: crate::sync::atomic
    ///
    /// # Panics
    ///
    /// This function may panic on some platforms if a thread attempts to join itself or otherwise may create a deadlock with joining threads. <br>如果某个线程尝试加入自身，则该函数在某些平台上可能为 panic，否则可能会在加入线程时产生死锁。<br>
    ///
    /// # Examples
    ///
    /// ```
    /// use std::thread;
    ///
    /// let builder = thread::Builder::new();
    ///
    /// let join_handle: thread::JoinHandle<_> = builder.spawn(|| {
    ///     // some work here <br>这里一些工作<br>
    /// }).unwrap();
    /// join_handle.join().expect("Couldn't join on the associated thread");
    /// ```
    ///
    ///
    ///
    ///
    #[stable(feature = "rust1", since = "1.0.0")]
    pub fn join(self) -> Result<T> {
        self.0.join()
    }

    /// Checks if the associated thread has finished running its main function. <br>检查关联线程是否已完成其 main 函数的运行。<br>
    ///
    /// `is_finished` supports implementing a non-blocking join operation, by checking `is_finished`, and calling `join` if it returns `true`. <br>`is_finished` 支持实现非阻塞连接操作，通过检查 `is_finished`，如果返回 `true` 则调用 `join`。<br>
    /// This function does not block. <br>该函数不会阻止。<br>
    /// To block while waiting on the thread to finish, use [`join`][Self::join]. <br>要在等待线程完成时阻塞，请使用 [`join`][Self::join]。<br>
    ///
    /// This might return `true` for a brief moment after the thread's main function has returned, but before the thread itself has stopped running. <br>这可能在线程的 main 函数返回之后，但在线程本身停止运行之前短暂返回 `true`。<br>
    /// However, once this returns `true`, [`join`][Self::join] can be expected to return quickly, without blocking for any significant amount of time. <br>但是，一旦返回 `true`，可以预期 [`join`][Self::join] 会快速返回，而不会被阻塞很长时间。<br>
    ///
    ///
    #[stable(feature = "thread_is_running", since = "1.61.0")]
    pub fn is_finished(&self) -> bool {
        Arc::strong_count(&self.0.packet) == 1
    }
}

impl<T> AsInner<imp::Thread> for JoinHandle<T> {
    fn as_inner(&self) -> &imp::Thread {
        &self.0.native
    }
}

impl<T> IntoInner<imp::Thread> for JoinHandle<T> {
    fn into_inner(self) -> imp::Thread {
        self.0.native
    }
}

#[stable(feature = "std_debug", since = "1.16.0")]
impl<T> fmt::Debug for JoinHandle<T> {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("JoinHandle").finish_non_exhaustive()
    }
}

fn _assert_sync_and_send() {
    fn _assert_both<T: Send + Sync>() {}
    _assert_both::<JoinHandle<()>>();
    _assert_both::<Thread>();
}

/// Returns an estimate of the default amount of parallelism a program should use. <br>返回程序应该使用的默认并行量的估计值。<br>
///
/// Parallelism is a resource. <br>并行性是一种资源。<br> A given machine provides a certain capacity for parallelism, i.e., a bound on the number of computations it can perform simultaneously. <br>给定的机器提供一定的并行能力，即它可以同时执行的计算数量的界限。<br> This number often corresponds to the amount of CPUs a computer has, but it may diverge in various cases. <br>这个数字通常与计算机拥有的 CPU 数量相对应，但在不同情况下可能会有所不同。<br>
///
/// Host environments such as VMs or container orchestrators may want to restrict the amount of parallelism made available to programs in them. <br>虚拟机或容器编排器等主机环境可能希望限制其中的程序可用的并行量。<br>
/// This is often done to limit the potential impact of (unintentionally) resource-intensive programs on other programs running on the same machine. <br>这样做通常是为了限制 (非故意地) 资源密集型程序对同一台机器上运行的其他程序的潜在影响。<br>
///
/// # Limitations
///
/// The purpose of this API is to provide an easy and portable way to query the default amount of parallelism the program should use. <br>此 API 的目的是提供一种简单且可移植的方式来查询程序应使用的默认并行量。<br>
/// Among other things it does not expose information on NUMA regions, does not account for differences in (co)processor capabilities or current system load, and will not modify the program's global state in order to more accurately query the amount of available parallelism. <br>除其他事项外，它不公开有关 NUMA 区域的信息，不考虑 (协同) 处理器功能或当前系统负载的差异，并且不会为了更准确地查询可用并行度的数量而修改程序的整体状态。<br>
///
///
/// Where both fixed steady-state and burst limits are available the steady-state capacity will be used to ensure more predictable latencies. <br>在固定稳态和突发限制都可用的情况下，稳态容量将用于确保更可预测的延迟。<br>
///
/// Resource limits can be changed during the runtime of a program, therefore the value is not cached and instead recomputed every time this function is called. <br>资源限制可以在程序运行时更改，因此该值不会被缓存，而是在每次调用此函数时重新计算。<br> It should not be called from hot code. <br>不应从热代码中调用它。<br>
///
/// The value returned by this function should be considered a simplified approximation of the actual amount of parallelism available at any given time. <br>这个函数返回的值应该被认为是任何给定时间可用的实际并行量的简化近似值。<br>
/// To get a more detailed or precise overview of the amount of parallelism available to the program, you may wish to use platform-specific APIs as well. <br>要更详细或更准确地了解程序可用的并行量，您可能还希望使用特定于平台的 API。<br>
/// The following platform limitations currently apply to `available_parallelism`: <br>以下平台限制目前适用于 `available_parallelism`：<br>
///
/// On Windows: <br>在 Windows 上：<br>
/// - It may undercount the amount of parallelism available on systems with more than 64 logical CPUs. <br>它可能会低估具有超过 64 个逻辑 CPU 的系统上可用的并行量。<br>
/// However, programs typically need specific support to take advantage of more than 64 logical CPUs, and in the absence of such support, the number returned by this function accurately reflects the number of logical CPUs the program can use by default. <br>但是，程序通常需要特定的支持才能利用超过 64 个逻辑 CPU，而在没有这种支持的情况下，此函数返回的数字准确地反映了程序默认可以使用的逻辑 CPU 的数量。<br>
/// - It may overcount the amount of parallelism available on systems limited by process-wide affinity masks, or job object limitations. <br>它可能会过度计算系统上可用的并行量，这些并行量受到进程范围的相似性屏蔽或作业对象限制的限制。<br>
///
/// On Linux: <br>在 Linux 上：<br>
/// - It may overcount the amount of parallelism available when limited by a process-wide affinity mask or cgroup quotas and cgroup2 fs or `sched_getaffinity()` can't be queried, e.g. due to sandboxing. <br>当受进程范围的关联掩码或 cgroup 配额限制，并且无法查询 cgroup2 fs 或 `sched_getaffinity()` (例如由于沙盒) 时，它可能会高估可用的并行数量。<br>
/// - It may undercount the amount of parallelism if the current thread's affinity mask does not reflect the process' cpuset, e.g. due to pinned threads. <br>如果当前线程的关联掩码没有反映进程的 cpuset，例如由于固定线程，它可能会低估并行的数量。<br>
///
/// On all targets: <br>在所有目标上：<br>
/// - It may overcount the amount of parallelism available when running in a VM with CPU usage limits (e.g. an overcommitted host). <br>当在具有 CPU 使用限制的 VM 中运行时 (例如过载的主机)，它可能会过度计算可用的并行量。<br>
///
/// # Errors
///
/// This function will, but is not limited to, return errors in the following cases: <br>在以下情况下，此函数将返回错误，但不限于此：<br>
///
/// - If the amount of parallelism is not known for the target platform. <br>如果目标平台的并行量未知。<br>
/// - If the program lacks permission to query the amount of parallelism made available to it. <br>如果程序没有查询可用的并行量的权限。<br>
///
/// # Examples
///
/// ```
/// # #![allow(dead_code)]
/// use std::{io, thread};
///
/// fn main() -> io::Result<()> {
///     let count = thread::available_parallelism()?.get();
///     assert!(count >= 1_usize);
///     Ok(())
/// }
/// ```
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
///
#[doc(alias = "available_concurrency")] // Alias for a previous name we gave this API on unstable. <br>我们为这个 API 提供了不稳定的前一个名称的别名。<br>
#[doc(alias = "hardware_concurrency")] // Alias for C++ `std::thread::hardware_concurrency`. <br>C++ `std::thread::hardware_concurrency` 的别名。<br>
#[doc(alias = "num_cpus")] // Alias for a popular ecosystem crate which provides similar functionality. <br>提供类似功能的流行生态系统 crate 的别名。<br>
#[stable(feature = "available_parallelism", since = "1.59.0")]
pub fn available_parallelism() -> io::Result<NonZeroUsize> {
    imp::available_parallelism()
}
