// 引用: ACLE 的 5.4.4 "LDREX / STREX" 部分

/// 删除由 LDREX 创建的排他锁
// 支持: v6，v6K，v7-M，v7-A，v7-R 不支持: v5，v6-M
// NOTE: v6 (<v6k) 中没有专用的 CLREX 指令； 清除独占监视器用户必须执行虚拟 STREX 操作
//
//
#[cfg(any(
    all(target_feature = "v6k", not(target_feature = "mclass")), // 不包括 v6-M
    all(target_feature = "v7", target_feature = "mclass"), // v7-M
    doc
))]
pub unsafe fn __clrex() {
    extern "unadjusted" {
        #[link_name = "llvm.arm.clrex"]
        fn clrex();
    }

    clrex()
}

/// 对 8 位值执行独占 LDR 指令。
// 支持: v6K，v7-M，v7-A，v7-R 不支持: v5，v6，v6-M
//
#[cfg(any(
    target_feature = "v6k", // 包括 v7-M，但不包括 v6-M
    doc
))]
pub unsafe fn __ldrexb(p: *const u8) -> u8 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.ldrex.p0i8"]
        fn ldrex8(p: *const u8) -> u32;
    }

    ldrex8(p) as u8
}

/// 对 16 位值执行独占 LDR 指令。
// 支持: v6K，v7-M，v7-A，v7-R，v8 不支持: v5，v6，v6-M
//
#[cfg(any(
    target_feature = "v6k", // 包括 v7-M，但不包括 v6-M
    doc
))]
pub unsafe fn __ldrexh(p: *const u16) -> u16 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.ldrex.p0i16"]
        fn ldrex16(p: *const u16) -> u32;
    }

    ldrex16(p) as u16
}

/// 对 32 位值执行独占 LDR 指令。
// 支援: v6，v7-M，v6K，v7-A，v7-R，v8 不支援: v5，v6-M
//
#[cfg(any(
    all(target_feature = "v6", not(target_feature = "mclass")), // 不包括 v6-M
    all(target_feature = "v7", target_feature = "mclass"), // v7-M
    doc
))]
pub unsafe fn __ldrex(p: *const u32) -> u32 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.ldrex.p0i32"]
        fn ldrex32(p: *const u32) -> u32;
    }

    ldrex32(p)
}

/// 对 8 位值执行独占 STR 指令
///
/// 如果操作成功，则返回 `0`; 如果操作失败，则返回 `1`
// 支持: v6K、v7-M、v7-A、v7-R 不支持: v5、v6、v6-M
//
#[cfg(any(
    target_feature = "v6k", // 包括 v7-M，但不包括 v6-M
    doc
))]
pub unsafe fn __strexb(value: u32, addr: *mut u8) -> u32 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.strex.p0i8"]
        fn strex8(value: u32, addr: *mut u8) -> u32;
    }

    strex8(value, addr)
}

/// 对 16 位值执行独占 STR 指令
///
/// 如果操作成功，则返回 `0`; 如果操作失败，则返回 `1`
// 支持: v6K，v7-M，v7-A，v7-R，v8 不支持: v5，v6，v6-M
//
#[cfg(target_feature = "aarch64")]
#[cfg(any(
    target_feature = "v6k", // 包括 v7-M，但不包括 v6-M
    doc
))]
pub unsafe fn __strexh(value: u16, addr: *mut u16) -> u32 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.strex.p0i16"]
        fn strex16(value: u32, addr: *mut u16) -> u32;
    }

    strex16(value as u32, addr)
}

/// 对 32 位值执行独占 STR 指令
///
/// 如果操作成功，则返回 `0`; 如果操作失败，则返回 `1`
// 支援: v6，v7-M，v6K，v7-A，v7-R，v8 不支援: v5，v6-M
//
#[cfg(any(
    all(target_feature = "v6", not(target_feature = "mclass")), // 不包括 v6-M
    all(target_feature = "v7", target_feature = "mclass"), // v7-M
    doc
))]
pub unsafe fn __strex(value: u32, addr: *mut u32) -> u32 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.strex.p0i32"]
        fn strex32(value: u32, addr: *mut u32) -> u32;
    }

    strex32(value, addr)
}
