//! ARM intrinsics. <br>ARM 内部函数。<br>
//!
//! The reference for NEON is [ARM's NEON Intrinsics Reference][arm_ref]. <br>NEON 的引用为 [ARM 的 NEON 内部函数参考][arm_ref]。<br>
//! The [ARM's NEON Intrinsics Online Database][arm_dat] is also useful. <br>[ARM 的 NEON Intrinsics 在线数据库][arm_dat] 也很有用。<br>
//!
//! [arm_ref]: http://infocenter.arm.com/help/topic/com.arm.doc.ihi0073a/IHI0073A_arm_neon_intrinsics_ref.pdf
//! [arm_dat]: https://developer.arm.com/technologies/neon/intrinsics

mod armclang;
pub use self::armclang::*;

mod v6;
pub use self::v6::*;

// Supported arches: 6, 7-M. <br>支持的 arches: 6、7-M。<br> See Section 10.1 of ACLE (e.g. SSAT) <br>请参见 ACLE 的 10.1 部分 (例如 SSAT)<br>
#[cfg(any(target_feature = "v6", doc))]
mod sat;

#[cfg(any(target_feature = "v6", doc))]
pub use self::sat::*;

// Supported arches: 5TE, 7E-M. <br>支持的 arches: 5TE，7E-M。<br> See Section 10.1 of ACLE (e.g. <br>参见 ACLE 的 10.1 部分 (例如<br>
// QADD) We also include the A profile even though DSP is deprecated on that profile as of ACLE 2.0 (see section 5.4.7) Here we workaround the difference between LLVM's +dsp and ACLE's __ARM_FEATURE_DSP by gating on '+v5te' rather than on '+dsp' <br>QADD) 即使从 ACLE 2.0 开始不赞成使用 DSP，我们也包括 A 配置文件 (请参见 5.4.7 部分)。在这里，我们通过在 '+v5te' 而非 '+dsp' 上进行选通来解决 LLVM 的 + dsp 和 ACLE 的 __ARM_FEATURE_DSP 之间的差异<br>
//
//
//
#[cfg(any(
    // >= v5TE but excludes v7-M <br>v5TE，但不包括 v7-M<br>
    all(target_feature = "v5te", not(target_feature = "mclass")),
    // v7E-M
    all(target_feature = "mclass", target_feature = "dsp"),
    doc,
))]
pub mod dsp;

#[cfg(any(
    // >= v5TE but excludes v7-M <br>v5TE，但不包括 v7-M<br>
    all(target_feature = "v5te", not(target_feature = "mclass")),
    // v7E-M
    all(target_feature = "mclass", target_feature = "dsp"),
    doc,
))]
pub use self::dsp::*;

// Deprecated in ACLE 2.0 for the A profile but fully supported on the M and R profiles, says Section 5.4.9 of ACLE. <br>ACLE 的 5.4.9 部分说，在 ACLE 2.0 中已不赞成 A 配置文件，但在 M 和 R 配置文件中完全支持。<br>
// We'll expose these for the A profile even if deprecated <br>即使不推荐使用，我们也会在 A 个人资料中公开这些内容<br>
#[cfg(any(
    // v7-A, v7-R
    all(target_feature = "v6", not(target_feature = "mclass")),
    // v7E-M
    all(target_feature = "mclass", target_feature = "dsp"),
    doc,
))]
mod simd32;

#[cfg(any(
    // v7-A, v7-R
    all(target_feature = "v6", not(target_feature = "mclass")),
    // v7E-M
    all(target_feature = "mclass", target_feature = "dsp"),
    doc,
))]
pub use self::simd32::*;

#[cfg(any(target_feature = "v7", doc))]
mod v7;
#[cfg(any(target_feature = "v7", doc))]
pub use self::v7::*;

mod ex;
pub use self::ex::*;

pub use crate::core_arch::arm_shared::*;

#[cfg(test)]
use stdarch_test::assert_instr;

#[cfg(any(target_feature = "v7", doc))]
pub(crate) mod neon;
#[cfg(any(target_feature = "v7", doc))]
pub use neon::*;

/// Generates the trap instruction `UDF` <br>生成陷阱指令 `UDF`<br>
#[cfg(target_arch = "arm")]
#[cfg_attr(test, assert_instr(udf))]
#[inline]
pub unsafe fn udf() -> ! {
    crate::intrinsics::abort()
}

/// Generates a DBG instruction. <br>生成 DBG 指令。<br>
///
/// This provides a hint to debugging and related systems. <br>这为调试和相关系统提供了提示。<br>
/// The argument must be a constant integer from 0 to 15 inclusive. <br>参数必须是 0 到 15 (含) 之间的常量整数。<br>
/// See implementation documentation for the effect (if any) of this instruction and the meaning of the argument. <br>有关此指令的效果 (如果有) 和参数的含义，请参见实现文档。<br>
/// This is available only when compiling for AArch32. <br>这仅在为 AArch32 编译时可用。<br>
// Section 10.1 of ACLE says that the supported arches are: 7, 7-M "The DBG hint instruction is added in ARMv7. <br>ACLE 的 10.1 部分说，支持的 arches 是: 7、7-M` 在 ARMv7 中添加了 DBG 提示指令。<br>
// It is UNDEFINED in the ARMv6 base architecture, and executes as a NOP instruction in ARMv6K and ARMv6T2." - ARM Architecture Reference Manual ARMv7-A and ARMv7-R edition (ARM DDI 0406C.c) sections D12.4.1 "ARM instruction set support" and D12.4.2 "Thumb instruction set support" <br>它在 ARMv6 基础架构中未定义，在 ARMv6K 和 ARMv6T2 中作为 NOP 指令执行。`-ARM 架构引用手册 ARMv7-A 和 ARMv7-R 版 (ARM DDI 0406C.c) 部分 D12.4.1 "ARM 指令集支持" 和 D12.4.2 "Thumb 指令集支持"<br>
//
//
//
#[cfg(any(target_feature = "v7", doc))]
#[inline(always)]
#[rustc_legacy_const_generics(0)]
pub unsafe fn __dbg<const IMM4: i32>() {
    static_assert_imm4!(IMM4);
    dbg(IMM4);
}

extern "unadjusted" {
    #[link_name = "llvm.arm.dbg"]
    fn dbg(_: i32);
}
