// # References
//
// - Section 7.4 "Hints" of ACLE <br>ACLE 第 7.4 节 "Hints" 部分<br>
// - Section 7.7 "NOP" of ACLE <br>ACLE 第 7.7 节 "NOP" 部分<br>

/// Generates a WFI (wait for interrupt) hint instruction, or nothing. <br>生成 WFI (等待中断) 提示指令，或不执行任何操作。<br>
///
/// The WFI instruction allows (but does not require) the processor to enter a low-power state until one of a number of asynchronous events occurs. <br>WFI 指令允许 (但不要求) 处理器进入低功耗状态，直到发生许多异步事件之一。<br>
///
// Section 10.1 of ACLE says that the supported arches are: 8, 6K, 6-M LLVM says "instruction requires: armv6k" <br>ACLE 的 10.1 部分说受支持的 arches 是: 8、6K，6-M LLVM 说 "instruction requires: armv6k"<br>
//
#[cfg(any(target_feature = "v6", target_arch = "aarch64", doc))]
#[inline(always)]
pub unsafe fn __wfi() {
    hint(HINT_WFI);
}

/// Generates a WFE (wait for event) hint instruction, or nothing. <br>生成 WFE (等待事件) 提示指令，否则不生成任何内容。<br>
///
/// The WFE instruction allows (but does not require) the processor to enter a low-power state until some event occurs such as a SEV being issued by another processor. <br>WFE 指令允许 (但不要求) 处理器进入低功耗状态，直到发生某些事件 (例如，另一处理器发出 SEV)。<br>
///
///
// Section 10.1 of ACLE says that the supported arches are: 8, 6K, 6-M LLVM says "instruction requires: armv6k" <br>ACLE 的 10.1 部分说受支持的 arches 是: 8、6K，6-M LLVM 说 "instruction requires: armv6k"<br>
//
#[cfg(any(target_feature = "v6", target_arch = "aarch64", doc))]
#[inline(always)]
pub unsafe fn __wfe() {
    hint(HINT_WFE);
}

/// Generates a SEV (send a global event) hint instruction. <br>生成 SEV (发送事件) 提示指令。<br>
///
/// This causes an event to be signaled to all processors in a multiprocessor system. <br>这导致事件被发送到多处理器系统中的所有处理器。<br>
/// It is a NOP on a uniprocessor system. <br>它是单处理器系统上的 NOP。<br>
// Section 10.1 of ACLE says that the supported arches are: 8, 6K, 6-M, 7-M LLVM says "instruction requires: armv6k" <br>ACLE 的 10.1 部分说受支持的 arches 是: 8，6K，6-M，7-M LLVM 说 "instruction requires: armv6k"<br>
//
#[cfg(any(target_feature = "v6", target_arch = "aarch64", doc))]
#[inline(always)]
pub unsafe fn __sev() {
    hint(HINT_SEV);
}

/// Generates a send a local event hint instruction. <br>生成发送本地事件提示指令。<br>
///
/// This causes an event to be signaled to only the processor executing this instruction. <br>这导致仅将信号通知执行该指令的处理器。<br>
/// In a multiprocessor system, it is not required to affect the other processors. <br>在多处理器系统中，不需要影响其他处理器。<br>
///
// LLVM says "instruction requires: armv8" <br>LLVM 说 "指令要求: armv8"<br>
#[cfg(any(
    target_feature = "v8", // 32-bit ARMv8 <br>32 位 ARMv8<br>
    target_arch = "aarch64", // AArch64
    doc,
))]
#[inline(always)]
pub unsafe fn __sevl() {
    hint(HINT_SEVL);
}

/// Generates a YIELD hint instruction. <br>生成一条 YIELD 提示指令。<br>
///
/// This enables multithreading software to indicate to the hardware that it is performing a task, for example a spin-lock, that could be swapped out to improve overall system performance. <br>这使多线程软件能够向硬件指示其正在执行任务 (例如自旋锁)，可以将其换出以提高整体系统性能。<br>
///
///
// Section 10.1 of ACLE says that the supported arches are: 8, 6K, 6-M LLVM says "instruction requires: armv6k" <br>ACLE 的 10.1 部分说受支持的 arches 是: 8、6K，6-M LLVM 说 "instruction requires: armv6k"<br>
//
#[cfg(any(target_feature = "v6", target_arch = "aarch64", doc))]
#[inline(always)]
pub unsafe fn __yield() {
    hint(HINT_YIELD);
}

/// Generates an unspecified no-op instruction. <br>生成未指定的无操作指令。<br>
///
/// Note that not all architectures provide a distinguished NOP instruction. <br>请注意，并非所有体系结构都提供专有的 NOP 指令。<br>
/// On those that do, it is unspecified whether this intrinsic generates it or another instruction. <br>在执行此操作的对象上，不确定此内部函数是生成它还是其他指令。<br>
///
/// It is not guaranteed that inserting this instruction will increase execution time. <br>不保证插入该指令会增加执行时间。<br>
#[inline(always)]
pub unsafe fn __nop() {
    crate::arch::asm!("nop", options(nomem, nostack, preserves_flags));
}

extern "unadjusted" {
    #[cfg_attr(target_arch = "aarch64", link_name = "llvm.aarch64.hint")]
    #[cfg_attr(target_arch = "arm", link_name = "llvm.arm.hint")]
    fn hint(_: i32);
}

// from LLVM 7.0.1's lib/Target/ARM/{ARMInstrThumb,ARMInstrInfo,ARMInstrThumb2}.td <br>来自 LLVM 7.0.1 的 lib/Target/ 分支 /{ARMInstrThumb,ARMInstrInfo,ARMInstrThumb2}.td<br>
const HINT_NOP: i32 = 0;
const HINT_YIELD: i32 = 1;
const HINT_WFE: i32 = 2;
const HINT_WFI: i32 = 3;
const HINT_SEV: i32 = 4;
const HINT_SEVL: i32 = 5;
