//! ARM C Language Extensions (ACLE) <br>ARM C 语言扩展 (ACLE)<br>
//!
//! # Developer notes <br>开发人员注意事项<br>
//!
//! Below is a list of built-in targets that are representative of the different ARM architectures; <br>以下是代表不同 ARM 体系结构的内置目标的列表；<br> the list includes the `target_feature`s they possess. <br>该列表包括他们拥有的 `target_feature`。<br>
//!
//!
//! - `armv4t-unknown-linux-gnueabi` - **ARMv4** - `+v4t`
//! - `armv5te-unknown-linux-gnueabi` - **ARMv5TE** - `+v4t +v5te`
//! - `arm-unknown-linux-gnueabi` - **ARMv6** - `+v4t +v5te +v6`
//! - `thumbv6m-none-eabi` - **ARMv6-M** - `+v4t +v5te +v6 +thumb-mode +mclass`
//! - `armv7-unknown-linux-gnueabihf` - **ARMv7-A** - `+v4t +v5te +v6 +v6k +v6t2 +v7 +dsp +thumb2 +aclass`
//! - `armv7r-none-eabi` - **ARMv7-R** - `+v4t +v5te +v6 +v6k +v6t2  +v7 +dsp +thumb2 +rclass`
//! - `thumbv7m-none-eabi` - **ARMv7-M** - `+v4t +v5te +v6 +v6k +v6t2 +v7 +thumb2 +thumb-mode +mclass`
//! - `thumbv7em-none-eabi` - **ARMv7E-M** - `+v4t +v5te +v6 +v6k +v6t2 +v7 +dsp +thumb2 +thumb-mode +mclass`
//! - `thumbv8m.main-none-eabi` - **ARMv8-M** - `+v4t +v5te +v6 +v6k +v6t2 +v7 +thumb2 +thumb-mode +mclass`
//! - `armv8r-none-eabi` - **ARMv8-R** - `+v4t +v5te +v6 +v6k +v6t2 +v7 +v8 +thumb2 +rclass`
//! - `aarch64-unknown-linux-gnu` - **ARMv8-A (AArch64)** - `+fp +neon`
//!
//! Section 10.1 of ACLE says: <br>ACLE 的第 10.1 节说：<br>
//!
//! - "In the sequence of Arm architectures { v5, v5TE, v6, v6T2, v7 } each architecture includes its predecessor instruction set."
//!
//! - "In the sequence of Thumb-only architectures { v6-M, v7-M, v7E-M } each architecture includes its predecessor instruction set."
//!
//! From that info and from looking at how LLVM features work (using custom targets) we can identify features that are subsets of others: <br>根据该信息以及查看 LLVM 特性的工作方式 (使用自定义目标)，我们可以识别属于其他特性的子集：<br>
//!
//! Legend: `a < b` reads as "`a` is a subset of `b`"; <br>图例: `a < b` 读为 "`a` 是 `b` 的子集"；<br> this means that if `b` is enabled then `a` is enabled as well. <br>这意味着如果启用了 `b`，那么还将启用 `a`。<br>
//!
//! - `v4t < v5te < v6 < v6k < v6t2 < v7 < v8`
//! - `v6 < v8m < v6t2`
//! - `v7 < v8m.main`
//!
//! *NOTE*: Section 5.4.7 of ACLE says: <br>ACLE 的第 5.4.7 节说：<br>
//!
//! - "__ARM_FEATURE_DSP is defined to 1 if the DSP (v5E) instructions are supported and the intrinsics defined in Saturating intrinsics are available."
//!
//! This does *not* match how LLVM uses the '+dsp' feature; <br>这与 LLVM 使用 '+dsp' 特性的方式不匹配。<br> this feature is not set for v5te targets so we have to work around this difference. <br>这个特性不是为 v5te 目标设置的，所以我们必须解决这个差异。<br>
//!
//! # References
//!
//! - [ACLE Q2 2018](https://developer.arm.com/docs/101028/latest)
//!
//!
//!
//!
//!
//!

// Only for 'neon' submodule <br>仅适用于 'neon' 子模块<br>
#![allow(non_camel_case_types)]

// 8, 7 and 6-M are supported via dedicated instructions like DMB. <br>通过专用指令 (如 DMB) 支持 8、7 和 6-M。<br> All other arches are supported via CP15 instructions. <br>通过 CP15 指令支持所有其他 arches。<br>
// See Section 10.1 of ACLE <br>参见 ACLE 的第 10.1 节<br>
mod barrier;

pub use self::barrier::*;

mod hints;
pub use self::hints::*;

mod registers;
pub use self::registers::*;

#[cfg(any(target_arch = "aarch64", target_feature = "v7", doc))]
mod crc;
#[cfg(any(target_arch = "aarch64", target_feature = "v7", doc))]
pub use crc::*;

#[cfg(any(target_arch = "aarch64", target_feature = "v7", doc))]
mod crypto;
#[cfg(any(target_arch = "aarch64", target_feature = "v7", doc))]
pub use self::crypto::*;

#[cfg(any(target_arch = "aarch64", target_feature = "v7", doc))]
pub(crate) mod neon;
#[cfg(any(target_arch = "aarch64", target_feature = "v7", doc))]
pub use self::neon::*;

#[cfg(test)]
#[cfg(any(target_arch = "aarch64", target_feature = "v7", doc))]
pub(crate) mod test_support;

mod sealed {
    pub trait Dmb {
        unsafe fn __dmb(&self);
    }

    pub trait Dsb {
        unsafe fn __dsb(&self);
    }

    pub trait Isb {
        unsafe fn __isb(&self);
    }

    pub trait Rsr {
        unsafe fn __rsr(&self) -> u32;
    }

    pub trait Rsr64 {
        unsafe fn __rsr64(&self) -> u64;
    }

    pub trait Rsrp {
        unsafe fn __rsrp(&self) -> *const u8;
    }

    pub trait Wsr {
        unsafe fn __wsr(&self, value: u32);
    }

    pub trait Wsr64 {
        unsafe fn __wsr64(&self, value: u64);
    }

    pub trait Wsrp {
        unsafe fn __wsrp(&self, value: *const u8);
    }
}
