//! `core_arch`

#[macro_use]
mod macros;

#[cfg(any(target_arch = "arm", target_arch = "aarch64", doc))]
mod arm_shared;

mod simd;

#[doc = include_str!("core_arch_docs.md")]
#[stable(feature = "simd_arch", since = "1.27.0")]
pub mod arch {
    /// Platform-specific intrinsics for the `x86` platform. <br>特定于平台的用于 `x86` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "x86", doc))]
    #[doc(cfg(target_arch = "x86"))]
    #[stable(feature = "simd_x86", since = "1.27.0")]
    pub mod x86 {
        #[stable(feature = "simd_x86", since = "1.27.0")]
        pub use crate::core_arch::x86::*;
    }

    /// Platform-specific intrinsics for the `x86_64` platform. <br>特定于平台的用于 `x86_64` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "x86_64", doc))]
    #[doc(cfg(target_arch = "x86_64"))]
    #[stable(feature = "simd_x86", since = "1.27.0")]
    pub mod x86_64 {
        #[stable(feature = "simd_x86", since = "1.27.0")]
        pub use crate::core_arch::x86::*;
        #[stable(feature = "simd_x86", since = "1.27.0")]
        pub use crate::core_arch::x86_64::*;
    }

    /// Platform-specific intrinsics for the `arm` platform. <br>特定于平台的用于 `arm` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "arm", doc))]
    #[doc(cfg(target_arch = "arm"))]
    #[unstable(feature = "stdsimd", issue = "27731")]
    pub mod arm {
        pub use crate::core_arch::arm::*;
    }

    /// Platform-specific intrinsics for the `aarch64` platform. <br>特定于平台的用于 `aarch64` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "aarch64", doc))]
    #[doc(cfg(target_arch = "aarch64"))]
    #[stable(feature = "neon_intrinsics", since = "1.59.0")]
    pub mod aarch64 {
        #[stable(feature = "neon_intrinsics", since = "1.59.0")]
        pub use crate::core_arch::aarch64::*;
    }

    /// Platform-specific intrinsics for the `riscv32` platform. <br>特定于平台的用于 `riscv32` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "riscv32", doc))]
    #[doc(cfg(any(target_arch = "riscv32")))]
    #[unstable(feature = "stdsimd", issue = "27731")]
    pub mod riscv32 {
        pub use crate::core_arch::riscv_shared::*;
    }

    /// Platform-specific intrinsics for the `riscv64` platform. <br>特定于平台的用于 `riscv64` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "riscv64", doc))]
    #[doc(cfg(any(target_arch = "riscv64")))]
    #[unstable(feature = "stdsimd", issue = "27731")]
    pub mod riscv64 {
        pub use crate::core_arch::riscv64::*;
        // RISC-V RV64 supports all RV32 instructions as well in current specifications (2022-01-05). <br>RISC-V RV64 支持当前规范 (2022-01-05) 中的所有 RV32 指令。<br>
        // Module `riscv_shared` includes instructions available under all RISC-V platforms, i.e. <br>模块 `riscv_shared` 包括所有 RISC-V 平台下可用的指令，即<br>
        // RISC-V RV32 instructions. <br>RISC-V RV32 指令。<br>
        pub use crate::core_arch::riscv_shared::*;
    }

    /// Platform-specific intrinsics for the `wasm32` platform. <br>特定于平台的用于 `wasm32` 平台的内部函数。<br>
    ///
    /// This module provides intrinsics specific to the WebAssembly architecture. <br>该模块提供特定于 WebAssembly 体系结构的内部函数。<br>
    /// Here you'll find intrinsics specific to WebAssembly that aren't otherwise surfaced somewhere in a cross-platform abstraction of `std`, and you'll also find functions for leveraging WebAssembly proposals such as [atomics] and [simd]. <br>在这里，您会发现特定于 WebAssembly 的内部函数，它们不会在 `std` 的跨平台抽象中以其他方式出现，您还会发现利用 WebAssembly 提案 (如 [原子性][atomics] 和 [simd]) 的函数。<br>
    ///
    /// Intrinsics in the `wasm32` module are modeled after the WebAssembly instructions that they represent. <br>`wasm32` 模块中的内部函数是根据它们所代表的 WebAssembly 指令建模的。<br> Most functions are named after the instruction they intend to correspond to, and the arguments/results correspond to the type signature of the instruction itself. <br>大多数函数都以它们想要对应的指令命名，arguments/results 对应指令本身的类型签名。<br>
    /// Stable WebAssembly instructions are [documented online][instrdoc]. <br>稳定的 WebAssembly 指令是 [在线文档][instrdoc]。<br>
    ///
    /// [instrdoc]: https://webassembly.github.io/spec/core/valid/instructions.html
    ///
    /// If a proposal is not yet stable in WebAssembly itself then the functions within this function may be unstable and require the nightly channel of Rust to use. <br>如果一个提案在 WebAssembly 本身中还不稳定，那么这个函数中的函数可能是不稳定的，需要使用 Rust 的夜间通道。<br> As the proposal itself stabilizes the intrinsics in this module should stabilize as well. <br>由于提案本身具有稳定性，该模块中的内部函数也应该是稳定的。<br>
    ///
    /// [atomics]: https://github.com/webassembly/threads
    /// [simd]: https://github.com/webassembly/simd
    ///
    /// See the [module documentation](../index.html) for general information about the `arch` module and platform intrinsics. <br>有关 `arch` 模块和平台内部函数的一般信息，请参见 [模块级文档](../index.html)。<br>
    ///
    /// ## Atomics
    ///
    /// The [threads proposal][atomics] for WebAssembly adds a number of instructions for dealing with multithreaded programs. <br>[线程提案][atomics] for WebAssembly 添加了许多有关处理多线程程序的指令。<br> Most instructions added in the [atomics] proposal are exposed in Rust through the `std::sync::atomic` module. <br>[atomics] 提案中添加的大多数指令都通过 `std::sync::atomic` 模块在 Rust 中公开。<br>
    /// Some instructions, however, don't have direct equivalents in Rust so they're exposed here instead. <br>但是，某些指令在 Rust 中没有直接等效项，因此它们在此处公开。<br>
    ///
    /// Note that the instructions added in the [atomics] proposal can work in either a context with a shared wasm memory and without. <br>请注意，在 [原子性][atomics] 提案中添加的指令可以在具有共享 wasm 内存和不具有共享 wasm 内存的上下文中工作。<br>
    /// These intrinsics are always available in the standard library, but you likely won't be able to use them too productively unless you recompile the standard library (and all your code) with `-Ctarget-feature=+atomics`. <br>这些内部函数始终在标准库中可用，但除非您使用 `-Ctarget-feature=+atomics` 重新编译标准库 (和所有代码)，否则您可能无法高效地使用它们。<br>
    ///
    /// It's also worth pointing out that multi-threaded WebAssembly and its story in Rust is still in a somewhat "early days" phase as of the time of this writing. <br>还值得指出的是，截至撰写本文时，多线程 WebAssembly 及其在 Rust 中的描述仍处于 "早期" 阶段。<br> Pieces should mostly work but it generally requires a good deal of manual setup. <br>件基本上可以工作，但它通常需要大量的手动设置。<br>
    /// At this time it's not as simple as "just call `std::thread::spawn`", but it will hopefully get there one day! <br>这个时候它不像 "只需调用 `std::thread::spawn`" 那么简单，但希望有一天它能被实现!<br>
    ///
    /// ## SIMD
    ///
    /// The [simd proposal][simd] for WebAssembly added a new `v128` type for a <br>WebAssembly 的 [simd 提案][simd] 添加了一个新的 `v128` 类型，用于<br>
    /// 128-bit SIMD register. <br>128 位 SIMD 寄存器。<br> It also added a large array of instructions to operate on the `v128` type to perform data processing. <br>它还添加了大量指令以对 `v128` 类型进行操作以执行数据处理。<br> Using SIMD on wasm is intended to be similar to as you would on `x86_64`, for example. <br>例如，在 wasm 上使用 SIMD 与在 `x86_64` 上使用类似。<br>
    /// You'd write a function such as: <br>您将编写一个函数，例如：<br>
    ///
    /// ```rust,ignore
    /// #[cfg(target_arch = "wasm32")]
    /// #[target_feature(enable = "simd128")]
    /// unsafe fn uses_simd() {
    ///     use std::arch::wasm32::*;
    ///     // ...
    /// }
    /// ```
    ///
    /// Unlike `x86_64`, however, WebAssembly does not currently have dynamic detection at runtime as to whether SIMD is supported (this is one of the motivators for the [conditional sections][condsections] and [feature detection] proposals, but that is still pretty early days). <br>然而，与 `x86_64` 不同的是，WebAssembly 目前没有在运行时动态检测是否支持 SIMD (这是 [条件部分][condsections] 和 [特性检测][feature detection] 提案的动机之一，但这还处于早期阶段)。<br>
    ///
    /// This means that your binary will either have SIMD and can only run on engines which support SIMD, or it will not have SIMD at all. <br>这意味着您的二进制文件将具有 SIMD，并且只能在支持 SIMD 的引擎上运行，或者根本不具有 SIMD。<br>
    /// For compatibility the standard library itself does not use any SIMD internally. <br>为了兼容，标准库本身内部未使用任何 SIMD。<br>
    /// Determining how best to ship your WebAssembly binary with SIMD is largely left up to you as it can can be pretty nuanced depending on your situation. <br>确定如何最好地将 WebAssembly 二进制文件与 SIMD 一起交付，很大程度上取决于您的情况，具体取决于您的情况。<br>
    ///
    /// [condsections]: https://github.com/webassembly/conditional-sections
    /// [feature detection]: https://github.com/WebAssembly/feature-detection
    ///
    /// To enable SIMD support at compile time you need to do one of two things: <br>要在编译时启用 SIMD 支持，您需要执行以下两项操作之一：<br>
    ///
    /// * First you can annotate functions with `#[target_feature(enable = "simd128")]`. <br>首先，您可以使用 `#[target_feature(enable = "simd128")]` 注解函数。<br> This causes just that one function to have SIMD support available to it, and intrinsics will get inlined as usual in this situation. <br>这将导致仅一个函数可以使用 SIMD 支持，并且在这种情况下，内部函数将像往常一样被内联。<br>
    ///
    /// * Second you can compile your program with `-Ctarget-feature=+simd128`. <br>其次，您可以使用 `-Ctarget-feature=+simd128` 编译程序。<br>
    ///   This compilation flag blanket enables SIMD support for your entire compilation. <br>该编译标志毯可为您的整个编译提供 SIMD 支持。<br> Note that this does not include the standard library unless you [recompile the standard library][buildstd]. <br>请注意，除非您使用 [重新编译标准库][buildstd]，否则这不包括标准库。<br>
    ///
    /// [buildstd]: https://doc.rust-lang.org/nightly/cargo/reference/unstable.html#build-std
    ///
    /// If you enable SIMD via either of these routes then you'll have a WebAssembly binary that uses SIMD instructions, and you'll need to ship that accordingly. <br>如果通过这两种途径之一启用 SIMD，那么您将拥有一个使用 SIMD 指令的 WebAssembly 二进制文件，并且需要相应地进行运送。<br>
    /// Also note that if you call SIMD intrinsics but don't enable SIMD via either of these mechanisms, you'll still have SIMD generated in your program. <br>还要注意，如果您调用 SIMD 内部函数但未通过这两种机制之一启用 SIMD，则程序中仍将生成 SIMD。<br>
    /// This means to generate a binary without SIMD you'll need to avoid both options above plus calling into any intrinsics in this module. <br>这意味着要生成没有 SIMD 的二进制文件，您将需要避免上述两个选项以及调用此模块中的任何内部函数。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    #[cfg(any(target_arch = "wasm32", doc))]
    #[doc(cfg(target_arch = "wasm32"))]
    #[stable(feature = "simd_wasm32", since = "1.33.0")]
    pub mod wasm32 {
        #[stable(feature = "simd_wasm32", since = "1.33.0")]
        pub use crate::core_arch::wasm32::*;
    }

    /// Platform-specific intrinsics for the `wasm64` platform. <br>特定于平台的用于 `wasm64` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "wasm64", doc))]
    #[doc(cfg(target_arch = "wasm64"))]
    #[unstable(feature = "simd_wasm64", issue = "90599")]
    pub mod wasm64 {
        #[unstable(feature = "simd_wasm64", issue = "90599")]
        pub use crate::core_arch::wasm32::*;
    }

    /// Platform-specific intrinsics for the `wasm` target family. <br>特定于平台的用于 `wasm` 目标家庭的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_family = "wasm", doc))]
    #[doc(cfg(target_family = "wasm"))]
    #[unstable(feature = "simd_wasm64", issue = "90599")]
    pub mod wasm {
        #[unstable(feature = "simd_wasm64", issue = "90599")]
        pub use crate::core_arch::wasm32::*;
    }

    /// Platform-specific intrinsics for the `mips` platform. <br>特定于平台的用于 `mips` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "mips", doc))]
    #[doc(cfg(target_arch = "mips"))]
    #[unstable(feature = "stdsimd", issue = "27731")]
    pub mod mips {
        pub use crate::core_arch::mips::*;
    }

    /// Platform-specific intrinsics for the `mips64` platform. <br>特定于平台的用于  `mips64` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "mips64", doc))]
    #[doc(cfg(target_arch = "mips64"))]
    #[unstable(feature = "stdsimd", issue = "27731")]
    pub mod mips64 {
        pub use crate::core_arch::mips::*;
    }

    /// Platform-specific intrinsics for the `PowerPC` platform. <br>特定于平台的用于 `PowerPC` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "powerpc", doc))]
    #[doc(cfg(target_arch = "powerpc"))]
    #[unstable(feature = "stdsimd", issue = "27731")]
    pub mod powerpc {
        pub use crate::core_arch::powerpc::*;
    }

    /// Platform-specific intrinsics for the `PowerPC64` platform. <br>特定于平台的用于 `PowerPC64` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "powerpc64", doc))]
    #[doc(cfg(target_arch = "powerpc64"))]
    #[unstable(feature = "stdsimd", issue = "27731")]
    pub mod powerpc64 {
        pub use crate::core_arch::powerpc64::*;
    }

    /// Platform-specific intrinsics for the `NVPTX` platform. <br>特定于平台的用于 `NVPTX` 平台的内部函数。<br>
    ///
    /// See the [module documentation](../index.html) for more details. <br>有关更多详细信息，请参见 [模块级文档](../index.html)。<br>
    #[cfg(any(target_arch = "nvptx", target_arch = "nvptx64", doc))]
    #[doc(cfg(any(target_arch = "nvptx", target_arch = "nvptx64")))]
    #[unstable(feature = "stdsimd", issue = "27731")]
    pub mod nvptx {
        pub use crate::core_arch::nvptx::*;
    }
}

mod simd_llvm;

#[cfg(any(target_arch = "x86", target_arch = "x86_64", doc))]
#[doc(cfg(any(target_arch = "x86", target_arch = "x86_64")))]
mod x86;
#[cfg(any(target_arch = "x86_64", doc))]
#[doc(cfg(target_arch = "x86_64"))]
mod x86_64;

#[cfg(any(target_arch = "aarch64", doc))]
#[doc(cfg(target_arch = "aarch64"))]
mod aarch64;
#[cfg(any(target_arch = "arm", doc))]
#[doc(cfg(any(target_arch = "arm")))]
mod arm;

#[cfg(any(target_arch = "riscv32", target_arch = "riscv64", doc))]
#[doc(cfg(any(target_arch = "riscv32", target_arch = "riscv64")))]
mod riscv_shared;

#[cfg(any(target_arch = "riscv64", doc))]
#[doc(cfg(any(target_arch = "riscv64")))]
mod riscv64;

#[cfg(any(target_family = "wasm", doc))]
#[doc(cfg(target_family = "wasm"))]
mod wasm32;

#[cfg(any(target_arch = "mips", target_arch = "mips64", doc))]
#[doc(cfg(any(target_arch = "mips", target_arch = "mips64")))]
mod mips;

#[cfg(any(target_arch = "powerpc", target_arch = "powerpc64", doc))]
#[doc(cfg(any(target_arch = "powerpc", target_arch = "powerpc64")))]
mod powerpc;

#[cfg(any(target_arch = "powerpc64", doc))]
#[doc(cfg(target_arch = "powerpc64"))]
mod powerpc64;

#[cfg(any(target_arch = "nvptx", target_arch = "nvptx64", doc))]
#[doc(cfg(any(target_arch = "nvptx", target_arch = "nvptx64")))]
mod nvptx;
