//! NVPTX intrinsics (experimental) <br>NVPTX 内部函数 (experimental)<br>
//!
//! These intrinsics form the foundation of the CUDA programming model. <br>这些内部函数构成了 CUDA 编程模型的基础。<br>
//!
//!
//! The reference is the [CUDA C Programming Guide][cuda_c]. <br>引用的是 [CUDA C 编程指南][cuda_c]。<br>
//! Relevant is also the [LLVM NVPTX Backend documentation][llvm_docs]. <br>也与 [LLVM NVPTX 后端文档][llvm_docs] 有关。<br>
//!
//! [cuda_c]:
//! http://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html
//! [llvm_docs]:
//! https://llvm.org/docs/NVPTXUsage.html

use crate::ffi::c_void;

#[allow(improper_ctypes)]
extern "C" {
    #[link_name = "llvm.nvvm.barrier0"]
    fn syncthreads() -> ();
    #[link_name = "llvm.nvvm.read.ptx.sreg.ntid.x"]
    fn block_dim_x() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.ntid.y"]
    fn block_dim_y() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.ntid.z"]
    fn block_dim_z() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.ctaid.x"]
    fn block_idx_x() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.ctaid.y"]
    fn block_idx_y() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.ctaid.z"]
    fn block_idx_z() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.nctaid.x"]
    fn grid_dim_x() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.nctaid.y"]
    fn grid_dim_y() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.nctaid.z"]
    fn grid_dim_z() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.tid.x"]
    fn thread_idx_x() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.tid.y"]
    fn thread_idx_y() -> i32;
    #[link_name = "llvm.nvvm.read.ptx.sreg.tid.z"]
    fn thread_idx_z() -> i32;
}

/// Synchronizes all threads in the block. <br>同步块中的所有线程。<br>
#[inline]
pub unsafe fn _syncthreads() -> () {
    syncthreads()
}

/// x-th thread-block dimension. <br>第 x 个线程块维度。<br>
#[inline]
pub unsafe fn _block_dim_x() -> i32 {
    block_dim_x()
}

/// y-th thread-block dimension. <br>第 y 个线程块的维度。<br>
#[inline]
pub unsafe fn _block_dim_y() -> i32 {
    block_dim_y()
}

/// z-th thread-block dimension. <br>第 z 个线程块的维度。<br>
#[inline]
pub unsafe fn _block_dim_z() -> i32 {
    block_dim_z()
}

/// x-th thread-block index. <br>第 x 个线程块索引。<br>
#[inline]
pub unsafe fn _block_idx_x() -> i32 {
    block_idx_x()
}

/// y-th thread-block index. <br>第 y 个线程块索引。<br>
#[inline]
pub unsafe fn _block_idx_y() -> i32 {
    block_idx_y()
}

/// z-th thread-block index. <br>第 z 个线程块索引。<br>
#[inline]
pub unsafe fn _block_idx_z() -> i32 {
    block_idx_z()
}

/// x-th block-grid dimension. <br>第 x 个块网格尺寸。<br>
#[inline]
pub unsafe fn _grid_dim_x() -> i32 {
    grid_dim_x()
}

/// y-th block-grid dimension. <br>第 y 个块网格尺寸。<br>
#[inline]
pub unsafe fn _grid_dim_y() -> i32 {
    grid_dim_y()
}

/// z-th block-grid dimension. <br>第 z 个块网格尺寸。<br>
#[inline]
pub unsafe fn _grid_dim_z() -> i32 {
    grid_dim_z()
}

/// x-th thread index. <br>第 x 个线程索引。<br>
#[inline]
pub unsafe fn _thread_idx_x() -> i32 {
    thread_idx_x()
}

/// y-th thread index. <br>第 y 个线程索引。<br>
#[inline]
pub unsafe fn _thread_idx_y() -> i32 {
    thread_idx_y()
}

/// z-th thread index. <br>第 z 个线程索引。<br>
#[inline]
pub unsafe fn _thread_idx_z() -> i32 {
    thread_idx_z()
}

/// Generates the trap instruction `TRAP` <br>生成陷阱指令 `TRAP`<br>
#[inline]
pub unsafe fn trap() -> ! {
    crate::intrinsics::abort()
}

// Basic CUDA syscall declarations. <br>基本的 CUDA syscall 声明。<br>
extern "C" {
    /// Print formatted output from a kernel to a host-side output stream. <br>将格式化的输出从内核打印到主机端输出流。<br>
    ///
    /// Syscall arguments: <br>Syscall 参数：<br>
    /// * `status`: The status value that is returned by `vprintf`. <br>`status`: `vprintf` 返回的状态值。<br>
    /// * `format`: A pointer to the format specifier input (uses common `printf` format). <br>`format`: 指向格式说明符输入的指针 (使用常见的 `printf` 格式)。<br>
    /// * `valist`: A pointer to the valist input. <br>`valist`: 指向 valist 输入的指针。<br>
    ///
    /// ```
    /// #[repr(C)]
    /// struct PrintArgs(f32, f32, f32, i32);
    ///
    /// vprintf(
    ///     "int(%f + %f) = int(%f) = %d\n".as_ptr(),
    ///     transmute(&PrintArgs(a, b, a + b, (a + b) as i32)),
    /// );
    /// ```
    ///
    /// Sources: [Programming Guide](https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#formatted-output), [PTX Interoperability](https://docs.nvidia.com/cuda/ptx-writers-guide-to-interoperability/index.html#system-calls). <br>资料来源: [编程指南](https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#formatted-output)、[PTX Interoperability](https://docs.nvidia.com/cuda/ptx-writers-guide-to-interoperability/index.html#system-calls)。<br>
    ///
    ///
    pub fn vprintf(format: *const u8, valist: *const c_void) -> i32;

    /// Allocate memory dynamically from a fixed-size heap in global memory. <br>从固定大小的堆中动态分配内存。<br>
    ///
    /// The CUDA in-kernel `malloc()` function allocates at least `size` bytes from the device heap and returns a pointer to the allocated memory or `NULL` if insufficient memory exists to fulfill the request. <br>CUDA 内核 `malloc()` 函数从设备堆中分配至少 `size` 字节，并返回指向分配的内存的指针; 如果内存不足，无法返回请求，则返回 `NULL`。<br>
    ///
    /// The returned pointer is guaranteed to be aligned to a 16-byte boundary. <br>返回的指针保证与 16 字节边界对齐。<br>
    ///
    /// The memory allocated by a given CUDA thread via `malloc()` remains allocated for the lifetime of the CUDA context, or until it is explicitly released by a call to `free()`. <br>给定 CUDA 线程通过 `malloc()` 分配的内存将保持分配给 CUDA 上下文的生命周期，或者直到被调用显式释放到 `free()` 为止。<br> It can be used by any other CUDA threads even from subsequent kernel launches. <br>它可以被其他任何 CUDA 线程使用，即使随后的内核启动也可以使用。<br>
    ///
    /// Sources: [Programming Guide](https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#dynamic-global-memory-allocation-and-operations), [PTX Interoperability](https://docs.nvidia.com/cuda/ptx-writers-guide-to-interoperability/index.html#system-calls). <br>资料来源: [编程指南](https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#dynamic-global-memory-allocation-and-operations)、[PTX Interoperability](https://docs.nvidia.com/cuda/ptx-writers-guide-to-interoperability/index.html#system-calls)。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    ///
    // FIXME(denzp): assign `malloc` and `nothrow` attributes. <br>分配 `malloc` 和 `nothrow` 属性。<br>
    pub fn malloc(size: usize) -> *mut c_void;

    /// Free previously dynamically allocated memory. <br>释放以前动态分配的内存。<br>
    ///
    /// The CUDA in-kernel `free()` function deallocates the memory pointed to by `ptr`, which must have been returned by a previous call to `malloc()`. <br>CUDA 内核 `free()` 函数将 `ptr` 指向的内存释放，该内存必须由先前的调用返回给 `malloc()`。<br>
    /// If `ptr` is NULL, the call to `free()` is ignored. <br>如果 `ptr` 为 NULL，则忽略对 `free()` 的调用。<br>
    ///
    /// Any CUDA thread may free memory allocated by another thread, but care should be taken to ensure that the same pointer is not freed more than once. <br>任何 CUDA 线程都可以释放另一个线程分配的内存，但应注意确保同一指针不会被释放超过一次。<br> Repeated calls to `free()` with the same `ptr` has undefined behavior. <br>使用相同的 `ptr` 重复调用 `free()` 具有未定义的行为。<br>
    ///
    /// Sources: [Programming Guide](https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#dynamic-global-memory-allocation-and-operations), [PTX Interoperability](https://docs.nvidia.com/cuda/ptx-writers-guide-to-interoperability/index.html#system-calls). <br>资料来源: [编程指南](https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#dynamic-global-memory-allocation-and-operations)、[PTX Interoperability](https://docs.nvidia.com/cuda/ptx-writers-guide-to-interoperability/index.html#system-calls)。<br>
    ///
    ///
    ///
    ///
    ///
    // FIXME(denzp): assign `nothrow` attribute. <br>分配 `nothrow` 属性。<br>
    pub fn free(ptr: *mut c_void);

    // Internal declaration of the syscall. <br>syscall 的内部声明。<br>
    // Exported variant has the `char_size` parameter set to `1` (single char size in bytes). <br>导出的变体将 `char_size` 参数设置为 `1` (以字节为单位的单个字符大小)。<br>
    fn __assertfail(
        message: *const u8,
        file: *const u8,
        line: u32,
        function: *const u8,
        char_size: usize,
    );
}

/// Syscall to be used whenever the *assert expression produces a `false` value*. <br>*assert expression 产生 `false` 值* 时将使用 Syscall。<br>
///
/// Syscall arguments: <br>Syscall 参数：<br>
/// * `message`: The pointer to the string that should be output. <br>`message`: 指向应该输出的字符串的指针。<br>
/// * `file`: The pointer to the file name string associated with the assert. <br>`file`: 指向与断言关联的文件名字符串的指针。<br>
/// * `line`: The line number associated with the assert. <br>`line`: 与断言关联的行号。<br>
/// * `function`: The pointer to the function name string associated with the assert. <br>`function`: 指向与断言关联的函数名称字符串的指针。<br>
///
/// Source: [PTX Interoperability](https://docs.nvidia.com/cuda/ptx-writers-guide-to-interoperability/index.html#system-calls). <br>来源: [PTX 互操作性](https://docs.nvidia.com/cuda/ptx-writers-guide-to-interoperability/index.html#system-calls)。<br>
///
#[inline]
pub unsafe fn __assert_fail(message: *const u8, file: *const u8, line: u32, function: *const u8) {
    __assertfail(message, file, line, function, 1)
}
