//! RISC-V RV64 特定的内部函数
use crate::arch::asm;

/// 通过无符号字整数加载虚拟机内存
///
/// 该指令执行显式内存访问，就像 `V=1` 一样;
/// 即，具有地址转换和保护以及字节序，适用于 VS 模式或 VU 模式下的内存访问。
///
/// 此操作在 RV32 基本指令集下不可用。
///
/// 这个函数是不安全的，因为它通过 `HLV.WU` 指令访问虚拟 supervisor 或用户，这实际上是对任何内存地址的解引用。
///
///
#[inline]
pub unsafe fn hlv_wu(src: *const u32) -> u32 {
    let value: u32;
    asm!(".insn i 0x73, 0x4, {}, {}, 0x681", out(reg) value, in(reg) src, options(readonly, nostack));
    value
}

/// 按双整数加载虚拟机内存
///
/// 该指令执行显式内存访问，就像 `V=1` 一样;
/// 即，具有地址转换和保护以及字节序，适用于 VS 模式或 VU 模式下的内存访问。
///
/// 此操作在 RV32 基本指令集下不可用。
///
/// 这个函数是不安全的，因为它通过 `HLV.D` 指令访问虚拟 supervisor 或用户，这实际上是对任何内存地址的解引用。
///
///
#[inline]
pub unsafe fn hlv_d(src: *const i64) -> i64 {
    let value: i64;
    asm!(".insn i 0x73, 0x4, {}, {}, 0x6C0", out(reg) value, in(reg) src, options(readonly, nostack));
    value
}

/// 按双整数存储虚拟机内存
///
/// 该指令执行显式内存访问，就像 `V=1` 一样;
/// 即，具有地址转换和保护以及字节序，适用于 VS 模式或 VU 模式下的内存访问。
///
/// 这个函数是不安全的，因为它通过 `HSV.D` 指令访问虚拟 supervisor 或用户，这实际上是对任何内存地址的解引用。
///
///
#[inline]
pub unsafe fn hsv_d(dst: *mut i64, src: i64) {
    asm!(".insn r 0x73, 0x4, 0x37, x0, {}, {}", in(reg) dst, in(reg) src, options(nostack));
}
