#[cfg(test)]
use stdarch_test::assert_instr;

extern "C" {
    #[link_name = "llvm.wasm.memory.atomic.wait32"]
    fn llvm_atomic_wait_i32(ptr: *mut i32, exp: i32, timeout: i64) -> i32;
    #[link_name = "llvm.wasm.memory.atomic.wait64"]
    fn llvm_atomic_wait_i64(ptr: *mut i64, exp: i64, timeout: i64) -> i32;
    #[link_name = "llvm.wasm.memory.atomic.notify"]
    fn llvm_atomic_notify(ptr: *mut i32, cnt: i32) -> i32;
}

/// Corresponding intrinsic to wasm's [`memory.atomic.wait32` instruction][instr] <br>对应 wasm 的 [`memory.atomic.wait32` 指令][instr]<br>
///
/// This function, when called, will block the current thread if the memory pointed to by `ptr` is equal to `expression` (performing this action atomically). <br>如果 `ptr` 指向的内存等于 `expression` (原子执行此操作)，则调用此函数时，它将阻塞当前线程。<br>
///
///
/// The argument `timeout_ns` is a maximum number of nanoseconds the calling thread will be blocked for, if it blocks. <br>参数 `timeout_ns` 是调用线程将被阻塞的最大纳秒数 (如果被阻塞的话)。<br>
/// If the timeout is negative then the calling thread will be blocked forever. <br>如果超时为负，则调用线程将永远被阻塞。<br>
///
/// The calling thread can only be woken up with a call to the `wake` intrinsic once it has been blocked. <br>一旦调用线程被阻塞，调用线程只能通过调用 `wake` 内部函数来唤醒。<br>
/// Changing the memory behind `ptr` will not wake the thread once it's blocked. <br>一旦线程被阻塞，在 `ptr` 后面更改内存将不会唤醒线程。<br>
///
/// # Return value <br>返回值<br>
///
/// * 0 - indicates that the thread blocked and then was woken up <br>0 - 表示线程被阻塞，然后被唤醒<br>
/// * 1 - the loaded value from `ptr` didn't match `expression`, the thread didn't block <br>1-`ptr` 的加载值与 `expression` 不匹配，线程未阻塞<br>
/// * 2 - the thread blocked, but the timeout expired. <br>2 - 线程被阻塞，但超时已过期。<br>
///
/// [instr]: https://webassembly.github.io/threads/core/syntax/instructions.html#syntax-instr-atomic-memory
///
///
///
///
#[inline]
#[cfg_attr(test, assert_instr(memory.atomic.wait32))]
#[target_feature(enable = "atomics")]
#[doc(alias("memory.atomic.wait32"))]
pub unsafe fn memory_atomic_wait32(ptr: *mut i32, expression: i32, timeout_ns: i64) -> i32 {
    llvm_atomic_wait_i32(ptr, expression, timeout_ns)
}

/// Corresponding intrinsic to wasm's [`memory.atomic.wait64` instruction][instr] <br>对应 wasm 的 [`memory.atomic.wait64` 指令][instr]<br>
///
/// This function, when called, will block the current thread if the memory pointed to by `ptr` is equal to `expression` (performing this action atomically). <br>如果 `ptr` 指向的内存等于 `expression` (原子执行此操作)，则调用此函数时，它将阻塞当前线程。<br>
///
///
/// The argument `timeout_ns` is a maximum number of nanoseconds the calling thread will be blocked for, if it blocks. <br>参数 `timeout_ns` 是调用线程将被阻塞的最大纳秒数 (如果被阻塞的话)。<br>
/// If the timeout is negative then the calling thread will be blocked forever. <br>如果超时为负，则调用线程将永远被阻塞。<br>
///
/// The calling thread can only be woken up with a call to the `wake` intrinsic once it has been blocked. <br>一旦调用线程被阻塞，调用线程只能通过调用 `wake` 内部函数来唤醒。<br>
/// Changing the memory behind `ptr` will not wake the thread once it's blocked. <br>一旦线程被阻塞，在 `ptr` 后面更改内存将不会唤醒线程。<br>
///
/// # Return value <br>返回值<br>
///
/// * 0 - indicates that the thread blocked and then was woken up <br>0 - 表示线程被阻塞，然后被唤醒<br>
/// * 1 - the loaded value from `ptr` didn't match `expression`, the thread didn't block <br>1-`ptr` 的加载值与 `expression` 不匹配，线程未阻塞<br>
/// * 2 - the thread blocked, but the timeout expired. <br>2 - 线程被阻塞，但超时已过期。<br>
///
/// [instr]: https://webassembly.github.io/threads/core/syntax/instructions.html#syntax-instr-atomic-memory
///
///
///
///
#[inline]
#[cfg_attr(test, assert_instr(memory.atomic.wait64))]
#[target_feature(enable = "atomics")]
#[doc(alias("memory.atomic.wait64"))]
pub unsafe fn memory_atomic_wait64(ptr: *mut i64, expression: i64, timeout_ns: i64) -> i32 {
    llvm_atomic_wait_i64(ptr, expression, timeout_ns)
}

/// Corresponding intrinsic to wasm's [`memory.atomic.notify` instruction][instr] <br>对应 wasm 的 [`memory.atomic.notify` 指令][instr]<br>
///
/// This function will notify a number of threads blocked on the address indicated by `ptr`. <br>该函数将通知 `ptr` 指示的地址上阻塞的多个线程。<br>
/// Threads previously blocked with the `i32_atomic_wait` and `i64_atomic_wait` functions above will be woken up. <br>先前被上述 `i32_atomic_wait` 和 `i64_atomic_wait` 函数阻塞的线程将被唤醒。<br>
///
///
/// The `waiters` argument indicates how many waiters should be woken up (a maximum). <br>`waiters` 参数指示应唤醒的等待者数量 (最大值)。<br>
/// If the value is zero no waiters are woken up. <br>如果值为零，则不会唤醒任何等待者。<br>
///
/// # Return value <br>返回值<br>
///
/// Returns the number of waiters which were actually notified. <br>返回实际被通知的等待者的数量。<br>
///
/// [instr]: https://webassembly.github.io/threads/core/syntax/instructions.html#syntax-instr-atomic-memory
#[inline]
#[cfg_attr(test, assert_instr(memory.atomic.notify))]
#[target_feature(enable = "atomics")]
#[doc(alias("memory.atomic.notify"))]
pub unsafe fn memory_atomic_notify(ptr: *mut i32, waiters: u32) -> u32 {
    llvm_atomic_notify(ptr, waiters as i32) as u32
}
