#[cfg(test)]
use stdarch_test::assert_instr;

extern "C" {
    #[link_name = "llvm.wasm.memory.grow"]
    fn llvm_memory_grow(mem: u32, pages: usize) -> usize;
    #[link_name = "llvm.wasm.memory.size"]
    fn llvm_memory_size(mem: u32) -> usize;
}

/// Corresponding intrinsic to wasm's [`memory.size` instruction][instr] <br>对应 wasm 的 [`memory.size` 指令][instr]<br>
///
/// This function, when called, will return the current memory size in units of pages. <br>调用此函数时，它将以页为单位返回当前的内存大小。<br>
/// The current WebAssembly page size is 65536 bytes (64 KB). <br>当前的 WebAssembly 页面大小为 65536 字节 (64 KB)。<br>
///
/// The argument `MEM` is the numerical index of which memory to return the size of. <br>参数 `MEM` 是返回内存大小的数字索引。<br>
/// Note that currently the WebAssembly specification only supports one memory, so it is required that zero is passed in. <br>请注意，当前 WebAssembly 规范仅支持一个内存，因此要求传入零。<br>
///
/// The argument is present to be forward-compatible with future WebAssembly revisions. <br>该参数与 future WebAssembly 修订版向前兼容。<br>
/// If a nonzero argument is passed to this function it will currently unconditionally abort. <br>如果将非零参数传递给此函数，则它将当前无条件终止。<br>
///
/// [instr]: http://webassembly.github.io/spec/core/exec/instructions.html#exec-memory-size
#[inline]
#[cfg_attr(test, assert_instr("memory.size", MEM = 0))]
#[rustc_legacy_const_generics(0)]
#[stable(feature = "simd_wasm32", since = "1.33.0")]
#[doc(alias("memory.size"))]
pub fn memory_size<const MEM: u32>() -> usize {
    static_assert!(MEM: u32 where MEM == 0);
    unsafe { llvm_memory_size(MEM) }
}

/// Corresponding intrinsic to wasm's [`memory.grow` instruction][instr] <br>对应 wasm 的 [`memory.grow` 指令][instr]<br>
///
/// This function, when called, will attempt to grow the default linear memory by the specified `delta` of pages. <br>调用此函数时，它将尝试通过指定的 `delta` 页增长默认的线性内存。<br>
/// The current WebAssembly page size is <br>当前的 WebAssembly 页面大小为<br>
/// 65536 bytes (64 KB). <br>65536 字节 (64 KB)。<br> If memory is successfully grown then the previous size of memory, in pages, is returned. <br>如果成功增加了内存，则将返回先前的内存大小 (以页为单位)。<br>
/// If memory cannot be grown then `usize::MAX` is returned. <br>如果无法增加内存，则返回 `usize::MAX`。<br>
///
/// The argument `MEM` is the numerical index of which memory to return the size of. <br>参数 `MEM` 是返回内存大小的数字索引。<br>
/// Note that currently the WebAssembly specification only supports one memory, so it is required that zero is passed in. <br>请注意，当前 WebAssembly 规范仅支持一个内存，因此要求传入零。<br>
///
/// The argument is present to be forward-compatible with future WebAssembly revisions. <br>该参数与 future WebAssembly 修订版向前兼容。<br>
/// If a nonzero argument is passed to this function it will currently unconditionally abort. <br>如果将非零参数传递给此函数，则它将当前无条件终止。<br>
///
/// [instr]: http://webassembly.github.io/spec/core/exec/instructions.html#exec-memory-grow
///
#[inline]
#[cfg_attr(test, assert_instr("memory.grow", MEM = 0))]
#[rustc_legacy_const_generics(0)]
#[stable(feature = "simd_wasm32", since = "1.33.0")]
#[doc(alias("memory.grow"))]
pub fn memory_grow<const MEM: u32>(delta: usize) -> usize {
    unsafe {
        static_assert!(MEM: u32 where MEM == 0);
        llvm_memory_grow(MEM, delta)
    }
}
