//! RDTSC instructions. <br>RDTSC 指令。<br>

#[cfg(test)]
use stdarch_test::assert_instr;

/// Reads the current value of the processor’s time-stamp counter. <br>读取处理器时间戳计数器的当前值。<br>
///
/// The processor monotonically increments the time-stamp counter MSR every clock cycle and resets it to 0 whenever the processor is reset. <br>处理器在每个时钟周期单调递增时间戳计数器 MSR，并在每次重置处理器时将其重置为 0。<br>
///
///
/// The RDTSC instruction is not a serializing instruction. <br>RDTSC 指令不是序列化指令。<br>
/// It does not necessarily wait until all previous instructions have been executed before reading the counter. <br>它不一定要等到所有先前的指令已执行才读取计数器。<br>
/// Similarly, subsequent instructions may begin execution before the read operation is performed. <br>类似地，后续指令可以在执行读取操作之前开始执行。<br>
///
/// On processors that support the Intel 64 architecture, the high-order 32 bits of each of RAX and RDX are cleared. <br>在支持 Intel 64 架构的处理器上，将清除 RAX 和 RDX 的每个高阶 32 位。<br>
///
/// [Intel's documentation](https://software.intel.com/sites/landingpage/IntrinsicsGuide/#text=_rdtsc)
///
///
///
///
#[inline]
#[cfg_attr(test, assert_instr(rdtsc))]
#[stable(feature = "simd_x86", since = "1.27.0")]
pub unsafe fn _rdtsc() -> u64 {
    rdtsc()
}

/// Reads the current value of the processor’s time-stamp counter and the `IA32_TSC_AUX MSR`. <br>读取处理器时间戳计数器和 `IA32_TSC_AUX MSR` 的当前值。<br>
///
/// The processor monotonically increments the time-stamp counter MSR every clock cycle and resets it to 0 whenever the processor is reset. <br>处理器在每个时钟周期单调递增时间戳计数器 MSR，并在每次重置处理器时将其重置为 0。<br>
///
///
/// The RDTSCP instruction waits until all previous instructions have been executed before reading the counter. <br>RDTSCP 指令等待，直到所有先前的指令都已执行，然后再读取计数器。<br>
/// However, subsequent instructions may begin execution before the read operation is performed. <br>但是，后续指令可能会在执行读取操作之前开始执行。<br>
///
/// On processors that support the Intel 64 architecture, the high-order 32 bits of each of RAX, RDX, and RCX are cleared. <br>在支持 Intel 64 架构的处理器上，将清除 RAX，RDX 和 RCX 的每个高阶 32 位。<br>
///
/// [Intel's documentation](https://software.intel.com/sites/landingpage/IntrinsicsGuide/#text=__rdtscp)
///
///
///
///
///
#[inline]
#[cfg_attr(test, assert_instr(rdtscp))]
#[stable(feature = "simd_x86", since = "1.27.0")]
pub unsafe fn __rdtscp(aux: *mut u32) -> u64 {
    rdtscp(aux as *mut _)
}

#[allow(improper_ctypes)]
extern "C" {
    #[link_name = "llvm.x86.rdtsc"]
    fn rdtsc() -> u64;
    #[link_name = "llvm.x86.rdtscp"]
    fn rdtscp(aux: *mut u8) -> u64;
}

#[cfg(test)]
mod tests {
    use crate::core_arch::x86::*;
    use stdarch_test::simd_test;

    #[simd_test(enable = "sse2")]
    unsafe fn _rdtsc() {
        let r = rdtsc::_rdtsc();
        assert_ne!(r, 0); // The chances of this being 0 are infinitesimal <br>此为 0 的机会是无穷的<br>
    }

    #[simd_test(enable = "sse2")]
    unsafe fn _rdtscp() {
        let mut aux = 0;
        let r = rdtsc::__rdtscp(&mut aux);
        assert_ne!(r, 0); // The chances of this being 0 are infinitesimal <br>此为 0 的机会是无穷的<br>
    }
}
