//! Intel's Restricted Transactional Memory (RTM). <br>英特尔受限事物内存 (RTM)。<br>
//!
//! This CPU feature is available on Intel Broadwell or later CPUs (and some Haswell). <br>此 CPU 特性在 Intel Broadwell 或更高版本的 CPU (和某些 Haswell) 上可用。<br>
//!
//! The reference is [Intel 64 and IA-32 Architectures Software Developer's Manual Volume 2: Instruction Set Reference, A-Z][intel64_ref]. <br>引用的是 [英特尔 64 位和 IA-32 架构软件开发人员手册第 2 卷：指令集参考，A-Z][intel64_ref]。<br>
//!
//! [Wikipedia][wikipedia_rtm] provides a quick overview of the assembly instructions, and Intel's [programming considerations][intel_consid] details what sorts of instructions within a transaction are likely to cause an abort. <br>[维基百科][wikipedia_rtm] 提供了汇编指令的快速概览，英特尔的 [编程注意事项][intel_consid] 详细说明了事务中的哪些类型的指令可能导致终止。<br>
//!
//!
//! [intel64_ref]: http://www.intel.de/content/dam/www/public/us/en/documents/manuals/64-ia-32-architectures-software-developer-instruction-set-reference-manual-325383.pdf
//! [wikipedia_rtm]: https://en.wikipedia.org/wiki/Transactional_Synchronization_Extensions#Restricted_Transactional_Memory
//! [intel_consid]: https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-intel-transactional-synchronization-extensions-intel-tsx-programming-considerations
//!
//!

#[cfg(test)]
use stdarch_test::assert_instr;

extern "C" {
    #[link_name = "llvm.x86.xbegin"]
    fn x86_xbegin() -> i32;
    #[link_name = "llvm.x86.xend"]
    fn x86_xend();
    #[link_name = "llvm.x86.xabort"]
    fn x86_xabort(imm8: i8);
    #[link_name = "llvm.x86.xtest"]
    fn x86_xtest() -> i32;
}

/// Transaction successfully started. <br>事务成功开始。<br>
pub const _XBEGIN_STARTED: u32 = !0;

/// Transaction explicitly aborted with xabort. <br>使用 xabort 显式中止了事务。<br>
/// The parameter passed to xabort is available with `_xabort_code(status)`. <br>传递给 xabort 的参数可用于 `_xabort_code(status)`。<br>
#[allow(clippy::identity_op)]
pub const _XABORT_EXPLICIT: u32 = 1 << 0;

/// Transaction retry is possible. <br>事务重试是可能的。<br>
pub const _XABORT_RETRY: u32 = 1 << 1;

/// Transaction abort due to a memory conflict with another thread. <br>由于与另一个线程发生内存冲突而导致事务终止。<br>
pub const _XABORT_CONFLICT: u32 = 1 << 2;

/// Transaction abort due to the transaction using too much memory. <br>事务终止，因为事务使用了过多的内存。<br>
pub const _XABORT_CAPACITY: u32 = 1 << 3;

/// Transaction abort due to a debug trap. <br>事务由于调试陷阱而终止。<br>
pub const _XABORT_DEBUG: u32 = 1 << 4;

/// Transaction abort in a inner nested transaction. <br>内部嵌套事务中的事务终止。<br>
pub const _XABORT_NESTED: u32 = 1 << 5;

/// Specifies the start of a restricted transactional memory (RTM) code region and returns a value indicating status. <br>指定受限制的事务性存储器 (RTM) 代码区域的开始，并返回一个指示状态的值。<br>
///
///  [Intel's documentation](https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-xbegin).
///
#[inline]
#[target_feature(enable = "rtm")]
#[cfg_attr(test, assert_instr(xbegin))]
pub unsafe fn _xbegin() -> u32 {
    x86_xbegin() as _
}

/// Specifies the end of a restricted transactional memory (RTM) code region. <br>指定受限制的事务性存储器 (RTM) 代码区域的结尾。<br>
///  [Intel's documentation](https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-xend).
///
#[inline]
#[target_feature(enable = "rtm")]
#[cfg_attr(test, assert_instr(xend))]
pub unsafe fn _xend() {
    x86_xend()
}

/// Forces a restricted transactional memory (RTM) region to abort. <br>强制将受限制的事务内存 (RTM) 区域终止。<br>
///  [Intel's documentation](https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-xabort).
///
#[inline]
#[target_feature(enable = "rtm")]
#[cfg_attr(test, assert_instr(xabort, IMM8 = 0x0))]
#[rustc_legacy_const_generics(0)]
pub unsafe fn _xabort<const IMM8: u32>() {
    static_assert_imm_u8!(IMM8);
    x86_xabort(IMM8 as i8)
}

/// Queries whether the processor is executing in a transactional region identified by restricted transactional memory (RTM) or hardware lock elision (HLE). <br>查询处理器是在由受限事务存储器 (RTM) 还是硬件锁定扩展 (HLE) 标识的事务区域中执行。<br>
///
///  [Intel's documentation](https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-xtest).
///
#[inline]
#[target_feature(enable = "rtm")]
#[cfg_attr(test, assert_instr(xtest))]
pub unsafe fn _xtest() -> u8 {
    x86_xtest() as _
}

/// Retrieves the parameter passed to [`_xabort`] when [`_xbegin`]'s status has the `_XABORT_EXPLICIT` flag set. <br>当 [`_xbegin`] 的状态设置为 `_XABORT_EXPLICIT` 标志时，检索传递给 [`_xabort`] 的参数。<br>
///
#[inline]
pub const fn _xabort_code(status: u32) -> u32 {
    (status >> 24) & 0xFF
}

#[cfg(test)]
mod tests {
    use stdarch_test::simd_test;

    use crate::core_arch::x86::*;

    #[simd_test(enable = "rtm")]
    unsafe fn test_xbegin_xend() {
        let mut x = 0;
        for _ in 0..10 {
            let code = rtm::_xbegin();
            if code == _XBEGIN_STARTED {
                x += 1;
                rtm::_xend();
                assert_eq!(x, 1);
                break;
            }
            assert_eq!(x, 0);
        }
    }

    #[simd_test(enable = "rtm")]
    unsafe fn test_xabort() {
        const ABORT_CODE: u32 = 42;
        // aborting outside a transactional region does nothing <br>在事务区域外中止不执行任何操作<br>
        _xabort::<ABORT_CODE>();

        for _ in 0..10 {
            let mut x = 0;
            let code = rtm::_xbegin();
            if code == _XBEGIN_STARTED {
                x += 1;
                rtm::_xabort::<ABORT_CODE>();
            } else if code & _XABORT_EXPLICIT != 0 {
                let test_abort_code = rtm::_xabort_code(code);
                assert_eq!(test_abort_code, ABORT_CODE);
            }
            assert_eq!(x, 0);
        }
    }

    #[simd_test(enable = "rtm")]
    unsafe fn test_xtest() {
        assert_eq!(_xtest(), 0);

        for _ in 0..10 {
            let code = rtm::_xbegin();
            if code == _XBEGIN_STARTED {
                let in_tx = _xtest();
                rtm::_xend();

                // putting the assert inside the transaction would abort the transaction on fail without any output/panic/etc <br>将断言放入事务中将在没有任何 output/panic/etc 的情况下终止事务失败<br>
                //
                assert_eq!(in_tx, 1);
                break;
            }
        }
    }
}
