//! 英特尔受限事物内存 (RTM)。
//!
//! 此 CPU 特性在 Intel Broadwell 或更高版本的 CPU (和某些 Haswell) 上可用。
//!
//! 引用的是 [英特尔 64 位和 IA-32 架构软件开发人员手册第 2 卷：指令集参考，A-Z][intel64_ref]。
//!
//! [维基百科][wikipedia_rtm] 提供了汇编指令的快速概览，英特尔的 [编程注意事项][intel_consid] 详细说明了事务中的哪些类型的指令可能导致终止。
//!
//!
//! [intel64_ref]: http://www.intel.de/content/dam/www/public/us/en/documents/manuals/64-ia-32-architectures-software-developer-instruction-set-reference-manual-325383.pdf
//! [wikipedia_rtm]: https://en.wikipedia.org/wiki/Transactional_Synchronization_Extensions#Restricted_Transactional_Memory
//! [intel_consid]: https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-intel-transactional-synchronization-extensions-intel-tsx-programming-considerations
//!
//!

#[cfg(test)]
use stdarch_test::assert_instr;

extern "C" {
    #[link_name = "llvm.x86.xbegin"]
    fn x86_xbegin() -> i32;
    #[link_name = "llvm.x86.xend"]
    fn x86_xend();
    #[link_name = "llvm.x86.xabort"]
    fn x86_xabort(imm8: i8);
    #[link_name = "llvm.x86.xtest"]
    fn x86_xtest() -> i32;
}

/// 事务成功开始。
pub const _XBEGIN_STARTED: u32 = !0;

/// 使用 xabort 显式中止了事务。
/// 传递给 xabort 的参数可用于 `_xabort_code(status)`。
#[allow(clippy::identity_op)]
pub const _XABORT_EXPLICIT: u32 = 1 << 0;

/// 事务重试是可能的。
pub const _XABORT_RETRY: u32 = 1 << 1;

/// 由于与另一个线程发生内存冲突而导致事务终止。
pub const _XABORT_CONFLICT: u32 = 1 << 2;

/// 事务终止，因为事务使用了过多的内存。
pub const _XABORT_CAPACITY: u32 = 1 << 3;

/// 事务由于调试陷阱而终止。
pub const _XABORT_DEBUG: u32 = 1 << 4;

/// 内部嵌套事务中的事务终止。
pub const _XABORT_NESTED: u32 = 1 << 5;

/// 指定受限制的事务性存储器 (RTM) 代码区域的开始，并返回一个指示状态的值。
///
///  [Intel's documentation](https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-xbegin).
///
#[inline]
#[target_feature(enable = "rtm")]
#[cfg_attr(test, assert_instr(xbegin))]
pub unsafe fn _xbegin() -> u32 {
    x86_xbegin() as _
}

/// 指定受限制的事务性存储器 (RTM) 代码区域的结尾。
///  [Intel's documentation](https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-xend).
///
#[inline]
#[target_feature(enable = "rtm")]
#[cfg_attr(test, assert_instr(xend))]
pub unsafe fn _xend() {
    x86_xend()
}

/// 强制将受限制的事务内存 (RTM) 区域终止。
///  [Intel's documentation](https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-xabort).
///
#[inline]
#[target_feature(enable = "rtm")]
#[cfg_attr(test, assert_instr(xabort, IMM8 = 0x0))]
#[rustc_legacy_const_generics(0)]
pub unsafe fn _xabort<const IMM8: u32>() {
    static_assert_imm_u8!(IMM8);
    x86_xabort(IMM8 as i8)
}

/// 查询处理器是在由受限事务存储器 (RTM) 还是硬件锁定扩展 (HLE) 标识的事务区域中执行。
///
///  [Intel's documentation](https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-xtest).
///
#[inline]
#[target_feature(enable = "rtm")]
#[cfg_attr(test, assert_instr(xtest))]
pub unsafe fn _xtest() -> u8 {
    x86_xtest() as _
}

/// 当 [`_xbegin`] 的状态设置为 `_XABORT_EXPLICIT` 标志时，检索传递给 [`_xabort`] 的参数。
///
#[inline]
pub const fn _xabort_code(status: u32) -> u32 {
    (status >> 24) & 0xFF
}

#[cfg(test)]
mod tests {
    use stdarch_test::simd_test;

    use crate::core_arch::x86::*;

    #[simd_test(enable = "rtm")]
    unsafe fn test_xbegin_xend() {
        let mut x = 0;
        for _ in 0..10 {
            let code = rtm::_xbegin();
            if code == _XBEGIN_STARTED {
                x += 1;
                rtm::_xend();
                assert_eq!(x, 1);
                break;
            }
            assert_eq!(x, 0);
        }
    }

    #[simd_test(enable = "rtm")]
    unsafe fn test_xabort() {
        const ABORT_CODE: u32 = 42;
        // 在事务区域外中止不执行任何操作
        _xabort::<ABORT_CODE>();

        for _ in 0..10 {
            let mut x = 0;
            let code = rtm::_xbegin();
            if code == _XBEGIN_STARTED {
                x += 1;
                rtm::_xabort::<ABORT_CODE>();
            } else if code & _XABORT_EXPLICIT != 0 {
                let test_abort_code = rtm::_xabort_code(code);
                assert_eq!(test_abort_code, ABORT_CODE);
            }
            assert_eq!(x, 0);
        }
    }

    #[simd_test(enable = "rtm")]
    unsafe fn test_xtest() {
        assert_eq!(_xtest(), 0);

        for _ in 0..10 {
            let code = rtm::_xbegin();
            if code == _XBEGIN_STARTED {
                let in_tx = _xtest();
                rtm::_xend();

                // 将断言放入事务中将在没有任何 output/panic/etc 的情况下终止事务失败
                //
                assert_eq!(in_tx, 1);
                break;
            }
        }
    }
}
