use crate::sync::atomic::Ordering;

#[cfg(test)]
use stdarch_test::assert_instr;

/// Compares and exchange 16 bytes (128 bits) of data atomically. <br>自动比较和交换 16 字节 (128 位) 的数据。<br>
///
/// This intrinsic corresponds to the `cmpxchg16b` instruction on `x86_64` processors. <br>此内部函数对应于 `x86_64` 处理器上的 `cmpxchg16b` 指令。<br>
/// It performs an atomic compare-and-swap, updating the `ptr` memory location to `val` if the current value in memory equals `old`. <br>它执行原子比较和交换，如果内存中的当前值等于 `old`，则将 `ptr` 的内存位置更新为 `val`。<br>
///
/// # Return value <br>返回值<br>
///
/// This function returns the previous value at the memory location. <br>此函数在内存位置返回前一个值。<br> If it is equal to `old` then the memory was updated to `new`. <br>如果等于 `old`，则内存已更新为 `new`。<br>
///
/// # Memory Orderings <br>内存顺序<br>
///
/// This atomic operations has the same semantics of memory orderings as `AtomicUsize::compare_exchange` does, only operating on 16 bytes of memory instead of just a pointer. <br>该原子操作与 `AtomicUsize::compare_exchange` 具有相同的内存顺序语义，只对 16 个字节的内存进行操作，而不仅仅是指针。<br>
///
/// For more information on memory orderings here see the `compare_exchange` documentation for other `Atomic*` types in the standard library. <br>有关此处的内存顺序的更多信息，请参见标准库中其他 `Atomic*` 类型的 `compare_exchange` 文档。<br>
///
/// # Unsafety
///
/// This method is unsafe because it takes a raw pointer and will attempt to read and possibly write the memory at the pointer. <br>此方法是不安全的，因为它使用裸指针，并且将尝试读取并可能在指针处写入内存。<br>
/// The pointer must also be aligned on a 16-byte boundary. <br>指针还必须在 16 字节边界上对齐。<br>
///
/// This method also requires the `cmpxchg16b` CPU feature to be available at runtime to work correctly. <br>此方法还要求 `cmpxchg16b` CPU 特性在运行时可用，才能正常工作。<br>
/// If the CPU running the binary does not actually support `cmpxchg16b` and the program enters an execution path that eventually would reach this function the behavior is undefined. <br>如果运行二进制文件的 CPU 实际上不支持 `cmpxchg16b`，并且程序输入了最终会到达该函数的执行路径，则该行为未定义。<br>
///
///
/// The `success` ordering must also be stronger or equal to `failure`, or this function call is undefined. <br>`success` 的顺序也必须大于或等于 `failure`，否则此函数调用是未定义的。<br>
/// See the `Atomic*` documentation's `compare_exchange` function for more information. <br>有关更多信息，请参见 `Atomic*` 文档的 `compare_exchange` 函数。<br>
/// When `compare_exchange` panics, this is undefined behavior. <br>当 `compare_exchange` panics 时，这是未定义的行为。<br>
/// Currently this function aborts the process with an undefined instruction. <br>当前，该函数使用未定义的指令中止进程。<br>
///
///
///
///
///
///
///
///
#[inline]
#[cfg_attr(test, assert_instr(cmpxchg16b, success = Ordering::SeqCst, failure = Ordering::SeqCst))]
#[target_feature(enable = "cmpxchg16b")]
pub unsafe fn cmpxchg16b(
    dst: *mut u128,
    old: u128,
    new: u128,
    success: Ordering,
    failure: Ordering,
) -> u128 {
    use crate::{intrinsics, sync::atomic::Ordering::*};

    debug_assert!(dst as usize % 16 == 0);

    let (val, _ok) = match (success, failure) {
        (Acquire, Acquire) => intrinsics::atomic_cxchg_acq(dst, old, new),
        (Release, Relaxed) => intrinsics::atomic_cxchg_rel(dst, old, new),
        (AcqRel, Acquire) => intrinsics::atomic_cxchg_acqrel(dst, old, new),
        (Relaxed, Relaxed) => intrinsics::atomic_cxchg_relaxed(dst, old, new),
        (SeqCst, SeqCst) => intrinsics::atomic_cxchg(dst, old, new),
        (Acquire, Relaxed) => intrinsics::atomic_cxchg_acq_failrelaxed(dst, old, new),
        (AcqRel, Relaxed) => intrinsics::atomic_cxchg_acqrel_failrelaxed(dst, old, new),
        (SeqCst, Relaxed) => intrinsics::atomic_cxchg_failrelaxed(dst, old, new),
        (SeqCst, Acquire) => intrinsics::atomic_cxchg_failacq(dst, old, new),

        // The above block is all copied from libcore, and this statement is also copied from libcore except that it's a panic in libcore and we have a little bit more of a lightweight panic here. <br>上面的代码块都是从 libcore 复制的，该语句也是从 libcore 复制的，除了它是 libcore 中的 panic，我们这里还有一些轻量级的 panic。<br>
        //
        //
        _ => crate::core_arch::x86::ud2(),
    };
    val
}
