//! Run-time feature detection on RISC-V. <br>RISC-V 上的运行时特性检测。<br>

features! {
    @TARGET: riscv;
    @CFG: any(target_arch = "riscv32", target_arch = "riscv64");
    @MACRO_NAME: is_riscv_feature_detected;
    @MACRO_ATTRS:
    /// A macro to test at *runtime* whether instruction sets are available on RISC-V platforms. <br>在运行时测试指令集在 RISC-V 平台上是否可用的宏。<br>
    ///
    /// RISC-V standard defined the base sets and the extension sets. <br>RISC-V 标准定义了基本集和扩展集。<br>
    /// The base sets are RV32I, RV64I, RV32E or RV128I. <br>基本套件为 RV32I、RV64I、RV32E 或 RV128I。<br>
    /// Any RISC-V platform must support one base set and/or multiple extension sets. <br>任何 RISC-V 平台都必须支持一个基本集或者多个扩展集。<br>
    ///
    /// Any RISC-V standard instruction sets can be in state of either ratified, frozen or draft. <br>任何 RISC-V 标准指令集都可以处于批准、冻结或草案状态。<br>
    /// The version and status of current standard instruction sets can be checked out from preface section of the [ISA manual]. <br>当前标准指令集的版本和状态可以从 [ISA 手册][ISA manual] 的前言部分查看。<br>
    ///
    ///
    /// Platform may define and support their own custom instruction sets with ISA prefix X. <br>平台可以定义和支持他们自己的带有 ISA 前缀 X 的自定义指令集。<br>
    /// These sets are highly platform specific and should be detected with their own platform support crates. <br>这些集合是高度特定于平台的，应该使用它们自己的平台支持 crates 来检测。<br>
    ///
    /// # Unprivileged Specification <br>非特权规范<br>
    ///
    /// The supported ratified RISC-V instruction sets are as follows: <br>支持的已批准 RISC-V 指令集如下：<br>
    ///
    /// * RV32I: `"rv32i"`
    /// * Zifencei: `"zifencei"`
    /// * Zihintpause: `"zihintpause"`
    /// * RV64I: `"rv64i"`
    /// * M: `"m"`
    /// * A: `"a"`
    /// * Zicsr: `"zicsr"`
    /// * Zicntr: `"zicntr"`
    /// * Zihpm: `"zihpm"`
    /// * F: `"f"`
    /// * D: `"d"`
    /// * Q: `"q"`
    /// * C: `"c"`
    ///
    /// There's also bases and extensions marked as standard instruction set, but they are in frozen or draft state. <br>还有一些基础和扩展标记为标准指令集，但它们处于冻结或草稿状态。<br>
    /// These instruction sets are also reserved by this macro and can be detected in the future platforms. <br>这些指令集也由这个宏保留，可以在未来的平台中检测到。<br>
    ///
    /// Frozen RISC-V instruction sets: <br>冻结的 RISC-V 指令集：<br>
    ///
    /// * Zfinx: `"zfinx"`
    /// * Zdinx: `"zdinx"`
    /// * Zhinx: `"zhinx"`
    /// * Zhinxmin: `"zhinxmin"`
    /// * Ztso: `"ztso"`
    ///
    /// Draft RISC-V instruction sets: <br>RISC-V 指令集草案：<br>
    ///
    /// * RV32E: `"rv32e"`
    /// * RV128I: `"rv128i"`
    /// * Zfh: `"zfh"`
    /// * Zfhmin: `"zfhmin"`
    /// * B: `"b"`
    /// * J: `"j"`
    /// * P: `"p"`
    /// * V: `"v"`
    /// * Zam: `"zam"`
    ///
    /// Defined by Privileged Specification: <br>由特权规范定义：<br>
    ///
    /// * Supervisor: `"s"`
    /// * Svnapot: `"svnapot"`
    /// * Svpbmt: `"svpbmt"`
    /// * Svinval: `"svinval"`
    /// * Hypervisor: `"h"`
    ///
    /// # RISC-V Bit-Manipulation ISA-extensions <br>RISC-V 位操作 ISA 扩展<br>
    ///
    /// This document defined the following extensions: <br>本文档定义了以下扩展：<br>
    ///
    /// * Zba: `"zba"`
    /// * Zbb: `"zbb"`
    /// * Zbc: `"zbc"`
    /// * Zbs: `"zbs"`
    ///
    /// # RISC-V Cryptography Extensions <br>RISC-V 加密扩展<br>
    ///
    /// These extensions are defined in Volume I, Scalar & Entropy Source Instructions: <br>这些扩展在第一卷，标量和熵源说明中定义：<br>
    ///
    /// * Zbkb: `"zbkb"`
    /// * Zbkc: `"zbkc"`
    /// * Zbkx: `"zbkx"`
    /// * Zknd: `"zknd"`
    /// * Zkne: `"zkne"`
    /// * Zknh: `"zknh"`
    /// * Zksed: `"zksed"`
    /// * Zksh: `"zksh"`
    /// * Zkr: `"zkr"`
    /// * Zkn: `"zkn"`
    /// * Zks: `"zks"`
    /// * Zk: `"zk"`
    /// * Zkt: `"zkt"`
    ///
    /// [ISA manual]: https://github.com/riscv/riscv-isa-manual/
    ///
    ///
    ///
    ///
    #[unstable(feature = "stdsimd", issue = "27731")]
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] rv32i: "rv32i";
    /// RV32I Base Integer Instruction Set <br>RV32I 基本整数指令集<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zifencei: "zifencei";
    /// "Zifencei" Instruction-Fetch Fence <br>"Zifencei" 获取指令栅栏<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zihintpause: "zihintpause";
    /// "Zihintpause" Pause Hint <br>"Zihintpause" 暂停提示<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] rv64i: "rv64i";
    /// RV64I Base Integer Instruction Set <br>RV64I 基本整数指令集<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] m: "m";
    /// "M" Standard Extension for Integer Multiplication and Division <br>整数乘法和除法的 "M" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] a: "a";
    /// "A" Standard Extension for Atomic Instructions <br>原子指令的 "A" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zicsr: "zicsr";
    /// "Zicsr", Control and Status Register (CSR) Instructions <br>"Zicsr"，控制和状态寄存器 (CSR) 指令<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zicntr: "zicntr";
    /// "Zicntr", Standard Extension for Base Counters and Timers <br>"Zicntr"，基本计数器和定时器的标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zihpm: "zihpm";
    /// "Zihpm", Standard Extension for Hardware Performance Counters <br>"Zihpm"，硬件性能计数器的标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] f: "f";
    /// "F" Standard Extension for Single-Precision Floating-Point <br>单精度浮点的 "F" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] d: "d";
    /// "D" Standard Extension for Double-Precision Floating-Point <br>双精度浮点的 "D" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] q: "q";
    /// "Q" Standard Extension for Quad-Precision Floating-Point <br>四精度浮点的 "Q" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] c: "c";
    /// "C" Standard Extension for Compressed Instructions <br>压缩指令的 "C" 标准扩展<br>

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zfinx: "zfinx";
    /// "Zfinx" Standard Extension for Single-Precision Floating-Point in Integer Registers <br>整数寄存器中单精度浮点的 "Zfinx" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zdinx: "zdinx";
    /// "Zdinx" Standard Extension for Double-Precision Floating-Point in Integer Registers <br>整数寄存器中双精度浮点的 "Zdinx" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zhinx: "zhinx";
    /// "Zhinx" Standard Extension for Half-Precision Floating-Point in Integer Registers <br>整数寄存器中半精度浮点的 "Zhinx" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zhinxmin: "zhinxmin";
    /// "Zhinxmin" Standard Extension for Minimal Half-Precision Floating-Point in Integer Registers <br>整数寄存器中最小半精度浮点数的 "Zhinxmin" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] ztso: "ztso";
    /// "Ztso" Standard Extension for Total Store Ordering <br> 全存储排序的 "Ztso" 标准扩展<br>

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] rv32e: "rv32e";
    /// RV32E Base Integer Instruction Set <br>RV32E 基本整数指令集<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] rv128i: "rv128i";
    /// RV128I Base Integer Instruction Set <br>RV128I 基本整数指令集<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zfh: "zfh";
    /// "Zfh" Standard Extension for 16-Bit Half-Precision Floating-Point <br>16 位半精度浮点的 "Zfh" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zfhmin: "zfhmin";
    /// "Zfhmin" Standard Extension for Minimal Half-Precision Floating-Point Support <br>用于最小半精度浮点支持的 "Zfhmin" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] b: "b";
    /// "B" Standard Extension for Bit Manipulation <br>位操作的 "B" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] j: "j";
    /// "J" Standard Extension for Dynamically Translated Languages <br>动态翻译语言的 "J" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] p: "p";
    /// "P" Standard Extension for Packed-SIMD Instructions <br>打包 SIMD 指令的 "P" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] v: "v";
    /// "V" Standard Extension for Vector Operations <br>用于 Vector 运算的 "V" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zam: "zam";
    /// "Zam" Standard Extension for Misaligned Atomics <br>未对齐原子的 "Zam" 标准扩展<br>

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] s: "s";
    /// Supervisor-Level ISA <br>supervisor 级别的 ISA<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] svnapot: "svnapot";
    /// "Svnapot" Standard Extension for NAPOT Translation Contiguity <br>NAPOT 翻译连续性的 "Svnapot" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] svpbmt: "svpbmt";
    /// "Svpbmt" Standard Extension for Page-Based Memory Types <br>基于页面的内存类型的 "Svpbmt" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] svinval: "svinval";
    /// "Svinval" Standard Extension for Fine-Grained Address-Translation Cache Invalidation <br>细粒度地址转换缓存失效的 "Svinval" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] h: "h";
    /// Hypervisor Extension <br>管理程序扩展<br>

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zba: "zba";
    /// "Zba" Standard Extension for Address Generation Instructions <br>地址生成指令的 "Zba" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbb: "zbb";
    /// "Zbb" Standard Extension for Basic Bit-Manipulation <br>基本位操作的 "Zbb" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbc: "zbc";
    /// "Zbc" Standard Extension for Carry-less Multiplication <br>无进位乘法的 "Zbc" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbs: "zbs";
    /// "Zbs" Standard Extension for Single-Bit instructions <br>单位指令的 "Zbs" 标准扩展<br>

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbkb: "zbkb";
    /// "Zbkb" Standard Extension for Bitmanip instructions for Cryptography <br>用于加密的 Bitmanip 指令的 "Zbkb" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbkc: "zbkc";
    /// "Zbkc" Standard Extension for Carry-less multiply instructions <br>无进位乘法指令的 "Zbkc" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbkx: "zbkx";
    /// "Zbkx" Standard Extension for Crossbar permutation instructions <br>Crossbar 置换指令的 "Zbkx" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zknd: "zknd";
    /// "Zknd" Standard Extension for NIST Suite: AES Decryption <br>NIST 套件的 "Zknd" 标准扩展: AES 解密<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zkne: "zkne";
    /// "Zkne" Standard Extension for NIST Suite: AES Encryption <br>NIST 套件的 "Zkne" 标准扩展: AES 加密<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zknh: "zknh";
    /// "Zknh" Standard Extension for NIST Suite: Hash Function Instructions <br>NIST Suite 的 "Zknh" 标准扩展: Hash 函数指令<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zksed: "zksed";
    /// "Zksed" Standard Extension for ShangMi Suite: SM4 Block Cipher Instructions <br>ShangMi 套件 "Zksed" 标准扩展: SM4 分组密码指令<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zksh: "zksh";
    /// "Zksh" Standard Extension for ShangMi Suite: SM3 Hash Function Instructions <br>ShangMi 套件 "Zksh" 标准扩展: SM3 哈希函数指令<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zkr: "zkr";
    /// "Zkr" Standard Extension for Entropy Source Extension <br>熵源扩展的 "Zkr" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zkn: "zkn";
    /// "Zkn" Standard Extension for NIST Algorithm Suite <br>NIST 算法套件的 "Zkn" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zks: "zks";
    /// "Zks" Standard Extension for ShangMi Algorithm Suite <br>ShangMi 算法套件 "Zks" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zk: "zk";
    /// "Zk" Standard Extension for Standard scalar cryptography extension <br>用于标准标量加密扩展的 "Zk" 标准扩展<br>
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zkt: "zkt";
    /// "Zkt" Standard Extension for Data Independent Execution Latency <br>数据独立执行延迟的 "Zkt" 标准扩展<br>
}
