//! RISC-V 上的运行时特性检测。

features! {
    @TARGET: riscv;
    @CFG: any(target_arch = "riscv32", target_arch = "riscv64");
    @MACRO_NAME: is_riscv_feature_detected;
    @MACRO_ATTRS:
    /// 在运行时测试指令集在 RISC-V 平台上是否可用的宏。
    ///
    /// RISC-V 标准定义了基本集和扩展集。
    /// 基本套件为 RV32I、RV64I、RV32E 或 RV128I。
    /// 任何 RISC-V 平台都必须支持一个基本集或者多个扩展集。
    ///
    /// 任何 RISC-V 标准指令集都可以处于批准、冻结或草案状态。
    /// 当前标准指令集的版本和状态可以从 [ISA 手册][ISA manual] 的前言部分查看。
    ///
    ///
    /// 平台可以定义和支持他们自己的带有 ISA 前缀 X 的自定义指令集。
    /// 这些集合是高度特定于平台的，应该使用它们自己的平台支持 crates 来检测。
    ///
    /// # 非特权规范
    ///
    /// 支持的已批准 RISC-V 指令集如下：
    ///
    /// * RV32I: `"rv32i"`
    /// * Zifencei: `"zifencei"`
    /// * Zihintpause: `"zihintpause"`
    /// * RV64I: `"rv64i"`
    /// * M: `"m"`
    /// * A: `"a"`
    /// * Zicsr: `"zicsr"`
    /// * Zicntr: `"zicntr"`
    /// * Zihpm: `"zihpm"`
    /// * F: `"f"`
    /// * D: `"d"`
    /// * Q: `"q"`
    /// * C: `"c"`
    ///
    /// 还有一些基础和扩展标记为标准指令集，但它们处于冻结或草稿状态。
    /// 这些指令集也由这个宏保留，可以在未来的平台中检测到。
    ///
    /// 冻结的 RISC-V 指令集：
    ///
    /// * Zfinx: `"zfinx"`
    /// * Zdinx: `"zdinx"`
    /// * Zhinx: `"zhinx"`
    /// * Zhinxmin: `"zhinxmin"`
    /// * Ztso: `"ztso"`
    ///
    /// RISC-V 指令集草案：
    ///
    /// * RV32E: `"rv32e"`
    /// * RV128I: `"rv128i"`
    /// * Zfh: `"zfh"`
    /// * Zfhmin: `"zfhmin"`
    /// * B: `"b"`
    /// * J: `"j"`
    /// * P: `"p"`
    /// * V: `"v"`
    /// * Zam: `"zam"`
    ///
    /// 由特权规范定义：
    ///
    /// * Supervisor: `"s"`
    /// * Svnapot: `"svnapot"`
    /// * Svpbmt: `"svpbmt"`
    /// * Svinval: `"svinval"`
    /// * Hypervisor: `"h"`
    ///
    /// # RISC-V 位操作 ISA 扩展
    ///
    /// 本文档定义了以下扩展：
    ///
    /// * Zba: `"zba"`
    /// * Zbb: `"zbb"`
    /// * Zbc: `"zbc"`
    /// * Zbs: `"zbs"`
    ///
    /// # RISC-V 加密扩展
    ///
    /// 这些扩展在第一卷，标量和熵源说明中定义：
    ///
    /// * Zbkb: `"zbkb"`
    /// * Zbkc: `"zbkc"`
    /// * Zbkx: `"zbkx"`
    /// * Zknd: `"zknd"`
    /// * Zkne: `"zkne"`
    /// * Zknh: `"zknh"`
    /// * Zksed: `"zksed"`
    /// * Zksh: `"zksh"`
    /// * Zkr: `"zkr"`
    /// * Zkn: `"zkn"`
    /// * Zks: `"zks"`
    /// * Zk: `"zk"`
    /// * Zkt: `"zkt"`
    ///
    /// [ISA manual]: https://github.com/riscv/riscv-isa-manual/
    ///
    ///
    ///
    ///
    #[unstable(feature = "stdsimd", issue = "27731")]
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] rv32i: "rv32i";
    /// RV32I 基本整数指令集
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zifencei: "zifencei";
    /// "Zifencei" 获取指令栅栏
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zihintpause: "zihintpause";
    /// "Zihintpause" 暂停提示
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] rv64i: "rv64i";
    /// RV64I 基本整数指令集
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] m: "m";
    /// 整数乘法和除法的 "M" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] a: "a";
    /// 原子指令的 "A" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zicsr: "zicsr";
    /// "Zicsr"，控制和状态寄存器 (CSR) 指令
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zicntr: "zicntr";
    /// "Zicntr"，基本计数器和定时器的标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zihpm: "zihpm";
    /// "Zihpm"，硬件性能计数器的标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] f: "f";
    /// 单精度浮点的 "F" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] d: "d";
    /// 双精度浮点的 "D" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] q: "q";
    /// 四精度浮点的 "Q" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] c: "c";
    /// 压缩指令的 "C" 标准扩展

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zfinx: "zfinx";
    /// 整数寄存器中单精度浮点的 "Zfinx" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zdinx: "zdinx";
    /// 整数寄存器中双精度浮点的 "Zdinx" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zhinx: "zhinx";
    /// 整数寄存器中半精度浮点的 "Zhinx" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zhinxmin: "zhinxmin";
    /// 整数寄存器中最小半精度浮点数的 "Zhinxmin" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] ztso: "ztso";
    ///  全存储排序的 "Ztso" 标准扩展

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] rv32e: "rv32e";
    /// RV32E 基本整数指令集
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] rv128i: "rv128i";
    /// RV128I 基本整数指令集
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zfh: "zfh";
    /// 16 位半精度浮点的 "Zfh" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zfhmin: "zfhmin";
    /// 用于最小半精度浮点支持的 "Zfhmin" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] b: "b";
    /// 位操作的 "B" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] j: "j";
    /// 动态翻译语言的 "J" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] p: "p";
    /// 打包 SIMD 指令的 "P" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] v: "v";
    /// 用于 Vector 运算的 "V" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zam: "zam";
    /// 未对齐原子的 "Zam" 标准扩展

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] s: "s";
    /// supervisor 级别的 ISA
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] svnapot: "svnapot";
    /// NAPOT 翻译连续性的 "Svnapot" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] svpbmt: "svpbmt";
    /// 基于页面的内存类型的 "Svpbmt" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] svinval: "svinval";
    /// 细粒度地址转换缓存失效的 "Svinval" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] h: "h";
    /// 管理程序扩展

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zba: "zba";
    /// 地址生成指令的 "Zba" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbb: "zbb";
    /// 基本位操作的 "Zbb" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbc: "zbc";
    /// 无进位乘法的 "Zbc" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbs: "zbs";
    /// 单位指令的 "Zbs" 标准扩展

    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbkb: "zbkb";
    /// 用于加密的 Bitmanip 指令的 "Zbkb" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbkc: "zbkc";
    /// 无进位乘法指令的 "Zbkc" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zbkx: "zbkx";
    /// Crossbar 置换指令的 "Zbkx" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zknd: "zknd";
    /// NIST 套件的 "Zknd" 标准扩展: AES 解密
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zkne: "zkne";
    /// NIST 套件的 "Zkne" 标准扩展: AES 加密
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zknh: "zknh";
    /// NIST Suite 的 "Zknh" 标准扩展: Hash 函数指令
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zksed: "zksed";
    /// ShangMi 套件 "Zksed" 标准扩展: SM4 分组密码指令
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zksh: "zksh";
    /// ShangMi 套件 "Zksh" 标准扩展: SM3 哈希函数指令
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zkr: "zkr";
    /// 熵源扩展的 "Zkr" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zkn: "zkn";
    /// NIST 算法套件的 "Zkn" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zks: "zks";
    /// ShangMi 算法套件 "Zks" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zk: "zk";
    /// 用于标准标量加密扩展的 "Zk" 标准扩展
    @FEATURE: #[unstable(feature = "stdsimd", issue = "27731")] zkt: "zkt";
    /// 数据独立执行延迟的 "Zkt" 标准扩展
}
