//! This module implements minimal run-time feature detection for x86. <br>该模块为 x86 实现了最小的运行时特性检测。<br>
//!
//! The features are detected using the `detect_features` function below. <br>使用下面的 `detect_features` 函数可以检测到这些特性。<br>
//! This function uses the CPUID instruction to read the feature flags from the CPU and encodes them in a `usize` where each bit position represents whether a feature is available (bit is set) or unavailable (bit is cleared). <br>这个函数使用 CPUID 指令从 CPU 读取特性标志并将它们编码在一个 `usize` 中，其中每个位位置代表一个特性是可用 (位被设置) 还是不可用 (位被清除)。<br>
//!
//!
//! The enum `Feature` is used to map bit positions to feature names, and the the `__crate::detect::check_for!` macro is used to map string literals (e.g., "avx") to these bit positions (e.g., `Feature::avx`). <br>枚举 `Feature` 用于 map 位位置以指定特性名称，而 `__crate::detect::check_for!` 宏用于 map 字符串字面量 (例如 "avx") 到这些位位置 (例如 `Feature::avx`)。<br>
//!
//! The run-time feature detection is performed by the `__crate::detect::check_for(Feature) -> bool` function. <br>运行时特性检测由 `__crate::detect::check_for(Feature) -> bool` 函数执行。<br>
//! On its first call, this functions queries the CPU for the available features and stores them in a global `AtomicUsize` variable. <br>在第一次调用时，此函数向 CPU 查询可用特性，并将其存储在 `AtomicUsize` 变量中。<br>
//! The query is performed by just checking whether the feature bit in this global variable is set or cleared. <br>通过仅检查此变量中的特性位是否已设置或清除来执行查询。<br>
//!
//!
//!
//!
//!

features! {
    @TARGET: x86;
    @CFG: any(target_arch = "x86", target_arch = "x86_64");
    @MACRO_NAME: is_x86_feature_detected;
    @MACRO_ATTRS:
    /// A macro to test at *runtime* whether a CPU feature is available on x86/x86-64 platforms. <br>一个在运行时测试 x86/x86-64 平台上是否具有 CPU 特性的宏。<br>
    ///
    /// This macro is provided in the standard library and will detect at runtime whether the specified CPU feature is detected. <br>标准库中提供了此宏，它将在运行时检测是否检测到指定的 CPU 特性。<br>
    ///
    /// This does **not** resolve at compile time unless the specified feature is already enabled for the entire crate. <br>除非在整个 crate 中启用了指定特性，否则在编译时不会解析。<br>
    /// Runtime detection currently relies mostly on the `cpuid` instruction. <br>当前，运行时检测主要依赖于 `cpuid` 指令。<br>
    ///
    /// This macro only takes one argument which is a string literal of the feature being tested for. <br>该宏仅使用一个参数，该参数是要测试的特性的字符串字面量。<br>
    /// The feature names supported are the lowercase versions of the ones defined by Intel in [their documentation][docs]. <br>支持的特性名称是 [their 文档][docs] 中 Intel 定义的特性的小写版本。<br>
    ///
    /// ## Supported arguments <br>支持的参数<br>
    ///
    /// This macro supports the same names that `#[target_feature]` supports. <br>该宏支持 `#[target_feature]` 支持的相同名称。<br>
    /// Unlike `#[target_feature]`, however, this macro does not support names separated with a comma. <br>但是，与 `#[target_feature]` 不同，此宏不支持用逗号分隔的名称。<br>
    /// Instead testing for multiple features must be done through separate macro invocations for now. <br>相反，现在必须通过单独的宏调用来测试多个特性。<br>
    ///
    /// Supported arguments are: <br>支持的参数有：<br>
    ///
    /// * `"aes"`
    /// * `"pclmulqdq"`
    /// * `"rdrand"`
    /// * `"rdseed"`
    /// * `"tsc"`
    /// * `"mmx"`
    /// * `"sse"`
    /// * `"sse2"`
    /// * `"sse3"`
    /// * `"ssse3"`
    /// * `"sse4.1"`
    /// * `"sse4.2"`
    /// * `"sse4a"`
    /// * `"sha"`
    /// * `"avx"`
    /// * `"avx2"`
    /// * `"avx512f"`
    /// * `"avx512cd"`
    /// * `"avx512er"`
    /// * `"avx512pf"`
    /// * `"avx512bw"`
    /// * `"avx512dq"`
    /// * `"avx512vl"`
    /// * `"avx512ifma"`
    /// * `"avx512vbmi"`
    /// * `"avx512vpopcntdq"`
    /// * `"avx512vbmi2"`
    /// * `"avx512gfni"`
    /// * `"avx512vaes"`
    /// * `"avx512vpclmulqdq"`
    /// * `"avx512vnni"`
    /// * `"avx512bitalg"`
    /// * `"avx512bf16"`
    /// * `"avx512vp2intersect"`
    /// * `"f16c"`
    /// * `"fma"`
    /// * `"bmi1"`
    /// * `"bmi2"`
    /// * `"abm"`
    /// * `"lzcnt"`
    /// * `"tbm"`
    /// * `"popcnt"`
    /// * `"fxsr"`
    /// * `"xsave"`
    /// * `"xsaveopt"`
    /// * `"xsaves"`
    /// * `"xsavec"`
    /// * `"cmpxchg16b"`
    /// * `"adx"`
    /// * `"rtm"`
    ///
    /// [docs]: https://software.intel.com/sites/landingpage/IntrinsicsGuide
    ///
    ///
    ///
    #[stable(feature = "simd_x86", since = "1.27.0")]
    @BIND_FEATURE_NAME: "abm"; "lzcnt"; // abm is a synonym for lzcnt <br>abm 是 lzcnt 的同义词<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] aes: "aes";
    /// AES (Advanced Encryption Standard New Instructions AES-NI) <br>AES (高级加密标准新指令 AES-NI)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] pclmulqdq: "pclmulqdq";
    /// CLMUL (Carry-less Multiplication) <br>CLMUL (无进位乘法)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] rdrand: "rdrand";
    /// RDRAND
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] rdseed: "rdseed";
    /// RDSEED
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] tsc: "tsc";
    /// TSC (Time Stamp Counter) <br>(时间戳计数器)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] mmx: "mmx";
    /// MMX (MultiMedia eXtensions) <br>MMX (多媒体扩展)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] sse: "sse";
    /// SSE (Streaming SIMD Extensions) <br>SSE (流式 SIMD 扩展)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] sse2: "sse2";
    /// SSE2 (Streaming SIMD Extensions 2) <br>SSE2 (流式 SIMD 扩展 2)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] sse3: "sse3";
    /// SSE3 (Streaming SIMD Extensions 3) <br>SSE3 (流式 SIMD 扩展 3)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] ssse3: "ssse3";
    /// SSSE3 (Supplemental Streaming SIMD Extensions 3) <br>SSSE3 (补充流 SIMD 扩展 3)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] sse4_1: "sse4.1";
    /// SSE4.1 (Streaming SIMD Extensions 4.1) <br>SSE4.1 (流式 SIMD 扩展 4.1)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] sse4_2: "sse4.2";
    /// SSE4.2 (Streaming SIMD Extensions 4.2) <br>SSE4.2 (流式 SIMD 扩展 4.2)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] sse4a: "sse4a";
    /// SSE4a (Streaming SIMD Extensions 4a) <br>SSE4a (流式 SIMD 扩展 4a)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] sha: "sha";
    /// SHA
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx: "avx";
    /// AVX (Advanced Vector Extensions) <br>AVX (高级 Vector 扩展)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx2: "avx2";
    /// AVX2 (Advanced Vector Extensions 2) <br>AVX2 (高级 Vector 扩展 2)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512f: "avx512f" ;
    /// AVX-512 F (Foundation)
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512cd: "avx512cd" ;
    /// AVX-512 CD (Conflict Detection Instructions) <br>AVX-512 CD (冲突检测指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512er: "avx512er";
    /// AVX-512 ER (Expo nential and Reciprocal Instructions) <br>AVX-512 ER (指数和倒数指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512pf: "avx512pf";
    /// AVX-512 PF (Prefetch Instructions) <br>AVX-512 PF (预取指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512bw: "avx512bw";
    /// AVX-512 BW (Byte and Word Instructions) <br>AVX-512 BW (字节和字指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512dq: "avx512dq";
    /// AVX-512 DQ (Doubleword and Quadword) <br>AVX-512 DQ (双字和四字)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512vl: "avx512vl";
    /// AVX-512 VL (Vector Length Extensions) <br>AVX-512 VL (Vector 长度扩展)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512ifma: "avx512ifma";
    /// AVX-512 IFMA (Integer Fused Multiply Add) <br>AVX-512 IFMA (整数融合乘加)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512vbmi: "avx512vbmi";
    /// AVX-512 VBMI (Vector Byte Manipulation Instructions) <br>AVX-512 VBMI (Vector 字节操作指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512vpopcntdq: "avx512vpopcntdq";
    /// AVX-512 VPOPCNTDQ (Vector Population Count Doubleword and Quadword) <br>AVX-512 VPOPCNTDQ (Vector 总体计数双字和四字)<br>
    ///
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512vbmi2: "avx512vbmi2";
    /// AVX-512 VBMI2 (Additional byte, word, dword and qword capabilities) <br>AVX-512 VBMI2 (附加的字节，字，dword 和 qword 功能)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512gfni: "avx512gfni";
    /// AVX-512 GFNI (Galois Field New Instruction) <br>AVX-512 GFNI (Galois Field 新指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512vaes: "avx512vaes";
    /// AVX-512 VAES (Vector AES instruction) <br>AVX-512 VAES (Vector AES 指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512vpclmulqdq: "avx512vpclmulqdq";
    /// AVX-512 VPCLMULQDQ (Vector PCLMULQDQ instructions) <br>AVX-512 VPCLMULQDQ (Vector PCLMULQDQ 指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512vnni: "avx512vnni";
    /// AVX-512 VNNI (Vector Neural Network Instructions) <br>AVX-512 VNNI (Vector 神经网络指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512bitalg: "avx512bitalg";
    /// AVX-512 BITALG (Support for VPOPCNT\[B,W\] and VPSHUFBITQMB) <br>AVX-512 BITALG (支持 VPOPCNT\[B,W\] 和 VPSHUFBITQMB)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512bf16: "avx512bf16";
    /// AVX-512 BF16 (BFLOAT16 instructions) <br>AVX-512 BF16 (BFLOAT16 指令)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] avx512vp2intersect: "avx512vp2intersect";
    /// AVX-512 P2INTERSECT <br>AVX-512 P2 相交<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] f16c: "f16c";
    /// F16C (Conversions between IEEE-754 `binary16` and `binary32` formats) <br>F16C (IEEE-754 `binary16` 和 `binary32` 格式之间的转换)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] fma: "fma";
    /// FMA (Fused Multiply Add) <br>FMA (融合乘加)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] bmi1: "bmi1" ;
    /// BMI1 (Bit Manipulation Instructions 1) <br>BMI1 (位操作指令 1)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] bmi2: "bmi2" ;
    /// BMI2 (Bit Manipulation Instructions 2) <br>BMI2 (位操作指令 2)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] lzcnt: "lzcnt";
    /// ABM (Advanced Bit Manipulation) / LZCNT (Leading Zero Count) <br>ABM (高级位操作) /LZCNT (前导零计数)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] tbm: "tbm";
    /// TBM (Trailing Bit Manipulation) <br>TBM (尾随位操作)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] popcnt: "popcnt";
    /// POPCNT (Population Count) <br>POPCNT (Population 计数)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] fxsr: "fxsr";
    /// FXSR (Floating-point context fast save and restore) <br>FXSR (浮点上下文快速保存和恢复)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] xsave: "xsave";
    /// XSAVE (Save Processor Extended States) <br>XSAVE (保存处理器扩展状态)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] xsaveopt: "xsaveopt";
    /// XSAVEOPT (Save Processor Extended States Optimized) <br>XSAVEOPT (保存处理器扩展状态优化)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] xsaves: "xsaves";
    /// XSAVES (Save Processor Extended States Supervisor) <br>XSAVES (保存处理器扩展状态管理器)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] xsavec: "xsavec";
    /// XSAVEC (Save Processor Extended States Compacted) <br>XSAVEC (保存处理器扩展状态压缩)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] cmpxchg16b: "cmpxchg16b";
    /// CMPXCH16B (16-byte compare-and-swap instruction) <br>CMPXCH16B (16 字节比较和交换指令)<br>
    @FEATURE: #[stable(feature = "simd_x86_adx", since = "1.33.0")] adx: "adx";
    /// ADX, Intel ADX (Multi-Precision Add-Carry Instruction Extensions) <br>ADX，Intel ADX (多精度加法进位指令扩展)<br>
    @FEATURE: #[stable(feature = "simd_x86", since = "1.27.0")] rtm: "rtm";
    /// RTM, Intel (Restricted Transactional Memory) <br>RTM，英特尔 (受限事务内存)<br>
}
