//! Caches run-time feature detection so that it only needs to be computed once. <br>缓存运行时特性检测，以便只需要计算一次。<br>
//!

#![allow(dead_code)] // not used on all platforms <br>并非在所有平台上都使用<br>

use core::sync::atomic::Ordering;

use core::sync::atomic::AtomicUsize;

/// Sets the `bit` of `x`. <br>设置 `x` 的 `bit`。<br>
#[inline]
const fn set_bit(x: u64, bit: u32) -> u64 {
    x | 1 << bit
}

/// Tests the `bit` of `x`. <br>测试 `x` 的 `bit`。<br>
#[inline]
const fn test_bit(x: u64, bit: u32) -> bool {
    x & (1 << bit) != 0
}

/// Unset the `bit of `x`. <br>取消设置 `x` 的位。<br>
#[inline]
const fn unset_bit(x: u64, bit: u32) -> u64 {
    x & !(1 << bit)
}

/// Maximum number of features that can be cached. <br>可以缓存的最大特性数。<br>
const CACHE_CAPACITY: u32 = 62;

/// This type is used to initialize the cache <br>此类型用于初始化缓存<br>
#[derive(Copy, Clone)]
pub(crate) struct Initializer(u64);

#[allow(clippy::use_self)]
impl Default for Initializer {
    fn default() -> Self {
        Initializer(0)
    }
}

// NOTE: the `debug_assert!` would catch that we do not add more Features than the one fitting our cache. <br>`debug_assert!` 会发现我们不会添加比适合我们缓存的特性更多的特性。<br>
//
impl Initializer {
    /// Tests the `bit` of the cache. <br>测试缓存的 `bit`。<br>
    #[inline]
    pub(crate) fn test(self, bit: u32) -> bool {
        debug_assert!(
            bit < CACHE_CAPACITY,
            "too many features, time to increase the cache size!"
        );
        test_bit(self.0, bit)
    }

    /// Sets the `bit` of the cache. <br>设置缓存的 `bit`。<br>
    #[inline]
    pub(crate) fn set(&mut self, bit: u32) {
        debug_assert!(
            bit < CACHE_CAPACITY,
            "too many features, time to increase the cache size!"
        );
        let v = self.0;
        self.0 = set_bit(v, bit);
    }

    /// Unsets the `bit` of the cache. <br>取消设置缓存的 `bit`。<br>
    #[inline]
    pub(crate) fn unset(&mut self, bit: u32) {
        debug_assert!(
            bit < CACHE_CAPACITY,
            "too many features, time to increase the cache size!"
        );
        let v = self.0;
        self.0 = unset_bit(v, bit);
    }
}

/// This global variable is a cache of the features supported by the CPU. <br>这个全局变量是 CPU 支持的特性缓存。<br>
// Note: on x64, we only use the first slot <br>在 x64 上，我们仅使用第一个插槽<br>
static CACHE: [Cache; 2] = [Cache::uninitialized(), Cache::uninitialized()];

/// Feature cache with capacity for `size_of::<usize::MAX>() * 8 - 1` features. <br>具有 `size_of::<usize::MAX>() * 8 - 1` 容量的特性缓存。<br>
///
/// Note: 0 is used to represent an uninitialized cache, and (at least) the most significant bit is set on any cache which has been initialized. <br>0 用于表示未初始化的缓存，并且 (至少) 最高有效位设置在任何已初始化的缓存上。<br>
///
///
/// Note: we use `Relaxed` atomic operations, because we are only interested in the effects of operations on a single memory location. <br>我们使用 `Relaxed` 原子操作，因为我们只对操作在单个内存位置上的影响感兴趣。<br>
/// That is, we only need "modification order", and not the full-blown "happens before". <br>也就是说，我们只需要 "modification order"，而不需要完全 "happens before"。<br>
///
struct Cache(AtomicUsize);

impl Cache {
    const CAPACITY: u32 = (core::mem::size_of::<usize>() * 8 - 1) as u32;
    const MASK: usize = (1 << Cache::CAPACITY) - 1;
    const INITIALIZED_BIT: usize = 1usize << Cache::CAPACITY;

    /// Creates an uninitialized cache. <br>创建一个未初始化的缓存。<br>
    #[allow(clippy::declare_interior_mutable_const)]
    const fn uninitialized() -> Self {
        Cache(AtomicUsize::new(0))
    }

    /// Is the `bit` in the cache set? <br>缓存中的 `bit` 是否已设置？<br> Returns `None` if the cache has not been initialized. <br>如果尚未初始化缓存，则返回 `None`。<br>
    #[inline]
    pub(crate) fn test(&self, bit: u32) -> Option<bool> {
        let cached = self.0.load(Ordering::Relaxed);
        if cached == 0 {
            None
        } else {
            Some(test_bit(cached as u64, bit))
        }
    }

    /// Initializes the cache. <br>初始化缓存。<br>
    #[inline]
    fn initialize(&self, value: usize) -> usize {
        debug_assert_eq!((value & !Cache::MASK), 0);
        self.0
            .store(value | Cache::INITIALIZED_BIT, Ordering::Relaxed);
        value
    }
}

cfg_if::cfg_if! {
    if #[cfg(feature = "std_detect_env_override")] {
        #[inline]
        fn initialize(mut value: Initializer) -> Initializer {
            let env = unsafe {
                libc::getenv(b"RUST_STD_DETECT_UNSTABLE\0".as_ptr() as *const libc::c_char)
            };
            if !env.is_null() {
                let len = unsafe { libc::strlen(env) };
                let env = unsafe { core::slice::from_raw_parts(env as *const u8, len) };
                if let Ok(disable) = core::str::from_utf8(env) {
                    for v in disable.split(" ") {
                        let _ = super::Feature::from_str(v).map(|v| value.unset(v as u32));
                    }
                }
            }
            do_initialize(value);
            value
        }
    } else {
        #[inline]
        fn initialize(value: Initializer) -> Initializer {
            do_initialize(value);
            value
        }
    }
}

#[inline]
fn do_initialize(value: Initializer) {
    CACHE[0].initialize((value.0) as usize & Cache::MASK);
    CACHE[1].initialize((value.0 >> Cache::CAPACITY) as usize & Cache::MASK);
}

// We only have to detect features once, and it's fairly costly, so hint to LLVM that it should assume that cache hits are more common than misses (which is the point of caching). <br>我们只需要检测一次特性，而且成本相当高，所以向 LLVM 提示它应该假设缓存命中比未命中更常见 (这是缓存的重点)。<br>
// It's possibly unfortunate that this function needs to reach across modules like this to call `os::detect_features`, but it produces the best code out of several attempted variants. <br>可能不幸的是，此函数需要跨这样的模块才能调用 `os::detect_features`，但它会在尝试的多个变体中产生最佳的代码。<br>
//
//
// The `Initializer` that the cache was initialized with is returned, so that the caller can call `test()` on it without having to load the value from the cache again. <br>返回用于初始化缓存的 `Initializer`，以便调用者可以在其上调用 `test()`，而不必再次从缓存中加载该值。<br>
//
//
//
//
#[cold]
fn detect_and_initialize() -> Initializer {
    initialize(super::os::detect_features())
}

/// Tests the `bit` of the storage. <br>测试存储器的 `bit`。<br>
/// If the storage has not been initialized, initializes it with the result of `os::detect_features()`. <br>如果尚未初始化存储，请使用 `os::detect_features()` 的结果对其进行初始化。<br>
///
/// On its first invocation, it detects the CPU features and caches them in the `CACHE` global variable as an `AtomicU64`. <br>首次调用时，它将检测到 CPU 特性，并将其作为 `AtomicU64` 缓存在 `CACHE` 变量中。<br>
///
/// It uses the `Feature` variant to index into this variable as a bitset. <br>它使用 `Feature` 变体作为位集索引到该变体中。<br> If the bit is set, the feature is enabled, and otherwise it is disabled. <br>如果该位置 1，则启用该特性，否则禁用该特性。<br>
///
/// If the feature `std_detect_env_override` is enabled looks for the env variable `RUST_STD_DETECT_UNSTABLE` and uses its its content to disable Features that would had been otherwise detected. <br>如果启用了特性 `std_detect_env_override`，则查找 env 变量 `RUST_STD_DETECT_UNSTABLE`，并使用其内容禁用本来可以检测到的特性。<br>
///
///
///
///
#[inline]
pub(crate) fn test(bit: u32) -> bool {
    let (relative_bit, idx) = if bit < Cache::CAPACITY {
        (bit, 0)
    } else {
        (bit - Cache::CAPACITY, 1)
    };
    CACHE[idx]
        .test(relative_bit)
        .unwrap_or_else(|| detect_and_initialize().test(bit))
}
