//! This module implements run-time feature detection. <br>此模块实现运行时特性检测。<br>
//!
//! The `is_{arch}_feature_detected!("feature-name")` macros take the name of a feature as a string-literal, and return a boolean indicating whether the feature is enabled at run-time or not. <br>`is_{arch}_feature_detected!("feature-name")` 宏将特性名称命名为字符串 - 字面量，并返回一个布尔值，指示是否在运行时启用该特性。<br>
//!
//!
//! These macros do two things: <br>这些宏可做两件事：<br>
//! * map the string-literal into an integer stored as a `Feature` enum, <br>将字符串字面量映射为存储为 `Feature` 枚举的整数，<br>
//! * call a `os::check_for(x: Feature)` function that returns `true` if the feature is enabled. <br>调用一个 `os::check_for(x: Feature)` 函数，如果启用了该特性，则返回 `true`。<br>
//!
//! The `Feature` enums are also implemented in the `arch/{target_arch}.rs` modules. <br>`Feature` 枚举也在 `arch/{target_arch}.rs` 模块中实现。<br>
//!
//! The `check_for` functions are, in general, Operating System dependent. <br>通常，`check_for` 函数通常依赖于操作系统。<br>
//! Most architectures do not allow user-space programs to query the feature bits due to security concerns (x86 is the big exception). <br>由于安全方面的考虑，大多数体系结构都不允许 user-space 程序查询特性位 (x86 是个大例外)。<br>
//! These functions are implemented in the `os/{target_os}.rs` modules. <br>这些函数在 `os/{target_os}.rs` 模块中实现。<br>
//!
//!
//!
//!

use cfg_if::cfg_if;

#[macro_use]
mod macros;

mod arch;

// This module needs to be public because the `is_{arch}_feature_detected!` macros expand calls to items within it in user crates. <br>该模块必须是公共的，因为 `is_{arch}_feature_detected!` 宏在用户 crates 中扩展了对其内项目的调用。<br>
//
#[doc(hidden)]
pub use self::arch::__is_feature_detected;

pub(crate) use self::arch::Feature;

mod bit;
mod cache;

cfg_if! {
    if #[cfg(miri)] {
        // When running under miri all target-features that are not enabled at compile-time are reported as disabled at run-time. <br>在 miri 下运行时，在编译时未启用的所有目标特性都将在运行时报告为已禁用。<br>
        //
        //
        // For features for which `cfg(target_feature)` returns true, this run-time detection logic is never called. <br>对于 `cfg(target_feature)` 返回 true 的特性，永远不会调用此运行时检测逻辑。<br>
        //
        #[path = "os/other.rs"]
        mod os;
    } else if #[cfg(any(target_arch = "x86", target_arch = "x86_64"))] {
        // On x86/x86_64 no OS specific functionality is required. <br>在 x86/x86_64 上，不需要特定于操作系统的功能。<br>
        #[path = "os/x86.rs"]
        mod os;
    } else if #[cfg(all(target_os = "linux", feature = "libc"))] {
        #[path = "os/linux/mod.rs"]
        mod os;
    } else if #[cfg(all(target_os = "freebsd", feature = "libc"))] {
        #[cfg(target_arch = "aarch64")]
        #[path = "os/aarch64.rs"]
        mod aarch64;
        #[path = "os/freebsd/mod.rs"]
        mod os;
    } else if #[cfg(all(target_os = "windows", target_arch = "aarch64"))] {
        #[path = "os/windows/aarch64.rs"]
        mod os;
    } else {
        #[path = "os/other.rs"]
        mod os;
    }
}

/// Performs run-time feature detection. <br>执行运行时特性检测。<br>
#[inline]
#[allow(dead_code)]
fn check_for(x: Feature) -> bool {
    cache::test(x as u32)
}

/// Returns an `Iterator<Item=(&'static str, bool)>` where `Item.0` is the feature name, and `Item.1` is a `bool` which is `true` if the feature is supported by the host and `false` otherwise. <br>返回 `Iterator<Item=(&'static str, bool)>`，其中 `Item.0` 是特性名称，`Item.1` 是 `bool`，如果主机支持该特性，则 `true` 是 `true`，否则返回 `false`。<br>
///
///
#[unstable(feature = "stdsimd", issue = "27731")]
pub fn features() -> impl Iterator<Item = (&'static str, bool)> {
    cfg_if! {
        if #[cfg(any(
            target_arch = "x86",
            target_arch = "x86_64",
            target_arch = "arm",
            target_arch = "aarch64",
            target_arch = "riscv32",
            target_arch = "riscv64",
            target_arch = "powerpc",
            target_arch = "powerpc64",
            target_arch = "mips",
            target_arch = "mips64",
        ))] {
            (0_u8..Feature::_last as u8).map(|discriminant: u8| {
                #[allow(bindings_with_variant_name)] // RISC-V has Feature::f <br>RISC-V 具有 Feature::f<br>
                let f: Feature = unsafe { core::mem::transmute(discriminant) };
                let name: &'static str = f.to_str();
                let enabled: bool = check_for(f);
                (name, enabled)
            })
        } else {
            None.into_iter()
        }
    }
}
