//! Linux 上 ARM 的运行时特性检测。

use super::auxvec;
use crate::detect::{bit, cache, Feature};

/// 尝试从辅助 vector 读取特性，如果失败，请尝试从 /proc/cpuinfo 读取特性。
///
pub(crate) fn detect_features() -> cache::Initializer {
    let mut value = cache::Initializer::default();
    let enable_feature = |value: &mut cache::Initializer, f, enable| {
        if enable {
            value.set(f as u32);
        }
    };

    // 这些值是特定于平台的 [asm/hwcap.h][hwcap] 的一部分
    //
    // [hwcap]: https://github.com/torvalds/linux/blob/master/arch/arm/include/uapi/asm/hwcap.h
    if let Ok(auxv) = auxvec::auxv() {
        enable_feature(&mut value, Feature::neon, bit::test(auxv.hwcap, 12));
        enable_feature(&mut value, Feature::pmull, bit::test(auxv.hwcap2, 1));
        enable_feature(&mut value, Feature::crc, bit::test(auxv.hwcap2, 4));
        enable_feature(
            &mut value,
            Feature::crypto,
            bit::test(auxv.hwcap2, 0)
                && bit::test(auxv.hwcap2, 1)
                && bit::test(auxv.hwcap2, 2)
                && bit::test(auxv.hwcap2, 3),
        );
        enable_feature(&mut value, Feature::aes, bit::test(auxv.hwcap2, 0));
        // SHA2 需要 SHA1 和 SHA2 特性
        enable_feature(
            &mut value,
            Feature::sha2,
            bit::test(auxv.hwcap2, 2) && bit::test(auxv.hwcap2, 3),
        );
        return value;
    }

    #[cfg(feature = "std_detect_file_io")]
    if let Ok(c) = super::cpuinfo::CpuInfo::new() {
        enable_feature(
            &mut value,
            Feature::neon,
            c.field("Features").has("neon") && !has_broken_neon(&c),
        );
        enable_feature(&mut value, Feature::pmull, c.field("Features").has("pmull"));
        enable_feature(&mut value, Feature::crc, c.field("Features").has("crc32"));
        enable_feature(
            &mut value,
            Feature::crypto,
            c.field("Features").has("aes")
                && c.field("Features").has("pmull")
                && c.field("Features").has("sha1")
                && c.field("Features").has("sha2"),
        );
        enable_feature(&mut value, Feature::aes, c.field("Features").has("aes"));
        enable_feature(
            &mut value,
            Feature::sha2,
            c.field("Features").has("sha1") && c.field("Features").has("sha2"),
        );
        return value;
    }
    value
}

/// 是否已知 CPU 有损坏的 NEON unit 吗？
///
///  See https://crbug.com/341598.
#[cfg(feature = "std_detect_file_io")]
fn has_broken_neon(cpuinfo: &super::cpuinfo::CpuInfo) -> bool {
    cpuinfo.field("CPU implementer") == "0x51"
        && cpuinfo.field("CPU architecture") == "7"
        && cpuinfo.field("CPU variant") == "0x1"
        && cpuinfo.field("CPU part") == "0x04d"
        && cpuinfo.field("CPU revision") == "0"
}
