//! Parses ELF auxiliary vectors. <br>解析 ELF 辅助 vectors。<br>
#![allow(dead_code)]

pub(crate) const AT_NULL: usize = 0;

/// Key to access the CPU Hardware capabilities bitfield. <br>访问 CPU 硬件功能位域的键。<br>
pub(crate) const AT_HWCAP: usize = 16;
/// Key to access the CPU Hardware capabilities 2 bitfield. <br>访问 CPU 硬件功能 2 位域的键。<br>
#[cfg(any(
    target_arch = "arm",
    target_arch = "powerpc",
    target_arch = "powerpc64"
))]
pub(crate) const AT_HWCAP2: usize = 26;

/// Cache HWCAP bitfields of the ELF Auxiliary Vector. <br>缓存 ELF 辅助 Vector 的 HWCAP 位域。<br>
///
/// If an entry cannot be read all the bits in the bitfield are set to zero. <br>如果无法读取条目，则位域中的所有位均设置为零。<br>
/// This should be interpreted as all the features being disabled. <br>这应该解释为所有特性都被禁用。<br>
#[derive(Debug, Copy, Clone)]
pub(crate) struct AuxVec {
    pub hwcap: usize,
    #[cfg(any(
        target_arch = "arm",
        target_arch = "powerpc",
        target_arch = "powerpc64"
    ))]
    pub hwcap2: usize,
}

/// ELF Auxiliary Vector <br>ELF 辅助 Vector<br>
///
/// The auxiliary vector is a memory region in a running ELF program's stack composed of (key: usize, value: usize) pairs. <br>辅助 vector 是正在运行的 ELF 程序栈中的存储区域，该栈由 (键: usize，值: usize) 对组成。<br>
///
/// The keys used in the aux vector are platform dependent. <br>辅助 vector 中使用的键取决于平台。<br> For Linux, they are defined in [linux/auxvec.h][auxvec_h]. <br>对于 Linux，它们在 [linux/auxvec.h][auxvec_h] 中定义。<br>
/// The hardware capabilities of a given CPU can be queried with the  `AT_HWCAP` and `AT_HWCAP2` keys. <br>可以使用 `AT_HWCAP` 和 `AT_HWCAP2` 键查询给定 CPU 的硬件功能。<br>
///
/// There is no perfect way of reading the auxiliary vector. <br>没有读取辅助 vector 的完美方法。<br>
///
/// - If the `std_detect_dlsym_getauxval` cargo feature is enabled, this will use `getauxval` if its linked to the binary, and otherwise proceed to a fallback implementation. <br>如果启用了 `std_detect_dlsym_getauxval` cargo 特性，则在链接到二进制文件时将使用 `getauxval`，否则将进行后备实现。<br>
///
/// When `std_detect_dlsym_getauxval` is disabled, this will assume that `getauxval` is linked to the binary - if that is not the case the behavior is undefined. <br>禁用 `std_detect_dlsym_getauxval` 时，将假定 `getauxval` 已链接到二进制文件 - 如果不是这种情况，则行为未定义。<br>
/// - Otherwise, if the `std_detect_file_io` cargo feature is enabled, it will try to read `/proc/self/auxv`. <br>否则，如果启用了 `std_detect_file_io` cargo 特性，它将尝试读取 `/proc/self/auxv`。<br>
/// - If that fails, this function returns an error. <br>如果失败，则此函数返回错误。<br>
///
/// Note that run-time feature detection is not invoked for features that can be detected at compile-time. <br>请注意，对于可在编译时检测到的特性，不会调用运行时特性检测。<br>
/// Also note that if this function returns an error, cpuinfo still can (and will) be used to try to perform run-time feature detecton on some platforms. <br>还要注意，如果此函数返回错误，则 cpuinfo 仍可以 (并且将) 用于在某些平台上尝试执行运行时特性检测。<br>
///
/// For more information about when `getauxval` is available check the great [`auxv` crate documentation][auxv_docs]. <br>有关 `getauxval` 何时可用的更多信息，请检查 [`auxv` crate documentation][auxv_docs]。<br>
///
/// [auxvec_h]: https://github.com/torvalds/linux/blob/master/include/uapi/linux/auxvec.h
/// [auxv_docs]: https://docs.rs/auxv/0.3.3/auxv/
///
///
///
///
///
///
///
pub(crate) fn auxv() -> Result<AuxVec, ()> {
    #[cfg(feature = "std_detect_dlsym_getauxval")]
    {
        // Try to call a dynamically-linked getauxval function. <br>尝试调用动态链接的 getauxval 函数。<br>
        if let Ok(hwcap) = getauxval(AT_HWCAP) {
            // Targets with only AT_HWCAP: <br>仅使用 AT_HWCAP 的目标：<br>
            #[cfg(any(
                target_arch = "aarch64",
                target_arch = "riscv32",
                target_arch = "riscv64",
                target_arch = "mips",
                target_arch = "mips64"
            ))]
            {
                if hwcap != 0 {
                    return Ok(AuxVec { hwcap });
                }
            }

            // Targets with AT_HWCAP and AT_HWCAP2: <br>使用 AT_HWCAP 和 AT_HWCAP2 的目标：<br>
            #[cfg(any(
                target_arch = "arm",
                target_arch = "powerpc",
                target_arch = "powerpc64"
            ))]
            {
                if let Ok(hwcap2) = getauxval(AT_HWCAP2) {
                    if hwcap != 0 && hwcap2 != 0 {
                        return Ok(AuxVec { hwcap, hwcap2 });
                    }
                }
            }
            drop(hwcap);
        }
    }

    #[cfg(not(feature = "std_detect_dlsym_getauxval"))]
    {
        // Targets with only AT_HWCAP: <br>仅使用 AT_HWCAP 的目标：<br>
        #[cfg(any(
            target_arch = "aarch64",
            target_arch = "riscv32",
            target_arch = "riscv64",
            target_arch = "mips",
            target_arch = "mips64"
        ))]
        {
            let hwcap = unsafe { libc::getauxval(AT_HWCAP as libc::c_ulong) as usize };
            if hwcap != 0 {
                return Ok(AuxVec { hwcap });
            }
        }

        // Targets with AT_HWCAP and AT_HWCAP2: <br>使用 AT_HWCAP 和 AT_HWCAP2 的目标：<br>
        #[cfg(any(
            target_arch = "arm",
            target_arch = "powerpc",
            target_arch = "powerpc64"
        ))]
        {
            let hwcap = unsafe { libc::getauxval(AT_HWCAP as libc::c_ulong) as usize };
            let hwcap2 = unsafe { libc::getauxval(AT_HWCAP2 as libc::c_ulong) as usize };
            if hwcap != 0 && hwcap2 != 0 {
                return Ok(AuxVec { hwcap, hwcap2 });
            }
        }
    }

    #[cfg(feature = "std_detect_file_io")]
    {
        // If calling getauxval fails, try to read the auxiliary vector from its file: <br>如果调用 getauxval 失败，请尝试从其文件中读取辅助 vector：<br>
        //
        auxv_from_file("/proc/self/auxv")
    }
    #[cfg(not(feature = "std_detect_file_io"))]
    {
        Err(())
    }
}

/// Tries to read the `key` from the auxiliary vector by calling the dynamically-linked `getauxval` function. <br>尝试通过调用动态链接的 `getauxval` 函数从辅助 vector 中读取 `key`。<br>
///
/// If the function is not linked, this function return `Err`. <br>如果未链接该函数，则此函数返回 `Err`。<br>
#[cfg(feature = "std_detect_dlsym_getauxval")]
fn getauxval(key: usize) -> Result<usize, ()> {
    use libc;
    pub type F = unsafe extern "C" fn(usize) -> usize;
    unsafe {
        let ptr = libc::dlsym(libc::RTLD_DEFAULT, "getauxval\0".as_ptr() as *const _);
        if ptr.is_null() {
            return Err(());
        }

        let ffi_getauxval: F = core::mem::transmute(ptr);
        Ok(ffi_getauxval(key))
    }
}

/// Tries to read the auxiliary vector from the `file`. <br>尝试从 `file` 读取辅助 vector。<br>
/// If this fails, this function returns `Err`. <br>如果失败，则此函数返回 `Err`。<br>
#[cfg(feature = "std_detect_file_io")]
fn auxv_from_file(file: &str) -> Result<AuxVec, ()> {
    let file = super::read_file(file)?;

    // See <https://github.com/torvalds/linux/blob/v3.19/include/uapi/linux/auxvec.h>. <br>请参见 <https://github.com/torvalds/linux/blob/v3.19/include/uapi/linux/auxvec.h>。<br>
    //
    // The auxiliary vector contains at most 32 (key,value) fields: from `AT_EXECFN = 31` to `AT_NULL = 0`. <br>辅助 vector 最多包含 32 个 (key,value) 字段：从 `AT_EXECFN = 31` 到 `AT_NULL = 0`。<br>
    // That is, a buffer of <br>就是说，<br>
    // 2*32 `usize` elements is enough to read the whole vector. <br>2 * 32 个 `usize` 元素足以读取整个 vector。<br>
    let mut buf = [0_usize; 64];
    let len = core::mem::size_of_val(&buf).max(file.len());
    unsafe {
        core::ptr::copy_nonoverlapping(file.as_ptr(), buf.as_mut_ptr() as *mut u8, len);
    }

    auxv_from_buf(&buf)
}

/// Tries to interpret the `buffer` as an auxiliary vector. <br>尝试将 `buffer` 解释为辅助 vector。<br>
/// If that fails, this function returns `Err`. <br>如果失败，则此函数返回 `Err`。<br>
#[cfg(feature = "std_detect_file_io")]
fn auxv_from_buf(buf: &[usize; 64]) -> Result<AuxVec, ()> {
    // Targets with only AT_HWCAP: <br>仅使用 AT_HWCAP 的目标：<br>
    #[cfg(any(
        target_arch = "aarch64",
        target_arch = "riscv32",
        target_arch = "riscv64",
        target_arch = "mips",
        target_arch = "mips64",
    ))]
    {
        for el in buf.chunks(2) {
            match el[0] {
                AT_NULL => break,
                AT_HWCAP => return Ok(AuxVec { hwcap: el[1] }),
                _ => (),
            }
        }
    }
    // Targets with AT_HWCAP and AT_HWCAP2: <br>使用 AT_HWCAP 和 AT_HWCAP2 的目标：<br>
    #[cfg(any(
        target_arch = "arm",
        target_arch = "powerpc",
        target_arch = "powerpc64"
    ))]
    {
        let mut hwcap = None;
        let mut hwcap2 = None;
        for el in buf.chunks(2) {
            match el[0] {
                AT_NULL => break,
                AT_HWCAP => hwcap = Some(el[1]),
                AT_HWCAP2 => hwcap2 = Some(el[1]),
                _ => (),
            }
        }

        if let (Some(hwcap), Some(hwcap2)) = (hwcap, hwcap2) {
            return Ok(AuxVec { hwcap, hwcap2 });
        }
    }
    drop(buf);
    Err(())
}

#[cfg(test)]
mod tests {
    extern crate auxv as auxv_crate;
    use super::*;

    // Reads the Auxiliary Vector key from /proc/self/auxv using the auxv crate. <br>使用 auxv crate 从 /proc/self/auxv 读取辅助 Vector 键。<br>
    //
    #[cfg(feature = "std_detect_file_io")]
    fn auxv_crate_getprocfs(key: usize) -> Option<usize> {
        use self::auxv_crate::procfs::search_procfs_auxv;
        use self::auxv_crate::AuxvType;
        let k = key as AuxvType;
        match search_procfs_auxv(&[k]) {
            Ok(v) => Some(v[&k] as usize),
            Err(_) => None,
        }
    }

    // Reads the Auxiliary Vector key from getauxval() using the auxv crate. <br>使用 auxv crate 从 getauxval() 读取辅助 Vector 键。<br>
    //
    #[cfg(not(any(target_arch = "mips", target_arch = "mips64")))]
    fn auxv_crate_getauxval(key: usize) -> Option<usize> {
        use self::auxv_crate::getauxval::Getauxval;
        use self::auxv_crate::AuxvType;
        let q = auxv_crate::getauxval::NativeGetauxval {};
        match q.getauxval(key as AuxvType) {
            Ok(v) => Some(v as usize),
            Err(_) => None,
        }
    }

    // FIXME: on mips/mips64 getauxval returns 0, and /proc/self/auxv does not always contain the AT_HWCAP key under qemu. <br>在 mips/mips64 上，getauxval 返回 0，并且 /proc/self/auxv 并不始终在 qemu 下包含 AT_HWCAP 密钥。<br>
    //
    #[cfg(any(
        target_arch = "aarch64",
        target_arch = "arm",
        target_arch = "powerpc",
        target_arch = "powerpc64"
    ))]
    #[test]
    fn auxv_crate() {
        let v = auxv();
        if let Some(hwcap) = auxv_crate_getauxval(AT_HWCAP) {
            let rt_hwcap = v.expect("failed to find hwcap key").hwcap;
            assert_eq!(rt_hwcap, hwcap);
        }

        // Targets with AT_HWCAP and AT_HWCAP2: <br>使用 AT_HWCAP 和 AT_HWCAP2 的目标：<br>
        #[cfg(any(
            target_arch = "arm",
            target_arch = "powerpc",
            target_arch = "powerpc64"
        ))]
        {
            if let Some(hwcap2) = auxv_crate_getauxval(AT_HWCAP2) {
                let rt_hwcap2 = v.expect("failed to find hwcap2 key").hwcap2;
                assert_eq!(rt_hwcap2, hwcap2);
            }
        }
    }

    #[test]
    fn auxv_dump() {
        if let Ok(auxvec) = auxv() {
            println!("{:?}", auxvec);
        } else {
            println!("both getauxval() and reading /proc/self/auxv failed!");
        }
    }

    #[cfg(feature = "std_detect_file_io")]
    cfg_if::cfg_if! {
        if #[cfg(target_arch = "arm")] {
            #[test]
            fn linux_rpi3() {
                let file = concat!(env!("CARGO_MANIFEST_DIR"), "/src/detect/test_data/linux-rpi3.auxv");
                println!("file: {}", file);
                let v = auxv_from_file(file).unwrap();
                assert_eq!(v.hwcap, 4174038);
                assert_eq!(v.hwcap2, 16);
            }

            #[test]
            #[should_panic]
            fn linux_macos_vb() {
                let file = concat!(env!("CARGO_MANIFEST_DIR"), "/src/detect/test_data/macos-virtualbox-linux-x86-4850HQ.auxv");
                println!("file: {}", file);
                let v = auxv_from_file(file).unwrap();
                // this file is incomplete (contains hwcap but not hwcap2), we want to fall back to /proc/cpuinfo in this case, so reading should fail. <br>该文件不完整 (包含 hwcap，但不包含 hwcap2)，在这种情况下，我们想回退到 /proc/cpuinfo，因此读取失败。<br>
                // assert_eq!(v.hwcap, 126614527);
                // assert_eq!(v.hwcap2, 0);
                //
                let _ = v;
            }
        } else if #[cfg(target_arch = "aarch64")] {
            #[test]
            fn linux_x64() {
                let file = concat!(env!("CARGO_MANIFEST_DIR"), "/src/detect/test_data/linux-x64-i7-6850k.auxv");
                println!("file: {}", file);
                let v = auxv_from_file(file).unwrap();
                assert_eq!(v.hwcap, 3219913727);
            }
        }
    }

    #[test]
    #[cfg(feature = "std_detect_file_io")]
    fn auxv_dump_procfs() {
        if let Ok(auxvec) = auxv_from_file("/proc/self/auxv") {
            println!("{:?}", auxvec);
        } else {
            println!("reading /proc/self/auxv failed!");
        }
    }

    #[cfg(any(
        target_arch = "aarch64",
        target_arch = "arm",
        target_arch = "powerpc",
        target_arch = "powerpc64"
    ))]
    #[test]
    #[cfg(feature = "std_detect_file_io")]
    fn auxv_crate_procfs() {
        let v = auxv();
        if let Some(hwcap) = auxv_crate_getprocfs(AT_HWCAP) {
            assert_eq!(v.unwrap().hwcap, hwcap);
        }

        // Targets with AT_HWCAP and AT_HWCAP2: <br>使用 AT_HWCAP 和 AT_HWCAP2 的目标：<br>
        #[cfg(any(
            target_arch = "arm",
            target_arch = "powerpc",
            target_arch = "powerpc64"
        ))]
        {
            if let Some(hwcap2) = auxv_crate_getprocfs(AT_HWCAP2) {
                assert_eq!(v.unwrap().hwcap2, hwcap2);
            }
        }
    }
}
