//! Linux 上 PowerPC 的运行时特性检测。

use super::auxvec;
use crate::detect::{cache, Feature};

/// 尝试从辅助 vector 读取特性，如果失败，请尝试从 /proc/cpuinfo 读取特性。
///
pub(crate) fn detect_features() -> cache::Initializer {
    let mut value = cache::Initializer::default();
    let enable_feature = |value: &mut cache::Initializer, f, enable| {
        if enable {
            value.set(f as u32);
        }
    };

    // 这些值是特定于平台的 [asm/cputable.h][cputable] 的一部分
    //
    // [cputable]: https://github.com/torvalds/linux/blob/master/arch/powerpc/include/uapi/asm/cputable.h
    if let Ok(auxv) = auxvec::auxv() {
        // note: PowerPC 值是进行测试的掩码 (而不是像 ARM 和 Aarch64 一样要测试的位的索引)
        //
        enable_feature(&mut value, Feature::altivec, auxv.hwcap & 0x10000000 != 0);
        enable_feature(&mut value, Feature::vsx, auxv.hwcap & 0x00000080 != 0);
        enable_feature(&mut value, Feature::power8, auxv.hwcap2 & 0x80000000 != 0);
        return value;
    }

    // PowerPC 的 /proc/cpuinfo 缺少适当的 Feature 字段，但 `cpu` 字段中指出了对 `altivec` 的支持。
    //
    #[cfg(feature = "std_detect_file_io")]
    if let Ok(c) = super::cpuinfo::CpuInfo::new() {
        enable_feature(&mut value, Feature::altivec, c.field("cpu").has("altivec"));
        return value;
    }
    value
}
