//! Run-time feature detection for Aarch64 on Windows. <br>Windows 上 Aarch64 的运行时特性检测。<br>

use crate::detect::{cache, Feature};

/// Try to read the features using IsProcessorFeaturePresent. <br>尝试使用 IsProcessorFeaturePresent 来读取特性。<br>
pub(crate) fn detect_features() -> cache::Initializer {
    type DWORD = u32;
    type BOOL = i32;

    const FALSE: BOOL = 0;
    // The following Microsoft documents isn't updated for aarch64. <br>下列 Microsoft 文档未针对 aarch64 更新。<br>
    // https://docs.microsoft.com/en-us/windows/win32/api/processthreadsapi/nf-processthreadsapi-isprocessorfeaturepresent
    // These are defined in winnt.h of Windows SDK <br>这些在 Windows SDK 的 winnt.h 中定义<br>
    const PF_ARM_NEON_INSTRUCTIONS_AVAILABLE: u32 = 19;
    const PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE: u32 = 30;
    const PF_ARM_V8_CRC32_INSTRUCTIONS_AVAILABLE: u32 = 31;

    extern "system" {
        pub fn IsProcessorFeaturePresent(ProcessorFeature: DWORD) -> BOOL;
    }

    let mut value = cache::Initializer::default();
    {
        let mut enable_feature = |f, enable| {
            if enable {
                value.set(f as u32);
            }
        };

        // Some features such Feature::fp may be supported on current CPU, but no way to detect it by OS API. <br>当前的 CPU 可能支持某些特性，例如 Feature::fp，但无法通过 OS API 进行检测。<br>
        //
        // Also, we require unsafe block for the extern "system" calls. <br>另外，我们要求外部 "system" 调用使用不安全的块。<br>
        unsafe {
            enable_feature(
                Feature::asimd,
                IsProcessorFeaturePresent(PF_ARM_NEON_INSTRUCTIONS_AVAILABLE) != FALSE,
            );
            enable_feature(
                Feature::crc,
                IsProcessorFeaturePresent(PF_ARM_V8_CRC32_INSTRUCTIONS_AVAILABLE) != FALSE,
            );
            // PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE means aes, sha1, sha2 and pmull support <br>PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE 表示对 AES，sha1，sha2 和 pmull 的支持<br>
            //
            enable_feature(
                Feature::aes,
                IsProcessorFeaturePresent(PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE) != FALSE,
            );
            enable_feature(
                Feature::pmull,
                IsProcessorFeaturePresent(PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE) != FALSE,
            );
            enable_feature(
                Feature::sha2,
                IsProcessorFeaturePresent(PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE) != FALSE,
            );
        }
    }
    value
}
