//! x86 运行时特性检测与操作系统无关。

#[cfg(target_arch = "x86")]
use core::arch::x86::*;
#[cfg(target_arch = "x86_64")]
use core::arch::x86_64::*;

use core::mem;

use crate::detect::{bit, cache, Feature};

/// x86 上的运行时特性检测通过使用 CPUID 指令进行。
///
/// [CPUID 维基百科][wiki_cpuid] 页面包含有关设置哪些标志以查询哪些值，以及在哪些寄存器中报告这些标志的所有信息。
///
/// 权威的引用是：
/// - [英特尔 64 位和 IA-32 架构软件开发人员手册第 2 卷：
///   指令集引用，AZ][intel64_ref]。
/// - [AMD64 Architecture Programmer's Manual, Volume 3: General-Purpose and System Instructions][amd64_ref].
///
/// [wiki_cpuid]: https://en.wikipedia.org/wiki/CPUID
/// [intel64_ref]: http://www.intel.de/content/dam/www/public/us/en/documents/manuals/64-ia-32-architectures-software-developer-instruction-set-reference-manual-325383.pdf
/// [amd64_ref]: http://support.amd.com/TechDocs/24594.pdf
///
///
///
#[allow(clippy::similar_names)]
pub(crate) fn detect_features() -> cache::Initializer {
    let mut value = cache::Initializer::default();

    // 如果 x86 CPU 不支持 CPUID 指令，则它太旧了，无法支持任何当前可检测的特性。
    //
    if !has_cpuid() {
        return value;
    }

    // 从这里开始调用 `__cpuid`/`__cpuid_count` 是安全的，因为 CPU 支持 `cpuid`。
    //

    // 0. EAX = 0: 基本信息：
    // - EAX 返回 "最高函数参数"，即 `cpuinfo` 后续调用的最大叶子值，范围为 [0，
    //
    // 0x8000_0000].
    // - 供应商 ID 存储在 12 个 u8 ASCII 字符中，并以 EBX，EDX 和 ECX (按此顺序) 返回：
    let (max_basic_leaf, vendor_id) = unsafe {
        let CpuidResult {
            eax: max_basic_leaf,
            ebx,
            ecx,
            edx,
        } = __cpuid(0);
        let vendor_id: [[u8; 4]; 3] = [
            mem::transmute(ebx),
            mem::transmute(edx),
            mem::transmute(ecx),
        ];
        let vendor_id: [u8; 12] = mem::transmute(vendor_id);
        (max_basic_leaf, vendor_id)
    };

    if max_basic_leaf < 1 {
        // 较早的 Intel 486，未实现 CPUID
        return value;
    }

    // EAX = 1, ECX = 0: Queries "Processor Info and Feature Bits";
    // 包含有关大多数 x86 特性的信息。
    let CpuidResult {
        ecx: proc_info_ecx,
        edx: proc_info_edx,
        ..
    } = unsafe { __cpuid(0x0000_0001_u32) };

    // EAX = 7, ECX = 0: Queries "Extended Features";
    // 包含有关 bmi，bmi2 和 avx2 支持的信息。
    let (extended_features_ebx, extended_features_ecx) = if max_basic_leaf >= 7 {
        let CpuidResult { ebx, ecx, .. } = unsafe { __cpuid(0x0000_0007_u32) };
        (ebx, ecx)
    } else {
        (0, 0) // CPUID 不支持 "扩展特性"
    };

    // EAX=0x8000_0000，ECX = 0: 获得最高的扩展函数支持
    // - EAX 返回扩展信息的最大叶子值，即范围 [0x8000_0000; u32::MAX] 中的 `cpuid` 调用：
    //
    let CpuidResult {
        eax: extended_max_basic_leaf,
        ..
    } = unsafe { __cpuid(0x8000_0000_u32) };

    // EAX = 0x8000_0001, ECX=0: 查询扩展的处理器信息和特性
    // Bits"
    let extended_proc_info_ecx = if extended_max_basic_leaf >= 1 {
        let CpuidResult { ecx, .. } = unsafe { __cpuid(0x8000_0001_u32) };
        ecx
    } else {
        0
    };

    {
        // 借用值，直到此作用域结束：
        let mut enable = |r, rb, f| {
            if bit::test(r as usize, rb) {
                value.set(f as u32);
            }
        };

        enable(proc_info_ecx, 0, Feature::sse3);
        enable(proc_info_ecx, 1, Feature::pclmulqdq);
        enable(proc_info_ecx, 9, Feature::ssse3);
        enable(proc_info_ecx, 13, Feature::cmpxchg16b);
        enable(proc_info_ecx, 19, Feature::sse4_1);
        enable(proc_info_ecx, 20, Feature::sse4_2);
        enable(proc_info_ecx, 23, Feature::popcnt);
        enable(proc_info_ecx, 25, Feature::aes);
        enable(proc_info_ecx, 29, Feature::f16c);
        enable(proc_info_ecx, 30, Feature::rdrand);
        enable(extended_features_ebx, 18, Feature::rdseed);
        enable(extended_features_ebx, 19, Feature::adx);
        enable(extended_features_ebx, 11, Feature::rtm);
        enable(proc_info_edx, 4, Feature::tsc);
        enable(proc_info_edx, 23, Feature::mmx);
        enable(proc_info_edx, 24, Feature::fxsr);
        enable(proc_info_edx, 25, Feature::sse);
        enable(proc_info_edx, 26, Feature::sse2);
        enable(extended_features_ebx, 29, Feature::sha);

        enable(extended_features_ebx, 3, Feature::bmi1);
        enable(extended_features_ebx, 8, Feature::bmi2);

        // `XSAVE` 和 `AVX` 支持：
        let cpu_xsave = bit::test(proc_info_ecx as usize, 26);
        if cpu_xsave {
            // 0. CPU 在这里支持 `XSAVE`。

            // 1.
            // 检测 `OSXSAVE`，即操作系统是否已启用 AVX 并支持在上下文切换器上保存 AVX/AVX2 vector 寄存器的状态，请参见：
            //
            //
            // - [intel: is avx enabled?][is_avx_enabled],
            // - [mozilla: sse.cpp][mozilla_sse_cpp].
            //
            // [is_avx_enabled]: https://software.intel.com/en-us/blogs/2011/04/14/is-avx-enabled
            // [mozilla_sse_cpp]: https://hg.mozilla.org/mozilla-central/file/64bab5cbb9b6/mozglue/build/SSE.cpp#l190
            let cpu_osxsave = bit::test(proc_info_ecx as usize, 27);

            if cpu_osxsave {
                // 2.
                // 操作系统必须已向 CPU 发出信号，表明它支持保存和恢复以下内容：
                //
                //
                // * SSE -> `XCR0.SSE[1]`
                // * AVX -> `XCR0.AVX[2]`
                // * AVX-512 -> `XCR0.AVX-512[7:5]`.
                //
                // 通过将 `XCR0` 的相应位设置为 `1`。
                //
                // 这是安全的，因为 CPU 支持 `xsave`，并且操作系统已设置 `osxsave`。
                let xcr0 = unsafe { _xgetbv(0) };
                // 使用掩码 `0b110 == 6` 测试 `XCR0.SSE[1]` 和 `XCR0.AVX[2]`：
                let os_avx_support = xcr0 & 6 == 6;
                // 使用面罩 `0b1110_0000 == 224` 测试 `XCR0.AVX-512[7:5]`：
                let os_avx512_support = xcr0 & 224 == 224;

                // 仅当 OS 和 CPU 支持 saving/restoring 和 AVX 寄存器时，我们才启用 `xsave` 支持：
                //
                if os_avx_support {
                    // 请参见 "Intel® 64 and IA-32 Architectures Software Developer’s Manual, Volume 1: Basic Architecture" 中的 "13.3 ENABLING THE XSAVE FEATURE SET AND XSAVE-ENABLED FEATURES"：
                    //
                    //
                    // ` 软件通过将 CR4.OSXSAVE [位 18] 设置为 1 (例如，使用 MOV 至 CR4 指令) 来启用 XSAVE 特性集。
                    // 如果此位为 0，则执行 XGETBV，XRSTOR，XRSTORS，XSAVE，XSAVEC，XSAVEOPT，XSAVES 和 XSETBV 中的任何一个都会导致无效操作码异常 (#UD) `
                    //
                    //
                    //
                    //
                    //
                    enable(proc_info_ecx, 26, Feature::xsave);

                    // 对于 `xsaveopt`，`xsavec` 和 `xsaves`，我们需要查询：
                    // 处理器扩展状态枚举子叶 (EAX=0DH，ECX=1) ：
                    //
                    if max_basic_leaf >= 0xd {
                        let CpuidResult {
                            eax: proc_extended_state1_eax,
                            ..
                        } = unsafe { __cpuid_count(0xd_u32, 1) };
                        enable(proc_extended_state1_eax, 0, Feature::xsaveopt);
                        enable(proc_extended_state1_eax, 1, Feature::xsavec);
                        enable(proc_extended_state1_eax, 3, Feature::xsaves);
                    }

                    // FMA (uses 256-bit wide registers):
                    enable(proc_info_ecx, 12, Feature::fma);

                    // 和 AVX/AVX2：
                    enable(proc_info_ecx, 28, Feature::avx);
                    enable(extended_features_ebx, 5, Feature::avx2);

                    // 对于 AVX-512，操作系统还需要支持 saving/restoring 扩展状态，只有这样我们才能启用 AVX-512 支持：
                    //
                    if os_avx512_support {
                        enable(extended_features_ebx, 16, Feature::avx512f);
                        enable(extended_features_ebx, 17, Feature::avx512dq);
                        enable(extended_features_ebx, 21, Feature::avx512ifma);
                        enable(extended_features_ebx, 26, Feature::avx512pf);
                        enable(extended_features_ebx, 27, Feature::avx512er);
                        enable(extended_features_ebx, 28, Feature::avx512cd);
                        enable(extended_features_ebx, 30, Feature::avx512bw);
                        enable(extended_features_ebx, 31, Feature::avx512vl);
                        enable(extended_features_ecx, 1, Feature::avx512vbmi);
                        enable(extended_features_ecx, 5, Feature::avx512bf16);
                        enable(extended_features_ecx, 6, Feature::avx512vbmi2);
                        enable(extended_features_ecx, 8, Feature::avx512gfni);
                        enable(extended_features_ecx, 8, Feature::avx512vp2intersect);
                        enable(extended_features_ecx, 9, Feature::avx512vaes);
                        enable(extended_features_ecx, 10, Feature::avx512vpclmulqdq);
                        enable(extended_features_ecx, 11, Feature::avx512vnni);
                        enable(extended_features_ecx, 12, Feature::avx512bitalg);
                        enable(extended_features_ecx, 14, Feature::avx512vpopcntdq);
                    }
                }
            }
        }

        // 这将检测 AMD CPU 上的 ABM 和 Intel CPU 上的 LZCNT。
        // 在具有 popcnt 的英特尔 CPU 上，lzcnt 实现了 ABM 的 "missing part"，因此我们 map 都具有相同的内部特性。
        //
        //
        // `is_x86_feature_detected!("lzcnt")` 宏然后在内部 maps 到 Feature::abm。
        //
        //
        enable(extended_proc_info_ecx, 5, Feature::lzcnt);

        // Hygon Dhyana 源于 AMD 技术，与 AMD 家族 17h 共享大多数架构，但是 CPU 供应商 ID (`HygonGenuine`) / 家族系列 number(Family 18h) 不同。
        //
        //
        // 对于 CPUID 特性位，Hygon Dhyana(family 18h) 与 AMD 系列 17h 具有相同的定义。
        //
        // 相关的 AMD CPUID 规范为 https://www.amd.com/system/files/TechDocs/25481.pdf。
        // 可以在以下位置找到相关的 Hygon 内核补丁
        // http://lkml.kernel.org/r/5ce86123a7b9dad925ac583d88d2f921040e859b.1538583282.git.puwen@hygon.cn
        //
        //
        if vendor_id == *b"AuthenticAMD" || vendor_id == *b"HygonGenuine" {
            // 这些特性在 AMD arch CPU 上可用：
            enable(extended_proc_info_ecx, 6, Feature::sse4a);
            enable(extended_proc_info_ecx, 21, Feature::tbm);
        }
    }

    // 不幸的是，一些 Skylake 芯片错误地报告了对 BMI1 和 BMI2 的支持，而没有实际支持。
    // 这些芯片不支持 AVX，而且似乎所有无错误支持 BMI 的 Intel 芯片都支持 (我没有检查其他供应商)，因此我们可以为未报告支持 AVX 的芯片禁用这些标志。
    //
    // 这可能会使支持 BMI 而不是 AVX 的 future 芯片感到悲观，但这与执行不受支持的指令时遇到的严重崩溃相比似乎微不足道 (换句话说，我们可以安全地报告 CPU 特性，但是不要过度报告它们)。
    //
    // 尽管如此，为了限制这在未来可能产生的任何影响，我们仅对英特尔芯片执行此操作，因为这是仅存在于其芯片中的错误。
    //
    // 此错误在本文档的勘误部分中记录为 `SKL052`：
    // http://www.intel.com/content/dam/www/public/us/en/documents/specification-updates/desktop-6th-gen-core-family-spec-update.pdf
    //
    //
    //
    //
    //
    //
    if vendor_id == *b"GenuineIntel" && !value.test(Feature::avx as u32) {
        value.unset(Feature::bmi1 as u32);
        value.unset(Feature::bmi2 as u32);
    }

    value
}
