//! 测试标准 crate 所需的运行时支持。
//!
//! 这基本上只是反汇编当前的可执行文件，然后在全局范围内解析一次输出，然后提供 `assert` 函数，该函数对反汇编函数进行断言。
//!
//!
#![feature(bench_black_box)] // 对于 black_box
#![deny(rust_2018_idioms)]
#![allow(clippy::missing_docs_in_private_items, clippy::print_stdout)]

#[macro_use]
extern crate lazy_static;
#[macro_use]
extern crate cfg_if;

pub use assert_instr_macro::*;
pub use simd_test_macro::*;
use std::{cmp, collections::HashSet, env, hash, hint::black_box, str};

cfg_if! {
    if #[cfg(target_arch = "wasm32")] {
        pub mod wasm;
        use wasm::disassemble_myself;
    } else {
        mod disassembly;
        use crate::disassembly::disassemble_myself;
    }
}

lazy_static! {
    static ref DISASSEMBLY: HashSet<Function> = disassemble_myself();
}

#[derive(Debug)]
struct Function {
    name: String,
    instrs: Vec<String>,
}
impl Function {
    fn new(n: &str) -> Self {
        Self {
            name: n.to_string(),
            instrs: Vec::new(),
        }
    }
}

impl cmp::PartialEq for Function {
    fn eq(&self, other: &Self) -> bool {
        self.name == other.name
    }
}
impl cmp::Eq for Function {}

impl hash::Hash for Function {
    fn hash<H: hash::Hasher>(&self, state: &mut H) {
        self.name.hash(state)
    }
}

/// crate 的主入口，由 `#[assert_instr]` 宏调用。
///
/// 这断言 `fnptr` 处的函数包含提供的指令 `expected`。
///
pub fn assert(shim_addr: usize, fnname: &str, expected: &str) {
    // 确保未移除垫片
    black_box(shim_addr);

    // eprintln!("shim name: {}", fnname);
    let function = &DISASSEMBLY
        .get(&Function::new(fnname))
        .unwrap_or_else(|| panic!("function \"{}\" not found in the disassembly", fnname));
    // eprintln!("  function: {:?}", function);

    let mut instrs = &function.instrs[..];
    while instrs.last().map_or(false, |s| s == "nop") {
        instrs = &instrs[..instrs.len() - 1];
    }

    // 在此函数的任何指令的第一部分中查找 `expected`，例如 tzcntl%rax，%rax 中的 tzcntl。
    //
    // 当预期指令为 nop 时，有两种情况：
    // 1. 预期的内部函数被编译掉了，所以我们不能检查它 -- 也就是内部函数没有生成任何代码。
    // 2.
    // 它是一个标记，表示该指令将被编译成其他指令 -- 主要是因为 llvm 优化。
    //
    //
    //
    let found = expected == "nop" || instrs.iter().any(|s| s.starts_with(expected));

    // 在反汇编中查找子程序调用指令来检测是否内联失败：所有内部函数都是 `#[inline(always)]`，所以从另一个调用一个内部函数不应生成子程序调用指令。
    //
    //
    let inlining_failed = if cfg!(target_arch = "x86_64") || cfg!(target_arch = "wasm32") {
        instrs.iter().any(|s| s.starts_with("call "))
    } else if cfg!(target_arch = "x86") {
        instrs.windows(2).any(|s| {
            // 在 32 位 x86 上，位置无关代码将自行调用，紧随其后的是 `pop`，以了解当前地址。
            //
            // 在考虑函数是否无法内联某些东西时，不要考虑这一点。
            //
            s[0].starts_with("call ") && s[1].starts_with("pop") // FIXME: 与注释不匹配的原始逻辑
        })
    } else if cfg!(target_arch = "aarch64") {
        instrs.iter().any(|s| s.starts_with("bl "))
    } else {
        // FIXME: 添加对其他 archs 的检测
        false
    };

    let instruction_limit = std::env::var("STDARCH_ASSERT_INSTR_LIMIT")
        .ok()
        .map_or_else(
            || match expected {
                // `cpuid` 返回一个相当大的聚合结构体，因此将其从下面稍微严格一点的 22 条指令中排除。
                //
                "cpuid" => 30,

                // 显然，在 Windows 上，LLVM 围绕这些指令生成了一堆 Xmm 的 saves/restores 寄存器，超过了下面的 20 个限制。
                // 正如 Windows 的 ABI (我相信吗？) 所规定的那样，我们可能对此无能为力。
                //
                //
                //
                "vzeroall" | "vzeroupper" if cfg!(windows) => 30,

                // 使用 `cvtpi2ps` 的内部函数通常是 "composites"，在某些情况下会超出限制。
                //
                "cvtpi2ps" => 25,
                // core_arch/src/arm_shared/simd32 vfmaq_n_f32_vfma : #instructions = 26 >= 22 (limit)
                //
                "usad8" | "vfma" | "vfms" => 27,
                "qadd8" | "qsub8" | "sadd8" | "sel" | "shadd8" | "shsub8" | "usub8" | "ssub8" => 29,
                // core_arch/src/arm_shared/simd32 vst1q_s64_x4_vst1 : #instructions = 22 >= 22 (limit)
                //
                "vld3" => 23,
                // core_arch/src/arm_shared/simd32 vld4q_lane_u32_vld4 : #instructions = 31 >= 22 (limit)
                //
                "vld4" => 32,
                // core_arch/src/arm_shared/simd32 vst1q_s64_x4_vst1 : #instructions = 40 >= 22 (limit)
                //
                "vst1" => 41,
                // core_arch/src/arm_shared/simd32 vst4q_u32_vst4 : #instructions = 26 >= 22 (limit)
                //
                "vst4" => 27,

                // 暂时的，目前 fptosi.sat 和 fptoui.sat LLVM 内部函数会在 arm 上发出不必要的代码。
                // 一旦在 LLVM 中解决了这个问题，就可以将其删除。
                //
                "fcvtzu" | "fcvtzs" | "vcvt" => 64,

                // core_arch/src/arm_shared/simd32 vst1q_p64_x4_nop : #instructions = 33 >= 22 (limit)
                //
                "nop" if fnname.contains("vst1q_p64") => 34,

                // 最初的限制是 20 条指令，但是 ARM DSP Intrinsics 恰好是 20 条指令。
                // 因此，将限制提高到 22，而不是在此处添加一长串例外。
                //
                _ => 22,
            },
            |v| v.parse().unwrap(),
        );
    let probably_only_one_instruction = instrs.len() < instruction_limit;

    if found && probably_only_one_instruction && !inlining_failed {
        return;
    }

    // 先打印发现的反汇编，然后打印 panic，以帮助进行调试，因为我们找不到指令。
    //
    println!("disassembly for {}: ", fnname,);
    for (i, instr) in instrs.iter().enumerate() {
        println!("\t{:2}: {}", i, instr);
    }

    if !found {
        panic!(
            "failed to find instruction `{}` in the disassembly",
            expected
        );
    } else if !probably_only_one_instruction {
        panic!(
            "instruction found, but the disassembly contains too many \
             instructions: #instructions = {} >= {} (limit)",
            instrs.len(),
            instruction_limit
        );
    } else if inlining_failed {
        panic!(
            "instruction found, but the disassembly contains subroutine \
             call instructions, which hint that inlining failed"
        );
    }
}

pub fn assert_skip_test_ok(name: &str) {
    if env::var("STDARCH_TEST_EVERYTHING").is_err() {
        return;
    }
    panic!("skipped test `{}` when it shouldn't be skipped", name);
}

// 请参见 `assert-instr-macro` crate 中的注释以了解为什么存在
pub static mut _DONT_DEDUP: *const u8 = std::ptr::null();
