#![allow(missing_docs)]

use std::mem;

#[cfg(test)]
mod tests;

fn local_sort(v: &mut [f64]) {
    v.sort_by(|x: &f64, y: &f64| x.total_cmp(y));
}

/// Trait that provides simple descriptive statistics on a univariate set of numeric samples. <br>一个 trait，它提供关于单变量数字样本集的简单描述性统计信息。<br>
pub trait Stats {
    /// Sum of the samples. <br>样本的总和。<br>
    ///
    /// Note: this method sacrifices performance at the altar of accuracy Depends on IEEE-754 arithmetic guarantees. <br>这种方法牺牲了性能的准确性。取决于 IEEE-754 的算法保证。<br>
    /// See proof of correctness at: <br>请参见以下网址的正确性证明：<br>
    /// ["Adaptive Precision Floating-Point Arithmetic and Fast Robust Geometric Predicates"][paper]
    ///
    /// [paper]: https://www.cs.cmu.edu/~quake-papers/robust-arithmetic.ps
    ///
    fn sum(&self) -> f64;

    /// Minimum value of the samples. <br>样品的最小值。<br>
    fn min(&self) -> f64;

    /// Maximum value of the samples. <br>样品的最大值。<br>
    fn max(&self) -> f64;

    /// Arithmetic mean (average) of the samples: sum divided by sample-count. <br>样本的算术平均值 (average): 总和除以样本数。<br>
    ///
    /// See: <https://en.wikipedia.org/wiki/Arithmetic_mean> <br>请参见：<https://en.wikipedia.org/wiki/Arithmetic_mean><br>
    fn mean(&self) -> f64;

    /// Median of the samples: value separating the lower half of the samples from the higher half. <br>样本中位数：将样本下半部分与样本上半部分分开的值。<br>
    /// Equal to `self.percentile(50.0)`. <br>等于 `self.percentile(50.0)`。<br>
    ///
    /// See: <https://en.wikipedia.org/wiki/Median> <br>请参见：<https://en.wikipedia.org/wiki/Median><br>
    fn median(&self) -> f64;

    /// Variance of the samples: bias-corrected mean of the squares of the differences of each sample from the sample mean. <br>样本方差：每个样本与样本均值之差的平方的偏差校正后的平均值。<br>
    /// Note that this calculates the _sample variance_ rather than the population variance, which is assumed to be unknown. <br>请注意，这是计算 _sample variance_ 而不是总体方差 (假定该方差是未知的)。<br>
    /// It therefore corrects the `(n-1)/n` bias that would appear if we calculated a population variance, by dividing by `(n-1)` rather than `n`. <br>因此，通过除以 `(n-1)` 而不是 `n`，它可以纠正如果我们计算总体方差时出现的 `(n-1)/n` 偏差。<br>
    ///
    ///
    /// See: <https://en.wikipedia.org/wiki/Variance> <br>请参见：<https://en.wikipedia.org/wiki/Variance><br>
    ///
    fn var(&self) -> f64;

    /// Standard deviation: the square root of the sample variance. <br>标准偏差：样本方差的平方根。<br>
    ///
    /// Note: this is not a robust statistic for non-normal distributions. <br>对于非正态分布，这不是一个可靠的统计数据。<br>
    /// Prefer the `median_abs_dev` for unknown distributions. <br>对于未知的发行版，最好使用 `median_abs_dev`。<br>
    ///
    /// See: <https://en.wikipedia.org/wiki/Standard_deviation> <br>请参见：<https://en.wikipedia.org/wiki/Standard_deviation><br>
    fn std_dev(&self) -> f64;

    /// Standard deviation as a percent of the mean value. <br>标准偏差，以平均值的百分比表示。<br> See `std_dev` and `mean`. <br>请参见 `std_dev` 和 `mean`。<br>
    ///
    /// Note: this is not a robust statistic for non-normal distributions. <br>对于非正态分布，这不是一个可靠的统计数据。<br>
    /// Prefer the `median_abs_dev_pct` for unknown distributions. <br>对于未知的发行版，最好使用 `median_abs_dev_pct`。<br>
    fn std_dev_pct(&self) -> f64;

    /// Scaled median of the absolute deviations of each sample from the sample median. <br>每个样本与样本中位数的绝对偏差的标度中位数。<br>
    /// This is a robust (distribution-agnostic) estimator of sample variability. <br>这是样本可变性的可靠 (distribution-agnostic) 估计器。<br>
    /// Use this in preference to `std_dev` if you cannot assume your sample is normally distributed. <br>如果您不能假定样本是正态分布的，则优先使用 `std_dev`。<br>
    /// Note that this is scaled by the constant `1.4826` to allow its use as a consistent estimator for the standard deviation. <br>请注意，常量 `1.4826` 对它进行了缩放，以使其可用作标准偏差的一致估计量。<br>
    ///
    ///
    /// See: <https://en.wikipedia.org/wiki/Median_absolute_deviation> <br>请参见：<https://en.wikipedia.org/wiki/Median_absolute_deviation><br>
    fn median_abs_dev(&self) -> f64;

    /// Median absolute deviation as a percent of the median. <br>中位数绝对偏差占中位数的百分比。<br> See `median_abs_dev` and `median`. <br>请参见 `median_abs_dev` 和 `median`。<br>
    fn median_abs_dev_pct(&self) -> f64;

    /// Percentile: the value below which `pct` percent of the values in `self` fall. <br>百分位数: `self` 中的值所占的 `pct` 百分比以下的值。<br>
    /// For example, percentile(95.0) will return the value `v` such that 95% of the samples `s` in `self` satisfy `s <= v`. <br>例如，percentile(95.0) 将返回值 `v`，以使 `self` 中 95% 的样本 `s` 满足 `s <= v`。<br>
    ///
    ///
    /// Calculated by linear interpolation between closest ranks. <br>通过最接近等级之间的线性插值计算。<br>
    ///
    /// See: <https://en.wikipedia.org/wiki/Percentile> <br>请参见：<https://en.wikipedia.org/wiki/Percentile><br>
    fn percentile(&self, pct: f64) -> f64;

    /// Quartiles of the sample: three values that divide the sample into four equal groups, each with 1/4 of the data. <br>样本四分位数：将样本分为四个相等组的三个值，每个组的数据为 1/4。<br>
    /// The middle value is the median. <br>中间值为中间值。<br>
    /// See `median` and `percentile`. <br>请参见 `median` 和 `percentile`。<br>
    /// This function may calculate the 3 quartiles more efficiently than 3 calls to `percentile`, but is otherwise equivalent. <br>与对 `percentile` 的 3 个调用相比，此函数可以更有效地计算 3 个四分位数，但在其他方面等效。<br>
    ///
    /// See also: <https://en.wikipedia.org/wiki/Quartile> <br>另请参见：<https://en.wikipedia.org/wiki/Quartile><br>
    fn quartiles(&self) -> (f64, f64, f64);

    /// Inter-quartile range: the difference between the 25th percentile (1st quartile) and the 75th percentile (3rd quartile). <br>四分位间距：第 25 个百分位数 (第 1 个四分位数) 和第 75 个百分位数 (第 3 个四分位数) 之间的差。<br>
    /// See `quartiles`. <br>请参见 `quartiles`。<br>
    ///
    /// See also: <https://en.wikipedia.org/wiki/Interquartile_range> <br>另请参见：<https://en.wikipedia.org/wiki/Interquartile_range><br>
    fn iqr(&self) -> f64;
}

/// Extracted collection of all the summary statistics of a sample set. <br>样本集的所有汇总统计信息的提取集合。<br>
#[derive(Debug, Clone, PartialEq, Copy)]
#[allow(missing_docs)]
pub struct Summary {
    pub sum: f64,
    pub min: f64,
    pub max: f64,
    pub mean: f64,
    pub median: f64,
    pub var: f64,
    pub std_dev: f64,
    pub std_dev_pct: f64,
    pub median_abs_dev: f64,
    pub median_abs_dev_pct: f64,
    pub quartiles: (f64, f64, f64),
    pub iqr: f64,
}

impl Summary {
    /// Construct a new summary of a sample set. <br>创建一个新的样本集总结。<br>
    pub fn new(samples: &[f64]) -> Summary {
        Summary {
            sum: samples.sum(),
            min: samples.min(),
            max: samples.max(),
            mean: samples.mean(),
            median: samples.median(),
            var: samples.var(),
            std_dev: samples.std_dev(),
            std_dev_pct: samples.std_dev_pct(),
            median_abs_dev: samples.median_abs_dev(),
            median_abs_dev_pct: samples.median_abs_dev_pct(),
            quartiles: samples.quartiles(),
            iqr: samples.iqr(),
        }
    }
}

impl Stats for [f64] {
    // FIXME #11059 handle NaN, inf and overflow <br>#11059 处理 NaN，inf 和溢出<br>
    fn sum(&self) -> f64 {
        let mut partials = vec![];

        for &x in self {
            let mut x = x;
            let mut j = 0;
            // This inner loop applies `hi`/`lo` summation to each partial so that the list of partial sums remains exact. <br>此内部循环将 `hi`/`lo` 总和应用于每个部分，以使部分总和的列表保持准确。<br>
            //
            for i in 0..partials.len() {
                let mut y: f64 = partials[i];
                if x.abs() < y.abs() {
                    mem::swap(&mut x, &mut y);
                }
                // Rounded `x+y` is stored in `hi` with round-off stored in `lo`. <br>四舍五入的 `x+y` 存储在 `hi` 中，四舍五入存储在 `lo` 中。<br>
                // Together `hi+lo` are exactly equal to `x+y`. <br>`hi+lo` 等于 `x+y`。<br>
                let hi = x + y;
                let lo = y - (hi - x);
                if lo != 0.0 {
                    partials[j] = lo;
                    j += 1;
                }
                x = hi;
            }
            if j >= partials.len() {
                partials.push(x);
            } else {
                partials[j] = x;
                partials.truncate(j + 1);
            }
        }
        let zero: f64 = 0.0;
        partials.iter().fold(zero, |p, q| p + *q)
    }

    fn min(&self) -> f64 {
        assert!(!self.is_empty());
        self.iter().fold(self[0], |p, q| p.min(*q))
    }

    fn max(&self) -> f64 {
        assert!(!self.is_empty());
        self.iter().fold(self[0], |p, q| p.max(*q))
    }

    fn mean(&self) -> f64 {
        assert!(!self.is_empty());
        self.sum() / (self.len() as f64)
    }

    fn median(&self) -> f64 {
        self.percentile(50_f64)
    }

    fn var(&self) -> f64 {
        if self.len() < 2 {
            0.0
        } else {
            let mean = self.mean();
            let mut v: f64 = 0.0;
            for s in self {
                let x = *s - mean;
                v += x * x;
            }
            // N.B., this is _supposed to be_ len-1, not len. <br>注意，这 _应该是 _ len-1，而不是 len。<br>
            // If you change it back to len, you will be calculating a population variance, not a sample variance. <br>如果将其更改回 len，则将计算总体方差，而不是样本方差。<br>
            //
            let denom = (self.len() - 1) as f64;
            v / denom
        }
    }

    fn std_dev(&self) -> f64 {
        self.var().sqrt()
    }

    fn std_dev_pct(&self) -> f64 {
        let hundred = 100_f64;
        (self.std_dev() / self.mean()) * hundred
    }

    fn median_abs_dev(&self) -> f64 {
        let med = self.median();
        let abs_devs: Vec<f64> = self.iter().map(|&v| (med - v).abs()).collect();
        // This constant is derived by smarter statistics brains than me, but it is consistent with how R and other packages treat the MAD. <br>这个常量是由比我更聪明的统计头脑得出的，但它与 R 和其他软件包对 MAD 的处理方式一致。<br>
        //
        let number = 1.4826;
        abs_devs.median() * number
    }

    fn median_abs_dev_pct(&self) -> f64 {
        let hundred = 100_f64;
        (self.median_abs_dev() / self.median()) * hundred
    }

    fn percentile(&self, pct: f64) -> f64 {
        let mut tmp = self.to_vec();
        local_sort(&mut tmp);
        percentile_of_sorted(&tmp, pct)
    }

    fn quartiles(&self) -> (f64, f64, f64) {
        let mut tmp = self.to_vec();
        local_sort(&mut tmp);
        let first = 25_f64;
        let a = percentile_of_sorted(&tmp, first);
        let second = 50_f64;
        let b = percentile_of_sorted(&tmp, second);
        let third = 75_f64;
        let c = percentile_of_sorted(&tmp, third);
        (a, b, c)
    }

    fn iqr(&self) -> f64 {
        let (a, _, c) = self.quartiles();
        c - a
    }
}

// Helper function: extract a value representing the `pct` percentile of a sorted sample-set, using linear interpolation. <br>Helper 函数：使用线性插值提取代表排序后的样本集的 `pct` 百分位数的值。<br>
// If samples are not sorted, return nonsensical value. <br>如果样品未排序，则返回无意义的值。<br>
fn percentile_of_sorted(sorted_samples: &[f64], pct: f64) -> f64 {
    assert!(!sorted_samples.is_empty());
    if sorted_samples.len() == 1 {
        return sorted_samples[0];
    }
    let zero: f64 = 0.0;
    assert!(zero <= pct);
    let hundred = 100_f64;
    assert!(pct <= hundred);
    if pct == hundred {
        return sorted_samples[sorted_samples.len() - 1];
    }
    let length = (sorted_samples.len() - 1) as f64;
    let rank = (pct / hundred) * length;
    let lrank = rank.floor();
    let d = rank - lrank;
    let n = lrank as usize;
    let lo = sorted_samples[n];
    let hi = sorted_samples[n + 1];
    lo + (hi - lo) * d
}

/// Winsorize a set of samples, replacing values above the `100-pct` percentile and below the `pct` percentile with those percentiles themselves. <br>Winsorize 一组样本，用这些百分数本身替换 `100-pct` 百分数以上和 `pct` 百分数以下的值。<br>
/// This is a way of minimizing the effect of outliers, at the cost of biasing the sample. <br>这是最小化离群值影响的方法，但要以使样本产生偏差为代价。<br>
/// It differs from trimming in that it does not change the number of samples, just changes the values of those that are outliers. <br>它与修整的不同之处在于，它不会更改样本数，而只会更改异常值的值。<br>
///
/// See: <https://en.wikipedia.org/wiki/Winsorising> <br>请参见：<https://en.wikipedia.org/wiki/Winsorising><br>
///
///
pub fn winsorize(samples: &mut [f64], pct: f64) {
    let mut tmp = samples.to_vec();
    local_sort(&mut tmp);
    let lo = percentile_of_sorted(&tmp, pct);
    let hundred = 100_f64;
    let hi = percentile_of_sorted(&tmp, hundred - pct);
    for samp in samples {
        if *samp > hi {
            *samp = hi
        } else if *samp < lo {
            *samp = lo
        }
    }
}
