//! Terminal formatting module. <br>终端格式化模块。<br>
//!
//! This module provides the `Terminal` trait, which abstracts over an [ANSI Terminal][ansi] to provide color printing, among other things. <br>该模块提供了 `Terminal` trait，它抽象了 [ANSI 终端][ansi] 以提供彩色打印等。<br>
//! There are two implementations, the `TerminfoTerminal`, which uses control characters from a [terminfo][ti] database, and `WinConsole`, which uses the [Win32 Console API][win]. <br>有两种实现，一种是 `TerminfoTerminal`，它使用来自 [术语信息][ti] 数据库的控制字符，另一种是 `WinConsole`，它使用 [Win32 控制台 API][win]。<br>
//!
//!
//! [ansi]: https://en.wikipedia.org/wiki/ANSI_escape_code
//! [win]: https://docs.microsoft.com/en-us/windows/console/character-mode-applications
//! [ti]: https://en.wikipedia.org/wiki/Terminfo
//!
//!

#![deny(missing_docs)]

use std::io::{self, prelude::*};

pub(crate) use terminfo::TerminfoTerminal;
#[cfg(windows)]
pub(crate) use win::WinConsole;

pub(crate) mod terminfo;

#[cfg(windows)]
mod win;

/// Alias for stdout terminals. <br>stdout 终端的别名。<br>
pub(crate) type StdoutTerminal = dyn Terminal + Send;

#[cfg(not(windows))]
/// Returns a Terminal wrapping stdout, or None if a terminal couldn't be opened. <br>返回包装 stdout 的终端，如果无法打开终端，则返回 None。<br>
///
pub(crate) fn stdout() -> Option<Box<StdoutTerminal>> {
    TerminfoTerminal::new(io::stdout()).map(|t| Box::new(t) as Box<StdoutTerminal>)
}

#[cfg(windows)]
/// Returns a Terminal wrapping stdout, or None if a terminal couldn't be opened. <br>返回包装 stdout 的终端，如果无法打开终端，则返回 None。<br>
///
pub(crate) fn stdout() -> Option<Box<StdoutTerminal>> {
    TerminfoTerminal::new(io::stdout())
        .map(|t| Box::new(t) as Box<StdoutTerminal>)
        .or_else(|| WinConsole::new(io::stdout()).ok().map(|t| Box::new(t) as Box<StdoutTerminal>))
}

/// Terminal color definitions <br>终端颜色定义<br>
#[allow(missing_docs)]
#[cfg_attr(not(windows), allow(dead_code))]
pub(crate) mod color {
    /// Number for a terminal color <br>终端颜色的编号<br>
    pub(crate) type Color = u32;

    pub(crate) const BLACK: Color = 0;
    pub(crate) const RED: Color = 1;
    pub(crate) const GREEN: Color = 2;
    pub(crate) const YELLOW: Color = 3;
    pub(crate) const BLUE: Color = 4;
    pub(crate) const MAGENTA: Color = 5;
    pub(crate) const CYAN: Color = 6;
    pub(crate) const WHITE: Color = 7;
}

/// A terminal with similar capabilities to an ANSI Terminal (foreground/background colors etc). <br>具有与 ANSI 终端 (foreground/background 颜色等) 相似的功能的终端。<br>
///
pub trait Terminal: Write {
    /// Sets the foreground color to the given color. <br>将前景色设置为给定的颜色。<br>
    ///
    /// If the color is a bright color, but the terminal only supports 8 colors, the corresponding normal color will be used instead. <br>如果颜色是亮色，但终端仅支持 8 种颜色，则将使用相应的常规颜色。<br>
    ///
    ///
    /// Returns `Ok(true)` if the color was set, `Ok(false)` otherwise, and `Err(e)` if there was an I/O error. <br>如果设置了颜色，则返回 `Ok(true)`，否则返回 `Ok(false)`，如果存在 I/O 错误，则返回 `Err(e)`。<br>
    ///
    fn fg(&mut self, color: color::Color) -> io::Result<bool>;

    /// Resets all terminal attributes and colors to their defaults. <br>将所有终端属性和颜色重置为其默认值。<br>
    ///
    /// Returns `Ok(true)` if the terminal was reset, `Ok(false)` otherwise, and `Err(e)` if there was an I/O error. <br>如果重置了终端，则返回 `Ok(true)`; 否则，返回 `Ok(false)`; 如果发生 I/O 错误，则返回 `Err(e)`。<br>
    ///
    /// *Note: This does not flush. <br>这不会刷新。<br>*
    ///
    /// That means the reset command may get buffered so, if you aren't planning on doing anything else that might flush stdout's buffer (e.g., writing a line of text), you should flush after calling reset. <br>这意味着 reset 命令可能会被缓冲，因此，如果您不打算做其他可能会刷新 stdout 缓冲区的事情 (例如，编写一行文本)，则应在调用 reset 之后刷新。<br>
    ///
    ///
    ///
    fn reset(&mut self) -> io::Result<bool>;
}
