//! Common types used by `libtest`. <br>`libtest` 使用的常见类型。<br>

use std::borrow::Cow;
use std::fmt;

use super::bench::Bencher;
use super::options;

pub use NamePadding::*;
pub use TestFn::*;
pub use TestName::*;

/// Type of the test according to the [rust book](https://doc.rust-lang.org/cargo/guide/tests.html) conventions. <br>根据 [rust 书](https://doc.rust-lang.org/cargo/guide/tests.html) 约定的测试类型。<br>
///
#[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
pub enum TestType {
    /// Unit-tests are expected to be in the `src` folder of the crate. <br>单元测试应位于 crate 的 `src` 文件夹中。<br>
    UnitTest,
    /// Integration-style tests are expected to be in the `tests` folder of the crate. <br>集成样式的测试应该位于 crate 的 `tests` 文件夹中。<br>
    IntegrationTest,
    /// Doctests are created by the `librustdoc` manually, so it's a different type of test. <br>Doctests 是由 `librustdoc` 手动创建的，因此它是另一种类型的测试。<br>
    DocTest,
    /// Tests for the sources that don't follow the project layout convention (e.g. <br>测试不遵循项目布局约定的源 (例如，<br>
    /// tests in raw `main.rs` compiled by calling `rustc --test` directly). <br>在直接调用 `rustc --test` 编译的原始 `main.rs` 中进行测试)。<br>
    Unknown,
}

#[derive(Clone, Copy, PartialEq, Eq, Hash, Debug)]
pub enum NamePadding {
    PadNone,
    PadOnRight,
}

// The name of a test. <br>测试的名称。<br>
// By convention this follows the rules for rust paths; <br>按照惯例，这遵循 rust 路径的规则；<br> i.e., it should be a series of identifiers separated by double colons. <br>即，它应该是由双冒号分隔的一系列标识符。<br>
//
// This way if some test runner wants to arrange the tests hierarchically it may. <br>这样，如果某些测试跑步者想要按层次排列测试，则可以。<br>
#[derive(Clone, PartialEq, Eq, Hash, Debug)]
pub enum TestName {
    StaticTestName(&'static str),
    DynTestName(String),
    AlignedTestName(Cow<'static, str>, NamePadding),
}

impl TestName {
    pub fn as_slice(&self) -> &str {
        match *self {
            StaticTestName(s) => s,
            DynTestName(ref s) => s,
            AlignedTestName(ref s, _) => &*s,
        }
    }

    pub fn padding(&self) -> NamePadding {
        match self {
            &AlignedTestName(_, p) => p,
            _ => PadNone,
        }
    }

    pub fn with_padding(&self, padding: NamePadding) -> TestName {
        let name = match *self {
            TestName::StaticTestName(name) => Cow::Borrowed(name),
            TestName::DynTestName(ref name) => Cow::Owned(name.clone()),
            TestName::AlignedTestName(ref name, _) => name.clone(),
        };

        TestName::AlignedTestName(name, padding)
    }
}
impl fmt::Display for TestName {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt::Display::fmt(self.as_slice(), f)
    }
}

// A function that runs a test. <br>运行测试的函数。<br>
// If the function returns successfully, the test succeeds; <br>如果函数成功返回，则测试成功; 否则，函数将失败。<br> if the function panics then the test fails. <br>如果函数 panics，则测试失败。<br>
// We may need to come up with a more clever definition of test in order to support isolation of tests into threads. <br>为了支持将测试隔离到线程中，我们可能需要提出一个更聪明的测试定义。<br>
//
pub enum TestFn {
    StaticTestFn(fn()),
    StaticBenchFn(fn(&mut Bencher)),
    DynTestFn(Box<dyn FnOnce() + Send>),
    DynBenchFn(Box<dyn Fn(&mut Bencher) + Send>),
}

impl TestFn {
    pub fn padding(&self) -> NamePadding {
        match *self {
            StaticTestFn(..) => PadNone,
            StaticBenchFn(..) => PadOnRight,
            DynTestFn(..) => PadNone,
            DynBenchFn(..) => PadOnRight,
        }
    }
}

impl fmt::Debug for TestFn {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.write_str(match *self {
            StaticTestFn(..) => "StaticTestFn(..)",
            StaticBenchFn(..) => "StaticBenchFn(..)",
            DynTestFn(..) => "DynTestFn(..)",
            DynBenchFn(..) => "DynBenchFn(..)",
        })
    }
}

// A unique integer associated with each test. <br>与每个测试关联的唯一整数。<br>
#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
pub struct TestId(pub usize);

// The definition of a single test. <br>单个测试的定义。<br>
// A test runner will run a list of these. <br>测试运行器将运行这些列表。<br>
#[derive(Clone, Debug)]
pub struct TestDesc {
    pub name: TestName,
    pub ignore: bool,
    pub ignore_message: Option<&'static str>,
    pub should_panic: options::ShouldPanic,
    pub compile_fail: bool,
    pub no_run: bool,
    pub test_type: TestType,
}

impl TestDesc {
    pub fn padded_name(&self, column_count: usize, align: NamePadding) -> String {
        let mut name = String::from(self.name.as_slice());
        let fill = column_count.saturating_sub(name.len());
        let pad = " ".repeat(fill);
        match align {
            PadNone => name,
            PadOnRight => {
                name.push_str(&pad);
                name
            }
        }
    }

    /// Returns None for ignored test or that that are just run, otherwise give a description of the type of test. <br>对于被忽略的测试或刚刚运行的测试，返回 None，否则给出测试类型的描述。<br>
    /// Descriptions include "should panic", "compile fail" and "compile". <br>描述包括 "应该 panic"、"编译失败" 和 "编译"。<br>
    pub fn test_mode(&self) -> Option<&'static str> {
        if self.ignore {
            return None;
        }
        match self.should_panic {
            options::ShouldPanic::Yes | options::ShouldPanic::YesWithMessage(_) => {
                return Some("should panic");
            }
            options::ShouldPanic::No => {}
        }
        if self.compile_fail {
            return Some("compile fail");
        }
        if self.no_run {
            return Some("compile");
        }
        None
    }
}

#[derive(Debug)]
pub struct TestDescAndFn {
    pub desc: TestDesc,
    pub testfn: TestFn,
}
